/**
 * vis-charts
 * http://visjs.org/
 *
 * A dynamic, browser-based visualization library.
 *
 * @version 3.0.0
 * @date    2020-05-17T12:00:20.796Z
 *
 * @copyright (c) 2011-2017 Almende B.V, http://almende.com
 * @copyright (c) 2017-2019 visjs contributors, https://github.com/visjs
 *
 * @license
 * vis.js is dual licensed under both
 *
 *   1. The Apache 2.0 License
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *   and
 *
 *   2. The MIT License
 *      http://opensource.org/licenses/MIT
 *
 * vis.js may be distributed under either license.
 */

(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('vis-network/styles/vis-network.css')) :
	typeof define === 'function' && define.amd ? define(['exports', 'vis-network/styles/vis-network.css'], factory) :
	(global = global || self, factory(global.vis = global.vis || {}));
}(this, (function (exports) {
	var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

	function unwrapExports (x) {
		return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
	}

	function createCommonjsModule(fn, module) {
		return module = { exports: {} }, fn(module, module.exports), module.exports;
	}

	function getCjsExportFromNamespace (n) {
		return n && n['default'] || n;
	}

	function commonjsRequire () {
		throw new Error('Dynamic requires are not currently supported by @rollup/plugin-commonjs');
	}

	var check = function (it) {
	  return it && it.Math == Math && it;
	}; // https://github.com/zloirock/core-js/issues/86#issuecomment-115759028


	var global_1 = // eslint-disable-next-line no-undef
	check(typeof globalThis == 'object' && globalThis) || check(typeof window == 'object' && window) || check(typeof self == 'object' && self) || check(typeof commonjsGlobal == 'object' && commonjsGlobal) || // eslint-disable-next-line no-new-func
	Function('return this')();

	var fails = function (exec) {
	  try {
	    return !!exec();
	  } catch (error) {
	    return true;
	  }
	};

	var descriptors = !fails(function () {
	  return Object.defineProperty({}, 1, {
	    get: function () {
	      return 7;
	    }
	  })[1] != 7;
	});

	var nativePropertyIsEnumerable = {}.propertyIsEnumerable;
	var getOwnPropertyDescriptor = Object.getOwnPropertyDescriptor; // Nashorn ~ JDK8 bug

	var NASHORN_BUG = getOwnPropertyDescriptor && !nativePropertyIsEnumerable.call({
	  1: 2
	}, 1); // `Object.prototype.propertyIsEnumerable` method implementation
	// https://tc39.github.io/ecma262/#sec-object.prototype.propertyisenumerable

	var f = NASHORN_BUG ? function propertyIsEnumerable(V) {
	  var descriptor = getOwnPropertyDescriptor(this, V);
	  return !!descriptor && descriptor.enumerable;
	} : nativePropertyIsEnumerable;
	var objectPropertyIsEnumerable = {
	  f: f
	};

	var createPropertyDescriptor = function (bitmap, value) {
	  return {
	    enumerable: !(bitmap & 1),
	    configurable: !(bitmap & 2),
	    writable: !(bitmap & 4),
	    value: value
	  };
	};

	var toString = {}.toString;

	var classofRaw = function (it) {
	  return toString.call(it).slice(8, -1);
	};

	var split = ''.split; // fallback for non-array-like ES3 and non-enumerable old V8 strings

	var indexedObject = fails(function () {
	  // throws an error in rhino, see https://github.com/mozilla/rhino/issues/346
	  // eslint-disable-next-line no-prototype-builtins
	  return !Object('z').propertyIsEnumerable(0);
	}) ? function (it) {
	  return classofRaw(it) == 'String' ? split.call(it, '') : Object(it);
	} : Object;

	// `RequireObjectCoercible` abstract operation
	// https://tc39.github.io/ecma262/#sec-requireobjectcoercible
	var requireObjectCoercible = function (it) {
	  if (it == undefined) throw TypeError("Can't call method on " + it);
	  return it;
	};

	var toIndexedObject = function (it) {
	  return indexedObject(requireObjectCoercible(it));
	};

	var isObject = function (it) {
	  return typeof it === 'object' ? it !== null : typeof it === 'function';
	};

	// https://tc39.github.io/ecma262/#sec-toprimitive
	// instead of the ES6 spec version, we didn't implement @@toPrimitive case
	// and the second argument - flag - preferred type is a string

	var toPrimitive = function (input, PREFERRED_STRING) {
	  if (!isObject(input)) return input;
	  var fn, val;
	  if (PREFERRED_STRING && typeof (fn = input.toString) == 'function' && !isObject(val = fn.call(input))) return val;
	  if (typeof (fn = input.valueOf) == 'function' && !isObject(val = fn.call(input))) return val;
	  if (!PREFERRED_STRING && typeof (fn = input.toString) == 'function' && !isObject(val = fn.call(input))) return val;
	  throw TypeError("Can't convert object to primitive value");
	};

	var hasOwnProperty = {}.hasOwnProperty;

	var has = function (it, key) {
	  return hasOwnProperty.call(it, key);
	};

	var document$1 = global_1.document; // typeof document.createElement is 'object' in old IE

	var EXISTS = isObject(document$1) && isObject(document$1.createElement);

	var documentCreateElement = function (it) {
	  return EXISTS ? document$1.createElement(it) : {};
	};

	var ie8DomDefine = !descriptors && !fails(function () {
	  return Object.defineProperty(documentCreateElement('div'), 'a', {
	    get: function () {
	      return 7;
	    }
	  }).a != 7;
	});

	var nativeGetOwnPropertyDescriptor = Object.getOwnPropertyDescriptor; // `Object.getOwnPropertyDescriptor` method
	// https://tc39.github.io/ecma262/#sec-object.getownpropertydescriptor

	var f$1 = descriptors ? nativeGetOwnPropertyDescriptor : function getOwnPropertyDescriptor(O, P) {
	  O = toIndexedObject(O);
	  P = toPrimitive(P, true);
	  if (ie8DomDefine) try {
	    return nativeGetOwnPropertyDescriptor(O, P);
	  } catch (error) {
	    /* empty */
	  }
	  if (has(O, P)) return createPropertyDescriptor(!objectPropertyIsEnumerable.f.call(O, P), O[P]);
	};
	var objectGetOwnPropertyDescriptor = {
	  f: f$1
	};

	var replacement = /#|\.prototype\./;

	var isForced = function (feature, detection) {
	  var value = data[normalize(feature)];
	  return value == POLYFILL ? true : value == NATIVE ? false : typeof detection == 'function' ? fails(detection) : !!detection;
	};

	var normalize = isForced.normalize = function (string) {
	  return String(string).replace(replacement, '.').toLowerCase();
	};

	var data = isForced.data = {};
	var NATIVE = isForced.NATIVE = 'N';
	var POLYFILL = isForced.POLYFILL = 'P';
	var isForced_1 = isForced;

	var path = {};

	var aFunction = function (it) {
	  if (typeof it != 'function') {
	    throw TypeError(String(it) + ' is not a function');
	  }

	  return it;
	};

	var functionBindContext = function (fn, that, length) {
	  aFunction(fn);
	  if (that === undefined) return fn;

	  switch (length) {
	    case 0:
	      return function () {
	        return fn.call(that);
	      };

	    case 1:
	      return function (a) {
	        return fn.call(that, a);
	      };

	    case 2:
	      return function (a, b) {
	        return fn.call(that, a, b);
	      };

	    case 3:
	      return function (a, b, c) {
	        return fn.call(that, a, b, c);
	      };
	  }

	  return function ()
	  /* ...args */
	  {
	    return fn.apply(that, arguments);
	  };
	};

	var anObject = function (it) {
	  if (!isObject(it)) {
	    throw TypeError(String(it) + ' is not an object');
	  }

	  return it;
	};

	var nativeDefineProperty = Object.defineProperty; // `Object.defineProperty` method
	// https://tc39.github.io/ecma262/#sec-object.defineproperty

	var f$2 = descriptors ? nativeDefineProperty : function defineProperty(O, P, Attributes) {
	  anObject(O);
	  P = toPrimitive(P, true);
	  anObject(Attributes);
	  if (ie8DomDefine) try {
	    return nativeDefineProperty(O, P, Attributes);
	  } catch (error) {
	    /* empty */
	  }
	  if ('get' in Attributes || 'set' in Attributes) throw TypeError('Accessors not supported');
	  if ('value' in Attributes) O[P] = Attributes.value;
	  return O;
	};
	var objectDefineProperty = {
	  f: f$2
	};

	var createNonEnumerableProperty = descriptors ? function (object, key, value) {
	  return objectDefineProperty.f(object, key, createPropertyDescriptor(1, value));
	} : function (object, key, value) {
	  object[key] = value;
	  return object;
	};

	var getOwnPropertyDescriptor$1 = objectGetOwnPropertyDescriptor.f;

	var wrapConstructor = function (NativeConstructor) {
	  var Wrapper = function (a, b, c) {
	    if (this instanceof NativeConstructor) {
	      switch (arguments.length) {
	        case 0:
	          return new NativeConstructor();

	        case 1:
	          return new NativeConstructor(a);

	        case 2:
	          return new NativeConstructor(a, b);
	      }

	      return new NativeConstructor(a, b, c);
	    }

	    return NativeConstructor.apply(this, arguments);
	  };

	  Wrapper.prototype = NativeConstructor.prototype;
	  return Wrapper;
	};
	/*
	  options.target      - name of the target object
	  options.global      - target is the global object
	  options.stat        - export as static methods of target
	  options.proto       - export as prototype methods of target
	  options.real        - real prototype method for the `pure` version
	  options.forced      - export even if the native feature is available
	  options.bind        - bind methods to the target, required for the `pure` version
	  options.wrap        - wrap constructors to preventing global pollution, required for the `pure` version
	  options.unsafe      - use the simple assignment of property instead of delete + defineProperty
	  options.sham        - add a flag to not completely full polyfills
	  options.enumerable  - export as enumerable property
	  options.noTargetGet - prevent calling a getter on target
	*/


	var _export = function (options, source) {
	  var TARGET = options.target;
	  var GLOBAL = options.global;
	  var STATIC = options.stat;
	  var PROTO = options.proto;
	  var nativeSource = GLOBAL ? global_1 : STATIC ? global_1[TARGET] : (global_1[TARGET] || {}).prototype;
	  var target = GLOBAL ? path : path[TARGET] || (path[TARGET] = {});
	  var targetPrototype = target.prototype;
	  var FORCED, USE_NATIVE, VIRTUAL_PROTOTYPE;
	  var key, sourceProperty, targetProperty, nativeProperty, resultProperty, descriptor;

	  for (key in source) {
	    FORCED = isForced_1(GLOBAL ? key : TARGET + (STATIC ? '.' : '#') + key, options.forced); // contains in native

	    USE_NATIVE = !FORCED && nativeSource && has(nativeSource, key);
	    targetProperty = target[key];
	    if (USE_NATIVE) if (options.noTargetGet) {
	      descriptor = getOwnPropertyDescriptor$1(nativeSource, key);
	      nativeProperty = descriptor && descriptor.value;
	    } else nativeProperty = nativeSource[key]; // export native or implementation

	    sourceProperty = USE_NATIVE && nativeProperty ? nativeProperty : source[key];
	    if (USE_NATIVE && typeof targetProperty === typeof sourceProperty) continue; // bind timers to global for call from export context

	    if (options.bind && USE_NATIVE) resultProperty = functionBindContext(sourceProperty, global_1); // wrap global constructors for prevent changs in this version
	    else if (options.wrap && USE_NATIVE) resultProperty = wrapConstructor(sourceProperty); // make static versions for prototype methods
	      else if (PROTO && typeof sourceProperty == 'function') resultProperty = functionBindContext(Function.call, sourceProperty); // default case
	        else resultProperty = sourceProperty; // add a flag to not completely full polyfills

	    if (options.sham || sourceProperty && sourceProperty.sham || targetProperty && targetProperty.sham) {
	      createNonEnumerableProperty(resultProperty, 'sham', true);
	    }

	    target[key] = resultProperty;

	    if (PROTO) {
	      VIRTUAL_PROTOTYPE = TARGET + 'Prototype';

	      if (!has(path, VIRTUAL_PROTOTYPE)) {
	        createNonEnumerableProperty(path, VIRTUAL_PROTOTYPE, {});
	      } // export virtual prototype methods


	      path[VIRTUAL_PROTOTYPE][key] = sourceProperty; // export real prototype methods

	      if (options.real && targetPrototype && !targetPrototype[key]) {
	        createNonEnumerableProperty(targetPrototype, key, sourceProperty);
	      }
	    }
	  }
	};

	var ceil = Math.ceil;
	var floor = Math.floor; // `ToInteger` abstract operation
	// https://tc39.github.io/ecma262/#sec-tointeger

	var toInteger = function (argument) {
	  return isNaN(argument = +argument) ? 0 : (argument > 0 ? floor : ceil)(argument);
	};

	var min = Math.min; // `ToLength` abstract operation
	// https://tc39.github.io/ecma262/#sec-tolength

	var toLength = function (argument) {
	  return argument > 0 ? min(toInteger(argument), 0x1FFFFFFFFFFFFF) : 0; // 2 ** 53 - 1 == 9007199254740991
	};

	var max = Math.max;
	var min$1 = Math.min; // Helper for a popular repeating case of the spec:
	// Let integer be ? ToInteger(index).
	// If integer < 0, let result be max((length + integer), 0); else let result be min(integer, length).

	var toAbsoluteIndex = function (index, length) {
	  var integer = toInteger(index);
	  return integer < 0 ? max(integer + length, 0) : min$1(integer, length);
	};

	var createMethod = function (IS_INCLUDES) {
	  return function ($this, el, fromIndex) {
	    var O = toIndexedObject($this);
	    var length = toLength(O.length);
	    var index = toAbsoluteIndex(fromIndex, length);
	    var value; // Array#includes uses SameValueZero equality algorithm
	    // eslint-disable-next-line no-self-compare

	    if (IS_INCLUDES && el != el) while (length > index) {
	      value = O[index++]; // eslint-disable-next-line no-self-compare

	      if (value != value) return true; // Array#indexOf ignores holes, Array#includes - not
	    } else for (; length > index; index++) {
	      if ((IS_INCLUDES || index in O) && O[index] === el) return IS_INCLUDES || index || 0;
	    }
	    return !IS_INCLUDES && -1;
	  };
	};

	var arrayIncludes = {
	  // `Array.prototype.includes` method
	  // https://tc39.github.io/ecma262/#sec-array.prototype.includes
	  includes: createMethod(true),
	  // `Array.prototype.indexOf` method
	  // https://tc39.github.io/ecma262/#sec-array.prototype.indexof
	  indexOf: createMethod(false)
	};

	var hiddenKeys = {};

	var indexOf = arrayIncludes.indexOf;

	var objectKeysInternal = function (object, names) {
	  var O = toIndexedObject(object);
	  var i = 0;
	  var result = [];
	  var key;

	  for (key in O) !has(hiddenKeys, key) && has(O, key) && result.push(key); // Don't enum bug & hidden keys


	  while (names.length > i) if (has(O, key = names[i++])) {
	    ~indexOf(result, key) || result.push(key);
	  }

	  return result;
	};

	// IE8- don't enum bug keys
	var enumBugKeys = ['constructor', 'hasOwnProperty', 'isPrototypeOf', 'propertyIsEnumerable', 'toLocaleString', 'toString', 'valueOf'];

	// https://tc39.github.io/ecma262/#sec-object.keys

	var objectKeys = Object.keys || function keys(O) {
	  return objectKeysInternal(O, enumBugKeys);
	};

	var f$3 = Object.getOwnPropertySymbols;
	var objectGetOwnPropertySymbols = {
	  f: f$3
	};

	// https://tc39.github.io/ecma262/#sec-toobject

	var toObject = function (argument) {
	  return Object(requireObjectCoercible(argument));
	};

	var nativeAssign = Object.assign;
	var defineProperty = Object.defineProperty; // `Object.assign` method
	// https://tc39.github.io/ecma262/#sec-object.assign

	var objectAssign = !nativeAssign || fails(function () {
	  // should have correct order of operations (Edge bug)
	  if (descriptors && nativeAssign({
	    b: 1
	  }, nativeAssign(defineProperty({}, 'a', {
	    enumerable: true,
	    get: function () {
	      defineProperty(this, 'b', {
	        value: 3,
	        enumerable: false
	      });
	    }
	  }), {
	    b: 2
	  })).b !== 1) return true; // should work with symbols and should have deterministic property order (V8 bug)

	  var A = {};
	  var B = {}; // eslint-disable-next-line no-undef

	  var symbol = Symbol();
	  var alphabet = 'abcdefghijklmnopqrst';
	  A[symbol] = 7;
	  alphabet.split('').forEach(function (chr) {
	    B[chr] = chr;
	  });
	  return nativeAssign({}, A)[symbol] != 7 || objectKeys(nativeAssign({}, B)).join('') != alphabet;
	}) ? function assign(target, source) {
	  // eslint-disable-line no-unused-vars
	  var T = toObject(target);
	  var argumentsLength = arguments.length;
	  var index = 1;
	  var getOwnPropertySymbols = objectGetOwnPropertySymbols.f;
	  var propertyIsEnumerable = objectPropertyIsEnumerable.f;

	  while (argumentsLength > index) {
	    var S = indexedObject(arguments[index++]);
	    var keys = getOwnPropertySymbols ? objectKeys(S).concat(getOwnPropertySymbols(S)) : objectKeys(S);
	    var length = keys.length;
	    var j = 0;
	    var key;

	    while (length > j) {
	      key = keys[j++];
	      if (!descriptors || propertyIsEnumerable.call(S, key)) T[key] = S[key];
	    }
	  }

	  return T;
	} : nativeAssign;

	// https://tc39.github.io/ecma262/#sec-object.assign

	_export({
	  target: 'Object',
	  stat: true,
	  forced: Object.assign !== objectAssign
	}, {
	  assign: objectAssign
	});

	var assign = path.Object.assign;

	var assign$1 = assign;

	var assign$2 = assign$1;

	function _assertThisInitialized(self) {
	  if (self === void 0) {
	    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
	  }

	  return self;
	}

	var assertThisInitialized = _assertThisInitialized;

	// https://tc39.github.io/ecma262/#sec-object.defineproperty

	_export({
	  target: 'Object',
	  stat: true,
	  forced: !descriptors,
	  sham: !descriptors
	}, {
	  defineProperty: objectDefineProperty.f
	});

	var defineProperty_1 = createCommonjsModule(function (module) {
	  var Object = path.Object;

	  var defineProperty = module.exports = function defineProperty(it, key, desc) {
	    return Object.defineProperty(it, key, desc);
	  };

	  if (Object.defineProperty.sham) defineProperty.sham = true;
	});

	var defineProperty$1 = defineProperty_1;

	var defineProperty$2 = defineProperty$1;

	// https://tc39.github.io/ecma262/#sec-object.defineproperties

	var objectDefineProperties = descriptors ? Object.defineProperties : function defineProperties(O, Properties) {
	  anObject(O);
	  var keys = objectKeys(Properties);
	  var length = keys.length;
	  var index = 0;
	  var key;

	  while (length > index) objectDefineProperty.f(O, key = keys[index++], Properties[key]);

	  return O;
	};

	// https://tc39.github.io/ecma262/#sec-object.defineproperties

	_export({
	  target: 'Object',
	  stat: true,
	  forced: !descriptors,
	  sham: !descriptors
	}, {
	  defineProperties: objectDefineProperties
	});

	var defineProperties_1 = createCommonjsModule(function (module) {
	  var Object = path.Object;

	  var defineProperties = module.exports = function defineProperties(T, D) {
	    return Object.defineProperties(T, D);
	  };

	  if (Object.defineProperties.sham) defineProperties.sham = true;
	});

	var defineProperties = defineProperties_1;

	var defineProperties$1 = defineProperties;

	var aFunction$1 = function (variable) {
	  return typeof variable == 'function' ? variable : undefined;
	};

	var getBuiltIn = function (namespace, method) {
	  return arguments.length < 2 ? aFunction$1(path[namespace]) || aFunction$1(global_1[namespace]) : path[namespace] && path[namespace][method] || global_1[namespace] && global_1[namespace][method];
	};

	var hiddenKeys$1 = enumBugKeys.concat('length', 'prototype'); // `Object.getOwnPropertyNames` method
	// https://tc39.github.io/ecma262/#sec-object.getownpropertynames

	var f$4 = Object.getOwnPropertyNames || function getOwnPropertyNames(O) {
	  return objectKeysInternal(O, hiddenKeys$1);
	};

	var objectGetOwnPropertyNames = {
	  f: f$4
	};

	var ownKeys = getBuiltIn('Reflect', 'ownKeys') || function ownKeys(it) {
	  var keys = objectGetOwnPropertyNames.f(anObject(it));
	  var getOwnPropertySymbols = objectGetOwnPropertySymbols.f;
	  return getOwnPropertySymbols ? keys.concat(getOwnPropertySymbols(it)) : keys;
	};

	var createProperty = function (object, key, value) {
	  var propertyKey = toPrimitive(key);
	  if (propertyKey in object) objectDefineProperty.f(object, propertyKey, createPropertyDescriptor(0, value));else object[propertyKey] = value;
	};

	// https://tc39.github.io/ecma262/#sec-object.getownpropertydescriptors

	_export({
	  target: 'Object',
	  stat: true,
	  sham: !descriptors
	}, {
	  getOwnPropertyDescriptors: function getOwnPropertyDescriptors(object) {
	    var O = toIndexedObject(object);
	    var getOwnPropertyDescriptor = objectGetOwnPropertyDescriptor.f;
	    var keys = ownKeys(O);
	    var result = {};
	    var index = 0;
	    var key, descriptor;

	    while (keys.length > index) {
	      descriptor = getOwnPropertyDescriptor(O, key = keys[index++]);
	      if (descriptor !== undefined) createProperty(result, key, descriptor);
	    }

	    return result;
	  }
	});

	var getOwnPropertyDescriptors = path.Object.getOwnPropertyDescriptors;

	var getOwnPropertyDescriptors$1 = getOwnPropertyDescriptors;

	var getOwnPropertyDescriptors$2 = getOwnPropertyDescriptors$1;

	var nativeGetOwnPropertyDescriptor$1 = objectGetOwnPropertyDescriptor.f;
	var FAILS_ON_PRIMITIVES = fails(function () {
	  nativeGetOwnPropertyDescriptor$1(1);
	});
	var FORCED = !descriptors || FAILS_ON_PRIMITIVES; // `Object.getOwnPropertyDescriptor` method
	// https://tc39.github.io/ecma262/#sec-object.getownpropertydescriptor

	_export({
	  target: 'Object',
	  stat: true,
	  forced: FORCED,
	  sham: !descriptors
	}, {
	  getOwnPropertyDescriptor: function getOwnPropertyDescriptor(it, key) {
	    return nativeGetOwnPropertyDescriptor$1(toIndexedObject(it), key);
	  }
	});

	var getOwnPropertyDescriptor_1 = createCommonjsModule(function (module) {
	  var Object = path.Object;

	  var getOwnPropertyDescriptor = module.exports = function getOwnPropertyDescriptor(it, key) {
	    return Object.getOwnPropertyDescriptor(it, key);
	  };

	  if (Object.getOwnPropertyDescriptor.sham) getOwnPropertyDescriptor.sham = true;
	});

	var getOwnPropertyDescriptor$2 = getOwnPropertyDescriptor_1;

	var getOwnPropertyDescriptor$3 = getOwnPropertyDescriptor$2;

	var nativeSymbol = !!Object.getOwnPropertySymbols && !fails(function () {
	  // Chrome 38 Symbol has incorrect toString conversion
	  // eslint-disable-next-line no-undef
	  return !String(Symbol());
	});

	var useSymbolAsUid = nativeSymbol // eslint-disable-next-line no-undef
	&& !Symbol.sham // eslint-disable-next-line no-undef
	&& typeof Symbol.iterator == 'symbol';

	// https://tc39.github.io/ecma262/#sec-isarray

	var isArray = Array.isArray || function isArray(arg) {
	  return classofRaw(arg) == 'Array';
	};

	var html = getBuiltIn('document', 'documentElement');

	var setGlobal = function (key, value) {
	  try {
	    createNonEnumerableProperty(global_1, key, value);
	  } catch (error) {
	    global_1[key] = value;
	  }

	  return value;
	};

	var SHARED = '__core-js_shared__';
	var store = global_1[SHARED] || setGlobal(SHARED, {});
	var sharedStore = store;

	var shared = createCommonjsModule(function (module) {
	  (module.exports = function (key, value) {
	    return sharedStore[key] || (sharedStore[key] = value !== undefined ? value : {});
	  })('versions', []).push({
	    version: '3.6.4',
	    mode:  'pure' ,
	    copyright: '© 2020 Denis Pushkarev (zloirock.ru)'
	  });
	});

	var id = 0;
	var postfix = Math.random();

	var uid = function (key) {
	  return 'Symbol(' + String(key === undefined ? '' : key) + ')_' + (++id + postfix).toString(36);
	};

	var keys = shared('keys');

	var sharedKey = function (key) {
	  return keys[key] || (keys[key] = uid(key));
	};

	var GT = '>';
	var LT = '<';
	var PROTOTYPE = 'prototype';
	var SCRIPT = 'script';
	var IE_PROTO = sharedKey('IE_PROTO');

	var EmptyConstructor = function () {
	  /* empty */
	};

	var scriptTag = function (content) {
	  return LT + SCRIPT + GT + content + LT + '/' + SCRIPT + GT;
	}; // Create object with fake `null` prototype: use ActiveX Object with cleared prototype


	var NullProtoObjectViaActiveX = function (activeXDocument) {
	  activeXDocument.write(scriptTag(''));
	  activeXDocument.close();
	  var temp = activeXDocument.parentWindow.Object;
	  activeXDocument = null; // avoid memory leak

	  return temp;
	}; // Create object with fake `null` prototype: use iframe Object with cleared prototype


	var NullProtoObjectViaIFrame = function () {
	  // Thrash, waste and sodomy: IE GC bug
	  var iframe = documentCreateElement('iframe');
	  var JS = 'java' + SCRIPT + ':';
	  var iframeDocument;
	  iframe.style.display = 'none';
	  html.appendChild(iframe); // https://github.com/zloirock/core-js/issues/475

	  iframe.src = String(JS);
	  iframeDocument = iframe.contentWindow.document;
	  iframeDocument.open();
	  iframeDocument.write(scriptTag('document.F=Object'));
	  iframeDocument.close();
	  return iframeDocument.F;
	}; // Check for document.domain and active x support
	// No need to use active x approach when document.domain is not set
	// see https://github.com/es-shims/es5-shim/issues/150
	// variation of https://github.com/kitcambridge/es5-shim/commit/4f738ac066346
	// avoid IE GC bug


	var activeXDocument;

	var NullProtoObject = function () {
	  try {
	    /* global ActiveXObject */
	    activeXDocument = document.domain && new ActiveXObject('htmlfile');
	  } catch (error) {
	    /* ignore */
	  }

	  NullProtoObject = activeXDocument ? NullProtoObjectViaActiveX(activeXDocument) : NullProtoObjectViaIFrame();
	  var length = enumBugKeys.length;

	  while (length--) delete NullProtoObject[PROTOTYPE][enumBugKeys[length]];

	  return NullProtoObject();
	};

	hiddenKeys[IE_PROTO] = true; // `Object.create` method
	// https://tc39.github.io/ecma262/#sec-object.create

	var objectCreate = Object.create || function create(O, Properties) {
	  var result;

	  if (O !== null) {
	    EmptyConstructor[PROTOTYPE] = anObject(O);
	    result = new EmptyConstructor();
	    EmptyConstructor[PROTOTYPE] = null; // add "__proto__" for Object.getPrototypeOf polyfill

	    result[IE_PROTO] = O;
	  } else result = NullProtoObject();

	  return Properties === undefined ? result : objectDefineProperties(result, Properties);
	};

	var nativeGetOwnPropertyNames = objectGetOwnPropertyNames.f;
	var toString$1 = {}.toString;
	var windowNames = typeof window == 'object' && window && Object.getOwnPropertyNames ? Object.getOwnPropertyNames(window) : [];

	var getWindowNames = function (it) {
	  try {
	    return nativeGetOwnPropertyNames(it);
	  } catch (error) {
	    return windowNames.slice();
	  }
	}; // fallback for IE11 buggy Object.getOwnPropertyNames with iframe and window


	var f$5 = function getOwnPropertyNames(it) {
	  return windowNames && toString$1.call(it) == '[object Window]' ? getWindowNames(it) : nativeGetOwnPropertyNames(toIndexedObject(it));
	};

	var objectGetOwnPropertyNamesExternal = {
	  f: f$5
	};

	var redefine = function (target, key, value, options) {
	  if (options && options.enumerable) target[key] = value;else createNonEnumerableProperty(target, key, value);
	};

	var WellKnownSymbolsStore = shared('wks');
	var Symbol$1 = global_1.Symbol;
	var createWellKnownSymbol = useSymbolAsUid ? Symbol$1 : Symbol$1 && Symbol$1.withoutSetter || uid;

	var wellKnownSymbol = function (name) {
	  if (!has(WellKnownSymbolsStore, name)) {
	    if (nativeSymbol && has(Symbol$1, name)) WellKnownSymbolsStore[name] = Symbol$1[name];else WellKnownSymbolsStore[name] = createWellKnownSymbol('Symbol.' + name);
	  }

	  return WellKnownSymbolsStore[name];
	};

	var f$6 = wellKnownSymbol;
	var wellKnownSymbolWrapped = {
	  f: f$6
	};

	var defineProperty$3 = objectDefineProperty.f;

	var defineWellKnownSymbol = function (NAME) {
	  var Symbol = path.Symbol || (path.Symbol = {});
	  if (!has(Symbol, NAME)) defineProperty$3(Symbol, NAME, {
	    value: wellKnownSymbolWrapped.f(NAME)
	  });
	};

	var TO_STRING_TAG = wellKnownSymbol('toStringTag');
	var test = {};
	test[TO_STRING_TAG] = 'z';
	var toStringTagSupport = String(test) === '[object z]';

	var TO_STRING_TAG$1 = wellKnownSymbol('toStringTag'); // ES3 wrong here

	var CORRECT_ARGUMENTS = classofRaw(function () {
	  return arguments;
	}()) == 'Arguments'; // fallback for IE11 Script Access Denied error

	var tryGet = function (it, key) {
	  try {
	    return it[key];
	  } catch (error) {
	    /* empty */
	  }
	}; // getting tag from ES6+ `Object.prototype.toString`


	var classof = toStringTagSupport ? classofRaw : function (it) {
	  var O, tag, result;
	  return it === undefined ? 'Undefined' : it === null ? 'Null' // @@toStringTag case
	  : typeof (tag = tryGet(O = Object(it), TO_STRING_TAG$1)) == 'string' ? tag // builtinTag case
	  : CORRECT_ARGUMENTS ? classofRaw(O) // ES3 arguments fallback
	  : (result = classofRaw(O)) == 'Object' && typeof O.callee == 'function' ? 'Arguments' : result;
	};

	// https://tc39.github.io/ecma262/#sec-object.prototype.tostring


	var objectToString = toStringTagSupport ? {}.toString : function toString() {
	  return '[object ' + classof(this) + ']';
	};

	var defineProperty$4 = objectDefineProperty.f;
	var TO_STRING_TAG$2 = wellKnownSymbol('toStringTag');

	var setToStringTag = function (it, TAG, STATIC, SET_METHOD) {
	  if (it) {
	    var target = STATIC ? it : it.prototype;

	    if (!has(target, TO_STRING_TAG$2)) {
	      defineProperty$4(target, TO_STRING_TAG$2, {
	        configurable: true,
	        value: TAG
	      });
	    }

	    if (SET_METHOD && !toStringTagSupport) {
	      createNonEnumerableProperty(target, 'toString', objectToString);
	    }
	  }
	};

	var functionToString = Function.toString; // this helper broken in `3.4.1-3.4.4`, so we can't use `shared` helper

	if (typeof sharedStore.inspectSource != 'function') {
	  sharedStore.inspectSource = function (it) {
	    return functionToString.call(it);
	  };
	}

	var inspectSource = sharedStore.inspectSource;

	var WeakMap = global_1.WeakMap;
	var nativeWeakMap = typeof WeakMap === 'function' && /native code/.test(inspectSource(WeakMap));

	var WeakMap$1 = global_1.WeakMap;
	var set, get, has$1;

	var enforce = function (it) {
	  return has$1(it) ? get(it) : set(it, {});
	};

	var getterFor = function (TYPE) {
	  return function (it) {
	    var state;

	    if (!isObject(it) || (state = get(it)).type !== TYPE) {
	      throw TypeError('Incompatible receiver, ' + TYPE + ' required');
	    }

	    return state;
	  };
	};

	if (nativeWeakMap) {
	  var store$1 = new WeakMap$1();
	  var wmget = store$1.get;
	  var wmhas = store$1.has;
	  var wmset = store$1.set;

	  set = function (it, metadata) {
	    wmset.call(store$1, it, metadata);
	    return metadata;
	  };

	  get = function (it) {
	    return wmget.call(store$1, it) || {};
	  };

	  has$1 = function (it) {
	    return wmhas.call(store$1, it);
	  };
	} else {
	  var STATE = sharedKey('state');
	  hiddenKeys[STATE] = true;

	  set = function (it, metadata) {
	    createNonEnumerableProperty(it, STATE, metadata);
	    return metadata;
	  };

	  get = function (it) {
	    return has(it, STATE) ? it[STATE] : {};
	  };

	  has$1 = function (it) {
	    return has(it, STATE);
	  };
	}

	var internalState = {
	  set: set,
	  get: get,
	  has: has$1,
	  enforce: enforce,
	  getterFor: getterFor
	};

	var SPECIES = wellKnownSymbol('species'); // `ArraySpeciesCreate` abstract operation
	// https://tc39.github.io/ecma262/#sec-arrayspeciescreate

	var arraySpeciesCreate = function (originalArray, length) {
	  var C;

	  if (isArray(originalArray)) {
	    C = originalArray.constructor; // cross-realm fallback

	    if (typeof C == 'function' && (C === Array || isArray(C.prototype))) C = undefined;else if (isObject(C)) {
	      C = C[SPECIES];
	      if (C === null) C = undefined;
	    }
	  }

	  return new (C === undefined ? Array : C)(length === 0 ? 0 : length);
	};

	var push = [].push; // `Array.prototype.{ forEach, map, filter, some, every, find, findIndex }` methods implementation

	var createMethod$1 = function (TYPE) {
	  var IS_MAP = TYPE == 1;
	  var IS_FILTER = TYPE == 2;
	  var IS_SOME = TYPE == 3;
	  var IS_EVERY = TYPE == 4;
	  var IS_FIND_INDEX = TYPE == 6;
	  var NO_HOLES = TYPE == 5 || IS_FIND_INDEX;
	  return function ($this, callbackfn, that, specificCreate) {
	    var O = toObject($this);
	    var self = indexedObject(O);
	    var boundFunction = functionBindContext(callbackfn, that, 3);
	    var length = toLength(self.length);
	    var index = 0;
	    var create = specificCreate || arraySpeciesCreate;
	    var target = IS_MAP ? create($this, length) : IS_FILTER ? create($this, 0) : undefined;
	    var value, result;

	    for (; length > index; index++) if (NO_HOLES || index in self) {
	      value = self[index];
	      result = boundFunction(value, index, O);

	      if (TYPE) {
	        if (IS_MAP) target[index] = result; // map
	        else if (result) switch (TYPE) {
	            case 3:
	              return true;
	            // some

	            case 5:
	              return value;
	            // find

	            case 6:
	              return index;
	            // findIndex

	            case 2:
	              push.call(target, value);
	            // filter
	          } else if (IS_EVERY) return false; // every
	      }
	    }

	    return IS_FIND_INDEX ? -1 : IS_SOME || IS_EVERY ? IS_EVERY : target;
	  };
	};

	var arrayIteration = {
	  // `Array.prototype.forEach` method
	  // https://tc39.github.io/ecma262/#sec-array.prototype.foreach
	  forEach: createMethod$1(0),
	  // `Array.prototype.map` method
	  // https://tc39.github.io/ecma262/#sec-array.prototype.map
	  map: createMethod$1(1),
	  // `Array.prototype.filter` method
	  // https://tc39.github.io/ecma262/#sec-array.prototype.filter
	  filter: createMethod$1(2),
	  // `Array.prototype.some` method
	  // https://tc39.github.io/ecma262/#sec-array.prototype.some
	  some: createMethod$1(3),
	  // `Array.prototype.every` method
	  // https://tc39.github.io/ecma262/#sec-array.prototype.every
	  every: createMethod$1(4),
	  // `Array.prototype.find` method
	  // https://tc39.github.io/ecma262/#sec-array.prototype.find
	  find: createMethod$1(5),
	  // `Array.prototype.findIndex` method
	  // https://tc39.github.io/ecma262/#sec-array.prototype.findIndex
	  findIndex: createMethod$1(6)
	};

	var $forEach = arrayIteration.forEach;
	var HIDDEN = sharedKey('hidden');
	var SYMBOL = 'Symbol';
	var PROTOTYPE$1 = 'prototype';
	var TO_PRIMITIVE = wellKnownSymbol('toPrimitive');
	var setInternalState = internalState.set;
	var getInternalState = internalState.getterFor(SYMBOL);
	var ObjectPrototype = Object[PROTOTYPE$1];
	var $Symbol = global_1.Symbol;
	var $stringify = getBuiltIn('JSON', 'stringify');
	var nativeGetOwnPropertyDescriptor$2 = objectGetOwnPropertyDescriptor.f;
	var nativeDefineProperty$1 = objectDefineProperty.f;
	var nativeGetOwnPropertyNames$1 = objectGetOwnPropertyNamesExternal.f;
	var nativePropertyIsEnumerable$1 = objectPropertyIsEnumerable.f;
	var AllSymbols = shared('symbols');
	var ObjectPrototypeSymbols = shared('op-symbols');
	var StringToSymbolRegistry = shared('string-to-symbol-registry');
	var SymbolToStringRegistry = shared('symbol-to-string-registry');
	var WellKnownSymbolsStore$1 = shared('wks');
	var QObject = global_1.QObject; // Don't use setters in Qt Script, https://github.com/zloirock/core-js/issues/173

	var USE_SETTER = !QObject || !QObject[PROTOTYPE$1] || !QObject[PROTOTYPE$1].findChild; // fallback for old Android, https://code.google.com/p/v8/issues/detail?id=687

	var setSymbolDescriptor = descriptors && fails(function () {
	  return objectCreate(nativeDefineProperty$1({}, 'a', {
	    get: function () {
	      return nativeDefineProperty$1(this, 'a', {
	        value: 7
	      }).a;
	    }
	  })).a != 7;
	}) ? function (O, P, Attributes) {
	  var ObjectPrototypeDescriptor = nativeGetOwnPropertyDescriptor$2(ObjectPrototype, P);
	  if (ObjectPrototypeDescriptor) delete ObjectPrototype[P];
	  nativeDefineProperty$1(O, P, Attributes);

	  if (ObjectPrototypeDescriptor && O !== ObjectPrototype) {
	    nativeDefineProperty$1(ObjectPrototype, P, ObjectPrototypeDescriptor);
	  }
	} : nativeDefineProperty$1;

	var wrap = function (tag, description) {
	  var symbol = AllSymbols[tag] = objectCreate($Symbol[PROTOTYPE$1]);
	  setInternalState(symbol, {
	    type: SYMBOL,
	    tag: tag,
	    description: description
	  });
	  if (!descriptors) symbol.description = description;
	  return symbol;
	};

	var isSymbol = useSymbolAsUid ? function (it) {
	  return typeof it == 'symbol';
	} : function (it) {
	  return Object(it) instanceof $Symbol;
	};

	var $defineProperty = function defineProperty(O, P, Attributes) {
	  if (O === ObjectPrototype) $defineProperty(ObjectPrototypeSymbols, P, Attributes);
	  anObject(O);
	  var key = toPrimitive(P, true);
	  anObject(Attributes);

	  if (has(AllSymbols, key)) {
	    if (!Attributes.enumerable) {
	      if (!has(O, HIDDEN)) nativeDefineProperty$1(O, HIDDEN, createPropertyDescriptor(1, {}));
	      O[HIDDEN][key] = true;
	    } else {
	      if (has(O, HIDDEN) && O[HIDDEN][key]) O[HIDDEN][key] = false;
	      Attributes = objectCreate(Attributes, {
	        enumerable: createPropertyDescriptor(0, false)
	      });
	    }

	    return setSymbolDescriptor(O, key, Attributes);
	  }

	  return nativeDefineProperty$1(O, key, Attributes);
	};

	var $defineProperties = function defineProperties(O, Properties) {
	  anObject(O);
	  var properties = toIndexedObject(Properties);
	  var keys = objectKeys(properties).concat($getOwnPropertySymbols(properties));
	  $forEach(keys, function (key) {
	    if (!descriptors || $propertyIsEnumerable.call(properties, key)) $defineProperty(O, key, properties[key]);
	  });
	  return O;
	};

	var $create = function create(O, Properties) {
	  return Properties === undefined ? objectCreate(O) : $defineProperties(objectCreate(O), Properties);
	};

	var $propertyIsEnumerable = function propertyIsEnumerable(V) {
	  var P = toPrimitive(V, true);
	  var enumerable = nativePropertyIsEnumerable$1.call(this, P);
	  if (this === ObjectPrototype && has(AllSymbols, P) && !has(ObjectPrototypeSymbols, P)) return false;
	  return enumerable || !has(this, P) || !has(AllSymbols, P) || has(this, HIDDEN) && this[HIDDEN][P] ? enumerable : true;
	};

	var $getOwnPropertyDescriptor = function getOwnPropertyDescriptor(O, P) {
	  var it = toIndexedObject(O);
	  var key = toPrimitive(P, true);
	  if (it === ObjectPrototype && has(AllSymbols, key) && !has(ObjectPrototypeSymbols, key)) return;
	  var descriptor = nativeGetOwnPropertyDescriptor$2(it, key);

	  if (descriptor && has(AllSymbols, key) && !(has(it, HIDDEN) && it[HIDDEN][key])) {
	    descriptor.enumerable = true;
	  }

	  return descriptor;
	};

	var $getOwnPropertyNames = function getOwnPropertyNames(O) {
	  var names = nativeGetOwnPropertyNames$1(toIndexedObject(O));
	  var result = [];
	  $forEach(names, function (key) {
	    if (!has(AllSymbols, key) && !has(hiddenKeys, key)) result.push(key);
	  });
	  return result;
	};

	var $getOwnPropertySymbols = function getOwnPropertySymbols(O) {
	  var IS_OBJECT_PROTOTYPE = O === ObjectPrototype;
	  var names = nativeGetOwnPropertyNames$1(IS_OBJECT_PROTOTYPE ? ObjectPrototypeSymbols : toIndexedObject(O));
	  var result = [];
	  $forEach(names, function (key) {
	    if (has(AllSymbols, key) && (!IS_OBJECT_PROTOTYPE || has(ObjectPrototype, key))) {
	      result.push(AllSymbols[key]);
	    }
	  });
	  return result;
	}; // `Symbol` constructor
	// https://tc39.github.io/ecma262/#sec-symbol-constructor


	if (!nativeSymbol) {
	  $Symbol = function Symbol() {
	    if (this instanceof $Symbol) throw TypeError('Symbol is not a constructor');
	    var description = !arguments.length || arguments[0] === undefined ? undefined : String(arguments[0]);
	    var tag = uid(description);

	    var setter = function (value) {
	      if (this === ObjectPrototype) setter.call(ObjectPrototypeSymbols, value);
	      if (has(this, HIDDEN) && has(this[HIDDEN], tag)) this[HIDDEN][tag] = false;
	      setSymbolDescriptor(this, tag, createPropertyDescriptor(1, value));
	    };

	    if (descriptors && USE_SETTER) setSymbolDescriptor(ObjectPrototype, tag, {
	      configurable: true,
	      set: setter
	    });
	    return wrap(tag, description);
	  };

	  redefine($Symbol[PROTOTYPE$1], 'toString', function toString() {
	    return getInternalState(this).tag;
	  });
	  redefine($Symbol, 'withoutSetter', function (description) {
	    return wrap(uid(description), description);
	  });
	  objectPropertyIsEnumerable.f = $propertyIsEnumerable;
	  objectDefineProperty.f = $defineProperty;
	  objectGetOwnPropertyDescriptor.f = $getOwnPropertyDescriptor;
	  objectGetOwnPropertyNames.f = objectGetOwnPropertyNamesExternal.f = $getOwnPropertyNames;
	  objectGetOwnPropertySymbols.f = $getOwnPropertySymbols;

	  wellKnownSymbolWrapped.f = function (name) {
	    return wrap(wellKnownSymbol(name), name);
	  };

	  if (descriptors) {
	    // https://github.com/tc39/proposal-Symbol-description
	    nativeDefineProperty$1($Symbol[PROTOTYPE$1], 'description', {
	      configurable: true,
	      get: function description() {
	        return getInternalState(this).description;
	      }
	    });
	  }
	}

	_export({
	  global: true,
	  wrap: true,
	  forced: !nativeSymbol,
	  sham: !nativeSymbol
	}, {
	  Symbol: $Symbol
	});
	$forEach(objectKeys(WellKnownSymbolsStore$1), function (name) {
	  defineWellKnownSymbol(name);
	});
	_export({
	  target: SYMBOL,
	  stat: true,
	  forced: !nativeSymbol
	}, {
	  // `Symbol.for` method
	  // https://tc39.github.io/ecma262/#sec-symbol.for
	  'for': function (key) {
	    var string = String(key);
	    if (has(StringToSymbolRegistry, string)) return StringToSymbolRegistry[string];
	    var symbol = $Symbol(string);
	    StringToSymbolRegistry[string] = symbol;
	    SymbolToStringRegistry[symbol] = string;
	    return symbol;
	  },
	  // `Symbol.keyFor` method
	  // https://tc39.github.io/ecma262/#sec-symbol.keyfor
	  keyFor: function keyFor(sym) {
	    if (!isSymbol(sym)) throw TypeError(sym + ' is not a symbol');
	    if (has(SymbolToStringRegistry, sym)) return SymbolToStringRegistry[sym];
	  },
	  useSetter: function () {
	    USE_SETTER = true;
	  },
	  useSimple: function () {
	    USE_SETTER = false;
	  }
	});
	_export({
	  target: 'Object',
	  stat: true,
	  forced: !nativeSymbol,
	  sham: !descriptors
	}, {
	  // `Object.create` method
	  // https://tc39.github.io/ecma262/#sec-object.create
	  create: $create,
	  // `Object.defineProperty` method
	  // https://tc39.github.io/ecma262/#sec-object.defineproperty
	  defineProperty: $defineProperty,
	  // `Object.defineProperties` method
	  // https://tc39.github.io/ecma262/#sec-object.defineproperties
	  defineProperties: $defineProperties,
	  // `Object.getOwnPropertyDescriptor` method
	  // https://tc39.github.io/ecma262/#sec-object.getownpropertydescriptors
	  getOwnPropertyDescriptor: $getOwnPropertyDescriptor
	});
	_export({
	  target: 'Object',
	  stat: true,
	  forced: !nativeSymbol
	}, {
	  // `Object.getOwnPropertyNames` method
	  // https://tc39.github.io/ecma262/#sec-object.getownpropertynames
	  getOwnPropertyNames: $getOwnPropertyNames,
	  // `Object.getOwnPropertySymbols` method
	  // https://tc39.github.io/ecma262/#sec-object.getownpropertysymbols
	  getOwnPropertySymbols: $getOwnPropertySymbols
	}); // Chrome 38 and 39 `Object.getOwnPropertySymbols` fails on primitives
	// https://bugs.chromium.org/p/v8/issues/detail?id=3443

	_export({
	  target: 'Object',
	  stat: true,
	  forced: fails(function () {
	    objectGetOwnPropertySymbols.f(1);
	  })
	}, {
	  getOwnPropertySymbols: function getOwnPropertySymbols(it) {
	    return objectGetOwnPropertySymbols.f(toObject(it));
	  }
	}); // `JSON.stringify` method behavior with symbols
	// https://tc39.github.io/ecma262/#sec-json.stringify

	if ($stringify) {
	  var FORCED_JSON_STRINGIFY = !nativeSymbol || fails(function () {
	    var symbol = $Symbol(); // MS Edge converts symbol values to JSON as {}

	    return $stringify([symbol]) != '[null]' // WebKit converts symbol values to JSON as null
	    || $stringify({
	      a: symbol
	    }) != '{}' // V8 throws on boxed symbols
	    || $stringify(Object(symbol)) != '{}';
	  });
	  _export({
	    target: 'JSON',
	    stat: true,
	    forced: FORCED_JSON_STRINGIFY
	  }, {
	    // eslint-disable-next-line no-unused-vars
	    stringify: function stringify(it, replacer, space) {
	      var args = [it];
	      var index = 1;
	      var $replacer;

	      while (arguments.length > index) args.push(arguments[index++]);

	      $replacer = replacer;
	      if (!isObject(replacer) && it === undefined || isSymbol(it)) return; // IE8 returns string on undefined

	      if (!isArray(replacer)) replacer = function (key, value) {
	        if (typeof $replacer == 'function') value = $replacer.call(this, key, value);
	        if (!isSymbol(value)) return value;
	      };
	      args[1] = replacer;
	      return $stringify.apply(null, args);
	    }
	  });
	} // `Symbol.prototype[@@toPrimitive]` method
	// https://tc39.github.io/ecma262/#sec-symbol.prototype-@@toprimitive


	if (!$Symbol[PROTOTYPE$1][TO_PRIMITIVE]) {
	  createNonEnumerableProperty($Symbol[PROTOTYPE$1], TO_PRIMITIVE, $Symbol[PROTOTYPE$1].valueOf);
	} // `Symbol.prototype[@@toStringTag]` property
	// https://tc39.github.io/ecma262/#sec-symbol.prototype-@@tostringtag


	setToStringTag($Symbol, SYMBOL);
	hiddenKeys[HIDDEN] = true;

	var getOwnPropertySymbols = path.Object.getOwnPropertySymbols;

	var getOwnPropertySymbols$1 = getOwnPropertySymbols;

	var getOwnPropertySymbols$2 = getOwnPropertySymbols$1;

	var slice = [].slice;
	var factories = {};

	var construct = function (C, argsLength, args) {
	  if (!(argsLength in factories)) {
	    for (var list = [], i = 0; i < argsLength; i++) list[i] = 'a[' + i + ']'; // eslint-disable-next-line no-new-func


	    factories[argsLength] = Function('C,a', 'return new C(' + list.join(',') + ')');
	  }

	  return factories[argsLength](C, args);
	}; // `Function.prototype.bind` method implementation
	// https://tc39.github.io/ecma262/#sec-function.prototype.bind


	var functionBind = Function.bind || function bind(that
	/* , ...args */
	) {
	  var fn = aFunction(this);
	  var partArgs = slice.call(arguments, 1);

	  var boundFunction = function bound()
	  /* args... */
	  {
	    var args = partArgs.concat(slice.call(arguments));
	    return this instanceof boundFunction ? construct(fn, args.length, args) : fn.apply(that, args);
	  };

	  if (isObject(fn.prototype)) boundFunction.prototype = fn.prototype;
	  return boundFunction;
	};

	var nativeConstruct = getBuiltIn('Reflect', 'construct'); // `Reflect.construct` method
	// https://tc39.github.io/ecma262/#sec-reflect.construct
	// MS Edge supports only 2 arguments and argumentsList argument is optional
	// FF Nightly sets third argument as `new.target`, but does not create `this` from it

	var NEW_TARGET_BUG = fails(function () {
	  function F() {
	    /* empty */
	  }

	  return !(nativeConstruct(function () {
	    /* empty */
	  }, [], F) instanceof F);
	});
	var ARGS_BUG = !fails(function () {
	  nativeConstruct(function () {
	    /* empty */
	  });
	});
	var FORCED$1 = NEW_TARGET_BUG || ARGS_BUG;
	_export({
	  target: 'Reflect',
	  stat: true,
	  forced: FORCED$1,
	  sham: FORCED$1
	}, {
	  construct: function construct(Target, args
	  /* , newTarget */
	  ) {
	    aFunction(Target);
	    anObject(args);
	    var newTarget = arguments.length < 3 ? Target : aFunction(arguments[2]);
	    if (ARGS_BUG && !NEW_TARGET_BUG) return nativeConstruct(Target, args, newTarget);

	    if (Target == newTarget) {
	      // w/o altered newTarget, optimization for 0-4 arguments
	      switch (args.length) {
	        case 0:
	          return new Target();

	        case 1:
	          return new Target(args[0]);

	        case 2:
	          return new Target(args[0], args[1]);

	        case 3:
	          return new Target(args[0], args[1], args[2]);

	        case 4:
	          return new Target(args[0], args[1], args[2], args[3]);
	      } // w/o altered newTarget, lot of arguments case


	      var $args = [null];
	      $args.push.apply($args, args);
	      return new (functionBind.apply(Target, $args))();
	    } // with altered newTarget, not support built-in constructors


	    var proto = newTarget.prototype;
	    var instance = objectCreate(isObject(proto) ? proto : Object.prototype);
	    var result = Function.apply.call(Target, instance, args);
	    return isObject(result) ? result : instance;
	  }
	});

	var construct$1 = path.Reflect.construct;

	var construct$2 = construct$1;

	var construct$3 = construct$2;

	var iterators = {};

	var correctPrototypeGetter = !fails(function () {
	  function F() {
	    /* empty */
	  }

	  F.prototype.constructor = null;
	  return Object.getPrototypeOf(new F()) !== F.prototype;
	});

	var IE_PROTO$1 = sharedKey('IE_PROTO');
	var ObjectPrototype$1 = Object.prototype; // `Object.getPrototypeOf` method
	// https://tc39.github.io/ecma262/#sec-object.getprototypeof

	var objectGetPrototypeOf = correctPrototypeGetter ? Object.getPrototypeOf : function (O) {
	  O = toObject(O);
	  if (has(O, IE_PROTO$1)) return O[IE_PROTO$1];

	  if (typeof O.constructor == 'function' && O instanceof O.constructor) {
	    return O.constructor.prototype;
	  }

	  return O instanceof Object ? ObjectPrototype$1 : null;
	};

	var ITERATOR = wellKnownSymbol('iterator');
	var BUGGY_SAFARI_ITERATORS = false;
	// https://tc39.github.io/ecma262/#sec-%iteratorprototype%-object


	var IteratorPrototype, PrototypeOfArrayIteratorPrototype, arrayIterator;

	if ([].keys) {
	  arrayIterator = [].keys(); // Safari 8 has buggy iterators w/o `next`

	  if (!('next' in arrayIterator)) BUGGY_SAFARI_ITERATORS = true;else {
	    PrototypeOfArrayIteratorPrototype = objectGetPrototypeOf(objectGetPrototypeOf(arrayIterator));
	    if (PrototypeOfArrayIteratorPrototype !== Object.prototype) IteratorPrototype = PrototypeOfArrayIteratorPrototype;
	  }
	}

	if (IteratorPrototype == undefined) IteratorPrototype = {}; // 25.1.2.1.1 %IteratorPrototype%[@@iterator]()

	var iteratorsCore = {
	  IteratorPrototype: IteratorPrototype,
	  BUGGY_SAFARI_ITERATORS: BUGGY_SAFARI_ITERATORS
	};

	var IteratorPrototype$1 = iteratorsCore.IteratorPrototype;

	var returnThis = function () {
	  return this;
	};

	var createIteratorConstructor = function (IteratorConstructor, NAME, next) {
	  var TO_STRING_TAG = NAME + ' Iterator';
	  IteratorConstructor.prototype = objectCreate(IteratorPrototype$1, {
	    next: createPropertyDescriptor(1, next)
	  });
	  setToStringTag(IteratorConstructor, TO_STRING_TAG, false, true);
	  iterators[TO_STRING_TAG] = returnThis;
	  return IteratorConstructor;
	};

	var aPossiblePrototype = function (it) {
	  if (!isObject(it) && it !== null) {
	    throw TypeError("Can't set " + String(it) + ' as a prototype');
	  }

	  return it;
	};

	// https://tc39.github.io/ecma262/#sec-object.setprototypeof
	// Works with __proto__ only. Old v8 can't work with null proto objects.

	/* eslint-disable no-proto */

	var objectSetPrototypeOf = Object.setPrototypeOf || ('__proto__' in {} ? function () {
	  var CORRECT_SETTER = false;
	  var test = {};
	  var setter;

	  try {
	    setter = Object.getOwnPropertyDescriptor(Object.prototype, '__proto__').set;
	    setter.call(test, []);
	    CORRECT_SETTER = test instanceof Array;
	  } catch (error) {
	    /* empty */
	  }

	  return function setPrototypeOf(O, proto) {
	    anObject(O);
	    aPossiblePrototype(proto);
	    if (CORRECT_SETTER) setter.call(O, proto);else O.__proto__ = proto;
	    return O;
	  };
	}() : undefined);

	var IteratorPrototype$2 = iteratorsCore.IteratorPrototype;
	var BUGGY_SAFARI_ITERATORS$1 = iteratorsCore.BUGGY_SAFARI_ITERATORS;
	var ITERATOR$1 = wellKnownSymbol('iterator');
	var KEYS = 'keys';
	var VALUES = 'values';
	var ENTRIES = 'entries';

	var returnThis$1 = function () {
	  return this;
	};

	var defineIterator = function (Iterable, NAME, IteratorConstructor, next, DEFAULT, IS_SET, FORCED) {
	  createIteratorConstructor(IteratorConstructor, NAME, next);

	  var getIterationMethod = function (KIND) {
	    if (KIND === DEFAULT && defaultIterator) return defaultIterator;
	    if (!BUGGY_SAFARI_ITERATORS$1 && KIND in IterablePrototype) return IterablePrototype[KIND];

	    switch (KIND) {
	      case KEYS:
	        return function keys() {
	          return new IteratorConstructor(this, KIND);
	        };

	      case VALUES:
	        return function values() {
	          return new IteratorConstructor(this, KIND);
	        };

	      case ENTRIES:
	        return function entries() {
	          return new IteratorConstructor(this, KIND);
	        };
	    }

	    return function () {
	      return new IteratorConstructor(this);
	    };
	  };

	  var TO_STRING_TAG = NAME + ' Iterator';
	  var INCORRECT_VALUES_NAME = false;
	  var IterablePrototype = Iterable.prototype;
	  var nativeIterator = IterablePrototype[ITERATOR$1] || IterablePrototype['@@iterator'] || DEFAULT && IterablePrototype[DEFAULT];
	  var defaultIterator = !BUGGY_SAFARI_ITERATORS$1 && nativeIterator || getIterationMethod(DEFAULT);
	  var anyNativeIterator = NAME == 'Array' ? IterablePrototype.entries || nativeIterator : nativeIterator;
	  var CurrentIteratorPrototype, methods, KEY; // fix native

	  if (anyNativeIterator) {
	    CurrentIteratorPrototype = objectGetPrototypeOf(anyNativeIterator.call(new Iterable()));

	    if (IteratorPrototype$2 !== Object.prototype && CurrentIteratorPrototype.next) {


	      setToStringTag(CurrentIteratorPrototype, TO_STRING_TAG, true, true);
	      iterators[TO_STRING_TAG] = returnThis$1;
	    }
	  } // fix Array#{values, @@iterator}.name in V8 / FF


	  if (DEFAULT == VALUES && nativeIterator && nativeIterator.name !== VALUES) {
	    INCORRECT_VALUES_NAME = true;

	    defaultIterator = function values() {
	      return nativeIterator.call(this);
	    };
	  } // define iterator


	  if (( FORCED) && IterablePrototype[ITERATOR$1] !== defaultIterator) {
	    createNonEnumerableProperty(IterablePrototype, ITERATOR$1, defaultIterator);
	  }

	  iterators[NAME] = defaultIterator; // export additional methods

	  if (DEFAULT) {
	    methods = {
	      values: getIterationMethod(VALUES),
	      keys: IS_SET ? defaultIterator : getIterationMethod(KEYS),
	      entries: getIterationMethod(ENTRIES)
	    };
	    if (FORCED) for (KEY in methods) {
	      if (BUGGY_SAFARI_ITERATORS$1 || INCORRECT_VALUES_NAME || !(KEY in IterablePrototype)) {
	        redefine(IterablePrototype, KEY, methods[KEY]);
	      }
	    } else _export({
	      target: NAME,
	      proto: true,
	      forced: BUGGY_SAFARI_ITERATORS$1 || INCORRECT_VALUES_NAME
	    }, methods);
	  }

	  return methods;
	};

	var ARRAY_ITERATOR = 'Array Iterator';
	var setInternalState$1 = internalState.set;
	var getInternalState$1 = internalState.getterFor(ARRAY_ITERATOR); // `Array.prototype.entries` method
	// https://tc39.github.io/ecma262/#sec-array.prototype.entries
	// `Array.prototype.keys` method
	// https://tc39.github.io/ecma262/#sec-array.prototype.keys
	// `Array.prototype.values` method
	// https://tc39.github.io/ecma262/#sec-array.prototype.values
	// `Array.prototype[@@iterator]` method
	// https://tc39.github.io/ecma262/#sec-array.prototype-@@iterator
	// `CreateArrayIterator` internal method
	// https://tc39.github.io/ecma262/#sec-createarrayiterator

	var es_array_iterator = defineIterator(Array, 'Array', function (iterated, kind) {
	  setInternalState$1(this, {
	    type: ARRAY_ITERATOR,
	    target: toIndexedObject(iterated),
	    // target
	    index: 0,
	    // next index
	    kind: kind // kind

	  }); // `%ArrayIteratorPrototype%.next` method
	  // https://tc39.github.io/ecma262/#sec-%arrayiteratorprototype%.next
	}, function () {
	  var state = getInternalState$1(this);
	  var target = state.target;
	  var kind = state.kind;
	  var index = state.index++;

	  if (!target || index >= target.length) {
	    state.target = undefined;
	    return {
	      value: undefined,
	      done: true
	    };
	  }

	  if (kind == 'keys') return {
	    value: index,
	    done: false
	  };
	  if (kind == 'values') return {
	    value: target[index],
	    done: false
	  };
	  return {
	    value: [index, target[index]],
	    done: false
	  };
	}, 'values'); // argumentsList[@@iterator] is %ArrayProto_values%
	// https://tc39.github.io/ecma262/#sec-createunmappedargumentsobject
	// https://tc39.github.io/ecma262/#sec-createmappedargumentsobject

	iterators.Arguments = iterators.Array; // https://tc39.github.io/ecma262/#sec-array.prototype-@@unscopables

	// iterable DOM collections
	// flag - `iterable` interface - 'entries', 'keys', 'values', 'forEach' methods
	var domIterables = {
	  CSSRuleList: 0,
	  CSSStyleDeclaration: 0,
	  CSSValueList: 0,
	  ClientRectList: 0,
	  DOMRectList: 0,
	  DOMStringList: 0,
	  DOMTokenList: 1,
	  DataTransferItemList: 0,
	  FileList: 0,
	  HTMLAllCollection: 0,
	  HTMLCollection: 0,
	  HTMLFormElement: 0,
	  HTMLSelectElement: 0,
	  MediaList: 0,
	  MimeTypeArray: 0,
	  NamedNodeMap: 0,
	  NodeList: 1,
	  PaintRequestList: 0,
	  Plugin: 0,
	  PluginArray: 0,
	  SVGLengthList: 0,
	  SVGNumberList: 0,
	  SVGPathSegList: 0,
	  SVGPointList: 0,
	  SVGStringList: 0,
	  SVGTransformList: 0,
	  SourceBufferList: 0,
	  StyleSheetList: 0,
	  TextTrackCueList: 0,
	  TextTrackList: 0,
	  TouchList: 0
	};

	var TO_STRING_TAG$3 = wellKnownSymbol('toStringTag');

	for (var COLLECTION_NAME in domIterables) {
	  var Collection = global_1[COLLECTION_NAME];
	  var CollectionPrototype = Collection && Collection.prototype;

	  if (CollectionPrototype && classof(CollectionPrototype) !== TO_STRING_TAG$3) {
	    createNonEnumerableProperty(CollectionPrototype, TO_STRING_TAG$3, COLLECTION_NAME);
	  }

	  iterators[COLLECTION_NAME] = iterators.Array;
	}

	var entryVirtual = function (CONSTRUCTOR) {
	  return path[CONSTRUCTOR + 'Prototype'];
	};

	var entries = entryVirtual('Array').entries;

	var entries$1 = entries;

	var ArrayPrototype = Array.prototype;
	var DOMIterables = {
	  DOMTokenList: true,
	  NodeList: true
	};

	var entries_1 = function (it) {
	  var own = it.entries;
	  return it === ArrayPrototype || it instanceof Array && own === ArrayPrototype.entries // eslint-disable-next-line no-prototype-builtins
	  || DOMIterables.hasOwnProperty(classof(it)) ? entries$1 : own;
	};

	var entries$2 = entries_1;

	var $stringify$1 = getBuiltIn('JSON', 'stringify');
	var re = /[\uD800-\uDFFF]/g;
	var low = /^[\uD800-\uDBFF]$/;
	var hi = /^[\uDC00-\uDFFF]$/;

	var fix = function (match, offset, string) {
	  var prev = string.charAt(offset - 1);
	  var next = string.charAt(offset + 1);

	  if (low.test(match) && !hi.test(next) || hi.test(match) && !low.test(prev)) {
	    return '\\u' + match.charCodeAt(0).toString(16);
	  }

	  return match;
	};

	var FORCED$2 = fails(function () {
	  return $stringify$1('\uDF06\uD834') !== '"\\udf06\\ud834"' || $stringify$1('\uDEAD') !== '"\\udead"';
	});

	if ($stringify$1) {
	  // https://github.com/tc39/proposal-well-formed-stringify
	  _export({
	    target: 'JSON',
	    stat: true,
	    forced: FORCED$2
	  }, {
	    // eslint-disable-next-line no-unused-vars
	    stringify: function stringify(it, replacer, space) {
	      var result = $stringify$1.apply(null, arguments);
	      return typeof result == 'string' ? result.replace(re, fix) : result;
	    }
	  });
	}

	if (!path.JSON) path.JSON = {
	  stringify: JSON.stringify
	}; // eslint-disable-next-line no-unused-vars

	var stringify = function stringify(it, replacer, space) {
	  return path.JSON.stringify.apply(null, arguments);
	};

	var stringify$1 = stringify;

	var stringify$2 = stringify$1;

	var values = entryVirtual('Array').values;

	var values$1 = values;

	var ArrayPrototype$1 = Array.prototype;
	var DOMIterables$1 = {
	  DOMTokenList: true,
	  NodeList: true
	};

	var values_1 = function (it) {
	  var own = it.values;
	  return it === ArrayPrototype$1 || it instanceof Array && own === ArrayPrototype$1.values // eslint-disable-next-line no-prototype-builtins
	  || DOMIterables$1.hasOwnProperty(classof(it)) ? values$1 : own;
	};

	var values$2 = values_1;

	var keys$1 = entryVirtual('Array').keys;

	var keys$2 = keys$1;

	var ArrayPrototype$2 = Array.prototype;
	var DOMIterables$2 = {
	  DOMTokenList: true,
	  NodeList: true
	};

	var keys_1 = function (it) {
	  var own = it.keys;
	  return it === ArrayPrototype$2 || it instanceof Array && own === ArrayPrototype$2.keys // eslint-disable-next-line no-prototype-builtins
	  || DOMIterables$2.hasOwnProperty(classof(it)) ? keys$2 : own;
	};

	var keys$3 = keys_1;

	var arrayMethodIsStrict = function (METHOD_NAME, argument) {
	  var method = [][METHOD_NAME];
	  return !!method && fails(function () {
	    // eslint-disable-next-line no-useless-call,no-throw-literal
	    method.call(null, argument || function () {
	      throw 1;
	    }, 1);
	  });
	};

	var defineProperty$5 = Object.defineProperty;
	var cache = {};

	var thrower = function (it) {
	  throw it;
	};

	var arrayMethodUsesToLength = function (METHOD_NAME, options) {
	  if (has(cache, METHOD_NAME)) return cache[METHOD_NAME];
	  if (!options) options = {};
	  var method = [][METHOD_NAME];
	  var ACCESSORS = has(options, 'ACCESSORS') ? options.ACCESSORS : false;
	  var argument0 = has(options, 0) ? options[0] : thrower;
	  var argument1 = has(options, 1) ? options[1] : undefined;
	  return cache[METHOD_NAME] = !!method && !fails(function () {
	    if (ACCESSORS && !descriptors) return true;
	    var O = {
	      length: -1
	    };
	    if (ACCESSORS) defineProperty$5(O, 1, {
	      enumerable: true,
	      get: thrower
	    });else O[1] = 1;
	    method.call(O, argument0, argument1);
	  });
	};

	var $some = arrayIteration.some;
	var STRICT_METHOD = arrayMethodIsStrict('some');
	var USES_TO_LENGTH = arrayMethodUsesToLength('some'); // `Array.prototype.some` method
	// https://tc39.github.io/ecma262/#sec-array.prototype.some

	_export({
	  target: 'Array',
	  proto: true,
	  forced: !STRICT_METHOD || !USES_TO_LENGTH
	}, {
	  some: function some(callbackfn
	  /* , thisArg */
	  ) {
	    return $some(this, callbackfn, arguments.length > 1 ? arguments[1] : undefined);
	  }
	});

	var some = entryVirtual('Array').some;

	var ArrayPrototype$3 = Array.prototype;

	var some_1 = function (it) {
	  var own = it.some;
	  return it === ArrayPrototype$3 || it instanceof Array && own === ArrayPrototype$3.some ? some : own;
	};

	var some$1 = some_1;

	var some$2 = some$1;

	// https://tc39.github.io/ecma262/#sec-symbol.iterator

	defineWellKnownSymbol('iterator');

	var createMethod$2 = function (CONVERT_TO_STRING) {
	  return function ($this, pos) {
	    var S = String(requireObjectCoercible($this));
	    var position = toInteger(pos);
	    var size = S.length;
	    var first, second;
	    if (position < 0 || position >= size) return CONVERT_TO_STRING ? '' : undefined;
	    first = S.charCodeAt(position);
	    return first < 0xD800 || first > 0xDBFF || position + 1 === size || (second = S.charCodeAt(position + 1)) < 0xDC00 || second > 0xDFFF ? CONVERT_TO_STRING ? S.charAt(position) : first : CONVERT_TO_STRING ? S.slice(position, position + 2) : (first - 0xD800 << 10) + (second - 0xDC00) + 0x10000;
	  };
	};

	var stringMultibyte = {
	  // `String.prototype.codePointAt` method
	  // https://tc39.github.io/ecma262/#sec-string.prototype.codepointat
	  codeAt: createMethod$2(false),
	  // `String.prototype.at` method
	  // https://github.com/mathiasbynens/String.prototype.at
	  charAt: createMethod$2(true)
	};

	var charAt = stringMultibyte.charAt;
	var STRING_ITERATOR = 'String Iterator';
	var setInternalState$2 = internalState.set;
	var getInternalState$2 = internalState.getterFor(STRING_ITERATOR); // `String.prototype[@@iterator]` method
	// https://tc39.github.io/ecma262/#sec-string.prototype-@@iterator

	defineIterator(String, 'String', function (iterated) {
	  setInternalState$2(this, {
	    type: STRING_ITERATOR,
	    string: String(iterated),
	    index: 0
	  }); // `%StringIteratorPrototype%.next` method
	  // https://tc39.github.io/ecma262/#sec-%stringiteratorprototype%.next
	}, function next() {
	  var state = getInternalState$2(this);
	  var string = state.string;
	  var index = state.index;
	  var point;
	  if (index >= string.length) return {
	    value: undefined,
	    done: true
	  };
	  point = charAt(string, index);
	  state.index += point.length;
	  return {
	    value: point,
	    done: false
	  };
	});

	var iterator = wellKnownSymbolWrapped.f('iterator');

	var iterator$1 = iterator;

	var iterator$2 = iterator$1;

	var engineUserAgent = getBuiltIn('navigator', 'userAgent') || '';

	var process = global_1.process;
	var versions = process && process.versions;
	var v8 = versions && versions.v8;
	var match, version;

	if (v8) {
	  match = v8.split('.');
	  version = match[0] + match[1];
	} else if (engineUserAgent) {
	  match = engineUserAgent.match(/Edge\/(\d+)/);

	  if (!match || match[1] >= 74) {
	    match = engineUserAgent.match(/Chrome\/(\d+)/);
	    if (match) version = match[1];
	  }
	}

	var engineV8Version = version && +version;

	var SPECIES$1 = wellKnownSymbol('species');

	var arrayMethodHasSpeciesSupport = function (METHOD_NAME) {
	  // We can't use this feature detection in V8 since it causes
	  // deoptimization and serious performance degradation
	  // https://github.com/zloirock/core-js/issues/677
	  return engineV8Version >= 51 || !fails(function () {
	    var array = [];
	    var constructor = array.constructor = {};

	    constructor[SPECIES$1] = function () {
	      return {
	        foo: 1
	      };
	    };

	    return array[METHOD_NAME](Boolean).foo !== 1;
	  });
	};

	var IS_CONCAT_SPREADABLE = wellKnownSymbol('isConcatSpreadable');
	var MAX_SAFE_INTEGER = 0x1FFFFFFFFFFFFF;
	var MAXIMUM_ALLOWED_INDEX_EXCEEDED = 'Maximum allowed index exceeded'; // We can't use this feature detection in V8 since it causes
	// deoptimization and serious performance degradation
	// https://github.com/zloirock/core-js/issues/679

	var IS_CONCAT_SPREADABLE_SUPPORT = engineV8Version >= 51 || !fails(function () {
	  var array = [];
	  array[IS_CONCAT_SPREADABLE] = false;
	  return array.concat()[0] !== array;
	});
	var SPECIES_SUPPORT = arrayMethodHasSpeciesSupport('concat');

	var isConcatSpreadable = function (O) {
	  if (!isObject(O)) return false;
	  var spreadable = O[IS_CONCAT_SPREADABLE];
	  return spreadable !== undefined ? !!spreadable : isArray(O);
	};

	var FORCED$3 = !IS_CONCAT_SPREADABLE_SUPPORT || !SPECIES_SUPPORT; // `Array.prototype.concat` method
	// https://tc39.github.io/ecma262/#sec-array.prototype.concat
	// with adding support of @@isConcatSpreadable and @@species

	_export({
	  target: 'Array',
	  proto: true,
	  forced: FORCED$3
	}, {
	  concat: function concat(arg) {
	    // eslint-disable-line no-unused-vars
	    var O = toObject(this);
	    var A = arraySpeciesCreate(O, 0);
	    var n = 0;
	    var i, k, length, len, E;

	    for (i = -1, length = arguments.length; i < length; i++) {
	      E = i === -1 ? O : arguments[i];

	      if (isConcatSpreadable(E)) {
	        len = toLength(E.length);
	        if (n + len > MAX_SAFE_INTEGER) throw TypeError(MAXIMUM_ALLOWED_INDEX_EXCEEDED);

	        for (k = 0; k < len; k++, n++) if (k in E) createProperty(A, n, E[k]);
	      } else {
	        if (n >= MAX_SAFE_INTEGER) throw TypeError(MAXIMUM_ALLOWED_INDEX_EXCEEDED);
	        createProperty(A, n++, E);
	      }
	    }

	    A.length = n;
	    return A;
	  }
	});

	// empty

	var es_object_toString = /*#__PURE__*/Object.freeze({
		__proto__: null
	});

	// https://tc39.github.io/ecma262/#sec-symbol.asynciterator

	defineWellKnownSymbol('asyncIterator');

	// empty

	var es_symbol_description = /*#__PURE__*/Object.freeze({
		__proto__: null
	});

	// https://tc39.github.io/ecma262/#sec-symbol.hasinstance

	defineWellKnownSymbol('hasInstance');

	// https://tc39.github.io/ecma262/#sec-symbol.isconcatspreadable

	defineWellKnownSymbol('isConcatSpreadable');

	// https://tc39.github.io/ecma262/#sec-symbol.match

	defineWellKnownSymbol('match');

	defineWellKnownSymbol('matchAll');

	// https://tc39.github.io/ecma262/#sec-symbol.replace

	defineWellKnownSymbol('replace');

	// https://tc39.github.io/ecma262/#sec-symbol.search

	defineWellKnownSymbol('search');

	// https://tc39.github.io/ecma262/#sec-symbol.species

	defineWellKnownSymbol('species');

	// https://tc39.github.io/ecma262/#sec-symbol.split

	defineWellKnownSymbol('split');

	// https://tc39.github.io/ecma262/#sec-symbol.toprimitive

	defineWellKnownSymbol('toPrimitive');

	// https://tc39.github.io/ecma262/#sec-symbol.tostringtag

	defineWellKnownSymbol('toStringTag');

	// https://tc39.github.io/ecma262/#sec-symbol.unscopables

	defineWellKnownSymbol('unscopables');

	// https://tc39.github.io/ecma262/#sec-math-@@tostringtag

	setToStringTag(Math, 'Math', true);

	// https://tc39.github.io/ecma262/#sec-json-@@tostringtag

	setToStringTag(global_1.JSON, 'JSON', true);

	getCjsExportFromNamespace(es_object_toString);

	getCjsExportFromNamespace(es_symbol_description);

	var symbol = path.Symbol;

	// https://github.com/tc39/proposal-using-statement

	defineWellKnownSymbol('asyncDispose');

	// https://github.com/tc39/proposal-using-statement

	defineWellKnownSymbol('dispose');

	// https://github.com/tc39/proposal-observable

	defineWellKnownSymbol('observable');

	// https://github.com/tc39/proposal-pattern-matching

	defineWellKnownSymbol('patternMatch');

	defineWellKnownSymbol('replaceAll');

	var symbol$1 = symbol;

	var symbol$2 = symbol$1;

	var _typeof_1 = createCommonjsModule(function (module) {
	  function _typeof(obj) {
	    "@babel/helpers - typeof";

	    if (typeof symbol$2 === "function" && typeof iterator$2 === "symbol") {
	      module.exports = _typeof = function _typeof(obj) {
	        return typeof obj;
	      };
	    } else {
	      module.exports = _typeof = function _typeof(obj) {
	        return obj && typeof symbol$2 === "function" && obj.constructor === symbol$2 && obj !== symbol$2.prototype ? "symbol" : typeof obj;
	      };
	    }

	    return _typeof(obj);
	  }

	  module.exports = _typeof;
	});

	var FAILS_ON_PRIMITIVES$1 = fails(function () {
	  objectKeys(1);
	}); // `Object.keys` method
	// https://tc39.github.io/ecma262/#sec-object.keys

	_export({
	  target: 'Object',
	  stat: true,
	  forced: FAILS_ON_PRIMITIVES$1
	}, {
	  keys: function keys(it) {
	    return objectKeys(toObject(it));
	  }
	});

	var keys$4 = path.Object.keys;

	var keys$5 = keys$4;

	var keys$6 = keys$5;

	// https://tc39.github.io/ecma262/#sec-object.create

	_export({
	  target: 'Object',
	  stat: true,
	  sham: !descriptors
	}, {
	  create: objectCreate
	});

	var Object$1 = path.Object;

	var create = function create(P, D) {
	  return Object$1.create(P, D);
	};

	var create$1 = create;

	var create$2 = create$1;

	// https://tc39.github.io/ecma262/#sec-object.setprototypeof

	_export({
	  target: 'Object',
	  stat: true
	}, {
	  setPrototypeOf: objectSetPrototypeOf
	});

	var setPrototypeOf = path.Object.setPrototypeOf;

	var setPrototypeOf$1 = setPrototypeOf;

	var setPrototypeOf$2 = setPrototypeOf$1;

	var setPrototypeOf$3 = createCommonjsModule(function (module) {
	  function _setPrototypeOf(o, p) {
	    module.exports = _setPrototypeOf = setPrototypeOf$2 || function _setPrototypeOf(o, p) {
	      o.__proto__ = p;
	      return o;
	    };

	    return _setPrototypeOf(o, p);
	  }

	  module.exports = _setPrototypeOf;
	});

	function _inherits(subClass, superClass) {
	  if (typeof superClass !== "function" && superClass !== null) {
	    throw new TypeError("Super expression must either be null or a function");
	  }

	  subClass.prototype = create$2(superClass && superClass.prototype, {
	    constructor: {
	      value: subClass,
	      writable: true,
	      configurable: true
	    }
	  });
	  if (superClass) setPrototypeOf$3(subClass, superClass);
	}

	var inherits = _inherits;

	function _possibleConstructorReturn(self, call) {
	  if (call && (_typeof_1(call) === "object" || typeof call === "function")) {
	    return call;
	  }

	  return assertThisInitialized(self);
	}

	var possibleConstructorReturn = _possibleConstructorReturn;

	var FAILS_ON_PRIMITIVES$2 = fails(function () {
	  objectGetPrototypeOf(1);
	}); // `Object.getPrototypeOf` method
	// https://tc39.github.io/ecma262/#sec-object.getprototypeof

	_export({
	  target: 'Object',
	  stat: true,
	  forced: FAILS_ON_PRIMITIVES$2,
	  sham: !correctPrototypeGetter
	}, {
	  getPrototypeOf: function getPrototypeOf(it) {
	    return objectGetPrototypeOf(toObject(it));
	  }
	});

	var getPrototypeOf = path.Object.getPrototypeOf;

	var getPrototypeOf$1 = getPrototypeOf;

	var getPrototypeOf$2 = getPrototypeOf$1;

	var getPrototypeOf$3 = createCommonjsModule(function (module) {
	  function _getPrototypeOf(o) {
	    module.exports = _getPrototypeOf = setPrototypeOf$2 ? getPrototypeOf$2 : function _getPrototypeOf(o) {
	      return o.__proto__ || getPrototypeOf$2(o);
	    };
	    return _getPrototypeOf(o);
	  }

	  module.exports = _getPrototypeOf;
	});

	// https://tc39.github.io/ecma262/#sec-array.isarray

	_export({
	  target: 'Array',
	  stat: true
	}, {
	  isArray: isArray
	});

	var isArray$1 = path.Array.isArray;

	var isArray$2 = isArray$1;

	var isArray$3 = isArray$2;

	var ITERATOR$2 = wellKnownSymbol('iterator');

	var getIteratorMethod = function (it) {
	  if (it != undefined) return it[ITERATOR$2] || it['@@iterator'] || iterators[classof(it)];
	};

	var getIteratorMethod_1 = getIteratorMethod;

	var getIteratorMethod$1 = getIteratorMethod_1;

	var symbol$3 = symbol;

	var symbol$4 = symbol$3;

	var callWithSafeIterationClosing = function (iterator, fn, value, ENTRIES) {
	  try {
	    return ENTRIES ? fn(anObject(value)[0], value[1]) : fn(value); // 7.4.6 IteratorClose(iterator, completion)
	  } catch (error) {
	    var returnMethod = iterator['return'];
	    if (returnMethod !== undefined) anObject(returnMethod.call(iterator));
	    throw error;
	  }
	};

	var ITERATOR$3 = wellKnownSymbol('iterator');
	var ArrayPrototype$4 = Array.prototype; // check on default Array iterator

	var isArrayIteratorMethod = function (it) {
	  return it !== undefined && (iterators.Array === it || ArrayPrototype$4[ITERATOR$3] === it);
	};

	// https://tc39.github.io/ecma262/#sec-array.from


	var arrayFrom = function from(arrayLike
	/* , mapfn = undefined, thisArg = undefined */
	) {
	  var O = toObject(arrayLike);
	  var C = typeof this == 'function' ? this : Array;
	  var argumentsLength = arguments.length;
	  var mapfn = argumentsLength > 1 ? arguments[1] : undefined;
	  var mapping = mapfn !== undefined;
	  var iteratorMethod = getIteratorMethod(O);
	  var index = 0;
	  var length, result, step, iterator, next, value;
	  if (mapping) mapfn = functionBindContext(mapfn, argumentsLength > 2 ? arguments[2] : undefined, 2); // if the target is not iterable or it's an array with the default iterator - use a simple case

	  if (iteratorMethod != undefined && !(C == Array && isArrayIteratorMethod(iteratorMethod))) {
	    iterator = iteratorMethod.call(O);
	    next = iterator.next;
	    result = new C();

	    for (; !(step = next.call(iterator)).done; index++) {
	      value = mapping ? callWithSafeIterationClosing(iterator, mapfn, [step.value, index], true) : step.value;
	      createProperty(result, index, value);
	    }
	  } else {
	    length = toLength(O.length);
	    result = new C(length);

	    for (; length > index; index++) {
	      value = mapping ? mapfn(O[index], index) : O[index];
	      createProperty(result, index, value);
	    }
	  }

	  result.length = index;
	  return result;
	};

	var ITERATOR$4 = wellKnownSymbol('iterator');
	var SAFE_CLOSING = false;

	try {
	  var called = 0;
	  var iteratorWithReturn = {
	    next: function () {
	      return {
	        done: !!called++
	      };
	    },
	    'return': function () {
	      SAFE_CLOSING = true;
	    }
	  };

	  iteratorWithReturn[ITERATOR$4] = function () {
	    return this;
	  }; // eslint-disable-next-line no-throw-literal


	  Array.from(iteratorWithReturn, function () {
	    throw 2;
	  });
	} catch (error) {
	  /* empty */
	}

	var checkCorrectnessOfIteration = function (exec, SKIP_CLOSING) {
	  if (!SKIP_CLOSING && !SAFE_CLOSING) return false;
	  var ITERATION_SUPPORT = false;

	  try {
	    var object = {};

	    object[ITERATOR$4] = function () {
	      return {
	        next: function () {
	          return {
	            done: ITERATION_SUPPORT = true
	          };
	        }
	      };
	    };

	    exec(object);
	  } catch (error) {
	    /* empty */
	  }

	  return ITERATION_SUPPORT;
	};

	var INCORRECT_ITERATION = !checkCorrectnessOfIteration(function (iterable) {
	  Array.from(iterable);
	}); // `Array.from` method
	// https://tc39.github.io/ecma262/#sec-array.from

	_export({
	  target: 'Array',
	  stat: true,
	  forced: INCORRECT_ITERATION
	}, {
	  from: arrayFrom
	});

	var from_1 = path.Array.from;

	var from_1$1 = from_1;

	var from_1$2 = from_1$1;

	var HAS_SPECIES_SUPPORT = arrayMethodHasSpeciesSupport('slice');
	var USES_TO_LENGTH$1 = arrayMethodUsesToLength('slice', {
	  ACCESSORS: true,
	  0: 0,
	  1: 2
	});
	var SPECIES$2 = wellKnownSymbol('species');
	var nativeSlice = [].slice;
	var max$1 = Math.max; // `Array.prototype.slice` method
	// https://tc39.github.io/ecma262/#sec-array.prototype.slice
	// fallback for not array-like ES3 strings and DOM objects

	_export({
	  target: 'Array',
	  proto: true,
	  forced: !HAS_SPECIES_SUPPORT || !USES_TO_LENGTH$1
	}, {
	  slice: function slice(start, end) {
	    var O = toIndexedObject(this);
	    var length = toLength(O.length);
	    var k = toAbsoluteIndex(start, length);
	    var fin = toAbsoluteIndex(end === undefined ? length : end, length); // inline `ArraySpeciesCreate` for usage native `Array#slice` where it's possible

	    var Constructor, result, n;

	    if (isArray(O)) {
	      Constructor = O.constructor; // cross-realm fallback

	      if (typeof Constructor == 'function' && (Constructor === Array || isArray(Constructor.prototype))) {
	        Constructor = undefined;
	      } else if (isObject(Constructor)) {
	        Constructor = Constructor[SPECIES$2];
	        if (Constructor === null) Constructor = undefined;
	      }

	      if (Constructor === Array || Constructor === undefined) {
	        return nativeSlice.call(O, k, fin);
	      }
	    }

	    result = new (Constructor === undefined ? Array : Constructor)(max$1(fin - k, 0));

	    for (n = 0; k < fin; k++, n++) if (k in O) createProperty(result, n, O[k]);

	    result.length = n;
	    return result;
	  }
	});

	var slice$1 = entryVirtual('Array').slice;

	var ArrayPrototype$5 = Array.prototype;

	var slice_1 = function (it) {
	  var own = it.slice;
	  return it === ArrayPrototype$5 || it instanceof Array && own === ArrayPrototype$5.slice ? slice$1 : own;
	};

	var slice$2 = slice_1;

	var slice$3 = slice$2;

	var test$1 = [];
	var nativeSort = test$1.sort; // IE8-

	var FAILS_ON_UNDEFINED = fails(function () {
	  test$1.sort(undefined);
	}); // V8 bug

	var FAILS_ON_NULL = fails(function () {
	  test$1.sort(null);
	}); // Old WebKit

	var STRICT_METHOD$1 = arrayMethodIsStrict('sort');
	var FORCED$4 = FAILS_ON_UNDEFINED || !FAILS_ON_NULL || !STRICT_METHOD$1; // `Array.prototype.sort` method
	// https://tc39.github.io/ecma262/#sec-array.prototype.sort

	_export({
	  target: 'Array',
	  proto: true,
	  forced: FORCED$4
	}, {
	  sort: function sort(comparefn) {
	    return comparefn === undefined ? nativeSort.call(toObject(this)) : nativeSort.call(toObject(this), aFunction(comparefn));
	  }
	});

	var sort = entryVirtual('Array').sort;

	var ArrayPrototype$6 = Array.prototype;

	var sort_1 = function (it) {
	  var own = it.sort;
	  return it === ArrayPrototype$6 || it instanceof Array && own === ArrayPrototype$6.sort ? sort : own;
	};

	var sort$1 = sort_1;

	var sort$2 = sort$1;

	var getIterator = function (it) {
	  var iteratorMethod = getIteratorMethod(it);

	  if (typeof iteratorMethod != 'function') {
	    throw TypeError(String(it) + ' is not iterable');
	  }

	  return anObject(iteratorMethod.call(it));
	};

	var getIterator_1 = getIterator;

	var getIterator$1 = getIterator_1;

	var defineProperty$6 = defineProperty_1;

	var defineProperty$7 = defineProperty$6;

	function _defineProperty(obj, key, value) {
	  if (key in obj) {
	    defineProperty$7(obj, key, {
	      value: value,
	      enumerable: true,
	      configurable: true,
	      writable: true
	    });
	  } else {
	    obj[key] = value;
	  }

	  return obj;
	}

	var defineProperty$8 = _defineProperty;

	var freezing = !fails(function () {
	  return Object.isExtensible(Object.preventExtensions({}));
	});

	var internalMetadata = createCommonjsModule(function (module) {
	  var defineProperty = objectDefineProperty.f;
	  var METADATA = uid('meta');
	  var id = 0;

	  var isExtensible = Object.isExtensible || function () {
	    return true;
	  };

	  var setMetadata = function (it) {
	    defineProperty(it, METADATA, {
	      value: {
	        objectID: 'O' + ++id,
	        // object ID
	        weakData: {} // weak collections IDs

	      }
	    });
	  };

	  var fastKey = function (it, create) {
	    // return a primitive with prefix
	    if (!isObject(it)) return typeof it == 'symbol' ? it : (typeof it == 'string' ? 'S' : 'P') + it;

	    if (!has(it, METADATA)) {
	      // can't set metadata to uncaught frozen object
	      if (!isExtensible(it)) return 'F'; // not necessary to add metadata

	      if (!create) return 'E'; // add missing metadata

	      setMetadata(it); // return object ID
	    }

	    return it[METADATA].objectID;
	  };

	  var getWeakData = function (it, create) {
	    if (!has(it, METADATA)) {
	      // can't set metadata to uncaught frozen object
	      if (!isExtensible(it)) return true; // not necessary to add metadata

	      if (!create) return false; // add missing metadata

	      setMetadata(it); // return the store of weak collections IDs
	    }

	    return it[METADATA].weakData;
	  }; // add metadata on freeze-family methods calling


	  var onFreeze = function (it) {
	    if (freezing && meta.REQUIRED && isExtensible(it) && !has(it, METADATA)) setMetadata(it);
	    return it;
	  };

	  var meta = module.exports = {
	    REQUIRED: false,
	    fastKey: fastKey,
	    getWeakData: getWeakData,
	    onFreeze: onFreeze
	  };
	  hiddenKeys[METADATA] = true;
	});
	var internalMetadata_1 = internalMetadata.REQUIRED;
	var internalMetadata_2 = internalMetadata.fastKey;
	var internalMetadata_3 = internalMetadata.getWeakData;
	var internalMetadata_4 = internalMetadata.onFreeze;

	var iterate_1 = createCommonjsModule(function (module) {
	  var Result = function (stopped, result) {
	    this.stopped = stopped;
	    this.result = result;
	  };

	  var iterate = module.exports = function (iterable, fn, that, AS_ENTRIES, IS_ITERATOR) {
	    var boundFunction = functionBindContext(fn, that, AS_ENTRIES ? 2 : 1);
	    var iterator, iterFn, index, length, result, next, step;

	    if (IS_ITERATOR) {
	      iterator = iterable;
	    } else {
	      iterFn = getIteratorMethod(iterable);
	      if (typeof iterFn != 'function') throw TypeError('Target is not iterable'); // optimisation for array iterators

	      if (isArrayIteratorMethod(iterFn)) {
	        for (index = 0, length = toLength(iterable.length); length > index; index++) {
	          result = AS_ENTRIES ? boundFunction(anObject(step = iterable[index])[0], step[1]) : boundFunction(iterable[index]);
	          if (result && result instanceof Result) return result;
	        }

	        return new Result(false);
	      }

	      iterator = iterFn.call(iterable);
	    }

	    next = iterator.next;

	    while (!(step = next.call(iterator)).done) {
	      result = callWithSafeIterationClosing(iterator, boundFunction, step.value, AS_ENTRIES);
	      if (typeof result == 'object' && result && result instanceof Result) return result;
	    }

	    return new Result(false);
	  };

	  iterate.stop = function (result) {
	    return new Result(true, result);
	  };
	});

	var anInstance = function (it, Constructor, name) {
	  if (!(it instanceof Constructor)) {
	    throw TypeError('Incorrect ' + (name ? name + ' ' : '') + 'invocation');
	  }

	  return it;
	};

	var defineProperty$9 = objectDefineProperty.f;
	var forEach = arrayIteration.forEach;
	var setInternalState$3 = internalState.set;
	var internalStateGetterFor = internalState.getterFor;

	var collection = function (CONSTRUCTOR_NAME, wrapper, common) {
	  var IS_MAP = CONSTRUCTOR_NAME.indexOf('Map') !== -1;
	  var IS_WEAK = CONSTRUCTOR_NAME.indexOf('Weak') !== -1;
	  var ADDER = IS_MAP ? 'set' : 'add';
	  var NativeConstructor = global_1[CONSTRUCTOR_NAME];
	  var NativePrototype = NativeConstructor && NativeConstructor.prototype;
	  var exported = {};
	  var Constructor;

	  if (!descriptors || typeof NativeConstructor != 'function' || !(IS_WEAK || NativePrototype.forEach && !fails(function () {
	    new NativeConstructor().entries().next();
	  }))) {
	    // create collection constructor
	    Constructor = common.getConstructor(wrapper, CONSTRUCTOR_NAME, IS_MAP, ADDER);
	    internalMetadata.REQUIRED = true;
	  } else {
	    Constructor = wrapper(function (target, iterable) {
	      setInternalState$3(anInstance(target, Constructor, CONSTRUCTOR_NAME), {
	        type: CONSTRUCTOR_NAME,
	        collection: new NativeConstructor()
	      });
	      if (iterable != undefined) iterate_1(iterable, target[ADDER], target, IS_MAP);
	    });
	    var getInternalState = internalStateGetterFor(CONSTRUCTOR_NAME);
	    forEach(['add', 'clear', 'delete', 'forEach', 'get', 'has', 'set', 'keys', 'values', 'entries'], function (KEY) {
	      var IS_ADDER = KEY == 'add' || KEY == 'set';

	      if (KEY in NativePrototype && !(IS_WEAK && KEY == 'clear')) {
	        createNonEnumerableProperty(Constructor.prototype, KEY, function (a, b) {
	          var collection = getInternalState(this).collection;
	          if (!IS_ADDER && IS_WEAK && !isObject(a)) return KEY == 'get' ? undefined : false;
	          var result = collection[KEY](a === 0 ? 0 : a, b);
	          return IS_ADDER ? this : result;
	        });
	      }
	    });
	    IS_WEAK || defineProperty$9(Constructor.prototype, 'size', {
	      configurable: true,
	      get: function () {
	        return getInternalState(this).collection.size;
	      }
	    });
	  }

	  setToStringTag(Constructor, CONSTRUCTOR_NAME, false, true);
	  exported[CONSTRUCTOR_NAME] = Constructor;
	  _export({
	    global: true,
	    forced: true
	  }, exported);
	  if (!IS_WEAK) common.setStrong(Constructor, CONSTRUCTOR_NAME, IS_MAP);
	  return Constructor;
	};

	var redefineAll = function (target, src, options) {
	  for (var key in src) {
	    if (options && options.unsafe && target[key]) target[key] = src[key];else redefine(target, key, src[key], options);
	  }

	  return target;
	};

	var SPECIES$3 = wellKnownSymbol('species');

	var setSpecies = function (CONSTRUCTOR_NAME) {
	  var Constructor = getBuiltIn(CONSTRUCTOR_NAME);
	  var defineProperty = objectDefineProperty.f;

	  if (descriptors && Constructor && !Constructor[SPECIES$3]) {
	    defineProperty(Constructor, SPECIES$3, {
	      configurable: true,
	      get: function () {
	        return this;
	      }
	    });
	  }
	};

	var defineProperty$a = objectDefineProperty.f;
	var fastKey = internalMetadata.fastKey;
	var setInternalState$4 = internalState.set;
	var internalStateGetterFor$1 = internalState.getterFor;
	var collectionStrong = {
	  getConstructor: function (wrapper, CONSTRUCTOR_NAME, IS_MAP, ADDER) {
	    var C = wrapper(function (that, iterable) {
	      anInstance(that, C, CONSTRUCTOR_NAME);
	      setInternalState$4(that, {
	        type: CONSTRUCTOR_NAME,
	        index: objectCreate(null),
	        first: undefined,
	        last: undefined,
	        size: 0
	      });
	      if (!descriptors) that.size = 0;
	      if (iterable != undefined) iterate_1(iterable, that[ADDER], that, IS_MAP);
	    });
	    var getInternalState = internalStateGetterFor$1(CONSTRUCTOR_NAME);

	    var define = function (that, key, value) {
	      var state = getInternalState(that);
	      var entry = getEntry(that, key);
	      var previous, index; // change existing entry

	      if (entry) {
	        entry.value = value; // create new entry
	      } else {
	        state.last = entry = {
	          index: index = fastKey(key, true),
	          key: key,
	          value: value,
	          previous: previous = state.last,
	          next: undefined,
	          removed: false
	        };
	        if (!state.first) state.first = entry;
	        if (previous) previous.next = entry;
	        if (descriptors) state.size++;else that.size++; // add to index

	        if (index !== 'F') state.index[index] = entry;
	      }

	      return that;
	    };

	    var getEntry = function (that, key) {
	      var state = getInternalState(that); // fast case

	      var index = fastKey(key);
	      var entry;
	      if (index !== 'F') return state.index[index]; // frozen object case

	      for (entry = state.first; entry; entry = entry.next) {
	        if (entry.key == key) return entry;
	      }
	    };

	    redefineAll(C.prototype, {
	      // 23.1.3.1 Map.prototype.clear()
	      // 23.2.3.2 Set.prototype.clear()
	      clear: function clear() {
	        var that = this;
	        var state = getInternalState(that);
	        var data = state.index;
	        var entry = state.first;

	        while (entry) {
	          entry.removed = true;
	          if (entry.previous) entry.previous = entry.previous.next = undefined;
	          delete data[entry.index];
	          entry = entry.next;
	        }

	        state.first = state.last = undefined;
	        if (descriptors) state.size = 0;else that.size = 0;
	      },
	      // 23.1.3.3 Map.prototype.delete(key)
	      // 23.2.3.4 Set.prototype.delete(value)
	      'delete': function (key) {
	        var that = this;
	        var state = getInternalState(that);
	        var entry = getEntry(that, key);

	        if (entry) {
	          var next = entry.next;
	          var prev = entry.previous;
	          delete state.index[entry.index];
	          entry.removed = true;
	          if (prev) prev.next = next;
	          if (next) next.previous = prev;
	          if (state.first == entry) state.first = next;
	          if (state.last == entry) state.last = prev;
	          if (descriptors) state.size--;else that.size--;
	        }

	        return !!entry;
	      },
	      // 23.2.3.6 Set.prototype.forEach(callbackfn, thisArg = undefined)
	      // 23.1.3.5 Map.prototype.forEach(callbackfn, thisArg = undefined)
	      forEach: function forEach(callbackfn
	      /* , that = undefined */
	      ) {
	        var state = getInternalState(this);
	        var boundFunction = functionBindContext(callbackfn, arguments.length > 1 ? arguments[1] : undefined, 3);
	        var entry;

	        while (entry = entry ? entry.next : state.first) {
	          boundFunction(entry.value, entry.key, this); // revert to the last existing entry

	          while (entry && entry.removed) entry = entry.previous;
	        }
	      },
	      // 23.1.3.7 Map.prototype.has(key)
	      // 23.2.3.7 Set.prototype.has(value)
	      has: function has(key) {
	        return !!getEntry(this, key);
	      }
	    });
	    redefineAll(C.prototype, IS_MAP ? {
	      // 23.1.3.6 Map.prototype.get(key)
	      get: function get(key) {
	        var entry = getEntry(this, key);
	        return entry && entry.value;
	      },
	      // 23.1.3.9 Map.prototype.set(key, value)
	      set: function set(key, value) {
	        return define(this, key === 0 ? 0 : key, value);
	      }
	    } : {
	      // 23.2.3.1 Set.prototype.add(value)
	      add: function add(value) {
	        return define(this, value = value === 0 ? 0 : value, value);
	      }
	    });
	    if (descriptors) defineProperty$a(C.prototype, 'size', {
	      get: function () {
	        return getInternalState(this).size;
	      }
	    });
	    return C;
	  },
	  setStrong: function (C, CONSTRUCTOR_NAME, IS_MAP) {
	    var ITERATOR_NAME = CONSTRUCTOR_NAME + ' Iterator';
	    var getInternalCollectionState = internalStateGetterFor$1(CONSTRUCTOR_NAME);
	    var getInternalIteratorState = internalStateGetterFor$1(ITERATOR_NAME); // add .keys, .values, .entries, [@@iterator]
	    // 23.1.3.4, 23.1.3.8, 23.1.3.11, 23.1.3.12, 23.2.3.5, 23.2.3.8, 23.2.3.10, 23.2.3.11

	    defineIterator(C, CONSTRUCTOR_NAME, function (iterated, kind) {
	      setInternalState$4(this, {
	        type: ITERATOR_NAME,
	        target: iterated,
	        state: getInternalCollectionState(iterated),
	        kind: kind,
	        last: undefined
	      });
	    }, function () {
	      var state = getInternalIteratorState(this);
	      var kind = state.kind;
	      var entry = state.last; // revert to the last existing entry

	      while (entry && entry.removed) entry = entry.previous; // get next entry


	      if (!state.target || !(state.last = entry = entry ? entry.next : state.state.first)) {
	        // or finish the iteration
	        state.target = undefined;
	        return {
	          value: undefined,
	          done: true
	        };
	      } // return step by kind


	      if (kind == 'keys') return {
	        value: entry.key,
	        done: false
	      };
	      if (kind == 'values') return {
	        value: entry.value,
	        done: false
	      };
	      return {
	        value: [entry.key, entry.value],
	        done: false
	      };
	    }, IS_MAP ? 'entries' : 'values', !IS_MAP, true); // add [@@species], 23.1.2.2, 23.2.2.2

	    setSpecies(CONSTRUCTOR_NAME);
	  }
	};

	// https://tc39.github.io/ecma262/#sec-set-objects


	var es_set = collection('Set', function (init) {
	  return function Set() {
	    return init(this, arguments.length ? arguments[0] : undefined);
	  };
	}, collectionStrong);

	var set$1 = path.Set;

	var set$2 = set$1;

	var set$3 = set$2;

	// https://tc39.github.io/ecma262/#sec-map-objects


	var es_map = collection('Map', function (init) {
	  return function Map() {
	    return init(this, arguments.length ? arguments[0] : undefined);
	  };
	}, collectionStrong);

	var map = path.Map;

	var map$1 = map;

	var map$2 = map$1;

	var create$3 = create;

	var create$4 = create$3;

	var runtime_1 = createCommonjsModule(function (module) {
	  /**
	   * Copyright (c) 2014-present, Facebook, Inc.
	   *
	   * This source code is licensed under the MIT license found in the
	   * LICENSE file in the root directory of this source tree.
	   */
	  var runtime = function (exports) {

	    var Op = Object.prototype;
	    var hasOwn = Op.hasOwnProperty;
	    var undefined$1; // More compressible than void 0.

	    var $Symbol = typeof Symbol === "function" ? Symbol : {};
	    var iteratorSymbol = $Symbol.iterator || "@@iterator";
	    var asyncIteratorSymbol = $Symbol.asyncIterator || "@@asyncIterator";
	    var toStringTagSymbol = $Symbol.toStringTag || "@@toStringTag";

	    function wrap(innerFn, outerFn, self, tryLocsList) {
	      // If outerFn provided and outerFn.prototype is a Generator, then outerFn.prototype instanceof Generator.
	      var protoGenerator = outerFn && outerFn.prototype instanceof Generator ? outerFn : Generator;
	      var generator = Object.create(protoGenerator.prototype);
	      var context = new Context(tryLocsList || []); // The ._invoke method unifies the implementations of the .next,
	      // .throw, and .return methods.

	      generator._invoke = makeInvokeMethod(innerFn, self, context);
	      return generator;
	    }

	    exports.wrap = wrap; // Try/catch helper to minimize deoptimizations. Returns a completion
	    // record like context.tryEntries[i].completion. This interface could
	    // have been (and was previously) designed to take a closure to be
	    // invoked without arguments, but in all the cases we care about we
	    // already have an existing method we want to call, so there's no need
	    // to create a new function object. We can even get away with assuming
	    // the method takes exactly one argument, since that happens to be true
	    // in every case, so we don't have to touch the arguments object. The
	    // only additional allocation required is the completion record, which
	    // has a stable shape and so hopefully should be cheap to allocate.

	    function tryCatch(fn, obj, arg) {
	      try {
	        return {
	          type: "normal",
	          arg: fn.call(obj, arg)
	        };
	      } catch (err) {
	        return {
	          type: "throw",
	          arg: err
	        };
	      }
	    }

	    var GenStateSuspendedStart = "suspendedStart";
	    var GenStateSuspendedYield = "suspendedYield";
	    var GenStateExecuting = "executing";
	    var GenStateCompleted = "completed"; // Returning this object from the innerFn has the same effect as
	    // breaking out of the dispatch switch statement.

	    var ContinueSentinel = {}; // Dummy constructor functions that we use as the .constructor and
	    // .constructor.prototype properties for functions that return Generator
	    // objects. For full spec compliance, you may wish to configure your
	    // minifier not to mangle the names of these two functions.

	    function Generator() {}

	    function GeneratorFunction() {}

	    function GeneratorFunctionPrototype() {} // This is a polyfill for %IteratorPrototype% for environments that
	    // don't natively support it.


	    var IteratorPrototype = {};

	    IteratorPrototype[iteratorSymbol] = function () {
	      return this;
	    };

	    var getProto = Object.getPrototypeOf;
	    var NativeIteratorPrototype = getProto && getProto(getProto(values([])));

	    if (NativeIteratorPrototype && NativeIteratorPrototype !== Op && hasOwn.call(NativeIteratorPrototype, iteratorSymbol)) {
	      // This environment has a native %IteratorPrototype%; use it instead
	      // of the polyfill.
	      IteratorPrototype = NativeIteratorPrototype;
	    }

	    var Gp = GeneratorFunctionPrototype.prototype = Generator.prototype = Object.create(IteratorPrototype);
	    GeneratorFunction.prototype = Gp.constructor = GeneratorFunctionPrototype;
	    GeneratorFunctionPrototype.constructor = GeneratorFunction;
	    GeneratorFunctionPrototype[toStringTagSymbol] = GeneratorFunction.displayName = "GeneratorFunction"; // Helper for defining the .next, .throw, and .return methods of the
	    // Iterator interface in terms of a single ._invoke method.

	    function defineIteratorMethods(prototype) {
	      ["next", "throw", "return"].forEach(function (method) {
	        prototype[method] = function (arg) {
	          return this._invoke(method, arg);
	        };
	      });
	    }

	    exports.isGeneratorFunction = function (genFun) {
	      var ctor = typeof genFun === "function" && genFun.constructor;
	      return ctor ? ctor === GeneratorFunction || // For the native GeneratorFunction constructor, the best we can
	      // do is to check its .name property.
	      (ctor.displayName || ctor.name) === "GeneratorFunction" : false;
	    };

	    exports.mark = function (genFun) {
	      if (Object.setPrototypeOf) {
	        Object.setPrototypeOf(genFun, GeneratorFunctionPrototype);
	      } else {
	        genFun.__proto__ = GeneratorFunctionPrototype;

	        if (!(toStringTagSymbol in genFun)) {
	          genFun[toStringTagSymbol] = "GeneratorFunction";
	        }
	      }

	      genFun.prototype = Object.create(Gp);
	      return genFun;
	    }; // Within the body of any async function, `await x` is transformed to
	    // `yield regeneratorRuntime.awrap(x)`, so that the runtime can test
	    // `hasOwn.call(value, "__await")` to determine if the yielded value is
	    // meant to be awaited.


	    exports.awrap = function (arg) {
	      return {
	        __await: arg
	      };
	    };

	    function AsyncIterator(generator, PromiseImpl) {
	      function invoke(method, arg, resolve, reject) {
	        var record = tryCatch(generator[method], generator, arg);

	        if (record.type === "throw") {
	          reject(record.arg);
	        } else {
	          var result = record.arg;
	          var value = result.value;

	          if (value && typeof value === "object" && hasOwn.call(value, "__await")) {
	            return PromiseImpl.resolve(value.__await).then(function (value) {
	              invoke("next", value, resolve, reject);
	            }, function (err) {
	              invoke("throw", err, resolve, reject);
	            });
	          }

	          return PromiseImpl.resolve(value).then(function (unwrapped) {
	            // When a yielded Promise is resolved, its final value becomes
	            // the .value of the Promise<{value,done}> result for the
	            // current iteration.
	            result.value = unwrapped;
	            resolve(result);
	          }, function (error) {
	            // If a rejected Promise was yielded, throw the rejection back
	            // into the async generator function so it can be handled there.
	            return invoke("throw", error, resolve, reject);
	          });
	        }
	      }

	      var previousPromise;

	      function enqueue(method, arg) {
	        function callInvokeWithMethodAndArg() {
	          return new PromiseImpl(function (resolve, reject) {
	            invoke(method, arg, resolve, reject);
	          });
	        }

	        return previousPromise = // If enqueue has been called before, then we want to wait until
	        // all previous Promises have been resolved before calling invoke,
	        // so that results are always delivered in the correct order. If
	        // enqueue has not been called before, then it is important to
	        // call invoke immediately, without waiting on a callback to fire,
	        // so that the async generator function has the opportunity to do
	        // any necessary setup in a predictable way. This predictability
	        // is why the Promise constructor synchronously invokes its
	        // executor callback, and why async functions synchronously
	        // execute code before the first await. Since we implement simple
	        // async functions in terms of async generators, it is especially
	        // important to get this right, even though it requires care.
	        previousPromise ? previousPromise.then(callInvokeWithMethodAndArg, // Avoid propagating failures to Promises returned by later
	        // invocations of the iterator.
	        callInvokeWithMethodAndArg) : callInvokeWithMethodAndArg();
	      } // Define the unified helper method that is used to implement .next,
	      // .throw, and .return (see defineIteratorMethods).


	      this._invoke = enqueue;
	    }

	    defineIteratorMethods(AsyncIterator.prototype);

	    AsyncIterator.prototype[asyncIteratorSymbol] = function () {
	      return this;
	    };

	    exports.AsyncIterator = AsyncIterator; // Note that simple async functions are implemented on top of
	    // AsyncIterator objects; they just return a Promise for the value of
	    // the final result produced by the iterator.

	    exports.async = function (innerFn, outerFn, self, tryLocsList, PromiseImpl) {
	      if (PromiseImpl === void 0) PromiseImpl = Promise;
	      var iter = new AsyncIterator(wrap(innerFn, outerFn, self, tryLocsList), PromiseImpl);
	      return exports.isGeneratorFunction(outerFn) ? iter // If outerFn is a generator, return the full iterator.
	      : iter.next().then(function (result) {
	        return result.done ? result.value : iter.next();
	      });
	    };

	    function makeInvokeMethod(innerFn, self, context) {
	      var state = GenStateSuspendedStart;
	      return function invoke(method, arg) {
	        if (state === GenStateExecuting) {
	          throw new Error("Generator is already running");
	        }

	        if (state === GenStateCompleted) {
	          if (method === "throw") {
	            throw arg;
	          } // Be forgiving, per 25.3.3.3.3 of the spec:
	          // https://people.mozilla.org/~jorendorff/es6-draft.html#sec-generatorresume


	          return doneResult();
	        }

	        context.method = method;
	        context.arg = arg;

	        while (true) {
	          var delegate = context.delegate;

	          if (delegate) {
	            var delegateResult = maybeInvokeDelegate(delegate, context);

	            if (delegateResult) {
	              if (delegateResult === ContinueSentinel) continue;
	              return delegateResult;
	            }
	          }

	          if (context.method === "next") {
	            // Setting context._sent for legacy support of Babel's
	            // function.sent implementation.
	            context.sent = context._sent = context.arg;
	          } else if (context.method === "throw") {
	            if (state === GenStateSuspendedStart) {
	              state = GenStateCompleted;
	              throw context.arg;
	            }

	            context.dispatchException(context.arg);
	          } else if (context.method === "return") {
	            context.abrupt("return", context.arg);
	          }

	          state = GenStateExecuting;
	          var record = tryCatch(innerFn, self, context);

	          if (record.type === "normal") {
	            // If an exception is thrown from innerFn, we leave state ===
	            // GenStateExecuting and loop back for another invocation.
	            state = context.done ? GenStateCompleted : GenStateSuspendedYield;

	            if (record.arg === ContinueSentinel) {
	              continue;
	            }

	            return {
	              value: record.arg,
	              done: context.done
	            };
	          } else if (record.type === "throw") {
	            state = GenStateCompleted; // Dispatch the exception by looping back around to the
	            // context.dispatchException(context.arg) call above.

	            context.method = "throw";
	            context.arg = record.arg;
	          }
	        }
	      };
	    } // Call delegate.iterator[context.method](context.arg) and handle the
	    // result, either by returning a { value, done } result from the
	    // delegate iterator, or by modifying context.method and context.arg,
	    // setting context.delegate to null, and returning the ContinueSentinel.


	    function maybeInvokeDelegate(delegate, context) {
	      var method = delegate.iterator[context.method];

	      if (method === undefined$1) {
	        // A .throw or .return when the delegate iterator has no .throw
	        // method always terminates the yield* loop.
	        context.delegate = null;

	        if (context.method === "throw") {
	          // Note: ["return"] must be used for ES3 parsing compatibility.
	          if (delegate.iterator["return"]) {
	            // If the delegate iterator has a return method, give it a
	            // chance to clean up.
	            context.method = "return";
	            context.arg = undefined$1;
	            maybeInvokeDelegate(delegate, context);

	            if (context.method === "throw") {
	              // If maybeInvokeDelegate(context) changed context.method from
	              // "return" to "throw", let that override the TypeError below.
	              return ContinueSentinel;
	            }
	          }

	          context.method = "throw";
	          context.arg = new TypeError("The iterator does not provide a 'throw' method");
	        }

	        return ContinueSentinel;
	      }

	      var record = tryCatch(method, delegate.iterator, context.arg);

	      if (record.type === "throw") {
	        context.method = "throw";
	        context.arg = record.arg;
	        context.delegate = null;
	        return ContinueSentinel;
	      }

	      var info = record.arg;

	      if (!info) {
	        context.method = "throw";
	        context.arg = new TypeError("iterator result is not an object");
	        context.delegate = null;
	        return ContinueSentinel;
	      }

	      if (info.done) {
	        // Assign the result of the finished delegate to the temporary
	        // variable specified by delegate.resultName (see delegateYield).
	        context[delegate.resultName] = info.value; // Resume execution at the desired location (see delegateYield).

	        context.next = delegate.nextLoc; // If context.method was "throw" but the delegate handled the
	        // exception, let the outer generator proceed normally. If
	        // context.method was "next", forget context.arg since it has been
	        // "consumed" by the delegate iterator. If context.method was
	        // "return", allow the original .return call to continue in the
	        // outer generator.

	        if (context.method !== "return") {
	          context.method = "next";
	          context.arg = undefined$1;
	        }
	      } else {
	        // Re-yield the result returned by the delegate method.
	        return info;
	      } // The delegate iterator is finished, so forget it and continue with
	      // the outer generator.


	      context.delegate = null;
	      return ContinueSentinel;
	    } // Define Generator.prototype.{next,throw,return} in terms of the
	    // unified ._invoke helper method.


	    defineIteratorMethods(Gp);
	    Gp[toStringTagSymbol] = "Generator"; // A Generator should always return itself as the iterator object when the
	    // @@iterator function is called on it. Some browsers' implementations of the
	    // iterator prototype chain incorrectly implement this, causing the Generator
	    // object to not be returned from this call. This ensures that doesn't happen.
	    // See https://github.com/facebook/regenerator/issues/274 for more details.

	    Gp[iteratorSymbol] = function () {
	      return this;
	    };

	    Gp.toString = function () {
	      return "[object Generator]";
	    };

	    function pushTryEntry(locs) {
	      var entry = {
	        tryLoc: locs[0]
	      };

	      if (1 in locs) {
	        entry.catchLoc = locs[1];
	      }

	      if (2 in locs) {
	        entry.finallyLoc = locs[2];
	        entry.afterLoc = locs[3];
	      }

	      this.tryEntries.push(entry);
	    }

	    function resetTryEntry(entry) {
	      var record = entry.completion || {};
	      record.type = "normal";
	      delete record.arg;
	      entry.completion = record;
	    }

	    function Context(tryLocsList) {
	      // The root entry object (effectively a try statement without a catch
	      // or a finally block) gives us a place to store values thrown from
	      // locations where there is no enclosing try statement.
	      this.tryEntries = [{
	        tryLoc: "root"
	      }];
	      tryLocsList.forEach(pushTryEntry, this);
	      this.reset(true);
	    }

	    exports.keys = function (object) {
	      var keys = [];

	      for (var key in object) {
	        keys.push(key);
	      }

	      keys.reverse(); // Rather than returning an object with a next method, we keep
	      // things simple and return the next function itself.

	      return function next() {
	        while (keys.length) {
	          var key = keys.pop();

	          if (key in object) {
	            next.value = key;
	            next.done = false;
	            return next;
	          }
	        } // To avoid creating an additional object, we just hang the .value
	        // and .done properties off the next function object itself. This
	        // also ensures that the minifier will not anonymize the function.


	        next.done = true;
	        return next;
	      };
	    };

	    function values(iterable) {
	      if (iterable) {
	        var iteratorMethod = iterable[iteratorSymbol];

	        if (iteratorMethod) {
	          return iteratorMethod.call(iterable);
	        }

	        if (typeof iterable.next === "function") {
	          return iterable;
	        }

	        if (!isNaN(iterable.length)) {
	          var i = -1,
	              next = function next() {
	            while (++i < iterable.length) {
	              if (hasOwn.call(iterable, i)) {
	                next.value = iterable[i];
	                next.done = false;
	                return next;
	              }
	            }

	            next.value = undefined$1;
	            next.done = true;
	            return next;
	          };

	          return next.next = next;
	        }
	      } // Return an iterator with no values.


	      return {
	        next: doneResult
	      };
	    }

	    exports.values = values;

	    function doneResult() {
	      return {
	        value: undefined$1,
	        done: true
	      };
	    }

	    Context.prototype = {
	      constructor: Context,
	      reset: function (skipTempReset) {
	        this.prev = 0;
	        this.next = 0; // Resetting context._sent for legacy support of Babel's
	        // function.sent implementation.

	        this.sent = this._sent = undefined$1;
	        this.done = false;
	        this.delegate = null;
	        this.method = "next";
	        this.arg = undefined$1;
	        this.tryEntries.forEach(resetTryEntry);

	        if (!skipTempReset) {
	          for (var name in this) {
	            // Not sure about the optimal order of these conditions:
	            if (name.charAt(0) === "t" && hasOwn.call(this, name) && !isNaN(+name.slice(1))) {
	              this[name] = undefined$1;
	            }
	          }
	        }
	      },
	      stop: function () {
	        this.done = true;
	        var rootEntry = this.tryEntries[0];
	        var rootRecord = rootEntry.completion;

	        if (rootRecord.type === "throw") {
	          throw rootRecord.arg;
	        }

	        return this.rval;
	      },
	      dispatchException: function (exception) {
	        if (this.done) {
	          throw exception;
	        }

	        var context = this;

	        function handle(loc, caught) {
	          record.type = "throw";
	          record.arg = exception;
	          context.next = loc;

	          if (caught) {
	            // If the dispatched exception was caught by a catch block,
	            // then let that catch block handle the exception normally.
	            context.method = "next";
	            context.arg = undefined$1;
	          }

	          return !!caught;
	        }

	        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
	          var entry = this.tryEntries[i];
	          var record = entry.completion;

	          if (entry.tryLoc === "root") {
	            // Exception thrown outside of any try block that could handle
	            // it, so set the completion value of the entire function to
	            // throw the exception.
	            return handle("end");
	          }

	          if (entry.tryLoc <= this.prev) {
	            var hasCatch = hasOwn.call(entry, "catchLoc");
	            var hasFinally = hasOwn.call(entry, "finallyLoc");

	            if (hasCatch && hasFinally) {
	              if (this.prev < entry.catchLoc) {
	                return handle(entry.catchLoc, true);
	              } else if (this.prev < entry.finallyLoc) {
	                return handle(entry.finallyLoc);
	              }
	            } else if (hasCatch) {
	              if (this.prev < entry.catchLoc) {
	                return handle(entry.catchLoc, true);
	              }
	            } else if (hasFinally) {
	              if (this.prev < entry.finallyLoc) {
	                return handle(entry.finallyLoc);
	              }
	            } else {
	              throw new Error("try statement without catch or finally");
	            }
	          }
	        }
	      },
	      abrupt: function (type, arg) {
	        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
	          var entry = this.tryEntries[i];

	          if (entry.tryLoc <= this.prev && hasOwn.call(entry, "finallyLoc") && this.prev < entry.finallyLoc) {
	            var finallyEntry = entry;
	            break;
	          }
	        }

	        if (finallyEntry && (type === "break" || type === "continue") && finallyEntry.tryLoc <= arg && arg <= finallyEntry.finallyLoc) {
	          // Ignore the finally entry if control is not jumping to a
	          // location outside the try/catch block.
	          finallyEntry = null;
	        }

	        var record = finallyEntry ? finallyEntry.completion : {};
	        record.type = type;
	        record.arg = arg;

	        if (finallyEntry) {
	          this.method = "next";
	          this.next = finallyEntry.finallyLoc;
	          return ContinueSentinel;
	        }

	        return this.complete(record);
	      },
	      complete: function (record, afterLoc) {
	        if (record.type === "throw") {
	          throw record.arg;
	        }

	        if (record.type === "break" || record.type === "continue") {
	          this.next = record.arg;
	        } else if (record.type === "return") {
	          this.rval = this.arg = record.arg;
	          this.method = "return";
	          this.next = "end";
	        } else if (record.type === "normal" && afterLoc) {
	          this.next = afterLoc;
	        }

	        return ContinueSentinel;
	      },
	      finish: function (finallyLoc) {
	        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
	          var entry = this.tryEntries[i];

	          if (entry.finallyLoc === finallyLoc) {
	            this.complete(entry.completion, entry.afterLoc);
	            resetTryEntry(entry);
	            return ContinueSentinel;
	          }
	        }
	      },
	      "catch": function (tryLoc) {
	        for (var i = this.tryEntries.length - 1; i >= 0; --i) {
	          var entry = this.tryEntries[i];

	          if (entry.tryLoc === tryLoc) {
	            var record = entry.completion;

	            if (record.type === "throw") {
	              var thrown = record.arg;
	              resetTryEntry(entry);
	            }

	            return thrown;
	          }
	        } // The context.catch method must only be called with a location
	        // argument that corresponds to a known catch block.


	        throw new Error("illegal catch attempt");
	      },
	      delegateYield: function (iterable, resultName, nextLoc) {
	        this.delegate = {
	          iterator: values(iterable),
	          resultName: resultName,
	          nextLoc: nextLoc
	        };

	        if (this.method === "next") {
	          // Deliberately forget the last sent value so that we don't
	          // accidentally pass it on to the delegate.
	          this.arg = undefined$1;
	        }

	        return ContinueSentinel;
	      }
	    }; // Regardless of whether this script is executing as a CommonJS module
	    // or not, return the runtime object so that we can declare the variable
	    // regeneratorRuntime in the outer scope, which allows this module to be
	    // injected easily by `bin/regenerator --include-runtime script.js`.

	    return exports;
	  }( // If this script is executing as a CommonJS module, use module.exports
	  // as the regeneratorRuntime namespace. Otherwise create a new empty
	  // object. Either way, the resulting object will be used to initialize
	  // the regeneratorRuntime variable at the top of this file.
	   module.exports );

	  try {
	    regeneratorRuntime = runtime;
	  } catch (accidentalStrictMode) {
	    // This module should not be running in strict mode, so the above
	    // assignment should always work unless something is misconfigured. Just
	    // in case runtime.js accidentally runs in strict mode, we can escape
	    // strict mode using a global Function call. This could conceivably fail
	    // if a Content Security Policy forbids using Function, but in that case
	    // the proper solution is to fix the accidental strict mode problem. If
	    // you've misconfigured your bundler to force strict mode and applied a
	    // CSP to forbid Function, and you're not willing to fix either of those
	    // problems, please detail your unique predicament in a GitHub issue.
	    Function("r", "regeneratorRuntime = r")(runtime);
	  }
	});

	var regenerator = runtime_1;

	var isArray$4 = isArray$1;

	var isArray$5 = isArray$4;

	function _arrayWithHoles(arr) {
	  if (isArray$5(arr)) return arr;
	}

	var arrayWithHoles = _arrayWithHoles;

	var ITERATOR$5 = wellKnownSymbol('iterator');

	var isIterable = function (it) {
	  var O = Object(it);
	  return O[ITERATOR$5] !== undefined || '@@iterator' in O // eslint-disable-next-line no-prototype-builtins
	  || iterators.hasOwnProperty(classof(O));
	};

	var isIterable_1 = isIterable;

	var isIterable$1 = isIterable_1;

	function _iterableToArrayLimit(arr, i) {
	  if (typeof symbol$2 === "undefined" || !isIterable$1(Object(arr))) return;
	  var _arr = [];
	  var _n = true;
	  var _d = false;
	  var _e = undefined;

	  try {
	    for (var _i = getIterator$1(arr), _s; !(_n = (_s = _i.next()).done); _n = true) {
	      _arr.push(_s.value);

	      if (i && _arr.length === i) break;
	    }
	  } catch (err) {
	    _d = true;
	    _e = err;
	  } finally {
	    try {
	      if (!_n && _i["return"] != null) _i["return"]();
	    } finally {
	      if (_d) throw _e;
	    }
	  }

	  return _arr;
	}

	var iterableToArrayLimit = _iterableToArrayLimit;

	var from_1$3 = from_1;

	var from_1$4 = from_1$3;

	var slice$4 = slice_1;

	var slice$5 = slice$4;

	function _arrayLikeToArray(arr, len) {
	  if (len == null || len > arr.length) len = arr.length;

	  for (var i = 0, arr2 = new Array(len); i < len; i++) {
	    arr2[i] = arr[i];
	  }

	  return arr2;
	}

	var arrayLikeToArray = _arrayLikeToArray;

	function _unsupportedIterableToArray(o, minLen) {
	  var _context;

	  if (!o) return;
	  if (typeof o === "string") return arrayLikeToArray(o, minLen);

	  var n = slice$5(_context = Object.prototype.toString.call(o)).call(_context, 8, -1);

	  if (n === "Object" && o.constructor) n = o.constructor.name;
	  if (n === "Map" || n === "Set") return from_1$4(o);
	  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return arrayLikeToArray(o, minLen);
	}

	var unsupportedIterableToArray = _unsupportedIterableToArray;

	function _nonIterableRest() {
	  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
	}

	var nonIterableRest = _nonIterableRest;

	function _slicedToArray(arr, i) {
	  return arrayWithHoles(arr) || iterableToArrayLimit(arr, i) || unsupportedIterableToArray(arr, i) || nonIterableRest();
	}

	var slicedToArray = _slicedToArray;

	var iterator$3 = iterator;

	var iterator$4 = iterator$3;

	var concat = entryVirtual('Array').concat;

	var ArrayPrototype$7 = Array.prototype;

	var concat_1 = function (it) {
	  var own = it.concat;
	  return it === ArrayPrototype$7 || it instanceof Array && own === ArrayPrototype$7.concat ? concat : own;
	};

	var concat$1 = concat_1;

	var concat$2 = concat$1;

	function _arrayWithoutHoles(arr) {
	  if (isArray$5(arr)) return arrayLikeToArray(arr);
	}

	var arrayWithoutHoles = _arrayWithoutHoles;

	function _iterableToArray(iter) {
	  if (typeof symbol$2 !== "undefined" && isIterable$1(Object(iter))) return from_1$4(iter);
	}

	var iterableToArray = _iterableToArray;

	function _nonIterableSpread() {
	  throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
	}

	var nonIterableSpread = _nonIterableSpread;

	function _toConsumableArray(arr) {
	  return arrayWithoutHoles(arr) || iterableToArray(arr) || unsupportedIterableToArray(arr) || nonIterableSpread();
	}

	var toConsumableArray = _toConsumableArray;

	var HAS_SPECIES_SUPPORT$1 = arrayMethodHasSpeciesSupport('splice');
	var USES_TO_LENGTH$2 = arrayMethodUsesToLength('splice', {
	  ACCESSORS: true,
	  0: 0,
	  1: 2
	});
	var max$2 = Math.max;
	var min$2 = Math.min;
	var MAX_SAFE_INTEGER$1 = 0x1FFFFFFFFFFFFF;
	var MAXIMUM_ALLOWED_LENGTH_EXCEEDED = 'Maximum allowed length exceeded'; // `Array.prototype.splice` method
	// https://tc39.github.io/ecma262/#sec-array.prototype.splice
	// with adding support of @@species

	_export({
	  target: 'Array',
	  proto: true,
	  forced: !HAS_SPECIES_SUPPORT$1 || !USES_TO_LENGTH$2
	}, {
	  splice: function splice(start, deleteCount
	  /* , ...items */
	  ) {
	    var O = toObject(this);
	    var len = toLength(O.length);
	    var actualStart = toAbsoluteIndex(start, len);
	    var argumentsLength = arguments.length;
	    var insertCount, actualDeleteCount, A, k, from, to;

	    if (argumentsLength === 0) {
	      insertCount = actualDeleteCount = 0;
	    } else if (argumentsLength === 1) {
	      insertCount = 0;
	      actualDeleteCount = len - actualStart;
	    } else {
	      insertCount = argumentsLength - 2;
	      actualDeleteCount = min$2(max$2(toInteger(deleteCount), 0), len - actualStart);
	    }

	    if (len + insertCount - actualDeleteCount > MAX_SAFE_INTEGER$1) {
	      throw TypeError(MAXIMUM_ALLOWED_LENGTH_EXCEEDED);
	    }

	    A = arraySpeciesCreate(O, actualDeleteCount);

	    for (k = 0; k < actualDeleteCount; k++) {
	      from = actualStart + k;
	      if (from in O) createProperty(A, k, O[from]);
	    }

	    A.length = actualDeleteCount;

	    if (insertCount < actualDeleteCount) {
	      for (k = actualStart; k < len - actualDeleteCount; k++) {
	        from = k + actualDeleteCount;
	        to = k + insertCount;
	        if (from in O) O[to] = O[from];else delete O[to];
	      }

	      for (k = len; k > len - actualDeleteCount + insertCount; k--) delete O[k - 1];
	    } else if (insertCount > actualDeleteCount) {
	      for (k = len - actualDeleteCount; k > actualStart; k--) {
	        from = k + actualDeleteCount - 1;
	        to = k + insertCount - 1;
	        if (from in O) O[to] = O[from];else delete O[to];
	      }
	    }

	    for (k = 0; k < insertCount; k++) {
	      O[k + actualStart] = arguments[k + 2];
	    }

	    O.length = len - actualDeleteCount + insertCount;
	    return A;
	  }
	});

	var splice = entryVirtual('Array').splice;

	var ArrayPrototype$8 = Array.prototype;

	var splice_1 = function (it) {
	  var own = it.splice;
	  return it === ArrayPrototype$8 || it instanceof Array && own === ArrayPrototype$8.splice ? splice : own;
	};

	var splice$1 = splice_1;

	var splice$2 = splice$1;

	var $forEach$1 = arrayIteration.forEach;
	var STRICT_METHOD$2 = arrayMethodIsStrict('forEach');
	var USES_TO_LENGTH$3 = arrayMethodUsesToLength('forEach'); // `Array.prototype.forEach` method implementation
	// https://tc39.github.io/ecma262/#sec-array.prototype.foreach

	var arrayForEach = !STRICT_METHOD$2 || !USES_TO_LENGTH$3 ? function forEach(callbackfn
	/* , thisArg */
	) {
	  return $forEach$1(this, callbackfn, arguments.length > 1 ? arguments[1] : undefined);
	} : [].forEach;

	// https://tc39.github.io/ecma262/#sec-array.prototype.foreach


	_export({
	  target: 'Array',
	  proto: true,
	  forced: [].forEach != arrayForEach
	}, {
	  forEach: arrayForEach
	});

	var forEach$1 = entryVirtual('Array').forEach;

	var forEach$2 = forEach$1;

	var ArrayPrototype$9 = Array.prototype;
	var DOMIterables$3 = {
	  DOMTokenList: true,
	  NodeList: true
	};

	var forEach_1 = function (it) {
	  var own = it.forEach;
	  return it === ArrayPrototype$9 || it instanceof Array && own === ArrayPrototype$9.forEach // eslint-disable-next-line no-prototype-builtins
	  || DOMIterables$3.hasOwnProperty(classof(it)) ? forEach$2 : own;
	};

	var forEach$3 = forEach_1;

	var slice$6 = [].slice;
	var MSIE = /MSIE .\./.test(engineUserAgent); // <- dirty ie9- check

	var wrap$1 = function (scheduler) {
	  return function (handler, timeout
	  /* , ...arguments */
	  ) {
	    var boundArgs = arguments.length > 2;
	    var args = boundArgs ? slice$6.call(arguments, 2) : undefined;
	    return scheduler(boundArgs ? function () {
	      // eslint-disable-next-line no-new-func
	      (typeof handler == 'function' ? handler : Function(handler)).apply(this, args);
	    } : handler, timeout);
	  };
	}; // ie9- setTimeout & setInterval additional parameters fix
	// https://html.spec.whatwg.org/multipage/timers-and-user-prompts.html#timers


	_export({
	  global: true,
	  bind: true,
	  forced: MSIE
	}, {
	  // `setTimeout` method
	  // https://html.spec.whatwg.org/multipage/timers-and-user-prompts.html#dom-settimeout
	  setTimeout: wrap$1(global_1.setTimeout),
	  // `setInterval` method
	  // https://html.spec.whatwg.org/multipage/timers-and-user-prompts.html#dom-setinterval
	  setInterval: wrap$1(global_1.setInterval)
	});

	var setTimeout$1 = path.setTimeout;

	var setTimeout$2 = setTimeout$1;

	// https://tc39.github.io/proposal-flatMap/#sec-FlattenIntoArray


	var flattenIntoArray = function (target, original, source, sourceLen, start, depth, mapper, thisArg) {
	  var targetIndex = start;
	  var sourceIndex = 0;
	  var mapFn = mapper ? functionBindContext(mapper, thisArg, 3) : false;
	  var element;

	  while (sourceIndex < sourceLen) {
	    if (sourceIndex in source) {
	      element = mapFn ? mapFn(source[sourceIndex], sourceIndex, original) : source[sourceIndex];

	      if (depth > 0 && isArray(element)) {
	        targetIndex = flattenIntoArray(target, original, element, toLength(element.length), targetIndex, depth - 1) - 1;
	      } else {
	        if (targetIndex >= 0x1FFFFFFFFFFFFF) throw TypeError('Exceed the acceptable array length');
	        target[targetIndex] = element;
	      }

	      targetIndex++;
	    }

	    sourceIndex++;
	  }

	  return targetIndex;
	};

	var flattenIntoArray_1 = flattenIntoArray;

	// https://github.com/tc39/proposal-flatMap


	_export({
	  target: 'Array',
	  proto: true
	}, {
	  flatMap: function flatMap(callbackfn
	  /* , thisArg */
	  ) {
	    var O = toObject(this);
	    var sourceLen = toLength(O.length);
	    var A;
	    aFunction(callbackfn);
	    A = arraySpeciesCreate(O, 0);
	    A.length = flattenIntoArray_1(A, O, O, sourceLen, 0, 1, callbackfn, arguments.length > 1 ? arguments[1] : undefined);
	    return A;
	  }
	});

	var flatMap = entryVirtual('Array').flatMap;

	var ArrayPrototype$a = Array.prototype;

	var flatMap_1 = function (it) {
	  var own = it.flatMap;
	  return it === ArrayPrototype$a || it instanceof Array && own === ArrayPrototype$a.flatMap ? flatMap : own;
	};

	var flatMap$1 = flatMap_1;

	var flatMap$2 = flatMap$1;

	var $map = arrayIteration.map;
	var HAS_SPECIES_SUPPORT$2 = arrayMethodHasSpeciesSupport('map'); // FF49- issue

	var USES_TO_LENGTH$4 = arrayMethodUsesToLength('map'); // `Array.prototype.map` method
	// https://tc39.github.io/ecma262/#sec-array.prototype.map
	// with adding support of @@species

	_export({
	  target: 'Array',
	  proto: true,
	  forced: !HAS_SPECIES_SUPPORT$2 || !USES_TO_LENGTH$4
	}, {
	  map: function map(callbackfn
	  /* , thisArg */
	  ) {
	    return $map(this, callbackfn, arguments.length > 1 ? arguments[1] : undefined);
	  }
	});

	var map$3 = entryVirtual('Array').map;

	var ArrayPrototype$b = Array.prototype;

	var map_1 = function (it) {
	  var own = it.map;
	  return it === ArrayPrototype$b || it instanceof Array && own === ArrayPrototype$b.map ? map$3 : own;
	};

	var map$4 = map_1;

	var map$5 = map$4;

	var $filter = arrayIteration.filter;
	var HAS_SPECIES_SUPPORT$3 = arrayMethodHasSpeciesSupport('filter'); // Edge 14- issue

	var USES_TO_LENGTH$5 = arrayMethodUsesToLength('filter'); // `Array.prototype.filter` method
	// https://tc39.github.io/ecma262/#sec-array.prototype.filter
	// with adding support of @@species

	_export({
	  target: 'Array',
	  proto: true,
	  forced: !HAS_SPECIES_SUPPORT$3 || !USES_TO_LENGTH$5
	}, {
	  filter: function filter(callbackfn
	  /* , thisArg */
	  ) {
	    return $filter(this, callbackfn, arguments.length > 1 ? arguments[1] : undefined);
	  }
	});

	var filter = entryVirtual('Array').filter;

	var ArrayPrototype$c = Array.prototype;

	var filter_1 = function (it) {
	  var own = it.filter;
	  return it === ArrayPrototype$c || it instanceof Array && own === ArrayPrototype$c.filter ? filter : own;
	};

	var filter$1 = filter_1;

	var filter$2 = filter$1;

	var createMethod$3 = function (IS_RIGHT) {
	  return function (that, callbackfn, argumentsLength, memo) {
	    aFunction(callbackfn);
	    var O = toObject(that);
	    var self = indexedObject(O);
	    var length = toLength(O.length);
	    var index = IS_RIGHT ? length - 1 : 0;
	    var i = IS_RIGHT ? -1 : 1;
	    if (argumentsLength < 2) while (true) {
	      if (index in self) {
	        memo = self[index];
	        index += i;
	        break;
	      }

	      index += i;

	      if (IS_RIGHT ? index < 0 : length <= index) {
	        throw TypeError('Reduce of empty array with no initial value');
	      }
	    }

	    for (; IS_RIGHT ? index >= 0 : length > index; index += i) if (index in self) {
	      memo = callbackfn(memo, self[index], index, O);
	    }

	    return memo;
	  };
	};

	var arrayReduce = {
	  // `Array.prototype.reduce` method
	  // https://tc39.github.io/ecma262/#sec-array.prototype.reduce
	  left: createMethod$3(false),
	  // `Array.prototype.reduceRight` method
	  // https://tc39.github.io/ecma262/#sec-array.prototype.reduceright
	  right: createMethod$3(true)
	};

	var $reduce = arrayReduce.left;
	var STRICT_METHOD$3 = arrayMethodIsStrict('reduce');
	var USES_TO_LENGTH$6 = arrayMethodUsesToLength('reduce', {
	  1: 0
	}); // `Array.prototype.reduce` method
	// https://tc39.github.io/ecma262/#sec-array.prototype.reduce

	_export({
	  target: 'Array',
	  proto: true,
	  forced: !STRICT_METHOD$3 || !USES_TO_LENGTH$6
	}, {
	  reduce: function reduce(callbackfn
	  /* , initialValue */
	  ) {
	    return $reduce(this, callbackfn, arguments.length, arguments.length > 1 ? arguments[1] : undefined);
	  }
	});

	var reduce = entryVirtual('Array').reduce;

	var ArrayPrototype$d = Array.prototype;

	var reduce_1 = function (it) {
	  var own = it.reduce;
	  return it === ArrayPrototype$d || it instanceof Array && own === ArrayPrototype$d.reduce ? reduce : own;
	};

	var reduce$1 = reduce_1;

	var reduce$2 = reduce$1;

	// https://tc39.github.io/ecma262/#sec-function.prototype.bind

	_export({
	  target: 'Function',
	  proto: true
	}, {
	  bind: functionBind
	});

	var bind = entryVirtual('Function').bind;

	var FunctionPrototype = Function.prototype;

	var bind_1 = function (it) {
	  var own = it.bind;
	  return it === FunctionPrototype || it instanceof Function && own === FunctionPrototype.bind ? bind : own;
	};

	var bind$1 = bind_1;

	var bind$2 = bind$1;

	function _classCallCheck(instance, Constructor) {
	  if (!(instance instanceof Constructor)) {
	    throw new TypeError("Cannot call a class as a function");
	  }
	}

	var classCallCheck = _classCallCheck;

	function _defineProperties(target, props) {
	  for (var i = 0; i < props.length; i++) {
	    var descriptor = props[i];
	    descriptor.enumerable = descriptor.enumerable || false;
	    descriptor.configurable = true;
	    if ("value" in descriptor) descriptor.writable = true;

	    defineProperty$7(target, descriptor.key, descriptor);
	  }
	}

	function _createClass(Constructor, protoProps, staticProps) {
	  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
	  if (staticProps) _defineProperties(Constructor, staticProps);
	  return Constructor;
	}

	var createClass = _createClass;

	// Unique ID creation requires a high quality random # generator. In the browser we therefore
	// require the crypto API and do not support built-in fallback to lower quality random number
	// generators (like Math.random()).
	// getRandomValues needs to be invoked in a context where "this" is a Crypto implementation. Also,
	// find the complete implementation of crypto (msCrypto) on IE11.
	var getRandomValues = typeof crypto != 'undefined' && crypto.getRandomValues && crypto.getRandomValues.bind(crypto) || typeof msCrypto != 'undefined' && typeof msCrypto.getRandomValues == 'function' && msCrypto.getRandomValues.bind(msCrypto);
	var rnds8 = new Uint8Array(16); // eslint-disable-line no-undef

	function rng() {
	  if (!getRandomValues) {
	    throw new Error('crypto.getRandomValues() not supported. See https://github.com/uuidjs/uuid#getrandomvalues-not-supported');
	  }

	  return getRandomValues(rnds8);
	}

	/**
	 * Convert array of 16 byte values to UUID string format of the form:
	 * XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX
	 */
	var byteToHex = [];

	for (var i = 0; i < 256; ++i) {
	  byteToHex[i] = (i + 0x100).toString(16).substr(1);
	}

	function bytesToUuid(buf, offset) {
	  var i = offset || 0;
	  var bth = byteToHex; // join used to fix memory issue caused by concatenation: https://bugs.chromium.org/p/v8/issues/detail?id=3175#c4

	  return [bth[buf[i++]], bth[buf[i++]], bth[buf[i++]], bth[buf[i++]], '-', bth[buf[i++]], bth[buf[i++]], '-', bth[buf[i++]], bth[buf[i++]], '-', bth[buf[i++]], bth[buf[i++]], '-', bth[buf[i++]], bth[buf[i++]], bth[buf[i++]], bth[buf[i++]], bth[buf[i++]], bth[buf[i++]]].join('');
	}

	function v4(options, buf, offset) {
	  var i = buf && offset || 0;

	  if (typeof options == 'string') {
	    buf = options === 'binary' ? new Array(16) : null;
	    options = null;
	  }

	  options = options || {};
	  var rnds = options.random || (options.rng || rng)(); // Per 4.4, set bits for version and `clock_seq_hi_and_reserved`

	  rnds[6] = rnds[6] & 0x0f | 0x40;
	  rnds[8] = rnds[8] & 0x3f | 0x80; // Copy bytes to buffer, if provided

	  if (buf) {
	    for (var ii = 0; ii < 16; ++ii) {
	      buf[i + ii] = rnds[ii];
	    }
	  }

	  return buf || bytesToUuid(rnds);
	}

	// a string of all valid unicode whitespaces
	// eslint-disable-next-line max-len
	var whitespaces = '\u0009\u000A\u000B\u000C\u000D\u0020\u00A0\u1680\u2000\u2001\u2002\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200A\u202F\u205F\u3000\u2028\u2029\uFEFF';

	var whitespace = '[' + whitespaces + ']';
	var ltrim = RegExp('^' + whitespace + whitespace + '*');
	var rtrim = RegExp(whitespace + whitespace + '*$'); // `String.prototype.{ trim, trimStart, trimEnd, trimLeft, trimRight }` methods implementation

	var createMethod$4 = function (TYPE) {
	  return function ($this) {
	    var string = String(requireObjectCoercible($this));
	    if (TYPE & 1) string = string.replace(ltrim, '');
	    if (TYPE & 2) string = string.replace(rtrim, '');
	    return string;
	  };
	};

	var stringTrim = {
	  // `String.prototype.{ trimLeft, trimStart }` methods
	  // https://tc39.github.io/ecma262/#sec-string.prototype.trimstart
	  start: createMethod$4(1),
	  // `String.prototype.{ trimRight, trimEnd }` methods
	  // https://tc39.github.io/ecma262/#sec-string.prototype.trimend
	  end: createMethod$4(2),
	  // `String.prototype.trim` method
	  // https://tc39.github.io/ecma262/#sec-string.prototype.trim
	  trim: createMethod$4(3)
	};

	var non = '\u200B\u0085\u180E'; // check that a method works with the correct list
	// of whitespaces and has a correct name

	var stringTrimForced = function (METHOD_NAME) {
	  return fails(function () {
	    return !!whitespaces[METHOD_NAME]() || non[METHOD_NAME]() != non || whitespaces[METHOD_NAME].name !== METHOD_NAME;
	  });
	};

	var $trim = stringTrim.trim; // `String.prototype.trim` method
	// https://tc39.github.io/ecma262/#sec-string.prototype.trim

	_export({
	  target: 'String',
	  proto: true,
	  forced: stringTrimForced('trim')
	}, {
	  trim: function trim() {
	    return $trim(this);
	  }
	});

	var trim = entryVirtual('String').trim;

	var StringPrototype = String.prototype;

	var trim_1 = function (it) {
	  var own = it.trim;
	  return typeof it === 'string' || it === StringPrototype || it instanceof String && own === StringPrototype.trim ? trim : own;
	};

	var trim$1 = trim_1;

	var trim$2 = trim$1;

	var trim$3 = stringTrim.trim;
	var $parseInt = global_1.parseInt;
	var hex = /^[+-]?0[Xx]/;
	var FORCED$5 = $parseInt(whitespaces + '08') !== 8 || $parseInt(whitespaces + '0x16') !== 22; // `parseInt` method
	// https://tc39.github.io/ecma262/#sec-parseint-string-radix

	var numberParseInt = FORCED$5 ? function parseInt(string, radix) {
	  var S = trim$3(String(string));
	  return $parseInt(S, radix >>> 0 || (hex.test(S) ? 16 : 10));
	} : $parseInt;

	// https://tc39.github.io/ecma262/#sec-parseint-string-radix

	_export({
	  global: true,
	  forced: parseInt != numberParseInt
	}, {
	  parseInt: numberParseInt
	});

	var _parseInt = path.parseInt;

	var _parseInt$1 = _parseInt;

	var _parseInt$2 = _parseInt$1;

	var propertyIsEnumerable = objectPropertyIsEnumerable.f; // `Object.{ entries, values }` methods implementation

	var createMethod$5 = function (TO_ENTRIES) {
	  return function (it) {
	    var O = toIndexedObject(it);
	    var keys = objectKeys(O);
	    var length = keys.length;
	    var i = 0;
	    var result = [];
	    var key;

	    while (length > i) {
	      key = keys[i++];

	      if (!descriptors || propertyIsEnumerable.call(O, key)) {
	        result.push(TO_ENTRIES ? [key, O[key]] : O[key]);
	      }
	    }

	    return result;
	  };
	};

	var objectToArray = {
	  // `Object.entries` method
	  // https://tc39.github.io/ecma262/#sec-object.entries
	  entries: createMethod$5(true),
	  // `Object.values` method
	  // https://tc39.github.io/ecma262/#sec-object.values
	  values: createMethod$5(false)
	};

	var $values = objectToArray.values; // `Object.values` method
	// https://tc39.github.io/ecma262/#sec-object.values

	_export({
	  target: 'Object',
	  stat: true
	}, {
	  values: function values(O) {
	    return $values(O);
	  }
	});

	var values$3 = path.Object.values;

	var values$4 = values$3;

	var values$5 = values$4;

	var getPrototypeOf$4 = getPrototypeOf;

	var getPrototypeOf$5 = getPrototypeOf$4;

	var $indexOf = arrayIncludes.indexOf;
	var nativeIndexOf = [].indexOf;
	var NEGATIVE_ZERO = !!nativeIndexOf && 1 / [1].indexOf(1, -0) < 0;
	var STRICT_METHOD$4 = arrayMethodIsStrict('indexOf');
	var USES_TO_LENGTH$7 = arrayMethodUsesToLength('indexOf', {
	  ACCESSORS: true,
	  1: 0
	}); // `Array.prototype.indexOf` method
	// https://tc39.github.io/ecma262/#sec-array.prototype.indexof

	_export({
	  target: 'Array',
	  proto: true,
	  forced: NEGATIVE_ZERO || !STRICT_METHOD$4 || !USES_TO_LENGTH$7
	}, {
	  indexOf: function indexOf(searchElement
	  /* , fromIndex = 0 */
	  ) {
	    return NEGATIVE_ZERO // convert -0 to +0
	    ? nativeIndexOf.apply(this, arguments) || 0 : $indexOf(this, searchElement, arguments.length > 1 ? arguments[1] : undefined);
	  }
	});

	var indexOf$1 = entryVirtual('Array').indexOf;

	var ArrayPrototype$e = Array.prototype;

	var indexOf_1 = function (it) {
	  var own = it.indexOf;
	  return it === ArrayPrototype$e || it instanceof Array && own === ArrayPrototype$e.indexOf ? indexOf$1 : own;
	};

	var indexOf$2 = indexOf_1;

	var indexOf$3 = indexOf$2;

	// https://tc39.github.io/ecma262/#sec-date.now

	_export({
	  target: 'Date',
	  stat: true
	}, {
	  now: function now() {
	    return new Date().getTime();
	  }
	});

	var now = path.Date.now;

	var now$1 = now;

	var now$2 = now$1;

	function _createForOfIteratorHelper(o) { if (typeof symbol$4 === "undefined" || getIteratorMethod$1(o) == null) { if (isArray$3(o) || (o = _unsupportedIterableToArray$1(o))) { var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var it, normalCompletion = true, didErr = false, err; return { s: function s() { it = getIterator$1(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }

	function _unsupportedIterableToArray$1(o, minLen) { var _context13; if (!o) return; if (typeof o === "string") return _arrayLikeToArray$1(o, minLen); var n = slice$3(_context13 = Object.prototype.toString.call(o)).call(_context13, 8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return from_1$2(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$1(o, minLen); }

	function _arrayLikeToArray$1(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

	function ownKeys$1(object, enumerableOnly) { var keys = keys$6(object); if (getOwnPropertySymbols$2) { var symbols = getOwnPropertySymbols$2(object); if (enumerableOnly) symbols = filter$2(symbols).call(symbols, function (sym) { return getOwnPropertyDescriptor$3(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

	function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { var _context11; forEach$3(_context11 = ownKeys$1(Object(source), true)).call(_context11, function (key) { defineProperty$8(target, key, source[key]); }); } else if (getOwnPropertyDescriptors$2) { defineProperties$1(target, getOwnPropertyDescriptors$2(source)); } else { var _context12; forEach$3(_context12 = ownKeys$1(Object(source))).call(_context12, function (key) { defineProperty$2(target, key, getOwnPropertyDescriptor$3(source, key)); }); } } return target; }

	/**
	 * vis-util
	 * https://github.com/visjs/vis-util
	 *
	 * utilitie collection for visjs
	 *
	 * @version 4.0.1
	 * @date    2020-05-16T21:18:02.369Z
	 *
	 * @copyright (c) 2011-2017 Almende B.V, http://almende.com
	 * @copyright (c) 2017-2019 visjs contributors, https://github.com/visjs
	 *
	 * @license
	 * vis.js is dual licensed under both
	 *
	 *   1. The Apache 2.0 License
	 *      http://www.apache.org/licenses/LICENSE-2.0
	 *
	 *   and
	 *
	 *   2. The MIT License
	 *      http://opensource.org/licenses/MIT
	 *
	 * vis.js may be distributed under either license.
	 */

	/**
	 * Seedable, fast and reasonably good (not crypto but more than okay for our
	 * needs) random number generator.
	 *
	 * @remarks
	 * Adapted from {@link https://web.archive.org/web/20110429100736/http://baagoe.com:80/en/RandomMusings/javascript}.
	 * Original algorithm created by Johannes Baagøe \<baagoe\@baagoe.com\> in 2010.
	 */

	/**
	 * Create a seeded pseudo random generator based on Alea by Johannes Baagøe.
	 *
	 * @param seed - All supplied arguments will be used as a seed. In case nothing
	 * is supplied the current time will be used to seed the generator.
	 *
	 * @returns A ready to use seeded generator.
	 */
	function Alea() {
	  for (var _len = arguments.length, seed = new Array(_len), _key = 0; _key < _len; _key++) {
	    seed[_key] = arguments[_key];
	  }

	  return AleaImplementation(seed.length ? seed : [now$2()]);
	}
	/**
	 * An implementation of [[Alea]] without user input validation.
	 *
	 * @param seed - The data that will be used to seed the generator.
	 *
	 * @returns A ready to use seeded generator.
	 */


	function AleaImplementation(seed) {
	  var _mashSeed = mashSeed(seed),
	      _mashSeed2 = slicedToArray(_mashSeed, 3),
	      s0 = _mashSeed2[0],
	      s1 = _mashSeed2[1],
	      s2 = _mashSeed2[2];

	  var c = 1;

	  var random = function random() {
	    var t = 2091639 * s0 + c * 2.3283064365386963e-10; // 2^-32

	    s0 = s1;
	    s1 = s2;
	    return s2 = t - (c = t | 0);
	  };

	  random.uint32 = function () {
	    return random() * 0x100000000;
	  }; // 2^32


	  random.fract53 = function () {
	    return random() + (random() * 0x200000 | 0) * 1.1102230246251565e-16;
	  }; // 2^-53


	  random.algorithm = "Alea";
	  random.seed = seed;
	  random.version = "0.9";
	  return random;
	}
	/**
	 * Turn arbitrary data into values [[AleaImplementation]] can use to generate
	 * random numbers.
	 *
	 * @param seed - Arbitrary data that will be used as the seed.
	 *
	 * @returns Three numbers to use as initial values for [[AleaImplementation]].
	 */


	function mashSeed() {
	  var mash = Mash();
	  var s0 = mash(" ");
	  var s1 = mash(" ");
	  var s2 = mash(" ");

	  for (var i = 0; i < arguments.length; i++) {
	    s0 -= mash(i < 0 || arguments.length <= i ? undefined : arguments[i]);

	    if (s0 < 0) {
	      s0 += 1;
	    }

	    s1 -= mash(i < 0 || arguments.length <= i ? undefined : arguments[i]);

	    if (s1 < 0) {
	      s1 += 1;
	    }

	    s2 -= mash(i < 0 || arguments.length <= i ? undefined : arguments[i]);

	    if (s2 < 0) {
	      s2 += 1;
	    }
	  }

	  return [s0, s1, s2];
	}
	/**
	 * Create a new mash function.
	 *
	 * @returns A nonpure function that takes arbitrary [[Mashable]] data and turns
	 * them into numbers.
	 */


	function Mash() {
	  var n = 0xefc8249d;
	  return function (data) {
	    var string = data.toString();

	    for (var i = 0; i < string.length; i++) {
	      n += string.charCodeAt(i);
	      var h = 0.02519603282416938 * n;
	      n = h >>> 0;
	      h -= n;
	      h *= n;
	      n = h >>> 0;
	      h -= n;
	      n += h * 0x100000000; // 2^32
	    }

	    return (n >>> 0) * 2.3283064365386963e-10; // 2^-32
	  };
	} // utility functions
	// parse ASP.Net Date pattern,
	// for example '/Date(1198908717056)/' or '/Date(1198908717056-0700)/'
	// code from http://momentjs.com/


	var ASPDateRegex = /^\/?Date\((-?\d+)/i; // Color REs

	var fullHexRE = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i;
	var shortHexRE = /^#?([a-f\d])([a-f\d])([a-f\d])$/i;
	var rgbRE = /^rgb\( *(1?\d{1,2}|2[0-4]\d|25[0-5]) *, *(1?\d{1,2}|2[0-4]\d|25[0-5]) *, *(1?\d{1,2}|2[0-4]\d|25[0-5]) *\)$/i;
	var rgbaRE = /^rgba\( *(1?\d{1,2}|2[0-4]\d|25[0-5]) *, *(1?\d{1,2}|2[0-4]\d|25[0-5]) *, *(1?\d{1,2}|2[0-4]\d|25[0-5]) *, *([01]|0?\.\d+) *\)$/i;
	/**
	 * Test whether given object is a number.
	 *
	 * @param value - Input value of unknown type.
	 *
	 * @returns True if number, false otherwise.
	 */

	function isNumber(value) {
	  return value instanceof Number || typeof value === "number";
	}
	/**
	 * Remove everything in the DOM object.
	 *
	 * @param DOMobject - Node whose child nodes will be recursively deleted.
	 */


	function recursiveDOMDelete(DOMobject) {
	  if (DOMobject) {
	    while (DOMobject.hasChildNodes() === true) {
	      var child = DOMobject.firstChild;

	      if (child) {
	        recursiveDOMDelete(child);
	        DOMobject.removeChild(child);
	      }
	    }
	  }
	}
	/**
	 * Test whether given object is a string.
	 *
	 * @param value - Input value of unknown type.
	 *
	 * @returns True if string, false otherwise.
	 */


	function isString(value) {
	  return value instanceof String || typeof value === "string";
	}
	/**
	 * Test whether given object is a object (not primitive or null).
	 *
	 * @param value - Input value of unknown type.
	 *
	 * @returns True if not null object, false otherwise.
	 */


	function isObject$1(value) {
	  return _typeof_1(value) === "object" && value !== null;
	}
	/**
	 * Test whether given object is a Date, or a String containing a Date.
	 *
	 * @param value - Input value of unknown type.
	 *
	 * @returns True if Date instance or string date representation, false otherwise.
	 */


	function isDate(value) {
	  if (value instanceof Date) {
	    return true;
	  } else if (isString(value)) {
	    // test whether this string contains a date
	    var match = ASPDateRegex.exec(value);

	    if (match) {
	      return true;
	    } else if (!isNaN(Date.parse(value))) {
	      return true;
	    }
	  }

	  return false;
	}
	/**
	 * Copy property from b to a if property present in a.
	 * If property in b explicitly set to null, delete it if `allowDeletion` set.
	 *
	 * Internal helper routine, should not be exported. Not added to `exports` for that reason.
	 *
	 * @param a - Target object.
	 * @param b - Source object.
	 * @param prop - Name of property to copy from b to a.
	 * @param allowDeletion - If true, delete property in a if explicitly set to null in b.
	 */


	function copyOrDelete(a, b, prop, allowDeletion) {
	  var doDeletion = false;

	  if (allowDeletion === true) {
	    doDeletion = b[prop] === null && a[prop] !== undefined;
	  }

	  if (doDeletion) {
	    delete a[prop];
	  } else {
	    a[prop] = b[prop]; // Remember, this is a reference copy!
	  }
	}
	/**
	 * Fill an object with a possibly partially defined other object.
	 *
	 * Only copies values for the properties already present in a.
	 * That means an object is not created on a property if only the b object has it.
	 *
	 * @param a - The object that will have it's properties updated.
	 * @param b - The object with property updates.
	 * @param allowDeletion - If true, delete properties in a that are explicitly set to null in b.
	 */


	function fillIfDefined(a, b) {
	  var allowDeletion = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

	  // NOTE: iteration of properties of a
	  // NOTE: prototype properties iterated over as well
	  for (var prop in a) {
	    if (b[prop] !== undefined) {
	      if (b[prop] === null || _typeof_1(b[prop]) !== "object") {
	        // Note: typeof null === 'object'
	        copyOrDelete(a, b, prop, allowDeletion);
	      } else {
	        var aProp = a[prop];
	        var bProp = b[prop];

	        if (isObject$1(aProp) && isObject$1(bProp)) {
	          fillIfDefined(aProp, bProp, allowDeletion);
	        }
	      }
	    }
	  }
	}
	/**
	 * Copy the values of all of the enumerable own properties from one or more source objects to a
	 * target object. Returns the target object.
	 *
	 * @param target - The target object to copy to.
	 * @param source - The source object from which to copy properties.
	 *
	 * @returns The target object.
	 */


	var extend = assign$2;
	/**
	 * Extend object a with selected properties of object b or a series of objects.
	 *
	 * @remarks
	 * Only properties with defined values are copied.
	 *
	 * @param props - Properties to be copied to a.
	 * @param a - The target.
	 * @param others - The sources.
	 *
	 * @returns Argument a.
	 */

	function selectiveExtend(props, a) {
	  if (!isArray$3(props)) {
	    throw new Error("Array with property names expected as first argument");
	  }

	  for (var _len2 = arguments.length, others = new Array(_len2 > 2 ? _len2 - 2 : 0), _key2 = 2; _key2 < _len2; _key2++) {
	    others[_key2 - 2] = arguments[_key2];
	  }

	  for (var _i = 0, _others = others; _i < _others.length; _i++) {
	    var other = _others[_i];

	    for (var p = 0; p < props.length; p++) {
	      var prop = props[p];

	      if (other && Object.prototype.hasOwnProperty.call(other, prop)) {
	        a[prop] = other[prop];
	      }
	    }
	  }

	  return a;
	}
	/**
	 * Extend object a with selected properties of object b.
	 * Only properties with defined values are copied.
	 *
	 * @remarks
	 * Previous version of this routine implied that multiple source objects could
	 * be used; however, the implementation was **wrong**. Since multiple (\>1)
	 * sources weren't used anywhere in the `vis.js` code, this has been removed
	 *
	 * @param props - Names of first-level properties to copy over.
	 * @param a - Target object.
	 * @param b - Source object.
	 * @param allowDeletion - If true, delete property in a if explicitly set to null in b.
	 *
	 * @returns Argument a.
	 */


	function selectiveDeepExtend(props, a, b) {
	  var allowDeletion = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

	  // TODO: add support for Arrays to deepExtend
	  if (isArray$3(b)) {
	    throw new TypeError("Arrays are not supported by deepExtend");
	  }

	  for (var p = 0; p < props.length; p++) {
	    var prop = props[p];

	    if (Object.prototype.hasOwnProperty.call(b, prop)) {
	      if (b[prop] && b[prop].constructor === Object) {
	        if (a[prop] === undefined) {
	          a[prop] = {};
	        }

	        if (a[prop].constructor === Object) {
	          deepExtend(a[prop], b[prop], false, allowDeletion);
	        } else {
	          copyOrDelete(a, b, prop, allowDeletion);
	        }
	      } else if (isArray$3(b[prop])) {
	        throw new TypeError("Arrays are not supported by deepExtend");
	      } else {
	        copyOrDelete(a, b, prop, allowDeletion);
	      }
	    }
	  }

	  return a;
	}
	/**
	 * Extend object `a` with properties of object `b`, ignoring properties which
	 * are explicitly specified to be excluded.
	 *
	 * @remarks
	 * The properties of `b` are considered for copying. Properties which are
	 * themselves objects are are also extended. Only properties with defined
	 * values are copied.
	 *
	 * @param propsToExclude - Names of properties which should *not* be copied.
	 * @param a - Object to extend.
	 * @param b - Object to take properties from for extension.
	 * @param allowDeletion - If true, delete properties in a that are explicitly
	 * set to null in b.
	 *
	 * @returns Argument a.
	 */


	function selectiveNotDeepExtend(propsToExclude, a, b) {
	  var allowDeletion = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

	  // TODO: add support for Arrays to deepExtend
	  // NOTE: array properties have an else-below; apparently, there is a problem here.
	  if (isArray$3(b)) {
	    throw new TypeError("Arrays are not supported by deepExtend");
	  }

	  for (var prop in b) {
	    if (!Object.prototype.hasOwnProperty.call(b, prop)) {
	      continue;
	    } // Handle local properties only


	    if (indexOf$3(propsToExclude).call(propsToExclude, prop) !== -1) {
	      continue;
	    } // In exclusion list, skip


	    if (b[prop] && b[prop].constructor === Object) {
	      if (a[prop] === undefined) {
	        a[prop] = {};
	      }

	      if (a[prop].constructor === Object) {
	        deepExtend(a[prop], b[prop]); // NOTE: allowDeletion not propagated!
	      } else {
	        copyOrDelete(a, b, prop, allowDeletion);
	      }
	    } else if (isArray$3(b[prop])) {
	      a[prop] = [];

	      for (var i = 0; i < b[prop].length; i++) {
	        a[prop].push(b[prop][i]);
	      }
	    } else {
	      copyOrDelete(a, b, prop, allowDeletion);
	    }
	  }

	  return a;
	}
	/**
	 * Deep extend an object a with the properties of object b.
	 *
	 * @param a - Target object.
	 * @param b - Source object.
	 * @param protoExtend - If true, the prototype values will also be extended.
	 * (That is the options objects that inherit from others will also get the
	 * inherited options).
	 * @param allowDeletion - If true, the values of fields that are null will be deleted.
	 *
	 * @returns Argument a.
	 */


	function deepExtend(a, b) {
	  var protoExtend = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
	  var allowDeletion = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

	  for (var prop in b) {
	    if (Object.prototype.hasOwnProperty.call(b, prop) || protoExtend === true) {
	      if (_typeof_1(b[prop]) === "object" && b[prop] !== null && getPrototypeOf$5(b[prop]) === Object.prototype) {
	        if (a[prop] === undefined) {
	          a[prop] = deepExtend({}, b[prop], protoExtend); // NOTE: allowDeletion not propagated!
	        } else if (_typeof_1(a[prop]) === "object" && a[prop] !== null && getPrototypeOf$5(a[prop]) === Object.prototype) {
	          deepExtend(a[prop], b[prop], protoExtend); // NOTE: allowDeletion not propagated!
	        } else {
	          copyOrDelete(a, b, prop, allowDeletion);
	        }
	      } else if (isArray$3(b[prop])) {
	        var _context;

	        a[prop] = slice$3(_context = b[prop]).call(_context);
	      } else {
	        copyOrDelete(a, b, prop, allowDeletion);
	      }
	    }
	  }

	  return a;
	}
	/**
	 * Test whether all elements in two arrays are equal.
	 *
	 * @param a - First array.
	 * @param b - Second array.
	 *
	 * @returns True if both arrays have the same length and same elements (1 = '1').
	 */


	function equalArray(a, b) {
	  if (a.length !== b.length) {
	    return false;
	  }

	  for (var i = 0, len = a.length; i < len; i++) {
	    if (a[i] != b[i]) {
	      return false;
	    }
	  }

	  return true;
	}
	/**
	 * Get the type of an object, for example exports.getType([]) returns 'Array'.
	 *
	 * @param object - Input value of unknown type.
	 *
	 * @returns Detected type.
	 */


	function getType(object) {
	  var type = _typeof_1(object);

	  if (type === "object") {
	    if (object === null) {
	      return "null";
	    }

	    if (object instanceof Boolean) {
	      return "Boolean";
	    }

	    if (object instanceof Number) {
	      return "Number";
	    }

	    if (object instanceof String) {
	      return "String";
	    }

	    if (isArray$3(object)) {
	      return "Array";
	    }

	    if (object instanceof Date) {
	      return "Date";
	    }

	    return "Object";
	  }

	  if (type === "number") {
	    return "Number";
	  }

	  if (type === "boolean") {
	    return "Boolean";
	  }

	  if (type === "string") {
	    return "String";
	  }

	  if (type === undefined) {
	    return "undefined";
	  }

	  return type;
	}
	/**
	 * Used to extend an array and copy it. This is used to propagate paths recursively.
	 *
	 * @param arr - First part.
	 * @param newValue - The value to be aadded into the array.
	 *
	 * @returns A new array with all items from arr and newValue (which is last).
	 */


	function copyAndExtendArray(arr, newValue) {
	  var _context2;

	  return concat$2(_context2 = []).call(_context2, toConsumableArray(arr), [newValue]);
	}
	/**
	 * Used to extend an array and copy it. This is used to propagate paths recursively.
	 *
	 * @param arr - The array to be copied.
	 *
	 * @returns Shallow copy of arr.
	 */


	function copyArray(arr) {
	  return slice$3(arr).call(arr);
	}
	/**
	 * Retrieve the absolute left value of a DOM element.
	 *
	 * @param elem - A dom element, for example a div.
	 *
	 * @returns The absolute left position of this element in the browser page.
	 */


	function getAbsoluteLeft(elem) {
	  return elem.getBoundingClientRect().left;
	}
	/**
	 * Retrieve the absolute right value of a DOM element.
	 *
	 * @param elem - A dom element, for example a div.
	 *
	 * @returns The absolute right position of this element in the browser page.
	 */


	function getAbsoluteRight(elem) {
	  return elem.getBoundingClientRect().right;
	}
	/**
	 * Retrieve the absolute top value of a DOM element.
	 *
	 * @param elem - A dom element, for example a div.
	 *
	 * @returns The absolute top position of this element in the browser page.
	 */


	function getAbsoluteTop(elem) {
	  return elem.getBoundingClientRect().top;
	}
	/**
	 * Add a className to the given elements style.
	 *
	 * @param elem - The element to which the classes will be added.
	 * @param classNames - Space separated list of classes.
	 */


	function addClassName(elem, classNames) {
	  var classes = elem.className.split(" ");
	  var newClasses = classNames.split(" ");
	  classes = concat$2(classes).call(classes, filter$2(newClasses).call(newClasses, function (className) {
	    return indexOf$3(classes).call(classes, className) < 0;
	  }));
	  elem.className = classes.join(" ");
	}
	/**
	 * Remove a className from the given elements style.
	 *
	 * @param elem - The element from which the classes will be removed.
	 * @param classNames - Space separated list of classes.
	 */


	function removeClassName(elem, classNames) {
	  var classes = elem.className.split(" ");
	  var oldClasses = classNames.split(" ");
	  classes = filter$2(classes).call(classes, function (className) {
	    return indexOf$3(oldClasses).call(oldClasses, className) < 0;
	  });
	  elem.className = classes.join(" ");
	}
	/**
	 * For each method for both arrays and objects.
	 * In case of an array, the built-in Array.forEach() is applied (**No, it's not!**).
	 * In case of an Object, the method loops over all properties of the object.
	 *
	 * @param object - An Object or Array to be iterated over.
	 * @param callback - Array.forEach-like callback.
	 */


	function forEach$4(object, callback) {
	  if (isArray$3(object)) {
	    // array
	    var len = object.length;

	    for (var i = 0; i < len; i++) {
	      callback(object[i], i, object);
	    }
	  } else {
	    // object
	    for (var key in object) {
	      if (Object.prototype.hasOwnProperty.call(object, key)) {
	        callback(object[key], key, object);
	      }
	    }
	  }
	}
	/**
	 * Convert an object into an array: all objects properties are put into the array. The resulting array is unordered.
	 *
	 * @param o - Object that contains the properties and methods.
	 *
	 * @returns An array of unordered values.
	 */


	var toArray = values$5;
	/**
	 * Update a property in an object.
	 *
	 * @param object - The object whose property will be updated.
	 * @param key - Name of the property to be updated.
	 * @param value - The new value to be assigned.
	 *
	 * @returns Whether the value was updated (true) or already strictly the same in the original object (false).
	 */

	function updateProperty(object, key, value) {
	  if (object[key] !== value) {
	    object[key] = value;
	    return true;
	  } else {
	    return false;
	  }
	}
	/**
	 * Throttle the given function to be only executed once per animation frame.
	 *
	 * @param fn - The original function.
	 *
	 * @returns The throttled function.
	 */


	function throttle(fn) {
	  var scheduled = false;
	  return function () {
	    if (!scheduled) {
	      scheduled = true;
	      requestAnimationFrame(function () {
	        scheduled = false;
	        fn();
	      });
	    }
	  };
	}
	/**
	 * Add and event listener. Works for all browsers.
	 *
	 * @param element - The element to bind the event listener to.
	 * @param action - Same as Element.addEventListener(action, —, —).
	 * @param listener - Same as Element.addEventListener(—, listener, —).
	 * @param useCapture - Same as Element.addEventListener(—, —, useCapture).
	 */


	function addEventListener(element, action, listener, useCapture) {
	  if (element.addEventListener) {
	    var _context3;

	    if (useCapture === undefined) {
	      useCapture = false;
	    }

	    if (action === "mousewheel" && indexOf$3(_context3 = navigator.userAgent).call(_context3, "Firefox") >= 0) {
	      action = "DOMMouseScroll"; // For Firefox
	    }

	    element.addEventListener(action, listener, useCapture);
	  } else {
	    // @TODO: IE types? Does anyone care?
	    element.attachEvent("on" + action, listener); // IE browsers
	  }
	}
	/**
	 * Remove an event listener from an element.
	 *
	 * @param element - The element to bind the event listener to.
	 * @param action - Same as Element.removeEventListener(action, —, —).
	 * @param listener - Same as Element.removeEventListener(—, listener, —).
	 * @param useCapture - Same as Element.removeEventListener(—, —, useCapture).
	 */


	function removeEventListener(element, action, listener, useCapture) {
	  if (element.removeEventListener) {
	    var _context4;

	    // non-IE browsers
	    if (useCapture === undefined) {
	      useCapture = false;
	    }

	    if (action === "mousewheel" && indexOf$3(_context4 = navigator.userAgent).call(_context4, "Firefox") >= 0) {
	      action = "DOMMouseScroll"; // For Firefox
	    }

	    element.removeEventListener(action, listener, useCapture);
	  } else {
	    // @TODO: IE types? Does anyone care?
	    element.detachEvent("on" + action, listener); // IE browsers
	  }
	}
	/**
	 * Cancels the event's default action if it is cancelable, without stopping further propagation of the event.
	 *
	 * @param event - The event whose default action should be prevented.
	 */


	function preventDefault(event) {
	  if (!event) {
	    event = window.event;
	  }

	  if (!event) ;else if (event.preventDefault) {
	    event.preventDefault(); // non-IE browsers
	  } else {
	    // @TODO: IE types? Does anyone care?
	    event.returnValue = false; // IE browsers
	  }
	}
	/**
	 * Get HTML element which is the target of the event.
	 *
	 * @param event - The event.
	 *
	 * @returns The element or null if not obtainable.
	 */


	function getTarget() {
	  var event = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : window.event;
	  // code from http://www.quirksmode.org/js/events_properties.html
	  // @TODO: EventTarget can be almost anything, is it okay to return only Elements?
	  var target = null;
	  if (!event) ;else if (event.target) {
	    target = event.target;
	  } else if (event.srcElement) {
	    target = event.srcElement;
	  }

	  if (!(target instanceof Element)) {
	    return null;
	  }

	  if (target.nodeType != null && target.nodeType == 3) {
	    // defeat Safari bug
	    target = target.parentNode;

	    if (!(target instanceof Element)) {
	      return null;
	    }
	  }

	  return target;
	}
	/**
	 * Check if given element contains given parent somewhere in the DOM tree.
	 *
	 * @param element - The element to be tested.
	 * @param parent - The ancestor (not necessarily parent) of the element.
	 *
	 * @returns True if parent is an ancestor of the element, false otherwise.
	 */


	function hasParent(element, parent) {
	  var elem = element;

	  while (elem) {
	    if (elem === parent) {
	      return true;
	    } else if (elem.parentNode) {
	      elem = elem.parentNode;
	    } else {
	      return false;
	    }
	  }

	  return false;
	}

	var option = {
	  /**
	   * Convert a value into a boolean.
	   *
	   * @param value - Value to be converted intoboolean, a function will be executed as `(() => unknown)`.
	   * @param defaultValue - If the value or the return value of the function == null then this will be returned.
	   *
	   * @returns Corresponding boolean value, if none then the default value, if none then null.
	   */
	  asBoolean: function asBoolean(value, defaultValue) {
	    if (typeof value == "function") {
	      value = value();
	    }

	    if (value != null) {
	      return value != false;
	    }

	    return defaultValue || null;
	  },

	  /**
	   * Convert a value into a number.
	   *
	   * @param value - Value to be converted intonumber, a function will be executed as `(() => unknown)`.
	   * @param defaultValue - If the value or the return value of the function == null then this will be returned.
	   *
	   * @returns Corresponding **boxed** number value, if none then the default value, if none then null.
	   */
	  asNumber: function asNumber(value, defaultValue) {
	    if (typeof value == "function") {
	      value = value();
	    }

	    if (value != null) {
	      return Number(value) || defaultValue || null;
	    }

	    return defaultValue || null;
	  },

	  /**
	   * Convert a value into a string.
	   *
	   * @param value - Value to be converted intostring, a function will be executed as `(() => unknown)`.
	   * @param defaultValue - If the value or the return value of the function == null then this will be returned.
	   *
	   * @returns Corresponding **boxed** string value, if none then the default value, if none then null.
	   */
	  asString: function asString(value, defaultValue) {
	    if (typeof value == "function") {
	      value = value();
	    }

	    if (value != null) {
	      return String(value);
	    }

	    return defaultValue || null;
	  },

	  /**
	   * Convert a value into a size.
	   *
	   * @param value - Value to be converted intosize, a function will be executed as `(() => unknown)`.
	   * @param defaultValue - If the value or the return value of the function == null then this will be returned.
	   *
	   * @returns Corresponding string value (number + 'px'), if none then the default value, if none then null.
	   */
	  asSize: function asSize(value, defaultValue) {
	    if (typeof value == "function") {
	      value = value();
	    }

	    if (isString(value)) {
	      return value;
	    } else if (isNumber(value)) {
	      return value + "px";
	    } else {
	      return defaultValue || null;
	    }
	  },

	  /**
	   * Convert a value into a DOM Element.
	   *
	   * @param value - Value to be converted into DOM Element, a function will be executed as `(() => unknown)`.
	   * @param defaultValue - If the value or the return value of the function == null then this will be returned.
	   *
	   * @returns The DOM Element, if none then the default value, if none then null.
	   */
	  asElement: function asElement(value, defaultValue) {
	    if (typeof value == "function") {
	      value = value();
	    }

	    return value || defaultValue || null;
	  }
	};
	/**
	 * Convert hex color string into RGB color object.
	 *
	 * @remarks
	 * {@link http://stackoverflow.com/questions/5623838/rgb-to-hex-and-hex-to-rgb}
	 *
	 * @param hex - Hex color string (3 or 6 digits, with or without #).
	 *
	 * @returns RGB color object.
	 */

	function hexToRGB(hex) {
	  var result;

	  switch (hex.length) {
	    case 3:
	    case 4:
	      result = shortHexRE.exec(hex);
	      return result ? {
	        r: _parseInt$2(result[1] + result[1], 16),
	        g: _parseInt$2(result[2] + result[2], 16),
	        b: _parseInt$2(result[3] + result[3], 16)
	      } : null;

	    case 6:
	    case 7:
	      result = fullHexRE.exec(hex);
	      return result ? {
	        r: _parseInt$2(result[1], 16),
	        g: _parseInt$2(result[2], 16),
	        b: _parseInt$2(result[3], 16)
	      } : null;

	    default:
	      return null;
	  }
	}
	/**
	 * This function takes string color in hex or RGB format and adds the opacity, RGBA is passed through unchanged.
	 *
	 * @param color - The color string (hex, RGB, RGBA).
	 * @param opacity - The new opacity.
	 *
	 * @returns RGBA string, for example 'rgba(255, 0, 127, 0.3)'.
	 */


	function overrideOpacity(color, opacity) {
	  if (indexOf$3(color).call(color, "rgba") !== -1) {
	    return color;
	  } else if (indexOf$3(color).call(color, "rgb") !== -1) {
	    var rgb = color.substr(indexOf$3(color).call(color, "(") + 1).replace(")", "").split(",");
	    return "rgba(" + rgb[0] + "," + rgb[1] + "," + rgb[2] + "," + opacity + ")";
	  } else {
	    var _rgb = hexToRGB(color);

	    if (_rgb == null) {
	      return color;
	    } else {
	      return "rgba(" + _rgb.r + "," + _rgb.g + "," + _rgb.b + "," + opacity + ")";
	    }
	  }
	}
	/**
	 * Convert RGB \<0, 255\> into hex color string.
	 *
	 * @param red - Red channel.
	 * @param green - Green channel.
	 * @param blue - Blue channel.
	 *
	 * @returns Hex color string (for example: '#0acdc0').
	 */


	function RGBToHex(red, green, blue) {
	  var _context5;

	  return "#" + slice$3(_context5 = ((1 << 24) + (red << 16) + (green << 8) + blue).toString(16)).call(_context5, 1);
	}
	/**
	 * Parse a color property into an object with border, background, and highlight colors.
	 *
	 * @param inputColor - Shorthand color string or input color object.
	 * @param defaultColor - Full color object to fill in missing values in inputColor.
	 *
	 * @returns Color object.
	 */


	function parseColor(inputColor, defaultColor) {
	  if (isString(inputColor)) {
	    var colorStr = inputColor;

	    if (isValidRGB(colorStr)) {
	      var _context6;

	      var rgb = map$5(_context6 = colorStr.substr(4).substr(0, colorStr.length - 5).split(",")).call(_context6, function (value) {
	        return _parseInt$2(value);
	      });

	      colorStr = RGBToHex(rgb[0], rgb[1], rgb[2]);
	    }

	    if (isValidHex(colorStr) === true) {
	      var hsv = hexToHSV(colorStr);
	      var lighterColorHSV = {
	        h: hsv.h,
	        s: hsv.s * 0.8,
	        v: Math.min(1, hsv.v * 1.02)
	      };
	      var darkerColorHSV = {
	        h: hsv.h,
	        s: Math.min(1, hsv.s * 1.25),
	        v: hsv.v * 0.8
	      };
	      var darkerColorHex = HSVToHex(darkerColorHSV.h, darkerColorHSV.s, darkerColorHSV.v);
	      var lighterColorHex = HSVToHex(lighterColorHSV.h, lighterColorHSV.s, lighterColorHSV.v);
	      return {
	        background: colorStr,
	        border: darkerColorHex,
	        highlight: {
	          background: lighterColorHex,
	          border: darkerColorHex
	        },
	        hover: {
	          background: lighterColorHex,
	          border: darkerColorHex
	        }
	      };
	    } else {
	      return {
	        background: colorStr,
	        border: colorStr,
	        highlight: {
	          background: colorStr,
	          border: colorStr
	        },
	        hover: {
	          background: colorStr,
	          border: colorStr
	        }
	      };
	    }
	  } else {
	    if (defaultColor) {
	      var color = {
	        background: inputColor.background || defaultColor.background,
	        border: inputColor.border || defaultColor.border,
	        highlight: isString(inputColor.highlight) ? {
	          border: inputColor.highlight,
	          background: inputColor.highlight
	        } : {
	          background: inputColor.highlight && inputColor.highlight.background || defaultColor.highlight.background,
	          border: inputColor.highlight && inputColor.highlight.border || defaultColor.highlight.border
	        },
	        hover: isString(inputColor.hover) ? {
	          border: inputColor.hover,
	          background: inputColor.hover
	        } : {
	          border: inputColor.hover && inputColor.hover.border || defaultColor.hover.border,
	          background: inputColor.hover && inputColor.hover.background || defaultColor.hover.background
	        }
	      };
	      return color;
	    } else {
	      var _color = {
	        background: inputColor.background || undefined,
	        border: inputColor.border || undefined,
	        highlight: isString(inputColor.highlight) ? {
	          border: inputColor.highlight,
	          background: inputColor.highlight
	        } : {
	          background: inputColor.highlight && inputColor.highlight.background || undefined,
	          border: inputColor.highlight && inputColor.highlight.border || undefined
	        },
	        hover: isString(inputColor.hover) ? {
	          border: inputColor.hover,
	          background: inputColor.hover
	        } : {
	          border: inputColor.hover && inputColor.hover.border || undefined,
	          background: inputColor.hover && inputColor.hover.background || undefined
	        }
	      };
	      return _color;
	    }
	  }
	}
	/**
	 * Convert RGB \<0, 255\> into HSV object.
	 *
	 * @remarks
	 * {@link http://www.javascripter.net/faq/rgb2hsv.htm}
	 *
	 * @param red - Red channel.
	 * @param green - Green channel.
	 * @param blue - Blue channel.
	 *
	 * @returns HSV color object.
	 */


	function RGBToHSV(red, green, blue) {
	  red = red / 255;
	  green = green / 255;
	  blue = blue / 255;
	  var minRGB = Math.min(red, Math.min(green, blue));
	  var maxRGB = Math.max(red, Math.max(green, blue)); // Black-gray-white

	  if (minRGB === maxRGB) {
	    return {
	      h: 0,
	      s: 0,
	      v: minRGB
	    };
	  } // Colors other than black-gray-white:


	  var d = red === minRGB ? green - blue : blue === minRGB ? red - green : blue - red;
	  var h = red === minRGB ? 3 : blue === minRGB ? 1 : 5;
	  var hue = 60 * (h - d / (maxRGB - minRGB)) / 360;
	  var saturation = (maxRGB - minRGB) / maxRGB;
	  var value = maxRGB;
	  return {
	    h: hue,
	    s: saturation,
	    v: value
	  };
	}

	var cssUtil = {
	  // split a string with css styles into an object with key/values
	  split: function split(cssText) {
	    var _context7;

	    var styles = {};

	    forEach$3(_context7 = cssText.split(";")).call(_context7, function (style) {
	      if (trim$2(style).call(style) != "") {
	        var _context8, _context9;

	        var parts = style.split(":");

	        var key = trim$2(_context8 = parts[0]).call(_context8);

	        var value = trim$2(_context9 = parts[1]).call(_context9);

	        styles[key] = value;
	      }
	    });

	    return styles;
	  },
	  // build a css text string from an object with key/values
	  join: function join(styles) {
	    var _context10;

	    return map$5(_context10 = keys$6(styles)).call(_context10, function (key) {
	      return key + ": " + styles[key];
	    }).join("; ");
	  }
	};
	/**
	 * Append a string with css styles to an element.
	 *
	 * @param element - The element that will receive new styles.
	 * @param cssText - The styles to be appended.
	 */

	function addCssText(element, cssText) {
	  var currentStyles = cssUtil.split(element.style.cssText);
	  var newStyles = cssUtil.split(cssText);

	  var styles = _objectSpread(_objectSpread({}, currentStyles), newStyles);

	  element.style.cssText = cssUtil.join(styles);
	}
	/**
	 * Remove a string with css styles from an element.
	 *
	 * @param element - The element from which styles should be removed.
	 * @param cssText - The styles to be removed.
	 */


	function removeCssText(element, cssText) {
	  var styles = cssUtil.split(element.style.cssText);
	  var removeStyles = cssUtil.split(cssText);

	  for (var key in removeStyles) {
	    if (Object.prototype.hasOwnProperty.call(removeStyles, key)) {
	      delete styles[key];
	    }
	  }

	  element.style.cssText = cssUtil.join(styles);
	}
	/**
	 * Convert HSV \<0, 1\> into RGB color object.
	 *
	 * @remarks
	 * {@link https://gist.github.com/mjijackson/5311256}
	 *
	 * @param h - Hue.
	 * @param s - Saturation.
	 * @param v - Value.
	 *
	 * @returns RGB color object.
	 */


	function HSVToRGB(h, s, v) {
	  var r;
	  var g;
	  var b;
	  var i = Math.floor(h * 6);
	  var f = h * 6 - i;
	  var p = v * (1 - s);
	  var q = v * (1 - f * s);
	  var t = v * (1 - (1 - f) * s);

	  switch (i % 6) {
	    case 0:
	      r = v, g = t, b = p;
	      break;

	    case 1:
	      r = q, g = v, b = p;
	      break;

	    case 2:
	      r = p, g = v, b = t;
	      break;

	    case 3:
	      r = p, g = q, b = v;
	      break;

	    case 4:
	      r = t, g = p, b = v;
	      break;

	    case 5:
	      r = v, g = p, b = q;
	      break;
	  }

	  return {
	    r: Math.floor(r * 255),
	    g: Math.floor(g * 255),
	    b: Math.floor(b * 255)
	  };
	}
	/**
	 * Convert HSV \<0, 1\> into hex color string.
	 *
	 * @param h - Hue.
	 * @param s - Saturation.
	 * @param v - Value.
	 *
	 * @returns Hex color string.
	 */


	function HSVToHex(h, s, v) {
	  var rgb = HSVToRGB(h, s, v);
	  return RGBToHex(rgb.r, rgb.g, rgb.b);
	}
	/**
	 * Convert hex color string into HSV \<0, 1\>.
	 *
	 * @param hex - Hex color string.
	 *
	 * @returns HSV color object.
	 */


	function hexToHSV(hex) {
	  var rgb = hexToRGB(hex);

	  if (!rgb) {
	    throw new TypeError("'".concat(hex, "' is not a valid color."));
	  }

	  return RGBToHSV(rgb.r, rgb.g, rgb.b);
	}
	/**
	 * Validate hex color string.
	 *
	 * @param hex - Unknown string that may contain a color.
	 *
	 * @returns True if the string is valid, false otherwise.
	 */


	function isValidHex(hex) {
	  var isOk = /(^#[0-9A-F]{6}$)|(^#[0-9A-F]{3}$)/i.test(hex);
	  return isOk;
	}
	/**
	 * Validate RGB color string.
	 *
	 * @param rgb - Unknown string that may contain a color.
	 *
	 * @returns True if the string is valid, false otherwise.
	 */


	function isValidRGB(rgb) {
	  return rgbRE.test(rgb);
	}
	/**
	 * Validate RGBA color string.
	 *
	 * @param rgba - Unknown string that may contain a color.
	 *
	 * @returns True if the string is valid, false otherwise.
	 */


	function isValidRGBA(rgba) {
	  return rgbaRE.test(rgba);
	}
	/**
	 * This recursively redirects the prototype of JSON objects to the referenceObject.
	 * This is used for default options.
	 *
	 * @param fields - Names of properties to be bridged.
	 * @param referenceObject - The original object.
	 *
	 * @returns A new object inheriting from the referenceObject.
	 */


	function selectiveBridgeObject(fields, referenceObject) {
	  if (referenceObject !== null && _typeof_1(referenceObject) === "object") {
	    // !!! typeof null === 'object'
	    var objectTo = create$4(referenceObject);

	    for (var i = 0; i < fields.length; i++) {
	      if (Object.prototype.hasOwnProperty.call(referenceObject, fields[i])) {
	        if (_typeof_1(referenceObject[fields[i]]) == "object") {
	          objectTo[fields[i]] = bridgeObject(referenceObject[fields[i]]);
	        }
	      }
	    }

	    return objectTo;
	  } else {
	    return null;
	  }
	}
	/**
	 * This recursively redirects the prototype of JSON objects to the referenceObject.
	 * This is used for default options.
	 *
	 * @param referenceObject - The original object.
	 *
	 * @returns The Element if the referenceObject is an Element, or a new object inheriting from the referenceObject.
	 */


	function bridgeObject(referenceObject) {
	  if (referenceObject === null || _typeof_1(referenceObject) !== "object") {
	    return null;
	  }

	  if (referenceObject instanceof Element) {
	    // Avoid bridging DOM objects
	    return referenceObject;
	  }

	  var objectTo = create$4(referenceObject);

	  for (var i in referenceObject) {
	    if (Object.prototype.hasOwnProperty.call(referenceObject, i)) {
	      if (_typeof_1(referenceObject[i]) == "object") {
	        objectTo[i] = bridgeObject(referenceObject[i]);
	      }
	    }
	  }

	  return objectTo;
	}
	/**
	 * This method provides a stable sort implementation, very fast for presorted data.
	 *
	 * @param a - The array to be sorted (in-place).
	 * @param compare - An order comparator.
	 *
	 * @returns The argument a.
	 */


	function insertSort(a, compare) {
	  for (var i = 0; i < a.length; i++) {
	    var k = a[i];
	    var j = void 0;

	    for (j = i; j > 0 && compare(k, a[j - 1]) < 0; j--) {
	      a[j] = a[j - 1];
	    }

	    a[j] = k;
	  }

	  return a;
	}
	/**
	 * This is used to set the options of subobjects in the options object.
	 *
	 * A requirement of these subobjects is that they have an 'enabled' element
	 * which is optional for the user but mandatory for the program.
	 *
	 * The added value here of the merge is that option 'enabled' is set as required.
	 *
	 * @param mergeTarget - Either this.options or the options used for the groups.
	 * @param options - Options.
	 * @param option - Option key in the options argument.
	 * @param globalOptions - Global options, passed in to determine value of option 'enabled'.
	 */


	function mergeOptions(mergeTarget, options, option) {
	  var globalOptions = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

	  // Local helpers
	  var isPresent = function isPresent(obj) {
	    return obj !== null && obj !== undefined;
	  };

	  var isObject = function isObject(obj) {
	    return obj !== null && _typeof_1(obj) === "object";
	  }; // https://stackoverflow.com/a/34491287/1223531


	  var isEmpty = function isEmpty(obj) {
	    for (var x in obj) {
	      if (Object.prototype.hasOwnProperty.call(obj, x)) {
	        return false;
	      }
	    }

	    return true;
	  }; // Guards


	  if (!isObject(mergeTarget)) {
	    throw new Error("Parameter mergeTarget must be an object");
	  }

	  if (!isObject(options)) {
	    throw new Error("Parameter options must be an object");
	  }

	  if (!isPresent(option)) {
	    throw new Error("Parameter option must have a value");
	  }

	  if (!isObject(globalOptions)) {
	    throw new Error("Parameter globalOptions must be an object");
	  } //
	  // Actual merge routine, separated from main logic
	  // Only a single level of options is merged. Deeper levels are ref'd. This may actually be an issue.
	  //


	  var doMerge = function doMerge(target, options, option) {
	    if (!isObject(target[option])) {
	      target[option] = {};
	    }

	    var src = options[option];
	    var dst = target[option];

	    for (var prop in src) {
	      if (Object.prototype.hasOwnProperty.call(src, prop)) {
	        dst[prop] = src[prop];
	      }
	    }
	  }; // Local initialization


	  var srcOption = options[option];
	  var globalPassed = isObject(globalOptions) && !isEmpty(globalOptions);
	  var globalOption = globalPassed ? globalOptions[option] : undefined;
	  var globalEnabled = globalOption ? globalOption.enabled : undefined; /////////////////////////////////////////
	  // Main routine
	  /////////////////////////////////////////

	  if (srcOption === undefined) {
	    return; // Nothing to do
	  }

	  if (typeof srcOption === "boolean") {
	    if (!isObject(mergeTarget[option])) {
	      mergeTarget[option] = {};
	    }

	    mergeTarget[option].enabled = srcOption;
	    return;
	  }

	  if (srcOption === null && !isObject(mergeTarget[option])) {
	    // If possible, explicit copy from globals
	    if (isPresent(globalOption)) {
	      mergeTarget[option] = create$4(globalOption);
	    } else {
	      return; // Nothing to do
	    }
	  }

	  if (!isObject(srcOption)) {
	    return;
	  } //
	  // Ensure that 'enabled' is properly set. It is required internally
	  // Note that the value from options will always overwrite the existing value
	  //


	  var enabled = true; // default value

	  if (srcOption.enabled !== undefined) {
	    enabled = srcOption.enabled;
	  } else {
	    // Take from globals, if present
	    if (globalEnabled !== undefined) {
	      enabled = globalOption.enabled;
	    }
	  }

	  doMerge(mergeTarget, options, option);
	  mergeTarget[option].enabled = enabled;
	}
	/**
	 * This function does a binary search for a visible item in a sorted list. If we find a visible item, the code that uses
	 * this function will then iterate in both directions over this sorted list to find all visible items.
	 *
	 * @param orderedItems - Items ordered by start.
	 * @param comparator - -1 is lower, 0 is equal, 1 is higher.
	 * @param field - Property name on an item (That is item[field]).
	 * @param field2 - Second property name on an item (That is item[field][field2]).
	 *
	 * @returns Index of the found item or -1 if nothing was found.
	 */


	function binarySearchCustom(orderedItems, comparator, field, field2) {
	  var maxIterations = 10000;
	  var iteration = 0;
	  var low = 0;
	  var high = orderedItems.length - 1;

	  while (low <= high && iteration < maxIterations) {
	    var middle = Math.floor((low + high) / 2);
	    var item = orderedItems[middle];
	    var value = field2 === undefined ? item[field] : item[field][field2];
	    var searchResult = comparator(value);

	    if (searchResult == 0) {
	      // jihaa, found a visible item!
	      return middle;
	    } else if (searchResult == -1) {
	      // it is too small --> increase low
	      low = middle + 1;
	    } else {
	      // it is too big --> decrease high
	      high = middle - 1;
	    }

	    iteration++;
	  }

	  return -1;
	}
	/**
	 * This function does a binary search for a specific value in a sorted array.
	 * If it does not exist but is in between of two values, we return either the
	 * one before or the one after, depending on user input If it is found, we
	 * return the index, else -1.
	 *
	 * @param orderedItems - Sorted array.
	 * @param target - The searched value.
	 * @param field - Name of the property in items to be searched.
	 * @param sidePreference - If the target is between two values, should the index of the before or the after be returned?
	 * @param comparator - An optional comparator, returning -1, 0, 1 for \<, ===, \>.
	 *
	 * @returns The index of found value or -1 if nothing was found.
	 */


	function binarySearchValue(orderedItems, target, field, sidePreference, comparator) {
	  var maxIterations = 10000;
	  var iteration = 0;
	  var low = 0;
	  var high = orderedItems.length - 1;
	  var prevValue;
	  var value;
	  var nextValue;
	  var middle;
	  comparator = comparator != undefined ? comparator : function (a, b) {
	    return a == b ? 0 : a < b ? -1 : 1;
	  };

	  while (low <= high && iteration < maxIterations) {
	    // get a new guess
	    middle = Math.floor(0.5 * (high + low));
	    prevValue = orderedItems[Math.max(0, middle - 1)][field];
	    value = orderedItems[middle][field];
	    nextValue = orderedItems[Math.min(orderedItems.length - 1, middle + 1)][field];

	    if (comparator(value, target) == 0) {
	      // we found the target
	      return middle;
	    } else if (comparator(prevValue, target) < 0 && comparator(value, target) > 0) {
	      // target is in between of the previous and the current
	      return sidePreference == "before" ? Math.max(0, middle - 1) : middle;
	    } else if (comparator(value, target) < 0 && comparator(nextValue, target) > 0) {
	      // target is in between of the current and the next
	      return sidePreference == "before" ? middle : Math.min(orderedItems.length - 1, middle + 1);
	    } else {
	      // didnt find the target, we need to change our boundaries.
	      if (comparator(value, target) < 0) {
	        // it is too small --> increase low
	        low = middle + 1;
	      } else {
	        // it is too big --> decrease high
	        high = middle - 1;
	      }
	    }

	    iteration++;
	  } // didnt find anything. Return -1.


	  return -1;
	}
	/*
	 * Easing Functions.
	 * Only considering the t value for the range [0, 1] => [0, 1].
	 *
	 * Inspiration: from http://gizma.com/easing/
	 * https://gist.github.com/gre/1650294
	 */


	var easingFunctions = {
	  /**
	   * Provides no easing and no acceleration.
	   *
	   * @param t - Time.
	   *
	   * @returns Value at time t.
	   */
	  linear: function linear(t) {
	    return t;
	  },

	  /**
	   * Accelerate from zero velocity.
	   *
	   * @param t - Time.
	   *
	   * @returns Value at time t.
	   */
	  easeInQuad: function easeInQuad(t) {
	    return t * t;
	  },

	  /**
	   * Decelerate to zero velocity.
	   *
	   * @param t - Time.
	   *
	   * @returns Value at time t.
	   */
	  easeOutQuad: function easeOutQuad(t) {
	    return t * (2 - t);
	  },

	  /**
	   * Accelerate until halfway, then decelerate.
	   *
	   * @param t - Time.
	   *
	   * @returns Value at time t.
	   */
	  easeInOutQuad: function easeInOutQuad(t) {
	    return t < 0.5 ? 2 * t * t : -1 + (4 - 2 * t) * t;
	  },

	  /**
	   * Accelerate from zero velocity.
	   *
	   * @param t - Time.
	   *
	   * @returns Value at time t.
	   */
	  easeInCubic: function easeInCubic(t) {
	    return t * t * t;
	  },

	  /**
	   * Decelerate to zero velocity.
	   *
	   * @param t - Time.
	   *
	   * @returns Value at time t.
	   */
	  easeOutCubic: function easeOutCubic(t) {
	    return --t * t * t + 1;
	  },

	  /**
	   * Accelerate until halfway, then decelerate.
	   *
	   * @param t - Time.
	   *
	   * @returns Value at time t.
	   */
	  easeInOutCubic: function easeInOutCubic(t) {
	    return t < 0.5 ? 4 * t * t * t : (t - 1) * (2 * t - 2) * (2 * t - 2) + 1;
	  },

	  /**
	   * Accelerate from zero velocity.
	   *
	   * @param t - Time.
	   *
	   * @returns Value at time t.
	   */
	  easeInQuart: function easeInQuart(t) {
	    return t * t * t * t;
	  },

	  /**
	   * Decelerate to zero velocity.
	   *
	   * @param t - Time.
	   *
	   * @returns Value at time t.
	   */
	  easeOutQuart: function easeOutQuart(t) {
	    return 1 - --t * t * t * t;
	  },

	  /**
	   * Accelerate until halfway, then decelerate.
	   *
	   * @param t - Time.
	   *
	   * @returns Value at time t.
	   */
	  easeInOutQuart: function easeInOutQuart(t) {
	    return t < 0.5 ? 8 * t * t * t * t : 1 - 8 * --t * t * t * t;
	  },

	  /**
	   * Accelerate from zero velocity.
	   *
	   * @param t - Time.
	   *
	   * @returns Value at time t.
	   */
	  easeInQuint: function easeInQuint(t) {
	    return t * t * t * t * t;
	  },

	  /**
	   * Decelerate to zero velocity.
	   *
	   * @param t - Time.
	   *
	   * @returns Value at time t.
	   */
	  easeOutQuint: function easeOutQuint(t) {
	    return 1 + --t * t * t * t * t;
	  },

	  /**
	   * Accelerate until halfway, then decelerate.
	   *
	   * @param t - Time.
	   *
	   * @returns Value at time t.
	   */
	  easeInOutQuint: function easeInOutQuint(t) {
	    return t < 0.5 ? 16 * t * t * t * t * t : 1 + 16 * --t * t * t * t * t;
	  }
	};
	/**
	 * Experimentaly compute the width of the scrollbar for this browser.
	 *
	 * @returns The width in pixels.
	 */

	function getScrollBarWidth() {
	  var inner = document.createElement("p");
	  inner.style.width = "100%";
	  inner.style.height = "200px";
	  var outer = document.createElement("div");
	  outer.style.position = "absolute";
	  outer.style.top = "0px";
	  outer.style.left = "0px";
	  outer.style.visibility = "hidden";
	  outer.style.width = "200px";
	  outer.style.height = "150px";
	  outer.style.overflow = "hidden";
	  outer.appendChild(inner);
	  document.body.appendChild(outer);
	  var w1 = inner.offsetWidth;
	  outer.style.overflow = "scroll";
	  var w2 = inner.offsetWidth;

	  if (w1 == w2) {
	    w2 = outer.clientWidth;
	  }

	  document.body.removeChild(outer);
	  return w1 - w2;
	} // @TODO: This doesn't work properly.
	// It works only for single property objects,
	// otherwise it combines all of the types in a union.
	// export function topMost<K1 extends string, V1> (
	//   pile: Record<K1, undefined | V1>[],
	//   accessors: K1 | [K1]
	// ): undefined | V1
	// export function topMost<K1 extends string, K2 extends string, V1, V2> (
	//   pile: Record<K1, undefined | V1 | Record<K2, undefined | V2>>[],
	//   accessors: [K1, K2]
	// ): undefined | V1 | V2
	// export function topMost<K1 extends string, K2 extends string, K3 extends string, V1, V2, V3> (
	//   pile: Record<K1, undefined | V1 | Record<K2, undefined | V2 | Record<K3, undefined | V3>>>[],
	//   accessors: [K1, K2, K3]
	// ): undefined | V1 | V2 | V3

	/**
	 * Get the top most property value from a pile of objects.
	 *
	 * @param pile - Array of objects, no required format.
	 * @param accessors - Array of property names.
	 * For example `object['foo']['bar']` → `['foo', 'bar']`.
	 *
	 * @returns Value of the property with given accessors path from the first pile item where it's not undefined.
	 */


	function topMost(pile, accessors) {
	  var candidate;

	  if (!isArray$3(accessors)) {
	    accessors = [accessors];
	  }

	  var _iterator = _createForOfIteratorHelper(pile),
	      _step;

	  try {
	    for (_iterator.s(); !(_step = _iterator.n()).done;) {
	      var member = _step.value;

	      if (member) {
	        candidate = member[accessors[0]];

	        for (var i = 1; i < accessors.length; i++) {
	          if (candidate) {
	            candidate = candidate[accessors[i]];
	          }
	        }

	        if (typeof candidate !== "undefined") {
	          break;
	        }
	      }
	    }
	  } catch (err) {
	    _iterator.e(err);
	  } finally {
	    _iterator.f();
	  }

	  return candidate;
	}

	var util = /*#__PURE__*/Object.freeze({
		__proto__: null,
		Alea: Alea,
		HSVToHex: HSVToHex,
		HSVToRGB: HSVToRGB,
		RGBToHSV: RGBToHSV,
		RGBToHex: RGBToHex,
		addClassName: addClassName,
		addCssText: addCssText,
		addEventListener: addEventListener,
		binarySearchCustom: binarySearchCustom,
		binarySearchValue: binarySearchValue,
		bridgeObject: bridgeObject,
		copyAndExtendArray: copyAndExtendArray,
		copyArray: copyArray,
		deepExtend: deepExtend,
		easingFunctions: easingFunctions,
		equalArray: equalArray,
		extend: extend,
		fillIfDefined: fillIfDefined,
		forEach: forEach$4,
		getAbsoluteLeft: getAbsoluteLeft,
		getAbsoluteRight: getAbsoluteRight,
		getAbsoluteTop: getAbsoluteTop,
		getScrollBarWidth: getScrollBarWidth,
		getTarget: getTarget,
		getType: getType,
		hasParent: hasParent,
		hexToHSV: hexToHSV,
		hexToRGB: hexToRGB,
		insertSort: insertSort,
		isDate: isDate,
		isNumber: isNumber,
		isObject: isObject$1,
		isString: isString,
		isValidHex: isValidHex,
		isValidRGB: isValidRGB,
		isValidRGBA: isValidRGBA,
		mergeOptions: mergeOptions,
		option: option,
		overrideOpacity: overrideOpacity,
		parseColor: parseColor,
		preventDefault: preventDefault,
		recursiveDOMDelete: recursiveDOMDelete,
		removeClassName: removeClassName,
		removeCssText: removeCssText,
		removeEventListener: removeEventListener,
		selectiveBridgeObject: selectiveBridgeObject,
		selectiveDeepExtend: selectiveDeepExtend,
		selectiveExtend: selectiveExtend,
		selectiveNotDeepExtend: selectiveNotDeepExtend,
		throttle: throttle,
		toArray: toArray,
		topMost: topMost,
		updateProperty: updateProperty
	});

	var moment = createCommonjsModule(function (module, exports) {

	  (function (global, factory) {
	     module.exports = factory() ;
	  })(commonjsGlobal, function () {

	    var hookCallback;

	    function hooks() {
	      return hookCallback.apply(null, arguments);
	    } // This is done to register the method called with moment()
	    // without creating circular dependencies.


	    function setHookCallback(callback) {
	      hookCallback = callback;
	    }

	    function isArray(input) {
	      return input instanceof Array || Object.prototype.toString.call(input) === '[object Array]';
	    }

	    function isObject(input) {
	      // IE8 will treat undefined and null as object if it wasn't for
	      // input != null
	      return input != null && Object.prototype.toString.call(input) === '[object Object]';
	    }

	    function hasOwnProp(a, b) {
	      return Object.prototype.hasOwnProperty.call(a, b);
	    }

	    function isObjectEmpty(obj) {
	      if (Object.getOwnPropertyNames) {
	        return Object.getOwnPropertyNames(obj).length === 0;
	      } else {
	        var k;

	        for (k in obj) {
	          if (hasOwnProp(obj, k)) {
	            return false;
	          }
	        }

	        return true;
	      }
	    }

	    function isUndefined(input) {
	      return input === void 0;
	    }

	    function isNumber(input) {
	      return typeof input === 'number' || Object.prototype.toString.call(input) === '[object Number]';
	    }

	    function isDate(input) {
	      return input instanceof Date || Object.prototype.toString.call(input) === '[object Date]';
	    }

	    function map(arr, fn) {
	      var res = [],
	          i;

	      for (i = 0; i < arr.length; ++i) {
	        res.push(fn(arr[i], i));
	      }

	      return res;
	    }

	    function extend(a, b) {
	      for (var i in b) {
	        if (hasOwnProp(b, i)) {
	          a[i] = b[i];
	        }
	      }

	      if (hasOwnProp(b, 'toString')) {
	        a.toString = b.toString;
	      }

	      if (hasOwnProp(b, 'valueOf')) {
	        a.valueOf = b.valueOf;
	      }

	      return a;
	    }

	    function createUTC(input, format, locale, strict) {
	      return createLocalOrUTC(input, format, locale, strict, true).utc();
	    }

	    function defaultParsingFlags() {
	      // We need to deep clone this object.
	      return {
	        empty: false,
	        unusedTokens: [],
	        unusedInput: [],
	        overflow: -2,
	        charsLeftOver: 0,
	        nullInput: false,
	        invalidEra: null,
	        invalidMonth: null,
	        invalidFormat: false,
	        userInvalidated: false,
	        iso: false,
	        parsedDateParts: [],
	        era: null,
	        meridiem: null,
	        rfc2822: false,
	        weekdayMismatch: false
	      };
	    }

	    function getParsingFlags(m) {
	      if (m._pf == null) {
	        m._pf = defaultParsingFlags();
	      }

	      return m._pf;
	    }

	    var some;

	    if (Array.prototype.some) {
	      some = Array.prototype.some;
	    } else {
	      some = function (fun) {
	        var t = Object(this),
	            len = t.length >>> 0,
	            i;

	        for (i = 0; i < len; i++) {
	          if (i in t && fun.call(this, t[i], i, t)) {
	            return true;
	          }
	        }

	        return false;
	      };
	    }

	    function isValid(m) {
	      if (m._isValid == null) {
	        var flags = getParsingFlags(m),
	            parsedParts = some.call(flags.parsedDateParts, function (i) {
	          return i != null;
	        }),
	            isNowValid = !isNaN(m._d.getTime()) && flags.overflow < 0 && !flags.empty && !flags.invalidEra && !flags.invalidMonth && !flags.invalidWeekday && !flags.weekdayMismatch && !flags.nullInput && !flags.invalidFormat && !flags.userInvalidated && (!flags.meridiem || flags.meridiem && parsedParts);

	        if (m._strict) {
	          isNowValid = isNowValid && flags.charsLeftOver === 0 && flags.unusedTokens.length === 0 && flags.bigHour === undefined;
	        }

	        if (Object.isFrozen == null || !Object.isFrozen(m)) {
	          m._isValid = isNowValid;
	        } else {
	          return isNowValid;
	        }
	      }

	      return m._isValid;
	    }

	    function createInvalid(flags) {
	      var m = createUTC(NaN);

	      if (flags != null) {
	        extend(getParsingFlags(m), flags);
	      } else {
	        getParsingFlags(m).userInvalidated = true;
	      }

	      return m;
	    } // Plugins that add properties should also add the key here (null value),
	    // so we can properly clone ourselves.


	    var momentProperties = hooks.momentProperties = [],
	        updateInProgress = false;

	    function copyConfig(to, from) {
	      var i, prop, val;

	      if (!isUndefined(from._isAMomentObject)) {
	        to._isAMomentObject = from._isAMomentObject;
	      }

	      if (!isUndefined(from._i)) {
	        to._i = from._i;
	      }

	      if (!isUndefined(from._f)) {
	        to._f = from._f;
	      }

	      if (!isUndefined(from._l)) {
	        to._l = from._l;
	      }

	      if (!isUndefined(from._strict)) {
	        to._strict = from._strict;
	      }

	      if (!isUndefined(from._tzm)) {
	        to._tzm = from._tzm;
	      }

	      if (!isUndefined(from._isUTC)) {
	        to._isUTC = from._isUTC;
	      }

	      if (!isUndefined(from._offset)) {
	        to._offset = from._offset;
	      }

	      if (!isUndefined(from._pf)) {
	        to._pf = getParsingFlags(from);
	      }

	      if (!isUndefined(from._locale)) {
	        to._locale = from._locale;
	      }

	      if (momentProperties.length > 0) {
	        for (i = 0; i < momentProperties.length; i++) {
	          prop = momentProperties[i];
	          val = from[prop];

	          if (!isUndefined(val)) {
	            to[prop] = val;
	          }
	        }
	      }

	      return to;
	    } // Moment prototype object


	    function Moment(config) {
	      copyConfig(this, config);
	      this._d = new Date(config._d != null ? config._d.getTime() : NaN);

	      if (!this.isValid()) {
	        this._d = new Date(NaN);
	      } // Prevent infinite loop in case updateOffset creates new moment
	      // objects.


	      if (updateInProgress === false) {
	        updateInProgress = true;
	        hooks.updateOffset(this);
	        updateInProgress = false;
	      }
	    }

	    function isMoment(obj) {
	      return obj instanceof Moment || obj != null && obj._isAMomentObject != null;
	    }

	    function warn(msg) {
	      if (hooks.suppressDeprecationWarnings === false && typeof console !== 'undefined' && console.warn) {
	        console.warn('Deprecation warning: ' + msg);
	      }
	    }

	    function deprecate(msg, fn) {
	      var firstTime = true;
	      return extend(function () {
	        if (hooks.deprecationHandler != null) {
	          hooks.deprecationHandler(null, msg);
	        }

	        if (firstTime) {
	          var args = [],
	              arg,
	              i,
	              key;

	          for (i = 0; i < arguments.length; i++) {
	            arg = '';

	            if (typeof arguments[i] === 'object') {
	              arg += '\n[' + i + '] ';

	              for (key in arguments[0]) {
	                if (hasOwnProp(arguments[0], key)) {
	                  arg += key + ': ' + arguments[0][key] + ', ';
	                }
	              }

	              arg = arg.slice(0, -2); // Remove trailing comma and space
	            } else {
	              arg = arguments[i];
	            }

	            args.push(arg);
	          }

	          warn(msg + '\nArguments: ' + Array.prototype.slice.call(args).join('') + '\n' + new Error().stack);
	          firstTime = false;
	        }

	        return fn.apply(this, arguments);
	      }, fn);
	    }

	    var deprecations = {};

	    function deprecateSimple(name, msg) {
	      if (hooks.deprecationHandler != null) {
	        hooks.deprecationHandler(name, msg);
	      }

	      if (!deprecations[name]) {
	        warn(msg);
	        deprecations[name] = true;
	      }
	    }

	    hooks.suppressDeprecationWarnings = false;
	    hooks.deprecationHandler = null;

	    function isFunction(input) {
	      return typeof Function !== 'undefined' && input instanceof Function || Object.prototype.toString.call(input) === '[object Function]';
	    }

	    function set(config) {
	      var prop, i;

	      for (i in config) {
	        if (hasOwnProp(config, i)) {
	          prop = config[i];

	          if (isFunction(prop)) {
	            this[i] = prop;
	          } else {
	            this['_' + i] = prop;
	          }
	        }
	      }

	      this._config = config; // Lenient ordinal parsing accepts just a number in addition to
	      // number + (possibly) stuff coming from _dayOfMonthOrdinalParse.
	      // TODO: Remove "ordinalParse" fallback in next major release.

	      this._dayOfMonthOrdinalParseLenient = new RegExp((this._dayOfMonthOrdinalParse.source || this._ordinalParse.source) + '|' + /\d{1,2}/.source);
	    }

	    function mergeConfigs(parentConfig, childConfig) {
	      var res = extend({}, parentConfig),
	          prop;

	      for (prop in childConfig) {
	        if (hasOwnProp(childConfig, prop)) {
	          if (isObject(parentConfig[prop]) && isObject(childConfig[prop])) {
	            res[prop] = {};
	            extend(res[prop], parentConfig[prop]);
	            extend(res[prop], childConfig[prop]);
	          } else if (childConfig[prop] != null) {
	            res[prop] = childConfig[prop];
	          } else {
	            delete res[prop];
	          }
	        }
	      }

	      for (prop in parentConfig) {
	        if (hasOwnProp(parentConfig, prop) && !hasOwnProp(childConfig, prop) && isObject(parentConfig[prop])) {
	          // make sure changes to properties don't modify parent config
	          res[prop] = extend({}, res[prop]);
	        }
	      }

	      return res;
	    }

	    function Locale(config) {
	      if (config != null) {
	        this.set(config);
	      }
	    }

	    var keys;

	    if (Object.keys) {
	      keys = Object.keys;
	    } else {
	      keys = function (obj) {
	        var i,
	            res = [];

	        for (i in obj) {
	          if (hasOwnProp(obj, i)) {
	            res.push(i);
	          }
	        }

	        return res;
	      };
	    }

	    var defaultCalendar = {
	      sameDay: '[Today at] LT',
	      nextDay: '[Tomorrow at] LT',
	      nextWeek: 'dddd [at] LT',
	      lastDay: '[Yesterday at] LT',
	      lastWeek: '[Last] dddd [at] LT',
	      sameElse: 'L'
	    };

	    function calendar(key, mom, now) {
	      var output = this._calendar[key] || this._calendar['sameElse'];
	      return isFunction(output) ? output.call(mom, now) : output;
	    }

	    function zeroFill(number, targetLength, forceSign) {
	      var absNumber = '' + Math.abs(number),
	          zerosToFill = targetLength - absNumber.length,
	          sign = number >= 0;
	      return (sign ? forceSign ? '+' : '' : '-') + Math.pow(10, Math.max(0, zerosToFill)).toString().substr(1) + absNumber;
	    }

	    var formattingTokens = /(\[[^\[]*\])|(\\)?([Hh]mm(ss)?|Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|Qo?|N{1,5}|YYYYYY|YYYYY|YYYY|YY|y{2,4}|yo?|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|kk?|mm?|ss?|S{1,9}|x|X|zz?|ZZ?|.)/g,
	        localFormattingTokens = /(\[[^\[]*\])|(\\)?(LTS|LT|LL?L?L?|l{1,4})/g,
	        formatFunctions = {},
	        formatTokenFunctions = {}; // token:    'M'
	    // padded:   ['MM', 2]
	    // ordinal:  'Mo'
	    // callback: function () { this.month() + 1 }

	    function addFormatToken(token, padded, ordinal, callback) {
	      var func = callback;

	      if (typeof callback === 'string') {
	        func = function () {
	          return this[callback]();
	        };
	      }

	      if (token) {
	        formatTokenFunctions[token] = func;
	      }

	      if (padded) {
	        formatTokenFunctions[padded[0]] = function () {
	          return zeroFill(func.apply(this, arguments), padded[1], padded[2]);
	        };
	      }

	      if (ordinal) {
	        formatTokenFunctions[ordinal] = function () {
	          return this.localeData().ordinal(func.apply(this, arguments), token);
	        };
	      }
	    }

	    function removeFormattingTokens(input) {
	      if (input.match(/\[[\s\S]/)) {
	        return input.replace(/^\[|\]$/g, '');
	      }

	      return input.replace(/\\/g, '');
	    }

	    function makeFormatFunction(format) {
	      var array = format.match(formattingTokens),
	          i,
	          length;

	      for (i = 0, length = array.length; i < length; i++) {
	        if (formatTokenFunctions[array[i]]) {
	          array[i] = formatTokenFunctions[array[i]];
	        } else {
	          array[i] = removeFormattingTokens(array[i]);
	        }
	      }

	      return function (mom) {
	        var output = '',
	            i;

	        for (i = 0; i < length; i++) {
	          output += isFunction(array[i]) ? array[i].call(mom, format) : array[i];
	        }

	        return output;
	      };
	    } // format date using native date object


	    function formatMoment(m, format) {
	      if (!m.isValid()) {
	        return m.localeData().invalidDate();
	      }

	      format = expandFormat(format, m.localeData());
	      formatFunctions[format] = formatFunctions[format] || makeFormatFunction(format);
	      return formatFunctions[format](m);
	    }

	    function expandFormat(format, locale) {
	      var i = 5;

	      function replaceLongDateFormatTokens(input) {
	        return locale.longDateFormat(input) || input;
	      }

	      localFormattingTokens.lastIndex = 0;

	      while (i >= 0 && localFormattingTokens.test(format)) {
	        format = format.replace(localFormattingTokens, replaceLongDateFormatTokens);
	        localFormattingTokens.lastIndex = 0;
	        i -= 1;
	      }

	      return format;
	    }

	    var defaultLongDateFormat = {
	      LTS: 'h:mm:ss A',
	      LT: 'h:mm A',
	      L: 'MM/DD/YYYY',
	      LL: 'MMMM D, YYYY',
	      LLL: 'MMMM D, YYYY h:mm A',
	      LLLL: 'dddd, MMMM D, YYYY h:mm A'
	    };

	    function longDateFormat(key) {
	      var format = this._longDateFormat[key],
	          formatUpper = this._longDateFormat[key.toUpperCase()];

	      if (format || !formatUpper) {
	        return format;
	      }

	      this._longDateFormat[key] = formatUpper.match(formattingTokens).map(function (tok) {
	        if (tok === 'MMMM' || tok === 'MM' || tok === 'DD' || tok === 'dddd') {
	          return tok.slice(1);
	        }

	        return tok;
	      }).join('');
	      return this._longDateFormat[key];
	    }

	    var defaultInvalidDate = 'Invalid date';

	    function invalidDate() {
	      return this._invalidDate;
	    }

	    var defaultOrdinal = '%d',
	        defaultDayOfMonthOrdinalParse = /\d{1,2}/;

	    function ordinal(number) {
	      return this._ordinal.replace('%d', number);
	    }

	    var defaultRelativeTime = {
	      future: 'in %s',
	      past: '%s ago',
	      s: 'a few seconds',
	      ss: '%d seconds',
	      m: 'a minute',
	      mm: '%d minutes',
	      h: 'an hour',
	      hh: '%d hours',
	      d: 'a day',
	      dd: '%d days',
	      w: 'a week',
	      ww: '%d weeks',
	      M: 'a month',
	      MM: '%d months',
	      y: 'a year',
	      yy: '%d years'
	    };

	    function relativeTime(number, withoutSuffix, string, isFuture) {
	      var output = this._relativeTime[string];
	      return isFunction(output) ? output(number, withoutSuffix, string, isFuture) : output.replace(/%d/i, number);
	    }

	    function pastFuture(diff, output) {
	      var format = this._relativeTime[diff > 0 ? 'future' : 'past'];
	      return isFunction(format) ? format(output) : format.replace(/%s/i, output);
	    }

	    var aliases = {};

	    function addUnitAlias(unit, shorthand) {
	      var lowerCase = unit.toLowerCase();
	      aliases[lowerCase] = aliases[lowerCase + 's'] = aliases[shorthand] = unit;
	    }

	    function normalizeUnits(units) {
	      return typeof units === 'string' ? aliases[units] || aliases[units.toLowerCase()] : undefined;
	    }

	    function normalizeObjectUnits(inputObject) {
	      var normalizedInput = {},
	          normalizedProp,
	          prop;

	      for (prop in inputObject) {
	        if (hasOwnProp(inputObject, prop)) {
	          normalizedProp = normalizeUnits(prop);

	          if (normalizedProp) {
	            normalizedInput[normalizedProp] = inputObject[prop];
	          }
	        }
	      }

	      return normalizedInput;
	    }

	    var priorities = {};

	    function addUnitPriority(unit, priority) {
	      priorities[unit] = priority;
	    }

	    function getPrioritizedUnits(unitsObj) {
	      var units = [],
	          u;

	      for (u in unitsObj) {
	        if (hasOwnProp(unitsObj, u)) {
	          units.push({
	            unit: u,
	            priority: priorities[u]
	          });
	        }
	      }

	      units.sort(function (a, b) {
	        return a.priority - b.priority;
	      });
	      return units;
	    }

	    function isLeapYear(year) {
	      return year % 4 === 0 && year % 100 !== 0 || year % 400 === 0;
	    }

	    function absFloor(number) {
	      if (number < 0) {
	        // -0 -> 0
	        return Math.ceil(number) || 0;
	      } else {
	        return Math.floor(number);
	      }
	    }

	    function toInt(argumentForCoercion) {
	      var coercedNumber = +argumentForCoercion,
	          value = 0;

	      if (coercedNumber !== 0 && isFinite(coercedNumber)) {
	        value = absFloor(coercedNumber);
	      }

	      return value;
	    }

	    function makeGetSet(unit, keepTime) {
	      return function (value) {
	        if (value != null) {
	          set$1(this, unit, value);
	          hooks.updateOffset(this, keepTime);
	          return this;
	        } else {
	          return get(this, unit);
	        }
	      };
	    }

	    function get(mom, unit) {
	      return mom.isValid() ? mom._d['get' + (mom._isUTC ? 'UTC' : '') + unit]() : NaN;
	    }

	    function set$1(mom, unit, value) {
	      if (mom.isValid() && !isNaN(value)) {
	        if (unit === 'FullYear' && isLeapYear(mom.year()) && mom.month() === 1 && mom.date() === 29) {
	          value = toInt(value);

	          mom._d['set' + (mom._isUTC ? 'UTC' : '') + unit](value, mom.month(), daysInMonth(value, mom.month()));
	        } else {
	          mom._d['set' + (mom._isUTC ? 'UTC' : '') + unit](value);
	        }
	      }
	    } // MOMENTS


	    function stringGet(units) {
	      units = normalizeUnits(units);

	      if (isFunction(this[units])) {
	        return this[units]();
	      }

	      return this;
	    }

	    function stringSet(units, value) {
	      if (typeof units === 'object') {
	        units = normalizeObjectUnits(units);
	        var prioritized = getPrioritizedUnits(units),
	            i;

	        for (i = 0; i < prioritized.length; i++) {
	          this[prioritized[i].unit](units[prioritized[i].unit]);
	        }
	      } else {
	        units = normalizeUnits(units);

	        if (isFunction(this[units])) {
	          return this[units](value);
	        }
	      }

	      return this;
	    }

	    var match1 = /\d/,
	        //       0 - 9
	    match2 = /\d\d/,
	        //      00 - 99
	    match3 = /\d{3}/,
	        //     000 - 999
	    match4 = /\d{4}/,
	        //    0000 - 9999
	    match6 = /[+-]?\d{6}/,
	        // -999999 - 999999
	    match1to2 = /\d\d?/,
	        //       0 - 99
	    match3to4 = /\d\d\d\d?/,
	        //     999 - 9999
	    match5to6 = /\d\d\d\d\d\d?/,
	        //   99999 - 999999
	    match1to3 = /\d{1,3}/,
	        //       0 - 999
	    match1to4 = /\d{1,4}/,
	        //       0 - 9999
	    match1to6 = /[+-]?\d{1,6}/,
	        // -999999 - 999999
	    matchUnsigned = /\d+/,
	        //       0 - inf
	    matchSigned = /[+-]?\d+/,
	        //    -inf - inf
	    matchOffset = /Z|[+-]\d\d:?\d\d/gi,
	        // +00:00 -00:00 +0000 -0000 or Z
	    matchShortOffset = /Z|[+-]\d\d(?::?\d\d)?/gi,
	        // +00 -00 +00:00 -00:00 +0000 -0000 or Z
	    matchTimestamp = /[+-]?\d+(\.\d{1,3})?/,
	        // 123456789 123456789.123
	    // any word (or two) characters or numbers including two/three word month in arabic.
	    // includes scottish gaelic two word and hyphenated months
	    matchWord = /[0-9]{0,256}['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFF07\uFF10-\uFFEF]{1,256}|[\u0600-\u06FF\/]{1,256}(\s*?[\u0600-\u06FF]{1,256}){1,2}/i,
	        regexes;
	    regexes = {};

	    function addRegexToken(token, regex, strictRegex) {
	      regexes[token] = isFunction(regex) ? regex : function (isStrict, localeData) {
	        return isStrict && strictRegex ? strictRegex : regex;
	      };
	    }

	    function getParseRegexForToken(token, config) {
	      if (!hasOwnProp(regexes, token)) {
	        return new RegExp(unescapeFormat(token));
	      }

	      return regexes[token](config._strict, config._locale);
	    } // Code from http://stackoverflow.com/questions/3561493/is-there-a-regexp-escape-function-in-javascript


	    function unescapeFormat(s) {
	      return regexEscape(s.replace('\\', '').replace(/\\(\[)|\\(\])|\[([^\]\[]*)\]|\\(.)/g, function (matched, p1, p2, p3, p4) {
	        return p1 || p2 || p3 || p4;
	      }));
	    }

	    function regexEscape(s) {
	      return s.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
	    }

	    var tokens = {};

	    function addParseToken(token, callback) {
	      var i,
	          func = callback;

	      if (typeof token === 'string') {
	        token = [token];
	      }

	      if (isNumber(callback)) {
	        func = function (input, array) {
	          array[callback] = toInt(input);
	        };
	      }

	      for (i = 0; i < token.length; i++) {
	        tokens[token[i]] = func;
	      }
	    }

	    function addWeekParseToken(token, callback) {
	      addParseToken(token, function (input, array, config, token) {
	        config._w = config._w || {};
	        callback(input, config._w, config, token);
	      });
	    }

	    function addTimeToArrayFromToken(token, input, config) {
	      if (input != null && hasOwnProp(tokens, token)) {
	        tokens[token](input, config._a, config, token);
	      }
	    }

	    var YEAR = 0,
	        MONTH = 1,
	        DATE = 2,
	        HOUR = 3,
	        MINUTE = 4,
	        SECOND = 5,
	        MILLISECOND = 6,
	        WEEK = 7,
	        WEEKDAY = 8;

	    function mod(n, x) {
	      return (n % x + x) % x;
	    }

	    var indexOf;

	    if (Array.prototype.indexOf) {
	      indexOf = Array.prototype.indexOf;
	    } else {
	      indexOf = function (o) {
	        // I know
	        var i;

	        for (i = 0; i < this.length; ++i) {
	          if (this[i] === o) {
	            return i;
	          }
	        }

	        return -1;
	      };
	    }

	    function daysInMonth(year, month) {
	      if (isNaN(year) || isNaN(month)) {
	        return NaN;
	      }

	      var modMonth = mod(month, 12);
	      year += (month - modMonth) / 12;
	      return modMonth === 1 ? isLeapYear(year) ? 29 : 28 : 31 - modMonth % 7 % 2;
	    } // FORMATTING


	    addFormatToken('M', ['MM', 2], 'Mo', function () {
	      return this.month() + 1;
	    });
	    addFormatToken('MMM', 0, 0, function (format) {
	      return this.localeData().monthsShort(this, format);
	    });
	    addFormatToken('MMMM', 0, 0, function (format) {
	      return this.localeData().months(this, format);
	    }); // ALIASES

	    addUnitAlias('month', 'M'); // PRIORITY

	    addUnitPriority('month', 8); // PARSING

	    addRegexToken('M', match1to2);
	    addRegexToken('MM', match1to2, match2);
	    addRegexToken('MMM', function (isStrict, locale) {
	      return locale.monthsShortRegex(isStrict);
	    });
	    addRegexToken('MMMM', function (isStrict, locale) {
	      return locale.monthsRegex(isStrict);
	    });
	    addParseToken(['M', 'MM'], function (input, array) {
	      array[MONTH] = toInt(input) - 1;
	    });
	    addParseToken(['MMM', 'MMMM'], function (input, array, config, token) {
	      var month = config._locale.monthsParse(input, token, config._strict); // if we didn't find a month name, mark the date as invalid.


	      if (month != null) {
	        array[MONTH] = month;
	      } else {
	        getParsingFlags(config).invalidMonth = input;
	      }
	    }); // LOCALES

	    var defaultLocaleMonths = 'January_February_March_April_May_June_July_August_September_October_November_December'.split('_'),
	        defaultLocaleMonthsShort = 'Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec'.split('_'),
	        MONTHS_IN_FORMAT = /D[oD]?(\[[^\[\]]*\]|\s)+MMMM?/,
	        defaultMonthsShortRegex = matchWord,
	        defaultMonthsRegex = matchWord;

	    function localeMonths(m, format) {
	      if (!m) {
	        return isArray(this._months) ? this._months : this._months['standalone'];
	      }

	      return isArray(this._months) ? this._months[m.month()] : this._months[(this._months.isFormat || MONTHS_IN_FORMAT).test(format) ? 'format' : 'standalone'][m.month()];
	    }

	    function localeMonthsShort(m, format) {
	      if (!m) {
	        return isArray(this._monthsShort) ? this._monthsShort : this._monthsShort['standalone'];
	      }

	      return isArray(this._monthsShort) ? this._monthsShort[m.month()] : this._monthsShort[MONTHS_IN_FORMAT.test(format) ? 'format' : 'standalone'][m.month()];
	    }

	    function handleStrictParse(monthName, format, strict) {
	      var i,
	          ii,
	          mom,
	          llc = monthName.toLocaleLowerCase();

	      if (!this._monthsParse) {
	        // this is not used
	        this._monthsParse = [];
	        this._longMonthsParse = [];
	        this._shortMonthsParse = [];

	        for (i = 0; i < 12; ++i) {
	          mom = createUTC([2000, i]);
	          this._shortMonthsParse[i] = this.monthsShort(mom, '').toLocaleLowerCase();
	          this._longMonthsParse[i] = this.months(mom, '').toLocaleLowerCase();
	        }
	      }

	      if (strict) {
	        if (format === 'MMM') {
	          ii = indexOf.call(this._shortMonthsParse, llc);
	          return ii !== -1 ? ii : null;
	        } else {
	          ii = indexOf.call(this._longMonthsParse, llc);
	          return ii !== -1 ? ii : null;
	        }
	      } else {
	        if (format === 'MMM') {
	          ii = indexOf.call(this._shortMonthsParse, llc);

	          if (ii !== -1) {
	            return ii;
	          }

	          ii = indexOf.call(this._longMonthsParse, llc);
	          return ii !== -1 ? ii : null;
	        } else {
	          ii = indexOf.call(this._longMonthsParse, llc);

	          if (ii !== -1) {
	            return ii;
	          }

	          ii = indexOf.call(this._shortMonthsParse, llc);
	          return ii !== -1 ? ii : null;
	        }
	      }
	    }

	    function localeMonthsParse(monthName, format, strict) {
	      var i, mom, regex;

	      if (this._monthsParseExact) {
	        return handleStrictParse.call(this, monthName, format, strict);
	      }

	      if (!this._monthsParse) {
	        this._monthsParse = [];
	        this._longMonthsParse = [];
	        this._shortMonthsParse = [];
	      } // TODO: add sorting
	      // Sorting makes sure if one month (or abbr) is a prefix of another
	      // see sorting in computeMonthsParse


	      for (i = 0; i < 12; i++) {
	        // make the regex if we don't have it already
	        mom = createUTC([2000, i]);

	        if (strict && !this._longMonthsParse[i]) {
	          this._longMonthsParse[i] = new RegExp('^' + this.months(mom, '').replace('.', '') + '$', 'i');
	          this._shortMonthsParse[i] = new RegExp('^' + this.monthsShort(mom, '').replace('.', '') + '$', 'i');
	        }

	        if (!strict && !this._monthsParse[i]) {
	          regex = '^' + this.months(mom, '') + '|^' + this.monthsShort(mom, '');
	          this._monthsParse[i] = new RegExp(regex.replace('.', ''), 'i');
	        } // test the regex


	        if (strict && format === 'MMMM' && this._longMonthsParse[i].test(monthName)) {
	          return i;
	        } else if (strict && format === 'MMM' && this._shortMonthsParse[i].test(monthName)) {
	          return i;
	        } else if (!strict && this._monthsParse[i].test(monthName)) {
	          return i;
	        }
	      }
	    } // MOMENTS


	    function setMonth(mom, value) {
	      var dayOfMonth;

	      if (!mom.isValid()) {
	        // No op
	        return mom;
	      }

	      if (typeof value === 'string') {
	        if (/^\d+$/.test(value)) {
	          value = toInt(value);
	        } else {
	          value = mom.localeData().monthsParse(value); // TODO: Another silent failure?

	          if (!isNumber(value)) {
	            return mom;
	          }
	        }
	      }

	      dayOfMonth = Math.min(mom.date(), daysInMonth(mom.year(), value));

	      mom._d['set' + (mom._isUTC ? 'UTC' : '') + 'Month'](value, dayOfMonth);

	      return mom;
	    }

	    function getSetMonth(value) {
	      if (value != null) {
	        setMonth(this, value);
	        hooks.updateOffset(this, true);
	        return this;
	      } else {
	        return get(this, 'Month');
	      }
	    }

	    function getDaysInMonth() {
	      return daysInMonth(this.year(), this.month());
	    }

	    function monthsShortRegex(isStrict) {
	      if (this._monthsParseExact) {
	        if (!hasOwnProp(this, '_monthsRegex')) {
	          computeMonthsParse.call(this);
	        }

	        if (isStrict) {
	          return this._monthsShortStrictRegex;
	        } else {
	          return this._monthsShortRegex;
	        }
	      } else {
	        if (!hasOwnProp(this, '_monthsShortRegex')) {
	          this._monthsShortRegex = defaultMonthsShortRegex;
	        }

	        return this._monthsShortStrictRegex && isStrict ? this._monthsShortStrictRegex : this._monthsShortRegex;
	      }
	    }

	    function monthsRegex(isStrict) {
	      if (this._monthsParseExact) {
	        if (!hasOwnProp(this, '_monthsRegex')) {
	          computeMonthsParse.call(this);
	        }

	        if (isStrict) {
	          return this._monthsStrictRegex;
	        } else {
	          return this._monthsRegex;
	        }
	      } else {
	        if (!hasOwnProp(this, '_monthsRegex')) {
	          this._monthsRegex = defaultMonthsRegex;
	        }

	        return this._monthsStrictRegex && isStrict ? this._monthsStrictRegex : this._monthsRegex;
	      }
	    }

	    function computeMonthsParse() {
	      function cmpLenRev(a, b) {
	        return b.length - a.length;
	      }

	      var shortPieces = [],
	          longPieces = [],
	          mixedPieces = [],
	          i,
	          mom;

	      for (i = 0; i < 12; i++) {
	        // make the regex if we don't have it already
	        mom = createUTC([2000, i]);
	        shortPieces.push(this.monthsShort(mom, ''));
	        longPieces.push(this.months(mom, ''));
	        mixedPieces.push(this.months(mom, ''));
	        mixedPieces.push(this.monthsShort(mom, ''));
	      } // Sorting makes sure if one month (or abbr) is a prefix of another it
	      // will match the longer piece.


	      shortPieces.sort(cmpLenRev);
	      longPieces.sort(cmpLenRev);
	      mixedPieces.sort(cmpLenRev);

	      for (i = 0; i < 12; i++) {
	        shortPieces[i] = regexEscape(shortPieces[i]);
	        longPieces[i] = regexEscape(longPieces[i]);
	      }

	      for (i = 0; i < 24; i++) {
	        mixedPieces[i] = regexEscape(mixedPieces[i]);
	      }

	      this._monthsRegex = new RegExp('^(' + mixedPieces.join('|') + ')', 'i');
	      this._monthsShortRegex = this._monthsRegex;
	      this._monthsStrictRegex = new RegExp('^(' + longPieces.join('|') + ')', 'i');
	      this._monthsShortStrictRegex = new RegExp('^(' + shortPieces.join('|') + ')', 'i');
	    } // FORMATTING


	    addFormatToken('Y', 0, 0, function () {
	      var y = this.year();
	      return y <= 9999 ? zeroFill(y, 4) : '+' + y;
	    });
	    addFormatToken(0, ['YY', 2], 0, function () {
	      return this.year() % 100;
	    });
	    addFormatToken(0, ['YYYY', 4], 0, 'year');
	    addFormatToken(0, ['YYYYY', 5], 0, 'year');
	    addFormatToken(0, ['YYYYYY', 6, true], 0, 'year'); // ALIASES

	    addUnitAlias('year', 'y'); // PRIORITIES

	    addUnitPriority('year', 1); // PARSING

	    addRegexToken('Y', matchSigned);
	    addRegexToken('YY', match1to2, match2);
	    addRegexToken('YYYY', match1to4, match4);
	    addRegexToken('YYYYY', match1to6, match6);
	    addRegexToken('YYYYYY', match1to6, match6);
	    addParseToken(['YYYYY', 'YYYYYY'], YEAR);
	    addParseToken('YYYY', function (input, array) {
	      array[YEAR] = input.length === 2 ? hooks.parseTwoDigitYear(input) : toInt(input);
	    });
	    addParseToken('YY', function (input, array) {
	      array[YEAR] = hooks.parseTwoDigitYear(input);
	    });
	    addParseToken('Y', function (input, array) {
	      array[YEAR] = parseInt(input, 10);
	    }); // HELPERS

	    function daysInYear(year) {
	      return isLeapYear(year) ? 366 : 365;
	    } // HOOKS


	    hooks.parseTwoDigitYear = function (input) {
	      return toInt(input) + (toInt(input) > 68 ? 1900 : 2000);
	    }; // MOMENTS


	    var getSetYear = makeGetSet('FullYear', true);

	    function getIsLeapYear() {
	      return isLeapYear(this.year());
	    }

	    function createDate(y, m, d, h, M, s, ms) {
	      // can't just apply() to create a date:
	      // https://stackoverflow.com/q/181348
	      var date; // the date constructor remaps years 0-99 to 1900-1999

	      if (y < 100 && y >= 0) {
	        // preserve leap years using a full 400 year cycle, then reset
	        date = new Date(y + 400, m, d, h, M, s, ms);

	        if (isFinite(date.getFullYear())) {
	          date.setFullYear(y);
	        }
	      } else {
	        date = new Date(y, m, d, h, M, s, ms);
	      }

	      return date;
	    }

	    function createUTCDate(y) {
	      var date, args; // the Date.UTC function remaps years 0-99 to 1900-1999

	      if (y < 100 && y >= 0) {
	        args = Array.prototype.slice.call(arguments); // preserve leap years using a full 400 year cycle, then reset

	        args[0] = y + 400;
	        date = new Date(Date.UTC.apply(null, args));

	        if (isFinite(date.getUTCFullYear())) {
	          date.setUTCFullYear(y);
	        }
	      } else {
	        date = new Date(Date.UTC.apply(null, arguments));
	      }

	      return date;
	    } // start-of-first-week - start-of-year


	    function firstWeekOffset(year, dow, doy) {
	      var // first-week day -- which january is always in the first week (4 for iso, 1 for other)
	      fwd = 7 + dow - doy,
	          // first-week day local weekday -- which local weekday is fwd
	      fwdlw = (7 + createUTCDate(year, 0, fwd).getUTCDay() - dow) % 7;
	      return -fwdlw + fwd - 1;
	    } // https://en.wikipedia.org/wiki/ISO_week_date#Calculating_a_date_given_the_year.2C_week_number_and_weekday


	    function dayOfYearFromWeeks(year, week, weekday, dow, doy) {
	      var localWeekday = (7 + weekday - dow) % 7,
	          weekOffset = firstWeekOffset(year, dow, doy),
	          dayOfYear = 1 + 7 * (week - 1) + localWeekday + weekOffset,
	          resYear,
	          resDayOfYear;

	      if (dayOfYear <= 0) {
	        resYear = year - 1;
	        resDayOfYear = daysInYear(resYear) + dayOfYear;
	      } else if (dayOfYear > daysInYear(year)) {
	        resYear = year + 1;
	        resDayOfYear = dayOfYear - daysInYear(year);
	      } else {
	        resYear = year;
	        resDayOfYear = dayOfYear;
	      }

	      return {
	        year: resYear,
	        dayOfYear: resDayOfYear
	      };
	    }

	    function weekOfYear(mom, dow, doy) {
	      var weekOffset = firstWeekOffset(mom.year(), dow, doy),
	          week = Math.floor((mom.dayOfYear() - weekOffset - 1) / 7) + 1,
	          resWeek,
	          resYear;

	      if (week < 1) {
	        resYear = mom.year() - 1;
	        resWeek = week + weeksInYear(resYear, dow, doy);
	      } else if (week > weeksInYear(mom.year(), dow, doy)) {
	        resWeek = week - weeksInYear(mom.year(), dow, doy);
	        resYear = mom.year() + 1;
	      } else {
	        resYear = mom.year();
	        resWeek = week;
	      }

	      return {
	        week: resWeek,
	        year: resYear
	      };
	    }

	    function weeksInYear(year, dow, doy) {
	      var weekOffset = firstWeekOffset(year, dow, doy),
	          weekOffsetNext = firstWeekOffset(year + 1, dow, doy);
	      return (daysInYear(year) - weekOffset + weekOffsetNext) / 7;
	    } // FORMATTING


	    addFormatToken('w', ['ww', 2], 'wo', 'week');
	    addFormatToken('W', ['WW', 2], 'Wo', 'isoWeek'); // ALIASES

	    addUnitAlias('week', 'w');
	    addUnitAlias('isoWeek', 'W'); // PRIORITIES

	    addUnitPriority('week', 5);
	    addUnitPriority('isoWeek', 5); // PARSING

	    addRegexToken('w', match1to2);
	    addRegexToken('ww', match1to2, match2);
	    addRegexToken('W', match1to2);
	    addRegexToken('WW', match1to2, match2);
	    addWeekParseToken(['w', 'ww', 'W', 'WW'], function (input, week, config, token) {
	      week[token.substr(0, 1)] = toInt(input);
	    }); // HELPERS
	    // LOCALES

	    function localeWeek(mom) {
	      return weekOfYear(mom, this._week.dow, this._week.doy).week;
	    }

	    var defaultLocaleWeek = {
	      dow: 0,
	      // Sunday is the first day of the week.
	      doy: 6 // The week that contains Jan 6th is the first week of the year.

	    };

	    function localeFirstDayOfWeek() {
	      return this._week.dow;
	    }

	    function localeFirstDayOfYear() {
	      return this._week.doy;
	    } // MOMENTS


	    function getSetWeek(input) {
	      var week = this.localeData().week(this);
	      return input == null ? week : this.add((input - week) * 7, 'd');
	    }

	    function getSetISOWeek(input) {
	      var week = weekOfYear(this, 1, 4).week;
	      return input == null ? week : this.add((input - week) * 7, 'd');
	    } // FORMATTING


	    addFormatToken('d', 0, 'do', 'day');
	    addFormatToken('dd', 0, 0, function (format) {
	      return this.localeData().weekdaysMin(this, format);
	    });
	    addFormatToken('ddd', 0, 0, function (format) {
	      return this.localeData().weekdaysShort(this, format);
	    });
	    addFormatToken('dddd', 0, 0, function (format) {
	      return this.localeData().weekdays(this, format);
	    });
	    addFormatToken('e', 0, 0, 'weekday');
	    addFormatToken('E', 0, 0, 'isoWeekday'); // ALIASES

	    addUnitAlias('day', 'd');
	    addUnitAlias('weekday', 'e');
	    addUnitAlias('isoWeekday', 'E'); // PRIORITY

	    addUnitPriority('day', 11);
	    addUnitPriority('weekday', 11);
	    addUnitPriority('isoWeekday', 11); // PARSING

	    addRegexToken('d', match1to2);
	    addRegexToken('e', match1to2);
	    addRegexToken('E', match1to2);
	    addRegexToken('dd', function (isStrict, locale) {
	      return locale.weekdaysMinRegex(isStrict);
	    });
	    addRegexToken('ddd', function (isStrict, locale) {
	      return locale.weekdaysShortRegex(isStrict);
	    });
	    addRegexToken('dddd', function (isStrict, locale) {
	      return locale.weekdaysRegex(isStrict);
	    });
	    addWeekParseToken(['dd', 'ddd', 'dddd'], function (input, week, config, token) {
	      var weekday = config._locale.weekdaysParse(input, token, config._strict); // if we didn't get a weekday name, mark the date as invalid


	      if (weekday != null) {
	        week.d = weekday;
	      } else {
	        getParsingFlags(config).invalidWeekday = input;
	      }
	    });
	    addWeekParseToken(['d', 'e', 'E'], function (input, week, config, token) {
	      week[token] = toInt(input);
	    }); // HELPERS

	    function parseWeekday(input, locale) {
	      if (typeof input !== 'string') {
	        return input;
	      }

	      if (!isNaN(input)) {
	        return parseInt(input, 10);
	      }

	      input = locale.weekdaysParse(input);

	      if (typeof input === 'number') {
	        return input;
	      }

	      return null;
	    }

	    function parseIsoWeekday(input, locale) {
	      if (typeof input === 'string') {
	        return locale.weekdaysParse(input) % 7 || 7;
	      }

	      return isNaN(input) ? null : input;
	    } // LOCALES


	    function shiftWeekdays(ws, n) {
	      return ws.slice(n, 7).concat(ws.slice(0, n));
	    }

	    var defaultLocaleWeekdays = 'Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday'.split('_'),
	        defaultLocaleWeekdaysShort = 'Sun_Mon_Tue_Wed_Thu_Fri_Sat'.split('_'),
	        defaultLocaleWeekdaysMin = 'Su_Mo_Tu_We_Th_Fr_Sa'.split('_'),
	        defaultWeekdaysRegex = matchWord,
	        defaultWeekdaysShortRegex = matchWord,
	        defaultWeekdaysMinRegex = matchWord;

	    function localeWeekdays(m, format) {
	      var weekdays = isArray(this._weekdays) ? this._weekdays : this._weekdays[m && m !== true && this._weekdays.isFormat.test(format) ? 'format' : 'standalone'];
	      return m === true ? shiftWeekdays(weekdays, this._week.dow) : m ? weekdays[m.day()] : weekdays;
	    }

	    function localeWeekdaysShort(m) {
	      return m === true ? shiftWeekdays(this._weekdaysShort, this._week.dow) : m ? this._weekdaysShort[m.day()] : this._weekdaysShort;
	    }

	    function localeWeekdaysMin(m) {
	      return m === true ? shiftWeekdays(this._weekdaysMin, this._week.dow) : m ? this._weekdaysMin[m.day()] : this._weekdaysMin;
	    }

	    function handleStrictParse$1(weekdayName, format, strict) {
	      var i,
	          ii,
	          mom,
	          llc = weekdayName.toLocaleLowerCase();

	      if (!this._weekdaysParse) {
	        this._weekdaysParse = [];
	        this._shortWeekdaysParse = [];
	        this._minWeekdaysParse = [];

	        for (i = 0; i < 7; ++i) {
	          mom = createUTC([2000, 1]).day(i);
	          this._minWeekdaysParse[i] = this.weekdaysMin(mom, '').toLocaleLowerCase();
	          this._shortWeekdaysParse[i] = this.weekdaysShort(mom, '').toLocaleLowerCase();
	          this._weekdaysParse[i] = this.weekdays(mom, '').toLocaleLowerCase();
	        }
	      }

	      if (strict) {
	        if (format === 'dddd') {
	          ii = indexOf.call(this._weekdaysParse, llc);
	          return ii !== -1 ? ii : null;
	        } else if (format === 'ddd') {
	          ii = indexOf.call(this._shortWeekdaysParse, llc);
	          return ii !== -1 ? ii : null;
	        } else {
	          ii = indexOf.call(this._minWeekdaysParse, llc);
	          return ii !== -1 ? ii : null;
	        }
	      } else {
	        if (format === 'dddd') {
	          ii = indexOf.call(this._weekdaysParse, llc);

	          if (ii !== -1) {
	            return ii;
	          }

	          ii = indexOf.call(this._shortWeekdaysParse, llc);

	          if (ii !== -1) {
	            return ii;
	          }

	          ii = indexOf.call(this._minWeekdaysParse, llc);
	          return ii !== -1 ? ii : null;
	        } else if (format === 'ddd') {
	          ii = indexOf.call(this._shortWeekdaysParse, llc);

	          if (ii !== -1) {
	            return ii;
	          }

	          ii = indexOf.call(this._weekdaysParse, llc);

	          if (ii !== -1) {
	            return ii;
	          }

	          ii = indexOf.call(this._minWeekdaysParse, llc);
	          return ii !== -1 ? ii : null;
	        } else {
	          ii = indexOf.call(this._minWeekdaysParse, llc);

	          if (ii !== -1) {
	            return ii;
	          }

	          ii = indexOf.call(this._weekdaysParse, llc);

	          if (ii !== -1) {
	            return ii;
	          }

	          ii = indexOf.call(this._shortWeekdaysParse, llc);
	          return ii !== -1 ? ii : null;
	        }
	      }
	    }

	    function localeWeekdaysParse(weekdayName, format, strict) {
	      var i, mom, regex;

	      if (this._weekdaysParseExact) {
	        return handleStrictParse$1.call(this, weekdayName, format, strict);
	      }

	      if (!this._weekdaysParse) {
	        this._weekdaysParse = [];
	        this._minWeekdaysParse = [];
	        this._shortWeekdaysParse = [];
	        this._fullWeekdaysParse = [];
	      }

	      for (i = 0; i < 7; i++) {
	        // make the regex if we don't have it already
	        mom = createUTC([2000, 1]).day(i);

	        if (strict && !this._fullWeekdaysParse[i]) {
	          this._fullWeekdaysParse[i] = new RegExp('^' + this.weekdays(mom, '').replace('.', '\\.?') + '$', 'i');
	          this._shortWeekdaysParse[i] = new RegExp('^' + this.weekdaysShort(mom, '').replace('.', '\\.?') + '$', 'i');
	          this._minWeekdaysParse[i] = new RegExp('^' + this.weekdaysMin(mom, '').replace('.', '\\.?') + '$', 'i');
	        }

	        if (!this._weekdaysParse[i]) {
	          regex = '^' + this.weekdays(mom, '') + '|^' + this.weekdaysShort(mom, '') + '|^' + this.weekdaysMin(mom, '');
	          this._weekdaysParse[i] = new RegExp(regex.replace('.', ''), 'i');
	        } // test the regex


	        if (strict && format === 'dddd' && this._fullWeekdaysParse[i].test(weekdayName)) {
	          return i;
	        } else if (strict && format === 'ddd' && this._shortWeekdaysParse[i].test(weekdayName)) {
	          return i;
	        } else if (strict && format === 'dd' && this._minWeekdaysParse[i].test(weekdayName)) {
	          return i;
	        } else if (!strict && this._weekdaysParse[i].test(weekdayName)) {
	          return i;
	        }
	      }
	    } // MOMENTS


	    function getSetDayOfWeek(input) {
	      if (!this.isValid()) {
	        return input != null ? this : NaN;
	      }

	      var day = this._isUTC ? this._d.getUTCDay() : this._d.getDay();

	      if (input != null) {
	        input = parseWeekday(input, this.localeData());
	        return this.add(input - day, 'd');
	      } else {
	        return day;
	      }
	    }

	    function getSetLocaleDayOfWeek(input) {
	      if (!this.isValid()) {
	        return input != null ? this : NaN;
	      }

	      var weekday = (this.day() + 7 - this.localeData()._week.dow) % 7;
	      return input == null ? weekday : this.add(input - weekday, 'd');
	    }

	    function getSetISODayOfWeek(input) {
	      if (!this.isValid()) {
	        return input != null ? this : NaN;
	      } // behaves the same as moment#day except
	      // as a getter, returns 7 instead of 0 (1-7 range instead of 0-6)
	      // as a setter, sunday should belong to the previous week.


	      if (input != null) {
	        var weekday = parseIsoWeekday(input, this.localeData());
	        return this.day(this.day() % 7 ? weekday : weekday - 7);
	      } else {
	        return this.day() || 7;
	      }
	    }

	    function weekdaysRegex(isStrict) {
	      if (this._weekdaysParseExact) {
	        if (!hasOwnProp(this, '_weekdaysRegex')) {
	          computeWeekdaysParse.call(this);
	        }

	        if (isStrict) {
	          return this._weekdaysStrictRegex;
	        } else {
	          return this._weekdaysRegex;
	        }
	      } else {
	        if (!hasOwnProp(this, '_weekdaysRegex')) {
	          this._weekdaysRegex = defaultWeekdaysRegex;
	        }

	        return this._weekdaysStrictRegex && isStrict ? this._weekdaysStrictRegex : this._weekdaysRegex;
	      }
	    }

	    function weekdaysShortRegex(isStrict) {
	      if (this._weekdaysParseExact) {
	        if (!hasOwnProp(this, '_weekdaysRegex')) {
	          computeWeekdaysParse.call(this);
	        }

	        if (isStrict) {
	          return this._weekdaysShortStrictRegex;
	        } else {
	          return this._weekdaysShortRegex;
	        }
	      } else {
	        if (!hasOwnProp(this, '_weekdaysShortRegex')) {
	          this._weekdaysShortRegex = defaultWeekdaysShortRegex;
	        }

	        return this._weekdaysShortStrictRegex && isStrict ? this._weekdaysShortStrictRegex : this._weekdaysShortRegex;
	      }
	    }

	    function weekdaysMinRegex(isStrict) {
	      if (this._weekdaysParseExact) {
	        if (!hasOwnProp(this, '_weekdaysRegex')) {
	          computeWeekdaysParse.call(this);
	        }

	        if (isStrict) {
	          return this._weekdaysMinStrictRegex;
	        } else {
	          return this._weekdaysMinRegex;
	        }
	      } else {
	        if (!hasOwnProp(this, '_weekdaysMinRegex')) {
	          this._weekdaysMinRegex = defaultWeekdaysMinRegex;
	        }

	        return this._weekdaysMinStrictRegex && isStrict ? this._weekdaysMinStrictRegex : this._weekdaysMinRegex;
	      }
	    }

	    function computeWeekdaysParse() {
	      function cmpLenRev(a, b) {
	        return b.length - a.length;
	      }

	      var minPieces = [],
	          shortPieces = [],
	          longPieces = [],
	          mixedPieces = [],
	          i,
	          mom,
	          minp,
	          shortp,
	          longp;

	      for (i = 0; i < 7; i++) {
	        // make the regex if we don't have it already
	        mom = createUTC([2000, 1]).day(i);
	        minp = regexEscape(this.weekdaysMin(mom, ''));
	        shortp = regexEscape(this.weekdaysShort(mom, ''));
	        longp = regexEscape(this.weekdays(mom, ''));
	        minPieces.push(minp);
	        shortPieces.push(shortp);
	        longPieces.push(longp);
	        mixedPieces.push(minp);
	        mixedPieces.push(shortp);
	        mixedPieces.push(longp);
	      } // Sorting makes sure if one weekday (or abbr) is a prefix of another it
	      // will match the longer piece.


	      minPieces.sort(cmpLenRev);
	      shortPieces.sort(cmpLenRev);
	      longPieces.sort(cmpLenRev);
	      mixedPieces.sort(cmpLenRev);
	      this._weekdaysRegex = new RegExp('^(' + mixedPieces.join('|') + ')', 'i');
	      this._weekdaysShortRegex = this._weekdaysRegex;
	      this._weekdaysMinRegex = this._weekdaysRegex;
	      this._weekdaysStrictRegex = new RegExp('^(' + longPieces.join('|') + ')', 'i');
	      this._weekdaysShortStrictRegex = new RegExp('^(' + shortPieces.join('|') + ')', 'i');
	      this._weekdaysMinStrictRegex = new RegExp('^(' + minPieces.join('|') + ')', 'i');
	    } // FORMATTING


	    function hFormat() {
	      return this.hours() % 12 || 12;
	    }

	    function kFormat() {
	      return this.hours() || 24;
	    }

	    addFormatToken('H', ['HH', 2], 0, 'hour');
	    addFormatToken('h', ['hh', 2], 0, hFormat);
	    addFormatToken('k', ['kk', 2], 0, kFormat);
	    addFormatToken('hmm', 0, 0, function () {
	      return '' + hFormat.apply(this) + zeroFill(this.minutes(), 2);
	    });
	    addFormatToken('hmmss', 0, 0, function () {
	      return '' + hFormat.apply(this) + zeroFill(this.minutes(), 2) + zeroFill(this.seconds(), 2);
	    });
	    addFormatToken('Hmm', 0, 0, function () {
	      return '' + this.hours() + zeroFill(this.minutes(), 2);
	    });
	    addFormatToken('Hmmss', 0, 0, function () {
	      return '' + this.hours() + zeroFill(this.minutes(), 2) + zeroFill(this.seconds(), 2);
	    });

	    function meridiem(token, lowercase) {
	      addFormatToken(token, 0, 0, function () {
	        return this.localeData().meridiem(this.hours(), this.minutes(), lowercase);
	      });
	    }

	    meridiem('a', true);
	    meridiem('A', false); // ALIASES

	    addUnitAlias('hour', 'h'); // PRIORITY

	    addUnitPriority('hour', 13); // PARSING

	    function matchMeridiem(isStrict, locale) {
	      return locale._meridiemParse;
	    }

	    addRegexToken('a', matchMeridiem);
	    addRegexToken('A', matchMeridiem);
	    addRegexToken('H', match1to2);
	    addRegexToken('h', match1to2);
	    addRegexToken('k', match1to2);
	    addRegexToken('HH', match1to2, match2);
	    addRegexToken('hh', match1to2, match2);
	    addRegexToken('kk', match1to2, match2);
	    addRegexToken('hmm', match3to4);
	    addRegexToken('hmmss', match5to6);
	    addRegexToken('Hmm', match3to4);
	    addRegexToken('Hmmss', match5to6);
	    addParseToken(['H', 'HH'], HOUR);
	    addParseToken(['k', 'kk'], function (input, array, config) {
	      var kInput = toInt(input);
	      array[HOUR] = kInput === 24 ? 0 : kInput;
	    });
	    addParseToken(['a', 'A'], function (input, array, config) {
	      config._isPm = config._locale.isPM(input);
	      config._meridiem = input;
	    });
	    addParseToken(['h', 'hh'], function (input, array, config) {
	      array[HOUR] = toInt(input);
	      getParsingFlags(config).bigHour = true;
	    });
	    addParseToken('hmm', function (input, array, config) {
	      var pos = input.length - 2;
	      array[HOUR] = toInt(input.substr(0, pos));
	      array[MINUTE] = toInt(input.substr(pos));
	      getParsingFlags(config).bigHour = true;
	    });
	    addParseToken('hmmss', function (input, array, config) {
	      var pos1 = input.length - 4,
	          pos2 = input.length - 2;
	      array[HOUR] = toInt(input.substr(0, pos1));
	      array[MINUTE] = toInt(input.substr(pos1, 2));
	      array[SECOND] = toInt(input.substr(pos2));
	      getParsingFlags(config).bigHour = true;
	    });
	    addParseToken('Hmm', function (input, array, config) {
	      var pos = input.length - 2;
	      array[HOUR] = toInt(input.substr(0, pos));
	      array[MINUTE] = toInt(input.substr(pos));
	    });
	    addParseToken('Hmmss', function (input, array, config) {
	      var pos1 = input.length - 4,
	          pos2 = input.length - 2;
	      array[HOUR] = toInt(input.substr(0, pos1));
	      array[MINUTE] = toInt(input.substr(pos1, 2));
	      array[SECOND] = toInt(input.substr(pos2));
	    }); // LOCALES

	    function localeIsPM(input) {
	      // IE8 Quirks Mode & IE7 Standards Mode do not allow accessing strings like arrays
	      // Using charAt should be more compatible.
	      return (input + '').toLowerCase().charAt(0) === 'p';
	    }

	    var defaultLocaleMeridiemParse = /[ap]\.?m?\.?/i,
	        // Setting the hour should keep the time, because the user explicitly
	    // specified which hour they want. So trying to maintain the same hour (in
	    // a new timezone) makes sense. Adding/subtracting hours does not follow
	    // this rule.
	    getSetHour = makeGetSet('Hours', true);

	    function localeMeridiem(hours, minutes, isLower) {
	      if (hours > 11) {
	        return isLower ? 'pm' : 'PM';
	      } else {
	        return isLower ? 'am' : 'AM';
	      }
	    }

	    var baseConfig = {
	      calendar: defaultCalendar,
	      longDateFormat: defaultLongDateFormat,
	      invalidDate: defaultInvalidDate,
	      ordinal: defaultOrdinal,
	      dayOfMonthOrdinalParse: defaultDayOfMonthOrdinalParse,
	      relativeTime: defaultRelativeTime,
	      months: defaultLocaleMonths,
	      monthsShort: defaultLocaleMonthsShort,
	      week: defaultLocaleWeek,
	      weekdays: defaultLocaleWeekdays,
	      weekdaysMin: defaultLocaleWeekdaysMin,
	      weekdaysShort: defaultLocaleWeekdaysShort,
	      meridiemParse: defaultLocaleMeridiemParse
	    }; // internal storage for locale config files

	    var locales = {},
	        localeFamilies = {},
	        globalLocale;

	    function commonPrefix(arr1, arr2) {
	      var i,
	          minl = Math.min(arr1.length, arr2.length);

	      for (i = 0; i < minl; i += 1) {
	        if (arr1[i] !== arr2[i]) {
	          return i;
	        }
	      }

	      return minl;
	    }

	    function normalizeLocale(key) {
	      return key ? key.toLowerCase().replace('_', '-') : key;
	    } // pick the locale from the array
	    // try ['en-au', 'en-gb'] as 'en-au', 'en-gb', 'en', as in move through the list trying each
	    // substring from most specific to least, but move to the next array item if it's a more specific variant than the current root


	    function chooseLocale(names) {
	      var i = 0,
	          j,
	          next,
	          locale,
	          split;

	      while (i < names.length) {
	        split = normalizeLocale(names[i]).split('-');
	        j = split.length;
	        next = normalizeLocale(names[i + 1]);
	        next = next ? next.split('-') : null;

	        while (j > 0) {
	          locale = loadLocale(split.slice(0, j).join('-'));

	          if (locale) {
	            return locale;
	          }

	          if (next && next.length >= j && commonPrefix(split, next) >= j - 1) {
	            //the next array item is better than a shallower substring of this one
	            break;
	          }

	          j--;
	        }

	        i++;
	      }

	      return globalLocale;
	    }

	    function loadLocale(name) {
	      var oldLocale = null,
	          aliasedRequire; // TODO: Find a better way to register and load all the locales in Node

	      if (locales[name] === undefined && 'object' !== 'undefined' && module && module.exports) {
	        try {
	          oldLocale = globalLocale._abbr;
	          aliasedRequire = commonjsRequire;
	          aliasedRequire('./locale/' + name);
	          getSetGlobalLocale(oldLocale);
	        } catch (e) {
	          // mark as not found to avoid repeating expensive file require call causing high CPU
	          // when trying to find en-US, en_US, en-us for every format call
	          locales[name] = null; // null means not found
	        }
	      }

	      return locales[name];
	    } // This function will load locale and then set the global locale.  If
	    // no arguments are passed in, it will simply return the current global
	    // locale key.


	    function getSetGlobalLocale(key, values) {
	      var data;

	      if (key) {
	        if (isUndefined(values)) {
	          data = getLocale(key);
	        } else {
	          data = defineLocale(key, values);
	        }

	        if (data) {
	          // moment.duration._locale = moment._locale = data;
	          globalLocale = data;
	        } else {
	          if (typeof console !== 'undefined' && console.warn) {
	            //warn user if arguments are passed but the locale could not be set
	            console.warn('Locale ' + key + ' not found. Did you forget to load it?');
	          }
	        }
	      }

	      return globalLocale._abbr;
	    }

	    function defineLocale(name, config) {
	      if (config !== null) {
	        var locale,
	            parentConfig = baseConfig;
	        config.abbr = name;

	        if (locales[name] != null) {
	          deprecateSimple('defineLocaleOverride', 'use moment.updateLocale(localeName, config) to change ' + 'an existing locale. moment.defineLocale(localeName, ' + 'config) should only be used for creating a new locale ' + 'See http://momentjs.com/guides/#/warnings/define-locale/ for more info.');
	          parentConfig = locales[name]._config;
	        } else if (config.parentLocale != null) {
	          if (locales[config.parentLocale] != null) {
	            parentConfig = locales[config.parentLocale]._config;
	          } else {
	            locale = loadLocale(config.parentLocale);

	            if (locale != null) {
	              parentConfig = locale._config;
	            } else {
	              if (!localeFamilies[config.parentLocale]) {
	                localeFamilies[config.parentLocale] = [];
	              }

	              localeFamilies[config.parentLocale].push({
	                name: name,
	                config: config
	              });
	              return null;
	            }
	          }
	        }

	        locales[name] = new Locale(mergeConfigs(parentConfig, config));

	        if (localeFamilies[name]) {
	          localeFamilies[name].forEach(function (x) {
	            defineLocale(x.name, x.config);
	          });
	        } // backwards compat for now: also set the locale
	        // make sure we set the locale AFTER all child locales have been
	        // created, so we won't end up with the child locale set.


	        getSetGlobalLocale(name);
	        return locales[name];
	      } else {
	        // useful for testing
	        delete locales[name];
	        return null;
	      }
	    }

	    function updateLocale(name, config) {
	      if (config != null) {
	        var locale,
	            tmpLocale,
	            parentConfig = baseConfig;

	        if (locales[name] != null && locales[name].parentLocale != null) {
	          // Update existing child locale in-place to avoid memory-leaks
	          locales[name].set(mergeConfigs(locales[name]._config, config));
	        } else {
	          // MERGE
	          tmpLocale = loadLocale(name);

	          if (tmpLocale != null) {
	            parentConfig = tmpLocale._config;
	          }

	          config = mergeConfigs(parentConfig, config);

	          if (tmpLocale == null) {
	            // updateLocale is called for creating a new locale
	            // Set abbr so it will have a name (getters return
	            // undefined otherwise).
	            config.abbr = name;
	          }

	          locale = new Locale(config);
	          locale.parentLocale = locales[name];
	          locales[name] = locale;
	        } // backwards compat for now: also set the locale


	        getSetGlobalLocale(name);
	      } else {
	        // pass null for config to unupdate, useful for tests
	        if (locales[name] != null) {
	          if (locales[name].parentLocale != null) {
	            locales[name] = locales[name].parentLocale;

	            if (name === getSetGlobalLocale()) {
	              getSetGlobalLocale(name);
	            }
	          } else if (locales[name] != null) {
	            delete locales[name];
	          }
	        }
	      }

	      return locales[name];
	    } // returns locale data


	    function getLocale(key) {
	      var locale;

	      if (key && key._locale && key._locale._abbr) {
	        key = key._locale._abbr;
	      }

	      if (!key) {
	        return globalLocale;
	      }

	      if (!isArray(key)) {
	        //short-circuit everything else
	        locale = loadLocale(key);

	        if (locale) {
	          return locale;
	        }

	        key = [key];
	      }

	      return chooseLocale(key);
	    }

	    function listLocales() {
	      return keys(locales);
	    }

	    function checkOverflow(m) {
	      var overflow,
	          a = m._a;

	      if (a && getParsingFlags(m).overflow === -2) {
	        overflow = a[MONTH] < 0 || a[MONTH] > 11 ? MONTH : a[DATE] < 1 || a[DATE] > daysInMonth(a[YEAR], a[MONTH]) ? DATE : a[HOUR] < 0 || a[HOUR] > 24 || a[HOUR] === 24 && (a[MINUTE] !== 0 || a[SECOND] !== 0 || a[MILLISECOND] !== 0) ? HOUR : a[MINUTE] < 0 || a[MINUTE] > 59 ? MINUTE : a[SECOND] < 0 || a[SECOND] > 59 ? SECOND : a[MILLISECOND] < 0 || a[MILLISECOND] > 999 ? MILLISECOND : -1;

	        if (getParsingFlags(m)._overflowDayOfYear && (overflow < YEAR || overflow > DATE)) {
	          overflow = DATE;
	        }

	        if (getParsingFlags(m)._overflowWeeks && overflow === -1) {
	          overflow = WEEK;
	        }

	        if (getParsingFlags(m)._overflowWeekday && overflow === -1) {
	          overflow = WEEKDAY;
	        }

	        getParsingFlags(m).overflow = overflow;
	      }

	      return m;
	    } // iso 8601 regex
	    // 0000-00-00 0000-W00 or 0000-W00-0 + T + 00 or 00:00 or 00:00:00 or 00:00:00.000 + +00:00 or +0000 or +00)


	    var extendedIsoRegex = /^\s*((?:[+-]\d{6}|\d{4})-(?:\d\d-\d\d|W\d\d-\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?::\d\d(?::\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,
	        basicIsoRegex = /^\s*((?:[+-]\d{6}|\d{4})(?:\d\d\d\d|W\d\d\d|W\d\d|\d\d\d|\d\d|))(?:(T| )(\d\d(?:\d\d(?:\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,
	        tzRegex = /Z|[+-]\d\d(?::?\d\d)?/,
	        isoDates = [['YYYYYY-MM-DD', /[+-]\d{6}-\d\d-\d\d/], ['YYYY-MM-DD', /\d{4}-\d\d-\d\d/], ['GGGG-[W]WW-E', /\d{4}-W\d\d-\d/], ['GGGG-[W]WW', /\d{4}-W\d\d/, false], ['YYYY-DDD', /\d{4}-\d{3}/], ['YYYY-MM', /\d{4}-\d\d/, false], ['YYYYYYMMDD', /[+-]\d{10}/], ['YYYYMMDD', /\d{8}/], ['GGGG[W]WWE', /\d{4}W\d{3}/], ['GGGG[W]WW', /\d{4}W\d{2}/, false], ['YYYYDDD', /\d{7}/], ['YYYYMM', /\d{6}/, false], ['YYYY', /\d{4}/, false]],
	        // iso time formats and regexes
	    isoTimes = [['HH:mm:ss.SSSS', /\d\d:\d\d:\d\d\.\d+/], ['HH:mm:ss,SSSS', /\d\d:\d\d:\d\d,\d+/], ['HH:mm:ss', /\d\d:\d\d:\d\d/], ['HH:mm', /\d\d:\d\d/], ['HHmmss.SSSS', /\d\d\d\d\d\d\.\d+/], ['HHmmss,SSSS', /\d\d\d\d\d\d,\d+/], ['HHmmss', /\d\d\d\d\d\d/], ['HHmm', /\d\d\d\d/], ['HH', /\d\d/]],
	        aspNetJsonRegex = /^\/?Date\((-?\d+)/i,
	        // RFC 2822 regex: For details see https://tools.ietf.org/html/rfc2822#section-3.3
	    rfc2822 = /^(?:(Mon|Tue|Wed|Thu|Fri|Sat|Sun),?\s)?(\d{1,2})\s(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s(\d{2,4})\s(\d\d):(\d\d)(?::(\d\d))?\s(?:(UT|GMT|[ECMP][SD]T)|([Zz])|([+-]\d{4}))$/,
	        obsOffsets = {
	      UT: 0,
	      GMT: 0,
	      EDT: -4 * 60,
	      EST: -5 * 60,
	      CDT: -5 * 60,
	      CST: -6 * 60,
	      MDT: -6 * 60,
	      MST: -7 * 60,
	      PDT: -7 * 60,
	      PST: -8 * 60
	    }; // date from iso format

	    function configFromISO(config) {
	      var i,
	          l,
	          string = config._i,
	          match = extendedIsoRegex.exec(string) || basicIsoRegex.exec(string),
	          allowTime,
	          dateFormat,
	          timeFormat,
	          tzFormat;

	      if (match) {
	        getParsingFlags(config).iso = true;

	        for (i = 0, l = isoDates.length; i < l; i++) {
	          if (isoDates[i][1].exec(match[1])) {
	            dateFormat = isoDates[i][0];
	            allowTime = isoDates[i][2] !== false;
	            break;
	          }
	        }

	        if (dateFormat == null) {
	          config._isValid = false;
	          return;
	        }

	        if (match[3]) {
	          for (i = 0, l = isoTimes.length; i < l; i++) {
	            if (isoTimes[i][1].exec(match[3])) {
	              // match[2] should be 'T' or space
	              timeFormat = (match[2] || ' ') + isoTimes[i][0];
	              break;
	            }
	          }

	          if (timeFormat == null) {
	            config._isValid = false;
	            return;
	          }
	        }

	        if (!allowTime && timeFormat != null) {
	          config._isValid = false;
	          return;
	        }

	        if (match[4]) {
	          if (tzRegex.exec(match[4])) {
	            tzFormat = 'Z';
	          } else {
	            config._isValid = false;
	            return;
	          }
	        }

	        config._f = dateFormat + (timeFormat || '') + (tzFormat || '');
	        configFromStringAndFormat(config);
	      } else {
	        config._isValid = false;
	      }
	    }

	    function extractFromRFC2822Strings(yearStr, monthStr, dayStr, hourStr, minuteStr, secondStr) {
	      var result = [untruncateYear(yearStr), defaultLocaleMonthsShort.indexOf(monthStr), parseInt(dayStr, 10), parseInt(hourStr, 10), parseInt(minuteStr, 10)];

	      if (secondStr) {
	        result.push(parseInt(secondStr, 10));
	      }

	      return result;
	    }

	    function untruncateYear(yearStr) {
	      var year = parseInt(yearStr, 10);

	      if (year <= 49) {
	        return 2000 + year;
	      } else if (year <= 999) {
	        return 1900 + year;
	      }

	      return year;
	    }

	    function preprocessRFC2822(s) {
	      // Remove comments and folding whitespace and replace multiple-spaces with a single space
	      return s.replace(/\([^)]*\)|[\n\t]/g, ' ').replace(/(\s\s+)/g, ' ').replace(/^\s\s*/, '').replace(/\s\s*$/, '');
	    }

	    function checkWeekday(weekdayStr, parsedInput, config) {
	      if (weekdayStr) {
	        // TODO: Replace the vanilla JS Date object with an independent day-of-week check.
	        var weekdayProvided = defaultLocaleWeekdaysShort.indexOf(weekdayStr),
	            weekdayActual = new Date(parsedInput[0], parsedInput[1], parsedInput[2]).getDay();

	        if (weekdayProvided !== weekdayActual) {
	          getParsingFlags(config).weekdayMismatch = true;
	          config._isValid = false;
	          return false;
	        }
	      }

	      return true;
	    }

	    function calculateOffset(obsOffset, militaryOffset, numOffset) {
	      if (obsOffset) {
	        return obsOffsets[obsOffset];
	      } else if (militaryOffset) {
	        // the only allowed military tz is Z
	        return 0;
	      } else {
	        var hm = parseInt(numOffset, 10),
	            m = hm % 100,
	            h = (hm - m) / 100;
	        return h * 60 + m;
	      }
	    } // date and time from ref 2822 format


	    function configFromRFC2822(config) {
	      var match = rfc2822.exec(preprocessRFC2822(config._i)),
	          parsedArray;

	      if (match) {
	        parsedArray = extractFromRFC2822Strings(match[4], match[3], match[2], match[5], match[6], match[7]);

	        if (!checkWeekday(match[1], parsedArray, config)) {
	          return;
	        }

	        config._a = parsedArray;
	        config._tzm = calculateOffset(match[8], match[9], match[10]);
	        config._d = createUTCDate.apply(null, config._a);

	        config._d.setUTCMinutes(config._d.getUTCMinutes() - config._tzm);

	        getParsingFlags(config).rfc2822 = true;
	      } else {
	        config._isValid = false;
	      }
	    } // date from 1) ASP.NET, 2) ISO, 3) RFC 2822 formats, or 4) optional fallback if parsing isn't strict


	    function configFromString(config) {
	      var matched = aspNetJsonRegex.exec(config._i);

	      if (matched !== null) {
	        config._d = new Date(+matched[1]);
	        return;
	      }

	      configFromISO(config);

	      if (config._isValid === false) {
	        delete config._isValid;
	      } else {
	        return;
	      }

	      configFromRFC2822(config);

	      if (config._isValid === false) {
	        delete config._isValid;
	      } else {
	        return;
	      }

	      if (config._strict) {
	        config._isValid = false;
	      } else {
	        // Final attempt, use Input Fallback
	        hooks.createFromInputFallback(config);
	      }
	    }

	    hooks.createFromInputFallback = deprecate('value provided is not in a recognized RFC2822 or ISO format. moment construction falls back to js Date(), ' + 'which is not reliable across all browsers and versions. Non RFC2822/ISO date formats are ' + 'discouraged and will be removed in an upcoming major release. Please refer to ' + 'http://momentjs.com/guides/#/warnings/js-date/ for more info.', function (config) {
	      config._d = new Date(config._i + (config._useUTC ? ' UTC' : ''));
	    }); // Pick the first defined of two or three arguments.

	    function defaults(a, b, c) {
	      if (a != null) {
	        return a;
	      }

	      if (b != null) {
	        return b;
	      }

	      return c;
	    }

	    function currentDateArray(config) {
	      // hooks is actually the exported moment object
	      var nowValue = new Date(hooks.now());

	      if (config._useUTC) {
	        return [nowValue.getUTCFullYear(), nowValue.getUTCMonth(), nowValue.getUTCDate()];
	      }

	      return [nowValue.getFullYear(), nowValue.getMonth(), nowValue.getDate()];
	    } // convert an array to a date.
	    // the array should mirror the parameters below
	    // note: all values past the year are optional and will default to the lowest possible value.
	    // [year, month, day , hour, minute, second, millisecond]


	    function configFromArray(config) {
	      var i,
	          date,
	          input = [],
	          currentDate,
	          expectedWeekday,
	          yearToUse;

	      if (config._d) {
	        return;
	      }

	      currentDate = currentDateArray(config); //compute day of the year from weeks and weekdays

	      if (config._w && config._a[DATE] == null && config._a[MONTH] == null) {
	        dayOfYearFromWeekInfo(config);
	      } //if the day of the year is set, figure out what it is


	      if (config._dayOfYear != null) {
	        yearToUse = defaults(config._a[YEAR], currentDate[YEAR]);

	        if (config._dayOfYear > daysInYear(yearToUse) || config._dayOfYear === 0) {
	          getParsingFlags(config)._overflowDayOfYear = true;
	        }

	        date = createUTCDate(yearToUse, 0, config._dayOfYear);
	        config._a[MONTH] = date.getUTCMonth();
	        config._a[DATE] = date.getUTCDate();
	      } // Default to current date.
	      // * if no year, month, day of month are given, default to today
	      // * if day of month is given, default month and year
	      // * if month is given, default only year
	      // * if year is given, don't default anything


	      for (i = 0; i < 3 && config._a[i] == null; ++i) {
	        config._a[i] = input[i] = currentDate[i];
	      } // Zero out whatever was not defaulted, including time


	      for (; i < 7; i++) {
	        config._a[i] = input[i] = config._a[i] == null ? i === 2 ? 1 : 0 : config._a[i];
	      } // Check for 24:00:00.000


	      if (config._a[HOUR] === 24 && config._a[MINUTE] === 0 && config._a[SECOND] === 0 && config._a[MILLISECOND] === 0) {
	        config._nextDay = true;
	        config._a[HOUR] = 0;
	      }

	      config._d = (config._useUTC ? createUTCDate : createDate).apply(null, input);
	      expectedWeekday = config._useUTC ? config._d.getUTCDay() : config._d.getDay(); // Apply timezone offset from input. The actual utcOffset can be changed
	      // with parseZone.

	      if (config._tzm != null) {
	        config._d.setUTCMinutes(config._d.getUTCMinutes() - config._tzm);
	      }

	      if (config._nextDay) {
	        config._a[HOUR] = 24;
	      } // check for mismatching day of week


	      if (config._w && typeof config._w.d !== 'undefined' && config._w.d !== expectedWeekday) {
	        getParsingFlags(config).weekdayMismatch = true;
	      }
	    }

	    function dayOfYearFromWeekInfo(config) {
	      var w, weekYear, week, weekday, dow, doy, temp, weekdayOverflow, curWeek;
	      w = config._w;

	      if (w.GG != null || w.W != null || w.E != null) {
	        dow = 1;
	        doy = 4; // TODO: We need to take the current isoWeekYear, but that depends on
	        // how we interpret now (local, utc, fixed offset). So create
	        // a now version of current config (take local/utc/offset flags, and
	        // create now).

	        weekYear = defaults(w.GG, config._a[YEAR], weekOfYear(createLocal(), 1, 4).year);
	        week = defaults(w.W, 1);
	        weekday = defaults(w.E, 1);

	        if (weekday < 1 || weekday > 7) {
	          weekdayOverflow = true;
	        }
	      } else {
	        dow = config._locale._week.dow;
	        doy = config._locale._week.doy;
	        curWeek = weekOfYear(createLocal(), dow, doy);
	        weekYear = defaults(w.gg, config._a[YEAR], curWeek.year); // Default to current week.

	        week = defaults(w.w, curWeek.week);

	        if (w.d != null) {
	          // weekday -- low day numbers are considered next week
	          weekday = w.d;

	          if (weekday < 0 || weekday > 6) {
	            weekdayOverflow = true;
	          }
	        } else if (w.e != null) {
	          // local weekday -- counting starts from beginning of week
	          weekday = w.e + dow;

	          if (w.e < 0 || w.e > 6) {
	            weekdayOverflow = true;
	          }
	        } else {
	          // default to beginning of week
	          weekday = dow;
	        }
	      }

	      if (week < 1 || week > weeksInYear(weekYear, dow, doy)) {
	        getParsingFlags(config)._overflowWeeks = true;
	      } else if (weekdayOverflow != null) {
	        getParsingFlags(config)._overflowWeekday = true;
	      } else {
	        temp = dayOfYearFromWeeks(weekYear, week, weekday, dow, doy);
	        config._a[YEAR] = temp.year;
	        config._dayOfYear = temp.dayOfYear;
	      }
	    } // constant that refers to the ISO standard


	    hooks.ISO_8601 = function () {}; // constant that refers to the RFC 2822 form


	    hooks.RFC_2822 = function () {}; // date from string and format string


	    function configFromStringAndFormat(config) {
	      // TODO: Move this to another part of the creation flow to prevent circular deps
	      if (config._f === hooks.ISO_8601) {
	        configFromISO(config);
	        return;
	      }

	      if (config._f === hooks.RFC_2822) {
	        configFromRFC2822(config);
	        return;
	      }

	      config._a = [];
	      getParsingFlags(config).empty = true; // This array is used to make a Date, either with `new Date` or `Date.UTC`

	      var string = '' + config._i,
	          i,
	          parsedInput,
	          tokens,
	          token,
	          skipped,
	          stringLength = string.length,
	          totalParsedInputLength = 0,
	          era;
	      tokens = expandFormat(config._f, config._locale).match(formattingTokens) || [];

	      for (i = 0; i < tokens.length; i++) {
	        token = tokens[i];
	        parsedInput = (string.match(getParseRegexForToken(token, config)) || [])[0]; // console.log('token', token, 'parsedInput', parsedInput,
	        //         'regex', getParseRegexForToken(token, config));

	        if (parsedInput) {
	          skipped = string.substr(0, string.indexOf(parsedInput));

	          if (skipped.length > 0) {
	            getParsingFlags(config).unusedInput.push(skipped);
	          }

	          string = string.slice(string.indexOf(parsedInput) + parsedInput.length);
	          totalParsedInputLength += parsedInput.length;
	        } // don't parse if it's not a known token


	        if (formatTokenFunctions[token]) {
	          if (parsedInput) {
	            getParsingFlags(config).empty = false;
	          } else {
	            getParsingFlags(config).unusedTokens.push(token);
	          }

	          addTimeToArrayFromToken(token, parsedInput, config);
	        } else if (config._strict && !parsedInput) {
	          getParsingFlags(config).unusedTokens.push(token);
	        }
	      } // add remaining unparsed input length to the string


	      getParsingFlags(config).charsLeftOver = stringLength - totalParsedInputLength;

	      if (string.length > 0) {
	        getParsingFlags(config).unusedInput.push(string);
	      } // clear _12h flag if hour is <= 12


	      if (config._a[HOUR] <= 12 && getParsingFlags(config).bigHour === true && config._a[HOUR] > 0) {
	        getParsingFlags(config).bigHour = undefined;
	      }

	      getParsingFlags(config).parsedDateParts = config._a.slice(0);
	      getParsingFlags(config).meridiem = config._meridiem; // handle meridiem

	      config._a[HOUR] = meridiemFixWrap(config._locale, config._a[HOUR], config._meridiem); // handle era

	      era = getParsingFlags(config).era;

	      if (era !== null) {
	        config._a[YEAR] = config._locale.erasConvertYear(era, config._a[YEAR]);
	      }

	      configFromArray(config);
	      checkOverflow(config);
	    }

	    function meridiemFixWrap(locale, hour, meridiem) {
	      var isPm;

	      if (meridiem == null) {
	        // nothing to do
	        return hour;
	      }

	      if (locale.meridiemHour != null) {
	        return locale.meridiemHour(hour, meridiem);
	      } else if (locale.isPM != null) {
	        // Fallback
	        isPm = locale.isPM(meridiem);

	        if (isPm && hour < 12) {
	          hour += 12;
	        }

	        if (!isPm && hour === 12) {
	          hour = 0;
	        }

	        return hour;
	      } else {
	        // this is not supposed to happen
	        return hour;
	      }
	    } // date from string and array of format strings


	    function configFromStringAndArray(config) {
	      var tempConfig,
	          bestMoment,
	          scoreToBeat,
	          i,
	          currentScore,
	          validFormatFound,
	          bestFormatIsValid = false;

	      if (config._f.length === 0) {
	        getParsingFlags(config).invalidFormat = true;
	        config._d = new Date(NaN);
	        return;
	      }

	      for (i = 0; i < config._f.length; i++) {
	        currentScore = 0;
	        validFormatFound = false;
	        tempConfig = copyConfig({}, config);

	        if (config._useUTC != null) {
	          tempConfig._useUTC = config._useUTC;
	        }

	        tempConfig._f = config._f[i];
	        configFromStringAndFormat(tempConfig);

	        if (isValid(tempConfig)) {
	          validFormatFound = true;
	        } // if there is any input that was not parsed add a penalty for that format


	        currentScore += getParsingFlags(tempConfig).charsLeftOver; //or tokens

	        currentScore += getParsingFlags(tempConfig).unusedTokens.length * 10;
	        getParsingFlags(tempConfig).score = currentScore;

	        if (!bestFormatIsValid) {
	          if (scoreToBeat == null || currentScore < scoreToBeat || validFormatFound) {
	            scoreToBeat = currentScore;
	            bestMoment = tempConfig;

	            if (validFormatFound) {
	              bestFormatIsValid = true;
	            }
	          }
	        } else {
	          if (currentScore < scoreToBeat) {
	            scoreToBeat = currentScore;
	            bestMoment = tempConfig;
	          }
	        }
	      }

	      extend(config, bestMoment || tempConfig);
	    }

	    function configFromObject(config) {
	      if (config._d) {
	        return;
	      }

	      var i = normalizeObjectUnits(config._i),
	          dayOrDate = i.day === undefined ? i.date : i.day;
	      config._a = map([i.year, i.month, dayOrDate, i.hour, i.minute, i.second, i.millisecond], function (obj) {
	        return obj && parseInt(obj, 10);
	      });
	      configFromArray(config);
	    }

	    function createFromConfig(config) {
	      var res = new Moment(checkOverflow(prepareConfig(config)));

	      if (res._nextDay) {
	        // Adding is smart enough around DST
	        res.add(1, 'd');
	        res._nextDay = undefined;
	      }

	      return res;
	    }

	    function prepareConfig(config) {
	      var input = config._i,
	          format = config._f;
	      config._locale = config._locale || getLocale(config._l);

	      if (input === null || format === undefined && input === '') {
	        return createInvalid({
	          nullInput: true
	        });
	      }

	      if (typeof input === 'string') {
	        config._i = input = config._locale.preparse(input);
	      }

	      if (isMoment(input)) {
	        return new Moment(checkOverflow(input));
	      } else if (isDate(input)) {
	        config._d = input;
	      } else if (isArray(format)) {
	        configFromStringAndArray(config);
	      } else if (format) {
	        configFromStringAndFormat(config);
	      } else {
	        configFromInput(config);
	      }

	      if (!isValid(config)) {
	        config._d = null;
	      }

	      return config;
	    }

	    function configFromInput(config) {
	      var input = config._i;

	      if (isUndefined(input)) {
	        config._d = new Date(hooks.now());
	      } else if (isDate(input)) {
	        config._d = new Date(input.valueOf());
	      } else if (typeof input === 'string') {
	        configFromString(config);
	      } else if (isArray(input)) {
	        config._a = map(input.slice(0), function (obj) {
	          return parseInt(obj, 10);
	        });
	        configFromArray(config);
	      } else if (isObject(input)) {
	        configFromObject(config);
	      } else if (isNumber(input)) {
	        // from milliseconds
	        config._d = new Date(input);
	      } else {
	        hooks.createFromInputFallback(config);
	      }
	    }

	    function createLocalOrUTC(input, format, locale, strict, isUTC) {
	      var c = {};

	      if (format === true || format === false) {
	        strict = format;
	        format = undefined;
	      }

	      if (locale === true || locale === false) {
	        strict = locale;
	        locale = undefined;
	      }

	      if (isObject(input) && isObjectEmpty(input) || isArray(input) && input.length === 0) {
	        input = undefined;
	      } // object construction must be done this way.
	      // https://github.com/moment/moment/issues/1423


	      c._isAMomentObject = true;
	      c._useUTC = c._isUTC = isUTC;
	      c._l = locale;
	      c._i = input;
	      c._f = format;
	      c._strict = strict;
	      return createFromConfig(c);
	    }

	    function createLocal(input, format, locale, strict) {
	      return createLocalOrUTC(input, format, locale, strict, false);
	    }

	    var prototypeMin = deprecate('moment().min is deprecated, use moment.max instead. http://momentjs.com/guides/#/warnings/min-max/', function () {
	      var other = createLocal.apply(null, arguments);

	      if (this.isValid() && other.isValid()) {
	        return other < this ? this : other;
	      } else {
	        return createInvalid();
	      }
	    }),
	        prototypeMax = deprecate('moment().max is deprecated, use moment.min instead. http://momentjs.com/guides/#/warnings/min-max/', function () {
	      var other = createLocal.apply(null, arguments);

	      if (this.isValid() && other.isValid()) {
	        return other > this ? this : other;
	      } else {
	        return createInvalid();
	      }
	    }); // Pick a moment m from moments so that m[fn](other) is true for all
	    // other. This relies on the function fn to be transitive.
	    //
	    // moments should either be an array of moment objects or an array, whose
	    // first element is an array of moment objects.

	    function pickBy(fn, moments) {
	      var res, i;

	      if (moments.length === 1 && isArray(moments[0])) {
	        moments = moments[0];
	      }

	      if (!moments.length) {
	        return createLocal();
	      }

	      res = moments[0];

	      for (i = 1; i < moments.length; ++i) {
	        if (!moments[i].isValid() || moments[i][fn](res)) {
	          res = moments[i];
	        }
	      }

	      return res;
	    } // TODO: Use [].sort instead?


	    function min() {
	      var args = [].slice.call(arguments, 0);
	      return pickBy('isBefore', args);
	    }

	    function max() {
	      var args = [].slice.call(arguments, 0);
	      return pickBy('isAfter', args);
	    }

	    var now = function () {
	      return Date.now ? Date.now() : +new Date();
	    };

	    var ordering = ['year', 'quarter', 'month', 'week', 'day', 'hour', 'minute', 'second', 'millisecond'];

	    function isDurationValid(m) {
	      var key,
	          unitHasDecimal = false,
	          i;

	      for (key in m) {
	        if (hasOwnProp(m, key) && !(indexOf.call(ordering, key) !== -1 && (m[key] == null || !isNaN(m[key])))) {
	          return false;
	        }
	      }

	      for (i = 0; i < ordering.length; ++i) {
	        if (m[ordering[i]]) {
	          if (unitHasDecimal) {
	            return false; // only allow non-integers for smallest unit
	          }

	          if (parseFloat(m[ordering[i]]) !== toInt(m[ordering[i]])) {
	            unitHasDecimal = true;
	          }
	        }
	      }

	      return true;
	    }

	    function isValid$1() {
	      return this._isValid;
	    }

	    function createInvalid$1() {
	      return createDuration(NaN);
	    }

	    function Duration(duration) {
	      var normalizedInput = normalizeObjectUnits(duration),
	          years = normalizedInput.year || 0,
	          quarters = normalizedInput.quarter || 0,
	          months = normalizedInput.month || 0,
	          weeks = normalizedInput.week || normalizedInput.isoWeek || 0,
	          days = normalizedInput.day || 0,
	          hours = normalizedInput.hour || 0,
	          minutes = normalizedInput.minute || 0,
	          seconds = normalizedInput.second || 0,
	          milliseconds = normalizedInput.millisecond || 0;
	      this._isValid = isDurationValid(normalizedInput); // representation for dateAddRemove

	      this._milliseconds = +milliseconds + seconds * 1e3 + // 1000
	      minutes * 6e4 + // 1000 * 60
	      hours * 1000 * 60 * 60; //using 1000 * 60 * 60 instead of 36e5 to avoid floating point rounding errors https://github.com/moment/moment/issues/2978
	      // Because of dateAddRemove treats 24 hours as different from a
	      // day when working around DST, we need to store them separately

	      this._days = +days + weeks * 7; // It is impossible to translate months into days without knowing
	      // which months you are are talking about, so we have to store
	      // it separately.

	      this._months = +months + quarters * 3 + years * 12;
	      this._data = {};
	      this._locale = getLocale();

	      this._bubble();
	    }

	    function isDuration(obj) {
	      return obj instanceof Duration;
	    }

	    function absRound(number) {
	      if (number < 0) {
	        return Math.round(-1 * number) * -1;
	      } else {
	        return Math.round(number);
	      }
	    } // compare two arrays, return the number of differences


	    function compareArrays(array1, array2, dontConvert) {
	      var len = Math.min(array1.length, array2.length),
	          lengthDiff = Math.abs(array1.length - array2.length),
	          diffs = 0,
	          i;

	      for (i = 0; i < len; i++) {
	        if (dontConvert && array1[i] !== array2[i] || !dontConvert && toInt(array1[i]) !== toInt(array2[i])) {
	          diffs++;
	        }
	      }

	      return diffs + lengthDiff;
	    } // FORMATTING


	    function offset(token, separator) {
	      addFormatToken(token, 0, 0, function () {
	        var offset = this.utcOffset(),
	            sign = '+';

	        if (offset < 0) {
	          offset = -offset;
	          sign = '-';
	        }

	        return sign + zeroFill(~~(offset / 60), 2) + separator + zeroFill(~~offset % 60, 2);
	      });
	    }

	    offset('Z', ':');
	    offset('ZZ', ''); // PARSING

	    addRegexToken('Z', matchShortOffset);
	    addRegexToken('ZZ', matchShortOffset);
	    addParseToken(['Z', 'ZZ'], function (input, array, config) {
	      config._useUTC = true;
	      config._tzm = offsetFromString(matchShortOffset, input);
	    }); // HELPERS
	    // timezone chunker
	    // '+10:00' > ['10',  '00']
	    // '-1530'  > ['-15', '30']

	    var chunkOffset = /([\+\-]|\d\d)/gi;

	    function offsetFromString(matcher, string) {
	      var matches = (string || '').match(matcher),
	          chunk,
	          parts,
	          minutes;

	      if (matches === null) {
	        return null;
	      }

	      chunk = matches[matches.length - 1] || [];
	      parts = (chunk + '').match(chunkOffset) || ['-', 0, 0];
	      minutes = +(parts[1] * 60) + toInt(parts[2]);
	      return minutes === 0 ? 0 : parts[0] === '+' ? minutes : -minutes;
	    } // Return a moment from input, that is local/utc/zone equivalent to model.


	    function cloneWithOffset(input, model) {
	      var res, diff;

	      if (model._isUTC) {
	        res = model.clone();
	        diff = (isMoment(input) || isDate(input) ? input.valueOf() : createLocal(input).valueOf()) - res.valueOf(); // Use low-level api, because this fn is low-level api.

	        res._d.setTime(res._d.valueOf() + diff);

	        hooks.updateOffset(res, false);
	        return res;
	      } else {
	        return createLocal(input).local();
	      }
	    }

	    function getDateOffset(m) {
	      // On Firefox.24 Date#getTimezoneOffset returns a floating point.
	      // https://github.com/moment/moment/pull/1871
	      return -Math.round(m._d.getTimezoneOffset());
	    } // HOOKS
	    // This function will be called whenever a moment is mutated.
	    // It is intended to keep the offset in sync with the timezone.


	    hooks.updateOffset = function () {}; // MOMENTS
	    // keepLocalTime = true means only change the timezone, without
	    // affecting the local hour. So 5:31:26 +0300 --[utcOffset(2, true)]-->
	    // 5:31:26 +0200 It is possible that 5:31:26 doesn't exist with offset
	    // +0200, so we adjust the time as needed, to be valid.
	    //
	    // Keeping the time actually adds/subtracts (one hour)
	    // from the actual represented time. That is why we call updateOffset
	    // a second time. In case it wants us to change the offset again
	    // _changeInProgress == true case, then we have to adjust, because
	    // there is no such time in the given timezone.


	    function getSetOffset(input, keepLocalTime, keepMinutes) {
	      var offset = this._offset || 0,
	          localAdjust;

	      if (!this.isValid()) {
	        return input != null ? this : NaN;
	      }

	      if (input != null) {
	        if (typeof input === 'string') {
	          input = offsetFromString(matchShortOffset, input);

	          if (input === null) {
	            return this;
	          }
	        } else if (Math.abs(input) < 16 && !keepMinutes) {
	          input = input * 60;
	        }

	        if (!this._isUTC && keepLocalTime) {
	          localAdjust = getDateOffset(this);
	        }

	        this._offset = input;
	        this._isUTC = true;

	        if (localAdjust != null) {
	          this.add(localAdjust, 'm');
	        }

	        if (offset !== input) {
	          if (!keepLocalTime || this._changeInProgress) {
	            addSubtract(this, createDuration(input - offset, 'm'), 1, false);
	          } else if (!this._changeInProgress) {
	            this._changeInProgress = true;
	            hooks.updateOffset(this, true);
	            this._changeInProgress = null;
	          }
	        }

	        return this;
	      } else {
	        return this._isUTC ? offset : getDateOffset(this);
	      }
	    }

	    function getSetZone(input, keepLocalTime) {
	      if (input != null) {
	        if (typeof input !== 'string') {
	          input = -input;
	        }

	        this.utcOffset(input, keepLocalTime);
	        return this;
	      } else {
	        return -this.utcOffset();
	      }
	    }

	    function setOffsetToUTC(keepLocalTime) {
	      return this.utcOffset(0, keepLocalTime);
	    }

	    function setOffsetToLocal(keepLocalTime) {
	      if (this._isUTC) {
	        this.utcOffset(0, keepLocalTime);
	        this._isUTC = false;

	        if (keepLocalTime) {
	          this.subtract(getDateOffset(this), 'm');
	        }
	      }

	      return this;
	    }

	    function setOffsetToParsedOffset() {
	      if (this._tzm != null) {
	        this.utcOffset(this._tzm, false, true);
	      } else if (typeof this._i === 'string') {
	        var tZone = offsetFromString(matchOffset, this._i);

	        if (tZone != null) {
	          this.utcOffset(tZone);
	        } else {
	          this.utcOffset(0, true);
	        }
	      }

	      return this;
	    }

	    function hasAlignedHourOffset(input) {
	      if (!this.isValid()) {
	        return false;
	      }

	      input = input ? createLocal(input).utcOffset() : 0;
	      return (this.utcOffset() - input) % 60 === 0;
	    }

	    function isDaylightSavingTime() {
	      return this.utcOffset() > this.clone().month(0).utcOffset() || this.utcOffset() > this.clone().month(5).utcOffset();
	    }

	    function isDaylightSavingTimeShifted() {
	      if (!isUndefined(this._isDSTShifted)) {
	        return this._isDSTShifted;
	      }

	      var c = {},
	          other;
	      copyConfig(c, this);
	      c = prepareConfig(c);

	      if (c._a) {
	        other = c._isUTC ? createUTC(c._a) : createLocal(c._a);
	        this._isDSTShifted = this.isValid() && compareArrays(c._a, other.toArray()) > 0;
	      } else {
	        this._isDSTShifted = false;
	      }

	      return this._isDSTShifted;
	    }

	    function isLocal() {
	      return this.isValid() ? !this._isUTC : false;
	    }

	    function isUtcOffset() {
	      return this.isValid() ? this._isUTC : false;
	    }

	    function isUtc() {
	      return this.isValid() ? this._isUTC && this._offset === 0 : false;
	    } // ASP.NET json date format regex


	    var aspNetRegex = /^(-|\+)?(?:(\d*)[. ])?(\d+):(\d+)(?::(\d+)(\.\d*)?)?$/,
	        // from http://docs.closure-library.googlecode.com/git/closure_goog_date_date.js.source.html
	    // somewhat more in line with 4.4.3.2 2004 spec, but allows decimal anywhere
	    // and further modified to allow for strings containing both week and day
	    isoRegex = /^(-|\+)?P(?:([-+]?[0-9,.]*)Y)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)W)?(?:([-+]?[0-9,.]*)D)?(?:T(?:([-+]?[0-9,.]*)H)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)S)?)?$/;

	    function createDuration(input, key) {
	      var duration = input,
	          // matching against regexp is expensive, do it on demand
	      match = null,
	          sign,
	          ret,
	          diffRes;

	      if (isDuration(input)) {
	        duration = {
	          ms: input._milliseconds,
	          d: input._days,
	          M: input._months
	        };
	      } else if (isNumber(input) || !isNaN(+input)) {
	        duration = {};

	        if (key) {
	          duration[key] = +input;
	        } else {
	          duration.milliseconds = +input;
	        }
	      } else if (match = aspNetRegex.exec(input)) {
	        sign = match[1] === '-' ? -1 : 1;
	        duration = {
	          y: 0,
	          d: toInt(match[DATE]) * sign,
	          h: toInt(match[HOUR]) * sign,
	          m: toInt(match[MINUTE]) * sign,
	          s: toInt(match[SECOND]) * sign,
	          ms: toInt(absRound(match[MILLISECOND] * 1000)) * sign // the millisecond decimal point is included in the match

	        };
	      } else if (match = isoRegex.exec(input)) {
	        sign = match[1] === '-' ? -1 : 1;
	        duration = {
	          y: parseIso(match[2], sign),
	          M: parseIso(match[3], sign),
	          w: parseIso(match[4], sign),
	          d: parseIso(match[5], sign),
	          h: parseIso(match[6], sign),
	          m: parseIso(match[7], sign),
	          s: parseIso(match[8], sign)
	        };
	      } else if (duration == null) {
	        // checks for null or undefined
	        duration = {};
	      } else if (typeof duration === 'object' && ('from' in duration || 'to' in duration)) {
	        diffRes = momentsDifference(createLocal(duration.from), createLocal(duration.to));
	        duration = {};
	        duration.ms = diffRes.milliseconds;
	        duration.M = diffRes.months;
	      }

	      ret = new Duration(duration);

	      if (isDuration(input) && hasOwnProp(input, '_locale')) {
	        ret._locale = input._locale;
	      }

	      if (isDuration(input) && hasOwnProp(input, '_isValid')) {
	        ret._isValid = input._isValid;
	      }

	      return ret;
	    }

	    createDuration.fn = Duration.prototype;
	    createDuration.invalid = createInvalid$1;

	    function parseIso(inp, sign) {
	      // We'd normally use ~~inp for this, but unfortunately it also
	      // converts floats to ints.
	      // inp may be undefined, so careful calling replace on it.
	      var res = inp && parseFloat(inp.replace(',', '.')); // apply sign while we're at it

	      return (isNaN(res) ? 0 : res) * sign;
	    }

	    function positiveMomentsDifference(base, other) {
	      var res = {};
	      res.months = other.month() - base.month() + (other.year() - base.year()) * 12;

	      if (base.clone().add(res.months, 'M').isAfter(other)) {
	        --res.months;
	      }

	      res.milliseconds = +other - +base.clone().add(res.months, 'M');
	      return res;
	    }

	    function momentsDifference(base, other) {
	      var res;

	      if (!(base.isValid() && other.isValid())) {
	        return {
	          milliseconds: 0,
	          months: 0
	        };
	      }

	      other = cloneWithOffset(other, base);

	      if (base.isBefore(other)) {
	        res = positiveMomentsDifference(base, other);
	      } else {
	        res = positiveMomentsDifference(other, base);
	        res.milliseconds = -res.milliseconds;
	        res.months = -res.months;
	      }

	      return res;
	    } // TODO: remove 'name' arg after deprecation is removed


	    function createAdder(direction, name) {
	      return function (val, period) {
	        var dur, tmp; //invert the arguments, but complain about it

	        if (period !== null && !isNaN(+period)) {
	          deprecateSimple(name, 'moment().' + name + '(period, number) is deprecated. Please use moment().' + name + '(number, period). ' + 'See http://momentjs.com/guides/#/warnings/add-inverted-param/ for more info.');
	          tmp = val;
	          val = period;
	          period = tmp;
	        }

	        dur = createDuration(val, period);
	        addSubtract(this, dur, direction);
	        return this;
	      };
	    }

	    function addSubtract(mom, duration, isAdding, updateOffset) {
	      var milliseconds = duration._milliseconds,
	          days = absRound(duration._days),
	          months = absRound(duration._months);

	      if (!mom.isValid()) {
	        // No op
	        return;
	      }

	      updateOffset = updateOffset == null ? true : updateOffset;

	      if (months) {
	        setMonth(mom, get(mom, 'Month') + months * isAdding);
	      }

	      if (days) {
	        set$1(mom, 'Date', get(mom, 'Date') + days * isAdding);
	      }

	      if (milliseconds) {
	        mom._d.setTime(mom._d.valueOf() + milliseconds * isAdding);
	      }

	      if (updateOffset) {
	        hooks.updateOffset(mom, days || months);
	      }
	    }

	    var add = createAdder(1, 'add'),
	        subtract = createAdder(-1, 'subtract');

	    function isString(input) {
	      return typeof input === 'string' || input instanceof String;
	    } // type MomentInput = Moment | Date | string | number | (number | string)[] | MomentInputObject | void; // null | undefined


	    function isMomentInput(input) {
	      return isMoment(input) || isDate(input) || isString(input) || isNumber(input) || isNumberOrStringArray(input) || isMomentInputObject(input) || input === null || input === undefined;
	    }

	    function isMomentInputObject(input) {
	      var objectTest = isObject(input) && !isObjectEmpty(input),
	          propertyTest = false,
	          properties = ['years', 'year', 'y', 'months', 'month', 'M', 'days', 'day', 'd', 'dates', 'date', 'D', 'hours', 'hour', 'h', 'minutes', 'minute', 'm', 'seconds', 'second', 's', 'milliseconds', 'millisecond', 'ms'],
	          i,
	          property;

	      for (i = 0; i < properties.length; i += 1) {
	        property = properties[i];
	        propertyTest = propertyTest || hasOwnProp(input, property);
	      }

	      return objectTest && propertyTest;
	    }

	    function isNumberOrStringArray(input) {
	      var arrayTest = isArray(input),
	          dataTypeTest = false;

	      if (arrayTest) {
	        dataTypeTest = input.filter(function (item) {
	          return !isNumber(item) && isString(input);
	        }).length === 0;
	      }

	      return arrayTest && dataTypeTest;
	    }

	    function isCalendarSpec(input) {
	      var objectTest = isObject(input) && !isObjectEmpty(input),
	          propertyTest = false,
	          properties = ['sameDay', 'nextDay', 'lastDay', 'nextWeek', 'lastWeek', 'sameElse'],
	          i,
	          property;

	      for (i = 0; i < properties.length; i += 1) {
	        property = properties[i];
	        propertyTest = propertyTest || hasOwnProp(input, property);
	      }

	      return objectTest && propertyTest;
	    }

	    function getCalendarFormat(myMoment, now) {
	      var diff = myMoment.diff(now, 'days', true);
	      return diff < -6 ? 'sameElse' : diff < -1 ? 'lastWeek' : diff < 0 ? 'lastDay' : diff < 1 ? 'sameDay' : diff < 2 ? 'nextDay' : diff < 7 ? 'nextWeek' : 'sameElse';
	    }

	    function calendar$1(time, formats) {
	      // Support for single parameter, formats only overload to the calendar function
	      if (arguments.length === 1) {
	        if (isMomentInput(arguments[0])) {
	          time = arguments[0];
	          formats = undefined;
	        } else if (isCalendarSpec(arguments[0])) {
	          formats = arguments[0];
	          time = undefined;
	        }
	      } // We want to compare the start of today, vs this.
	      // Getting start-of-today depends on whether we're local/utc/offset or not.


	      var now = time || createLocal(),
	          sod = cloneWithOffset(now, this).startOf('day'),
	          format = hooks.calendarFormat(this, sod) || 'sameElse',
	          output = formats && (isFunction(formats[format]) ? formats[format].call(this, now) : formats[format]);
	      return this.format(output || this.localeData().calendar(format, this, createLocal(now)));
	    }

	    function clone() {
	      return new Moment(this);
	    }

	    function isAfter(input, units) {
	      var localInput = isMoment(input) ? input : createLocal(input);

	      if (!(this.isValid() && localInput.isValid())) {
	        return false;
	      }

	      units = normalizeUnits(units) || 'millisecond';

	      if (units === 'millisecond') {
	        return this.valueOf() > localInput.valueOf();
	      } else {
	        return localInput.valueOf() < this.clone().startOf(units).valueOf();
	      }
	    }

	    function isBefore(input, units) {
	      var localInput = isMoment(input) ? input : createLocal(input);

	      if (!(this.isValid() && localInput.isValid())) {
	        return false;
	      }

	      units = normalizeUnits(units) || 'millisecond';

	      if (units === 'millisecond') {
	        return this.valueOf() < localInput.valueOf();
	      } else {
	        return this.clone().endOf(units).valueOf() < localInput.valueOf();
	      }
	    }

	    function isBetween(from, to, units, inclusivity) {
	      var localFrom = isMoment(from) ? from : createLocal(from),
	          localTo = isMoment(to) ? to : createLocal(to);

	      if (!(this.isValid() && localFrom.isValid() && localTo.isValid())) {
	        return false;
	      }

	      inclusivity = inclusivity || '()';
	      return (inclusivity[0] === '(' ? this.isAfter(localFrom, units) : !this.isBefore(localFrom, units)) && (inclusivity[1] === ')' ? this.isBefore(localTo, units) : !this.isAfter(localTo, units));
	    }

	    function isSame(input, units) {
	      var localInput = isMoment(input) ? input : createLocal(input),
	          inputMs;

	      if (!(this.isValid() && localInput.isValid())) {
	        return false;
	      }

	      units = normalizeUnits(units) || 'millisecond';

	      if (units === 'millisecond') {
	        return this.valueOf() === localInput.valueOf();
	      } else {
	        inputMs = localInput.valueOf();
	        return this.clone().startOf(units).valueOf() <= inputMs && inputMs <= this.clone().endOf(units).valueOf();
	      }
	    }

	    function isSameOrAfter(input, units) {
	      return this.isSame(input, units) || this.isAfter(input, units);
	    }

	    function isSameOrBefore(input, units) {
	      return this.isSame(input, units) || this.isBefore(input, units);
	    }

	    function diff(input, units, asFloat) {
	      var that, zoneDelta, output;

	      if (!this.isValid()) {
	        return NaN;
	      }

	      that = cloneWithOffset(input, this);

	      if (!that.isValid()) {
	        return NaN;
	      }

	      zoneDelta = (that.utcOffset() - this.utcOffset()) * 6e4;
	      units = normalizeUnits(units);

	      switch (units) {
	        case 'year':
	          output = monthDiff(this, that) / 12;
	          break;

	        case 'month':
	          output = monthDiff(this, that);
	          break;

	        case 'quarter':
	          output = monthDiff(this, that) / 3;
	          break;

	        case 'second':
	          output = (this - that) / 1e3;
	          break;
	        // 1000

	        case 'minute':
	          output = (this - that) / 6e4;
	          break;
	        // 1000 * 60

	        case 'hour':
	          output = (this - that) / 36e5;
	          break;
	        // 1000 * 60 * 60

	        case 'day':
	          output = (this - that - zoneDelta) / 864e5;
	          break;
	        // 1000 * 60 * 60 * 24, negate dst

	        case 'week':
	          output = (this - that - zoneDelta) / 6048e5;
	          break;
	        // 1000 * 60 * 60 * 24 * 7, negate dst

	        default:
	          output = this - that;
	      }

	      return asFloat ? output : absFloor(output);
	    }

	    function monthDiff(a, b) {
	      if (a.date() < b.date()) {
	        // end-of-month calculations work correct when the start month has more
	        // days than the end month.
	        return -monthDiff(b, a);
	      } // difference in months


	      var wholeMonthDiff = (b.year() - a.year()) * 12 + (b.month() - a.month()),
	          // b is in (anchor - 1 month, anchor + 1 month)
	      anchor = a.clone().add(wholeMonthDiff, 'months'),
	          anchor2,
	          adjust;

	      if (b - anchor < 0) {
	        anchor2 = a.clone().add(wholeMonthDiff - 1, 'months'); // linear across the month

	        adjust = (b - anchor) / (anchor - anchor2);
	      } else {
	        anchor2 = a.clone().add(wholeMonthDiff + 1, 'months'); // linear across the month

	        adjust = (b - anchor) / (anchor2 - anchor);
	      } //check for negative zero, return zero if negative zero


	      return -(wholeMonthDiff + adjust) || 0;
	    }

	    hooks.defaultFormat = 'YYYY-MM-DDTHH:mm:ssZ';
	    hooks.defaultFormatUtc = 'YYYY-MM-DDTHH:mm:ss[Z]';

	    function toString() {
	      return this.clone().locale('en').format('ddd MMM DD YYYY HH:mm:ss [GMT]ZZ');
	    }

	    function toISOString(keepOffset) {
	      if (!this.isValid()) {
	        return null;
	      }

	      var utc = keepOffset !== true,
	          m = utc ? this.clone().utc() : this;

	      if (m.year() < 0 || m.year() > 9999) {
	        return formatMoment(m, utc ? 'YYYYYY-MM-DD[T]HH:mm:ss.SSS[Z]' : 'YYYYYY-MM-DD[T]HH:mm:ss.SSSZ');
	      }

	      if (isFunction(Date.prototype.toISOString)) {
	        // native implementation is ~50x faster, use it when we can
	        if (utc) {
	          return this.toDate().toISOString();
	        } else {
	          return new Date(this.valueOf() + this.utcOffset() * 60 * 1000).toISOString().replace('Z', formatMoment(m, 'Z'));
	        }
	      }

	      return formatMoment(m, utc ? 'YYYY-MM-DD[T]HH:mm:ss.SSS[Z]' : 'YYYY-MM-DD[T]HH:mm:ss.SSSZ');
	    }
	    /**
	     * Return a human readable representation of a moment that can
	     * also be evaluated to get a new moment which is the same
	     *
	     * @link https://nodejs.org/dist/latest/docs/api/util.html#util_custom_inspect_function_on_objects
	     */


	    function inspect() {
	      if (!this.isValid()) {
	        return 'moment.invalid(/* ' + this._i + ' */)';
	      }

	      var func = 'moment',
	          zone = '',
	          prefix,
	          year,
	          datetime,
	          suffix;

	      if (!this.isLocal()) {
	        func = this.utcOffset() === 0 ? 'moment.utc' : 'moment.parseZone';
	        zone = 'Z';
	      }

	      prefix = '[' + func + '("]';
	      year = 0 <= this.year() && this.year() <= 9999 ? 'YYYY' : 'YYYYYY';
	      datetime = '-MM-DD[T]HH:mm:ss.SSS';
	      suffix = zone + '[")]';
	      return this.format(prefix + year + datetime + suffix);
	    }

	    function format(inputString) {
	      if (!inputString) {
	        inputString = this.isUtc() ? hooks.defaultFormatUtc : hooks.defaultFormat;
	      }

	      var output = formatMoment(this, inputString);
	      return this.localeData().postformat(output);
	    }

	    function from(time, withoutSuffix) {
	      if (this.isValid() && (isMoment(time) && time.isValid() || createLocal(time).isValid())) {
	        return createDuration({
	          to: this,
	          from: time
	        }).locale(this.locale()).humanize(!withoutSuffix);
	      } else {
	        return this.localeData().invalidDate();
	      }
	    }

	    function fromNow(withoutSuffix) {
	      return this.from(createLocal(), withoutSuffix);
	    }

	    function to(time, withoutSuffix) {
	      if (this.isValid() && (isMoment(time) && time.isValid() || createLocal(time).isValid())) {
	        return createDuration({
	          from: this,
	          to: time
	        }).locale(this.locale()).humanize(!withoutSuffix);
	      } else {
	        return this.localeData().invalidDate();
	      }
	    }

	    function toNow(withoutSuffix) {
	      return this.to(createLocal(), withoutSuffix);
	    } // If passed a locale key, it will set the locale for this
	    // instance.  Otherwise, it will return the locale configuration
	    // variables for this instance.


	    function locale(key) {
	      var newLocaleData;

	      if (key === undefined) {
	        return this._locale._abbr;
	      } else {
	        newLocaleData = getLocale(key);

	        if (newLocaleData != null) {
	          this._locale = newLocaleData;
	        }

	        return this;
	      }
	    }

	    var lang = deprecate('moment().lang() is deprecated. Instead, use moment().localeData() to get the language configuration. Use moment().locale() to change languages.', function (key) {
	      if (key === undefined) {
	        return this.localeData();
	      } else {
	        return this.locale(key);
	      }
	    });

	    function localeData() {
	      return this._locale;
	    }

	    var MS_PER_SECOND = 1000,
	        MS_PER_MINUTE = 60 * MS_PER_SECOND,
	        MS_PER_HOUR = 60 * MS_PER_MINUTE,
	        MS_PER_400_YEARS = (365 * 400 + 97) * 24 * MS_PER_HOUR; // actual modulo - handles negative numbers (for dates before 1970):

	    function mod$1(dividend, divisor) {
	      return (dividend % divisor + divisor) % divisor;
	    }

	    function localStartOfDate(y, m, d) {
	      // the date constructor remaps years 0-99 to 1900-1999
	      if (y < 100 && y >= 0) {
	        // preserve leap years using a full 400 year cycle, then reset
	        return new Date(y + 400, m, d) - MS_PER_400_YEARS;
	      } else {
	        return new Date(y, m, d).valueOf();
	      }
	    }

	    function utcStartOfDate(y, m, d) {
	      // Date.UTC remaps years 0-99 to 1900-1999
	      if (y < 100 && y >= 0) {
	        // preserve leap years using a full 400 year cycle, then reset
	        return Date.UTC(y + 400, m, d) - MS_PER_400_YEARS;
	      } else {
	        return Date.UTC(y, m, d);
	      }
	    }

	    function startOf(units) {
	      var time, startOfDate;
	      units = normalizeUnits(units);

	      if (units === undefined || units === 'millisecond' || !this.isValid()) {
	        return this;
	      }

	      startOfDate = this._isUTC ? utcStartOfDate : localStartOfDate;

	      switch (units) {
	        case 'year':
	          time = startOfDate(this.year(), 0, 1);
	          break;

	        case 'quarter':
	          time = startOfDate(this.year(), this.month() - this.month() % 3, 1);
	          break;

	        case 'month':
	          time = startOfDate(this.year(), this.month(), 1);
	          break;

	        case 'week':
	          time = startOfDate(this.year(), this.month(), this.date() - this.weekday());
	          break;

	        case 'isoWeek':
	          time = startOfDate(this.year(), this.month(), this.date() - (this.isoWeekday() - 1));
	          break;

	        case 'day':
	        case 'date':
	          time = startOfDate(this.year(), this.month(), this.date());
	          break;

	        case 'hour':
	          time = this._d.valueOf();
	          time -= mod$1(time + (this._isUTC ? 0 : this.utcOffset() * MS_PER_MINUTE), MS_PER_HOUR);
	          break;

	        case 'minute':
	          time = this._d.valueOf();
	          time -= mod$1(time, MS_PER_MINUTE);
	          break;

	        case 'second':
	          time = this._d.valueOf();
	          time -= mod$1(time, MS_PER_SECOND);
	          break;
	      }

	      this._d.setTime(time);

	      hooks.updateOffset(this, true);
	      return this;
	    }

	    function endOf(units) {
	      var time, startOfDate;
	      units = normalizeUnits(units);

	      if (units === undefined || units === 'millisecond' || !this.isValid()) {
	        return this;
	      }

	      startOfDate = this._isUTC ? utcStartOfDate : localStartOfDate;

	      switch (units) {
	        case 'year':
	          time = startOfDate(this.year() + 1, 0, 1) - 1;
	          break;

	        case 'quarter':
	          time = startOfDate(this.year(), this.month() - this.month() % 3 + 3, 1) - 1;
	          break;

	        case 'month':
	          time = startOfDate(this.year(), this.month() + 1, 1) - 1;
	          break;

	        case 'week':
	          time = startOfDate(this.year(), this.month(), this.date() - this.weekday() + 7) - 1;
	          break;

	        case 'isoWeek':
	          time = startOfDate(this.year(), this.month(), this.date() - (this.isoWeekday() - 1) + 7) - 1;
	          break;

	        case 'day':
	        case 'date':
	          time = startOfDate(this.year(), this.month(), this.date() + 1) - 1;
	          break;

	        case 'hour':
	          time = this._d.valueOf();
	          time += MS_PER_HOUR - mod$1(time + (this._isUTC ? 0 : this.utcOffset() * MS_PER_MINUTE), MS_PER_HOUR) - 1;
	          break;

	        case 'minute':
	          time = this._d.valueOf();
	          time += MS_PER_MINUTE - mod$1(time, MS_PER_MINUTE) - 1;
	          break;

	        case 'second':
	          time = this._d.valueOf();
	          time += MS_PER_SECOND - mod$1(time, MS_PER_SECOND) - 1;
	          break;
	      }

	      this._d.setTime(time);

	      hooks.updateOffset(this, true);
	      return this;
	    }

	    function valueOf() {
	      return this._d.valueOf() - (this._offset || 0) * 60000;
	    }

	    function unix() {
	      return Math.floor(this.valueOf() / 1000);
	    }

	    function toDate() {
	      return new Date(this.valueOf());
	    }

	    function toArray() {
	      var m = this;
	      return [m.year(), m.month(), m.date(), m.hour(), m.minute(), m.second(), m.millisecond()];
	    }

	    function toObject() {
	      var m = this;
	      return {
	        years: m.year(),
	        months: m.month(),
	        date: m.date(),
	        hours: m.hours(),
	        minutes: m.minutes(),
	        seconds: m.seconds(),
	        milliseconds: m.milliseconds()
	      };
	    }

	    function toJSON() {
	      // new Date(NaN).toJSON() === null
	      return this.isValid() ? this.toISOString() : null;
	    }

	    function isValid$2() {
	      return isValid(this);
	    }

	    function parsingFlags() {
	      return extend({}, getParsingFlags(this));
	    }

	    function invalidAt() {
	      return getParsingFlags(this).overflow;
	    }

	    function creationData() {
	      return {
	        input: this._i,
	        format: this._f,
	        locale: this._locale,
	        isUTC: this._isUTC,
	        strict: this._strict
	      };
	    }

	    addFormatToken('N', 0, 0, 'eraAbbr');
	    addFormatToken('NN', 0, 0, 'eraAbbr');
	    addFormatToken('NNN', 0, 0, 'eraAbbr');
	    addFormatToken('NNNN', 0, 0, 'eraName');
	    addFormatToken('NNNNN', 0, 0, 'eraNarrow');
	    addFormatToken('y', ['y', 1], 'yo', 'eraYear');
	    addFormatToken('y', ['yy', 2], 0, 'eraYear');
	    addFormatToken('y', ['yyy', 3], 0, 'eraYear');
	    addFormatToken('y', ['yyyy', 4], 0, 'eraYear');
	    addRegexToken('N', matchEraAbbr);
	    addRegexToken('NN', matchEraAbbr);
	    addRegexToken('NNN', matchEraAbbr);
	    addRegexToken('NNNN', matchEraName);
	    addRegexToken('NNNNN', matchEraNarrow);
	    addParseToken(['N', 'NN', 'NNN', 'NNNN', 'NNNNN'], function (input, array, config, token) {
	      var era = config._locale.erasParse(input, token, config._strict);

	      if (era) {
	        getParsingFlags(config).era = era;
	      } else {
	        getParsingFlags(config).invalidEra = input;
	      }
	    });
	    addRegexToken('y', matchUnsigned);
	    addRegexToken('yy', matchUnsigned);
	    addRegexToken('yyy', matchUnsigned);
	    addRegexToken('yyyy', matchUnsigned);
	    addRegexToken('yo', matchEraYearOrdinal);
	    addParseToken(['y', 'yy', 'yyy', 'yyyy'], YEAR);
	    addParseToken(['yo'], function (input, array, config, token) {
	      var match;

	      if (config._locale._eraYearOrdinalRegex) {
	        match = input.match(config._locale._eraYearOrdinalRegex);
	      }

	      if (config._locale.eraYearOrdinalParse) {
	        array[YEAR] = config._locale.eraYearOrdinalParse(input, match);
	      } else {
	        array[YEAR] = parseInt(input, 10);
	      }
	    });

	    function localeEras(m, format) {
	      var i,
	          l,
	          date,
	          eras = this._eras || getLocale('en')._eras;

	      for (i = 0, l = eras.length; i < l; ++i) {
	        switch (typeof eras[i].since) {
	          case 'string':
	            // truncate time
	            date = hooks(eras[i].since).startOf('day');
	            eras[i].since = date.valueOf();
	            break;
	        }

	        switch (typeof eras[i].until) {
	          case 'undefined':
	            eras[i].until = +Infinity;
	            break;

	          case 'string':
	            // truncate time
	            date = hooks(eras[i].until).startOf('day').valueOf();
	            eras[i].until = date.valueOf();
	            break;
	        }
	      }

	      return eras;
	    }

	    function localeErasParse(eraName, format, strict) {
	      var i,
	          l,
	          eras = this.eras(),
	          name,
	          abbr,
	          narrow;
	      eraName = eraName.toUpperCase();

	      for (i = 0, l = eras.length; i < l; ++i) {
	        name = eras[i].name.toUpperCase();
	        abbr = eras[i].abbr.toUpperCase();
	        narrow = eras[i].narrow.toUpperCase();

	        if (strict) {
	          switch (format) {
	            case 'N':
	            case 'NN':
	            case 'NNN':
	              if (abbr === eraName) {
	                return eras[i];
	              }

	              break;

	            case 'NNNN':
	              if (name === eraName) {
	                return eras[i];
	              }

	              break;

	            case 'NNNNN':
	              if (narrow === eraName) {
	                return eras[i];
	              }

	              break;
	          }
	        } else if ([name, abbr, narrow].indexOf(eraName) >= 0) {
	          return eras[i];
	        }
	      }
	    }

	    function localeErasConvertYear(era, year) {
	      var dir = era.since <= era.until ? +1 : -1;

	      if (year === undefined) {
	        return hooks(era.since).year();
	      } else {
	        return hooks(era.since).year() + (year - era.offset) * dir;
	      }
	    }

	    function getEraName() {
	      var i,
	          l,
	          val,
	          eras = this.localeData().eras();

	      for (i = 0, l = eras.length; i < l; ++i) {
	        // truncate time
	        val = this.startOf('day').valueOf();

	        if (eras[i].since <= val && val <= eras[i].until) {
	          return eras[i].name;
	        }

	        if (eras[i].until <= val && val <= eras[i].since) {
	          return eras[i].name;
	        }
	      }

	      return '';
	    }

	    function getEraNarrow() {
	      var i,
	          l,
	          val,
	          eras = this.localeData().eras();

	      for (i = 0, l = eras.length; i < l; ++i) {
	        // truncate time
	        val = this.startOf('day').valueOf();

	        if (eras[i].since <= val && val <= eras[i].until) {
	          return eras[i].narrow;
	        }

	        if (eras[i].until <= val && val <= eras[i].since) {
	          return eras[i].narrow;
	        }
	      }

	      return '';
	    }

	    function getEraAbbr() {
	      var i,
	          l,
	          val,
	          eras = this.localeData().eras();

	      for (i = 0, l = eras.length; i < l; ++i) {
	        // truncate time
	        val = this.startOf('day').valueOf();

	        if (eras[i].since <= val && val <= eras[i].until) {
	          return eras[i].abbr;
	        }

	        if (eras[i].until <= val && val <= eras[i].since) {
	          return eras[i].abbr;
	        }
	      }

	      return '';
	    }

	    function getEraYear() {
	      var i,
	          l,
	          dir,
	          val,
	          eras = this.localeData().eras();

	      for (i = 0, l = eras.length; i < l; ++i) {
	        dir = eras[i].since <= eras[i].until ? +1 : -1; // truncate time

	        val = this.startOf('day').valueOf();

	        if (eras[i].since <= val && val <= eras[i].until || eras[i].until <= val && val <= eras[i].since) {
	          return (this.year() - hooks(eras[i].since).year()) * dir + eras[i].offset;
	        }
	      }

	      return this.year();
	    }

	    function erasNameRegex(isStrict) {
	      if (!hasOwnProp(this, '_erasNameRegex')) {
	        computeErasParse.call(this);
	      }

	      return isStrict ? this._erasNameRegex : this._erasRegex;
	    }

	    function erasAbbrRegex(isStrict) {
	      if (!hasOwnProp(this, '_erasAbbrRegex')) {
	        computeErasParse.call(this);
	      }

	      return isStrict ? this._erasAbbrRegex : this._erasRegex;
	    }

	    function erasNarrowRegex(isStrict) {
	      if (!hasOwnProp(this, '_erasNarrowRegex')) {
	        computeErasParse.call(this);
	      }

	      return isStrict ? this._erasNarrowRegex : this._erasRegex;
	    }

	    function matchEraAbbr(isStrict, locale) {
	      return locale.erasAbbrRegex(isStrict);
	    }

	    function matchEraName(isStrict, locale) {
	      return locale.erasNameRegex(isStrict);
	    }

	    function matchEraNarrow(isStrict, locale) {
	      return locale.erasNarrowRegex(isStrict);
	    }

	    function matchEraYearOrdinal(isStrict, locale) {
	      return locale._eraYearOrdinalRegex || matchUnsigned;
	    }

	    function computeErasParse() {
	      var abbrPieces = [],
	          namePieces = [],
	          narrowPieces = [],
	          mixedPieces = [],
	          i,
	          l,
	          eras = this.eras();

	      for (i = 0, l = eras.length; i < l; ++i) {
	        namePieces.push(regexEscape(eras[i].name));
	        abbrPieces.push(regexEscape(eras[i].abbr));
	        narrowPieces.push(regexEscape(eras[i].narrow));
	        mixedPieces.push(regexEscape(eras[i].name));
	        mixedPieces.push(regexEscape(eras[i].abbr));
	        mixedPieces.push(regexEscape(eras[i].narrow));
	      }

	      this._erasRegex = new RegExp('^(' + mixedPieces.join('|') + ')', 'i');
	      this._erasNameRegex = new RegExp('^(' + namePieces.join('|') + ')', 'i');
	      this._erasAbbrRegex = new RegExp('^(' + abbrPieces.join('|') + ')', 'i');
	      this._erasNarrowRegex = new RegExp('^(' + narrowPieces.join('|') + ')', 'i');
	    } // FORMATTING


	    addFormatToken(0, ['gg', 2], 0, function () {
	      return this.weekYear() % 100;
	    });
	    addFormatToken(0, ['GG', 2], 0, function () {
	      return this.isoWeekYear() % 100;
	    });

	    function addWeekYearFormatToken(token, getter) {
	      addFormatToken(0, [token, token.length], 0, getter);
	    }

	    addWeekYearFormatToken('gggg', 'weekYear');
	    addWeekYearFormatToken('ggggg', 'weekYear');
	    addWeekYearFormatToken('GGGG', 'isoWeekYear');
	    addWeekYearFormatToken('GGGGG', 'isoWeekYear'); // ALIASES

	    addUnitAlias('weekYear', 'gg');
	    addUnitAlias('isoWeekYear', 'GG'); // PRIORITY

	    addUnitPriority('weekYear', 1);
	    addUnitPriority('isoWeekYear', 1); // PARSING

	    addRegexToken('G', matchSigned);
	    addRegexToken('g', matchSigned);
	    addRegexToken('GG', match1to2, match2);
	    addRegexToken('gg', match1to2, match2);
	    addRegexToken('GGGG', match1to4, match4);
	    addRegexToken('gggg', match1to4, match4);
	    addRegexToken('GGGGG', match1to6, match6);
	    addRegexToken('ggggg', match1to6, match6);
	    addWeekParseToken(['gggg', 'ggggg', 'GGGG', 'GGGGG'], function (input, week, config, token) {
	      week[token.substr(0, 2)] = toInt(input);
	    });
	    addWeekParseToken(['gg', 'GG'], function (input, week, config, token) {
	      week[token] = hooks.parseTwoDigitYear(input);
	    }); // MOMENTS

	    function getSetWeekYear(input) {
	      return getSetWeekYearHelper.call(this, input, this.week(), this.weekday(), this.localeData()._week.dow, this.localeData()._week.doy);
	    }

	    function getSetISOWeekYear(input) {
	      return getSetWeekYearHelper.call(this, input, this.isoWeek(), this.isoWeekday(), 1, 4);
	    }

	    function getISOWeeksInYear() {
	      return weeksInYear(this.year(), 1, 4);
	    }

	    function getISOWeeksInISOWeekYear() {
	      return weeksInYear(this.isoWeekYear(), 1, 4);
	    }

	    function getWeeksInYear() {
	      var weekInfo = this.localeData()._week;

	      return weeksInYear(this.year(), weekInfo.dow, weekInfo.doy);
	    }

	    function getWeeksInWeekYear() {
	      var weekInfo = this.localeData()._week;

	      return weeksInYear(this.weekYear(), weekInfo.dow, weekInfo.doy);
	    }

	    function getSetWeekYearHelper(input, week, weekday, dow, doy) {
	      var weeksTarget;

	      if (input == null) {
	        return weekOfYear(this, dow, doy).year;
	      } else {
	        weeksTarget = weeksInYear(input, dow, doy);

	        if (week > weeksTarget) {
	          week = weeksTarget;
	        }

	        return setWeekAll.call(this, input, week, weekday, dow, doy);
	      }
	    }

	    function setWeekAll(weekYear, week, weekday, dow, doy) {
	      var dayOfYearData = dayOfYearFromWeeks(weekYear, week, weekday, dow, doy),
	          date = createUTCDate(dayOfYearData.year, 0, dayOfYearData.dayOfYear);
	      this.year(date.getUTCFullYear());
	      this.month(date.getUTCMonth());
	      this.date(date.getUTCDate());
	      return this;
	    } // FORMATTING


	    addFormatToken('Q', 0, 'Qo', 'quarter'); // ALIASES

	    addUnitAlias('quarter', 'Q'); // PRIORITY

	    addUnitPriority('quarter', 7); // PARSING

	    addRegexToken('Q', match1);
	    addParseToken('Q', function (input, array) {
	      array[MONTH] = (toInt(input) - 1) * 3;
	    }); // MOMENTS

	    function getSetQuarter(input) {
	      return input == null ? Math.ceil((this.month() + 1) / 3) : this.month((input - 1) * 3 + this.month() % 3);
	    } // FORMATTING


	    addFormatToken('D', ['DD', 2], 'Do', 'date'); // ALIASES

	    addUnitAlias('date', 'D'); // PRIORITY

	    addUnitPriority('date', 9); // PARSING

	    addRegexToken('D', match1to2);
	    addRegexToken('DD', match1to2, match2);
	    addRegexToken('Do', function (isStrict, locale) {
	      // TODO: Remove "ordinalParse" fallback in next major release.
	      return isStrict ? locale._dayOfMonthOrdinalParse || locale._ordinalParse : locale._dayOfMonthOrdinalParseLenient;
	    });
	    addParseToken(['D', 'DD'], DATE);
	    addParseToken('Do', function (input, array) {
	      array[DATE] = toInt(input.match(match1to2)[0]);
	    }); // MOMENTS

	    var getSetDayOfMonth = makeGetSet('Date', true); // FORMATTING

	    addFormatToken('DDD', ['DDDD', 3], 'DDDo', 'dayOfYear'); // ALIASES

	    addUnitAlias('dayOfYear', 'DDD'); // PRIORITY

	    addUnitPriority('dayOfYear', 4); // PARSING

	    addRegexToken('DDD', match1to3);
	    addRegexToken('DDDD', match3);
	    addParseToken(['DDD', 'DDDD'], function (input, array, config) {
	      config._dayOfYear = toInt(input);
	    }); // HELPERS
	    // MOMENTS

	    function getSetDayOfYear(input) {
	      var dayOfYear = Math.round((this.clone().startOf('day') - this.clone().startOf('year')) / 864e5) + 1;
	      return input == null ? dayOfYear : this.add(input - dayOfYear, 'd');
	    } // FORMATTING


	    addFormatToken('m', ['mm', 2], 0, 'minute'); // ALIASES

	    addUnitAlias('minute', 'm'); // PRIORITY

	    addUnitPriority('minute', 14); // PARSING

	    addRegexToken('m', match1to2);
	    addRegexToken('mm', match1to2, match2);
	    addParseToken(['m', 'mm'], MINUTE); // MOMENTS

	    var getSetMinute = makeGetSet('Minutes', false); // FORMATTING

	    addFormatToken('s', ['ss', 2], 0, 'second'); // ALIASES

	    addUnitAlias('second', 's'); // PRIORITY

	    addUnitPriority('second', 15); // PARSING

	    addRegexToken('s', match1to2);
	    addRegexToken('ss', match1to2, match2);
	    addParseToken(['s', 'ss'], SECOND); // MOMENTS

	    var getSetSecond = makeGetSet('Seconds', false); // FORMATTING

	    addFormatToken('S', 0, 0, function () {
	      return ~~(this.millisecond() / 100);
	    });
	    addFormatToken(0, ['SS', 2], 0, function () {
	      return ~~(this.millisecond() / 10);
	    });
	    addFormatToken(0, ['SSS', 3], 0, 'millisecond');
	    addFormatToken(0, ['SSSS', 4], 0, function () {
	      return this.millisecond() * 10;
	    });
	    addFormatToken(0, ['SSSSS', 5], 0, function () {
	      return this.millisecond() * 100;
	    });
	    addFormatToken(0, ['SSSSSS', 6], 0, function () {
	      return this.millisecond() * 1000;
	    });
	    addFormatToken(0, ['SSSSSSS', 7], 0, function () {
	      return this.millisecond() * 10000;
	    });
	    addFormatToken(0, ['SSSSSSSS', 8], 0, function () {
	      return this.millisecond() * 100000;
	    });
	    addFormatToken(0, ['SSSSSSSSS', 9], 0, function () {
	      return this.millisecond() * 1000000;
	    }); // ALIASES

	    addUnitAlias('millisecond', 'ms'); // PRIORITY

	    addUnitPriority('millisecond', 16); // PARSING

	    addRegexToken('S', match1to3, match1);
	    addRegexToken('SS', match1to3, match2);
	    addRegexToken('SSS', match1to3, match3);
	    var token, getSetMillisecond;

	    for (token = 'SSSS'; token.length <= 9; token += 'S') {
	      addRegexToken(token, matchUnsigned);
	    }

	    function parseMs(input, array) {
	      array[MILLISECOND] = toInt(('0.' + input) * 1000);
	    }

	    for (token = 'S'; token.length <= 9; token += 'S') {
	      addParseToken(token, parseMs);
	    }

	    getSetMillisecond = makeGetSet('Milliseconds', false); // FORMATTING

	    addFormatToken('z', 0, 0, 'zoneAbbr');
	    addFormatToken('zz', 0, 0, 'zoneName'); // MOMENTS

	    function getZoneAbbr() {
	      return this._isUTC ? 'UTC' : '';
	    }

	    function getZoneName() {
	      return this._isUTC ? 'Coordinated Universal Time' : '';
	    }

	    var proto = Moment.prototype;
	    proto.add = add;
	    proto.calendar = calendar$1;
	    proto.clone = clone;
	    proto.diff = diff;
	    proto.endOf = endOf;
	    proto.format = format;
	    proto.from = from;
	    proto.fromNow = fromNow;
	    proto.to = to;
	    proto.toNow = toNow;
	    proto.get = stringGet;
	    proto.invalidAt = invalidAt;
	    proto.isAfter = isAfter;
	    proto.isBefore = isBefore;
	    proto.isBetween = isBetween;
	    proto.isSame = isSame;
	    proto.isSameOrAfter = isSameOrAfter;
	    proto.isSameOrBefore = isSameOrBefore;
	    proto.isValid = isValid$2;
	    proto.lang = lang;
	    proto.locale = locale;
	    proto.localeData = localeData;
	    proto.max = prototypeMax;
	    proto.min = prototypeMin;
	    proto.parsingFlags = parsingFlags;
	    proto.set = stringSet;
	    proto.startOf = startOf;
	    proto.subtract = subtract;
	    proto.toArray = toArray;
	    proto.toObject = toObject;
	    proto.toDate = toDate;
	    proto.toISOString = toISOString;
	    proto.inspect = inspect;

	    if (typeof Symbol !== 'undefined' && Symbol.for != null) {
	      proto[Symbol.for('nodejs.util.inspect.custom')] = function () {
	        return 'Moment<' + this.format() + '>';
	      };
	    }

	    proto.toJSON = toJSON;
	    proto.toString = toString;
	    proto.unix = unix;
	    proto.valueOf = valueOf;
	    proto.creationData = creationData;
	    proto.eraName = getEraName;
	    proto.eraNarrow = getEraNarrow;
	    proto.eraAbbr = getEraAbbr;
	    proto.eraYear = getEraYear;
	    proto.year = getSetYear;
	    proto.isLeapYear = getIsLeapYear;
	    proto.weekYear = getSetWeekYear;
	    proto.isoWeekYear = getSetISOWeekYear;
	    proto.quarter = proto.quarters = getSetQuarter;
	    proto.month = getSetMonth;
	    proto.daysInMonth = getDaysInMonth;
	    proto.week = proto.weeks = getSetWeek;
	    proto.isoWeek = proto.isoWeeks = getSetISOWeek;
	    proto.weeksInYear = getWeeksInYear;
	    proto.weeksInWeekYear = getWeeksInWeekYear;
	    proto.isoWeeksInYear = getISOWeeksInYear;
	    proto.isoWeeksInISOWeekYear = getISOWeeksInISOWeekYear;
	    proto.date = getSetDayOfMonth;
	    proto.day = proto.days = getSetDayOfWeek;
	    proto.weekday = getSetLocaleDayOfWeek;
	    proto.isoWeekday = getSetISODayOfWeek;
	    proto.dayOfYear = getSetDayOfYear;
	    proto.hour = proto.hours = getSetHour;
	    proto.minute = proto.minutes = getSetMinute;
	    proto.second = proto.seconds = getSetSecond;
	    proto.millisecond = proto.milliseconds = getSetMillisecond;
	    proto.utcOffset = getSetOffset;
	    proto.utc = setOffsetToUTC;
	    proto.local = setOffsetToLocal;
	    proto.parseZone = setOffsetToParsedOffset;
	    proto.hasAlignedHourOffset = hasAlignedHourOffset;
	    proto.isDST = isDaylightSavingTime;
	    proto.isLocal = isLocal;
	    proto.isUtcOffset = isUtcOffset;
	    proto.isUtc = isUtc;
	    proto.isUTC = isUtc;
	    proto.zoneAbbr = getZoneAbbr;
	    proto.zoneName = getZoneName;
	    proto.dates = deprecate('dates accessor is deprecated. Use date instead.', getSetDayOfMonth);
	    proto.months = deprecate('months accessor is deprecated. Use month instead', getSetMonth);
	    proto.years = deprecate('years accessor is deprecated. Use year instead', getSetYear);
	    proto.zone = deprecate('moment().zone is deprecated, use moment().utcOffset instead. http://momentjs.com/guides/#/warnings/zone/', getSetZone);
	    proto.isDSTShifted = deprecate('isDSTShifted is deprecated. See http://momentjs.com/guides/#/warnings/dst-shifted/ for more information', isDaylightSavingTimeShifted);

	    function createUnix(input) {
	      return createLocal(input * 1000);
	    }

	    function createInZone() {
	      return createLocal.apply(null, arguments).parseZone();
	    }

	    function preParsePostFormat(string) {
	      return string;
	    }

	    var proto$1 = Locale.prototype;
	    proto$1.calendar = calendar;
	    proto$1.longDateFormat = longDateFormat;
	    proto$1.invalidDate = invalidDate;
	    proto$1.ordinal = ordinal;
	    proto$1.preparse = preParsePostFormat;
	    proto$1.postformat = preParsePostFormat;
	    proto$1.relativeTime = relativeTime;
	    proto$1.pastFuture = pastFuture;
	    proto$1.set = set;
	    proto$1.eras = localeEras;
	    proto$1.erasParse = localeErasParse;
	    proto$1.erasConvertYear = localeErasConvertYear;
	    proto$1.erasAbbrRegex = erasAbbrRegex;
	    proto$1.erasNameRegex = erasNameRegex;
	    proto$1.erasNarrowRegex = erasNarrowRegex;
	    proto$1.months = localeMonths;
	    proto$1.monthsShort = localeMonthsShort;
	    proto$1.monthsParse = localeMonthsParse;
	    proto$1.monthsRegex = monthsRegex;
	    proto$1.monthsShortRegex = monthsShortRegex;
	    proto$1.week = localeWeek;
	    proto$1.firstDayOfYear = localeFirstDayOfYear;
	    proto$1.firstDayOfWeek = localeFirstDayOfWeek;
	    proto$1.weekdays = localeWeekdays;
	    proto$1.weekdaysMin = localeWeekdaysMin;
	    proto$1.weekdaysShort = localeWeekdaysShort;
	    proto$1.weekdaysParse = localeWeekdaysParse;
	    proto$1.weekdaysRegex = weekdaysRegex;
	    proto$1.weekdaysShortRegex = weekdaysShortRegex;
	    proto$1.weekdaysMinRegex = weekdaysMinRegex;
	    proto$1.isPM = localeIsPM;
	    proto$1.meridiem = localeMeridiem;

	    function get$1(format, index, field, setter) {
	      var locale = getLocale(),
	          utc = createUTC().set(setter, index);
	      return locale[field](utc, format);
	    }

	    function listMonthsImpl(format, index, field) {
	      if (isNumber(format)) {
	        index = format;
	        format = undefined;
	      }

	      format = format || '';

	      if (index != null) {
	        return get$1(format, index, field, 'month');
	      }

	      var i,
	          out = [];

	      for (i = 0; i < 12; i++) {
	        out[i] = get$1(format, i, field, 'month');
	      }

	      return out;
	    } // ()
	    // (5)
	    // (fmt, 5)
	    // (fmt)
	    // (true)
	    // (true, 5)
	    // (true, fmt, 5)
	    // (true, fmt)


	    function listWeekdaysImpl(localeSorted, format, index, field) {
	      if (typeof localeSorted === 'boolean') {
	        if (isNumber(format)) {
	          index = format;
	          format = undefined;
	        }

	        format = format || '';
	      } else {
	        format = localeSorted;
	        index = format;
	        localeSorted = false;

	        if (isNumber(format)) {
	          index = format;
	          format = undefined;
	        }

	        format = format || '';
	      }

	      var locale = getLocale(),
	          shift = localeSorted ? locale._week.dow : 0,
	          i,
	          out = [];

	      if (index != null) {
	        return get$1(format, (index + shift) % 7, field, 'day');
	      }

	      for (i = 0; i < 7; i++) {
	        out[i] = get$1(format, (i + shift) % 7, field, 'day');
	      }

	      return out;
	    }

	    function listMonths(format, index) {
	      return listMonthsImpl(format, index, 'months');
	    }

	    function listMonthsShort(format, index) {
	      return listMonthsImpl(format, index, 'monthsShort');
	    }

	    function listWeekdays(localeSorted, format, index) {
	      return listWeekdaysImpl(localeSorted, format, index, 'weekdays');
	    }

	    function listWeekdaysShort(localeSorted, format, index) {
	      return listWeekdaysImpl(localeSorted, format, index, 'weekdaysShort');
	    }

	    function listWeekdaysMin(localeSorted, format, index) {
	      return listWeekdaysImpl(localeSorted, format, index, 'weekdaysMin');
	    }

	    getSetGlobalLocale('en', {
	      eras: [{
	        since: '0001-01-01',
	        until: +Infinity,
	        offset: 1,
	        name: 'Anno Domini',
	        narrow: 'AD',
	        abbr: 'AD'
	      }, {
	        since: '0000-12-31',
	        until: -Infinity,
	        offset: 1,
	        name: 'Before Christ',
	        narrow: 'BC',
	        abbr: 'BC'
	      }],
	      dayOfMonthOrdinalParse: /\d{1,2}(th|st|nd|rd)/,
	      ordinal: function (number) {
	        var b = number % 10,
	            output = toInt(number % 100 / 10) === 1 ? 'th' : b === 1 ? 'st' : b === 2 ? 'nd' : b === 3 ? 'rd' : 'th';
	        return number + output;
	      }
	    }); // Side effect imports

	    hooks.lang = deprecate('moment.lang is deprecated. Use moment.locale instead.', getSetGlobalLocale);
	    hooks.langData = deprecate('moment.langData is deprecated. Use moment.localeData instead.', getLocale);
	    var mathAbs = Math.abs;

	    function abs() {
	      var data = this._data;
	      this._milliseconds = mathAbs(this._milliseconds);
	      this._days = mathAbs(this._days);
	      this._months = mathAbs(this._months);
	      data.milliseconds = mathAbs(data.milliseconds);
	      data.seconds = mathAbs(data.seconds);
	      data.minutes = mathAbs(data.minutes);
	      data.hours = mathAbs(data.hours);
	      data.months = mathAbs(data.months);
	      data.years = mathAbs(data.years);
	      return this;
	    }

	    function addSubtract$1(duration, input, value, direction) {
	      var other = createDuration(input, value);
	      duration._milliseconds += direction * other._milliseconds;
	      duration._days += direction * other._days;
	      duration._months += direction * other._months;
	      return duration._bubble();
	    } // supports only 2.0-style add(1, 's') or add(duration)


	    function add$1(input, value) {
	      return addSubtract$1(this, input, value, 1);
	    } // supports only 2.0-style subtract(1, 's') or subtract(duration)


	    function subtract$1(input, value) {
	      return addSubtract$1(this, input, value, -1);
	    }

	    function absCeil(number) {
	      if (number < 0) {
	        return Math.floor(number);
	      } else {
	        return Math.ceil(number);
	      }
	    }

	    function bubble() {
	      var milliseconds = this._milliseconds,
	          days = this._days,
	          months = this._months,
	          data = this._data,
	          seconds,
	          minutes,
	          hours,
	          years,
	          monthsFromDays; // if we have a mix of positive and negative values, bubble down first
	      // check: https://github.com/moment/moment/issues/2166

	      if (!(milliseconds >= 0 && days >= 0 && months >= 0 || milliseconds <= 0 && days <= 0 && months <= 0)) {
	        milliseconds += absCeil(monthsToDays(months) + days) * 864e5;
	        days = 0;
	        months = 0;
	      } // The following code bubbles up values, see the tests for
	      // examples of what that means.


	      data.milliseconds = milliseconds % 1000;
	      seconds = absFloor(milliseconds / 1000);
	      data.seconds = seconds % 60;
	      minutes = absFloor(seconds / 60);
	      data.minutes = minutes % 60;
	      hours = absFloor(minutes / 60);
	      data.hours = hours % 24;
	      days += absFloor(hours / 24); // convert days to months

	      monthsFromDays = absFloor(daysToMonths(days));
	      months += monthsFromDays;
	      days -= absCeil(monthsToDays(monthsFromDays)); // 12 months -> 1 year

	      years = absFloor(months / 12);
	      months %= 12;
	      data.days = days;
	      data.months = months;
	      data.years = years;
	      return this;
	    }

	    function daysToMonths(days) {
	      // 400 years have 146097 days (taking into account leap year rules)
	      // 400 years have 12 months === 4800
	      return days * 4800 / 146097;
	    }

	    function monthsToDays(months) {
	      // the reverse of daysToMonths
	      return months * 146097 / 4800;
	    }

	    function as(units) {
	      if (!this.isValid()) {
	        return NaN;
	      }

	      var days,
	          months,
	          milliseconds = this._milliseconds;
	      units = normalizeUnits(units);

	      if (units === 'month' || units === 'quarter' || units === 'year') {
	        days = this._days + milliseconds / 864e5;
	        months = this._months + daysToMonths(days);

	        switch (units) {
	          case 'month':
	            return months;

	          case 'quarter':
	            return months / 3;

	          case 'year':
	            return months / 12;
	        }
	      } else {
	        // handle milliseconds separately because of floating point math errors (issue #1867)
	        days = this._days + Math.round(monthsToDays(this._months));

	        switch (units) {
	          case 'week':
	            return days / 7 + milliseconds / 6048e5;

	          case 'day':
	            return days + milliseconds / 864e5;

	          case 'hour':
	            return days * 24 + milliseconds / 36e5;

	          case 'minute':
	            return days * 1440 + milliseconds / 6e4;

	          case 'second':
	            return days * 86400 + milliseconds / 1000;
	          // Math.floor prevents floating point math errors here

	          case 'millisecond':
	            return Math.floor(days * 864e5) + milliseconds;

	          default:
	            throw new Error('Unknown unit ' + units);
	        }
	      }
	    } // TODO: Use this.as('ms')?


	    function valueOf$1() {
	      if (!this.isValid()) {
	        return NaN;
	      }

	      return this._milliseconds + this._days * 864e5 + this._months % 12 * 2592e6 + toInt(this._months / 12) * 31536e6;
	    }

	    function makeAs(alias) {
	      return function () {
	        return this.as(alias);
	      };
	    }

	    var asMilliseconds = makeAs('ms'),
	        asSeconds = makeAs('s'),
	        asMinutes = makeAs('m'),
	        asHours = makeAs('h'),
	        asDays = makeAs('d'),
	        asWeeks = makeAs('w'),
	        asMonths = makeAs('M'),
	        asQuarters = makeAs('Q'),
	        asYears = makeAs('y');

	    function clone$1() {
	      return createDuration(this);
	    }

	    function get$2(units) {
	      units = normalizeUnits(units);
	      return this.isValid() ? this[units + 's']() : NaN;
	    }

	    function makeGetter(name) {
	      return function () {
	        return this.isValid() ? this._data[name] : NaN;
	      };
	    }

	    var milliseconds = makeGetter('milliseconds'),
	        seconds = makeGetter('seconds'),
	        minutes = makeGetter('minutes'),
	        hours = makeGetter('hours'),
	        days = makeGetter('days'),
	        months = makeGetter('months'),
	        years = makeGetter('years');

	    function weeks() {
	      return absFloor(this.days() / 7);
	    }

	    var round = Math.round,
	        thresholds = {
	      ss: 44,
	      // a few seconds to seconds
	      s: 45,
	      // seconds to minute
	      m: 45,
	      // minutes to hour
	      h: 22,
	      // hours to day
	      d: 26,
	      // days to month/week
	      w: null,
	      // weeks to month
	      M: 11 // months to year

	    }; // helper function for moment.fn.from, moment.fn.fromNow, and moment.duration.fn.humanize

	    function substituteTimeAgo(string, number, withoutSuffix, isFuture, locale) {
	      return locale.relativeTime(number || 1, !!withoutSuffix, string, isFuture);
	    }

	    function relativeTime$1(posNegDuration, withoutSuffix, thresholds, locale) {
	      var duration = createDuration(posNegDuration).abs(),
	          seconds = round(duration.as('s')),
	          minutes = round(duration.as('m')),
	          hours = round(duration.as('h')),
	          days = round(duration.as('d')),
	          months = round(duration.as('M')),
	          weeks = round(duration.as('w')),
	          years = round(duration.as('y')),
	          a = seconds <= thresholds.ss && ['s', seconds] || seconds < thresholds.s && ['ss', seconds] || minutes <= 1 && ['m'] || minutes < thresholds.m && ['mm', minutes] || hours <= 1 && ['h'] || hours < thresholds.h && ['hh', hours] || days <= 1 && ['d'] || days < thresholds.d && ['dd', days];

	      if (thresholds.w != null) {
	        a = a || weeks <= 1 && ['w'] || weeks < thresholds.w && ['ww', weeks];
	      }

	      a = a || months <= 1 && ['M'] || months < thresholds.M && ['MM', months] || years <= 1 && ['y'] || ['yy', years];
	      a[2] = withoutSuffix;
	      a[3] = +posNegDuration > 0;
	      a[4] = locale;
	      return substituteTimeAgo.apply(null, a);
	    } // This function allows you to set the rounding function for relative time strings


	    function getSetRelativeTimeRounding(roundingFunction) {
	      if (roundingFunction === undefined) {
	        return round;
	      }

	      if (typeof roundingFunction === 'function') {
	        round = roundingFunction;
	        return true;
	      }

	      return false;
	    } // This function allows you to set a threshold for relative time strings


	    function getSetRelativeTimeThreshold(threshold, limit) {
	      if (thresholds[threshold] === undefined) {
	        return false;
	      }

	      if (limit === undefined) {
	        return thresholds[threshold];
	      }

	      thresholds[threshold] = limit;

	      if (threshold === 's') {
	        thresholds.ss = limit - 1;
	      }

	      return true;
	    }

	    function humanize(argWithSuffix, argThresholds) {
	      if (!this.isValid()) {
	        return this.localeData().invalidDate();
	      }

	      var withSuffix = false,
	          th = thresholds,
	          locale,
	          output;

	      if (typeof argWithSuffix === 'object') {
	        argThresholds = argWithSuffix;
	        argWithSuffix = false;
	      }

	      if (typeof argWithSuffix === 'boolean') {
	        withSuffix = argWithSuffix;
	      }

	      if (typeof argThresholds === 'object') {
	        th = Object.assign({}, thresholds, argThresholds);

	        if (argThresholds.s != null && argThresholds.ss == null) {
	          th.ss = argThresholds.s - 1;
	        }
	      }

	      locale = this.localeData();
	      output = relativeTime$1(this, !withSuffix, th, locale);

	      if (withSuffix) {
	        output = locale.pastFuture(+this, output);
	      }

	      return locale.postformat(output);
	    }

	    var abs$1 = Math.abs;

	    function sign(x) {
	      return (x > 0) - (x < 0) || +x;
	    }

	    function toISOString$1() {
	      // for ISO strings we do not use the normal bubbling rules:
	      //  * milliseconds bubble up until they become hours
	      //  * days do not bubble at all
	      //  * months bubble up until they become years
	      // This is because there is no context-free conversion between hours and days
	      // (think of clock changes)
	      // and also not between days and months (28-31 days per month)
	      if (!this.isValid()) {
	        return this.localeData().invalidDate();
	      }

	      var seconds = abs$1(this._milliseconds) / 1000,
	          days = abs$1(this._days),
	          months = abs$1(this._months),
	          minutes,
	          hours,
	          years,
	          s,
	          total = this.asSeconds(),
	          totalSign,
	          ymSign,
	          daysSign,
	          hmsSign;

	      if (!total) {
	        // this is the same as C#'s (Noda) and python (isodate)...
	        // but not other JS (goog.date)
	        return 'P0D';
	      } // 3600 seconds -> 60 minutes -> 1 hour


	      minutes = absFloor(seconds / 60);
	      hours = absFloor(minutes / 60);
	      seconds %= 60;
	      minutes %= 60; // 12 months -> 1 year

	      years = absFloor(months / 12);
	      months %= 12; // inspired by https://github.com/dordille/moment-isoduration/blob/master/moment.isoduration.js

	      s = seconds ? seconds.toFixed(3).replace(/\.?0+$/, '') : '';
	      totalSign = total < 0 ? '-' : '';
	      ymSign = sign(this._months) !== sign(total) ? '-' : '';
	      daysSign = sign(this._days) !== sign(total) ? '-' : '';
	      hmsSign = sign(this._milliseconds) !== sign(total) ? '-' : '';
	      return totalSign + 'P' + (years ? ymSign + years + 'Y' : '') + (months ? ymSign + months + 'M' : '') + (days ? daysSign + days + 'D' : '') + (hours || minutes || seconds ? 'T' : '') + (hours ? hmsSign + hours + 'H' : '') + (minutes ? hmsSign + minutes + 'M' : '') + (seconds ? hmsSign + s + 'S' : '');
	    }

	    var proto$2 = Duration.prototype;
	    proto$2.isValid = isValid$1;
	    proto$2.abs = abs;
	    proto$2.add = add$1;
	    proto$2.subtract = subtract$1;
	    proto$2.as = as;
	    proto$2.asMilliseconds = asMilliseconds;
	    proto$2.asSeconds = asSeconds;
	    proto$2.asMinutes = asMinutes;
	    proto$2.asHours = asHours;
	    proto$2.asDays = asDays;
	    proto$2.asWeeks = asWeeks;
	    proto$2.asMonths = asMonths;
	    proto$2.asQuarters = asQuarters;
	    proto$2.asYears = asYears;
	    proto$2.valueOf = valueOf$1;
	    proto$2._bubble = bubble;
	    proto$2.clone = clone$1;
	    proto$2.get = get$2;
	    proto$2.milliseconds = milliseconds;
	    proto$2.seconds = seconds;
	    proto$2.minutes = minutes;
	    proto$2.hours = hours;
	    proto$2.days = days;
	    proto$2.weeks = weeks;
	    proto$2.months = months;
	    proto$2.years = years;
	    proto$2.humanize = humanize;
	    proto$2.toISOString = toISOString$1;
	    proto$2.toString = toISOString$1;
	    proto$2.toJSON = toISOString$1;
	    proto$2.locale = locale;
	    proto$2.localeData = localeData;
	    proto$2.toIsoString = deprecate('toIsoString() is deprecated. Please use toISOString() instead (notice the capitals)', toISOString$1);
	    proto$2.lang = lang; // FORMATTING

	    addFormatToken('X', 0, 0, 'unix');
	    addFormatToken('x', 0, 0, 'valueOf'); // PARSING

	    addRegexToken('x', matchSigned);
	    addRegexToken('X', matchTimestamp);
	    addParseToken('X', function (input, array, config) {
	      config._d = new Date(parseFloat(input) * 1000);
	    });
	    addParseToken('x', function (input, array, config) {
	      config._d = new Date(toInt(input));
	    }); //! moment.js

	    hooks.version = '2.25.3';
	    setHookCallback(createLocal);
	    hooks.fn = proto;
	    hooks.min = min;
	    hooks.max = max;
	    hooks.now = now;
	    hooks.utc = createUTC;
	    hooks.unix = createUnix;
	    hooks.months = listMonths;
	    hooks.isDate = isDate;
	    hooks.locale = getSetGlobalLocale;
	    hooks.invalid = createInvalid;
	    hooks.duration = createDuration;
	    hooks.isMoment = isMoment;
	    hooks.weekdays = listWeekdays;
	    hooks.parseZone = createInZone;
	    hooks.localeData = getLocale;
	    hooks.isDuration = isDuration;
	    hooks.monthsShort = listMonthsShort;
	    hooks.weekdaysMin = listWeekdaysMin;
	    hooks.defineLocale = defineLocale;
	    hooks.updateLocale = updateLocale;
	    hooks.locales = listLocales;
	    hooks.weekdaysShort = listWeekdaysShort;
	    hooks.normalizeUnits = normalizeUnits;
	    hooks.relativeTimeRounding = getSetRelativeTimeRounding;
	    hooks.relativeTimeThreshold = getSetRelativeTimeThreshold;
	    hooks.calendarFormat = getCalendarFormat;
	    hooks.prototype = proto; // currently HTML5 input type only supports 24-hour formats

	    hooks.HTML5_FMT = {
	      DATETIME_LOCAL: 'YYYY-MM-DDTHH:mm',
	      // <input type="datetime-local" />
	      DATETIME_LOCAL_SECONDS: 'YYYY-MM-DDTHH:mm:ss',
	      // <input type="datetime-local" step="1" />
	      DATETIME_LOCAL_MS: 'YYYY-MM-DDTHH:mm:ss.SSS',
	      // <input type="datetime-local" step="0.001" />
	      DATE: 'YYYY-MM-DD',
	      // <input type="date" />
	      TIME: 'HH:mm',
	      // <input type="time" />
	      TIME_SECONDS: 'HH:mm:ss',
	      // <input type="time" step="1" />
	      TIME_MS: 'HH:mm:ss.SSS',
	      // <input type="time" step="0.001" />
	      WEEK: 'GGGG-[W]WW',
	      // <input type="week" />
	      MONTH: 'YYYY-MM' // <input type="month" />

	    };
	    return hooks;
	  });
	});

	function ownKeys$2(object, enumerableOnly) { var keys = keys$6(object); if (getOwnPropertySymbols$2) { var symbols = getOwnPropertySymbols$2(object); if (enumerableOnly) symbols = filter$2(symbols).call(symbols, function (sym) { return getOwnPropertyDescriptor$3(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

	function _objectSpread$1(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { var _context28; forEach$3(_context28 = ownKeys$2(Object(source), true)).call(_context28, function (key) { defineProperty$8(target, key, source[key]); }); } else if (getOwnPropertyDescriptors$2) { defineProperties$1(target, getOwnPropertyDescriptors$2(source)); } else { var _context29; forEach$3(_context29 = ownKeys$2(Object(source))).call(_context29, function (key) { defineProperty$2(target, key, getOwnPropertyDescriptor$3(source, key)); }); } } return target; }

	function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function () { var Super = getPrototypeOf$3(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = getPrototypeOf$3(this).constructor; result = construct$3(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return possibleConstructorReturn(this, result); }; }

	function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !construct$3) return false; if (construct$3.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(construct$3(Date, [], function () {})); return true; } catch (e) { return false; } }

	function _createForOfIteratorHelper$1(o) { if (typeof symbol$4 === "undefined" || getIteratorMethod$1(o) == null) { if (isArray$3(o) || (o = _unsupportedIterableToArray$2(o))) { var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var it, normalCompletion = true, didErr = false, err; return { s: function s() { it = getIterator$1(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }

	function _unsupportedIterableToArray$2(o, minLen) { var _context19; if (!o) return; if (typeof o === "string") return _arrayLikeToArray$2(o, minLen); var n = slice$3(_context19 = Object.prototype.toString.call(o)).call(_context19, 8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return from_1$2(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$2(o, minLen); }

	function _arrayLikeToArray$2(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }
	/* eslint @typescript-eslint/member-ordering: ["error", { "classes": ["field", "constructor", "method"] }] */

	/**
	 * Create new data pipe.
	 *
	 * @param from - The source data set or data view.
	 *
	 * @remarks
	 * Example usage:
	 * ```typescript
	 * interface AppItem {
	 *   whoami: string;
	 *   appData: unknown;
	 *   visData: VisItem;
	 * }
	 * interface VisItem {
	 *   id: number;
	 *   label: string;
	 *   color: string;
	 *   x: number;
	 *   y: number;
	 * }
	 *
	 * const ds1 = new DataSet<AppItem, "whoami">([], { fieldId: "whoami" });
	 * const ds2 = new DataSet<VisItem, "id">();
	 *
	 * const pipe = createNewDataPipeFrom(ds1)
	 *   .filter((item): boolean => item.enabled === true)
	 *   .map<VisItem, "id">((item): VisItem => item.visData)
	 *   .to(ds2);
	 *
	 * pipe.start();
	 * ```
	 *
	 * @returns A factory whose methods can be used to configure the pipe.
	 */

	function createNewDataPipeFrom(from) {
	  return new DataPipeUnderConstruction(from);
	}
	/**
	 * Internal implementation of the pipe. This should be accessible only through
	 * `createNewDataPipeFrom` from the outside.
	 *
	 * @typeparam SI - Source item type.
	 * @typeparam SP - Source item type's id property name.
	 * @typeparam TI - Target item type.
	 * @typeparam TP - Target item type's id property name.
	 */


	var SimpleDataPipe = /*#__PURE__*/function () {
	  /**
	   * Create a new data pipe.
	   *
	   * @param _source - The data set or data view that will be observed.
	   * @param _transformers - An array of transforming functions to be used to
	   * filter or transform the items in the pipe.
	   * @param _target - The data set or data view that will receive the items.
	   */
	  function SimpleDataPipe(_source, _transformers, _target) {
	    var _context, _context2, _context3;

	    classCallCheck(this, SimpleDataPipe);

	    this._source = _source;
	    this._transformers = _transformers;
	    this._target = _target;
	    /**
	     * Bound listeners for use with `DataInterface['on' | 'off']`.
	     */

	    this._listeners = {
	      add: bind$2(_context = this._add).call(_context, this),
	      remove: bind$2(_context2 = this._remove).call(_context2, this),
	      update: bind$2(_context3 = this._update).call(_context3, this)
	    };
	  }
	  /** @inheritdoc */


	  createClass(SimpleDataPipe, [{
	    key: "all",
	    value: function all() {
	      this._target.update(this._transformItems(this._source.get()));

	      return this;
	    }
	    /** @inheritdoc */

	  }, {
	    key: "start",
	    value: function start() {
	      this._source.on("add", this._listeners.add);

	      this._source.on("remove", this._listeners.remove);

	      this._source.on("update", this._listeners.update);

	      return this;
	    }
	    /** @inheritdoc */

	  }, {
	    key: "stop",
	    value: function stop() {
	      this._source.off("add", this._listeners.add);

	      this._source.off("remove", this._listeners.remove);

	      this._source.off("update", this._listeners.update);

	      return this;
	    }
	    /**
	     * Apply the transformers to the items.
	     *
	     * @param items - The items to be transformed.
	     *
	     * @returns The transformed items.
	     */

	  }, {
	    key: "_transformItems",
	    value: function _transformItems(items) {
	      var _context4;

	      return reduce$2(_context4 = this._transformers).call(_context4, function (items, transform) {
	        return transform(items);
	      }, items);
	    }
	    /**
	     * Handle an add event.
	     *
	     * @param _name - Ignored.
	     * @param payload - The payload containing the ids of the added items.
	     */

	  }, {
	    key: "_add",
	    value: function _add(_name, payload) {
	      if (payload == null) {
	        return;
	      }

	      this._target.add(this._transformItems(this._source.get(payload.items)));
	    }
	    /**
	     * Handle an update event.
	     *
	     * @param _name - Ignored.
	     * @param payload - The payload containing the ids of the updated items.
	     */

	  }, {
	    key: "_update",
	    value: function _update(_name, payload) {
	      if (payload == null) {
	        return;
	      }

	      this._target.update(this._transformItems(this._source.get(payload.items)));
	    }
	    /**
	     * Handle a remove event.
	     *
	     * @param _name - Ignored.
	     * @param payload - The payload containing the data of the removed items.
	     */

	  }, {
	    key: "_remove",
	    value: function _remove(_name, payload) {
	      if (payload == null) {
	        return;
	      }

	      this._target.remove(this._transformItems(payload.oldData));
	    }
	  }]);

	  return SimpleDataPipe;
	}();
	/**
	 * Internal implementation of the pipe factory. This should be accessible
	 * only through `createNewDataPipeFrom` from the outside.
	 *
	 * @typeparam TI - Target item type.
	 * @typeparam TP - Target item type's id property name.
	 */


	var DataPipeUnderConstruction = /*#__PURE__*/function () {
	  /**
	   * Create a new data pipe factory. This is an internal constructor that
	   * should never be called from outside of this file.
	   *
	   * @param _source - The source data set or data view for this pipe.
	   */
	  function DataPipeUnderConstruction(_source) {
	    classCallCheck(this, DataPipeUnderConstruction);

	    this._source = _source;
	    /**
	     * Array transformers used to transform items within the pipe. This is typed
	     * as any for the sake of simplicity.
	     */

	    this._transformers = [];
	  }
	  /**
	   * Filter the items.
	   *
	   * @param callback - A filtering function that returns true if given item
	   * should be piped and false if not.
	   *
	   * @returns This factory for further configuration.
	   */


	  createClass(DataPipeUnderConstruction, [{
	    key: "filter",
	    value: function filter(callback) {
	      this._transformers.push(function (input) {
	        return filter$2(input).call(input, callback);
	      });

	      return this;
	    }
	    /**
	     * Map each source item to a new type.
	     *
	     * @param callback - A mapping function that takes a source item and returns
	     * corresponding mapped item.
	     *
	     * @typeparam TI - Target item type.
	     * @typeparam TP - Target item type's id property name.
	     *
	     * @returns This factory for further configuration.
	     */

	  }, {
	    key: "map",
	    value: function map(callback) {
	      this._transformers.push(function (input) {
	        return map$5(input).call(input, callback);
	      });

	      return this;
	    }
	    /**
	     * Map each source item to zero or more items of a new type.
	     *
	     * @param callback - A mapping function that takes a source item and returns
	     * an array of corresponding mapped items.
	     *
	     * @typeparam TI - Target item type.
	     * @typeparam TP - Target item type's id property name.
	     *
	     * @returns This factory for further configuration.
	     */

	  }, {
	    key: "flatMap",
	    value: function flatMap(callback) {
	      this._transformers.push(function (input) {
	        return flatMap$2(input).call(input, callback);
	      });

	      return this;
	    }
	    /**
	     * Connect this pipe to given data set.
	     *
	     * @param target - The data set that will receive the items from this pipe.
	     *
	     * @returns The pipe connected between given data sets and performing
	     * configured transformation on the processed items.
	     */

	  }, {
	    key: "to",
	    value: function to(target) {
	      return new SimpleDataPipe(this._source, this._transformers, target);
	    }
	  }]);

	  return DataPipeUnderConstruction;
	}(); // utility functions
	// parse ASP.Net Date pattern,
	// for example '/Date(1198908717056)/' or '/Date(1198908717056-0700)/'
	// code from http://momentjs.com/


	var ASPDateRegex$1 = /^\/?Date\((-?\d+)/i;
	/**
	 * Test whether given object is a Moment date.
	 *
	 * @param value - Input value of unknown type.
	 *
	 * @returns True if Moment instance, false otherwise.
	 */

	function isMoment(value) {
	  return moment.isMoment(value);
	}
	/**
	 * Convert an object into another type
	 *
	 * @param object - Value of unknown type.
	 * @param type - Name of the desired type.
	 *
	 * @returns Object in the desired type.
	 * @throws Error
	 */


	function convert(object, type) {
	  var match;

	  if (object === undefined) {
	    return undefined;
	  }

	  if (object === null) {
	    return null;
	  }

	  if (!type) {
	    return object;
	  }

	  if (!(typeof type === "string") && !(type instanceof String)) {
	    throw new Error("Type must be a string");
	  } //noinspection FallthroughInSwitchStatementJS


	  switch (type) {
	    case "boolean":
	    case "Boolean":
	      return Boolean(object);

	    case "number":
	    case "Number":
	      if (isString(object) && !isNaN(Date.parse(object))) {
	        return moment(object).valueOf();
	      } else {
	        // @TODO: I don't think that Number and String constructors are a good idea.
	        // This could also fail if the object doesn't have valueOf method or if it's redefined.
	        // For example: Object.create(null) or { valueOf: 7 }.
	        return Number(object.valueOf());
	      }

	    case "string":
	    case "String":
	      return String(object);

	    case "Date":
	      if (isNumber(object)) {
	        return new Date(object);
	      }

	      if (object instanceof Date) {
	        return new Date(object.valueOf());
	      } else if (isMoment(object)) {
	        return new Date(object.valueOf());
	      }

	      if (isString(object)) {
	        match = ASPDateRegex$1.exec(object);

	        if (match) {
	          // object is an ASP date
	          return new Date(Number(match[1])); // parse number
	        } else {
	          return moment(new Date(object)).toDate(); // parse string
	        }
	      } else {
	        throw new Error("Cannot convert object of type " + getType(object) + " to type Date");
	      }

	    case "Moment":
	      if (isNumber(object)) {
	        return moment(object);
	      }

	      if (object instanceof Date) {
	        return moment(object.valueOf());
	      } else if (isMoment(object)) {
	        return moment(object);
	      }

	      if (isString(object)) {
	        match = ASPDateRegex$1.exec(object);

	        if (match) {
	          // object is an ASP date
	          return moment(Number(match[1])); // parse number
	        } else {
	          return moment(object); // parse string
	        }
	      } else {
	        throw new Error("Cannot convert object of type " + getType(object) + " to type Date");
	      }

	    case "ISODate":
	      if (isNumber(object)) {
	        return new Date(object);
	      } else if (object instanceof Date) {
	        return object.toISOString();
	      } else if (isMoment(object)) {
	        return object.toDate().toISOString();
	      } else if (isString(object)) {
	        match = ASPDateRegex$1.exec(object);

	        if (match) {
	          // object is an ASP date
	          return new Date(Number(match[1])).toISOString(); // parse number
	        } else {
	          return moment(object).format(); // ISO 8601
	        }
	      } else {
	        throw new Error("Cannot convert object of type " + getType(object) + " to type ISODate");
	      }

	    case "ASPDate":
	      if (isNumber(object)) {
	        return "/Date(" + object + ")/";
	      } else if (object instanceof Date || isMoment(object)) {
	        return "/Date(" + object.valueOf() + ")/";
	      } else if (isString(object)) {
	        match = ASPDateRegex$1.exec(object);
	        var value;

	        if (match) {
	          // object is an ASP date
	          value = new Date(Number(match[1])).valueOf(); // parse number
	        } else {
	          value = new Date(object).valueOf(); // parse string
	        }

	        return "/Date(" + value + ")/";
	      } else {
	        throw new Error("Cannot convert object of type " + getType(object) + " to type ASPDate");
	      }

	    default:
	      var never = type;
	      throw new Error("Unknown type ".concat(never));
	  }
	}
	/**
	 * Determine whether a value can be used as an id.
	 *
	 * @param value - Input value of unknown type.
	 *
	 * @returns True if the value is valid id, false otherwise.
	 */


	function isId(value) {
	  return typeof value === "string" || typeof value === "number";
	}
	/* eslint @typescript-eslint/member-ordering: ["error", { "classes": ["field", "constructor", "method"] }] */

	/**
	 * A queue.
	 *
	 * @typeParam T - The type of method names to be replaced by queued versions.
	 */


	var Queue = /*#__PURE__*/function () {
	  /**
	   * Construct a new Queue.
	   *
	   * @param options - Queue configuration.
	   */
	  function Queue(options) {
	    classCallCheck(this, Queue);

	    this._queue = [];
	    this._timeout = null;
	    this._extended = null; // options

	    this.delay = null;
	    this.max = Infinity;
	    this.setOptions(options);
	  }
	  /**
	   * Update the configuration of the queue.
	   *
	   * @param options - Queue configuration.
	   */


	  createClass(Queue, [{
	    key: "setOptions",
	    value: function setOptions(options) {
	      if (options && typeof options.delay !== "undefined") {
	        this.delay = options.delay;
	      }

	      if (options && typeof options.max !== "undefined") {
	        this.max = options.max;
	      }

	      this._flushIfNeeded();
	    }
	    /**
	     * Extend an object with queuing functionality.
	     * The object will be extended with a function flush, and the methods provided in options.replace will be replaced with queued ones.
	     *
	     * @param object - The object to be extended.
	     * @param options - Additional options.
	     *
	     * @returns The created queue.
	     */

	  }, {
	    key: "destroy",

	    /**
	     * Destroy the queue. The queue will first flush all queued actions, and in case it has extended an object, will restore the original object.
	     */
	    value: function destroy() {
	      this.flush();

	      if (this._extended) {
	        var object = this._extended.object;
	        var methods = this._extended.methods;

	        for (var i = 0; i < methods.length; i++) {
	          var method = methods[i];

	          if (method.original) {
	            // @TODO: better solution?
	            object[method.name] = method.original;
	          } else {
	            // @TODO: better solution?
	            delete object[method.name];
	          }
	        }

	        this._extended = null;
	      }
	    }
	    /**
	     * Replace a method on an object with a queued version.
	     *
	     * @param object - Object having the method.
	     * @param method - The method name.
	     */

	  }, {
	    key: "replace",
	    value: function replace(object, method) {
	      /* eslint-disable-next-line @typescript-eslint/no-this-alias */
	      var me = this;
	      var original = object[method];

	      if (!original) {
	        throw new Error("Method " + method + " undefined");
	      }

	      object[method] = function () {
	        for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
	          args[_key] = arguments[_key];
	        }

	        // add this call to the queue
	        me.queue({
	          args: args,
	          fn: original,
	          context: this
	        });
	      };
	    }
	    /**
	     * Queue a call.
	     *
	     * @param entry - The function or entry to be queued.
	     */

	  }, {
	    key: "queue",
	    value: function queue(entry) {
	      if (typeof entry === "function") {
	        this._queue.push({
	          fn: entry
	        });
	      } else {
	        this._queue.push(entry);
	      }

	      this._flushIfNeeded();
	    }
	    /**
	     * Check whether the queue needs to be flushed.
	     */

	  }, {
	    key: "_flushIfNeeded",
	    value: function _flushIfNeeded() {
	      var _this = this;

	      // flush when the maximum is exceeded.
	      if (this._queue.length > this.max) {
	        this.flush();
	      } // flush after a period of inactivity when a delay is configured


	      if (this._timeout != null) {
	        clearTimeout(this._timeout);
	        this._timeout = null;
	      }

	      if (this.queue.length > 0 && typeof this.delay === "number") {
	        this._timeout = setTimeout$2(function () {
	          _this.flush();
	        }, this.delay);
	      }
	    }
	    /**
	     * Flush all queued calls
	     */

	  }, {
	    key: "flush",
	    value: function flush() {
	      var _context5, _context6;

	      forEach$3(_context5 = splice$2(_context6 = this._queue).call(_context6, 0)).call(_context5, function (entry) {
	        entry.fn.apply(entry.context || entry.fn, entry.args || []);
	      });
	    }
	  }], [{
	    key: "extend",
	    value: function extend(object, options) {
	      var queue = new Queue(options);

	      if (object.flush !== undefined) {
	        throw new Error("Target object already has a property flush");
	      }

	      object.flush = function () {
	        queue.flush();
	      };

	      var methods = [{
	        name: "flush",
	        original: undefined
	      }];

	      if (options && options.replace) {
	        for (var i = 0; i < options.replace.length; i++) {
	          var name = options.replace[i];
	          methods.push({
	            name: name,
	            // @TODO: better solution?
	            original: object[name]
	          }); // @TODO: better solution?

	          queue.replace(object, name);
	        }
	      }

	      queue._extended = {
	        object: object,
	        methods: methods
	      };
	      return queue;
	    }
	  }]);

	  return Queue;
	}();
	/* eslint-disable @typescript-eslint/member-ordering */

	/**
	 * [[DataSet]] code that can be reused in [[DataView]] or other similar implementations of [[DataInterface]].
	 *
	 * @typeParam Item - Item type that may or may not have an id.
	 * @typeParam IdProp - Name of the property that contains the id.
	 */


	var DataSetPart = /*#__PURE__*/function () {
	  function DataSetPart() {
	    classCallCheck(this, DataSetPart);

	    this._subscribers = {
	      "*": [],
	      add: [],
	      remove: [],
	      update: []
	    };
	    /**
	     * @deprecated Use on instead (PS: DataView.subscribe === DataView.on).
	     */

	    this.subscribe = DataSetPart.prototype.on;
	    /**
	     * @deprecated Use off instead (PS: DataView.unsubscribe === DataView.off).
	     */

	    this.unsubscribe = DataSetPart.prototype.off;
	  }
	  /**
	   * Trigger an event
	   *
	   * @param event - Event name.
	   * @param payload - Event payload.
	   * @param senderId - Id of the sender.
	   */


	  createClass(DataSetPart, [{
	    key: "_trigger",
	    value: function _trigger(event, payload, senderId) {
	      var _context7, _context8;

	      if (event === "*") {
	        throw new Error("Cannot trigger event *");
	      }

	      forEach$3(_context7 = concat$2(_context8 = []).call(_context8, toConsumableArray(this._subscribers[event]), toConsumableArray(this._subscribers["*"]))).call(_context7, function (subscriber) {
	        subscriber(event, payload, senderId != null ? senderId : null);
	      });
	    }
	    /**
	     * Subscribe to an event, add an event listener.
	     *
	     * @remarks Non-function callbacks are ignored.
	     *
	     * @param event - Event name.
	     * @param callback - Callback method.
	     */

	  }, {
	    key: "on",
	    value: function on(event, callback) {
	      if (typeof callback === "function") {
	        this._subscribers[event].push(callback);
	      } // @TODO: Maybe throw for invalid callbacks?

	    }
	    /**
	     * Unsubscribe from an event, remove an event listener.
	     *
	     * @remarks If the same callback was subscribed more than once **all** occurences will be removed.
	     *
	     * @param event - Event name.
	     * @param callback - Callback method.
	     */

	  }, {
	    key: "off",
	    value: function off(event, callback) {
	      var _context9;

	      this._subscribers[event] = filter$2(_context9 = this._subscribers[event]).call(_context9, function (subscriber) {
	        return subscriber !== callback;
	      });
	    }
	  }]);

	  return DataSetPart;
	}();
	/**
	 * Data stream
	 *
	 * @remarks
	 * [[DataStream]] offers an always up to date stream of items from a [[DataSet]] or [[DataView]].
	 * That means that the stream is evaluated at the time of iteration, conversion to another data type or when [[cache]] is called, not when the [[DataStream]] was created.
	 * Multiple invocations of for example [[toItemArray]] may yield different results (if the data source like for example [[DataSet]] gets modified).
	 *
	 * @typeparam Item - The item type this stream is going to work with.
	 */


	var DataStream = /*#__PURE__*/function () {
	  /**
	   * Create a new data stream.
	   *
	   * @param _pairs - The id, item pairs.
	   */
	  function DataStream(_pairs) {
	    classCallCheck(this, DataStream);

	    this._pairs = _pairs;
	  }
	  /**
	   * Return an iterable of key, value pairs for every entry in the stream.
	   */


	  createClass(DataStream, [{
	    key: iterator$4,
	    value: /*#__PURE__*/regenerator.mark(function value() {
	      var _iterator, _step, _step$value, id, item;

	      return regenerator.wrap(function value$(_context10) {
	        while (1) {
	          switch (_context10.prev = _context10.next) {
	            case 0:
	              _iterator = _createForOfIteratorHelper$1(this._pairs);
	              _context10.prev = 1;

	              _iterator.s();

	            case 3:
	              if ((_step = _iterator.n()).done) {
	                _context10.next = 9;
	                break;
	              }

	              _step$value = slicedToArray(_step.value, 2), id = _step$value[0], item = _step$value[1];
	              _context10.next = 7;
	              return [id, item];

	            case 7:
	              _context10.next = 3;
	              break;

	            case 9:
	              _context10.next = 14;
	              break;

	            case 11:
	              _context10.prev = 11;
	              _context10.t0 = _context10["catch"](1);

	              _iterator.e(_context10.t0);

	            case 14:
	              _context10.prev = 14;

	              _iterator.f();

	              return _context10.finish(14);

	            case 17:
	            case "end":
	              return _context10.stop();
	          }
	        }
	      }, value, this, [[1, 11, 14, 17]]);
	    })
	    /**
	     * Return an iterable of key, value pairs for every entry in the stream.
	     */

	  }, {
	    key: "entries",
	    value: /*#__PURE__*/regenerator.mark(function entries() {
	      var _iterator2, _step2, _step2$value, id, item;

	      return regenerator.wrap(function entries$(_context11) {
	        while (1) {
	          switch (_context11.prev = _context11.next) {
	            case 0:
	              _iterator2 = _createForOfIteratorHelper$1(this._pairs);
	              _context11.prev = 1;

	              _iterator2.s();

	            case 3:
	              if ((_step2 = _iterator2.n()).done) {
	                _context11.next = 9;
	                break;
	              }

	              _step2$value = slicedToArray(_step2.value, 2), id = _step2$value[0], item = _step2$value[1];
	              _context11.next = 7;
	              return [id, item];

	            case 7:
	              _context11.next = 3;
	              break;

	            case 9:
	              _context11.next = 14;
	              break;

	            case 11:
	              _context11.prev = 11;
	              _context11.t0 = _context11["catch"](1);

	              _iterator2.e(_context11.t0);

	            case 14:
	              _context11.prev = 14;

	              _iterator2.f();

	              return _context11.finish(14);

	            case 17:
	            case "end":
	              return _context11.stop();
	          }
	        }
	      }, entries, this, [[1, 11, 14, 17]]);
	    })
	    /**
	     * Return an iterable of keys in the stream.
	     */

	  }, {
	    key: "keys",
	    value: /*#__PURE__*/regenerator.mark(function keys() {
	      var _iterator3, _step3, _step3$value, id;

	      return regenerator.wrap(function keys$(_context12) {
	        while (1) {
	          switch (_context12.prev = _context12.next) {
	            case 0:
	              _iterator3 = _createForOfIteratorHelper$1(this._pairs);
	              _context12.prev = 1;

	              _iterator3.s();

	            case 3:
	              if ((_step3 = _iterator3.n()).done) {
	                _context12.next = 9;
	                break;
	              }

	              _step3$value = slicedToArray(_step3.value, 1), id = _step3$value[0];
	              _context12.next = 7;
	              return id;

	            case 7:
	              _context12.next = 3;
	              break;

	            case 9:
	              _context12.next = 14;
	              break;

	            case 11:
	              _context12.prev = 11;
	              _context12.t0 = _context12["catch"](1);

	              _iterator3.e(_context12.t0);

	            case 14:
	              _context12.prev = 14;

	              _iterator3.f();

	              return _context12.finish(14);

	            case 17:
	            case "end":
	              return _context12.stop();
	          }
	        }
	      }, keys, this, [[1, 11, 14, 17]]);
	    })
	    /**
	     * Return an iterable of values in the stream.
	     */

	  }, {
	    key: "values",
	    value: /*#__PURE__*/regenerator.mark(function values() {
	      var _iterator4, _step4, _step4$value, item;

	      return regenerator.wrap(function values$(_context13) {
	        while (1) {
	          switch (_context13.prev = _context13.next) {
	            case 0:
	              _iterator4 = _createForOfIteratorHelper$1(this._pairs);
	              _context13.prev = 1;

	              _iterator4.s();

	            case 3:
	              if ((_step4 = _iterator4.n()).done) {
	                _context13.next = 9;
	                break;
	              }

	              _step4$value = slicedToArray(_step4.value, 2), item = _step4$value[1];
	              _context13.next = 7;
	              return item;

	            case 7:
	              _context13.next = 3;
	              break;

	            case 9:
	              _context13.next = 14;
	              break;

	            case 11:
	              _context13.prev = 11;
	              _context13.t0 = _context13["catch"](1);

	              _iterator4.e(_context13.t0);

	            case 14:
	              _context13.prev = 14;

	              _iterator4.f();

	              return _context13.finish(14);

	            case 17:
	            case "end":
	              return _context13.stop();
	          }
	        }
	      }, values, this, [[1, 11, 14, 17]]);
	    })
	    /**
	     * Return an array containing all the ids in this stream.
	     *
	     * @remarks
	     * The array may contain duplicities.
	     *
	     * @returns The array with all ids from this stream.
	     */

	  }, {
	    key: "toIdArray",
	    value: function toIdArray() {
	      var _context14;

	      return map$5(_context14 = toConsumableArray(this._pairs)).call(_context14, function (pair) {
	        return pair[0];
	      });
	    }
	    /**
	     * Return an array containing all the items in this stream.
	     *
	     * @remarks
	     * The array may contain duplicities.
	     *
	     * @returns The array with all items from this stream.
	     */

	  }, {
	    key: "toItemArray",
	    value: function toItemArray() {
	      var _context15;

	      return map$5(_context15 = toConsumableArray(this._pairs)).call(_context15, function (pair) {
	        return pair[1];
	      });
	    }
	    /**
	     * Return an array containing all the entries in this stream.
	     *
	     * @remarks
	     * The array may contain duplicities.
	     *
	     * @returns The array with all entries from this stream.
	     */

	  }, {
	    key: "toEntryArray",
	    value: function toEntryArray() {
	      return toConsumableArray(this._pairs);
	    }
	    /**
	     * Return an object map containing all the items in this stream accessible by ids.
	     *
	     * @remarks
	     * In case of duplicate ids (coerced to string so `7 == '7'`) the last encoutered appears in the returned object.
	     *
	     * @returns The object map of all id → item pairs from this stream.
	     */

	  }, {
	    key: "toObjectMap",
	    value: function toObjectMap() {
	      var map = create$4(null);

	      var _iterator5 = _createForOfIteratorHelper$1(this._pairs),
	          _step5;

	      try {
	        for (_iterator5.s(); !(_step5 = _iterator5.n()).done;) {
	          var _step5$value = slicedToArray(_step5.value, 2),
	              id = _step5$value[0],
	              item = _step5$value[1];

	          map[id] = item;
	        }
	      } catch (err) {
	        _iterator5.e(err);
	      } finally {
	        _iterator5.f();
	      }

	      return map;
	    }
	    /**
	     * Return a map containing all the items in this stream accessible by ids.
	     *
	     * @returns The map of all id → item pairs from this stream.
	     */

	  }, {
	    key: "toMap",
	    value: function toMap() {
	      return new map$2(this._pairs);
	    }
	    /**
	     * Return a set containing all the (unique) ids in this stream.
	     *
	     * @returns The set of all ids from this stream.
	     */

	  }, {
	    key: "toIdSet",
	    value: function toIdSet() {
	      return new set$3(this.toIdArray());
	    }
	    /**
	     * Return a set containing all the (unique) items in this stream.
	     *
	     * @returns The set of all items from this stream.
	     */

	  }, {
	    key: "toItemSet",
	    value: function toItemSet() {
	      return new set$3(this.toItemArray());
	    }
	    /**
	     * Cache the items from this stream.
	     *
	     * @remarks
	     * This method allows for items to be fetched immediatelly and used (possibly multiple times) later.
	     * It can also be used to optimize performance as [[DataStream]] would otherwise reevaluate everything upon each iteration.
	     *
	     * ## Example
	     * ```javascript
	     * const ds = new DataSet([…])
	     *
	     * const cachedStream = ds.stream()
	     *   .filter(…)
	     *   .sort(…)
	     *   .map(…)
	     *   .cached(…) // Data are fetched, processed and cached here.
	     *
	     * ds.clear()
	     * chachedStream // Still has all the items.
	     * ```
	     *
	     * @returns A new [[DataStream]] with cached items (detached from the original [[DataSet]]).
	     */

	  }, {
	    key: "cache",
	    value: function cache() {
	      return new DataStream(toConsumableArray(this._pairs));
	    }
	    /**
	     * Get the distinct values of given property.
	     *
	     * @param callback - The function that picks and possibly converts the property.
	     *
	     * @typeparam T - The type of the distinct value.
	     *
	     * @returns A set of all distinct properties.
	     */

	  }, {
	    key: "distinct",
	    value: function distinct(callback) {
	      var set = new set$3();

	      var _iterator6 = _createForOfIteratorHelper$1(this._pairs),
	          _step6;

	      try {
	        for (_iterator6.s(); !(_step6 = _iterator6.n()).done;) {
	          var _step6$value = slicedToArray(_step6.value, 2),
	              id = _step6$value[0],
	              item = _step6$value[1];

	          set.add(callback(item, id));
	        }
	      } catch (err) {
	        _iterator6.e(err);
	      } finally {
	        _iterator6.f();
	      }

	      return set;
	    }
	    /**
	     * Filter the items of the stream.
	     *
	     * @param callback - The function that decides whether an item will be included.
	     *
	     * @returns A new data stream with the filtered items.
	     */

	  }, {
	    key: "filter",
	    value: function filter(callback) {
	      var pairs = this._pairs;
	      return new DataStream(defineProperty$8({}, iterator$4, /*#__PURE__*/regenerator.mark(function _callee() {
	        var _iterator7, _step7, _step7$value, id, item;

	        return regenerator.wrap(function _callee$(_context16) {
	          while (1) {
	            switch (_context16.prev = _context16.next) {
	              case 0:
	                _iterator7 = _createForOfIteratorHelper$1(pairs);
	                _context16.prev = 1;

	                _iterator7.s();

	              case 3:
	                if ((_step7 = _iterator7.n()).done) {
	                  _context16.next = 10;
	                  break;
	                }

	                _step7$value = slicedToArray(_step7.value, 2), id = _step7$value[0], item = _step7$value[1];

	                if (!callback(item, id)) {
	                  _context16.next = 8;
	                  break;
	                }

	                _context16.next = 8;
	                return [id, item];

	              case 8:
	                _context16.next = 3;
	                break;

	              case 10:
	                _context16.next = 15;
	                break;

	              case 12:
	                _context16.prev = 12;
	                _context16.t0 = _context16["catch"](1);

	                _iterator7.e(_context16.t0);

	              case 15:
	                _context16.prev = 15;

	                _iterator7.f();

	                return _context16.finish(15);

	              case 18:
	              case "end":
	                return _context16.stop();
	            }
	          }
	        }, _callee, null, [[1, 12, 15, 18]]);
	      })));
	    }
	    /**
	     * Execute a callback for each item of the stream.
	     *
	     * @param callback - The function that will be invoked for each item.
	     */

	  }, {
	    key: "forEach",
	    value: function forEach(callback) {
	      var _iterator8 = _createForOfIteratorHelper$1(this._pairs),
	          _step8;

	      try {
	        for (_iterator8.s(); !(_step8 = _iterator8.n()).done;) {
	          var _step8$value = slicedToArray(_step8.value, 2),
	              id = _step8$value[0],
	              item = _step8$value[1];

	          callback(item, id);
	        }
	      } catch (err) {
	        _iterator8.e(err);
	      } finally {
	        _iterator8.f();
	      }
	    }
	    /**
	     * Map the items into a different type.
	     *
	     * @param callback - The function that does the conversion.
	     *
	     * @typeparam Mapped - The type of the item after mapping.
	     *
	     * @returns A new data stream with the mapped items.
	     */

	  }, {
	    key: "map",
	    value: function map(callback) {
	      var pairs = this._pairs;
	      return new DataStream(defineProperty$8({}, iterator$4, /*#__PURE__*/regenerator.mark(function _callee2() {
	        var _iterator9, _step9, _step9$value, id, item;

	        return regenerator.wrap(function _callee2$(_context17) {
	          while (1) {
	            switch (_context17.prev = _context17.next) {
	              case 0:
	                _iterator9 = _createForOfIteratorHelper$1(pairs);
	                _context17.prev = 1;

	                _iterator9.s();

	              case 3:
	                if ((_step9 = _iterator9.n()).done) {
	                  _context17.next = 9;
	                  break;
	                }

	                _step9$value = slicedToArray(_step9.value, 2), id = _step9$value[0], item = _step9$value[1];
	                _context17.next = 7;
	                return [id, callback(item, id)];

	              case 7:
	                _context17.next = 3;
	                break;

	              case 9:
	                _context17.next = 14;
	                break;

	              case 11:
	                _context17.prev = 11;
	                _context17.t0 = _context17["catch"](1);

	                _iterator9.e(_context17.t0);

	              case 14:
	                _context17.prev = 14;

	                _iterator9.f();

	                return _context17.finish(14);

	              case 17:
	              case "end":
	                return _context17.stop();
	            }
	          }
	        }, _callee2, null, [[1, 11, 14, 17]]);
	      })));
	    }
	    /**
	     * Get the item with the maximum value of given property.
	     *
	     * @param callback - The function that picks and possibly converts the property.
	     *
	     * @returns The item with the maximum if found otherwise null.
	     */

	  }, {
	    key: "max",
	    value: function max(callback) {
	      var iter = getIterator$1(this._pairs);

	      var curr = iter.next();

	      if (curr.done) {
	        return null;
	      }

	      var maxItem = curr.value[1];
	      var maxValue = callback(curr.value[1], curr.value[0]);

	      while (!(curr = iter.next()).done) {
	        var _curr$value = slicedToArray(curr.value, 2),
	            id = _curr$value[0],
	            item = _curr$value[1];

	        var _value = callback(item, id);

	        if (_value > maxValue) {
	          maxValue = _value;
	          maxItem = item;
	        }
	      }

	      return maxItem;
	    }
	    /**
	     * Get the item with the minimum value of given property.
	     *
	     * @param callback - The function that picks and possibly converts the property.
	     *
	     * @returns The item with the minimum if found otherwise null.
	     */

	  }, {
	    key: "min",
	    value: function min(callback) {
	      var iter = getIterator$1(this._pairs);

	      var curr = iter.next();

	      if (curr.done) {
	        return null;
	      }

	      var minItem = curr.value[1];
	      var minValue = callback(curr.value[1], curr.value[0]);

	      while (!(curr = iter.next()).done) {
	        var _curr$value2 = slicedToArray(curr.value, 2),
	            id = _curr$value2[0],
	            item = _curr$value2[1];

	        var _value2 = callback(item, id);

	        if (_value2 < minValue) {
	          minValue = _value2;
	          minItem = item;
	        }
	      }

	      return minItem;
	    }
	    /**
	     * Reduce the items into a single value.
	     *
	     * @param callback - The function that does the reduction.
	     * @param accumulator - The initial value of the accumulator.
	     *
	     * @typeparam T - The type of the accumulated value.
	     *
	     * @returns The reduced value.
	     */

	  }, {
	    key: "reduce",
	    value: function reduce(callback, accumulator) {
	      var _iterator10 = _createForOfIteratorHelper$1(this._pairs),
	          _step10;

	      try {
	        for (_iterator10.s(); !(_step10 = _iterator10.n()).done;) {
	          var _step10$value = slicedToArray(_step10.value, 2),
	              id = _step10$value[0],
	              item = _step10$value[1];

	          accumulator = callback(accumulator, item, id);
	        }
	      } catch (err) {
	        _iterator10.e(err);
	      } finally {
	        _iterator10.f();
	      }

	      return accumulator;
	    }
	    /**
	     * Sort the items.
	     *
	     * @param callback - Item comparator.
	     *
	     * @returns A new stream with sorted items.
	     */

	  }, {
	    key: "sort",
	    value: function sort(callback) {
	      var _this2 = this;

	      return new DataStream(defineProperty$8({}, iterator$4, function () {
	        var _context18;

	        return getIterator$1(sort$2(_context18 = toConsumableArray(_this2._pairs)).call(_context18, function (_ref, _ref2) {
	          var _ref3 = slicedToArray(_ref, 2),
	              idA = _ref3[0],
	              itemA = _ref3[1];

	          var _ref4 = slicedToArray(_ref2, 2),
	              idB = _ref4[0],
	              itemB = _ref4[1];

	          return callback(itemA, itemB, idA, idB);
	        }));
	      }));
	    }
	  }]);

	  return DataStream;
	}();
	/* eslint @typescript-eslint/member-ordering: ["error", { "classes": ["field", "constructor", "method"] }] */


	var warnTypeCorectionDeprecation = function warnTypeCorectionDeprecation() {
	  console.warn("Type coercion has been deprecated. " + "Please, use data pipes instead. " + "See https://visjs.github.io/vis-data/data/datapipe.html#TypeCoercion for more details with working migration example.");
	};
	/**
	 * # DataSet
	 *
	 * Vis.js comes with a flexible DataSet, which can be used to hold and manipulate unstructured data and listen for changes in the data. The DataSet is key/value based. Data items can be added, updated and removed from the DataSet, and one can subscribe to changes in the DataSet. The data in the DataSet can be filtered and ordered, and fields (like dates) can be converted to a specific type. Data can be normalized when appending it to the DataSet as well.
	 *
	 * ## Example
	 *
	 * The following example shows how to use a DataSet.
	 *
	 * ```javascript
	 * // create a DataSet
	 * var options = {};
	 * var data = new vis.DataSet(options);
	 *
	 * // add items
	 * // note that the data items can contain different properties and data formats
	 * data.add([
	 *   {id: 1, text: 'item 1', date: new Date(2013, 6, 20), group: 1, first: true},
	 *   {id: 2, text: 'item 2', date: '2013-06-23', group: 2},
	 *   {id: 3, text: 'item 3', date: '2013-06-25', group: 2},
	 *   {id: 4, text: 'item 4'}
	 * ]);
	 *
	 * // subscribe to any change in the DataSet
	 * data.on('*', function (event, properties, senderId) {
	 *   console.log('event', event, properties);
	 * });
	 *
	 * // update an existing item
	 * data.update({id: 2, group: 1});
	 *
	 * // remove an item
	 * data.remove(4);
	 *
	 * // get all ids
	 * var ids = data.getIds();
	 * console.log('ids', ids);
	 *
	 * // get a specific item
	 * var item1 = data.get(1);
	 * console.log('item1', item1);
	 *
	 * // retrieve a filtered subset of the data
	 * var items = data.get({
	 *   filter: function (item) {
	 *     return item.group == 1;
	 *   }
	 * });
	 * console.log('filtered items', items);
	 * ```
	 *
	 * @typeParam Item - Item type that may or may not have an id.
	 * @typeParam IdProp - Name of the property that contains the id.
	 */


	var DataSet = /*#__PURE__*/function (_DataSetPart) {
	  inherits(DataSet, _DataSetPart);

	  var _super = _createSuper(DataSet);

	  /**
	   * Construct a new DataSet.
	   *
	   * @param data - Initial data or options.
	   * @param options - Options (type error if data is also options).
	   */
	  function DataSet(data, options) {
	    var _this3;

	    classCallCheck(this, DataSet);

	    _this3 = _super.call(this); // correctly read optional arguments

	    if (data && !isArray$3(data)) {
	      options = data;
	      data = [];
	    }

	    _this3._options = options || {};
	    _this3._data = new map$2(); // map with data indexed by id

	    _this3.length = 0; // number of items in the DataSet

	    _this3._idProp = _this3._options.fieldId || "id"; // name of the field containing id

	    _this3._type = {}; // internal field types (NOTE: this can differ from this._options.type)
	    // all variants of a Date are internally stored as Date, so we can convert
	    // from everything to everything (also from ISODate to Number for example)

	    if (_this3._options.type) {
	      warnTypeCorectionDeprecation();

	      var fields = keys$6(_this3._options.type);

	      for (var i = 0, len = fields.length; i < len; i++) {
	        var field = fields[i];
	        var _value3 = _this3._options.type[field];

	        if (_value3 == "Date" || _value3 == "ISODate" || _value3 == "ASPDate") {
	          _this3._type[field] = "Date";
	        } else {
	          _this3._type[field] = _value3;
	        }
	      }
	    } // add initial data when provided


	    if (data && data.length) {
	      _this3.add(data);
	    }

	    _this3.setOptions(options);

	    return _this3;
	  }
	  /**
	   * Set new options.
	   *
	   * @param options - The new options.
	   */


	  createClass(DataSet, [{
	    key: "setOptions",
	    value: function setOptions(options) {
	      if (options && options.queue !== undefined) {
	        if (options.queue === false) {
	          // delete queue if loaded
	          if (this._queue) {
	            this._queue.destroy();

	            delete this._queue;
	          }
	        } else {
	          // create queue and update its options
	          if (!this._queue) {
	            this._queue = Queue.extend(this, {
	              replace: ["add", "update", "remove"]
	            });
	          }

	          if (options.queue && _typeof_1(options.queue) === "object") {
	            this._queue.setOptions(options.queue);
	          }
	        }
	      }
	    }
	    /**
	     * Add a data item or an array with items.
	     *
	     * After the items are added to the DataSet, the DataSet will trigger an event `add`. When a `senderId` is provided, this id will be passed with the triggered event to all subscribers.
	     *
	     * ## Example
	     *
	     * ```javascript
	     * // create a DataSet
	     * const data = new vis.DataSet()
	     *
	     * // add items
	     * const ids = data.add([
	     *   { id: 1, text: 'item 1' },
	     *   { id: 2, text: 'item 2' },
	     *   { text: 'item without an id' }
	     * ])
	     *
	     * console.log(ids) // [1, 2, '<UUIDv4>']
	     * ```
	     *
	     * @param data - Items to be added (ids will be generated if missing).
	     * @param senderId - Sender id.
	     *
	     * @returns addedIds - Array with the ids (generated if not present) of the added items.
	     *
	     * @throws When an item with the same id as any of the added items already exists.
	     */

	  }, {
	    key: "add",
	    value: function add(data, senderId) {
	      var _this4 = this;

	      var addedIds = [];
	      var id;

	      if (isArray$3(data)) {
	        // Array
	        var idsToAdd = map$5(data).call(data, function (d) {
	          return d[_this4._idProp];
	        });

	        if (some$2(idsToAdd).call(idsToAdd, function (id) {
	          return _this4._data.has(id);
	        })) {
	          throw new Error("A duplicate id was found in the parameter array.");
	        }

	        for (var i = 0, len = data.length; i < len; i++) {
	          id = this._addItem(data[i]);
	          addedIds.push(id);
	        }
	      } else if (data && _typeof_1(data) === "object") {
	        // Single item
	        id = this._addItem(data);
	        addedIds.push(id);
	      } else {
	        throw new Error("Unknown dataType");
	      }

	      if (addedIds.length) {
	        this._trigger("add", {
	          items: addedIds
	        }, senderId);
	      }

	      return addedIds;
	    }
	    /**
	     * Update existing items. When an item does not exist, it will be created.
	     *
	     * @remarks
	     * The provided properties will be merged in the existing item. When an item does not exist, it will be created.
	     *
	     * After the items are updated, the DataSet will trigger an event `add` for the added items, and an event `update`. When a `senderId` is provided, this id will be passed with the triggered event to all subscribers.
	     *
	     * ## Example
	     *
	     * ```javascript
	     * // create a DataSet
	     * const data = new vis.DataSet([
	     *   { id: 1, text: 'item 1' },
	     *   { id: 2, text: 'item 2' },
	     *   { id: 3, text: 'item 3' }
	     * ])
	     *
	     * // update items
	     * const ids = data.update([
	     *   { id: 2, text: 'item 2 (updated)' },
	     *   { id: 4, text: 'item 4 (new)' }
	     * ])
	     *
	     * console.log(ids) // [2, 4]
	     * ```
	     *
	     * ## Warning for TypeScript users
	     * This method may introduce partial items into the data set. Use add or updateOnly instead for better type safety.
	     *
	     * @param data - Items to be updated (if the id is already present) or added (if the id is missing).
	     * @param senderId - Sender id.
	     *
	     * @returns updatedIds - The ids of the added (these may be newly generated if there was no id in the item from the data) or updated items.
	     *
	     * @throws When the supplied data is neither an item nor an array of items.
	     */

	  }, {
	    key: "update",
	    value: function update(data, senderId) {
	      var _this5 = this;

	      var addedIds = [];
	      var updatedIds = [];
	      var oldData = [];
	      var updatedData = [];
	      var idProp = this._idProp;

	      var addOrUpdate = function addOrUpdate(item) {
	        var origId = item[idProp];

	        if (origId != null && _this5._data.has(origId)) {
	          var fullItem = item; // it has an id, therefore it is a fullitem

	          var oldItem = assign$2({}, _this5._data.get(origId)); // update item


	          var id = _this5._updateItem(fullItem);

	          updatedIds.push(id);
	          updatedData.push(fullItem);
	          oldData.push(oldItem);
	        } else {
	          // add new item
	          var _id = _this5._addItem(item);

	          addedIds.push(_id);
	        }
	      };

	      if (isArray$3(data)) {
	        // Array
	        for (var i = 0, len = data.length; i < len; i++) {
	          if (data[i] && _typeof_1(data[i]) === "object") {
	            addOrUpdate(data[i]);
	          } else {
	            console.warn("Ignoring input item, which is not an object at index " + i);
	          }
	        }
	      } else if (data && _typeof_1(data) === "object") {
	        // Single item
	        addOrUpdate(data);
	      } else {
	        throw new Error("Unknown dataType");
	      }

	      if (addedIds.length) {
	        this._trigger("add", {
	          items: addedIds
	        }, senderId);
	      }

	      if (updatedIds.length) {
	        var props = {
	          items: updatedIds,
	          oldData: oldData,
	          data: updatedData
	        }; // TODO: remove deprecated property 'data' some day
	        //Object.defineProperty(props, 'data', {
	        //  'get': (function() {
	        //    console.warn('Property data is deprecated. Use DataSet.get(ids) to retrieve the new data, use the oldData property on this object to get the old data');
	        //    return updatedData;
	        //  }).bind(this)
	        //});

	        this._trigger("update", props, senderId);
	      }

	      return concat$2(addedIds).call(addedIds, updatedIds);
	    }
	    /**
	     * Update existing items. When an item does not exist, an error will be thrown.
	     *
	     * @remarks
	     * The provided properties will be deeply merged into the existing item.
	     * When an item does not exist (id not present in the data set or absent), an error will be thrown and nothing will be changed.
	     *
	     * After the items are updated, the DataSet will trigger an event `update`.
	     * When a `senderId` is provided, this id will be passed with the triggered event to all subscribers.
	     *
	     * ## Example
	     *
	     * ```javascript
	     * // create a DataSet
	     * const data = new vis.DataSet([
	     *   { id: 1, text: 'item 1' },
	     *   { id: 2, text: 'item 2' },
	     *   { id: 3, text: 'item 3' },
	     * ])
	     *
	     * // update items
	     * const ids = data.update([
	     *   { id: 2, text: 'item 2 (updated)' }, // works
	     *   // { id: 4, text: 'item 4 (new)' }, // would throw
	     *   // { text: 'item 4 (new)' }, // would also throw
	     * ])
	     *
	     * console.log(ids) // [2]
	     * ```
	     *
	     * @param data - Updates (the id and optionally other props) to the items in this data set.
	     * @param senderId - Sender id.
	     *
	     * @returns updatedIds - The ids of the updated items.
	     *
	     * @throws When the supplied data is neither an item nor an array of items, when the ids are missing.
	     */

	  }, {
	    key: "updateOnly",
	    value: function updateOnly(data, senderId) {
	      var _context20,
	          _this6 = this;

	      if (!isArray$3(data)) {
	        data = [data];
	      }

	      var updateEventData = map$5(_context20 = map$5(data).call(data, function (update) {
	        var oldData = _this6._data.get(update[_this6._idProp]);

	        if (oldData == null) {
	          throw new Error("Updating non-existent items is not allowed.");
	        }

	        return {
	          oldData: oldData,
	          update: update
	        };
	      })).call(_context20, function (_ref5) {
	        var oldData = _ref5.oldData,
	            update = _ref5.update;
	        var id = oldData[_this6._idProp];
	        var updatedData = deepExtend(deepExtend({}, oldData), update);

	        _this6._data.set(id, updatedData);

	        return {
	          id: id,
	          oldData: oldData,
	          updatedData: updatedData
	        };
	      });

	      if (updateEventData.length) {
	        var props = {
	          items: map$5(updateEventData).call(updateEventData, function (value) {
	            return value.id;
	          }),
	          oldData: map$5(updateEventData).call(updateEventData, function (value) {
	            return value.oldData;
	          }),
	          data: map$5(updateEventData).call(updateEventData, function (value) {
	            return value.updatedData;
	          })
	        }; // TODO: remove deprecated property 'data' some day
	        //Object.defineProperty(props, 'data', {
	        //  'get': (function() {
	        //    console.warn('Property data is deprecated. Use DataSet.get(ids) to retrieve the new data, use the oldData property on this object to get the old data');
	        //    return updatedData;
	        //  }).bind(this)
	        //});

	        this._trigger("update", props, senderId);

	        return props.items;
	      } else {
	        return [];
	      }
	    }
	    /** @inheritdoc */

	  }, {
	    key: "get",
	    value: function get(first, second) {
	      // @TODO: Woudn't it be better to split this into multiple methods?
	      // parse the arguments
	      var id = undefined;
	      var ids = undefined;
	      var options = undefined;

	      if (isId(first)) {
	        // get(id [, options])
	        id = first;
	        options = second;
	      } else if (isArray$3(first)) {
	        // get(ids [, options])
	        ids = first;
	        options = second;
	      } else {
	        // get([, options])
	        options = first;
	      } // determine the return type


	      var returnType = options && options.returnType === "Object" ? "Object" : "Array"; // @TODO: WTF is this? Or am I missing something?
	      // var returnType
	      // if (options && options.returnType) {
	      //   var allowedValues = ['Array', 'Object']
	      //   returnType =
	      //     allowedValues.indexOf(options.returnType) == -1
	      //       ? 'Array'
	      //       : options.returnType
	      // } else {
	      //   returnType = 'Array'
	      // }
	      // build options

	      var type = options && options.type || this._options.type;

	      var filter = options && filter$2(options);

	      var items = [];
	      var item = null;
	      var itemIds = null;
	      var itemId = null; // convert items

	      if (id != null) {
	        // return a single item
	        item = this._getItem(id, type);

	        if (item && filter && !filter(item)) {
	          item = null;
	        }
	      } else if (ids != null) {
	        // return a subset of items
	        for (var i = 0, len = ids.length; i < len; i++) {
	          item = this._getItem(ids[i], type);

	          if (item != null && (!filter || filter(item))) {
	            items.push(item);
	          }
	        }
	      } else {
	        var _context21;

	        // return all items
	        itemIds = toConsumableArray(keys$3(_context21 = this._data).call(_context21));

	        for (var _i = 0, _len2 = itemIds.length; _i < _len2; _i++) {
	          itemId = itemIds[_i];
	          item = this._getItem(itemId, type);

	          if (item != null && (!filter || filter(item))) {
	            items.push(item);
	          }
	        }
	      } // order the results


	      if (options && options.order && id == undefined) {
	        this._sort(items, options.order);
	      } // filter fields of the items


	      if (options && options.fields) {
	        var fields = options.fields;

	        if (id != undefined && item != null) {
	          item = this._filterFields(item, fields);
	        } else {
	          for (var _i2 = 0, _len3 = items.length; _i2 < _len3; _i2++) {
	            items[_i2] = this._filterFields(items[_i2], fields);
	          }
	        }
	      } // return the results


	      if (returnType == "Object") {
	        var result = {};

	        for (var _i3 = 0, _len4 = items.length; _i3 < _len4; _i3++) {
	          var resultant = items[_i3]; // @TODO: Shoudn't this be this._fieldId?
	          // result[resultant.id] = resultant

	          var _id2 = resultant[this._idProp];
	          result[_id2] = resultant;
	        }

	        return result;
	      } else {
	        if (id != null) {
	          // a single item
	          return item;
	        } else {
	          // just return our array
	          return items;
	        }
	      }
	    }
	    /** @inheritdoc */

	  }, {
	    key: "getIds",
	    value: function getIds(options) {
	      var data = this._data;

	      var filter = options && filter$2(options);

	      var order = options && options.order;
	      var type = options && options.type || this._options.type;

	      var itemIds = toConsumableArray(keys$3(data).call(data));

	      var ids = [];
	      var item;
	      var items;

	      if (filter) {
	        // get filtered items
	        if (order) {
	          // create ordered list
	          items = [];

	          for (var i = 0, len = itemIds.length; i < len; i++) {
	            var id = itemIds[i];
	            item = this._getItem(id, type);

	            if (filter(item)) {
	              items.push(item);
	            }
	          }

	          this._sort(items, order);

	          for (var _i4 = 0, _len5 = items.length; _i4 < _len5; _i4++) {
	            ids.push(items[_i4][this._idProp]);
	          }
	        } else {
	          // create unordered list
	          for (var _i5 = 0, _len6 = itemIds.length; _i5 < _len6; _i5++) {
	            var _id3 = itemIds[_i5];
	            item = this._getItem(_id3, type);

	            if (filter(item)) {
	              ids.push(item[this._idProp]);
	            }
	          }
	        }
	      } else {
	        // get all items
	        if (order) {
	          // create an ordered list
	          items = [];

	          for (var _i6 = 0, _len7 = itemIds.length; _i6 < _len7; _i6++) {
	            var _id4 = itemIds[_i6];
	            items.push(data.get(_id4));
	          }

	          this._sort(items, order);

	          for (var _i7 = 0, _len8 = items.length; _i7 < _len8; _i7++) {
	            ids.push(items[_i7][this._idProp]);
	          }
	        } else {
	          // create unordered list
	          for (var _i8 = 0, _len9 = itemIds.length; _i8 < _len9; _i8++) {
	            var _id5 = itemIds[_i8];
	            item = data.get(_id5);
	            ids.push(item[this._idProp]);
	          }
	        }
	      }

	      return ids;
	    }
	    /** @inheritdoc */

	  }, {
	    key: "getDataSet",
	    value: function getDataSet() {
	      return this;
	    }
	    /** @inheritdoc */

	  }, {
	    key: "forEach",
	    value: function forEach(callback, options) {
	      var filter = options && filter$2(options);

	      var type = options && options.type || this._options.type;
	      var data = this._data;

	      var itemIds = toConsumableArray(keys$3(data).call(data));

	      if (options && options.order) {
	        // execute forEach on ordered list
	        var items = this.get(options);

	        for (var i = 0, len = items.length; i < len; i++) {
	          var item = items[i];
	          var id = item[this._idProp];
	          callback(item, id);
	        }
	      } else {
	        // unordered
	        for (var _i9 = 0, _len10 = itemIds.length; _i9 < _len10; _i9++) {
	          var _id6 = itemIds[_i9];

	          var _item = this._getItem(_id6, type);

	          if (!filter || filter(_item)) {
	            callback(_item, _id6);
	          }
	        }
	      }
	    }
	    /** @inheritdoc */

	  }, {
	    key: "map",
	    value: function map(callback, options) {
	      var filter = options && filter$2(options);

	      var type = options && options.type || this._options.type;
	      var mappedItems = [];
	      var data = this._data;

	      var itemIds = toConsumableArray(keys$3(data).call(data)); // convert and filter items


	      for (var i = 0, len = itemIds.length; i < len; i++) {
	        var id = itemIds[i];

	        var item = this._getItem(id, type);

	        if (!filter || filter(item)) {
	          mappedItems.push(callback(item, id));
	        }
	      } // order items


	      if (options && options.order) {
	        this._sort(mappedItems, options.order);
	      }

	      return mappedItems;
	    }
	    /**
	     * Filter the fields of an item.
	     *
	     * @param item - The item whose fields should be filtered.
	     * @param fields - The names of the fields that will be kept.
	     *
	     * @typeParam K - Field name type.
	     *
	     * @returns The item without any additional fields.
	     */

	  }, {
	    key: "_filterFields",
	    value: function _filterFields(item, fields) {
	      var _context22;

	      if (!item) {
	        // item is null
	        return item;
	      }

	      return reduce$2(_context22 = isArray$3(fields) ? // Use the supplied array
	      fields : // Use the keys of the supplied object
	      keys$6(fields)).call(_context22, function (filteredItem, field) {
	        filteredItem[field] = item[field];
	        return filteredItem;
	      }, {});
	    }
	    /**
	     * Sort the provided array with items.
	     *
	     * @param items - Items to be sorted in place.
	     * @param order - A field name or custom sort function.
	     *
	     * @typeParam T - The type of the items in the items array.
	     */

	  }, {
	    key: "_sort",
	    value: function _sort(items, order) {
	      if (typeof order === "string") {
	        // order by provided field name
	        var name = order; // field name

	        sort$2(items).call(items, function (a, b) {
	          // @TODO: How to treat missing properties?
	          var av = a[name];
	          var bv = b[name];
	          return av > bv ? 1 : av < bv ? -1 : 0;
	        });
	      } else if (typeof order === "function") {
	        // order by sort function
	        sort$2(items).call(items, order);
	      } else {
	        // TODO: extend order by an Object {field:string, direction:string}
	        //       where direction can be 'asc' or 'desc'
	        throw new TypeError("Order must be a function or a string");
	      }
	    }
	    /**
	     * Remove an item or multiple items by “reference” (only the id is used) or by id.
	     *
	     * The method ignores removal of non-existing items, and returns an array containing the ids of the items which are actually removed from the DataSet.
	     *
	     * After the items are removed, the DataSet will trigger an event `remove` for the removed items. When a `senderId` is provided, this id will be passed with the triggered event to all subscribers.
	     *
	     * ## Example
	     * ```javascript
	     * // create a DataSet
	     * const data = new vis.DataSet([
	     *   { id: 1, text: 'item 1' },
	     *   { id: 2, text: 'item 2' },
	     *   { id: 3, text: 'item 3' }
	     * ])
	     *
	     * // remove items
	     * const ids = data.remove([2, { id: 3 }, 4])
	     *
	     * console.log(ids) // [2, 3]
	     * ```
	     *
	     * @param id - One or more items or ids of items to be removed.
	     * @param senderId - Sender id.
	     *
	     * @returns The ids of the removed items.
	     */

	  }, {
	    key: "remove",
	    value: function remove(id, senderId) {
	      var removedIds = [];
	      var removedItems = []; // force everything to be an array for simplicity

	      var ids = isArray$3(id) ? id : [id];

	      for (var i = 0, len = ids.length; i < len; i++) {
	        var item = this._remove(ids[i]);

	        if (item) {
	          var itemId = item[this._idProp];

	          if (itemId != null) {
	            removedIds.push(itemId);
	            removedItems.push(item);
	          }
	        }
	      }

	      if (removedIds.length) {
	        this._trigger("remove", {
	          items: removedIds,
	          oldData: removedItems
	        }, senderId);
	      }

	      return removedIds;
	    }
	    /**
	     * Remove an item by its id or reference.
	     *
	     * @param id - Id of an item or the item itself.
	     *
	     * @returns The removed item if removed, null otherwise.
	     */

	  }, {
	    key: "_remove",
	    value: function _remove(id) {
	      // @TODO: It origianlly returned the item although the docs say id.
	      // The code expects the item, so probably an error in the docs.
	      var ident; // confirm the id to use based on the args type

	      if (isId(id)) {
	        ident = id;
	      } else if (id && _typeof_1(id) === "object") {
	        ident = id[this._idProp]; // look for the identifier field using ._idProp
	      } // do the removing if the item is found


	      if (ident != null && this._data.has(ident)) {
	        var item = this._data.get(ident) || null;

	        this._data.delete(ident);

	        --this.length;
	        return item;
	      }

	      return null;
	    }
	    /**
	     * Clear the entire data set.
	     *
	     * After the items are removed, the [[DataSet]] will trigger an event `remove` for all removed items. When a `senderId` is provided, this id will be passed with the triggered event to all subscribers.
	     *
	     * @param senderId - Sender id.
	     *
	     * @returns removedIds - The ids of all removed items.
	     */

	  }, {
	    key: "clear",
	    value: function clear(senderId) {
	      var _context23;

	      var ids = toConsumableArray(keys$3(_context23 = this._data).call(_context23));

	      var items = [];

	      for (var i = 0, len = ids.length; i < len; i++) {
	        items.push(this._data.get(ids[i]));
	      }

	      this._data.clear();

	      this.length = 0;

	      this._trigger("remove", {
	        items: ids,
	        oldData: items
	      }, senderId);

	      return ids;
	    }
	    /**
	     * Find the item with maximum value of a specified field.
	     *
	     * @param field - Name of the property that should be searched for max value.
	     *
	     * @returns Item containing max value, or null if no items.
	     */

	  }, {
	    key: "max",
	    value: function max(field) {
	      var _context24;

	      var max = null;
	      var maxField = null;

	      var _iterator11 = _createForOfIteratorHelper$1(values$2(_context24 = this._data).call(_context24)),
	          _step11;

	      try {
	        for (_iterator11.s(); !(_step11 = _iterator11.n()).done;) {
	          var item = _step11.value;
	          var itemField = item[field];

	          if (typeof itemField === "number" && (maxField == null || itemField > maxField)) {
	            max = item;
	            maxField = itemField;
	          }
	        }
	      } catch (err) {
	        _iterator11.e(err);
	      } finally {
	        _iterator11.f();
	      }

	      return max || null;
	    }
	    /**
	     * Find the item with minimum value of a specified field.
	     *
	     * @param field - Name of the property that should be searched for min value.
	     *
	     * @returns Item containing min value, or null if no items.
	     */

	  }, {
	    key: "min",
	    value: function min(field) {
	      var _context25;

	      var min = null;
	      var minField = null;

	      var _iterator12 = _createForOfIteratorHelper$1(values$2(_context25 = this._data).call(_context25)),
	          _step12;

	      try {
	        for (_iterator12.s(); !(_step12 = _iterator12.n()).done;) {
	          var item = _step12.value;
	          var itemField = item[field];

	          if (typeof itemField === "number" && (minField == null || itemField < minField)) {
	            min = item;
	            minField = itemField;
	          }
	        }
	      } catch (err) {
	        _iterator12.e(err);
	      } finally {
	        _iterator12.f();
	      }

	      return min || null;
	    }
	    /**
	     * Find all distinct values of a specified field
	     *
	     * @param prop - The property name whose distinct values should be returned.
	     *
	     * @returns Unordered array containing all distinct values. Items without specified property are ignored.
	     */

	  }, {
	    key: "distinct",
	    value: function distinct(prop) {
	      var data = this._data;

	      var itemIds = toConsumableArray(keys$3(data).call(data));

	      var values = [];
	      var fieldType = this._options.type && this._options.type[prop] || null;
	      var count = 0;

	      for (var i = 0, len = itemIds.length; i < len; i++) {
	        var id = itemIds[i];
	        var item = data.get(id);
	        var _value4 = item[prop];
	        var exists = false;

	        for (var j = 0; j < count; j++) {
	          if (values[j] == _value4) {
	            exists = true;
	            break;
	          }
	        }

	        if (!exists && _value4 !== undefined) {
	          values[count] = _value4;
	          count++;
	        }
	      }

	      if (fieldType) {
	        for (var _i10 = 0, _len11 = values.length; _i10 < _len11; _i10++) {
	          values[_i10] = convert(values[_i10], fieldType);
	        }
	      }

	      return values;
	    }
	    /**
	     * Add a single item. Will fail when an item with the same id already exists.
	     *
	     * @param item - A new item to be added.
	     *
	     * @returns Added item's id. An id is generated when it is not present in the item.
	     */

	  }, {
	    key: "_addItem",
	    value: function _addItem(item) {
	      var id = item[this._idProp];

	      if (id != null) {
	        // check whether this id is already taken
	        if (this._data.has(id)) {
	          // item already exists
	          throw new Error("Cannot add item: item with id " + id + " already exists");
	        }
	      } else {
	        // generate an id
	        id = v4();
	        item[this._idProp] = id;
	      }

	      var d = {};

	      var fields = keys$6(item);

	      for (var i = 0, len = fields.length; i < len; i++) {
	        var field = fields[i];
	        var fieldType = this._type[field]; // type may be undefined

	        d[field] = convert(item[field], fieldType);
	      }

	      this._data.set(id, d);

	      ++this.length;
	      return id;
	    }
	    /**
	     * Get an item. Fields can be converted to a specific type
	     *
	     * @param id - Id of the requested item.
	     * @param types - Property name to type name object map of type converstions.
	     *
	     * @returns The item, optionally after type conversion.
	     */

	  }, {
	    key: "_getItem",
	    value: function _getItem(id, types) {
	      // @TODO: I have no idea how to type this.
	      // get the item from the dataset
	      var raw = this._data.get(id);

	      if (!raw) {
	        return null;
	      } // convert the items field types


	      var converted;

	      var fields = keys$6(raw);

	      if (types) {
	        warnTypeCorectionDeprecation();
	        converted = {};

	        for (var i = 0, len = fields.length; i < len; i++) {
	          var field = fields[i];
	          var _value5 = raw[field];
	          converted[field] = convert(_value5, types[field]);
	        }
	      } else {
	        // no field types specified, no converting needed
	        converted = _objectSpread$1({}, raw);
	      }

	      if (converted[this._idProp] == null) {
	        converted[this._idProp] = raw.id;
	      }

	      return converted;
	    }
	    /**
	     * Update a single item: merge with existing item.
	     * Will fail when the item has no id, or when there does not exist an item with the same id.
	     *
	     * @param item - The new item
	     *
	     * @returns The id of the updated item.
	     */

	  }, {
	    key: "_updateItem",
	    value: function _updateItem(item) {
	      var id = item[this._idProp];

	      if (id == null) {
	        throw new Error("Cannot update item: item has no id (item: " + stringify$2(item) + ")");
	      }

	      var d = this._data.get(id);

	      if (!d) {
	        // item doesn't exist
	        throw new Error("Cannot update item: no item with id " + id + " found");
	      } // merge with current item


	      var fields = keys$6(item);

	      for (var i = 0, len = fields.length; i < len; i++) {
	        var field = fields[i];
	        var fieldType = this._type[field]; // type may be undefined

	        d[field] = convert(item[field], fieldType);
	      }

	      return id;
	    }
	    /** @inheritdoc */

	  }, {
	    key: "stream",
	    value: function stream(ids) {
	      if (ids) {
	        var data = this._data;
	        return new DataStream(defineProperty$8({}, iterator$4, /*#__PURE__*/regenerator.mark(function _callee3() {
	          var _iterator13, _step13, id, item;

	          return regenerator.wrap(function _callee3$(_context26) {
	            while (1) {
	              switch (_context26.prev = _context26.next) {
	                case 0:
	                  _iterator13 = _createForOfIteratorHelper$1(ids);
	                  _context26.prev = 1;

	                  _iterator13.s();

	                case 3:
	                  if ((_step13 = _iterator13.n()).done) {
	                    _context26.next = 11;
	                    break;
	                  }

	                  id = _step13.value;
	                  item = data.get(id);

	                  if (!(item != null)) {
	                    _context26.next = 9;
	                    break;
	                  }

	                  _context26.next = 9;
	                  return [id, item];

	                case 9:
	                  _context26.next = 3;
	                  break;

	                case 11:
	                  _context26.next = 16;
	                  break;

	                case 13:
	                  _context26.prev = 13;
	                  _context26.t0 = _context26["catch"](1);

	                  _iterator13.e(_context26.t0);

	                case 16:
	                  _context26.prev = 16;

	                  _iterator13.f();

	                  return _context26.finish(16);

	                case 19:
	                case "end":
	                  return _context26.stop();
	              }
	            }
	          }, _callee3, null, [[1, 13, 16, 19]]);
	        })));
	      } else {
	        var _context27;

	        return new DataStream(defineProperty$8({}, iterator$4, bind$2(_context27 = entries$2(this._data)).call(_context27, this._data)));
	      }
	    }
	  }]);

	  return DataSet;
	}(DataSetPart);
	/**
	 * DataView
	 *
	 * A DataView offers a filtered and/or formatted view on a DataSet. One can subscribe to changes in a DataView, and easily get filtered or formatted data without having to specify filters and field types all the time.
	 *
	 * ## Example
	 * ```javascript
	 * // create a DataSet
	 * var data = new vis.DataSet();
	 * data.add([
	 *   {id: 1, text: 'item 1', date: new Date(2013, 6, 20), group: 1, first: true},
	 *   {id: 2, text: 'item 2', date: '2013-06-23', group: 2},
	 *   {id: 3, text: 'item 3', date: '2013-06-25', group: 2},
	 *   {id: 4, text: 'item 4'}
	 * ]);
	 *
	 * // create a DataView
	 * // the view will only contain items having a property group with value 1,
	 * // and will only output fields id, text, and date.
	 * var view = new vis.DataView(data, {
	 *   filter: function (item) {
	 *     return (item.group == 1);
	 *   },
	 *   fields: ['id', 'text', 'date']
	 * });
	 *
	 * // subscribe to any change in the DataView
	 * view.on('*', function (event, properties, senderId) {
	 *   console.log('event', event, properties);
	 * });
	 *
	 * // update an item in the data set
	 * data.update({id: 2, group: 1});
	 *
	 * // get all ids in the view
	 * var ids = view.getIds();
	 * console.log('ids', ids); // will output [1, 2]
	 *
	 * // get all items in the view
	 * var items = view.get();
	 * ```
	 *
	 * @typeParam Item - Item type that may or may not have an id.
	 * @typeParam IdProp - Name of the property that contains the id.
	 */


	var DataView = /*#__PURE__*/function (_DataSetPart2) {
	  inherits(DataView, _DataSetPart2);

	  var _super2 = _createSuper(DataView);

	  /**
	   * Create a DataView.
	   *
	   * @param data - The instance containing data (directly or indirectly).
	   * @param options - Options to configure this data view.
	   */
	  function DataView(data, options) {
	    var _context30;

	    var _this7;

	    classCallCheck(this, DataView);

	    _this7 = _super2.call(this);
	    /** @inheritdoc */

	    _this7.length = 0;
	    _this7._ids = new set$3(); // ids of the items currently in memory (just contains a boolean true)

	    _this7._options = options || {};
	    _this7._listener = bind$2(_context30 = _this7._onEvent).call(_context30, assertThisInitialized(_this7));

	    _this7.setData(data);

	    return _this7;
	  } // TODO: implement a function .config() to dynamically update things like configured filter
	  // and trigger changes accordingly

	  /**
	   * Set a data source for the view.
	   *
	   * @param data - The instance containing data (directly or indirectly).
	   *
	   * @remarks
	   * Note that when the data view is bound to a data set it won't be garbage
	   * collected unless the data set is too. Use `dataView.setData(null)` or
	   * `dataView.dispose()` to enable garbage collection before you lose the last
	   * reference.
	   */


	  createClass(DataView, [{
	    key: "setData",
	    value: function setData(data) {
	      if (this._data) {
	        // unsubscribe from current dataset
	        if (this._data.off) {
	          this._data.off("*", this._listener);
	        } // trigger a remove of all items in memory


	        var ids = this._data.getIds({
	          filter: filter$2(this._options)
	        });

	        var items = this._data.get(ids);

	        this._ids.clear();

	        this.length = 0;

	        this._trigger("remove", {
	          items: ids,
	          oldData: items
	        });
	      }

	      if (data != null) {
	        this._data = data; // trigger an add of all added items

	        var _ids = this._data.getIds({
	          filter: filter$2(this._options)
	        });

	        for (var i = 0, len = _ids.length; i < len; i++) {
	          var id = _ids[i];

	          this._ids.add(id);
	        }

	        this.length = _ids.length;

	        this._trigger("add", {
	          items: _ids
	        });
	      } else {
	        this._data = new DataSet();
	      } // subscribe to new dataset


	      if (this._data.on) {
	        this._data.on("*", this._listener);
	      }
	    }
	    /**
	     * Refresh the DataView.
	     * Useful when the DataView has a filter function containing a variable parameter.
	     */

	  }, {
	    key: "refresh",
	    value: function refresh() {
	      var ids = this._data.getIds({
	        filter: filter$2(this._options)
	      });

	      var oldIds = toConsumableArray(this._ids);

	      var newIds = {};
	      var addedIds = [];
	      var removedIds = [];
	      var removedItems = []; // check for additions

	      for (var i = 0, len = ids.length; i < len; i++) {
	        var id = ids[i];
	        newIds[id] = true;

	        if (!this._ids.has(id)) {
	          addedIds.push(id);

	          this._ids.add(id);
	        }
	      } // check for removals


	      for (var _i11 = 0, _len12 = oldIds.length; _i11 < _len12; _i11++) {
	        var _id7 = oldIds[_i11];

	        var item = this._data.get(_id7);

	        if (item == null) {
	          // @TODO: Investigate.
	          // Doesn't happen during tests or examples.
	          // Is it really impossible or could it eventually happen?
	          // How to handle it if it does? The types guarantee non-nullable items.
	          console.error("If you see this, report it please.");
	        } else if (!newIds[_id7]) {
	          removedIds.push(_id7);
	          removedItems.push(item);

	          this._ids.delete(_id7);
	        }
	      }

	      this.length += addedIds.length - removedIds.length; // trigger events

	      if (addedIds.length) {
	        this._trigger("add", {
	          items: addedIds
	        });
	      }

	      if (removedIds.length) {
	        this._trigger("remove", {
	          items: removedIds,
	          oldData: removedItems
	        });
	      }
	    }
	    /** @inheritdoc */

	  }, {
	    key: "get",
	    value: function get(first, second) {
	      if (this._data == null) {
	        return null;
	      } // parse the arguments


	      var ids = null;
	      var options;

	      if (isId(first) || isArray$3(first)) {
	        ids = first;
	        options = second;
	      } else {
	        options = first;
	      } // extend the options with the default options and provided options


	      var viewOptions = assign$2({}, this._options, options); // create a combined filter method when needed


	      var thisFilter = filter$2(this._options);

	      var optionsFilter = options && filter$2(options);

	      if (thisFilter && optionsFilter) {
	        viewOptions.filter = function (item) {
	          return thisFilter(item) && optionsFilter(item);
	        };
	      }

	      if (ids == null) {
	        return this._data.get(viewOptions);
	      } else {
	        return this._data.get(ids, viewOptions);
	      }
	    }
	    /** @inheritdoc */

	  }, {
	    key: "getIds",
	    value: function getIds(options) {
	      if (this._data.length) {
	        var defaultFilter = filter$2(this._options);

	        var optionsFilter = options != null ? filter$2(options) : null;
	        var filter;

	        if (optionsFilter) {
	          if (defaultFilter) {
	            filter = function filter(item) {
	              return defaultFilter(item) && optionsFilter(item);
	            };
	          } else {
	            filter = optionsFilter;
	          }
	        } else {
	          filter = defaultFilter;
	        }

	        return this._data.getIds({
	          filter: filter,
	          order: options && options.order
	        });
	      } else {
	        return [];
	      }
	    }
	    /** @inheritdoc */

	  }, {
	    key: "forEach",
	    value: function forEach(callback, options) {
	      if (this._data) {
	        var _context31;

	        var defaultFilter = filter$2(this._options);

	        var optionsFilter = options && filter$2(options);

	        var filter;

	        if (optionsFilter) {
	          if (defaultFilter) {
	            filter = function filter(item) {
	              return defaultFilter(item) && optionsFilter(item);
	            };
	          } else {
	            filter = optionsFilter;
	          }
	        } else {
	          filter = defaultFilter;
	        }

	        forEach$3(_context31 = this._data).call(_context31, callback, {
	          filter: filter,
	          order: options && options.order
	        });
	      }
	    }
	    /** @inheritdoc */

	  }, {
	    key: "map",
	    value: function map(callback, options) {
	      if (this._data) {
	        var _context32;

	        var defaultFilter = filter$2(this._options);

	        var optionsFilter = options && filter$2(options);

	        var filter;

	        if (optionsFilter) {
	          if (defaultFilter) {
	            filter = function filter(item) {
	              return defaultFilter(item) && optionsFilter(item);
	            };
	          } else {
	            filter = optionsFilter;
	          }
	        } else {
	          filter = defaultFilter;
	        }

	        return map$5(_context32 = this._data).call(_context32, callback, {
	          filter: filter,
	          order: options && options.order
	        });
	      } else {
	        return [];
	      }
	    }
	    /** @inheritdoc */

	  }, {
	    key: "getDataSet",
	    value: function getDataSet() {
	      return this._data.getDataSet();
	    }
	    /** @inheritdoc */

	  }, {
	    key: "stream",
	    value: function stream(ids) {
	      var _context33;

	      return this._data.stream(ids || defineProperty$8({}, iterator$4, bind$2(_context33 = keys$3(this._ids)).call(_context33, this._ids)));
	    }
	    /**
	     * Render the instance unusable prior to garbage collection.
	     *
	     * @remarks
	     * The intention of this method is to help discover scenarios where the data
	     * view is being used when the programmer thinks it has been garbage collected
	     * already. It's stricter version of `dataView.setData(null)`.
	     */

	  }, {
	    key: "dispose",
	    value: function dispose() {
	      var _a;

	      if ((_a = this._data) === null || _a === void 0 ? void 0 : _a.off) {
	        this._data.off("*", this._listener);
	      }

	      var message = "This data view has already been disposed of.";

	      defineProperty$2(this, "_data", {
	        get: function get() {
	          throw new Error(message);
	        },
	        set: function set() {
	          throw new Error(message);
	        },
	        configurable: false
	      });
	    }
	    /**
	     * Event listener. Will propagate all events from the connected data set to the subscribers of the DataView, but will filter the items and only trigger when there are changes in the filtered data set.
	     *
	     * @param event - The name of the event.
	     * @param params - Parameters of the event.
	     * @param senderId - Id supplied by the sender.
	     */

	  }, {
	    key: "_onEvent",
	    value: function _onEvent(event, params, senderId) {
	      if (!params || !params.items || !this._data) {
	        return;
	      }

	      var ids = params.items;
	      var addedIds = [];
	      var updatedIds = [];
	      var removedIds = [];
	      var oldItems = [];
	      var updatedItems = [];
	      var removedItems = [];

	      switch (event) {
	        case "add":
	          // filter the ids of the added items
	          for (var i = 0, len = ids.length; i < len; i++) {
	            var id = ids[i];
	            var item = this.get(id);

	            if (item) {
	              this._ids.add(id);

	              addedIds.push(id);
	            }
	          }

	          break;

	        case "update":
	          // determine the event from the views viewpoint: an updated
	          // item can be added, updated, or removed from this view.
	          for (var _i12 = 0, _len13 = ids.length; _i12 < _len13; _i12++) {
	            var _id8 = ids[_i12];

	            var _item2 = this.get(_id8);

	            if (_item2) {
	              if (this._ids.has(_id8)) {
	                updatedIds.push(_id8);
	                updatedItems.push(params.data[_i12]);
	                oldItems.push(params.oldData[_i12]);
	              } else {
	                this._ids.add(_id8);

	                addedIds.push(_id8);
	              }
	            } else {
	              if (this._ids.has(_id8)) {
	                this._ids.delete(_id8);

	                removedIds.push(_id8);
	                removedItems.push(params.oldData[_i12]);
	              }
	            }
	          }

	          break;

	        case "remove":
	          // filter the ids of the removed items
	          for (var _i13 = 0, _len14 = ids.length; _i13 < _len14; _i13++) {
	            var _id9 = ids[_i13];

	            if (this._ids.has(_id9)) {
	              this._ids.delete(_id9);

	              removedIds.push(_id9);
	              removedItems.push(params.oldData[_i13]);
	            }
	          }

	          break;
	      }

	      this.length += addedIds.length - removedIds.length;

	      if (addedIds.length) {
	        this._trigger("add", {
	          items: addedIds
	        }, senderId);
	      }

	      if (updatedIds.length) {
	        this._trigger("update", {
	          items: updatedIds,
	          oldData: oldItems,
	          data: updatedItems
	        }, senderId);
	      }

	      if (removedIds.length) {
	        this._trigger("remove", {
	          items: removedIds,
	          oldData: removedItems
	        }, senderId);
	      }
	    }
	  }]);

	  return DataView;
	}(DataSetPart);

	var nsData = /*#__PURE__*/Object.freeze({
		__proto__: null,
		DataSet: DataSet,
		DataStream: DataStream,
		DataView: DataView,
		Queue: Queue,
		createNewDataPipeFrom: createNewDataPipeFrom
	});

	// https://tc39.github.io/ecma262/#sec-number.isnan

	_export({
	  target: 'Number',
	  stat: true
	}, {
	  isNaN: function isNaN(number) {
	    // eslint-disable-next-line no-self-compare
	    return number != number;
	  }
	});

	var isNan = path.Number.isNaN;

	var isNan$1 = isNan;

	var isNan$2 = isNan$1;

	var nativeReverse = [].reverse;
	var test$2 = [1, 2]; // `Array.prototype.reverse` method
	// https://tc39.github.io/ecma262/#sec-array.prototype.reverse
	// fix for Safari 12.0 bug
	// https://bugs.webkit.org/show_bug.cgi?id=188794

	_export({
	  target: 'Array',
	  proto: true,
	  forced: String(test$2) === String(test$2.reverse())
	}, {
	  reverse: function reverse() {
	    // eslint-disable-next-line no-self-assign
	    if (isArray(this)) this.length = this.length;
	    return nativeReverse.call(this);
	  }
	});

	var reverse = entryVirtual('Array').reverse;

	var ArrayPrototype$f = Array.prototype;

	var reverse_1 = function (it) {
	  var own = it.reverse;
	  return it === ArrayPrototype$f || it instanceof Array && own === ArrayPrototype$f.reverse ? reverse : own;
	};

	var reverse$1 = reverse_1;

	var reverse$2 = reverse$1;

	// https://tc39.github.io/ecma262/#sec-array.prototype.fill


	var arrayFill = function fill(value
	/* , start = 0, end = @length */
	) {
	  var O = toObject(this);
	  var length = toLength(O.length);
	  var argumentsLength = arguments.length;
	  var index = toAbsoluteIndex(argumentsLength > 1 ? arguments[1] : undefined, length);
	  var end = argumentsLength > 2 ? arguments[2] : undefined;
	  var endPos = end === undefined ? length : toAbsoluteIndex(end, length);

	  while (endPos > index) O[index++] = value;

	  return O;
	};

	// https://tc39.github.io/ecma262/#sec-array.prototype.fill

	_export({
	  target: 'Array',
	  proto: true
	}, {
	  fill: arrayFill
	}); // https://tc39.github.io/ecma262/#sec-array.prototype-@@unscopables

	var fill = entryVirtual('Array').fill;

	var ArrayPrototype$g = Array.prototype;

	var fill_1 = function (it) {
	  var own = it.fill;
	  return it === ArrayPrototype$g || it instanceof Array && own === ArrayPrototype$g.fill ? fill : own;
	};

	var fill$1 = fill_1;

	var fill$2 = fill$1;

	// `Math.sign` method implementation
	// https://tc39.github.io/ecma262/#sec-math.sign
	var mathSign = Math.sign || function sign(x) {
	  // eslint-disable-next-line no-self-compare
	  return (x = +x) == 0 || x != x ? x : x < 0 ? -1 : 1;
	};

	// https://tc39.github.io/ecma262/#sec-math.sign

	_export({
	  target: 'Math',
	  stat: true
	}, {
	  sign: mathSign
	});

	var sign = path.Math.sign;

	var sign$1 = sign;

	var sign$2 = sign$1;

	var trim$4 = stringTrim.trim;
	var $parseFloat = global_1.parseFloat;
	var FORCED$6 = 1 / $parseFloat(whitespaces + '-0') !== -Infinity; // `parseFloat` method
	// https://tc39.github.io/ecma262/#sec-parsefloat-string

	var numberParseFloat = FORCED$6 ? function parseFloat(string) {
	  var trimmedString = trim$4(String(string));
	  var result = $parseFloat(trimmedString);
	  return result === 0 && trimmedString.charAt(0) == '-' ? -0 : result;
	} : $parseFloat;

	// https://tc39.github.io/ecma262/#sec-parsefloat-string

	_export({
	  global: true,
	  forced: parseFloat != numberParseFloat
	}, {
	  parseFloat: numberParseFloat
	});

	var _parseFloat = path.parseFloat;

	var _parseFloat$1 = _parseFloat;

	var _parseFloat$2 = _parseFloat$1;

	var componentEmitter = createCommonjsModule(function (module) {
	  /**
	   * Expose `Emitter`.
	   */
	  {
	    module.exports = Emitter;
	  }
	  /**
	   * Initialize a new `Emitter`.
	   *
	   * @api public
	   */


	  function Emitter(obj) {
	    if (obj) return mixin(obj);
	  }
	  /**
	   * Mixin the emitter properties.
	   *
	   * @param {Object} obj
	   * @return {Object}
	   * @api private
	   */

	  function mixin(obj) {
	    for (var key in Emitter.prototype) {
	      obj[key] = Emitter.prototype[key];
	    }

	    return obj;
	  }
	  /**
	   * Listen on the given `event` with `fn`.
	   *
	   * @param {String} event
	   * @param {Function} fn
	   * @return {Emitter}
	   * @api public
	   */


	  Emitter.prototype.on = Emitter.prototype.addEventListener = function (event, fn) {
	    this._callbacks = this._callbacks || {};
	    (this._callbacks['$' + event] = this._callbacks['$' + event] || []).push(fn);
	    return this;
	  };
	  /**
	   * Adds an `event` listener that will be invoked a single
	   * time then automatically removed.
	   *
	   * @param {String} event
	   * @param {Function} fn
	   * @return {Emitter}
	   * @api public
	   */


	  Emitter.prototype.once = function (event, fn) {
	    function on() {
	      this.off(event, on);
	      fn.apply(this, arguments);
	    }

	    on.fn = fn;
	    this.on(event, on);
	    return this;
	  };
	  /**
	   * Remove the given callback for `event` or all
	   * registered callbacks.
	   *
	   * @param {String} event
	   * @param {Function} fn
	   * @return {Emitter}
	   * @api public
	   */


	  Emitter.prototype.off = Emitter.prototype.removeListener = Emitter.prototype.removeAllListeners = Emitter.prototype.removeEventListener = function (event, fn) {
	    this._callbacks = this._callbacks || {}; // all

	    if (0 == arguments.length) {
	      this._callbacks = {};
	      return this;
	    } // specific event


	    var callbacks = this._callbacks['$' + event];
	    if (!callbacks) return this; // remove all handlers

	    if (1 == arguments.length) {
	      delete this._callbacks['$' + event];
	      return this;
	    } // remove specific handler


	    var cb;

	    for (var i = 0; i < callbacks.length; i++) {
	      cb = callbacks[i];

	      if (cb === fn || cb.fn === fn) {
	        callbacks.splice(i, 1);
	        break;
	      }
	    } // Remove event specific arrays for event types that no
	    // one is subscribed for to avoid memory leak.


	    if (callbacks.length === 0) {
	      delete this._callbacks['$' + event];
	    }

	    return this;
	  };
	  /**
	   * Emit `event` with the given args.
	   *
	   * @param {String} event
	   * @param {Mixed} ...
	   * @return {Emitter}
	   */


	  Emitter.prototype.emit = function (event) {
	    this._callbacks = this._callbacks || {};
	    var args = new Array(arguments.length - 1),
	        callbacks = this._callbacks['$' + event];

	    for (var i = 1; i < arguments.length; i++) {
	      args[i - 1] = arguments[i];
	    }

	    if (callbacks) {
	      callbacks = callbacks.slice(0);

	      for (var i = 0, len = callbacks.length; i < len; ++i) {
	        callbacks[i].apply(this, args);
	      }
	    }

	    return this;
	  };
	  /**
	   * Return array of callbacks for `event`.
	   *
	   * @param {String} event
	   * @return {Array}
	   * @api public
	   */


	  Emitter.prototype.listeners = function (event) {
	    this._callbacks = this._callbacks || {};
	    return this._callbacks['$' + event] || [];
	  };
	  /**
	   * Check if this emitter has `event` handlers.
	   *
	   * @param {String} event
	   * @return {Boolean}
	   * @api public
	   */


	  Emitter.prototype.hasListeners = function (event) {
	    return !!this.listeners(event).length;
	  };
	});

	/**
	 * @prototype Point3d
	 * @param {number} [x]
	 * @param {number} [y]
	 * @param {number} [z]
	 */

	function Point3d(x, y, z) {
	  this.x = x !== undefined ? x : 0;
	  this.y = y !== undefined ? y : 0;
	  this.z = z !== undefined ? z : 0;
	}
	/**
	 * Subtract the two provided points, returns a-b
	 * @param {Point3d} a
	 * @param {Point3d} b
	 * @return {Point3d} a-b
	 */


	Point3d.subtract = function (a, b) {
	  var sub = new Point3d();
	  sub.x = a.x - b.x;
	  sub.y = a.y - b.y;
	  sub.z = a.z - b.z;
	  return sub;
	};
	/**
	 * Add the two provided points, returns a+b
	 * @param {Point3d} a
	 * @param {Point3d} b
	 * @return {Point3d} a+b
	 */


	Point3d.add = function (a, b) {
	  var sum = new Point3d();
	  sum.x = a.x + b.x;
	  sum.y = a.y + b.y;
	  sum.z = a.z + b.z;
	  return sum;
	};
	/**
	 * Calculate the average of two 3d points
	 * @param {Point3d} a
	 * @param {Point3d} b
	 * @return {Point3d} The average, (a+b)/2
	 */


	Point3d.avg = function (a, b) {
	  return new Point3d((a.x + b.x) / 2, (a.y + b.y) / 2, (a.z + b.z) / 2);
	};
	/**
	 * Scale the provided point by a scalar, returns p*c
	 * @param {Point3d} p
	 * @param {number} c
	 * @return {Point3d} p*c
	 */


	Point3d.scalarProduct = function (p, c) {
	  return new Point3d(p.x * c, p.y * c, p.z * c);
	};
	/**
	 * Calculate the dot product of the two provided points, returns a.b
	 * Documentation: http://en.wikipedia.org/wiki/Dot_product
	 * @param {Point3d} a
	 * @param {Point3d} b
	 * @return {Point3d} dot product a.b
	 */


	Point3d.dotProduct = function (a, b) {
	  return a.x * b.x + a.y * b.y + a.z * b.z;
	};
	/**
	 * Calculate the cross product of the two provided points, returns axb
	 * Documentation: http://en.wikipedia.org/wiki/Cross_product
	 * @param {Point3d} a
	 * @param {Point3d} b
	 * @return {Point3d} cross product axb
	 */


	Point3d.crossProduct = function (a, b) {
	  var crossproduct = new Point3d();
	  crossproduct.x = a.y * b.z - a.z * b.y;
	  crossproduct.y = a.z * b.x - a.x * b.z;
	  crossproduct.z = a.x * b.y - a.y * b.x;
	  return crossproduct;
	};
	/**
	 * Retrieve the length of the vector (or the distance from this point to the origin
	 * @return {number}  length
	 */


	Point3d.prototype.length = function () {
	  return Math.sqrt(this.x * this.x + this.y * this.y + this.z * this.z);
	};
	/**
	 * Return a normalized vector pointing in the same direction.
	 * @return {Point3d}  normalized
	 */


	Point3d.prototype.normalize = function () {
	  return Point3d.scalarProduct(this, 1 / this.length());
	};

	var Point3d_1 = Point3d;
	/**
	 * @prototype Point2d
	 * @param {number} [x]
	 * @param {number} [y]
	 */

	function Point2d(x, y) {
	  this.x = x !== undefined ? x : 0;
	  this.y = y !== undefined ? y : 0;
	}

	var Point2d_1 = Point2d;
	/**
	 * An html slider control with start/stop/prev/next buttons
	 *
	 * @constructor Slider
	 * @param {Element} container  The element where the slider will be created
	 * @param {Object} options   Available options:
	 *                 {boolean} visible   If true (default) the
	 *                           slider is visible.
	 */

	function Slider(container, options) {
	  if (container === undefined) {
	    throw new Error('No container element defined');
	  }

	  this.container = container;
	  this.visible = options && options.visible != undefined ? options.visible : true;

	  if (this.visible) {
	    this.frame = document.createElement('DIV'); //this.frame.style.backgroundColor = '#E5E5E5';

	    this.frame.style.width = '100%';
	    this.frame.style.position = 'relative';
	    this.container.appendChild(this.frame);
	    this.frame.prev = document.createElement('INPUT');
	    this.frame.prev.type = 'BUTTON';
	    this.frame.prev.value = 'Prev';
	    this.frame.appendChild(this.frame.prev);
	    this.frame.play = document.createElement('INPUT');
	    this.frame.play.type = 'BUTTON';
	    this.frame.play.value = 'Play';
	    this.frame.appendChild(this.frame.play);
	    this.frame.next = document.createElement('INPUT');
	    this.frame.next.type = 'BUTTON';
	    this.frame.next.value = 'Next';
	    this.frame.appendChild(this.frame.next);
	    this.frame.bar = document.createElement('INPUT');
	    this.frame.bar.type = 'BUTTON';
	    this.frame.bar.style.position = 'absolute';
	    this.frame.bar.style.border = '1px solid red';
	    this.frame.bar.style.width = '100px';
	    this.frame.bar.style.height = '6px';
	    this.frame.bar.style.borderRadius = '2px';
	    this.frame.bar.style.MozBorderRadius = '2px';
	    this.frame.bar.style.border = '1px solid #7F7F7F';
	    this.frame.bar.style.backgroundColor = '#E5E5E5';
	    this.frame.appendChild(this.frame.bar);
	    this.frame.slide = document.createElement('INPUT');
	    this.frame.slide.type = 'BUTTON';
	    this.frame.slide.style.margin = '0px';
	    this.frame.slide.value = ' ';
	    this.frame.slide.style.position = 'relative';
	    this.frame.slide.style.left = '-100px';
	    this.frame.appendChild(this.frame.slide); // create events

	    var me = this;

	    this.frame.slide.onmousedown = function (event) {
	      me._onMouseDown(event);
	    };

	    this.frame.prev.onclick = function (event) {
	      me.prev(event);
	    };

	    this.frame.play.onclick = function (event) {
	      me.togglePlay(event);
	    };

	    this.frame.next.onclick = function (event) {
	      me.next(event);
	    };
	  }

	  this.onChangeCallback = undefined;
	  this.values = [];
	  this.index = undefined;
	  this.playTimeout = undefined;
	  this.playInterval = 1000; // milliseconds

	  this.playLoop = true;
	}
	/**
	 * Select the previous index
	 */


	Slider.prototype.prev = function () {
	  var index = this.getIndex();

	  if (index > 0) {
	    index--;
	    this.setIndex(index);
	  }
	};
	/**
	 * Select the next index
	 */


	Slider.prototype.next = function () {
	  var index = this.getIndex();

	  if (index < values$2(this).length - 1) {
	    index++;
	    this.setIndex(index);
	  }
	};
	/**
	 * Select the next index
	 */


	Slider.prototype.playNext = function () {
	  var start = new Date();
	  var index = this.getIndex();

	  if (index < values$2(this).length - 1) {
	    index++;
	    this.setIndex(index);
	  } else if (this.playLoop) {
	    // jump to the start
	    index = 0;
	    this.setIndex(index);
	  }

	  var end = new Date();
	  var diff = end - start; // calculate how much time it to to set the index and to execute the callback
	  // function.

	  var interval = Math.max(this.playInterval - diff, 0); // document.title = diff // TODO: cleanup

	  var me = this;
	  this.playTimeout = setTimeout$2(function () {
	    me.playNext();
	  }, interval);
	};
	/**
	 * Toggle start or stop playing
	 */


	Slider.prototype.togglePlay = function () {
	  if (this.playTimeout === undefined) {
	    this.play();
	  } else {
	    this.stop();
	  }
	};
	/**
	 * Start playing
	 */


	Slider.prototype.play = function () {
	  // Test whether already playing
	  if (this.playTimeout) return;
	  this.playNext();

	  if (this.frame) {
	    this.frame.play.value = 'Stop';
	  }
	};
	/**
	 * Stop playing
	 */


	Slider.prototype.stop = function () {
	  clearInterval(this.playTimeout);
	  this.playTimeout = undefined;

	  if (this.frame) {
	    this.frame.play.value = 'Play';
	  }
	};
	/**
	 * Set a callback function which will be triggered when the value of the
	 * slider bar has changed.
	 *
	 * @param {function} callback
	 */


	Slider.prototype.setOnChangeCallback = function (callback) {
	  this.onChangeCallback = callback;
	};
	/**
	 * Set the interval for playing the list
	 * @param {number} interval   The interval in milliseconds
	 */


	Slider.prototype.setPlayInterval = function (interval) {
	  this.playInterval = interval;
	};
	/**
	 * Retrieve the current play interval
	 * @return {number} interval   The interval in milliseconds
	 */


	Slider.prototype.getPlayInterval = function () {
	  return this.playInterval;
	};
	/**
	 * Set looping on or off
	 * @param {boolean} doLoop  If true, the slider will jump to the start when
	 *               the end is passed, and will jump to the end
	 *               when the start is passed.
	 *
	 */


	Slider.prototype.setPlayLoop = function (doLoop) {
	  this.playLoop = doLoop;
	};
	/**
	 * Execute the onchange callback function
	 */


	Slider.prototype.onChange = function () {
	  if (this.onChangeCallback !== undefined) {
	    this.onChangeCallback();
	  }
	};
	/**
	 * redraw the slider on the correct place
	 */


	Slider.prototype.redraw = function () {
	  if (this.frame) {
	    // resize the bar
	    this.frame.bar.style.top = this.frame.clientHeight / 2 - this.frame.bar.offsetHeight / 2 + 'px';
	    this.frame.bar.style.width = this.frame.clientWidth - this.frame.prev.clientWidth - this.frame.play.clientWidth - this.frame.next.clientWidth - 30 + 'px'; // position the slider button

	    var left = this.indexToLeft(this.index);
	    this.frame.slide.style.left = left + 'px';
	  }
	};
	/**
	 * Set the list with values for the slider
	 * @param {Array} values   A javascript array with values (any type)
	 */


	Slider.prototype.setValues = function (values) {
	  this.values = values;
	  if (values$2(this).length > 0) this.setIndex(0);else this.index = undefined;
	};
	/**
	 * Select a value by its index
	 * @param {number} index
	 */


	Slider.prototype.setIndex = function (index) {
	  if (index < values$2(this).length) {
	    this.index = index;
	    this.redraw();
	    this.onChange();
	  } else {
	    throw new Error('Index out of range');
	  }
	};
	/**
	 * retrieve the index of the currently selected vaue
	 * @return {number} index
	 */


	Slider.prototype.getIndex = function () {
	  return this.index;
	};
	/**
	 * retrieve the currently selected value
	 * @return {*} value
	 */


	Slider.prototype.get = function () {
	  return values$2(this)[this.index];
	};

	Slider.prototype._onMouseDown = function (event) {
	  // only react on left mouse button down
	  var leftButtonDown = event.which ? event.which === 1 : event.button === 1;
	  if (!leftButtonDown) return;
	  this.startClientX = event.clientX;
	  this.startSlideX = _parseFloat$2(this.frame.slide.style.left);
	  this.frame.style.cursor = 'move'; // add event listeners to handle moving the contents
	  // we store the function onmousemove and onmouseup in the graph, so we can
	  // remove the eventlisteners lateron in the function mouseUp()

	  var me = this;

	  this.onmousemove = function (event) {
	    me._onMouseMove(event);
	  };

	  this.onmouseup = function (event) {
	    me._onMouseUp(event);
	  };

	  addEventListener(document, 'mousemove', this.onmousemove);
	  addEventListener(document, 'mouseup', this.onmouseup);
	  preventDefault(event);
	};

	Slider.prototype.leftToIndex = function (left) {
	  var width = _parseFloat$2(this.frame.bar.style.width) - this.frame.slide.clientWidth - 10;
	  var x = left - 3;
	  var index = Math.round(x / width * (values$2(this).length - 1));
	  if (index < 0) index = 0;
	  if (index > values$2(this).length - 1) index = values$2(this).length - 1;
	  return index;
	};

	Slider.prototype.indexToLeft = function (index) {
	  var width = _parseFloat$2(this.frame.bar.style.width) - this.frame.slide.clientWidth - 10;
	  var x = index / (values$2(this).length - 1) * width;
	  var left = x + 3;
	  return left;
	};

	Slider.prototype._onMouseMove = function (event) {
	  var diff = event.clientX - this.startClientX;
	  var x = this.startSlideX + diff;
	  var index = this.leftToIndex(x);
	  this.setIndex(index);
	  preventDefault();
	};

	Slider.prototype._onMouseUp = function (event) {
	  // eslint-disable-line no-unused-vars
	  this.frame.style.cursor = 'auto'; // remove event listeners

	  removeEventListener(document, 'mousemove', this.onmousemove);
	  removeEventListener(document, 'mouseup', this.onmouseup);
	  preventDefault();
	};
	/**
	 * @prototype StepNumber
	 * The class StepNumber is an iterator for Numbers. You provide a start and end
	 * value, and a best step size. StepNumber itself rounds to fixed values and
	 * a finds the step that best fits the provided step.
	 *
	 * If prettyStep is true, the step size is chosen as close as possible to the
	 * provided step, but being a round value like 1, 2, 5, 10, 20, 50, ....
	 *
	 * Example usage:
	 *   var step = new StepNumber(0, 10, 2.5, true);
	 *   step.start();
	 *   while (!step.end()) {
	 *   alert(step.getCurrent());
	 *   step.next();
	 *   }
	 *
	 * Version: 1.0
	 *
	 * @param {number} start     The start value
	 * @param {number} end     The end value
	 * @param {number} step    Optional. Step size. Must be a positive value.
	 * @param {boolean} prettyStep Optional. If true, the step size is rounded
	 *               To a pretty step size (like 1, 2, 5, 10, 20, 50, ...)
	 */


	function StepNumber(start, end, step, prettyStep) {
	  // set default values
	  this._start = 0;
	  this._end = 0;
	  this._step = 1;
	  this.prettyStep = true;
	  this.precision = 5;
	  this._current = 0;
	  this.setRange(start, end, step, prettyStep);
	}
	/**
	 * Check for input values, to prevent disasters from happening
	 *
	 * Source: http://stackoverflow.com/a/1830844
	 *
	 * @param {string} n
	 * @returns {boolean}
	 */


	StepNumber.prototype.isNumeric = function (n) {
	  return !isNaN(_parseFloat$2(n)) && isFinite(n);
	};
	/**
	 * Set a new range: start, end and step.
	 *
	 * @param {number} start     The start value
	 * @param {number} end     The end value
	 * @param {number} step    Optional. Step size. Must be a positive value.
	 * @param {boolean} prettyStep Optional. If true, the step size is rounded
	 *               To a pretty step size (like 1, 2, 5, 10, 20, 50, ...)
	 */


	StepNumber.prototype.setRange = function (start, end, step, prettyStep) {
	  if (!this.isNumeric(start)) {
	    throw new Error('Parameter \'start\' is not numeric; value: ' + start);
	  }

	  if (!this.isNumeric(end)) {
	    throw new Error('Parameter \'end\' is not numeric; value: ' + start);
	  }

	  if (!this.isNumeric(step)) {
	    throw new Error('Parameter \'step\' is not numeric; value: ' + start);
	  }

	  this._start = start ? start : 0;
	  this._end = end ? end : 0;
	  this.setStep(step, prettyStep);
	};
	/**
	 * Set a new step size
	 * @param {number} step    New step size. Must be a positive value
	 * @param {boolean} prettyStep Optional. If true, the provided step is rounded
	 *               to a pretty step size (like 1, 2, 5, 10, 20, 50, ...)
	 */


	StepNumber.prototype.setStep = function (step, prettyStep) {
	  if (step === undefined || step <= 0) return;
	  if (prettyStep !== undefined) this.prettyStep = prettyStep;
	  if (this.prettyStep === true) this._step = StepNumber.calculatePrettyStep(step);else this._step = step;
	};
	/**
	 * Calculate a nice step size, closest to the desired step size.
	 * Returns a value in one of the ranges 1*10^n, 2*10^n, or 5*10^n, where n is an
	 * integer Number. For example 1, 2, 5, 10, 20, 50, etc...
	 * @param {number}  step  Desired step size
	 * @return {number}     Nice step size
	 */


	StepNumber.calculatePrettyStep = function (step) {
	  var log10 = function log10(x) {
	    return Math.log(x) / Math.LN10;
	  }; // try three steps (multiple of 1, 2, or 5


	  var step1 = Math.pow(10, Math.round(log10(step))),
	      step2 = 2 * Math.pow(10, Math.round(log10(step / 2))),
	      step5 = 5 * Math.pow(10, Math.round(log10(step / 5))); // choose the best step (closest to minimum step)

	  var prettyStep = step1;
	  if (Math.abs(step2 - step) <= Math.abs(prettyStep - step)) prettyStep = step2;
	  if (Math.abs(step5 - step) <= Math.abs(prettyStep - step)) prettyStep = step5; // for safety

	  if (prettyStep <= 0) {
	    prettyStep = 1;
	  }

	  return prettyStep;
	};
	/**
	 * returns the current value of the step
	 * @return {number} current value
	 */


	StepNumber.prototype.getCurrent = function () {
	  return _parseFloat$2(this._current.toPrecision(this.precision));
	};
	/**
	 * returns the current step size
	 * @return {number} current step size
	 */


	StepNumber.prototype.getStep = function () {
	  return this._step;
	};
	/**
	 * Set the current to its starting value.
	 *
	 * By default, this will be the largest value smaller than start, which
	 * is a multiple of the step size.
	 *
	 * Parameters checkFirst is optional, default false.
	 * If set to true, move the current value one step if smaller than start.
	 *
	 * @param {boolean} [checkFirst=false]
	 */


	StepNumber.prototype.start = function (checkFirst) {
	  if (checkFirst === undefined) {
	    checkFirst = false;
	  }

	  this._current = this._start - this._start % this._step;

	  if (checkFirst) {
	    if (this.getCurrent() < this._start) {
	      this.next();
	    }
	  }
	};
	/**
	 * Do a step, add the step size to the current value
	 */


	StepNumber.prototype.next = function () {
	  this._current += this._step;
	};
	/**
	 * Returns true whether the end is reached
	 * @return {boolean}  True if the current value has passed the end value.
	 */


	StepNumber.prototype.end = function () {
	  return this._current > this._end;
	};

	var StepNumber_1 = StepNumber;
	/**
	 * The camera is mounted on a (virtual) camera arm. The camera arm can rotate
	 * The camera is always looking in the direction of the origin of the arm.
	 * This way, the camera always rotates around one fixed point, the location
	 * of the camera arm.
	 *
	 * Documentation:
	 *   http://en.wikipedia.org/wiki/3D_projection
	 * @class Camera
	 */

	function Camera() {
	  this.armLocation = new Point3d_1();
	  this.armRotation = {};
	  this.armRotation.horizontal = 0;
	  this.armRotation.vertical = 0;
	  this.armLength = 1.7;
	  this.cameraOffset = new Point3d_1();
	  this.offsetMultiplier = 0.6;
	  this.cameraLocation = new Point3d_1();
	  this.cameraRotation = new Point3d_1(0.5 * Math.PI, 0, 0);
	  this.calculateCameraOrientation();
	}
	/**
	 * Set offset camera in camera coordinates
	 * @param {number} x offset by camera horisontal
	 * @param {number} y offset by camera vertical
	 */


	Camera.prototype.setOffset = function (x, y) {
	  var abs = Math.abs,
	      sign = sign$2,
	      mul = this.offsetMultiplier,
	      border = this.armLength * mul;

	  if (abs(x) > border) {
	    x = sign(x) * border;
	  }

	  if (abs(y) > border) {
	    y = sign(y) * border;
	  }

	  this.cameraOffset.x = x;
	  this.cameraOffset.y = y;
	  this.calculateCameraOrientation();
	};
	/**
	 * Get camera offset by horizontal and vertical
	 * @returns {number}
	 */


	Camera.prototype.getOffset = function () {
	  return this.cameraOffset;
	};
	/**
	 * Set the location (origin) of the arm
	 * @param {number} x  Normalized value of x
	 * @param {number} y  Normalized value of y
	 * @param {number} z  Normalized value of z
	 */


	Camera.prototype.setArmLocation = function (x, y, z) {
	  this.armLocation.x = x;
	  this.armLocation.y = y;
	  this.armLocation.z = z;
	  this.calculateCameraOrientation();
	};
	/**
	 * Set the rotation of the camera arm
	 * @param {number} horizontal   The horizontal rotation, between 0 and 2*PI.
	 *                Optional, can be left undefined.
	 * @param {number} vertical   The vertical rotation, between 0 and 0.5*PI
	 *                if vertical=0.5*PI, the graph is shown from the
	 *                top. Optional, can be left undefined.
	 */


	Camera.prototype.setArmRotation = function (horizontal, vertical) {
	  if (horizontal !== undefined) {
	    this.armRotation.horizontal = horizontal;
	  }

	  if (vertical !== undefined) {
	    this.armRotation.vertical = vertical;
	    if (this.armRotation.vertical < 0) this.armRotation.vertical = 0;
	    if (this.armRotation.vertical > 0.5 * Math.PI) this.armRotation.vertical = 0.5 * Math.PI;
	  }

	  if (horizontal !== undefined || vertical !== undefined) {
	    this.calculateCameraOrientation();
	  }
	};
	/**
	 * Retrieve the current arm rotation
	 * @return {object}   An object with parameters horizontal and vertical
	 */


	Camera.prototype.getArmRotation = function () {
	  var rot = {};
	  rot.horizontal = this.armRotation.horizontal;
	  rot.vertical = this.armRotation.vertical;
	  return rot;
	};
	/**
	 * Set the (normalized) length of the camera arm.
	 * @param {number} length A length between 0.71 and 5.0
	 */


	Camera.prototype.setArmLength = function (length) {
	  if (length === undefined) return;
	  this.armLength = length; // Radius must be larger than the corner of the graph,
	  // which has a distance of sqrt(0.5^2+0.5^2) = 0.71 from the center of the
	  // graph

	  if (this.armLength < 0.71) this.armLength = 0.71;
	  if (this.armLength > 5.0) this.armLength = 5.0;
	  this.setOffset(this.cameraOffset.x, this.cameraOffset.y);
	  this.calculateCameraOrientation();
	};
	/**
	 * Retrieve the arm length
	 * @return {number} length
	 */


	Camera.prototype.getArmLength = function () {
	  return this.armLength;
	};
	/**
	 * Retrieve the camera location
	 * @return {Point3d} cameraLocation
	 */


	Camera.prototype.getCameraLocation = function () {
	  return this.cameraLocation;
	};
	/**
	 * Retrieve the camera rotation
	 * @return {Point3d} cameraRotation
	 */


	Camera.prototype.getCameraRotation = function () {
	  return this.cameraRotation;
	};
	/**
	 * Calculate the location and rotation of the camera based on the
	 * position and orientation of the camera arm
	 */


	Camera.prototype.calculateCameraOrientation = function () {
	  // calculate location of the camera
	  this.cameraLocation.x = this.armLocation.x - this.armLength * Math.sin(this.armRotation.horizontal) * Math.cos(this.armRotation.vertical);
	  this.cameraLocation.y = this.armLocation.y - this.armLength * Math.cos(this.armRotation.horizontal) * Math.cos(this.armRotation.vertical);
	  this.cameraLocation.z = this.armLocation.z + this.armLength * Math.sin(this.armRotation.vertical); // calculate rotation of the camera

	  this.cameraRotation.x = Math.PI / 2 - this.armRotation.vertical;
	  this.cameraRotation.y = 0;
	  this.cameraRotation.z = -this.armRotation.horizontal;
	  var xa = this.cameraRotation.x;
	  var za = this.cameraRotation.z;
	  var dx = this.cameraOffset.x;
	  var dy = this.cameraOffset.y;
	  var sin = Math.sin,
	      cos = Math.cos;
	  this.cameraLocation.x = this.cameraLocation.x + dx * cos(za) + dy * -sin(za) * cos(xa);
	  this.cameraLocation.y = this.cameraLocation.y + dx * sin(za) + dy * cos(za) * cos(xa);
	  this.cameraLocation.z = this.cameraLocation.z + dy * sin(xa);
	}; ////////////////////////////////////////////////////////////////////////////////
	// enumerate the available styles


	var STYLE = {
	  BAR: 0,
	  BARCOLOR: 1,
	  BARSIZE: 2,
	  DOT: 3,
	  DOTLINE: 4,
	  DOTCOLOR: 5,
	  DOTSIZE: 6,
	  GRID: 7,
	  LINE: 8,
	  SURFACE: 9
	}; // The string representations of the styles

	var STYLENAME = {
	  'dot': STYLE.DOT,
	  'dot-line': STYLE.DOTLINE,
	  'dot-color': STYLE.DOTCOLOR,
	  'dot-size': STYLE.DOTSIZE,
	  'line': STYLE.LINE,
	  'grid': STYLE.GRID,
	  'surface': STYLE.SURFACE,
	  'bar': STYLE.BAR,
	  'bar-color': STYLE.BARCOLOR,
	  'bar-size': STYLE.BARSIZE
	};
	/**
	 * Field names in the options hash which are of relevance to the user.
	 *
	 * Specifically, these are the fields which require no special handling,
	 * and can be directly copied over.
	 */

	var OPTIONKEYS = ['width', 'height', 'filterLabel', 'legendLabel', 'xLabel', 'yLabel', 'zLabel', 'xValueLabel', 'yValueLabel', 'zValueLabel', 'showXAxis', 'showYAxis', 'showZAxis', 'showGrayBottom', 'showGrid', 'showPerspective', 'showShadow', 'showSurfaceGrid', 'keepAspectRatio', 'rotateAxisLabels', 'verticalRatio', 'dotSizeRatio', 'dotSizeMinFraction', 'dotSizeMaxFraction', 'showAnimationControls', 'animationInterval', 'animationPreload', 'animationAutoStart', 'axisColor', 'axisFontSize', 'axisFontType', 'gridColor', 'xCenter', 'yCenter', 'zoomable', 'tooltipDelay', 'ctrlToZoom'];
	/**
	 * Field names in the options hash which are of relevance to the user.
	 *
	 * Same as OPTIONKEYS, but internally these fields are stored with 
	 * prefix 'default' in the name.
	 */

	var PREFIXEDOPTIONKEYS = ['xBarWidth', 'yBarWidth', 'valueMin', 'valueMax', 'xMin', 'xMax', 'xStep', 'yMin', 'yMax', 'yStep', 'zMin', 'zMax', 'zStep']; // Placeholder for DEFAULTS reference

	var DEFAULTS = undefined;
	/**
	 * Check if given hash is empty.
	 *
	 * Source: http://stackoverflow.com/a/679937
	 *
	 * @param {object} obj
	 * @returns {boolean}
	 */

	function isEmpty(obj) {
	  for (var prop in obj) {
	    if (obj.hasOwnProperty(prop)) return false;
	  }

	  return true;
	}
	/**
	 * Make first letter of parameter upper case.
	 *
	 * Source: http://stackoverflow.com/a/1026087
	 *
	 * @param {string} str
	 * @returns {string}
	 */


	function capitalize(str) {
	  if (str === undefined || str === "" || typeof str != "string") {
	    return str;
	  }

	  return str.charAt(0).toUpperCase() + slice$3(str).call(str, 1);
	}
	/**
	 * Add a prefix to a field name, taking style guide into account
	 *
	 * @param {string} prefix
	 * @param {string} fieldName
	 * @returns {string}
	 */


	function prefixFieldName(prefix, fieldName) {
	  if (prefix === undefined || prefix === "") {
	    return fieldName;
	  }

	  return prefix + capitalize(fieldName);
	}
	/**
	 * Forcibly copy fields from src to dst in a controlled manner.
	 *
	 * A given field in dst will always be overwitten. If this field
	 * is undefined or not present in src, the field in dst will 
	 * be explicitly set to undefined.
	 * 
	 * The intention here is to be able to reset all option fields.
	 * 
	 * Only the fields mentioned in array 'fields' will be handled.
	 *
	 * @param {object} src
	 * @param {object} dst
	 * @param {array<string>} fields array with names of fields to copy
	 * @param {string} [prefix] prefix to use for the target fields.
	 */


	function forceCopy(src, dst, fields, prefix) {
	  var srcKey;
	  var dstKey;

	  for (var i = 0; i < fields.length; ++i) {
	    srcKey = fields[i];
	    dstKey = prefixFieldName(prefix, srcKey);
	    dst[dstKey] = src[srcKey];
	  }
	}
	/**
	 * Copy fields from src to dst in a safe and controlled manner.
	 *
	 * Only the fields mentioned in array 'fields' will be copied over,
	 * and only if these are actually defined.
	 *
	 * @param {object} src
	 * @param {object} dst
	 * @param {array<string>} fields array with names of fields to copy
	 * @param {string} [prefix] prefix to use for the target fields.
	 */


	function safeCopy(src, dst, fields, prefix) {
	  var srcKey;
	  var dstKey;

	  for (var i = 0; i < fields.length; ++i) {
	    srcKey = fields[i];
	    if (src[srcKey] === undefined) continue;
	    dstKey = prefixFieldName(prefix, srcKey);
	    dst[dstKey] = src[srcKey];
	  }
	}
	/**
	 * Initialize dst with the values in src.
	 *
	 * src is the hash with the default values. 
	 * A reference DEFAULTS to this hash is stored locally for 
	 * further handling.
	 *
	 * For now, dst is assumed to be a Graph3d instance.
	 * @param {object} src
	 * @param {object} dst
	 */


	function setDefaults(src, dst) {
	  if (src === undefined || isEmpty(src)) {
	    throw new Error('No DEFAULTS passed');
	  }

	  if (dst === undefined) {
	    throw new Error('No dst passed');
	  } // Remember defaults for future reference


	  DEFAULTS = src; // Handle the defaults which can be simply copied over

	  forceCopy(src, dst, OPTIONKEYS);
	  forceCopy(src, dst, PREFIXEDOPTIONKEYS, 'default'); // Handle the more complex ('special') fields

	  setSpecialSettings(src, dst); // Following are internal fields, not part of the user settings

	  dst.margin = 10; // px

	  dst.showTooltip = false;
	  dst.onclick_callback = null;
	  dst.eye = new Point3d_1(0, 0, -1); // TODO: set eye.z about 3/4 of the width of the window?
	}
	/**
	 *
	 * @param {object} options
	 * @param {object} dst
	 */


	function setOptions(options, dst) {
	  if (options === undefined) {
	    return;
	  }

	  if (dst === undefined) {
	    throw new Error('No dst passed');
	  }

	  if (DEFAULTS === undefined || isEmpty(DEFAULTS)) {
	    throw new Error('DEFAULTS not set for module Settings');
	  } // Handle the parameters which can be simply copied over


	  safeCopy(options, dst, OPTIONKEYS);
	  safeCopy(options, dst, PREFIXEDOPTIONKEYS, 'default'); // Handle the more complex ('special') fields

	  setSpecialSettings(options, dst);
	}
	/**
	 * Special handling for certain parameters
	 *
	 * 'Special' here means: setting requires more than a simple copy
	 *
	 * @param {object} src
	 * @param {object} dst
	 */


	function setSpecialSettings(src, dst) {
	  if (src.backgroundColor !== undefined) {
	    setBackgroundColor(src.backgroundColor, dst);
	  }

	  setDataColor(src.dataColor, dst);
	  setStyle(src.style, dst);

	  if (src.surfaceColors !== undefined) {
	    console.warn('`options.surfaceColors` is deprecated and may be removed in a future ' + 'version. Please use `options.colormap` instead. Note that the `colormap` ' + 'option uses the inverse array ordering (running from vMin to vMax).');

	    if (src.colormap !== undefined) {
	      throw new Error('The `colormap` and `surfaceColors` options are mutually exclusive.');
	    }

	    if (dst.style !== 'surface') {
	      console.warn('Ignoring `surfaceColors` in graph style `' + dst.style + '` for ' + 'backward compatibility (only effective in `surface` plots).');
	    } else {
	      setSurfaceColor(src.surfaceColors, dst);
	    }
	  } else {
	    setColormap(src.colormap, dst);
	  }

	  setShowLegend(src.showLegend, dst);
	  setCameraPosition(src.cameraPosition, dst); // As special fields go, this is an easy one; just a translation of the name.
	  // Can't use this.tooltip directly, because that field exists internally

	  if (src.tooltip !== undefined) {
	    dst.showTooltip = src.tooltip;
	  }

	  if (src.onclick != undefined) {
	    dst.onclick_callback = src.onclick;
	    console.warn("`options.onclick` is deprecated and may be removed in a future version." + " Please use `Graph3d.on('click', handler)` instead.");
	  }

	  if (src.tooltipStyle !== undefined) {
	    selectiveDeepExtend(['tooltipStyle'], dst, src);
	  }
	}
	/**
	 * Set the value of setting 'showLegend'
	 *
	 * This depends on the value of the style fields, so it must be called
	 * after the style field has been initialized.
	 *
	 * @param {boolean} showLegend
	 * @param {object} dst
	 */


	function setShowLegend(showLegend, dst) {
	  if (showLegend === undefined) {
	    // If the default was auto, make a choice for this field
	    var isAutoByDefault = DEFAULTS.showLegend === undefined;

	    if (isAutoByDefault) {
	      // these styles default to having legends
	      var isLegendGraphStyle = dst.style === STYLE.DOTCOLOR || dst.style === STYLE.DOTSIZE;
	      dst.showLegend = isLegendGraphStyle;
	    }
	  } else {
	    dst.showLegend = showLegend;
	  }
	}
	/**
	 * Retrieve the style index from given styleName
	 * @param {string} styleName  Style name such as 'dot', 'grid', 'dot-line'
	 * @return {number} styleNumber Enumeration value representing the style, or -1
	 *                when not found
	 */


	function getStyleNumberByName(styleName) {
	  var number = STYLENAME[styleName];

	  if (number === undefined) {
	    return -1;
	  }

	  return number;
	}
	/**
	 * Check if given number is a valid style number.
	 *
	 * @param {string | number} style
	 * @return {boolean} true if valid, false otherwise
	 */


	function checkStyleNumber(style) {
	  var valid = false;

	  for (var n in STYLE) {
	    if (STYLE[n] === style) {
	      valid = true;
	      break;
	    }
	  }

	  return valid;
	}
	/**
	 *
	 * @param {string | number} style
	 * @param {Object} dst
	 */


	function setStyle(style, dst) {
	  if (style === undefined) {
	    return; // Nothing to do
	  }

	  var styleNumber;

	  if (typeof style === 'string') {
	    styleNumber = getStyleNumberByName(style);

	    if (styleNumber === -1) {
	      throw new Error('Style \'' + style + '\' is invalid');
	    }
	  } else {
	    // Do a pedantic check on style number value
	    if (!checkStyleNumber(style)) {
	      throw new Error('Style \'' + style + '\' is invalid');
	    }

	    styleNumber = style;
	  }

	  dst.style = styleNumber;
	}
	/**
	 * Set the background styling for the graph
	 * @param {string | {fill: string, stroke: string, strokeWidth: string}} backgroundColor
	 * @param {Object} dst
	 */


	function setBackgroundColor(backgroundColor, dst) {
	  var fill = 'white';
	  var stroke = 'gray';
	  var strokeWidth = 1;

	  if (typeof backgroundColor === 'string') {
	    fill = backgroundColor;
	    stroke = 'none';
	    strokeWidth = 0;
	  } else if (_typeof_1(backgroundColor) === 'object') {
	    if (fill$2(backgroundColor) !== undefined) fill = fill$2(backgroundColor);
	    if (backgroundColor.stroke !== undefined) stroke = backgroundColor.stroke;
	    if (backgroundColor.strokeWidth !== undefined) strokeWidth = backgroundColor.strokeWidth;
	  } else {
	    throw new Error('Unsupported type of backgroundColor');
	  }

	  dst.frame.style.backgroundColor = fill;
	  dst.frame.style.borderColor = stroke;
	  dst.frame.style.borderWidth = strokeWidth + 'px';
	  dst.frame.style.borderStyle = 'solid';
	}
	/**
	 *
	 * @param {string | Object} dataColor
	 * @param {Object} dst
	 */


	function setDataColor(dataColor, dst) {
	  if (dataColor === undefined) {
	    return; // Nothing to do
	  }

	  if (dst.dataColor === undefined) {
	    dst.dataColor = {};
	  }

	  if (typeof dataColor === 'string') {
	    dst.dataColor.fill = dataColor;
	    dst.dataColor.stroke = dataColor;
	  } else {
	    if (fill$2(dataColor)) {
	      dst.dataColor.fill = fill$2(dataColor);
	    }

	    if (dataColor.stroke) {
	      dst.dataColor.stroke = dataColor.stroke;
	    }

	    if (dataColor.strokeWidth !== undefined) {
	      dst.dataColor.strokeWidth = dataColor.strokeWidth;
	    }
	  }
	}
	/**
	 * 
	 * @param {Object | Array<string>} surfaceColors Either an object that describes the HUE, or an array of HTML hex color codes
	 * @param {Object} dst 
	 */


	function setSurfaceColor(surfaceColors, dst) {
	  if (surfaceColors === undefined || surfaceColors === true) {
	    return; // Nothing to do
	  }

	  if (surfaceColors === false) {
	    dst.surfaceColors = undefined;
	    return;
	  }

	  if (dst.surfaceColors === undefined) {
	    dst.surfaceColors = {};
	  }

	  var rgbColors;

	  if (isArray$3(surfaceColors)) {
	    rgbColors = parseColorArray(surfaceColors);
	  } else if (_typeof_1(surfaceColors) === 'object') {
	    rgbColors = parseColorObject(surfaceColors.hue);
	  } else {
	    throw new Error('Unsupported type of surfaceColors');
	  } // for some reason surfaceColors goes from vMax to vMin:


	  reverse$2(rgbColors).call(rgbColors);

	  dst.colormap = rgbColors;
	}
	/**
	 *
	 * @param {Object | Array<string>} colormap Either an object that describes the HUE, or an array of HTML hex color codes
	 * @param {Object} dst
	 */


	function setColormap(colormap, dst) {
	  if (colormap === undefined) {
	    return;
	  }

	  var rgbColors;

	  if (isArray$3(colormap)) {
	    rgbColors = parseColorArray(colormap);
	  } else if (_typeof_1(colormap) === 'object') {
	    rgbColors = parseColorObject(colormap.hue);
	  } else if (typeof colormap === 'function') {
	    rgbColors = colormap;
	  } else {
	    throw new Error('Unsupported type of colormap');
	  }

	  dst.colormap = rgbColors;
	}
	/**
	 *
	 * @param {Array} colormap
	 */


	function parseColorArray(colormap) {
	  if (colormap.length < 2) {
	    throw new Error('Colormap array length must be 2 or above.');
	  }

	  return map$5(colormap).call(colormap, function (colorCode) {
	    if (!isValidHex(colorCode)) {
	      throw new Error("Invalid hex color code supplied to colormap.");
	    }

	    return hexToRGB(colorCode);
	  });
	}
	/**
	 * Converts an object to a certain amount of hex color stops. At which point:
	 * the HTML hex color codes is converted into an RGB color object.
	 *
	 * @param {Object} hues
	 */


	function parseColorObject(hues) {
	  if (hues === undefined) {
	    throw new Error('Unsupported type of colormap');
	  }

	  if (!(hues.saturation >= 0 && hues.saturation <= 100)) {
	    throw new Error('Saturation is out of bounds. Expected range is 0-100.');
	  }

	  if (!(hues.brightness >= 0 && hues.brightness <= 100)) {
	    throw new Error('Brightness is out of bounds. Expected range is 0-100.');
	  }

	  if (!(hues.colorStops >= 2)) {
	    throw new Error('colorStops is out of bounds. Expected 2 or above.');
	  }

	  var hueStep = (hues.end - hues.start) / (hues.colorStops - 1);
	  var rgbColors = [];

	  for (var i = 0; i < hues.colorStops; ++i) {
	    var hue = (hues.start + hueStep * i) % 360 / 360;
	    rgbColors.push(HSVToRGB(hue < 0 ? hue + 1 : hue, hues.saturation / 100, hues.brightness / 100));
	  }

	  return rgbColors;
	}
	/**
	 *
	 * @param {Object} cameraPosition
	 * @param {Object} dst
	 */


	function setCameraPosition(cameraPosition, dst) {
	  var camPos = cameraPosition;

	  if (camPos === undefined) {
	    return;
	  }

	  if (dst.camera === undefined) {
	    dst.camera = new Camera();
	  }

	  dst.camera.setArmRotation(camPos.horizontal, camPos.vertical);
	  dst.camera.setArmLength(camPos.distance);
	}

	var errorFound = false;
	var allOptions;
	var printStyle = 'background: #FFeeee; color: #dd0000';
	/**
	 *  Used to validate options.
	 */

	var Validator = /*#__PURE__*/function () {
	  /**
	   * @ignore
	   */
	  function Validator() {
	    classCallCheck(this, Validator);
	  }
	  /**
	   * Main function to be called
	   * @param {Object} options
	   * @param {Object} referenceOptions
	   * @param {Object} subObject
	   * @returns {boolean}
	   * @static
	   */


	  createClass(Validator, null, [{
	    key: "validate",
	    value: function validate(options, referenceOptions, subObject) {
	      errorFound = false;
	      allOptions = referenceOptions;
	      var usedOptions = referenceOptions;

	      if (subObject !== undefined) {
	        usedOptions = referenceOptions[subObject];
	      }

	      Validator.parse(options, usedOptions, []);
	      return errorFound;
	    }
	    /**
	     * Will traverse an object recursively and check every value
	     * @param {Object} options
	     * @param {Object} referenceOptions
	     * @param {array} path    | where to look for the actual option
	     * @static
	     */

	  }, {
	    key: "parse",
	    value: function parse(options, referenceOptions, path) {
	      for (var option in options) {
	        if (options.hasOwnProperty(option)) {
	          Validator.check(option, options, referenceOptions, path);
	        }
	      }
	    }
	    /**
	     * Check every value. If the value is an object, call the parse function on that object.
	     * @param {string} option
	     * @param {Object} options
	     * @param {Object} referenceOptions
	     * @param {array} path    | where to look for the actual option
	     * @static
	     */

	  }, {
	    key: "check",
	    value: function check(option, options, referenceOptions, path) {
	      if (referenceOptions[option] === undefined && referenceOptions.__any__ === undefined) {
	        Validator.getSuggestion(option, referenceOptions, path);
	        return;
	      }

	      var referenceOption = option;
	      var is_object = true;

	      if (referenceOptions[option] === undefined && referenceOptions.__any__ !== undefined) {
	        // NOTE: This only triggers if the __any__ is in the top level of the options object.
	        //       THAT'S A REALLY BAD PLACE TO ALLOW IT!!!!
	        // TODO: Examine if needed, remove if possible
	        // __any__ is a wildcard. Any value is accepted and will be further analysed by reference.
	        referenceOption = '__any__'; // if the any-subgroup is not a predefined object in the configurator,
	        // we do not look deeper into the object.

	        is_object = Validator.getType(options[option]) === 'object';
	      }

	      var refOptionObj = referenceOptions[referenceOption];

	      if (is_object && refOptionObj.__type__ !== undefined) {
	        refOptionObj = refOptionObj.__type__;
	      }

	      Validator.checkFields(option, options, referenceOptions, referenceOption, refOptionObj, path);
	    }
	    /**
	     *
	     * @param {string}  option           | the option property
	     * @param {Object}  options          | The supplied options object
	     * @param {Object}  referenceOptions | The reference options containing all options and their allowed formats
	     * @param {string}  referenceOption  | Usually this is the same as option, except when handling an __any__ tag.
	     * @param {string}  refOptionObj     | This is the type object from the reference options
	     * @param {Array}   path             | where in the object is the option
	     * @static
	     */

	  }, {
	    key: "checkFields",
	    value: function checkFields(option, options, referenceOptions, referenceOption, refOptionObj, path) {
	      var log = function log(message) {
	        console.log('%c' + message + Validator.printLocation(path, option), printStyle);
	      };

	      var optionType = Validator.getType(options[option]);
	      var refOptionType = refOptionObj[optionType];

	      if (refOptionType !== undefined) {
	        // if the type is correct, we check if it is supposed to be one of a few select values
	        if (Validator.getType(refOptionType) === 'array' && indexOf$3(refOptionType).call(refOptionType, options[option]) === -1) {
	          log('Invalid option detected in "' + option + '".' + ' Allowed values are:' + Validator.print(refOptionType) + ' not "' + options[option] + '". ');
	          errorFound = true;
	        } else if (optionType === 'object' && referenceOption !== "__any__") {
	          path = copyAndExtendArray(path, option);
	          Validator.parse(options[option], referenceOptions[referenceOption], path);
	        }
	      } else if (refOptionObj['any'] === undefined) {
	        // type of the field is incorrect and the field cannot be any
	        log('Invalid type received for "' + option + '". Expected: ' + Validator.print(keys$6(refOptionObj)) + '. Received [' + optionType + '] "' + options[option] + '"');
	        errorFound = true;
	      }
	    }
	    /**
	     *
	     * @param {Object|boolean|number|string|Array.<number>|Date|Node|Moment|undefined|null} object
	     * @returns {string}
	     * @static
	     */

	  }, {
	    key: "getType",
	    value: function getType(object) {
	      var type = _typeof_1(object);

	      if (type === 'object') {
	        if (object === null) {
	          return 'null';
	        }

	        if (object instanceof Boolean) {
	          return 'boolean';
	        }

	        if (object instanceof Number) {
	          return 'number';
	        }

	        if (object instanceof String) {
	          return 'string';
	        }

	        if (isArray$3(object)) {
	          return 'array';
	        }

	        if (object instanceof Date) {
	          return 'date';
	        }

	        if (object.nodeType !== undefined) {
	          return 'dom';
	        }

	        if (object._isAMomentObject === true) {
	          return 'moment';
	        }

	        return 'object';
	      } else if (type === 'number') {
	        return 'number';
	      } else if (type === 'boolean') {
	        return 'boolean';
	      } else if (type === 'string') {
	        return 'string';
	      } else if (type === undefined) {
	        return 'undefined';
	      }

	      return type;
	    }
	    /**
	     * @param {string} option
	     * @param {Object} options
	     * @param {Array.<string>} path
	     * @static
	     */

	  }, {
	    key: "getSuggestion",
	    value: function getSuggestion(option, options, path) {
	      var localSearch = Validator.findInOptions(option, options, path, false);
	      var globalSearch = Validator.findInOptions(option, allOptions, [], true);
	      var localSearchThreshold = 8;
	      var globalSearchThreshold = 4;
	      var msg;

	      if (localSearch.indexMatch !== undefined) {
	        msg = ' in ' + Validator.printLocation(localSearch.path, option, '') + 'Perhaps it was incomplete? Did you mean: "' + localSearch.indexMatch + '"?\n\n';
	      } else if (globalSearch.distance <= globalSearchThreshold && localSearch.distance > globalSearch.distance) {
	        msg = ' in ' + Validator.printLocation(localSearch.path, option, '') + 'Perhaps it was misplaced? Matching option found at: ' + Validator.printLocation(globalSearch.path, globalSearch.closestMatch, '');
	      } else if (localSearch.distance <= localSearchThreshold) {
	        msg = '. Did you mean "' + localSearch.closestMatch + '"?' + Validator.printLocation(localSearch.path, option);
	      } else {
	        msg = '. Did you mean one of these: ' + Validator.print(keys$6(options)) + Validator.printLocation(path, option);
	      }

	      console.log('%cUnknown option detected: "' + option + '"' + msg, printStyle);
	      errorFound = true;
	    }
	    /**
	     * traverse the options in search for a match.
	     * @param {string} option
	     * @param {Object} options
	     * @param {Array} path    | where to look for the actual option
	     * @param {boolean} [recursive=false]
	     * @returns {{closestMatch: string, path: Array, distance: number}}
	     * @static
	     */

	  }, {
	    key: "findInOptions",
	    value: function findInOptions(option, options, path) {
	      var recursive = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
	      var min = 1e9;
	      var closestMatch = '';
	      var closestMatchPath = [];
	      var lowerCaseOption = option.toLowerCase();
	      var indexMatch = undefined;

	      for (var op in options) {
	        // eslint-disable-line guard-for-in
	        var distance = void 0;

	        if (options[op].__type__ !== undefined && recursive === true) {
	          var result = Validator.findInOptions(option, options[op], copyAndExtendArray(path, op));

	          if (min > result.distance) {
	            closestMatch = result.closestMatch;
	            closestMatchPath = result.path;
	            min = result.distance;
	            indexMatch = result.indexMatch;
	          }
	        } else {
	          var _context;

	          if (indexOf$3(_context = op.toLowerCase()).call(_context, lowerCaseOption) !== -1) {
	            indexMatch = op;
	          }

	          distance = Validator.levenshteinDistance(option, op);

	          if (min > distance) {
	            closestMatch = op;
	            closestMatchPath = copyArray(path);
	            min = distance;
	          }
	        }
	      }

	      return {
	        closestMatch: closestMatch,
	        path: closestMatchPath,
	        distance: min,
	        indexMatch: indexMatch
	      };
	    }
	    /**
	     * @param {Array.<string>} path
	     * @param {Object} option
	     * @param {string} prefix
	     * @returns {String}
	     * @static
	     */

	  }, {
	    key: "printLocation",
	    value: function printLocation(path, option) {
	      var prefix = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'Problem value found at: \n';
	      var str = '\n\n' + prefix + 'options = {\n';

	      for (var i = 0; i < path.length; i++) {
	        for (var j = 0; j < i + 1; j++) {
	          str += '  ';
	        }

	        str += path[i] + ': {\n';
	      }

	      for (var _j = 0; _j < path.length + 1; _j++) {
	        str += '  ';
	      }

	      str += option + '\n';

	      for (var _i = 0; _i < path.length + 1; _i++) {
	        for (var _j2 = 0; _j2 < path.length - _i; _j2++) {
	          str += '  ';
	        }

	        str += '}\n';
	      }

	      return str + '\n\n';
	    }
	    /**
	     * @param {Object} options
	     * @returns {String}
	     * @static
	     */

	  }, {
	    key: "print",
	    value: function print(options) {
	      return stringify$2(options).replace(/(\")|(\[)|(\])|(,"__type__")/g, "").replace(/(\,)/g, ', ');
	    }
	    /**
	     *  Compute the edit distance between the two given strings
	     * http://en.wikibooks.org/wiki/Algorithm_Implementation/Strings/Levenshtein_distance#JavaScript
	     *
	     * Copyright (c) 2011 Andrei Mackenzie
	     *
	     * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
	     *
	     * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
	     *
	     * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
	     *
	     * @param {string} a
	     * @param {string} b
	     * @returns {Array.<Array.<number>>}}
	     * @static
	     */

	  }, {
	    key: "levenshteinDistance",
	    value: function levenshteinDistance(a, b) {
	      if (a.length === 0) return b.length;
	      if (b.length === 0) return a.length;
	      var matrix = []; // increment along the first column of each row

	      var i;

	      for (i = 0; i <= b.length; i++) {
	        matrix[i] = [i];
	      } // increment each column in the first row


	      var j;

	      for (j = 0; j <= a.length; j++) {
	        matrix[0][j] = j;
	      } // Fill in the rest of the matrix


	      for (i = 1; i <= b.length; i++) {
	        for (j = 1; j <= a.length; j++) {
	          if (b.charAt(i - 1) == a.charAt(j - 1)) {
	            matrix[i][j] = matrix[i - 1][j - 1];
	          } else {
	            matrix[i][j] = Math.min(matrix[i - 1][j - 1] + 1, // substitution
	            Math.min(matrix[i][j - 1] + 1, // insertion
	            matrix[i - 1][j] + 1)); // deletion
	          }
	        }
	      }

	      return matrix[b.length][a.length];
	    }
	  }]);

	  return Validator;
	}();
	/**
	 * This object contains all possible options. It will check if the types are correct, if required if the option is one
	 * of the allowed values.
	 *
	 * __any__ means that the name of the property does not matter.
	 * __type__ is a required field for all objects and contains the allowed types of all objects
	 */


	var string = 'string';
	var bool = 'boolean';
	var number = 'number';
	var object = 'object'; // should only be in a __type__ property

	var array = 'array'; // Following not used here, but useful for reference
	//let dom      = 'dom';
	//let any      = 'any';

	var colorOptions = {
	  fill: {
	    string: string
	  },
	  stroke: {
	    string: string
	  },
	  strokeWidth: {
	    number: number
	  },
	  __type__: {
	    string: string,
	    object: object,
	    'undefined': 'undefined'
	  }
	};
	var surfaceColorsOptions = {
	  hue: {
	    start: {
	      number: number
	    },
	    end: {
	      number: number
	    },
	    saturation: {
	      number: number
	    },
	    brightness: {
	      number: number
	    },
	    colorStops: {
	      number: number
	    },
	    __type__: {
	      object: object
	    }
	  },
	  __type__: {
	    boolean: bool,
	    array: array,
	    object: object,
	    'undefined': 'undefined'
	  }
	};
	var colormapOptions = {
	  hue: {
	    start: {
	      number: number
	    },
	    end: {
	      number: number
	    },
	    saturation: {
	      number: number
	    },
	    brightness: {
	      number: number
	    },
	    colorStops: {
	      number: number
	    },
	    __type__: {
	      object: object
	    }
	  },
	  __type__: {
	    array: array,
	    object: object,
	    'function': 'function',
	    'undefined': 'undefined'
	  }
	};
	/**
	 * Order attempted to be alphabetical.
	 *   - x/y/z-prefixes ignored in sorting
	 *   - __type__ always at end
	 *   - globals at end
	 */

	var allOptions$1 = {
	  animationAutoStart: {
	    boolean: bool,
	    'undefined': 'undefined'
	  },
	  animationInterval: {
	    number: number
	  },
	  animationPreload: {
	    boolean: bool
	  },
	  axisColor: {
	    string: string
	  },
	  axisFontSize: {
	    number: number
	  },
	  axisFontType: {
	    string: string
	  },
	  backgroundColor: colorOptions,
	  xBarWidth: {
	    number: number,
	    'undefined': 'undefined'
	  },
	  yBarWidth: {
	    number: number,
	    'undefined': 'undefined'
	  },
	  cameraPosition: {
	    distance: {
	      number: number
	    },
	    horizontal: {
	      number: number
	    },
	    vertical: {
	      number: number
	    },
	    __type__: {
	      object: object
	    }
	  },
	  zoomable: {
	    boolean: bool
	  },
	  ctrlToZoom: {
	    boolean: bool
	  },
	  xCenter: {
	    string: string
	  },
	  yCenter: {
	    string: string
	  },
	  colormap: colormapOptions,
	  dataColor: colorOptions,
	  dotSizeMinFraction: {
	    number: number
	  },
	  dotSizeMaxFraction: {
	    number: number
	  },
	  dotSizeRatio: {
	    number: number
	  },
	  filterLabel: {
	    string: string
	  },
	  gridColor: {
	    string: string
	  },
	  onclick: {
	    'function': 'function'
	  },
	  keepAspectRatio: {
	    boolean: bool
	  },
	  xLabel: {
	    string: string
	  },
	  yLabel: {
	    string: string
	  },
	  zLabel: {
	    string: string
	  },
	  legendLabel: {
	    string: string
	  },
	  xMin: {
	    number: number,
	    'undefined': 'undefined'
	  },
	  yMin: {
	    number: number,
	    'undefined': 'undefined'
	  },
	  zMin: {
	    number: number,
	    'undefined': 'undefined'
	  },
	  xMax: {
	    number: number,
	    'undefined': 'undefined'
	  },
	  yMax: {
	    number: number,
	    'undefined': 'undefined'
	  },
	  zMax: {
	    number: number,
	    'undefined': 'undefined'
	  },
	  showAnimationControls: {
	    boolean: bool,
	    'undefined': 'undefined'
	  },
	  showGrayBottom: {
	    boolean: bool
	  },
	  showGrid: {
	    boolean: bool
	  },
	  showLegend: {
	    boolean: bool,
	    'undefined': 'undefined'
	  },
	  showPerspective: {
	    boolean: bool
	  },
	  showShadow: {
	    boolean: bool
	  },
	  showSurfaceGrid: {
	    boolean: bool
	  },
	  showXAxis: {
	    boolean: bool
	  },
	  showYAxis: {
	    boolean: bool
	  },
	  showZAxis: {
	    boolean: bool
	  },
	  rotateAxisLabels: {
	    boolean: bool
	  },
	  surfaceColors: surfaceColorsOptions,
	  xStep: {
	    number: number,
	    'undefined': 'undefined'
	  },
	  yStep: {
	    number: number,
	    'undefined': 'undefined'
	  },
	  zStep: {
	    number: number,
	    'undefined': 'undefined'
	  },
	  style: {
	    number: number,
	    // TODO: either Graph3d.DEFAULT has string, or number allowed in documentation
	    string: ['bar', 'bar-color', 'bar-size', 'dot', 'dot-line', 'dot-color', 'dot-size', 'line', 'grid', 'surface']
	  },
	  tooltip: {
	    boolean: bool,
	    'function': 'function'
	  },
	  tooltipDelay: {
	    number: number
	  },
	  tooltipStyle: {
	    content: {
	      color: {
	        string: string
	      },
	      background: {
	        string: string
	      },
	      border: {
	        string: string
	      },
	      borderRadius: {
	        string: string
	      },
	      boxShadow: {
	        string: string
	      },
	      padding: {
	        string: string
	      },
	      __type__: {
	        object: object
	      }
	    },
	    line: {
	      borderLeft: {
	        string: string
	      },
	      height: {
	        string: string
	      },
	      width: {
	        string: string
	      },
	      pointerEvents: {
	        string: string
	      },
	      __type__: {
	        object: object
	      }
	    },
	    dot: {
	      border: {
	        string: string
	      },
	      borderRadius: {
	        string: string
	      },
	      height: {
	        string: string
	      },
	      width: {
	        string: string
	      },
	      pointerEvents: {
	        string: string
	      },
	      __type__: {
	        object: object
	      }
	    },
	    __type__: {
	      object: object
	    }
	  },
	  xValueLabel: {
	    'function': 'function'
	  },
	  yValueLabel: {
	    'function': 'function'
	  },
	  zValueLabel: {
	    'function': 'function'
	  },
	  valueMax: {
	    number: number,
	    'undefined': 'undefined'
	  },
	  valueMin: {
	    number: number,
	    'undefined': 'undefined'
	  },
	  verticalRatio: {
	    number: number
	  },
	  //globals :
	  height: {
	    string: string
	  },
	  width: {
	    string: string
	  },
	  __type__: {
	    object: object
	  }
	};
	/**
	 * @prototype Range
	 *
	 * Helper class to make working with related min and max values easier.
	 *
	 * The range is inclusive; a given value is considered part of the range if:
	 *
	 *    this.min <= value <= this.max
	 */

	function Range() {
	  this.min = undefined;
	  this.max = undefined;
	}
	/**
	 * Adjust the range so that the passed value fits in it.
	 *
	 * If the value is outside of the current extremes, adjust
	 * the min or max so that the value is within the range.
	 *
	 * @param {number} value Numeric value to fit in range
	 */


	Range.prototype.adjust = function (value) {
	  if (value === undefined) return;

	  if (this.min === undefined || this.min > value) {
	    this.min = value;
	  }

	  if (this.max === undefined || this.max < value) {
	    this.max = value;
	  }
	};
	/**
	 * Adjust the current range so that the passed range fits in it.
	 *
	 * @param {Range} range Range instance to fit in current instance
	 */


	Range.prototype.combine = function (range) {
	  this.add(range.min);
	  this.add(range.max);
	};
	/**
	 * Expand the range by the given value
	 *
	 * min will be lowered by given value;
	 * max will be raised by given value
	 *
	 * Shrinking by passing a negative value is allowed.
	 *
	 * @param {number} val Amount by which to expand or shrink current range with
	 */


	Range.prototype.expand = function (val) {
	  if (val === undefined) {
	    return;
	  }

	  var newMin = this.min - val;
	  var newMax = this.max + val; // Note that following allows newMin === newMax.
	  // This should be OK, since method expand() allows this also.

	  if (newMin > newMax) {
	    throw new Error('Passed expansion value makes range invalid');
	  }

	  this.min = newMin;
	  this.max = newMax;
	};
	/**
	 * Determine the full range width of current instance.
	 *
	 * @returns {num} The calculated width of this range
	 */


	Range.prototype.range = function () {
	  return this.max - this.min;
	};
	/**
	 * Determine the central point of current instance.
	 *
	 * @returns {number} the value in the middle of min and max
	 */


	Range.prototype.center = function () {
	  return (this.min + this.max) / 2;
	};

	var Range_1 = Range;
	/**
	 * @class Filter
	 *
	 * @param {DataGroup} dataGroup the data group 
	 * @param {number}  column             The index of the column to be filtered
	 * @param {Graph3d} graph              The graph
	 */

	function Filter(dataGroup, column, graph) {
	  this.dataGroup = dataGroup;
	  this.column = column;
	  this.graph = graph; // the parent graph

	  this.index = undefined;
	  this.value = undefined; // read all distinct values and select the first one

	  this.values = dataGroup.getDistinctValues(this.column);

	  if (values$2(this).length > 0) {
	    this.selectValue(0);
	  } // create an array with the filtered datapoints. this will be loaded afterwards


	  this.dataPoints = [];
	  this.loaded = false;
	  this.onLoadCallback = undefined;

	  if (graph.animationPreload) {
	    this.loaded = false;
	    this.loadInBackground();
	  } else {
	    this.loaded = true;
	  }
	}
	/**
	 * Return the label
	 * @return {string} label
	 */


	Filter.prototype.isLoaded = function () {
	  return this.loaded;
	};
	/**
	 * Return the loaded progress
	 * @return {number} percentage between 0 and 100
	 */


	Filter.prototype.getLoadedProgress = function () {
	  var len = values$2(this).length;

	  var i = 0;

	  while (this.dataPoints[i]) {
	    i++;
	  }

	  return Math.round(i / len * 100);
	};
	/**
	 * Return the label
	 * @return {string} label
	 */


	Filter.prototype.getLabel = function () {
	  return this.graph.filterLabel;
	};
	/**
	 * Return the columnIndex of the filter
	 * @return {number} columnIndex
	 */


	Filter.prototype.getColumn = function () {
	  return this.column;
	};
	/**
	 * Return the currently selected value. Returns undefined if there is no selection
	 * @return {*} value
	 */


	Filter.prototype.getSelectedValue = function () {
	  if (this.index === undefined) return undefined;
	  return values$2(this)[this.index];
	};
	/**
	 * Retrieve all values of the filter
	 * @return {Array} values
	 */


	Filter.prototype.getValues = function () {
	  return values$2(this);
	};
	/**
	 * Retrieve one value of the filter
	 * @param {number}  index
	 * @return {*} value
	 */


	Filter.prototype.getValue = function (index) {
	  if (index >= values$2(this).length) throw new Error('Index out of range');
	  return values$2(this)[index];
	};
	/**
	 * Retrieve the (filtered) dataPoints for the currently selected filter index
	 * @param {number} [index] (optional)
	 * @return {Array} dataPoints
	 */


	Filter.prototype._getDataPoints = function (index) {
	  if (index === undefined) index = this.index;
	  if (index === undefined) return [];
	  var dataPoints;

	  if (this.dataPoints[index]) {
	    dataPoints = this.dataPoints[index];
	  } else {
	    var f = {};
	    f.column = this.column;
	    f.value = values$2(this)[index];
	    var dataView = new DataView(this.dataGroup.getDataSet(), {
	      filter: function filter(item) {
	        return item[f.column] == f.value;
	      }
	    }).get();
	    dataPoints = this.dataGroup._getDataPoints(dataView);
	    this.dataPoints[index] = dataPoints;
	  }

	  return dataPoints;
	};
	/**
	 * Set a callback function when the filter is fully loaded.
	 *
	 * @param {function} callback
	 */


	Filter.prototype.setOnLoadCallback = function (callback) {
	  this.onLoadCallback = callback;
	};
	/**
	 * Add a value to the list with available values for this filter
	 * No double entries will be created.
	 * @param {number} index
	 */


	Filter.prototype.selectValue = function (index) {
	  if (index >= values$2(this).length) throw new Error('Index out of range');
	  this.index = index;
	  this.value = values$2(this)[index];
	};
	/**
	 * Load all filtered rows in the background one by one
	 * Start this method without providing an index!
	 *
	 * @param {number} [index=0]
	 */


	Filter.prototype.loadInBackground = function (index) {
	  if (index === undefined) index = 0;
	  var frame = this.graph.frame;

	  if (index < values$2(this).length) {
	    // create a progress box
	    if (frame.progress === undefined) {
	      frame.progress = document.createElement('DIV');
	      frame.progress.style.position = 'absolute';
	      frame.progress.style.color = 'gray';
	      frame.appendChild(frame.progress);
	    }

	    var progress = this.getLoadedProgress();
	    frame.progress.innerHTML = 'Loading animation... ' + progress + '%'; // TODO: this is no nice solution...

	    frame.progress.style.bottom = 60 + 'px'; // TODO: use height of slider

	    frame.progress.style.left = 10 + 'px';
	    var me = this;

	    setTimeout$2(function () {
	      me.loadInBackground(index + 1);
	    }, 10);

	    this.loaded = false;
	  } else {
	    this.loaded = true; // remove the progress box

	    if (frame.progress !== undefined) {
	      frame.removeChild(frame.progress);
	      frame.progress = undefined;
	    }

	    if (this.onLoadCallback) this.onLoadCallback();
	  }
	};
	/**
	 * Creates a container for all data of one specific 3D-graph.
	 *
	 * On construction, the container is totally empty; the data
	 * needs to be initialized with method initializeData().
	 * Failure to do so will result in the following exception begin thrown
	 * on instantiation of Graph3D:
	 *
	 *     Error: Array, DataSet, or DataView expected
	 *
	 * @constructor DataGroup
	 */


	function DataGroup() {
	  this.dataTable = null; // The original data table
	}
	/**
	 * Initializes the instance from the passed data.
	 *
	 * Calculates minimum and maximum values and column index values.
	 *
	 * The graph3d instance is used internally to access the settings for
	 * the given instance.
	 * TODO: Pass settings only instead.
	 *
	 * @param {vis.Graph3d}  graph3d Reference to the calling Graph3D instance.
	 * @param {Array | DataSet | DataView} rawData The data containing the items for
	 *                                             the Graph.
	 * @param {number}   style   Style Number
	 * @returns {Array.<Object>}
	 */


	DataGroup.prototype.initializeData = function (graph3d, rawData, style) {
	  if (rawData === undefined) return;

	  if (isArray$3(rawData)) {
	    rawData = new DataSet(rawData);
	  }

	  var data;

	  if (rawData instanceof DataSet || rawData instanceof DataView) {
	    data = rawData.get();
	  } else {
	    throw new Error('Array, DataSet, or DataView expected');
	  }

	  if (data.length == 0) return;
	  this.style = style; // unsubscribe from the dataTable

	  if (this.dataSet) {
	    this.dataSet.off('*', this._onChange);
	  }

	  this.dataSet = rawData;
	  this.dataTable = data; // subscribe to changes in the dataset

	  var me = this;

	  this._onChange = function () {
	    graph3d.setData(me.dataSet);
	  };

	  this.dataSet.on('*', this._onChange); // determine the location of x,y,z,value,filter columns

	  this.colX = 'x';
	  this.colY = 'y';
	  this.colZ = 'z';
	  var withBars = graph3d.hasBars(style); // determine barWidth from data

	  if (withBars) {
	    if (graph3d.defaultXBarWidth !== undefined) {
	      this.xBarWidth = graph3d.defaultXBarWidth;
	    } else {
	      this.xBarWidth = this.getSmallestDifference(data, this.colX) || 1;
	    }

	    if (graph3d.defaultYBarWidth !== undefined) {
	      this.yBarWidth = graph3d.defaultYBarWidth;
	    } else {
	      this.yBarWidth = this.getSmallestDifference(data, this.colY) || 1;
	    }
	  } // calculate minima and maxima


	  this._initializeRange(data, this.colX, graph3d, withBars);

	  this._initializeRange(data, this.colY, graph3d, withBars);

	  this._initializeRange(data, this.colZ, graph3d, false);

	  if (data[0].hasOwnProperty('style')) {
	    this.colValue = 'style';
	    var valueRange = this.getColumnRange(data, this.colValue);

	    this._setRangeDefaults(valueRange, graph3d.defaultValueMin, graph3d.defaultValueMax);

	    this.valueRange = valueRange;
	  } else {
	    this.colValue = 'z';
	    this.valueRange = this.zRange;
	  } // Initialize data filter if a filter column is provided


	  var table = this.getDataTable();

	  if (table[0].hasOwnProperty('filter')) {
	    if (this.dataFilter === undefined) {
	      this.dataFilter = new Filter(this, 'filter', graph3d);
	      this.dataFilter.setOnLoadCallback(function () {
	        graph3d.redraw();
	      });
	    }
	  }

	  var dataPoints;

	  if (this.dataFilter) {
	    // apply filtering
	    dataPoints = this.dataFilter._getDataPoints();
	  } else {
	    // no filtering. load all data
	    dataPoints = this._getDataPoints(this.getDataTable());
	  }

	  return dataPoints;
	};
	/**
	 * Collect the range settings for the given data column.
	 *
	 * This internal method is intended to make the range 
	 * initalization more generic.
	 *
	 * TODO: if/when combined settings per axis defined, get rid of this.
	 *
	 * @private
	 *
	 * @param {'x'|'y'|'z'} column  The data column to process
	 * @param {vis.Graph3d} graph3d Reference to the calling Graph3D instance;
	 *                              required for access to settings
	 * @returns {Object}
	 */


	DataGroup.prototype._collectRangeSettings = function (column, graph3d) {
	  var _context2;

	  var index = indexOf$3(_context2 = ['x', 'y', 'z']).call(_context2, column);

	  if (index == -1) {
	    throw new Error('Column \'' + column + '\' invalid');
	  }

	  var upper = column.toUpperCase();
	  return {
	    barWidth: this[column + 'BarWidth'],
	    min: graph3d['default' + upper + 'Min'],
	    max: graph3d['default' + upper + 'Max'],
	    step: graph3d['default' + upper + 'Step'],
	    range_label: column + 'Range',
	    // Name of instance field to write to
	    step_label: column + 'Step' // Name of instance field to write to

	  };
	};
	/**
	 * Initializes the settings per given column.
	 *
	 * TODO: if/when combined settings per axis defined, rewrite this.
	 *
	 * @private
	 *
	 * @param {DataSet | DataView} data     The data containing the items for the Graph
	 * @param {'x'|'y'|'z'}        column   The data column to process
	 * @param {vis.Graph3d}        graph3d  Reference to the calling Graph3D instance;
	 *                                      required for access to settings
	 * @param {boolean}            withBars True if initializing for bar graph
	 */


	DataGroup.prototype._initializeRange = function (data, column, graph3d, withBars) {
	  var NUMSTEPS = 5;

	  var settings = this._collectRangeSettings(column, graph3d);

	  var range = this.getColumnRange(data, column);

	  if (withBars && column != 'z') {
	    // Safeguard for 'z'; it doesn't have a bar width
	    range.expand(settings.barWidth / 2);
	  }

	  this._setRangeDefaults(range, settings.min, settings.max);

	  this[settings.range_label] = range;
	  this[settings.step_label] = settings.step !== undefined ? settings.step : range.range() / NUMSTEPS;
	};
	/**
	 * Creates a list with all the different values in the data for the given column.
	 *
	 * If no data passed, use the internal data of this instance.
	 *
	 * @param {'x'|'y'|'z'}                column The data column to process
	 * @param {DataSet|DataView|undefined} data   The data containing the items for the Graph
	 *
	 * @returns {Array} All distinct values in the given column data, sorted ascending.
	 */


	DataGroup.prototype.getDistinctValues = function (column, data) {
	  if (data === undefined) {
	    data = this.dataTable;
	  }

	  var values = [];

	  for (var i = 0; i < data.length; i++) {
	    var value = data[i][column] || 0;

	    if (indexOf$3(values).call(values, value) === -1) {
	      values.push(value);
	    }
	  }

	  return sort$2(values).call(values, function (a, b) {
	    return a - b;
	  });
	};
	/**
	 * Determine the smallest difference between the values for given
	 * column in the passed data set.
	 *
	 * @param {DataSet|DataView|undefined} data   The data containing the items for the Graph
	 * @param {'x'|'y'|'z'}                column The data column to process
	 *
	 * @returns {number|null} Smallest difference value or
	 *                        null, if it can't be determined.
	 */


	DataGroup.prototype.getSmallestDifference = function (data, column) {
	  var values = this.getDistinctValues(data, column); // Get all the distinct diffs
	  // Array values is assumed to be sorted here

	  var smallest_diff = null;

	  for (var i = 1; i < values.length; i++) {
	    var diff = values[i] - values[i - 1];

	    if (smallest_diff == null || smallest_diff > diff) {
	      smallest_diff = diff;
	    }
	  }

	  return smallest_diff;
	};
	/**
	 * Get the absolute min/max values for the passed data column.
	 *
	 * @param {DataSet|DataView|undefined} data   The data containing the items for the Graph
	 * @param {'x'|'y'|'z'}                column The data column to process
	 *
	 * @returns {Range} A Range instance with min/max members properly set.
	 */


	DataGroup.prototype.getColumnRange = function (data, column) {
	  var range = new Range_1(); // Adjust the range so that it covers all values in the passed data elements.

	  for (var i = 0; i < data.length; i++) {
	    var item = data[i][column];
	    range.adjust(item);
	  }

	  return range;
	};
	/**
	 * Determines the number of rows in the current data.
	 *
	 * @returns {number}
	 */


	DataGroup.prototype.getNumberOfRows = function () {
	  return this.dataTable.length;
	};
	/**
	 * Set default values for range
	 *
	 * The default values override the range values, if defined.
	 *
	 * Because it's possible that only defaultMin or defaultMax is set, it's better
	 * to pass in a range already set with the min/max set from the data. Otherwise,
	 * it's quite hard to process the min/max properly.
	 *
	 * @param {vis.Range} range
	 * @param {number} [defaultMin=range.min]
	 * @param {number} [defaultMax=range.max]
	 * @private
	 */


	DataGroup.prototype._setRangeDefaults = function (range, defaultMin, defaultMax) {
	  if (defaultMin !== undefined) {
	    range.min = defaultMin;
	  }

	  if (defaultMax !== undefined) {
	    range.max = defaultMax;
	  } // This is the original way that the default min/max values were adjusted.
	  // TODO: Perhaps it's better if an error is thrown if the values do not agree.
	  //       But this will change the behaviour.


	  if (range.max <= range.min) range.max = range.min + 1;
	};

	DataGroup.prototype.getDataTable = function () {
	  return this.dataTable;
	};

	DataGroup.prototype.getDataSet = function () {
	  return this.dataSet;
	};
	/**
	 * Return all data values as a list of Point3d objects
	 * @param {Array.<Object>} data
	 * @returns {Array.<Object>}
	 */


	DataGroup.prototype.getDataPoints = function (data) {
	  var dataPoints = [];

	  for (var i = 0; i < data.length; i++) {
	    var point = new Point3d_1();
	    point.x = data[i][this.colX] || 0;
	    point.y = data[i][this.colY] || 0;
	    point.z = data[i][this.colZ] || 0;
	    point.data = data[i];
	    point.value = data[i][this.colValue] || 0;
	    var obj = {};
	    obj.point = point;
	    obj.bottom = new Point3d_1(point.x, point.y, this.zRange.min);
	    obj.trans = undefined;
	    obj.screen = undefined;
	    dataPoints.push(obj);
	  }

	  return dataPoints;
	};
	/**
	 * Copy all values from the data table to a matrix.
	 *
	 * The provided values are supposed to form a grid of (x,y) positions.
	 * @param {Array.<Object>} data
	 * @returns {Array.<Object>}
	 * @private
	 */


	DataGroup.prototype.initDataAsMatrix = function (data) {
	  // TODO: store the created matrix dataPoints in the filters instead of
	  //       reloading each time.
	  var x, y, i, obj; // create two lists with all present x and y values

	  var dataX = this.getDistinctValues(this.colX, data);
	  var dataY = this.getDistinctValues(this.colY, data);
	  var dataPoints = this.getDataPoints(data); // create a grid, a 2d matrix, with all values.

	  var dataMatrix = []; // temporary data matrix

	  for (i = 0; i < dataPoints.length; i++) {
	    obj = dataPoints[i]; // TODO: implement Array().indexOf() for Internet Explorer

	    var xIndex = indexOf$3(dataX).call(dataX, obj.point.x);

	    var yIndex = indexOf$3(dataY).call(dataY, obj.point.y);

	    if (dataMatrix[xIndex] === undefined) {
	      dataMatrix[xIndex] = [];
	    }

	    dataMatrix[xIndex][yIndex] = obj;
	  } // fill in the pointers to the neighbors.


	  for (x = 0; x < dataMatrix.length; x++) {
	    for (y = 0; y < dataMatrix[x].length; y++) {
	      if (dataMatrix[x][y]) {
	        dataMatrix[x][y].pointRight = x < dataMatrix.length - 1 ? dataMatrix[x + 1][y] : undefined;
	        dataMatrix[x][y].pointTop = y < dataMatrix[x].length - 1 ? dataMatrix[x][y + 1] : undefined;
	        dataMatrix[x][y].pointCross = x < dataMatrix.length - 1 && y < dataMatrix[x].length - 1 ? dataMatrix[x + 1][y + 1] : undefined;
	      }
	    }
	  }

	  return dataPoints;
	};
	/**
	 * Return common information, if present
	 *
	 * @returns {string}
	 */


	DataGroup.prototype.getInfo = function () {
	  var dataFilter = this.dataFilter;
	  if (!dataFilter) return undefined;
	  return dataFilter.getLabel() + ': ' + dataFilter.getSelectedValue();
	};
	/**
	 * Reload the data
	 */


	DataGroup.prototype.reload = function () {
	  if (this.dataTable) {
	    this.setData(this.dataTable);
	  }
	};
	/**
	 * Filter the data based on the current filter
	 *
	 * @param   {Array} data
	 * @returns {Array} dataPoints Array with point objects which can be drawn on
	 *                             screen
	 */


	DataGroup.prototype._getDataPoints = function (data) {
	  var dataPoints = [];

	  if (this.style === STYLE.GRID || this.style === STYLE.SURFACE) {
	    dataPoints = this.initDataAsMatrix(data);
	  } else {
	    // 'dot', 'dot-line', etc.
	    dataPoints = this.getDataPoints(data);

	    if (this.style === STYLE.LINE) {
	      // Add next member points for line drawing
	      for (var i = 0; i < dataPoints.length; i++) {
	        if (i > 0) {
	          dataPoints[i - 1].pointNext = dataPoints[i];
	        }
	      }
	    }
	  }

	  return dataPoints;
	}; /// enumerate the available styles


	Graph3d.STYLE = STYLE;
	/**
	 * Following label is used in the settings to describe values which should be
	 * determined by the code while running, from the current data and graph style.
	 *
	 * Using 'undefined' directly achieves the same thing, but this is more
	 * descriptive by describing the intent.
	 */

	var autoByDefault = undefined;
	/**
	 * Default values for option settings.
	 *
	 * These are the values used when a Graph3d instance is initialized without
	 * custom settings.
	 *
	 * If a field is not in this list, a default value of 'autoByDefault' is assumed,
	 * which is just an alias for 'undefined'.
	 */

	Graph3d.DEFAULTS = {
	  width: '400px',
	  height: '400px',
	  filterLabel: 'time',
	  legendLabel: 'value',
	  xLabel: 'x',
	  yLabel: 'y',
	  zLabel: 'z',
	  xValueLabel: function xValueLabel(v) {
	    return v;
	  },
	  yValueLabel: function yValueLabel(v) {
	    return v;
	  },
	  zValueLabel: function zValueLabel(v) {
	    return v;
	  },
	  showXAxis: true,
	  showYAxis: true,
	  showZAxis: true,
	  showGrayBottom: false,
	  showGrid: true,
	  showPerspective: true,
	  showShadow: false,
	  showSurfaceGrid: true,
	  keepAspectRatio: true,
	  rotateAxisLabels: true,
	  verticalRatio: 0.5,
	  // 0.1 to 1.0, where 1.0 results in a 'cube'
	  dotSizeRatio: 0.02,
	  // size of the dots as a fraction of the graph width
	  dotSizeMinFraction: 0.5,
	  // size of min-value dot as a fraction of dotSizeRatio	
	  dotSizeMaxFraction: 2.5,
	  // size of max-value dot as a fraction of dotSizeRatio	
	  showAnimationControls: autoByDefault,
	  animationInterval: 1000,
	  // milliseconds
	  animationPreload: false,
	  animationAutoStart: autoByDefault,
	  axisFontSize: 14,
	  axisFontType: 'arial',
	  axisColor: '#4D4D4D',
	  gridColor: '#D3D3D3',
	  xCenter: '55%',
	  yCenter: '50%',
	  style: Graph3d.STYLE.DOT,
	  tooltip: false,
	  tooltipDelay: 300,
	  // milliseconds
	  tooltipStyle: {
	    content: {
	      padding: '10px',
	      border: '1px solid #4d4d4d',
	      color: '#1a1a1a',
	      background: 'rgba(255,255,255,0.7)',
	      borderRadius: '2px',
	      boxShadow: '5px 5px 10px rgba(128,128,128,0.5)'
	    },
	    line: {
	      height: '40px',
	      width: '0',
	      borderLeft: '1px solid #4d4d4d',
	      pointerEvents: 'none'
	    },
	    dot: {
	      height: '0',
	      width: '0',
	      border: '5px solid #4d4d4d',
	      borderRadius: '5px',
	      pointerEvents: 'none'
	    }
	  },
	  dataColor: {
	    fill: '#7DC1FF',
	    stroke: '#3267D2',
	    strokeWidth: 1 // px

	  },
	  surfaceColors: autoByDefault,
	  colormap: autoByDefault,
	  cameraPosition: {
	    horizontal: 1.0,
	    vertical: 0.5,
	    distance: 1.7
	  },
	  zoomable: true,
	  ctrlToZoom: false,

	  /*
	    The following fields are 'auto by default', see above.
	   */
	  showLegend: autoByDefault,
	  // determined by graph style
	  backgroundColor: autoByDefault,
	  xBarWidth: autoByDefault,
	  yBarWidth: autoByDefault,
	  valueMin: autoByDefault,
	  valueMax: autoByDefault,
	  xMin: autoByDefault,
	  xMax: autoByDefault,
	  xStep: autoByDefault,
	  yMin: autoByDefault,
	  yMax: autoByDefault,
	  yStep: autoByDefault,
	  zMin: autoByDefault,
	  zMax: autoByDefault,
	  zStep: autoByDefault
	}; // -----------------------------------------------------------------------------
	// Class Graph3d
	// -----------------------------------------------------------------------------

	/**
	 * Graph3d displays data in 3d.
	 *
	 * Graph3d is developed in javascript as a Google Visualization Chart.
	 *
	 * @constructor Graph3d
	 * @param {Element} container   The DOM element in which the Graph3d will
	 *                              be created. Normally a div element.
	 * @param {DataSet | DataView | Array} [data]
	 * @param {Object} [options]
	 */

	function Graph3d(container, data, options) {
	  if (!(this instanceof Graph3d)) {
	    throw new SyntaxError('Constructor must be called with the new operator');
	  } // create variables and set default values


	  this.containerElement = container;
	  this.dataGroup = new DataGroup();
	  this.dataPoints = null; // The table with point objects
	  // create a frame and canvas

	  this.create();
	  setDefaults(Graph3d.DEFAULTS, this); // the column indexes

	  this.colX = undefined;
	  this.colY = undefined;
	  this.colZ = undefined;
	  this.colValue = undefined; // TODO: customize axis range
	  // apply options (also when undefined)

	  this.setOptions(options); // apply data

	  this.setData(data);
	} // Extend Graph3d with an Emitter mixin


	componentEmitter(Graph3d.prototype);
	/**
	 * Calculate the scaling values, dependent on the range in x, y, and z direction
	 */

	Graph3d.prototype._setScale = function () {
	  this.scale = new Point3d_1(1 / this.xRange.range(), 1 / this.yRange.range(), 1 / this.zRange.range()); // keep aspect ration between x and y scale if desired

	  if (this.keepAspectRatio) {
	    if (this.scale.x < this.scale.y) {
	      //noinspection JSSuspiciousNameCombination
	      this.scale.y = this.scale.x;
	    } else {
	      //noinspection JSSuspiciousNameCombination
	      this.scale.x = this.scale.y;
	    }
	  } // scale the vertical axis


	  this.scale.z *= this.verticalRatio; // TODO: can this be automated? verticalRatio?
	  // determine scale for (optional) value

	  if (this.valueRange !== undefined) {
	    this.scale.value = 1 / this.valueRange.range();
	  } // position the camera arm


	  var xCenter = this.xRange.center() * this.scale.x;
	  var yCenter = this.yRange.center() * this.scale.y;
	  var zCenter = this.zRange.center() * this.scale.z;
	  this.camera.setArmLocation(xCenter, yCenter, zCenter);
	};
	/**
	 * Convert a 3D location to a 2D location on screen
	 * Source: ttp://en.wikipedia.org/wiki/3D_projection
	 *
	 * @param   {Point3d} point3d  A 3D point with parameters x, y, z
	 * @returns {Point2d} point2d  A 2D point with parameters x, y
	 */


	Graph3d.prototype._convert3Dto2D = function (point3d) {
	  var translation = this._convertPointToTranslation(point3d);

	  return this._convertTranslationToScreen(translation);
	};
	/**
	 * Convert a 3D location its translation seen from the camera
	 * Source: http://en.wikipedia.org/wiki/3D_projection
	 *
	 * @param   {Point3d} point3d     A 3D point with parameters x, y, z
	 * @returns {Point3d} translation A 3D point with parameters x, y, z This is
	 *                                the translation of the point, seen from the
	 *                                camera.
	 */


	Graph3d.prototype._convertPointToTranslation = function (point3d) {
	  var cameraLocation = this.camera.getCameraLocation(),
	      cameraRotation = this.camera.getCameraRotation(),
	      ax = point3d.x * this.scale.x,
	      ay = point3d.y * this.scale.y,
	      az = point3d.z * this.scale.z,
	      cx = cameraLocation.x,
	      cy = cameraLocation.y,
	      cz = cameraLocation.z,
	      // calculate angles
	  sinTx = Math.sin(cameraRotation.x),
	      cosTx = Math.cos(cameraRotation.x),
	      sinTy = Math.sin(cameraRotation.y),
	      cosTy = Math.cos(cameraRotation.y),
	      sinTz = Math.sin(cameraRotation.z),
	      cosTz = Math.cos(cameraRotation.z),
	      // calculate translation
	  dx = cosTy * (sinTz * (ay - cy) + cosTz * (ax - cx)) - sinTy * (az - cz),
	      dy = sinTx * (cosTy * (az - cz) + sinTy * (sinTz * (ay - cy) + cosTz * (ax - cx))) + cosTx * (cosTz * (ay - cy) - sinTz * (ax - cx)),
	      dz = cosTx * (cosTy * (az - cz) + sinTy * (sinTz * (ay - cy) + cosTz * (ax - cx))) - sinTx * (cosTz * (ay - cy) - sinTz * (ax - cx));
	  return new Point3d_1(dx, dy, dz);
	};
	/**
	 * Convert a translation point to a point on the screen
	 *
	 * @param   {Point3d} translation A 3D point with parameters x, y, z This is
	 *                                the translation of the point, seen from the
	 *                                camera.
	 * @returns {Point2d} point2d     A 2D point with parameters x, y
	 */


	Graph3d.prototype._convertTranslationToScreen = function (translation) {
	  var ex = this.eye.x,
	      ey = this.eye.y,
	      ez = this.eye.z,
	      dx = translation.x,
	      dy = translation.y,
	      dz = translation.z; // calculate position on screen from translation

	  var bx;
	  var by;

	  if (this.showPerspective) {
	    bx = (dx - ex) * (ez / dz);
	    by = (dy - ey) * (ez / dz);
	  } else {
	    bx = dx * -(ez / this.camera.getArmLength());
	    by = dy * -(ez / this.camera.getArmLength());
	  } // shift and scale the point to the center of the screen
	  // use the width of the graph to scale both horizontally and vertically.


	  return new Point2d_1(this.currentXCenter + bx * this.frame.canvas.clientWidth, this.currentYCenter - by * this.frame.canvas.clientWidth);
	};
	/**
	 * Calculate the translations and screen positions of all points
	 *
	 * @param {Array.<Point3d>} points
	 * @private
	 */


	Graph3d.prototype._calcTranslations = function (points) {
	  for (var i = 0; i < points.length; i++) {
	    var point = points[i];
	    point.trans = this._convertPointToTranslation(point.point);
	    point.screen = this._convertTranslationToScreen(point.trans); // calculate the translation of the point at the bottom (needed for sorting)

	    var transBottom = this._convertPointToTranslation(point.bottom);

	    point.dist = this.showPerspective ? transBottom.length() : -transBottom.z;
	  } // sort the points on depth of their (x,y) position (not on z)


	  var sortDepth = function sortDepth(a, b) {
	    return b.dist - a.dist;
	  };

	  sort$2(points).call(points, sortDepth);
	};
	/**
	 * Transfer min/max values to the Graph3d instance.
	 */


	Graph3d.prototype._initializeRanges = function () {
	  // TODO: later on, all min/maxes of all datagroups will be combined here
	  var dg = this.dataGroup;
	  this.xRange = dg.xRange;
	  this.yRange = dg.yRange;
	  this.zRange = dg.zRange;
	  this.valueRange = dg.valueRange; // Values currently needed but which need to be sorted out for
	  // the multiple graph case.

	  this.xStep = dg.xStep;
	  this.yStep = dg.yStep;
	  this.zStep = dg.zStep;
	  this.xBarWidth = dg.xBarWidth;
	  this.yBarWidth = dg.yBarWidth;
	  this.colX = dg.colX;
	  this.colY = dg.colY;
	  this.colZ = dg.colZ;
	  this.colValue = dg.colValue; // set the scale dependent on the ranges.

	  this._setScale();
	};
	/**
	 * Return all data values as a list of Point3d objects
	 *
	 * @param {vis.DataSet} data
	 * @returns {Array.<Object>}
	 */


	Graph3d.prototype.getDataPoints = function (data) {
	  var dataPoints = [];

	  for (var i = 0; i < data.length; i++) {
	    var point = new Point3d_1();
	    point.x = data[i][this.colX] || 0;
	    point.y = data[i][this.colY] || 0;
	    point.z = data[i][this.colZ] || 0;
	    point.data = data[i];
	    point.value = data[i][this.colValue] || 0;
	    var obj = {};
	    obj.point = point;
	    obj.bottom = new Point3d_1(point.x, point.y, this.zRange.min);
	    obj.trans = undefined;
	    obj.screen = undefined;
	    dataPoints.push(obj);
	  }

	  return dataPoints;
	};
	/**
	 * Filter the data based on the current filter
	 *
	 * @param   {Array} data
	 * @returns {Array} dataPoints Array with point objects which can be drawn on
	 *                             screen
	 */


	Graph3d.prototype._getDataPoints = function (data) {
	  // TODO: store the created matrix dataPoints in the filters instead of
	  //       reloading each time.
	  var x, y, i, obj;
	  var dataPoints = [];

	  if (this.style === Graph3d.STYLE.GRID || this.style === Graph3d.STYLE.SURFACE) {
	    // copy all values from the data table to a matrix
	    // the provided values are supposed to form a grid of (x,y) positions
	    // create two lists with all present x and y values
	    var dataX = this.dataGroup.getDistinctValues(this.colX, data);
	    var dataY = this.dataGroup.getDistinctValues(this.colY, data);
	    dataPoints = this.getDataPoints(data); // create a grid, a 2d matrix, with all values.

	    var dataMatrix = []; // temporary data matrix

	    for (i = 0; i < dataPoints.length; i++) {
	      obj = dataPoints[i]; // TODO: implement Array().indexOf() for Internet Explorer

	      var xIndex = indexOf$3(dataX).call(dataX, obj.point.x);

	      var yIndex = indexOf$3(dataY).call(dataY, obj.point.y);

	      if (dataMatrix[xIndex] === undefined) {
	        dataMatrix[xIndex] = [];
	      }

	      dataMatrix[xIndex][yIndex] = obj;
	    } // fill in the pointers to the neighbors.


	    for (x = 0; x < dataMatrix.length; x++) {
	      for (y = 0; y < dataMatrix[x].length; y++) {
	        if (dataMatrix[x][y]) {
	          dataMatrix[x][y].pointRight = x < dataMatrix.length - 1 ? dataMatrix[x + 1][y] : undefined;
	          dataMatrix[x][y].pointTop = y < dataMatrix[x].length - 1 ? dataMatrix[x][y + 1] : undefined;
	          dataMatrix[x][y].pointCross = x < dataMatrix.length - 1 && y < dataMatrix[x].length - 1 ? dataMatrix[x + 1][y + 1] : undefined;
	        }
	      }
	    }
	  } else {
	    // 'dot', 'dot-line', etc.
	    dataPoints = this.getDataPoints(data);

	    if (this.style === Graph3d.STYLE.LINE) {
	      // Add next member points for line drawing
	      for (i = 0; i < dataPoints.length; i++) {
	        if (i > 0) {
	          dataPoints[i - 1].pointNext = dataPoints[i];
	        }
	      }
	    }
	  }

	  return dataPoints;
	};
	/**
	 * Create the main frame for the Graph3d.
	 *
	 * This function is executed once when a Graph3d object is created. The frame
	 * contains a canvas, and this canvas contains all objects like the axis and
	 * nodes.
	 */


	Graph3d.prototype.create = function () {
	  // remove all elements from the container element.
	  while (this.containerElement.hasChildNodes()) {
	    this.containerElement.removeChild(this.containerElement.firstChild);
	  }

	  this.frame = document.createElement('div');
	  this.frame.style.position = 'relative';
	  this.frame.style.overflow = 'hidden'; // create the graph canvas (HTML canvas element)

	  this.frame.canvas = document.createElement('canvas');
	  this.frame.canvas.style.position = 'relative';
	  this.frame.appendChild(this.frame.canvas); //if (!this.frame.canvas.getContext) {

	  {
	    var noCanvas = document.createElement('DIV');
	    noCanvas.style.color = 'red';
	    noCanvas.style.fontWeight = 'bold';
	    noCanvas.style.padding = '10px';
	    noCanvas.innerHTML = 'Error: your browser does not support HTML canvas';
	    this.frame.canvas.appendChild(noCanvas);
	  }
	  this.frame.filter = document.createElement('div');
	  filter$2(this.frame).style.position = 'absolute';
	  filter$2(this.frame).style.bottom = '0px';
	  filter$2(this.frame).style.left = '0px';
	  filter$2(this.frame).style.width = '100%';
	  this.frame.appendChild(filter$2(this.frame)); // add event listeners to handle moving and zooming the contents

	  var me = this;

	  var onmousedown = function onmousedown(event) {
	    me._onMouseDown(event);
	  };

	  var ontouchstart = function ontouchstart(event) {
	    me._onTouchStart(event);
	  };

	  var onmousewheel = function onmousewheel(event) {
	    me._onWheel(event);
	  };

	  var ontooltip = function ontooltip(event) {
	    me._onTooltip(event);
	  };

	  var onclick = function onclick(event) {
	    me._onClick(event);
	  }; // TODO: these events are never cleaned up... can give a 'memory leakage'


	  addEventListener(this.frame.canvas, 'mousedown', onmousedown);
	  addEventListener(this.frame.canvas, 'touchstart', ontouchstart);
	  addEventListener(this.frame.canvas, 'mousewheel', onmousewheel);
	  addEventListener(this.frame.canvas, 'mousemove', ontooltip);
	  addEventListener(this.frame.canvas, 'click', onclick); // add the new graph to the container element

	  this.containerElement.appendChild(this.frame);
	};
	/**
	 * Set a new size for the graph
	 *
	 * @param {number} width
	 * @param {number} height
	 * @private
	 */


	Graph3d.prototype._setSize = function (width, height) {
	  this.frame.style.width = width;
	  this.frame.style.height = height;

	  this._resizeCanvas();
	};
	/**
	 * Resize the canvas to the current size of the frame
	 */


	Graph3d.prototype._resizeCanvas = function () {
	  this.frame.canvas.style.width = '100%';
	  this.frame.canvas.style.height = '100%';
	  this.frame.canvas.width = this.frame.canvas.clientWidth;
	  this.frame.canvas.height = this.frame.canvas.clientHeight; // adjust with for margin

	  filter$2(this.frame).style.width = this.frame.canvas.clientWidth - 2 * 10 + 'px';
	};
	/**
	 * Start playing the animation, if requested and filter present. Only applicable
	 * when animation data is available.
	 */


	Graph3d.prototype.animationStart = function () {
	  // start animation when option is true
	  if (!this.animationAutoStart || !this.dataGroup.dataFilter) return;
	  if (!filter$2(this.frame) || !filter$2(this.frame).slider) throw new Error('No animation available');

	  filter$2(this.frame).slider.play();
	};
	/**
	 * Stop animation
	 */


	Graph3d.prototype.animationStop = function () {
	  if (!filter$2(this.frame) || !filter$2(this.frame).slider) return;

	  filter$2(this.frame).slider.stop();
	};
	/**
	 * Resize the center position based on the current values in this.xCenter
	 * and this.yCenter (which are strings with a percentage or a value
	 * in pixels). The center positions are the variables this.currentXCenter
	 * and this.currentYCenter
	 */


	Graph3d.prototype._resizeCenter = function () {
	  // calculate the horizontal center position
	  if (this.xCenter.charAt(this.xCenter.length - 1) === '%') {
	    this.currentXCenter = _parseFloat$2(this.xCenter) / 100 * this.frame.canvas.clientWidth;
	  } else {
	    this.currentXCenter = _parseFloat$2(this.xCenter); // supposed to be in px
	  } // calculate the vertical center position


	  if (this.yCenter.charAt(this.yCenter.length - 1) === '%') {
	    this.currentYCenter = _parseFloat$2(this.yCenter) / 100 * (this.frame.canvas.clientHeight - filter$2(this.frame).clientHeight);
	  } else {
	    this.currentYCenter = _parseFloat$2(this.yCenter); // supposed to be in px
	  }
	};
	/**
	 * Retrieve the current camera rotation
	 *
	 * @returns {object} An object with parameters horizontal, vertical, and
	 *                   distance
	 */


	Graph3d.prototype.getCameraPosition = function () {
	  var pos = this.camera.getArmRotation();
	  pos.distance = this.camera.getArmLength();
	  return pos;
	};
	/**
	 * Load data into the 3D Graph
	 *
	 * @param {vis.DataSet} data
	 * @private
	 */


	Graph3d.prototype._readData = function (data) {
	  // read the data
	  this.dataPoints = this.dataGroup.initializeData(this, data, this.style);

	  this._initializeRanges();

	  this._redrawFilter();
	};
	/**
	 * Replace the dataset of the Graph3d
	 *
	 * @param {Array | DataSet | DataView} data
	 */


	Graph3d.prototype.setData = function (data) {
	  if (data === undefined || data === null) return;

	  this._readData(data);

	  this.redraw();
	  this.animationStart();
	};
	/**
	 * Update the options. Options will be merged with current options
	 *
	 * @param {Object} options
	 */


	Graph3d.prototype.setOptions = function (options) {
	  if (options === undefined) return;
	  var errorFound = Validator.validate(options, allOptions$1);

	  if (errorFound === true) {
	    console.log('%cErrors have been found in the supplied options object.', printStyle);
	  }

	  this.animationStop();
	  setOptions(options, this);
	  this.setPointDrawingMethod();

	  this._setSize(this.width, this.height);

	  this.setAxisLabelMethod();
	  this.setData(this.dataGroup.getDataTable());
	  this.animationStart();
	};
	/**
	 * Determine which point drawing method to use for the current graph style.
	 */


	Graph3d.prototype.setPointDrawingMethod = function () {
	  var method = undefined;

	  switch (this.style) {
	    case Graph3d.STYLE.BAR:
	      method = this._redrawBarGraphPoint;
	      break;

	    case Graph3d.STYLE.BARCOLOR:
	      method = this._redrawBarColorGraphPoint;
	      break;

	    case Graph3d.STYLE.BARSIZE:
	      method = this._redrawBarSizeGraphPoint;
	      break;

	    case Graph3d.STYLE.DOT:
	      method = this._redrawDotGraphPoint;
	      break;

	    case Graph3d.STYLE.DOTLINE:
	      method = this._redrawDotLineGraphPoint;
	      break;

	    case Graph3d.STYLE.DOTCOLOR:
	      method = this._redrawDotColorGraphPoint;
	      break;

	    case Graph3d.STYLE.DOTSIZE:
	      method = this._redrawDotSizeGraphPoint;
	      break;

	    case Graph3d.STYLE.SURFACE:
	      method = this._redrawSurfaceGraphPoint;
	      break;

	    case Graph3d.STYLE.GRID:
	      method = this._redrawGridGraphPoint;
	      break;

	    case Graph3d.STYLE.LINE:
	      method = this._redrawLineGraphPoint;
	      break;

	    default:
	      throw new Error('Can not determine point drawing method ' + 'for graph style \'' + this.style + '\'');
	  }

	  this._pointDrawingMethod = method;
	};
	/**
	 * Determine which functions to use to draw axis labels.
	 */


	Graph3d.prototype.setAxisLabelMethod = function () {
	  if (this.rotateAxisLabels) {
	    this._drawAxisLabelX = this.drawAxisLabelXRotate;
	    this._drawAxisLabelY = this.drawAxisLabelYRotate;
	    this._drawAxisLabelZ = this.drawAxisLabelZRotate;
	  } else {
	    this._drawAxisLabelX = this.drawAxisLabelX;
	    this._drawAxisLabelY = this.drawAxisLabelY;
	    this._drawAxisLabelZ = this.drawAxisLabelZ;
	  }
	};
	/**
	 * Redraw the Graph.
	 */


	Graph3d.prototype.redraw = function () {
	  if (this.dataPoints === undefined) {
	    throw new Error('Graph data not initialized');
	  }

	  this._resizeCanvas();

	  this._resizeCenter();

	  this._redrawSlider();

	  this._redrawClear();

	  this._redrawAxis();

	  this._redrawDataGraph();

	  this._redrawInfo();

	  this._redrawLegend();
	};
	/**
	 * Get drawing context without exposing canvas
	 *
	 * @returns {CanvasRenderingContext2D}
	 * @private
	 */


	Graph3d.prototype._getContext = function () {
	  var canvas = this.frame.canvas;
	  var ctx = canvas.getContext('2d');
	  ctx.lineJoin = 'round';
	  ctx.lineCap = 'round';
	  return ctx;
	};
	/**
	 * Clear the canvas before redrawing
	 */


	Graph3d.prototype._redrawClear = function () {
	  var canvas = this.frame.canvas;
	  var ctx = canvas.getContext('2d');
	  ctx.clearRect(0, 0, canvas.width, canvas.height);
	};

	Graph3d.prototype._dotSize = function () {
	  return this.frame.clientWidth * this.dotSizeRatio;
	};
	/**
	 * Get legend width
	 *
	 * @returns {*}
	 * @private
	 */


	Graph3d.prototype._getLegendWidth = function () {
	  var width;

	  if (this.style === Graph3d.STYLE.DOTSIZE) {
	    var dotSize = this._dotSize(); //width =  dotSize / 2 + dotSize * 2;


	    width = dotSize * this.dotSizeMaxFraction;
	  } else if (this.style === Graph3d.STYLE.BARSIZE) {
	    width = this.xBarWidth;
	  } else {
	    width = 20;
	  }

	  return width;
	};
	/**
	 * Redraw the legend based on size, dot color, or surface height
	 */


	Graph3d.prototype._redrawLegend = function () {
	  //Return without drawing anything, if no legend is specified
	  if (this.showLegend !== true) {
	    return;
	  } // Do not draw legend when graph style does not support


	  if (this.style === Graph3d.STYLE.LINE || this.style === Graph3d.STYLE.BARSIZE //TODO add legend support for BARSIZE
	  ) {
	      return;
	    } // Legend types - size and color. Determine if size legend.


	  var isSizeLegend = this.style === Graph3d.STYLE.BARSIZE || this.style === Graph3d.STYLE.DOTSIZE; // Legend is either tracking z values or style values. This flag if false means use z values.

	  var isValueLegend = this.style === Graph3d.STYLE.DOTSIZE || this.style === Graph3d.STYLE.DOTCOLOR || this.style === Graph3d.STYLE.SURFACE || this.style === Graph3d.STYLE.BARCOLOR;
	  var height = Math.max(this.frame.clientHeight * 0.25, 100);
	  var top = this.margin;

	  var width = this._getLegendWidth(); // px - overwritten by size legend


	  var right = this.frame.clientWidth - this.margin;
	  var left = right - width;
	  var bottom = top + height;

	  var ctx = this._getContext();

	  ctx.lineWidth = 1;
	  ctx.font = '14px arial'; // TODO: put in options

	  if (isSizeLegend === false) {
	    // draw the color bar
	    var ymin = 0;
	    var ymax = height; // Todo: make height customizable

	    var y;

	    for (y = ymin; y < ymax; y++) {
	      // Need (1 - x) because y runs from top to bottom:
	      var f = 1 - (y - ymin) / (ymax - ymin);

	      var color = this._colormap(f, 1);

	      ctx.strokeStyle = color;
	      ctx.beginPath();
	      ctx.moveTo(left, top + y);
	      ctx.lineTo(right, top + y);
	      ctx.stroke();
	    }

	    ctx.strokeStyle = this.axisColor;
	    ctx.strokeRect(left, top, width, height);
	  } else {
	    // draw the size legend box
	    var widthMin;

	    if (this.style === Graph3d.STYLE.DOTSIZE) {
	      // Get the proportion to max and min right
	      widthMin = width * (this.dotSizeMinFraction / this.dotSizeMaxFraction);
	    } else if (this.style === Graph3d.STYLE.BARSIZE) ;

	    ctx.strokeStyle = this.axisColor;
	    ctx.fillStyle = fill$2(this.dataColor);
	    ctx.beginPath();
	    ctx.moveTo(left, top);
	    ctx.lineTo(right, top);
	    ctx.lineTo(left + widthMin, bottom);
	    ctx.lineTo(left, bottom);
	    ctx.closePath();

	    fill$2(ctx).call(ctx);

	    ctx.stroke();
	  } // print value text along the legend edge


	  var gridLineLen = 5; // px

	  var legendMin = isValueLegend ? this.valueRange.min : this.zRange.min;
	  var legendMax = isValueLegend ? this.valueRange.max : this.zRange.max;
	  var step = new StepNumber_1(legendMin, legendMax, (legendMax - legendMin) / 5, true);
	  step.start(true);
	  var from;
	  var to;

	  while (!step.end()) {
	    y = bottom - (step.getCurrent() - legendMin) / (legendMax - legendMin) * height;
	    from = new Point2d_1(left - gridLineLen, y);
	    to = new Point2d_1(left, y);

	    this._line(ctx, from, to);

	    ctx.textAlign = 'right';
	    ctx.textBaseline = 'middle';
	    ctx.fillStyle = this.axisColor;
	    ctx.fillText(step.getCurrent(), left - 2 * gridLineLen, y);
	    step.next();
	  }

	  ctx.textAlign = 'right';
	  ctx.textBaseline = 'top';
	  var label = this.legendLabel;
	  ctx.fillText(label, right, bottom + this.margin);
	};
	/**
	 * Redraw the filter
	 */


	Graph3d.prototype._redrawFilter = function () {
	  var dataFilter = this.dataGroup.dataFilter;

	  var filter = filter$2(this.frame);

	  filter.innerHTML = '';

	  if (!dataFilter) {
	    filter.slider = undefined;
	    return;
	  }

	  var options = {
	    'visible': this.showAnimationControls
	  };
	  var slider = new Slider(filter, options);
	  filter.slider = slider; // TODO: css here is not nice here...

	  filter.style.padding = '10px'; //this.frame.filter.style.backgroundColor = '#EFEFEF';

	  slider.setValues(values$2(dataFilter));
	  slider.setPlayInterval(this.animationInterval); // create an event handler

	  var me = this;

	  var onchange = function onchange() {
	    var dataFilter = me.dataGroup.dataFilter;
	    var index = slider.getIndex();
	    dataFilter.selectValue(index);
	    me.dataPoints = dataFilter._getDataPoints();
	    me.redraw();
	  };

	  slider.setOnChangeCallback(onchange);
	};
	/**
	 * Redraw the slider
	 */


	Graph3d.prototype._redrawSlider = function () {
	  if (filter$2(this.frame).slider !== undefined) {
	    filter$2(this.frame).slider.redraw();
	  }
	};
	/**
	 * Redraw common information
	 */


	Graph3d.prototype._redrawInfo = function () {
	  var info = this.dataGroup.getInfo();
	  if (info === undefined) return;

	  var ctx = this._getContext();

	  ctx.font = '14px arial'; // TODO: put in options

	  ctx.lineStyle = 'gray';
	  ctx.fillStyle = 'gray';
	  ctx.textAlign = 'left';
	  ctx.textBaseline = 'top';
	  var x = this.margin;
	  var y = this.margin;
	  ctx.fillText(info, x, y);
	};
	/**
	 * Draw a line between 2d points 'from' and 'to'.
	 *
	 * If stroke style specified, set that as well.
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {vis.Point2d} from
	 * @param {vis.Point2d} to
	 * @param {string} [strokeStyle]
	 * @private
	 */


	Graph3d.prototype._line = function (ctx, from, to, strokeStyle) {
	  if (strokeStyle !== undefined) {
	    ctx.strokeStyle = strokeStyle;
	  }

	  ctx.beginPath();
	  ctx.moveTo(from.x, from.y);
	  ctx.lineTo(to.x, to.y);
	  ctx.stroke();
	};
	/**
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {vis.Point3d} point3d
	 * @param {string} text
	 * @param {number} armAngle
	 * @param {number} [yMargin=0]
	 */


	Graph3d.prototype.drawAxisLabelX = function (ctx, point3d, text, armAngle, yMargin) {
	  if (yMargin === undefined) {
	    yMargin = 0;
	  }

	  var point2d = this._convert3Dto2D(point3d);

	  if (Math.cos(armAngle * 2) > 0) {
	    ctx.textAlign = 'center';
	    ctx.textBaseline = 'top';
	    point2d.y += yMargin;
	  } else if (Math.sin(armAngle * 2) < 0) {
	    ctx.textAlign = 'right';
	    ctx.textBaseline = 'middle';
	  } else {
	    ctx.textAlign = 'left';
	    ctx.textBaseline = 'middle';
	  }

	  ctx.fillStyle = this.axisColor;
	  ctx.fillText(text, point2d.x, point2d.y);
	};
	/**
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {vis.Point3d} point3d
	 * @param {string} text
	 * @param {number} armAngle
	 * @param {number} [yMargin=0]
	 */


	Graph3d.prototype.drawAxisLabelY = function (ctx, point3d, text, armAngle, yMargin) {
	  if (yMargin === undefined) {
	    yMargin = 0;
	  }

	  var point2d = this._convert3Dto2D(point3d);

	  if (Math.cos(armAngle * 2) < 0) {
	    ctx.textAlign = 'center';
	    ctx.textBaseline = 'top';
	    point2d.y += yMargin;
	  } else if (Math.sin(armAngle * 2) > 0) {
	    ctx.textAlign = 'right';
	    ctx.textBaseline = 'middle';
	  } else {
	    ctx.textAlign = 'left';
	    ctx.textBaseline = 'middle';
	  }

	  ctx.fillStyle = this.axisColor;
	  ctx.fillText(text, point2d.x, point2d.y);
	};
	/**
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {vis.Point3d} point3d
	 * @param {string} text
	 * @param {number} [offset=0]
	 */


	Graph3d.prototype.drawAxisLabelZ = function (ctx, point3d, text, offset) {
	  if (offset === undefined) {
	    offset = 0;
	  }

	  var point2d = this._convert3Dto2D(point3d);

	  ctx.textAlign = 'right';
	  ctx.textBaseline = 'middle';
	  ctx.fillStyle = this.axisColor;
	  ctx.fillText(text, point2d.x - offset, point2d.y);
	};
	/**
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {vis.Point3d} point3d
	 * @param {string} text
	 * @param {number} armAngle
	 * @param {number} [yMargin=0]
	 */


	Graph3d.prototype.drawAxisLabelXRotate = function (ctx, point3d, text, armAngle, yMargin) {
	  var point2d = this._convert3Dto2D(point3d);

	  if (Math.cos(armAngle * 2) > 0) {
	    ctx.save();
	    ctx.translate(point2d.x, point2d.y);
	    ctx.rotate(-Math.PI / 2);
	    ctx.textAlign = 'right';
	    ctx.textBaseline = 'middle';
	    ctx.fillStyle = this.axisColor;
	    ctx.fillText(text, 0, 0);
	    ctx.restore();
	  } else if (Math.sin(armAngle * 2) < 0) {
	    ctx.textAlign = 'right';
	    ctx.textBaseline = 'middle';
	    ctx.fillStyle = this.axisColor;
	    ctx.fillText(text, point2d.x, point2d.y);
	  } else {
	    ctx.textAlign = 'left';
	    ctx.textBaseline = 'middle';
	    ctx.fillStyle = this.axisColor;
	    ctx.fillText(text, point2d.x, point2d.y);
	  }
	};
	/**
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {vis.Point3d} point3d
	 * @param {string} text
	 * @param {number} armAngle
	 * @param {number} [yMargin=0]
	 */


	Graph3d.prototype.drawAxisLabelYRotate = function (ctx, point3d, text, armAngle, yMargin) {
	  var point2d = this._convert3Dto2D(point3d);

	  if (Math.cos(armAngle * 2) < 0) {
	    ctx.save();
	    ctx.translate(point2d.x, point2d.y);
	    ctx.rotate(-Math.PI / 2);
	    ctx.textAlign = 'right';
	    ctx.textBaseline = 'middle';
	    ctx.fillStyle = this.axisColor;
	    ctx.fillText(text, 0, 0);
	    ctx.restore();
	  } else if (Math.sin(armAngle * 2) > 0) {
	    ctx.textAlign = 'right';
	    ctx.textBaseline = 'middle';
	    ctx.fillStyle = this.axisColor;
	    ctx.fillText(text, point2d.x, point2d.y);
	  } else {
	    ctx.textAlign = 'left';
	    ctx.textBaseline = 'middle';
	    ctx.fillStyle = this.axisColor;
	    ctx.fillText(text, point2d.x, point2d.y);
	  }
	};
	/**
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {vis.Point3d} point3d
	 * @param {string} text
	 * @param {number} [offset=0]
	 */


	Graph3d.prototype.drawAxisLabelZRotate = function (ctx, point3d, text, offset) {
	  if (offset === undefined) {
	    offset = 0;
	  }

	  var point2d = this._convert3Dto2D(point3d);

	  ctx.textAlign = 'right';
	  ctx.textBaseline = 'middle';
	  ctx.fillStyle = this.axisColor;
	  ctx.fillText(text, point2d.x - offset, point2d.y);
	};
	/**


	/**
	 * Draw a line between 2d points 'from' and 'to'.
	 *
	 * If stroke style specified, set that as well.
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {vis.Point2d} from
	 * @param {vis.Point2d} to
	 * @param {string} [strokeStyle]
	 * @private
	 */


	Graph3d.prototype._line3d = function (ctx, from, to, strokeStyle) {
	  var from2d = this._convert3Dto2D(from);

	  var to2d = this._convert3Dto2D(to);

	  this._line(ctx, from2d, to2d, strokeStyle);
	};
	/**
	 * Redraw the axis
	 */


	Graph3d.prototype._redrawAxis = function () {
	  var ctx = this._getContext(),
	      from,
	      to,
	      step,
	      prettyStep,
	      text,
	      xText,
	      yText,
	      zText,
	      offset,
	      xOffset,
	      yOffset; // TODO: get the actual rendered style of the containerElement
	  //ctx.font = this.containerElement.style.font;
	  //ctx.font = 24 / this.camera.getArmLength() + 'px arial';


	  ctx.font = this.axisFontSize / this.camera.getArmLength() + 'px ' + this.axisFontType; // calculate the length for the short grid lines

	  var gridLenX = 0.025 / this.scale.x;
	  var gridLenY = 0.025 / this.scale.y;
	  var textMargin = 5 / this.camera.getArmLength(); // px

	  var armAngle = this.camera.getArmRotation().horizontal;
	  var armVector = new Point2d_1(Math.cos(armAngle), Math.sin(armAngle));
	  var xRange = this.xRange;
	  var yRange = this.yRange;
	  var zRange = this.zRange;
	  var point3d; // draw x-grid lines

	  ctx.lineWidth = 1;
	  prettyStep = this.defaultXStep === undefined;
	  step = new StepNumber_1(xRange.min, xRange.max, this.xStep, prettyStep);
	  step.start(true);

	  while (!step.end()) {
	    var x = step.getCurrent();

	    if (this.showGrid) {
	      from = new Point3d_1(x, yRange.min, zRange.min);
	      to = new Point3d_1(x, yRange.max, zRange.min);

	      this._line3d(ctx, from, to, this.gridColor);
	    } else if (this.showXAxis) {
	      from = new Point3d_1(x, yRange.min, zRange.min);
	      to = new Point3d_1(x, yRange.min + gridLenX, zRange.min);

	      this._line3d(ctx, from, to, this.axisColor);

	      from = new Point3d_1(x, yRange.max, zRange.min);
	      to = new Point3d_1(x, yRange.max - gridLenX, zRange.min);

	      this._line3d(ctx, from, to, this.axisColor);
	    }

	    if (this.showXAxis) {
	      yText = armVector.x > 0 ? yRange.min : yRange.max;
	      point3d = new Point3d_1(x, yText, zRange.min);
	      var msg = '  ' + this.xValueLabel(x) + '  ';

	      this._drawAxisLabelX.call(this, ctx, point3d, msg, armAngle, textMargin);
	    }

	    step.next();
	  } // draw y-grid lines


	  ctx.lineWidth = 1;
	  prettyStep = this.defaultYStep === undefined;
	  step = new StepNumber_1(yRange.min, yRange.max, this.yStep, prettyStep);
	  step.start(true);

	  while (!step.end()) {
	    var y = step.getCurrent();

	    if (this.showGrid) {
	      from = new Point3d_1(xRange.min, y, zRange.min);
	      to = new Point3d_1(xRange.max, y, zRange.min);

	      this._line3d(ctx, from, to, this.gridColor);
	    } else if (this.showYAxis) {
	      from = new Point3d_1(xRange.min, y, zRange.min);
	      to = new Point3d_1(xRange.min + gridLenY, y, zRange.min);

	      this._line3d(ctx, from, to, this.axisColor);

	      from = new Point3d_1(xRange.max, y, zRange.min);
	      to = new Point3d_1(xRange.max - gridLenY, y, zRange.min);

	      this._line3d(ctx, from, to, this.axisColor);
	    }

	    if (this.showYAxis) {
	      xText = armVector.y > 0 ? xRange.min : xRange.max;
	      point3d = new Point3d_1(xText, y, zRange.min);

	      var _msg = '  ' + this.yValueLabel(y) + '  ';

	      this._drawAxisLabelY.call(this, ctx, point3d, _msg, armAngle, textMargin);
	    }

	    step.next();
	  } // draw z-grid lines and axis


	  if (this.showZAxis) {
	    ctx.lineWidth = 1;
	    prettyStep = this.defaultZStep === undefined;
	    step = new StepNumber_1(zRange.min, zRange.max, this.zStep, prettyStep);
	    step.start(true);
	    xText = armVector.x > 0 ? xRange.min : xRange.max;
	    yText = armVector.y < 0 ? yRange.min : yRange.max;

	    while (!step.end()) {
	      var z = step.getCurrent(); // TODO: make z-grid lines really 3d?

	      var from3d = new Point3d_1(xText, yText, z);

	      var from2d = this._convert3Dto2D(from3d);

	      to = new Point2d_1(from2d.x - textMargin, from2d.y);

	      this._line(ctx, from2d, to, this.axisColor);

	      var _msg2 = this.zValueLabel(z) + ' ';

	      this._drawAxisLabelZ.call(this, ctx, from3d, _msg2, 5);

	      step.next();
	    }

	    ctx.lineWidth = 1;
	    from = new Point3d_1(xText, yText, zRange.min);
	    to = new Point3d_1(xText, yText, zRange.max);

	    this._line3d(ctx, from, to, this.axisColor);
	  } // draw x-axis


	  if (this.showXAxis) {
	    var xMin2d;
	    var xMax2d;
	    ctx.lineWidth = 1; // line at yMin

	    xMin2d = new Point3d_1(xRange.min, yRange.min, zRange.min);
	    xMax2d = new Point3d_1(xRange.max, yRange.min, zRange.min);

	    this._line3d(ctx, xMin2d, xMax2d, this.axisColor); // line at ymax


	    xMin2d = new Point3d_1(xRange.min, yRange.max, zRange.min);
	    xMax2d = new Point3d_1(xRange.max, yRange.max, zRange.min);

	    this._line3d(ctx, xMin2d, xMax2d, this.axisColor);
	  } // draw y-axis


	  if (this.showYAxis) {
	    ctx.lineWidth = 1; // line at xMin

	    from = new Point3d_1(xRange.min, yRange.min, zRange.min);
	    to = new Point3d_1(xRange.min, yRange.max, zRange.min);

	    this._line3d(ctx, from, to, this.axisColor); // line at xMax


	    from = new Point3d_1(xRange.max, yRange.min, zRange.min);
	    to = new Point3d_1(xRange.max, yRange.max, zRange.min);

	    this._line3d(ctx, from, to, this.axisColor);
	  } // draw x-label


	  var xLabel = this.xLabel;

	  if (xLabel.length > 0 && this.showXAxis) {
	    yOffset = 0.1 / this.scale.y;
	    xText = (xRange.max + 3 * xRange.min) / 4;
	    yText = armVector.x > 0 ? yRange.min - yOffset : yRange.max + yOffset;
	    text = new Point3d_1(xText, yText, zRange.min);
	    this.drawAxisLabelX(ctx, text, xLabel, armAngle);
	  } // draw y-label


	  var yLabel = this.yLabel;

	  if (yLabel.length > 0 && this.showYAxis) {
	    xOffset = 0.1 / this.scale.x;
	    xText = armVector.y > 0 ? xRange.min - xOffset : xRange.max + xOffset;
	    yText = (yRange.max + 3 * yRange.min) / 4;
	    text = new Point3d_1(xText, yText, zRange.min);
	    this.drawAxisLabelY(ctx, text, yLabel, armAngle);
	  } // draw z-label


	  var zLabel = this.zLabel;

	  if (zLabel.length > 0 && this.showZAxis) {
	    offset = 30; // pixels.  // TODO: relate to the max width of the values on the z axis?

	    xText = armVector.x > 0 ? xRange.min : xRange.max;
	    yText = armVector.y < 0 ? yRange.min : yRange.max;
	    zText = (zRange.max + 3 * zRange.min) / 4;
	    text = new Point3d_1(xText, yText, zText);
	    this.drawAxisLabelZ(ctx, text, zLabel, offset);
	  }
	};
	/**
	 *
	 * @param {vis.Point3d} point
	 * @returns {*}
	 * @private
	 */


	Graph3d.prototype._getStrokeWidth = function (point) {
	  if (point !== undefined) {
	    if (this.showPerspective) {
	      return 1 / -point.trans.z * this.dataColor.strokeWidth;
	    } else {
	      return -(this.eye.z / this.camera.getArmLength()) * this.dataColor.strokeWidth;
	    }
	  }

	  return this.dataColor.strokeWidth;
	}; // -----------------------------------------------------------------------------
	// Drawing primitives for the graphs
	// -----------------------------------------------------------------------------

	/**
	 * Draw a bar element in the view with the given properties.
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {Object} point
	 * @param {number} xWidth
	 * @param {number} yWidth
	 * @param {string} color
	 * @param {string} borderColor
	 * @private
	 */


	Graph3d.prototype._redrawBar = function (ctx, point, xWidth, yWidth, color, borderColor) {
	  var surface; // calculate all corner points

	  var me = this;
	  var point3d = point.point;
	  var zMin = this.zRange.min;
	  var top = [{
	    point: new Point3d_1(point3d.x - xWidth, point3d.y - yWidth, point3d.z)
	  }, {
	    point: new Point3d_1(point3d.x + xWidth, point3d.y - yWidth, point3d.z)
	  }, {
	    point: new Point3d_1(point3d.x + xWidth, point3d.y + yWidth, point3d.z)
	  }, {
	    point: new Point3d_1(point3d.x - xWidth, point3d.y + yWidth, point3d.z)
	  }];
	  var bottom = [{
	    point: new Point3d_1(point3d.x - xWidth, point3d.y - yWidth, zMin)
	  }, {
	    point: new Point3d_1(point3d.x + xWidth, point3d.y - yWidth, zMin)
	  }, {
	    point: new Point3d_1(point3d.x + xWidth, point3d.y + yWidth, zMin)
	  }, {
	    point: new Point3d_1(point3d.x - xWidth, point3d.y + yWidth, zMin)
	  }]; // calculate screen location of the points

	  forEach$3(top).call(top, function (obj) {
	    obj.screen = me._convert3Dto2D(obj.point);
	  });

	  forEach$3(bottom).call(bottom, function (obj) {
	    obj.screen = me._convert3Dto2D(obj.point);
	  }); // create five sides, calculate both corner points and center points


	  var surfaces = [{
	    corners: top,
	    center: Point3d_1.avg(bottom[0].point, bottom[2].point)
	  }, {
	    corners: [top[0], top[1], bottom[1], bottom[0]],
	    center: Point3d_1.avg(bottom[1].point, bottom[0].point)
	  }, {
	    corners: [top[1], top[2], bottom[2], bottom[1]],
	    center: Point3d_1.avg(bottom[2].point, bottom[1].point)
	  }, {
	    corners: [top[2], top[3], bottom[3], bottom[2]],
	    center: Point3d_1.avg(bottom[3].point, bottom[2].point)
	  }, {
	    corners: [top[3], top[0], bottom[0], bottom[3]],
	    center: Point3d_1.avg(bottom[0].point, bottom[3].point)
	  }];
	  point.surfaces = surfaces; // calculate the distance of each of the surface centers to the camera

	  for (var j = 0; j < surfaces.length; j++) {
	    surface = surfaces[j];

	    var transCenter = this._convertPointToTranslation(surface.center);

	    surface.dist = this.showPerspective ? transCenter.length() : -transCenter.z; // TODO: this dept calculation doesn't work 100% of the cases due to perspective,
	    //     but the current solution is fast/simple and works in 99.9% of all cases
	    //     the issue is visible in example 14, with graph.setCameraPosition({horizontal: 2.97, vertical: 0.5, distance: 0.9})
	  } // order the surfaces by their (translated) depth


	  sort$2(surfaces).call(surfaces, function (a, b) {
	    var diff = b.dist - a.dist;
	    if (diff) return diff; // if equal depth, sort the top surface last

	    if (a.corners === top) return 1;
	    if (b.corners === top) return -1; // both are equal

	    return 0;
	  }); // draw the ordered surfaces


	  ctx.lineWidth = this._getStrokeWidth(point);
	  ctx.strokeStyle = borderColor;
	  ctx.fillStyle = color; // NOTE: we start at j=2 instead of j=0 as we don't need to draw the two surfaces at the backside

	  for (var _j3 = 2; _j3 < surfaces.length; _j3++) {
	    surface = surfaces[_j3];

	    this._polygon(ctx, surface.corners);
	  }
	};
	/**
	 * Draw a polygon using the passed points and fill it with the passed style and stroke.
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {Array.<vis.Point3d>} points      an array of points.
	 * @param {string} [fillStyle] the fill style to set
	 * @param {string} [strokeStyle] the stroke style to set
	 */


	Graph3d.prototype._polygon = function (ctx, points, fillStyle, strokeStyle) {
	  if (points.length < 2) {
	    return;
	  }

	  if (fillStyle !== undefined) {
	    ctx.fillStyle = fillStyle;
	  }

	  if (strokeStyle !== undefined) {
	    ctx.strokeStyle = strokeStyle;
	  }

	  ctx.beginPath();
	  ctx.moveTo(points[0].screen.x, points[0].screen.y);

	  for (var i = 1; i < points.length; ++i) {
	    var point = points[i];
	    ctx.lineTo(point.screen.x, point.screen.y);
	  }

	  ctx.closePath();

	  fill$2(ctx).call(ctx);

	  ctx.stroke(); // TODO: only draw stroke when strokeWidth > 0
	};
	/**
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {object} point
	 * @param {string} color
	 * @param {string} borderColor
	 * @param {number} [size=this._dotSize()]
	 * @private
	 */


	Graph3d.prototype._drawCircle = function (ctx, point, color, borderColor, size) {
	  var radius = this._calcRadius(point, size);

	  ctx.lineWidth = this._getStrokeWidth(point);
	  ctx.strokeStyle = borderColor;
	  ctx.fillStyle = color;
	  ctx.beginPath();
	  ctx.arc(point.screen.x, point.screen.y, radius, 0, Math.PI * 2, true);

	  fill$2(ctx).call(ctx);

	  ctx.stroke();
	};
	/**
	 * Determine the colors for the 'regular' graph styles.
	 *
	 * @param {object} point
	 * @returns {{fill, border}}
	 * @private
	 */


	Graph3d.prototype._getColorsRegular = function (point) {
	  var f = (point.point.value - this.valueRange.min) * this.scale.value;

	  var color = this._colormap(f, 1);

	  var borderColor = this._colormap(f, 0.8);

	  return {
	    fill: color,
	    border: borderColor
	  };
	};
	/**
	 * Get the colors for the 'color' graph styles.
	 * These styles are currently: 'bar-color' and 'dot-color'
	 * Color may be set as a string representation of HTML color, like #ff00ff,
	 * or calculated from a number, for example, distance from this point
	 * The first option is useful when we have some pre-given legend, to which we have to adjust ourselves
	 * The second option is useful when we are interested in automatically setting the color, from some value,
	 * using some color scale
	 * @param {object} point
	 * @returns {{fill: *, border: *}}
	 * @private
	 */


	Graph3d.prototype._getColorsColor = function (point) {
	  // calculate the color based on the value
	  var color, borderColor, pointStyle;

	  if (point && point.point && point.point.data && point.point.data.style) {
	    pointStyle = point.point.data.style;
	  }

	  if (pointStyle && _typeof_1(pointStyle) === 'object' && fill$2(pointStyle) && pointStyle.stroke) {
	    return {
	      fill: fill$2(pointStyle),
	      border: pointStyle.stroke
	    };
	  }

	  if (typeof point.point.value === "string") {
	    color = point.point.value;
	    borderColor = point.point.value;
	  } else {
	    var f = (point.point.value - this.valueRange.min) * this.scale.value;
	    color = this._colormap(f, 1);
	    borderColor = this._colormap(f, 0.8);
	  }

	  return {
	    fill: color,
	    border: borderColor
	  };
	};
	/**
	 * Get the colors for the 'size' graph styles.
	 * These styles are currently: 'bar-size' and 'dot-size'
	 *
	 * @returns {{fill: *, border: (string|colorOptions.stroke|{string, undefined}|string|colorOptions.stroke|{string}|*)}}
	 * @private
	 */


	Graph3d.prototype._getColorsSize = function () {
	  return {
	    fill: fill$2(this.dataColor),
	    border: this.dataColor.stroke
	  };
	};
	/**
	 * Determine the color corresponding to a given value on the color scale.
	 *
	 * @param {number} [x] the data value to be mapped running from 0 to 1
	 * @param {number} [v] scale factor between 0 and 1 for the color brightness
	 * @returns {string}
	 * @private
	 */


	Graph3d.prototype._colormap = function (x) {
	  var v = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;
	  var r, g, b, a;
	  var colormap = this.colormap;

	  if (isArray$3(colormap)) {
	    var maxIndex = colormap.length - 1;
	    var startIndex = Math.max(Math.floor(x * maxIndex), 0);
	    var endIndex = Math.min(startIndex + 1, maxIndex);
	    var innerRatio = x * maxIndex - startIndex;
	    var min = colormap[startIndex];
	    var max = colormap[endIndex];
	    r = min.r + innerRatio * (max.r - min.r);
	    g = min.g + innerRatio * (max.g - min.g);
	    b = min.b + innerRatio * (max.b - min.b);
	  } else if (typeof colormap === 'function') {
	    var _colormap = colormap(x);

	    r = _colormap.r;
	    g = _colormap.g;
	    b = _colormap.b;
	    a = _colormap.a;
	  } else {
	    var hue = (1 - x) * 240;

	    var _HSVToRGB = HSVToRGB(hue / 360, 1, 1);

	    r = _HSVToRGB.r;
	    g = _HSVToRGB.g;
	    b = _HSVToRGB.b;
	  }

	  if (typeof a === 'number' && !isNan$2(a)) {
	    var _context3, _context4, _context5;

	    return concat$2(_context3 = concat$2(_context4 = concat$2(_context5 = "RGBA(".concat(Math.round(r * v), ", ")).call(_context5, Math.round(g * v), ", ")).call(_context4, Math.round(b * v), ", ")).call(_context3, a, ")");
	  } else {
	    var _context6, _context7;

	    return concat$2(_context6 = concat$2(_context7 = "RGB(".concat(Math.round(r * v), ", ")).call(_context7, Math.round(g * v), ", ")).call(_context6, Math.round(b * v), ")");
	  }
	};
	/**
	 * Determine the size of a point on-screen, as determined by the
	 * distance to the camera.
	 *
	 * @param {Object} point
	 * @param {number} [size=this._dotSize()] the size that needs to be translated to screen coordinates.
	 *             optional; if not passed, use the default point size.
	 * @returns {number}
	 * @private
	 */


	Graph3d.prototype._calcRadius = function (point, size) {
	  if (size === undefined) {
	    size = this._dotSize();
	  }

	  var radius;

	  if (this.showPerspective) {
	    radius = size / -point.trans.z;
	  } else {
	    radius = size * -(this.eye.z / this.camera.getArmLength());
	  }

	  if (radius < 0) {
	    radius = 0;
	  }

	  return radius;
	}; // -----------------------------------------------------------------------------
	// Methods for drawing points per graph style.
	// -----------------------------------------------------------------------------

	/**
	 * Draw single datapoint for graph style 'bar'.
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {Object} point
	 * @private
	 */


	Graph3d.prototype._redrawBarGraphPoint = function (ctx, point) {
	  var xWidth = this.xBarWidth / 2;
	  var yWidth = this.yBarWidth / 2;

	  var colors = this._getColorsRegular(point);

	  this._redrawBar(ctx, point, xWidth, yWidth, fill$2(colors), colors.border);
	};
	/**
	 * Draw single datapoint for graph style 'bar-color'.
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {Object} point
	 * @private
	 */


	Graph3d.prototype._redrawBarColorGraphPoint = function (ctx, point) {
	  var xWidth = this.xBarWidth / 2;
	  var yWidth = this.yBarWidth / 2;

	  var colors = this._getColorsColor(point);

	  this._redrawBar(ctx, point, xWidth, yWidth, fill$2(colors), colors.border);
	};
	/**
	 * Draw single datapoint for graph style 'bar-size'.
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {Object} point
	 * @private
	 */


	Graph3d.prototype._redrawBarSizeGraphPoint = function (ctx, point) {
	  // calculate size for the bar
	  var fraction = (point.point.value - this.valueRange.min) / this.valueRange.range();
	  var xWidth = this.xBarWidth / 2 * (fraction * 0.8 + 0.2);
	  var yWidth = this.yBarWidth / 2 * (fraction * 0.8 + 0.2);

	  var colors = this._getColorsSize();

	  this._redrawBar(ctx, point, xWidth, yWidth, fill$2(colors), colors.border);
	};
	/**
	 * Draw single datapoint for graph style 'dot'.
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {Object} point
	 * @private
	 */


	Graph3d.prototype._redrawDotGraphPoint = function (ctx, point) {
	  var colors = this._getColorsRegular(point);

	  this._drawCircle(ctx, point, fill$2(colors), colors.border);
	};
	/**
	 * Draw single datapoint for graph style 'dot-line'.
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {Object} point
	 * @private
	 */


	Graph3d.prototype._redrawDotLineGraphPoint = function (ctx, point) {
	  // draw a vertical line from the XY-plane to the graph value
	  var from = this._convert3Dto2D(point.bottom);

	  ctx.lineWidth = 1;

	  this._line(ctx, from, point.screen, this.gridColor);

	  this._redrawDotGraphPoint(ctx, point);
	};
	/**
	 * Draw single datapoint for graph style 'dot-color'.
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {Object} point
	 * @private
	 */


	Graph3d.prototype._redrawDotColorGraphPoint = function (ctx, point) {
	  var colors = this._getColorsColor(point);

	  this._drawCircle(ctx, point, fill$2(colors), colors.border);
	};
	/**
	 * Draw single datapoint for graph style 'dot-size'.
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {Object} point
	 * @private
	 */


	Graph3d.prototype._redrawDotSizeGraphPoint = function (ctx, point) {
	  var dotSize = this._dotSize();

	  var fraction = (point.point.value - this.valueRange.min) / this.valueRange.range();
	  var sizeMin = dotSize * this.dotSizeMinFraction;
	  var sizeRange = dotSize * this.dotSizeMaxFraction - sizeMin;
	  var size = sizeMin + sizeRange * fraction;

	  var colors = this._getColorsSize();

	  this._drawCircle(ctx, point, fill$2(colors), colors.border, size);
	};
	/**
	 * Draw single datapoint for graph style 'surface'.
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {Object} point
	 * @private
	 */


	Graph3d.prototype._redrawSurfaceGraphPoint = function (ctx, point) {
	  var right = point.pointRight;
	  var top = point.pointTop;
	  var cross = point.pointCross;

	  if (point === undefined || right === undefined || top === undefined || cross === undefined) {
	    return;
	  }

	  var topSideVisible = true;
	  var fillStyle;
	  var strokeStyle;
	  var cosViewAngle;

	  if (this.showGrayBottom || this.showShadow) {
	    // calculate the cross product of the two vectors from center
	    // to left and right, in order to know whether we are looking at the
	    // bottom or at the top side. We can also use the cross product
	    // for calculating light intensity
	    var aDiff = Point3d_1.subtract(cross.trans, point.trans);
	    var bDiff = Point3d_1.subtract(top.trans, right.trans);
	    var surfaceNormal = Point3d_1.crossProduct(aDiff, bDiff);

	    if (this.showPerspective) {
	      var surfacePosition = Point3d_1.avg(Point3d_1.avg(point.trans, cross.trans), Point3d_1.avg(right.trans, top.trans)); // This corresponds to diffuse lighting with light source at (0, 0, 0).
	      // More generally, we would need `surfacePosition - lightPosition`:

	      cosViewAngle = -Point3d_1.dotProduct(surfaceNormal.normalize(), surfacePosition.normalize());
	    } else {
	      cosViewAngle = surfaceNormal.z / surfaceNormal.length();
	    }

	    topSideVisible = cosViewAngle > 0;
	  }

	  if (topSideVisible || !this.showGrayBottom) {
	    var vAvg = (point.point.value + right.point.value + top.point.value + cross.point.value) / 4;
	    var ratio = (vAvg - this.valueRange.min) * this.scale.value; // lighting factor. TODO: let user specify lighting model as function(?)

	    var v = this.showShadow ? (1 + cosViewAngle) / 2 : 1;
	    fillStyle = this._colormap(ratio, v);
	  } else {
	    fillStyle = 'gray';
	  }

	  if (this.showSurfaceGrid) {
	    strokeStyle = this.axisColor; // TODO: should be customizable
	  } else {
	    strokeStyle = fillStyle;
	  }

	  ctx.lineWidth = this._getStrokeWidth(point); // TODO: only draw stroke when strokeWidth > 0

	  var points = [point, right, cross, top];

	  this._polygon(ctx, points, fillStyle, strokeStyle);
	};
	/**
	 * Helper method for _redrawGridGraphPoint()
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {Object} from
	 * @param {Object} to
	 * @private
	 */


	Graph3d.prototype._drawGridLine = function (ctx, from, to) {
	  if (from === undefined || to === undefined) {
	    return;
	  }

	  var vAvg = (from.point.value + to.point.value) / 2;
	  var f = (vAvg - this.valueRange.min) * this.scale.value;
	  ctx.lineWidth = this._getStrokeWidth(from) * 2;
	  ctx.strokeStyle = this._colormap(f, 1);

	  this._line(ctx, from.screen, to.screen);
	};
	/**
	 * Draw single datapoint for graph style 'Grid'.
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {Object} point
	 * @private
	 */


	Graph3d.prototype._redrawGridGraphPoint = function (ctx, point) {
	  this._drawGridLine(ctx, point, point.pointRight);

	  this._drawGridLine(ctx, point, point.pointTop);
	};
	/**
	 * Draw single datapoint for graph style 'line'.
	 *
	 * @param {CanvasRenderingContext2D} ctx
	 * @param {Object} point
	 * @private
	 */


	Graph3d.prototype._redrawLineGraphPoint = function (ctx, point) {
	  if (point.pointNext === undefined) {
	    return;
	  }

	  ctx.lineWidth = this._getStrokeWidth(point);
	  ctx.strokeStyle = this.dataColor.stroke;

	  this._line(ctx, point.screen, point.pointNext.screen);
	};
	/**
	 * Draw all datapoints for currently selected graph style.
	 *
	 */


	Graph3d.prototype._redrawDataGraph = function () {
	  var ctx = this._getContext();

	  var i;
	  if (this.dataPoints === undefined || this.dataPoints.length <= 0) return; // TODO: throw exception?

	  this._calcTranslations(this.dataPoints);

	  for (i = 0; i < this.dataPoints.length; i++) {
	    var point = this.dataPoints[i]; // Using call() ensures that the correct context is used

	    this._pointDrawingMethod.call(this, ctx, point);
	  }
	}; // -----------------------------------------------------------------------------
	// End methods for drawing points per graph style.
	// -----------------------------------------------------------------------------

	/**
	 * Store startX, startY and startOffset for mouse operations
	 *
	 * @param {Event}     event     The event that occurred
	 */


	Graph3d.prototype._storeMousePosition = function (event) {
	  // get mouse position (different code for IE and all other browsers)
	  this.startMouseX = getMouseX(event);
	  this.startMouseY = getMouseY(event);
	  this._startCameraOffset = this.camera.getOffset();
	};
	/**
	 * Start a moving operation inside the provided parent element
	 * @param {Event}     event     The event that occurred (required for
	 *                  retrieving the  mouse position)
	 */


	Graph3d.prototype._onMouseDown = function (event) {
	  event = event || window.event; // check if mouse is still down (may be up when focus is lost for example
	  // in an iframe)

	  if (this.leftButtonDown) {
	    this._onMouseUp(event);
	  } // only react on left mouse button down


	  this.leftButtonDown = event.which ? event.which === 1 : event.button === 1;
	  if (!this.leftButtonDown && !this.touchDown) return;

	  this._storeMousePosition(event);

	  this.startStart = new Date(this.start);
	  this.startEnd = new Date(this.end);
	  this.startArmRotation = this.camera.getArmRotation();
	  this.frame.style.cursor = 'move'; // add event listeners to handle moving the contents
	  // we store the function onmousemove and onmouseup in the graph, so we can
	  // remove the eventlisteners lateron in the function mouseUp()

	  var me = this;

	  this.onmousemove = function (event) {
	    me._onMouseMove(event);
	  };

	  this.onmouseup = function (event) {
	    me._onMouseUp(event);
	  };

	  addEventListener(document, 'mousemove', me.onmousemove);
	  addEventListener(document, 'mouseup', me.onmouseup);
	  preventDefault(event);
	};
	/**
	 * Perform moving operating.
	 * This function activated from within the funcion Graph.mouseDown().
	 * @param {Event}   event  Well, eehh, the event
	 */


	Graph3d.prototype._onMouseMove = function (event) {
	  this.moving = true;
	  event = event || window.event; // calculate change in mouse position

	  var diffX = _parseFloat$2(getMouseX(event)) - this.startMouseX;
	  var diffY = _parseFloat$2(getMouseY(event)) - this.startMouseY; // move with ctrl or rotate by other

	  if (event && event.ctrlKey === true) {
	    // calculate change in mouse position
	    var scaleX = this.frame.clientWidth * 0.5;
	    var scaleY = this.frame.clientHeight * 0.5;
	    var offXNew = (this._startCameraOffset.x || 0) - diffX / scaleX * this.camera.armLength * 0.8;
	    var offYNew = (this._startCameraOffset.y || 0) + diffY / scaleY * this.camera.armLength * 0.8;
	    this.camera.setOffset(offXNew, offYNew);

	    this._storeMousePosition(event);
	  } else {
	    var horizontalNew = this.startArmRotation.horizontal + diffX / 200;
	    var verticalNew = this.startArmRotation.vertical + diffY / 200;
	    var snapAngle = 4; // degrees

	    var snapValue = Math.sin(snapAngle / 360 * 2 * Math.PI); // snap horizontally to nice angles at 0pi, 0.5pi, 1pi, 1.5pi, etc...
	    // the -0.001 is to take care that the vertical axis is always drawn at the left front corner

	    if (Math.abs(Math.sin(horizontalNew)) < snapValue) {
	      horizontalNew = Math.round(horizontalNew / Math.PI) * Math.PI - 0.001;
	    }

	    if (Math.abs(Math.cos(horizontalNew)) < snapValue) {
	      horizontalNew = (Math.round(horizontalNew / Math.PI - 0.5) + 0.5) * Math.PI - 0.001;
	    } // snap vertically to nice angles


	    if (Math.abs(Math.sin(verticalNew)) < snapValue) {
	      verticalNew = Math.round(verticalNew / Math.PI) * Math.PI;
	    }

	    if (Math.abs(Math.cos(verticalNew)) < snapValue) {
	      verticalNew = (Math.round(verticalNew / Math.PI - 0.5) + 0.5) * Math.PI;
	    }

	    this.camera.setArmRotation(horizontalNew, verticalNew);
	  }

	  this.redraw(); // fire a cameraPositionChange event

	  var parameters = this.getCameraPosition();
	  this.emit('cameraPositionChange', parameters);
	  preventDefault(event);
	};
	/**
	 * Stop moving operating.
	 * This function activated from within the funcion Graph.mouseDown().
	 * @param {Event}  event   The event
	 */


	Graph3d.prototype._onMouseUp = function (event) {
	  this.frame.style.cursor = 'auto';
	  this.leftButtonDown = false; // remove event listeners here

	  removeEventListener(document, 'mousemove', this.onmousemove);
	  removeEventListener(document, 'mouseup', this.onmouseup);
	  preventDefault(event);
	};
	/**
	 * @param {Event}  event   The event
	 */


	Graph3d.prototype._onClick = function (event) {
	  // NOTE: onclick_callback is deprecated and may be removed in a future version.
	  if (!this.onclick_callback && !this.hasListeners('click')) return;

	  if (!this.moving) {
	    var boundingRect = this.frame.getBoundingClientRect();
	    var mouseX = getMouseX(event) - boundingRect.left;
	    var mouseY = getMouseY(event) - boundingRect.top;

	    var dataPoint = this._dataPointFromXY(mouseX, mouseY);

	    if (dataPoint) {
	      if (this.onclick_callback) this.onclick_callback(dataPoint.point.data);
	      this.emit('click', dataPoint.point.data);
	    }
	  } else {
	    // disable onclick callback, if it came immediately after rotate/pan
	    this.moving = false;
	  }

	  preventDefault(event);
	};
	/**
	 * After having moved the mouse, a tooltip should pop up when the mouse is resting on a data point
	 * @param {Event}  event   A mouse move event
	 */


	Graph3d.prototype._onTooltip = function (event) {
	  var delay = this.tooltipDelay; // ms

	  var boundingRect = this.frame.getBoundingClientRect();
	  var mouseX = getMouseX(event) - boundingRect.left;
	  var mouseY = getMouseY(event) - boundingRect.top;

	  if (!this.showTooltip) {
	    return;
	  }

	  if (this.tooltipTimeout) {
	    clearTimeout(this.tooltipTimeout);
	  } // (delayed) display of a tooltip only if no mouse button is down


	  if (this.leftButtonDown) {
	    this._hideTooltip();

	    return;
	  }

	  if (this.tooltip && this.tooltip.dataPoint) {
	    // tooltip is currently visible
	    var dataPoint = this._dataPointFromXY(mouseX, mouseY);

	    if (dataPoint !== this.tooltip.dataPoint) {
	      // datapoint changed
	      if (dataPoint) {
	        this._showTooltip(dataPoint);
	      } else {
	        this._hideTooltip();
	      }
	    }
	  } else {
	    // tooltip is currently not visible
	    var me = this;
	    this.tooltipTimeout = setTimeout$2(function () {
	      me.tooltipTimeout = null; // show a tooltip if we have a data point

	      var dataPoint = me._dataPointFromXY(mouseX, mouseY);

	      if (dataPoint) {
	        me._showTooltip(dataPoint);
	      }
	    }, delay);
	  }
	};
	/**
	 * Event handler for touchstart event on mobile devices
	 * @param {Event}  event   The event
	 */


	Graph3d.prototype._onTouchStart = function (event) {
	  this.touchDown = true;
	  var me = this;

	  this.ontouchmove = function (event) {
	    me._onTouchMove(event);
	  };

	  this.ontouchend = function (event) {
	    me._onTouchEnd(event);
	  };

	  addEventListener(document, 'touchmove', me.ontouchmove);
	  addEventListener(document, 'touchend', me.ontouchend);

	  this._onMouseDown(event);
	};
	/**
	 * Event handler for touchmove event on mobile devices
	 * @param {Event}  event   The event
	 */


	Graph3d.prototype._onTouchMove = function (event) {
	  this._onMouseMove(event);
	};
	/**
	 * Event handler for touchend event on mobile devices
	 * @param {Event}  event   The event
	 */


	Graph3d.prototype._onTouchEnd = function (event) {
	  this.touchDown = false;
	  removeEventListener(document, 'touchmove', this.ontouchmove);
	  removeEventListener(document, 'touchend', this.ontouchend);

	  this._onMouseUp(event);
	};
	/**
	 * Event handler for mouse wheel event, used to zoom the graph
	 * Code from http://adomas.org/javascript-mouse-wheel/
	 * @param {Event}  event   The event
	 */


	Graph3d.prototype._onWheel = function (event) {
	  if (!event)
	    /* For IE. */
	    event = window.event;

	  if (this.zoomable && (!this.ctrlToZoom || event.ctrlKey)) {
	    // retrieve delta
	    var delta = 0;

	    if (event.wheelDelta) {
	      /* IE/Opera. */
	      delta = event.wheelDelta / 120;
	    } else if (event.detail) {
	      /* Mozilla case. */
	      // In Mozilla, sign of delta is different than in IE.
	      // Also, delta is multiple of 3.
	      delta = -event.detail / 3;
	    } // If delta is nonzero, handle it.
	    // Basically, delta is now positive if wheel was scrolled up,
	    // and negative, if wheel was scrolled down.


	    if (delta) {
	      var oldLength = this.camera.getArmLength();
	      var newLength = oldLength * (1 - delta / 10);
	      this.camera.setArmLength(newLength);
	      this.redraw();

	      this._hideTooltip();
	    } // fire a cameraPositionChange event


	    var parameters = this.getCameraPosition();
	    this.emit('cameraPositionChange', parameters); // Prevent default actions caused by mouse wheel.
	    // That might be ugly, but we handle scrolls somehow
	    // anyway, so don't bother here..

	    preventDefault(event);
	  }
	};
	/**
	 * Test whether a point lies inside given 2D triangle
	 *
	 * @param   {vis.Point2d}   point
	 * @param   {vis.Point2d[]} triangle
	 * @returns {boolean}   true if given point lies inside or on the edge of the
	 *                      triangle, false otherwise
	 * @private
	 */


	Graph3d.prototype._insideTriangle = function (point, triangle) {
	  var a = triangle[0],
	      b = triangle[1],
	      c = triangle[2];
	  /**
	   *
	   * @param {number} x
	   * @returns {number}
	   */

	  function sign(x) {
	    return x > 0 ? 1 : x < 0 ? -1 : 0;
	  }

	  var as = sign((b.x - a.x) * (point.y - a.y) - (b.y - a.y) * (point.x - a.x));
	  var bs = sign((c.x - b.x) * (point.y - b.y) - (c.y - b.y) * (point.x - b.x));
	  var cs = sign((a.x - c.x) * (point.y - c.y) - (a.y - c.y) * (point.x - c.x)); // each of the three signs must be either equal to each other or zero

	  return (as == 0 || bs == 0 || as == bs) && (bs == 0 || cs == 0 || bs == cs) && (as == 0 || cs == 0 || as == cs);
	};
	/**
	 * Find a data point close to given screen position (x, y)
	 *
	 * @param   {number} x
	 * @param   {number} y
	 * @returns {Object | null} The closest data point or null if not close to any
	 *                          data point
	 * @private
	 */


	Graph3d.prototype._dataPointFromXY = function (x, y) {
	  var i,
	      distMax = 100,
	      // px
	  dataPoint = null,
	      closestDataPoint = null,
	      closestDist = null,
	      center = new Point2d_1(x, y);

	  if (this.style === Graph3d.STYLE.BAR || this.style === Graph3d.STYLE.BARCOLOR || this.style === Graph3d.STYLE.BARSIZE) {
	    // the data points are ordered from far away to closest
	    for (i = this.dataPoints.length - 1; i >= 0; i--) {
	      dataPoint = this.dataPoints[i];
	      var surfaces = dataPoint.surfaces;

	      if (surfaces) {
	        for (var s = surfaces.length - 1; s >= 0; s--) {
	          // split each surface in two triangles, and see if the center point is inside one of these
	          var surface = surfaces[s];
	          var corners = surface.corners;
	          var triangle1 = [corners[0].screen, corners[1].screen, corners[2].screen];
	          var triangle2 = [corners[2].screen, corners[3].screen, corners[0].screen];

	          if (this._insideTriangle(center, triangle1) || this._insideTriangle(center, triangle2)) {
	            // return immediately at the first hit
	            return dataPoint;
	          }
	        }
	      }
	    }
	  } else {
	    // find the closest data point, using distance to the center of the point on 2d screen
	    for (i = 0; i < this.dataPoints.length; i++) {
	      dataPoint = this.dataPoints[i];
	      var point = dataPoint.screen;

	      if (point) {
	        var distX = Math.abs(x - point.x);
	        var distY = Math.abs(y - point.y);
	        var dist = Math.sqrt(distX * distX + distY * distY);

	        if ((closestDist === null || dist < closestDist) && dist < distMax) {
	          closestDist = dist;
	          closestDataPoint = dataPoint;
	        }
	      }
	    }
	  }

	  return closestDataPoint;
	};
	/**
	 * Determine if the given style has bars
	 *
	 * @param   {number} style the style to check
	 * @returns {boolean} true if bar style, false otherwise
	 */


	Graph3d.prototype.hasBars = function (style) {
	  return style == Graph3d.STYLE.BAR || style == Graph3d.STYLE.BARCOLOR || style == Graph3d.STYLE.BARSIZE;
	};
	/**
	 * Display a tooltip for given data point
	 * @param {Object} dataPoint
	 * @private
	 */


	Graph3d.prototype._showTooltip = function (dataPoint) {
	  var content, line, dot;

	  if (!this.tooltip) {
	    content = document.createElement('div');

	    assign$2(content.style, {}, this.tooltipStyle.content);

	    content.style.position = 'absolute';
	    line = document.createElement('div');

	    assign$2(line.style, {}, this.tooltipStyle.line);

	    line.style.position = 'absolute';
	    dot = document.createElement('div');

	    assign$2(dot.style, {}, this.tooltipStyle.dot);

	    dot.style.position = 'absolute';
	    this.tooltip = {
	      dataPoint: null,
	      dom: {
	        content: content,
	        line: line,
	        dot: dot
	      }
	    };
	  } else {
	    content = this.tooltip.dom.content;
	    line = this.tooltip.dom.line;
	    dot = this.tooltip.dom.dot;
	  }

	  this._hideTooltip();

	  this.tooltip.dataPoint = dataPoint;

	  if (typeof this.showTooltip === 'function') {
	    content.innerHTML = this.showTooltip(dataPoint.point);
	  } else {
	    content.innerHTML = '<table>' + '<tr><td>' + this.xLabel + ':</td><td>' + dataPoint.point.x + '</td></tr>' + '<tr><td>' + this.yLabel + ':</td><td>' + dataPoint.point.y + '</td></tr>' + '<tr><td>' + this.zLabel + ':</td><td>' + dataPoint.point.z + '</td></tr>' + '</table>';
	  }

	  content.style.left = '0';
	  content.style.top = '0';
	  this.frame.appendChild(content);
	  this.frame.appendChild(line);
	  this.frame.appendChild(dot); // calculate sizes

	  var contentWidth = content.offsetWidth;
	  var contentHeight = content.offsetHeight;
	  var lineHeight = line.offsetHeight;
	  var dotWidth = dot.offsetWidth;
	  var dotHeight = dot.offsetHeight;
	  var left = dataPoint.screen.x - contentWidth / 2;
	  left = Math.min(Math.max(left, 10), this.frame.clientWidth - 10 - contentWidth);
	  line.style.left = dataPoint.screen.x + 'px';
	  line.style.top = dataPoint.screen.y - lineHeight + 'px';
	  content.style.left = left + 'px';
	  content.style.top = dataPoint.screen.y - lineHeight - contentHeight + 'px';
	  dot.style.left = dataPoint.screen.x - dotWidth / 2 + 'px';
	  dot.style.top = dataPoint.screen.y - dotHeight / 2 + 'px';
	};
	/**
	 * Hide the tooltip when displayed
	 * @private
	 */


	Graph3d.prototype._hideTooltip = function () {
	  if (this.tooltip) {
	    this.tooltip.dataPoint = null;

	    for (var prop in this.tooltip.dom) {
	      if (this.tooltip.dom.hasOwnProperty(prop)) {
	        var elem = this.tooltip.dom[prop];

	        if (elem && elem.parentNode) {
	          elem.parentNode.removeChild(elem);
	        }
	      }
	    }
	  }
	};
	/**--------------------------------------------------------------------------**/

	/**
	 * Get the horizontal mouse position from a mouse event
	 *
	 * @param   {Event}  event
	 * @returns {number} mouse x
	 */


	function getMouseX(event) {
	  if ('clientX' in event) return event.clientX;
	  return event.targetTouches[0] && event.targetTouches[0].clientX || 0;
	}
	/**
	 * Get the vertical mouse position from a mouse event
	 *
	 * @param   {Event}  event
	 * @returns {number} mouse y
	 */


	function getMouseY(event) {
	  if ('clientY' in event) return event.clientY;
	  return event.targetTouches[0] && event.targetTouches[0].clientY || 0;
	} // -----------------------------------------------------------------------------
	//  Public methods for specific settings
	// -----------------------------------------------------------------------------

	/**
	 * Set the rotation and distance of the camera
	 *
	 * @param {Object}  pos            An object with the camera position
	 * @param {number} [pos.horizontal] The horizontal rotation, between 0 and 2*PI.
	 *                                 Optional, can be left undefined.
	 * @param {number} [pos.vertical]  The vertical rotation, between 0 and 0.5*PI.
	 *                                 if vertical=0.5*PI, the graph is shown from
	 *                                 the top. Optional, can be left undefined.
	 * @param {number} [pos.distance]  The (normalized) distance of the camera to the
	 *                                 center of the graph, a value between 0.71 and
	 *                                 5.0. Optional, can be left undefined.
	 */


	Graph3d.prototype.setCameraPosition = function (pos) {
	  setCameraPosition(pos, this);
	  this.redraw();
	};
	/**
	 * Set a new size for the graph
	 *
	 * @param {string} width  Width in pixels or percentage (for example '800px'
	 *                        or '50%')
	 * @param {string} height Height in pixels or percentage  (for example '400px'
	 *                        or '30%')
	 */


	Graph3d.prototype.setSize = function (width, height) {
	  this._setSize(width, height);

	  this.redraw();
	};

	var nsGraph3d = /*#__PURE__*/Object.freeze({
		__proto__: null,
		Graph3d: Graph3d,
		Graph3dCamera: Camera,
		Graph3dFilter: Filter,
		Graph3dPoint2d: Point2d_1,
		Graph3dPoint3d: Point3d_1,
		Graph3dSlider: Slider,
		Graph3dStepNumber: StepNumber_1
	});

	var $every = arrayIteration.every;
	var STRICT_METHOD$5 = arrayMethodIsStrict('every');
	var USES_TO_LENGTH$8 = arrayMethodUsesToLength('every'); // `Array.prototype.every` method
	// https://tc39.github.io/ecma262/#sec-array.prototype.every

	_export({
	  target: 'Array',
	  proto: true,
	  forced: !STRICT_METHOD$5 || !USES_TO_LENGTH$8
	}, {
	  every: function every(callbackfn
	  /* , thisArg */
	  ) {
	    return $every(this, callbackfn, arguments.length > 1 ? arguments[1] : undefined);
	  }
	});

	var every = entryVirtual('Array').every;

	var ArrayPrototype$h = Array.prototype;

	var every_1 = function (it) {
	  var own = it.every;
	  return it === ArrayPrototype$h || it instanceof Array && own === ArrayPrototype$h.every ? every : own;
	};

	var every$1 = every_1;

	var every$2 = every$1;

	var setInterval = path.setInterval;

	var setInterval$1 = setInterval;

	var getOwnPropertyDescriptor$4 = getOwnPropertyDescriptor_1;

	var getOwnPropertyDescriptor$5 = getOwnPropertyDescriptor$4;

	// https://tc39.github.io/ecma262/#sec-reflect.get

	function get$1(target, propertyKey
	/* , receiver */
	) {
	  var receiver = arguments.length < 3 ? target : arguments[2];
	  var descriptor, prototype;
	  if (anObject(target) === receiver) return target[propertyKey];
	  if (descriptor = objectGetOwnPropertyDescriptor.f(target, propertyKey)) return has(descriptor, 'value') ? descriptor.value : descriptor.get === undefined ? undefined : descriptor.get.call(receiver);
	  if (isObject(prototype = objectGetPrototypeOf(target))) return get$1(prototype, propertyKey, receiver);
	}

	_export({
	  target: 'Reflect',
	  stat: true
	}, {
	  get: get$1
	});

	var get$2 = path.Reflect.get;

	var get$3 = get$2;

	var get$4 = get$3;

	function _superPropBase(object, property) {
	  while (!Object.prototype.hasOwnProperty.call(object, property)) {
	    object = getPrototypeOf$3(object);
	    if (object === null) break;
	  }

	  return object;
	}

	var superPropBase = _superPropBase;

	var get$5 = createCommonjsModule(function (module) {
	  function _get(target, property, receiver) {
	    if (typeof Reflect !== "undefined" && get$4) {
	      module.exports = _get = get$4;
	    } else {
	      module.exports = _get = function _get(target, property, receiver) {
	        var base = superPropBase(target, property);
	        if (!base) return;

	        var desc = getOwnPropertyDescriptor$5(base, property);

	        if (desc.get) {
	          return desc.get.call(receiver);
	        }

	        return desc.value;
	      };
	    }

	    return _get(target, property, receiver || target);
	  }

	  module.exports = _get;
	});

	var $hypot = Math.hypot;
	var abs = Math.abs;
	var sqrt = Math.sqrt; // Chrome 77 bug
	// https://bugs.chromium.org/p/v8/issues/detail?id=9546

	var BUGGY = !!$hypot && $hypot(Infinity, NaN) !== Infinity; // `Math.hypot` method
	// https://tc39.github.io/ecma262/#sec-math.hypot

	_export({
	  target: 'Math',
	  stat: true,
	  forced: BUGGY
	}, {
	  hypot: function hypot(value1, value2) {
	    // eslint-disable-line no-unused-vars
	    var sum = 0;
	    var i = 0;
	    var aLen = arguments.length;
	    var larg = 0;
	    var arg, div;

	    while (i < aLen) {
	      arg = abs(arguments[i++]);

	      if (larg < arg) {
	        div = larg / arg;
	        sum = sum * div * div + 1;
	        larg = arg;
	      } else if (arg > 0) {
	        div = arg / larg;
	        sum += div * div;
	      } else sum += arg;
	    }

	    return larg === Infinity ? Infinity : larg * sqrt(sum);
	  }
	});

	var hypot = path.Math.hypot;

	var hypot$1 = hypot;

	var hypot$2 = hypot$1;

	var globalIsFinite = global_1.isFinite; // `Number.isFinite` method
	// https://tc39.github.io/ecma262/#sec-number.isfinite

	var numberIsFinite = Number.isFinite || function isFinite(it) {
	  return typeof it == 'number' && globalIsFinite(it);
	};

	// https://tc39.github.io/ecma262/#sec-number.isfinite

	_export({
	  target: 'Number',
	  stat: true
	}, {
	  isFinite: numberIsFinite
	});

	var _isFinite = path.Number.isFinite;

	var _isFinite$1 = _isFinite;

	var _isFinite$2 = _isFinite$1;

	var nativeGetOwnPropertyNames$2 = objectGetOwnPropertyNamesExternal.f;
	var FAILS_ON_PRIMITIVES$3 = fails(function () {
	  return !Object.getOwnPropertyNames(1);
	}); // `Object.getOwnPropertyNames` method
	// https://tc39.github.io/ecma262/#sec-object.getownpropertynames

	_export({
	  target: 'Object',
	  stat: true,
	  forced: FAILS_ON_PRIMITIVES$3
	}, {
	  getOwnPropertyNames: nativeGetOwnPropertyNames$2
	});

	var Object$2 = path.Object;

	var getOwnPropertyNames = function getOwnPropertyNames(it) {
	  return Object$2.getOwnPropertyNames(it);
	};

	var getOwnPropertyNames$1 = getOwnPropertyNames;

	var getOwnPropertyNames$2 = getOwnPropertyNames$1;

	var setPrototypeOf$4 = setPrototypeOf;

	var setPrototypeOf$5 = setPrototypeOf$4;

	var onFreeze = internalMetadata.onFreeze;
	var nativeFreeze = Object.freeze;
	var FAILS_ON_PRIMITIVES$4 = fails(function () {
	  nativeFreeze(1);
	}); // `Object.freeze` method
	// https://tc39.github.io/ecma262/#sec-object.freeze

	_export({
	  target: 'Object',
	  stat: true,
	  forced: FAILS_ON_PRIMITIVES$4,
	  sham: !freezing
	}, {
	  freeze: function freeze(it) {
	    return nativeFreeze && isObject(it) ? nativeFreeze(onFreeze(it)) : it;
	  }
	});

	var freeze = path.Object.freeze;

	var freeze$1 = freeze;

	var freeze$2 = freeze$1;

	var $includes = arrayIncludes.includes;
	var USES_TO_LENGTH$9 = arrayMethodUsesToLength('indexOf', {
	  ACCESSORS: true,
	  1: 0
	}); // `Array.prototype.includes` method
	// https://tc39.github.io/ecma262/#sec-array.prototype.includes

	_export({
	  target: 'Array',
	  proto: true,
	  forced: !USES_TO_LENGTH$9
	}, {
	  includes: function includes(el
	  /* , fromIndex = 0 */
	  ) {
	    return $includes(this, el, arguments.length > 1 ? arguments[1] : undefined);
	  }
	}); // https://tc39.github.io/ecma262/#sec-array.prototype-@@unscopables

	var includes = entryVirtual('Array').includes;

	var MATCH = wellKnownSymbol('match'); // `IsRegExp` abstract operation
	// https://tc39.github.io/ecma262/#sec-isregexp

	var isRegexp = function (it) {
	  var isRegExp;
	  return isObject(it) && ((isRegExp = it[MATCH]) !== undefined ? !!isRegExp : classofRaw(it) == 'RegExp');
	};

	var notARegexp = function (it) {
	  if (isRegexp(it)) {
	    throw TypeError("The method doesn't accept regular expressions");
	  }

	  return it;
	};

	var MATCH$1 = wellKnownSymbol('match');

	var correctIsRegexpLogic = function (METHOD_NAME) {
	  var regexp = /./;

	  try {
	    '/./'[METHOD_NAME](regexp);
	  } catch (e) {
	    try {
	      regexp[MATCH$1] = false;
	      return '/./'[METHOD_NAME](regexp);
	    } catch (f) {
	      /* empty */
	    }
	  }

	  return false;
	};

	// https://tc39.github.io/ecma262/#sec-string.prototype.includes


	_export({
	  target: 'String',
	  proto: true,
	  forced: !correctIsRegexpLogic('includes')
	}, {
	  includes: function includes(searchString
	  /* , position = 0 */
	  ) {
	    return !!~String(requireObjectCoercible(this)).indexOf(notARegexp(searchString), arguments.length > 1 ? arguments[1] : undefined);
	  }
	});

	var includes$1 = entryVirtual('String').includes;

	var ArrayPrototype$i = Array.prototype;
	var StringPrototype$1 = String.prototype;

	var includes$2 = function (it) {
	  var own = it.includes;
	  if (it === ArrayPrototype$i || it instanceof Array && own === ArrayPrototype$i.includes) return includes;

	  if (typeof it === 'string' || it === StringPrototype$1 || it instanceof String && own === StringPrototype$1.includes) {
	    return includes$1;
	  }

	  return own;
	};

	var includes$3 = includes$2;

	var includes$4 = includes$3;

	var keycharm = createCommonjsModule(function (module, exports) {
	  /**
	   * Created by Alex on 11/6/2014.
	   */
	  // https://github.com/umdjs/umd/blob/master/returnExports.js#L40-L60
	  // if the module has no dependencies, the above pattern can be simplified to

	  (function (root, factory) {
	    {
	      // Node. Does not work with strict CommonJS, but
	      // only CommonJS-like environments that support module.exports,
	      // like Node.
	      module.exports = factory();
	    }
	  })(commonjsGlobal, function () {
	    function keycharm(options) {
	      var preventDefault = options && options.preventDefault || false;
	      var container = options && options.container || window;
	      var _exportFunctions = {};
	      var _bound = {
	        keydown: {},
	        keyup: {}
	      };
	      var _keys = {};
	      var i; // a - z

	      for (i = 97; i <= 122; i++) {
	        _keys[String.fromCharCode(i)] = {
	          code: 65 + (i - 97),
	          shift: false
	        };
	      } // A - Z


	      for (i = 65; i <= 90; i++) {
	        _keys[String.fromCharCode(i)] = {
	          code: i,
	          shift: true
	        };
	      } // 0 - 9


	      for (i = 0; i <= 9; i++) {
	        _keys['' + i] = {
	          code: 48 + i,
	          shift: false
	        };
	      } // F1 - F12


	      for (i = 1; i <= 12; i++) {
	        _keys['F' + i] = {
	          code: 111 + i,
	          shift: false
	        };
	      } // num0 - num9


	      for (i = 0; i <= 9; i++) {
	        _keys['num' + i] = {
	          code: 96 + i,
	          shift: false
	        };
	      } // numpad misc


	      _keys['num*'] = {
	        code: 106,
	        shift: false
	      };
	      _keys['num+'] = {
	        code: 107,
	        shift: false
	      };
	      _keys['num-'] = {
	        code: 109,
	        shift: false
	      };
	      _keys['num/'] = {
	        code: 111,
	        shift: false
	      };
	      _keys['num.'] = {
	        code: 110,
	        shift: false
	      }; // arrows

	      _keys['left'] = {
	        code: 37,
	        shift: false
	      };
	      _keys['up'] = {
	        code: 38,
	        shift: false
	      };
	      _keys['right'] = {
	        code: 39,
	        shift: false
	      };
	      _keys['down'] = {
	        code: 40,
	        shift: false
	      }; // extra keys

	      _keys['space'] = {
	        code: 32,
	        shift: false
	      };
	      _keys['enter'] = {
	        code: 13,
	        shift: false
	      };
	      _keys['shift'] = {
	        code: 16,
	        shift: undefined
	      };
	      _keys['esc'] = {
	        code: 27,
	        shift: false
	      };
	      _keys['backspace'] = {
	        code: 8,
	        shift: false
	      };
	      _keys['tab'] = {
	        code: 9,
	        shift: false
	      };
	      _keys['ctrl'] = {
	        code: 17,
	        shift: false
	      };
	      _keys['alt'] = {
	        code: 18,
	        shift: false
	      };
	      _keys['delete'] = {
	        code: 46,
	        shift: false
	      };
	      _keys['pageup'] = {
	        code: 33,
	        shift: false
	      };
	      _keys['pagedown'] = {
	        code: 34,
	        shift: false
	      }; // symbols

	      _keys['='] = {
	        code: 187,
	        shift: false
	      };
	      _keys['-'] = {
	        code: 189,
	        shift: false
	      };
	      _keys[']'] = {
	        code: 221,
	        shift: false
	      };
	      _keys['['] = {
	        code: 219,
	        shift: false
	      };

	      var down = function (event) {
	        handleEvent(event, 'keydown');
	      };

	      var up = function (event) {
	        handleEvent(event, 'keyup');
	      }; // handle the actualy bound key with the event


	      var handleEvent = function (event, type) {
	        if (_bound[type][event.keyCode] !== undefined) {
	          var bound = _bound[type][event.keyCode];

	          for (var i = 0; i < bound.length; i++) {
	            if (bound[i].shift === undefined) {
	              bound[i].fn(event);
	            } else if (bound[i].shift == true && event.shiftKey == true) {
	              bound[i].fn(event);
	            } else if (bound[i].shift == false && event.shiftKey == false) {
	              bound[i].fn(event);
	            }
	          }

	          if (preventDefault == true) {
	            event.preventDefault();
	          }
	        }
	      }; // bind a key to a callback


	      _exportFunctions.bind = function (key, callback, type) {
	        if (type === undefined) {
	          type = 'keydown';
	        }

	        if (_keys[key] === undefined) {
	          throw new Error("unsupported key: " + key);
	        }

	        if (_bound[type][_keys[key].code] === undefined) {
	          _bound[type][_keys[key].code] = [];
	        }

	        _bound[type][_keys[key].code].push({
	          fn: callback,
	          shift: _keys[key].shift
	        });
	      }; // bind all keys to a call back (demo purposes)


	      _exportFunctions.bindAll = function (callback, type) {
	        if (type === undefined) {
	          type = 'keydown';
	        }

	        for (var key in _keys) {
	          if (_keys.hasOwnProperty(key)) {
	            _exportFunctions.bind(key, callback, type);
	          }
	        }
	      }; // get the key label from an event


	      _exportFunctions.getKey = function (event) {
	        for (var key in _keys) {
	          if (_keys.hasOwnProperty(key)) {
	            if (event.shiftKey == true && _keys[key].shift == true && event.keyCode == _keys[key].code) {
	              return key;
	            } else if (event.shiftKey == false && _keys[key].shift == false && event.keyCode == _keys[key].code) {
	              return key;
	            } else if (event.keyCode == _keys[key].code && key == 'shift') {
	              return key;
	            }
	          }
	        }

	        return "unknown key, currently not supported";
	      }; // unbind either a specific callback from a key or all of them (by leaving callback undefined)


	      _exportFunctions.unbind = function (key, callback, type) {
	        if (type === undefined) {
	          type = 'keydown';
	        }

	        if (_keys[key] === undefined) {
	          throw new Error("unsupported key: " + key);
	        }

	        if (callback !== undefined) {
	          var newBindings = [];
	          var bound = _bound[type][_keys[key].code];

	          if (bound !== undefined) {
	            for (var i = 0; i < bound.length; i++) {
	              if (!(bound[i].fn == callback && bound[i].shift == _keys[key].shift)) {
	                newBindings.push(_bound[type][_keys[key].code][i]);
	              }
	            }
	          }

	          _bound[type][_keys[key].code] = newBindings;
	        } else {
	          _bound[type][_keys[key].code] = [];
	        }
	      }; // reset all bound variables.


	      _exportFunctions.reset = function () {
	        _bound = {
	          keydown: {},
	          keyup: {}
	        };
	      }; // unbind all listeners and reset all variables.


	      _exportFunctions.destroy = function () {
	        _bound = {
	          keydown: {},
	          keyup: {}
	        };
	        container.removeEventListener('keydown', down, true);
	        container.removeEventListener('keyup', up, true);
	      }; // create listeners.


	      container.addEventListener('keydown', down, true);
	      container.addEventListener('keyup', up, true); // return the public functions.

	      return _exportFunctions;
	    }

	    return keycharm;
	  });
	});

	var keycharm$1 = /*#__PURE__*/Object.freeze({
		__proto__: null,
		'default': keycharm,
		__moduleExports: keycharm
	});

	/*! Hammer.JS - v2.0.17-rc - 2019-12-16
	 * http://naver.github.io/egjs
	 *
	 * Forked By Naver egjs
	 * Copyright (c) hammerjs
	 * Licensed under the MIT license */
	function _extends() {
	  _extends = Object.assign || function (target) {
	    for (var i = 1; i < arguments.length; i++) {
	      var source = arguments[i];

	      for (var key in source) {
	        if (Object.prototype.hasOwnProperty.call(source, key)) {
	          target[key] = source[key];
	        }
	      }
	    }

	    return target;
	  };

	  return _extends.apply(this, arguments);
	}

	function _inheritsLoose(subClass, superClass) {
	  subClass.prototype = Object.create(superClass.prototype);
	  subClass.prototype.constructor = subClass;
	  subClass.__proto__ = superClass;
	}

	function _assertThisInitialized$1(self) {
	  if (self === void 0) {
	    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
	  }

	  return self;
	}
	/**
	 * @private
	 * extend object.
	 * means that properties in dest will be overwritten by the ones in src.
	 * @param {Object} target
	 * @param {...Object} objects_to_assign
	 * @returns {Object} target
	 */


	var assign$3;

	if (typeof Object.assign !== 'function') {
	  assign$3 = function assign(target) {
	    if (target === undefined || target === null) {
	      throw new TypeError('Cannot convert undefined or null to object');
	    }

	    var output = Object(target);

	    for (var index = 1; index < arguments.length; index++) {
	      var source = arguments[index];

	      if (source !== undefined && source !== null) {
	        for (var nextKey in source) {
	          if (source.hasOwnProperty(nextKey)) {
	            output[nextKey] = source[nextKey];
	          }
	        }
	      }
	    }

	    return output;
	  };
	} else {
	  assign$3 = Object.assign;
	}

	var assign$1$1 = assign$3;
	var VENDOR_PREFIXES = ['', 'webkit', 'Moz', 'MS', 'ms', 'o'];
	var TEST_ELEMENT = typeof document === "undefined" ? {
	  style: {}
	} : document.createElement('div');
	var TYPE_FUNCTION = 'function';
	var round = Math.round,
	    abs$1 = Math.abs;
	var now$3 = Date.now;
	/**
	 * @private
	 * get the prefixed property
	 * @param {Object} obj
	 * @param {String} property
	 * @returns {String|Undefined} prefixed
	 */

	function prefixed(obj, property) {
	  var prefix;
	  var prop;
	  var camelProp = property[0].toUpperCase() + property.slice(1);
	  var i = 0;

	  while (i < VENDOR_PREFIXES.length) {
	    prefix = VENDOR_PREFIXES[i];
	    prop = prefix ? prefix + camelProp : property;

	    if (prop in obj) {
	      return prop;
	    }

	    i++;
	  }

	  return undefined;
	}
	/* eslint-disable no-new-func, no-nested-ternary */


	var win;

	if (typeof window === "undefined") {
	  // window is undefined in node.js
	  win = {};
	} else {
	  win = window;
	}

	var PREFIXED_TOUCH_ACTION = prefixed(TEST_ELEMENT.style, 'touchAction');
	var NATIVE_TOUCH_ACTION = PREFIXED_TOUCH_ACTION !== undefined;

	function getTouchActionProps() {
	  if (!NATIVE_TOUCH_ACTION) {
	    return false;
	  }

	  var touchMap = {};
	  var cssSupports = win.CSS && win.CSS.supports;
	  ['auto', 'manipulation', 'pan-y', 'pan-x', 'pan-x pan-y', 'none'].forEach(function (val) {
	    // If css.supports is not supported but there is native touch-action assume it supports
	    // all values. This is the case for IE 10 and 11.
	    return touchMap[val] = cssSupports ? win.CSS.supports('touch-action', val) : true;
	  });
	  return touchMap;
	}

	var TOUCH_ACTION_COMPUTE = 'compute';
	var TOUCH_ACTION_AUTO = 'auto';
	var TOUCH_ACTION_MANIPULATION = 'manipulation'; // not implemented

	var TOUCH_ACTION_NONE = 'none';
	var TOUCH_ACTION_PAN_X = 'pan-x';
	var TOUCH_ACTION_PAN_Y = 'pan-y';
	var TOUCH_ACTION_MAP = getTouchActionProps();
	var MOBILE_REGEX = /mobile|tablet|ip(ad|hone|od)|android/i;
	var SUPPORT_TOUCH = ('ontouchstart' in win);
	var SUPPORT_POINTER_EVENTS = prefixed(win, 'PointerEvent') !== undefined;
	var SUPPORT_ONLY_TOUCH = SUPPORT_TOUCH && MOBILE_REGEX.test(navigator.userAgent);
	var INPUT_TYPE_TOUCH = 'touch';
	var INPUT_TYPE_PEN = 'pen';
	var INPUT_TYPE_MOUSE = 'mouse';
	var INPUT_TYPE_KINECT = 'kinect';
	var COMPUTE_INTERVAL = 25;
	var INPUT_START = 1;
	var INPUT_MOVE = 2;
	var INPUT_END = 4;
	var INPUT_CANCEL = 8;
	var DIRECTION_NONE = 1;
	var DIRECTION_LEFT = 2;
	var DIRECTION_RIGHT = 4;
	var DIRECTION_UP = 8;
	var DIRECTION_DOWN = 16;
	var DIRECTION_HORIZONTAL = DIRECTION_LEFT | DIRECTION_RIGHT;
	var DIRECTION_VERTICAL = DIRECTION_UP | DIRECTION_DOWN;
	var DIRECTION_ALL = DIRECTION_HORIZONTAL | DIRECTION_VERTICAL;
	var PROPS_XY = ['x', 'y'];
	var PROPS_CLIENT_XY = ['clientX', 'clientY'];
	/**
	 * @private
	 * walk objects and arrays
	 * @param {Object} obj
	 * @param {Function} iterator
	 * @param {Object} context
	 */

	function each(obj, iterator, context) {
	  var i;

	  if (!obj) {
	    return;
	  }

	  if (obj.forEach) {
	    obj.forEach(iterator, context);
	  } else if (obj.length !== undefined) {
	    i = 0;

	    while (i < obj.length) {
	      iterator.call(context, obj[i], i, obj);
	      i++;
	    }
	  } else {
	    for (i in obj) {
	      obj.hasOwnProperty(i) && iterator.call(context, obj[i], i, obj);
	    }
	  }
	}
	/**
	 * @private
	 * let a boolean value also be a function that must return a boolean
	 * this first item in args will be used as the context
	 * @param {Boolean|Function} val
	 * @param {Array} [args]
	 * @returns {Boolean}
	 */


	function boolOrFn(val, args) {
	  if (typeof val === TYPE_FUNCTION) {
	    return val.apply(args ? args[0] || undefined : undefined, args);
	  }

	  return val;
	}
	/**
	 * @private
	 * small indexOf wrapper
	 * @param {String} str
	 * @param {String} find
	 * @returns {Boolean} found
	 */


	function inStr(str, find) {
	  return str.indexOf(find) > -1;
	}
	/**
	 * @private
	 * when the touchActions are collected they are not a valid value, so we need to clean things up. *
	 * @param {String} actions
	 * @returns {*}
	 */


	function cleanTouchActions(actions) {
	  // none
	  if (inStr(actions, TOUCH_ACTION_NONE)) {
	    return TOUCH_ACTION_NONE;
	  }

	  var hasPanX = inStr(actions, TOUCH_ACTION_PAN_X);
	  var hasPanY = inStr(actions, TOUCH_ACTION_PAN_Y); // if both pan-x and pan-y are set (different recognizers
	  // for different directions, e.g. horizontal pan but vertical swipe?)
	  // we need none (as otherwise with pan-x pan-y combined none of these
	  // recognizers will work, since the browser would handle all panning

	  if (hasPanX && hasPanY) {
	    return TOUCH_ACTION_NONE;
	  } // pan-x OR pan-y


	  if (hasPanX || hasPanY) {
	    return hasPanX ? TOUCH_ACTION_PAN_X : TOUCH_ACTION_PAN_Y;
	  } // manipulation


	  if (inStr(actions, TOUCH_ACTION_MANIPULATION)) {
	    return TOUCH_ACTION_MANIPULATION;
	  }

	  return TOUCH_ACTION_AUTO;
	}
	/**
	 * @private
	 * Touch Action
	 * sets the touchAction property or uses the js alternative
	 * @param {Manager} manager
	 * @param {String} value
	 * @constructor
	 */


	var TouchAction = /*#__PURE__*/function () {
	  function TouchAction(manager, value) {
	    this.manager = manager;
	    this.set(value);
	  }
	  /**
	   * @private
	   * set the touchAction value on the element or enable the polyfill
	   * @param {String} value
	   */


	  var _proto = TouchAction.prototype;

	  _proto.set = function set(value) {
	    // find out the touch-action by the event handlers
	    if (value === TOUCH_ACTION_COMPUTE) {
	      value = this.compute();
	    }

	    if (NATIVE_TOUCH_ACTION && this.manager.element.style && TOUCH_ACTION_MAP[value]) {
	      this.manager.element.style[PREFIXED_TOUCH_ACTION] = value;
	    }

	    this.actions = value.toLowerCase().trim();
	  };
	  /**
	   * @private
	   * just re-set the touchAction value
	   */


	  _proto.update = function update() {
	    this.set(this.manager.options.touchAction);
	  };
	  /**
	   * @private
	   * compute the value for the touchAction property based on the recognizer's settings
	   * @returns {String} value
	   */


	  _proto.compute = function compute() {
	    var actions = [];
	    each(this.manager.recognizers, function (recognizer) {
	      if (boolOrFn(recognizer.options.enable, [recognizer])) {
	        actions = actions.concat(recognizer.getTouchAction());
	      }
	    });
	    return cleanTouchActions(actions.join(' '));
	  };
	  /**
	   * @private
	   * this method is called on each input cycle and provides the preventing of the browser behavior
	   * @param {Object} input
	   */


	  _proto.preventDefaults = function preventDefaults(input) {
	    var srcEvent = input.srcEvent;
	    var direction = input.offsetDirection; // if the touch action did prevented once this session

	    if (this.manager.session.prevented) {
	      srcEvent.preventDefault();
	      return;
	    }

	    var actions = this.actions;
	    var hasNone = inStr(actions, TOUCH_ACTION_NONE) && !TOUCH_ACTION_MAP[TOUCH_ACTION_NONE];
	    var hasPanY = inStr(actions, TOUCH_ACTION_PAN_Y) && !TOUCH_ACTION_MAP[TOUCH_ACTION_PAN_Y];
	    var hasPanX = inStr(actions, TOUCH_ACTION_PAN_X) && !TOUCH_ACTION_MAP[TOUCH_ACTION_PAN_X];

	    if (hasNone) {
	      // do not prevent defaults if this is a tap gesture
	      var isTapPointer = input.pointers.length === 1;
	      var isTapMovement = input.distance < 2;
	      var isTapTouchTime = input.deltaTime < 250;

	      if (isTapPointer && isTapMovement && isTapTouchTime) {
	        return;
	      }
	    }

	    if (hasPanX && hasPanY) {
	      // `pan-x pan-y` means browser handles all scrolling/panning, do not prevent
	      return;
	    }

	    if (hasNone || hasPanY && direction & DIRECTION_HORIZONTAL || hasPanX && direction & DIRECTION_VERTICAL) {
	      return this.preventSrc(srcEvent);
	    }
	  };
	  /**
	   * @private
	   * call preventDefault to prevent the browser's default behavior (scrolling in most cases)
	   * @param {Object} srcEvent
	   */


	  _proto.preventSrc = function preventSrc(srcEvent) {
	    this.manager.session.prevented = true;
	    srcEvent.preventDefault();
	  };

	  return TouchAction;
	}();
	/**
	 * @private
	 * find if a node is in the given parent
	 * @method hasParent
	 * @param {HTMLElement} node
	 * @param {HTMLElement} parent
	 * @return {Boolean} found
	 */


	function hasParent$1(node, parent) {
	  while (node) {
	    if (node === parent) {
	      return true;
	    }

	    node = node.parentNode;
	  }

	  return false;
	}
	/**
	 * @private
	 * get the center of all the pointers
	 * @param {Array} pointers
	 * @return {Object} center contains `x` and `y` properties
	 */


	function getCenter(pointers) {
	  var pointersLength = pointers.length; // no need to loop when only one touch

	  if (pointersLength === 1) {
	    return {
	      x: round(pointers[0].clientX),
	      y: round(pointers[0].clientY)
	    };
	  }

	  var x = 0;
	  var y = 0;
	  var i = 0;

	  while (i < pointersLength) {
	    x += pointers[i].clientX;
	    y += pointers[i].clientY;
	    i++;
	  }

	  return {
	    x: round(x / pointersLength),
	    y: round(y / pointersLength)
	  };
	}
	/**
	 * @private
	 * create a simple clone from the input used for storage of firstInput and firstMultiple
	 * @param {Object} input
	 * @returns {Object} clonedInputData
	 */


	function simpleCloneInputData(input) {
	  // make a simple copy of the pointers because we will get a reference if we don't
	  // we only need clientXY for the calculations
	  var pointers = [];
	  var i = 0;

	  while (i < input.pointers.length) {
	    pointers[i] = {
	      clientX: round(input.pointers[i].clientX),
	      clientY: round(input.pointers[i].clientY)
	    };
	    i++;
	  }

	  return {
	    timeStamp: now$3(),
	    pointers: pointers,
	    center: getCenter(pointers),
	    deltaX: input.deltaX,
	    deltaY: input.deltaY
	  };
	}
	/**
	 * @private
	 * calculate the absolute distance between two points
	 * @param {Object} p1 {x, y}
	 * @param {Object} p2 {x, y}
	 * @param {Array} [props] containing x and y keys
	 * @return {Number} distance
	 */


	function getDistance(p1, p2, props) {
	  if (!props) {
	    props = PROPS_XY;
	  }

	  var x = p2[props[0]] - p1[props[0]];
	  var y = p2[props[1]] - p1[props[1]];
	  return Math.sqrt(x * x + y * y);
	}
	/**
	 * @private
	 * calculate the angle between two coordinates
	 * @param {Object} p1
	 * @param {Object} p2
	 * @param {Array} [props] containing x and y keys
	 * @return {Number} angle
	 */


	function getAngle(p1, p2, props) {
	  if (!props) {
	    props = PROPS_XY;
	  }

	  var x = p2[props[0]] - p1[props[0]];
	  var y = p2[props[1]] - p1[props[1]];
	  return Math.atan2(y, x) * 180 / Math.PI;
	}
	/**
	 * @private
	 * get the direction between two points
	 * @param {Number} x
	 * @param {Number} y
	 * @return {Number} direction
	 */


	function getDirection(x, y) {
	  if (x === y) {
	    return DIRECTION_NONE;
	  }

	  if (abs$1(x) >= abs$1(y)) {
	    return x < 0 ? DIRECTION_LEFT : DIRECTION_RIGHT;
	  }

	  return y < 0 ? DIRECTION_UP : DIRECTION_DOWN;
	}

	function computeDeltaXY(session, input) {
	  var center = input.center; // let { offsetDelta:offset = {}, prevDelta = {}, prevInput = {} } = session;
	  // jscs throwing error on defalut destructured values and without defaults tests fail

	  var offset = session.offsetDelta || {};
	  var prevDelta = session.prevDelta || {};
	  var prevInput = session.prevInput || {};

	  if (input.eventType === INPUT_START || prevInput.eventType === INPUT_END) {
	    prevDelta = session.prevDelta = {
	      x: prevInput.deltaX || 0,
	      y: prevInput.deltaY || 0
	    };
	    offset = session.offsetDelta = {
	      x: center.x,
	      y: center.y
	    };
	  }

	  input.deltaX = prevDelta.x + (center.x - offset.x);
	  input.deltaY = prevDelta.y + (center.y - offset.y);
	}
	/**
	 * @private
	 * calculate the velocity between two points. unit is in px per ms.
	 * @param {Number} deltaTime
	 * @param {Number} x
	 * @param {Number} y
	 * @return {Object} velocity `x` and `y`
	 */


	function getVelocity(deltaTime, x, y) {
	  return {
	    x: x / deltaTime || 0,
	    y: y / deltaTime || 0
	  };
	}
	/**
	 * @private
	 * calculate the scale factor between two pointersets
	 * no scale is 1, and goes down to 0 when pinched together, and bigger when pinched out
	 * @param {Array} start array of pointers
	 * @param {Array} end array of pointers
	 * @return {Number} scale
	 */


	function getScale(start, end) {
	  return getDistance(end[0], end[1], PROPS_CLIENT_XY) / getDistance(start[0], start[1], PROPS_CLIENT_XY);
	}
	/**
	 * @private
	 * calculate the rotation degrees between two pointersets
	 * @param {Array} start array of pointers
	 * @param {Array} end array of pointers
	 * @return {Number} rotation
	 */


	function getRotation(start, end) {
	  return getAngle(end[1], end[0], PROPS_CLIENT_XY) + getAngle(start[1], start[0], PROPS_CLIENT_XY);
	}
	/**
	 * @private
	 * velocity is calculated every x ms
	 * @param {Object} session
	 * @param {Object} input
	 */


	function computeIntervalInputData(session, input) {
	  var last = session.lastInterval || input;
	  var deltaTime = input.timeStamp - last.timeStamp;
	  var velocity;
	  var velocityX;
	  var velocityY;
	  var direction;

	  if (input.eventType !== INPUT_CANCEL && (deltaTime > COMPUTE_INTERVAL || last.velocity === undefined)) {
	    var deltaX = input.deltaX - last.deltaX;
	    var deltaY = input.deltaY - last.deltaY;
	    var v = getVelocity(deltaTime, deltaX, deltaY);
	    velocityX = v.x;
	    velocityY = v.y;
	    velocity = abs$1(v.x) > abs$1(v.y) ? v.x : v.y;
	    direction = getDirection(deltaX, deltaY);
	    session.lastInterval = input;
	  } else {
	    // use latest velocity info if it doesn't overtake a minimum period
	    velocity = last.velocity;
	    velocityX = last.velocityX;
	    velocityY = last.velocityY;
	    direction = last.direction;
	  }

	  input.velocity = velocity;
	  input.velocityX = velocityX;
	  input.velocityY = velocityY;
	  input.direction = direction;
	}
	/**
	* @private
	 * extend the data with some usable properties like scale, rotate, velocity etc
	 * @param {Object} manager
	 * @param {Object} input
	 */


	function computeInputData(manager, input) {
	  var session = manager.session;
	  var pointers = input.pointers;
	  var pointersLength = pointers.length; // store the first input to calculate the distance and direction

	  if (!session.firstInput) {
	    session.firstInput = simpleCloneInputData(input);
	  } // to compute scale and rotation we need to store the multiple touches


	  if (pointersLength > 1 && !session.firstMultiple) {
	    session.firstMultiple = simpleCloneInputData(input);
	  } else if (pointersLength === 1) {
	    session.firstMultiple = false;
	  }

	  var firstInput = session.firstInput,
	      firstMultiple = session.firstMultiple;
	  var offsetCenter = firstMultiple ? firstMultiple.center : firstInput.center;
	  var center = input.center = getCenter(pointers);
	  input.timeStamp = now$3();
	  input.deltaTime = input.timeStamp - firstInput.timeStamp;
	  input.angle = getAngle(offsetCenter, center);
	  input.distance = getDistance(offsetCenter, center);
	  computeDeltaXY(session, input);
	  input.offsetDirection = getDirection(input.deltaX, input.deltaY);
	  var overallVelocity = getVelocity(input.deltaTime, input.deltaX, input.deltaY);
	  input.overallVelocityX = overallVelocity.x;
	  input.overallVelocityY = overallVelocity.y;
	  input.overallVelocity = abs$1(overallVelocity.x) > abs$1(overallVelocity.y) ? overallVelocity.x : overallVelocity.y;
	  input.scale = firstMultiple ? getScale(firstMultiple.pointers, pointers) : 1;
	  input.rotation = firstMultiple ? getRotation(firstMultiple.pointers, pointers) : 0;
	  input.maxPointers = !session.prevInput ? input.pointers.length : input.pointers.length > session.prevInput.maxPointers ? input.pointers.length : session.prevInput.maxPointers;
	  computeIntervalInputData(session, input); // find the correct target

	  var target = manager.element;
	  var srcEvent = input.srcEvent;
	  var srcEventTarget;

	  if (srcEvent.composedPath) {
	    srcEventTarget = srcEvent.composedPath()[0];
	  } else if (srcEvent.path) {
	    srcEventTarget = srcEvent.path[0];
	  } else {
	    srcEventTarget = srcEvent.target;
	  }

	  if (hasParent$1(srcEventTarget, target)) {
	    target = srcEventTarget;
	  }

	  input.target = target;
	}
	/**
	 * @private
	 * handle input events
	 * @param {Manager} manager
	 * @param {String} eventType
	 * @param {Object} input
	 */


	function inputHandler(manager, eventType, input) {
	  var pointersLen = input.pointers.length;
	  var changedPointersLen = input.changedPointers.length;
	  var isFirst = eventType & INPUT_START && pointersLen - changedPointersLen === 0;
	  var isFinal = eventType & (INPUT_END | INPUT_CANCEL) && pointersLen - changedPointersLen === 0;
	  input.isFirst = !!isFirst;
	  input.isFinal = !!isFinal;

	  if (isFirst) {
	    manager.session = {};
	  } // source event is the normalized value of the domEvents
	  // like 'touchstart, mouseup, pointerdown'


	  input.eventType = eventType; // compute scale, rotation etc

	  computeInputData(manager, input); // emit secret event

	  manager.emit('hammer.input', input);
	  manager.recognize(input);
	  manager.session.prevInput = input;
	}
	/**
	 * @private
	 * split string on whitespace
	 * @param {String} str
	 * @returns {Array} words
	 */


	function splitStr(str) {
	  return str.trim().split(/\s+/g);
	}
	/**
	 * @private
	 * addEventListener with multiple events at once
	 * @param {EventTarget} target
	 * @param {String} types
	 * @param {Function} handler
	 */


	function addEventListeners(target, types, handler) {
	  each(splitStr(types), function (type) {
	    target.addEventListener(type, handler, false);
	  });
	}
	/**
	 * @private
	 * removeEventListener with multiple events at once
	 * @param {EventTarget} target
	 * @param {String} types
	 * @param {Function} handler
	 */


	function removeEventListeners(target, types, handler) {
	  each(splitStr(types), function (type) {
	    target.removeEventListener(type, handler, false);
	  });
	}
	/**
	 * @private
	 * get the window object of an element
	 * @param {HTMLElement} element
	 * @returns {DocumentView|Window}
	 */


	function getWindowForElement(element) {
	  var doc = element.ownerDocument || element;
	  return doc.defaultView || doc.parentWindow || window;
	}
	/**
	 * @private
	 * create new input type manager
	 * @param {Manager} manager
	 * @param {Function} callback
	 * @returns {Input}
	 * @constructor
	 */


	var Input = /*#__PURE__*/function () {
	  function Input(manager, callback) {
	    var self = this;
	    this.manager = manager;
	    this.callback = callback;
	    this.element = manager.element;
	    this.target = manager.options.inputTarget; // smaller wrapper around the handler, for the scope and the enabled state of the manager,
	    // so when disabled the input events are completely bypassed.

	    this.domHandler = function (ev) {
	      if (boolOrFn(manager.options.enable, [manager])) {
	        self.handler(ev);
	      }
	    };

	    this.init();
	  }
	  /**
	   * @private
	   * should handle the inputEvent data and trigger the callback
	   * @virtual
	   */


	  var _proto = Input.prototype;

	  _proto.handler = function handler() {};
	  /**
	   * @private
	   * bind the events
	   */


	  _proto.init = function init() {
	    this.evEl && addEventListeners(this.element, this.evEl, this.domHandler);
	    this.evTarget && addEventListeners(this.target, this.evTarget, this.domHandler);
	    this.evWin && addEventListeners(getWindowForElement(this.element), this.evWin, this.domHandler);
	  };
	  /**
	   * @private
	   * unbind the events
	   */


	  _proto.destroy = function destroy() {
	    this.evEl && removeEventListeners(this.element, this.evEl, this.domHandler);
	    this.evTarget && removeEventListeners(this.target, this.evTarget, this.domHandler);
	    this.evWin && removeEventListeners(getWindowForElement(this.element), this.evWin, this.domHandler);
	  };

	  return Input;
	}();
	/**
	 * @private
	 * find if a array contains the object using indexOf or a simple polyFill
	 * @param {Array} src
	 * @param {String} find
	 * @param {String} [findByKey]
	 * @return {Boolean|Number} false when not found, or the index
	 */


	function inArray(src, find, findByKey) {
	  if (src.indexOf && !findByKey) {
	    return src.indexOf(find);
	  } else {
	    var i = 0;

	    while (i < src.length) {
	      if (findByKey && src[i][findByKey] == find || !findByKey && src[i] === find) {
	        // do not use === here, test fails
	        return i;
	      }

	      i++;
	    }

	    return -1;
	  }
	}

	var POINTER_INPUT_MAP = {
	  pointerdown: INPUT_START,
	  pointermove: INPUT_MOVE,
	  pointerup: INPUT_END,
	  pointercancel: INPUT_CANCEL,
	  pointerout: INPUT_CANCEL
	}; // in IE10 the pointer types is defined as an enum

	var IE10_POINTER_TYPE_ENUM = {
	  2: INPUT_TYPE_TOUCH,
	  3: INPUT_TYPE_PEN,
	  4: INPUT_TYPE_MOUSE,
	  5: INPUT_TYPE_KINECT // see https://twitter.com/jacobrossi/status/480596438489890816

	};
	var POINTER_ELEMENT_EVENTS = 'pointerdown';
	var POINTER_WINDOW_EVENTS = 'pointermove pointerup pointercancel'; // IE10 has prefixed support, and case-sensitive

	if (win.MSPointerEvent && !win.PointerEvent) {
	  POINTER_ELEMENT_EVENTS = 'MSPointerDown';
	  POINTER_WINDOW_EVENTS = 'MSPointerMove MSPointerUp MSPointerCancel';
	}
	/**
	 * @private
	 * Pointer events input
	 * @constructor
	 * @extends Input
	 */


	var PointerEventInput = /*#__PURE__*/function (_Input) {
	  _inheritsLoose(PointerEventInput, _Input);

	  function PointerEventInput() {
	    var _this;

	    var proto = PointerEventInput.prototype;
	    proto.evEl = POINTER_ELEMENT_EVENTS;
	    proto.evWin = POINTER_WINDOW_EVENTS;
	    _this = _Input.apply(this, arguments) || this;
	    _this.store = _this.manager.session.pointerEvents = [];
	    return _this;
	  }
	  /**
	   * @private
	   * handle mouse events
	   * @param {Object} ev
	   */


	  var _proto = PointerEventInput.prototype;

	  _proto.handler = function handler(ev) {
	    var store = this.store;
	    var removePointer = false;
	    var eventTypeNormalized = ev.type.toLowerCase().replace('ms', '');
	    var eventType = POINTER_INPUT_MAP[eventTypeNormalized];
	    var pointerType = IE10_POINTER_TYPE_ENUM[ev.pointerType] || ev.pointerType;
	    var isTouch = pointerType === INPUT_TYPE_TOUCH; // get index of the event in the store

	    var storeIndex = inArray(store, ev.pointerId, 'pointerId'); // start and mouse must be down

	    if (eventType & INPUT_START && (ev.button === 0 || isTouch)) {
	      if (storeIndex < 0) {
	        store.push(ev);
	        storeIndex = store.length - 1;
	      }
	    } else if (eventType & (INPUT_END | INPUT_CANCEL)) {
	      removePointer = true;
	    } // it not found, so the pointer hasn't been down (so it's probably a hover)


	    if (storeIndex < 0) {
	      return;
	    } // update the event in the store


	    store[storeIndex] = ev;
	    this.callback(this.manager, eventType, {
	      pointers: store,
	      changedPointers: [ev],
	      pointerType: pointerType,
	      srcEvent: ev
	    });

	    if (removePointer) {
	      // remove from the store
	      store.splice(storeIndex, 1);
	    }
	  };

	  return PointerEventInput;
	}(Input);
	/**
	 * @private
	 * convert array-like objects to real arrays
	 * @param {Object} obj
	 * @returns {Array}
	 */


	function toArray$1(obj) {
	  return Array.prototype.slice.call(obj, 0);
	}
	/**
	 * @private
	 * unique array with objects based on a key (like 'id') or just by the array's value
	 * @param {Array} src [{id:1},{id:2},{id:1}]
	 * @param {String} [key]
	 * @param {Boolean} [sort=False]
	 * @returns {Array} [{id:1},{id:2}]
	 */


	function uniqueArray(src, key, sort) {
	  var results = [];
	  var values = [];
	  var i = 0;

	  while (i < src.length) {
	    var val = key ? src[i][key] : src[i];

	    if (inArray(values, val) < 0) {
	      results.push(src[i]);
	    }

	    values[i] = val;
	    i++;
	  }

	  if (sort) {
	    if (!key) {
	      results = results.sort();
	    } else {
	      results = results.sort(function (a, b) {
	        return a[key] > b[key];
	      });
	    }
	  }

	  return results;
	}

	var TOUCH_INPUT_MAP = {
	  touchstart: INPUT_START,
	  touchmove: INPUT_MOVE,
	  touchend: INPUT_END,
	  touchcancel: INPUT_CANCEL
	};
	var TOUCH_TARGET_EVENTS = 'touchstart touchmove touchend touchcancel';
	/**
	 * @private
	 * Multi-user touch events input
	 * @constructor
	 * @extends Input
	 */

	var TouchInput = /*#__PURE__*/function (_Input) {
	  _inheritsLoose(TouchInput, _Input);

	  function TouchInput() {
	    var _this;

	    TouchInput.prototype.evTarget = TOUCH_TARGET_EVENTS;
	    _this = _Input.apply(this, arguments) || this;
	    _this.targetIds = {}; // this.evTarget = TOUCH_TARGET_EVENTS;

	    return _this;
	  }

	  var _proto = TouchInput.prototype;

	  _proto.handler = function handler(ev) {
	    var type = TOUCH_INPUT_MAP[ev.type];
	    var touches = getTouches.call(this, ev, type);

	    if (!touches) {
	      return;
	    }

	    this.callback(this.manager, type, {
	      pointers: touches[0],
	      changedPointers: touches[1],
	      pointerType: INPUT_TYPE_TOUCH,
	      srcEvent: ev
	    });
	  };

	  return TouchInput;
	}(Input);

	function getTouches(ev, type) {
	  var allTouches = toArray$1(ev.touches);
	  var targetIds = this.targetIds; // when there is only one touch, the process can be simplified

	  if (type & (INPUT_START | INPUT_MOVE) && allTouches.length === 1) {
	    targetIds[allTouches[0].identifier] = true;
	    return [allTouches, allTouches];
	  }

	  var i;
	  var targetTouches;
	  var changedTouches = toArray$1(ev.changedTouches);
	  var changedTargetTouches = [];
	  var target = this.target; // get target touches from touches

	  targetTouches = allTouches.filter(function (touch) {
	    return hasParent$1(touch.target, target);
	  }); // collect touches

	  if (type === INPUT_START) {
	    i = 0;

	    while (i < targetTouches.length) {
	      targetIds[targetTouches[i].identifier] = true;
	      i++;
	    }
	  } // filter changed touches to only contain touches that exist in the collected target ids


	  i = 0;

	  while (i < changedTouches.length) {
	    if (targetIds[changedTouches[i].identifier]) {
	      changedTargetTouches.push(changedTouches[i]);
	    } // cleanup removed touches


	    if (type & (INPUT_END | INPUT_CANCEL)) {
	      delete targetIds[changedTouches[i].identifier];
	    }

	    i++;
	  }

	  if (!changedTargetTouches.length) {
	    return;
	  }

	  return [// merge targetTouches with changedTargetTouches so it contains ALL touches, including 'end' and 'cancel'
	  uniqueArray(targetTouches.concat(changedTargetTouches), 'identifier', true), changedTargetTouches];
	}

	var MOUSE_INPUT_MAP = {
	  mousedown: INPUT_START,
	  mousemove: INPUT_MOVE,
	  mouseup: INPUT_END
	};
	var MOUSE_ELEMENT_EVENTS = 'mousedown';
	var MOUSE_WINDOW_EVENTS = 'mousemove mouseup';
	/**
	 * @private
	 * Mouse events input
	 * @constructor
	 * @extends Input
	 */

	var MouseInput = /*#__PURE__*/function (_Input) {
	  _inheritsLoose(MouseInput, _Input);

	  function MouseInput() {
	    var _this;

	    var proto = MouseInput.prototype;
	    proto.evEl = MOUSE_ELEMENT_EVENTS;
	    proto.evWin = MOUSE_WINDOW_EVENTS;
	    _this = _Input.apply(this, arguments) || this;
	    _this.pressed = false; // mousedown state

	    return _this;
	  }
	  /**
	   * @private
	   * handle mouse events
	   * @param {Object} ev
	   */


	  var _proto = MouseInput.prototype;

	  _proto.handler = function handler(ev) {
	    var eventType = MOUSE_INPUT_MAP[ev.type]; // on start we want to have the left mouse button down

	    if (eventType & INPUT_START && ev.button === 0) {
	      this.pressed = true;
	    }

	    if (eventType & INPUT_MOVE && ev.which !== 1) {
	      eventType = INPUT_END;
	    } // mouse must be down


	    if (!this.pressed) {
	      return;
	    }

	    if (eventType & INPUT_END) {
	      this.pressed = false;
	    }

	    this.callback(this.manager, eventType, {
	      pointers: [ev],
	      changedPointers: [ev],
	      pointerType: INPUT_TYPE_MOUSE,
	      srcEvent: ev
	    });
	  };

	  return MouseInput;
	}(Input);
	/**
	 * @private
	 * Combined touch and mouse input
	 *
	 * Touch has a higher priority then mouse, and while touching no mouse events are allowed.
	 * This because touch devices also emit mouse events while doing a touch.
	 *
	 * @constructor
	 * @extends Input
	 */


	var DEDUP_TIMEOUT = 2500;
	var DEDUP_DISTANCE = 25;

	function setLastTouch(eventData) {
	  var _eventData$changedPoi = eventData.changedPointers,
	      touch = _eventData$changedPoi[0];

	  if (touch.identifier === this.primaryTouch) {
	    var lastTouch = {
	      x: touch.clientX,
	      y: touch.clientY
	    };
	    var lts = this.lastTouches;
	    this.lastTouches.push(lastTouch);

	    var removeLastTouch = function removeLastTouch() {
	      var i = lts.indexOf(lastTouch);

	      if (i > -1) {
	        lts.splice(i, 1);
	      }
	    };

	    setTimeout(removeLastTouch, DEDUP_TIMEOUT);
	  }
	}

	function recordTouches(eventType, eventData) {
	  if (eventType & INPUT_START) {
	    this.primaryTouch = eventData.changedPointers[0].identifier;
	    setLastTouch.call(this, eventData);
	  } else if (eventType & (INPUT_END | INPUT_CANCEL)) {
	    setLastTouch.call(this, eventData);
	  }
	}

	function isSyntheticEvent(eventData) {
	  var x = eventData.srcEvent.clientX;
	  var y = eventData.srcEvent.clientY;

	  for (var i = 0; i < this.lastTouches.length; i++) {
	    var t = this.lastTouches[i];
	    var dx = Math.abs(x - t.x);
	    var dy = Math.abs(y - t.y);

	    if (dx <= DEDUP_DISTANCE && dy <= DEDUP_DISTANCE) {
	      return true;
	    }
	  }

	  return false;
	}

	var TouchMouseInput = /*#__PURE__*/function () {
	  var TouchMouseInput = /*#__PURE__*/function (_Input) {
	    _inheritsLoose(TouchMouseInput, _Input);

	    function TouchMouseInput(_manager, callback) {
	      var _this;

	      _this = _Input.call(this, _manager, callback) || this;

	      _this.handler = function (manager, inputEvent, inputData) {
	        var isTouch = inputData.pointerType === INPUT_TYPE_TOUCH;
	        var isMouse = inputData.pointerType === INPUT_TYPE_MOUSE;

	        if (isMouse && inputData.sourceCapabilities && inputData.sourceCapabilities.firesTouchEvents) {
	          return;
	        } // when we're in a touch event, record touches to  de-dupe synthetic mouse event


	        if (isTouch) {
	          recordTouches.call(_assertThisInitialized$1(_assertThisInitialized$1(_this)), inputEvent, inputData);
	        } else if (isMouse && isSyntheticEvent.call(_assertThisInitialized$1(_assertThisInitialized$1(_this)), inputData)) {
	          return;
	        }

	        _this.callback(manager, inputEvent, inputData);
	      };

	      _this.touch = new TouchInput(_this.manager, _this.handler);
	      _this.mouse = new MouseInput(_this.manager, _this.handler);
	      _this.primaryTouch = null;
	      _this.lastTouches = [];
	      return _this;
	    }
	    /**
	     * @private
	     * handle mouse and touch events
	     * @param {Hammer} manager
	     * @param {String} inputEvent
	     * @param {Object} inputData
	     */


	    var _proto = TouchMouseInput.prototype;
	    /**
	     * @private
	     * remove the event listeners
	     */

	    _proto.destroy = function destroy() {
	      this.touch.destroy();
	      this.mouse.destroy();
	    };

	    return TouchMouseInput;
	  }(Input);

	  return TouchMouseInput;
	}();
	/**
	 * @private
	 * create new input type manager
	 * called by the Manager constructor
	 * @param {Hammer} manager
	 * @returns {Input}
	 */


	function createInputInstance(manager) {
	  var Type; // let inputClass = manager.options.inputClass;

	  var inputClass = manager.options.inputClass;

	  if (inputClass) {
	    Type = inputClass;
	  } else if (SUPPORT_POINTER_EVENTS) {
	    Type = PointerEventInput;
	  } else if (SUPPORT_ONLY_TOUCH) {
	    Type = TouchInput;
	  } else if (!SUPPORT_TOUCH) {
	    Type = MouseInput;
	  } else {
	    Type = TouchMouseInput;
	  }

	  return new Type(manager, inputHandler);
	}
	/**
	 * @private
	 * if the argument is an array, we want to execute the fn on each entry
	 * if it aint an array we don't want to do a thing.
	 * this is used by all the methods that accept a single and array argument.
	 * @param {*|Array} arg
	 * @param {String} fn
	 * @param {Object} [context]
	 * @returns {Boolean}
	 */


	function invokeArrayArg(arg, fn, context) {
	  if (Array.isArray(arg)) {
	    each(arg, context[fn], context);
	    return true;
	  }

	  return false;
	}

	var STATE_POSSIBLE = 1;
	var STATE_BEGAN = 2;
	var STATE_CHANGED = 4;
	var STATE_ENDED = 8;
	var STATE_RECOGNIZED = STATE_ENDED;
	var STATE_CANCELLED = 16;
	var STATE_FAILED = 32;
	/**
	 * @private
	 * get a unique id
	 * @returns {number} uniqueId
	 */

	var _uniqueId = 1;

	function uniqueId() {
	  return _uniqueId++;
	}
	/**
	 * @private
	 * get a recognizer by name if it is bound to a manager
	 * @param {Recognizer|String} otherRecognizer
	 * @param {Recognizer} recognizer
	 * @returns {Recognizer}
	 */


	function getRecognizerByNameIfManager(otherRecognizer, recognizer) {
	  var manager = recognizer.manager;

	  if (manager) {
	    return manager.get(otherRecognizer);
	  }

	  return otherRecognizer;
	}
	/**
	 * @private
	 * get a usable string, used as event postfix
	 * @param {constant} state
	 * @returns {String} state
	 */


	function stateStr(state) {
	  if (state & STATE_CANCELLED) {
	    return 'cancel';
	  } else if (state & STATE_ENDED) {
	    return 'end';
	  } else if (state & STATE_CHANGED) {
	    return 'move';
	  } else if (state & STATE_BEGAN) {
	    return 'start';
	  }

	  return '';
	}
	/**
	 * @private
	 * Recognizer flow explained; *
	 * All recognizers have the initial state of POSSIBLE when a input session starts.
	 * The definition of a input session is from the first input until the last input, with all it's movement in it. *
	 * Example session for mouse-input: mousedown -> mousemove -> mouseup
	 *
	 * On each recognizing cycle (see Manager.recognize) the .recognize() method is executed
	 * which determines with state it should be.
	 *
	 * If the recognizer has the state FAILED, CANCELLED or RECOGNIZED (equals ENDED), it is reset to
	 * POSSIBLE to give it another change on the next cycle.
	 *
	 *               Possible
	 *                  |
	 *            +-----+---------------+
	 *            |                     |
	 *      +-----+-----+               |
	 *      |           |               |
	 *   Failed      Cancelled          |
	 *                          +-------+------+
	 *                          |              |
	 *                      Recognized       Began
	 *                                         |
	 *                                      Changed
	 *                                         |
	 *                                  Ended/Recognized
	 */

	/**
	 * @private
	 * Recognizer
	 * Every recognizer needs to extend from this class.
	 * @constructor
	 * @param {Object} options
	 */


	var Recognizer = /*#__PURE__*/function () {
	  function Recognizer(options) {
	    if (options === void 0) {
	      options = {};
	    }

	    this.options = _extends({
	      enable: true
	    }, options);
	    this.id = uniqueId();
	    this.manager = null; // default is enable true

	    this.state = STATE_POSSIBLE;
	    this.simultaneous = {};
	    this.requireFail = [];
	  }
	  /**
	   * @private
	   * set options
	   * @param {Object} options
	   * @return {Recognizer}
	   */


	  var _proto = Recognizer.prototype;

	  _proto.set = function set(options) {
	    assign$1$1(this.options, options); // also update the touchAction, in case something changed about the directions/enabled state

	    this.manager && this.manager.touchAction.update();
	    return this;
	  };
	  /**
	   * @private
	   * recognize simultaneous with an other recognizer.
	   * @param {Recognizer} otherRecognizer
	   * @returns {Recognizer} this
	   */


	  _proto.recognizeWith = function recognizeWith(otherRecognizer) {
	    if (invokeArrayArg(otherRecognizer, 'recognizeWith', this)) {
	      return this;
	    }

	    var simultaneous = this.simultaneous;
	    otherRecognizer = getRecognizerByNameIfManager(otherRecognizer, this);

	    if (!simultaneous[otherRecognizer.id]) {
	      simultaneous[otherRecognizer.id] = otherRecognizer;
	      otherRecognizer.recognizeWith(this);
	    }

	    return this;
	  };
	  /**
	   * @private
	   * drop the simultaneous link. it doesnt remove the link on the other recognizer.
	   * @param {Recognizer} otherRecognizer
	   * @returns {Recognizer} this
	   */


	  _proto.dropRecognizeWith = function dropRecognizeWith(otherRecognizer) {
	    if (invokeArrayArg(otherRecognizer, 'dropRecognizeWith', this)) {
	      return this;
	    }

	    otherRecognizer = getRecognizerByNameIfManager(otherRecognizer, this);
	    delete this.simultaneous[otherRecognizer.id];
	    return this;
	  };
	  /**
	   * @private
	   * recognizer can only run when an other is failing
	   * @param {Recognizer} otherRecognizer
	   * @returns {Recognizer} this
	   */


	  _proto.requireFailure = function requireFailure(otherRecognizer) {
	    if (invokeArrayArg(otherRecognizer, 'requireFailure', this)) {
	      return this;
	    }

	    var requireFail = this.requireFail;
	    otherRecognizer = getRecognizerByNameIfManager(otherRecognizer, this);

	    if (inArray(requireFail, otherRecognizer) === -1) {
	      requireFail.push(otherRecognizer);
	      otherRecognizer.requireFailure(this);
	    }

	    return this;
	  };
	  /**
	   * @private
	   * drop the requireFailure link. it does not remove the link on the other recognizer.
	   * @param {Recognizer} otherRecognizer
	   * @returns {Recognizer} this
	   */


	  _proto.dropRequireFailure = function dropRequireFailure(otherRecognizer) {
	    if (invokeArrayArg(otherRecognizer, 'dropRequireFailure', this)) {
	      return this;
	    }

	    otherRecognizer = getRecognizerByNameIfManager(otherRecognizer, this);
	    var index = inArray(this.requireFail, otherRecognizer);

	    if (index > -1) {
	      this.requireFail.splice(index, 1);
	    }

	    return this;
	  };
	  /**
	   * @private
	   * has require failures boolean
	   * @returns {boolean}
	   */


	  _proto.hasRequireFailures = function hasRequireFailures() {
	    return this.requireFail.length > 0;
	  };
	  /**
	   * @private
	   * if the recognizer can recognize simultaneous with an other recognizer
	   * @param {Recognizer} otherRecognizer
	   * @returns {Boolean}
	   */


	  _proto.canRecognizeWith = function canRecognizeWith(otherRecognizer) {
	    return !!this.simultaneous[otherRecognizer.id];
	  };
	  /**
	   * @private
	   * You should use `tryEmit` instead of `emit` directly to check
	   * that all the needed recognizers has failed before emitting.
	   * @param {Object} input
	   */


	  _proto.emit = function emit(input) {
	    var self = this;
	    var state = this.state;

	    function emit(event) {
	      self.manager.emit(event, input);
	    } // 'panstart' and 'panmove'


	    if (state < STATE_ENDED) {
	      emit(self.options.event + stateStr(state));
	    }

	    emit(self.options.event); // simple 'eventName' events

	    if (input.additionalEvent) {
	      // additional event(panleft, panright, pinchin, pinchout...)
	      emit(input.additionalEvent);
	    } // panend and pancancel


	    if (state >= STATE_ENDED) {
	      emit(self.options.event + stateStr(state));
	    }
	  };
	  /**
	   * @private
	   * Check that all the require failure recognizers has failed,
	   * if true, it emits a gesture event,
	   * otherwise, setup the state to FAILED.
	   * @param {Object} input
	   */


	  _proto.tryEmit = function tryEmit(input) {
	    if (this.canEmit()) {
	      return this.emit(input);
	    } // it's failing anyway


	    this.state = STATE_FAILED;
	  };
	  /**
	   * @private
	   * can we emit?
	   * @returns {boolean}
	   */


	  _proto.canEmit = function canEmit() {
	    var i = 0;

	    while (i < this.requireFail.length) {
	      if (!(this.requireFail[i].state & (STATE_FAILED | STATE_POSSIBLE))) {
	        return false;
	      }

	      i++;
	    }

	    return true;
	  };
	  /**
	   * @private
	   * update the recognizer
	   * @param {Object} inputData
	   */


	  _proto.recognize = function recognize(inputData) {
	    // make a new copy of the inputData
	    // so we can change the inputData without messing up the other recognizers
	    var inputDataClone = assign$1$1({}, inputData); // is is enabled and allow recognizing?

	    if (!boolOrFn(this.options.enable, [this, inputDataClone])) {
	      this.reset();
	      this.state = STATE_FAILED;
	      return;
	    } // reset when we've reached the end


	    if (this.state & (STATE_RECOGNIZED | STATE_CANCELLED | STATE_FAILED)) {
	      this.state = STATE_POSSIBLE;
	    }

	    this.state = this.process(inputDataClone); // the recognizer has recognized a gesture
	    // so trigger an event

	    if (this.state & (STATE_BEGAN | STATE_CHANGED | STATE_ENDED | STATE_CANCELLED)) {
	      this.tryEmit(inputDataClone);
	    }
	  };
	  /**
	   * @private
	   * return the state of the recognizer
	   * the actual recognizing happens in this method
	   * @virtual
	   * @param {Object} inputData
	   * @returns {constant} STATE
	   */

	  /* jshint ignore:start */


	  _proto.process = function process(inputData) {};
	  /* jshint ignore:end */

	  /**
	   * @private
	   * return the preferred touch-action
	   * @virtual
	   * @returns {Array}
	   */


	  _proto.getTouchAction = function getTouchAction() {};
	  /**
	   * @private
	   * called when the gesture isn't allowed to recognize
	   * like when another is being recognized or it is disabled
	   * @virtual
	   */


	  _proto.reset = function reset() {};

	  return Recognizer;
	}();
	/**
	 * @private
	 * A tap is recognized when the pointer is doing a small tap/click. Multiple taps are recognized if they occur
	 * between the given interval and position. The delay option can be used to recognize multi-taps without firing
	 * a single tap.
	 *
	 * The eventData from the emitted event contains the property `tapCount`, which contains the amount of
	 * multi-taps being recognized.
	 * @constructor
	 * @extends Recognizer
	 */


	var TapRecognizer = /*#__PURE__*/function (_Recognizer) {
	  _inheritsLoose(TapRecognizer, _Recognizer);

	  function TapRecognizer(options) {
	    var _this;

	    if (options === void 0) {
	      options = {};
	    }

	    _this = _Recognizer.call(this, _extends({
	      event: 'tap',
	      pointers: 1,
	      taps: 1,
	      interval: 300,
	      // max time between the multi-tap taps
	      time: 250,
	      // max time of the pointer to be down (like finger on the screen)
	      threshold: 9,
	      // a minimal movement is ok, but keep it low
	      posThreshold: 10
	    }, options)) || this; // previous time and center,
	    // used for tap counting

	    _this.pTime = false;
	    _this.pCenter = false;
	    _this._timer = null;
	    _this._input = null;
	    _this.count = 0;
	    return _this;
	  }

	  var _proto = TapRecognizer.prototype;

	  _proto.getTouchAction = function getTouchAction() {
	    return [TOUCH_ACTION_MANIPULATION];
	  };

	  _proto.process = function process(input) {
	    var _this2 = this;

	    var options = this.options;
	    var validPointers = input.pointers.length === options.pointers;
	    var validMovement = input.distance < options.threshold;
	    var validTouchTime = input.deltaTime < options.time;
	    this.reset();

	    if (input.eventType & INPUT_START && this.count === 0) {
	      return this.failTimeout();
	    } // we only allow little movement
	    // and we've reached an end event, so a tap is possible


	    if (validMovement && validTouchTime && validPointers) {
	      if (input.eventType !== INPUT_END) {
	        return this.failTimeout();
	      }

	      var validInterval = this.pTime ? input.timeStamp - this.pTime < options.interval : true;
	      var validMultiTap = !this.pCenter || getDistance(this.pCenter, input.center) < options.posThreshold;
	      this.pTime = input.timeStamp;
	      this.pCenter = input.center;

	      if (!validMultiTap || !validInterval) {
	        this.count = 1;
	      } else {
	        this.count += 1;
	      }

	      this._input = input; // if tap count matches we have recognized it,
	      // else it has began recognizing...

	      var tapCount = this.count % options.taps;

	      if (tapCount === 0) {
	        // no failing requirements, immediately trigger the tap event
	        // or wait as long as the multitap interval to trigger
	        if (!this.hasRequireFailures()) {
	          return STATE_RECOGNIZED;
	        } else {
	          this._timer = setTimeout(function () {
	            _this2.state = STATE_RECOGNIZED;

	            _this2.tryEmit();
	          }, options.interval);
	          return STATE_BEGAN;
	        }
	      }
	    }

	    return STATE_FAILED;
	  };

	  _proto.failTimeout = function failTimeout() {
	    var _this3 = this;

	    this._timer = setTimeout(function () {
	      _this3.state = STATE_FAILED;
	    }, this.options.interval);
	    return STATE_FAILED;
	  };

	  _proto.reset = function reset() {
	    clearTimeout(this._timer);
	  };

	  _proto.emit = function emit() {
	    if (this.state === STATE_RECOGNIZED) {
	      this._input.tapCount = this.count;
	      this.manager.emit(this.options.event, this._input);
	    }
	  };

	  return TapRecognizer;
	}(Recognizer);
	/**
	 * @private
	 * This recognizer is just used as a base for the simple attribute recognizers.
	 * @constructor
	 * @extends Recognizer
	 */


	var AttrRecognizer = /*#__PURE__*/function (_Recognizer) {
	  _inheritsLoose(AttrRecognizer, _Recognizer);

	  function AttrRecognizer(options) {
	    if (options === void 0) {
	      options = {};
	    }

	    return _Recognizer.call(this, _extends({
	      pointers: 1
	    }, options)) || this;
	  }
	  /**
	   * @private
	   * Used to check if it the recognizer receives valid input, like input.distance > 10.
	   * @memberof AttrRecognizer
	   * @param {Object} input
	   * @returns {Boolean} recognized
	   */


	  var _proto = AttrRecognizer.prototype;

	  _proto.attrTest = function attrTest(input) {
	    var optionPointers = this.options.pointers;
	    return optionPointers === 0 || input.pointers.length === optionPointers;
	  };
	  /**
	   * @private
	   * Process the input and return the state for the recognizer
	   * @memberof AttrRecognizer
	   * @param {Object} input
	   * @returns {*} State
	   */


	  _proto.process = function process(input) {
	    var state = this.state;
	    var eventType = input.eventType;
	    var isRecognized = state & (STATE_BEGAN | STATE_CHANGED);
	    var isValid = this.attrTest(input); // on cancel input and we've recognized before, return STATE_CANCELLED

	    if (isRecognized && (eventType & INPUT_CANCEL || !isValid)) {
	      return state | STATE_CANCELLED;
	    } else if (isRecognized || isValid) {
	      if (eventType & INPUT_END) {
	        return state | STATE_ENDED;
	      } else if (!(state & STATE_BEGAN)) {
	        return STATE_BEGAN;
	      }

	      return state | STATE_CHANGED;
	    }

	    return STATE_FAILED;
	  };

	  return AttrRecognizer;
	}(Recognizer);
	/**
	 * @private
	 * direction cons to string
	 * @param {constant} direction
	 * @returns {String}
	 */


	function directionStr(direction) {
	  if (direction === DIRECTION_DOWN) {
	    return 'down';
	  } else if (direction === DIRECTION_UP) {
	    return 'up';
	  } else if (direction === DIRECTION_LEFT) {
	    return 'left';
	  } else if (direction === DIRECTION_RIGHT) {
	    return 'right';
	  }

	  return '';
	}
	/**
	 * @private
	 * Pan
	 * Recognized when the pointer is down and moved in the allowed direction.
	 * @constructor
	 * @extends AttrRecognizer
	 */


	var PanRecognizer = /*#__PURE__*/function (_AttrRecognizer) {
	  _inheritsLoose(PanRecognizer, _AttrRecognizer);

	  function PanRecognizer(options) {
	    var _this;

	    if (options === void 0) {
	      options = {};
	    }

	    _this = _AttrRecognizer.call(this, _extends({
	      event: 'pan',
	      threshold: 10,
	      pointers: 1,
	      direction: DIRECTION_ALL
	    }, options)) || this;
	    _this.pX = null;
	    _this.pY = null;
	    return _this;
	  }

	  var _proto = PanRecognizer.prototype;

	  _proto.getTouchAction = function getTouchAction() {
	    var direction = this.options.direction;
	    var actions = [];

	    if (direction & DIRECTION_HORIZONTAL) {
	      actions.push(TOUCH_ACTION_PAN_Y);
	    }

	    if (direction & DIRECTION_VERTICAL) {
	      actions.push(TOUCH_ACTION_PAN_X);
	    }

	    return actions;
	  };

	  _proto.directionTest = function directionTest(input) {
	    var options = this.options;
	    var hasMoved = true;
	    var distance = input.distance;
	    var direction = input.direction;
	    var x = input.deltaX;
	    var y = input.deltaY; // lock to axis?

	    if (!(direction & options.direction)) {
	      if (options.direction & DIRECTION_HORIZONTAL) {
	        direction = x === 0 ? DIRECTION_NONE : x < 0 ? DIRECTION_LEFT : DIRECTION_RIGHT;
	        hasMoved = x !== this.pX;
	        distance = Math.abs(input.deltaX);
	      } else {
	        direction = y === 0 ? DIRECTION_NONE : y < 0 ? DIRECTION_UP : DIRECTION_DOWN;
	        hasMoved = y !== this.pY;
	        distance = Math.abs(input.deltaY);
	      }
	    }

	    input.direction = direction;
	    return hasMoved && distance > options.threshold && direction & options.direction;
	  };

	  _proto.attrTest = function attrTest(input) {
	    return AttrRecognizer.prototype.attrTest.call(this, input) && ( // replace with a super call
	    this.state & STATE_BEGAN || !(this.state & STATE_BEGAN) && this.directionTest(input));
	  };

	  _proto.emit = function emit(input) {
	    this.pX = input.deltaX;
	    this.pY = input.deltaY;
	    var direction = directionStr(input.direction);

	    if (direction) {
	      input.additionalEvent = this.options.event + direction;
	    }

	    _AttrRecognizer.prototype.emit.call(this, input);
	  };

	  return PanRecognizer;
	}(AttrRecognizer);
	/**
	 * @private
	 * Swipe
	 * Recognized when the pointer is moving fast (velocity), with enough distance in the allowed direction.
	 * @constructor
	 * @extends AttrRecognizer
	 */


	var SwipeRecognizer = /*#__PURE__*/function (_AttrRecognizer) {
	  _inheritsLoose(SwipeRecognizer, _AttrRecognizer);

	  function SwipeRecognizer(options) {
	    if (options === void 0) {
	      options = {};
	    }

	    return _AttrRecognizer.call(this, _extends({
	      event: 'swipe',
	      threshold: 10,
	      velocity: 0.3,
	      direction: DIRECTION_HORIZONTAL | DIRECTION_VERTICAL,
	      pointers: 1
	    }, options)) || this;
	  }

	  var _proto = SwipeRecognizer.prototype;

	  _proto.getTouchAction = function getTouchAction() {
	    return PanRecognizer.prototype.getTouchAction.call(this);
	  };

	  _proto.attrTest = function attrTest(input) {
	    var direction = this.options.direction;
	    var velocity;

	    if (direction & (DIRECTION_HORIZONTAL | DIRECTION_VERTICAL)) {
	      velocity = input.overallVelocity;
	    } else if (direction & DIRECTION_HORIZONTAL) {
	      velocity = input.overallVelocityX;
	    } else if (direction & DIRECTION_VERTICAL) {
	      velocity = input.overallVelocityY;
	    }

	    return _AttrRecognizer.prototype.attrTest.call(this, input) && direction & input.offsetDirection && input.distance > this.options.threshold && input.maxPointers === this.options.pointers && abs$1(velocity) > this.options.velocity && input.eventType & INPUT_END;
	  };

	  _proto.emit = function emit(input) {
	    var direction = directionStr(input.offsetDirection);

	    if (direction) {
	      this.manager.emit(this.options.event + direction, input);
	    }

	    this.manager.emit(this.options.event, input);
	  };

	  return SwipeRecognizer;
	}(AttrRecognizer);
	/**
	 * @private
	 * Pinch
	 * Recognized when two or more pointers are moving toward (zoom-in) or away from each other (zoom-out).
	 * @constructor
	 * @extends AttrRecognizer
	 */


	var PinchRecognizer = /*#__PURE__*/function (_AttrRecognizer) {
	  _inheritsLoose(PinchRecognizer, _AttrRecognizer);

	  function PinchRecognizer(options) {
	    if (options === void 0) {
	      options = {};
	    }

	    return _AttrRecognizer.call(this, _extends({
	      event: 'pinch',
	      threshold: 0,
	      pointers: 2
	    }, options)) || this;
	  }

	  var _proto = PinchRecognizer.prototype;

	  _proto.getTouchAction = function getTouchAction() {
	    return [TOUCH_ACTION_NONE];
	  };

	  _proto.attrTest = function attrTest(input) {
	    return _AttrRecognizer.prototype.attrTest.call(this, input) && (Math.abs(input.scale - 1) > this.options.threshold || this.state & STATE_BEGAN);
	  };

	  _proto.emit = function emit(input) {
	    if (input.scale !== 1) {
	      var inOut = input.scale < 1 ? 'in' : 'out';
	      input.additionalEvent = this.options.event + inOut;
	    }

	    _AttrRecognizer.prototype.emit.call(this, input);
	  };

	  return PinchRecognizer;
	}(AttrRecognizer);
	/**
	 * @private
	 * Rotate
	 * Recognized when two or more pointer are moving in a circular motion.
	 * @constructor
	 * @extends AttrRecognizer
	 */


	var RotateRecognizer = /*#__PURE__*/function (_AttrRecognizer) {
	  _inheritsLoose(RotateRecognizer, _AttrRecognizer);

	  function RotateRecognizer(options) {
	    if (options === void 0) {
	      options = {};
	    }

	    return _AttrRecognizer.call(this, _extends({
	      event: 'rotate',
	      threshold: 0,
	      pointers: 2
	    }, options)) || this;
	  }

	  var _proto = RotateRecognizer.prototype;

	  _proto.getTouchAction = function getTouchAction() {
	    return [TOUCH_ACTION_NONE];
	  };

	  _proto.attrTest = function attrTest(input) {
	    return _AttrRecognizer.prototype.attrTest.call(this, input) && (Math.abs(input.rotation) > this.options.threshold || this.state & STATE_BEGAN);
	  };

	  return RotateRecognizer;
	}(AttrRecognizer);
	/**
	 * @private
	 * Press
	 * Recognized when the pointer is down for x ms without any movement.
	 * @constructor
	 * @extends Recognizer
	 */


	var PressRecognizer = /*#__PURE__*/function (_Recognizer) {
	  _inheritsLoose(PressRecognizer, _Recognizer);

	  function PressRecognizer(options) {
	    var _this;

	    if (options === void 0) {
	      options = {};
	    }

	    _this = _Recognizer.call(this, _extends({
	      event: 'press',
	      pointers: 1,
	      time: 251,
	      // minimal time of the pointer to be pressed
	      threshold: 9
	    }, options)) || this;
	    _this._timer = null;
	    _this._input = null;
	    return _this;
	  }

	  var _proto = PressRecognizer.prototype;

	  _proto.getTouchAction = function getTouchAction() {
	    return [TOUCH_ACTION_AUTO];
	  };

	  _proto.process = function process(input) {
	    var _this2 = this;

	    var options = this.options;
	    var validPointers = input.pointers.length === options.pointers;
	    var validMovement = input.distance < options.threshold;
	    var validTime = input.deltaTime > options.time;
	    this._input = input; // we only allow little movement
	    // and we've reached an end event, so a tap is possible

	    if (!validMovement || !validPointers || input.eventType & (INPUT_END | INPUT_CANCEL) && !validTime) {
	      this.reset();
	    } else if (input.eventType & INPUT_START) {
	      this.reset();
	      this._timer = setTimeout(function () {
	        _this2.state = STATE_RECOGNIZED;

	        _this2.tryEmit();
	      }, options.time);
	    } else if (input.eventType & INPUT_END) {
	      return STATE_RECOGNIZED;
	    }

	    return STATE_FAILED;
	  };

	  _proto.reset = function reset() {
	    clearTimeout(this._timer);
	  };

	  _proto.emit = function emit(input) {
	    if (this.state !== STATE_RECOGNIZED) {
	      return;
	    }

	    if (input && input.eventType & INPUT_END) {
	      this.manager.emit(this.options.event + "up", input);
	    } else {
	      this._input.timeStamp = now$3();
	      this.manager.emit(this.options.event, this._input);
	    }
	  };

	  return PressRecognizer;
	}(Recognizer);

	var defaults = {
	  /**
	   * @private
	   * set if DOM events are being triggered.
	   * But this is slower and unused by simple implementations, so disabled by default.
	   * @type {Boolean}
	   * @default false
	   */
	  domEvents: false,

	  /**
	   * @private
	   * The value for the touchAction property/fallback.
	   * When set to `compute` it will magically set the correct value based on the added recognizers.
	   * @type {String}
	   * @default compute
	   */
	  touchAction: TOUCH_ACTION_COMPUTE,

	  /**
	   * @private
	   * @type {Boolean}
	   * @default true
	   */
	  enable: true,

	  /**
	   * @private
	   * EXPERIMENTAL FEATURE -- can be removed/changed
	   * Change the parent input target element.
	   * If Null, then it is being set the to main element.
	   * @type {Null|EventTarget}
	   * @default null
	   */
	  inputTarget: null,

	  /**
	   * @private
	   * force an input class
	   * @type {Null|Function}
	   * @default null
	   */
	  inputClass: null,

	  /**
	   * @private
	   * Some CSS properties can be used to improve the working of Hammer.
	   * Add them to this method and they will be set when creating a new Manager.
	   * @namespace
	   */
	  cssProps: {
	    /**
	     * @private
	     * Disables text selection to improve the dragging gesture. Mainly for desktop browsers.
	     * @type {String}
	     * @default 'none'
	     */
	    userSelect: "none",

	    /**
	     * @private
	     * Disable the Windows Phone grippers when pressing an element.
	     * @type {String}
	     * @default 'none'
	     */
	    touchSelect: "none",

	    /**
	     * @private
	     * Disables the default callout shown when you touch and hold a touch target.
	     * On iOS, when you touch and hold a touch target such as a link, Safari displays
	     * a callout containing information about the link. This property allows you to disable that callout.
	     * @type {String}
	     * @default 'none'
	     */
	    touchCallout: "none",

	    /**
	     * @private
	     * Specifies whether zooming is enabled. Used by IE10>
	     * @type {String}
	     * @default 'none'
	     */
	    contentZooming: "none",

	    /**
	     * @private
	     * Specifies that an entire element should be draggable instead of its contents. Mainly for desktop browsers.
	     * @type {String}
	     * @default 'none'
	     */
	    userDrag: "none",

	    /**
	     * @private
	     * Overrides the highlight color shown when the user taps a link or a JavaScript
	     * clickable element in iOS. This property obeys the alpha value, if specified.
	     * @type {String}
	     * @default 'rgba(0,0,0,0)'
	     */
	    tapHighlightColor: "rgba(0,0,0,0)"
	  }
	};
	/**
	 * @private
	 * Default recognizer setup when calling `Hammer()`
	 * When creating a new Manager these will be skipped.
	 * This is separated with other defaults because of tree-shaking.
	 * @type {Array}
	 */

	var preset = [[RotateRecognizer, {
	  enable: false
	}], [PinchRecognizer, {
	  enable: false
	}, ['rotate']], [SwipeRecognizer, {
	  direction: DIRECTION_HORIZONTAL
	}], [PanRecognizer, {
	  direction: DIRECTION_HORIZONTAL
	}, ['swipe']], [TapRecognizer], [TapRecognizer, {
	  event: 'doubletap',
	  taps: 2
	}, ['tap']], [PressRecognizer]];
	var STOP = 1;
	var FORCED_STOP = 2;
	/**
	 * @private
	 * add/remove the css properties as defined in manager.options.cssProps
	 * @param {Manager} manager
	 * @param {Boolean} add
	 */

	function toggleCssProps(manager, add) {
	  var element = manager.element;

	  if (!element.style) {
	    return;
	  }

	  var prop;
	  each(manager.options.cssProps, function (value, name) {
	    prop = prefixed(element.style, name);

	    if (add) {
	      manager.oldCssProps[prop] = element.style[prop];
	      element.style[prop] = value;
	    } else {
	      element.style[prop] = manager.oldCssProps[prop] || "";
	    }
	  });

	  if (!add) {
	    manager.oldCssProps = {};
	  }
	}
	/**
	 * @private
	 * trigger dom event
	 * @param {String} event
	 * @param {Object} data
	 */


	function triggerDomEvent(event, data) {
	  var gestureEvent = document.createEvent("Event");
	  gestureEvent.initEvent(event, true, true);
	  gestureEvent.gesture = data;
	  data.target.dispatchEvent(gestureEvent);
	}
	/**
	* @private
	 * Manager
	 * @param {HTMLElement} element
	 * @param {Object} [options]
	 * @constructor
	 */


	var Manager = /*#__PURE__*/function () {
	  function Manager(element, options) {
	    var _this = this;

	    this.options = assign$1$1({}, defaults, options || {});
	    this.options.inputTarget = this.options.inputTarget || element;
	    this.handlers = {};
	    this.session = {};
	    this.recognizers = [];
	    this.oldCssProps = {};
	    this.element = element;
	    this.input = createInputInstance(this);
	    this.touchAction = new TouchAction(this, this.options.touchAction);
	    toggleCssProps(this, true);
	    each(this.options.recognizers, function (item) {
	      var recognizer = _this.add(new item[0](item[1]));

	      item[2] && recognizer.recognizeWith(item[2]);
	      item[3] && recognizer.requireFailure(item[3]);
	    }, this);
	  }
	  /**
	   * @private
	   * set options
	   * @param {Object} options
	   * @returns {Manager}
	   */


	  var _proto = Manager.prototype;

	  _proto.set = function set(options) {
	    assign$1$1(this.options, options); // Options that need a little more setup

	    if (options.touchAction) {
	      this.touchAction.update();
	    }

	    if (options.inputTarget) {
	      // Clean up existing event listeners and reinitialize
	      this.input.destroy();
	      this.input.target = options.inputTarget;
	      this.input.init();
	    }

	    return this;
	  };
	  /**
	   * @private
	   * stop recognizing for this session.
	   * This session will be discarded, when a new [input]start event is fired.
	   * When forced, the recognizer cycle is stopped immediately.
	   * @param {Boolean} [force]
	   */


	  _proto.stop = function stop(force) {
	    this.session.stopped = force ? FORCED_STOP : STOP;
	  };
	  /**
	   * @private
	   * run the recognizers!
	   * called by the inputHandler function on every movement of the pointers (touches)
	   * it walks through all the recognizers and tries to detect the gesture that is being made
	   * @param {Object} inputData
	   */


	  _proto.recognize = function recognize(inputData) {
	    var session = this.session;

	    if (session.stopped) {
	      return;
	    } // run the touch-action polyfill


	    this.touchAction.preventDefaults(inputData);
	    var recognizer;
	    var recognizers = this.recognizers; // this holds the recognizer that is being recognized.
	    // so the recognizer's state needs to be BEGAN, CHANGED, ENDED or RECOGNIZED
	    // if no recognizer is detecting a thing, it is set to `null`

	    var curRecognizer = session.curRecognizer; // reset when the last recognizer is recognized
	    // or when we're in a new session

	    if (!curRecognizer || curRecognizer && curRecognizer.state & STATE_RECOGNIZED) {
	      session.curRecognizer = null;
	      curRecognizer = null;
	    }

	    var i = 0;

	    while (i < recognizers.length) {
	      recognizer = recognizers[i]; // find out if we are allowed try to recognize the input for this one.
	      // 1.   allow if the session is NOT forced stopped (see the .stop() method)
	      // 2.   allow if we still haven't recognized a gesture in this session, or the this recognizer is the one
	      //      that is being recognized.
	      // 3.   allow if the recognizer is allowed to run simultaneous with the current recognized recognizer.
	      //      this can be setup with the `recognizeWith()` method on the recognizer.

	      if (session.stopped !== FORCED_STOP && ( // 1
	      !curRecognizer || recognizer === curRecognizer || // 2
	      recognizer.canRecognizeWith(curRecognizer))) {
	        // 3
	        recognizer.recognize(inputData);
	      } else {
	        recognizer.reset();
	      } // if the recognizer has been recognizing the input as a valid gesture, we want to store this one as the
	      // current active recognizer. but only if we don't already have an active recognizer


	      if (!curRecognizer && recognizer.state & (STATE_BEGAN | STATE_CHANGED | STATE_ENDED)) {
	        session.curRecognizer = recognizer;
	        curRecognizer = recognizer;
	      }

	      i++;
	    }
	  };
	  /**
	   * @private
	   * get a recognizer by its event name.
	   * @param {Recognizer|String} recognizer
	   * @returns {Recognizer|Null}
	   */


	  _proto.get = function get(recognizer) {
	    if (recognizer instanceof Recognizer) {
	      return recognizer;
	    }

	    var recognizers = this.recognizers;

	    for (var i = 0; i < recognizers.length; i++) {
	      if (recognizers[i].options.event === recognizer) {
	        return recognizers[i];
	      }
	    }

	    return null;
	  };
	  /**
	   * @private add a recognizer to the manager
	   * existing recognizers with the same event name will be removed
	   * @param {Recognizer} recognizer
	   * @returns {Recognizer|Manager}
	   */


	  _proto.add = function add(recognizer) {
	    if (invokeArrayArg(recognizer, "add", this)) {
	      return this;
	    } // remove existing


	    var existing = this.get(recognizer.options.event);

	    if (existing) {
	      this.remove(existing);
	    }

	    this.recognizers.push(recognizer);
	    recognizer.manager = this;
	    this.touchAction.update();
	    return recognizer;
	  };
	  /**
	   * @private
	   * remove a recognizer by name or instance
	   * @param {Recognizer|String} recognizer
	   * @returns {Manager}
	   */


	  _proto.remove = function remove(recognizer) {
	    if (invokeArrayArg(recognizer, "remove", this)) {
	      return this;
	    }

	    var targetRecognizer = this.get(recognizer); // let's make sure this recognizer exists

	    if (recognizer) {
	      var recognizers = this.recognizers;
	      var index = inArray(recognizers, targetRecognizer);

	      if (index !== -1) {
	        recognizers.splice(index, 1);
	        this.touchAction.update();
	      }
	    }

	    return this;
	  };
	  /**
	   * @private
	   * bind event
	   * @param {String} events
	   * @param {Function} handler
	   * @returns {EventEmitter} this
	   */


	  _proto.on = function on(events, handler) {
	    if (events === undefined || handler === undefined) {
	      return this;
	    }

	    var handlers = this.handlers;
	    each(splitStr(events), function (event) {
	      handlers[event] = handlers[event] || [];
	      handlers[event].push(handler);
	    });
	    return this;
	  };
	  /**
	   * @private unbind event, leave emit blank to remove all handlers
	   * @param {String} events
	   * @param {Function} [handler]
	   * @returns {EventEmitter} this
	   */


	  _proto.off = function off(events, handler) {
	    if (events === undefined) {
	      return this;
	    }

	    var handlers = this.handlers;
	    each(splitStr(events), function (event) {
	      if (!handler) {
	        delete handlers[event];
	      } else {
	        handlers[event] && handlers[event].splice(inArray(handlers[event], handler), 1);
	      }
	    });
	    return this;
	  };
	  /**
	   * @private emit event to the listeners
	   * @param {String} event
	   * @param {Object} data
	   */


	  _proto.emit = function emit(event, data) {
	    // we also want to trigger dom events
	    if (this.options.domEvents) {
	      triggerDomEvent(event, data);
	    } // no handlers, so skip it all


	    var handlers = this.handlers[event] && this.handlers[event].slice();

	    if (!handlers || !handlers.length) {
	      return;
	    }

	    data.type = event;

	    data.preventDefault = function () {
	      data.srcEvent.preventDefault();
	    };

	    var i = 0;

	    while (i < handlers.length) {
	      handlers[i](data);
	      i++;
	    }
	  };
	  /**
	   * @private
	   * destroy the manager and unbinds all events
	   * it doesn't unbind dom events, that is the user own responsibility
	   */


	  _proto.destroy = function destroy() {
	    this.element && toggleCssProps(this, false);
	    this.handlers = {};
	    this.session = {};
	    this.input.destroy();
	    this.element = null;
	  };

	  return Manager;
	}();

	var SINGLE_TOUCH_INPUT_MAP = {
	  touchstart: INPUT_START,
	  touchmove: INPUT_MOVE,
	  touchend: INPUT_END,
	  touchcancel: INPUT_CANCEL
	};
	var SINGLE_TOUCH_TARGET_EVENTS = 'touchstart';
	var SINGLE_TOUCH_WINDOW_EVENTS = 'touchstart touchmove touchend touchcancel';
	/**
	 * @private
	 * Touch events input
	 * @constructor
	 * @extends Input
	 */

	var SingleTouchInput = /*#__PURE__*/function (_Input) {
	  _inheritsLoose(SingleTouchInput, _Input);

	  function SingleTouchInput() {
	    var _this;

	    var proto = SingleTouchInput.prototype;
	    proto.evTarget = SINGLE_TOUCH_TARGET_EVENTS;
	    proto.evWin = SINGLE_TOUCH_WINDOW_EVENTS;
	    _this = _Input.apply(this, arguments) || this;
	    _this.started = false;
	    return _this;
	  }

	  var _proto = SingleTouchInput.prototype;

	  _proto.handler = function handler(ev) {
	    var type = SINGLE_TOUCH_INPUT_MAP[ev.type]; // should we handle the touch events?

	    if (type === INPUT_START) {
	      this.started = true;
	    }

	    if (!this.started) {
	      return;
	    }

	    var touches = normalizeSingleTouches.call(this, ev, type); // when done, reset the started state

	    if (type & (INPUT_END | INPUT_CANCEL) && touches[0].length - touches[1].length === 0) {
	      this.started = false;
	    }

	    this.callback(this.manager, type, {
	      pointers: touches[0],
	      changedPointers: touches[1],
	      pointerType: INPUT_TYPE_TOUCH,
	      srcEvent: ev
	    });
	  };

	  return SingleTouchInput;
	}(Input);

	function normalizeSingleTouches(ev, type) {
	  var all = toArray$1(ev.touches);
	  var changed = toArray$1(ev.changedTouches);

	  if (type & (INPUT_END | INPUT_CANCEL)) {
	    all = uniqueArray(all.concat(changed), 'identifier', true);
	  }

	  return [all, changed];
	}
	/**
	 * @private
	 * wrap a method with a deprecation warning and stack trace
	 * @param {Function} method
	 * @param {String} name
	 * @param {String} message
	 * @returns {Function} A new function wrapping the supplied method.
	 */


	function deprecate(method, name, message) {
	  var deprecationMessage = "DEPRECATED METHOD: " + name + "\n" + message + " AT \n";
	  return function () {
	    var e = new Error('get-stack-trace');
	    var stack = e && e.stack ? e.stack.replace(/^[^\(]+?[\n$]/gm, '').replace(/^\s+at\s+/gm, '').replace(/^Object.<anonymous>\s*\(/gm, '{anonymous}()@') : 'Unknown Stack Trace';
	    var log = window.console && (window.console.warn || window.console.log);

	    if (log) {
	      log.call(window.console, deprecationMessage, stack);
	    }

	    return method.apply(this, arguments);
	  };
	}
	/**
	 * @private
	 * extend object.
	 * means that properties in dest will be overwritten by the ones in src.
	 * @param {Object} dest
	 * @param {Object} src
	 * @param {Boolean} [merge=false]
	 * @returns {Object} dest
	 */


	var extend$1 = deprecate(function (dest, src, merge) {
	  var keys = Object.keys(src);
	  var i = 0;

	  while (i < keys.length) {
	    if (!merge || merge && dest[keys[i]] === undefined) {
	      dest[keys[i]] = src[keys[i]];
	    }

	    i++;
	  }

	  return dest;
	}, 'extend', 'Use `assign`.');
	/**
	 * @private
	 * merge the values from src in the dest.
	 * means that properties that exist in dest will not be overwritten by src
	 * @param {Object} dest
	 * @param {Object} src
	 * @returns {Object} dest
	 */

	var merge = deprecate(function (dest, src) {
	  return extend$1(dest, src, true);
	}, 'merge', 'Use `assign`.');
	/**
	 * @private
	 * simple class inheritance
	 * @param {Function} child
	 * @param {Function} base
	 * @param {Object} [properties]
	 */

	function inherit(child, base, properties) {
	  var baseP = base.prototype;
	  var childP;
	  childP = child.prototype = Object.create(baseP);
	  childP.constructor = child;
	  childP._super = baseP;

	  if (properties) {
	    assign$1$1(childP, properties);
	  }
	}
	/**
	 * @private
	 * simple function bind
	 * @param {Function} fn
	 * @param {Object} context
	 * @returns {Function}
	 */


	function bindFn(fn, context) {
	  return function boundFn() {
	    return fn.apply(context, arguments);
	  };
	}
	/**
	 * @private
	 * Simple way to create a manager with a default set of recognizers.
	 * @param {HTMLElement} element
	 * @param {Object} [options]
	 * @constructor
	 */


	var Hammer = /*#__PURE__*/function () {
	  var Hammer =
	  /**
	    * @private
	    * @const {string}
	    */
	  function Hammer(element, options) {
	    if (options === void 0) {
	      options = {};
	    }

	    return new Manager(element, _extends({
	      recognizers: preset.concat()
	    }, options));
	  };

	  Hammer.VERSION = "2.0.17-rc";
	  Hammer.DIRECTION_ALL = DIRECTION_ALL;
	  Hammer.DIRECTION_DOWN = DIRECTION_DOWN;
	  Hammer.DIRECTION_LEFT = DIRECTION_LEFT;
	  Hammer.DIRECTION_RIGHT = DIRECTION_RIGHT;
	  Hammer.DIRECTION_UP = DIRECTION_UP;
	  Hammer.DIRECTION_HORIZONTAL = DIRECTION_HORIZONTAL;
	  Hammer.DIRECTION_VERTICAL = DIRECTION_VERTICAL;
	  Hammer.DIRECTION_NONE = DIRECTION_NONE;
	  Hammer.DIRECTION_DOWN = DIRECTION_DOWN;
	  Hammer.INPUT_START = INPUT_START;
	  Hammer.INPUT_MOVE = INPUT_MOVE;
	  Hammer.INPUT_END = INPUT_END;
	  Hammer.INPUT_CANCEL = INPUT_CANCEL;
	  Hammer.STATE_POSSIBLE = STATE_POSSIBLE;
	  Hammer.STATE_BEGAN = STATE_BEGAN;
	  Hammer.STATE_CHANGED = STATE_CHANGED;
	  Hammer.STATE_ENDED = STATE_ENDED;
	  Hammer.STATE_RECOGNIZED = STATE_RECOGNIZED;
	  Hammer.STATE_CANCELLED = STATE_CANCELLED;
	  Hammer.STATE_FAILED = STATE_FAILED;
	  Hammer.Manager = Manager;
	  Hammer.Input = Input;
	  Hammer.TouchAction = TouchAction;
	  Hammer.TouchInput = TouchInput;
	  Hammer.MouseInput = MouseInput;
	  Hammer.PointerEventInput = PointerEventInput;
	  Hammer.TouchMouseInput = TouchMouseInput;
	  Hammer.SingleTouchInput = SingleTouchInput;
	  Hammer.Recognizer = Recognizer;
	  Hammer.AttrRecognizer = AttrRecognizer;
	  Hammer.Tap = TapRecognizer;
	  Hammer.Pan = PanRecognizer;
	  Hammer.Swipe = SwipeRecognizer;
	  Hammer.Pinch = PinchRecognizer;
	  Hammer.Rotate = RotateRecognizer;
	  Hammer.Press = PressRecognizer;
	  Hammer.on = addEventListeners;
	  Hammer.off = removeEventListeners;
	  Hammer.each = each;
	  Hammer.merge = merge;
	  Hammer.extend = extend$1;
	  Hammer.bindFn = bindFn;
	  Hammer.assign = assign$1$1;
	  Hammer.inherit = inherit;
	  Hammer.bindFn = bindFn;
	  Hammer.prefixed = prefixed;
	  Hammer.toArray = toArray$1;
	  Hammer.inArray = inArray;
	  Hammer.uniqueArray = uniqueArray;
	  Hammer.splitStr = splitStr;
	  Hammer.boolOrFn = boolOrFn;
	  Hammer.hasParent = hasParent$1;
	  Hammer.addEventListeners = addEventListeners;
	  Hammer.removeEventListeners = removeEventListeners;
	  Hammer.defaults = assign$1$1({}, defaults, {
	    preset: preset
	  });
	  return Hammer;
	}(); //  style loader but by script tag, not by the loader.

	var timsort = createCommonjsModule(function (module, exports) {
	  /****
	   * The MIT License
	   *
	   * Copyright (c) 2015 Marco Ziccardi
	   *
	   * Permission is hereby granted, free of charge, to any person obtaining a copy
	   * of this software and associated documentation files (the "Software"), to deal
	   * in the Software without restriction, including without limitation the rights
	   * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
	   * copies of the Software, and to permit persons to whom the Software is
	   * furnished to do so, subject to the following conditions:
	   *
	   * The above copyright notice and this permission notice shall be included in
	   * all copies or substantial portions of the Software.
	   *
	   * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	   * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	   * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	   * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	   * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
	   * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
	   * THE SOFTWARE.
	   *
	   ****/
	  (function (global, factory) {
	    {
	      factory(exports);
	    }
	  })(commonjsGlobal, function (exports) {

	    exports.__esModule = true;
	    exports.sort = sort;

	    function _classCallCheck(instance, Constructor) {
	      if (!(instance instanceof Constructor)) {
	        throw new TypeError('Cannot call a class as a function');
	      }
	    }

	    var DEFAULT_MIN_MERGE = 32;
	    var DEFAULT_MIN_GALLOPING = 7;
	    var DEFAULT_TMP_STORAGE_LENGTH = 256;
	    var POWERS_OF_TEN = [1e0, 1e1, 1e2, 1e3, 1e4, 1e5, 1e6, 1e7, 1e8, 1e9];

	    function log10(x) {
	      if (x < 1e5) {
	        if (x < 1e2) {
	          return x < 1e1 ? 0 : 1;
	        }

	        if (x < 1e4) {
	          return x < 1e3 ? 2 : 3;
	        }

	        return 4;
	      }

	      if (x < 1e7) {
	        return x < 1e6 ? 5 : 6;
	      }

	      if (x < 1e9) {
	        return x < 1e8 ? 7 : 8;
	      }

	      return 9;
	    }

	    function alphabeticalCompare(a, b) {
	      if (a === b) {
	        return 0;
	      }

	      if (~~a === a && ~~b === b) {
	        if (a === 0 || b === 0) {
	          return a < b ? -1 : 1;
	        }

	        if (a < 0 || b < 0) {
	          if (b >= 0) {
	            return -1;
	          }

	          if (a >= 0) {
	            return 1;
	          }

	          a = -a;
	          b = -b;
	        }

	        var al = log10(a);
	        var bl = log10(b);
	        var t = 0;

	        if (al < bl) {
	          a *= POWERS_OF_TEN[bl - al - 1];
	          b /= 10;
	          t = -1;
	        } else if (al > bl) {
	          b *= POWERS_OF_TEN[al - bl - 1];
	          a /= 10;
	          t = 1;
	        }

	        if (a === b) {
	          return t;
	        }

	        return a < b ? -1 : 1;
	      }

	      var aStr = String(a);
	      var bStr = String(b);

	      if (aStr === bStr) {
	        return 0;
	      }

	      return aStr < bStr ? -1 : 1;
	    }

	    function minRunLength(n) {
	      var r = 0;

	      while (n >= DEFAULT_MIN_MERGE) {
	        r |= n & 1;
	        n >>= 1;
	      }

	      return n + r;
	    }

	    function makeAscendingRun(array, lo, hi, compare) {
	      var runHi = lo + 1;

	      if (runHi === hi) {
	        return 1;
	      }

	      if (compare(array[runHi++], array[lo]) < 0) {
	        while (runHi < hi && compare(array[runHi], array[runHi - 1]) < 0) {
	          runHi++;
	        }

	        reverseRun(array, lo, runHi);
	      } else {
	        while (runHi < hi && compare(array[runHi], array[runHi - 1]) >= 0) {
	          runHi++;
	        }
	      }

	      return runHi - lo;
	    }

	    function reverseRun(array, lo, hi) {
	      hi--;

	      while (lo < hi) {
	        var t = array[lo];
	        array[lo++] = array[hi];
	        array[hi--] = t;
	      }
	    }

	    function binaryInsertionSort(array, lo, hi, start, compare) {
	      if (start === lo) {
	        start++;
	      }

	      for (; start < hi; start++) {
	        var pivot = array[start];
	        var left = lo;
	        var right = start;

	        while (left < right) {
	          var mid = left + right >>> 1;

	          if (compare(pivot, array[mid]) < 0) {
	            right = mid;
	          } else {
	            left = mid + 1;
	          }
	        }

	        var n = start - left;

	        switch (n) {
	          case 3:
	            array[left + 3] = array[left + 2];

	          case 2:
	            array[left + 2] = array[left + 1];

	          case 1:
	            array[left + 1] = array[left];
	            break;

	          default:
	            while (n > 0) {
	              array[left + n] = array[left + n - 1];
	              n--;
	            }

	        }

	        array[left] = pivot;
	      }
	    }

	    function gallopLeft(value, array, start, length, hint, compare) {
	      var lastOffset = 0;
	      var maxOffset = 0;
	      var offset = 1;

	      if (compare(value, array[start + hint]) > 0) {
	        maxOffset = length - hint;

	        while (offset < maxOffset && compare(value, array[start + hint + offset]) > 0) {
	          lastOffset = offset;
	          offset = (offset << 1) + 1;

	          if (offset <= 0) {
	            offset = maxOffset;
	          }
	        }

	        if (offset > maxOffset) {
	          offset = maxOffset;
	        }

	        lastOffset += hint;
	        offset += hint;
	      } else {
	        maxOffset = hint + 1;

	        while (offset < maxOffset && compare(value, array[start + hint - offset]) <= 0) {
	          lastOffset = offset;
	          offset = (offset << 1) + 1;

	          if (offset <= 0) {
	            offset = maxOffset;
	          }
	        }

	        if (offset > maxOffset) {
	          offset = maxOffset;
	        }

	        var tmp = lastOffset;
	        lastOffset = hint - offset;
	        offset = hint - tmp;
	      }

	      lastOffset++;

	      while (lastOffset < offset) {
	        var m = lastOffset + (offset - lastOffset >>> 1);

	        if (compare(value, array[start + m]) > 0) {
	          lastOffset = m + 1;
	        } else {
	          offset = m;
	        }
	      }

	      return offset;
	    }

	    function gallopRight(value, array, start, length, hint, compare) {
	      var lastOffset = 0;
	      var maxOffset = 0;
	      var offset = 1;

	      if (compare(value, array[start + hint]) < 0) {
	        maxOffset = hint + 1;

	        while (offset < maxOffset && compare(value, array[start + hint - offset]) < 0) {
	          lastOffset = offset;
	          offset = (offset << 1) + 1;

	          if (offset <= 0) {
	            offset = maxOffset;
	          }
	        }

	        if (offset > maxOffset) {
	          offset = maxOffset;
	        }

	        var tmp = lastOffset;
	        lastOffset = hint - offset;
	        offset = hint - tmp;
	      } else {
	        maxOffset = length - hint;

	        while (offset < maxOffset && compare(value, array[start + hint + offset]) >= 0) {
	          lastOffset = offset;
	          offset = (offset << 1) + 1;

	          if (offset <= 0) {
	            offset = maxOffset;
	          }
	        }

	        if (offset > maxOffset) {
	          offset = maxOffset;
	        }

	        lastOffset += hint;
	        offset += hint;
	      }

	      lastOffset++;

	      while (lastOffset < offset) {
	        var m = lastOffset + (offset - lastOffset >>> 1);

	        if (compare(value, array[start + m]) < 0) {
	          offset = m;
	        } else {
	          lastOffset = m + 1;
	        }
	      }

	      return offset;
	    }

	    var TimSort = function () {
	      function TimSort(array, compare) {
	        _classCallCheck(this, TimSort);

	        this.array = null;
	        this.compare = null;
	        this.minGallop = DEFAULT_MIN_GALLOPING;
	        this.length = 0;
	        this.tmpStorageLength = DEFAULT_TMP_STORAGE_LENGTH;
	        this.stackLength = 0;
	        this.runStart = null;
	        this.runLength = null;
	        this.stackSize = 0;
	        this.array = array;
	        this.compare = compare;
	        this.length = array.length;

	        if (this.length < 2 * DEFAULT_TMP_STORAGE_LENGTH) {
	          this.tmpStorageLength = this.length >>> 1;
	        }

	        this.tmp = new Array(this.tmpStorageLength);
	        this.stackLength = this.length < 120 ? 5 : this.length < 1542 ? 10 : this.length < 119151 ? 19 : 40;
	        this.runStart = new Array(this.stackLength);
	        this.runLength = new Array(this.stackLength);
	      }

	      TimSort.prototype.pushRun = function pushRun(runStart, runLength) {
	        this.runStart[this.stackSize] = runStart;
	        this.runLength[this.stackSize] = runLength;
	        this.stackSize += 1;
	      };

	      TimSort.prototype.mergeRuns = function mergeRuns() {
	        while (this.stackSize > 1) {
	          var n = this.stackSize - 2;

	          if (n >= 1 && this.runLength[n - 1] <= this.runLength[n] + this.runLength[n + 1] || n >= 2 && this.runLength[n - 2] <= this.runLength[n] + this.runLength[n - 1]) {
	            if (this.runLength[n - 1] < this.runLength[n + 1]) {
	              n--;
	            }
	          } else if (this.runLength[n] > this.runLength[n + 1]) {
	            break;
	          }

	          this.mergeAt(n);
	        }
	      };

	      TimSort.prototype.forceMergeRuns = function forceMergeRuns() {
	        while (this.stackSize > 1) {
	          var n = this.stackSize - 2;

	          if (n > 0 && this.runLength[n - 1] < this.runLength[n + 1]) {
	            n--;
	          }

	          this.mergeAt(n);
	        }
	      };

	      TimSort.prototype.mergeAt = function mergeAt(i) {
	        var compare = this.compare;
	        var array = this.array;
	        var start1 = this.runStart[i];
	        var length1 = this.runLength[i];
	        var start2 = this.runStart[i + 1];
	        var length2 = this.runLength[i + 1];
	        this.runLength[i] = length1 + length2;

	        if (i === this.stackSize - 3) {
	          this.runStart[i + 1] = this.runStart[i + 2];
	          this.runLength[i + 1] = this.runLength[i + 2];
	        }

	        this.stackSize--;
	        var k = gallopRight(array[start2], array, start1, length1, 0, compare);
	        start1 += k;
	        length1 -= k;

	        if (length1 === 0) {
	          return;
	        }

	        length2 = gallopLeft(array[start1 + length1 - 1], array, start2, length2, length2 - 1, compare);

	        if (length2 === 0) {
	          return;
	        }

	        if (length1 <= length2) {
	          this.mergeLow(start1, length1, start2, length2);
	        } else {
	          this.mergeHigh(start1, length1, start2, length2);
	        }
	      };

	      TimSort.prototype.mergeLow = function mergeLow(start1, length1, start2, length2) {
	        var compare = this.compare;
	        var array = this.array;
	        var tmp = this.tmp;
	        var i = 0;

	        for (i = 0; i < length1; i++) {
	          tmp[i] = array[start1 + i];
	        }

	        var cursor1 = 0;
	        var cursor2 = start2;
	        var dest = start1;
	        array[dest++] = array[cursor2++];

	        if (--length2 === 0) {
	          for (i = 0; i < length1; i++) {
	            array[dest + i] = tmp[cursor1 + i];
	          }

	          return;
	        }

	        if (length1 === 1) {
	          for (i = 0; i < length2; i++) {
	            array[dest + i] = array[cursor2 + i];
	          }

	          array[dest + length2] = tmp[cursor1];
	          return;
	        }

	        var minGallop = this.minGallop;

	        while (true) {
	          var count1 = 0;
	          var count2 = 0;
	          var exit = false;

	          do {
	            if (compare(array[cursor2], tmp[cursor1]) < 0) {
	              array[dest++] = array[cursor2++];
	              count2++;
	              count1 = 0;

	              if (--length2 === 0) {
	                exit = true;
	                break;
	              }
	            } else {
	              array[dest++] = tmp[cursor1++];
	              count1++;
	              count2 = 0;

	              if (--length1 === 1) {
	                exit = true;
	                break;
	              }
	            }
	          } while ((count1 | count2) < minGallop);

	          if (exit) {
	            break;
	          }

	          do {
	            count1 = gallopRight(array[cursor2], tmp, cursor1, length1, 0, compare);

	            if (count1 !== 0) {
	              for (i = 0; i < count1; i++) {
	                array[dest + i] = tmp[cursor1 + i];
	              }

	              dest += count1;
	              cursor1 += count1;
	              length1 -= count1;

	              if (length1 <= 1) {
	                exit = true;
	                break;
	              }
	            }

	            array[dest++] = array[cursor2++];

	            if (--length2 === 0) {
	              exit = true;
	              break;
	            }

	            count2 = gallopLeft(tmp[cursor1], array, cursor2, length2, 0, compare);

	            if (count2 !== 0) {
	              for (i = 0; i < count2; i++) {
	                array[dest + i] = array[cursor2 + i];
	              }

	              dest += count2;
	              cursor2 += count2;
	              length2 -= count2;

	              if (length2 === 0) {
	                exit = true;
	                break;
	              }
	            }

	            array[dest++] = tmp[cursor1++];

	            if (--length1 === 1) {
	              exit = true;
	              break;
	            }

	            minGallop--;
	          } while (count1 >= DEFAULT_MIN_GALLOPING || count2 >= DEFAULT_MIN_GALLOPING);

	          if (exit) {
	            break;
	          }

	          if (minGallop < 0) {
	            minGallop = 0;
	          }

	          minGallop += 2;
	        }

	        this.minGallop = minGallop;

	        if (minGallop < 1) {
	          this.minGallop = 1;
	        }

	        if (length1 === 1) {
	          for (i = 0; i < length2; i++) {
	            array[dest + i] = array[cursor2 + i];
	          }

	          array[dest + length2] = tmp[cursor1];
	        } else if (length1 === 0) {
	          throw new Error('mergeLow preconditions were not respected');
	        } else {
	          for (i = 0; i < length1; i++) {
	            array[dest + i] = tmp[cursor1 + i];
	          }
	        }
	      };

	      TimSort.prototype.mergeHigh = function mergeHigh(start1, length1, start2, length2) {
	        var compare = this.compare;
	        var array = this.array;
	        var tmp = this.tmp;
	        var i = 0;

	        for (i = 0; i < length2; i++) {
	          tmp[i] = array[start2 + i];
	        }

	        var cursor1 = start1 + length1 - 1;
	        var cursor2 = length2 - 1;
	        var dest = start2 + length2 - 1;
	        var customCursor = 0;
	        var customDest = 0;
	        array[dest--] = array[cursor1--];

	        if (--length1 === 0) {
	          customCursor = dest - (length2 - 1);

	          for (i = 0; i < length2; i++) {
	            array[customCursor + i] = tmp[i];
	          }

	          return;
	        }

	        if (length2 === 1) {
	          dest -= length1;
	          cursor1 -= length1;
	          customDest = dest + 1;
	          customCursor = cursor1 + 1;

	          for (i = length1 - 1; i >= 0; i--) {
	            array[customDest + i] = array[customCursor + i];
	          }

	          array[dest] = tmp[cursor2];
	          return;
	        }

	        var minGallop = this.minGallop;

	        while (true) {
	          var count1 = 0;
	          var count2 = 0;
	          var exit = false;

	          do {
	            if (compare(tmp[cursor2], array[cursor1]) < 0) {
	              array[dest--] = array[cursor1--];
	              count1++;
	              count2 = 0;

	              if (--length1 === 0) {
	                exit = true;
	                break;
	              }
	            } else {
	              array[dest--] = tmp[cursor2--];
	              count2++;
	              count1 = 0;

	              if (--length2 === 1) {
	                exit = true;
	                break;
	              }
	            }
	          } while ((count1 | count2) < minGallop);

	          if (exit) {
	            break;
	          }

	          do {
	            count1 = length1 - gallopRight(tmp[cursor2], array, start1, length1, length1 - 1, compare);

	            if (count1 !== 0) {
	              dest -= count1;
	              cursor1 -= count1;
	              length1 -= count1;
	              customDest = dest + 1;
	              customCursor = cursor1 + 1;

	              for (i = count1 - 1; i >= 0; i--) {
	                array[customDest + i] = array[customCursor + i];
	              }

	              if (length1 === 0) {
	                exit = true;
	                break;
	              }
	            }

	            array[dest--] = tmp[cursor2--];

	            if (--length2 === 1) {
	              exit = true;
	              break;
	            }

	            count2 = length2 - gallopLeft(array[cursor1], tmp, 0, length2, length2 - 1, compare);

	            if (count2 !== 0) {
	              dest -= count2;
	              cursor2 -= count2;
	              length2 -= count2;
	              customDest = dest + 1;
	              customCursor = cursor2 + 1;

	              for (i = 0; i < count2; i++) {
	                array[customDest + i] = tmp[customCursor + i];
	              }

	              if (length2 <= 1) {
	                exit = true;
	                break;
	              }
	            }

	            array[dest--] = array[cursor1--];

	            if (--length1 === 0) {
	              exit = true;
	              break;
	            }

	            minGallop--;
	          } while (count1 >= DEFAULT_MIN_GALLOPING || count2 >= DEFAULT_MIN_GALLOPING);

	          if (exit) {
	            break;
	          }

	          if (minGallop < 0) {
	            minGallop = 0;
	          }

	          minGallop += 2;
	        }

	        this.minGallop = minGallop;

	        if (minGallop < 1) {
	          this.minGallop = 1;
	        }

	        if (length2 === 1) {
	          dest -= length1;
	          cursor1 -= length1;
	          customDest = dest + 1;
	          customCursor = cursor1 + 1;

	          for (i = length1 - 1; i >= 0; i--) {
	            array[customDest + i] = array[customCursor + i];
	          }

	          array[dest] = tmp[cursor2];
	        } else if (length2 === 0) {
	          throw new Error('mergeHigh preconditions were not respected');
	        } else {
	          customCursor = dest - (length2 - 1);

	          for (i = 0; i < length2; i++) {
	            array[customCursor + i] = tmp[i];
	          }
	        }
	      };

	      return TimSort;
	    }();

	    function sort(array, compare, lo, hi) {
	      if (!Array.isArray(array)) {
	        throw new TypeError('Can only sort arrays');
	      }

	      if (!compare) {
	        compare = alphabeticalCompare;
	      } else if (typeof compare !== 'function') {
	        hi = lo;
	        lo = compare;
	        compare = alphabeticalCompare;
	      }

	      if (!lo) {
	        lo = 0;
	      }

	      if (!hi) {
	        hi = array.length;
	      }

	      var remaining = hi - lo;

	      if (remaining < 2) {
	        return;
	      }

	      var runLength = 0;

	      if (remaining < DEFAULT_MIN_MERGE) {
	        runLength = makeAscendingRun(array, lo, hi, compare);
	        binaryInsertionSort(array, lo, hi, lo + runLength, compare);
	        return;
	      }

	      var ts = new TimSort(array, compare);
	      var minRun = minRunLength(remaining);

	      do {
	        runLength = makeAscendingRun(array, lo, hi, compare);

	        if (runLength < minRun) {
	          var force = remaining;

	          if (force > minRun) {
	            force = minRun;
	          }

	          binaryInsertionSort(array, lo, lo + force, lo + runLength, compare);
	          runLength = force;
	        }

	        ts.pushRun(lo, runLength);
	        ts.mergeRuns();
	        remaining -= runLength;
	        lo += runLength;
	      } while (remaining !== 0);

	      ts.forceMergeRuns();
	    }
	  });
	});
	unwrapExports(timsort);

	var timsort$1 = timsort;
	var timsort_1 = timsort$1.sort;

	function _createForOfIteratorHelper$2(o){if(typeof symbol$4==="undefined"||getIteratorMethod$1(o)==null){if(isArray$3(o)||(o=_unsupportedIterableToArray$3(o))){var i=0;var F=function F(){};return {s:F,n:function n(){if(i>=o.length)return {done:true};return {done:false,value:o[i++]};},e:function e(_e2){throw _e2;},f:F};}throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");}var it,normalCompletion=true,didErr=false,err;return {s:function s(){it=getIterator$1(o);},n:function n(){var step=it.next();normalCompletion=step.done;return step;},e:function e(_e3){didErr=true;err=_e3;},f:function f(){try{if(!normalCompletion&&it.return!=null)it.return();}finally{if(didErr)throw err;}}};}function _unsupportedIterableToArray$3(o,minLen){var _context24;if(!o)return;if(typeof o==="string")return _arrayLikeToArray$3(o,minLen);var n=slice$3(_context24=Object.prototype.toString.call(o)).call(_context24,8,-1);if(n==="Object"&&o.constructor)n=o.constructor.name;if(n==="Map"||n==="Set")return from_1$2(o);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return _arrayLikeToArray$3(o,minLen);}function _arrayLikeToArray$3(arr,len){if(len==null||len>arr.length)len=arr.length;for(var i=0,arr2=new Array(len);i<len;i++){arr2[i]=arr[i];}return arr2;}function ownKeys$3(object,enumerableOnly){var keys=keys$6(object);if(getOwnPropertySymbols$2){var symbols=getOwnPropertySymbols$2(object);if(enumerableOnly)symbols=filter$2(symbols).call(symbols,function(sym){return getOwnPropertyDescriptor$3(object,sym).enumerable;});keys.push.apply(keys,symbols);}return keys;}function _objectSpread$2(target){for(var i=1;i<arguments.length;i++){var source=arguments[i]!=null?arguments[i]:{};if(i%2){var _context134;forEach$3(_context134=ownKeys$3(Object(source),true)).call(_context134,function(key){defineProperty$8(target,key,source[key]);});}else if(getOwnPropertyDescriptors$2){defineProperties$1(target,getOwnPropertyDescriptors$2(source));}else {var _context135;forEach$3(_context135=ownKeys$3(Object(source))).call(_context135,function(key){defineProperty$2(target,key,getOwnPropertyDescriptor$3(source,key));});}}return target;}function _createSuper$1(Derived){var hasNativeReflectConstruct=_isNativeReflectConstruct$1();return function(){var Super=getPrototypeOf$3(Derived),result;if(hasNativeReflectConstruct){var NewTarget=getPrototypeOf$3(this).constructor;result=construct$3(Super,arguments,NewTarget);}else {result=Super.apply(this,arguments);}return possibleConstructorReturn(this,result);};}function _isNativeReflectConstruct$1(){if(typeof Reflect==="undefined"||!construct$3)return false;if(construct$3.sham)return false;if(typeof Proxy==="function")return true;try{Date.prototype.toString.call(construct$3(Date,[],function(){}));return true;}catch(e){return false;}}/**
	 * Draw a circle.
	 *
	 * @param ctx - The context this shape will be rendered to.
	 * @param x - The position of the center on the x axis.
	 * @param y - The position of the center on the y axis.
	 * @param r - The radius of the circle.
	 */function drawCircle(ctx,x,y,r){ctx.beginPath();ctx.arc(x,y,r,0,2*Math.PI,false);ctx.closePath();}/**
	 * Draw a square.
	 *
	 * @param ctx - The context this shape will be rendered to.
	 * @param x - The position of the center on the x axis.
	 * @param y - The position of the center on the y axis.
	 * @param r - Half of the width and height of the square.
	 */function drawSquare(ctx,x,y,r){ctx.beginPath();ctx.rect(x-r,y-r,r*2,r*2);ctx.closePath();}/**
	 * Draw an equilateral triangle standing on a side.
	 *
	 * @param ctx - The context this shape will be rendered to.
	 * @param x - The position of the center on the x axis.
	 * @param y - The position of the center on the y axis.
	 * @param r - Half of the length of the sides.
	 *
	 * @remarks
	 * http://en.wikipedia.org/wiki/Equilateral_triangle
	 */function drawTriangle(ctx,x,y,r){ctx.beginPath();// the change in radius and the offset is here to center the shape
	r*=1.15;y+=0.275*r;var s=r*2;var s2=s/2;var ir=Math.sqrt(3)/6*s;// radius of inner circle
	var h=Math.sqrt(s*s-s2*s2);// height
	ctx.moveTo(x,y-(h-ir));ctx.lineTo(x+s2,y+ir);ctx.lineTo(x-s2,y+ir);ctx.lineTo(x,y-(h-ir));ctx.closePath();}/**
	 * Draw an equilateral triangle standing on a vertex.
	 *
	 * @param ctx - The context this shape will be rendered to.
	 * @param x - The position of the center on the x axis.
	 * @param y - The position of the center on the y axis.
	 * @param r - Half of the length of the sides.
	 *
	 * @remarks
	 * http://en.wikipedia.org/wiki/Equilateral_triangle
	 */function drawTriangleDown(ctx,x,y,r){ctx.beginPath();// the change in radius and the offset is here to center the shape
	r*=1.15;y-=0.275*r;var s=r*2;var s2=s/2;var ir=Math.sqrt(3)/6*s;// radius of inner circle
	var h=Math.sqrt(s*s-s2*s2);// height
	ctx.moveTo(x,y+(h-ir));ctx.lineTo(x+s2,y-ir);ctx.lineTo(x-s2,y-ir);ctx.lineTo(x,y+(h-ir));ctx.closePath();}/**
	 * Draw a star.
	 *
	 * @param ctx - The context this shape will be rendered to.
	 * @param x - The position of the center on the x axis.
	 * @param y - The position of the center on the y axis.
	 * @param r - The outer radius of the star.
	 */function drawStar(ctx,x,y,r){// http://www.html5canvastutorials.com/labs/html5-canvas-star-spinner/
	ctx.beginPath();// the change in radius and the offset is here to center the shape
	r*=0.82;y+=0.1*r;for(var n=0;n<10;n++){var radius=n%2===0?r*1.3:r*0.5;ctx.lineTo(x+radius*Math.sin(n*2*Math.PI/10),y-radius*Math.cos(n*2*Math.PI/10));}ctx.closePath();}/**
	 * Draw a diamond.
	 *
	 * @param ctx - The context this shape will be rendered to.
	 * @param x - The position of the center on the x axis.
	 * @param y - The position of the center on the y axis.
	 * @param r - Half of the width and height of the diamond.
	 *
	 * @remarks
	 * http://www.html5canvastutorials.com/labs/html5-canvas-star-spinner/
	 */function drawDiamond(ctx,x,y,r){ctx.beginPath();ctx.lineTo(x,y+r);ctx.lineTo(x+r,y);ctx.lineTo(x,y-r);ctx.lineTo(x-r,y);ctx.closePath();}/**
	 * Draw a rectangle with rounded corners.
	 *
	 * @param ctx - The context this shape will be rendered to.
	 * @param x - The position of the center on the x axis.
	 * @param y - The position of the center on the y axis.
	 * @param w - The width of the rectangle.
	 * @param h - The height of the rectangle.
	 * @param r - The radius of the corners.
	 *
	 * @remarks
	 * http://stackoverflow.com/questions/1255512/how-to-draw-a-rounded-rectangle-on-html-canvas
	 */function drawRoundRect(ctx,x,y,w,h,r){var r2d=Math.PI/180;if(w-2*r<0){r=w/2;}//ensure that the radius isn't too large for x
	if(h-2*r<0){r=h/2;}//ensure that the radius isn't too large for y
	ctx.beginPath();ctx.moveTo(x+r,y);ctx.lineTo(x+w-r,y);ctx.arc(x+w-r,y+r,r,r2d*270,r2d*360,false);ctx.lineTo(x+w,y+h-r);ctx.arc(x+w-r,y+h-r,r,0,r2d*90,false);ctx.lineTo(x+r,y+h);ctx.arc(x+r,y+h-r,r,r2d*90,r2d*180,false);ctx.lineTo(x,y+r);ctx.arc(x+r,y+r,r,r2d*180,r2d*270,false);ctx.closePath();}/**
	 * Draw an ellipse.
	 *
	 * @param ctx - The context this shape will be rendered to.
	 * @param x - The position of the center on the x axis.
	 * @param y - The position of the center on the y axis.
	 * @param w - The width of the ellipse.
	 * @param h - The height of the ellipse.
	 *
	 * @remarks
	 * http://stackoverflow.com/questions/2172798/how-to-draw-an-oval-in-html5-canvas
	 *
	 * Postfix '_vis' added to discern it from standard method ellipse().
	 */function drawEllipse(ctx,x,y,w,h){var kappa=0.5522848,ox=w/2*kappa,// control point offset horizontal
	oy=h/2*kappa,// control point offset vertical
	xe=x+w,// x-end
	ye=y+h,// y-end
	xm=x+w/2,// x-middle
	ym=y+h/2;// y-middle
	ctx.beginPath();ctx.moveTo(x,ym);ctx.bezierCurveTo(x,ym-oy,xm-ox,y,xm,y);ctx.bezierCurveTo(xm+ox,y,xe,ym-oy,xe,ym);ctx.bezierCurveTo(xe,ym+oy,xm+ox,ye,xm,ye);ctx.bezierCurveTo(xm-ox,ye,x,ym+oy,x,ym);ctx.closePath();}/**
	 * Draw an isometric cylinder.
	 *
	 * @param ctx - The context this shape will be rendered to.
	 * @param x - The position of the center on the x axis.
	 * @param y - The position of the center on the y axis.
	 * @param w - The width of the database.
	 * @param h - The height of the database.
	 *
	 * @remarks
	 * http://stackoverflow.com/questions/2172798/how-to-draw-an-oval-in-html5-canvas
	 */function drawDatabase(ctx,x,y,w,h){var f=1/3;var wEllipse=w;var hEllipse=h*f;var kappa=0.5522848,ox=wEllipse/2*kappa,// control point offset horizontal
	oy=hEllipse/2*kappa,// control point offset vertical
	xe=x+wEllipse,// x-end
	ye=y+hEllipse,// y-end
	xm=x+wEllipse/2,// x-middle
	ym=y+hEllipse/2,// y-middle
	ymb=y+(h-hEllipse/2),// y-midlle, bottom ellipse
	yeb=y+h;// y-end, bottom ellipse
	ctx.beginPath();ctx.moveTo(xe,ym);ctx.bezierCurveTo(xe,ym+oy,xm+ox,ye,xm,ye);ctx.bezierCurveTo(xm-ox,ye,x,ym+oy,x,ym);ctx.bezierCurveTo(x,ym-oy,xm-ox,y,xm,y);ctx.bezierCurveTo(xm+ox,y,xe,ym-oy,xe,ym);ctx.lineTo(xe,ymb);ctx.bezierCurveTo(xe,ymb+oy,xm+ox,yeb,xm,yeb);ctx.bezierCurveTo(xm-ox,yeb,x,ymb+oy,x,ymb);ctx.lineTo(x,ym);}/**
	 * Draw a dashed line.
	 *
	 * @param ctx - The context this shape will be rendered to.
	 * @param x - The start position on the x axis.
	 * @param y - The start position on the y axis.
	 * @param x2 - The end position on the x axis.
	 * @param y2 - The end position on the y axis.
	 * @param pattern - List of lengths starting with line and then alternating between space and line.
	 *
	 * @author David Jordan
	 * @date 2012-08-08
	 * @remarks
	 * http://stackoverflow.com/questions/4576724/dotted-stroke-in-canvas
	 */function drawDashedLine(ctx,x,y,x2,y2,pattern){ctx.beginPath();ctx.moveTo(x,y);var patternLength=pattern.length;var dx=x2-x;var dy=y2-y;var slope=dy/dx;var distRemaining=Math.sqrt(dx*dx+dy*dy);var patternIndex=0;var draw=true;var xStep=0;var dashLength=+pattern[0];while(distRemaining>=0.1){dashLength=+pattern[patternIndex++%patternLength];if(dashLength>distRemaining){dashLength=distRemaining;}xStep=Math.sqrt(dashLength*dashLength/(1+slope*slope));xStep=dx<0?-xStep:xStep;x+=xStep;y+=slope*xStep;if(draw===true){ctx.lineTo(x,y);}else {ctx.moveTo(x,y);}distRemaining-=dashLength;draw=!draw;}}/**
	 * Draw a hexagon.
	 *
	 * @param ctx - The context this shape will be rendered to.
	 * @param x - The position of the center on the x axis.
	 * @param y - The position of the center on the y axis.
	 * @param r - The radius of the hexagon.
	 */function drawHexagon(ctx,x,y,r){ctx.beginPath();var sides=6;var a=Math.PI*2/sides;ctx.moveTo(x+r,y);for(var i=1;i<sides;i++){ctx.lineTo(x+r*Math.cos(a*i),y+r*Math.sin(a*i));}ctx.closePath();}var shapeMap={circle:drawCircle,dashedLine:drawDashedLine,database:drawDatabase,diamond:drawDiamond,ellipse:drawEllipse,ellipse_vis:drawEllipse,hexagon:drawHexagon,roundRect:drawRoundRect,square:drawSquare,star:drawStar,triangle:drawTriangle,triangleDown:drawTriangleDown};/**
	 * Returns either custom or native drawing function base on supplied name.
	 *
	 * @param name - The name of the function. Either the name of a
	 * CanvasRenderingContext2D property or an export from shapes.ts without the
	 * draw prefix.
	 *
	 * @returns The function that can be used for rendering. In case of native
	 * CanvasRenderingContext2D function the API is normalized to
	 * `(ctx: CanvasRenderingContext2D, ...originalArgs) => void`.
	 */function getShape(name){if(Object.prototype.hasOwnProperty.call(shapeMap,name)){return shapeMap[name];}else {return function(ctx){for(var _len=arguments.length,args=new Array(_len>1?_len-1:0),_key=1;_key<_len;_key++){args[_key-1]=arguments[_key];}CanvasRenderingContext2D.prototype[name].call(ctx,args);};}}/**
	 * Parse a text source containing data in DOT language into a JSON object.
	 * The object contains two lists: one with nodes and one with edges.
	 *
	 * DOT language reference: http://www.graphviz.org/doc/info/lang.html
	 *
	 * DOT language attributes: http://graphviz.org/content/attrs
	 *
	 * @param {string} data     Text containing a graph in DOT-notation
	 * @return {Object} graph   An object containing two parameters:
	 *                          {Object[]} nodes
	 *                          {Object[]} edges
	 *
	 * -------------------------------------------
	 * TODO
	 * ====
	 *
	 * For label handling, this is an incomplete implementation. From docs (quote #3015):
	 * 
	 * > the escape sequences "\n", "\l" and "\r" divide the label into lines, centered, 
	 * > left-justified, and right-justified, respectively.
	 *
	 * Source: http://www.graphviz.org/content/attrs#kescString
	 *
	 * > As another aid for readability, dot allows double-quoted strings to span multiple physical
	 * > lines using the standard C convention of a backslash immediately preceding a newline
	 * > character
	 * > In addition, double-quoted strings can be concatenated using a '+' operator.
	 * > As HTML strings can contain newline characters, which are used solely for formatting,
	 * > the language does not allow escaped newlines or concatenation operators to be used
	 * > within them.
	 *
	 * - Currently, only '\\n' is handled
	 * - Note that text explicitly says 'labels'; the dot parser currently handles escape
	 *   sequences in **all** strings.
	 */function parseDOT(data){dot=data;return parseGraph();}// mapping of attributes from DOT (the keys) to vis.js (the values)
	var NODE_ATTR_MAPPING={'fontsize':'font.size','fontcolor':'font.color','labelfontcolor':'font.color','fontname':'font.face','color':['color.border','color.background'],'fillcolor':'color.background','tooltip':'title','labeltooltip':'title'};var EDGE_ATTR_MAPPING=create$4(NODE_ATTR_MAPPING);EDGE_ATTR_MAPPING.color='color.color';EDGE_ATTR_MAPPING.style='dashes';// token types enumeration
	var TOKENTYPE={NULL:0,DELIMITER:1,IDENTIFIER:2,UNKNOWN:3};// map with all delimiters
	var DELIMITERS={'{':true,'}':true,'[':true,']':true,';':true,'=':true,',':true,'->':true,'--':true};var dot='';// current dot file
	var index=0;// current index in dot file
	var c='';// current token character in expr
	var token='';// current token
	var tokenType=TOKENTYPE.NULL;// type of the token
	/**
	 * Get the first character from the dot file.
	 * The character is stored into the char c. If the end of the dot file is
	 * reached, the function puts an empty string in c.
	 */function first(){index=0;c=dot.charAt(0);}/**
	 * Get the next character from the dot file.
	 * The character is stored into the char c. If the end of the dot file is
	 * reached, the function puts an empty string in c.
	 */function next(){index++;c=dot.charAt(index);}/**
	 * Preview the next character from the dot file.
	 * @return {string} cNext
	 */function nextPreview(){return dot.charAt(index+1);}var regexAlphaNumeric=/[a-zA-Z_0-9.:#]/;/**
	 * Test whether given character is alphabetic or numeric
	 * @param {string} c
	 * @return {Boolean} isAlphaNumeric
	 */function isAlphaNumeric(c){return regexAlphaNumeric.test(c);}/**
	 * Merge all options of object b into object b
	 * @param {Object} a
	 * @param {Object} b
	 * @return {Object} a
	 */function merge$1(a,b){if(!a){a={};}if(b){for(var name in b){if(b.hasOwnProperty(name)){a[name]=b[name];}}}return a;}/**
	 * Set a value in an object, where the provided parameter name can be a
	 * path with nested parameters. For example:
	 *
	 *     var obj = {a: 2};
	 *     setValue(obj, 'b.c', 3);     // obj = {a: 2, b: {c: 3}}
	 *
	 * @param {Object} obj
	 * @param {string} path  A parameter name or dot-separated parameter path,
	 *                      like "color.highlight.border".
	 * @param {*} value
	 */function setValue(obj,path,value){var keys=path.split('.');var o=obj;while(keys.length){var key=keys.shift();if(keys.length){// this isn't the end point
	if(!o[key]){o[key]={};}o=o[key];}else {// this is the end point
	o[key]=value;}}}/**
	 * Add a node to a graph object. If there is already a node with
	 * the same id, their attributes will be merged.
	 * @param {Object} graph
	 * @param {Object} node
	 */function addNode(graph,node){var i,len;var current=null;// find root graph (in case of subgraph)
	var graphs=[graph];// list with all graphs from current graph to root graph
	var root=graph;while(root.parent){graphs.push(root.parent);root=root.parent;}// find existing node (at root level) by its id
	if(root.nodes){for(i=0,len=root.nodes.length;i<len;i++){if(node.id===root.nodes[i].id){current=root.nodes[i];break;}}}if(!current){// this is a new node
	current={id:node.id};if(graph.node){// clone default attributes
	current.attr=merge$1(current.attr,graph.node);}}// add node to this (sub)graph and all its parent graphs
	for(i=graphs.length-1;i>=0;i--){var _context;var g=graphs[i];if(!g.nodes){g.nodes=[];}if(indexOf$3(_context=g.nodes).call(_context,current)===-1){g.nodes.push(current);}}// merge attributes
	if(node.attr){current.attr=merge$1(current.attr,node.attr);}}/**
	 * Add an edge to a graph object
	 * @param {Object} graph
	 * @param {Object} edge
	 */function addEdge(graph,edge){if(!graph.edges){graph.edges=[];}graph.edges.push(edge);if(graph.edge){var attr=merge$1({},graph.edge);// clone default attributes
	edge.attr=merge$1(attr,edge.attr);// merge attributes
	}}/**
	 * Create an edge to a graph object
	 * @param {Object} graph
	 * @param {string | number | Object} from
	 * @param {string | number | Object} to
	 * @param {string} type
	 * @param {Object | null} attr
	 * @return {Object} edge
	 */function createEdge(graph,from,to,type,attr){var edge={from:from,to:to,type:type};if(graph.edge){edge.attr=merge$1({},graph.edge);// clone default attributes
	}edge.attr=merge$1(edge.attr||{},attr);// merge attributes
	// Move arrows attribute from attr to edge temporally created in
	// parseAttributeList().
	if(attr!=null){if(attr.hasOwnProperty('arrows')&&attr['arrows']!=null){edge['arrows']={to:{enabled:true,type:attr.arrows.type}};attr['arrows']=null;}}return edge;}/**
	 * Get next token in the current dot file.
	 * The token and token type are available as token and tokenType
	 */function getToken(){tokenType=TOKENTYPE.NULL;token='';// skip over whitespaces
	while(c===' '||c==='\t'||c==='\n'||c==='\r'){// space, tab, enter
	next();}do{var isComment=false;// skip comment
	if(c==='#'){// find the previous non-space character
	var i=index-1;while(dot.charAt(i)===' '||dot.charAt(i)==='\t'){i--;}if(dot.charAt(i)==='\n'||dot.charAt(i)===''){// the # is at the start of a line, this is indeed a line comment
	while(c!=''&&c!='\n'){next();}isComment=true;}}if(c==='/'&&nextPreview()==='/'){// skip line comment
	while(c!=''&&c!='\n'){next();}isComment=true;}if(c==='/'&&nextPreview()==='*'){// skip block comment
	while(c!=''){if(c==='*'&&nextPreview()==='/'){// end of block comment found. skip these last two characters
	next();next();break;}else {next();}}isComment=true;}// skip over whitespaces
	while(c===' '||c==='\t'||c==='\n'||c==='\r'){// space, tab, enter
	next();}}while(isComment);// check for end of dot file
	if(c===''){// token is still empty
	tokenType=TOKENTYPE.DELIMITER;return;}// check for delimiters consisting of 2 characters
	var c2=c+nextPreview();if(DELIMITERS[c2]){tokenType=TOKENTYPE.DELIMITER;token=c2;next();next();return;}// check for delimiters consisting of 1 character
	if(DELIMITERS[c]){tokenType=TOKENTYPE.DELIMITER;token=c;next();return;}// check for an identifier (number or string)
	// TODO: more precise parsing of numbers/strings (and the port separator ':')
	if(isAlphaNumeric(c)||c==='-'){token+=c;next();while(isAlphaNumeric(c)){token+=c;next();}if(token==='false'){token=false;// convert to boolean
	}else if(token==='true'){token=true;// convert to boolean
	}else if(!isNaN(Number(token))){token=Number(token);// convert to number
	}tokenType=TOKENTYPE.IDENTIFIER;return;}// check for a string enclosed by double quotes
	if(c==='"'){next();while(c!=''&&(c!='"'||c==='"'&&nextPreview()==='"')){if(c==='"'){// skip the escape character
	token+=c;next();}else if(c==='\\'&&nextPreview()==='n'){// Honor a newline escape sequence
	token+='\n';next();}else {token+=c;}next();}if(c!='"'){throw newSyntaxError('End of string " expected');}next();tokenType=TOKENTYPE.IDENTIFIER;return;}// something unknown is found, wrong characters, a syntax error
	tokenType=TOKENTYPE.UNKNOWN;while(c!=''){token+=c;next();}throw new SyntaxError('Syntax error in part "'+chop(token,30)+'"');}/**
	 * Parse a graph.
	 * @returns {Object} graph
	 */function parseGraph(){var graph={};first();getToken();// optional strict keyword
	if(token==='strict'){graph.strict=true;getToken();}// graph or digraph keyword
	if(token==='graph'||token==='digraph'){graph.type=token;getToken();}// optional graph id
	if(tokenType===TOKENTYPE.IDENTIFIER){graph.id=token;getToken();}// open angle bracket
	if(token!='{'){throw newSyntaxError('Angle bracket { expected');}getToken();// statements
	parseStatements(graph);// close angle bracket
	if(token!='}'){throw newSyntaxError('Angle bracket } expected');}getToken();// end of file
	if(token!==''){throw newSyntaxError('End of file expected');}getToken();// remove temporary default options
	delete graph.node;delete graph.edge;delete graph.graph;return graph;}/**
	 * Parse a list with statements.
	 * @param {Object} graph
	 */function parseStatements(graph){while(token!==''&&token!='}'){parseStatement(graph);if(token===';'){getToken();}}}/**
	 * Parse a single statement. Can be a an attribute statement, node
	 * statement, a series of node statements and edge statements, or a
	 * parameter.
	 * @param {Object} graph
	 */function parseStatement(graph){// parse subgraph
	var subgraph=parseSubgraph(graph);if(subgraph){// edge statements
	parseEdge(graph,subgraph);return;}// parse an attribute statement
	var attr=parseAttributeStatement(graph);if(attr){return;}// parse node
	if(tokenType!=TOKENTYPE.IDENTIFIER){throw newSyntaxError('Identifier expected');}var id=token;// id can be a string or a number
	getToken();if(token==='='){// id statement
	getToken();if(tokenType!=TOKENTYPE.IDENTIFIER){throw newSyntaxError('Identifier expected');}graph[id]=token;getToken();// TODO: implement comma separated list with "a_list: ID=ID [','] [a_list] "
	}else {parseNodeStatement(graph,id);}}/**
	 * Parse a subgraph
	 * @param {Object} graph    parent graph object
	 * @return {Object | null} subgraph
	 */function parseSubgraph(graph){var subgraph=null;// optional subgraph keyword
	if(token==='subgraph'){subgraph={};subgraph.type='subgraph';getToken();// optional graph id
	if(tokenType===TOKENTYPE.IDENTIFIER){subgraph.id=token;getToken();}}// open angle bracket
	if(token==='{'){getToken();if(!subgraph){subgraph={};}subgraph.parent=graph;subgraph.node=graph.node;subgraph.edge=graph.edge;subgraph.graph=graph.graph;// statements
	parseStatements(subgraph);// close angle bracket
	if(token!='}'){throw newSyntaxError('Angle bracket } expected');}getToken();// remove temporary default options
	delete subgraph.node;delete subgraph.edge;delete subgraph.graph;delete subgraph.parent;// register at the parent graph
	if(!graph.subgraphs){graph.subgraphs=[];}graph.subgraphs.push(subgraph);}return subgraph;}/**
	 * parse an attribute statement like "node [shape=circle fontSize=16]".
	 * Available keywords are 'node', 'edge', 'graph'.
	 * The previous list with default attributes will be replaced
	 * @param {Object} graph
	 * @returns {String | null} keyword Returns the name of the parsed attribute
	 *                                  (node, edge, graph), or null if nothing
	 *                                  is parsed.
	 */function parseAttributeStatement(graph){// attribute statements
	if(token==='node'){getToken();// node attributes
	graph.node=parseAttributeList();return 'node';}else if(token==='edge'){getToken();// edge attributes
	graph.edge=parseAttributeList();return 'edge';}else if(token==='graph'){getToken();// graph attributes
	graph.graph=parseAttributeList();return 'graph';}return null;}/**
	 * parse a node statement
	 * @param {Object} graph
	 * @param {string | number} id
	 */function parseNodeStatement(graph,id){// node statement
	var node={id:id};var attr=parseAttributeList();if(attr){node.attr=attr;}addNode(graph,node);// edge statements
	parseEdge(graph,id);}/**
	 * Parse an edge or a series of edges
	 * @param {Object} graph
	 * @param {string | number} from        Id of the from node
	 */function parseEdge(graph,from){while(token==='->'||token==='--'){var to;var type=token;getToken();var subgraph=parseSubgraph(graph);if(subgraph){to=subgraph;}else {if(tokenType!=TOKENTYPE.IDENTIFIER){throw newSyntaxError('Identifier or subgraph expected');}to=token;addNode(graph,{id:to});getToken();}// parse edge attributes
	var attr=parseAttributeList();// create edge
	var edge=createEdge(graph,from,to,type,attr);addEdge(graph,edge);from=to;}}/**
	 * Parse a set with attributes,
	 * for example [label="1.000", shape=solid]
	 * @return {Object | null} attr
	 */function parseAttributeList(){var i;var attr=null;// edge styles of dot and vis
	var edgeStyles={'dashed':true,'solid':false,'dotted':[1,5]};/**
	   * Define arrow types.
	   * vis currently supports types defined in 'arrowTypes'.
	   * Details of arrow shapes are described in
	   * http://www.graphviz.org/content/arrow-shapes
	   */var arrowTypes={dot:'circle',box:'box',crow:'crow',curve:'curve',icurve:'inv_curve',normal:'triangle',inv:'inv_triangle',diamond:'diamond',tee:'bar',vee:'vee'};/**
	   * 'attr_list' contains attributes for checking if some of them are affected
	   * later. For instance, both of 'arrowhead' and 'dir' (edge style defined
	   * in DOT) make changes to 'arrows' attribute in vis.
	   */var attr_list=new Array();var attr_names=new Array();// used for checking the case.
	// parse attributes
	while(token==='['){getToken();attr={};while(token!==''&&token!=']'){if(tokenType!=TOKENTYPE.IDENTIFIER){throw newSyntaxError('Attribute name expected');}var name=token;getToken();if(token!='='){throw newSyntaxError('Equal sign = expected');}getToken();if(tokenType!=TOKENTYPE.IDENTIFIER){throw newSyntaxError('Attribute value expected');}var value=token;// convert from dot style to vis
	if(name==='style'){value=edgeStyles[value];}var arrowType;if(name==='arrowhead'){arrowType=arrowTypes[value];name='arrows';value={'to':{'enabled':true,'type':arrowType}};}if(name==='arrowtail'){arrowType=arrowTypes[value];name='arrows';value={'from':{'enabled':true,'type':arrowType}};}attr_list.push({'attr':attr,'name':name,'value':value});attr_names.push(name);getToken();if(token==','){getToken();}}if(token!=']'){throw newSyntaxError('Bracket ] expected');}getToken();}/**
	   * As explained in [1], graphviz has limitations for combination of
	   * arrow[head|tail] and dir. If attribute list includes 'dir',
	   * following cases just be supported.
	   *   1. both or none + arrowhead, arrowtail
	   *   2. forward + arrowhead (arrowtail is not affedted)
	   *   3. back + arrowtail (arrowhead is not affected)
	   * [1] https://www.graphviz.org/doc/info/attrs.html#h:undir_note
	   */if(includes$4(attr_names).call(attr_names,'dir')){var idx={};// get index of 'arrows' and 'dir'
	idx.arrows={};for(i=0;i<attr_list.length;i++){if(attr_list[i].name==='arrows'){if(attr_list[i].value.to!=null){idx.arrows.to=i;}else if(attr_list[i].value.from!=null){idx.arrows.from=i;}else {throw newSyntaxError('Invalid value of arrows');}}else if(attr_list[i].name==='dir'){idx.dir=i;}}// first, add default arrow shape if it is not assigned to avoid error
	var dir_type=attr_list[idx.dir].value;if(!includes$4(attr_names).call(attr_names,'arrows')){if(dir_type==='both'){attr_list.push({'attr':attr_list[idx.dir].attr,'name':'arrows','value':{to:{enabled:true}}});idx.arrows.to=attr_list.length-1;attr_list.push({'attr':attr_list[idx.dir].attr,'name':'arrows','value':{from:{enabled:true}}});idx.arrows.from=attr_list.length-1;}else if(dir_type==='forward'){attr_list.push({'attr':attr_list[idx.dir].attr,'name':'arrows','value':{to:{enabled:true}}});idx.arrows.to=attr_list.length-1;}else if(dir_type==='back'){attr_list.push({'attr':attr_list[idx.dir].attr,'name':'arrows','value':{from:{enabled:true}}});idx.arrows.from=attr_list.length-1;}else if(dir_type==='none'){attr_list.push({'attr':attr_list[idx.dir].attr,'name':'arrows','value':''});idx.arrows.to=attr_list.length-1;}else {throw newSyntaxError('Invalid dir type "'+dir_type+'"');}}var from_type;var to_type;// update 'arrows' attribute from 'dir'.
	if(dir_type==='both'){// both of shapes of 'from' and 'to' are given
	if(idx.arrows.to&&idx.arrows.from){to_type=attr_list[idx.arrows.to].value.to.type;from_type=attr_list[idx.arrows.from].value.from.type;attr_list[idx.arrows.to]={'attr':attr_list[idx.arrows.to].attr,'name':attr_list[idx.arrows.to].name,'value':{to:{enabled:true,type:to_type},from:{enabled:true,type:from_type}}};splice$2(attr_list).call(attr_list,idx.arrows.from,1);// shape of 'to' is assigned and use default to 'from'
	}else if(idx.arrows.to){to_type=attr_list[idx.arrows.to].value.to.type;from_type='arrow';attr_list[idx.arrows.to]={'attr':attr_list[idx.arrows.to].attr,'name':attr_list[idx.arrows.to].name,'value':{to:{enabled:true,type:to_type},from:{enabled:true,type:from_type}}};// only shape of 'from' is assigned and use default for 'to'
	}else if(idx.arrows.from){to_type='arrow';from_type=attr_list[idx.arrows.from].value.from.type;attr_list[idx.arrows.from]={'attr':attr_list[idx.arrows.from].attr,'name':attr_list[idx.arrows.from].name,'value':{to:{enabled:true,type:to_type},from:{enabled:true,type:from_type}}};}}else if(dir_type==='back'){// given both of shapes, but use only 'from'
	if(idx.arrows.to&&idx.arrows.from){to_type='';from_type=attr_list[idx.arrows.from].value.from.type;attr_list[idx.arrows.from]={'attr':attr_list[idx.arrows.from].attr,'name':attr_list[idx.arrows.from].name,'value':{to:{enabled:true,type:to_type},from:{enabled:true,type:from_type}}};// given shape of 'to', but does not use it
	}else if(idx.arrows.to){to_type='';from_type='arrow';idx.arrows.from=idx.arrows.to;attr_list[idx.arrows.from]={'attr':attr_list[idx.arrows.from].attr,'name':attr_list[idx.arrows.from].name,'value':{to:{enabled:true,type:to_type},from:{enabled:true,type:from_type}}};// assign given 'from' shape
	}else if(idx.arrows.from){to_type='';from_type=attr_list[idx.arrows.from].value.from.type;attr_list[idx.arrows.to]={'attr':attr_list[idx.arrows.from].attr,'name':attr_list[idx.arrows.from].name,'value':{to:{enabled:true,type:to_type},from:{enabled:true,type:from_type}}};}attr_list[idx.arrows.from]={'attr':attr_list[idx.arrows.from].attr,'name':attr_list[idx.arrows.from].name,'value':{from:{enabled:true,type:attr_list[idx.arrows.from].value.from.type}}};}else if(dir_type==='none'){var idx_arrow;if(idx.arrows.to){idx_arrow=idx.arrows.to;}else {idx_arrow=idx.arrows.from;}attr_list[idx_arrow]={'attr':attr_list[idx_arrow].attr,'name':attr_list[idx_arrow].name,'value':''};}else if(dir_type==='forward'){// given both of shapes, but use only 'to'
	if(idx.arrows.to&&idx.arrows.from){to_type=attr_list[idx.arrows.to].value.to.type;from_type='';attr_list[idx.arrows.to]={'attr':attr_list[idx.arrows.to].attr,'name':attr_list[idx.arrows.to].name,'value':{to:{enabled:true,type:to_type},from:{enabled:true,type:from_type}}};// assign given 'to' shape
	}else if(idx.arrows.to){to_type=attr_list[idx.arrows.to].value.to.type;from_type='';attr_list[idx.arrows.to]={'attr':attr_list[idx.arrows.to].attr,'name':attr_list[idx.arrows.to].name,'value':{to:{enabled:true,type:to_type},from:{enabled:true,type:from_type}}};// given shape of 'from', but does not use it
	}else if(idx.arrows.from){to_type='arrow';from_type='';idx.arrows.to=idx.arrows.from;attr_list[idx.arrows.to]={'attr':attr_list[idx.arrows.to].attr,'name':attr_list[idx.arrows.to].name,'value':{to:{enabled:true,type:to_type},from:{enabled:true,type:from_type}}};}attr_list[idx.arrows.to]={'attr':attr_list[idx.arrows.to].attr,'name':attr_list[idx.arrows.to].name,'value':{to:{enabled:true,type:attr_list[idx.arrows.to].value.to.type}}};}else {throw newSyntaxError('Invalid dir type "'+dir_type+'"');}// remove 'dir' attribute no need anymore
	splice$2(attr_list).call(attr_list,idx.dir,1);}// parse 'penwidth'
	var nof_attr_list;if(includes$4(attr_names).call(attr_names,'penwidth')){var tmp_attr_list=[];nof_attr_list=attr_list.length;for(i=0;i<nof_attr_list;i++){// exclude 'width' from attr_list if 'penwidth' exists
	if(attr_list[i].name!=='width'){if(attr_list[i].name==='penwidth'){attr_list[i].name='width';}tmp_attr_list.push(attr_list[i]);}}attr_list=tmp_attr_list;}nof_attr_list=attr_list.length;for(i=0;i<nof_attr_list;i++){setValue(attr_list[i].attr,attr_list[i].name,attr_list[i].value);}return attr;}/**
	 * Create a syntax error with extra information on current token and index.
	 * @param {string} message
	 * @returns {SyntaxError} err
	 */function newSyntaxError(message){return new SyntaxError(message+', got "'+chop(token,30)+'" (char '+index+')');}/**
	 * Chop off text after a maximum length
	 * @param {string} text
	 * @param {number} maxLength
	 * @returns {String}
	 */function chop(text,maxLength){return text.length<=maxLength?text:text.substr(0,27)+'...';}/**
	 * Execute a function fn for each pair of elements in two arrays
	 * @param {Array | *} array1
	 * @param {Array | *} array2
	 * @param {function} fn
	 */function forEach2(array1,array2,fn){if(isArray$3(array1)){forEach$3(array1).call(array1,function(elem1){if(isArray$3(array2)){forEach$3(array2).call(array2,function(elem2){fn(elem1,elem2);});}else {fn(elem1,array2);}});}else {if(isArray$3(array2)){forEach$3(array2).call(array2,function(elem2){fn(array1,elem2);});}else {fn(array1,array2);}}}/**
	 * Set a nested property on an object
	 * When nested objects are missing, they will be created.
	 * For example setProp({}, 'font.color', 'red') will return {font: {color: 'red'}}
	 * @param {Object} object
	 * @param {string} path   A dot separated string like 'font.color'
	 * @param {*} value       Value for the property
	 * @return {Object} Returns the original object, allows for chaining.
	 */function setProp(object,path,value){var names=path.split('.');var prop=names.pop();// traverse over the nested objects
	var obj=object;for(var i=0;i<names.length;i++){var name=names[i];if(!(name in obj)){obj[name]={};}obj=obj[name];}// set the property value
	obj[prop]=value;return object;}/**
	 * Convert an object with DOT attributes to their vis.js equivalents.
	 * @param {Object} attr     Object with DOT attributes
	 * @param {Object} mapping
	 * @return {Object}         Returns an object with vis.js attributes
	 */function convertAttr(attr,mapping){var converted={};for(var prop in attr){if(attr.hasOwnProperty(prop)){var visProp=mapping[prop];if(isArray$3(visProp)){forEach$3(visProp).call(visProp,function(visPropI){setProp(converted,visPropI,attr[prop]);});}else if(typeof visProp==='string'){setProp(converted,visProp,attr[prop]);}else {setProp(converted,prop,attr[prop]);}}}return converted;}/**
	 * Convert a string containing a graph in DOT language into a map containing
	 * with nodes and edges in the format of graph.
	 * @param {string} data         Text containing a graph in DOT-notation
	 * @return {Object} graphData
	 */function DOTToGraph(data){// parse the DOT file
	var dotData=parseDOT(data);var graphData={nodes:[],edges:[],options:{}};// copy the nodes
	if(dotData.nodes){var _context2;forEach$3(_context2=dotData.nodes).call(_context2,function(dotNode){var graphNode={id:dotNode.id,label:String(dotNode.label||dotNode.id)};merge$1(graphNode,convertAttr(dotNode.attr,NODE_ATTR_MAPPING));if(graphNode.image){graphNode.shape='image';}graphData.nodes.push(graphNode);});}// copy the edges
	if(dotData.edges){var _context3;/**
	     * Convert an edge in DOT format to an edge with VisGraph format
	     * @param {Object} dotEdge
	     * @returns {Object} graphEdge
	     */var convertEdge=function convertEdge(dotEdge){var graphEdge={from:dotEdge.from,to:dotEdge.to};merge$1(graphEdge,convertAttr(dotEdge.attr,EDGE_ATTR_MAPPING));// Add arrows attribute to default styled arrow.
	// The reason why default style is not added in parseAttributeList() is
	// because only default is cleared before here.
	if(graphEdge.arrows==null&&dotEdge.type==='->'){graphEdge.arrows='to';}return graphEdge;};forEach$3(_context3=dotData.edges).call(_context3,function(dotEdge){var from,to;if(dotEdge.from instanceof Object){from=dotEdge.from.nodes;}else {from={id:dotEdge.from};}if(dotEdge.to instanceof Object){to=dotEdge.to.nodes;}else {to={id:dotEdge.to};}if(dotEdge.from instanceof Object&&dotEdge.from.edges){var _context4;forEach$3(_context4=dotEdge.from.edges).call(_context4,function(subEdge){var graphEdge=convertEdge(subEdge);graphData.edges.push(graphEdge);});}forEach2(from,to,function(from,to){var subEdge=createEdge(graphData,from.id,to.id,dotEdge.type,dotEdge.attr);var graphEdge=convertEdge(subEdge);graphData.edges.push(graphEdge);});if(dotEdge.to instanceof Object&&dotEdge.to.edges){var _context5;forEach$3(_context5=dotEdge.to.edges).call(_context5,function(subEdge){var graphEdge=convertEdge(subEdge);graphData.edges.push(graphEdge);});}});}// copy the options
	if(dotData.attr){graphData.options=dotData.attr;}return graphData;}// exports
	var parseDOT_1=parseDOT;var DOTToGraph_1=DOTToGraph;var dotparser={parseDOT:parseDOT_1,DOTToGraph:DOTToGraph_1};var dotparser$1=/*#__PURE__*/freeze$2({__proto__:null,'default':dotparser,__moduleExports:dotparser,parseDOT:parseDOT_1,DOTToGraph:DOTToGraph_1});/**
	 * Convert Gephi to Vis.
	 *
	 * @param gephiJSON - The parsed JSON data in Gephi format.
	 * @param optionsObj - Additional options.
	 *
	 * @returns The converted data ready to be used in Vis.
	 */function parseGephi(gephiJSON,optionsObj){var _context6;var options={edges:{inheritColor:false},nodes:{fixed:false,parseColor:false}};if(optionsObj!=null){if(optionsObj.fixed!=null){options.nodes.fixed=optionsObj.fixed;}if(optionsObj.parseColor!=null){options.nodes.parseColor=optionsObj.parseColor;}if(optionsObj.inheritColor!=null){options.edges.inheritColor=optionsObj.inheritColor;}}var gEdges=gephiJSON.edges;var vEdges=map$5(gEdges).call(gEdges,function(gEdge){var vEdge={from:gEdge.source,id:gEdge.id,to:gEdge.target};if(gEdge.attributes!=null){vEdge.attributes=gEdge.attributes;}if(gEdge.label!=null){vEdge.label=gEdge.label;}if(gEdge.attributes!=null&&gEdge.attributes.title!=null){vEdge.title=gEdge.attributes.title;}if(gEdge.type==='Directed'){vEdge.arrows='to';}// edge['value'] = gEdge.attributes != null ? gEdge.attributes.Weight : undefined;
	// edge['width'] = edge['value'] != null ? undefined : edgegEdge.size;
	if(gEdge.color&&options.edges.inheritColor===false){vEdge.color=gEdge.color;}return vEdge;});var vNodes=map$5(_context6=gephiJSON.nodes).call(_context6,function(gNode){var vNode={id:gNode.id,fixed:options.nodes.fixed&&gNode.x!=null&&gNode.y!=null};if(gNode.attributes!=null){vNode.attributes=gNode.attributes;}if(gNode.label!=null){vNode.label=gNode.label;}if(gNode.size!=null){vNode.size=gNode.size;}if(gNode.attributes!=null&&gNode.attributes.title!=null){vNode.title=gNode.attributes.title;}if(gNode.title!=null){vNode.title=gNode.title;}if(gNode.x!=null){vNode.x=gNode.x;}if(gNode.y!=null){vNode.y=gNode.y;}if(gNode.color!=null){if(options.nodes.parseColor===true){vNode.color=gNode.color;}else {vNode.color={background:gNode.color,border:gNode.color,highlight:{background:gNode.color,border:gNode.color},hover:{background:gNode.color,border:gNode.color}};}}return vNode;});return {nodes:vNodes,edges:vEdges};}var gephiParser=/*#__PURE__*/freeze$2({__proto__:null,parseGephi:parseGephi});var Activator=/*#__PURE__*/freeze$2({__proto__:null,'default':undefined});var commonjsGlobal$1=typeof globalThis!=='undefined'?globalThis:typeof window!=='undefined'?window:typeof global!=='undefined'?global:typeof self!=='undefined'?self:{};function unwrapExports$1(x){return x&&x.__esModule&&Object.prototype.hasOwnProperty.call(x,'default')?x['default']:x;}function createCommonjsModule$1(fn,module){return module={exports:{}},fn(module,module.exports),module.exports;}function getCjsExportFromNamespace$1(n){return n&&n['default']||n;}var hammer=createCommonjsModule$1(function(module){/**
	 * Setup a mock hammer.js object, for unit testing.
	 *
	 * Inspiration: https://github.com/uber/deck.gl/pull/658
	 *
	 * @returns {{on: noop, off: noop, destroy: noop, emit: noop, get: get}}
	 */function hammerMock(){var noop=function noop(){};return {on:noop,off:noop,destroy:noop,emit:noop,get:function get(m){//eslint-disable-line no-unused-vars
	return {set:noop};}};}if(typeof window!=='undefined'){var Hammer$1=window['Hammer']||Hammer;module.exports=Hammer$1;}else {module.exports=function(){// hammer.js is only available in a browser, not in node.js. Replacing it with a mock object.
	return hammerMock();};}});var visUtil_min=createCommonjsModule$1(function(module,exports){/**
	 * vis-util
	 * https://github.com/visjs/vis-util
	 *
	 * utilitie collection for visjs
	 *
	 * @version 4.0.0
	 * @date    2020-02-29T15:35:13.178Z
	 *
	 * @copyright (c) 2011-2017 Almende B.V, http://almende.com
	 * @copyright (c) 2017-2019 visjs contributors, https://github.com/visjs
	 *
	 * @license
	 * vis.js is dual licensed under both
	 *
	 *   1. The Apache 2.0 License
	 *      http://www.apache.org/licenses/LICENSE-2.0
	 *
	 *   and
	 *
	 *   2. The MIT License
	 *      http://opensource.org/licenses/MIT
	 *
	 * vis.js may be distributed under either license.
	 */!function(t,r){r(exports);}(commonjsGlobal$1,function(t){var _context8,_context10;var r="undefined"!=typeof globalThis?globalThis:"undefined"!=typeof window?window:"undefined"!=typeof commonjsGlobal$1?commonjsGlobal$1:"undefined"!=typeof self?self:{};function e(t,r){return t(r={exports:{}},r.exports),r.exports;}function n(t){return t&&t.default||t;}var o=function o(t){return t&&t.Math==Math&&t;},i=o("object"==(typeof globalThis==="undefined"?"undefined":_typeof_1(globalThis))&&globalThis)||o("object"==(typeof window==="undefined"?"undefined":_typeof_1(window))&&window)||o("object"==(typeof self==="undefined"?"undefined":_typeof_1(self))&&self)||o("object"==_typeof_1(r)&&r)||Function("return this")(),a=function a(t){try{return !!t();}catch(t){return !0;}},u=!a(function(){return 7!=defineProperty$2({},1,{get:function get(){return 7;}})[1];}),c={}.propertyIsEnumerable,f=getOwnPropertyDescriptor$3,l={f:f&&!c.call({1:2},1)?function(t){var r=f(this,t);return !!r&&r.enumerable;}:c},s=function s(t,r){return {enumerable:!(1&t),configurable:!(2&t),writable:!(4&t),value:r};},p={}.toString,v=function v(t){var _context7;return slice$3(_context7=p.call(t)).call(_context7,8,-1);},h="".split,d=a(function(){return !Object("z").propertyIsEnumerable(0);})?function(t){return "String"==v(t)?h.call(t,""):Object(t);}:Object,y=function y(t){if(null==t)throw TypeError("Can't call method on "+t);return t;},g=function g(t){return d(y(t));},b=function b(t){return "object"==_typeof_1(t)?null!==t:"function"==typeof t;},m=function m(t,r){if(!b(t))return t;var e,n;if(r&&"function"==typeof(e=t.toString)&&!b(n=e.call(t)))return n;if("function"==typeof(e=t.valueOf)&&!b(n=e.call(t)))return n;if(!r&&"function"==typeof(e=t.toString)&&!b(n=e.call(t)))return n;throw TypeError("Can't convert object to primitive value");},O={}.hasOwnProperty,w=function w(t,r){return O.call(t,r);},S=i.document,j=b(S)&&b(S.createElement),A=function A(t){return j?S.createElement(t):{};},x=!u&&!a(function(){return 7!=defineProperty$2(A("div"),"a",{get:function get(){return 7;}}).a;}),E=getOwnPropertyDescriptor$3,P={f:u?E:function(t,r){if(t=g(t),r=m(r,!0),x)try{return E(t,r);}catch(t){}if(w(t,r))return s(!l.f.call(t,r),t[r]);}},T=/#|\.prototype\./,k=function k(t,r){var e=C[M(t)];return e==I||e!=L&&("function"==typeof r?a(r):!!r);},M=k.normalize=function(t){return String(t).replace(T,".").toLowerCase();},C=k.data={},L=k.NATIVE="N",I=k.POLYFILL="P",N=k,D={},_=function _(t,r,e){if(function(t){if("function"!=typeof t)throw TypeError(String(t)+" is not a function");}(t),void 0===r)return t;switch(e){case 0:return function(){return t.call(r);};case 1:return function(e){return t.call(r,e);};case 2:return function(e,n){return t.call(r,e,n);};case 3:return function(e,n,o){return t.call(r,e,n,o);};}return function(){return t.apply(r,arguments);};},R=function R(t){if(!b(t))throw TypeError(String(t)+" is not an object");return t;},F=defineProperty$2,B={f:u?F:function(t,r,e){if(R(t),r=m(r,!0),R(e),x)try{return F(t,r,e);}catch(t){}if("get"in e||"set"in e)throw TypeError("Accessors not supported");return "value"in e&&(t[r]=e.value),t;}},G=u?function(t,r,e){return B.f(t,r,s(1,e));}:function(t,r,e){return t[r]=e,t;},V=P.f,H=function H(t){var r=function r(_r2,e,n){if(this instanceof t){switch(arguments.length){case 0:return new t();case 1:return new t(_r2);case 2:return new t(_r2,e);}return new t(_r2,e,n);}return t.apply(this,arguments);};return r.prototype=t.prototype,r;},Q=function Q(t,r){var e,n,o,a,u,c,f,l,s=t.target,p=t.global,v=t.stat,h=t.proto,d=p?i:v?i[s]:(i[s]||{}).prototype,y=p?D:D[s]||(D[s]={}),g=y.prototype;for(o in r){e=!N(p?o:s+(v?".":"#")+o,t.forced)&&d&&w(d,o),u=y[o],e&&(c=t.noTargetGet?(l=V(d,o))&&l.value:d[o]),a=e&&c?c:r[o],e&&_typeof_1(u)==_typeof_1(a)||(f=bind$2(t)&&e?_(a,i):t.wrap&&e?H(a):h&&"function"==typeof a?_(Function.call,a):a,(t.sham||a&&a.sham||u&&u.sham)&&G(f,"sham",!0),y[o]=f,h&&(w(D,n=s+"Prototype")||G(D,n,{}),D[n][o]=a,t.real&&g&&!g[o]&&G(g,o,a)));}},W=isArray$3||function(t){return "Array"==v(t);};Q({target:"Array",stat:!0},{isArray:W});var z=D.Array.isArray,$=z;var q=function q(t){if($(t))return t;},J={},U=i["__core-js_shared__"]||function(t,r){try{G(i,t,r);}catch(e){i[t]=r;}return r;}("__core-js_shared__",{}),Y=Function.toString;"function"!=typeof U.inspectSource&&(U.inspectSource=function(t){return Y.call(t);});var X,K,Z,tt=U.inspectSource,rt=i.WeakMap,et="function"==typeof rt&&/native code/.test(tt(rt)),nt=e(function(t){(t.exports=function(t,r){return U[t]||(U[t]=void 0!==r?r:{});})("versions",[]).push({version:"3.6.4",mode:"pure",copyright:"© 2020 Denis Pushkarev (zloirock.ru)"});}),ot=0,it=Math.random(),at=function at(t){return "Symbol("+String(void 0===t?"":t)+")_"+(++ot+it).toString(36);},ut=nt("keys"),ct=function ct(t){return ut[t]||(ut[t]=at(t));},ft={},lt=i.WeakMap;if(et){var st=new lt(),pt=st.get,vt=st.has,ht=st.set;X=function X(t,r){return ht.call(st,t,r),r;},K=function K(t){return pt.call(st,t)||{};},Z=function Z(t){return vt.call(st,t);};}else {var dt=ct("state");ft[dt]=!0,X=function X(t,r){return G(t,dt,r),r;},K=function K(t){return w(t,dt)?t[dt]:{};},Z=function Z(t){return w(t,dt);};}var yt,gt,bt,mt={set:X,get:K,has:Z,enforce:function enforce(t){return Z(t)?K(t):X(t,{});},getterFor:function getterFor(t){return function(r){var e;if(!b(r)||(e=K(r)).type!==t)throw TypeError("Incompatible receiver, "+t+" required");return e;};}},Ot=function Ot(t){return Object(y(t));},wt=!a(function(){function t(){}return t.prototype.constructor=null,getPrototypeOf$5(new t())!==t.prototype;}),St=ct("IE_PROTO"),jt=Object.prototype,At=wt?getPrototypeOf$5:function(t){return t=Ot(t),w(t,St)?t[St]:"function"==typeof t.constructor&&t instanceof t.constructor?t.constructor.prototype:t instanceof Object?jt:null;},xt=!!getOwnPropertySymbols$2&&!a(function(){return !String(symbol$4());}),Et=xt&&!symbol$4.sham&&"symbol"==_typeof_1(iterator$4),Pt=nt("wks"),Tt=i.Symbol,kt=Et?Tt:Tt&&Tt.withoutSetter||at,Mt=function Mt(t){return w(Pt,t)||(xt&&w(Tt,t)?Pt[t]=Tt[t]:Pt[t]=kt("Symbol."+t)),Pt[t];},Ct=(Mt("iterator"),!1);keys$3([])&&("next"in(bt=keys$3(_context8=[]).call(_context8))?(gt=At(At(bt)))!==Object.prototype&&(yt=gt):Ct=!0),null==yt&&(yt={});var Lt,It={IteratorPrototype:yt,BUGGY_SAFARI_ITERATORS:Ct},Nt=Math.ceil,Dt=Math.floor,_t=function _t(t){return isNaN(t=+t)?0:(t>0?Dt:Nt)(t);},Rt=Math.min,Ft=function Ft(t){return t>0?Rt(_t(t),9007199254740991):0;},Bt=Math.max,Gt=Math.min,Vt=function Vt(t,r){var e=_t(t);return e<0?Bt(e+r,0):Gt(e,r);},Ht=function Ht(t){return function(r,e,n){var o,i=g(r),a=Ft(i.length),u=Vt(n,a);if(t&&e!=e){for(;a>u;){if((o=i[u++])!=o)return !0;}}else for(;a>u;u++){if((t||u in i)&&i[u]===e)return t||u||0;}return !t&&-1;};},Qt={includes:Ht(!0),indexOf:Ht(!1)},Wt=indexOf$3(Qt),zt=function zt(t,r){var e,n=g(t),o=0,i=[];for(e in n){!w(ft,e)&&w(n,e)&&i.push(e);}for(;r.length>o;){w(n,e=r[o++])&&(~Wt(i,e)||i.push(e));}return i;},$t=["constructor","hasOwnProperty","isPrototypeOf","propertyIsEnumerable","toLocaleString","toString","valueOf"],qt=keys$6||function(t){return zt(t,$t);},Jt=u?defineProperties$1:function(t,r){R(t);for(var e,n=qt(r),o=n.length,i=0;o>i;){B.f(t,e=n[i++],r[e]);}return t;},Ut=function Ut(t){return "function"==typeof t?t:void 0;},Yt=function Yt(t,r){return arguments.length<2?Ut(D[t])||Ut(i[t]):D[t]&&D[t][r]||i[t]&&i[t][r];},Xt=Yt("document","documentElement"),Kt=ct("IE_PROTO"),Zt=function Zt(){},tr=function tr(t){return "<script>"+t+"<\/script>";},_rr=function rr(){try{Lt=document.domain&&new ActiveXObject("htmlfile");}catch(t){}var t,r;_rr=Lt?function(t){t.write(tr("")),t.close();var r=t.parentWindow.Object;return t=null,r;}(Lt):((r=A("iframe")).style.display="none",Xt.appendChild(r),r.src=String("javascript:"),(t=r.contentWindow.document).open(),t.write(tr("document.F=Object")),t.close(),t.F);for(var e=$t.length;e--;){delete _rr.prototype[$t[e]];}return _rr();};ft[Kt]=!0;var er=create$4||function(t,r){var e;return null!==t?(Zt.prototype=R(t),e=new Zt(),Zt.prototype=null,e[Kt]=t):e=_rr(),void 0===r?e:Jt(e,r);},nr={};nr[Mt("toStringTag")]="z";var or="[object z]"===String(nr),ir=Mt("toStringTag"),ar="Arguments"==v(function(){return arguments;}()),ur=or?v:function(t){var r,e,n;return void 0===t?"Undefined":null===t?"Null":"string"==typeof(e=function(t,r){try{return t[r];}catch(t){}}(r=Object(t),ir))?e:ar?v(r):"Object"==(n=v(r))&&"function"==typeof r.callee?"Arguments":n;},cr=or?{}.toString:function(){return "[object "+ur(this)+"]";},fr=B.f,lr=Mt("toStringTag"),sr=function sr(t,r,e,n){if(t){var o=e?t:t.prototype;w(o,lr)||fr(o,lr,{configurable:!0,value:r}),n&&!or&&G(o,"toString",cr);}},pr=It.IteratorPrototype,vr=function vr(){return this;},hr=(setPrototypeOf$5||"__proto__"in{}&&function(){var t,r=!1,e={};try{(t=getOwnPropertyDescriptor$3(Object.prototype,"__proto__").set).call(e,[]),r=e instanceof Array;}catch(t){}}(),function(t,r,e,n){n&&n.enumerable?t[r]=e:G(t,r,e);}),dr=It.IteratorPrototype,yr=It.BUGGY_SAFARI_ITERATORS,gr=Mt("iterator"),br=function br(){return this;},mr=function mr(t,r,e,n,o,i,a){!function(t,r,e){var n=r+" Iterator";t.prototype=er(pr,{next:s(1,e)}),sr(t,n,!1,!0),J[n]=vr;}(e,r,n);var u,c,f,l=function l(t){if(t===o&&y)return y;if(!yr&&t in h)return h[t];switch(t){case"keys":case"values":case"entries":return function(){return new e(this,t);};}return function(){return new e(this);};},p=r+" Iterator",v=!1,h=t.prototype,d=h[gr]||h["@@iterator"]||o&&h[o],y=!yr&&d||l(o),g="Array"==r&&entries$2(h)||d;if(g&&(u=At(g.call(new t())),dr!==Object.prototype&&u.next&&(sr(u,p,!0,!0),J[p]=br)),"values"==o&&d&&"values"!==d.name&&(v=!0,y=function y(){return d.call(this);}),a&&h[gr]!==y&&G(h,gr,y),J[r]=y,o)if(c={values:l("values"),keys:i?y:l("keys"),entries:l("entries")},a)for(f in c){!yr&&!v&&f in h||hr(h,f,c[f]);}else Q({target:r,proto:!0,forced:yr||v},c);return c;},Or=mt.set,wr=mt.getterFor("Array Iterator");mr(Array,"Array",function(t,r){Or(this,{type:"Array Iterator",target:g(t),index:0,kind:r});},function(){var t=wr(this),r=t.target,e=t.kind,n=t.index++;return !r||n>=r.length?(t.target=void 0,{value:void 0,done:!0}):"keys"==e?{value:n,done:!1}:"values"==e?{value:r[n],done:!1}:{value:[n,r[n]],done:!1};},"values");J.Arguments=J.Array;var Sr=Mt("toStringTag");for(var jr in {CSSRuleList:0,CSSStyleDeclaration:0,CSSValueList:0,ClientRectList:0,DOMRectList:0,DOMStringList:0,DOMTokenList:1,DataTransferItemList:0,FileList:0,HTMLAllCollection:0,HTMLCollection:0,HTMLFormElement:0,HTMLSelectElement:0,MediaList:0,MimeTypeArray:0,NamedNodeMap:0,NodeList:1,PaintRequestList:0,Plugin:0,PluginArray:0,SVGLengthList:0,SVGNumberList:0,SVGPathSegList:0,SVGPointList:0,SVGStringList:0,SVGTransformList:0,SourceBufferList:0,StyleSheetList:0,TextTrackCueList:0,TextTrackList:0,TouchList:0}){var Ar=i[jr],xr=Ar&&Ar.prototype;xr&&ur(xr)!==Sr&&G(xr,Sr,jr),J[jr]=J.Array;}var Er=function Er(t){return function(r,e){var n,o,i=String(y(r)),a=_t(e),u=i.length;return a<0||a>=u?t?"":void 0:(n=i.charCodeAt(a))<55296||n>56319||a+1===u||(o=i.charCodeAt(a+1))<56320||o>57343?t?i.charAt(a):n:t?slice$3(i).call(i,a,a+2):o-56320+(n-55296<<10)+65536;};},Pr={codeAt:Er(!1),charAt:Er(!0)}.charAt,Tr=mt.set,kr=mt.getterFor("String Iterator");mr(String,"String",function(t){Tr(this,{type:"String Iterator",string:String(t),index:0});},function(){var t,r=kr(this),e=r.string,n=r.index;return n>=e.length?{value:void 0,done:!0}:(t=Pr(e,n),r.index+=t.length,{value:t,done:!1});});var Mr=Mt("iterator"),Cr=function Cr(t){if(null!=t)return t[Mr]||t["@@iterator"]||J[ur(t)];},Lr=function Lr(t){var r=Cr(t);if("function"!=typeof r)throw TypeError(String(t)+" is not iterable");return R(r.call(t));},Ir=Mt("iterator"),Nr=function Nr(t){var r=Object(t);return void 0!==r[Ir]||"@@iterator"in r||J.hasOwnProperty(ur(r));};var Dr=function Dr(t,r){if(Nr(Object(t))||"[object Arguments]"===Object.prototype.toString.call(t)){var e=[],n=!0,o=!1,i=void 0;try{for(var a,u=Lr(t);!(n=(a=u.next()).done)&&(e.push(a.value),!r||e.length!==r);n=!0){;}}catch(t){o=!0,i=t;}finally{try{n||null==u.return||u.return();}finally{if(o)throw i;}}return e;}};var _r=function _r(){throw new TypeError("Invalid attempt to destructure non-iterable instance");};var Rr=function Rr(t,r){return q(t)||Dr(t,r)||_r();};Q({target:"Date",stat:!0},{now:function now(){return new Date().getTime();}});var Fr=D.Date.now;function Br(t){var r=function(){for(var t=Gr(),r=t(" "),e=t(" "),n=t(" "),o=0;o<arguments.length;o++){(r-=t(o<0||arguments.length<=o?void 0:arguments[o]))<0&&(r+=1),(e-=t(o<0||arguments.length<=o?void 0:arguments[o]))<0&&(e+=1),(n-=t(o<0||arguments.length<=o?void 0:arguments[o]))<0&&(n+=1);}return [r,e,n];}(t),e=Rr(r,3),n=e[0],o=e[1],i=e[2],a=1,u=function u(){var t=2091639*n+2.3283064365386963e-10*a;return n=o,o=i,i=t-(a=0|t);};return u.uint32=function(){return 4294967296*u();},u.fract53=function(){return u()+11102230246251565e-32*(2097152*u()|0);},u.algorithm="Alea",u.seed=t,u.version="0.9",u;}function Gr(){var t=4022871197;return function(r){for(var e=r.toString(),n=0;n<e.length;n++){var o=.02519603282416938*(t+=e.charCodeAt(n));o-=t=o>>>0,t=(o*=t)>>>0,t+=4294967296*(o-=t);}return 2.3283064365386963e-10*(t>>>0);};}Q({target:"Object",stat:!0,forced:!u,sham:!u},{defineProperty:B.f});var Vr=e(function(t){var r=D.Object,e=t.exports=function(t,e,n){return r.defineProperty(t,e,n);};r.defineProperty.sham&&(e.sham=!0);}),Hr=Vr;Q({target:"Object",stat:!0,forced:!u,sham:!u},{defineProperties:Jt});var Qr=e(function(t){var r=D.Object,e=t.exports=function(t,e){return r.defineProperties(t,e);};r.defineProperties.sham&&(e.sham=!0);}),Wr=concat$2($t).call($t,"length","prototype"),zr={f:getOwnPropertyNames$2||function(t){return zt(t,Wr);}},$r={f:getOwnPropertySymbols$2},qr=Yt("Reflect","ownKeys")||function(t){var r=zr.f(R(t)),e=$r.f;return e?concat$2(r).call(r,e(t)):r;},Jr=function Jr(t,r,e){var n=m(r);n in t?B.f(t,n,s(0,e)):t[n]=e;};Q({target:"Object",stat:!0,sham:!u},{getOwnPropertyDescriptors:function getOwnPropertyDescriptors(t){for(var r,e,n=g(t),o=P.f,i=qr(n),a={},u=0;i.length>u;){void 0!==(e=o(n,r=i[u++]))&&Jr(a,r,e);}return a;}});var Ur=D.Object.getOwnPropertyDescriptors,Yr=P.f,Xr=a(function(){Yr(1);});Q({target:"Object",stat:!0,forced:!u||Xr,sham:!u},{getOwnPropertyDescriptor:function getOwnPropertyDescriptor(t,r){return Yr(g(t),r);}});var Kr=e(function(t){var r=D.Object,e=t.exports=function(t,e){return r.getOwnPropertyDescriptor(t,e);};r.getOwnPropertyDescriptor.sham&&(e.sham=!0);}),Zr=zr.f,te={}.toString,re="object"==(typeof window==="undefined"?"undefined":_typeof_1(window))&&window&&getOwnPropertyNames$2?getOwnPropertyNames$2(window):[],ee={f:function f(t){return re&&"[object Window]"==te.call(t)?function(t){try{return Zr(t);}catch(t){return slice$3(re).call(re);}}(t):Zr(g(t));}},ne={f:Mt},oe=B.f,ie=function ie(t){var r=D.Symbol||(D.Symbol={});w(r,t)||oe(r,t,{value:ne.f(t)});},ae=Mt("species"),ue=function ue(t,r){var e;return W(t)&&("function"!=typeof(e=t.constructor)||e!==Array&&!W(e.prototype)?b(e)&&null===(e=e[ae])&&(e=void 0):e=void 0),new(void 0===e?Array:e)(0===r?0:r);},ce=[].push,fe=function fe(t){var r=1==t,e=2==t,n=3==t,o=4==t,i=6==t,a=5==t||i;return function(u,c,f,l){for(var s,p,v=Ot(u),h=d(v),y=_(c,f,3),g=Ft(h.length),b=0,m=l||ue,O=r?m(u,g):e?m(u,0):void 0;g>b;b++){if((a||b in h)&&(p=y(s=h[b],b,v),t))if(r)O[b]=p;else if(p)switch(t){case 3:return !0;case 5:return s;case 6:return b;case 2:ce.call(O,s);}else if(o)return !1;}return i?-1:n||o?o:O;};},le={forEach:fe(0),map:fe(1),filter:fe(2),some:fe(3),every:fe(4),find:fe(5),findIndex:fe(6)},se=forEach$3(le),pe=ct("hidden"),ve=Mt("toPrimitive"),he=mt.set,de=mt.getterFor("Symbol"),ye=Object.prototype,_ge=i.Symbol,be=Yt("JSON","stringify"),me=P.f,Oe=B.f,we=ee.f,Se=l.f,je=nt("symbols"),Ae=nt("op-symbols"),xe=nt("string-to-symbol-registry"),Ee=nt("symbol-to-string-registry"),Pe=nt("wks"),Te=i.QObject,ke=!Te||!Te.prototype||!Te.prototype.findChild,Me=u&&a(function(){return 7!=er(Oe({},"a",{get:function get(){return Oe(this,"a",{value:7}).a;}})).a;})?function(t,r,e){var n=me(ye,r);n&&delete ye[r],Oe(t,r,e),n&&t!==ye&&Oe(ye,r,n);}:Oe,Ce=function Ce(t,r){var e=je[t]=er(_ge.prototype);return he(e,{type:"Symbol",tag:t,description:r}),u||(e.description=r),e;},Le=Et?function(t){return "symbol"==_typeof_1(t);}:function(t){return Object(t)instanceof _ge;},Ie=function Ie(t,r,e){t===ye&&Ie(Ae,r,e),R(t);var n=m(r,!0);return R(e),w(je,n)?(e.enumerable?(w(t,pe)&&t[pe][n]&&(t[pe][n]=!1),e=er(e,{enumerable:s(0,!1)})):(w(t,pe)||Oe(t,pe,s(1,{})),t[pe][n]=!0),Me(t,n,e)):Oe(t,n,e);},Ne=function Ne(t,r){var _context9;R(t);var e=g(r),n=concat$2(_context9=qt(e)).call(_context9,Fe(e));return se(n,function(r){u&&!De.call(e,r)||Ie(t,r,e[r]);}),t;},De=function De(t){var r=m(t,!0),e=Se.call(this,r);return !(this===ye&&w(je,r)&&!w(Ae,r))&&(!(e||!w(this,r)||!w(je,r)||w(this,pe)&&this[pe][r])||e);},_e=function _e(t,r){var e=g(t),n=m(r,!0);if(e!==ye||!w(je,n)||w(Ae,n)){var o=me(e,n);return !o||!w(je,n)||w(e,pe)&&e[pe][n]||(o.enumerable=!0),o;}},Re=function Re(t){var r=we(g(t)),e=[];return se(r,function(t){w(je,t)||w(ft,t)||e.push(t);}),e;},Fe=function Fe(t){var r=t===ye,e=we(r?Ae:g(t)),n=[];return se(e,function(t){!w(je,t)||r&&!w(ye,t)||n.push(je[t]);}),n;};if(xt||(hr((_ge=function ge(){if(this instanceof _ge)throw TypeError("Symbol is not a constructor");var t=arguments.length&&void 0!==arguments[0]?String(arguments[0]):void 0,r=at(t),e=function e(t){this===ye&&e.call(Ae,t),w(this,pe)&&w(this[pe],r)&&(this[pe][r]=!1),Me(this,r,s(1,t));};return u&&ke&&Me(ye,r,{configurable:!0,set:e}),Ce(r,t);}).prototype,"toString",function(){return de(this).tag;}),hr(_ge,"withoutSetter",function(t){return Ce(at(t),t);}),l.f=De,B.f=Ie,P.f=_e,zr.f=ee.f=Re,$r.f=Fe,ne.f=function(t){return Ce(Mt(t),t);},u&&Oe(_ge.prototype,"description",{configurable:!0,get:function get(){return de(this).description;}})),Q({global:!0,wrap:!0,forced:!xt,sham:!xt},{Symbol:_ge}),se(qt(Pe),function(t){ie(t);}),Q({target:"Symbol",stat:!0,forced:!xt},{for:function _for(t){var r=String(t);if(w(xe,r))return xe[r];var e=_ge(r);return xe[r]=e,Ee[e]=r,e;},keyFor:function keyFor(t){if(!Le(t))throw TypeError(t+" is not a symbol");if(w(Ee,t))return Ee[t];},useSetter:function useSetter(){ke=!0;},useSimple:function useSimple(){ke=!1;}}),Q({target:"Object",stat:!0,forced:!xt,sham:!u},{create:function create(t,r){return void 0===r?er(t):Ne(er(t),r);},defineProperty:Ie,defineProperties:Ne,getOwnPropertyDescriptor:_e}),Q({target:"Object",stat:!0,forced:!xt},{getOwnPropertyNames:Re,getOwnPropertySymbols:Fe}),Q({target:"Object",stat:!0,forced:a(function(){$r.f(1);})},{getOwnPropertySymbols:function getOwnPropertySymbols(t){return $r.f(Ot(t));}}),be){var Be=!xt||a(function(){var t=_ge();return "[null]"!=be([t])||"{}"!=be({a:t})||"{}"!=be(Object(t));});Q({target:"JSON",stat:!0,forced:Be},{stringify:function stringify(t,r,e){for(var n,o=[t],i=1;arguments.length>i;){o.push(arguments[i++]);}if(n=r,(b(r)||void 0!==t)&&!Le(t))return W(r)||(r=function r(t,_r3){if("function"==typeof n&&(_r3=n.call(this,t,_r3)),!Le(_r3))return _r3;}),o[1]=r,be.apply(null,o);}});}_ge.prototype[ve]||G(_ge.prototype,ve,_ge.prototype.valueOf),sr(_ge,"Symbol"),ft[pe]=!0;var Ge=D.Object.getOwnPropertySymbols;Q({target:"Object",stat:!0,sham:!u},{create:er});var Ve=D.Object,He=function He(t,r){return Ve.create(t,r);},Qe=Vr;var We=function We(t,r,e){return r in t?Qe(t,r,{value:e,enumerable:!0,configurable:!0,writable:!0}):t[r]=e,t;},ze=a(function(){qt(1);});Q({target:"Object",stat:!0,forced:ze},{keys:function keys(t){return qt(Ot(t));}});var $e,qe=keys$3(D.Object),Je="\t\n\x0B\f\r \xA0\u1680\u2000\u2001\u2002\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200A\u202F\u205F\u3000\u2028\u2029\uFEFF",Ue="["+Je+"]",Ye=RegExp("^"+Ue+Ue+"*"),Xe=RegExp(Ue+Ue+"*$"),Ke=function Ke(t){return function(r){var e=String(y(r));return 1&t&&(e=e.replace(Ye,"")),2&t&&(e=e.replace(Xe,"")),e;};},Ze={start:Ke(1),end:Ke(2),trim:Ke(3)},tn=trim$2(Ze);Q({target:"String",proto:!0,forced:($e="trim",a(function(){return !!Je[$e]()||"​᠎"!="​᠎"[$e]()||Je[$e].name!==$e;}))},{trim:function trim(){return tn(this);}});var rn=function rn(t){return D[t+"Prototype"];},en=trim$2(rn("String")),nn=String.prototype,on=function on(t){var r=trim$2(t);return "string"==typeof t||t===nn||t instanceof String&&r===trim$2(nn)?en:r;},an=function an(t,r){var e=[][t];return !!e&&a(function(){e.call(null,r||function(){throw 1;},1);});},un=defineProperty$2,cn={},fn=function fn(t){throw t;},ln=function ln(t,r){if(w(cn,t))return cn[t];r||(r={});var e=[][t],n=!!w(r,"ACCESSORS")&&r.ACCESSORS,o=w(r,0)?r[0]:fn,i=w(r,1)?r[1]:void 0;return cn[t]=!!e&&!a(function(){if(n&&!u)return !0;var t={length:-1};n?un(t,1,{enumerable:!0,get:fn}):t[1]=1,e.call(t,o,i);});},sn=forEach$3(le),pn=an("forEach"),vn=ln("forEach"),hn=pn&&vn?forEach$3([]):function(t){return sn(this,t,arguments.length>1?arguments[1]:void 0);};Q({target:"Array",proto:!0,forced:forEach$3([])!=hn},{forEach:hn});var dn,yn,gn=forEach$3(rn("Array")),bn=Array.prototype,mn={DOMTokenList:!0,NodeList:!0},On=function On(t){var r=forEach$3(t);return t===bn||t instanceof Array&&r===forEach$3(bn)||mn.hasOwnProperty(ur(t))?gn:r;},wn=Yt("navigator","userAgent")||"",Sn=i.process,jn=Sn&&Sn.versions,An=jn&&jn.v8;An?yn=(dn=An.split("."))[0]+dn[1]:wn&&(!(dn=wn.match(/Edge\/(\d+)/))||dn[1]>=74)&&(dn=wn.match(/Chrome\/(\d+)/))&&(yn=dn[1]);var xn=yn&&+yn,En=Mt("species"),Pn=function Pn(t){return xn>=51||!a(function(){var r=[];return (r.constructor={})[En]=function(){return {foo:1};},1!==r[t](Boolean).foo;});},Tn=map$5(le),kn=Pn("map"),Mn=ln("map");Q({target:"Array",proto:!0,forced:!kn||!Mn},{map:function map(t){return Tn(this,t,arguments.length>1?arguments[1]:void 0);}});var Cn=map$5(rn("Array")),Ln=Array.prototype,In=function In(t){var r=map$5(t);return t===Ln||t instanceof Array&&r===map$5(Ln)?Cn:r;},Nn=trim$2(Ze),Dn=i.parseInt,_n=/^[+-]?0[Xx]/,Rn=8!==Dn(Je+"08")||22!==Dn(Je+"0x16")?function(t,r){var e=Nn(String(t));return Dn(e,r>>>0||(_n.test(e)?16:10));}:Dn;Q({global:!0,forced:_parseInt$2!=Rn},{parseInt:Rn});var Fn=D.parseInt,Bn=l.f,Gn=function Gn(t){return function(r){for(var e,n=g(r),o=qt(n),i=o.length,a=0,c=[];i>a;){e=o[a++],u&&!Bn.call(n,e)||c.push(t?[e,n[e]]:n[e]);}return c;};},Vn=values$2({entries:Gn(!0),values:Gn(!1)});Q({target:"Object",stat:!0},{values:function values(t){return Vn(t);}});var Hn=values$2(D.Object),Qn=filter$2(le),Wn=Pn("filter"),zn=ln("filter");Q({target:"Array",proto:!0,forced:!Wn||!zn},{filter:function filter(t){return Qn(this,t,arguments.length>1?arguments[1]:void 0);}});var $n=filter$2(rn("Array")),qn=Array.prototype,Jn=function Jn(t){var r=filter$2(t);return t===qn||t instanceof Array&&r===filter$2(qn)?$n:r;},Un=Mt("isConcatSpreadable"),Yn=xn>=51||!a(function(){var t=[];return t[Un]=!1,concat$2(t).call(t)[0]!==t;}),Xn=Pn("concat"),Kn=function Kn(t){if(!b(t))return !1;var r=t[Un];return void 0!==r?!!r:W(t);};Q({target:"Array",proto:!0,forced:!Yn||!Xn},{concat:function concat(t){var r,e,n,o,i,a=Ot(this),u=ue(a,0),c=0;for(r=-1,n=arguments.length;r<n;r++){if(i=-1===r?a:arguments[r],Kn(i)){if(c+(o=Ft(i.length))>9007199254740991)throw TypeError("Maximum allowed index exceeded");for(e=0;e<o;e++,c++){e in i&&Jr(u,c,i[e]);}}else {if(c>=9007199254740991)throw TypeError("Maximum allowed index exceeded");Jr(u,c++,i);}}return u.length=c,u;}});var Zn=concat$2(rn("Array")),to=Array.prototype,ro=function ro(t){var r=concat$2(t);return t===to||t instanceof Array&&r===concat$2(to)?Zn:r;};var eo=function eo(t){if($(t)){for(var r=0,e=new Array(t.length);r<t.length;r++){e[r]=t[r];}return e;}},no=function no(t,r,e,n){try{return n?r(R(e)[0],e[1]):r(e);}catch(r){var o=t.return;throw void 0!==o&&R(o.call(t)),r;}},oo=Mt("iterator"),io=Array.prototype,ao=function ao(t){return void 0!==t&&(J.Array===t||io[oo]===t);},uo=Mt("iterator"),co=!1;try{var fo=0,lo={next:function next(){return {done:!!fo++};},return:function _return(){co=!0;}};lo[uo]=function(){return this;},from_1$2(lo,function(){throw 2;});}catch(t){}var so=!function(t,r){if(!r&&!co)return !1;var e=!1;try{var n={};n[uo]=function(){return {next:function next(){return {done:e=!0};}};},t(n);}catch(t){}return e;}(function(t){from_1$2(t);});Q({target:"Array",stat:!0,forced:so},{from:function from(t){var r,e,n,o,i,a,u=Ot(t),c="function"==typeof this?this:Array,f=arguments.length,l=f>1?arguments[1]:void 0,s=void 0!==l,p=Cr(u),v=0;if(s&&(l=_(l,f>2?arguments[2]:void 0,2)),null==p||c==Array&&ao(p))for(e=new c(r=Ft(u.length));r>v;v++){a=s?l(u[v],v):u[v],Jr(e,v,a);}else for(i=(o=p.call(u)).next,e=new c();!(n=i.call(o)).done;v++){a=s?no(o,l,[n.value,v],!0):n.value,Jr(e,v,a);}return e.length=v,e;}});var po=D.Array.from;var vo=function vo(t){if(Nr(Object(t))||"[object Arguments]"===Object.prototype.toString.call(t))return po(t);};var ho=function ho(){throw new TypeError("Invalid attempt to spread non-iterable instance");};var yo=function yo(t){return eo(t)||vo(t)||ho();},go=Pn("slice"),bo=ln("slice",{ACCESSORS:!0,0:0,1:2}),mo=Mt("species"),Oo=slice$3([]),wo=Math.max;Q({target:"Array",proto:!0,forced:!go||!bo},{slice:function slice(t,r){var e,n,o,i=g(this),a=Ft(i.length),u=Vt(t,a),c=Vt(void 0===r?a:r,a);if(W(i)&&("function"!=typeof(e=i.constructor)||e!==Array&&!W(e.prototype)?b(e)&&null===(e=e[mo])&&(e=void 0):e=void 0,e===Array||void 0===e))return Oo.call(i,u,c);for(n=new(void 0===e?Array:e)(wo(c-u,0)),o=0;u<c;u++,o++){u in i&&Jr(n,o,i[u]);}return n.length=o,n;}});var So=slice$3(rn("Array")),jo=Array.prototype,Ao=function Ao(t){var r=slice$3(t);return t===jo||t instanceof Array&&r===slice$3(jo)?So:r;},xo=a(function(){At(1);});Q({target:"Object",stat:!0,forced:xo,sham:!wt},{getPrototypeOf:function getPrototypeOf(t){return At(Ot(t));}});var Eo=D.Object.getPrototypeOf,Po=indexOf$3(Qt),To=indexOf$3([]),ko=!!To&&1/indexOf$3(_context10=[1]).call(_context10,1,-0)<0,Mo=an("indexOf"),Co=ln("indexOf",{ACCESSORS:!0,1:0});Q({target:"Array",proto:!0,forced:ko||!Mo||!Co},{indexOf:function indexOf(t){return ko?To.apply(this,arguments)||0:Po(this,t,arguments.length>1?arguments[1]:void 0);}});var Lo=indexOf$3(rn("Array")),Io=Array.prototype,No=function No(t){var r=indexOf$3(t);return t===Io||t instanceof Array&&r===indexOf$3(Io)?Lo:r;},Do=z,_o=assign$2,Ro=defineProperty$2,Fo=!_o||a(function(){var _context11;if(u&&1!==_o({b:1},_o(Ro({},"a",{enumerable:!0,get:function get(){Ro(this,"b",{value:3,enumerable:!1});}}),{b:2})).b)return !0;var t={},r={},e=symbol$4();return t[e]=7,forEach$3(_context11="abcdefghijklmnopqrst".split("")).call(_context11,function(t){r[t]=t;}),7!=_o({},t)[e]||"abcdefghijklmnopqrst"!=qt(_o({},r)).join("");})?function(t,r){for(var e=Ot(t),n=arguments.length,o=1,i=$r.f,a=l.f;n>o;){for(var c,f=d(arguments[o++]),s=i?concat$2(_context12=qt(f)).call(_context12,i(f)):qt(f),p=s.length,v=0;p>v;){var _context12;c=s[v++],u&&!a.call(f,c)||(e[c]=f[c]);}}return e;}:_o;Q({target:"Object",stat:!0,forced:assign$2!==Fo},{assign:Fo});var Bo=D.Object.assign;ie("iterator");var Go=ne.f("iterator"),Vo=freeze$2({__proto__:null});ie("asyncIterator");var Ho=freeze$2({__proto__:null});ie("hasInstance"),ie("isConcatSpreadable"),ie("match"),ie("matchAll"),ie("replace"),ie("search"),ie("species"),ie("split"),ie("toPrimitive"),ie("toStringTag"),ie("unscopables"),sr(Math,"Math",!0),sr(i.JSON,"JSON",!0),n(Vo),n(Ho);var Qo=D.Symbol;ie("asyncDispose"),ie("dispose"),ie("observable"),ie("patternMatch"),ie("replaceAll");var Wo=Qo,zo=e(function(t){function r(e){return t.exports=r="function"==typeof Wo&&"symbol"==_typeof_1(Go)?function(t){return _typeof_1(t);}:function(t){return t&&"function"==typeof Wo&&t.constructor===Wo&&t!==Wo.prototype?"symbol":_typeof_1(t);},r(e);}t.exports=r;});function $o(t,r){var e=qe(t);if(Ge){var n=Ge(t);r&&(n=Jn(n).call(n,function(r){return Kr(t,r).enumerable;})),e.push.apply(e,n);}return e;}var qo=/^\/?Date\((-?\d+)/i,Jo=/^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i,Uo=/^#?([a-f\d])([a-f\d])([a-f\d])$/i,Yo=/^rgb\( *(1?\d{1,2}|2[0-4]\d|25[0-5]) *, *(1?\d{1,2}|2[0-4]\d|25[0-5]) *, *(1?\d{1,2}|2[0-4]\d|25[0-5]) *\)$/i,Xo=/^rgba\( *(1?\d{1,2}|2[0-4]\d|25[0-5]) *, *(1?\d{1,2}|2[0-4]\d|25[0-5]) *, *(1?\d{1,2}|2[0-4]\d|25[0-5]) *, *([01]|0?\.\d+) *\)$/i;function Ko(t){return t instanceof Number||"number"==typeof t;}function Zo(t){return t instanceof String||"string"==typeof t;}function ti(t){return "object"===zo(t)&&null!==t;}function ri(t,r,e,n){var o=!1;!0===n&&(o=null===r[e]&&void 0!==t[e]),o?delete t[e]:t[e]=r[e];}var ei=Bo;function ni(t,r){var e=arguments.length>2&&void 0!==arguments[2]&&arguments[2],n=arguments.length>3&&void 0!==arguments[3]&&arguments[3];for(var o in r){if(Object.prototype.hasOwnProperty.call(r,o)||!0===e)if("object"===zo(r[o])&&null!==r[o]&&Eo(r[o])===Object.prototype)void 0===t[o]?t[o]=ni({},r[o],e):"object"===zo(t[o])&&null!==t[o]&&Eo(t[o])===Object.prototype?ni(t[o],r[o],e):ri(t,r,o,n);else if(Do(r[o])){var i;t[o]=Ao(i=r[o]).call(i);}else ri(t,r,o,n);}return t;}var oi=Hn;var ii={asBoolean:function asBoolean(t,r){return "function"==typeof t&&(t=t()),null!=t?0!=t:r||null;},asNumber:function asNumber(t,r){return "function"==typeof t&&(t=t()),null!=t?Number(t)||r||null:r||null;},asString:function asString(t,r){return "function"==typeof t&&(t=t()),null!=t?String(t):r||null;},asSize:function asSize(t,r){return "function"==typeof t&&(t=t()),Zo(t)?t:Ko(t)?t+"px":r||null;},asElement:function asElement(t,r){return "function"==typeof t&&(t=t()),t||r||null;}};function ai(t){var r;switch(t.length){case 3:case 4:return (r=Uo.exec(t))?{r:Fn(r[1]+r[1],16),g:Fn(r[2]+r[2],16),b:Fn(r[3]+r[3],16)}:null;case 6:case 7:return (r=Jo.exec(t))?{r:Fn(r[1],16),g:Fn(r[2],16),b:Fn(r[3],16)}:null;default:return null;}}function ui(t,r,e){var n;return "#"+Ao(n=((1<<24)+(t<<16)+(r<<8)+e).toString(16)).call(n,1);}function ci(t,r,e){t/=255,r/=255,e/=255;var n=Math.min(t,Math.min(r,e)),o=Math.max(t,Math.max(r,e));return n===o?{h:0,s:0,v:n}:{h:60*((t===n?3:e===n?1:5)-(t===n?r-e:e===n?t-r:e-t)/(o-n))/360,s:(o-n)/o,v:o};}var fi=function fi(t){var r,e={};return On(r=t.split(";")).call(r,function(t){if(""!=on(t).call(t)){var r,n,o=t.split(":"),i=on(r=o[0]).call(r),a=on(n=o[1]).call(n);e[i]=a;}}),e;},li=function li(t){var r;return In(r=qe(t)).call(r,function(r){return r+": "+t[r];}).join("; ");};function si(t,r,e){var n,o,i,a=Math.floor(6*t),u=6*t-a,c=e*(1-r),f=e*(1-u*r),l=e*(1-(1-u)*r);switch(a%6){case 0:n=e,o=l,i=c;break;case 1:n=f,o=e,i=c;break;case 2:n=c,o=e,i=l;break;case 3:n=c,o=f,i=e;break;case 4:n=l,o=c,i=e;break;case 5:n=e,o=c,i=f;}return {r:Math.floor(255*n),g:Math.floor(255*o),b:Math.floor(255*i)};}function pi(t,r,e){var n=si(t,r,e);return ui(n.r,n.g,n.b);}function vi(t){var r=ai(t);if(!r)throw new TypeError("'".concat(t,"' is not a valid color."));return ci(r.r,r.g,r.b);}function hi(t){return /(^#[0-9A-F]{6}$)|(^#[0-9A-F]{3}$)/i.test(t);}function di(t){return Yo.test(t);}function yi(t){if(null===t||"object"!==zo(t))return null;if(t instanceof Element)return t;var r=He(t);for(var e in t){Object.prototype.hasOwnProperty.call(t,e)&&"object"==zo(t[e])&&(r[e]=yi(t[e]));}return r;}t.Alea=function(){for(var t=arguments.length,r=new Array(t),e=0;e<t;e++){r[e]=arguments[e];}return Br(r.length?r:[Fr()]);},t.HSVToHex=pi,t.HSVToRGB=si,t.RGBToHSV=ci,t.RGBToHex=ui,t.addClassName=function(t,r){var e=t.className.split(" "),n=r.split(" ");e=ro(e).call(e,Jn(n).call(n,function(t){return No(e).call(e,t)<0;})),t.className=e.join(" ");},t.addCssText=function(t,r){var e=function(t){for(var r=1;r<arguments.length;r++){var e,n=null!=arguments[r]?arguments[r]:{};if(r%2)On(e=$o(Object(n),!0)).call(e,function(r){We(t,r,n[r]);});else if(Ur)Qr(t,Ur(n));else {var o;On(o=$o(Object(n))).call(o,function(r){Hr(t,r,Kr(n,r));});}}return t;}({},fi(t.style.cssText),{},fi(r));t.style.cssText=li(e);},t.addEventListener=function(t,r,e,n){var o;t.addEventListener?(void 0===n&&(n=!1),"mousewheel"===r&&No(o=navigator.userAgent).call(o,"Firefox")>=0&&(r="DOMMouseScroll"),t.addEventListener(r,e,n)):t.attachEvent("on"+r,e);},t.binarySearchCustom=function(t,r,e,n){for(var o=0,i=0,a=t.length-1;i<=a&&o<1e4;){var u=Math.floor((i+a)/2),c=t[u],f=r(void 0===n?c[e]:c[e][n]);if(0==f)return u;-1==f?i=u+1:a=u-1,o++;}return -1;},t.binarySearchValue=function(t,r,e,n,o){var i,a,u,c,f=0,l=0,s=t.length-1;for(o=null!=o?o:function(t,r){return t==r?0:t<r?-1:1;};l<=s&&f<1e4;){if(c=Math.floor(.5*(s+l)),i=t[Math.max(0,c-1)][e],a=t[c][e],u=t[Math.min(t.length-1,c+1)][e],0==o(a,r))return c;if(o(i,r)<0&&o(a,r)>0)return "before"==n?Math.max(0,c-1):c;if(o(a,r)<0&&o(u,r)>0)return "before"==n?c:Math.min(t.length-1,c+1);o(a,r)<0?l=c+1:s=c-1,f++;}return -1;},t.bridgeObject=yi,t.copyAndExtendArray=function(t,r){var e;return ro(e=[]).call(e,yo(t),[r]);},t.copyArray=function(t){return Ao(t).call(t);},t.deepExtend=ni,t.easingFunctions={linear:function linear(t){return t;},easeInQuad:function easeInQuad(t){return t*t;},easeOutQuad:function easeOutQuad(t){return t*(2-t);},easeInOutQuad:function easeInOutQuad(t){return t<.5?2*t*t:(4-2*t)*t-1;},easeInCubic:function easeInCubic(t){return t*t*t;},easeOutCubic:function easeOutCubic(t){return --t*t*t+1;},easeInOutCubic:function easeInOutCubic(t){return t<.5?4*t*t*t:(t-1)*(2*t-2)*(2*t-2)+1;},easeInQuart:function easeInQuart(t){return t*t*t*t;},easeOutQuart:function easeOutQuart(t){return 1- --t*t*t*t;},easeInOutQuart:function easeInOutQuart(t){return t<.5?8*t*t*t*t:1-8*--t*t*t*t;},easeInQuint:function easeInQuint(t){return t*t*t*t*t;},easeOutQuint:function easeOutQuint(t){return 1+--t*t*t*t*t;},easeInOutQuint:function easeInOutQuint(t){return t<.5?16*t*t*t*t*t:1+16*--t*t*t*t*t;}},t.equalArray=function(t,r){if(t.length!==r.length)return !1;for(var e=0,n=t.length;e<n;e++){if(t[e]!=r[e])return !1;}return !0;},t.extend=ei,t.fillIfDefined=function t(r,e){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2];for(var o in r){if(void 0!==e[o])if(null===e[o]||"object"!==zo(e[o]))ri(r,e,o,n);else {var i=r[o],a=e[o];ti(i)&&ti(a)&&t(i,a,n);}}},t.forEach=function(t,r){if(Do(t))for(var e=t.length,n=0;n<e;n++){r(t[n],n,t);}else for(var o in t){Object.prototype.hasOwnProperty.call(t,o)&&r(t[o],o,t);}},t.getAbsoluteLeft=function(t){return t.getBoundingClientRect().left;},t.getAbsoluteRight=function(t){return t.getBoundingClientRect().right;},t.getAbsoluteTop=function(t){return t.getBoundingClientRect().top;},t.getScrollBarWidth=function(){var t=document.createElement("p");t.style.width="100%",t.style.height="200px";var r=document.createElement("div");r.style.position="absolute",r.style.top="0px",r.style.left="0px",r.style.visibility="hidden",r.style.width="200px",r.style.height="150px",r.style.overflow="hidden",r.appendChild(t),document.body.appendChild(r);var e=t.offsetWidth;r.style.overflow="scroll";var n=t.offsetWidth;return e==n&&(n=r.clientWidth),document.body.removeChild(r),e-n;},t.getTarget=function(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:window.event,r=null;return t&&(t.target?r=t.target:t.srcElement&&(r=t.srcElement)),r instanceof Element&&(null==r.nodeType||3!=r.nodeType||(r=r.parentNode)instanceof Element)?r:null;},t.getType=function(t){var r=zo(t);return "object"===r?null===t?"null":t instanceof Boolean?"Boolean":t instanceof Number?"Number":t instanceof String?"String":Do(t)?"Array":t instanceof Date?"Date":"Object":"number"===r?"Number":"boolean"===r?"Boolean":"string"===r?"String":void 0===r?"undefined":r;},t.hasParent=function(t,r){for(var e=t;e;){if(e===r)return !0;if(!e.parentNode)return !1;e=e.parentNode;}return !1;},t.hexToHSV=vi,t.hexToRGB=ai,t.insertSort=function(t,r){for(var e=0;e<t.length;e++){var n=t[e],o=void 0;for(o=e;o>0&&r(n,t[o-1])<0;o--){t[o]=t[o-1];}t[o]=n;}return t;},t.isDate=function(t){if(t instanceof Date)return !0;if(Zo(t)){if(qo.exec(t))return !0;if(!isNaN(Date.parse(t)))return !0;}return !1;},t.isNumber=Ko,t.isObject=ti,t.isString=Zo,t.isValidHex=hi,t.isValidRGB=di,t.isValidRGBA=function(t){return Xo.test(t);},t.mergeOptions=function(t,r,e){var n=arguments.length>3&&void 0!==arguments[3]?arguments[3]:{},o=function o(t){return null!=t;},i=function i(t){return null!==t&&"object"===zo(t);},a=function a(t){for(var r in t){if(Object.prototype.hasOwnProperty.call(t,r))return !1;}return !0;};if(!i(t))throw new Error("Parameter mergeTarget must be an object");if(!i(r))throw new Error("Parameter options must be an object");if(!o(e))throw new Error("Parameter option must have a value");if(!i(n))throw new Error("Parameter globalOptions must be an object");var u=function u(t,r,e){i(t[e])||(t[e]={});var n=r[e],o=t[e];for(var a in n){Object.prototype.hasOwnProperty.call(n,a)&&(o[a]=n[a]);}},c=r[e],f=i(n)&&!a(n),l=f?n[e]:void 0,s=l?l.enabled:void 0;if(void 0!==c){if("boolean"==typeof c)return i(t[e])||(t[e]={}),void(t[e].enabled=c);if(null===c&&!i(t[e])){if(!o(l))return;t[e]=He(l);}if(i(c)){var p=!0;void 0!==c.enabled?p=c.enabled:void 0!==s&&(p=l.enabled),u(t,r,e),t[e].enabled=p;}}},t.option=ii,t.overrideOpacity=function(t,r){if(-1!==No(t).call(t,"rgba"))return t;if(-1!==No(t).call(t,"rgb")){var e=t.substr(No(t).call(t,"(")+1).replace(")","").split(",");return "rgba("+e[0]+","+e[1]+","+e[2]+","+r+")";}var n=ai(t);return null==n?t:"rgba("+n.r+","+n.g+","+n.b+","+r+")";},t.parseColor=function(t,r){if(Zo(t)){var e=t;if(di(e)){var n,o=In(n=e.substr(4).substr(0,e.length-5).split(",")).call(n,function(t){return Fn(t);});e=ui(o[0],o[1],o[2]);}if(!0===hi(e)){var i=vi(e),a={h:i.h,s:.8*i.s,v:Math.min(1,1.02*i.v)},u={h:i.h,s:Math.min(1,1.25*i.s),v:.8*i.v},c=pi(u.h,u.s,u.v),f=pi(a.h,a.s,a.v);return {background:e,border:c,highlight:{background:f,border:c},hover:{background:f,border:c}};}return {background:e,border:e,highlight:{background:e,border:e},hover:{background:e,border:e}};}return r?{background:t.background||r.background,border:t.border||r.border,highlight:Zo(t.highlight)?{border:t.highlight,background:t.highlight}:{background:t.highlight&&t.highlight.background||r.highlight.background,border:t.highlight&&t.highlight.border||r.highlight.border},hover:Zo(t.hover)?{border:t.hover,background:t.hover}:{border:t.hover&&t.hover.border||r.hover.border,background:t.hover&&t.hover.background||r.hover.background}}:{background:t.background||void 0,border:t.border||void 0,highlight:Zo(t.highlight)?{border:t.highlight,background:t.highlight}:{background:t.highlight&&t.highlight.background||void 0,border:t.highlight&&t.highlight.border||void 0},hover:Zo(t.hover)?{border:t.hover,background:t.hover}:{border:t.hover&&t.hover.border||void 0,background:t.hover&&t.hover.background||void 0}};},t.preventDefault=function(t){t||(t=window.event),t&&(t.preventDefault?t.preventDefault():t.returnValue=!1);},t.recursiveDOMDelete=function t(r){if(r)for(;!0===r.hasChildNodes();){var e=r.firstChild;e&&(t(e),r.removeChild(e));}},t.removeClassName=function(t,r){var e=t.className.split(" "),n=r.split(" ");e=Jn(e).call(e,function(t){return No(n).call(n,t)<0;}),t.className=e.join(" ");},t.removeCssText=function(t,r){var e=fi(t.style.cssText),n=fi(r);for(var o in n){Object.prototype.hasOwnProperty.call(n,o)&&delete e[o];}t.style.cssText=li(e);},t.removeEventListener=function(t,r,e,n){var o;t.removeEventListener?(void 0===n&&(n=!1),"mousewheel"===r&&No(o=navigator.userAgent).call(o,"Firefox")>=0&&(r="DOMMouseScroll"),t.removeEventListener(r,e,n)):t.detachEvent("on"+r,e);},t.selectiveBridgeObject=function(t,r){if(null!==r&&"object"===zo(r)){for(var e=He(r),n=0;n<t.length;n++){Object.prototype.hasOwnProperty.call(r,t[n])&&"object"==zo(r[t[n]])&&(e[t[n]]=yi(r[t[n]]));}return e;}return null;},t.selectiveDeepExtend=function(t,r,e){var n=arguments.length>3&&void 0!==arguments[3]&&arguments[3];if(Do(e))throw new TypeError("Arrays are not supported by deepExtend");for(var o=0;o<t.length;o++){var i=t[o];if(Object.prototype.hasOwnProperty.call(e,i))if(e[i]&&e[i].constructor===Object)void 0===r[i]&&(r[i]={}),r[i].constructor===Object?ni(r[i],e[i],!1,n):ri(r,e,i,n);else {if(Do(e[i]))throw new TypeError("Arrays are not supported by deepExtend");ri(r,e,i,n);}}return r;},t.selectiveExtend=function(t,r){if(!Do(t))throw new Error("Array with property names expected as first argument");for(var e=arguments.length,n=new Array(e>2?e-2:0),o=2;o<e;o++){n[o-2]=arguments[o];}for(var i=0,a=n;i<a.length;i++){for(var u=a[i],c=0;c<t.length;c++){var f=t[c];u&&Object.prototype.hasOwnProperty.call(u,f)&&(r[f]=u[f]);}}return r;},t.selectiveNotDeepExtend=function(t,r,e){var n=arguments.length>3&&void 0!==arguments[3]&&arguments[3];if(Do(e))throw new TypeError("Arrays are not supported by deepExtend");for(var o in e){if(Object.prototype.hasOwnProperty.call(e,o)&&-1===No(t).call(t,o))if(e[o]&&e[o].constructor===Object)void 0===r[o]&&(r[o]={}),r[o].constructor===Object?ni(r[o],e[o]):ri(r,e,o,n);else if(Do(e[o])){r[o]=[];for(var i=0;i<e[o].length;i++){r[o].push(e[o][i]);}}else ri(r,e,o,n);}return r;},t.throttle=function(t){var r=!1;return function(){r||(r=!0,requestAnimationFrame(function(){r=!1,t();}));};},t.toArray=oi,t.topMost=function(t,r){var e;Do(r)||(r=[r]);var n=!0,o=!1,i=void 0;try{for(var a,u=Lr(t);!(n=(a=u.next()).done);n=!0){var c=a.value;if(c){e=c[r[0]];for(var f=1;f<r.length;f++){e&&(e=e[r[f]]);}if(void 0!==e)break;}}}catch(t){o=!0,i=t;}finally{try{n||null==u.return||u.return();}finally{if(o)throw i;}}return e;},t.updateProperty=function(t,r,e){return t[r]!==e&&(t[r]=e,!0);},defineProperty$2(t,"__esModule",{value:!0});});});unwrapExports$1(visUtil_min);getCjsExportFromNamespace$1(Activator);/**
	 * Turn an element into an clickToUse element.
	 * When not active, the element has a transparent overlay. When the overlay is
	 * clicked, the mode is changed to active.
	 * When active, the element is displayed with a blue border around it, and
	 * the interactive contents of the element can be used. When clicked outside
	 * the element, the elements mode is changed to inactive.
	 * @param {Element} container
	 * @constructor Activator
	 */function Activator$1(container){var _context13,_this=this,_context14;this.active=false;this.dom={container:container};this.dom.overlay=document.createElement('div');this.dom.overlay.className='vis-overlay';this.dom.container.appendChild(this.dom.overlay);this.hammer=hammer(this.dom.overlay);this.hammer.on('tap',bind$2(_context13=this._onTapOverlay).call(_context13,this));// block all touch events (except tap)
	var events=['tap','doubletap','press','pinch','pan','panstart','panmove','panend'];forEach$3(events).call(events,function(event){_this.hammer.on(event,function(event){event.srcEvent.stopPropagation();});});// attach a click event to the window, in order to deactivate when clicking outside the timeline
	if(document&&document.body){this.onClick=function(event){if(!_hasParent(event.target,container)){_this.deactivate();}};document.body.addEventListener('click',this.onClick);}if(this.keycharm!==undefined){this.keycharm.destroy();}this.keycharm=keycharm();// keycharm listener only bounded when active)
	this.escListener=bind$2(_context14=this.deactivate).call(_context14,this);}// turn into an event emitter
	componentEmitter(Activator$1.prototype);// The currently active activator
	Activator$1.current=null;/**
	 * Destroy the activator. Cleans up all created DOM and event listeners
	 */Activator$1.prototype.destroy=function(){this.deactivate();// remove dom
	this.dom.overlay.parentNode.removeChild(this.dom.overlay);// remove global event listener
	if(this.onClick){document.body.removeEventListener('click',this.onClick);}// remove keycharm
	if(this.keycharm!==undefined){this.keycharm.destroy();}this.keycharm=null;// cleanup hammer instances
	this.hammer.destroy();this.hammer=null;// FIXME: cleaning up hammer instances doesn't work (Timeline not removed from memory)
	};/**
	 * Activate the element
	 * Overlay is hidden, element is decorated with a blue shadow border
	 */Activator$1.prototype.activate=function(){var _context15;// we allow only one active activator at a time
	if(Activator$1.current){Activator$1.current.deactivate();}Activator$1.current=this;this.active=true;this.dom.overlay.style.display='none';visUtil_min.addClassName(this.dom.container,'vis-active');this.emit('change');this.emit('activate');// ugly hack: bind ESC after emitting the events, as the Network rebinds all
	// keyboard events on a 'change' event
	bind$2(_context15=this.keycharm).call(_context15,'esc',this.escListener);};/**
	 * Deactivate the element
	 * Overlay is displayed on top of the element
	 */Activator$1.prototype.deactivate=function(){this.active=false;this.dom.overlay.style.display='block';visUtil_min.removeClassName(this.dom.container,'vis-active');this.keycharm.unbind('esc',this.escListener);this.emit('change');this.emit('deactivate');};/**
	 * Handle a tap event: activate the container
	 * @param {Event}  event   The event
	 * @private
	 */Activator$1.prototype._onTapOverlay=function(event){// activate the container
	this.activate();event.srcEvent.stopPropagation();};/**
	 * Test whether the element has the requested parent element somewhere in
	 * its chain of parent nodes.
	 * @param {HTMLElement} element
	 * @param {HTMLElement} parent
	 * @returns {boolean} Returns true when the parent is found somewhere in the
	 *                    chain of parent nodes.
	 * @private
	 */function _hasParent(element,parent){while(element){if(element===parent){return true;}element=element.parentNode;}return false;}var Activator_1=Activator$1;var locales=createCommonjsModule$1(function(module,exports){// English
	exports['en']={edit:'Edit',del:'Delete selected',back:'Back',addNode:'Add Node',addEdge:'Add Edge',editNode:'Edit Node',editEdge:'Edit Edge',addDescription:'Click in an empty space to place a new node.',edgeDescription:'Click on a node and drag the edge to another node to connect them.',editEdgeDescription:'Click on the control points and drag them to a node to connect to it.',createEdgeError:'Cannot link edges to a cluster.',deleteClusterError:'Clusters cannot be deleted.',editClusterError:'Clusters cannot be edited.'};exports['en_EN']=exports['en'];exports['en_US']=exports['en'];// German
	exports['de']={edit:'Editieren',del:"L\xF6sche Auswahl",back:"Zur\xFCck",addNode:"Knoten hinzuf\xFCgen",addEdge:"Kante hinzuf\xFCgen",editNode:'Knoten editieren',editEdge:'Kante editieren',addDescription:'Klicke auf eine freie Stelle, um einen neuen Knoten zu plazieren.',edgeDescription:'Klicke auf einen Knoten und ziehe die Kante zu einem anderen Knoten, um diese zu verbinden.',editEdgeDescription:'Klicke auf die Verbindungspunkte und ziehe diese auf einen Knoten, um sie zu verbinden.',createEdgeError:"Es ist nicht m\xF6glich, Kanten mit Clustern zu verbinden.",deleteClusterError:"Cluster k\xF6nnen nicht gel\xF6scht werden.",editClusterError:"Cluster k\xF6nnen nicht editiert werden."};exports['de_DE']=exports['de'];// Spanish
	exports['es']={edit:'Editar',del:"Eliminar selecci\xF3n",back:"Atr\xE1s",addNode:"A\xF1adir nodo",addEdge:"A\xF1adir arista",editNode:'Editar nodo',editEdge:'Editar arista',addDescription:"Haga clic en un lugar vac\xEDo para colocar un nuevo nodo.",edgeDescription:'Haga clic en un nodo y arrastre la arista hacia otro nodo para conectarlos.',editEdgeDescription:'Haga clic en un punto de control y arrastrelo a un nodo para conectarlo.',createEdgeError:'No se puede conectar una arista a un grupo.',deleteClusterError:'No es posible eliminar grupos.',editClusterError:'No es posible editar grupos.'};exports['es_ES']=exports['es'];//Italiano
	exports['it']={edit:'Modifica',del:'Cancella la selezione',back:'Indietro',addNode:'Aggiungi un nodo',addEdge:'Aggiungi un vertice',editNode:'Modifica il nodo',editEdge:'Modifica il vertice',addDescription:'Clicca per aggiungere un nuovo nodo',edgeDescription:'Clicca su un nodo e trascinalo ad un altro nodo per connetterli.',editEdgeDescription:'Clicca sui Punti di controllo e trascinali ad un nodo per connetterli.',createEdgeError:'Non si possono collegare vertici ad un cluster',deleteClusterError:'I cluster non possono essere cancellati',editClusterError:'I clusters non possono essere modificati.'};exports['it_IT']=exports['it'];// Dutch
	exports['nl']={edit:'Wijzigen',del:'Selectie verwijderen',back:'Terug',addNode:'Node toevoegen',addEdge:'Link toevoegen',editNode:'Node wijzigen',editEdge:'Link wijzigen',addDescription:'Klik op een leeg gebied om een nieuwe node te maken.',edgeDescription:'Klik op een node en sleep de link naar een andere node om ze te verbinden.',editEdgeDescription:'Klik op de verbindingspunten en sleep ze naar een node om daarmee te verbinden.',createEdgeError:'Kan geen link maken naar een cluster.',deleteClusterError:'Clusters kunnen niet worden verwijderd.',editClusterError:'Clusters kunnen niet worden aangepast.'};exports['nl_NL']=exports['nl'];exports['nl_BE']=exports['nl'];// Portuguese Brazil
	exports['pt-br']={edit:'Editar',del:'Remover selecionado',back:'Voltar',addNode:'Adicionar nó',addEdge:'Adicionar aresta',editNode:'Editar nó',editEdge:'Editar aresta',addDescription:'Clique em um espaço em branco para adicionar um novo nó',edgeDescription:'Clique em um nó e arraste a aresta até outro nó para conectá-los',editEdgeDescription:'Clique nos pontos de controle e os arraste para um nó para conectá-los',createEdgeError:'Não foi possível linkar arestas a um cluster.',deleteClusterError:'Clusters não puderam ser removidos.',editClusterError:'Clusters não puderam ser editados.'};exports['pt-BR']=exports['pt-br'];exports['pt_BR']=exports['pt-br'];exports['pt_br']=exports['pt-br'];// Russian
	exports['ru']={edit:'Редактировать',del:'Удалить выбранное',back:'Назад',addNode:'Добавить узел',addEdge:'Добавить ребро',editNode:'Редактировать узел',editEdge:'Редактировать ребро',addDescription:'Кликните в свободное место, чтобы добавить новый узел.',edgeDescription:'Кликните на узел и протяните ребро к другому узлу, чтобы соединить их.',editEdgeDescription:'Кликните на контрольные точки и перетащите их в узел, чтобы подключиться к нему.',createEdgeError:'Невозможно соединить ребра в кластер.',deleteClusterError:'Кластеры не могут быть удалены',editClusterError:'Кластеры недоступны для редактирования.'};exports['ru_RU']=exports['ru'];// Chinese
	exports['cn']={edit:'编辑',del:'删除选定',back:'返回',addNode:'添加节点',addEdge:'添加连接线',editNode:'编辑节点',editEdge:'编辑连接线',addDescription:'单击空白处放置新节点。',edgeDescription:'单击某个节点并将该连接线拖动到另一个节点以连接它们。',editEdgeDescription:'单击控制节点并将它们拖到节点上连接。',createEdgeError:'无法将连接线连接到群集。',deleteClusterError:'无法删除群集。',editClusterError:'无法编辑群集。'};exports['zh_CN']=exports['cn'];// Ukrainian
	exports['uk']={edit:'Редагувати',del:'Видалити обране',back:'Назад',addNode:'Додати вузол',addEdge:'Додати край',editNode:'Редагувати вузол',editEdge:'Редагувати край',addDescription:'Kлікніть на вільне місце, щоб додати новий вузол.',edgeDescription:'Клікніть на вузол і перетягніть край до іншого вузла, щоб їх з\'єднати.',editEdgeDescription:'Клікніть на контрольні точки і перетягніть їх у вузол, щоб підключитися до нього.',createEdgeError:'Не можливо об\'єднати краї в групу.',deleteClusterError:'Групи не можуть бути видалені.',editClusterError:'Групи недоступні для редагування.'};exports['uk_UA']=exports['uk'];// French
	exports['fr']={edit:'Éditer',del:'Effacer la sélection',back:'Retour',addNode:'Ajouter un nœud',addEdge:'Ajouter un lien',editNode:'Éditer le nœud',editEdge:'Éditer le lien',addDescription:'Cliquez dans un endroit vide pour placer un nœud.',edgeDescription:'Cliquez sur un nœud et glissez le lien vers un autre nœud pour les connecter.',editEdgeDescription:'Cliquez sur les points de contrôle et glissez-les pour connecter un nœud.',createEdgeError:'Impossible de créer un lien vers un cluster.',deleteClusterError:'Les clusters ne peuvent pas être effacés.',editClusterError:'Les clusters ne peuvent pas être édités.'};exports['fr_FR']=exports['fr'];// Czech
	exports['cs']={edit:'Upravit',del:'Smazat výběr',back:'Zpět',addNode:'Přidat vrchol',addEdge:'Přidat hranu',editNode:'Upravit vrchol',editEdge:'Upravit hranu',addDescription:'Kluknutím do prázdného prostoru můžete přidat nový vrchol.',edgeDescription:'Přetažením z jednoho vrcholu do druhého můžete spojit tyto vrcholy novou hranou.',editEdgeDescription:'Přetažením kontrolního vrcholu hrany ji můžete připojit k jinému vrcholu.',createEdgeError:'Nelze připojit hranu ke shluku.',deleteClusterError:'Nelze mazat shluky.',editClusterError:'Nelze upravovat shluky.'};exports['cs_CZ']=exports['cs'];});/**
	 * Associates a canvas to a given image, containing a number of renderings
	 * of the image at various sizes.
	 *
	 * This technique is known as 'mipmapping'.
	 *
	 * NOTE: Images can also be of type 'data:svg+xml`. This code also works
	 *       for svg, but the mipmapping may not be necessary.
	 *
	 * @param {Image} image
	 */var CachedImage=/*#__PURE__*/function(){/**
	   * @ignore
	   */function CachedImage(){classCallCheck(this,CachedImage);// eslint-disable-line no-unused-vars
	this.NUM_ITERATIONS=4;// Number of items in the coordinates array
	this.image=new Image();this.canvas=document.createElement('canvas');}/**
	   * Called when the image has been successfully loaded.
	   */createClass(CachedImage,[{key:"init",value:function init(){if(this.initialized())return;this.src=this.image.src;// For same interface with Image
	var w=this.image.width;var h=this.image.height;// Ease external access
	this.width=w;this.height=h;var h2=Math.floor(h/2);var h4=Math.floor(h/4);var h8=Math.floor(h/8);var h16=Math.floor(h/16);var w2=Math.floor(w/2);var w4=Math.floor(w/4);var w8=Math.floor(w/8);var w16=Math.floor(w/16);// Make canvas as small as possible
	this.canvas.width=3*w4;this.canvas.height=h2;// Coordinates and sizes of images contained in the canvas
	// Values per row:  [top x, left y, width, height]
	this.coordinates=[[0,0,w2,h2],[w2,0,w4,h4],[w2,h4,w8,h8],[5*w8,h4,w16,h16]];this._fillMipMap();}/**
	   * @return {Boolean} true if init() has been called, false otherwise.
	   */},{key:"initialized",value:function initialized(){return this.coordinates!==undefined;}/**
	   * Redraw main image in various sizes to the context.
	   *
	   * The rationale behind this is to reduce artefacts due to interpolation
	   * at differing zoom levels.
	   *
	   * Source: http://stackoverflow.com/q/18761404/1223531
	   *
	   * This methods takes the resizing out of the drawing loop, in order to
	   * reduce performance overhead.
	   *
	   * TODO: The code assumes that a 2D context can always be gotten. This is
	   *       not necessarily true! OTOH, if not true then usage of this class
	   *       is senseless.
	   *
	   * @private
	   */},{key:"_fillMipMap",value:function _fillMipMap(){var ctx=this.canvas.getContext('2d');// First zoom-level comes from the image
	var to=this.coordinates[0];ctx.drawImage(this.image,to[0],to[1],to[2],to[3]);// The rest are copy actions internal to the canvas/context
	for(var iterations=1;iterations<this.NUM_ITERATIONS;iterations++){var from=this.coordinates[iterations-1];var _to=this.coordinates[iterations];ctx.drawImage(this.canvas,from[0],from[1],from[2],from[3],_to[0],_to[1],_to[2],_to[3]);}}/**
	   * Draw the image, using the mipmap if necessary.
	   *
	   * MipMap is only used if param factor > 2; otherwise, original bitmap
	   * is resized. This is also used to skip mipmap usage, e.g. by setting factor = 1
	   *
	   * Credits to 'Alex de Mulder' for original implementation.
	   *
	   * @param {CanvasRenderingContext2D} ctx  context on which to draw zoomed image
	   * @param {Float} factor scale factor at which to draw
	   * @param {number} left
	   * @param {number} top
	   * @param {number} width
	   * @param {number} height
	   */},{key:"drawImageAtPosition",value:function drawImageAtPosition(ctx,factor,left,top,width,height){if(!this.initialized())return;//can't draw image yet not intialized
	if(factor>2){// Determine which zoomed image to use
	factor*=0.5;var iterations=0;while(factor>2&&iterations<this.NUM_ITERATIONS){factor*=0.5;iterations+=1;}if(iterations>=this.NUM_ITERATIONS){iterations=this.NUM_ITERATIONS-1;}//console.log("iterations: " + iterations);
	var from=this.coordinates[iterations];ctx.drawImage(this.canvas,from[0],from[1],from[2],from[3],left,top,width,height);}else {// Draw image directly
	ctx.drawImage(this.image,left,top,width,height);}}}]);return CachedImage;}();/**
	 * This callback is a callback that accepts an Image.
	 * @callback ImageCallback
	 * @param {Image} image
	 */ /**
	 * This class loads images and keeps them stored.
	 *
	 * @param {ImageCallback} callback
	 */var Images=/*#__PURE__*/function(){/**
	   * @param {ImageCallback} callback
	   */function Images(callback){classCallCheck(this,Images);this.images={};this.imageBroken={};this.callback=callback;}/**
	     * @param {string} url                      The original Url that failed to load, if the broken image is successfully loaded it will be added to the cache using this Url as the key so that subsequent requests for this Url will return the broken image
	     * @param {string} brokenUrl                Url the broken image to try and load
	     * @param {Image} imageToLoadBrokenUrlOn   The image object
	     */createClass(Images,[{key:"_tryloadBrokenUrl",value:function _tryloadBrokenUrl(url,brokenUrl,imageToLoadBrokenUrlOn){//If these parameters aren't specified then exit the function because nothing constructive can be done
	if(url===undefined||imageToLoadBrokenUrlOn===undefined)return;if(brokenUrl===undefined){console.warn("No broken url image defined");return;}//Clear the old subscription to the error event and put a new in place that only handle errors in loading the brokenImageUrl
	imageToLoadBrokenUrlOn.image.onerror=function(){console.error("Could not load brokenImage:",brokenUrl);// cache item will contain empty image, this should be OK for default
	};//Set the source of the image to the brokenUrl, this is actually what kicks off the loading of the broken image
	imageToLoadBrokenUrlOn.image.src=brokenUrl;}/**
	   *
	   * @param {vis.Image} imageToRedrawWith
	   * @private
	   */},{key:"_redrawWithImage",value:function _redrawWithImage(imageToRedrawWith){if(this.callback){this.callback(imageToRedrawWith);}}/**
	     * @param {string} url          Url of the image
	     * @param {string} brokenUrl    Url of an image to use if the url image is not found
	     * @return {Image} img          The image object
	     */},{key:"load",value:function load(url,brokenUrl){var _this2=this;//Try and get the image from the cache, if successful then return the cached image   
	var cachedImage=this.images[url];if(cachedImage)return cachedImage;//Create a new image
	var img=new CachedImage();// Need to add to cache here, otherwise final return will spawn different copies of the same image,
	// Also, there will be multiple loads of the same image.
	this.images[url]=img;//Subscribe to the event that is raised if the image loads successfully 
	img.image.onload=function(){// Properly init the cached item and then request a redraw
	_this2._fixImageCoordinates(img.image);img.init();_this2._redrawWithImage(img);};//Subscribe to the event that is raised if the image fails to load
	img.image.onerror=function(){console.error("Could not load image:",url);//Try and load the image specified by the brokenUrl using
	_this2._tryloadBrokenUrl(url,brokenUrl,img);};//Set the source of the image to the url, this is what actually kicks off the loading of the image
	img.image.src=url;//Return the new image
	return img;}/**
	   * IE11 fix -- thanks dponch!
	   *
	   * Local helper function
	   * @param {vis.Image} imageToCache
	   * @private
	   */},{key:"_fixImageCoordinates",value:function _fixImageCoordinates(imageToCache){if(imageToCache.width===0){document.body.appendChild(imageToCache);imageToCache.width=imageToCache.offsetWidth;imageToCache.height=imageToCache.offsetHeight;document.body.removeChild(imageToCache);}}}]);return Images;}();/**
	 * This class can store groups and options specific for groups.
	 */var Groups=/*#__PURE__*/function(){/**
	   * @ignore
	   */function Groups(){classCallCheck(this,Groups);this.clear();this.defaultIndex=0;this.groupsArray=[];this.groupIndex=0;this.defaultGroups=[{border:"#2B7CE9",background:"#97C2FC",highlight:{border:"#2B7CE9",background:"#D2E5FF"},hover:{border:"#2B7CE9",background:"#D2E5FF"}},// 0: blue
	{border:"#FFA500",background:"#FFFF00",highlight:{border:"#FFA500",background:"#FFFFA3"},hover:{border:"#FFA500",background:"#FFFFA3"}},// 1: yellow
	{border:"#FA0A10",background:"#FB7E81",highlight:{border:"#FA0A10",background:"#FFAFB1"},hover:{border:"#FA0A10",background:"#FFAFB1"}},// 2: red
	{border:"#41A906",background:"#7BE141",highlight:{border:"#41A906",background:"#A1EC76"},hover:{border:"#41A906",background:"#A1EC76"}},// 3: green
	{border:"#E129F0",background:"#EB7DF4",highlight:{border:"#E129F0",background:"#F0B3F5"},hover:{border:"#E129F0",background:"#F0B3F5"}},// 4: magenta
	{border:"#7C29F0",background:"#AD85E4",highlight:{border:"#7C29F0",background:"#D3BDF0"},hover:{border:"#7C29F0",background:"#D3BDF0"}},// 5: purple
	{border:"#C37F00",background:"#FFA807",highlight:{border:"#C37F00",background:"#FFCA66"},hover:{border:"#C37F00",background:"#FFCA66"}},// 6: orange
	{border:"#4220FB",background:"#6E6EFD",highlight:{border:"#4220FB",background:"#9B9BFD"},hover:{border:"#4220FB",background:"#9B9BFD"}},// 7: darkblue
	{border:"#FD5A77",background:"#FFC0CB",highlight:{border:"#FD5A77",background:"#FFD1D9"},hover:{border:"#FD5A77",background:"#FFD1D9"}},// 8: pink
	{border:"#4AD63A",background:"#C2FABC",highlight:{border:"#4AD63A",background:"#E6FFE3"},hover:{border:"#4AD63A",background:"#E6FFE3"}},// 9: mint
	{border:"#990000",background:"#EE0000",highlight:{border:"#BB0000",background:"#FF3333"},hover:{border:"#BB0000",background:"#FF3333"}},// 10:bright red
	{border:"#FF6000",background:"#FF6000",highlight:{border:"#FF6000",background:"#FF6000"},hover:{border:"#FF6000",background:"#FF6000"}},// 12: real orange
	{border:"#97C2FC",background:"#2B7CE9",highlight:{border:"#D2E5FF",background:"#2B7CE9"},hover:{border:"#D2E5FF",background:"#2B7CE9"}},// 13: blue
	{border:"#399605",background:"#255C03",highlight:{border:"#399605",background:"#255C03"},hover:{border:"#399605",background:"#255C03"}},// 14: green
	{border:"#B70054",background:"#FF007E",highlight:{border:"#B70054",background:"#FF007E"},hover:{border:"#B70054",background:"#FF007E"}},// 15: magenta
	{border:"#AD85E4",background:"#7C29F0",highlight:{border:"#D3BDF0",background:"#7C29F0"},hover:{border:"#D3BDF0",background:"#7C29F0"}},// 16: purple
	{border:"#4557FA",background:"#000EA1",highlight:{border:"#6E6EFD",background:"#000EA1"},hover:{border:"#6E6EFD",background:"#000EA1"}},// 17: darkblue
	{border:"#FFC0CB",background:"#FD5A77",highlight:{border:"#FFD1D9",background:"#FD5A77"},hover:{border:"#FFD1D9",background:"#FD5A77"}},// 18: pink
	{border:"#C2FABC",background:"#74D66A",highlight:{border:"#E6FFE3",background:"#74D66A"},hover:{border:"#E6FFE3",background:"#74D66A"}},// 19: mint
	{border:"#EE0000",background:"#990000",highlight:{border:"#FF3333",background:"#BB0000"},hover:{border:"#FF3333",background:"#BB0000"}}// 20:bright red
	];this.options={};this.defaultOptions={useDefaultGroups:true};extend(this.options,this.defaultOptions);}/**
	   *
	   * @param {Object} options
	   */createClass(Groups,[{key:"setOptions",value:function setOptions(options){var optionFields=['useDefaultGroups'];if(options!==undefined){for(var groupName in options){if(options.hasOwnProperty(groupName)){if(indexOf$3(optionFields).call(optionFields,groupName)===-1){var group=options[groupName];this.add(groupName,group);}}}}}/**
	   * Clear all groups
	   */},{key:"clear",value:function clear(){this.groups={};this.groupsArray=[];}/**
	   * Get group options of a groupname.
	   * If groupname is not found, a new group may be created.
	   *
	   * @param {*}       groupname     Can be a number, string, Date, etc.
	   * @param {boolean} [shouldCreate=true] If true, create a new group
	   * @return {Object} The found or created group
	   */},{key:"get",value:function get(groupname){var shouldCreate=arguments.length>1&&arguments[1]!==undefined?arguments[1]:true;var group=this.groups[groupname];if(group===undefined&&shouldCreate){if(this.options.useDefaultGroups===false&&this.groupsArray.length>0){// create new group
	var _index=this.groupIndex%this.groupsArray.length;this.groupIndex++;group={};group.color=this.groups[this.groupsArray[_index]];this.groups[groupname]=group;}else {// create new group
	var _index2=this.defaultIndex%this.defaultGroups.length;this.defaultIndex++;group={};group.color=this.defaultGroups[_index2];this.groups[groupname]=group;}}return group;}/**
	   * Add a custom group style
	   * @param {string} groupName
	   * @param {Object} style       An object containing borderColor,
	   *                             backgroundColor, etc.
	   * @return {Object} group      The created group object
	   */},{key:"add",value:function add(groupName,style){this.groups[groupName]=style;this.groupsArray.push(groupName);return style;}}]);return Groups;}();/**
	 * Helper functions for components
	 * @class
	 */var ComponentUtil=/*#__PURE__*/function(){function ComponentUtil(){classCallCheck(this,ComponentUtil);}createClass(ComponentUtil,null,[{key:"choosify",/**
	   * Determine values to use for (sub)options of 'chosen'.
	   *
	   * This option is either a boolean or an object whose values should be examined further.
	   * The relevant structures are:
	   *
	   * - chosen: <boolean value>
	   * - chosen: { subOption: <boolean or function> }
	   *
	   * Where subOption is 'node', 'edge' or 'label'.
	   *
	   * The intention of this method appears to be to set a specific priority to the options;
	   * Since most properties are either bridged or merged into the local options objects, there
	   * is not much point in handling them separately.
	   * TODO: examine if 'most' in previous sentence can be replaced with 'all'. In that case, we
	   *       should be able to get rid of this method.
	   *
	   * @param {string}  subOption  option within object 'chosen' to consider; either 'node', 'edge' or 'label'
	   * @param {Object}  pile       array of options objects to consider
	   * 
	   * @return {boolean|function}  value for passed subOption of 'chosen' to use
	   */value:function choosify(subOption,pile){// allowed values for subOption
	var allowed=['node','edge','label'];var value=true;var chosen=topMost(pile,'chosen');if(typeof chosen==='boolean'){value=chosen;}else if(_typeof_1(chosen)==='object'){if(indexOf$3(allowed).call(allowed,subOption)===-1){throw new Error('choosify: subOption \''+subOption+'\' should be one of '+"'"+allowed.join("', '")+"'");}var chosenEdge=topMost(pile,['chosen',subOption]);if(typeof chosenEdge==='boolean'||typeof chosenEdge==='function'){value=chosenEdge;}}return value;}/**
	   * Check if the point falls within the given rectangle.
	   *
	   * @param {rect} rect
	   * @param {point} point
	   * @param {rotationPoint} [rotationPoint] if specified, the rotation that applies to the rectangle.
	   * @returns {boolean}  true if point within rectangle, false otherwise
	   * @static
	   */},{key:"pointInRect",value:function pointInRect(rect,point,rotationPoint){if(rect.width<=0||rect.height<=0){return false;// early out
	}if(rotationPoint!==undefined){// Rotate the point the same amount as the rectangle
	var tmp={x:point.x-rotationPoint.x,y:point.y-rotationPoint.y};if(rotationPoint.angle!==0){// In order to get the coordinates the same, you need to 
	// rotate in the reverse direction
	var angle=-rotationPoint.angle;var tmp2={x:Math.cos(angle)*tmp.x-Math.sin(angle)*tmp.y,y:Math.sin(angle)*tmp.x+Math.cos(angle)*tmp.y};point=tmp2;}else {point=tmp;}// Note that if a rotation is specified, the rectangle coordinates
	// are **not* the full canvas coordinates. They are relative to the
	// rotationPoint. Hence, the point coordinates need not be translated
	// back in this case.
	}var right=rect.x+rect.width;var bottom=rect.y+rect.width;return rect.left<point.x&&right>point.x&&rect.top<point.y&&bottom>point.y;}/**
	   * Check if given value is acceptable as a label text.
	   *
	   * @param {*} text value to check; can be anything at this point
	   * @returns {boolean} true if valid label value, false otherwise
	   */},{key:"isValidLabel",value:function isValidLabel(text){// Note that this is quite strict: types that *might* be converted to string are disallowed
	return typeof text==='string'&&text!=='';}/**
	   * Returns x, y of self reference circle based on provided angle
	   *
	   * @param {Object} ctx
	   * @param {number} angle 
	   * @param {number} radius 
	   * @return {Object} node
	   * @returns {Object} x and y coordinates
	   */},{key:"getSelfRefCoordinates",value:function getSelfRefCoordinates(ctx,angle,radius,node){var x=node.x;var y=node.y;if(typeof node.distanceToBorder==="function"){//calculating opposite and adjacent
	//distaneToBorder becomes Hypotenuse. 
	//Formulas sin(a) = Opposite / Hypotenuse and cos(a) = Adjacent / Hypotenuse
	var toBorderDist=node.distanceToBorder(ctx,angle);var yFromNodeCenter=Math.sin(angle)*toBorderDist;var xFromNodeCenter=Math.cos(angle)*toBorderDist;//xFromNodeCenter is basically x and if xFromNodeCenter equals to the distance to border then it means
	//that y does not need calculation because it is equal node.height / 2 or node.y
	//same thing with yFromNodeCenter and if yFromNodeCenter equals to the distance to border then it means
	//that x is equal node.width / 2 or node.x
	if(xFromNodeCenter===toBorderDist){x+=toBorderDist;y=node.y;}else if(yFromNodeCenter===toBorderDist){x=node.x;y-=toBorderDist;}else {x+=xFromNodeCenter;y-=yFromNodeCenter;}}else if(node.shape.width>node.shape.height){x=node.x+node.shape.width*0.5;y=node.y-radius;}else {x=node.x+radius;y=node.y-node.shape.height*0.5;}return {x:x,y:y};}}]);return ComponentUtil;}();/**
	 * Callback to determine text dimensions, using the parent label settings.
	 * @callback MeasureText
	 * @param {text} text
	 * @param {text} mod
	 * @return {Object} { width, values} width in pixels and font attributes
	 */ /**
	 * Helper class for Label which collects results of splitting labels into lines and blocks.
	 *
	 * @private
	 */var LabelAccumulator=/*#__PURE__*/function(){/**
	   * @param {MeasureText} measureText
	   */function LabelAccumulator(measureText){classCallCheck(this,LabelAccumulator);this.measureText=measureText;this.current=0;this.width=0;this.height=0;this.lines=[];}/**
	   * Append given text to the given line.
	   *
	   * @param {number}  l    index of line to add to
	   * @param {string}  text string to append to line
	   * @param {'bold'|'ital'|'boldital'|'mono'|'normal'} [mod='normal']
	   * @private
	   */createClass(LabelAccumulator,[{key:"_add",value:function _add(l,text){var mod=arguments.length>2&&arguments[2]!==undefined?arguments[2]:'normal';if(this.lines[l]===undefined){this.lines[l]={width:0,height:0,blocks:[]};}// We still need to set a block for undefined and empty texts, hence return at this point
	// This is necessary because we don't know at this point if we're at the
	// start of an empty line or not.
	// To compensate, empty blocks are removed in `finalize()`.
	//
	// Empty strings should still have a height
	var tmpText=text;if(text===undefined||text==="")tmpText=" ";// Determine width and get the font properties
	var result=this.measureText(tmpText,mod);var block=assign$2({},values$2(result));block.text=text;block.width=result.width;block.mod=mod;if(text===undefined||text===""){block.width=0;}this.lines[l].blocks.push(block);// Update the line width. We need this for determining if a string goes over max width
	this.lines[l].width+=block.width;}/**
	   * Returns the width in pixels of the current line.
	   *
	   * @returns {number}
	   */},{key:"curWidth",value:function curWidth(){var line=this.lines[this.current];if(line===undefined)return 0;return line.width;}/**
	    * Add text in block to current line
	    *
	    * @param {string} text
	    * @param {'bold'|'ital'|'boldital'|'mono'|'normal'} [mod='normal']
	    */},{key:"append",value:function append(text){var mod=arguments.length>1&&arguments[1]!==undefined?arguments[1]:'normal';this._add(this.current,text,mod);}/**
	   * Add text in block to current line and start a new line
	   *
	   * @param {string} text
	   * @param {'bold'|'ital'|'boldital'|'mono'|'normal'} [mod='normal']
	   */},{key:"newLine",value:function newLine(text){var mod=arguments.length>1&&arguments[1]!==undefined?arguments[1]:'normal';this._add(this.current,text,mod);this.current++;}/**
	   * Determine and set the heights of all the lines currently contained in this instance
	   *
	   * Note that width has already been set.
	   * 
	   * @private
	   */},{key:"determineLineHeights",value:function determineLineHeights(){for(var k=0;k<this.lines.length;k++){var line=this.lines[k];// Looking for max height of blocks in line
	var height=0;if(line.blocks!==undefined){// Can happen if text contains e.g. '\n '
	for(var l=0;l<line.blocks.length;l++){var block=line.blocks[l];if(height<block.height){height=block.height;}}}line.height=height;}}/**
	   * Determine the full size of the label text, as determined by current lines and blocks
	   * 
	   * @private
	   */},{key:"determineLabelSize",value:function determineLabelSize(){var width=0;var height=0;for(var k=0;k<this.lines.length;k++){var line=this.lines[k];if(line.width>width){width=line.width;}height+=line.height;}this.width=width;this.height=height;}/**
	   * Remove all empty blocks and empty lines we don't need
	   * 
	   * This must be done after the width/height determination,
	   * so that these are set properly for processing here.
	   *
	   * @returns {Array<Line>} Lines with empty blocks (and some empty lines) removed
	   * @private
	   */},{key:"removeEmptyBlocks",value:function removeEmptyBlocks(){var tmpLines=[];for(var k=0;k<this.lines.length;k++){var line=this.lines[k];// Note: an empty line in between text has width zero but is still relevant to layout.
	// So we can't use width for testing empty line here
	if(line.blocks.length===0)continue;// Discard final empty line always
	if(k===this.lines.length-1){if(line.width===0)continue;}var tmpLine={};assign$2(tmpLine,line);tmpLine.blocks=[];var firstEmptyBlock=void 0;var tmpBlocks=[];for(var l=0;l<line.blocks.length;l++){var block=line.blocks[l];if(block.width!==0){tmpBlocks.push(block);}else {if(firstEmptyBlock===undefined){firstEmptyBlock=block;}}}// Ensure that there is *some* text present
	if(tmpBlocks.length===0&&firstEmptyBlock!==undefined){tmpBlocks.push(firstEmptyBlock);}tmpLine.blocks=tmpBlocks;tmpLines.push(tmpLine);}return tmpLines;}/**
	   * Set the sizes for all lines and the whole thing.
	   *
	   * @returns {{width: (number|*), height: (number|*), lines: Array}}
	   */},{key:"finalize",value:function finalize(){//console.log(JSON.stringify(this.lines, null, 2));
	this.determineLineHeights();this.determineLabelSize();var tmpLines=this.removeEmptyBlocks();// Return a simple hash object for further processing.
	return {width:this.width,height:this.height,lines:tmpLines};}}]);return LabelAccumulator;}();// Hash of prepared regexp's for tags
	var tagPattern={// HTML
	'<b>':/<b>/,'<i>':/<i>/,'<code>':/<code>/,'</b>':/<\/b>/,'</i>':/<\/i>/,'</code>':/<\/code>/,// Markdown
	'*':/\*/,// bold
	'_':/\_/,// ital
	'`':/`/,// mono
	'afterBold':/[^\*]/,'afterItal':/[^_]/,'afterMono':/[^`]/};/**
	 * Internal helper class for parsing the markup tags for HTML and Markdown.
	 *
	 * NOTE: Sequences of tabs and spaces are reduced to single space.
	 *       Scan usage of `this.spacing` within method
	 */var MarkupAccumulator=/*#__PURE__*/function(){/**
	   * Create an instance
	   *
	   * @param {string} text  text to parse for markup
	   */function MarkupAccumulator(text){classCallCheck(this,MarkupAccumulator);this.text=text;this.bold=false;this.ital=false;this.mono=false;this.spacing=false;this.position=0;this.buffer="";this.modStack=[];this.blocks=[];}/**
	   * Return the mod label currently on the top of the stack
	   *
	   * @returns {string}  label of topmost mod 
	   * @private
	   */createClass(MarkupAccumulator,[{key:"mod",value:function mod(){return this.modStack.length===0?'normal':this.modStack[0];}/**
	   * Return the mod label currently active
	   * 
	   * @returns {string}  label of active mod 
	   * @private
	   */},{key:"modName",value:function modName(){if(this.modStack.length===0)return 'normal';else if(this.modStack[0]==='mono')return 'mono';else {if(this.bold&&this.ital){return 'boldital';}else if(this.bold){return 'bold';}else if(this.ital){return 'ital';}}}/**
	   * @private
	   */},{key:"emitBlock",value:function emitBlock(){if(this.spacing){this.add(" ");this.spacing=false;}if(this.buffer.length>0){this.blocks.push({text:this.buffer,mod:this.modName()});this.buffer="";}}/**
	   * Output text to buffer
	   *
	   * @param {string} text  text to add
	   * @private
	   */},{key:"add",value:function add(text){if(text===" "){this.spacing=true;}if(this.spacing){this.buffer+=" ";this.spacing=false;}if(text!=" "){this.buffer+=text;}}/**
	   * Handle parsing of whitespace
	   *
	   * @param {string} ch  the character to check
	   * @returns {boolean} true if the character was processed as whitespace, false otherwise
	   */},{key:"parseWS",value:function parseWS(ch){if(/[ \t]/.test(ch)){if(!this.mono){this.spacing=true;}else {this.add(ch);}return true;}return false;}/**
	   * @param {string} tagName  label for block type to set
	   * @private
	   */},{key:"setTag",value:function setTag(tagName){this.emitBlock();this[tagName]=true;this.modStack.unshift(tagName);}/**
	   * @param {string} tagName  label for block type to unset
	   * @private
	   */},{key:"unsetTag",value:function unsetTag(tagName){this.emitBlock();this[tagName]=false;this.modStack.shift();}/**
	   * @param {string} tagName label for block type we are currently processing
	   * @param {string|RegExp} tag string to match in text
	   * @returns {boolean} true if the tag was processed, false otherwise
	   */},{key:"parseStartTag",value:function parseStartTag(tagName,tag){// Note: if 'mono' passed as tagName, there is a double check here. This is OK
	if(!this.mono&&!this[tagName]&&this.match(tag)){this.setTag(tagName);return true;}return false;}/**
	   * @param {string|RegExp} tag
	   * @param {number} [advance=true] if set, advance current position in text
	   * @returns {boolean} true if match at given position, false otherwise
	   * @private
	   */},{key:"match",value:function match(tag){var advance=arguments.length>1&&arguments[1]!==undefined?arguments[1]:true;var _this$prepareRegExp=this.prepareRegExp(tag),_this$prepareRegExp2=slicedToArray(_this$prepareRegExp,2),regExp=_this$prepareRegExp2[0],length=_this$prepareRegExp2[1];var matched=regExp.test(this.text.substr(this.position,length));if(matched&&advance){this.position+=length-1;}return matched;}/**
	   * @param {string} tagName label for block type we are currently processing
	   * @param {string|RegExp} tag string to match in text
	   * @param {RegExp} [nextTag] regular expression to match for characters *following* the current tag 
	   * @returns {boolean} true if the tag was processed, false otherwise
	   */},{key:"parseEndTag",value:function parseEndTag(tagName,tag,nextTag){var checkTag=this.mod()===tagName;if(tagName==='mono'){// special handling for 'mono'
	checkTag=checkTag&&this.mono;}else {checkTag=checkTag&&!this.mono;}if(checkTag&&this.match(tag)){if(nextTag!==undefined){// Purpose of the following match is to prevent a direct unset/set of a given tag
	// E.g. '*bold **still bold*' => '*bold still bold*'
	if(this.position===this.text.length-1||this.match(nextTag,false)){this.unsetTag(tagName);}}else {this.unsetTag(tagName);}return true;}return false;}/**
	   * @param {string|RegExp} tag  string to match in text
	   * @param {value} value  string to replace tag with, if found at current position
	   * @returns {boolean} true if the tag was processed, false otherwise
	   */},{key:"replace",value:function replace(tag,value){if(this.match(tag)){this.add(value);this.position+=length-1;return true;}return false;}/**
	   * Create a regular expression for the tag if it isn't already one.
	   *
	   * The return value is an array `[RegExp, number]`, with exactly two value, where:
	   *  - RegExp is the regular expression to use
	   *  - number is the lenth of the input string to match
	   *
	   * @param {string|RegExp} tag  string to match in text
	   * @returns {Array}  regular expression to use and length of input string to match
	   * @private
	   */},{key:"prepareRegExp",value:function prepareRegExp(tag){var length;var regExp;if(tag instanceof RegExp){regExp=tag;length=1;// ASSUMPTION: regexp only tests one character
	}else {// use prepared regexp if present
	var prepared=tagPattern[tag];if(prepared!==undefined){regExp=prepared;}else {regExp=new RegExp(tag);}length=tag.length;}return [regExp,length];}}]);return MarkupAccumulator;}();/**
	 * Helper class for Label which explodes the label text into lines and blocks within lines
	 *
	 * @private
	 */var LabelSplitter=/*#__PURE__*/function(){/**
	   * @param {CanvasRenderingContext2D} ctx Canvas rendering context
	   * @param {Label} parent reference to the Label instance using current instance
	   * @param {boolean} selected 
	   * @param {boolean} hover
	   */function LabelSplitter(ctx,parent,selected,hover){var _this3=this;classCallCheck(this,LabelSplitter);this.ctx=ctx;this.parent=parent;this.selected=selected;this.hover=hover;/**
	     * Callback to determine text width; passed to LabelAccumulator instance
	     *
	     * @param  {String} text string to determine width of
	     * @param  {String} mod  font type to use for this text
	     * @return {Object} { width, values} width in pixels and font attributes
	     */var textWidth=function textWidth(text,mod){if(text===undefined)return 0;// TODO: This can be done more efficiently with caching
	// This will set the ctx.font correctly, depending on selected/hover and mod - so that ctx.measureText() will be accurate.
	var values=_this3.parent.getFormattingValues(ctx,selected,hover,mod);var width=0;if(text!==''){var measure=_this3.ctx.measureText(text);width=measure.width;}return {width:width,values:values};};this.lines=new LabelAccumulator(textWidth);}/**
	   * Split passed text of a label into lines and blocks.
	   *
	   * # NOTE
	   *
	   * The handling of spacing is option dependent:
	   *
	   * - if `font.multi : false`, all spaces are retained
	   * - if `font.multi : true`, every sequence of spaces is compressed to a single space
	   *
	   * This might not be the best way to do it, but this is as it has been working till now.
	   * In order not to break existing functionality, for the time being this behaviour will
	   * be retained in any code changes. 
	   *
	   * @param {string} text  text to split
	   * @returns {Array<line>}
	   */createClass(LabelSplitter,[{key:"process",value:function process(text){if(!ComponentUtil.isValidLabel(text)){return this.lines.finalize();}var font=this.parent.fontOptions;// Normalize the end-of-line's to a single representation - order important
	text=text.replace(/\r\n/g,'\n');// Dos EOL's
	text=text.replace(/\r/g,'\n');// Mac EOL's
	// Note that at this point, there can be no \r's in the text.
	// This is used later on splitStringIntoLines() to split multifont texts.
	var nlLines=String(text).split('\n');var lineCount=nlLines.length;if(font.multi){// Multi-font case: styling tags active
	for(var i=0;i<lineCount;i++){var blocks=this.splitBlocks(nlLines[i],font.multi);// Post: Sequences of tabs and spaces are reduced to single space
	if(blocks===undefined)continue;if(blocks.length===0){this.lines.newLine("");continue;}if(font.maxWdt>0){// widthConstraint.maximum defined
	//console.log('Running widthConstraint multi, max: ' + this.fontOptions.maxWdt);
	for(var j=0;j<blocks.length;j++){var mod=blocks[j].mod;var _text=blocks[j].text;this.splitStringIntoLines(_text,mod,true);}}else {// widthConstraint.maximum NOT defined
	for(var _j=0;_j<blocks.length;_j++){var _mod=blocks[_j].mod;var _text2=blocks[_j].text;this.lines.append(_text2,_mod);}}this.lines.newLine();}}else {// Single-font case
	if(font.maxWdt>0){// widthConstraint.maximum defined
	// console.log('Running widthConstraint normal, max: ' + this.fontOptions.maxWdt);
	for(var _i=0;_i<lineCount;_i++){this.splitStringIntoLines(nlLines[_i]);}}else {// widthConstraint.maximum NOT defined
	for(var _i2=0;_i2<lineCount;_i2++){this.lines.newLine(nlLines[_i2]);}}}return this.lines.finalize();}/**
	   * normalize the markup system
	   *
	   * @param {boolean|'md'|'markdown'|'html'} markupSystem
	   * @returns {string}
	   */},{key:"decodeMarkupSystem",value:function decodeMarkupSystem(markupSystem){var system='none';if(markupSystem==='markdown'||markupSystem==='md'){system='markdown';}else if(markupSystem===true||markupSystem==='html'){system='html';}return system;}/**
	   *
	   * @param {string} text
	   * @returns {Array}
	   */},{key:"splitHtmlBlocks",value:function splitHtmlBlocks(text){var s=new MarkupAccumulator(text);var parseEntities=function parseEntities(ch){if(/&/.test(ch)){var parsed=s.replace(s.text,'&lt;','<')||s.replace(s.text,'&amp;','&');if(!parsed){s.add("&");}return true;}return false;};while(s.position<s.text.length){var ch=s.text.charAt(s.position);var parsed=s.parseWS(ch)||/</.test(ch)&&(s.parseStartTag('bold','<b>')||s.parseStartTag('ital','<i>')||s.parseStartTag('mono','<code>')||s.parseEndTag('bold','</b>')||s.parseEndTag('ital','</i>')||s.parseEndTag('mono','</code>'))||parseEntities(ch);if(!parsed){s.add(ch);}s.position++;}s.emitBlock();return s.blocks;}/**
	   *
	   * @param {string} text
	   * @returns {Array}
	   */},{key:"splitMarkdownBlocks",value:function splitMarkdownBlocks(text){var _this4=this;var s=new MarkupAccumulator(text);var beginable=true;var parseOverride=function parseOverride(ch){if(/\\/.test(ch)){if(s.position<_this4.text.length+1){s.position++;ch=_this4.text.charAt(s.position);if(/ \t/.test(ch)){s.spacing=true;}else {s.add(ch);beginable=false;}}return true;}return false;};while(s.position<s.text.length){var ch=s.text.charAt(s.position);var parsed=s.parseWS(ch)||parseOverride(ch)||(beginable||s.spacing)&&(s.parseStartTag('bold','*')||s.parseStartTag('ital','_')||s.parseStartTag('mono','`'))||s.parseEndTag('bold','*','afterBold')||s.parseEndTag('ital','_','afterItal')||s.parseEndTag('mono','`','afterMono');if(!parsed){s.add(ch);beginable=false;}s.position++;}s.emitBlock();return s.blocks;}/**
	   * Explodes a piece of text into single-font blocks using a given markup
	   *
	   * @param {string} text
	   * @param {boolean|'md'|'markdown'|'html'} markupSystem
	   * @returns {Array.<{text: string, mod: string}>}
	   * @private
	   */},{key:"splitBlocks",value:function splitBlocks(text,markupSystem){var system=this.decodeMarkupSystem(markupSystem);if(system==='none'){return [{text:text,mod:'normal'}];}else if(system==='markdown'){return this.splitMarkdownBlocks(text);}else if(system==='html'){return this.splitHtmlBlocks(text);}}/**
	   * @param {string} text
	   * @returns {boolean} true if text length over the current max with
	   * @private
	   */},{key:"overMaxWidth",value:function overMaxWidth(text){var width=this.ctx.measureText(text).width;return this.lines.curWidth()+width>this.parent.fontOptions.maxWdt;}/**
	   * Determine the longest part of the sentence which still fits in the 
	   * current max width.
	   * 
	   * @param {Array} words  Array of strings signifying a text lines
	   * @return {number}      index of first item in string making string go over max
	   * @private
	   */},{key:"getLongestFit",value:function getLongestFit(words){var text='';var w=0;while(w<words.length){var pre=text===''?'':' ';var newText=text+pre+words[w];if(this.overMaxWidth(newText))break;text=newText;w++;}return w;}/**
	   * Determine the longest part of the string which still fits in the
	   * current max width.
	   * 
	   * @param {Array} words Array of strings signifying a text lines
	   * @return {number} index of first item in string making string go over max
	   */},{key:"getLongestFitWord",value:function getLongestFitWord(words){var w=0;while(w<words.length){if(this.overMaxWidth(slice$3(words).call(words,0,w)))break;w++;}return w;}/**
	   * Split the passed text into lines, according to width constraint (if any).
	   * 
	   * The method assumes that the input string is a single line, i.e. without lines break.
	   *
	   * This method retains spaces, if still present (case `font.multi: false`).
	   * A space which falls on an internal line break, will be replaced by a newline.
	   * There is no special handling of tabs; these go along with the flow.
	   * 
	   * @param {string} str
	   * @param {string} [mod='normal']
	   * @param {boolean} [appendLast=false]
	   * @private
	   */},{key:"splitStringIntoLines",value:function splitStringIntoLines(str){var mod=arguments.length>1&&arguments[1]!==undefined?arguments[1]:'normal';var appendLast=arguments.length>2&&arguments[2]!==undefined?arguments[2]:false;// Set the canvas context font, based upon the current selected/hover state
	// and the provided mod, so the text measurement performed by getLongestFit
	// will be accurate - and not just use the font of whoever last used the canvas.
	this.parent.getFormattingValues(this.ctx,this.selected,this.hover,mod);// Still-present spaces are relevant, retain them
	str=str.replace(/^( +)/g,'$1\r');str=str.replace(/([^\r][^ ]*)( +)/g,'$1\r$2\r');var words=str.split('\r');while(words.length>0){var w=this.getLongestFit(words);if(w===0){// Special case: the first word is already larger than the max width.
	var word=words[0];// Break the word to the largest part that fits the line
	var x=this.getLongestFitWord(word);this.lines.newLine(slice$3(word).call(word,0,x),mod);// Adjust the word, so that the rest will be done next iteration
	words[0]=slice$3(word).call(word,x);}else {// skip any space that is replaced by a newline
	var newW=w;if(words[w-1]===' '){w--;}else if(words[newW]===' '){newW++;}var text=slice$3(words).call(words,0,w).join("");if(w==words.length&&appendLast){this.lines.append(text,mod);}else {this.lines.newLine(text,mod);}// Adjust the word, so that the rest will be done next iteration
	words=slice$3(words).call(words,newW);}}}}]);return LabelSplitter;}();/**
	 * List of special styles for multi-fonts
	 * @private
	 */var multiFontStyle=['bold','ital','boldital','mono'];/**
	 * A Label to be used for Nodes or Edges.
	 */var Label=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   * @param {Object} options
	   * @param {boolean} [edgelabel=false]
	   */function Label(body,options){var edgelabel=arguments.length>2&&arguments[2]!==undefined?arguments[2]:false;classCallCheck(this,Label);this.body=body;this.pointToSelf=false;this.baseSize=undefined;this.fontOptions={};// instance variable containing the *instance-local* font options
	this.setOptions(options);this.size={top:0,left:0,width:0,height:0,yLine:0};this.isEdgeLabel=edgelabel;}/**
	   * @param {Object} options the options of the parent Node-instance
	   */createClass(Label,[{key:"setOptions",value:function setOptions(options){this.elementOptions=options;// Reference to the options of the parent Node-instance 
	this.initFontOptions(options.font);if(ComponentUtil.isValidLabel(options.label)){this.labelDirty=true;}else {// Bad label! Change the option value to prevent bad stuff happening
	options.label=undefined;}if(options.font!==undefined&&options.font!==null){// font options can be deleted at various levels
	if(typeof options.font==='string'){this.baseSize=this.fontOptions.size;}else if(_typeof_1(options.font)==='object'){var size=options.font.size;if(size!==undefined){this.baseSize=size;}}}}/**
	   * Init the font Options structure.
	   *
	   * Member fontOptions serves as an accumulator for the current font options.
	   * As such, it needs to be completely separated from the node options.
	   *
	   * @param {Object} newFontOptions the new font options to process
	   * @private
	   */},{key:"initFontOptions",value:function initFontOptions(newFontOptions){var _this5=this;// Prepare the multi-font option objects.
	// These will be filled in propagateFonts(), if required
	forEach$4(multiFontStyle,function(style){_this5.fontOptions[style]={};});// Handle shorthand option, if present
	if(Label.parseFontString(this.fontOptions,newFontOptions)){this.fontOptions.vadjust=0;return;}// Copy over the non-multifont options, if specified
	forEach$4(newFontOptions,function(prop,n){if(prop!==undefined&&prop!==null&&_typeof_1(prop)!=='object'){_this5.fontOptions[n]=prop;}});}/**
	   * If in-variable is a string, parse it as a font specifier.
	   *
	   * Note that following is not done here and have to be done after the call:
	   * - Not all font options are set (vadjust, mod)
	   *
	   * @param {Object} outOptions  out-parameter, object in which to store the parse results (if any)
	   * @param {Object} inOptions  font options to parse
	   * @return {boolean} true if font parsed as string, false otherwise
	   * @static
	   */},{key:"constrain",/**
	   * Set the width and height constraints based on 'nearest' value
	   *
	   * @param {Array} pile array of option objects to consider
	   * @returns {object} the actual constraint values to use
	   * @private
	   */value:function constrain(pile){// NOTE: constrainWidth and  constrainHeight never set!
	// NOTE: for edge labels, only 'maxWdt' set
	// Node labels can set all the fields
	var fontOptions={constrainWidth:false,maxWdt:-1,minWdt:-1,constrainHeight:false,minHgt:-1,valign:'middle'};var widthConstraint=topMost(pile,'widthConstraint');if(typeof widthConstraint==='number'){fontOptions.maxWdt=Number(widthConstraint);fontOptions.minWdt=Number(widthConstraint);}else if(_typeof_1(widthConstraint)==='object'){var widthConstraintMaximum=topMost(pile,['widthConstraint','maximum']);if(typeof widthConstraintMaximum==='number'){fontOptions.maxWdt=Number(widthConstraintMaximum);}var widthConstraintMinimum=topMost(pile,['widthConstraint','minimum']);if(typeof widthConstraintMinimum==='number'){fontOptions.minWdt=Number(widthConstraintMinimum);}}var heightConstraint=topMost(pile,'heightConstraint');if(typeof heightConstraint==='number'){fontOptions.minHgt=Number(heightConstraint);}else if(_typeof_1(heightConstraint)==='object'){var heightConstraintMinimum=topMost(pile,['heightConstraint','minimum']);if(typeof heightConstraintMinimum==='number'){fontOptions.minHgt=Number(heightConstraintMinimum);}var heightConstraintValign=topMost(pile,['heightConstraint','valign']);if(typeof heightConstraintValign==='string'){if(heightConstraintValign==='top'||heightConstraintValign==='bottom'){fontOptions.valign=heightConstraintValign;}}}return fontOptions;}/**
	   * Set options and update internal state
	   *
	   * @param {Object} options  options to set
	   * @param {Array}  pile     array of option objects to consider for option 'chosen'
	   */},{key:"update",value:function update(options,pile){this.setOptions(options,true);this.propagateFonts(pile);deepExtend(this.fontOptions,this.constrain(pile));this.fontOptions.chooser=ComponentUtil.choosify('label',pile);}/**
	   * When margins are set in an element, adjust sizes is called to remove them
	   * from the width/height constraints. This must be done prior to label sizing.
	   *
	   * @param {{top: number, right: number, bottom: number, left: number}} margins
	   */},{key:"adjustSizes",value:function adjustSizes(margins){var widthBias=margins?margins.right+margins.left:0;if(this.fontOptions.constrainWidth){this.fontOptions.maxWdt-=widthBias;this.fontOptions.minWdt-=widthBias;}var heightBias=margins?margins.top+margins.bottom:0;if(this.fontOptions.constrainHeight){this.fontOptions.minHgt-=heightBias;}}/////////////////////////////////////////////////////////
	// Methods for handling options piles
	// Eventually, these will be moved to a separate class
	/////////////////////////////////////////////////////////
	/**
	   * Add the font members of the passed list of option objects to the pile.
	   *
	   * @param {Pile} dstPile  pile of option objects add to 
	   * @param {Pile} srcPile  pile of option objects to take font options from
	   * @private
	   */},{key:"addFontOptionsToPile",value:function addFontOptionsToPile(dstPile,srcPile){for(var i=0;i<srcPile.length;++i){this.addFontToPile(dstPile,srcPile[i]);}}/**
	   * Add given font option object to the list of objects (the 'pile') to consider for determining
	   * multi-font option values.
	   *
	   * @param {Pile} pile  pile of option objects to use
	   * @param {object} options  instance to add to pile
	   * @private
	   */},{key:"addFontToPile",value:function addFontToPile(pile,options){if(options===undefined)return;if(options.font===undefined||options.font===null)return;var item=options.font;pile.push(item);}/**
	   * Collect all own-property values from the font pile that aren't multi-font option objectss.
	   *
	   * @param {Pile} pile  pile of option objects to use
	   * @returns {object} object with all current own basic font properties
	   * @private
	   */},{key:"getBasicOptions",value:function getBasicOptions(pile){var ret={};// Scans the whole pile to get all options present
	for(var n=0;n<pile.length;++n){var fontOptions=pile[n];// Convert shorthand if necessary
	var tmpShorthand={};if(Label.parseFontString(tmpShorthand,fontOptions)){fontOptions=tmpShorthand;}forEach$4(fontOptions,function(opt,name){if(opt===undefined)return;// multi-font option need not be present 
	if(ret.hasOwnProperty(name))return;// Keep first value we encounter
	if(indexOf$3(multiFontStyle).call(multiFontStyle,name)!==-1){// Skip multi-font properties but we do need the structure
	ret[name]={};}else {ret[name]=opt;}});}return ret;}/**
	   * Return the value for given option for the given multi-font.
	   *
	   * All available option objects are trawled in the set order to construct the option values.
	   *
	   * ---------------------------------------------------------------------
	   * ## Traversal of pile for multi-fonts
	   *
	   * The determination of multi-font option values is a special case, because any values not
	   * present in the multi-font options should by definition be taken from the main font options,
	   * i.e. from the current 'parent' object of the multi-font option.
	   *
	   * ### Search order for multi-fonts
	   *
	   * 'bold' used as example:
	   *
	   *   - search in option group 'bold' in local properties
	   *   - search in main font option group in local properties
	   *
	   * ---------------------------------------------------------------------
	   *
	   * @param {Pile} pile  pile of option objects to use
	   * @param {MultiFontStyle} multiName sub path for the multi-font
	   * @param {string} option  the option to search for, for the given multi-font
	   * @returns {string|number} the value for the given option
	   * @private
	   */},{key:"getFontOption",value:function getFontOption(pile,multiName,option){var multiFont;// Search multi font in local properties
	for(var n=0;n<pile.length;++n){var fontOptions=pile[n];if(fontOptions.hasOwnProperty(multiName)){multiFont=fontOptions[multiName];if(multiFont===undefined||multiFont===null)continue;// Convert shorthand if necessary
	// TODO: inefficient to do this conversion every time; find a better way.
	var tmpShorthand={};if(Label.parseFontString(tmpShorthand,multiFont)){multiFont=tmpShorthand;}if(multiFont.hasOwnProperty(option)){return multiFont[option];}}}// Option is not mentioned in the multi font options; take it from the parent font options.
	// These have already been converted with getBasicOptions(), so use the converted values.
	if(this.fontOptions.hasOwnProperty(option)){return this.fontOptions[option];}// A value **must** be found; you should never get here.
	throw new Error("Did not find value for multi-font for property: '"+option+"'");}/**
	   * Return all options values for the given multi-font.
	   *
	   * All available option objects are trawled in the set order to construct the option values.
	   *
	   * @param {Pile} pile  pile of option objects to use
	   * @param {MultiFontStyle} multiName sub path for the mod-font
	   * @returns {MultiFontOptions}
	   * @private
	   */},{key:"getFontOptions",value:function getFontOptions(pile,multiName){var result={};var optionNames=['color','size','face','mod','vadjust'];// List of allowed options per multi-font
	for(var i=0;i<optionNames.length;++i){var mod=optionNames[i];result[mod]=this.getFontOption(pile,multiName,mod);}return result;}/////////////////////////////////////////////////////////
	// End methods for handling options piles
	/////////////////////////////////////////////////////////
	/**
	   * Collapse the font options for the multi-font to single objects, from
	   * the chain of option objects passed (the 'pile').
	   *
	   * @param {Pile} pile  sequence of option objects to consider.
	   *                     First item in list assumed to be the newly set options.
	   */},{key:"propagateFonts",value:function propagateFonts(pile){var _this6=this;var fontPile=[];// sequence of font objects to consider, order important
	// Note that this.elementOptions is not used here.
	this.addFontOptionsToPile(fontPile,pile);this.fontOptions=this.getBasicOptions(fontPile);// We set multifont values even if multi === false, for consistency (things break otherwise)
	var _loop=function _loop(i){var mod=multiFontStyle[i];var modOptions=_this6.fontOptions[mod];var tmpMultiFontOptions=_this6.getFontOptions(fontPile,mod);// Copy over found values
	forEach$4(tmpMultiFontOptions,function(option,n){modOptions[n]=option;});modOptions.size=Number(modOptions.size);modOptions.vadjust=Number(modOptions.vadjust);};for(var i=0;i<multiFontStyle.length;++i){_loop(i);}}/**
	   * Main function. This is called from anything that wants to draw a label.
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x
	   * @param {number} y
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {string} [baseline='middle']
	   */},{key:"draw",value:function draw(ctx,x,y,selected,hover){var baseline=arguments.length>5&&arguments[5]!==undefined?arguments[5]:'middle';// if no label, return
	if(this.elementOptions.label===undefined)return;// check if we have to render the label
	var viewFontSize=this.fontOptions.size*this.body.view.scale;if(this.elementOptions.label&&viewFontSize<this.elementOptions.scaling.label.drawThreshold-1)return;// This ensures that there will not be HUGE letters on screen
	// by setting an upper limit on the visible text size (regardless of zoomLevel)
	if(viewFontSize>=this.elementOptions.scaling.label.maxVisible){viewFontSize=Number(this.elementOptions.scaling.label.maxVisible)/this.body.view.scale;}// update the size cache if required
	this.calculateLabelSize(ctx,selected,hover,x,y,baseline);this._drawBackground(ctx);this._drawText(ctx,x,this.size.yLine,baseline,viewFontSize);}/**
	   * Draws the label background
	   * @param {CanvasRenderingContext2D} ctx
	   * @private
	   */},{key:"_drawBackground",value:function _drawBackground(ctx){if(this.fontOptions.background!==undefined&&this.fontOptions.background!=="none"){ctx.fillStyle=this.fontOptions.background;var size=this.getSize();ctx.fillRect(size.left,size.top,size.width,size.height);}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x
	   * @param {number} y
	   * @param {string} [baseline='middle']
	   * @param {number} viewFontSize 
	   * @private
	   */},{key:"_drawText",value:function _drawText(ctx,x,y){var baseline=arguments.length>3&&arguments[3]!==undefined?arguments[3]:'middle';var viewFontSize=arguments.length>4?arguments[4]:undefined;var _this$_setAlignment=this._setAlignment(ctx,x,y,baseline);var _this$_setAlignment2=slicedToArray(_this$_setAlignment,2);x=_this$_setAlignment2[0];y=_this$_setAlignment2[1];ctx.textAlign='left';x=x-this.size.width/2;// Shift label 1/2-distance to the left
	if(this.fontOptions.valign&&this.size.height>this.size.labelHeight){if(this.fontOptions.valign==='top'){y-=(this.size.height-this.size.labelHeight)/2;}if(this.fontOptions.valign==='bottom'){y+=(this.size.height-this.size.labelHeight)/2;}}// draw the text
	for(var i=0;i<this.lineCount;i++){var line=this.lines[i];if(line&&line.blocks){var width=0;if(this.isEdgeLabel||this.fontOptions.align==='center'){width+=(this.size.width-line.width)/2;}else if(this.fontOptions.align==='right'){width+=this.size.width-line.width;}for(var j=0;j<line.blocks.length;j++){var block=line.blocks[j];ctx.font=block.font;var _this$_getColor=this._getColor(block.color,viewFontSize,block.strokeColor),_this$_getColor2=slicedToArray(_this$_getColor,2),fontColor=_this$_getColor2[0],strokeColor=_this$_getColor2[1];if(block.strokeWidth>0){ctx.lineWidth=block.strokeWidth;ctx.strokeStyle=strokeColor;ctx.lineJoin='round';}ctx.fillStyle=fontColor;if(block.strokeWidth>0){ctx.strokeText(block.text,x+width,y+block.vadjust);}ctx.fillText(block.text,x+width,y+block.vadjust);width+=block.width;}y+=line.height;}}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x
	   * @param {number} y
	   * @param {string} baseline
	   * @returns {Array.<number>}
	   * @private
	   */},{key:"_setAlignment",value:function _setAlignment(ctx,x,y,baseline){// check for label alignment (for edges)
	// TODO: make alignment for nodes
	if(this.isEdgeLabel&&this.fontOptions.align!=='horizontal'&&this.pointToSelf===false){x=0;y=0;var lineMargin=2;if(this.fontOptions.align==='top'){ctx.textBaseline='alphabetic';y-=2*lineMargin;// distance from edge, required because we use alphabetic. Alphabetic has less difference between browsers
	}else if(this.fontOptions.align==='bottom'){ctx.textBaseline='hanging';y+=2*lineMargin;// distance from edge, required because we use hanging. Hanging has less difference between browsers
	}else {ctx.textBaseline='middle';}}else {ctx.textBaseline=baseline;}return [x,y];}/**
	   * fade in when relative scale is between threshold and threshold - 1.
	   * If the relative scale would be smaller than threshold -1 the draw function would have returned before coming here.
	   *
	   * @param {string} color  The font color to use
	   * @param {number} viewFontSize
	   * @param {string} initialStrokeColor
	   * @returns {Array.<string>} An array containing the font color and stroke color
	   * @private
	   */},{key:"_getColor",value:function _getColor(color,viewFontSize,initialStrokeColor){var fontColor=color||'#000000';var strokeColor=initialStrokeColor||'#ffffff';if(viewFontSize<=this.elementOptions.scaling.label.drawThreshold){var opacity=Math.max(0,Math.min(1,1-(this.elementOptions.scaling.label.drawThreshold-viewFontSize)));fontColor=overrideOpacity(fontColor,opacity);strokeColor=overrideOpacity(strokeColor,opacity);}return [fontColor,strokeColor];}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @returns {{width: number, height: number}}
	   */},{key:"getTextSize",value:function getTextSize(ctx){var selected=arguments.length>1&&arguments[1]!==undefined?arguments[1]:false;var hover=arguments.length>2&&arguments[2]!==undefined?arguments[2]:false;this._processLabel(ctx,selected,hover);return {width:this.size.width,height:this.size.height,lineCount:this.lineCount};}/**
	   * Get the current dimensions of the label
	   *
	   * @return {rect}
	   */},{key:"getSize",value:function getSize(){var lineMargin=2;var x=this.size.left;// default values which might be overridden below
	var y=this.size.top-0.5*lineMargin;// idem
	if(this.isEdgeLabel){var x2=-this.size.width*0.5;switch(this.fontOptions.align){case'middle':x=x2;y=-this.size.height*0.5;break;case'top':x=x2;y=-(this.size.height+lineMargin);break;case'bottom':x=x2;y=lineMargin;break;}}var ret={left:x,top:y,width:this.size.width,height:this.size.height};return ret;}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {number} [x=0]
	   * @param {number} [y=0]
	   * @param {'middle'|'hanging'} [baseline='middle']
	   */},{key:"calculateLabelSize",value:function calculateLabelSize(ctx,selected,hover){var x=arguments.length>3&&arguments[3]!==undefined?arguments[3]:0;var y=arguments.length>4&&arguments[4]!==undefined?arguments[4]:0;var baseline=arguments.length>5&&arguments[5]!==undefined?arguments[5]:'middle';this._processLabel(ctx,selected,hover);this.size.left=x-this.size.width*0.5;this.size.top=y-this.size.height*0.5;this.size.yLine=y+(1-this.lineCount)*0.5*this.fontOptions.size;if(baseline==="hanging"){this.size.top+=0.5*this.fontOptions.size;this.size.top+=4;// distance from node, required because we use hanging. Hanging has less difference between browsers
	this.size.yLine+=4;// distance from node
	}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {string} mod
	   * @returns {{color, size, face, mod, vadjust, strokeWidth: *, strokeColor: (*|string|allOptions.edges.font.strokeColor|{string}|allOptions.nodes.font.strokeColor|Array)}}
	   */},{key:"getFormattingValues",value:function getFormattingValues(ctx,selected,hover,mod){var getValue=function getValue(fontOptions,mod,option){if(mod==="normal"){if(option==='mod')return "";return fontOptions[option];}if(fontOptions[mod][option]!==undefined){// Grumbl leaving out test on undefined equals false for "" 
	return fontOptions[mod][option];}else {// Take from parent font option
	return fontOptions[option];}};var values={color:getValue(this.fontOptions,mod,'color'),size:getValue(this.fontOptions,mod,'size'),face:getValue(this.fontOptions,mod,'face'),mod:getValue(this.fontOptions,mod,'mod'),vadjust:getValue(this.fontOptions,mod,'vadjust'),strokeWidth:this.fontOptions.strokeWidth,strokeColor:this.fontOptions.strokeColor};if(selected||hover){if(mod==="normal"&&this.fontOptions.chooser===true&&this.elementOptions.labelHighlightBold){values.mod='bold';}else {if(typeof this.fontOptions.chooser==='function'){this.fontOptions.chooser(values,this.elementOptions.id,selected,hover);}}}var fontString="";if(values.mod!==undefined&&values.mod!==""){// safeguard for undefined - this happened
	fontString+=values.mod+" ";}fontString+=values.size+"px "+values.face;ctx.font=fontString.replace(/"/g,"");values.font=ctx.font;values.height=values.size;return values;}/**
	   *
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @returns {boolean}
	   */},{key:"differentState",value:function differentState(selected,hover){return selected!==this.selectedState||hover!==this.hoverState;}/**
	   * This explodes the passed text into lines and determines the width, height and number of lines.
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {string} inText  the text to explode
	   * @returns {{width, height, lines}|*}
	   * @private
	   */},{key:"_processLabelText",value:function _processLabelText(ctx,selected,hover,inText){var splitter=new LabelSplitter(ctx,this,selected,hover);return splitter.process(inText);}/**
	   * This explodes the label string into lines and sets the width, height and number of lines.
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @private
	   */},{key:"_processLabel",value:function _processLabel(ctx,selected,hover){if(this.labelDirty===false&&!this.differentState(selected,hover))return;var state=this._processLabelText(ctx,selected,hover,this.elementOptions.label);if(this.fontOptions.minWdt>0&&state.width<this.fontOptions.minWdt){state.width=this.fontOptions.minWdt;}this.size.labelHeight=state.height;if(this.fontOptions.minHgt>0&&state.height<this.fontOptions.minHgt){state.height=this.fontOptions.minHgt;}this.lines=state.lines;this.lineCount=state.lines.length;this.size.width=state.width;this.size.height=state.height;this.selectedState=selected;this.hoverState=hover;this.labelDirty=false;}/**
	   * Check if this label is visible
	   *
	   * @return {boolean} true if this label will be show, false otherwise
	   */},{key:"visible",value:function visible(){if(this.size.width===0||this.size.height===0||this.elementOptions.label===undefined){return false;// nothing to display
	}var viewFontSize=this.fontOptions.size*this.body.view.scale;if(viewFontSize<this.elementOptions.scaling.label.drawThreshold-1){return false;// Too small or too far away to show
	}return true;}}],[{key:"parseFontString",value:function parseFontString(outOptions,inOptions){if(!inOptions||typeof inOptions!=='string')return false;var newOptionsArray=inOptions.split(" ");outOptions.size=+newOptionsArray[0].replace("px",'');outOptions.face=newOptionsArray[1];outOptions.color=newOptionsArray[2];return true;}}]);return Label;}();/**
	 * The Base class for all Nodes.
	 */var NodeBase=/*#__PURE__*/function(){/**
	   * @param {Object} options
	   * @param {Object} body
	   * @param {Label} labelModule
	   */function NodeBase(options,body,labelModule){classCallCheck(this,NodeBase);this.body=body;this.labelModule=labelModule;this.setOptions(options);this.top=undefined;this.left=undefined;this.height=undefined;this.width=undefined;this.radius=undefined;this.margin=undefined;this.refreshNeeded=true;this.boundingBox={top:0,left:0,right:0,bottom:0};}/**
	   *
	   * @param {Object} options
	   */createClass(NodeBase,[{key:"setOptions",value:function setOptions(options){this.options=options;}/**
	   *
	   * @param {Label} labelModule
	   * @private
	   */},{key:"_setMargins",value:function _setMargins(labelModule){this.margin={};if(this.options.margin){if(_typeof_1(this.options.margin)=='object'){this.margin.top=this.options.margin.top;this.margin.right=this.options.margin.right;this.margin.bottom=this.options.margin.bottom;this.margin.left=this.options.margin.left;}else {this.margin.top=this.options.margin;this.margin.right=this.options.margin;this.margin.bottom=this.options.margin;this.margin.left=this.options.margin;}}labelModule.adjustSizes(this.margin);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} angle
	   * @returns {number}
	   * @private
	   */},{key:"_distanceToBorder",value:function _distanceToBorder(ctx,angle){var borderWidth=this.options.borderWidth;if(ctx){this.resize(ctx);}return Math.min(Math.abs(this.width/2/Math.cos(angle)),Math.abs(this.height/2/Math.sin(angle)))+borderWidth;}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {ArrowOptions} values
	   */},{key:"enableShadow",value:function enableShadow(ctx,values){if(values.shadow){ctx.shadowColor=values.shadowColor;ctx.shadowBlur=values.shadowSize;ctx.shadowOffsetX=values.shadowX;ctx.shadowOffsetY=values.shadowY;}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {ArrowOptions} values
	   */},{key:"disableShadow",value:function disableShadow(ctx,values){if(values.shadow){ctx.shadowColor='rgba(0,0,0,0)';ctx.shadowBlur=0;ctx.shadowOffsetX=0;ctx.shadowOffsetY=0;}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {ArrowOptions} values
	   */},{key:"enableBorderDashes",value:function enableBorderDashes(ctx,values){if(values.borderDashes!==false){if(ctx.setLineDash!==undefined){var dashes=values.borderDashes;if(dashes===true){dashes=[5,15];}ctx.setLineDash(dashes);}else {console.warn("setLineDash is not supported in this browser. The dashed borders cannot be used.");this.options.shapeProperties.borderDashes=false;values.borderDashes=false;}}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {ArrowOptions} values
	   */},{key:"disableBorderDashes",value:function disableBorderDashes(ctx,values){if(values.borderDashes!==false){if(ctx.setLineDash!==undefined){ctx.setLineDash([0]);}else {console.warn("setLineDash is not supported in this browser. The dashed borders cannot be used.");this.options.shapeProperties.borderDashes=false;values.borderDashes=false;}}}/**
	   * Determine if the shape of a node needs to be recalculated.
	   *
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @returns {boolean}
	   * @protected
	   */},{key:"needsRefresh",value:function needsRefresh(selected,hover){if(this.refreshNeeded===true){// This is probably not the best location to reset this member.
	// However, in the current logic, it is the most convenient one.
	this.refreshNeeded=false;return true;}return this.width===undefined||this.labelModule.differentState(selected,hover);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {ArrowOptions} values
	   */},{key:"initContextForDraw",value:function initContextForDraw(ctx,values){var borderWidth=values.borderWidth/this.body.view.scale;ctx.lineWidth=Math.min(this.width,borderWidth);ctx.strokeStyle=values.borderColor;ctx.fillStyle=values.color;}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {ArrowOptions} values
	   */},{key:"performStroke",value:function performStroke(ctx,values){var borderWidth=values.borderWidth/this.body.view.scale;//draw dashed border if enabled, save and restore is required for firefox not to crash on unix.
	ctx.save();// if borders are zero width, they will be drawn with width 1 by default. This prevents that
	if(borderWidth>0){this.enableBorderDashes(ctx,values);//draw the border
	ctx.stroke();//disable dashed border for other elements
	this.disableBorderDashes(ctx,values);}ctx.restore();}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {ArrowOptions} values
	   */},{key:"performFill",value:function performFill(ctx,values){ctx.save();ctx.fillStyle=values.color;// draw shadow if enabled
	this.enableShadow(ctx,values);// draw the background
	fill$2(ctx).call(ctx);// disable shadows for other elements.
	this.disableShadow(ctx,values);ctx.restore();this.performStroke(ctx,values);}/**
	   *
	   * @param {number} margin
	   * @private
	   */},{key:"_addBoundingBoxMargin",value:function _addBoundingBoxMargin(margin){this.boundingBox.left-=margin;this.boundingBox.top-=margin;this.boundingBox.bottom+=margin;this.boundingBox.right+=margin;}/**
	   * Actual implementation of this method call.
	   *
	   * Doing it like this makes it easier to override
	   * in the child classes.
	   *
	   * @param {number} x width
	   * @param {number} y height
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @private
	   */},{key:"_updateBoundingBox",value:function _updateBoundingBox(x,y,ctx,selected,hover){if(ctx!==undefined){this.resize(ctx,selected,hover);}this.left=x-this.width/2;this.top=y-this.height/2;this.boundingBox.left=this.left;this.boundingBox.top=this.top;this.boundingBox.bottom=this.top+this.height;this.boundingBox.right=this.left+this.width;}/**
	   * Default implementation of this method call.
	   * This acts as a stub which can be overridden.
	   *
	   * @param {number} x width
	   * @param {number} y height
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {boolean} selected
	   * @param {boolean} hover
	   */},{key:"updateBoundingBox",value:function updateBoundingBox(x,y,ctx,selected,hover){this._updateBoundingBox(x,y,ctx,selected,hover);}/**
	   * Determine the dimensions to use for nodes with an internal label
	   *
	   * Currently, these are: Circle, Ellipse, Database, Box
	   * The other nodes have external labels, and will not call this method
	   *
	   * If there is no label, decent default values are supplied.
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {boolean} [selected]
	   * @param {boolean} [hover]
	   * @returns {{width:number, height:number}}
	   */},{key:"getDimensionsFromLabel",value:function getDimensionsFromLabel(ctx,selected,hover){// NOTE: previously 'textSize' was not put in 'this' for Ellipse
	// TODO: examine the consequences.
	this.textSize=this.labelModule.getTextSize(ctx,selected,hover);var width=this.textSize.width;var height=this.textSize.height;var DEFAULT_SIZE=14;if(width===0){// This happens when there is no label text set
	width=DEFAULT_SIZE;// use a decent default
	height=DEFAULT_SIZE;// if width zero, then height also always zero
	}return {width:width,height:height};}}]);return NodeBase;}();/**
	 * A Box Node/Cluster shape.
	 *
	 * @extends NodeBase
	 */var Box=/*#__PURE__*/function(_NodeBase){inherits(Box,_NodeBase);var _super=_createSuper$1(Box);/**
	   * @param {Object} options
	   * @param {Object} body
	   * @param {Label} labelModule
	   */function Box(options,body,labelModule){var _this7;classCallCheck(this,Box);_this7=_super.call(this,options,body,labelModule);_this7._setMargins(labelModule);return _this7;}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {boolean} [selected]
	   * @param {boolean} [hover]
	   */createClass(Box,[{key:"resize",value:function resize(ctx){var selected=arguments.length>1&&arguments[1]!==undefined?arguments[1]:this.selected;var hover=arguments.length>2&&arguments[2]!==undefined?arguments[2]:this.hover;if(this.needsRefresh(selected,hover)){var dimensions=this.getDimensionsFromLabel(ctx,selected,hover);this.width=dimensions.width+this.margin.right+this.margin.left;this.height=dimensions.height+this.margin.top+this.margin.bottom;this.radius=this.width/2;}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x width
	   * @param {number} y height
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {ArrowOptions} values
	   */},{key:"draw",value:function draw(ctx,x,y,selected,hover,values){this.resize(ctx,selected,hover);this.left=x-this.width/2;this.top=y-this.height/2;this.initContextForDraw(ctx,values);drawRoundRect(ctx,this.left,this.top,this.width,this.height,values.borderRadius);this.performFill(ctx,values);this.updateBoundingBox(x,y,ctx,selected,hover);this.labelModule.draw(ctx,this.left+this.textSize.width/2+this.margin.left,this.top+this.textSize.height/2+this.margin.top,selected,hover);}/**
	   *
	   * @param {number} x width
	   * @param {number} y height
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {boolean} selected
	   * @param {boolean} hover
	   */},{key:"updateBoundingBox",value:function updateBoundingBox(x,y,ctx,selected,hover){this._updateBoundingBox(x,y,ctx,selected,hover);var borderRadius=this.options.shapeProperties.borderRadius;// only effective for box
	this._addBoundingBoxMargin(borderRadius);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} angle
	   * @returns {number}
	   */},{key:"distanceToBorder",value:function distanceToBorder(ctx,angle){if(ctx){this.resize(ctx);}var borderWidth=this.options.borderWidth;return Math.min(Math.abs(this.width/2/Math.cos(angle)),Math.abs(this.height/2/Math.sin(angle)))+borderWidth;}}]);return Box;}(NodeBase);/**
	 * NOTE: This is a bad base class
	 *
	 * Child classes are:
	 *
	 *   Image       - uses *only* image methods
	 *   Circle      - uses *only* _drawRawCircle
	 *   CircleImage - uses all
	 *
	 * TODO: Refactor, move _drawRawCircle to different module, derive Circle from NodeBase
	 *       Rename this to ImageBase
	 *       Consolidate common code in Image and CircleImage to base class
	 *
	 * @extends NodeBase
	 */var CircleImageBase=/*#__PURE__*/function(_NodeBase2){inherits(CircleImageBase,_NodeBase2);var _super2=_createSuper$1(CircleImageBase);/**
	   * @param {Object} options
	   * @param {Object} body
	   * @param {Label} labelModule
	   */function CircleImageBase(options,body,labelModule){var _this8;classCallCheck(this,CircleImageBase);_this8=_super2.call(this,options,body,labelModule);_this8.labelOffset=0;_this8.selected=false;return _this8;}/**
	   *
	   * @param {Object} options
	   * @param {Object} [imageObj]
	   * @param {Object} [imageObjAlt]
	   */createClass(CircleImageBase,[{key:"setOptions",value:function setOptions(options,imageObj,imageObjAlt){this.options=options;if(!(imageObj===undefined&&imageObjAlt===undefined)){this.setImages(imageObj,imageObjAlt);}}/**
	   * Set the images for this node.
	   *
	   * The images can be updated after the initial setting of options;
	   * therefore, this method needs to be reentrant. 
	   *
	   * For correct working in error cases, it is necessary to properly set
	   * field 'nodes.brokenImage' in the options.
	   *
	   * @param {Image} imageObj  required; main image to show for this node
	   * @param {Image|undefined} imageObjAlt optional; image to show when node is selected
	   */},{key:"setImages",value:function setImages(imageObj,imageObjAlt){if(imageObjAlt&&this.selected){this.imageObj=imageObjAlt;this.imageObjAlt=imageObj;}else {this.imageObj=imageObj;this.imageObjAlt=imageObjAlt;}}/**
	   * Set selection and switch between the base and the selected image.
	   *
	   * Do the switch only if imageObjAlt exists.
	   *
	   * @param {boolean} selected value of new selected state for current node
	   */},{key:"switchImages",value:function switchImages(selected){var selection_changed=selected&&!this.selected||!selected&&this.selected;this.selected=selected;// Remember new selection
	if(this.imageObjAlt!==undefined&&selection_changed){var imageTmp=this.imageObj;this.imageObj=this.imageObjAlt;this.imageObjAlt=imageTmp;}}/**
	   * Returns Image Padding from node options
	   *
	   * @returns {{top: number,left: number,bottom: number,right: number}} image padding inside this shape
	   * @private
	   */},{key:"_getImagePadding",value:function _getImagePadding(){var imgPadding={top:0,right:0,bottom:0,left:0};if(this.options.imagePadding){var optImgPadding=this.options.imagePadding;if(_typeof_1(optImgPadding)=='object'){imgPadding.top=optImgPadding.top;imgPadding.right=optImgPadding.right;imgPadding.bottom=optImgPadding.bottom;imgPadding.left=optImgPadding.left;}else {imgPadding.top=optImgPadding;imgPadding.right=optImgPadding;imgPadding.bottom=optImgPadding;imgPadding.left=optImgPadding;}}return imgPadding;}/**
	   * Adjust the node dimensions for a loaded image.
	   *
	   * Pre: this.imageObj is valid
	   */},{key:"_resizeImage",value:function _resizeImage(){var width,height;if(this.options.shapeProperties.useImageSize===false){// Use the size property
	var ratio_width=1;var ratio_height=1;// Only calculate the proper ratio if both width and height not zero
	if(this.imageObj.width&&this.imageObj.height){if(this.imageObj.width>this.imageObj.height){ratio_width=this.imageObj.width/this.imageObj.height;}else {ratio_height=this.imageObj.height/this.imageObj.width;}}width=this.options.size*2*ratio_width;height=this.options.size*2*ratio_height;}else {// Use the image size with image padding
	var imgPadding=this._getImagePadding();width=this.imageObj.width+imgPadding.left+imgPadding.right;height=this.imageObj.height+imgPadding.top+imgPadding.bottom;}this.width=width;this.height=height;this.radius=0.5*this.width;}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x width
	   * @param {number} y height
	   * @param {ArrowOptions} values
	   * @private
	   */},{key:"_drawRawCircle",value:function _drawRawCircle(ctx,x,y,values){this.initContextForDraw(ctx,values);drawCircle(ctx,x,y,values.size);this.performFill(ctx,values);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {ArrowOptions} values
	   * @private
	   */},{key:"_drawImageAtPosition",value:function _drawImageAtPosition(ctx,values){if(this.imageObj.width!=0){// draw the image
	ctx.globalAlpha=values.opacity!==undefined?values.opacity:1;// draw shadow if enabled
	this.enableShadow(ctx,values);var factor=1;if(this.options.shapeProperties.interpolation===true){factor=this.imageObj.width/this.width/this.body.view.scale;}var imgPadding=this._getImagePadding();var imgPosLeft=this.left+imgPadding.left;var imgPosTop=this.top+imgPadding.top;var imgWidth=this.width-imgPadding.left-imgPadding.right;var imgHeight=this.height-imgPadding.top-imgPadding.bottom;this.imageObj.drawImageAtPosition(ctx,factor,imgPosLeft,imgPosTop,imgWidth,imgHeight);// disable shadows for other elements.
	this.disableShadow(ctx,values);}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x width
	   * @param {number} y height
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @private
	   */},{key:"_drawImageLabel",value:function _drawImageLabel(ctx,x,y,selected,hover){var yLabel;var offset=0;if(this.height!==undefined){offset=this.height*0.5;var labelDimensions=this.labelModule.getTextSize(ctx,selected,hover);if(labelDimensions.lineCount>=1){offset+=labelDimensions.height/2;}}yLabel=y+offset;if(this.options.label){this.labelOffset=offset;}this.labelModule.draw(ctx,x,yLabel,selected,hover,'hanging');}}]);return CircleImageBase;}(NodeBase);/**
	 * A Circle Node/Cluster shape.
	 *
	 * @extends CircleImageBase
	 */var Circle=/*#__PURE__*/function(_CircleImageBase){inherits(Circle,_CircleImageBase);var _super3=_createSuper$1(Circle);/**
	   * @param {Object} options
	   * @param {Object} body
	   * @param {Label} labelModule
	   */function Circle(options,body,labelModule){var _this9;classCallCheck(this,Circle);_this9=_super3.call(this,options,body,labelModule);_this9._setMargins(labelModule);return _this9;}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {boolean} [selected]
	   * @param {boolean} [hover]
	   */createClass(Circle,[{key:"resize",value:function resize(ctx){var selected=arguments.length>1&&arguments[1]!==undefined?arguments[1]:this.selected;var hover=arguments.length>2&&arguments[2]!==undefined?arguments[2]:this.hover;if(this.needsRefresh(selected,hover)){var dimensions=this.getDimensionsFromLabel(ctx,selected,hover);var diameter=Math.max(dimensions.width+this.margin.right+this.margin.left,dimensions.height+this.margin.top+this.margin.bottom);this.options.size=diameter/2;// NOTE: this size field only set here, not in Ellipse, Database, Box
	this.width=diameter;this.height=diameter;this.radius=this.width/2;}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x width
	   * @param {number} y height
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {ArrowOptions} values
	   */},{key:"draw",value:function draw(ctx,x,y,selected,hover,values){this.resize(ctx,selected,hover);this.left=x-this.width/2;this.top=y-this.height/2;this._drawRawCircle(ctx,x,y,values);this.updateBoundingBox(x,y);this.labelModule.draw(ctx,this.left+this.textSize.width/2+this.margin.left,y,selected,hover);}/**
	   *
	   * @param {number} x width
	   * @param {number} y height
	   */},{key:"updateBoundingBox",value:function updateBoundingBox(x,y){this.boundingBox.top=y-this.options.size;this.boundingBox.left=x-this.options.size;this.boundingBox.right=x+this.options.size;this.boundingBox.bottom=y+this.options.size;}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} angle - Unused
	   * @returns {number}
	   */},{key:"distanceToBorder",value:function distanceToBorder(ctx,angle){// eslint-disable-line no-unused-vars
	if(ctx){this.resize(ctx);}return this.width*0.5;}}]);return Circle;}(CircleImageBase);/**
	 * A CircularImage Node/Cluster shape.
	 *
	 * @extends CircleImageBase
	 */var CircularImage=/*#__PURE__*/function(_CircleImageBase2){inherits(CircularImage,_CircleImageBase2);var _super4=_createSuper$1(CircularImage);/**
	   * @param {Object} options
	   * @param {Object} body
	   * @param {Label} labelModule
	   * @param {Image} imageObj
	   * @param {Image} imageObjAlt
	   */function CircularImage(options,body,labelModule,imageObj,imageObjAlt){var _this10;classCallCheck(this,CircularImage);_this10=_super4.call(this,options,body,labelModule);_this10.setImages(imageObj,imageObjAlt);return _this10;}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {boolean} [selected]
	   * @param {boolean} [hover]
	   */createClass(CircularImage,[{key:"resize",value:function resize(ctx){var selected=arguments.length>1&&arguments[1]!==undefined?arguments[1]:this.selected;var hover=arguments.length>2&&arguments[2]!==undefined?arguments[2]:this.hover;var imageAbsent=this.imageObj.src===undefined||this.imageObj.width===undefined||this.imageObj.height===undefined;if(imageAbsent){var diameter=this.options.size*2;this.width=diameter;this.height=diameter;this.radius=0.5*this.width;return;}// At this point, an image is present, i.e. this.imageObj is valid.
	if(this.needsRefresh(selected,hover)){this._resizeImage();}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x width
	   * @param {number} y height
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {ArrowOptions} values
	   */},{key:"draw",value:function draw(ctx,x,y,selected,hover,values){this.switchImages(selected);this.resize();this.left=x-this.width/2;this.top=y-this.height/2;// draw the background circle. IMPORTANT: the stroke in this method is used by the clip method below.
	this._drawRawCircle(ctx,x,y,values);// now we draw in the circle, we save so we can revert the clip operation after drawing.
	ctx.save();// clip is used to use the stroke in drawRawCircle as an area that we can draw in.
	ctx.clip();// draw the image
	this._drawImageAtPosition(ctx,values);// restore so we can again draw on the full canvas
	ctx.restore();this._drawImageLabel(ctx,x,y,selected,hover);this.updateBoundingBox(x,y);}// TODO: compare with Circle.updateBoundingBox(), consolidate? More stuff is happening here
	/**
	   *
	   * @param {number} x width
	   * @param {number} y height
	   */},{key:"updateBoundingBox",value:function updateBoundingBox(x,y){this.boundingBox.top=y-this.options.size;this.boundingBox.left=x-this.options.size;this.boundingBox.right=x+this.options.size;this.boundingBox.bottom=y+this.options.size;// TODO: compare with Image.updateBoundingBox(), consolidate?
	this.boundingBox.left=Math.min(this.boundingBox.left,this.labelModule.size.left);this.boundingBox.right=Math.max(this.boundingBox.right,this.labelModule.size.left+this.labelModule.size.width);this.boundingBox.bottom=Math.max(this.boundingBox.bottom,this.boundingBox.bottom+this.labelOffset);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} angle - Unused
	   * @returns {number}
	   */},{key:"distanceToBorder",value:function distanceToBorder(ctx,angle){// eslint-disable-line no-unused-vars
	if(ctx){this.resize(ctx);}return this.width*0.5;}}]);return CircularImage;}(CircleImageBase);/**
	 * A Database Node/Cluster shape.
	 *
	 * @extends NodeBase
	 */var Database=/*#__PURE__*/function(_NodeBase3){inherits(Database,_NodeBase3);var _super5=_createSuper$1(Database);/**
	   * @param {Object} options
	   * @param {Object} body
	   * @param {Label} labelModule
	   */function Database(options,body,labelModule){var _this11;classCallCheck(this,Database);_this11=_super5.call(this,options,body,labelModule);_this11._setMargins(labelModule);return _this11;}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {boolean} selected
	   * @param {boolean} hover
	   */createClass(Database,[{key:"resize",value:function resize(ctx,selected,hover){if(this.needsRefresh(selected,hover)){var dimensions=this.getDimensionsFromLabel(ctx,selected,hover);var size=dimensions.width+this.margin.right+this.margin.left;this.width=size;this.height=size;this.radius=this.width/2;}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x width
	   * @param {number} y height
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {ArrowOptions} values
	   */},{key:"draw",value:function draw(ctx,x,y,selected,hover,values){this.resize(ctx,selected,hover);this.left=x-this.width/2;this.top=y-this.height/2;this.initContextForDraw(ctx,values);drawDatabase(ctx,x-this.width/2,y-this.height/2,this.width,this.height);this.performFill(ctx,values);this.updateBoundingBox(x,y,ctx,selected,hover);this.labelModule.draw(ctx,this.left+this.textSize.width/2+this.margin.left,this.top+this.textSize.height/2+this.margin.top,selected,hover);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} angle
	   * @returns {number}
	   */},{key:"distanceToBorder",value:function distanceToBorder(ctx,angle){return this._distanceToBorder(ctx,angle);}}]);return Database;}(NodeBase);/**
	 * Base class for constructing Node/Cluster Shapes.
	 *
	 * @extends NodeBase
	 */var ShapeBase=/*#__PURE__*/function(_NodeBase4){inherits(ShapeBase,_NodeBase4);var _super6=_createSuper$1(ShapeBase);/**
	   * @param {Object} options
	   * @param {Object} body
	   * @param {Label} labelModule
	   */function ShapeBase(options,body,labelModule){classCallCheck(this,ShapeBase);return _super6.call(this,options,body,labelModule);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {boolean} [selected]
	   * @param {boolean} [hover]
	   * @param {Object} [values={size: this.options.size}]
	   */createClass(ShapeBase,[{key:"resize",value:function resize(ctx){var selected=arguments.length>1&&arguments[1]!==undefined?arguments[1]:this.selected;var hover=arguments.length>2&&arguments[2]!==undefined?arguments[2]:this.hover;var values=arguments.length>3&&arguments[3]!==undefined?arguments[3]:{size:this.options.size};if(this.needsRefresh(selected,hover)){this.labelModule.getTextSize(ctx,selected,hover);var size=2*values.size;this.width=size;this.height=size;this.radius=0.5*this.width;}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {string} shape
	   * @param {number} sizeMultiplier - Unused! TODO: Remove next major release
	   * @param {number} x
	   * @param {number} y
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {ArrowOptions} values
	   * @private
	   */},{key:"_drawShape",value:function _drawShape(ctx,shape,sizeMultiplier,x,y,selected,hover,values){this.resize(ctx,selected,hover,values);this.left=x-this.width/2;this.top=y-this.height/2;this.initContextForDraw(ctx,values);getShape(shape)(ctx,x,y,values.size);this.performFill(ctx,values);if(this.options.icon!==undefined){if(this.options.icon.code!==undefined){ctx.font=(selected?"bold ":"")+this.height/2+"px "+(this.options.icon.face||'FontAwesome');ctx.fillStyle=this.options.icon.color||"black";ctx.textAlign="center";ctx.textBaseline="middle";ctx.fillText(this.options.icon.code,x,y);}}if(this.options.label!==undefined){// Need to call following here in order to ensure value for `this.labelModule.size.height`
	this.labelModule.calculateLabelSize(ctx,selected,hover,x,y,'hanging');var yLabel=y+0.5*this.height+0.5*this.labelModule.size.height;this.labelModule.draw(ctx,x,yLabel,selected,hover,'hanging');}this.updateBoundingBox(x,y);}/**
	   *
	   * @param {number} x
	   * @param {number} y
	   */},{key:"updateBoundingBox",value:function updateBoundingBox(x,y){this.boundingBox.top=y-this.options.size;this.boundingBox.left=x-this.options.size;this.boundingBox.right=x+this.options.size;this.boundingBox.bottom=y+this.options.size;if(this.options.label!==undefined&&this.labelModule.size.width>0){this.boundingBox.left=Math.min(this.boundingBox.left,this.labelModule.size.left);this.boundingBox.right=Math.max(this.boundingBox.right,this.labelModule.size.left+this.labelModule.size.width);this.boundingBox.bottom=Math.max(this.boundingBox.bottom,this.boundingBox.bottom+this.labelModule.size.height);}}}]);return ShapeBase;}(NodeBase);/**
	 * A Diamond Node/Cluster shape.
	 *
	 * @extends ShapeBase
	 */var Diamond=/*#__PURE__*/function(_ShapeBase){inherits(Diamond,_ShapeBase);var _super7=_createSuper$1(Diamond);/**
	   * @param {Object} options
	   * @param {Object} body
	   * @param {Label} labelModule
	   */function Diamond(options,body,labelModule){classCallCheck(this,Diamond);return _super7.call(this,options,body,labelModule);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x width
	   * @param {number} y height
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {ArrowOptions} values
	   */createClass(Diamond,[{key:"draw",value:function draw(ctx,x,y,selected,hover,values){this._drawShape(ctx,'diamond',4,x,y,selected,hover,values);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} angle
	   * @returns {number}
	   */},{key:"distanceToBorder",value:function distanceToBorder(ctx,angle){return this._distanceToBorder(ctx,angle);}}]);return Diamond;}(ShapeBase);/**
	 * A Dot Node/Cluster shape.
	 *
	 * @extends ShapeBase
	 */var Dot=/*#__PURE__*/function(_ShapeBase2){inherits(Dot,_ShapeBase2);var _super8=_createSuper$1(Dot);/**
	   * @param {Object} options
	   * @param {Object} body
	   * @param {Label} labelModule
	   */function Dot(options,body,labelModule){classCallCheck(this,Dot);return _super8.call(this,options,body,labelModule);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x width
	   * @param {number} y height
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {ArrowOptions} values
	   */createClass(Dot,[{key:"draw",value:function draw(ctx,x,y,selected,hover,values){this._drawShape(ctx,'circle',2,x,y,selected,hover,values);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} angle
	   * @returns {number}
	   */},{key:"distanceToBorder",value:function distanceToBorder(ctx,angle){// eslint-disable-line no-unused-vars
	if(ctx){this.resize(ctx);}return this.options.size;}}]);return Dot;}(ShapeBase);/**
	 * Am Ellipse Node/Cluster shape.
	 *
	 * @extends NodeBase
	 */var Ellipse=/*#__PURE__*/function(_NodeBase5){inherits(Ellipse,_NodeBase5);var _super9=_createSuper$1(Ellipse);/**
	   * @param {Object} options
	   * @param {Object} body
	   * @param {Label} labelModule
	   */function Ellipse(options,body,labelModule){classCallCheck(this,Ellipse);return _super9.call(this,options,body,labelModule);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {boolean} [selected]
	   * @param {boolean} [hover]
	   */createClass(Ellipse,[{key:"resize",value:function resize(ctx){var selected=arguments.length>1&&arguments[1]!==undefined?arguments[1]:this.selected;var hover=arguments.length>2&&arguments[2]!==undefined?arguments[2]:this.hover;if(this.needsRefresh(selected,hover)){var dimensions=this.getDimensionsFromLabel(ctx,selected,hover);this.height=dimensions.height*2;this.width=dimensions.width+dimensions.height;this.radius=0.5*this.width;}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x width
	   * @param {number} y height
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {ArrowOptions} values
	   */},{key:"draw",value:function draw(ctx,x,y,selected,hover,values){this.resize(ctx,selected,hover);this.left=x-this.width*0.5;this.top=y-this.height*0.5;this.initContextForDraw(ctx,values);drawEllipse(ctx,this.left,this.top,this.width,this.height);this.performFill(ctx,values);this.updateBoundingBox(x,y,ctx,selected,hover);this.labelModule.draw(ctx,x,y,selected,hover);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} angle
	   * @returns {number}
	   */},{key:"distanceToBorder",value:function distanceToBorder(ctx,angle){if(ctx){this.resize(ctx);}var a=this.width*0.5;var b=this.height*0.5;var w=Math.sin(angle)*a;var h=Math.cos(angle)*b;return a*b/Math.sqrt(w*w+h*h);}}]);return Ellipse;}(NodeBase);/**
	 * An icon replacement for the default Node shape.
	 *
	 * @extends NodeBase
	 */var Icon=/*#__PURE__*/function(_NodeBase6){inherits(Icon,_NodeBase6);var _super10=_createSuper$1(Icon);/**
	   * @param {Object} options
	   * @param {Object} body
	   * @param {Label} labelModule
	   */function Icon(options,body,labelModule){var _this12;classCallCheck(this,Icon);_this12=_super10.call(this,options,body,labelModule);_this12._setMargins(labelModule);return _this12;}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx - Unused.
	   * @param {boolean} [selected]
	   * @param {boolean} [hover]
	   */createClass(Icon,[{key:"resize",value:function resize(ctx,selected,hover){if(this.needsRefresh(selected,hover)){this.iconSize={width:Number(this.options.icon.size),height:Number(this.options.icon.size)};this.width=this.iconSize.width+this.margin.right+this.margin.left;this.height=this.iconSize.height+this.margin.top+this.margin.bottom;this.radius=0.5*this.width;}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x width
	   * @param {number} y height
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {ArrowOptions} values
	   */},{key:"draw",value:function draw(ctx,x,y,selected,hover,values){this.resize(ctx,selected,hover);this.options.icon.size=this.options.icon.size||50;this.left=x-this.width/2;this.top=y-this.height/2;this._icon(ctx,x,y,selected,hover,values);if(this.options.label!==undefined){var iconTextSpacing=5;this.labelModule.draw(ctx,this.left+this.iconSize.width/2+this.margin.left,y+this.height/2+iconTextSpacing,selected);}this.updateBoundingBox(x,y);}/**
	   *
	   * @param {number} x
	   * @param {number} y
	   */},{key:"updateBoundingBox",value:function updateBoundingBox(x,y){this.boundingBox.top=y-this.options.icon.size*0.5;this.boundingBox.left=x-this.options.icon.size*0.5;this.boundingBox.right=x+this.options.icon.size*0.5;this.boundingBox.bottom=y+this.options.icon.size*0.5;if(this.options.label!==undefined&&this.labelModule.size.width>0){var iconTextSpacing=5;this.boundingBox.left=Math.min(this.boundingBox.left,this.labelModule.size.left);this.boundingBox.right=Math.max(this.boundingBox.right,this.labelModule.size.left+this.labelModule.size.width);this.boundingBox.bottom=Math.max(this.boundingBox.bottom,this.boundingBox.bottom+this.labelModule.size.height+iconTextSpacing);}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x width
	   * @param {number} y height
	   * @param {boolean} selected
	   * @param {boolean} hover - Unused
	   * @param {ArrowOptions} values
	   */},{key:"_icon",value:function _icon(ctx,x,y,selected,hover,values){var iconSize=Number(this.options.icon.size);if(this.options.icon.code!==undefined){ctx.font=[this.options.icon.weight!=null?this.options.icon.weight:selected?"bold":"",// If the weight is forced (for example to make Font Awesome 5 work
	// properly) substitute slightly bigger size for bold font face.
	(this.options.icon.weight!=null&&selected?5:0)+iconSize+"px",this.options.icon.face].join(" ");// draw icon
	ctx.fillStyle=this.options.icon.color||"black";ctx.textAlign="center";ctx.textBaseline="middle";// draw shadow if enabled
	this.enableShadow(ctx,values);ctx.fillText(this.options.icon.code,x,y);// disable shadows for other elements.
	this.disableShadow(ctx,values);}else {console.error('When using the icon shape, you need to define the code in the icon options object. This can be done per node or globally.');}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} angle
	   * @returns {number}
	   */},{key:"distanceToBorder",value:function distanceToBorder(ctx,angle){return this._distanceToBorder(ctx,angle);}}]);return Icon;}(NodeBase);/**
	 * An image-based replacement for the default Node shape.
	 *
	 * @extends CircleImageBase
	 */var Image$1=/*#__PURE__*/function(_CircleImageBase3){inherits(Image$1,_CircleImageBase3);var _super11=_createSuper$1(Image$1);/**
	   * @param {Object} options
	   * @param {Object} body
	   * @param {Label} labelModule
	   * @param {Image} imageObj
	   * @param {Image} imageObjAlt
	   */function Image$1(options,body,labelModule,imageObj,imageObjAlt){var _this13;classCallCheck(this,Image$1);_this13=_super11.call(this,options,body,labelModule);_this13.setImages(imageObj,imageObjAlt);return _this13;}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx - Unused.
	   * @param {boolean} [selected]
	   * @param {boolean} [hover]
	   */createClass(Image$1,[{key:"resize",value:function resize(ctx){var selected=arguments.length>1&&arguments[1]!==undefined?arguments[1]:this.selected;var hover=arguments.length>2&&arguments[2]!==undefined?arguments[2]:this.hover;var imageAbsent=this.imageObj.src===undefined||this.imageObj.width===undefined||this.imageObj.height===undefined;if(imageAbsent){var side=this.options.size*2;this.width=side;this.height=side;return;}if(this.needsRefresh(selected,hover)){this._resizeImage();}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x width
	   * @param {number} y height
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {ArrowOptions} values
	   */},{key:"draw",value:function draw(ctx,x,y,selected,hover,values){ctx.save();this.switchImages(selected);this.resize();this.left=x-this.width/2;this.top=y-this.height/2;if(this.options.shapeProperties.useBorderWithImage===true){var neutralborderWidth=this.options.borderWidth;var selectionLineWidth=this.options.borderWidthSelected||2*this.options.borderWidth;var borderWidth=(selected?selectionLineWidth:neutralborderWidth)/this.body.view.scale;ctx.lineWidth=Math.min(this.width,borderWidth);ctx.beginPath();var strokeStyle=selected?this.options.color.highlight.border:hover?this.options.color.hover.border:this.options.color.border;var fillStyle=selected?this.options.color.highlight.background:hover?this.options.color.hover.background:this.options.color.background;if(values.opacity!==undefined){strokeStyle=overrideOpacity(strokeStyle,values.opacity);fillStyle=overrideOpacity(fillStyle,values.opacity);}// setup the line properties.
	ctx.strokeStyle=strokeStyle;// set a fillstyle
	ctx.fillStyle=fillStyle;// draw a rectangle to form the border around. This rectangle is filled so the opacity of a picture (in future vis releases?) can be used to tint the image
	ctx.rect(this.left-0.5*ctx.lineWidth,this.top-0.5*ctx.lineWidth,this.width+ctx.lineWidth,this.height+ctx.lineWidth);fill$2(ctx).call(ctx);this.performStroke(ctx,values);ctx.closePath();}this._drawImageAtPosition(ctx,values);this._drawImageLabel(ctx,x,y,selected,hover);this.updateBoundingBox(x,y);ctx.restore();}/**
	   *
	   * @param {number} x
	   * @param {number} y
	   */},{key:"updateBoundingBox",value:function updateBoundingBox(x,y){this.resize();this._updateBoundingBox(x,y);if(this.options.label!==undefined&&this.labelModule.size.width>0){this.boundingBox.left=Math.min(this.boundingBox.left,this.labelModule.size.left);this.boundingBox.right=Math.max(this.boundingBox.right,this.labelModule.size.left+this.labelModule.size.width);this.boundingBox.bottom=Math.max(this.boundingBox.bottom,this.boundingBox.bottom+this.labelOffset);}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} angle
	   * @returns {number}
	   */},{key:"distanceToBorder",value:function distanceToBorder(ctx,angle){return this._distanceToBorder(ctx,angle);}}]);return Image$1;}(CircleImageBase);/**
	 * A Square Node/Cluster shape.
	 *
	 * @extends ShapeBase
	 */var Square=/*#__PURE__*/function(_ShapeBase3){inherits(Square,_ShapeBase3);var _super12=_createSuper$1(Square);/**
	   * @param {Object} options
	   * @param {Object} body
	   * @param {Label} labelModule
	   */function Square(options,body,labelModule){classCallCheck(this,Square);return _super12.call(this,options,body,labelModule);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x width
	   * @param {number} y height
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {ArrowOptions} values
	   */createClass(Square,[{key:"draw",value:function draw(ctx,x,y,selected,hover,values){this._drawShape(ctx,'square',2,x,y,selected,hover,values);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} angle
	   * @returns {number}
	   */},{key:"distanceToBorder",value:function distanceToBorder(ctx,angle){return this._distanceToBorder(ctx,angle);}}]);return Square;}(ShapeBase);/**
	 * A Hexagon Node/Cluster shape.
	 *
	 * @extends ShapeBase
	 */var Hexagon=/*#__PURE__*/function(_ShapeBase4){inherits(Hexagon,_ShapeBase4);var _super13=_createSuper$1(Hexagon);/**
	   * @param {Object} options
	   * @param {Object} body
	   * @param {Label} labelModule
	   */function Hexagon(options,body,labelModule){classCallCheck(this,Hexagon);return _super13.call(this,options,body,labelModule);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x width
	   * @param {number} y height
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {ArrowOptions} values
	   */createClass(Hexagon,[{key:"draw",value:function draw(ctx,x,y,selected,hover,values){this._drawShape(ctx,'hexagon',4,x,y,selected,hover,values);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} angle
	   * @returns {number}
	   */},{key:"distanceToBorder",value:function distanceToBorder(ctx,angle){return this._distanceToBorder(ctx,angle);}}]);return Hexagon;}(ShapeBase);/**
	 * A Star Node/Cluster shape.
	 *
	 * @extends ShapeBase
	 */var Star=/*#__PURE__*/function(_ShapeBase5){inherits(Star,_ShapeBase5);var _super14=_createSuper$1(Star);/**
	   * @param {Object} options
	   * @param {Object} body
	   * @param {Label} labelModule
	   */function Star(options,body,labelModule){classCallCheck(this,Star);return _super14.call(this,options,body,labelModule);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x width
	   * @param {number} y height
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {ArrowOptions} values
	   */createClass(Star,[{key:"draw",value:function draw(ctx,x,y,selected,hover,values){this._drawShape(ctx,'star',4,x,y,selected,hover,values);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} angle
	   * @returns {number}
	   */},{key:"distanceToBorder",value:function distanceToBorder(ctx,angle){return this._distanceToBorder(ctx,angle);}}]);return Star;}(ShapeBase);/**
	 * A text-based replacement for the default Node shape.
	 *
	 * @extends NodeBase
	 */var Text=/*#__PURE__*/function(_NodeBase7){inherits(Text,_NodeBase7);var _super15=_createSuper$1(Text);/**
	   * @param {Object} options
	   * @param {Object} body
	   * @param {Label} labelModule
	   */function Text(options,body,labelModule){var _this14;classCallCheck(this,Text);_this14=_super15.call(this,options,body,labelModule);_this14._setMargins(labelModule);return _this14;}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {boolean} selected
	   * @param {boolean} hover
	   */createClass(Text,[{key:"resize",value:function resize(ctx,selected,hover){if(this.needsRefresh(selected,hover)){this.textSize=this.labelModule.getTextSize(ctx,selected,hover);this.width=this.textSize.width+this.margin.right+this.margin.left;this.height=this.textSize.height+this.margin.top+this.margin.bottom;this.radius=0.5*this.width;}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x width
	   * @param {number} y height
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {ArrowOptions} values
	   */},{key:"draw",value:function draw(ctx,x,y,selected,hover,values){this.resize(ctx,selected,hover);this.left=x-this.width/2;this.top=y-this.height/2;// draw shadow if enabled
	this.enableShadow(ctx,values);this.labelModule.draw(ctx,this.left+this.textSize.width/2+this.margin.left,this.top+this.textSize.height/2+this.margin.top,selected,hover);// disable shadows for other elements.
	this.disableShadow(ctx,values);this.updateBoundingBox(x,y,ctx,selected,hover);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} angle
	   * @returns {number}
	   */},{key:"distanceToBorder",value:function distanceToBorder(ctx,angle){return this._distanceToBorder(ctx,angle);}}]);return Text;}(NodeBase);/**
	 * A Triangle Node/Cluster shape.
	 *
	 * @extends ShapeBase
	 */var Triangle=/*#__PURE__*/function(_ShapeBase6){inherits(Triangle,_ShapeBase6);var _super16=_createSuper$1(Triangle);/**
	   * @param {Object} options
	   * @param {Object} body
	   * @param {Label} labelModule
	   */function Triangle(options,body,labelModule){classCallCheck(this,Triangle);return _super16.call(this,options,body,labelModule);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x
	   * @param {number} y
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {ArrowOptions} values
	   */createClass(Triangle,[{key:"draw",value:function draw(ctx,x,y,selected,hover,values){this._drawShape(ctx,'triangle',3,x,y,selected,hover,values);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} angle
	   * @returns {number}
	   */},{key:"distanceToBorder",value:function distanceToBorder(ctx,angle){return this._distanceToBorder(ctx,angle);}}]);return Triangle;}(ShapeBase);/**
	 * A downward facing Triangle Node/Cluster shape.
	 *
	 * @extends ShapeBase
	 */var TriangleDown=/*#__PURE__*/function(_ShapeBase7){inherits(TriangleDown,_ShapeBase7);var _super17=_createSuper$1(TriangleDown);/**
	   * @param {Object} options
	   * @param {Object} body
	   * @param {Label} labelModule
	   */function TriangleDown(options,body,labelModule){classCallCheck(this,TriangleDown);return _super17.call(this,options,body,labelModule);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} x
	   * @param {number} y
	   * @param {boolean} selected
	   * @param {boolean} hover
	   * @param {ArrowOptions} values
	   */createClass(TriangleDown,[{key:"draw",value:function draw(ctx,x,y,selected,hover,values){this._drawShape(ctx,'triangleDown',3,x,y,selected,hover,values);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {number} angle
	   * @returns {number}
	   */},{key:"distanceToBorder",value:function distanceToBorder(ctx,angle){return this._distanceToBorder(ctx,angle);}}]);return TriangleDown;}(ShapeBase);var errorFound$1=false;var allOptions$2;var printStyle$1='background: #FFeeee; color: #dd0000';/**
	 *  Used to validate options.
	 */var Validator$1=/*#__PURE__*/function(){/**
	   * @ignore
	   */function Validator(){classCallCheck(this,Validator);}/**
	   * Main function to be called
	   * @param {Object} options
	   * @param {Object} referenceOptions
	   * @param {Object} subObject
	   * @returns {boolean}
	   * @static
	   */createClass(Validator,null,[{key:"validate",value:function validate(options,referenceOptions,subObject){errorFound$1=false;allOptions$2=referenceOptions;var usedOptions=referenceOptions;if(subObject!==undefined){usedOptions=referenceOptions[subObject];}Validator.parse(options,usedOptions,[]);return errorFound$1;}/**
	   * Will traverse an object recursively and check every value
	   * @param {Object} options
	   * @param {Object} referenceOptions
	   * @param {array} path    | where to look for the actual option
	   * @static
	   */},{key:"parse",value:function parse(options,referenceOptions,path){for(var option in options){if(options.hasOwnProperty(option)){Validator.check(option,options,referenceOptions,path);}}}/**
	   * Check every value. If the value is an object, call the parse function on that object.
	   * @param {string} option
	   * @param {Object} options
	   * @param {Object} referenceOptions
	   * @param {array} path    | where to look for the actual option
	   * @static
	   */},{key:"check",value:function check(option,options,referenceOptions,path){if(referenceOptions[option]===undefined&&referenceOptions.__any__===undefined){Validator.getSuggestion(option,referenceOptions,path);return;}var referenceOption=option;var is_object=true;if(referenceOptions[option]===undefined&&referenceOptions.__any__!==undefined){// NOTE: This only triggers if the __any__ is in the top level of the options object.
	//       THAT'S A REALLY BAD PLACE TO ALLOW IT!!!!
	// TODO: Examine if needed, remove if possible
	// __any__ is a wildcard. Any value is accepted and will be further analysed by reference.
	referenceOption='__any__';// if the any-subgroup is not a predefined object in the configurator,
	// we do not look deeper into the object.
	is_object=Validator.getType(options[option])==='object';}var refOptionObj=referenceOptions[referenceOption];if(is_object&&refOptionObj.__type__!==undefined){refOptionObj=refOptionObj.__type__;}Validator.checkFields(option,options,referenceOptions,referenceOption,refOptionObj,path);}/**
	   *
	   * @param {string}  option           | the option property
	   * @param {Object}  options          | The supplied options object
	   * @param {Object}  referenceOptions | The reference options containing all options and their allowed formats
	   * @param {string}  referenceOption  | Usually this is the same as option, except when handling an __any__ tag.
	   * @param {string}  refOptionObj     | This is the type object from the reference options
	   * @param {Array}   path             | where in the object is the option
	   * @static
	   */},{key:"checkFields",value:function checkFields(option,options,referenceOptions,referenceOption,refOptionObj,path){var log=function log(message){console.log('%c'+message+Validator.printLocation(path,option),printStyle$1);};var optionType=Validator.getType(options[option]);var refOptionType=refOptionObj[optionType];if(refOptionType!==undefined){// if the type is correct, we check if it is supposed to be one of a few select values
	if(Validator.getType(refOptionType)==='array'&&indexOf$3(refOptionType).call(refOptionType,options[option])===-1){log('Invalid option detected in "'+option+'".'+' Allowed values are:'+Validator.print(refOptionType)+' not "'+options[option]+'". ');errorFound$1=true;}else if(optionType==='object'&&referenceOption!=="__any__"){path=copyAndExtendArray(path,option);Validator.parse(options[option],referenceOptions[referenceOption],path);}}else if(refOptionObj['any']===undefined){// type of the field is incorrect and the field cannot be any
	log('Invalid type received for "'+option+'". Expected: '+Validator.print(keys$6(refOptionObj))+'. Received ['+optionType+'] "'+options[option]+'"');errorFound$1=true;}}/**
	   *
	   * @param {Object|boolean|number|string|Array.<number>|Date|Node|Moment|undefined|null} object
	   * @returns {string}
	   * @static
	   */},{key:"getType",value:function getType(object){var type=_typeof_1(object);if(type==='object'){if(object===null){return 'null';}if(object instanceof Boolean){return 'boolean';}if(object instanceof Number){return 'number';}if(object instanceof String){return 'string';}if(isArray$3(object)){return 'array';}if(object instanceof Date){return 'date';}if(object.nodeType!==undefined){return 'dom';}if(object._isAMomentObject===true){return 'moment';}return 'object';}else if(type==='number'){return 'number';}else if(type==='boolean'){return 'boolean';}else if(type==='string'){return 'string';}else if(type===undefined){return 'undefined';}return type;}/**
	   * @param {string} option
	   * @param {Object} options
	   * @param {Array.<string>} path
	   * @static
	   */},{key:"getSuggestion",value:function getSuggestion(option,options,path){var localSearch=Validator.findInOptions(option,options,path,false);var globalSearch=Validator.findInOptions(option,allOptions$2,[],true);var localSearchThreshold=8;var globalSearchThreshold=4;var msg;if(localSearch.indexMatch!==undefined){msg=' in '+Validator.printLocation(localSearch.path,option,'')+'Perhaps it was incomplete? Did you mean: "'+localSearch.indexMatch+'"?\n\n';}else if(globalSearch.distance<=globalSearchThreshold&&localSearch.distance>globalSearch.distance){msg=' in '+Validator.printLocation(localSearch.path,option,'')+'Perhaps it was misplaced? Matching option found at: '+Validator.printLocation(globalSearch.path,globalSearch.closestMatch,'');}else if(localSearch.distance<=localSearchThreshold){msg='. Did you mean "'+localSearch.closestMatch+'"?'+Validator.printLocation(localSearch.path,option);}else {msg='. Did you mean one of these: '+Validator.print(keys$6(options))+Validator.printLocation(path,option);}console.log('%cUnknown option detected: "'+option+'"'+msg,printStyle$1);errorFound$1=true;}/**
	   * traverse the options in search for a match.
	   * @param {string} option
	   * @param {Object} options
	   * @param {Array} path    | where to look for the actual option
	   * @param {boolean} [recursive=false]
	   * @returns {{closestMatch: string, path: Array, distance: number}}
	   * @static
	   */},{key:"findInOptions",value:function findInOptions(option,options,path){var recursive=arguments.length>3&&arguments[3]!==undefined?arguments[3]:false;var min=1e9;var closestMatch='';var closestMatchPath=[];var lowerCaseOption=option.toLowerCase();var indexMatch=undefined;for(var op in options){// eslint-disable-line guard-for-in
	var distance=void 0;if(options[op].__type__!==undefined&&recursive===true){var result=Validator.findInOptions(option,options[op],copyAndExtendArray(path,op));if(min>result.distance){closestMatch=result.closestMatch;closestMatchPath=result.path;min=result.distance;indexMatch=result.indexMatch;}}else {var _context16;if(indexOf$3(_context16=op.toLowerCase()).call(_context16,lowerCaseOption)!==-1){indexMatch=op;}distance=Validator.levenshteinDistance(option,op);if(min>distance){closestMatch=op;closestMatchPath=copyArray(path);min=distance;}}}return {closestMatch:closestMatch,path:closestMatchPath,distance:min,indexMatch:indexMatch};}/**
	   * @param {Array.<string>} path
	   * @param {Object} option
	   * @param {string} prefix
	   * @returns {String}
	   * @static
	   */},{key:"printLocation",value:function printLocation(path,option){var prefix=arguments.length>2&&arguments[2]!==undefined?arguments[2]:'Problem value found at: \n';var str='\n\n'+prefix+'options = {\n';for(var i=0;i<path.length;i++){for(var j=0;j<i+1;j++){str+='  ';}str+=path[i]+': {\n';}for(var _j2=0;_j2<path.length+1;_j2++){str+='  ';}str+=option+'\n';for(var _i3=0;_i3<path.length+1;_i3++){for(var _j3=0;_j3<path.length-_i3;_j3++){str+='  ';}str+='}\n';}return str+'\n\n';}/**
	   * @param {Object} options
	   * @returns {String}
	   * @static
	   */},{key:"print",value:function print(options){return stringify$2(options).replace(/(\")|(\[)|(\])|(,"__type__")/g,"").replace(/(\,)/g,', ');}/**
	   *  Compute the edit distance between the two given strings
	   * http://en.wikibooks.org/wiki/Algorithm_Implementation/Strings/Levenshtein_distance#JavaScript
	   *
	   * Copyright (c) 2011 Andrei Mackenzie
	   *
	   * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
	   *
	   * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
	   *
	   * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
	   *
	   * @param {string} a
	   * @param {string} b
	   * @returns {Array.<Array.<number>>}}
	   * @static
	   */},{key:"levenshteinDistance",value:function levenshteinDistance(a,b){if(a.length===0)return b.length;if(b.length===0)return a.length;var matrix=[];// increment along the first column of each row
	var i;for(i=0;i<=b.length;i++){matrix[i]=[i];}// increment each column in the first row
	var j;for(j=0;j<=a.length;j++){matrix[0][j]=j;}// Fill in the rest of the matrix
	for(i=1;i<=b.length;i++){for(j=1;j<=a.length;j++){if(b.charAt(i-1)==a.charAt(j-1)){matrix[i][j]=matrix[i-1][j-1];}else {matrix[i][j]=Math.min(matrix[i-1][j-1]+1,// substitution
	Math.min(matrix[i][j-1]+1,// insertion
	matrix[i-1][j]+1));// deletion
	}}}return matrix[b.length][a.length];}}]);return Validator;}();/**
	 * A node. A node can be connected to other nodes via one or multiple edges.
	 */var Node=/*#__PURE__*/function(){/**
	   *
	   * @param {object} options An object containing options for the node. All
	   *                            options are optional, except for the id.
	   *                              {number} id     Id of the node. Required
	   *                              {string} label  Text label for the node
	   *                              {number} x      Horizontal position of the node
	   *                              {number} y      Vertical position of the node
	   *                              {string} shape  Node shape
	   *                              {string} image  An image url
	   *                              {string} title  A title text, can be HTML
	   *                              {anytype} group A group name or number
	   *
	   * @param {Object} body               Shared state of current network instance
	   * @param {Network.Images} imagelist  A list with images. Only needed when the node has an image
	   * @param {Groups} grouplist          A list with groups. Needed for retrieving group options
	   * @param {Object} globalOptions      Current global node options; these serve as defaults for the node instance
	   * @param {Object} defaultOptions     Global default options for nodes; note that this is also the prototype
	   *                                    for parameter `globalOptions`.
	   */function Node(options,body,imagelist,grouplist,globalOptions,defaultOptions){classCallCheck(this,Node);this.options=bridgeObject(globalOptions);this.globalOptions=globalOptions;this.defaultOptions=defaultOptions;this.body=body;this.edges=[];// all edges connected to this node
	// set defaults for the options
	this.id=undefined;this.imagelist=imagelist;this.grouplist=grouplist;// state options
	this.x=undefined;this.y=undefined;this.baseSize=this.options.size;this.baseFontSize=this.options.font.size;this.predefinedPosition=false;// used to check if initial fit should just take the range or approximate
	this.selected=false;this.hover=false;this.labelModule=new Label(this.body,this.options,false/* Not edge label */);this.setOptions(options);}/**
	   * Attach a edge to the node
	   * @param {Edge} edge
	   */createClass(Node,[{key:"attachEdge",value:function attachEdge(edge){var _context17;if(indexOf$3(_context17=this.edges).call(_context17,edge)===-1){this.edges.push(edge);}}/**
	   * Detach a edge from the node
	   *
	   * @param {Edge} edge
	   */},{key:"detachEdge",value:function detachEdge(edge){var _context18;var index=indexOf$3(_context18=this.edges).call(_context18,edge);if(index!=-1){var _context19;splice$2(_context19=this.edges).call(_context19,index,1);}}/**
	   * Set or overwrite options for the node
	   *
	   * @param {Object} options an object with options
	   * @returns {null|boolean}
	   */},{key:"setOptions",value:function setOptions(options){var currentShape=this.options.shape;if(!options){return;// Note that the return value will be 'undefined'! This is OK.
	}// Save the color for later.
	// This is necessary in order to prevent local color from being overwritten by group color.
	// TODO: To prevent such workarounds the way options are handled should be rewritten from scratch.
	// This is not the only problem with current options handling.
	if(typeof options.color!=='undefined'){this._localColor=options.color;}// basic options
	if(options.id!==undefined){this.id=options.id;}if(this.id===undefined){throw new Error("Node must have an id");}Node.checkMass(options,this.id);// set these options locally
	// clear x and y positions
	if(options.x!==undefined){if(options.x===null){this.x=undefined;this.predefinedPosition=false;}else {this.x=_parseInt$2(options.x);this.predefinedPosition=true;}}if(options.y!==undefined){if(options.y===null){this.y=undefined;this.predefinedPosition=false;}else {this.y=_parseInt$2(options.y);this.predefinedPosition=true;}}if(options.size!==undefined){this.baseSize=options.size;}if(options.value!==undefined){options.value=_parseFloat$2(options.value);}// this transforms all shorthands into fully defined options
	Node.parseOptions(this.options,options,true,this.globalOptions,this.grouplist);var pile=[options,this.options,this.defaultOptions];this.chooser=ComponentUtil.choosify('node',pile);this._load_images();this.updateLabelModule(options);// Need to set local opacity after `this.updateLabelModule(options);` because `this.updateLabelModule(options);` overrites local opacity with group opacity
	if(options.opacity!==undefined&&Node.checkOpacity(options.opacity)){this.options.opacity=options.opacity;}this.updateShape(currentShape);return options.hidden!==undefined||options.physics!==undefined;}/**
	   * Load the images from the options, for the nodes that need them.
	   *
	   * Images are always loaded, even if they are not used in the current shape.
	   * The user may switch to an image shape later on.
	   *
	   * @private
	   */},{key:"_load_images",value:function _load_images(){if(this.options.shape==='circularImage'||this.options.shape==='image'){if(this.options.image===undefined){throw new Error("Option image must be defined for node type '"+this.options.shape+"'");}}if(this.options.image===undefined){return;}if(this.imagelist===undefined){throw new Error("Internal Error: No images provided");}if(typeof this.options.image==='string'){this.imageObj=this.imagelist.load(this.options.image,this.options.brokenImage,this.id);}else {if(this.options.image.unselected===undefined){throw new Error("No unselected image provided");}this.imageObj=this.imagelist.load(this.options.image.unselected,this.options.brokenImage,this.id);if(this.options.image.selected!==undefined){this.imageObjAlt=this.imagelist.load(this.options.image.selected,this.options.brokenImage,this.id);}else {this.imageObjAlt=undefined;}}}/**
	   * Check that opacity is only between 0 and 1
	   * 
	   * @param {Number} opacity 
	   * @returns {boolean}
	   */},{key:"getFormattingValues",/**
	   *
	   * @returns {{color: *, borderWidth: *, borderColor: *, size: *, borderDashes: (boolean|Array|allOptions.nodes.shapeProperties.borderDashes|{boolean, array}), borderRadius: (number|allOptions.nodes.shapeProperties.borderRadius|{number}|Array), shadow: *, shadowColor: *, shadowSize: *, shadowX: *, shadowY: *}}
	   */value:function getFormattingValues(){var values={color:this.options.color.background,opacity:this.options.opacity,borderWidth:this.options.borderWidth,borderColor:this.options.color.border,size:this.options.size,borderDashes:this.options.shapeProperties.borderDashes,borderRadius:this.options.shapeProperties.borderRadius,shadow:this.options.shadow.enabled,shadowColor:this.options.shadow.color,shadowSize:this.options.shadow.size,shadowX:this.options.shadow.x,shadowY:this.options.shadow.y};if(this.selected||this.hover){if(this.chooser===true){if(this.selected){values.borderWidth*=2;values.color=this.options.color.highlight.background;values.borderColor=this.options.color.highlight.border;values.shadow=this.options.shadow.enabled;}else if(this.hover){values.color=this.options.color.hover.background;values.borderColor=this.options.color.hover.border;values.shadow=this.options.shadow.enabled;}}else if(typeof this.chooser==='function'){this.chooser(values,this.options.id,this.selected,this.hover);if(values.shadow===false){if(values.shadowColor!==this.options.shadow.color||values.shadowSize!==this.options.shadow.size||values.shadowX!==this.options.shadow.x||values.shadowY!==this.options.shadow.y){values.shadow=true;}}}}else {values.shadow=this.options.shadow.enabled;}if(this.options.opacity!==undefined){var opacity=this.options.opacity;values.borderColor=overrideOpacity(values.borderColor,opacity);values.color=overrideOpacity(values.color,opacity);values.shadowColor=overrideOpacity(values.shadowColor,opacity);}return values;}/**
	   *
	   * @param {Object} options
	   */},{key:"updateLabelModule",value:function updateLabelModule(options){if(this.options.label===undefined||this.options.label===null){this.options.label='';}Node.updateGroupOptions(this.options,_objectSpread$2(_objectSpread$2({},options),{},{color:options&&options.color||this._localColor||undefined}),this.grouplist);//
	// Note:The prototype chain for this.options is:
	//
	// this.options ->    NodesHandler.options    -> NodesHandler.defaultOptions
	//                 (also: this.globalOptions)
	//
	// Note that the prototypes are mentioned explicitly in the pile list below;
	// WE DON'T WANT THE ORDER OF THE PROTOTYPES!!!! At least, not for font handling of labels.
	// This is a good indication that the prototype usage of options is deficient.
	//
	var currentGroup=this.grouplist.get(this.options.group,false);var pile=[options,// new options
	this.options,// current node options, see comment above for prototype
	currentGroup,// group options, if any
	this.globalOptions,// Currently set global node options
	this.defaultOptions// Default global node options
	];this.labelModule.update(this.options,pile);if(this.labelModule.baseSize!==undefined){this.baseFontSize=this.labelModule.baseSize;}}/**
	   *
	   * @param {string} currentShape
	   */},{key:"updateShape",value:function updateShape(currentShape){if(currentShape===this.options.shape&&this.shape){this.shape.setOptions(this.options,this.imageObj,this.imageObjAlt);}else {// choose draw method depending on the shape
	switch(this.options.shape){case'box':this.shape=new Box(this.options,this.body,this.labelModule);break;case'circle':this.shape=new Circle(this.options,this.body,this.labelModule);break;case'circularImage':this.shape=new CircularImage(this.options,this.body,this.labelModule,this.imageObj,this.imageObjAlt);break;case'database':this.shape=new Database(this.options,this.body,this.labelModule);break;case'diamond':this.shape=new Diamond(this.options,this.body,this.labelModule);break;case'dot':this.shape=new Dot(this.options,this.body,this.labelModule);break;case'ellipse':this.shape=new Ellipse(this.options,this.body,this.labelModule);break;case'icon':this.shape=new Icon(this.options,this.body,this.labelModule);break;case'image':this.shape=new Image$1(this.options,this.body,this.labelModule,this.imageObj,this.imageObjAlt);break;case'square':this.shape=new Square(this.options,this.body,this.labelModule);break;case'hexagon':this.shape=new Hexagon(this.options,this.body,this.labelModule);break;case'star':this.shape=new Star(this.options,this.body,this.labelModule);break;case'text':this.shape=new Text(this.options,this.body,this.labelModule);break;case'triangle':this.shape=new Triangle(this.options,this.body,this.labelModule);break;case'triangleDown':this.shape=new TriangleDown(this.options,this.body,this.labelModule);break;default:this.shape=new Ellipse(this.options,this.body,this.labelModule);break;}}this.needsRefresh();}/**
	   * select this node
	   */},{key:"select",value:function select(){this.selected=true;this.needsRefresh();}/**
	   * unselect this node
	   */},{key:"unselect",value:function unselect(){this.selected=false;this.needsRefresh();}/**
	   * Reset the calculated size of the node, forces it to recalculate its size
	   */},{key:"needsRefresh",value:function needsRefresh(){this.shape.refreshNeeded=true;}/**
	   * get the title of this node.
	   * @return {string} title    The title of the node, or undefined when no title
	   *                           has been set.
	   */},{key:"getTitle",value:function getTitle(){return this.options.title;}/**
	   * Calculate the distance to the border of the Node
	   * @param {CanvasRenderingContext2D}   ctx
	   * @param {number} angle        Angle in radians
	   * @returns {number} distance   Distance to the border in pixels
	   */},{key:"distanceToBorder",value:function distanceToBorder(ctx,angle){return this.shape.distanceToBorder(ctx,angle);}/**
	   * Check if this node has a fixed x and y position
	   * @return {boolean}      true if fixed, false if not
	   */},{key:"isFixed",value:function isFixed(){return this.options.fixed.x&&this.options.fixed.y;}/**
	   * check if this node is selecte
	   * @return {boolean} selected   True if node is selected, else false
	   */},{key:"isSelected",value:function isSelected(){return this.selected;}/**
	   * Retrieve the value of the node. Can be undefined
	   * @return {number} value
	   */},{key:"getValue",value:function getValue(){return this.options.value;}/**
	   * Get the current dimensions of the label
	   *
	   * @return {rect}
	   */},{key:"getLabelSize",value:function getLabelSize(){return this.labelModule.size();}/**
	   * Adjust the value range of the node. The node will adjust it's size
	   * based on its value.
	   * @param {number} min
	   * @param {number} max
	   * @param {number} total
	   */},{key:"setValueRange",value:function setValueRange(min,max,total){if(this.options.value!==undefined){var scale=this.options.scaling.customScalingFunction(min,max,total,this.options.value);var sizeDiff=this.options.scaling.max-this.options.scaling.min;if(this.options.scaling.label.enabled===true){var fontDiff=this.options.scaling.label.max-this.options.scaling.label.min;this.options.font.size=this.options.scaling.label.min+scale*fontDiff;}this.options.size=this.options.scaling.min+scale*sizeDiff;}else {this.options.size=this.baseSize;this.options.font.size=this.baseFontSize;}this.updateLabelModule();}/**
	   * Draw this node in the given canvas
	   * The 2d context of a HTML canvas can be retrieved by canvas.getContext("2d");
	   * @param {CanvasRenderingContext2D}   ctx
	   */},{key:"draw",value:function draw(ctx){var values=this.getFormattingValues();this.shape.draw(ctx,this.x,this.y,this.selected,this.hover,values);}/**
	   * Update the bounding box of the shape
	   * @param {CanvasRenderingContext2D}   ctx
	   */},{key:"updateBoundingBox",value:function updateBoundingBox(ctx){this.shape.updateBoundingBox(this.x,this.y,ctx);}/**
	   * Recalculate the size of this node in the given canvas
	   * The 2d context of a HTML canvas can be retrieved by canvas.getContext("2d");
	   * @param {CanvasRenderingContext2D}   ctx
	   */},{key:"resize",value:function resize(ctx){var values=this.getFormattingValues();this.shape.resize(ctx,this.selected,this.hover,values);}/**
	   * Determine all visual elements of this node instance, in which the given
	   * point falls within the bounding shape.
	   *
	   * @param {point} point
	   * @returns {Array.<nodeClickItem|nodeLabelClickItem>} list with the items which are on the point
	   */},{key:"getItemsOnPoint",value:function getItemsOnPoint(point){var ret=[];if(this.labelModule.visible()){if(ComponentUtil.pointInRect(this.labelModule.getSize(),point)){ret.push({nodeId:this.id,labelId:0});}}if(ComponentUtil.pointInRect(this.shape.boundingBox,point)){ret.push({nodeId:this.id});}return ret;}/**
	   * Check if this object is overlapping with the provided object
	   * @param {Object} obj   an object with parameters left, top, right, bottom
	   * @return {boolean}     True if location is located on node
	   */},{key:"isOverlappingWith",value:function isOverlappingWith(obj){return this.shape.left<obj.right&&this.shape.left+this.shape.width>obj.left&&this.shape.top<obj.bottom&&this.shape.top+this.shape.height>obj.top;}/**
	   * Check if this object is overlapping with the provided object
	   * @param {Object} obj   an object with parameters left, top, right, bottom
	   * @return {boolean}     True if location is located on node
	   */},{key:"isBoundingBoxOverlappingWith",value:function isBoundingBoxOverlappingWith(obj){return this.shape.boundingBox.left<obj.right&&this.shape.boundingBox.right>obj.left&&this.shape.boundingBox.top<obj.bottom&&this.shape.boundingBox.bottom>obj.top;}/**
	  * Check valid values for mass
	  *
	  * The mass may not be negative or zero. If it is, reset to 1
	  *
	  * @param {object} options
	  * @param {Node.id} id
	   * @static
	  */}],[{key:"checkOpacity",value:function checkOpacity(opacity){return 0<=opacity&&opacity<=1;}/**
	   * Copy group option values into the node options.
	   *
	   * The group options override the global node options, so the copy of group options
	   *  must happen *after* the global node options have been set.
	   *
	   * This method must also be called also if the global node options have changed and the group options did not.
	   *
	   * @param {Object} parentOptions
	   * @param {Object} newOptions  new values for the options, currently only passed in for check
	   * @param {Object} groupList
	   */},{key:"updateGroupOptions",value:function updateGroupOptions(parentOptions,newOptions,groupList){var _context20;if(groupList===undefined)return;// No groups, nothing to do
	var group=parentOptions.group;// paranoia: the selected group is already merged into node options, check.
	if(newOptions!==undefined&&newOptions.group!==undefined&&group!==newOptions.group){throw new Error("updateGroupOptions: group values in options don't match.");}var hasGroup=typeof group==='number'||typeof group==='string'&&group!='';if(!hasGroup)return;// current node has no group, no need to merge
	var groupObj=groupList.get(group);if(groupObj.opacity!==undefined&&newOptions.opacity===undefined){if(!Node.checkOpacity(groupObj.opacity)){console.error("Invalid option for node opacity. Value must be between 0 and 1, found: "+groupObj.opacity);groupObj.opacity=undefined;}}// Skip any new option to avoid them being overridden by the group options.
	var skipProperties=filter$2(_context20=getOwnPropertyNames$2(newOptions)).call(_context20,function(p){return newOptions[p]!=null;});// Always skip merging group font options into parent; these are required to be distinct for labels
	skipProperties.push('font');selectiveNotDeepExtend(skipProperties,parentOptions,groupObj);// the color object needs to be completely defined.
	// Since groups can partially overwrite the colors, we parse it again, just in case.
	parentOptions.color=parseColor(parentOptions.color);}/**
	   * This process all possible shorthands in the new options and makes sure that the parentOptions are fully defined.
	   * Static so it can also be used by the handler.
	   *
	   * @param {Object} parentOptions
	   * @param {Object} newOptions
	   * @param {boolean} [allowDeletion=false]
	   * @param {Object} [globalOptions={}]
	   * @param {Object} [groupList]
	   * @static
	   */},{key:"parseOptions",value:function parseOptions(parentOptions,newOptions){var allowDeletion=arguments.length>2&&arguments[2]!==undefined?arguments[2]:false;var globalOptions=arguments.length>3&&arguments[3]!==undefined?arguments[3]:{};var groupList=arguments.length>4?arguments[4]:undefined;var fields=['color','fixed','shadow'];selectiveNotDeepExtend(fields,parentOptions,newOptions,allowDeletion);Node.checkMass(newOptions);if(parentOptions.opacity!==undefined){if(!Node.checkOpacity(parentOptions.opacity)){console.error("Invalid option for node opacity. Value must be between 0 and 1, found: "+parentOptions.opacity);parentOptions.opacity=undefined;}}if(newOptions.opacity!==undefined){if(!Node.checkOpacity(newOptions.opacity)){console.error("Invalid option for node opacity. Value must be between 0 and 1, found: "+newOptions.opacity);newOptions.opacity=undefined;}}// merge the shadow options into the parent.
	mergeOptions(parentOptions,newOptions,'shadow',globalOptions);// individual shape newOptions
	if(newOptions.color!==undefined&&newOptions.color!==null){var parsedColor=parseColor(newOptions.color);fillIfDefined(parentOptions.color,parsedColor);}else if(allowDeletion===true&&newOptions.color===null){parentOptions.color=bridgeObject(globalOptions.color);// set the object back to the global options
	}// handle the fixed options
	if(newOptions.fixed!==undefined&&newOptions.fixed!==null){if(typeof newOptions.fixed==='boolean'){parentOptions.fixed.x=newOptions.fixed;parentOptions.fixed.y=newOptions.fixed;}else {if(newOptions.fixed.x!==undefined&&typeof newOptions.fixed.x==='boolean'){parentOptions.fixed.x=newOptions.fixed.x;}if(newOptions.fixed.y!==undefined&&typeof newOptions.fixed.y==='boolean'){parentOptions.fixed.y=newOptions.fixed.y;}}}if(allowDeletion===true&&newOptions.font===null){parentOptions.font=bridgeObject(globalOptions.font);// set the object back to the global options
	}Node.updateGroupOptions(parentOptions,newOptions,groupList);// handle the scaling options, specifically the label part
	if(newOptions.scaling!==undefined){mergeOptions(parentOptions.scaling,newOptions.scaling,'label',globalOptions.scaling);}}},{key:"checkMass",value:function checkMass(options,id){if(options.mass!==undefined&&options.mass<=0){var strId='';if(id!==undefined){strId=' in node id: '+id;}console.log('%cNegative or zero mass disallowed'+strId+', setting mass to 1.',printStyle$1);options.mass=1;}}}]);return Node;}();/**
	 * Handler for Nodes
	 */var NodesHandler=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   * @param {Images} images
	   * @param {Array.<Group>} groups
	   * @param {LayoutEngine} layoutEngine
	   */function NodesHandler(body,images,groups,layoutEngine){var _context21,_this15=this;classCallCheck(this,NodesHandler);this.body=body;this.images=images;this.groups=groups;this.layoutEngine=layoutEngine;// create the node API in the body container
	this.body.functions.createNode=bind$2(_context21=this.create).call(_context21,this);this.nodesListeners={add:function add(event,params){_this15.add(params.items);},update:function update(event,params){_this15.update(params.items,params.data,params.oldData);},remove:function remove(event,params){_this15.remove(params.items);}};this.defaultOptions={borderWidth:1,borderWidthSelected:2,brokenImage:undefined,color:{border:'#2B7CE9',background:'#97C2FC',highlight:{border:'#2B7CE9',background:'#D2E5FF'},hover:{border:'#2B7CE9',background:'#D2E5FF'}},opacity:undefined,// number between 0 and 1
	fixed:{x:false,y:false},font:{color:'#343434',size:14,// px
	face:'arial',background:'none',strokeWidth:0,// px
	strokeColor:'#ffffff',align:'center',vadjust:0,multi:false,bold:{mod:'bold'},boldital:{mod:'bold italic'},ital:{mod:'italic'},mono:{mod:'',size:15,// px
	face:'monospace',vadjust:2}},group:undefined,hidden:false,icon:{face:'FontAwesome',//'FontAwesome',
	code:undefined,//'\uf007',
	size:50,//50,
	color:'#2B7CE9'//'#aa00ff'
	},image:undefined,// --> URL
	imagePadding:{// only for image shape
	top:0,right:0,bottom:0,left:0},label:undefined,labelHighlightBold:true,level:undefined,margin:{top:5,right:5,bottom:5,left:5},mass:1,physics:true,scaling:{min:10,max:30,label:{enabled:false,min:14,max:30,maxVisible:30,drawThreshold:5},customScalingFunction:function customScalingFunction(min,max,total,value){if(max===min){return 0.5;}else {var scale=1/(max-min);return Math.max(0,(value-min)*scale);}}},shadow:{enabled:false,color:'rgba(0,0,0,0.5)',size:10,x:5,y:5},shape:'ellipse',shapeProperties:{borderDashes:false,// only for borders
	borderRadius:6,// only for box shape
	interpolation:true,// only for image and circularImage shapes
	useImageSize:false,// only for image and circularImage shapes
	useBorderWithImage:false// only for image shape
	},size:25,title:undefined,value:undefined,x:undefined,y:undefined};// Protect from idiocy
	if(this.defaultOptions.mass<=0){throw 'Internal error: mass in defaultOptions of NodesHandler may not be zero or negative';}this.options=bridgeObject(this.defaultOptions);this.bindEventListeners();}/**
	   * Binds event listeners
	   */createClass(NodesHandler,[{key:"bindEventListeners",value:function bindEventListeners(){var _context22,_context23,_this16=this;// refresh the nodes. Used when reverting from hierarchical layout
	this.body.emitter.on('refreshNodes',bind$2(_context22=this.refresh).call(_context22,this));this.body.emitter.on('refresh',bind$2(_context23=this.refresh).call(_context23,this));this.body.emitter.on('destroy',function(){forEach$4(_this16.nodesListeners,function(callback,event){if(_this16.body.data.nodes)_this16.body.data.nodes.off(event,callback);});delete _this16.body.functions.createNode;delete _this16.nodesListeners.add;delete _this16.nodesListeners.update;delete _this16.nodesListeners.remove;delete _this16.nodesListeners;});}/**
	   *
	   * @param {Object} options
	   */},{key:"setOptions",value:function setOptions(options){if(options!==undefined){Node.parseOptions(this.options,options);// Need to set opacity here because Node.parseOptions is also used for groups,
	// if you set opacity in Node.parseOptions it overwrites group opacity.
	if(options.opacity!==undefined){if(isNan$2(options.opacity)||!_isFinite$2(options.opacity)||options.opacity<0||options.opacity>1){console.error("Invalid option for node opacity. Value must be between 0 and 1, found: "+options.opacity);}else {this.options.opacity=options.opacity;}}// update the shape in all nodes
	if(options.shape!==undefined){for(var nodeId in this.body.nodes){if(this.body.nodes.hasOwnProperty(nodeId)){this.body.nodes[nodeId].updateShape();}}}// Update the labels of nodes if any relevant options changed.
	if(typeof options.font!=="undefined"||typeof options.widthConstraint!=="undefined"||typeof options.heightConstraint!=="undefined"){for(var _i4=0,_Object$keys2=keys$6(this.body.nodes);_i4<_Object$keys2.length;_i4++){var _nodeId=_Object$keys2[_i4];this.body.nodes[_nodeId].updateLabelModule();this.body.nodes[_nodeId].needsRefresh();}}// update the shape size in all nodes
	if(options.size!==undefined){for(var _nodeId2 in this.body.nodes){if(this.body.nodes.hasOwnProperty(_nodeId2)){this.body.nodes[_nodeId2].needsRefresh();}}}// update the state of the variables if needed
	if(options.hidden!==undefined||options.physics!==undefined){this.body.emitter.emit('_dataChanged');}}}/**
	   * Set a data set with nodes for the network
	   * @param {Array | DataSet | DataView} nodes         The data containing the nodes.
	   * @param {boolean} [doNotEmit=false]
	   * @private
	   */},{key:"setData",value:function setData(nodes){var doNotEmit=arguments.length>1&&arguments[1]!==undefined?arguments[1]:false;var oldNodesData=this.body.data.nodes;if(nodes instanceof DataSet||nodes instanceof DataView){this.body.data.nodes=nodes;}else if(isArray$3(nodes)){this.body.data.nodes=new DataSet();this.body.data.nodes.add(nodes);}else if(!nodes){this.body.data.nodes=new DataSet();}else {throw new TypeError('Array or DataSet expected');}if(oldNodesData){// unsubscribe from old dataset
	forEach$4(this.nodesListeners,function(callback,event){oldNodesData.off(event,callback);});}// remove drawn nodes
	this.body.nodes={};if(this.body.data.nodes){// subscribe to new dataset
	var me=this;forEach$4(this.nodesListeners,function(callback,event){me.body.data.nodes.on(event,callback);});// draw all new nodes
	var ids=this.body.data.nodes.getIds();this.add(ids,true);}if(doNotEmit===false){this.body.emitter.emit("_dataChanged");}}/**
	   * Add nodes
	   * @param {number[] | string[]} ids
	   * @param {boolean} [doNotEmit=false]
	   * @private
	   */},{key:"add",value:function add(ids){var doNotEmit=arguments.length>1&&arguments[1]!==undefined?arguments[1]:false;var id;var newNodes=[];for(var i=0;i<ids.length;i++){id=ids[i];var properties=this.body.data.nodes.get(id);var node=this.create(properties);newNodes.push(node);this.body.nodes[id]=node;// note: this may replace an existing node
	}this.layoutEngine.positionInitially(newNodes);if(doNotEmit===false){this.body.emitter.emit("_dataChanged");}}/**
	   * Update existing nodes, or create them when not yet existing
	   * @param {number[] | string[]} ids id's of changed nodes
	   * @param {Array} changedData array with changed data
	   * @param {Array|undefined} oldData optional; array with previous data
	   * @private
	   */},{key:"update",value:function update(ids,changedData,oldData){var nodes=this.body.nodes;var dataChanged=false;for(var i=0;i<ids.length;i++){var id=ids[i];var node=nodes[id];var data=changedData[i];if(node!==undefined){// update node
	if(node.setOptions(data)){dataChanged=true;}}else {dataChanged=true;// create node
	node=this.create(data);nodes[id]=node;}}if(!dataChanged&&oldData!==undefined){// Check for any changes which should trigger a layout recalculation
	// For now, this is just 'level' for hierarchical layout
	// Assumption: old and new data arranged in same order; at time of writing, this holds.
	dataChanged=some$2(changedData).call(changedData,function(newValue,index){var oldValue=oldData[index];return oldValue&&oldValue.level!==newValue.level;});}if(dataChanged===true){this.body.emitter.emit("_dataChanged");}else {this.body.emitter.emit("_dataUpdated");}}/**
	   * Remove existing nodes. If nodes do not exist, the method will just ignore it.
	   * @param {number[] | string[]} ids
	   * @private
	   */},{key:"remove",value:function remove(ids){var nodes=this.body.nodes;for(var i=0;i<ids.length;i++){var id=ids[i];delete nodes[id];}this.body.emitter.emit("_dataChanged");}/**
	   * create a node
	   * @param {Object} properties
	   * @param {class} [constructorClass=Node.default]
	   * @returns {*}
	   */},{key:"create",value:function create(properties){var constructorClass=arguments.length>1&&arguments[1]!==undefined?arguments[1]:Node;return new constructorClass(properties,this.body,this.images,this.groups,this.options,this.defaultOptions);}/**
	   *
	   * @param {boolean} [clearPositions=false]
	   */},{key:"refresh",value:function refresh(){var _this17=this;var clearPositions=arguments.length>0&&arguments[0]!==undefined?arguments[0]:false;forEach$4(this.body.nodes,function(node,nodeId){var data=_this17.body.data.nodes.get(nodeId);if(data!==undefined){if(clearPositions===true){node.setOptions({x:null,y:null});}node.setOptions({fixed:false});node.setOptions(data);}});}/**
	   * Returns the positions of the nodes.
	   * @param {Array.<Node.id>|String} [ids]  --> optional, can be array of nodeIds, can be string
	   * @returns {{}}
	   */},{key:"getPositions",value:function getPositions(ids){var dataArray={};if(ids!==undefined){if(isArray$3(ids)===true){for(var i=0;i<ids.length;i++){if(this.body.nodes[ids[i]]!==undefined){var node=this.body.nodes[ids[i]];dataArray[ids[i]]={x:Math.round(node.x),y:Math.round(node.y)};}}}else {if(this.body.nodes[ids]!==undefined){var _node=this.body.nodes[ids];dataArray[ids]={x:Math.round(_node.x),y:Math.round(_node.y)};}}}else {for(var _i5=0;_i5<this.body.nodeIndices.length;_i5++){var _node2=this.body.nodes[this.body.nodeIndices[_i5]];dataArray[this.body.nodeIndices[_i5]]={x:Math.round(_node2.x),y:Math.round(_node2.y)};}}return dataArray;}/**
	   * Retrieves the x y position of a specific id.
	   * @param {string} id The id to retrieve.
	   * @throws {TypeError} If no id is included.
	   * @throws {ReferenceError} If an invalid id is provided.
	   */},{key:"getPosition",value:function getPosition(id){if(id==undefined){throw new TypeError("No id was specified for getPosition method.");}else if(this.body.nodes[id]==undefined){throw new ReferenceError("NodeId provided for getPosition does not exist. Provided: ".concat(id));}else {return {x:Math.round(this.body.nodes[id].x),y:Math.round(this.body.nodes[id].y)};}}/**
	   * Load the XY positions of the nodes into the dataset.
	   */},{key:"storePositions",value:function storePositions(){// todo: add support for clusters and hierarchical.
	var dataArray=[];var dataset=this.body.data.nodes.getDataSet();var _iterator=_createForOfIteratorHelper$2(dataset.get()),_step;try{for(_iterator.s();!(_step=_iterator.n()).done;){var dsNode=_step.value;var id=dsNode.id;var bodyNode=this.body.nodes[id];var x=Math.round(bodyNode.x);var y=Math.round(bodyNode.y);if(dsNode.x!==x||dsNode.y!==y){dataArray.push({id:id,x:x,y:y});}}}catch(err){_iterator.e(err);}finally{_iterator.f();}dataset.update(dataArray);}/**
	   * get the bounding box of a node.
	   * @param {Node.id} nodeId
	   * @returns {j|*}
	   */},{key:"getBoundingBox",value:function getBoundingBox(nodeId){if(this.body.nodes[nodeId]!==undefined){return this.body.nodes[nodeId].shape.boundingBox;}}/**
	   * Get the Ids of nodes connected to this node.
	   * @param {Node.id} nodeId
	   * @param {'to'|'from'|undefined} direction values 'from' and 'to' select respectively parent and child nodes only.
	   *                                          Any other value returns both parent and child nodes.
	   * @returns {Array}
	   */},{key:"getConnectedNodes",value:function getConnectedNodes(nodeId,direction){var nodeList=[];if(this.body.nodes[nodeId]!==undefined){var node=this.body.nodes[nodeId];var nodeObj={};// used to quickly check if node already exists
	for(var i=0;i<node.edges.length;i++){var edge=node.edges[i];if(direction!=='to'&&edge.toId==node.id){// these are double equals since ids can be numeric or string
	if(nodeObj[edge.fromId]===undefined){nodeList.push(edge.fromId);nodeObj[edge.fromId]=true;}}else if(direction!=='from'&&edge.fromId==node.id){// these are double equals since ids can be numeric or string
	if(nodeObj[edge.toId]===undefined){nodeList.push(edge.toId);nodeObj[edge.toId]=true;}}}}return nodeList;}/**
	   * Get the ids of the edges connected to this node.
	   * @param {Node.id} nodeId
	   * @returns {*}
	   */},{key:"getConnectedEdges",value:function getConnectedEdges(nodeId){var edgeList=[];if(this.body.nodes[nodeId]!==undefined){var node=this.body.nodes[nodeId];for(var i=0;i<node.edges.length;i++){edgeList.push(node.edges[i].id);}}else {console.log("NodeId provided for getConnectedEdges does not exist. Provided: ",nodeId);}return edgeList;}/**
	   * Move a node.
	   *
	   * @param {Node.id} nodeId
	   * @param {number} x
	   * @param {number} y
	   */},{key:"moveNode",value:function moveNode(nodeId,x,y){var _this18=this;if(this.body.nodes[nodeId]!==undefined){this.body.nodes[nodeId].x=Number(x);this.body.nodes[nodeId].y=Number(y);setTimeout$2(function(){_this18.body.emitter.emit("startSimulation");},0);}else {console.log("Node id supplied to moveNode does not exist. Provided: ",nodeId);}}}]);return NodesHandler;}();/** ============================================================================
	 * Location of all the endpoint drawing routines.
	 *
	 * Every endpoint has its own drawing routine, which contains an endpoint definition.
	 *
	 * The endpoint definitions must have the following properies:
	 *
	 * - (0,0) is the connection point to the node it attaches to
	 * - The endpoints are orientated to the positive x-direction
	 * - The length of the endpoint is at most 1
	 *
	 * As long as the endpoint classes remain simple and not too numerous, they will
	 * be contained within this module.
	 * All classes here except `EndPoints` should be considered as private to this module.
	 *
	 * -----------------------------------------------------------------------------
	 * ### Further Actions
	 *
	 * After adding a new endpoint here, you also need to do the following things:
	 *
	 * - Add the new endpoint name to `network/options.js` in array `endPoints`.
	 * - Add the new endpoint name to the documentation.
	 *   Scan for 'arrows.to.type` and add it to the description.
	 * - Add the endpoint to the examples. At the very least, add it to example
	 *   `edgeStyles/arrowTypes`.
	 * ============================================================================= */ /**
	 * Common methods for endpoints
	 *
	 * @class
	 */var EndPoint=/*#__PURE__*/function(){function EndPoint(){classCallCheck(this,EndPoint);}createClass(EndPoint,null,[{key:"transform",/**
	     * Apply transformation on points for display.
	     *
	     * The following is done:
	     * - rotate by the specified angle
	     * - multiply the (normalized) coordinates by the passed length
	     * - offset by the target coordinates
	     *
	     * @param points - The point(s) to be transformed.
	     * @param arrowData - The data determining the result of the transformation.
	     */value:function transform(points,arrowData){if(!isArray$3(points)){points=[points];}var x=arrowData.point.x;var y=arrowData.point.y;var angle=arrowData.angle;var length=arrowData.length;for(var i=0;i<points.length;++i){var p=points[i];var xt=p.x*Math.cos(angle)-p.y*Math.sin(angle);var yt=p.x*Math.sin(angle)+p.y*Math.cos(angle);p.x=x+length*xt;p.y=y+length*yt;}}/**
	     * Draw a closed path using the given real coordinates.
	     *
	     * @param ctx - The path will be rendered into this context.
	     * @param points - The points of the path.
	     */},{key:"drawPath",value:function drawPath(ctx,points){ctx.beginPath();ctx.moveTo(points[0].x,points[0].y);for(var i=1;i<points.length;++i){ctx.lineTo(points[i].x,points[i].y);}ctx.closePath();}}]);return EndPoint;}();/**
	 * Drawing methods for the arrow endpoint.
	 */var Image$2=/*#__PURE__*/function(_EndPoint){inherits(Image$2,_EndPoint);var _super18=_createSuper$1(Image$2);function Image$2(){classCallCheck(this,Image$2);return _super18.apply(this,arguments);}createClass(Image$2,null,[{key:"draw",/**
	     * Draw this shape at the end of a line.
	     *
	     * @param ctx - The shape will be rendered into this context.
	     * @param arrowData - The data determining the shape.
	     *
	     * @returns False as there is no way to fill an image.
	     */value:function draw(ctx,arrowData){if(arrowData.image){ctx.save();ctx.translate(arrowData.point.x,arrowData.point.y);ctx.rotate(Math.PI/2+arrowData.angle);var width=arrowData.imageWidth!=null?arrowData.imageWidth:arrowData.image.width;var height=arrowData.imageHeight!=null?arrowData.imageHeight:arrowData.image.height;arrowData.image.drawImageAtPosition(ctx,1,// scale
	-width/2,// x
	0,// y
	width,height);ctx.restore();}return false;}}]);return Image$2;}(EndPoint);/**
	 * Drawing methods for the arrow endpoint.
	 */var Arrow=/*#__PURE__*/function(_EndPoint2){inherits(Arrow,_EndPoint2);var _super19=_createSuper$1(Arrow);function Arrow(){classCallCheck(this,Arrow);return _super19.apply(this,arguments);}createClass(Arrow,null,[{key:"draw",/**
	     * Draw this shape at the end of a line.
	     *
	     * @param ctx - The shape will be rendered into this context.
	     * @param arrowData - The data determining the shape.
	     *
	     * @returns True because ctx.fill() can be used to fill the arrow.
	     */value:function draw(ctx,arrowData){// Normalized points of closed path, in the order that they should be drawn.
	// (0, 0) is the attachment point, and the point around which should be rotated
	var points=[{x:0,y:0},{x:-1,y:0.3},{x:-0.9,y:0},{x:-1,y:-0.3}];EndPoint.transform(points,arrowData);EndPoint.drawPath(ctx,points);return true;}}]);return Arrow;}(EndPoint);/**
	 * Drawing methods for the crow endpoint.
	 */var Crow=/*#__PURE__*/function(){function Crow(){classCallCheck(this,Crow);}createClass(Crow,null,[{key:"draw",/**
	     * Draw this shape at the end of a line.
	     *
	     * @param ctx - The shape will be rendered into this context.
	     * @param arrowData - The data determining the shape.
	     *
	     * @returns True because ctx.fill() can be used to fill the arrow.
	     */value:function draw(ctx,arrowData){// Normalized points of closed path, in the order that they should be drawn.
	// (0, 0) is the attachment point, and the point around which should be rotated
	var points=[{x:-1,y:0},{x:0,y:0.3},{x:-0.4,y:0},{x:0,y:-0.3}];EndPoint.transform(points,arrowData);EndPoint.drawPath(ctx,points);return true;}}]);return Crow;}();/**
	 * Drawing methods for the curve endpoint.
	 */var Curve=/*#__PURE__*/function(){function Curve(){classCallCheck(this,Curve);}createClass(Curve,null,[{key:"draw",/**
	     * Draw this shape at the end of a line.
	     *
	     * @param ctx - The shape will be rendered into this context.
	     * @param arrowData - The data determining the shape.
	     *
	     * @returns True because ctx.fill() can be used to fill the arrow.
	     */value:function draw(ctx,arrowData){// Normalized points of closed path, in the order that they should be drawn.
	// (0, 0) is the attachment point, and the point around which should be rotated
	var point={x:-0.4,y:0};EndPoint.transform(point,arrowData);// Update endpoint style for drawing transparent arc.
	ctx.strokeStyle=ctx.fillStyle;ctx.fillStyle="rgba(0, 0, 0, 0)";// Define curve endpoint as semicircle.
	var pi=Math.PI;var startAngle=arrowData.angle-pi/2;var endAngle=arrowData.angle+pi/2;ctx.beginPath();ctx.arc(point.x,point.y,arrowData.length*0.4,startAngle,endAngle,false);ctx.stroke();return true;}}]);return Curve;}();/**
	 * Drawing methods for the inverted curve endpoint.
	 */var InvertedCurve=/*#__PURE__*/function(){function InvertedCurve(){classCallCheck(this,InvertedCurve);}createClass(InvertedCurve,null,[{key:"draw",/**
	     * Draw this shape at the end of a line.
	     *
	     * @param ctx - The shape will be rendered into this context.
	     * @param arrowData - The data determining the shape.
	     *
	     * @returns True because ctx.fill() can be used to fill the arrow.
	     */value:function draw(ctx,arrowData){// Normalized points of closed path, in the order that they should be drawn.
	// (0, 0) is the attachment point, and the point around which should be rotated
	var point={x:-0.3,y:0};EndPoint.transform(point,arrowData);// Update endpoint style for drawing transparent arc.
	ctx.strokeStyle=ctx.fillStyle;ctx.fillStyle="rgba(0, 0, 0, 0)";// Define inverted curve endpoint as semicircle.
	var pi=Math.PI;var startAngle=arrowData.angle+pi/2;var endAngle=arrowData.angle+3*pi/2;ctx.beginPath();ctx.arc(point.x,point.y,arrowData.length*0.4,startAngle,endAngle,false);ctx.stroke();return true;}}]);return InvertedCurve;}();/**
	 * Drawing methods for the trinagle endpoint.
	 */var Triangle$1=/*#__PURE__*/function(){function Triangle$1(){classCallCheck(this,Triangle$1);}createClass(Triangle$1,null,[{key:"draw",/**
	     * Draw this shape at the end of a line.
	     *
	     * @param ctx - The shape will be rendered into this context.
	     * @param arrowData - The data determining the shape.
	     *
	     * @returns True because ctx.fill() can be used to fill the arrow.
	     */value:function draw(ctx,arrowData){// Normalized points of closed path, in the order that they should be drawn.
	// (0, 0) is the attachment point, and the point around which should be rotated
	var points=[{x:0.02,y:0},{x:-1,y:0.3},{x:-1,y:-0.3}];EndPoint.transform(points,arrowData);EndPoint.drawPath(ctx,points);return true;}}]);return Triangle$1;}();/**
	 * Drawing methods for the inverted trinagle endpoint.
	 */var InvertedTriangle=/*#__PURE__*/function(){function InvertedTriangle(){classCallCheck(this,InvertedTriangle);}createClass(InvertedTriangle,null,[{key:"draw",/**
	     * Draw this shape at the end of a line.
	     *
	     * @param ctx - The shape will be rendered into this context.
	     * @param arrowData - The data determining the shape.
	     *
	     * @returns True because ctx.fill() can be used to fill the arrow.
	     */value:function draw(ctx,arrowData){// Normalized points of closed path, in the order that they should be drawn.
	// (0, 0) is the attachment point, and the point around which should be rotated
	var points=[{x:0,y:0.3},{x:0,y:-0.3},{x:-1,y:0}];EndPoint.transform(points,arrowData);EndPoint.drawPath(ctx,points);return true;}}]);return InvertedTriangle;}();/**
	 * Drawing methods for the circle endpoint.
	 */var Circle$1=/*#__PURE__*/function(){function Circle$1(){classCallCheck(this,Circle$1);}createClass(Circle$1,null,[{key:"draw",/**
	     * Draw this shape at the end of a line.
	     *
	     * @param ctx - The shape will be rendered into this context.
	     * @param arrowData - The data determining the shape.
	     *
	     * @returns True because ctx.fill() can be used to fill the arrow.
	     */value:function draw(ctx,arrowData){var point={x:-0.4,y:0};EndPoint.transform(point,arrowData);drawCircle(ctx,point.x,point.y,arrowData.length*0.4);return true;}}]);return Circle$1;}();/**
	 * Drawing methods for the bar endpoint.
	 */var Bar=/*#__PURE__*/function(){function Bar(){classCallCheck(this,Bar);}createClass(Bar,null,[{key:"draw",/**
	     * Draw this shape at the end of a line.
	     *
	     * @param ctx - The shape will be rendered into this context.
	     * @param arrowData - The data determining the shape.
	     *
	     * @returns True because ctx.fill() can be used to fill the arrow.
	     */value:function draw(ctx,arrowData){/*
	        var points = [
	          {x:0, y:0.5},
	          {x:0, y:-0.5}
	        ];
	    
	        EndPoint.transform(points, arrowData);
	        ctx.beginPath();
	        ctx.moveTo(points[0].x, points[0].y);
	        ctx.lineTo(points[1].x, points[1].y);
	        ctx.stroke();
	    */var points=[{x:0,y:0.5},{x:0,y:-0.5},{x:-0.15,y:-0.5},{x:-0.15,y:0.5}];EndPoint.transform(points,arrowData);EndPoint.drawPath(ctx,points);return true;}}]);return Bar;}();/**
	 * Drawing methods for the box endpoint.
	 */var Box$1=/*#__PURE__*/function(){function Box$1(){classCallCheck(this,Box$1);}createClass(Box$1,null,[{key:"draw",/**
	     * Draw this shape at the end of a line.
	     *
	     * @param ctx - The shape will be rendered into this context.
	     * @param arrowData - The data determining the shape.
	     *
	     * @returns True because ctx.fill() can be used to fill the arrow.
	     */value:function draw(ctx,arrowData){var points=[{x:0,y:0.3},{x:0,y:-0.3},{x:-0.6,y:-0.3},{x:-0.6,y:0.3}];EndPoint.transform(points,arrowData);EndPoint.drawPath(ctx,points);return true;}}]);return Box$1;}();/**
	 * Drawing methods for the diamond endpoint.
	 */var Diamond$1=/*#__PURE__*/function(){function Diamond$1(){classCallCheck(this,Diamond$1);}createClass(Diamond$1,null,[{key:"draw",/**
	     * Draw this shape at the end of a line.
	     *
	     * @param ctx - The shape will be rendered into this context.
	     * @param arrowData - The data determining the shape.
	     *
	     * @returns True because ctx.fill() can be used to fill the arrow.
	     */value:function draw(ctx,arrowData){var points=[{x:0,y:0},{x:-0.5,y:-0.3},{x:-1,y:0},{x:-0.5,y:0.3}];EndPoint.transform(points,arrowData);EndPoint.drawPath(ctx,points);return true;}}]);return Diamond$1;}();/**
	 * Drawing methods for the vee endpoint.
	 */var Vee=/*#__PURE__*/function(){function Vee(){classCallCheck(this,Vee);}createClass(Vee,null,[{key:"draw",/**
	     * Draw this shape at the end of a line.
	     *
	     * @param ctx - The shape will be rendered into this context.
	     * @param arrowData - The data determining the shape.
	     *
	     * @returns True because ctx.fill() can be used to fill the arrow.
	     */value:function draw(ctx,arrowData){// Normalized points of closed path, in the order that they should be drawn.
	// (0, 0) is the attachment point, and the point around which should be rotated
	var points=[{x:-1,y:0.3},{x:-0.5,y:0},{x:-1,y:-0.3},{x:0,y:0}];EndPoint.transform(points,arrowData);EndPoint.drawPath(ctx,points);return true;}}]);return Vee;}();/**
	 * Drawing methods for the endpoints.
	 */var EndPoints=/*#__PURE__*/function(){function EndPoints(){classCallCheck(this,EndPoints);}createClass(EndPoints,null,[{key:"draw",/**
	     * Draw an endpoint.
	     *
	     * @param ctx - The shape will be rendered into this context.
	     * @param arrowData - The data determining the shape.
	     *
	     * @returns True if ctx.fill() can be used to fill the arrow, false otherwise.
	     */value:function draw(ctx,arrowData){var type;if(arrowData.type){type=arrowData.type.toLowerCase();}switch(type){case"image":return Image$2.draw(ctx,arrowData);case"circle":return Circle$1.draw(ctx,arrowData);case"box":return Box$1.draw(ctx,arrowData);case"crow":return Crow.draw(ctx,arrowData);case"curve":return Curve.draw(ctx,arrowData);case"diamond":return Diamond$1.draw(ctx,arrowData);case"inv_curve":return InvertedCurve.draw(ctx,arrowData);case"triangle":return Triangle$1.draw(ctx,arrowData);case"inv_triangle":return InvertedTriangle.draw(ctx,arrowData);case"bar":return Bar.draw(ctx,arrowData);case"vee":return Vee.draw(ctx,arrowData);case"arrow":// fall-through
	default:return Arrow.draw(ctx,arrowData);}}}]);return EndPoints;}();/**
	 * The Base Class for all edges.
	 */var EdgeBase=/*#__PURE__*/function(){/**
	     * Create a new instance.
	     *
	     * @param options - The options object of given edge.
	     * @param _body - The body of the network.
	     * @param _labelModule - Label module.
	     */function EdgeBase(options,_body,_labelModule){classCallCheck(this,EdgeBase);this._body=_body;this._labelModule=_labelModule;this.color={};this.colorDirty=true;this.hoverWidth=1.5;this.selectionWidth=2;this.setOptions(options);this.fromPoint=this.from;this.toPoint=this.to;}/** @inheritdoc */createClass(EdgeBase,[{key:"connect",value:function connect(){this.from=this._body.nodes[this.options.from];this.to=this._body.nodes[this.options.to];}/** @inheritdoc */},{key:"cleanup",value:function cleanup(){return false;}/**
	     * Set new edge options.
	     *
	     * @param options - The new edge options object.
	     */},{key:"setOptions",value:function setOptions(options){this.options=options;this.from=this._body.nodes[this.options.from];this.to=this._body.nodes[this.options.to];this.id=this.options.id;}/** @inheritdoc */},{key:"drawLine",value:function drawLine(ctx,values,_selected,_hover){var viaNode=arguments.length>4&&arguments[4]!==undefined?arguments[4]:this.getViaNode();// set style
	ctx.strokeStyle=this.getColor(ctx,values);ctx.lineWidth=values.width;if(values.dashes!==false){this._drawDashedLine(ctx,values,viaNode);}else {this._drawLine(ctx,values,viaNode);}}/**
	     * Draw a line with given style between two nodes through supplied node(s).
	     *
	     * @param ctx - The context that will be used for rendering.
	     * @param values - Formatting values like color, opacity or shadow.
	     * @param viaNode - Additional control point(s) for the edge.
	     * @param fromPoint - TODO: Seems ignored, remove?
	     * @param toPoint - TODO: Seems ignored, remove?
	     */},{key:"_drawLine",value:function _drawLine(ctx,values,viaNode,fromPoint,toPoint){if(this.from!=this.to){// draw line
	this._line(ctx,values,viaNode,fromPoint,toPoint);}else {var _this$_getCircleData=this._getCircleData(ctx),_this$_getCircleData2=slicedToArray(_this$_getCircleData,3),x=_this$_getCircleData2[0],y=_this$_getCircleData2[1],radius=_this$_getCircleData2[2];this._circle(ctx,values,x,y,radius);}}/**
	     * Draw a dashed line with given style between two nodes through supplied node(s).
	     *
	     * @param ctx - The context that will be used for rendering.
	     * @param values - Formatting values like color, opacity or shadow.
	     * @param viaNode - Additional control point(s) for the edge.
	     * @param _fromPoint - Ignored (TODO: remove in the future).
	     * @param _toPoint - Ignored (TODO: remove in the future).
	     */},{key:"_drawDashedLine",value:function _drawDashedLine(ctx,values,viaNode,_fromPoint,_toPoint){ctx.lineCap="round";var pattern=isArray$3(values.dashes)?values.dashes:[5,5];// only firefox and chrome support this method, else we use the legacy one.
	if(ctx.setLineDash!==undefined){ctx.save();// set dash settings for chrome or firefox
	ctx.setLineDash(pattern);ctx.lineDashOffset=0;// draw the line
	if(this.from!=this.to){// draw line
	this._line(ctx,values,viaNode);}else {var _this$_getCircleData3=this._getCircleData(ctx),_this$_getCircleData4=slicedToArray(_this$_getCircleData3,3),x=_this$_getCircleData4[0],y=_this$_getCircleData4[1],radius=_this$_getCircleData4[2];this._circle(ctx,values,x,y,radius);}// restore the dash settings.
	ctx.setLineDash([0]);ctx.lineDashOffset=0;ctx.restore();}else {// unsupporting smooth lines
	if(this.from!=this.to){// draw line
	drawDashedLine(ctx,this.from.x,this.from.y,this.to.x,this.to.y,pattern);}else {var _this$_getCircleData5=this._getCircleData(ctx),_this$_getCircleData6=slicedToArray(_this$_getCircleData5,3),_x=_this$_getCircleData6[0],_y=_this$_getCircleData6[1],_radius=_this$_getCircleData6[2];this._circle(ctx,values,_x,_y,_radius);}// draw shadow if enabled
	this.enableShadow(ctx,values);ctx.stroke();// disable shadows for other elements.
	this.disableShadow(ctx,values);}}/**
	     * Find the intersection between the border of the node and the edge.
	     *
	     * @param node - The node (either from or to node of the edge).
	     * @param ctx - The context that will be used for rendering.
	     * @param options - Additional options.
	     *
	     * @returns Cartesian coordinates of the intersection between the border of the node and the edge.
	     */},{key:"findBorderPosition",value:function findBorderPosition(node,ctx,options){if(this.from!=this.to){return this._findBorderPosition(node,ctx,options);}else {return this._findBorderPositionCircle(node,ctx,options);}}/** @inheritdoc */},{key:"findBorderPositions",value:function findBorderPositions(ctx){if(this.from!=this.to){return {from:this._findBorderPosition(this.from,ctx),to:this._findBorderPosition(this.to,ctx)};}else {var _context25;var _this$_getCircleData$=slice$3(_context25=this._getCircleData(ctx)).call(_context25,0,2),_this$_getCircleData$2=slicedToArray(_this$_getCircleData$,2),x=_this$_getCircleData$2[0],y=_this$_getCircleData$2[1];return {from:this._findBorderPositionCircle(this.from,ctx,{x:x,y:y,low:0.25,high:0.6,direction:-1}),to:this._findBorderPositionCircle(this.from,ctx,{x:x,y:y,low:0.6,high:0.8,direction:1})};}}/**
	     * Compute the center point and radius of an edge connected to the same node at both ends.
	     *
	     * @param ctx - The context that will be used for rendering.
	     *
	     * @returns `[x, y, radius]`
	     */},{key:"_getCircleData",value:function _getCircleData(ctx){var radius=this.options.selfReference.size;if(ctx!==undefined){if(this.from.shape.width===undefined){this.from.shape.resize(ctx);}}// get circle coordinates
	var coordinates=ComponentUtil.getSelfRefCoordinates(ctx,this.options.selfReference.angle,radius,this.from);return [coordinates.x,coordinates.y,radius];}/**
	     * Get a point on a circle.
	     *
	     * @param x - Center of the circle on the x axis.
	     * @param y - Center of the circle on the y axis.
	     * @param radius - Radius of the circle.
	     * @param position - Value between 0 (line start) and 1 (line end).
	     *
	     * @returns Cartesian coordinates of requested point on the circle.
	     */},{key:"_pointOnCircle",value:function _pointOnCircle(x,y,radius,position){var angle=position*2*Math.PI;return {x:x+radius*Math.cos(angle),y:y-radius*Math.sin(angle)};}/**
	     * Find the intersection between the border of the node and the edge.
	     *
	     * @remarks
	     * This function uses binary search to look for the point where the circle crosses the border of the node.
	     *
	     * @param nearNode - The node (either from or to node of the edge).
	     * @param ctx - The context that will be used for rendering.
	     * @param options - Additional options.
	     *
	     * @returns Cartesian coordinates of the intersection between the border of the node and the edge.
	     */},{key:"_findBorderPositionCircle",value:function _findBorderPositionCircle(nearNode,ctx,options){var x=options.x;var y=options.y;var low=options.low;var high=options.high;var direction=options.direction;var maxIterations=10;var radius=this.options.selfReference.size;var threshold=0.05;var pos;var middle=(low+high)*0.5;var iteration=0;do{middle=(low+high)*0.5;pos=this._pointOnCircle(x,y,radius,middle);var angle=Math.atan2(nearNode.y-pos.y,nearNode.x-pos.x);var distanceToBorder=nearNode.distanceToBorder(ctx,angle);var distanceToPoint=Math.sqrt(Math.pow(pos.x-nearNode.x,2)+Math.pow(pos.y-nearNode.y,2));var difference=distanceToBorder-distanceToPoint;if(Math.abs(difference)<threshold){break;// found
	}else if(difference>0){// distance to nodes is larger than distance to border --> t needs to be bigger if we're looking at the to node.
	if(direction>0){low=middle;}else {high=middle;}}else {if(direction>0){high=middle;}else {low=middle;}}++iteration;}while(low<=high&&iteration<maxIterations);return _objectSpread$2(_objectSpread$2({},pos),{},{t:middle});}/**
	     * Get the line width of the edge. Depends on width and whether one of the connected nodes is selected.
	     *
	     * @param selected - Determines wheter the line is selected.
	     * @param hover - Determines wheter the line is being hovered, only applies if selected is false.
	     *
	     * @returns The width of the line.
	     */},{key:"getLineWidth",value:function getLineWidth(selected,hover){if(selected===true){return Math.max(this.selectionWidth,0.3/this._body.view.scale);}else if(hover===true){return Math.max(this.hoverWidth,0.3/this._body.view.scale);}else {return Math.max(this.options.width,0.3/this._body.view.scale);}}/**
	     * Compute the color or gradient for given edge.
	     *
	     * @param ctx - The context that will be used for rendering.
	     * @param values - Formatting values like color, opacity or shadow.
	     * @param _selected - Ignored (TODO: remove in the future).
	     * @param _hover - Ignored (TODO: remove in the future).
	     *
	     * @returns Color string if single color is inherited or gradient if two.
	     */},{key:"getColor",value:function getColor(ctx,values){if(values.inheritsColor!==false){// when this is a loop edge, just use the 'from' method
	if(values.inheritsColor==="both"&&this.from.id!==this.to.id){var grd=ctx.createLinearGradient(this.from.x,this.from.y,this.to.x,this.to.y);var fromColor=this.from.options.color.highlight.border;var toColor=this.to.options.color.highlight.border;if(this.from.selected===false&&this.to.selected===false){fromColor=overrideOpacity(this.from.options.color.border,values.opacity);toColor=overrideOpacity(this.to.options.color.border,values.opacity);}else if(this.from.selected===true&&this.to.selected===false){toColor=this.to.options.color.border;}else if(this.from.selected===false&&this.to.selected===true){fromColor=this.from.options.color.border;}grd.addColorStop(0,fromColor);grd.addColorStop(1,toColor);// -------------------- this returns -------------------- //
	return grd;}if(values.inheritsColor==="to"){return overrideOpacity(this.to.options.color.border,values.opacity);}else {// "from"
	return overrideOpacity(this.from.options.color.border,values.opacity);}}else {return overrideOpacity(values.color,values.opacity);}}/**
	     * Draw a line from a node to itself, a circle.
	     *
	     * @param ctx - The context that will be used for rendering.
	     * @param values - Formatting values like color, opacity or shadow.
	     * @param x - Center of the circle on the x axis.
	     * @param y - Center of the circle on the y axis.
	     * @param radius - Radius of the circle.
	     */},{key:"_circle",value:function _circle(ctx,values,x,y,radius){// draw shadow if enabled
	this.enableShadow(ctx,values);//full circle
	var angleFrom=0;var angleTo=Math.PI*2;if(!this.options.selfReference.renderBehindTheNode){//render only parts which are not overlaping with parent node
	//need to find x,y of from point and x,y to point
	//calculating radians
	var low=this.options.selfReference.angle;var high=this.options.selfReference.angle+Math.PI;var pointTFrom=this._findBorderPositionCircle(this.from,ctx,{x:x,y:y,low:low,high:high,direction:-1});var pointTTo=this._findBorderPositionCircle(this.from,ctx,{x:x,y:y,low:low,high:high,direction:1});angleFrom=Math.atan2(pointTFrom.y-y,pointTFrom.x-x);angleTo=Math.atan2(pointTTo.y-y,pointTTo.x-x);}// draw a circle
	ctx.beginPath();ctx.arc(x,y,radius,angleFrom,angleTo,false);ctx.stroke();// disable shadows for other elements.
	this.disableShadow(ctx,values);}/**
	     * @inheritdoc
	     *
	     * @remarks
	     * http://stackoverflow.com/questions/849211/shortest-distancae-between-a-point-and-a-line-segment
	     */},{key:"getDistanceToEdge",value:function getDistanceToEdge(x1,y1,x2,y2,x3,y3){if(this.from!=this.to){return this._getDistanceToEdge(x1,y1,x2,y2,x3,y3);}else {var _this$_getCircleData7=this._getCircleData(undefined),_this$_getCircleData8=slicedToArray(_this$_getCircleData7,3),x=_this$_getCircleData8[0],y=_this$_getCircleData8[1],radius=_this$_getCircleData8[2];var dx=x-x3;var dy=y-y3;return Math.abs(Math.sqrt(dx*dx+dy*dy)-radius);}}/**
	     * Calculate the distance between a point (x3, y3) and a line segment from (x1, y1) to (x2, y2).
	     *
	     * @param x1 - First end of the line segment on the x axis.
	     * @param y1 - First end of the line segment on the y axis.
	     * @param x2 - Second end of the line segment on the x axis.
	     * @param y2 - Second end of the line segment on the y axis.
	     * @param x3 - Position of the point on the x axis.
	     * @param y3 - Position of the point on the y axis.
	     *
	     * @returns The distance between the line segment and the point.
	     */},{key:"_getDistanceToLine",value:function _getDistanceToLine(x1,y1,x2,y2,x3,y3){var px=x2-x1;var py=y2-y1;var something=px*px+py*py;var u=((x3-x1)*px+(y3-y1)*py)/something;if(u>1){u=1;}else if(u<0){u=0;}var x=x1+u*px;var y=y1+u*py;var dx=x-x3;var dy=y-y3;//# Note: If the actual distance does not matter,
	//# if you only want to compare what this function
	//# returns to other results of this function, you
	//# can just return the squared distance instead
	//# (i.e. remove the sqrt) to gain a little performance
	return Math.sqrt(dx*dx+dy*dy);}/** @inheritdoc */},{key:"getArrowData",value:function getArrowData(ctx,position,viaNode,_selected,_hover,values){// set lets
	var angle;var arrowPoint;var node1;var node2;var reversed;var scaleFactor;var type;var lineWidth=values.width;if(position==="from"){node1=this.from;node2=this.to;reversed=values.fromArrowScale<0;scaleFactor=Math.abs(values.fromArrowScale);type=values.fromArrowType;}else if(position==="to"){node1=this.to;node2=this.from;reversed=values.toArrowScale<0;scaleFactor=Math.abs(values.toArrowScale);type=values.toArrowType;}else {node1=this.to;node2=this.from;reversed=values.middleArrowScale<0;scaleFactor=Math.abs(values.middleArrowScale);type=values.middleArrowType;}var length=15*scaleFactor+3*lineWidth;// 3* lineWidth is the width of the edge.
	// if not connected to itself
	if(node1!=node2){var approximateEdgeLength=hypot$2(node1.x-node2.x,node1.y-node2.y);var relativeLength=length/approximateEdgeLength;if(position!=="middle"){// draw arrow head
	if(this.options.smooth.enabled===true){var pointT=this._findBorderPosition(node1,ctx,{via:viaNode});var guidePos=this.getPoint(pointT.t+relativeLength*(position==="from"?1:-1),viaNode);angle=Math.atan2(pointT.y-guidePos.y,pointT.x-guidePos.x);arrowPoint=pointT;}else {angle=Math.atan2(node1.y-node2.y,node1.x-node2.x);arrowPoint=this._findBorderPosition(node1,ctx);}}else {// Negative half length reverses arrow direction.
	var halfLength=(reversed?-relativeLength:relativeLength)/2;var guidePos1=this.getPoint(0.5+halfLength,viaNode);var guidePos2=this.getPoint(0.5-halfLength,viaNode);angle=Math.atan2(guidePos1.y-guidePos2.y,guidePos1.x-guidePos2.x);arrowPoint=this.getPoint(0.5,viaNode);}}else {// draw circle
	var _this$_getCircleData9=this._getCircleData(ctx),_this$_getCircleData10=slicedToArray(_this$_getCircleData9,3),x=_this$_getCircleData10[0],y=_this$_getCircleData10[1],radius=_this$_getCircleData10[2];if(position==="from"){var low=this.options.selfReference.angle;var high=this.options.selfReference.angle+Math.PI;var _pointT=this._findBorderPositionCircle(this.from,ctx,{x:x,y:y,low:low,high:high,direction:-1});angle=_pointT.t*-2*Math.PI+1.5*Math.PI+0.1*Math.PI;arrowPoint=_pointT;}else if(position==="to"){var _low=this.options.selfReference.angle;var _high=this.options.selfReference.angle+Math.PI;var _pointT2=this._findBorderPositionCircle(this.from,ctx,{x:x,y:y,low:_low,high:_high,direction:1});angle=_pointT2.t*-2*Math.PI+1.5*Math.PI-1.1*Math.PI;arrowPoint=_pointT2;}else {var pos=this.options.selfReference.angle/(2*Math.PI);arrowPoint=this._pointOnCircle(x,y,radius,pos);angle=pos*-2*Math.PI+1.5*Math.PI+0.1*Math.PI;}}var xi=arrowPoint.x-length*0.9*Math.cos(angle);var yi=arrowPoint.y-length*0.9*Math.sin(angle);var arrowCore={x:xi,y:yi};return {point:arrowPoint,core:arrowCore,angle:angle,length:length,type:type};}/** @inheritdoc */},{key:"drawArrowHead",value:function drawArrowHead(ctx,values,_selected,_hover,arrowData){// set style
	ctx.strokeStyle=this.getColor(ctx,values);ctx.fillStyle=ctx.strokeStyle;ctx.lineWidth=values.width;var canFill=EndPoints.draw(ctx,arrowData);if(canFill){// draw shadow if enabled
	this.enableShadow(ctx,values);fill$2(ctx).call(ctx);// disable shadows for other elements.
	this.disableShadow(ctx,values);}}/**
	     * Set the shadow formatting values in the context if enabled, do nothing otherwise.
	     *
	     * @param ctx - The context that will be used for rendering.
	     * @param values - Formatting values for the shadow.
	     */},{key:"enableShadow",value:function enableShadow(ctx,values){if(values.shadow===true){ctx.shadowColor=values.shadowColor;ctx.shadowBlur=values.shadowSize;ctx.shadowOffsetX=values.shadowX;ctx.shadowOffsetY=values.shadowY;}}/**
	     * Reset the shadow formatting values in the context if enabled, do nothing otherwise.
	     *
	     * @param ctx - The context that will be used for rendering.
	     * @param values - Formatting values for the shadow.
	     */},{key:"disableShadow",value:function disableShadow(ctx,values){if(values.shadow===true){ctx.shadowColor="rgba(0,0,0,0)";ctx.shadowBlur=0;ctx.shadowOffsetX=0;ctx.shadowOffsetY=0;}}/**
	     * Render the background according to the formatting values.
	     *
	     * @param ctx - The context that will be used for rendering.
	     * @param values - Formatting values for the background.
	     */},{key:"drawBackground",value:function drawBackground(ctx,values){if(values.background!==false){// save original line attrs
	var origCtxAttr={strokeStyle:ctx.strokeStyle,lineWidth:ctx.lineWidth,dashes:ctx.dashes};ctx.strokeStyle=values.backgroundColor;ctx.lineWidth=values.backgroundSize;this.setStrokeDashed(ctx,values.backgroundDashes);ctx.stroke();// restore original line attrs
	ctx.strokeStyle=origCtxAttr.strokeStyle;ctx.lineWidth=origCtxAttr.lineWidth;ctx.dashes=origCtxAttr.dashes;this.setStrokeDashed(ctx,values.dashes);}}/**
	     * Set the line dash pattern if supported. Logs a warning to the console if it isn't supported.
	     *
	     * @param ctx - The context that will be used for rendering.
	     * @param dashes - The pattern [line, space, line…], true for default dashed line or false for normal line.
	     */},{key:"setStrokeDashed",value:function setStrokeDashed(ctx,dashes){if(dashes!==false){if(ctx.setLineDash!==undefined){var pattern=isArray$3(dashes)?dashes:[5,5];ctx.setLineDash(pattern);}else {console.warn("setLineDash is not supported in this browser. The dashed stroke cannot be used.");}}else {if(ctx.setLineDash!==undefined){ctx.setLineDash([]);}else {console.warn("setLineDash is not supported in this browser. The dashed stroke cannot be used.");}}}}]);return EdgeBase;}();/**
	 * The Base Class for all Bezier edges.
	 * Bezier curves are used to model smooth gradual curves in paths between nodes.
	 */var BezierEdgeBase=/*#__PURE__*/function(_EdgeBase){inherits(BezierEdgeBase,_EdgeBase);var _super20=_createSuper$1(BezierEdgeBase);/**
	     * Create a new instance.
	     *
	     * @param options - The options object of given edge.
	     * @param body - The body of the network.
	     * @param labelModule - Label module.
	     */function BezierEdgeBase(options,body,labelModule){classCallCheck(this,BezierEdgeBase);return _super20.call(this,options,body,labelModule);}/**
	     * Find the intersection between the border of the node and the edge.
	     *
	     * @remarks
	     * This function uses binary search to look for the point where the bezier curve crosses the border of the node.
	     *
	     * @param nearNode - The node (either from or to node of the edge).
	     * @param ctx - The context that will be used for rendering.
	     * @param viaNode - Additional node(s) the edge passes through.
	     *
	     * @returns Cartesian coordinates of the intersection between the border of the node and the edge.
	     */createClass(BezierEdgeBase,[{key:"_findBorderPositionBezier",value:function _findBorderPositionBezier(nearNode,ctx){var viaNode=arguments.length>2&&arguments[2]!==undefined?arguments[2]:this._getViaCoordinates();var maxIterations=10;var threshold=0.2;var from=false;var high=1;var low=0;var node=this.to;var pos;var middle;if(nearNode.id===this.from.id){node=this.from;from=true;}var iteration=0;do{middle=(low+high)*0.5;pos=this.getPoint(middle,viaNode);var angle=Math.atan2(node.y-pos.y,node.x-pos.x);var distanceToBorder=node.distanceToBorder(ctx,angle);var distanceToPoint=Math.sqrt(Math.pow(pos.x-node.x,2)+Math.pow(pos.y-node.y,2));var difference=distanceToBorder-distanceToPoint;if(Math.abs(difference)<threshold){break;// found
	}else if(difference<0){// distance to nodes is larger than distance to border --> t needs to be bigger if we're looking at the to node.
	if(from===false){low=middle;}else {high=middle;}}else {if(from===false){high=middle;}else {low=middle;}}++iteration;}while(low<=high&&iteration<maxIterations);return _objectSpread$2(_objectSpread$2({},pos),{},{t:middle});}/**
	     * Calculate the distance between a point (x3,y3) and a line segment from (x1,y1) to (x2,y2).
	     *
	     * @remarks
	     * http://stackoverflow.com/questions/849211/shortest-distancae-between-a-point-and-a-line-segment
	     *
	     * @param x1 - First end of the line segment on the x axis.
	     * @param y1 - First end of the line segment on the y axis.
	     * @param x2 - Second end of the line segment on the x axis.
	     * @param y2 - Second end of the line segment on the y axis.
	     * @param x3 - Position of the point on the x axis.
	     * @param y3 - Position of the point on the y axis.
	     * @param via - The control point for the edge.
	     *
	     * @returns The distance between the line segment and the point.
	     */},{key:"_getDistanceToBezierEdge",value:function _getDistanceToBezierEdge(x1,y1,x2,y2,x3,y3,via){// x3,y3 is the point
	var minDistance=1e9;var distance;var i,t,x,y;var lastX=x1;var lastY=y1;for(i=1;i<10;i++){t=0.1*i;x=Math.pow(1-t,2)*x1+2*t*(1-t)*via.x+Math.pow(t,2)*x2;y=Math.pow(1-t,2)*y1+2*t*(1-t)*via.y+Math.pow(t,2)*y2;if(i>0){distance=this._getDistanceToLine(lastX,lastY,x,y,x3,y3);minDistance=distance<minDistance?distance:minDistance;}lastX=x;lastY=y;}return minDistance;}/**
	     * Render a bezier curve between two nodes.
	     *
	     * @remarks
	     * The method accepts zero, one or two control points.
	     * Passing zero control points just draws a straight line.
	     *
	     * @param ctx - The context that will be used for rendering.
	     * @param values - Style options for edge drawing.
	     * @param viaNode1 - First control point for curve drawing.
	     * @param viaNode2 - Second control point for curve drawing.
	     */},{key:"_bezierCurve",value:function _bezierCurve(ctx,values,viaNode1,viaNode2){ctx.beginPath();ctx.moveTo(this.fromPoint.x,this.fromPoint.y);if(viaNode1!=null&&viaNode1.x!=null){if(viaNode2!=null&&viaNode2.x!=null){ctx.bezierCurveTo(viaNode1.x,viaNode1.y,viaNode2.x,viaNode2.y,this.toPoint.x,this.toPoint.y);}else {ctx.quadraticCurveTo(viaNode1.x,viaNode1.y,this.toPoint.x,this.toPoint.y);}}else {// fallback to normal straight edge
	ctx.lineTo(this.toPoint.x,this.toPoint.y);}// draw a background
	this.drawBackground(ctx,values);// draw shadow if enabled
	this.enableShadow(ctx,values);ctx.stroke();this.disableShadow(ctx,values);}/** @inheritdoc */},{key:"getViaNode",value:function getViaNode(){return this._getViaCoordinates();}}]);return BezierEdgeBase;}(EdgeBase);/**
	 * A Dynamic Bezier Edge. Bezier curves are used to model smooth gradual
	 * curves in paths between nodes. The Dynamic piece refers to how the curve
	 * reacts to physics changes.
	 *
	 * @extends BezierEdgeBase
	 */var BezierEdgeDynamic=/*#__PURE__*/function(_BezierEdgeBase){inherits(BezierEdgeDynamic,_BezierEdgeBase);var _super21=_createSuper$1(BezierEdgeDynamic);/**
	     * Create a new instance.
	     *
	     * @param options - The options object of given edge.
	     * @param body - The body of the network.
	     * @param labelModule - Label module.
	     */function BezierEdgeDynamic(options,body,labelModule){var _this19;classCallCheck(this,BezierEdgeDynamic);//this.via = undefined; // Here for completeness but not allowed to defined before super() is invoked.
	_this19=_super21.call(this,options,body,labelModule);// --> this calls the setOptions below
	_this19.via=_this19.via;// constructor → super → super → setOptions → setupSupportNode
	_this19._boundFunction=function(){_this19.positionBezierNode();};_this19._body.emitter.on("_repositionBezierNodes",_this19._boundFunction);return _this19;}/** @inheritdoc */createClass(BezierEdgeDynamic,[{key:"setOptions",value:function setOptions(options){get$5(getPrototypeOf$3(BezierEdgeDynamic.prototype),"setOptions",this).call(this,options);// check if the physics has changed.
	var physicsChange=false;if(this.options.physics!==options.physics){physicsChange=true;}// set the options and the to and from nodes
	this.options=options;this.id=this.options.id;this.from=this._body.nodes[this.options.from];this.to=this._body.nodes[this.options.to];// setup the support node and connect
	this.setupSupportNode();this.connect();// when we change the physics state of the edge, we reposition the support node.
	if(physicsChange===true){this.via.setOptions({physics:this.options.physics});this.positionBezierNode();}}/** @inheritdoc */},{key:"connect",value:function connect(){this.from=this._body.nodes[this.options.from];this.to=this._body.nodes[this.options.to];if(this.from===undefined||this.to===undefined||this.options.physics===false){this.via.setOptions({physics:false});}else {// fix weird behaviour where a self referencing node has physics enabled
	if(this.from.id===this.to.id){this.via.setOptions({physics:false});}else {this.via.setOptions({physics:true});}}}/** @inheritdoc */},{key:"cleanup",value:function cleanup(){this._body.emitter.off("_repositionBezierNodes",this._boundFunction);if(this.via!==undefined){delete this._body.nodes[this.via.id];this.via=undefined;return true;}return false;}/**
	     * Create and add a support node if not already present.
	     *
	     * @remarks
	     * Bezier curves require an anchor point to calculate the smooth flow.
	     * These points are nodes.
	     * These nodes are invisible but are used for the force calculation.
	     *
	     * The changed data is not called, if needed, it is returned by the main edge constructor.
	     */},{key:"setupSupportNode",value:function setupSupportNode(){if(this.via===undefined){var nodeId="edgeId:"+this.id;var node=this._body.functions.createNode({id:nodeId,shape:"circle",physics:true,hidden:true});this._body.nodes[nodeId]=node;this.via=node;this.via.parentEdgeId=this.id;this.positionBezierNode();}}/**
	     * Position bezier node.
	     */},{key:"positionBezierNode",value:function positionBezierNode(){if(this.via!==undefined&&this.from!==undefined&&this.to!==undefined){this.via.x=0.5*(this.from.x+this.to.x);this.via.y=0.5*(this.from.y+this.to.y);}else if(this.via!==undefined){this.via.x=0;this.via.y=0;}}/** @inheritdoc */},{key:"_line",value:function _line(ctx,values,viaNode){this._bezierCurve(ctx,values,viaNode);}/** @inheritdoc */},{key:"_getViaCoordinates",value:function _getViaCoordinates(){return this.via;}/** @inheritdoc */},{key:"getViaNode",value:function getViaNode(){return this.via;}/** @inheritdoc */},{key:"getPoint",value:function getPoint(position){var viaNode=arguments.length>1&&arguments[1]!==undefined?arguments[1]:this.via;if(this.from===this.to){var _this$_getCircleData11=this._getCircleData(),_this$_getCircleData12=slicedToArray(_this$_getCircleData11,3),cx=_this$_getCircleData12[0],cy=_this$_getCircleData12[1],cr=_this$_getCircleData12[2];var a=2*Math.PI*(1-position);return {x:cx+cr*Math.sin(a),y:cy+cr-cr*(1-Math.cos(a))};}else {return {x:Math.pow(1-position,2)*this.fromPoint.x+2*position*(1-position)*viaNode.x+Math.pow(position,2)*this.toPoint.x,y:Math.pow(1-position,2)*this.fromPoint.y+2*position*(1-position)*viaNode.y+Math.pow(position,2)*this.toPoint.y};}}/** @inheritdoc */},{key:"_findBorderPosition",value:function _findBorderPosition(nearNode,ctx){return this._findBorderPositionBezier(nearNode,ctx,this.via);}/** @inheritdoc */},{key:"_getDistanceToEdge",value:function _getDistanceToEdge(x1,y1,x2,y2,x3,y3){// x3,y3 is the point
	return this._getDistanceToBezierEdge(x1,y1,x2,y2,x3,y3,this.via);}}]);return BezierEdgeDynamic;}(BezierEdgeBase);/**
	 * A Static Bezier Edge. Bezier curves are used to model smooth gradual curves in paths between nodes.
	 */var BezierEdgeStatic=/*#__PURE__*/function(_BezierEdgeBase2){inherits(BezierEdgeStatic,_BezierEdgeBase2);var _super22=_createSuper$1(BezierEdgeStatic);/**
	     * Create a new instance.
	     *
	     * @param options - The options object of given edge.
	     * @param body - The body of the network.
	     * @param labelModule - Label module.
	     */function BezierEdgeStatic(options,body,labelModule){classCallCheck(this,BezierEdgeStatic);return _super22.call(this,options,body,labelModule);}/** @inheritdoc */createClass(BezierEdgeStatic,[{key:"_line",value:function _line(ctx,values,viaNode){this._bezierCurve(ctx,values,viaNode);}/** @inheritdoc */},{key:"getViaNode",value:function getViaNode(){return this._getViaCoordinates();}/**
	     * Compute the coordinates of the via node.
	     *
	     * @remarks
	     * We do not use the to and fromPoints here to make the via nodes the same as edges without arrows.
	     *
	     * @returns Cartesian coordinates of the via node.
	     */},{key:"_getViaCoordinates",value:function _getViaCoordinates(){// Assumption: x/y coordinates in from/to always defined
	var factor=this.options.smooth.roundness;var type=this.options.smooth.type;var dx=Math.abs(this.from.x-this.to.x);var dy=Math.abs(this.from.y-this.to.y);if(type==="discrete"||type==="diagonalCross"){var stepX;var stepY;if(dx<=dy){stepX=stepY=factor*dy;}else {stepX=stepY=factor*dx;}if(this.from.x>this.to.x){stepX=-stepX;}if(this.from.y>=this.to.y){stepY=-stepY;}var xVia=this.from.x+stepX;var yVia=this.from.y+stepY;if(type==="discrete"){if(dx<=dy){xVia=dx<factor*dy?this.from.x:xVia;}else {yVia=dy<factor*dx?this.from.y:yVia;}}return {x:xVia,y:yVia};}else if(type==="straightCross"){var _stepX=(1-factor)*dx;var _stepY=(1-factor)*dy;if(dx<=dy){// up - down
	_stepX=0;if(this.from.y<this.to.y){_stepY=-_stepY;}}else {// left - right
	if(this.from.x<this.to.x){_stepX=-_stepX;}_stepY=0;}return {x:this.to.x+_stepX,y:this.to.y+_stepY};}else if(type==="horizontal"){var _stepX2=(1-factor)*dx;if(this.from.x<this.to.x){_stepX2=-_stepX2;}return {x:this.to.x+_stepX2,y:this.from.y};}else if(type==="vertical"){var _stepY2=(1-factor)*dy;if(this.from.y<this.to.y){_stepY2=-_stepY2;}return {x:this.from.x,y:this.to.y+_stepY2};}else if(type==="curvedCW"){dx=this.to.x-this.from.x;dy=this.from.y-this.to.y;var radius=Math.sqrt(dx*dx+dy*dy);var pi=Math.PI;var originalAngle=Math.atan2(dy,dx);var myAngle=(originalAngle+(factor*0.5+0.5)*pi)%(2*pi);return {x:this.from.x+(factor*0.5+0.5)*radius*Math.sin(myAngle),y:this.from.y+(factor*0.5+0.5)*radius*Math.cos(myAngle)};}else if(type==="curvedCCW"){dx=this.to.x-this.from.x;dy=this.from.y-this.to.y;var _radius2=Math.sqrt(dx*dx+dy*dy);var _pi=Math.PI;var _originalAngle=Math.atan2(dy,dx);var _myAngle=(_originalAngle+(-factor*0.5+0.5)*_pi)%(2*_pi);return {x:this.from.x+(factor*0.5+0.5)*_radius2*Math.sin(_myAngle),y:this.from.y+(factor*0.5+0.5)*_radius2*Math.cos(_myAngle)};}else {// continuous
	var _stepX3;var _stepY3;if(dx<=dy){_stepX3=_stepY3=factor*dy;}else {_stepX3=_stepY3=factor*dx;}if(this.from.x>this.to.x){_stepX3=-_stepX3;}if(this.from.y>=this.to.y){_stepY3=-_stepY3;}var _xVia=this.from.x+_stepX3;var _yVia=this.from.y+_stepY3;if(dx<=dy){if(this.from.x<=this.to.x){_xVia=this.to.x<_xVia?this.to.x:_xVia;}else {_xVia=this.to.x>_xVia?this.to.x:_xVia;}}else {if(this.from.y>=this.to.y){_yVia=this.to.y>_yVia?this.to.y:_yVia;}else {_yVia=this.to.y<_yVia?this.to.y:_yVia;}}return {x:_xVia,y:_yVia};}}/** @inheritdoc */},{key:"_findBorderPosition",value:function _findBorderPosition(nearNode,ctx){var options=arguments.length>2&&arguments[2]!==undefined?arguments[2]:{};return this._findBorderPositionBezier(nearNode,ctx,options.via);}/** @inheritdoc */},{key:"_getDistanceToEdge",value:function _getDistanceToEdge(x1,y1,x2,y2,x3,y3){var viaNode=arguments.length>6&&arguments[6]!==undefined?arguments[6]:this._getViaCoordinates();// x3,y3 is the point
	return this._getDistanceToBezierEdge(x1,y1,x2,y2,x3,y3,viaNode);}/** @inheritdoc */},{key:"getPoint",value:function getPoint(position){var viaNode=arguments.length>1&&arguments[1]!==undefined?arguments[1]:this._getViaCoordinates();var t=position;var x=Math.pow(1-t,2)*this.fromPoint.x+2*t*(1-t)*viaNode.x+Math.pow(t,2)*this.toPoint.x;var y=Math.pow(1-t,2)*this.fromPoint.y+2*t*(1-t)*viaNode.y+Math.pow(t,2)*this.toPoint.y;return {x:x,y:y};}}]);return BezierEdgeStatic;}(BezierEdgeBase);/**
	 * A Base Class for all Cubic Bezier Edges. Bezier curves are used to model
	 * smooth gradual curves in paths between nodes.
	 *
	 * @extends BezierEdgeBase
	 */var CubicBezierEdgeBase=/*#__PURE__*/function(_BezierEdgeBase3){inherits(CubicBezierEdgeBase,_BezierEdgeBase3);var _super23=_createSuper$1(CubicBezierEdgeBase);/**
	     * Create a new instance.
	     *
	     * @param options - The options object of given edge.
	     * @param body - The body of the network.
	     * @param labelModule - Label module.
	     */function CubicBezierEdgeBase(options,body,labelModule){classCallCheck(this,CubicBezierEdgeBase);return _super23.call(this,options,body,labelModule);}/**
	     * Calculate the distance between a point (x3,y3) and a line segment from (x1,y1) to (x2,y2).
	     *
	     * @remarks
	     * http://stackoverflow.com/questions/849211/shortest-distancae-between-a-point-and-a-line-segment
	     * https://en.wikipedia.org/wiki/B%C3%A9zier_curve
	     *
	     * @param x1 - First end of the line segment on the x axis.
	     * @param y1 - First end of the line segment on the y axis.
	     * @param x2 - Second end of the line segment on the x axis.
	     * @param y2 - Second end of the line segment on the y axis.
	     * @param x3 - Position of the point on the x axis.
	     * @param y3 - Position of the point on the y axis.
	     * @param via1 - The first point this edge passes through.
	     * @param via2 - The second point this edge passes through.
	     *
	     * @returns The distance between the line segment and the point.
	     */createClass(CubicBezierEdgeBase,[{key:"_getDistanceToBezierEdge2",value:function _getDistanceToBezierEdge2(x1,y1,x2,y2,x3,y3,via1,via2){// x3,y3 is the point
	var minDistance=1e9;var lastX=x1;var lastY=y1;var vec=[0,0,0,0];for(var i=1;i<10;i++){var t=0.1*i;vec[0]=Math.pow(1-t,3);vec[1]=3*t*Math.pow(1-t,2);vec[2]=3*Math.pow(t,2)*(1-t);vec[3]=Math.pow(t,3);var x=vec[0]*x1+vec[1]*via1.x+vec[2]*via2.x+vec[3]*x2;var y=vec[0]*y1+vec[1]*via1.y+vec[2]*via2.y+vec[3]*y2;if(i>0){var distance=this._getDistanceToLine(lastX,lastY,x,y,x3,y3);minDistance=distance<minDistance?distance:minDistance;}lastX=x;lastY=y;}return minDistance;}}]);return CubicBezierEdgeBase;}(BezierEdgeBase);/**
	 * A Cubic Bezier Edge. Bezier curves are used to model smooth gradual curves in paths between nodes.
	 */var CubicBezierEdge=/*#__PURE__*/function(_CubicBezierEdgeBase){inherits(CubicBezierEdge,_CubicBezierEdgeBase);var _super24=_createSuper$1(CubicBezierEdge);/**
	     * Create a new instance.
	     *
	     * @param options - The options object of given edge.
	     * @param body - The body of the network.
	     * @param labelModule - Label module.
	     */function CubicBezierEdge(options,body,labelModule){classCallCheck(this,CubicBezierEdge);return _super24.call(this,options,body,labelModule);}/** @inheritdoc */createClass(CubicBezierEdge,[{key:"_line",value:function _line(ctx,values,viaNodes){// get the coordinates of the support points.
	var via1=viaNodes[0];var via2=viaNodes[1];this._bezierCurve(ctx,values,via1,via2);}/**
	     * Compute the additional points the edge passes through.
	     *
	     * @returns Cartesian coordinates of the points the edge passes through.
	     */},{key:"_getViaCoordinates",value:function _getViaCoordinates(){var dx=this.from.x-this.to.x;var dy=this.from.y-this.to.y;var x1;var y1;var x2;var y2;var roundness=this.options.smooth.roundness;// horizontal if x > y or if direction is forced or if direction is horizontal
	if((Math.abs(dx)>Math.abs(dy)||this.options.smooth.forceDirection===true||this.options.smooth.forceDirection==="horizontal")&&this.options.smooth.forceDirection!=="vertical"){y1=this.from.y;y2=this.to.y;x1=this.from.x-roundness*dx;x2=this.to.x+roundness*dx;}else {y1=this.from.y-roundness*dy;y2=this.to.y+roundness*dy;x1=this.from.x;x2=this.to.x;}return [{x:x1,y:y1},{x:x2,y:y2}];}/** @inheritdoc */},{key:"getViaNode",value:function getViaNode(){return this._getViaCoordinates();}/** @inheritdoc */},{key:"_findBorderPosition",value:function _findBorderPosition(nearNode,ctx){return this._findBorderPositionBezier(nearNode,ctx);}/** @inheritdoc */},{key:"_getDistanceToEdge",value:function _getDistanceToEdge(x1,y1,x2,y2,x3,y3){var _ref=arguments.length>6&&arguments[6]!==undefined?arguments[6]:this._getViaCoordinates(),_ref2=slicedToArray(_ref,2),via1=_ref2[0],via2=_ref2[1];// x3,y3 is the point
	return this._getDistanceToBezierEdge2(x1,y1,x2,y2,x3,y3,via1,via2);}/** @inheritdoc */},{key:"getPoint",value:function getPoint(position){var _ref3=arguments.length>1&&arguments[1]!==undefined?arguments[1]:this._getViaCoordinates(),_ref4=slicedToArray(_ref3,2),via1=_ref4[0],via2=_ref4[1];var t=position;var vec=[Math.pow(1-t,3),3*t*Math.pow(1-t,2),3*Math.pow(t,2)*(1-t),Math.pow(t,3)];var x=vec[0]*this.fromPoint.x+vec[1]*via1.x+vec[2]*via2.x+vec[3]*this.toPoint.x;var y=vec[0]*this.fromPoint.y+vec[1]*via1.y+vec[2]*via2.y+vec[3]*this.toPoint.y;return {x:x,y:y};}}]);return CubicBezierEdge;}(CubicBezierEdgeBase);/**
	 * A Straight Edge.
	 */var StraightEdge=/*#__PURE__*/function(_EdgeBase2){inherits(StraightEdge,_EdgeBase2);var _super25=_createSuper$1(StraightEdge);/**
	     * Create a new instance.
	     *
	     * @param options - The options object of given edge.
	     * @param body - The body of the network.
	     * @param labelModule - Label module.
	     */function StraightEdge(options,body,labelModule){classCallCheck(this,StraightEdge);return _super25.call(this,options,body,labelModule);}/** @inheritdoc */createClass(StraightEdge,[{key:"_line",value:function _line(ctx,values){// draw a straight line
	ctx.beginPath();ctx.moveTo(this.fromPoint.x,this.fromPoint.y);ctx.lineTo(this.toPoint.x,this.toPoint.y);// draw shadow if enabled
	this.enableShadow(ctx,values);ctx.stroke();this.disableShadow(ctx,values);}/** @inheritdoc */},{key:"getViaNode",value:function getViaNode(){return undefined;}/** @inheritdoc */},{key:"getPoint",value:function getPoint(position){return {x:(1-position)*this.fromPoint.x+position*this.toPoint.x,y:(1-position)*this.fromPoint.y+position*this.toPoint.y};}/** @inheritdoc */},{key:"_findBorderPosition",value:function _findBorderPosition(nearNode,ctx){var node1=this.to;var node2=this.from;if(nearNode.id===this.from.id){node1=this.from;node2=this.to;}var angle=Math.atan2(node1.y-node2.y,node1.x-node2.x);var dx=node1.x-node2.x;var dy=node1.y-node2.y;var edgeSegmentLength=Math.sqrt(dx*dx+dy*dy);var toBorderDist=nearNode.distanceToBorder(ctx,angle);var toBorderPoint=(edgeSegmentLength-toBorderDist)/edgeSegmentLength;return {x:(1-toBorderPoint)*node2.x+toBorderPoint*node1.x,y:(1-toBorderPoint)*node2.y+toBorderPoint*node1.y,t:0};}/** @inheritdoc */},{key:"_getDistanceToEdge",value:function _getDistanceToEdge(x1,y1,x2,y2,x3,y3){// x3,y3 is the point
	return this._getDistanceToLine(x1,y1,x2,y2,x3,y3);}}]);return StraightEdge;}(EdgeBase);/**
	 * An edge connects two nodes and has a specific direction.
	 */var Edge=/*#__PURE__*/function(){/**
	   * @param {Object} options        values specific to this edge, must contain at least 'from' and 'to'
	   * @param {Object} body           shared state from Network instance
	   * @param {Network.Images} imagelist  A list with images. Only needed when the edge has image arrows.
	   * @param {Object} globalOptions  options from the EdgesHandler instance
	   * @param {Object} defaultOptions default options from the EdgeHandler instance. Value and reference are constant
	   */function Edge(options,body,imagelist,globalOptions,defaultOptions){classCallCheck(this,Edge);if(body===undefined){throw new Error("No body provided");}// Since globalOptions is constant in values as well as reference,
	// Following needs to be done only once.
	this.options=bridgeObject(globalOptions);this.globalOptions=globalOptions;this.defaultOptions=defaultOptions;this.body=body;this.imagelist=imagelist;// initialize variables
	this.id=undefined;this.fromId=undefined;this.toId=undefined;this.selected=false;this.hover=false;this.labelDirty=true;this.baseWidth=this.options.width;this.baseFontSize=this.options.font.size;this.from=undefined;// a node
	this.to=undefined;// a node
	this.edgeType=undefined;this.connected=false;this.labelModule=new Label(this.body,this.options,true/* It's an edge label */);this.setOptions(options);}/**
	   * Set or overwrite options for the edge
	   * @param {Object} options  an object with options
	   * @returns {undefined|boolean} undefined if no options, true if layout affecting data changed, false otherwise.
	   */createClass(Edge,[{key:"setOptions",value:function setOptions(options){if(!options){return;}// Following options if changed affect the layout.
	var affectsLayout=typeof options.physics!=="undefined"&&this.options.physics!==options.physics||typeof options.hidden!=="undefined"&&(this.options.hidden||false)!==(options.hidden||false)||typeof options.from!=="undefined"&&this.options.from!==options.from||typeof options.to!=="undefined"&&this.options.to!==options.to;Edge.parseOptions(this.options,options,true,this.globalOptions);if(options.id!==undefined){this.id=options.id;}if(options.from!==undefined){this.fromId=options.from;}if(options.to!==undefined){this.toId=options.to;}if(options.title!==undefined){this.title=options.title;}if(options.value!==undefined){options.value=_parseFloat$2(options.value);}var pile=[options,this.options,this.defaultOptions];this.chooser=ComponentUtil.choosify('edge',pile);// update label Module
	this.updateLabelModule(options);// Update edge type, this if changed affects the layout.
	affectsLayout=this.updateEdgeType()||affectsLayout;// if anything has been updates, reset the selection width and the hover width
	this._setInteractionWidths();// A node is connected when it has a from and to node that both exist in the network.body.nodes.
	this.connect();return affectsLayout;}/**
	   *
	   * @param {Object} parentOptions
	   * @param {Object} newOptions
	   * @param {boolean} [allowDeletion=false]
	   * @param {Object} [globalOptions={}]
	   * @param {boolean} [copyFromGlobals=false]
	   */},{key:"getFormattingValues",/**
	   *
	   * @returns {ArrowOptions}
	   */value:function getFormattingValues(){var toArrow=this.options.arrows.to===true||this.options.arrows.to.enabled===true;var fromArrow=this.options.arrows.from===true||this.options.arrows.from.enabled===true;var middleArrow=this.options.arrows.middle===true||this.options.arrows.middle.enabled===true;var inheritsColor=this.options.color.inherit;var values={toArrow:toArrow,toArrowScale:this.options.arrows.to.scaleFactor,toArrowType:this.options.arrows.to.type,toArrowSrc:this.options.arrows.to.src,toArrowImageWidth:this.options.arrows.to.imageWidth,toArrowImageHeight:this.options.arrows.to.imageHeight,middleArrow:middleArrow,middleArrowScale:this.options.arrows.middle.scaleFactor,middleArrowType:this.options.arrows.middle.type,middleArrowSrc:this.options.arrows.middle.src,middleArrowImageWidth:this.options.arrows.middle.imageWidth,middleArrowImageHeight:this.options.arrows.middle.imageHeight,fromArrow:fromArrow,fromArrowScale:this.options.arrows.from.scaleFactor,fromArrowType:this.options.arrows.from.type,fromArrowSrc:this.options.arrows.from.src,fromArrowImageWidth:this.options.arrows.from.imageWidth,fromArrowImageHeight:this.options.arrows.from.imageHeight,arrowStrikethrough:this.options.arrowStrikethrough,color:inheritsColor?undefined:this.options.color.color,inheritsColor:inheritsColor,opacity:this.options.color.opacity,hidden:this.options.hidden,length:this.options.length,shadow:this.options.shadow.enabled,shadowColor:this.options.shadow.color,shadowSize:this.options.shadow.size,shadowX:this.options.shadow.x,shadowY:this.options.shadow.y,dashes:this.options.dashes,width:this.options.width,background:this.options.background.enabled,backgroundColor:this.options.background.color,backgroundSize:this.options.background.size,backgroundDashes:this.options.background.dashes};if(this.selected||this.hover){if(this.chooser===true){if(this.selected){var selectedWidth=this.options.selectionWidth;if(typeof selectedWidth==='function'){values.width=selectedWidth(values.width);}else if(typeof selectedWidth==='number'){values.width+=selectedWidth;}values.width=Math.max(values.width,0.3/this.body.view.scale);values.color=this.options.color.highlight;values.shadow=this.options.shadow.enabled;}else if(this.hover){var hoverWidth=this.options.hoverWidth;if(typeof hoverWidth==='function'){values.width=hoverWidth(values.width);}else if(typeof hoverWidth==='number'){values.width+=hoverWidth;}values.width=Math.max(values.width,0.3/this.body.view.scale);values.color=this.options.color.hover;values.shadow=this.options.shadow.enabled;}}else if(typeof this.chooser==='function'){this.chooser(values,this.options.id,this.selected,this.hover);if(values.color!==undefined){values.inheritsColor=false;}if(values.shadow===false){if(values.shadowColor!==this.options.shadow.color||values.shadowSize!==this.options.shadow.size||values.shadowX!==this.options.shadow.x||values.shadowY!==this.options.shadow.y){values.shadow=true;}}}}else {values.shadow=this.options.shadow.enabled;values.width=Math.max(values.width,0.3/this.body.view.scale);}return values;}/**
	   * update the options in the label module
	   *
	   * @param {Object} options
	   */},{key:"updateLabelModule",value:function updateLabelModule(options){var pile=[options,this.options,this.globalOptions,// Currently set global edge options
	this.defaultOptions];this.labelModule.update(this.options,pile);if(this.labelModule.baseSize!==undefined){this.baseFontSize=this.labelModule.baseSize;}}/**
	   * update the edge type, set the options
	   * @returns {boolean}
	   */},{key:"updateEdgeType",value:function updateEdgeType(){var smooth=this.options.smooth;var dataChanged=false;var changeInType=true;if(this.edgeType!==undefined){if(this.edgeType instanceof BezierEdgeDynamic&&smooth.enabled===true&&smooth.type==='dynamic'||this.edgeType instanceof CubicBezierEdge&&smooth.enabled===true&&smooth.type==='cubicBezier'||this.edgeType instanceof BezierEdgeStatic&&smooth.enabled===true&&smooth.type!=='dynamic'&&smooth.type!=='cubicBezier'||this.edgeType instanceof StraightEdge&&smooth.type.enabled===false){changeInType=false;}if(changeInType===true){dataChanged=this.cleanup();}}if(changeInType===true){if(smooth.enabled===true){if(smooth.type==='dynamic'){dataChanged=true;this.edgeType=new BezierEdgeDynamic(this.options,this.body,this.labelModule);}else if(smooth.type==='cubicBezier'){this.edgeType=new CubicBezierEdge(this.options,this.body,this.labelModule);}else {this.edgeType=new BezierEdgeStatic(this.options,this.body,this.labelModule);}}else {this.edgeType=new StraightEdge(this.options,this.body,this.labelModule);}}else {// if nothing changes, we just set the options.
	this.edgeType.setOptions(this.options);}return dataChanged;}/**
	   * Connect an edge to its nodes
	   */},{key:"connect",value:function connect(){this.disconnect();this.from=this.body.nodes[this.fromId]||undefined;this.to=this.body.nodes[this.toId]||undefined;this.connected=this.from!==undefined&&this.to!==undefined;if(this.connected===true){this.from.attachEdge(this);this.to.attachEdge(this);}else {if(this.from){this.from.detachEdge(this);}if(this.to){this.to.detachEdge(this);}}this.edgeType.connect();}/**
	   * Disconnect an edge from its nodes
	   */},{key:"disconnect",value:function disconnect(){if(this.from){this.from.detachEdge(this);this.from=undefined;}if(this.to){this.to.detachEdge(this);this.to=undefined;}this.connected=false;}/**
	   * get the title of this edge.
	   * @return {string} title    The title of the edge, or undefined when no title
	   *                           has been set.
	   */},{key:"getTitle",value:function getTitle(){return this.title;}/**
	   * check if this node is selecte
	   * @return {boolean} selected   True if node is selected, else false
	   */},{key:"isSelected",value:function isSelected(){return this.selected;}/**
	   * Retrieve the value of the edge. Can be undefined
	   * @return {number} value
	   */},{key:"getValue",value:function getValue(){return this.options.value;}/**
	   * Adjust the value range of the edge. The edge will adjust it's width
	   * based on its value.
	   * @param {number} min
	   * @param {number} max
	   * @param {number} total
	   */},{key:"setValueRange",value:function setValueRange(min,max,total){if(this.options.value!==undefined){var scale=this.options.scaling.customScalingFunction(min,max,total,this.options.value);var widthDiff=this.options.scaling.max-this.options.scaling.min;if(this.options.scaling.label.enabled===true){var fontDiff=this.options.scaling.label.max-this.options.scaling.label.min;this.options.font.size=this.options.scaling.label.min+scale*fontDiff;}this.options.width=this.options.scaling.min+scale*widthDiff;}else {this.options.width=this.baseWidth;this.options.font.size=this.baseFontSize;}this._setInteractionWidths();this.updateLabelModule();}/**
	   *
	   * @private
	   */},{key:"_setInteractionWidths",value:function _setInteractionWidths(){if(typeof this.options.hoverWidth==='function'){this.edgeType.hoverWidth=this.options.hoverWidth(this.options.width);}else {this.edgeType.hoverWidth=this.options.hoverWidth+this.options.width;}if(typeof this.options.selectionWidth==='function'){this.edgeType.selectionWidth=this.options.selectionWidth(this.options.width);}else {this.edgeType.selectionWidth=this.options.selectionWidth+this.options.width;}}/**
	   * Redraw a edge
	   * Draw this edge in the given canvas
	   * The 2d context of a HTML canvas can be retrieved by canvas.getContext("2d");
	   * @param {CanvasRenderingContext2D}   ctx
	   */},{key:"draw",value:function draw(ctx){var values=this.getFormattingValues();if(values.hidden){return;}// get the via node from the edge type
	var viaNode=this.edgeType.getViaNode();var arrowData={};// restore edge targets to defaults
	this.edgeType.fromPoint=this.edgeType.from;this.edgeType.toPoint=this.edgeType.to;// from and to arrows give a different end point for edges. we set them here
	if(values.fromArrow){arrowData.from=this.edgeType.getArrowData(ctx,"from",viaNode,this.selected,this.hover,values);if(values.arrowStrikethrough===false)this.edgeType.fromPoint=arrowData.from.core;if(values.fromArrowSrc){arrowData.from.image=this.imagelist.load(values.fromArrowSrc);}if(values.fromArrowImageWidth){arrowData.from.imageWidth=values.fromArrowImageWidth;}if(values.fromArrowImageHeight){arrowData.from.imageHeight=values.fromArrowImageHeight;}}if(values.toArrow){arrowData.to=this.edgeType.getArrowData(ctx,"to",viaNode,this.selected,this.hover,values);if(values.arrowStrikethrough===false)this.edgeType.toPoint=arrowData.to.core;if(values.toArrowSrc){arrowData.to.image=this.imagelist.load(values.toArrowSrc);}if(values.toArrowImageWidth){arrowData.to.imageWidth=values.toArrowImageWidth;}if(values.toArrowImageHeight){arrowData.to.imageHeight=values.toArrowImageHeight;}}// the middle arrow depends on the line, which can depend on the to and from arrows so we do this one lastly.
	if(values.middleArrow){arrowData.middle=this.edgeType.getArrowData(ctx,"middle",viaNode,this.selected,this.hover,values);if(values.middleArrowSrc){arrowData.middle.image=this.imagelist.load(values.middleArrowSrc);}if(values.middleArrowImageWidth){arrowData.middle.imageWidth=values.middleArrowImageWidth;}if(values.middleArrowImageHeight){arrowData.middle.imageHeight=values.middleArrowImageHeight;}}// draw everything
	this.edgeType.drawLine(ctx,values,this.selected,this.hover,viaNode);this.drawArrows(ctx,arrowData,values);this.drawLabel(ctx,viaNode);}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {Object} arrowData
	   * @param {ArrowOptions} values
	   */},{key:"drawArrows",value:function drawArrows(ctx,arrowData,values){if(values.fromArrow){this.edgeType.drawArrowHead(ctx,values,this.selected,this.hover,arrowData.from);}if(values.middleArrow){this.edgeType.drawArrowHead(ctx,values,this.selected,this.hover,arrowData.middle);}if(values.toArrow){this.edgeType.drawArrowHead(ctx,values,this.selected,this.hover,arrowData.to);}}/**
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {Node} viaNode
	   */},{key:"drawLabel",value:function drawLabel(ctx,viaNode){if(this.options.label!==undefined){// set style
	var node1=this.from;var node2=this.to;if(this.labelModule.differentState(this.selected,this.hover)){this.labelModule.getTextSize(ctx,this.selected,this.hover);}if(node1.id!=node2.id){this.labelModule.pointToSelf=false;var point=this.edgeType.getPoint(0.5,viaNode);ctx.save();var rotationPoint=this._getRotation(ctx);if(rotationPoint.angle!=0){ctx.translate(rotationPoint.x,rotationPoint.y);ctx.rotate(rotationPoint.angle);}// draw the label
	this.labelModule.draw(ctx,point.x,point.y,this.selected,this.hover);/*
	        // Useful debug code: draw a border around the label
	        // This should **not** be enabled in production!
	        var size = this.labelModule.getSize();; // ;; intentional so lint catches it
	        ctx.strokeStyle = "#ff0000";
	        ctx.strokeRect(size.left, size.top, size.width, size.height);
	        // End  debug code
	*/ctx.restore();}else {// Ignore the orientations.
	this.labelModule.pointToSelf=true;// get circle coordinates
	var coordinates=ComponentUtil.getSelfRefCoordinates(ctx,this.options.selfReference.angle,this.options.selfReference.size,node1);point=this._pointOnCircle(coordinates.x,coordinates.y,this.options.selfReference.size,this.options.selfReference.angle);this.labelModule.draw(ctx,point.x,point.y,this.selected,this.hover);}}}/**
	   * Determine all visual elements of this edge instance, in which the given
	   * point falls within the bounding shape.
	   *
	   * @param {point} point
	   * @returns {Array.<edgeClickItem|edgeLabelClickItem>} list with the items which are on the point
	   */},{key:"getItemsOnPoint",value:function getItemsOnPoint(point){var ret=[];if(this.labelModule.visible()){var rotationPoint=this._getRotation();if(ComponentUtil.pointInRect(this.labelModule.getSize(),point,rotationPoint)){ret.push({edgeId:this.id,labelId:0});}}var obj={left:point.x,top:point.y};if(this.isOverlappingWith(obj)){ret.push({edgeId:this.id});}return ret;}/**
	   * Check if this object is overlapping with the provided object
	   * @param {Object} obj   an object with parameters left, top
	   * @return {boolean}     True if location is located on the edge
	   */},{key:"isOverlappingWith",value:function isOverlappingWith(obj){if(this.connected){var distMax=10;var xFrom=this.from.x;var yFrom=this.from.y;var xTo=this.to.x;var yTo=this.to.y;var xObj=obj.left;var yObj=obj.top;var dist=this.edgeType.getDistanceToEdge(xFrom,yFrom,xTo,yTo,xObj,yObj);return dist<distMax;}else {return false;}}/** 
	   * Determine the rotation point, if any.
	   *
	   * @param {CanvasRenderingContext2D} [ctx] if passed, do a recalculation of the label size
	   * @returns {rotationPoint} the point to rotate around and the angle in radians to rotate
	   * @private
	   */},{key:"_getRotation",value:function _getRotation(ctx){var viaNode=this.edgeType.getViaNode();var point=this.edgeType.getPoint(0.5,viaNode);if(ctx!==undefined){this.labelModule.calculateLabelSize(ctx,this.selected,this.hover,point.x,point.y);}var ret={x:point.x,y:this.labelModule.size.yLine,angle:0};if(!this.labelModule.visible()){return ret;// Don't even bother doing the atan2, there's nothing to draw
	}if(this.options.font.align==="horizontal"){return ret;// No need to calculate angle
	}var dy=this.from.y-this.to.y;var dx=this.from.x-this.to.x;var angle=Math.atan2(dy,dx);// radians
	// rotate so that label is readable
	if(angle<-1&&dx<0||angle>0&&dx<0){angle+=Math.PI;}ret.angle=angle;return ret;}/**
	   * Get a point on a circle
	   * @param {number} x
	   * @param {number} y
	   * @param {number} radius
	   * @param {number} angle 
	   * @return {Object} point
	   * @private
	   */},{key:"_pointOnCircle",value:function _pointOnCircle(x,y,radius,angle){return {x:x+radius*Math.cos(angle),y:y-radius*Math.sin(angle)};}/**
	   * Sets selected state to true
	   */},{key:"select",value:function select(){this.selected=true;}/**
	   * Sets selected state to false
	   */},{key:"unselect",value:function unselect(){this.selected=false;}/**
	   * cleans all required things on delete
	   * @returns {*}
	   */},{key:"cleanup",value:function cleanup(){return this.edgeType.cleanup();}/**
	   * Remove edge from the list and perform necessary cleanup.
	   */},{key:"remove",value:function remove(){this.cleanup();this.disconnect();delete this.body.edges[this.id];}/**
	   * Check if both connecting nodes exist
	   * @returns {boolean}
	   */},{key:"endPointsValid",value:function endPointsValid(){return this.body.nodes[this.fromId]!==undefined&&this.body.nodes[this.toId]!==undefined;}}],[{key:"parseOptions",value:function parseOptions(parentOptions,newOptions){var allowDeletion=arguments.length>2&&arguments[2]!==undefined?arguments[2]:false;var globalOptions=arguments.length>3&&arguments[3]!==undefined?arguments[3]:{};var copyFromGlobals=arguments.length>4&&arguments[4]!==undefined?arguments[4]:false;var fields=['arrowStrikethrough','id','from','hidden','hoverWidth','labelHighlightBold','length','line','opacity','physics','scaling','selectionWidth','selfReferenceSize','selfReference','to','title','value','width','font','chosen','widthConstraint'];// only deep extend the items in the field array. These do not have shorthand.
	selectiveDeepExtend(fields,parentOptions,newOptions,allowDeletion);// Only copy label if it's a legal value.
	if(ComponentUtil.isValidLabel(newOptions.label)){parentOptions.label=newOptions.label;}else if(!ComponentUtil.isValidLabel(parentOptions.label)){parentOptions.label=undefined;}mergeOptions(parentOptions,newOptions,'smooth',globalOptions);mergeOptions(parentOptions,newOptions,'shadow',globalOptions);mergeOptions(parentOptions,newOptions,'background',globalOptions);if(newOptions.dashes!==undefined&&newOptions.dashes!==null){parentOptions.dashes=newOptions.dashes;}else if(allowDeletion===true&&newOptions.dashes===null){parentOptions.dashes=create$4(globalOptions.dashes);// this sets the pointer of the option back to the global option.
	}// set the scaling newOptions
	if(newOptions.scaling!==undefined&&newOptions.scaling!==null){if(newOptions.scaling.min!==undefined){parentOptions.scaling.min=newOptions.scaling.min;}if(newOptions.scaling.max!==undefined){parentOptions.scaling.max=newOptions.scaling.max;}mergeOptions(parentOptions.scaling,newOptions.scaling,'label',globalOptions.scaling);}else if(allowDeletion===true&&newOptions.scaling===null){parentOptions.scaling=create$4(globalOptions.scaling);// this sets the pointer of the option back to the global option.
	}// handle multiple input cases for arrows
	if(newOptions.arrows!==undefined&&newOptions.arrows!==null){if(typeof newOptions.arrows==='string'){var arrows=newOptions.arrows.toLowerCase();parentOptions.arrows.to.enabled=indexOf$3(arrows).call(arrows,"to")!=-1;parentOptions.arrows.middle.enabled=indexOf$3(arrows).call(arrows,"middle")!=-1;parentOptions.arrows.from.enabled=indexOf$3(arrows).call(arrows,"from")!=-1;}else if(_typeof_1(newOptions.arrows)==='object'){mergeOptions(parentOptions.arrows,newOptions.arrows,'to',globalOptions.arrows);mergeOptions(parentOptions.arrows,newOptions.arrows,'middle',globalOptions.arrows);mergeOptions(parentOptions.arrows,newOptions.arrows,'from',globalOptions.arrows);}else {throw new Error("The arrow newOptions can only be an object or a string. Refer to the documentation. You used:"+stringify$2(newOptions.arrows));}}else if(allowDeletion===true&&newOptions.arrows===null){parentOptions.arrows=create$4(globalOptions.arrows);// this sets the pointer of the option back to the global option.
	}// handle multiple input cases for color
	if(newOptions.color!==undefined&&newOptions.color!==null){var fromColor=isString(newOptions.color)?{color:newOptions.color,highlight:newOptions.color,hover:newOptions.color,inherit:false,opacity:1}:newOptions.color;var toColor=parentOptions.color;// If passed, fill in values from default options - required in the case of no prototype bridging
	if(copyFromGlobals){deepExtend(toColor,globalOptions.color,false,allowDeletion);}else {// Clear local properties - need to do it like this in order to retain prototype bridges
	for(var i in toColor){if(toColor.hasOwnProperty(i)){delete toColor[i];}}}if(isString(toColor)){toColor.color=toColor;toColor.highlight=toColor;toColor.hover=toColor;toColor.inherit=false;if(fromColor.opacity===undefined){toColor.opacity=1.0;// set default
	}}else {var colorsDefined=false;if(fromColor.color!==undefined){toColor.color=fromColor.color;colorsDefined=true;}if(fromColor.highlight!==undefined){toColor.highlight=fromColor.highlight;colorsDefined=true;}if(fromColor.hover!==undefined){toColor.hover=fromColor.hover;colorsDefined=true;}if(fromColor.inherit!==undefined){toColor.inherit=fromColor.inherit;}if(fromColor.opacity!==undefined){toColor.opacity=Math.min(1,Math.max(0,fromColor.opacity));}if(colorsDefined===true){toColor.inherit=false;}else {if(toColor.inherit===undefined){toColor.inherit='from';// Set default
	}}}}else if(allowDeletion===true&&newOptions.color===null){parentOptions.color=bridgeObject(globalOptions.color);// set the object back to the global options
	}if(allowDeletion===true&&newOptions.font===null){parentOptions.font=bridgeObject(globalOptions.font);// set the object back to the global options
	}if(newOptions.hasOwnProperty("selfReferenceSize")){console.log('The selfReferenceSize property has been deprecated. Please use selfReference property instead. The selfReference can be set like thise selfReference:{size:30, angle:Math.PI / 4}');parentOptions.selfReference.size=newOptions.selfReferenceSize;}}}]);return Edge;}();/**
	 * Handler for Edges
	 */var EdgesHandler=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   * @param {Array.<Image>} images
	   * @param {Array.<Group>} groups
	   */function EdgesHandler(body,images,groups){var _context26,_this20=this;classCallCheck(this,EdgesHandler);this.body=body;this.images=images;this.groups=groups;// create the edge API in the body container
	this.body.functions.createEdge=bind$2(_context26=this.create).call(_context26,this);this.edgesListeners={add:function add(event,params){_this20.add(params.items);},update:function update(event,params){_this20.update(params.items);},remove:function remove(event,params){_this20.remove(params.items);}};this.options={};this.defaultOptions={arrows:{to:{enabled:false,scaleFactor:1,type:'arrow'},// boolean / {arrowScaleFactor:1} / {enabled: false, arrowScaleFactor:1}
	middle:{enabled:false,scaleFactor:1,type:'arrow'},from:{enabled:false,scaleFactor:1,type:'arrow'}},arrowStrikethrough:true,color:{color:'#848484',highlight:'#848484',hover:'#848484',inherit:'from',opacity:1.0},dashes:false,font:{color:'#343434',size:14,// px
	face:'arial',background:'none',strokeWidth:2,// px
	strokeColor:'#ffffff',align:'horizontal',multi:false,vadjust:0,bold:{mod:'bold'},boldital:{mod:'bold italic'},ital:{mod:'italic'},mono:{mod:'',size:15,// px
	face:'courier new',vadjust:2}},hidden:false,hoverWidth:1.5,label:undefined,labelHighlightBold:true,length:undefined,physics:true,scaling:{min:1,max:15,label:{enabled:true,min:14,max:30,maxVisible:30,drawThreshold:5},customScalingFunction:function customScalingFunction(min,max,total,value){if(max===min){return 0.5;}else {var scale=1/(max-min);return Math.max(0,(value-min)*scale);}}},selectionWidth:1.5,selfReference:{size:20,angle:Math.PI/4,renderBehindTheNode:true},shadow:{enabled:false,color:'rgba(0,0,0,0.5)',size:10,x:5,y:5},background:{enabled:false,color:'rgba(111,111,111,1)',size:10,dashes:false},smooth:{enabled:true,type:"dynamic",forceDirection:'none',roundness:0.5},title:undefined,width:1,value:undefined};deepExtend(this.options,this.defaultOptions);this.bindEventListeners();}/**
	   * Binds event listeners
	   */createClass(EdgesHandler,[{key:"bindEventListeners",value:function bindEventListeners(){var _this21=this,_context27,_context28;// this allows external modules to force all dynamic curves to turn static.
	this.body.emitter.on("_forceDisableDynamicCurves",function(type){var emit=arguments.length>1&&arguments[1]!==undefined?arguments[1]:true;if(type==='dynamic'){type='continuous';}var dataChanged=false;for(var edgeId in _this21.body.edges){if(_this21.body.edges.hasOwnProperty(edgeId)){var edge=_this21.body.edges[edgeId];var edgeData=_this21.body.data.edges.get(edgeId);// only forcibly remove the smooth curve if the data has been set of the edge has the smooth curves defined.
	// this is because a change in the global would not affect these curves.
	if(edgeData!=null){var smoothOptions=edgeData.smooth;if(smoothOptions!==undefined){if(smoothOptions.enabled===true&&smoothOptions.type==='dynamic'){if(type===undefined){edge.setOptions({smooth:false});}else {edge.setOptions({smooth:{type:type}});}dataChanged=true;}}}}}if(emit===true&&dataChanged===true){_this21.body.emitter.emit("_dataChanged");}});// this is called when options of EXISTING nodes or edges have changed.
	//
	// NOTE: Not true, called when options have NOT changed, for both existing as well as new nodes.
	//       See update() for logic.
	// TODO: Verify and examine the consequences of this. It might still trigger when
	//       non-option fields have changed, but then reconnecting edges is still useless.
	//       Alternatively, it might also be called when edges are removed.
	//
	this.body.emitter.on("_dataUpdated",function(){_this21.reconnectEdges();});// refresh the edges. Used when reverting from hierarchical layout
	this.body.emitter.on("refreshEdges",bind$2(_context27=this.refresh).call(_context27,this));this.body.emitter.on("refresh",bind$2(_context28=this.refresh).call(_context28,this));this.body.emitter.on("destroy",function(){forEach$4(_this21.edgesListeners,function(callback,event){if(_this21.body.data.edges)_this21.body.data.edges.off(event,callback);});delete _this21.body.functions.createEdge;delete _this21.edgesListeners.add;delete _this21.edgesListeners.update;delete _this21.edgesListeners.remove;delete _this21.edgesListeners;});}/**
	   *
	   * @param {Object} options
	   */},{key:"setOptions",value:function setOptions(options){if(options!==undefined){// use the parser from the Edge class to fill in all shorthand notations
	Edge.parseOptions(this.options,options,true,this.defaultOptions,true);// update smooth settings in all edges
	var dataChanged=false;if(options.smooth!==undefined){for(var edgeId in this.body.edges){if(this.body.edges.hasOwnProperty(edgeId)){dataChanged=this.body.edges[edgeId].updateEdgeType()||dataChanged;}}}// update fonts in all edges
	if(options.font!==undefined){for(var _edgeId in this.body.edges){if(this.body.edges.hasOwnProperty(_edgeId)){this.body.edges[_edgeId].updateLabelModule();}}}// update the state of the variables if needed
	if(options.hidden!==undefined||options.physics!==undefined||dataChanged===true){this.body.emitter.emit('_dataChanged');}}}/**
	   * Load edges by reading the data table
	   * @param {Array | DataSet | DataView} edges    The data containing the edges.
	   * @param {boolean} [doNotEmit=false]
	   * @private
	   */},{key:"setData",value:function setData(edges){var _this22=this;var doNotEmit=arguments.length>1&&arguments[1]!==undefined?arguments[1]:false;var oldEdgesData=this.body.data.edges;if(edges instanceof DataSet||edges instanceof DataView){this.body.data.edges=edges;}else if(isArray$3(edges)){this.body.data.edges=new DataSet();this.body.data.edges.add(edges);}else if(!edges){this.body.data.edges=new DataSet();}else {throw new TypeError('Array or DataSet expected');}// TODO: is this null or undefined or false?
	if(oldEdgesData){// unsubscribe from old dataset
	forEach$4(this.edgesListeners,function(callback,event){oldEdgesData.off(event,callback);});}// remove drawn edges
	this.body.edges={};// TODO: is this null or undefined or false?
	if(this.body.data.edges){// subscribe to new dataset
	forEach$4(this.edgesListeners,function(callback,event){_this22.body.data.edges.on(event,callback);});// draw all new nodes
	var ids=this.body.data.edges.getIds();this.add(ids,true);}this.body.emitter.emit('_adjustEdgesForHierarchicalLayout');if(doNotEmit===false){this.body.emitter.emit("_dataChanged");}}/**
	   * Add edges
	   * @param {number[] | string[]} ids
	   * @param {boolean} [doNotEmit=false]
	   * @private
	   */},{key:"add",value:function add(ids){var doNotEmit=arguments.length>1&&arguments[1]!==undefined?arguments[1]:false;var edges=this.body.edges;var edgesData=this.body.data.edges;for(var i=0;i<ids.length;i++){var id=ids[i];var oldEdge=edges[id];if(oldEdge){oldEdge.disconnect();}var data=edgesData.get(id,{"showInternalIds":true});edges[id]=this.create(data);}this.body.emitter.emit('_adjustEdgesForHierarchicalLayout');if(doNotEmit===false){this.body.emitter.emit("_dataChanged");}}/**
	   * Update existing edges, or create them when not yet existing
	   * @param {number[] | string[]} ids
	   * @private
	   */},{key:"update",value:function update(ids){var edges=this.body.edges;var edgesData=this.body.data.edges;var dataChanged=false;for(var i=0;i<ids.length;i++){var id=ids[i];var data=edgesData.get(id);var edge=edges[id];if(edge!==undefined){// update edge
	edge.disconnect();dataChanged=edge.setOptions(data)||dataChanged;// if a support node is added, data can be changed.
	edge.connect();}else {// create edge
	this.body.edges[id]=this.create(data);dataChanged=true;}}if(dataChanged===true){this.body.emitter.emit('_adjustEdgesForHierarchicalLayout');this.body.emitter.emit("_dataChanged");}else {this.body.emitter.emit("_dataUpdated");}}/**
	   * Remove existing edges. Non existing ids will be ignored
	   * @param {number[] | string[]} ids
	   * @param {boolean} [emit=true]
	   * @private
	   */},{key:"remove",value:function remove(ids){var emit=arguments.length>1&&arguments[1]!==undefined?arguments[1]:true;if(ids.length===0)return;// early out
	var edges=this.body.edges;forEach$4(ids,function(id){var edge=edges[id];if(edge!==undefined){edge.remove();}});if(emit){this.body.emitter.emit("_dataChanged");}}/**
	   * Refreshes Edge Handler
	   */},{key:"refresh",value:function refresh(){var _this23=this;forEach$4(this.body.edges,function(edge,edgeId){var data=_this23.body.data.edges.get(edgeId);if(data!==undefined){edge.setOptions(data);}});}/**
	   *
	   * @param {Object} properties
	   * @returns {Edge}
	   */},{key:"create",value:function create(properties){return new Edge(properties,this.body,this.images,this.options,this.defaultOptions);}/**
	   * Reconnect all edges
	   * @private
	   */},{key:"reconnectEdges",value:function reconnectEdges(){var id;var nodes=this.body.nodes;var edges=this.body.edges;for(id in nodes){if(nodes.hasOwnProperty(id)){nodes[id].edges=[];}}for(id in edges){if(edges.hasOwnProperty(id)){var edge=edges[id];edge.from=null;edge.to=null;edge.connect();}}}/**
	   *
	   * @param {Edge.id} edgeId
	   * @returns {Array}
	   */},{key:"getConnectedNodes",value:function getConnectedNodes(edgeId){var nodeList=[];if(this.body.edges[edgeId]!==undefined){var edge=this.body.edges[edgeId];if(edge.fromId!==undefined){nodeList.push(edge.fromId);}if(edge.toId!==undefined){nodeList.push(edge.toId);}}return nodeList;}/**
	   * There is no direct relation between the nodes and the edges DataSet,
	   * so the right place to do call this is in the handler for event `_dataUpdated`.
	   */},{key:"_updateState",value:function _updateState(){this._addMissingEdges();this._removeInvalidEdges();}/**
	   * Scan for missing nodes and remove corresponding edges, if any.
	   * @private
	   */},{key:"_removeInvalidEdges",value:function _removeInvalidEdges(){var _this24=this;var edgesToDelete=[];forEach$4(this.body.edges,function(edge,id){var toNode=_this24.body.nodes[edge.toId];var fromNode=_this24.body.nodes[edge.fromId];// Skip clustering edges here, let the Clustering module handle those
	if(toNode!==undefined&&toNode.isCluster===true||fromNode!==undefined&&fromNode.isCluster===true){return;}if(toNode===undefined||fromNode===undefined){edgesToDelete.push(id);}});this.remove(edgesToDelete,false);}/**
	   * add all edges from dataset that are not in the cached state
	   * @private
	   */},{key:"_addMissingEdges",value:function _addMissingEdges(){var edgesData=this.body.data.edges;if(edgesData===undefined||edgesData===null){return;// No edges DataSet yet; can happen on startup 
	}var edges=this.body.edges;var addIds=[];forEach$3(edgesData).call(edgesData,function(edgeData,edgeId){var edge=edges[edgeId];if(edge===undefined){addIds.push(edgeId);}});this.add(addIds,true);}}]);return EdgesHandler;}();/**
	 * Barnes Hut Solver
	 */var BarnesHutSolver=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   * @param {{physicsNodeIndices: Array, physicsEdgeIndices: Array, forces: {}, velocities: {}}} physicsBody
	   * @param {Object} options
	   */function BarnesHutSolver(body,physicsBody,options){classCallCheck(this,BarnesHutSolver);this.body=body;this.physicsBody=physicsBody;this.barnesHutTree;this.setOptions(options);this._rng=Alea("BARNES HUT SOLVER");// debug: show grid
	// this.body.emitter.on("afterDrawing", (ctx) => {this._debug(ctx,'#ff0000')})
	}/**
	   *
	   * @param {Object} options
	   */createClass(BarnesHutSolver,[{key:"setOptions",value:function setOptions(options){this.options=options;this.thetaInversed=1/this.options.theta;// if 1 then min distance = 0.5, if 0.5 then min distance = 0.5 + 0.5*node.shape.radius
	this.overlapAvoidanceFactor=1-Math.max(0,Math.min(1,this.options.avoidOverlap));}/**
	   * This function calculates the forces the nodes apply on each other based on a gravitational model.
	   * The Barnes Hut method is used to speed up this N-body simulation.
	   *
	   * @private
	   */},{key:"solve",value:function solve(){if(this.options.gravitationalConstant!==0&&this.physicsBody.physicsNodeIndices.length>0){var node;var nodes=this.body.nodes;var nodeIndices=this.physicsBody.physicsNodeIndices;var nodeCount=nodeIndices.length;// create the tree
	var barnesHutTree=this._formBarnesHutTree(nodes,nodeIndices);// for debugging
	this.barnesHutTree=barnesHutTree;// place the nodes one by one recursively
	for(var i=0;i<nodeCount;i++){node=nodes[nodeIndices[i]];if(node.options.mass>0){// starting with root is irrelevant, it never passes the BarnesHutSolver condition
	this._getForceContributions(barnesHutTree.root,node);}}}}/**
	   * @param {Object} parentBranch
	   * @param {Node} node
	   * @private
	   */},{key:"_getForceContributions",value:function _getForceContributions(parentBranch,node){this._getForceContribution(parentBranch.children.NW,node);this._getForceContribution(parentBranch.children.NE,node);this._getForceContribution(parentBranch.children.SW,node);this._getForceContribution(parentBranch.children.SE,node);}/**
	   * This function traverses the barnesHutTree. It checks when it can approximate distant nodes with their center of mass.
	   * If a region contains a single node, we check if it is not itself, then we apply the force.
	   *
	   * @param {Object} parentBranch
	   * @param {Node} node
	   * @private
	   */},{key:"_getForceContribution",value:function _getForceContribution(parentBranch,node){// we get no force contribution from an empty region
	if(parentBranch.childrenCount>0){var dx,dy,distance;// get the distance from the center of mass to the node.
	dx=parentBranch.centerOfMass.x-node.x;dy=parentBranch.centerOfMass.y-node.y;distance=Math.sqrt(dx*dx+dy*dy);// BarnesHutSolver condition
	// original condition : s/d < theta = passed  ===  d/s > 1/theta = passed
	// calcSize = 1/s --> d * 1/s > 1/theta = passed
	if(distance*parentBranch.calcSize>this.thetaInversed){this._calculateForces(distance,dx,dy,node,parentBranch);}else {// Did not pass the condition, go into children if available
	if(parentBranch.childrenCount===4){this._getForceContributions(parentBranch,node);}else {// parentBranch must have only one node, if it was empty we wouldnt be here
	if(parentBranch.children.data.id!=node.id){// if it is not self
	this._calculateForces(distance,dx,dy,node,parentBranch);}}}}}/**
	   * Calculate the forces based on the distance.
	   *
	   * @param {number} distance
	   * @param {number} dx
	   * @param {number} dy
	   * @param {Node} node
	   * @param {Object} parentBranch
	   * @private
	   */},{key:"_calculateForces",value:function _calculateForces(distance,dx,dy,node,parentBranch){if(distance===0){distance=0.1;dx=distance;}if(this.overlapAvoidanceFactor<1&&node.shape.radius){distance=Math.max(0.1+this.overlapAvoidanceFactor*node.shape.radius,distance-node.shape.radius);}// the dividing by the distance cubed instead of squared allows us to get the fx and fy components without sines and cosines
	// it is shorthand for gravityforce with distance squared and fx = dx/distance * gravityForce
	var gravityForce=this.options.gravitationalConstant*parentBranch.mass*node.options.mass/Math.pow(distance,3);var fx=dx*gravityForce;var fy=dy*gravityForce;this.physicsBody.forces[node.id].x+=fx;this.physicsBody.forces[node.id].y+=fy;}/**
	   * This function constructs the barnesHut tree recursively. It creates the root, splits it and starts placing the nodes.
	   *
	   * @param {Array.<Node>} nodes
	   * @param {Array.<number>} nodeIndices
	   * @returns {{root: {centerOfMass: {x: number, y: number}, mass: number, range: {minX: number, maxX: number, minY: number, maxY: number}, size: number, calcSize: number, children: {data: null}, maxWidth: number, level: number, childrenCount: number}}} BarnesHutTree
	   * @private
	   */},{key:"_formBarnesHutTree",value:function _formBarnesHutTree(nodes,nodeIndices){var node;var nodeCount=nodeIndices.length;var minX=nodes[nodeIndices[0]].x;var minY=nodes[nodeIndices[0]].y;var maxX=nodes[nodeIndices[0]].x;var maxY=nodes[nodeIndices[0]].y;// get the range of the nodes
	for(var i=1;i<nodeCount;i++){var _node3=nodes[nodeIndices[i]];var x=_node3.x;var y=_node3.y;if(_node3.options.mass>0){if(x<minX){minX=x;}if(x>maxX){maxX=x;}if(y<minY){minY=y;}if(y>maxY){maxY=y;}}}// make the range a square
	var sizeDiff=Math.abs(maxX-minX)-Math.abs(maxY-minY);// difference between X and Y
	if(sizeDiff>0){minY-=0.5*sizeDiff;maxY+=0.5*sizeDiff;}// xSize > ySize
	else {minX+=0.5*sizeDiff;maxX-=0.5*sizeDiff;}// xSize < ySize
	var minimumTreeSize=1e-5;var rootSize=Math.max(minimumTreeSize,Math.abs(maxX-minX));var halfRootSize=0.5*rootSize;var centerX=0.5*(minX+maxX),centerY=0.5*(minY+maxY);// construct the barnesHutTree
	var barnesHutTree={root:{centerOfMass:{x:0,y:0},mass:0,range:{minX:centerX-halfRootSize,maxX:centerX+halfRootSize,minY:centerY-halfRootSize,maxY:centerY+halfRootSize},size:rootSize,calcSize:1/rootSize,children:{data:null},maxWidth:0,level:0,childrenCount:4}};this._splitBranch(barnesHutTree.root);// place the nodes one by one recursively
	for(var _i6=0;_i6<nodeCount;_i6++){node=nodes[nodeIndices[_i6]];if(node.options.mass>0){this._placeInTree(barnesHutTree.root,node);}}// make global
	return barnesHutTree;}/**
	   * this updates the mass of a branch. this is increased by adding a node.
	   *
	   * @param {Object} parentBranch
	   * @param {Node} node
	   * @private
	   */},{key:"_updateBranchMass",value:function _updateBranchMass(parentBranch,node){var centerOfMass=parentBranch.centerOfMass;var totalMass=parentBranch.mass+node.options.mass;var totalMassInv=1/totalMass;centerOfMass.x=centerOfMass.x*parentBranch.mass+node.x*node.options.mass;centerOfMass.x*=totalMassInv;centerOfMass.y=centerOfMass.y*parentBranch.mass+node.y*node.options.mass;centerOfMass.y*=totalMassInv;parentBranch.mass=totalMass;var biggestSize=Math.max(Math.max(node.height,node.radius),node.width);parentBranch.maxWidth=parentBranch.maxWidth<biggestSize?biggestSize:parentBranch.maxWidth;}/**
	   * determine in which branch the node will be placed.
	   *
	   * @param {Object} parentBranch
	   * @param {Node} node
	   * @param {boolean} skipMassUpdate
	   * @private
	   */},{key:"_placeInTree",value:function _placeInTree(parentBranch,node,skipMassUpdate){if(skipMassUpdate!=true||skipMassUpdate===undefined){// update the mass of the branch.
	this._updateBranchMass(parentBranch,node);}var range=parentBranch.children.NW.range;var region;if(range.maxX>node.x){// in NW or SW
	if(range.maxY>node.y){region="NW";}else {region="SW";}}else {// in NE or SE
	if(range.maxY>node.y){region="NE";}else {region="SE";}}this._placeInRegion(parentBranch,node,region);}/**
	   * actually place the node in a region (or branch)
	   *
	   * @param {Object} parentBranch
	   * @param {Node} node
	   * @param {'NW'| 'NE' | 'SW' | 'SE'} region
	   * @private
	   */},{key:"_placeInRegion",value:function _placeInRegion(parentBranch,node,region){var children=parentBranch.children[region];switch(children.childrenCount){case 0:// place node here
	children.children.data=node;children.childrenCount=1;this._updateBranchMass(children,node);break;case 1:// convert into children
	// if there are two nodes exactly overlapping (on init, on opening of cluster etc.)
	// we move one node a little bit and we do not put it in the tree.
	if(children.children.data.x===node.x&&children.children.data.y===node.y){node.x+=this._rng();node.y+=this._rng();}else {this._splitBranch(children);this._placeInTree(children,node);}break;case 4:// place in branch
	this._placeInTree(children,node);break;}}/**
	   * this function splits a branch into 4 sub branches. If the branch contained a node, we place it in the subbranch
	   * after the split is complete.
	   *
	   * @param {Object} parentBranch
	   * @private
	   */},{key:"_splitBranch",value:function _splitBranch(parentBranch){// if the branch is shaded with a node, replace the node in the new subset.
	var containedNode=null;if(parentBranch.childrenCount===1){containedNode=parentBranch.children.data;parentBranch.mass=0;parentBranch.centerOfMass.x=0;parentBranch.centerOfMass.y=0;}parentBranch.childrenCount=4;parentBranch.children.data=null;this._insertRegion(parentBranch,"NW");this._insertRegion(parentBranch,"NE");this._insertRegion(parentBranch,"SW");this._insertRegion(parentBranch,"SE");if(containedNode!=null){this._placeInTree(parentBranch,containedNode);}}/**
	   * This function subdivides the region into four new segments.
	   * Specifically, this inserts a single new segment.
	   * It fills the children section of the parentBranch
	   *
	   * @param {Object} parentBranch
	   * @param {'NW'| 'NE' | 'SW' | 'SE'} region
	   * @private
	   */},{key:"_insertRegion",value:function _insertRegion(parentBranch,region){var minX,maxX,minY,maxY;var childSize=0.5*parentBranch.size;switch(region){case"NW":minX=parentBranch.range.minX;maxX=parentBranch.range.minX+childSize;minY=parentBranch.range.minY;maxY=parentBranch.range.minY+childSize;break;case"NE":minX=parentBranch.range.minX+childSize;maxX=parentBranch.range.maxX;minY=parentBranch.range.minY;maxY=parentBranch.range.minY+childSize;break;case"SW":minX=parentBranch.range.minX;maxX=parentBranch.range.minX+childSize;minY=parentBranch.range.minY+childSize;maxY=parentBranch.range.maxY;break;case"SE":minX=parentBranch.range.minX+childSize;maxX=parentBranch.range.maxX;minY=parentBranch.range.minY+childSize;maxY=parentBranch.range.maxY;break;}parentBranch.children[region]={centerOfMass:{x:0,y:0},mass:0,range:{minX:minX,maxX:maxX,minY:minY,maxY:maxY},size:0.5*parentBranch.size,calcSize:2*parentBranch.calcSize,children:{data:null},maxWidth:0,level:parentBranch.level+1,childrenCount:0};}//---------------------------  DEBUGGING BELOW  ---------------------------//
	/**
	   * This function is for debugging purposed, it draws the tree.
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {string} color
	   * @private
	   */},{key:"_debug",value:function _debug(ctx,color){if(this.barnesHutTree!==undefined){ctx.lineWidth=1;this._drawBranch(this.barnesHutTree.root,ctx,color);}}/**
	   * This function is for debugging purposes. It draws the branches recursively.
	   *
	   * @param {Object} branch
	   * @param {CanvasRenderingContext2D} ctx
	   * @param {string} color
	   * @private
	   */},{key:"_drawBranch",value:function _drawBranch(branch,ctx,color){if(color===undefined){color="#FF0000";}if(branch.childrenCount===4){this._drawBranch(branch.children.NW,ctx);this._drawBranch(branch.children.NE,ctx);this._drawBranch(branch.children.SE,ctx);this._drawBranch(branch.children.SW,ctx);}ctx.strokeStyle=color;ctx.beginPath();ctx.moveTo(branch.range.minX,branch.range.minY);ctx.lineTo(branch.range.maxX,branch.range.minY);ctx.stroke();ctx.beginPath();ctx.moveTo(branch.range.maxX,branch.range.minY);ctx.lineTo(branch.range.maxX,branch.range.maxY);ctx.stroke();ctx.beginPath();ctx.moveTo(branch.range.maxX,branch.range.maxY);ctx.lineTo(branch.range.minX,branch.range.maxY);ctx.stroke();ctx.beginPath();ctx.moveTo(branch.range.minX,branch.range.maxY);ctx.lineTo(branch.range.minX,branch.range.minY);ctx.stroke();/*
	     if (branch.mass > 0) {
	     ctx.circle(branch.centerOfMass.x, branch.centerOfMass.y, 3*branch.mass);
	     ctx.stroke();
	     }
	     */}}]);return BarnesHutSolver;}();/**
	 * Repulsion Solver
	 */var RepulsionSolver=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   * @param {{physicsNodeIndices: Array, physicsEdgeIndices: Array, forces: {}, velocities: {}}} physicsBody
	   * @param {Object} options
	   */function RepulsionSolver(body,physicsBody,options){classCallCheck(this,RepulsionSolver);this._rng=Alea("REPULSION SOLVER");this.body=body;this.physicsBody=physicsBody;this.setOptions(options);}/**
	   *
	   * @param {Object} options
	   */createClass(RepulsionSolver,[{key:"setOptions",value:function setOptions(options){this.options=options;}/**
	   * Calculate the forces the nodes apply on each other based on a repulsion field.
	   * This field is linearly approximated.
	   *
	   * @private
	   */},{key:"solve",value:function solve(){var dx,dy,distance,fx,fy,repulsingForce,node1,node2;var nodes=this.body.nodes;var nodeIndices=this.physicsBody.physicsNodeIndices;var forces=this.physicsBody.forces;// repulsing forces between nodes
	var nodeDistance=this.options.nodeDistance;// approximation constants
	var a=-2/3/nodeDistance;var b=4/3;// we loop from i over all but the last entree in the array
	// j loops from i+1 to the last. This way we do not double count any of the indices, nor i === j
	for(var i=0;i<nodeIndices.length-1;i++){node1=nodes[nodeIndices[i]];for(var j=i+1;j<nodeIndices.length;j++){node2=nodes[nodeIndices[j]];dx=node2.x-node1.x;dy=node2.y-node1.y;distance=Math.sqrt(dx*dx+dy*dy);// same condition as BarnesHutSolver, making sure nodes are never 100% overlapping.
	if(distance===0){distance=0.1*this._rng();dx=distance;}if(distance<2*nodeDistance){if(distance<0.5*nodeDistance){repulsingForce=1.0;}else {repulsingForce=a*distance+b;// linear approx of  1 / (1 + Math.exp((distance / nodeDistance - 1) * steepness))
	}repulsingForce=repulsingForce/distance;fx=dx*repulsingForce;fy=dy*repulsingForce;forces[node1.id].x-=fx;forces[node1.id].y-=fy;forces[node2.id].x+=fx;forces[node2.id].y+=fy;}}}}}]);return RepulsionSolver;}();/**
	 * Hierarchical Repulsion Solver
	 */var HierarchicalRepulsionSolver=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   * @param {{physicsNodeIndices: Array, physicsEdgeIndices: Array, forces: {}, velocities: {}}} physicsBody
	   * @param {Object} options
	   */function HierarchicalRepulsionSolver(body,physicsBody,options){classCallCheck(this,HierarchicalRepulsionSolver);this.body=body;this.physicsBody=physicsBody;this.setOptions(options);}/**
	   *
	   * @param {Object} options
	   */createClass(HierarchicalRepulsionSolver,[{key:"setOptions",value:function setOptions(options){this.options=options;this.overlapAvoidanceFactor=Math.max(0,Math.min(1,this.options.avoidOverlap||0));}/**
	   * Calculate the forces the nodes apply on each other based on a repulsion field.
	   * This field is linearly approximated.
	   *
	   * @private
	   */},{key:"solve",value:function solve(){var nodes=this.body.nodes;var nodeIndices=this.physicsBody.physicsNodeIndices;var forces=this.physicsBody.forces;// repulsing forces between nodes
	var nodeDistance=this.options.nodeDistance;// we loop from i over all but the last entree in the array
	// j loops from i+1 to the last. This way we do not double count any of the indices, nor i === j
	for(var i=0;i<nodeIndices.length-1;i++){var node1=nodes[nodeIndices[i]];for(var j=i+1;j<nodeIndices.length;j++){var node2=nodes[nodeIndices[j]];// nodes only affect nodes on their level
	if(node1.level===node2.level){var theseNodesDistance=nodeDistance+this.overlapAvoidanceFactor*((node1.shape.radius||0)/2+(node2.shape.radius||0)/2);var dx=node2.x-node1.x;var dy=node2.y-node1.y;var distance=Math.sqrt(dx*dx+dy*dy);var steepness=0.05;var repulsingForce=void 0;if(distance<theseNodesDistance){repulsingForce=-Math.pow(steepness*distance,2)+Math.pow(steepness*theseNodesDistance,2);}else {repulsingForce=0;}// normalize force with
	if(distance!==0){repulsingForce=repulsingForce/distance;}var fx=dx*repulsingForce;var fy=dy*repulsingForce;forces[node1.id].x-=fx;forces[node1.id].y-=fy;forces[node2.id].x+=fx;forces[node2.id].y+=fy;}}}}}]);return HierarchicalRepulsionSolver;}();/**
	 * Spring Solver
	 */var SpringSolver=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   * @param {{physicsNodeIndices: Array, physicsEdgeIndices: Array, forces: {}, velocities: {}}} physicsBody
	   * @param {Object} options
	   */function SpringSolver(body,physicsBody,options){classCallCheck(this,SpringSolver);this.body=body;this.physicsBody=physicsBody;this.setOptions(options);}/**
	   *
	   * @param {Object} options
	   */createClass(SpringSolver,[{key:"setOptions",value:function setOptions(options){this.options=options;}/**
	   * This function calculates the springforces on the nodes, accounting for the support nodes.
	   *
	   * @private
	   */},{key:"solve",value:function solve(){var edgeLength,edge;var edgeIndices=this.physicsBody.physicsEdgeIndices;var edges=this.body.edges;var node1,node2,node3;// forces caused by the edges, modelled as springs
	for(var i=0;i<edgeIndices.length;i++){edge=edges[edgeIndices[i]];if(edge.connected===true&&edge.toId!==edge.fromId){// only calculate forces if nodes are in the same sector
	if(this.body.nodes[edge.toId]!==undefined&&this.body.nodes[edge.fromId]!==undefined){if(edge.edgeType.via!==undefined){edgeLength=edge.options.length===undefined?this.options.springLength:edge.options.length;node1=edge.to;node2=edge.edgeType.via;node3=edge.from;this._calculateSpringForce(node1,node2,0.5*edgeLength);this._calculateSpringForce(node2,node3,0.5*edgeLength);}else {// the * 1.5 is here so the edge looks as large as a smooth edge. It does not initially because the smooth edges use
	// the support nodes which exert a repulsive force on the to and from nodes, making the edge appear larger.
	edgeLength=edge.options.length===undefined?this.options.springLength*1.5:edge.options.length;this._calculateSpringForce(edge.from,edge.to,edgeLength);}}}}}/**
	   * This is the code actually performing the calculation for the function above.
	   *
	   * @param {Node} node1
	   * @param {Node} node2
	   * @param {number} edgeLength
	   * @private
	   */},{key:"_calculateSpringForce",value:function _calculateSpringForce(node1,node2,edgeLength){var dx=node1.x-node2.x;var dy=node1.y-node2.y;var distance=Math.max(Math.sqrt(dx*dx+dy*dy),0.01);// the 1/distance is so the fx and fy can be calculated without sine or cosine.
	var springForce=this.options.springConstant*(edgeLength-distance)/distance;var fx=dx*springForce;var fy=dy*springForce;// handle the case where one node is not part of the physcis
	if(this.physicsBody.forces[node1.id]!==undefined){this.physicsBody.forces[node1.id].x+=fx;this.physicsBody.forces[node1.id].y+=fy;}if(this.physicsBody.forces[node2.id]!==undefined){this.physicsBody.forces[node2.id].x-=fx;this.physicsBody.forces[node2.id].y-=fy;}}}]);return SpringSolver;}();/**
	 * Hierarchical Spring Solver
	 */var HierarchicalSpringSolver=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   * @param {{physicsNodeIndices: Array, physicsEdgeIndices: Array, forces: {}, velocities: {}}} physicsBody
	   * @param {Object} options
	   */function HierarchicalSpringSolver(body,physicsBody,options){classCallCheck(this,HierarchicalSpringSolver);this.body=body;this.physicsBody=physicsBody;this.setOptions(options);}/**
	   *
	   * @param {Object} options
	   */createClass(HierarchicalSpringSolver,[{key:"setOptions",value:function setOptions(options){this.options=options;}/**
	   * This function calculates the springforces on the nodes, accounting for the support nodes.
	   *
	   * @private
	   */},{key:"solve",value:function solve(){var edgeLength,edge;var dx,dy,fx,fy,springForce,distance;var edges=this.body.edges;var factor=0.5;var edgeIndices=this.physicsBody.physicsEdgeIndices;var nodeIndices=this.physicsBody.physicsNodeIndices;var forces=this.physicsBody.forces;// initialize the spring force counters
	for(var i=0;i<nodeIndices.length;i++){var nodeId=nodeIndices[i];forces[nodeId].springFx=0;forces[nodeId].springFy=0;}// forces caused by the edges, modelled as springs
	for(var _i7=0;_i7<edgeIndices.length;_i7++){edge=edges[edgeIndices[_i7]];if(edge.connected===true){edgeLength=edge.options.length===undefined?this.options.springLength:edge.options.length;dx=edge.from.x-edge.to.x;dy=edge.from.y-edge.to.y;distance=Math.sqrt(dx*dx+dy*dy);distance=distance===0?0.01:distance;// the 1/distance is so the fx and fy can be calculated without sine or cosine.
	springForce=this.options.springConstant*(edgeLength-distance)/distance;fx=dx*springForce;fy=dy*springForce;if(edge.to.level!=edge.from.level){if(forces[edge.toId]!==undefined){forces[edge.toId].springFx-=fx;forces[edge.toId].springFy-=fy;}if(forces[edge.fromId]!==undefined){forces[edge.fromId].springFx+=fx;forces[edge.fromId].springFy+=fy;}}else {if(forces[edge.toId]!==undefined){forces[edge.toId].x-=factor*fx;forces[edge.toId].y-=factor*fy;}if(forces[edge.fromId]!==undefined){forces[edge.fromId].x+=factor*fx;forces[edge.fromId].y+=factor*fy;}}}}// normalize spring forces
	springForce=1;var springFx,springFy;for(var _i8=0;_i8<nodeIndices.length;_i8++){var _nodeId3=nodeIndices[_i8];springFx=Math.min(springForce,Math.max(-springForce,forces[_nodeId3].springFx));springFy=Math.min(springForce,Math.max(-springForce,forces[_nodeId3].springFy));forces[_nodeId3].x+=springFx;forces[_nodeId3].y+=springFy;}// retain energy balance
	var totalFx=0;var totalFy=0;for(var _i9=0;_i9<nodeIndices.length;_i9++){var _nodeId4=nodeIndices[_i9];totalFx+=forces[_nodeId4].x;totalFy+=forces[_nodeId4].y;}var correctionFx=totalFx/nodeIndices.length;var correctionFy=totalFy/nodeIndices.length;for(var _i10=0;_i10<nodeIndices.length;_i10++){var _nodeId5=nodeIndices[_i10];forces[_nodeId5].x-=correctionFx;forces[_nodeId5].y-=correctionFy;}}}]);return HierarchicalSpringSolver;}();/**
	 * Central Gravity Solver
	 */var CentralGravitySolver=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   * @param {{physicsNodeIndices: Array, physicsEdgeIndices: Array, forces: {}, velocities: {}}} physicsBody
	   * @param {Object} options
	   */function CentralGravitySolver(body,physicsBody,options){classCallCheck(this,CentralGravitySolver);this.body=body;this.physicsBody=physicsBody;this.setOptions(options);}/**
	   *
	   * @param {Object} options
	   */createClass(CentralGravitySolver,[{key:"setOptions",value:function setOptions(options){this.options=options;}/**
	   * Calculates forces for each node
	   */},{key:"solve",value:function solve(){var dx,dy,distance,node;var nodes=this.body.nodes;var nodeIndices=this.physicsBody.physicsNodeIndices;var forces=this.physicsBody.forces;for(var i=0;i<nodeIndices.length;i++){var nodeId=nodeIndices[i];node=nodes[nodeId];dx=-node.x;dy=-node.y;distance=Math.sqrt(dx*dx+dy*dy);this._calculateForces(distance,dx,dy,forces,node);}}/**
	   * Calculate the forces based on the distance.
	   * @param {number} distance
	   * @param {number} dx
	   * @param {number} dy
	   * @param {Object<Node.id, vis.Node>} forces
	   * @param {Node} node
	   * @private
	   */},{key:"_calculateForces",value:function _calculateForces(distance,dx,dy,forces,node){var gravityForce=distance===0?0:this.options.centralGravity/distance;forces[node.id].x=dx*gravityForce;forces[node.id].y=dy*gravityForce;}}]);return CentralGravitySolver;}();/**
	 * @extends BarnesHutSolver
	 */var ForceAtlas2BasedRepulsionSolver=/*#__PURE__*/function(_BarnesHutSolver){inherits(ForceAtlas2BasedRepulsionSolver,_BarnesHutSolver);var _super26=_createSuper$1(ForceAtlas2BasedRepulsionSolver);/**
	   * @param {Object} body
	   * @param {{physicsNodeIndices: Array, physicsEdgeIndices: Array, forces: {}, velocities: {}}} physicsBody
	   * @param {Object} options
	   */function ForceAtlas2BasedRepulsionSolver(body,physicsBody,options){var _this25;classCallCheck(this,ForceAtlas2BasedRepulsionSolver);_this25=_super26.call(this,body,physicsBody,options);_this25._rng=Alea("FORCE ATLAS 2 BASED REPULSION SOLVER");return _this25;}/**
	   * Calculate the forces based on the distance.
	   *
	   * @param {number} distance
	   * @param {number} dx
	   * @param {number} dy
	   * @param {Node} node
	   * @param {Object} parentBranch
	   * @private
	   */createClass(ForceAtlas2BasedRepulsionSolver,[{key:"_calculateForces",value:function _calculateForces(distance,dx,dy,node,parentBranch){if(distance===0){distance=0.1*this._rng();dx=distance;}if(this.overlapAvoidanceFactor<1&&node.shape.radius){distance=Math.max(0.1+this.overlapAvoidanceFactor*node.shape.radius,distance-node.shape.radius);}var degree=node.edges.length+1;// the dividing by the distance cubed instead of squared allows us to get the fx and fy components without sines and cosines
	// it is shorthand for gravityforce with distance squared and fx = dx/distance * gravityForce
	var gravityForce=this.options.gravitationalConstant*parentBranch.mass*node.options.mass*degree/Math.pow(distance,2);var fx=dx*gravityForce;var fy=dy*gravityForce;this.physicsBody.forces[node.id].x+=fx;this.physicsBody.forces[node.id].y+=fy;}}]);return ForceAtlas2BasedRepulsionSolver;}(BarnesHutSolver);/**
	 * @extends CentralGravitySolver
	 */var ForceAtlas2BasedCentralGravitySolver=/*#__PURE__*/function(_CentralGravitySolver){inherits(ForceAtlas2BasedCentralGravitySolver,_CentralGravitySolver);var _super27=_createSuper$1(ForceAtlas2BasedCentralGravitySolver);/**
	   * @param {Object} body
	   * @param {{physicsNodeIndices: Array, physicsEdgeIndices: Array, forces: {}, velocities: {}}} physicsBody
	   * @param {Object} options
	   */function ForceAtlas2BasedCentralGravitySolver(body,physicsBody,options){classCallCheck(this,ForceAtlas2BasedCentralGravitySolver);return _super27.call(this,body,physicsBody,options);}/**
	   * Calculate the forces based on the distance.
	   *
	   * @param {number} distance
	   * @param {number} dx
	   * @param {number} dy
	   * @param {Object<Node.id, Node>} forces
	   * @param {Node} node
	   * @private
	   */createClass(ForceAtlas2BasedCentralGravitySolver,[{key:"_calculateForces",value:function _calculateForces(distance,dx,dy,forces,node){if(distance>0){var degree=node.edges.length+1;var gravityForce=this.options.centralGravity*degree*node.options.mass;forces[node.id].x=dx*gravityForce;forces[node.id].y=dy*gravityForce;}}}]);return ForceAtlas2BasedCentralGravitySolver;}(CentralGravitySolver);/**
	 * The physics engine
	 */var PhysicsEngine=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   */function PhysicsEngine(body){classCallCheck(this,PhysicsEngine);this.body=body;this.physicsBody={physicsNodeIndices:[],physicsEdgeIndices:[],forces:{},velocities:{}};this.physicsEnabled=true;this.simulationInterval=1000/60;this.requiresTimeout=true;this.previousStates={};this.referenceState={};this.freezeCache={};this.renderTimer=undefined;// parameters for the adaptive timestep
	this.adaptiveTimestep=false;this.adaptiveTimestepEnabled=false;this.adaptiveCounter=0;this.adaptiveInterval=3;this.stabilized=false;this.startedStabilization=false;this.stabilizationIterations=0;this.ready=false;// will be set to true if the stabilize
	// default options
	this.options={};this.defaultOptions={enabled:true,barnesHut:{theta:0.5,gravitationalConstant:-2000,centralGravity:0.3,springLength:95,springConstant:0.04,damping:0.09,avoidOverlap:0},forceAtlas2Based:{theta:0.5,gravitationalConstant:-50,centralGravity:0.01,springConstant:0.08,springLength:100,damping:0.4,avoidOverlap:0},repulsion:{centralGravity:0.2,springLength:200,springConstant:0.05,nodeDistance:100,damping:0.09,avoidOverlap:0},hierarchicalRepulsion:{centralGravity:0.0,springLength:100,springConstant:0.01,nodeDistance:120,damping:0.09},maxVelocity:50,minVelocity:0.75,// px/s
	solver:'barnesHut',stabilization:{enabled:true,iterations:1000,// maximum number of iteration to stabilize
	updateInterval:50,onlyDynamicEdges:false,fit:true},timestep:0.5,adaptiveTimestep:true,wind:{x:0,y:0}};extend(this.options,this.defaultOptions);this.timestep=0.5;this.layoutFailed=false;this.bindEventListeners();}/**
	   * Binds event listeners
	   */createClass(PhysicsEngine,[{key:"bindEventListeners",value:function bindEventListeners(){var _this26=this;this.body.emitter.on('initPhysics',function(){_this26.initPhysics();});this.body.emitter.on('_layoutFailed',function(){_this26.layoutFailed=true;});this.body.emitter.on('resetPhysics',function(){_this26.stopSimulation();_this26.ready=false;});this.body.emitter.on('disablePhysics',function(){_this26.physicsEnabled=false;_this26.stopSimulation();});this.body.emitter.on('restorePhysics',function(){_this26.setOptions(_this26.options);if(_this26.ready===true){_this26.startSimulation();}});this.body.emitter.on('startSimulation',function(){if(_this26.ready===true){_this26.startSimulation();}});this.body.emitter.on('stopSimulation',function(){_this26.stopSimulation();});this.body.emitter.on('destroy',function(){_this26.stopSimulation(false);_this26.body.emitter.off();});this.body.emitter.on("_dataChanged",function(){// Nodes and/or edges have been added or removed, update shortcut lists.
	_this26.updatePhysicsData();});// debug: show forces
	// this.body.emitter.on("afterDrawing", (ctx) => {this._drawForces(ctx);});
	}/**
	   * set the physics options
	   * @param {Object} options
	   */},{key:"setOptions",value:function setOptions(options){if(options!==undefined){if(options===false){this.options.enabled=false;this.physicsEnabled=false;this.stopSimulation();}else if(options===true){this.options.enabled=true;this.physicsEnabled=true;this.startSimulation();}else {this.physicsEnabled=true;selectiveNotDeepExtend(['stabilization'],this.options,options);mergeOptions(this.options,options,'stabilization');if(options.enabled===undefined){this.options.enabled=true;}if(this.options.enabled===false){this.physicsEnabled=false;this.stopSimulation();}var wind=this.options.wind;if(wind){if(typeof wind.x!=='number'||isNan$2(wind.x)){wind.x=0;}if(typeof wind.y!=='number'||isNan$2(wind.y)){wind.y=0;}}// set the timestep
	this.timestep=this.options.timestep;}}this.init();}/**
	   * configure the engine.
	   */},{key:"init",value:function init(){var options;if(this.options.solver==='forceAtlas2Based'){options=this.options.forceAtlas2Based;this.nodesSolver=new ForceAtlas2BasedRepulsionSolver(this.body,this.physicsBody,options);this.edgesSolver=new SpringSolver(this.body,this.physicsBody,options);this.gravitySolver=new ForceAtlas2BasedCentralGravitySolver(this.body,this.physicsBody,options);}else if(this.options.solver==='repulsion'){options=this.options.repulsion;this.nodesSolver=new RepulsionSolver(this.body,this.physicsBody,options);this.edgesSolver=new SpringSolver(this.body,this.physicsBody,options);this.gravitySolver=new CentralGravitySolver(this.body,this.physicsBody,options);}else if(this.options.solver==='hierarchicalRepulsion'){options=this.options.hierarchicalRepulsion;this.nodesSolver=new HierarchicalRepulsionSolver(this.body,this.physicsBody,options);this.edgesSolver=new HierarchicalSpringSolver(this.body,this.physicsBody,options);this.gravitySolver=new CentralGravitySolver(this.body,this.physicsBody,options);}else {// barnesHut
	options=this.options.barnesHut;this.nodesSolver=new BarnesHutSolver(this.body,this.physicsBody,options);this.edgesSolver=new SpringSolver(this.body,this.physicsBody,options);this.gravitySolver=new CentralGravitySolver(this.body,this.physicsBody,options);}this.modelOptions=options;}/**
	   * initialize the engine
	   */},{key:"initPhysics",value:function initPhysics(){if(this.physicsEnabled===true&&this.options.enabled===true){if(this.options.stabilization.enabled===true){this.stabilize();}else {this.stabilized=false;this.ready=true;this.body.emitter.emit('fit',{},this.layoutFailed);// if the layout failed, we use the approximation for the zoom
	this.startSimulation();}}else {this.ready=true;this.body.emitter.emit('fit');}}/**
	   * Start the simulation
	   */},{key:"startSimulation",value:function startSimulation(){if(this.physicsEnabled===true&&this.options.enabled===true){this.stabilized=false;// when visible, adaptivity is disabled.
	this.adaptiveTimestep=false;// this sets the width of all nodes initially which could be required for the avoidOverlap
	this.body.emitter.emit("_resizeNodes");if(this.viewFunction===undefined){var _context29;this.viewFunction=bind$2(_context29=this.simulationStep).call(_context29,this);this.body.emitter.on('initRedraw',this.viewFunction);this.body.emitter.emit('_startRendering');}}else {this.body.emitter.emit('_redraw');}}/**
	   * Stop the simulation, force stabilization.
	   * @param {boolean} [emit=true]
	   */},{key:"stopSimulation",value:function stopSimulation(){var emit=arguments.length>0&&arguments[0]!==undefined?arguments[0]:true;this.stabilized=true;if(emit===true){this._emitStabilized();}if(this.viewFunction!==undefined){this.body.emitter.off('initRedraw',this.viewFunction);this.viewFunction=undefined;if(emit===true){this.body.emitter.emit('_stopRendering');}}}/**
	   * The viewFunction inserts this step into each render loop. It calls the physics tick and handles the cleanup at stabilized.
	   *
	   */},{key:"simulationStep",value:function simulationStep(){// check if the physics have settled
	var startTime=now$2();this.physicsTick();var physicsTime=now$2()-startTime;// run double speed if it is a little graph
	if((physicsTime<0.4*this.simulationInterval||this.runDoubleSpeed===true)&&this.stabilized===false){this.physicsTick();// this makes sure there is no jitter. The decision is taken once to run it at double speed.
	this.runDoubleSpeed=true;}if(this.stabilized===true){this.stopSimulation();}}/**
	   * trigger the stabilized event.
	   *
	   * @param {number} [amountOfIterations=this.stabilizationIterations]
	   * @private
	   */},{key:"_emitStabilized",value:function _emitStabilized(){var _this27=this;var amountOfIterations=arguments.length>0&&arguments[0]!==undefined?arguments[0]:this.stabilizationIterations;if(this.stabilizationIterations>1||this.startedStabilization===true){setTimeout$2(function(){_this27.body.emitter.emit('stabilized',{iterations:amountOfIterations});_this27.startedStabilization=false;_this27.stabilizationIterations=0;},0);}}/**
	   * Calculate the forces for one physics iteration and move the nodes.
	   * @private
	   */},{key:"physicsStep",value:function physicsStep(){this.gravitySolver.solve();this.nodesSolver.solve();this.edgesSolver.solve();this.moveNodes();}/**
	   * Make dynamic adjustments to the timestep, based on current state.
	   *
	   * Helper function for physicsTick().
	   * @private
	   */},{key:"adjustTimeStep",value:function adjustTimeStep(){var factor=1.2;// Factor for increasing the timestep on success.
	// we compare the two steps. if it is acceptable we double the step.
	if(this._evaluateStepQuality()===true){this.timestep=factor*this.timestep;}else {// if not, we decrease the step to a minimum of the options timestep.
	// if the decreased timestep is smaller than the options step, we do not reset the counter
	// we assume that the options timestep is stable enough.
	if(this.timestep/factor<this.options.timestep){this.timestep=this.options.timestep;}else {// if the timestep was larger than 2 times the option one we check the adaptivity again to ensure
	// that large instabilities do not form.
	this.adaptiveCounter=-1;// check again next iteration
	this.timestep=Math.max(this.options.timestep,this.timestep/factor);}}}/**
	   * A single simulation step (or 'tick') in the physics simulation
	   *
	   * @private
	   */},{key:"physicsTick",value:function physicsTick(){this._startStabilizing();// this ensures that there is no start event when the network is already stable.
	if(this.stabilized===true)return;// adaptivity means the timestep adapts to the situation, only applicable for stabilization
	if(this.adaptiveTimestep===true&&this.adaptiveTimestepEnabled===true){// timestep remains stable for "interval" iterations.
	var doAdaptive=this.adaptiveCounter%this.adaptiveInterval===0;if(doAdaptive){// first the big step and revert.
	this.timestep=2*this.timestep;this.physicsStep();this.revert();// saves the reference state
	// now the normal step. Since this is the last step, it is the more stable one and we will take this.
	this.timestep=0.5*this.timestep;// since it's half the step, we do it twice.
	this.physicsStep();this.physicsStep();this.adjustTimeStep();}else {this.physicsStep();// normal step, keeping timestep constant
	}this.adaptiveCounter+=1;}else {// case for the static timestep, we reset it to the one in options and take a normal step.
	this.timestep=this.options.timestep;this.physicsStep();}if(this.stabilized===true)this.revert();this.stabilizationIterations++;}/**
	   * Nodes and edges can have the physics toggles on or off. A collection of indices is created here so we can skip the check all the time.
	   *
	   * @private
	   */},{key:"updatePhysicsData",value:function updatePhysicsData(){this.physicsBody.forces={};this.physicsBody.physicsNodeIndices=[];this.physicsBody.physicsEdgeIndices=[];var nodes=this.body.nodes;var edges=this.body.edges;// get node indices for physics
	for(var nodeId in nodes){if(nodes.hasOwnProperty(nodeId)){if(nodes[nodeId].options.physics===true){this.physicsBody.physicsNodeIndices.push(nodes[nodeId].id);}}}// get edge indices for physics
	for(var edgeId in edges){if(edges.hasOwnProperty(edgeId)){if(edges[edgeId].options.physics===true){this.physicsBody.physicsEdgeIndices.push(edges[edgeId].id);}}}// get the velocity and the forces vector
	for(var i=0;i<this.physicsBody.physicsNodeIndices.length;i++){var _nodeId6=this.physicsBody.physicsNodeIndices[i];this.physicsBody.forces[_nodeId6]={x:0,y:0};// forces can be reset because they are recalculated. Velocities have to persist.
	if(this.physicsBody.velocities[_nodeId6]===undefined){this.physicsBody.velocities[_nodeId6]={x:0,y:0};}}// clean deleted nodes from the velocity vector
	for(var _nodeId7 in this.physicsBody.velocities){if(nodes[_nodeId7]===undefined){delete this.physicsBody.velocities[_nodeId7];}}}/**
	   * Revert the simulation one step. This is done so after stabilization, every new start of the simulation will also say stabilized.
	   */},{key:"revert",value:function revert(){var nodeIds=keys$6(this.previousStates);var nodes=this.body.nodes;var velocities=this.physicsBody.velocities;this.referenceState={};for(var i=0;i<nodeIds.length;i++){var nodeId=nodeIds[i];if(nodes[nodeId]!==undefined){if(nodes[nodeId].options.physics===true){this.referenceState[nodeId]={positions:{x:nodes[nodeId].x,y:nodes[nodeId].y}};velocities[nodeId].x=this.previousStates[nodeId].vx;velocities[nodeId].y=this.previousStates[nodeId].vy;nodes[nodeId].x=this.previousStates[nodeId].x;nodes[nodeId].y=this.previousStates[nodeId].y;}}else {delete this.previousStates[nodeId];}}}/**
	   * This compares the reference state to the current state
	   *
	   * @returns {boolean}
	   * @private
	   */},{key:"_evaluateStepQuality",value:function _evaluateStepQuality(){var dx,dy,dpos;var nodes=this.body.nodes;var reference=this.referenceState;var posThreshold=0.3;for(var nodeId in this.referenceState){if(this.referenceState.hasOwnProperty(nodeId)&&nodes[nodeId]!==undefined){dx=nodes[nodeId].x-reference[nodeId].positions.x;dy=nodes[nodeId].y-reference[nodeId].positions.y;dpos=Math.sqrt(Math.pow(dx,2)+Math.pow(dy,2));if(dpos>posThreshold){return false;}}}return true;}/**
	   * move the nodes one timestep and check if they are stabilized
	   */},{key:"moveNodes",value:function moveNodes(){var nodeIndices=this.physicsBody.physicsNodeIndices;var maxNodeVelocity=0;var averageNodeVelocity=0;// the velocity threshold (energy in the system) for the adaptivity toggle
	var velocityAdaptiveThreshold=5;for(var i=0;i<nodeIndices.length;i++){var nodeId=nodeIndices[i];var nodeVelocity=this._performStep(nodeId);// stabilized is true if stabilized is true and velocity is smaller than vmin --> all nodes must be stabilized
	maxNodeVelocity=Math.max(maxNodeVelocity,nodeVelocity);averageNodeVelocity+=nodeVelocity;}// evaluating the stabilized and adaptiveTimestepEnabled conditions
	this.adaptiveTimestepEnabled=averageNodeVelocity/nodeIndices.length<velocityAdaptiveThreshold;this.stabilized=maxNodeVelocity<this.options.minVelocity;}/**
	   * Calculate new velocity for a coordinate direction
	   *
	   * @param {number} v  velocity for current coordinate
	   * @param {number} f  regular force for current coordinate
	   * @param {number} m  mass of current node
	   * @returns {number} new velocity for current coordinate
	   * @private
	   */},{key:"calculateComponentVelocity",value:function calculateComponentVelocity(v,f,m){var df=this.modelOptions.damping*v;// damping force
	var a=(f-df)/m;// acceleration
	v+=a*this.timestep;// Put a limit on the velocities if it is really high
	var maxV=this.options.maxVelocity||1e9;if(Math.abs(v)>maxV){v=v>0?maxV:-maxV;}return v;}/**
	   * Perform the actual step
	   *
	   * @param {Node.id} nodeId
	   * @returns {number} the new velocity of given node
	   * @private
	   */},{key:"_performStep",value:function _performStep(nodeId){var node=this.body.nodes[nodeId];var force=this.physicsBody.forces[nodeId];if(this.options.wind){force.x+=this.options.wind.x;force.y+=this.options.wind.y;}var velocity=this.physicsBody.velocities[nodeId];// store the state so we can revert
	this.previousStates[nodeId]={x:node.x,y:node.y,vx:velocity.x,vy:velocity.y};if(node.options.fixed.x===false){velocity.x=this.calculateComponentVelocity(velocity.x,force.x,node.options.mass);node.x+=velocity.x*this.timestep;}else {force.x=0;velocity.x=0;}if(node.options.fixed.y===false){velocity.y=this.calculateComponentVelocity(velocity.y,force.y,node.options.mass);node.y+=velocity.y*this.timestep;}else {force.y=0;velocity.y=0;}var totalVelocity=Math.sqrt(Math.pow(velocity.x,2)+Math.pow(velocity.y,2));return totalVelocity;}/**
	   * When initializing and stabilizing, we can freeze nodes with a predefined position.
	   * This greatly speeds up stabilization because only the supportnodes for the smoothCurves have to settle.
	   *
	   * @private
	   */},{key:"_freezeNodes",value:function _freezeNodes(){var nodes=this.body.nodes;for(var id in nodes){if(nodes.hasOwnProperty(id)){if(nodes[id].x&&nodes[id].y){var fixed=nodes[id].options.fixed;this.freezeCache[id]={x:fixed.x,y:fixed.y};fixed.x=true;fixed.y=true;}}}}/**
	   * Unfreezes the nodes that have been frozen by _freezeDefinedNodes.
	   *
	   * @private
	   */},{key:"_restoreFrozenNodes",value:function _restoreFrozenNodes(){var nodes=this.body.nodes;for(var id in nodes){if(nodes.hasOwnProperty(id)){if(this.freezeCache[id]!==undefined){nodes[id].options.fixed.x=this.freezeCache[id].x;nodes[id].options.fixed.y=this.freezeCache[id].y;}}}this.freezeCache={};}/**
	   * Find a stable position for all nodes
	   *
	   * @param {number} [iterations=this.options.stabilization.iterations]
	   */},{key:"stabilize",value:function stabilize(){var _this28=this;var iterations=arguments.length>0&&arguments[0]!==undefined?arguments[0]:this.options.stabilization.iterations;if(typeof iterations!=='number'){iterations=this.options.stabilization.iterations;console.log('The stabilize method needs a numeric amount of iterations. Switching to default: ',iterations);}if(this.physicsBody.physicsNodeIndices.length===0){this.ready=true;return;}// enable adaptive timesteps
	this.adaptiveTimestep=this.options.adaptiveTimestep;// this sets the width of all nodes initially which could be required for the avoidOverlap
	this.body.emitter.emit("_resizeNodes");this.stopSimulation();// stop the render loop
	this.stabilized=false;// block redraw requests
	this.body.emitter.emit('_blockRedraw');this.targetIterations=iterations;// start the stabilization
	if(this.options.stabilization.onlyDynamicEdges===true){this._freezeNodes();}this.stabilizationIterations=0;setTimeout$2(function(){return _this28._stabilizationBatch();},0);}/**
	   * If not already stabilizing, start it and emit a start event.
	   *
	   * @returns {boolean} true if stabilization started with this call
	   * @private
	   */},{key:"_startStabilizing",value:function _startStabilizing(){if(this.startedStabilization===true)return false;this.body.emitter.emit('startStabilizing');this.startedStabilization=true;return true;}/**
	   * One batch of stabilization
	   * @private
	   */},{key:"_stabilizationBatch",value:function _stabilizationBatch(){var _this29=this;var running=function running(){return _this29.stabilized===false&&_this29.stabilizationIterations<_this29.targetIterations;};var sendProgress=function sendProgress(){_this29.body.emitter.emit('stabilizationProgress',{iterations:_this29.stabilizationIterations,total:_this29.targetIterations});};if(this._startStabilizing()){sendProgress();// Ensure that there is at least one start event.
	}var count=0;while(running()&&count<this.options.stabilization.updateInterval){this.physicsTick();count++;}sendProgress();if(running()){var _context30;setTimeout$2(bind$2(_context30=this._stabilizationBatch).call(_context30,this),0);}else {this._finalizeStabilization();}}/**
	   * Wrap up the stabilization, fit and emit the events.
	   * @private
	   */},{key:"_finalizeStabilization",value:function _finalizeStabilization(){this.body.emitter.emit('_allowRedraw');if(this.options.stabilization.fit===true){this.body.emitter.emit('fit');}if(this.options.stabilization.onlyDynamicEdges===true){this._restoreFrozenNodes();}this.body.emitter.emit('stabilizationIterationsDone');this.body.emitter.emit('_requestRedraw');if(this.stabilized===true){this._emitStabilized();}else {this.startSimulation();}this.ready=true;}//---------------------------  DEBUGGING BELOW  ---------------------------//
	/**
	   * Debug function that display arrows for the forces currently active in the network.
	   *
	   * Use this when debugging only.
	   *
	   * @param {CanvasRenderingContext2D} ctx
	   * @private
	   */},{key:"_drawForces",value:function _drawForces(ctx){for(var i=0;i<this.physicsBody.physicsNodeIndices.length;i++){var _index3=this.physicsBody.physicsNodeIndices[i];var node=this.body.nodes[_index3];var force=this.physicsBody.forces[_index3];var factor=20;var colorFactor=0.03;var forceSize=Math.sqrt(Math.pow(force.x,2)+Math.pow(force.x,2));var size=Math.min(Math.max(5,forceSize),15);var arrowSize=3*size;var color=HSVToHex((180-Math.min(1,Math.max(0,colorFactor*forceSize))*180)/360,1,1);var point={x:node.x+factor*force.x,y:node.y+factor*force.y};ctx.lineWidth=size;ctx.strokeStyle=color;ctx.beginPath();ctx.moveTo(node.x,node.y);ctx.lineTo(point.x,point.y);ctx.stroke();var angle=Math.atan2(force.y,force.x);ctx.fillStyle=color;EndPoints.draw(ctx,{type:'arrow',point:point,angle:angle,length:arrowSize});fill$2(ctx).call(ctx);}}}]);return PhysicsEngine;}();/**
	 * Utility Class
	 */var NetworkUtil=/*#__PURE__*/function(){/**
	   * @ignore
	   */function NetworkUtil(){classCallCheck(this,NetworkUtil);}/**
	   * Find the center position of the network considering the bounding boxes
	   *
	   * @param {Array.<Node>} allNodes
	   * @param {Array.<Node>} [specificNodes=[]]
	   * @returns {{minX: number, maxX: number, minY: number, maxY: number}}
	   * @static
	   */createClass(NetworkUtil,null,[{key:"getRange",value:function getRange(allNodes){var specificNodes=arguments.length>1&&arguments[1]!==undefined?arguments[1]:[];var minY=1e9,maxY=-1e9,minX=1e9,maxX=-1e9,node;if(specificNodes.length>0){for(var i=0;i<specificNodes.length;i++){node=allNodes[specificNodes[i]];if(minX>node.shape.boundingBox.left){minX=node.shape.boundingBox.left;}if(maxX<node.shape.boundingBox.right){maxX=node.shape.boundingBox.right;}if(minY>node.shape.boundingBox.top){minY=node.shape.boundingBox.top;}// top is negative, bottom is positive
	if(maxY<node.shape.boundingBox.bottom){maxY=node.shape.boundingBox.bottom;}// top is negative, bottom is positive
	}}if(minX===1e9&&maxX===-1e9&&minY===1e9&&maxY===-1e9){minY=0,maxY=0,minX=0,maxX=0;}return {minX:minX,maxX:maxX,minY:minY,maxY:maxY};}/**
	   * Find the center position of the network
	   *
	   * @param {Array.<Node>} allNodes
	   * @param {Array.<Node>} [specificNodes=[]]
	   * @returns {{minX: number, maxX: number, minY: number, maxY: number}}
	   * @static
	   */},{key:"getRangeCore",value:function getRangeCore(allNodes){var specificNodes=arguments.length>1&&arguments[1]!==undefined?arguments[1]:[];var minY=1e9,maxY=-1e9,minX=1e9,maxX=-1e9,node;if(specificNodes.length>0){for(var i=0;i<specificNodes.length;i++){node=allNodes[specificNodes[i]];if(minX>node.x){minX=node.x;}if(maxX<node.x){maxX=node.x;}if(minY>node.y){minY=node.y;}// top is negative, bottom is positive
	if(maxY<node.y){maxY=node.y;}// top is negative, bottom is positive
	}}if(minX===1e9&&maxX===-1e9&&minY===1e9&&maxY===-1e9){minY=0,maxY=0,minX=0,maxX=0;}return {minX:minX,maxX:maxX,minY:minY,maxY:maxY};}/**
	   * @param {object} range = {minX: minX, maxX: maxX, minY: minY, maxY: maxY};
	   * @returns {{x: number, y: number}}
	   * @static
	   */},{key:"findCenter",value:function findCenter(range){return {x:0.5*(range.maxX+range.minX),y:0.5*(range.maxY+range.minY)};}/**
	   * This returns a clone of the options or options of the edge or node to be used for construction of new edges or check functions for new nodes.
	   * @param {vis.Item} item
	   * @param {'node'|undefined} type
	   * @returns {{}}
	   * @static
	   */},{key:"cloneOptions",value:function cloneOptions(item,type){var clonedOptions={};if(type===undefined||type==='node'){deepExtend(clonedOptions,item.options,true);clonedOptions.x=item.x;clonedOptions.y=item.y;clonedOptions.amountOfConnections=item.edges.length;}else {deepExtend(clonedOptions,item.options,true);}return clonedOptions;}}]);return NetworkUtil;}();/**
	 * A Cluster is a special Node that allows a group of Nodes positioned closely together
	 * to be represented by a single Cluster Node.
	 *
	 * @extends Node
	 */var Cluster=/*#__PURE__*/function(_Node){inherits(Cluster,_Node);var _super28=_createSuper$1(Cluster);/**
	   * @param {Object} options
	   * @param {Object} body
	   * @param {Array.<HTMLImageElement>}imagelist
	   * @param {Array} grouplist
	   * @param {Object} globalOptions
	   * @param {Object} defaultOptions     Global default options for nodes
	   */function Cluster(options,body,imagelist,grouplist,globalOptions,defaultOptions){var _this30;classCallCheck(this,Cluster);_this30=_super28.call(this,options,body,imagelist,grouplist,globalOptions,defaultOptions);_this30.isCluster=true;_this30.containedNodes={};_this30.containedEdges={};return _this30;}/**
	   * Transfer child cluster data to current and disconnect the child cluster.
	   *
	   * Please consult the header comment in 'Clustering.js' for the fields set here.
	   *
	   * @param {string|number} childClusterId  id of child cluster to open
	   */createClass(Cluster,[{key:"_openChildCluster",value:function _openChildCluster(childClusterId){var _this31=this;var childCluster=this.body.nodes[childClusterId];if(this.containedNodes[childClusterId]===undefined){throw new Error('node with id: '+childClusterId+' not in current cluster');}if(!childCluster.isCluster){throw new Error('node with id: '+childClusterId+' is not a cluster');}// Disconnect child cluster from current cluster
	delete this.containedNodes[childClusterId];forEach$4(childCluster.edges,function(edge){delete _this31.containedEdges[edge.id];});// Transfer nodes and edges
	forEach$4(childCluster.containedNodes,function(node,nodeId){_this31.containedNodes[nodeId]=node;});childCluster.containedNodes={};forEach$4(childCluster.containedEdges,function(edge,edgeId){_this31.containedEdges[edgeId]=edge;});childCluster.containedEdges={};// Transfer edges within cluster edges which are clustered
	forEach$4(childCluster.edges,function(clusterEdge){forEach$4(_this31.edges,function(parentClusterEdge){var _context31,_context32;// Assumption: a clustered edge can only be present in a single clustering edge
	// Not tested here
	var index=indexOf$3(_context31=parentClusterEdge.clusteringEdgeReplacingIds).call(_context31,clusterEdge.id);if(index===-1)return;forEach$4(clusterEdge.clusteringEdgeReplacingIds,function(srcId){parentClusterEdge.clusteringEdgeReplacingIds.push(srcId);// Maintain correct bookkeeping for transferred edge
	_this31.body.edges[srcId].edgeReplacedById=parentClusterEdge.id;});// Remove cluster edge from parent cluster edge
	splice$2(_context32=parentClusterEdge.clusteringEdgeReplacingIds).call(_context32,index,1);});});childCluster.edges=[];}}]);return Cluster;}(Node);/* ===========================================================================

	# TODO

	- `edgeReplacedById` not cleaned up yet on cluster edge removal
	- allowSingleNodeCluster could be a global option as well; currently needs to always
	  be passed to clustering methods

	----------------------------------------------

	# State Model for Clustering

	The total state for clustering is non-trivial. It is useful to have a model
	available as to how it works. The following documents the relevant state items.


	## Network State

	The following `network`-members are relevant to clustering:

	- `body.nodes`       - all nodes actively participating in the network
	- `body.edges`       - same for edges
	- `body.nodeIndices` - id's of nodes that are visible at a given moment
	- `body.edgeIndices` - same for edges

	This includes:

	- helper nodes for dragging in `manipulation`
	- helper nodes for edge type `dynamic`
	- cluster nodes and edges
	- there may be more than this.

	A node/edge may be missing in the `Indices` member if:

	- it is a helper node
	- the node or edge state has option `hidden` set
	- It is not visible due to clustering


	## Clustering State

	For the hashes, the id's of the nodes/edges are used as key.

	Member `network.clustering` contains the following items:

	- `clusteredNodes` - hash with values: { clusterId: <id of cluster>, node: <node instance>}
	- `clusteredEdges` - hash with values: restore information for given edge


	Due to nesting of clusters, these members can contain cluster nodes and edges as well.

	The important thing to note here, is that the clustered nodes and edges also
	appear in the members of the cluster nodes. For data update, it is therefore 
	important to scan these lists as well as the cluster nodes.


	### Cluster Node

	A cluster node has the following extra fields:

	- `isCluster : true` - indication that this is a cluster node
	- `containedNodes`   - hash of nodes contained in this cluster
	- `containedEdges`   - same for edges
	- `edges`            - array of cluster edges for this node 


	**NOTE:**

	- `containedEdges` can also contain edges which are not clustered; e.g. an edge
	   connecting two nodes in the same cluster.


	### Cluster Edge

	These are the items in the `edges` member of a clustered node. They have the
	following relevant members:

	- 'clusteringEdgeReplacingIds` - array of id's of edges replaced by this edge

	Note that it's possible to nest clusters, so that `clusteringEdgeReplacingIds`
	can contain edge id's of other clusters.


	### Clustered Edge

	This is any edge contained by a cluster edge. It gets the following additional
	member:

	- `edgeReplacedById` - id of the cluster edge in which current edge is clustered


	   =========================================================================== */ /**
	 * The clustering engine
	 */var ClusterEngine=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   */function ClusterEngine(body){var _this32=this;classCallCheck(this,ClusterEngine);this.body=body;this.clusteredNodes={};// key: node id, value: { clusterId: <id of cluster>, node: <node instance>}
	this.clusteredEdges={};// key: edge id, value: restore information for given edge
	this.options={};this.defaultOptions={};extend(this.options,this.defaultOptions);this.body.emitter.on('_resetData',function(){_this32.clusteredNodes={};_this32.clusteredEdges={};});}/**
	  *
	  * @param {number} hubsize
	  * @param {Object} options
	  */createClass(ClusterEngine,[{key:"clusterByHubsize",value:function clusterByHubsize(hubsize,options){if(hubsize===undefined){hubsize=this._getHubSize();}else if(_typeof_1(hubsize)==="object"){options=this._checkOptions(hubsize);hubsize=this._getHubSize();}var nodesToCluster=[];for(var i=0;i<this.body.nodeIndices.length;i++){var node=this.body.nodes[this.body.nodeIndices[i]];if(node.edges.length>=hubsize){nodesToCluster.push(node.id);}}for(var _i11=0;_i11<nodesToCluster.length;_i11++){this.clusterByConnection(nodesToCluster[_i11],options,true);}this.body.emitter.emit('_dataChanged');}/**
	   * loop over all nodes, check if they adhere to the condition and cluster if needed.
	   * @param {Object} options
	   * @param {boolean} [refreshData=true]
	   */},{key:"cluster",value:function cluster(){var _this33=this;var options=arguments.length>0&&arguments[0]!==undefined?arguments[0]:{};var refreshData=arguments.length>1&&arguments[1]!==undefined?arguments[1]:true;if(options.joinCondition===undefined){throw new Error("Cannot call clusterByNodeData without a joinCondition function in the options.");}// check if the options object is fine, append if needed
	options=this._checkOptions(options);var childNodesObj={};var childEdgesObj={};// collect the nodes that will be in the cluster
	forEach$4(this.body.nodes,function(node,nodeId){if(node.options&&options.joinCondition(node.options)===true){childNodesObj[nodeId]=node;// collect the edges that will be in the cluster
	forEach$4(node.edges,function(edge){if(_this33.clusteredEdges[edge.id]===undefined){childEdgesObj[edge.id]=edge;}});}});this._cluster(childNodesObj,childEdgesObj,options,refreshData);}/**
	   * Cluster all nodes in the network that have only X edges
	   * @param {number} edgeCount
	   * @param {Object} options
	   * @param {boolean} [refreshData=true]
	   */},{key:"clusterByEdgeCount",value:function clusterByEdgeCount(edgeCount,options){var _this34=this;var refreshData=arguments.length>2&&arguments[2]!==undefined?arguments[2]:true;options=this._checkOptions(options);var clusters=[];var usedNodes={};var edge,edges,relevantEdgeCount;// collect the nodes that will be in the cluster
	var _loop2=function _loop2(i){var childNodesObj={};var childEdgesObj={};var nodeId=_this34.body.nodeIndices[i];var node=_this34.body.nodes[nodeId];// if this node is already used in another cluster this session, we do not have to re-evaluate it.
	if(usedNodes[nodeId]===undefined){relevantEdgeCount=0;edges=[];for(var j=0;j<node.edges.length;j++){edge=node.edges[j];if(_this34.clusteredEdges[edge.id]===undefined){if(edge.toId!==edge.fromId){relevantEdgeCount++;}edges.push(edge);}}// this node qualifies, we collect its neighbours to start the clustering process.
	if(relevantEdgeCount===edgeCount){checkJoinCondition=function checkJoinCondition(node){if(options.joinCondition===undefined||options.joinCondition===null){return true;}var clonedOptions=NetworkUtil.cloneOptions(node);return options.joinCondition(clonedOptions);};var gatheringSuccessful=true;for(var _j4=0;_j4<edges.length;_j4++){edge=edges[_j4];var childNodeId=_this34._getConnectedId(edge,nodeId);// add the nodes to the list by the join condition.
	if(checkJoinCondition(node)){childEdgesObj[edge.id]=edge;childNodesObj[nodeId]=node;childNodesObj[childNodeId]=_this34.body.nodes[childNodeId];usedNodes[nodeId]=true;}else {// this node does not qualify after all.
	gatheringSuccessful=false;break;}}// add to the cluster queue
	if(keys$6(childNodesObj).length>0&&keys$6(childEdgesObj).length>0&&gatheringSuccessful===true){/**
	             * Search for cluster data that contains any of the node id's
	             * @returns {Boolean} true if no joinCondition, otherwise return value of joinCondition
	             */findClusterData=function findClusterData(){for(var n=0;n<clusters.length;++n){// Search for a cluster containing any of the node id's
	for(var m in childNodesObj){if(clusters[n].nodes[m]!==undefined){return clusters[n];}}}return undefined;};// If any of the found nodes is part of a cluster found in this method,
	// add the current values to that cluster
	foundCluster=findClusterData();if(foundCluster!==undefined){// Add nodes to found cluster if not present
	for(var m in childNodesObj){if(foundCluster.nodes[m]===undefined){foundCluster.nodes[m]=childNodesObj[m];}}// Add edges to found cluster, if not present
	for(var _m in childEdgesObj){if(foundCluster.edges[_m]===undefined){foundCluster.edges[_m]=childEdgesObj[_m];}}}else {// Create a new cluster group
	clusters.push({nodes:childNodesObj,edges:childEdgesObj});}}}}};for(var i=0;i<this.body.nodeIndices.length;i++){var checkJoinCondition;var findClusterData;var foundCluster;_loop2(i);}for(var _i12=0;_i12<clusters.length;_i12++){this._cluster(clusters[_i12].nodes,clusters[_i12].edges,options,false);}if(refreshData===true){this.body.emitter.emit('_dataChanged');}}/**
	   * Cluster all nodes in the network that have only 1 edge
	   * @param {Object} options
	   * @param {boolean} [refreshData=true]
	   */},{key:"clusterOutliers",value:function clusterOutliers(options){var refreshData=arguments.length>1&&arguments[1]!==undefined?arguments[1]:true;this.clusterByEdgeCount(1,options,refreshData);}/**
	   * Cluster all nodes in the network that have only 2 edge
	   * @param {Object} options
	   * @param {boolean} [refreshData=true]
	   */},{key:"clusterBridges",value:function clusterBridges(options){var refreshData=arguments.length>1&&arguments[1]!==undefined?arguments[1]:true;this.clusterByEdgeCount(2,options,refreshData);}/**
	  * suck all connected nodes of a node into the node.
	  * @param {Node.id} nodeId
	  * @param {Object} options
	  * @param {boolean} [refreshData=true]
	  */},{key:"clusterByConnection",value:function clusterByConnection(nodeId,options){var _context33;var refreshData=arguments.length>2&&arguments[2]!==undefined?arguments[2]:true;// kill conditions
	if(nodeId===undefined){throw new Error("No nodeId supplied to clusterByConnection!");}if(this.body.nodes[nodeId]===undefined){throw new Error("The nodeId given to clusterByConnection does not exist!");}var node=this.body.nodes[nodeId];options=this._checkOptions(options,node);if(options.clusterNodeProperties.x===undefined){options.clusterNodeProperties.x=node.x;}if(options.clusterNodeProperties.y===undefined){options.clusterNodeProperties.y=node.y;}if(options.clusterNodeProperties.fixed===undefined){options.clusterNodeProperties.fixed={};options.clusterNodeProperties.fixed.x=node.options.fixed.x;options.clusterNodeProperties.fixed.y=node.options.fixed.y;}var childNodesObj={};var childEdgesObj={};var parentNodeId=node.id;var parentClonedOptions=NetworkUtil.cloneOptions(node);childNodesObj[parentNodeId]=node;// collect the nodes that will be in the cluster
	for(var i=0;i<node.edges.length;i++){var edge=node.edges[i];if(this.clusteredEdges[edge.id]===undefined){var childNodeId=this._getConnectedId(edge,parentNodeId);// if the child node is not in a cluster
	if(this.clusteredNodes[childNodeId]===undefined){if(childNodeId!==parentNodeId){if(options.joinCondition===undefined){childEdgesObj[edge.id]=edge;childNodesObj[childNodeId]=this.body.nodes[childNodeId];}else {// clone the options and insert some additional parameters that could be interesting.
	var childClonedOptions=NetworkUtil.cloneOptions(this.body.nodes[childNodeId]);if(options.joinCondition(parentClonedOptions,childClonedOptions)===true){childEdgesObj[edge.id]=edge;childNodesObj[childNodeId]=this.body.nodes[childNodeId];}}}else {// swallow the edge if it is self-referencing.
	childEdgesObj[edge.id]=edge;}}}}var childNodeIDs=map$5(_context33=keys$6(childNodesObj)).call(_context33,function(childNode){return childNodesObj[childNode].id;});for(childNode in childNodesObj){if(!childNodesObj.hasOwnProperty(childNode))continue;var childNode=childNodesObj[childNode];for(var y=0;y<childNode.edges.length;y++){var childEdge=childNode.edges[y];if(indexOf$3(childNodeIDs).call(childNodeIDs,this._getConnectedId(childEdge,childNode.id))>-1){childEdgesObj[childEdge.id]=childEdge;}}}this._cluster(childNodesObj,childEdgesObj,options,refreshData);}/**
	  * This function creates the edges that will be attached to the cluster
	  * It looks for edges that are connected to the nodes from the "outside' of the cluster.
	  *
	  * @param {{Node.id: vis.Node}} childNodesObj
	  * @param {{vis.Edge.id: vis.Edge}} childEdgesObj
	  * @param {Object} clusterNodeProperties
	  * @param {Object} clusterEdgeProperties
	  * @private
	  */},{key:"_createClusterEdges",value:function _createClusterEdges(childNodesObj,childEdgesObj,clusterNodeProperties,clusterEdgeProperties){var edge,childNodeId,childNode,toId,fromId,otherNodeId;// loop over all child nodes and their edges to find edges going out of the cluster
	// these edges will be replaced by clusterEdges.
	var childKeys=keys$6(childNodesObj);var createEdges=[];for(var i=0;i<childKeys.length;i++){childNodeId=childKeys[i];childNode=childNodesObj[childNodeId];// construct new edges from the cluster to others
	for(var j=0;j<childNode.edges.length;j++){edge=childNode.edges[j];// we only handle edges that are visible to the system, not the disabled ones from the clustering process.
	if(this.clusteredEdges[edge.id]===undefined){// self-referencing edges will be added to the "hidden" list
	if(edge.toId==edge.fromId){childEdgesObj[edge.id]=edge;}else {// set up the from and to.
	if(edge.toId==childNodeId){// this is a double equals because ints and strings can be interchanged here.
	toId=clusterNodeProperties.id;fromId=edge.fromId;otherNodeId=fromId;}else {toId=edge.toId;fromId=clusterNodeProperties.id;otherNodeId=toId;}}// Only edges from the cluster outwards are being replaced.
	if(childNodesObj[otherNodeId]===undefined){createEdges.push({edge:edge,fromId:fromId,toId:toId});}}}}//
	// Here we actually create the replacement edges.
	//
	// We could not do this in the loop above as the creation process
	// would add an edge to the edges array we are iterating over.
	//
	// NOTE: a clustered edge can have multiple base edges!
	//
	var newEdges=[];/**
	     * Find a cluster edge which matches the given created edge.
	     * @param {vis.Edge} createdEdge
	     * @returns {vis.Edge}
	     */var getNewEdge=function getNewEdge(createdEdge){for(var _j5=0;_j5<newEdges.length;_j5++){var newEdge=newEdges[_j5];// We replace both to and from edges with a single cluster edge
	var matchToDirection=createdEdge.fromId===newEdge.fromId&&createdEdge.toId===newEdge.toId;var matchFromDirection=createdEdge.fromId===newEdge.toId&&createdEdge.toId===newEdge.fromId;if(matchToDirection||matchFromDirection){return newEdge;}}return null;};for(var _j6=0;_j6<createEdges.length;_j6++){var createdEdge=createEdges[_j6];var _edge=createdEdge.edge;var newEdge=getNewEdge(createdEdge);if(newEdge===null){// Create a clustered edge for this connection
	newEdge=this._createClusteredEdge(createdEdge.fromId,createdEdge.toId,_edge,clusterEdgeProperties);newEdges.push(newEdge);}else {newEdge.clusteringEdgeReplacingIds.push(_edge.id);}// also reference the new edge in the old edge
	this.body.edges[_edge.id].edgeReplacedById=newEdge.id;// hide the replaced edge
	this._backupEdgeOptions(_edge);_edge.setOptions({physics:false});}}/**
	  * This function checks the options that can be supplied to the different cluster functions
	  * for certain fields and inserts defaults if needed
	  * @param {Object} options
	  * @returns {*}
	  * @private
	  */},{key:"_checkOptions",value:function _checkOptions(){var options=arguments.length>0&&arguments[0]!==undefined?arguments[0]:{};if(options.clusterEdgeProperties===undefined){options.clusterEdgeProperties={};}if(options.clusterNodeProperties===undefined){options.clusterNodeProperties={};}return options;}/**
	  *
	  * @param {Object}    childNodesObj         | object with node objects, id as keys, same as childNodes except it also contains a source node
	  * @param {Object}    childEdgesObj         | object with edge objects, id as keys
	  * @param {Array}     options               | object with {clusterNodeProperties, clusterEdgeProperties, processProperties}
	  * @param {boolean}   refreshData | when true, do not wrap up
	  * @private
	  */},{key:"_cluster",value:function _cluster(childNodesObj,childEdgesObj,options){var refreshData=arguments.length>3&&arguments[3]!==undefined?arguments[3]:true;// Remove nodes which are already clustered
	var tmpNodesToRemove=[];for(var nodeId in childNodesObj){if(childNodesObj.hasOwnProperty(nodeId)){if(this.clusteredNodes[nodeId]!==undefined){tmpNodesToRemove.push(nodeId);}}}for(var n=0;n<tmpNodesToRemove.length;++n){delete childNodesObj[tmpNodesToRemove[n]];}// kill condition: no nodes don't bother
	if(keys$6(childNodesObj).length==0){return;}// allow clusters of 1 if options allow
	if(keys$6(childNodesObj).length==1&&options.clusterNodeProperties.allowSingleNodeCluster!=true){return;}var clusterNodeProperties=deepExtend({},options.clusterNodeProperties);// construct the clusterNodeProperties
	if(options.processProperties!==undefined){// get the childNode options
	var childNodesOptions=[];for(var _nodeId8 in childNodesObj){if(childNodesObj.hasOwnProperty(_nodeId8)){var clonedOptions=NetworkUtil.cloneOptions(childNodesObj[_nodeId8]);childNodesOptions.push(clonedOptions);}}// get cluster properties based on childNodes
	var childEdgesOptions=[];for(var edgeId in childEdgesObj){if(childEdgesObj.hasOwnProperty(edgeId)){// these cluster edges will be removed on creation of the cluster.
	if(edgeId.substr(0,12)!=="clusterEdge:"){var _clonedOptions=NetworkUtil.cloneOptions(childEdgesObj[edgeId],'edge');childEdgesOptions.push(_clonedOptions);}}}clusterNodeProperties=options.processProperties(clusterNodeProperties,childNodesOptions,childEdgesOptions);if(!clusterNodeProperties){throw new Error("The processProperties function does not return properties!");}}// check if we have an unique id;
	if(clusterNodeProperties.id===undefined){clusterNodeProperties.id='cluster:'+v4();}var clusterId=clusterNodeProperties.id;if(clusterNodeProperties.label===undefined){clusterNodeProperties.label='cluster';}// give the clusterNode a position if it does not have one.
	var pos=undefined;if(clusterNodeProperties.x===undefined){pos=this._getClusterPosition(childNodesObj);clusterNodeProperties.x=pos.x;}if(clusterNodeProperties.y===undefined){if(pos===undefined){pos=this._getClusterPosition(childNodesObj);}clusterNodeProperties.y=pos.y;}// force the ID to remain the same
	clusterNodeProperties.id=clusterId;// create the cluster Node
	// Note that allowSingleNodeCluster, if present, is stored in the options as well
	var clusterNode=this.body.functions.createNode(clusterNodeProperties,Cluster);clusterNode.containedNodes=childNodesObj;clusterNode.containedEdges=childEdgesObj;// cache a copy from the cluster edge properties if we have to reconnect others later on
	clusterNode.clusterEdgeProperties=options.clusterEdgeProperties;// finally put the cluster node into global
	this.body.nodes[clusterNodeProperties.id]=clusterNode;this._clusterEdges(childNodesObj,childEdgesObj,clusterNodeProperties,options.clusterEdgeProperties);// set ID to undefined so no duplicates arise
	clusterNodeProperties.id=undefined;// wrap up
	if(refreshData===true){this.body.emitter.emit('_dataChanged');}}/**
	   *
	   * @param {Edge} edge
	   * @private
	   */},{key:"_backupEdgeOptions",value:function _backupEdgeOptions(edge){if(this.clusteredEdges[edge.id]===undefined){this.clusteredEdges[edge.id]={physics:edge.options.physics};}}/**
	   *
	   * @param {Edge} edge
	   * @private
	   */},{key:"_restoreEdge",value:function _restoreEdge(edge){var originalOptions=this.clusteredEdges[edge.id];if(originalOptions!==undefined){edge.setOptions({physics:originalOptions.physics});delete this.clusteredEdges[edge.id];}}/**
	  * Check if a node is a cluster.
	  * @param {Node.id} nodeId
	  * @returns {*}
	  */},{key:"isCluster",value:function isCluster(nodeId){if(this.body.nodes[nodeId]!==undefined){return this.body.nodes[nodeId].isCluster===true;}else {console.log("Node does not exist.");return false;}}/**
	  * get the position of the cluster node based on what's inside
	  * @param {object} childNodesObj    | object with node objects, id as keys
	  * @returns {{x: number, y: number}}
	  * @private
	  */},{key:"_getClusterPosition",value:function _getClusterPosition(childNodesObj){var childKeys=keys$6(childNodesObj);var minX=childNodesObj[childKeys[0]].x;var maxX=childNodesObj[childKeys[0]].x;var minY=childNodesObj[childKeys[0]].y;var maxY=childNodesObj[childKeys[0]].y;var node;for(var i=1;i<childKeys.length;i++){node=childNodesObj[childKeys[i]];minX=node.x<minX?node.x:minX;maxX=node.x>maxX?node.x:maxX;minY=node.y<minY?node.y:minY;maxY=node.y>maxY?node.y:maxY;}return {x:0.5*(minX+maxX),y:0.5*(minY+maxY)};}/**
	   * Open a cluster by calling this function.
	   * @param {vis.Edge.id}  clusterNodeId | the ID of the cluster node
	   * @param {Object} options
	   * @param {boolean} refreshData | wrap up afterwards if not true
	   */},{key:"openCluster",value:function openCluster(clusterNodeId,options){var refreshData=arguments.length>2&&arguments[2]!==undefined?arguments[2]:true;// kill conditions
	if(clusterNodeId===undefined){throw new Error("No clusterNodeId supplied to openCluster.");}var clusterNode=this.body.nodes[clusterNodeId];if(clusterNode===undefined){throw new Error("The clusterNodeId supplied to openCluster does not exist.");}if(clusterNode.isCluster!==true||clusterNode.containedNodes===undefined||clusterNode.containedEdges===undefined){throw new Error("The node:"+clusterNodeId+" is not a valid cluster.");}// Check if current cluster is clustered itself
	var stack=this.findNode(clusterNodeId);var parentIndex=indexOf$3(stack).call(stack,clusterNodeId)-1;if(parentIndex>=0){// Current cluster is clustered; transfer contained nodes and edges to parent
	var parentClusterNodeId=stack[parentIndex];var parentClusterNode=this.body.nodes[parentClusterNodeId];// clustering.clusteredNodes and clustering.clusteredEdges remain unchanged
	parentClusterNode._openChildCluster(clusterNodeId);// All components of child cluster node have been transferred. It can die now.
	delete this.body.nodes[clusterNodeId];if(refreshData===true){this.body.emitter.emit('_dataChanged');}return;}// main body 
	var containedNodes=clusterNode.containedNodes;var containedEdges=clusterNode.containedEdges;// allow the user to position the nodes after release.
	if(options!==undefined&&options.releaseFunction!==undefined&&typeof options.releaseFunction==='function'){var positions={};var clusterPosition={x:clusterNode.x,y:clusterNode.y};for(var nodeId in containedNodes){if(containedNodes.hasOwnProperty(nodeId)){var containedNode=this.body.nodes[nodeId];positions[nodeId]={x:containedNode.x,y:containedNode.y};}}var newPositions=options.releaseFunction(clusterPosition,positions);for(var _nodeId9 in containedNodes){if(containedNodes.hasOwnProperty(_nodeId9)){var _containedNode=this.body.nodes[_nodeId9];if(newPositions[_nodeId9]!==undefined){_containedNode.x=newPositions[_nodeId9].x===undefined?clusterNode.x:newPositions[_nodeId9].x;_containedNode.y=newPositions[_nodeId9].y===undefined?clusterNode.y:newPositions[_nodeId9].y;}}}}else {// copy the position from the cluster
	forEach$4(containedNodes,function(containedNode){// inherit position
	if(containedNode.options.fixed.x===false){containedNode.x=clusterNode.x;}if(containedNode.options.fixed.y===false){containedNode.y=clusterNode.y;}});}// release nodes
	for(var _nodeId10 in containedNodes){if(containedNodes.hasOwnProperty(_nodeId10)){var _containedNode2=this.body.nodes[_nodeId10];// inherit speed
	_containedNode2.vx=clusterNode.vx;_containedNode2.vy=clusterNode.vy;_containedNode2.setOptions({physics:true});delete this.clusteredNodes[_nodeId10];}}// copy the clusterNode edges because we cannot iterate over an object that we add or remove from.
	var edgesToBeDeleted=[];for(var i=0;i<clusterNode.edges.length;i++){edgesToBeDeleted.push(clusterNode.edges[i]);}// actually handling the deleting.
	for(var _i13=0;_i13<edgesToBeDeleted.length;_i13++){var edge=edgesToBeDeleted[_i13];var otherNodeId=this._getConnectedId(edge,clusterNodeId);var otherNode=this.clusteredNodes[otherNodeId];for(var j=0;j<edge.clusteringEdgeReplacingIds.length;j++){var transferId=edge.clusteringEdgeReplacingIds[j];var transferEdge=this.body.edges[transferId];if(transferEdge===undefined)continue;// if the other node is in another cluster, we transfer ownership of this edge to the other cluster
	if(otherNode!==undefined){// transfer ownership:
	var otherCluster=this.body.nodes[otherNode.clusterId];otherCluster.containedEdges[transferEdge.id]=transferEdge;// delete local reference
	delete containedEdges[transferEdge.id];// get to and from
	var fromId=transferEdge.fromId;var toId=transferEdge.toId;if(transferEdge.toId==otherNodeId){toId=otherNode.clusterId;}else {fromId=otherNode.clusterId;}// create new cluster edge from the otherCluster
	this._createClusteredEdge(fromId,toId,transferEdge,otherCluster.clusterEdgeProperties,{hidden:false,physics:true});}else {this._restoreEdge(transferEdge);}}edge.remove();}// handle the releasing of the edges
	for(var edgeId in containedEdges){if(containedEdges.hasOwnProperty(edgeId)){this._restoreEdge(containedEdges[edgeId]);}}// remove clusterNode
	delete this.body.nodes[clusterNodeId];if(refreshData===true){this.body.emitter.emit('_dataChanged');}}/**
	   *
	   * @param {Cluster.id} clusterId
	   * @returns {Array.<Node.id>}
	   */},{key:"getNodesInCluster",value:function getNodesInCluster(clusterId){var nodesArray=[];if(this.isCluster(clusterId)===true){var containedNodes=this.body.nodes[clusterId].containedNodes;for(var nodeId in containedNodes){if(containedNodes.hasOwnProperty(nodeId)){nodesArray.push(this.body.nodes[nodeId].id);}}}return nodesArray;}/**
	  * Get the stack clusterId's that a certain node resides in. cluster A -> cluster B -> cluster C -> node
	  *
	  * If a node can't be found in the chain, return an empty array.
	  *
	  * @param {string|number} nodeId
	  * @returns {Array}
	  */},{key:"findNode",value:function findNode(nodeId){var stack=[];var max=100;var counter=0;var node;while(this.clusteredNodes[nodeId]!==undefined&&counter<max){node=this.body.nodes[nodeId];if(node===undefined)return [];stack.push(node.id);nodeId=this.clusteredNodes[nodeId].clusterId;counter++;}node=this.body.nodes[nodeId];if(node===undefined)return [];stack.push(node.id);reverse$2(stack).call(stack);return stack;}/**
	  * Using a clustered nodeId, update with the new options
	  * @param {Node.id} clusteredNodeId
	  * @param {object} newOptions
	  */},{key:"updateClusteredNode",value:function updateClusteredNode(clusteredNodeId,newOptions){if(clusteredNodeId===undefined){throw new Error("No clusteredNodeId supplied to updateClusteredNode.");}if(newOptions===undefined){throw new Error("No newOptions supplied to updateClusteredNode.");}if(this.body.nodes[clusteredNodeId]===undefined){throw new Error("The clusteredNodeId supplied to updateClusteredNode does not exist.");}this.body.nodes[clusteredNodeId].setOptions(newOptions);this.body.emitter.emit('_dataChanged');}/**
	  * Using a base edgeId, update all related clustered edges with the new options
	  * @param {vis.Edge.id} startEdgeId
	  * @param {object} newOptions
	  */},{key:"updateEdge",value:function updateEdge(startEdgeId,newOptions){if(startEdgeId===undefined){throw new Error("No startEdgeId supplied to updateEdge.");}if(newOptions===undefined){throw new Error("No newOptions supplied to updateEdge.");}if(this.body.edges[startEdgeId]===undefined){throw new Error("The startEdgeId supplied to updateEdge does not exist.");}var allEdgeIds=this.getClusteredEdges(startEdgeId);for(var i=0;i<allEdgeIds.length;i++){var edge=this.body.edges[allEdgeIds[i]];edge.setOptions(newOptions);}this.body.emitter.emit('_dataChanged');}/**
	  * Get a stack of clusterEdgeId's (+base edgeid) that a base edge is the same as. cluster edge C -> cluster edge B -> cluster edge A -> base edge(edgeId)
	  * @param {vis.Edge.id} edgeId
	  * @returns {Array.<vis.Edge.id>}
	  */},{key:"getClusteredEdges",value:function getClusteredEdges(edgeId){var stack=[];var max=100;var counter=0;while(edgeId!==undefined&&this.body.edges[edgeId]!==undefined&&counter<max){stack.push(this.body.edges[edgeId].id);edgeId=this.body.edges[edgeId].edgeReplacedById;counter++;}reverse$2(stack).call(stack);return stack;}/**
	  * Get the base edge id of clusterEdgeId. cluster edge (clusteredEdgeId) -> cluster edge B -> cluster edge C -> base edge
	  * @param {vis.Edge.id} clusteredEdgeId
	  * @returns {vis.Edge.id} baseEdgeId
	  *
	  * TODO: deprecate in 5.0.0. Method getBaseEdges() is the correct one to use.
	  */},{key:"getBaseEdge",value:function getBaseEdge(clusteredEdgeId){// Just kludge this by returning the first base edge id found
	return this.getBaseEdges(clusteredEdgeId)[0];}/**
	   * Get all regular edges for this clustered edge id.
	   *
	   * @param {vis.Edge.id} clusteredEdgeId
	   * @returns {Array.<vis.Edge.id>} all baseEdgeId's under this clustered edge
	   */},{key:"getBaseEdges",value:function getBaseEdges(clusteredEdgeId){var IdsToHandle=[clusteredEdgeId];var doneIds=[];var foundIds=[];var max=100;var counter=0;while(IdsToHandle.length>0&&counter<max){var nextId=IdsToHandle.pop();if(nextId===undefined)continue;// Paranoia here and onwards
	var nextEdge=this.body.edges[nextId];if(nextEdge===undefined)continue;counter++;var replacingIds=nextEdge.clusteringEdgeReplacingIds;if(replacingIds===undefined){// nextId is a base id
	foundIds.push(nextId);}else {// Another cluster edge, unravel this one as well
	for(var i=0;i<replacingIds.length;++i){var replacingId=replacingIds[i];// Don't add if already handled
	// TODO: never triggers; find a test-case which does
	if(indexOf$3(IdsToHandle).call(IdsToHandle,replacingIds)!==-1||indexOf$3(doneIds).call(doneIds,replacingIds)!==-1){continue;}IdsToHandle.push(replacingId);}}doneIds.push(nextId);}return foundIds;}/**
	  * Get the Id the node is connected to
	  * @param {vis.Edge} edge
	  * @param {Node.id} nodeId
	  * @returns {*}
	  * @private
	  */},{key:"_getConnectedId",value:function _getConnectedId(edge,nodeId){if(edge.toId!=nodeId){return edge.toId;}else if(edge.fromId!=nodeId){return edge.fromId;}else {return edge.fromId;}}/**
	  * We determine how many connections denote an important hub.
	  * We take the mean + 2*std as the important hub size. (Assuming a normal distribution of data, ~2.2%)
	  *
	  * @returns {number}
	  * @private
	  */},{key:"_getHubSize",value:function _getHubSize(){var average=0;var averageSquared=0;var hubCounter=0;var largestHub=0;for(var i=0;i<this.body.nodeIndices.length;i++){var node=this.body.nodes[this.body.nodeIndices[i]];if(node.edges.length>largestHub){largestHub=node.edges.length;}average+=node.edges.length;averageSquared+=Math.pow(node.edges.length,2);hubCounter+=1;}average=average/hubCounter;averageSquared=averageSquared/hubCounter;var variance=averageSquared-Math.pow(average,2);var standardDeviation=Math.sqrt(variance);var hubThreshold=Math.floor(average+2*standardDeviation);// always have at least one to cluster
	if(hubThreshold>largestHub){hubThreshold=largestHub;}return hubThreshold;}/**
	   * Create an edge for the cluster representation.
	   *
	   * @param {Node.id} fromId
	   * @param {Node.id} toId
	   * @param {vis.Edge} baseEdge
	   * @param {Object} clusterEdgeProperties
	   * @param {Object} extraOptions
	   * @returns {Edge} newly created clustered edge
	   * @private
	   */},{key:"_createClusteredEdge",value:function _createClusteredEdge(fromId,toId,baseEdge,clusterEdgeProperties,extraOptions){// copy the options of the edge we will replace
	var clonedOptions=NetworkUtil.cloneOptions(baseEdge,'edge');// make sure the properties of clusterEdges are superimposed on it
	deepExtend(clonedOptions,clusterEdgeProperties);// set up the edge
	clonedOptions.from=fromId;clonedOptions.to=toId;clonedOptions.id='clusterEdge:'+v4();// apply the edge specific options to it if specified
	if(extraOptions!==undefined){deepExtend(clonedOptions,extraOptions);}var newEdge=this.body.functions.createEdge(clonedOptions);newEdge.clusteringEdgeReplacingIds=[baseEdge.id];newEdge.connect();// Register the new edge
	this.body.edges[newEdge.id]=newEdge;return newEdge;}/**
	   * Add the passed child nodes and edges to the given cluster node.
	   *
	   * @param {Object|Node} childNodes  hash of nodes or single node to add in cluster
	   * @param {Object|Edge} childEdges  hash of edges or single edge to take into account when clustering
	   * @param {Node} clusterNode  cluster node to add nodes and edges to
	   * @param {Object} [clusterEdgeProperties]
	   * @private
	   */},{key:"_clusterEdges",value:function _clusterEdges(childNodes,childEdges,clusterNode,clusterEdgeProperties){if(childEdges instanceof Edge){var edge=childEdges;var obj={};obj[edge.id]=edge;childEdges=obj;}if(childNodes instanceof Node){var node=childNodes;var _obj={};_obj[node.id]=node;childNodes=_obj;}if(clusterNode===undefined||clusterNode===null){throw new Error("_clusterEdges: parameter clusterNode required");}if(clusterEdgeProperties===undefined){// Take the required properties from the cluster node
	clusterEdgeProperties=clusterNode.clusterEdgeProperties;}// create the new edges that will connect to the cluster.
	// All self-referencing edges will be added to childEdges here.
	this._createClusterEdges(childNodes,childEdges,clusterNode,clusterEdgeProperties);// disable the childEdges
	for(var edgeId in childEdges){if(childEdges.hasOwnProperty(edgeId)){if(this.body.edges[edgeId]!==undefined){var _edge2=this.body.edges[edgeId];// cache the options before changing
	this._backupEdgeOptions(_edge2);// disable physics and hide the edge
	_edge2.setOptions({physics:false});}}}// disable the childNodes
	for(var nodeId in childNodes){if(childNodes.hasOwnProperty(nodeId)){this.clusteredNodes[nodeId]={clusterId:clusterNode.id,node:this.body.nodes[nodeId]};this.body.nodes[nodeId].setOptions({physics:false});}}}/**
	   * Determine in which cluster given nodeId resides.
	   *
	   * If not in cluster, return undefined.
	   *
	   * NOTE: If you know a cleaner way to do this, please enlighten me (wimrijnders).
	   *
	   * @param {Node.id} nodeId
	   * @returns {Node|undefined} Node instance for cluster, if present
	   * @private
	   */},{key:"_getClusterNodeForNode",value:function _getClusterNodeForNode(nodeId){if(nodeId===undefined)return undefined;var clusteredNode=this.clusteredNodes[nodeId];// NOTE: If no cluster info found, it should actually be an error
	if(clusteredNode===undefined)return undefined;var clusterId=clusteredNode.clusterId;if(clusterId===undefined)return undefined;return this.body.nodes[clusterId];}/**
	   * Internal helper function for conditionally removing items in array
	   *
	   * Done like this because Array.filter() is not fully supported by all IE's.
	   *
	   * @param {Array} arr
	   * @param {function} callback
	   * @returns {Array}
	   * @private
	   */},{key:"_filter",value:function _filter(arr,callback){var ret=[];forEach$4(arr,function(item){if(callback(item)){ret.push(item);}});return ret;}/**
	   * Scan all edges for changes in clustering and adjust this if necessary.
	   *
	   * Call this (internally) after there has been a change in node or edge data.
	   *
	   * Pre: States of this.body.nodes and this.body.edges consistent
	   * Pre: this.clusteredNodes and this.clusteredEdge consistent with containedNodes and containedEdges
	   *      of cluster nodes.
	   */},{key:"_updateState",value:function _updateState(){var _this35=this;var nodeId;var deletedNodeIds=[];var deletedEdgeIds={};/**
	     * Utility function to iterate over clustering nodes only
	     *
	     * @param {Function} callback  function to call for each cluster node
	     */var eachClusterNode=function eachClusterNode(callback){forEach$4(_this35.body.nodes,function(node){if(node.isCluster===true){callback(node);}});};//
	// Remove deleted regular nodes from clustering
	//
	// Determine the deleted nodes
	for(nodeId in this.clusteredNodes){if(!this.clusteredNodes.hasOwnProperty(nodeId))continue;var node=this.body.nodes[nodeId];if(node===undefined){deletedNodeIds.push(nodeId);}}// Remove nodes from cluster nodes
	eachClusterNode(function(clusterNode){for(var n=0;n<deletedNodeIds.length;n++){delete clusterNode.containedNodes[deletedNodeIds[n]];}});// Remove nodes from cluster list
	for(var n=0;n<deletedNodeIds.length;n++){delete this.clusteredNodes[deletedNodeIds[n]];}//
	// Remove deleted edges from clustering
	//
	// Add the deleted clustered edges to the list
	forEach$4(this.clusteredEdges,function(edgeId){var edge=_this35.body.edges[edgeId];if(edge===undefined||!edge.endPointsValid()){deletedEdgeIds[edgeId]=edgeId;}});// Cluster nodes can also contain edges which are not clustered, 
	// i.e. nodes 1-2 within cluster with an edge in between.
	// So the cluster nodes also need to be scanned for invalid edges
	eachClusterNode(function(clusterNode){forEach$4(clusterNode.containedEdges,function(edge,edgeId){if(!edge.endPointsValid()&&!deletedEdgeIds[edgeId]){deletedEdgeIds[edgeId]=edgeId;}});});// Also scan for cluster edges which need to be removed in the active list.
	// Regular edges have been removed beforehand, so this only picks up the cluster edges.
	forEach$4(this.body.edges,function(edge,edgeId){// Explicitly scan the contained edges for validity
	var isValid=true;var replacedIds=edge.clusteringEdgeReplacingIds;if(replacedIds!==undefined){var numValid=0;forEach$4(replacedIds,function(containedEdgeId){var containedEdge=_this35.body.edges[containedEdgeId];if(containedEdge!==undefined&&containedEdge.endPointsValid()){numValid+=1;}});isValid=numValid>0;}if(!edge.endPointsValid()||!isValid){deletedEdgeIds[edgeId]=edgeId;}});// Remove edges from cluster nodes
	eachClusterNode(function(clusterNode){forEach$4(deletedEdgeIds,function(deletedEdgeId){delete clusterNode.containedEdges[deletedEdgeId];forEach$4(clusterNode.edges,function(edge,m){if(edge.id===deletedEdgeId){clusterNode.edges[m]=null;// Don't want to directly delete here, because in the loop
	return;}edge.clusteringEdgeReplacingIds=_this35._filter(edge.clusteringEdgeReplacingIds,function(id){return !deletedEdgeIds[id];});});// Clean up the nulls
	clusterNode.edges=_this35._filter(clusterNode.edges,function(item){return item!==null;});});});// Remove from cluster list
	forEach$4(deletedEdgeIds,function(edgeId){delete _this35.clusteredEdges[edgeId];});// Remove cluster edges from active list (this.body.edges).
	// deletedEdgeIds still contains id of regular edges, but these should all
	// be gone when you reach here.
	forEach$4(deletedEdgeIds,function(edgeId){delete _this35.body.edges[edgeId];});//
	// Check changed cluster state of edges
	//
	// Iterating over keys here, because edges may be removed in the loop
	var ids=keys$6(this.body.edges);forEach$4(ids,function(edgeId){var edge=_this35.body.edges[edgeId];var shouldBeClustered=_this35._isClusteredNode(edge.fromId)||_this35._isClusteredNode(edge.toId);if(shouldBeClustered===_this35._isClusteredEdge(edge.id)){return;// all is well
	}if(shouldBeClustered){// add edge to clustering
	var clusterFrom=_this35._getClusterNodeForNode(edge.fromId);if(clusterFrom!==undefined){_this35._clusterEdges(_this35.body.nodes[edge.fromId],edge,clusterFrom);}var clusterTo=_this35._getClusterNodeForNode(edge.toId);if(clusterTo!==undefined){_this35._clusterEdges(_this35.body.nodes[edge.toId],edge,clusterTo);}// TODO: check that it works for both edges clustered
	//       (This might be paranoia)
	}else {delete _this35._clusterEdges[edgeId];_this35._restoreEdge(edge);// This should not be happening, the state should
	// be properly updated at this point.
	// 
	// If it *is* reached during normal operation, then we have to implement
	// undo clustering for this edge here.
	// throw new Error('remove edge from clustering not implemented!')
	}});// Clusters may be nested to any level. Keep on opening until nothing to open
	var changed=false;var continueLoop=true;var _loop3=function _loop3(){var clustersToOpen=[];// Determine the id's of clusters that need opening
	eachClusterNode(function(clusterNode){var numNodes=keys$6(clusterNode.containedNodes).length;var allowSingle=clusterNode.options.allowSingleNodeCluster===true;if(allowSingle&&numNodes<1||!allowSingle&&numNodes<2){clustersToOpen.push(clusterNode.id);}});// Open them
	for(var _n2=0;_n2<clustersToOpen.length;++_n2){_this35.openCluster(clustersToOpen[_n2],{},false/* Don't refresh, we're in an refresh/update already */);}continueLoop=clustersToOpen.length>0;changed=changed||continueLoop;};while(continueLoop){_loop3();}if(changed){this._updateState();// Redo this method (recursion possible! should be safe)
	}}/**
	  * Determine if node with given id is part of a cluster.
	  *
	  * @param {Node.id} nodeId
	  * @return {boolean} true if part of a cluster.
	  */},{key:"_isClusteredNode",value:function _isClusteredNode(nodeId){return this.clusteredNodes[nodeId]!==undefined;}/**
	  * Determine if edge with given id is not visible due to clustering.
	  *
	  * An edge is considered clustered if:
	  * - it is directly replaced by a clustering edge
	  * - any of its connecting nodes is in a cluster
	  *
	  * @param {vis.Edge.id} edgeId
	  * @return {boolean} true if part of a cluster.
	  */},{key:"_isClusteredEdge",value:function _isClusteredEdge(edgeId){return this.clusteredEdges[edgeId]!==undefined;}}]);return ClusterEngine;}();/**
	 * Initializes window.requestAnimationFrame() to a usable form.
	 *
	 * Specifically, set up this method for the case of running on node.js with jsdom enabled.
	 *
	 * NOTES:
	 *
	 * * On node.js, when calling this directly outside of this class, `window` is not defined.
	 *   This happens even if jsdom is used.
	 * * For node.js + jsdom, `window` is available at the moment the constructor is called.
	 *   For this reason, the called is placed within the constructor.
	 * * Even then, `window.requestAnimationFrame()` is not defined, so it still needs to be added.
	 * * During unit testing, it happens that the window object is reset during execution, causing
	 *   a runtime error due to missing `requestAnimationFrame()`. This needs to be compensated for,
	 *   see `_requestNextFrame()`.
	 * * Since this is a global object, it may affect other modules besides `Network`. With normal
	 *   usage, this does not cause any problems. During unit testing, errors may occur. These have
	 *   been compensated for, see comment block in _requestNextFrame().
	 *
	 * @private
	 */function _initRequestAnimationFrame(){var func;if(window!==undefined){func=window.requestAnimationFrame||window.mozRequestAnimationFrame||window.webkitRequestAnimationFrame||window.msRequestAnimationFrame;}if(func===undefined){// window or method not present, setting mock requestAnimationFrame
	window.requestAnimationFrame=function(callback){//console.log("Called mock requestAnimationFrame");
	callback();};}else {window.requestAnimationFrame=func;}}/**
	 * The canvas renderer
	 */var CanvasRenderer=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   * @param {Canvas} canvas
	   */function CanvasRenderer(body,canvas){classCallCheck(this,CanvasRenderer);_initRequestAnimationFrame();this.body=body;this.canvas=canvas;this.redrawRequested=false;this.renderTimer=undefined;this.requiresTimeout=true;this.renderingActive=false;this.renderRequests=0;this.allowRedraw=true;this.dragging=false;this.zooming=false;this.options={};this.defaultOptions={hideEdgesOnDrag:false,hideEdgesOnZoom:false,hideNodesOnDrag:false};extend(this.options,this.defaultOptions);this._determineBrowserMethod();this.bindEventListeners();}/**
	   * Binds event listeners
	   */createClass(CanvasRenderer,[{key:"bindEventListeners",value:function bindEventListeners(){var _this36=this,_context35;this.body.emitter.on("dragStart",function(){_this36.dragging=true;});this.body.emitter.on("dragEnd",function(){_this36.dragging=false;});this.body.emitter.on("zoom",function(){_this36.zooming=true;window.clearTimeout(_this36.zoomTimeoutId);_this36.zoomTimeoutId=window.setTimeout(function(){var _context34;_this36.zooming=false;bind$2(_context34=_this36._requestRedraw).call(_context34,_this36)();},250);});this.body.emitter.on("_resizeNodes",function(){_this36._resizeNodes();});this.body.emitter.on("_redraw",function(){if(_this36.renderingActive===false){_this36._redraw();}});this.body.emitter.on("_blockRedraw",function(){_this36.allowRedraw=false;});this.body.emitter.on("_allowRedraw",function(){_this36.allowRedraw=true;_this36.redrawRequested=false;});this.body.emitter.on("_requestRedraw",bind$2(_context35=this._requestRedraw).call(_context35,this));this.body.emitter.on("_startRendering",function(){_this36.renderRequests+=1;_this36.renderingActive=true;_this36._startRendering();});this.body.emitter.on("_stopRendering",function(){_this36.renderRequests-=1;_this36.renderingActive=_this36.renderRequests>0;_this36.renderTimer=undefined;});this.body.emitter.on('destroy',function(){_this36.renderRequests=0;_this36.allowRedraw=false;_this36.renderingActive=false;if(_this36.requiresTimeout===true){clearTimeout(_this36.renderTimer);}else {window.cancelAnimationFrame(_this36.renderTimer);}_this36.body.emitter.off();});}/**
	   *
	   * @param {Object} options
	   */},{key:"setOptions",value:function setOptions(options){if(options!==undefined){var fields=['hideEdgesOnDrag','hideEdgesOnZoom','hideNodesOnDrag'];selectiveDeepExtend(fields,this.options,options);}}/**
	   * Prepare the drawing of the next frame.
	   *
	   * Calls the callback when the next frame can or will be drawn.
	   *
	   * @param {function} callback
	   * @param {number} delay - timeout case only, wait this number of milliseconds
	   * @returns {function|undefined}
	   * @private
	   */},{key:"_requestNextFrame",value:function _requestNextFrame(callback,delay){// During unit testing, it happens that the mock window object is reset while
	// the next frame is still pending. Then, either 'window' is not present, or
	// 'requestAnimationFrame()' is not present because it is not defined on the
	// mock window object.
	//
	// As a consequence, unrelated unit tests may appear to fail, even if the problem
	// described happens in the current unit test.
	//
	// This is not something that will happen in normal operation, but we still need
	// to take it into account.
	//
	if(typeof window==='undefined')return;// Doing `if (window === undefined)` does not work here!
	var timer;var myWindow=window;// Grab a reference to reduce the possibility that 'window' is reset
	// while running this method.
	if(this.requiresTimeout===true){// wait given number of milliseconds and perform the animation step function
	timer=myWindow.setTimeout(callback,delay);}else {if(myWindow.requestAnimationFrame){timer=myWindow.requestAnimationFrame(callback);}}return timer;}/**
	   *
	   * @private
	   */},{key:"_startRendering",value:function _startRendering(){if(this.renderingActive===true){if(this.renderTimer===undefined){var _context36;this.renderTimer=this._requestNextFrame(bind$2(_context36=this._renderStep).call(_context36,this),this.simulationInterval);}}}/**
	   *
	   * @private
	   */},{key:"_renderStep",value:function _renderStep(){if(this.renderingActive===true){// reset the renderTimer so a new scheduled animation step can be set
	this.renderTimer=undefined;if(this.requiresTimeout===true){// this schedules a new simulation step
	this._startRendering();}this._redraw();if(this.requiresTimeout===false){// this schedules a new simulation step
	this._startRendering();}}}/**
	   * Redraw the network with the current data
	   * chart will be resized too.
	   */},{key:"redraw",value:function redraw(){this.body.emitter.emit('setSize');this._redraw();}/**
	   * Redraw the network with the current data
	   * @private
	   */},{key:"_requestRedraw",value:function _requestRedraw(){var _this37=this;if(this.redrawRequested!==true&&this.renderingActive===false&&this.allowRedraw===true){this.redrawRequested=true;this._requestNextFrame(function(){_this37._redraw(false);},0);}}/**
	   * Redraw the network with the current data
	   * @param {boolean} [hidden=false] | Used to get the first estimate of the node sizes.
	   *                                   Only the nodes are drawn after which they are quickly drawn over.
	   * @private
	   */},{key:"_redraw",value:function _redraw(){var hidden=arguments.length>0&&arguments[0]!==undefined?arguments[0]:false;if(this.allowRedraw===true){this.body.emitter.emit("initRedraw");this.redrawRequested=false;// when the container div was hidden, this fixes it back up!
	if(this.canvas.frame.canvas.width===0||this.canvas.frame.canvas.height===0){this.canvas.setSize();}this.canvas.setTransform();var ctx=this.canvas.getContext();// clear the canvas
	var w=this.canvas.frame.canvas.clientWidth;var h=this.canvas.frame.canvas.clientHeight;ctx.clearRect(0,0,w,h);// if the div is hidden, we stop the redraw here for performance.
	if(this.canvas.frame.clientWidth===0){return;}// set scaling and translation
	ctx.save();ctx.translate(this.body.view.translation.x,this.body.view.translation.y);ctx.scale(this.body.view.scale,this.body.view.scale);ctx.beginPath();this.body.emitter.emit("beforeDrawing",ctx);ctx.closePath();if(hidden===false){if((this.dragging===false||this.dragging===true&&this.options.hideEdgesOnDrag===false)&&(this.zooming===false||this.zooming===true&&this.options.hideEdgesOnZoom===false)){this._drawEdges(ctx);}}if(this.dragging===false||this.dragging===true&&this.options.hideNodesOnDrag===false){this._drawNodes(ctx,hidden);}ctx.beginPath();this.body.emitter.emit("afterDrawing",ctx);ctx.closePath();// restore original scaling and translation
	ctx.restore();if(hidden===true){ctx.clearRect(0,0,w,h);}}}/**
	   * Redraw all nodes
	   *
	   * @param {CanvasRenderingContext2D}   ctx
	   * @param {boolean} [alwaysShow]
	   * @private
	   */},{key:"_resizeNodes",value:function _resizeNodes(){this.canvas.setTransform();var ctx=this.canvas.getContext();ctx.save();ctx.translate(this.body.view.translation.x,this.body.view.translation.y);ctx.scale(this.body.view.scale,this.body.view.scale);var nodes=this.body.nodes;var node;// resize all nodes
	for(var nodeId in nodes){if(nodes.hasOwnProperty(nodeId)){node=nodes[nodeId];node.resize(ctx);node.updateBoundingBox(ctx,node.selected);}}// restore original scaling and translation
	ctx.restore();}/**
	   * Redraw all nodes
	   *
	   * @param {CanvasRenderingContext2D} ctx  2D context of a HTML canvas
	   * @param {boolean} [alwaysShow]
	   * @private
	   */},{key:"_drawNodes",value:function _drawNodes(ctx){var alwaysShow=arguments.length>1&&arguments[1]!==undefined?arguments[1]:false;var nodes=this.body.nodes;var nodeIndices=this.body.nodeIndices;var node;var selected=[];var hovered=[];var margin=20;var topLeft=this.canvas.DOMtoCanvas({x:-margin,y:-margin});var bottomRight=this.canvas.DOMtoCanvas({x:this.canvas.frame.canvas.clientWidth+margin,y:this.canvas.frame.canvas.clientHeight+margin});var viewableArea={top:topLeft.y,left:topLeft.x,bottom:bottomRight.y,right:bottomRight.x};// draw unselected nodes;
	for(var _i14=0;_i14<nodeIndices.length;_i14++){node=nodes[nodeIndices[_i14]];// set selected and hovered nodes aside
	if(node.hover){hovered.push(nodeIndices[_i14]);}else if(node.isSelected()){selected.push(nodeIndices[_i14]);}else {if(alwaysShow===true){node.draw(ctx);}else if(node.isBoundingBoxOverlappingWith(viewableArea)===true){node.draw(ctx);}else {node.updateBoundingBox(ctx,node.selected);}}}var i;var selectedLength=selected.length;var hoveredLength=hovered.length;// draw the selected nodes on top
	for(i=0;i<selectedLength;i++){node=nodes[selected[i]];node.draw(ctx);}// draw hovered nodes above everything else: fixes https://github.com/visjs/vis-network/issues/226
	for(i=0;i<hoveredLength;i++){node=nodes[hovered[i]];node.draw(ctx);}}/**
	   * Redraw all edges
	   * @param {CanvasRenderingContext2D} ctx  2D context of a HTML canvas
	   * @private
	   */},{key:"_drawEdges",value:function _drawEdges(ctx){var edges=this.body.edges;var edgeIndices=this.body.edgeIndices;var edge;for(var i=0;i<edgeIndices.length;i++){edge=edges[edgeIndices[i]];if(edge.connected===true){edge.draw(ctx);}}}/**
	   * Determine if the browser requires a setTimeout or a requestAnimationFrame. This was required because
	   * some implementations (safari and IE9) did not support requestAnimationFrame
	   * @private
	   */},{key:"_determineBrowserMethod",value:function _determineBrowserMethod(){if(typeof window!=='undefined'){var browserType=navigator.userAgent.toLowerCase();this.requiresTimeout=false;if(indexOf$3(browserType).call(browserType,'msie 9.0')!=-1){// IE 9
	this.requiresTimeout=true;}else if(indexOf$3(browserType).call(browserType,'safari')!=-1){// safari
	if(indexOf$3(browserType).call(browserType,'chrome')<=-1){this.requiresTimeout=true;}}}else {this.requiresTimeout=true;}}}]);return CanvasRenderer;}();var hammerUtil=createCommonjsModule$1(function(module,exports){/**
	 * Register a touch event, taking place before a gesture
	 * @param {Hammer} hammer       A hammer instance
	 * @param {function} callback   Callback, called as callback(event)
	 */exports.onTouch=function(hammer,callback){callback.inputHandler=function(event){if(event.isFirst){callback(event);}};hammer.on('hammer.input',callback.inputHandler);};/**
	 * Register a release event, taking place after a gesture
	 * @param {Hammer} hammer       A hammer instance
	 * @param {function} callback   Callback, called as callback(event)
	 * @returns {*}
	 */exports.onRelease=function(hammer,callback){callback.inputHandler=function(event){if(event.isFinal){callback(event);}};return hammer.on('hammer.input',callback.inputHandler);};/**
	 * Unregister a touch event, taking place before a gesture
	 * @param {Hammer} hammer       A hammer instance
	 * @param {function} callback   Callback, called as callback(event)
	 */exports.offTouch=function(hammer,callback){hammer.off('hammer.input',callback.inputHandler);};/**
	 * Unregister a release event, taking place before a gesture
	 * @param {Hammer} hammer       A hammer instance
	 * @param {function} callback   Callback, called as callback(event)
	 */exports.offRelease=exports.offTouch;/**
	 * Hack the PinchRecognizer such that it doesn't prevent default behavior
	 * for vertical panning.
	 *
	 * Yeah ... this is quite a hack ... see https://github.com/hammerjs/hammer.js/issues/932
	 *
	 * @param {Hammer.Pinch} pinchRecognizer
	 * @return {Hammer.Pinch} returns the pinchRecognizer
	 */exports.disablePreventDefaultVertically=function(pinchRecognizer){var TOUCH_ACTION_PAN_Y='pan-y';pinchRecognizer.getTouchAction=function(){// default method returns [TOUCH_ACTION_NONE]
	return [TOUCH_ACTION_PAN_Y];};return pinchRecognizer;};});var hammerUtil_1=hammerUtil.onTouch;var hammerUtil_2=hammerUtil.onRelease;var hammerUtil_3=hammerUtil.offTouch;var hammerUtil_4=hammerUtil.offRelease;var hammerUtil_5=hammerUtil.disablePreventDefaultVertically;/**
	 * Create the main frame for the Network.
	 * This function is executed once when a Network object is created. The frame
	 * contains a canvas, and this canvas contains all objects like the axis and
	 * nodes.
	 */var Canvas=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   */function Canvas(body){var _context37;classCallCheck(this,Canvas);this.body=body;this.pixelRatio=1;this.resizeTimer=undefined;this.resizeFunction=bind$2(_context37=this._onResize).call(_context37,this);this.cameraState={};this.initialized=false;this.canvasViewCenter={};this.options={};this.defaultOptions={autoResize:true,height:'100%',width:'100%'};extend(this.options,this.defaultOptions);this.bindEventListeners();}/**
	   * Binds event listeners
	   */createClass(Canvas,[{key:"bindEventListeners",value:function bindEventListeners(){var _this38=this,_context38;// bind the events
	this.body.emitter.once("resize",function(obj){if(obj.width!==0){_this38.body.view.translation.x=obj.width*0.5;}if(obj.height!==0){_this38.body.view.translation.y=obj.height*0.5;}});this.body.emitter.on("setSize",bind$2(_context38=this.setSize).call(_context38,this));this.body.emitter.on("destroy",function(){_this38.hammerFrame.destroy();_this38.hammer.destroy();_this38._cleanUp();});}/**
	   * @param {Object} options
	   */},{key:"setOptions",value:function setOptions(options){var _this39=this;if(options!==undefined){var fields=['width','height','autoResize'];selectiveDeepExtend(fields,this.options,options);}if(this.options.autoResize===true){var _context39;// automatically adapt to a changing size of the browser.
	this._cleanUp();this.resizeTimer=setInterval$1(function(){var changed=_this39.setSize();if(changed===true){_this39.body.emitter.emit("_requestRedraw");}},1000);this.resizeFunction=bind$2(_context39=this._onResize).call(_context39,this);addEventListener(window,'resize',this.resizeFunction);}}/**
	   * @private
	   */},{key:"_cleanUp",value:function _cleanUp(){// automatically adapt to a changing size of the browser.
	if(this.resizeTimer!==undefined){clearInterval(this.resizeTimer);}removeEventListener(window,'resize',this.resizeFunction);this.resizeFunction=undefined;}/**
	   * @private
	   */},{key:"_onResize",value:function _onResize(){this.setSize();this.body.emitter.emit("_redraw");}/**
	   * Get and store the cameraState
	   *
	   * @param {number} [pixelRatio=this.pixelRatio]
	   * @private
	   */},{key:"_getCameraState",value:function _getCameraState(){var pixelRatio=arguments.length>0&&arguments[0]!==undefined?arguments[0]:this.pixelRatio;if(this.initialized===true){this.cameraState.previousWidth=this.frame.canvas.width/pixelRatio;this.cameraState.previousHeight=this.frame.canvas.height/pixelRatio;this.cameraState.scale=this.body.view.scale;this.cameraState.position=this.DOMtoCanvas({x:0.5*this.frame.canvas.width/pixelRatio,y:0.5*this.frame.canvas.height/pixelRatio});}}/**
	   * Set the cameraState
	   * @private
	   */},{key:"_setCameraState",value:function _setCameraState(){if(this.cameraState.scale!==undefined&&this.frame.canvas.clientWidth!==0&&this.frame.canvas.clientHeight!==0&&this.pixelRatio!==0&&this.cameraState.previousWidth>0&&this.cameraState.previousHeight>0){var widthRatio=this.frame.canvas.width/this.pixelRatio/this.cameraState.previousWidth;var heightRatio=this.frame.canvas.height/this.pixelRatio/this.cameraState.previousHeight;var newScale=this.cameraState.scale;if(widthRatio!=1&&heightRatio!=1){newScale=this.cameraState.scale*0.5*(widthRatio+heightRatio);}else if(widthRatio!=1){newScale=this.cameraState.scale*widthRatio;}else if(heightRatio!=1){newScale=this.cameraState.scale*heightRatio;}this.body.view.scale=newScale;// this comes from the view module.
	var currentViewCenter=this.DOMtoCanvas({x:0.5*this.frame.canvas.clientWidth,y:0.5*this.frame.canvas.clientHeight});var distanceFromCenter={// offset from view, distance view has to change by these x and y to center the node
	x:currentViewCenter.x-this.cameraState.position.x,y:currentViewCenter.y-this.cameraState.position.y};this.body.view.translation.x+=distanceFromCenter.x*this.body.view.scale;this.body.view.translation.y+=distanceFromCenter.y*this.body.view.scale;}}/**
	   *
	   * @param {number|string} value
	   * @returns {string}
	   * @private
	   */},{key:"_prepareValue",value:function _prepareValue(value){if(typeof value==='number'){return value+'px';}else if(typeof value==='string'){if(indexOf$3(value).call(value,'%')!==-1||indexOf$3(value).call(value,'px')!==-1){return value;}else if(indexOf$3(value).call(value,'%')===-1){return value+'px';}}throw new Error('Could not use the value supplied for width or height:'+value);}/**
	   * Create the HTML
	   */},{key:"_create",value:function _create(){// remove all elements from the container element.
	while(this.body.container.hasChildNodes()){this.body.container.removeChild(this.body.container.firstChild);}this.frame=document.createElement('div');this.frame.className='vis-network';this.frame.style.position='relative';this.frame.style.overflow='hidden';this.frame.tabIndex=900;// tab index is required for keycharm to bind keystrokes to the div instead of the window
	//////////////////////////////////////////////////////////////////
	this.frame.canvas=document.createElement("canvas");this.frame.canvas.style.position='relative';this.frame.appendChild(this.frame.canvas);if(!this.frame.canvas.getContext){var noCanvas=document.createElement('DIV');noCanvas.style.color='red';noCanvas.style.fontWeight='bold';noCanvas.style.padding='10px';noCanvas.innerHTML='Error: your browser does not support HTML canvas';this.frame.canvas.appendChild(noCanvas);}else {this._setPixelRatio();this.setTransform();}// add the frame to the container element
	this.body.container.appendChild(this.frame);this.body.view.scale=1;this.body.view.translation={x:0.5*this.frame.canvas.clientWidth,y:0.5*this.frame.canvas.clientHeight};this._bindHammer();}/**
	   * This function binds hammer, it can be repeated over and over due to the uniqueness check.
	   * @private
	   */},{key:"_bindHammer",value:function _bindHammer(){var _this40=this;if(this.hammer!==undefined){this.hammer.destroy();}this.drag={};this.pinch={};// init hammer
	this.hammer=new hammer(this.frame.canvas);this.hammer.get('pinch').set({enable:true});// enable to get better response, todo: test on mobile.
	this.hammer.get('pan').set({threshold:5,direction:hammer.DIRECTION_ALL});hammerUtil.onTouch(this.hammer,function(event){_this40.body.eventListeners.onTouch(event);});this.hammer.on('tap',function(event){_this40.body.eventListeners.onTap(event);});this.hammer.on('doubletap',function(event){_this40.body.eventListeners.onDoubleTap(event);});this.hammer.on('press',function(event){_this40.body.eventListeners.onHold(event);});this.hammer.on('panstart',function(event){_this40.body.eventListeners.onDragStart(event);});this.hammer.on('panmove',function(event){_this40.body.eventListeners.onDrag(event);});this.hammer.on('panend',function(event){_this40.body.eventListeners.onDragEnd(event);});this.hammer.on('pinch',function(event){_this40.body.eventListeners.onPinch(event);});// TODO: neatly cleanup these handlers when re-creating the Canvas, IF these are done with hammer, event.stopPropagation will not work?
	this.frame.canvas.addEventListener('wheel',function(event){_this40.body.eventListeners.onMouseWheel(event);});this.frame.canvas.addEventListener('mousemove',function(event){_this40.body.eventListeners.onMouseMove(event);});this.frame.canvas.addEventListener('contextmenu',function(event){_this40.body.eventListeners.onContext(event);});this.hammerFrame=new hammer(this.frame);hammerUtil.onRelease(this.hammerFrame,function(event){_this40.body.eventListeners.onRelease(event);});}/**
	   * Set a new size for the network
	   * @param {string} width   Width in pixels or percentage (for example '800px'
	   *                         or '50%')
	   * @param {string} height  Height in pixels or percentage  (for example '400px'
	   *                         or '30%')
	   * @returns {boolean}
	   */},{key:"setSize",value:function setSize(){var width=arguments.length>0&&arguments[0]!==undefined?arguments[0]:this.options.width;var height=arguments.length>1&&arguments[1]!==undefined?arguments[1]:this.options.height;width=this._prepareValue(width);height=this._prepareValue(height);var emitEvent=false;var oldWidth=this.frame.canvas.width;var oldHeight=this.frame.canvas.height;// update the pixel ratio
	//
	// NOTE: Comment in following is rather inconsistent; this is the ONLY place in the code
	//       where it is assumed that the pixel ratio could change at runtime.
	//       The only way I can think of this happening is a rotating screen or tablet; but then
	//       there should be a mechanism for reloading the data (TODO: check if this is present).
	//
	//       If the assumption is true (i.e. pixel ratio can change at runtime), then *all* usage
	//       of pixel ratio must be overhauled for this.
	//
	//       For the time being, I will humor the assumption here, and in the rest of the code assume it is
	//       constant.
	var previousRatio=this.pixelRatio;// we cache this because the camera state storage needs the old value
	this._setPixelRatio();if(width!=this.options.width||height!=this.options.height||this.frame.style.width!=width||this.frame.style.height!=height){this._getCameraState(previousRatio);this.frame.style.width=width;this.frame.style.height=height;this.frame.canvas.style.width='100%';this.frame.canvas.style.height='100%';this.frame.canvas.width=Math.round(this.frame.canvas.clientWidth*this.pixelRatio);this.frame.canvas.height=Math.round(this.frame.canvas.clientHeight*this.pixelRatio);this.options.width=width;this.options.height=height;this.canvasViewCenter={x:0.5*this.frame.clientWidth,y:0.5*this.frame.clientHeight};emitEvent=true;}else {// this would adapt the width of the canvas to the width from 100% if and only if
	// there is a change.
	var newWidth=Math.round(this.frame.canvas.clientWidth*this.pixelRatio);var newHeight=Math.round(this.frame.canvas.clientHeight*this.pixelRatio);// store the camera if there is a change in size.
	if(this.frame.canvas.width!==newWidth||this.frame.canvas.height!==newHeight){this._getCameraState(previousRatio);}if(this.frame.canvas.width!==newWidth){this.frame.canvas.width=newWidth;emitEvent=true;}if(this.frame.canvas.height!==newHeight){this.frame.canvas.height=newHeight;emitEvent=true;}}if(emitEvent===true){this.body.emitter.emit('resize',{width:Math.round(this.frame.canvas.width/this.pixelRatio),height:Math.round(this.frame.canvas.height/this.pixelRatio),oldWidth:Math.round(oldWidth/this.pixelRatio),oldHeight:Math.round(oldHeight/this.pixelRatio)});// restore the camera on change.
	this._setCameraState();}// set initialized so the get and set camera will work from now on.
	this.initialized=true;return emitEvent;}/**
	   *
	   * @returns {CanvasRenderingContext2D}
	   */},{key:"getContext",value:function getContext(){return this.frame.canvas.getContext("2d");}/**
	   * Determine the pixel ratio for various browsers.
	   *
	   * @returns {number}
	   * @private
	   */},{key:"_determinePixelRatio",value:function _determinePixelRatio(){var ctx=this.getContext();if(ctx===undefined){throw new Error("Could not get canvax context");}var numerator=1;if(typeof window!=='undefined'){// (window !== undefined) doesn't work here!
	// Protection during unit tests, where 'window' can be missing
	numerator=window.devicePixelRatio||1;}var denominator=ctx.webkitBackingStorePixelRatio||ctx.mozBackingStorePixelRatio||ctx.msBackingStorePixelRatio||ctx.oBackingStorePixelRatio||ctx.backingStorePixelRatio||1;return numerator/denominator;}/**
	   * Lazy determination of pixel ratio.
	   *
	   * @private
	   */},{key:"_setPixelRatio",value:function _setPixelRatio(){this.pixelRatio=this._determinePixelRatio();}/**
	   * Set the transform in the contained context, based on its pixelRatio
	   */},{key:"setTransform",value:function setTransform(){var ctx=this.getContext();if(ctx===undefined){throw new Error("Could not get canvax context");}ctx.setTransform(this.pixelRatio,0,0,this.pixelRatio,0,0);}/**
	   * Convert the X coordinate in DOM-space (coordinate point in browser relative to the container div) to
	   * the X coordinate in canvas-space (the simulation sandbox, which the camera looks upon)
	   * @param {number} x
	   * @returns {number}
	   * @private
	   */},{key:"_XconvertDOMtoCanvas",value:function _XconvertDOMtoCanvas(x){return (x-this.body.view.translation.x)/this.body.view.scale;}/**
	   * Convert the X coordinate in canvas-space (the simulation sandbox, which the camera looks upon) to
	   * the X coordinate in DOM-space (coordinate point in browser relative to the container div)
	   * @param {number} x
	   * @returns {number}
	   * @private
	   */},{key:"_XconvertCanvasToDOM",value:function _XconvertCanvasToDOM(x){return x*this.body.view.scale+this.body.view.translation.x;}/**
	   * Convert the Y coordinate in DOM-space (coordinate point in browser relative to the container div) to
	   * the Y coordinate in canvas-space (the simulation sandbox, which the camera looks upon)
	   * @param {number} y
	   * @returns {number}
	   * @private
	   */},{key:"_YconvertDOMtoCanvas",value:function _YconvertDOMtoCanvas(y){return (y-this.body.view.translation.y)/this.body.view.scale;}/**
	   * Convert the Y coordinate in canvas-space (the simulation sandbox, which the camera looks upon) to
	   * the Y coordinate in DOM-space (coordinate point in browser relative to the container div)
	   * @param {number} y
	   * @returns {number}
	   * @private
	   */},{key:"_YconvertCanvasToDOM",value:function _YconvertCanvasToDOM(y){return y*this.body.view.scale+this.body.view.translation.y;}/**
	   * @param {point} pos
	   * @returns {point}
	   */},{key:"canvasToDOM",value:function canvasToDOM(pos){return {x:this._XconvertCanvasToDOM(pos.x),y:this._YconvertCanvasToDOM(pos.y)};}/**
	   *
	   * @param {point} pos
	   * @returns {point}
	   */},{key:"DOMtoCanvas",value:function DOMtoCanvas(pos){return {x:this._XconvertDOMtoCanvas(pos.x),y:this._YconvertDOMtoCanvas(pos.y)};}}]);return Canvas;}();/**
	 * The view
	 */var View=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   * @param {Canvas} canvas
	   */function View(body,canvas){var _context40,_this41=this,_context41;classCallCheck(this,View);this.body=body;this.canvas=canvas;this.animationSpeed=1/this.renderRefreshRate;this.animationEasingFunction="easeInOutQuint";this.easingTime=0;this.sourceScale=0;this.targetScale=0;this.sourceTranslation=0;this.targetTranslation=0;this.lockedOnNodeId=undefined;this.lockedOnNodeOffset=undefined;this.touchTime=0;this.viewFunction=undefined;this.body.emitter.on("fit",bind$2(_context40=this.fit).call(_context40,this));this.body.emitter.on("animationFinished",function(){_this41.body.emitter.emit("_stopRendering");});this.body.emitter.on("unlockNode",bind$2(_context41=this.releaseNode).call(_context41,this));}/**
	   *
	   * @param {Object} [options={}]
	   */createClass(View,[{key:"setOptions",value:function setOptions(){var options=arguments.length>0&&arguments[0]!==undefined?arguments[0]:{};this.options=options;}/**
	   * This function zooms out to fit all data on screen based on amount of nodes
	   * @param {Object} [options={{nodes=Array}}]
	   * @param {boolean} [initialZoom=false]  | zoom based on fitted formula or range, true = fitted, default = false;
	   */},{key:"fit",value:function fit(){var options=arguments.length>0&&arguments[0]!==undefined?arguments[0]:{nodes:[]};var initialZoom=arguments.length>1&&arguments[1]!==undefined?arguments[1]:false;var range;var zoomLevel;options=assign$2({},options);if(options.nodes===undefined||options.nodes.length===0){options.nodes=this.body.nodeIndices;}var canvasWidth=this.canvas.frame.canvas.clientWidth;var canvasHeight=this.canvas.frame.canvas.clientHeight;if(canvasWidth===0||canvasHeight===0){// There's no point in trying to fit into zero sized canvas. This could
	// potentially even result in invalid values being computed. For example
	// for network without nodes and zero sized canvas the zoom level would
	// end up being computed as 0/0 which results in NaN. In any other case
	// this would be 0/something which is again pointless to compute.
	zoomLevel=1;range=NetworkUtil.getRange(this.body.nodes,options.nodes);}else if(initialZoom===true){// check if more than half of the nodes have a predefined position. If so, we use the range, not the approximation.
	var positionDefined=0;for(var nodeId in this.body.nodes){if(this.body.nodes.hasOwnProperty(nodeId)){var node=this.body.nodes[nodeId];if(node.predefinedPosition===true){positionDefined+=1;}}}if(positionDefined>0.5*this.body.nodeIndices.length){this.fit(options,false);return;}range=NetworkUtil.getRange(this.body.nodes,options.nodes);var numberOfNodes=this.body.nodeIndices.length;zoomLevel=12.662/(numberOfNodes+7.4147)+0.0964822;// this is obtained from fitting a dataset from 5 points with scale levels that looked good.
	// correct for larger canvasses.
	var factor=Math.min(canvasWidth/600,canvasHeight/600);zoomLevel*=factor;}else {this.body.emitter.emit("_resizeNodes");range=NetworkUtil.getRange(this.body.nodes,options.nodes);var xDistance=Math.abs(range.maxX-range.minX)*1.1;var yDistance=Math.abs(range.maxY-range.minY)*1.1;var xZoomLevel=canvasWidth/xDistance;var yZoomLevel=canvasHeight/yDistance;zoomLevel=xZoomLevel<=yZoomLevel?xZoomLevel:yZoomLevel;}if(zoomLevel>1.0){zoomLevel=1.0;}else if(zoomLevel===0){zoomLevel=1.0;}var center=NetworkUtil.findCenter(range);var animationOptions={position:center,scale:zoomLevel,animation:options.animation};this.moveTo(animationOptions);}// animation
	/**
	   * Center a node in view.
	   *
	   * @param {number} nodeId
	   * @param {number} [options]
	   */},{key:"focus",value:function focus(nodeId){var options=arguments.length>1&&arguments[1]!==undefined?arguments[1]:{};if(this.body.nodes[nodeId]!==undefined){var nodePosition={x:this.body.nodes[nodeId].x,y:this.body.nodes[nodeId].y};options.position=nodePosition;options.lockedOnNode=nodeId;this.moveTo(options);}else {console.log("Node: "+nodeId+" cannot be found.");}}/**
	   *
	   * @param {Object} options  |  options.offset   = {x:number, y:number}   // offset from the center in DOM pixels
	   *                          |  options.scale    = number                 // scale to move to
	   *                          |  options.position = {x:number, y:number}   // position to move to
	   *                          |  options.animation = {duration:number, easingFunction:String} || Boolean   // position to move to
	   */},{key:"moveTo",value:function moveTo(options){if(options===undefined){options={};return;}if(options.offset!=null){if(options.offset.x!=null){// Coerce and verify that x is valid.
	options.offset.x=+options.offset.x;if(!_isFinite$2(options.offset.x)){throw new TypeError('The option "offset.x" has to be a finite number.');}}else {options.offset.x=0;}if(options.offset.y!=null){// Coerce and verify that y is valid.
	options.offset.y=+options.offset.y;if(!_isFinite$2(options.offset.y)){throw new TypeError('The option "offset.y" has to be a finite number.');}}else {options.offset.x=0;}}else {options.offset={x:0,y:0};}if(options.position!=null){if(options.position.x!=null){// Coerce and verify that x is valid.
	options.position.x=+options.position.x;if(!_isFinite$2(options.position.x)){throw new TypeError('The option "position.x" has to be a finite number.');}}else {options.position.x=0;}if(options.position.y!=null){// Coerce and verify that y is valid.
	options.position.y=+options.position.y;if(!_isFinite$2(options.position.y)){throw new TypeError('The option "position.y" has to be a finite number.');}}else {options.position.x=0;}}else {options.position=this.getViewPosition();}if(options.scale!=null){// Coerce and verify that the scale is valid.
	options.scale=+options.scale;if(!(options.scale>0)){throw new TypeError('The option "scale" has to be a number greater than zero.');}}else {options.scale=this.body.view.scale;}if(options.animation===undefined){options.animation={duration:0};}if(options.animation===false){options.animation={duration:0};}if(options.animation===true){options.animation={};}if(options.animation.duration===undefined){options.animation.duration=1000;}// default duration
	if(options.animation.easingFunction===undefined){options.animation.easingFunction="easeInOutQuad";}// default easing function
	this.animateView(options);}/**
	   *
	   * @param {Object} options  |  options.offset   = {x:number, y:number}   // offset from the center in DOM pixels
	   *                          |  options.time     = number                 // animation time in milliseconds
	   *                          |  options.scale    = number                 // scale to animate to
	   *                          |  options.position = {x:number, y:number}   // position to animate to
	   *                          |  options.easingFunction = String           // linear, easeInQuad, easeOutQuad, easeInOutQuad,
	   *                                                                       // easeInCubic, easeOutCubic, easeInOutCubic,
	   *                                                                       // easeInQuart, easeOutQuart, easeInOutQuart,
	   *                                                                       // easeInQuint, easeOutQuint, easeInOutQuint
	   */},{key:"animateView",value:function animateView(options){if(options===undefined){return;}this.animationEasingFunction=options.animation.easingFunction;// release if something focussed on the node
	this.releaseNode();if(options.locked===true){this.lockedOnNodeId=options.lockedOnNode;this.lockedOnNodeOffset=options.offset;}// forcefully complete the old animation if it was still running
	if(this.easingTime!=0){this._transitionRedraw(true);// by setting easingtime to 1, we finish the animation.
	}this.sourceScale=this.body.view.scale;this.sourceTranslation=this.body.view.translation;this.targetScale=options.scale;// set the scale so the viewCenter is based on the correct zoom level. This is overridden in the transitionRedraw
	// but at least then we'll have the target transition
	this.body.view.scale=this.targetScale;var viewCenter=this.canvas.DOMtoCanvas({x:0.5*this.canvas.frame.canvas.clientWidth,y:0.5*this.canvas.frame.canvas.clientHeight});var distanceFromCenter={// offset from view, distance view has to change by these x and y to center the node
	x:viewCenter.x-options.position.x,y:viewCenter.y-options.position.y};this.targetTranslation={x:this.sourceTranslation.x+distanceFromCenter.x*this.targetScale+options.offset.x,y:this.sourceTranslation.y+distanceFromCenter.y*this.targetScale+options.offset.y};// if the time is set to 0, don't do an animation
	if(options.animation.duration===0){if(this.lockedOnNodeId!=undefined){var _context42;this.viewFunction=bind$2(_context42=this._lockedRedraw).call(_context42,this);this.body.emitter.on("initRedraw",this.viewFunction);}else {this.body.view.scale=this.targetScale;this.body.view.translation=this.targetTranslation;this.body.emitter.emit("_requestRedraw");}}else {var _context43;this.animationSpeed=1/(60*options.animation.duration*0.001)||1/60;// 60 for 60 seconds, 0.001 for milli's
	this.animationEasingFunction=options.animation.easingFunction;this.viewFunction=bind$2(_context43=this._transitionRedraw).call(_context43,this);this.body.emitter.on("initRedraw",this.viewFunction);this.body.emitter.emit("_startRendering");}}/**
	   * used to animate smoothly by hijacking the redraw function.
	   * @private
	   */},{key:"_lockedRedraw",value:function _lockedRedraw(){var nodePosition={x:this.body.nodes[this.lockedOnNodeId].x,y:this.body.nodes[this.lockedOnNodeId].y};var viewCenter=this.canvas.DOMtoCanvas({x:0.5*this.canvas.frame.canvas.clientWidth,y:0.5*this.canvas.frame.canvas.clientHeight});var distanceFromCenter={// offset from view, distance view has to change by these x and y to center the node
	x:viewCenter.x-nodePosition.x,y:viewCenter.y-nodePosition.y};var sourceTranslation=this.body.view.translation;var targetTranslation={x:sourceTranslation.x+distanceFromCenter.x*this.body.view.scale+this.lockedOnNodeOffset.x,y:sourceTranslation.y+distanceFromCenter.y*this.body.view.scale+this.lockedOnNodeOffset.y};this.body.view.translation=targetTranslation;}/**
	   * Resets state of a locked on Node
	   */},{key:"releaseNode",value:function releaseNode(){if(this.lockedOnNodeId!==undefined&&this.viewFunction!==undefined){this.body.emitter.off("initRedraw",this.viewFunction);this.lockedOnNodeId=undefined;this.lockedOnNodeOffset=undefined;}}/**
	   * @param {boolean} [finished=false]
	   * @private
	   */},{key:"_transitionRedraw",value:function _transitionRedraw(){var finished=arguments.length>0&&arguments[0]!==undefined?arguments[0]:false;this.easingTime+=this.animationSpeed;this.easingTime=finished===true?1.0:this.easingTime;var progress=easingFunctions[this.animationEasingFunction](this.easingTime);this.body.view.scale=this.sourceScale+(this.targetScale-this.sourceScale)*progress;this.body.view.translation={x:this.sourceTranslation.x+(this.targetTranslation.x-this.sourceTranslation.x)*progress,y:this.sourceTranslation.y+(this.targetTranslation.y-this.sourceTranslation.y)*progress};// cleanup
	if(this.easingTime>=1.0){this.body.emitter.off("initRedraw",this.viewFunction);this.easingTime=0;if(this.lockedOnNodeId!=undefined){var _context44;this.viewFunction=bind$2(_context44=this._lockedRedraw).call(_context44,this);this.body.emitter.on("initRedraw",this.viewFunction);}this.body.emitter.emit("animationFinished");}}/**
	   *
	   * @returns {number}
	   */},{key:"getScale",value:function getScale(){return this.body.view.scale;}/**
	   *
	   * @returns {{x: number, y: number}}
	   */},{key:"getViewPosition",value:function getViewPosition(){return this.canvas.DOMtoCanvas({x:0.5*this.canvas.frame.canvas.clientWidth,y:0.5*this.canvas.frame.canvas.clientHeight});}}]);return View;}();/**
	 * Navigation Handler
	 */var NavigationHandler=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   * @param {Canvas} canvas
	   */function NavigationHandler(body,canvas){var _this42=this;classCallCheck(this,NavigationHandler);this.body=body;this.canvas=canvas;this.iconsCreated=false;this.navigationHammers=[];this.boundFunctions={};this.touchTime=0;this.activated=false;this.body.emitter.on("activate",function(){_this42.activated=true;_this42.configureKeyboardBindings();});this.body.emitter.on("deactivate",function(){_this42.activated=false;_this42.configureKeyboardBindings();});this.body.emitter.on("destroy",function(){if(_this42.keycharm!==undefined){_this42.keycharm.destroy();}});this.options={};}/**
	   *
	   * @param {Object} options
	   */createClass(NavigationHandler,[{key:"setOptions",value:function setOptions(options){if(options!==undefined){this.options=options;this.create();}}/**
	   * Creates or refreshes navigation and sets key bindings
	   */},{key:"create",value:function create(){if(this.options.navigationButtons===true){if(this.iconsCreated===false){this.loadNavigationElements();}}else if(this.iconsCreated===true){this.cleanNavigation();}this.configureKeyboardBindings();}/**
	   * Cleans up previous navigation items
	   */},{key:"cleanNavigation",value:function cleanNavigation(){// clean hammer bindings
	if(this.navigationHammers.length!=0){for(var i=0;i<this.navigationHammers.length;i++){this.navigationHammers[i].destroy();}this.navigationHammers=[];}// clean up previous navigation items
	if(this.navigationDOM&&this.navigationDOM['wrapper']&&this.navigationDOM['wrapper'].parentNode){this.navigationDOM['wrapper'].parentNode.removeChild(this.navigationDOM['wrapper']);}this.iconsCreated=false;}/**
	   * Creation of the navigation controls nodes. They are drawn over the rest of the nodes and are not affected by scale and translation
	   * they have a triggerFunction which is called on click. If the position of the navigation controls is dependent
	   * on this.frame.canvas.clientWidth or this.frame.canvas.clientHeight, we flag horizontalAlignLeft and verticalAlignTop false.
	   * This means that the location will be corrected by the _relocateNavigation function on a size change of the canvas.
	   *
	   * @private
	   */},{key:"loadNavigationElements",value:function loadNavigationElements(){var _this43=this;this.cleanNavigation();this.navigationDOM={};var navigationDivs=['up','down','left','right','zoomIn','zoomOut','zoomExtends'];var navigationDivActions=['_moveUp','_moveDown','_moveLeft','_moveRight','_zoomIn','_zoomOut','_fit'];this.navigationDOM['wrapper']=document.createElement('div');this.navigationDOM['wrapper'].className='vis-navigation';this.canvas.frame.appendChild(this.navigationDOM['wrapper']);for(var i=0;i<navigationDivs.length;i++){this.navigationDOM[navigationDivs[i]]=document.createElement('div');this.navigationDOM[navigationDivs[i]].className='vis-button vis-'+navigationDivs[i];this.navigationDOM['wrapper'].appendChild(this.navigationDOM[navigationDivs[i]]);var hammer$1=new hammer(this.navigationDOM[navigationDivs[i]]);if(navigationDivActions[i]==="_fit"){var _context45;hammerUtil.onTouch(hammer$1,bind$2(_context45=this._fit).call(_context45,this));}else {var _context46;hammerUtil.onTouch(hammer$1,bind$2(_context46=this.bindToRedraw).call(_context46,this,navigationDivActions[i]));}this.navigationHammers.push(hammer$1);}// use a hammer for the release so we do not require the one used in the rest of the network
	// the one the rest uses can be overloaded by the manipulation system.
	var hammerFrame=new hammer(this.canvas.frame);hammerUtil.onRelease(hammerFrame,function(){_this43._stopMovement();});this.navigationHammers.push(hammerFrame);this.iconsCreated=true;}/**
	   *
	   * @param {string} action
	   */},{key:"bindToRedraw",value:function bindToRedraw(action){if(this.boundFunctions[action]===undefined){var _context47;this.boundFunctions[action]=bind$2(_context47=this[action]).call(_context47,this);this.body.emitter.on("initRedraw",this.boundFunctions[action]);this.body.emitter.emit("_startRendering");}}/**
	   *
	   * @param {string} action
	   */},{key:"unbindFromRedraw",value:function unbindFromRedraw(action){if(this.boundFunctions[action]!==undefined){this.body.emitter.off("initRedraw",this.boundFunctions[action]);this.body.emitter.emit("_stopRendering");delete this.boundFunctions[action];}}/**
	   * this stops all movement induced by the navigation buttons
	   *
	   * @private
	   */},{key:"_fit",value:function _fit(){if(new Date().valueOf()-this.touchTime>700){// TODO: fix ugly hack to avoid hammer's double fireing of event (because we use release?)
	this.body.emitter.emit("fit",{duration:700});this.touchTime=new Date().valueOf();}}/**
	   * this stops all movement induced by the navigation buttons
	   *
	   * @private
	   */},{key:"_stopMovement",value:function _stopMovement(){for(var boundAction in this.boundFunctions){if(this.boundFunctions.hasOwnProperty(boundAction)){this.body.emitter.off("initRedraw",this.boundFunctions[boundAction]);this.body.emitter.emit("_stopRendering");}}this.boundFunctions={};}/**
	   *
	   * @private
	   */},{key:"_moveUp",value:function _moveUp(){this.body.view.translation.y+=this.options.keyboard.speed.y;}/**
	   *
	   * @private
	   */},{key:"_moveDown",value:function _moveDown(){this.body.view.translation.y-=this.options.keyboard.speed.y;}/**
	   *
	   * @private
	   */},{key:"_moveLeft",value:function _moveLeft(){this.body.view.translation.x+=this.options.keyboard.speed.x;}/**
	   *
	   * @private
	   */},{key:"_moveRight",value:function _moveRight(){this.body.view.translation.x-=this.options.keyboard.speed.x;}/**
	   *
	   * @private
	   */},{key:"_zoomIn",value:function _zoomIn(){var scaleOld=this.body.view.scale;var scale=this.body.view.scale*(1+this.options.keyboard.speed.zoom);var translation=this.body.view.translation;var scaleFrac=scale/scaleOld;var tx=(1-scaleFrac)*this.canvas.canvasViewCenter.x+translation.x*scaleFrac;var ty=(1-scaleFrac)*this.canvas.canvasViewCenter.y+translation.y*scaleFrac;this.body.view.scale=scale;this.body.view.translation={x:tx,y:ty};this.body.emitter.emit('zoom',{direction:'+',scale:this.body.view.scale,pointer:null});}/**
	   *
	   * @private
	   */},{key:"_zoomOut",value:function _zoomOut(){var scaleOld=this.body.view.scale;var scale=this.body.view.scale/(1+this.options.keyboard.speed.zoom);var translation=this.body.view.translation;var scaleFrac=scale/scaleOld;var tx=(1-scaleFrac)*this.canvas.canvasViewCenter.x+translation.x*scaleFrac;var ty=(1-scaleFrac)*this.canvas.canvasViewCenter.y+translation.y*scaleFrac;this.body.view.scale=scale;this.body.view.translation={x:tx,y:ty};this.body.emitter.emit('zoom',{direction:'-',scale:this.body.view.scale,pointer:null});}/**
	   * bind all keys using keycharm.
	   */},{key:"configureKeyboardBindings",value:function configureKeyboardBindings(){var _this44=this;if(this.keycharm!==undefined){this.keycharm.destroy();}if(this.options.keyboard.enabled===true){if(this.options.keyboard.bindToWindow===true){this.keycharm=keycharm({container:window,preventDefault:true});}else {this.keycharm=keycharm({container:this.canvas.frame,preventDefault:true});}this.keycharm.reset();if(this.activated===true){var _context48,_context49,_context50,_context51,_context52,_context53,_context54,_context55,_context56,_context57,_context58,_context59,_context60,_context61,_context62,_context63,_context64,_context65,_context66,_context67,_context68,_context69,_context70,_context71;bind$2(_context48=this.keycharm).call(_context48,"up",function(){_this44.bindToRedraw("_moveUp");},"keydown");bind$2(_context49=this.keycharm).call(_context49,"down",function(){_this44.bindToRedraw("_moveDown");},"keydown");bind$2(_context50=this.keycharm).call(_context50,"left",function(){_this44.bindToRedraw("_moveLeft");},"keydown");bind$2(_context51=this.keycharm).call(_context51,"right",function(){_this44.bindToRedraw("_moveRight");},"keydown");bind$2(_context52=this.keycharm).call(_context52,"=",function(){_this44.bindToRedraw("_zoomIn");},"keydown");bind$2(_context53=this.keycharm).call(_context53,"num+",function(){_this44.bindToRedraw("_zoomIn");},"keydown");bind$2(_context54=this.keycharm).call(_context54,"num-",function(){_this44.bindToRedraw("_zoomOut");},"keydown");bind$2(_context55=this.keycharm).call(_context55,"-",function(){_this44.bindToRedraw("_zoomOut");},"keydown");bind$2(_context56=this.keycharm).call(_context56,"[",function(){_this44.bindToRedraw("_zoomOut");},"keydown");bind$2(_context57=this.keycharm).call(_context57,"]",function(){_this44.bindToRedraw("_zoomIn");},"keydown");bind$2(_context58=this.keycharm).call(_context58,"pageup",function(){_this44.bindToRedraw("_zoomIn");},"keydown");bind$2(_context59=this.keycharm).call(_context59,"pagedown",function(){_this44.bindToRedraw("_zoomOut");},"keydown");bind$2(_context60=this.keycharm).call(_context60,"up",function(){_this44.unbindFromRedraw("_moveUp");},"keyup");bind$2(_context61=this.keycharm).call(_context61,"down",function(){_this44.unbindFromRedraw("_moveDown");},"keyup");bind$2(_context62=this.keycharm).call(_context62,"left",function(){_this44.unbindFromRedraw("_moveLeft");},"keyup");bind$2(_context63=this.keycharm).call(_context63,"right",function(){_this44.unbindFromRedraw("_moveRight");},"keyup");bind$2(_context64=this.keycharm).call(_context64,"=",function(){_this44.unbindFromRedraw("_zoomIn");},"keyup");bind$2(_context65=this.keycharm).call(_context65,"num+",function(){_this44.unbindFromRedraw("_zoomIn");},"keyup");bind$2(_context66=this.keycharm).call(_context66,"num-",function(){_this44.unbindFromRedraw("_zoomOut");},"keyup");bind$2(_context67=this.keycharm).call(_context67,"-",function(){_this44.unbindFromRedraw("_zoomOut");},"keyup");bind$2(_context68=this.keycharm).call(_context68,"[",function(){_this44.unbindFromRedraw("_zoomOut");},"keyup");bind$2(_context69=this.keycharm).call(_context69,"]",function(){_this44.unbindFromRedraw("_zoomIn");},"keyup");bind$2(_context70=this.keycharm).call(_context70,"pageup",function(){_this44.unbindFromRedraw("_zoomIn");},"keyup");bind$2(_context71=this.keycharm).call(_context71,"pagedown",function(){_this44.unbindFromRedraw("_zoomOut");},"keyup");}}}}]);return NavigationHandler;}();/**
	 * Popup is a class to create a popup window with some text
	 */var Popup=/*#__PURE__*/function(){/**
	   * @param {Element} container       The container object.
	   * @param {string}  overflowMethod  How the popup should act to overflowing ('flip' or 'cap')
	   */function Popup(container,overflowMethod){classCallCheck(this,Popup);this.container=container;this.overflowMethod=overflowMethod||'cap';this.x=0;this.y=0;this.padding=5;this.hidden=false;// create the frame
	this.frame=document.createElement('div');this.frame.className='vis-tooltip';this.container.appendChild(this.frame);}/**
	   * @param {number} x   Horizontal position of the popup window
	   * @param {number} y   Vertical position of the popup window
	   */createClass(Popup,[{key:"setPosition",value:function setPosition(x,y){this.x=_parseInt$2(x);this.y=_parseInt$2(y);}/**
	   * Set the content for the popup window. This can be HTML code or text.
	   * @param {string | Element} content
	   */},{key:"setText",value:function setText(content){if(content instanceof Element){this.frame.innerHTML='';this.frame.appendChild(content);}else {this.frame.innerHTML=content;// string containing text or HTML
	}}/**
	   * Show the popup window
	   * @param {boolean} [doShow]    Show or hide the window
	   */},{key:"show",value:function show(doShow){if(doShow===undefined){doShow=true;}if(doShow===true){var height=this.frame.clientHeight;var width=this.frame.clientWidth;var maxHeight=this.frame.parentNode.clientHeight;var maxWidth=this.frame.parentNode.clientWidth;var left=0,top=0;if(this.overflowMethod=='flip'){var isLeft=false,isTop=true;// Where around the position it's located
	if(this.y-height<this.padding){isTop=false;}if(this.x+width>maxWidth-this.padding){isLeft=true;}if(isLeft){left=this.x-width;}else {left=this.x;}if(isTop){top=this.y-height;}else {top=this.y;}}else {top=this.y-height;if(top+height+this.padding>maxHeight){top=maxHeight-height-this.padding;}if(top<this.padding){top=this.padding;}left=this.x;if(left+width+this.padding>maxWidth){left=maxWidth-width-this.padding;}if(left<this.padding){left=this.padding;}}this.frame.style.left=left+"px";this.frame.style.top=top+"px";this.frame.style.visibility="visible";this.hidden=false;}else {this.hide();}}/**
	   * Hide the popup window
	   */},{key:"hide",value:function hide(){this.hidden=true;this.frame.style.left="0";this.frame.style.top="0";this.frame.style.visibility="hidden";}/**
	   * Remove the popup window
	   */},{key:"destroy",value:function destroy(){this.frame.parentNode.removeChild(this.frame);// Remove element from DOM
	}}]);return Popup;}();/**
	 * Handler for interactions
	 */var InteractionHandler=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   * @param {Canvas} canvas
	   * @param {SelectionHandler} selectionHandler
	   */function InteractionHandler(body,canvas,selectionHandler){var _context72,_context73,_context74,_context75,_context76,_context77,_context78,_context79,_context80,_context81,_context82,_context83,_context84;classCallCheck(this,InteractionHandler);this.body=body;this.canvas=canvas;this.selectionHandler=selectionHandler;this.navigationHandler=new NavigationHandler(body,canvas);// bind the events from hammer to functions in this object
	this.body.eventListeners.onTap=bind$2(_context72=this.onTap).call(_context72,this);this.body.eventListeners.onTouch=bind$2(_context73=this.onTouch).call(_context73,this);this.body.eventListeners.onDoubleTap=bind$2(_context74=this.onDoubleTap).call(_context74,this);this.body.eventListeners.onHold=bind$2(_context75=this.onHold).call(_context75,this);this.body.eventListeners.onDragStart=bind$2(_context76=this.onDragStart).call(_context76,this);this.body.eventListeners.onDrag=bind$2(_context77=this.onDrag).call(_context77,this);this.body.eventListeners.onDragEnd=bind$2(_context78=this.onDragEnd).call(_context78,this);this.body.eventListeners.onMouseWheel=bind$2(_context79=this.onMouseWheel).call(_context79,this);this.body.eventListeners.onPinch=bind$2(_context80=this.onPinch).call(_context80,this);this.body.eventListeners.onMouseMove=bind$2(_context81=this.onMouseMove).call(_context81,this);this.body.eventListeners.onRelease=bind$2(_context82=this.onRelease).call(_context82,this);this.body.eventListeners.onContext=bind$2(_context83=this.onContext).call(_context83,this);this.touchTime=0;this.drag={};this.pinch={};this.popup=undefined;this.popupObj=undefined;this.popupTimer=undefined;this.body.functions.getPointer=bind$2(_context84=this.getPointer).call(_context84,this);this.options={};this.defaultOptions={dragNodes:true,dragView:true,hover:false,keyboard:{enabled:false,speed:{x:10,y:10,zoom:0.02},bindToWindow:true},navigationButtons:false,tooltipDelay:300,zoomView:true,zoomSpeed:1};extend(this.options,this.defaultOptions);this.bindEventListeners();}/**
	   * Binds event listeners
	   */createClass(InteractionHandler,[{key:"bindEventListeners",value:function bindEventListeners(){var _this45=this;this.body.emitter.on('destroy',function(){clearTimeout(_this45.popupTimer);delete _this45.body.functions.getPointer;});}/**
	   *
	   * @param {Object} options
	   */},{key:"setOptions",value:function setOptions(options){if(options!==undefined){// extend all but the values in fields
	var fields=['hideEdgesOnDrag','hideEdgesOnZoom','hideNodesOnDrag','keyboard','multiselect','selectable','selectConnectedEdges'];selectiveNotDeepExtend(fields,this.options,options);// merge the keyboard options in.
	mergeOptions(this.options,options,'keyboard');if(options.tooltip){extend(this.options.tooltip,options.tooltip);if(options.tooltip.color){this.options.tooltip.color=parseColor(options.tooltip.color);}}}this.navigationHandler.setOptions(this.options);}/**
	   * Get the pointer location from a touch location
	   * @param {{x: number, y: number}} touch
	   * @return {{x: number, y: number}} pointer
	   * @private
	   */},{key:"getPointer",value:function getPointer(touch){return {x:touch.x-getAbsoluteLeft(this.canvas.frame.canvas),y:touch.y-getAbsoluteTop(this.canvas.frame.canvas)};}/**
	   * On start of a touch gesture, store the pointer
	   * @param {Event}  event   The event
	   * @private
	   */},{key:"onTouch",value:function onTouch(event){if(new Date().valueOf()-this.touchTime>50){this.drag.pointer=this.getPointer(event.center);this.drag.pinched=false;this.pinch.scale=this.body.view.scale;// to avoid double fireing of this event because we have two hammer instances. (on canvas and on frame)
	this.touchTime=new Date().valueOf();}}/**
	   * handle tap/click event: select/unselect a node
	   * @param {Event} event
	   * @private
	   */},{key:"onTap",value:function onTap(event){var pointer=this.getPointer(event.center);var multiselect=this.selectionHandler.options.multiselect&&(event.changedPointers[0].ctrlKey||event.changedPointers[0].metaKey);this.checkSelectionChanges(pointer,event,multiselect);this.selectionHandler._generateClickEvent('click',event,pointer);}/**
	   * handle doubletap event
	   * @param {Event} event
	   * @private
	   */},{key:"onDoubleTap",value:function onDoubleTap(event){var pointer=this.getPointer(event.center);this.selectionHandler._generateClickEvent('doubleClick',event,pointer);}/**
	   * handle long tap event: multi select nodes
	   * @param {Event} event
	   * @private
	   */},{key:"onHold",value:function onHold(event){var pointer=this.getPointer(event.center);var multiselect=this.selectionHandler.options.multiselect;this.checkSelectionChanges(pointer,event,multiselect);this.selectionHandler._generateClickEvent('click',event,pointer);this.selectionHandler._generateClickEvent('hold',event,pointer);}/**
	   * handle the release of the screen
	   *
	   * @param {Event} event
	   * @private
	   */},{key:"onRelease",value:function onRelease(event){if(new Date().valueOf()-this.touchTime>10){var pointer=this.getPointer(event.center);this.selectionHandler._generateClickEvent('release',event,pointer);// to avoid double fireing of this event because we have two hammer instances. (on canvas and on frame)
	this.touchTime=new Date().valueOf();}}/**
	   *
	   * @param {Event} event
	   */},{key:"onContext",value:function onContext(event){var pointer=this.getPointer({x:event.clientX,y:event.clientY});this.selectionHandler._generateClickEvent('oncontext',event,pointer);}/**
	   * Select and deselect nodes depending current selection change.
	   *
	   * For changing nodes, select/deselect events are fired.
	   *
	   * NOTE: For a given edge, if one connecting node is deselected and with the same
	   *       click the other node is selected, no events for the edge will fire.
	   *       It was selected and it will remain selected.
	   *
	   * TODO: This is all SelectionHandler calls; the method should be moved to there.
	   *
	   * @param {{x: number, y: number}} pointer
	   * @param {Event} event
	   * @param {boolean} [add=false]
	   */},{key:"checkSelectionChanges",value:function checkSelectionChanges(pointer,event){var add=arguments.length>2&&arguments[2]!==undefined?arguments[2]:false;var previousSelection=this.selectionHandler.getSelection();var selected=false;if(add===true){selected=this.selectionHandler.selectAdditionalOnPoint(pointer);}else {selected=this.selectionHandler.selectOnPoint(pointer);}var currentSelection=this.selectionHandler.getSelection();// See NOTE in method comment for the reason to do it like this
	var deselectedItems=this._determineDifference(previousSelection,currentSelection);var selectedItems=this._determineDifference(currentSelection,previousSelection);if(deselectedItems.edges.length>0){this.selectionHandler._generateClickEvent('deselectEdge',event,pointer,previousSelection);selected=true;}if(deselectedItems.nodes.length>0){this.selectionHandler._generateClickEvent('deselectNode',event,pointer,previousSelection);selected=true;}if(selectedItems.nodes.length>0){this.selectionHandler._generateClickEvent('selectNode',event,pointer);selected=true;}if(selectedItems.edges.length>0){this.selectionHandler._generateClickEvent('selectEdge',event,pointer);selected=true;}// fire the select event if anything has been selected or deselected
	if(selected===true){// select or unselect
	this.selectionHandler._generateClickEvent('select',event,pointer);}}/**
	   * Remove all node and edge id's from the first set that are present in the second one.
	   *
	   * @param {{nodes: Array.<Node>, edges: Array.<vis.Edge>}} firstSet
	   * @param {{nodes: Array.<Node>, edges: Array.<vis.Edge>}} secondSet
	   * @returns {{nodes: Array.<Node>, edges: Array.<vis.Edge>}}
	   * @private
	   */},{key:"_determineDifference",value:function _determineDifference(firstSet,secondSet){var arrayDiff=function arrayDiff(firstArr,secondArr){var result=[];for(var i=0;i<firstArr.length;i++){var value=firstArr[i];if(indexOf$3(secondArr).call(secondArr,value)===-1){result.push(value);}}return result;};return {nodes:arrayDiff(firstSet.nodes,secondSet.nodes),edges:arrayDiff(firstSet.edges,secondSet.edges)};}/**
	   * This function is called by onDragStart.
	   * It is separated out because we can then overload it for the datamanipulation system.
	   *
	   * @param {Event} event
	   * @private
	   */},{key:"onDragStart",value:function onDragStart(event){//in case the touch event was triggered on an external div, do the initial touch now.
	if(this.drag.pointer===undefined){this.onTouch(event);}// note: drag.pointer is set in onTouch to get the initial touch location
	var node=this.selectionHandler.getNodeAt(this.drag.pointer);this.drag.dragging=true;this.drag.selection=[];this.drag.translation=extend({},this.body.view.translation);// copy the object
	this.drag.nodeId=undefined;if(node!==undefined&&this.options.dragNodes===true){this.drag.nodeId=node.id;// select the clicked node if not yet selected
	if(node.isSelected()===false){this.selectionHandler.unselectAll();this.selectionHandler.selectObject(node);}// after select to contain the node
	this.selectionHandler._generateClickEvent('dragStart',event,this.drag.pointer);var selection=this.selectionHandler.selectionObj.nodes;// create an array with the selected nodes and their original location and status
	for(var nodeId in selection){if(selection.hasOwnProperty(nodeId)){var _object=selection[nodeId];var s={id:_object.id,node:_object,// store original x, y, xFixed and yFixed, make the node temporarily Fixed
	x:_object.x,y:_object.y,xFixed:_object.options.fixed.x,yFixed:_object.options.fixed.y};_object.options.fixed.x=true;_object.options.fixed.y=true;this.drag.selection.push(s);}}}else {// fallback if no node is selected and thus the view is dragged.
	this.selectionHandler._generateClickEvent('dragStart',event,this.drag.pointer,undefined,true);}}/**
	   * handle drag event
	   * @param {Event} event
	   * @private
	   */},{key:"onDrag",value:function onDrag(event){var _this46=this;if(this.drag.pinched===true){return;}// remove the focus on node if it is focussed on by the focusOnNode
	this.body.emitter.emit('unlockNode');var pointer=this.getPointer(event.center);var selection=this.drag.selection;if(selection&&selection.length&&this.options.dragNodes===true){this.selectionHandler._generateClickEvent('dragging',event,pointer);// calculate delta's and new location
	var deltaX=pointer.x-this.drag.pointer.x;var deltaY=pointer.y-this.drag.pointer.y;// update position of all selected nodes
	forEach$3(selection).call(selection,function(selection){var node=selection.node;// only move the node if it was not fixed initially
	if(selection.xFixed===false){node.x=_this46.canvas._XconvertDOMtoCanvas(_this46.canvas._XconvertCanvasToDOM(selection.x)+deltaX);}// only move the node if it was not fixed initially
	if(selection.yFixed===false){node.y=_this46.canvas._YconvertDOMtoCanvas(_this46.canvas._YconvertCanvasToDOM(selection.y)+deltaY);}});// start the simulation of the physics
	this.body.emitter.emit('startSimulation');}else {// move the network
	if(this.options.dragView===true){this.selectionHandler._generateClickEvent('dragging',event,pointer,undefined,true);// if the drag was not started properly because the click started outside the network div, start it now.
	if(this.drag.pointer===undefined){this.onDragStart(event);return;}var diffX=pointer.x-this.drag.pointer.x;var diffY=pointer.y-this.drag.pointer.y;this.body.view.translation={x:this.drag.translation.x+diffX,y:this.drag.translation.y+diffY};this.body.emitter.emit('_requestRedraw');}}}/**
	   * handle drag start event
	   * @param {Event} event
	   * @private
	   */},{key:"onDragEnd",value:function onDragEnd(event){this.drag.dragging=false;var selection=this.drag.selection;if(selection&&selection.length){forEach$3(selection).call(selection,function(s){// restore original xFixed and yFixed
	s.node.options.fixed.x=s.xFixed;s.node.options.fixed.y=s.yFixed;});this.selectionHandler._generateClickEvent('dragEnd',event,this.getPointer(event.center));this.body.emitter.emit('startSimulation');}else {this.selectionHandler._generateClickEvent('dragEnd',event,this.getPointer(event.center),undefined,true);this.body.emitter.emit('_requestRedraw');}}/**
	   * Handle pinch event
	   * @param {Event}  event   The event
	   * @private
	   */},{key:"onPinch",value:function onPinch(event){var pointer=this.getPointer(event.center);this.drag.pinched=true;if(this.pinch['scale']===undefined){this.pinch.scale=1;}// TODO: enabled moving while pinching?
	var scale=this.pinch.scale*event.scale;this.zoom(scale,pointer);}/**
	   * Zoom the network in or out
	   * @param {number} scale a number around 1, and between 0.01 and 10
	   * @param {{x: number, y: number}} pointer    Position on screen
	   * @private
	   */},{key:"zoom",value:function zoom(scale,pointer){if(this.options.zoomView===true){var scaleOld=this.body.view.scale;if(scale<0.00001){scale=0.00001;}if(scale>10){scale=10;}var preScaleDragPointer=undefined;if(this.drag!==undefined){if(this.drag.dragging===true){preScaleDragPointer=this.canvas.DOMtoCanvas(this.drag.pointer);}}// + this.canvas.frame.canvas.clientHeight / 2
	var translation=this.body.view.translation;var scaleFrac=scale/scaleOld;var tx=(1-scaleFrac)*pointer.x+translation.x*scaleFrac;var ty=(1-scaleFrac)*pointer.y+translation.y*scaleFrac;this.body.view.scale=scale;this.body.view.translation={x:tx,y:ty};if(preScaleDragPointer!=undefined){var postScaleDragPointer=this.canvas.canvasToDOM(preScaleDragPointer);this.drag.pointer.x=postScaleDragPointer.x;this.drag.pointer.y=postScaleDragPointer.y;}this.body.emitter.emit('_requestRedraw');if(scaleOld<scale){this.body.emitter.emit('zoom',{direction:'+',scale:this.body.view.scale,pointer:pointer});}else {this.body.emitter.emit('zoom',{direction:'-',scale:this.body.view.scale,pointer:pointer});}}}/**
	   * Event handler for mouse wheel event, used to zoom the timeline
	   * See http://adomas.org/javascript-mouse-wheel/
	   *     https://github.com/EightMedia/hammer.js/issues/256
	   * @param {MouseEvent}  event
	   * @private
	   */},{key:"onMouseWheel",value:function onMouseWheel(event){if(this.options.zoomView===true){// If delta is nonzero, handle it.
	// Basically, delta is now positive if wheel was scrolled up,
	// and negative, if wheel was scrolled down.
	if(event.deltaY!==0){// calculate the new scale
	var scale=this.body.view.scale;scale*=1+(event.deltaY<0?1:-1)*(this.options.zoomSpeed*0.1);// calculate the pointer location
	var pointer=this.getPointer({x:event.clientX,y:event.clientY});// apply the new scale
	this.zoom(scale,pointer);}// Prevent default actions caused by mouse wheel.
	event.preventDefault();}}/**
	   * Mouse move handler for checking whether the title moves over a node with a title.
	   * @param  {Event} event
	   * @private
	   */},{key:"onMouseMove",value:function onMouseMove(event){var _this47=this;var pointer=this.getPointer({x:event.clientX,y:event.clientY});var popupVisible=false;// check if the previously selected node is still selected
	if(this.popup!==undefined){if(this.popup.hidden===false){this._checkHidePopup(pointer);}// if the popup was not hidden above
	if(this.popup.hidden===false){popupVisible=true;this.popup.setPosition(pointer.x+3,pointer.y-5);this.popup.show();}}// if we bind the keyboard to the div, we have to highlight it to use it. This highlights it on mouse over.
	if(this.options.keyboard.bindToWindow===false&&this.options.keyboard.enabled===true){this.canvas.frame.focus();}// start a timeout that will check if the mouse is positioned above an element
	if(popupVisible===false){if(this.popupTimer!==undefined){clearInterval(this.popupTimer);// stop any running calculationTimer
	this.popupTimer=undefined;}if(!this.drag.dragging){this.popupTimer=setTimeout$2(function(){return _this47._checkShowPopup(pointer);},this.options.tooltipDelay);}}// adding hover highlights
	if(this.options.hover===true){this.selectionHandler.hoverObject(event,pointer);}}/**
	   * Check if there is an element on the given position in the network
	   * (a node or edge). If so, and if this element has a title,
	   * show a popup window with its title.
	   *
	   * @param {{x:number, y:number}} pointer
	   * @private
	   */},{key:"_checkShowPopup",value:function _checkShowPopup(pointer){var x=this.canvas._XconvertDOMtoCanvas(pointer.x);var y=this.canvas._YconvertDOMtoCanvas(pointer.y);var pointerObj={left:x,top:y,right:x,bottom:y};var previousPopupObjId=this.popupObj===undefined?undefined:this.popupObj.id;var nodeUnderCursor=false;var popupType='node';// check if a node is under the cursor.
	if(this.popupObj===undefined){// search the nodes for overlap, select the top one in case of multiple nodes
	var nodeIndices=this.body.nodeIndices;var nodes=this.body.nodes;var node;var overlappingNodes=[];for(var i=0;i<nodeIndices.length;i++){node=nodes[nodeIndices[i]];if(node.isOverlappingWith(pointerObj)===true){nodeUnderCursor=true;if(node.getTitle()!==undefined){overlappingNodes.push(nodeIndices[i]);}}}if(overlappingNodes.length>0){// if there are overlapping nodes, select the last one, this is the one which is drawn on top of the others
	this.popupObj=nodes[overlappingNodes[overlappingNodes.length-1]];// if you hover over a node, the title of the edge is not supposed to be shown.
	nodeUnderCursor=true;}}if(this.popupObj===undefined&&nodeUnderCursor===false){// search the edges for overlap
	var edgeIndices=this.body.edgeIndices;var edges=this.body.edges;var edge;var overlappingEdges=[];for(var _i15=0;_i15<edgeIndices.length;_i15++){edge=edges[edgeIndices[_i15]];if(edge.isOverlappingWith(pointerObj)===true){if(edge.connected===true&&edge.getTitle()!==undefined){overlappingEdges.push(edgeIndices[_i15]);}}}if(overlappingEdges.length>0){this.popupObj=edges[overlappingEdges[overlappingEdges.length-1]];popupType='edge';}}if(this.popupObj!==undefined){// show popup message window
	if(this.popupObj.id!==previousPopupObjId){if(this.popup===undefined){this.popup=new Popup(this.canvas.frame);}this.popup.popupTargetType=popupType;this.popup.popupTargetId=this.popupObj.id;// adjust a small offset such that the mouse cursor is located in the
	// bottom left location of the popup, and you can easily move over the
	// popup area
	this.popup.setPosition(pointer.x+3,pointer.y-5);this.popup.setText(this.popupObj.getTitle());this.popup.show();this.body.emitter.emit('showPopup',this.popupObj.id);}}else {if(this.popup!==undefined){this.popup.hide();this.body.emitter.emit('hidePopup');}}}/**
	   * Check if the popup must be hidden, which is the case when the mouse is no
	   * longer hovering on the object
	   * @param {{x:number, y:number}} pointer
	   * @private
	   */},{key:"_checkHidePopup",value:function _checkHidePopup(pointer){var pointerObj=this.selectionHandler._pointerToPositionObject(pointer);var stillOnObj=false;if(this.popup.popupTargetType==='node'){if(this.body.nodes[this.popup.popupTargetId]!==undefined){stillOnObj=this.body.nodes[this.popup.popupTargetId].isOverlappingWith(pointerObj);// if the mouse is still one the node, we have to check if it is not also on one that is drawn on top of it.
	// we initially only check stillOnObj because this is much faster.
	if(stillOnObj===true){var overNode=this.selectionHandler.getNodeAt(pointer);stillOnObj=overNode===undefined?false:overNode.id===this.popup.popupTargetId;}}}else {if(this.selectionHandler.getNodeAt(pointer)===undefined){if(this.body.edges[this.popup.popupTargetId]!==undefined){stillOnObj=this.body.edges[this.popup.popupTargetId].isOverlappingWith(pointerObj);}}}if(stillOnObj===false){this.popupObj=undefined;this.popup.hide();this.body.emitter.emit('hidePopup');}}}]);return InteractionHandler;}();/**
	 * The handler for selections
	 */var SelectionHandler=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   * @param {Canvas} canvas
	   */function SelectionHandler(body,canvas){var _this48=this;classCallCheck(this,SelectionHandler);this.body=body;this.canvas=canvas;this.selectionObj={nodes:[],edges:[]};this.hoverObj={nodes:{},edges:{}};this.options={};this.defaultOptions={multiselect:false,selectable:true,selectConnectedEdges:true,hoverConnectedEdges:true};extend(this.options,this.defaultOptions);this.body.emitter.on("_dataChanged",function(){_this48.updateSelection();});}/**
	   *
	   * @param {Object} [options]
	   */createClass(SelectionHandler,[{key:"setOptions",value:function setOptions(options){if(options!==undefined){var fields=['multiselect','hoverConnectedEdges','selectable','selectConnectedEdges'];selectiveDeepExtend(fields,this.options,options);}}/**
	   * handles the selection part of the tap;
	   *
	   * @param {{x: number, y: number}} pointer
	   * @returns {boolean}
	   */},{key:"selectOnPoint",value:function selectOnPoint(pointer){var selected=false;if(this.options.selectable===true){var obj=this.getNodeAt(pointer)||this.getEdgeAt(pointer);// unselect after getting the objects in order to restore width and height.
	this.unselectAll();if(obj!==undefined){selected=this.selectObject(obj);}this.body.emitter.emit("_requestRedraw");}return selected;}/**
	   *
	   * @param {{x: number, y: number}} pointer
	   * @returns {boolean}
	   */},{key:"selectAdditionalOnPoint",value:function selectAdditionalOnPoint(pointer){var selectionChanged=false;if(this.options.selectable===true){var obj=this.getNodeAt(pointer)||this.getEdgeAt(pointer);if(obj!==undefined){selectionChanged=true;if(obj.isSelected()===true){this.deselectObject(obj);}else {this.selectObject(obj);}this.body.emitter.emit("_requestRedraw");}}return selectionChanged;}/**
	   * Create an object containing the standard fields for an event.
	   *
	   * @param {Event} event
	   * @param {{x: number, y: number}} pointer Object with the x and y screen coordinates of the mouse
	   * @returns {{}}
	   * @private
	   */},{key:"_initBaseEvent",value:function _initBaseEvent(event,pointer){var properties={};properties['pointer']={DOM:{x:pointer.x,y:pointer.y},canvas:this.canvas.DOMtoCanvas(pointer)};properties['event']=event;return properties;}/**
	   * Generate an event which the user can catch.
	   *
	   * This adds some extra data to the event with respect to cursor position and
	   * selected nodes and edges.
	   *
	   * @param {string} eventType                          Name of event to send
	   * @param {Event}  event
	   * @param {{x: number, y: number}} pointer            Object with the x and y screen coordinates of the mouse
	   * @param {Object|undefined} oldSelection             If present, selection state before event occured
	   * @param {boolean|undefined} [emptySelection=false]  Indicate if selection data should be passed
	   */},{key:"_generateClickEvent",value:function _generateClickEvent(eventType,event,pointer,oldSelection){var emptySelection=arguments.length>4&&arguments[4]!==undefined?arguments[4]:false;var properties=this._initBaseEvent(event,pointer);if(emptySelection===true){properties.nodes=[];properties.edges=[];}else {var tmp=this.getSelection();properties.nodes=tmp.nodes;properties.edges=tmp.edges;}if(oldSelection!==undefined){properties['previousSelection']=oldSelection;}if(eventType=='click'){// For the time being, restrict this functionality to
	// just the click event.
	properties.items=this.getClickedItems(pointer);}if(event.controlEdge!==undefined){properties.controlEdge=event.controlEdge;}this.body.emitter.emit(eventType,properties);}/**
	   *
	   * @param {Object} obj
	   * @param {boolean} [highlightEdges=this.options.selectConnectedEdges]
	   * @returns {boolean}
	   */},{key:"selectObject",value:function selectObject(obj){var highlightEdges=arguments.length>1&&arguments[1]!==undefined?arguments[1]:this.options.selectConnectedEdges;if(obj!==undefined){if(obj instanceof Node){if(highlightEdges===true){this._selectConnectedEdges(obj);}}obj.select();this._addToSelection(obj);return true;}return false;}/**
	   *
	   * @param {Object} obj
	   */},{key:"deselectObject",value:function deselectObject(obj){if(obj.isSelected()===true){obj.selected=false;this._removeFromSelection(obj);}}/**
	   * retrieve all nodes overlapping with given object
	   * @param {Object} object  An object with parameters left, top, right, bottom
	   * @return {number[]}   An array with id's of the overlapping nodes
	   * @private
	   */},{key:"_getAllNodesOverlappingWith",value:function _getAllNodesOverlappingWith(object){var overlappingNodes=[];var nodes=this.body.nodes;for(var i=0;i<this.body.nodeIndices.length;i++){var nodeId=this.body.nodeIndices[i];if(nodes[nodeId].isOverlappingWith(object)){overlappingNodes.push(nodeId);}}return overlappingNodes;}/**
	   * Return a position object in canvasspace from a single point in screenspace
	   *
	   * @param {{x: number, y: number}} pointer
	   * @returns {{left: number, top: number, right: number, bottom: number}}
	   * @private
	   */},{key:"_pointerToPositionObject",value:function _pointerToPositionObject(pointer){var canvasPos=this.canvas.DOMtoCanvas(pointer);return {left:canvasPos.x-1,top:canvasPos.y+1,right:canvasPos.x+1,bottom:canvasPos.y-1};}/**
	   * Get the top node at the passed point (like a click)
	   *
	   * @param {{x: number, y: number}} pointer
	   * @param {boolean} [returnNode=true]
	   * @return {Node | undefined} node
	   */},{key:"getNodeAt",value:function getNodeAt(pointer){var returnNode=arguments.length>1&&arguments[1]!==undefined?arguments[1]:true;// we first check if this is an navigation controls element
	var positionObject=this._pointerToPositionObject(pointer);var overlappingNodes=this._getAllNodesOverlappingWith(positionObject);// if there are overlapping nodes, select the last one, this is the
	// one which is drawn on top of the others
	if(overlappingNodes.length>0){if(returnNode===true){return this.body.nodes[overlappingNodes[overlappingNodes.length-1]];}else {return overlappingNodes[overlappingNodes.length-1];}}else {return undefined;}}/**
	   * retrieve all edges overlapping with given object, selector is around center
	   * @param {Object} object  An object with parameters left, top, right, bottom
	   * @param {number[]} overlappingEdges An array with id's of the overlapping nodes
	   * @private
	   */},{key:"_getEdgesOverlappingWith",value:function _getEdgesOverlappingWith(object,overlappingEdges){var edges=this.body.edges;for(var i=0;i<this.body.edgeIndices.length;i++){var edgeId=this.body.edgeIndices[i];if(edges[edgeId].isOverlappingWith(object)){overlappingEdges.push(edgeId);}}}/**
	   * retrieve all nodes overlapping with given object
	   * @param {Object} object  An object with parameters left, top, right, bottom
	   * @return {number[]}   An array with id's of the overlapping nodes
	   * @private
	   */},{key:"_getAllEdgesOverlappingWith",value:function _getAllEdgesOverlappingWith(object){var overlappingEdges=[];this._getEdgesOverlappingWith(object,overlappingEdges);return overlappingEdges;}/**
	   * Get the edges nearest to the passed point (like a click)
	   *
	   * @param {{x: number, y: number}} pointer
	   * @param {boolean} [returnEdge=true]
	   * @return {Edge | undefined} node
	   */},{key:"getEdgeAt",value:function getEdgeAt(pointer){var returnEdge=arguments.length>1&&arguments[1]!==undefined?arguments[1]:true;// Iterate over edges, pick closest within 10
	var canvasPos=this.canvas.DOMtoCanvas(pointer);var mindist=10;var overlappingEdge=null;var edges=this.body.edges;for(var i=0;i<this.body.edgeIndices.length;i++){var edgeId=this.body.edgeIndices[i];var edge=edges[edgeId];if(edge.connected){var xFrom=edge.from.x;var yFrom=edge.from.y;var xTo=edge.to.x;var yTo=edge.to.y;var dist=edge.edgeType.getDistanceToEdge(xFrom,yFrom,xTo,yTo,canvasPos.x,canvasPos.y);if(dist<mindist){overlappingEdge=edgeId;mindist=dist;}}}if(overlappingEdge!==null){if(returnEdge===true){return this.body.edges[overlappingEdge];}else {return overlappingEdge;}}else {return undefined;}}/**
	   * Add object to the selection array.
	   *
	   * @param {Object} obj
	   * @private
	   */},{key:"_addToSelection",value:function _addToSelection(obj){if(obj instanceof Node){this.selectionObj.nodes[obj.id]=obj;}else {this.selectionObj.edges[obj.id]=obj;}}/**
	   * Add object to the selection array.
	   *
	   * @param {Object} obj
	   * @private
	   */},{key:"_addToHover",value:function _addToHover(obj){if(obj instanceof Node){this.hoverObj.nodes[obj.id]=obj;}else {this.hoverObj.edges[obj.id]=obj;}}/**
	   * Remove a single option from selection.
	   *
	   * @param {Object} obj
	   * @private
	   */},{key:"_removeFromSelection",value:function _removeFromSelection(obj){if(obj instanceof Node){delete this.selectionObj.nodes[obj.id];this._unselectConnectedEdges(obj);}else {delete this.selectionObj.edges[obj.id];}}/**
	   * Unselect all. The selectionObj is useful for this.
	   */},{key:"unselectAll",value:function unselectAll(){for(var nodeId in this.selectionObj.nodes){if(this.selectionObj.nodes.hasOwnProperty(nodeId)){this.selectionObj.nodes[nodeId].unselect();}}for(var edgeId in this.selectionObj.edges){if(this.selectionObj.edges.hasOwnProperty(edgeId)){this.selectionObj.edges[edgeId].unselect();}}this.selectionObj={nodes:{},edges:{}};}/**
	   * return the number of selected nodes
	   *
	   * @returns {number}
	   * @private
	   */},{key:"_getSelectedNodeCount",value:function _getSelectedNodeCount(){var count=0;for(var nodeId in this.selectionObj.nodes){if(this.selectionObj.nodes.hasOwnProperty(nodeId)){count+=1;}}return count;}/**
	   * return the selected node
	   *
	   * @returns {number}
	   * @private
	   */},{key:"_getSelectedNode",value:function _getSelectedNode(){for(var nodeId in this.selectionObj.nodes){if(this.selectionObj.nodes.hasOwnProperty(nodeId)){return this.selectionObj.nodes[nodeId];}}return undefined;}/**
	   * return the selected edge
	   *
	   * @returns {number}
	   * @private
	   */},{key:"_getSelectedEdge",value:function _getSelectedEdge(){for(var edgeId in this.selectionObj.edges){if(this.selectionObj.edges.hasOwnProperty(edgeId)){return this.selectionObj.edges[edgeId];}}return undefined;}/**
	   * return the number of selected edges
	   *
	   * @returns {number}
	   * @private
	   */},{key:"_getSelectedEdgeCount",value:function _getSelectedEdgeCount(){var count=0;for(var edgeId in this.selectionObj.edges){if(this.selectionObj.edges.hasOwnProperty(edgeId)){count+=1;}}return count;}/**
	   * return the number of selected objects.
	   *
	   * @returns {number}
	   * @private
	   */},{key:"_getSelectedObjectCount",value:function _getSelectedObjectCount(){var count=0;for(var nodeId in this.selectionObj.nodes){if(this.selectionObj.nodes.hasOwnProperty(nodeId)){count+=1;}}for(var edgeId in this.selectionObj.edges){if(this.selectionObj.edges.hasOwnProperty(edgeId)){count+=1;}}return count;}/**
	   * Check if anything is selected
	   *
	   * @returns {boolean}
	   * @private
	   */},{key:"_selectionIsEmpty",value:function _selectionIsEmpty(){for(var nodeId in this.selectionObj.nodes){if(this.selectionObj.nodes.hasOwnProperty(nodeId)){return false;}}for(var edgeId in this.selectionObj.edges){if(this.selectionObj.edges.hasOwnProperty(edgeId)){return false;}}return true;}/**
	   * check if one of the selected nodes is a cluster.
	   *
	   * @returns {boolean}
	   * @private
	   */},{key:"_clusterInSelection",value:function _clusterInSelection(){for(var nodeId in this.selectionObj.nodes){if(this.selectionObj.nodes.hasOwnProperty(nodeId)){if(this.selectionObj.nodes[nodeId].clusterSize>1){return true;}}}return false;}/**
	   * select the edges connected to the node that is being selected
	   *
	   * @param {Node} node
	   * @private
	   */},{key:"_selectConnectedEdges",value:function _selectConnectedEdges(node){for(var i=0;i<node.edges.length;i++){var edge=node.edges[i];edge.select();this._addToSelection(edge);}}/**
	   * select the edges connected to the node that is being selected
	   *
	   * @param {Node} node
	   * @private
	   */},{key:"_hoverConnectedEdges",value:function _hoverConnectedEdges(node){for(var i=0;i<node.edges.length;i++){var edge=node.edges[i];edge.hover=true;this._addToHover(edge);}}/**
	   * unselect the edges connected to the node that is being selected
	   *
	   * @param {Node} node
	   * @private
	   */},{key:"_unselectConnectedEdges",value:function _unselectConnectedEdges(node){for(var i=0;i<node.edges.length;i++){var edge=node.edges[i];edge.unselect();this._removeFromSelection(edge);}}/**
	   * Remove the highlight from a node or edge, in response to mouse movement
	   *
	   * @param {Event}  event
	   * @param {{x: number, y: number}} pointer object with the x and y screen coordinates of the mouse
	   * @param {Node|vis.Edge} object
	   * @private
	   */},{key:"emitBlurEvent",value:function emitBlurEvent(event,pointer,object){var properties=this._initBaseEvent(event,pointer);if(object.hover===true){object.hover=false;if(object instanceof Node){properties.node=object.id;this.body.emitter.emit("blurNode",properties);}else {properties.edge=object.id;this.body.emitter.emit("blurEdge",properties);}}}/**
	   * Create the highlight for a node or edge, in response to mouse movement
	   *
	   * @param {Event}  event
	   * @param {{x: number, y: number}} pointer object with the x and y screen coordinates of the mouse
	   * @param {Node|vis.Edge} object
	   * @returns {boolean} hoverChanged
	   * @private
	   */},{key:"emitHoverEvent",value:function emitHoverEvent(event,pointer,object){var properties=this._initBaseEvent(event,pointer);var hoverChanged=false;if(object.hover===false){object.hover=true;this._addToHover(object);hoverChanged=true;if(object instanceof Node){properties.node=object.id;this.body.emitter.emit("hoverNode",properties);}else {properties.edge=object.id;this.body.emitter.emit("hoverEdge",properties);}}return hoverChanged;}/**
	   * Perform actions in response to a mouse movement.
	   *
	   * @param {Event}  event
	   * @param {{x: number, y: number}} pointer | object with the x and y screen coordinates of the mouse
	   */},{key:"hoverObject",value:function hoverObject(event,pointer){var object=this.getNodeAt(pointer);if(object===undefined){object=this.getEdgeAt(pointer);}var hoverChanged=false;// remove all node hover highlights
	for(var nodeId in this.hoverObj.nodes){if(this.hoverObj.nodes.hasOwnProperty(nodeId)){if(object===undefined||object instanceof Node&&object.id!=nodeId||object instanceof Edge){this.emitBlurEvent(event,pointer,this.hoverObj.nodes[nodeId]);delete this.hoverObj.nodes[nodeId];hoverChanged=true;}}}// removing all edge hover highlights
	for(var edgeId in this.hoverObj.edges){if(this.hoverObj.edges.hasOwnProperty(edgeId)){// if the hover has been changed here it means that the node has been hovered over or off
	// we then do not use the emitBlurEvent method here.
	if(hoverChanged===true){this.hoverObj.edges[edgeId].hover=false;delete this.hoverObj.edges[edgeId];}// if the blur remains the same and the object is undefined (mouse off) or another
	// edge has been hovered, or another node has been hovered we blur the edge.
	else if(object===undefined||object instanceof Edge&&object.id!=edgeId||object instanceof Node&&!object.hover){this.emitBlurEvent(event,pointer,this.hoverObj.edges[edgeId]);delete this.hoverObj.edges[edgeId];hoverChanged=true;}}}if(object!==undefined){var hoveredEdgesCount=keys$6(this.hoverObj.edges).length;var hoveredNodesCount=keys$6(this.hoverObj.nodes).length;var newOnlyHoveredEdge=object instanceof Edge&&hoveredEdgesCount===0&&hoveredNodesCount===0;var newOnlyHoveredNode=object instanceof Node&&hoveredEdgesCount===0&&hoveredNodesCount===0;if(hoverChanged||newOnlyHoveredEdge||newOnlyHoveredNode){hoverChanged=this.emitHoverEvent(event,pointer,object);}if(object instanceof Node&&this.options.hoverConnectedEdges===true){this._hoverConnectedEdges(object);}}if(hoverChanged===true){this.body.emitter.emit('_requestRedraw');}}/**
	   *
	   * retrieve the currently selected objects
	   * @return {{nodes: Array.<string>, edges: Array.<string>}} selection
	   */},{key:"getSelection",value:function getSelection(){var nodeIds=this.getSelectedNodes();var edgeIds=this.getSelectedEdges();return {nodes:nodeIds,edges:edgeIds};}/**
	   *
	   * retrieve the currently selected nodes
	   * @return {string[]} selection    An array with the ids of the
	   *                                            selected nodes.
	   */},{key:"getSelectedNodes",value:function getSelectedNodes(){var idArray=[];if(this.options.selectable===true){for(var nodeId in this.selectionObj.nodes){if(this.selectionObj.nodes.hasOwnProperty(nodeId)){idArray.push(this.selectionObj.nodes[nodeId].id);}}}return idArray;}/**
	   *
	   * retrieve the currently selected edges
	   * @return {Array} selection    An array with the ids of the
	   *                                            selected nodes.
	   */},{key:"getSelectedEdges",value:function getSelectedEdges(){var idArray=[];if(this.options.selectable===true){for(var edgeId in this.selectionObj.edges){if(this.selectionObj.edges.hasOwnProperty(edgeId)){idArray.push(this.selectionObj.edges[edgeId].id);}}}return idArray;}/**
	   * Updates the current selection
	   * @param {{nodes: Array.<string>, edges: Array.<string>}} selection
	   * @param {Object} options                                 Options
	   */},{key:"setSelection",value:function setSelection(selection){var options=arguments.length>1&&arguments[1]!==undefined?arguments[1]:{};var i,id;if(!selection||!selection.nodes&&!selection.edges)throw 'Selection must be an object with nodes and/or edges properties';// first unselect any selected node, if option is true or undefined
	if(options.unselectAll||options.unselectAll===undefined){this.unselectAll();}if(selection.nodes){for(i=0;i<selection.nodes.length;i++){id=selection.nodes[i];var node=this.body.nodes[id];if(!node){throw new RangeError('Node with id "'+id+'" not found');}// don't select edges with it
	this.selectObject(node,options.highlightEdges);}}if(selection.edges){for(i=0;i<selection.edges.length;i++){id=selection.edges[i];var edge=this.body.edges[id];if(!edge){throw new RangeError('Edge with id "'+id+'" not found');}this.selectObject(edge);}}this.body.emitter.emit('_requestRedraw');}/**
	   * select zero or more nodes with the option to highlight edges
	   * @param {number[] | string[]} selection     An array with the ids of the
	   *                                            selected nodes.
	   * @param {boolean} [highlightEdges]
	   */},{key:"selectNodes",value:function selectNodes(selection){var highlightEdges=arguments.length>1&&arguments[1]!==undefined?arguments[1]:true;if(!selection||selection.length===undefined)throw 'Selection must be an array with ids';this.setSelection({nodes:selection},{highlightEdges:highlightEdges});}/**
	   * select zero or more edges
	   * @param {number[] | string[]} selection     An array with the ids of the
	   *                                            selected nodes.
	   */},{key:"selectEdges",value:function selectEdges(selection){if(!selection||selection.length===undefined)throw 'Selection must be an array with ids';this.setSelection({edges:selection});}/**
	   * Validate the selection: remove ids of nodes which no longer exist
	   * @private
	   */},{key:"updateSelection",value:function updateSelection(){for(var nodeId in this.selectionObj.nodes){if(this.selectionObj.nodes.hasOwnProperty(nodeId)){if(!this.body.nodes.hasOwnProperty(nodeId)){delete this.selectionObj.nodes[nodeId];}}}for(var edgeId in this.selectionObj.edges){if(this.selectionObj.edges.hasOwnProperty(edgeId)){if(!this.body.edges.hasOwnProperty(edgeId)){delete this.selectionObj.edges[edgeId];}}}}/**
	   * Determine all the visual elements clicked which are on the given point.
	   *
	   * All elements are returned; this includes nodes, edges and their labels.
	   * The order returned is from highest to lowest, i.e. element 0 of the return
	   * value is the topmost item clicked on.
	   *
	   * The return value consists of an array of the following possible elements:
	   *
	   * - `{nodeId:number}`             - node with given id clicked on
	   * - `{nodeId:number, labelId:0}`  - label of node with given id clicked on
	   * - `{edgeId:number}`             - edge with given id clicked on
	   * - `{edge:number, labelId:0}`    - label of edge with given id clicked on
	   *
	   * ## NOTES
	   *
	   * - Currently, there is only one label associated with a node or an edge,
	   *   but this is expected to change somewhere in the future.
	   * - Since there is no z-indexing yet, it is not really possible to set the nodes and
	   *   edges in the correct order. For the time being, nodes come first.
	   *
	   * @param {point} pointer  mouse position in screen coordinates
	   * @returns {Array.<nodeClickItem|nodeLabelClickItem|edgeClickItem|edgeLabelClickItem>}
	   * @private
	   */},{key:"getClickedItems",value:function getClickedItems(pointer){var point=this.canvas.DOMtoCanvas(pointer);var items=[];// Note reverse order; we want the topmost clicked items to be first in the array
	// Also note that selected nodes are disregarded here; these normally display on top
	var nodeIndices=this.body.nodeIndices;var nodes=this.body.nodes;for(var i=nodeIndices.length-1;i>=0;i--){var node=nodes[nodeIndices[i]];var ret=node.getItemsOnPoint(point);items.push.apply(items,ret);// Append the return value to the running list.
	}var edgeIndices=this.body.edgeIndices;var edges=this.body.edges;for(var _i16=edgeIndices.length-1;_i16>=0;_i16--){var edge=edges[edgeIndices[_i16]];var _ret=edge.getItemsOnPoint(point);items.push.apply(items,_ret);// Append the return value to the running list.
	}return items;}}]);return SelectionHandler;}();/**
	 * Helper classes for LayoutEngine.
	 *
	 * Strategy pattern for usage of direction methods for hierarchical layouts.
	 */ /**
	 * Interface definition for direction strategy classes.
	 *
	 * This class describes the interface for the Strategy
	 * pattern classes used to differentiate horizontal and vertical
	 * direction of hierarchical results.
	 *
	 * For a given direction, one coordinate will be 'fixed', meaning that it is
	 * determined by level.
	 * The other coordinate is 'unfixed', meaning that the nodes on a given level
	 * can still move along that coordinate. So:
	 *
	 * - `vertical` layout: `x` unfixed, `y` fixed per level
	 * - `horizontal` layout: `x` fixed per level, `y` unfixed
	 *
	 * The local methods are stubs and should be regarded as abstract.
	 * Derived classes **must** implement all the methods themselves.
	 *
	 * @private
	 */var DirectionInterface=/*#__PURE__*/function(){function DirectionInterface(){classCallCheck(this,DirectionInterface);}createClass(DirectionInterface,[{key:"abstract",/** @ignore **/value:function abstract(){throw new Error("Can't instantiate abstract class!");}/**
	   * This is a dummy call which is used to suppress the jsdoc errors of type:
	   *
	   *   "'param' is assigned a value but never used"
	   *
	   * @ignore
	   **/},{key:"fake_use",value:function fake_use(){}// Do nothing special
	/**
	   * Type to use to translate dynamic curves to, in the case of hierarchical layout.
	   * Dynamic curves do not work for these.
	   *
	   * The value should be perpendicular to the actual direction of the layout.
	   *
	   * @return {string} Direction, either 'vertical' or 'horizontal'
	   */},{key:"curveType",value:function curveType(){return this.abstract();}/**
	   * Return the value of the coordinate that is not fixed for this direction.
	   *
	   * @param {Node} node The node to read
	   * @return {number} Value of the unfixed coordinate
	   */},{key:"getPosition",value:function getPosition(node){this.fake_use(node);return this.abstract();}/**
	   * Set the value of the coordinate that is not fixed for this direction.
	   *
	   * @param {Node} node The node to adjust
	   * @param {number} position
	   * @param {number} [level] if specified, the hierarchy level that this node should be fixed to
	   */},{key:"setPosition",value:function setPosition(node,position){var level=arguments.length>2&&arguments[2]!==undefined?arguments[2]:undefined;this.fake_use(node,position,level);this.abstract();}/**
	   * Get the width of a tree.
	   *
	   * A `tree` here is a subset of nodes within the network which are not connected to other nodes,
	   * only among themselves. In essence, it is a sub-network.
	   *
	   * @param {number} index The index number of a tree
	   * @return {number} the width of a tree in the view coordinates
	   */},{key:"getTreeSize",value:function getTreeSize(index){this.fake_use(index);return this.abstract();}/**
	   * Sort array of nodes on the unfixed coordinates.
	   *
	   * **Note:** chrome has non-stable sorting implementation, which
	   * has a tendency to change the order of the array items,
	   * even if the custom sort function returns 0.
	   *
	   * For this reason, an external sort implementation is used,
	   * which has the added benefit of being faster than the standard
	   * platforms implementation. This has been verified on `node.js`,
	   * `firefox` and `chrome` (all linux).
	   *
	   * @param {Array.<Node>} nodeArray array of nodes to sort
	   */},{key:"sort",value:function sort(nodeArray){this.fake_use(nodeArray);this.abstract();}/**
	   * Assign the fixed coordinate of the node to the given level
	   *
	   * @param {Node} node The node to adjust
	   * @param {number} level The level to fix to
	   */},{key:"fix",value:function fix(node,level){this.fake_use(node,level);this.abstract();}/**
	   * Add an offset to the unfixed coordinate of the given node. 
	   *
	   * @param {NodeId} nodeId Id of the node to adjust
	   * @param {number} diff Offset to add to the unfixed coordinate
	   */},{key:"shift",value:function shift(nodeId,diff){this.fake_use(nodeId,diff);this.abstract();}}]);return DirectionInterface;}();/**
	 * Vertical Strategy
	 *
	 * Coordinate `y` is fixed on levels, coordinate `x` is unfixed.
	 *
	 * @extends DirectionInterface
	 * @private
	 */var VerticalStrategy=/*#__PURE__*/function(_DirectionInterface){inherits(VerticalStrategy,_DirectionInterface);var _super29=_createSuper$1(VerticalStrategy);/**
	   * Constructor
	   *
	   * @param {Object} layout reference to the parent LayoutEngine instance.
	   */function VerticalStrategy(layout){var _this49;classCallCheck(this,VerticalStrategy);_this49=_super29.call(this);_this49.layout=layout;return _this49;}/** @inheritdoc */createClass(VerticalStrategy,[{key:"curveType",value:function curveType(){return 'horizontal';}/** @inheritdoc */},{key:"getPosition",value:function getPosition(node){return node.x;}/** @inheritdoc */},{key:"setPosition",value:function setPosition(node,position){var level=arguments.length>2&&arguments[2]!==undefined?arguments[2]:undefined;if(level!==undefined){this.layout.hierarchical.addToOrdering(node,level);}node.x=position;}/** @inheritdoc */},{key:"getTreeSize",value:function getTreeSize(index){var res=this.layout.hierarchical.getTreeSize(this.layout.body.nodes,index);return {min:res.min_x,max:res.max_x};}/** @inheritdoc */},{key:"sort",value:function sort(nodeArray){timsort_1(nodeArray,function(a,b){return a.x-b.x;});}/** @inheritdoc */},{key:"fix",value:function fix(node,level){node.y=this.layout.options.hierarchical.levelSeparation*level;node.options.fixed.y=true;}/** @inheritdoc */},{key:"shift",value:function shift(nodeId,diff){this.layout.body.nodes[nodeId].x+=diff;}}]);return VerticalStrategy;}(DirectionInterface);/**
	 * Horizontal Strategy
	 *
	 * Coordinate `x` is fixed on levels, coordinate `y` is unfixed.
	 *
	 * @extends DirectionInterface
	 * @private
	 */var HorizontalStrategy=/*#__PURE__*/function(_DirectionInterface2){inherits(HorizontalStrategy,_DirectionInterface2);var _super30=_createSuper$1(HorizontalStrategy);/**
	   * Constructor
	   *
	   * @param {Object} layout reference to the parent LayoutEngine instance.
	   */function HorizontalStrategy(layout){var _this50;classCallCheck(this,HorizontalStrategy);_this50=_super30.call(this);_this50.layout=layout;return _this50;}/** @inheritdoc */createClass(HorizontalStrategy,[{key:"curveType",value:function curveType(){return 'vertical';}/** @inheritdoc */},{key:"getPosition",value:function getPosition(node){return node.y;}/** @inheritdoc */},{key:"setPosition",value:function setPosition(node,position){var level=arguments.length>2&&arguments[2]!==undefined?arguments[2]:undefined;if(level!==undefined){this.layout.hierarchical.addToOrdering(node,level);}node.y=position;}/** @inheritdoc */},{key:"getTreeSize",value:function getTreeSize(index){var res=this.layout.hierarchical.getTreeSize(this.layout.body.nodes,index);return {min:res.min_y,max:res.max_y};}/** @inheritdoc */},{key:"sort",value:function sort(nodeArray){timsort_1(nodeArray,function(a,b){return a.y-b.y;});}/** @inheritdoc */},{key:"fix",value:function fix(node,level){node.x=this.layout.options.hierarchical.levelSeparation*level;node.options.fixed.x=true;}/** @inheritdoc */},{key:"shift",value:function shift(nodeId,diff){this.layout.body.nodes[nodeId].y+=diff;}}]);return HorizontalStrategy;}(DirectionInterface);/**
	 * Try to assign levels to nodes according to their positions in the cyclic “hierarchy”.
	 *
	 * @param nodes - Visible nodes of the graph.
	 * @param levels - If present levels will be added to it, if not a new object will be created.
	 *
	 * @returns Populated node levels.
	 */function fillLevelsByDirectionCyclic(nodes,levels){var edges=new set$3();forEach$3(nodes).call(nodes,function(node){var _context85;forEach$3(_context85=node.edges).call(_context85,function(edge){if(edge.connected){edges.add(edge);}});});forEach$3(edges).call(edges,function(edge){var fromId=edge.from.id;var toId=edge.to.id;if(levels[fromId]==null){levels[fromId]=0;}if(levels[toId]==null||levels[fromId]>=levels[toId]){levels[toId]=levels[fromId]+1;}});return levels;}/**
	 * Assign levels to nodes according to their positions in the hierarchy. Leaves will be lined up at the bottom and all other nodes as close to their children as possible.
	 *
	 * @param nodes - Visible nodes of the graph.
	 * @param levels - If present levels will be added to it, if not a new object will be created.
	 *
	 * @returns Populated node levels.
	 */function fillLevelsByDirectionLeaves(nodes){var levels=arguments.length>1&&arguments[1]!==undefined?arguments[1]:create$4(null);return fillLevelsByDirection(// Pick only leaves (nodes without children).
	function(node){var _context86,_context87;return every$2(_context86=filter$2(_context87=node.edges// Take only visible nodes into account.
	).call(_context87,function(edge){return nodes.has(edge.toId);})// Check that all edges lead to this node (leaf).
	).call(_context86,function(edge){return edge.to===node;});},// Use the lowest level.
	function(newLevel,oldLevel){return oldLevel>newLevel;},// Go against the direction of the edges.
	"from",nodes,levels);}/**
	 * Assign levels to nodes according to their positions in the hierarchy. Roots will be lined up at the top and all nodes as close to their parents as possible.
	 *
	 * @param nodes - Visible nodes of the graph.
	 * @param levels - If present levels will be added to it, if not a new object will be created.
	 *
	 * @returns Populated node levels.
	 */function fillLevelsByDirectionRoots(nodes){var levels=arguments.length>1&&arguments[1]!==undefined?arguments[1]:create$4(null);return fillLevelsByDirection(// Pick only roots (nodes without parents).
	function(node){var _context88,_context89;return every$2(_context88=filter$2(_context89=node.edges// Take only visible nodes into account.
	).call(_context89,function(edge){return nodes.has(edge.toId);})// Check that all edges lead from this node (root).
	).call(_context88,function(edge){return edge.from===node;});},// Use the highest level.
	function(newLevel,oldLevel){return oldLevel<newLevel;},// Go in the direction of the edges.
	"to",nodes,levels);}/**
	 * Assign levels to nodes according to their positions in the hierarchy.
	 *
	 * @param isEntryNode - Checks and return true if the graph should be traversed from this node.
	 * @param shouldLevelBeReplaced - Checks and returns true if the level of given node should be updated to the new value.
	 * @param direction - Wheter the graph should be traversed in the direction of the edges `"to"` or in the other way `"from"`.
	 * @param nodes - Visible nodes of the graph.
	 * @param levels - If present levels will be added to it, if not a new object will be created.
	 *
	 * @returns Populated node levels.
	 */function fillLevelsByDirection(isEntryNode,shouldLevelBeReplaced,direction,nodes,levels){var limit=nodes.size;var edgeIdProp=direction+"Id";var newLevelDiff=direction==="to"?1:-1;var _iterator2=_createForOfIteratorHelper$2(nodes),_step2;try{var _loop4=function _loop4(){var _step2$value=slicedToArray(_step2.value,2),entryNodeId=_step2$value[0],entryNode=_step2$value[1];if(// Skip if the node is not visible.
	!nodes.has(entryNodeId)||// Skip if the node is not an entry node.
	!isEntryNode(entryNode)){return "continue";}// Line up all the entry nodes on level 0.
	levels[entryNodeId]=0;var stack=[entryNode];var done=0;var node=void 0;var _loop5=function _loop5(){var _context90,_context91;if(!nodes.has(entryNodeId)){// Skip if the node is not visible.
	return "continue";}var newLevel=levels[node.id]+newLevelDiff;forEach$3(_context90=filter$2(_context91=node.edges).call(_context91,function(edge){return(// Ignore disconnected edges.
	edge.connected&&// Ignore circular edges.
	edge.to!==edge.from&&// Ignore edges leading to the node that's currently being processed.
	edge[direction]!==node&&// Ignore edges connecting to an invisible node.
	nodes.has(edge.toId)&&// Ignore edges connecting from an invisible node.
	nodes.has(edge.fromId));})).call(_context90,function(edge){var targetNodeId=edge[edgeIdProp];var oldLevel=levels[targetNodeId];if(oldLevel==null||shouldLevelBeReplaced(newLevel,oldLevel)){levels[targetNodeId]=newLevel;stack.push(edge[direction]);}});if(done>limit){// This would run forever on a cyclic graph.
	return {v:{v:fillLevelsByDirectionCyclic(nodes,levels)}};}else {++done;}};while(node=stack.pop()){var _ret3=_loop5();switch(_ret3){case"continue":continue;default:if(_typeof_1(_ret3)==="object")return _ret3.v;}}};for(_iterator2.s();!(_step2=_iterator2.n()).done;){var _ret2=_loop4();switch(_ret2){case"continue":continue;default:if(_typeof_1(_ret2)==="object")return _ret2.v;}}}catch(err){_iterator2.e(err);}finally{_iterator2.f();}return levels;}/**
	 * There's a mix-up with terms in the code. Following are the formal definitions:
	 *
	 *   tree   - a strict hierarchical network, i.e. every node has at most one parent
	 *   forest - a collection of trees. These distinct trees are thus not connected.
	 *
	 * So:
	 * - in a network that is not a tree, there exist nodes with multiple parents.
	 * - a network consisting of unconnected sub-networks, of which at least one
	 *   is not a tree, is not a forest.
	 *
	 * In the code, the definitions are:
	 *
	 *   tree   - any disconnected sub-network, strict hierarchical or not.
	 *   forest - a bunch of these sub-networks
	 *
	 * The difference between tree and not-tree is important in the code, notably within
	 * to the block-shifting algorithm. The algorithm assumes formal trees and fails
	 * for not-trees, often in a spectacular manner (search for 'exploding network' in the issues).
	 *
	 * In order to distinguish the definitions in the following code, the adjective 'formal' is
	 * used. If 'formal' is absent, you must assume the non-formal definition.
	 *
	 * ----------------------------------------------------------------------------------
	 * NOTES
	 * =====
	 *
	 * A hierarchical layout is a different thing from a hierarchical network.
	 * The layout is a way to arrange the nodes in the view; this can be done
	 * on non-hierarchical networks as well. The converse is also possible.
	 */ /**
	 * Container for derived data on current network, relating to hierarchy.
	 *
	 * @private
	 */var HierarchicalStatus=/*#__PURE__*/function(){/**
	   * @ignore
	   */function HierarchicalStatus(){classCallCheck(this,HierarchicalStatus);this.childrenReference={};// child id's per node id
	this.parentReference={};// parent id's per node id
	this.trees={};// tree id per node id; i.e. to which tree does given node id belong
	this.distributionOrdering={};// The nodes per level, in the display order
	this.levels={};// hierarchy level per node id
	this.distributionIndex={};// The position of the node in the level sorting order, per node id.
	this.isTree=false;// True if current network is a formal tree 
	this.treeIndex=-1;// Highest tree id in current network.
	}/**
	   * Add the relation between given nodes to the current state.
	   *
	   * @param {Node.id} parentNodeId
	   * @param {Node.id} childNodeId
	   */createClass(HierarchicalStatus,[{key:"addRelation",value:function addRelation(parentNodeId,childNodeId){if(this.childrenReference[parentNodeId]===undefined){this.childrenReference[parentNodeId]=[];}this.childrenReference[parentNodeId].push(childNodeId);if(this.parentReference[childNodeId]===undefined){this.parentReference[childNodeId]=[];}this.parentReference[childNodeId].push(parentNodeId);}/**
	   * Check if the current state is for a formal tree or formal forest.
	   *
	   * This is the case if every node has at most one parent.
	   *
	   * Pre: parentReference init'ed properly for current network
	   */},{key:"checkIfTree",value:function checkIfTree(){for(var i in this.parentReference){if(this.parentReference[i].length>1){this.isTree=false;return;}}this.isTree=true;}/**
	   * Return the number of separate trees in the current network.
	   * @returns {number}
	   */},{key:"numTrees",value:function numTrees(){return this.treeIndex+1;// This assumes the indexes are assigned consecitively
	}/**
	   * Assign a tree id to a node
	   * @param {Node} node
	   * @param {string|number} treeId
	   */},{key:"setTreeIndex",value:function setTreeIndex(node,treeId){if(treeId===undefined)return;// Don't bother
	if(this.trees[node.id]===undefined){this.trees[node.id]=treeId;this.treeIndex=Math.max(treeId,this.treeIndex);}}/**
	   * Ensure level for given id is defined.
	   *
	   * Sets level to zero for given node id if not already present
	   *
	   * @param {Node.id} nodeId
	   */},{key:"ensureLevel",value:function ensureLevel(nodeId){if(this.levels[nodeId]===undefined){this.levels[nodeId]=0;}}/**
	   * get the maximum level of a branch.
	   *
	   * TODO: Never entered; find a test case to test this!
	   * @param {Node.id} nodeId
	   * @returns {number}
	   */},{key:"getMaxLevel",value:function getMaxLevel(nodeId){var _this51=this;var accumulator={};var _getMaxLevel=function _getMaxLevel(nodeId){if(accumulator[nodeId]!==undefined){return accumulator[nodeId];}var level=_this51.levels[nodeId];if(_this51.childrenReference[nodeId]){var children=_this51.childrenReference[nodeId];if(children.length>0){for(var i=0;i<children.length;i++){level=Math.max(level,_getMaxLevel(children[i]));}}}accumulator[nodeId]=level;return level;};return _getMaxLevel(nodeId);}/**
	   *
	   * @param {Node} nodeA
	   * @param {Node} nodeB
	   */},{key:"levelDownstream",value:function levelDownstream(nodeA,nodeB){if(this.levels[nodeB.id]===undefined){// set initial level
	if(this.levels[nodeA.id]===undefined){this.levels[nodeA.id]=0;}// set level
	this.levels[nodeB.id]=this.levels[nodeA.id]+1;}}/**
	   * Small util method to set the minimum levels of the nodes to zero.
	   *
	   * @param {Array.<Node>} nodes
	   */},{key:"setMinLevelToZero",value:function setMinLevelToZero(nodes){var minLevel=1e9;// get the minimum level
	for(var nodeId in nodes){if(nodes.hasOwnProperty(nodeId)){if(this.levels[nodeId]!==undefined){minLevel=Math.min(this.levels[nodeId],minLevel);}}}// subtract the minimum from the set so we have a range starting from 0
	for(var _nodeId11 in nodes){if(nodes.hasOwnProperty(_nodeId11)){if(this.levels[_nodeId11]!==undefined){this.levels[_nodeId11]-=minLevel;}}}}/**
	   * Get the min and max xy-coordinates of a given tree
	   *
	   * @param {Array.<Node>} nodes
	   * @param {number} index
	   * @returns {{min_x: number, max_x: number, min_y: number, max_y: number}}
	   */},{key:"getTreeSize",value:function getTreeSize(nodes,index){var min_x=1e9;var max_x=-1e9;var min_y=1e9;var max_y=-1e9;for(var nodeId in this.trees){if(this.trees.hasOwnProperty(nodeId)){if(this.trees[nodeId]===index){var node=nodes[nodeId];min_x=Math.min(node.x,min_x);max_x=Math.max(node.x,max_x);min_y=Math.min(node.y,min_y);max_y=Math.max(node.y,max_y);}}}return {min_x:min_x,max_x:max_x,min_y:min_y,max_y:max_y};}/**
	   * Check if two nodes have the same parent(s)
	   *
	   * @param {Node} node1
	   * @param {Node} node2
	   * @return {boolean} true if the two nodes have a same ancestor node, false otherwise
	   */},{key:"hasSameParent",value:function hasSameParent(node1,node2){var parents1=this.parentReference[node1.id];var parents2=this.parentReference[node2.id];if(parents1===undefined||parents2===undefined){return false;}for(var i=0;i<parents1.length;i++){for(var j=0;j<parents2.length;j++){if(parents1[i]==parents2[j]){return true;}}}return false;}/**
	   * Check if two nodes are in the same tree.
	   *
	   * @param {Node} node1
	   * @param {Node} node2
	   * @return {Boolean} true if this is so, false otherwise
	   */},{key:"inSameSubNetwork",value:function inSameSubNetwork(node1,node2){return this.trees[node1.id]===this.trees[node2.id];}/**
	   * Get a list of the distinct levels in the current network
	   *
	   * @returns {Array}
	   */},{key:"getLevels",value:function getLevels(){return keys$6(this.distributionOrdering);}/**
	   * Add a node to the ordering per level
	   *
	   * @param {Node} node
	   * @param {number} level
	   */},{key:"addToOrdering",value:function addToOrdering(node,level){if(this.distributionOrdering[level]===undefined){this.distributionOrdering[level]=[];}var isPresent=false;var curLevel=this.distributionOrdering[level];for(var n in curLevel){//if (curLevel[n].id === node.id) {
	if(curLevel[n]===node){isPresent=true;break;}}if(!isPresent){this.distributionOrdering[level].push(node);this.distributionIndex[node.id]=this.distributionOrdering[level].length-1;}}}]);return HierarchicalStatus;}();/**
	 * The Layout Engine
	 */var LayoutEngine=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   */function LayoutEngine(body){classCallCheck(this,LayoutEngine);this.body=body;// Make sure there always is some RNG because the setOptions method won't
	// set it unless there's a seed for it.
	this._resetRNG(Math.random()+":"+now$2());this.setPhysics=false;this.options={};this.optionsBackup={physics:{}};this.defaultOptions={randomSeed:undefined,improvedLayout:true,clusterThreshold:150,hierarchical:{enabled:false,levelSeparation:150,nodeSpacing:100,treeSpacing:200,blockShifting:true,edgeMinimization:true,parentCentralization:true,direction:'UD',// UD, DU, LR, RL
	sortMethod:'hubsize'// hubsize, directed
	}};extend(this.options,this.defaultOptions);this.bindEventListeners();}/**
	   * Binds event listeners
	   */createClass(LayoutEngine,[{key:"bindEventListeners",value:function bindEventListeners(){var _this52=this;this.body.emitter.on('_dataChanged',function(){_this52.setupHierarchicalLayout();});this.body.emitter.on('_dataLoaded',function(){_this52.layoutNetwork();});this.body.emitter.on('_resetHierarchicalLayout',function(){_this52.setupHierarchicalLayout();});this.body.emitter.on('_adjustEdgesForHierarchicalLayout',function(){if(_this52.options.hierarchical.enabled!==true){return;}// get the type of static smooth curve in case it is required
	var type=_this52.direction.curveType();// force all edges into static smooth curves.
	_this52.body.emitter.emit('_forceDisableDynamicCurves',type,false);});}/**
	   *
	   * @param {Object} options
	   * @param {Object} allOptions
	   * @returns {Object}
	   */},{key:"setOptions",value:function setOptions(options,allOptions){if(options!==undefined){var hierarchical=this.options.hierarchical;var prevHierarchicalState=hierarchical.enabled;selectiveDeepExtend(["randomSeed","improvedLayout","clusterThreshold"],this.options,options);mergeOptions(this.options,options,'hierarchical');if(options.randomSeed!==undefined){this._resetRNG(options.randomSeed);}if(hierarchical.enabled===true){if(prevHierarchicalState===true){// refresh the overridden options for nodes and edges.
	this.body.emitter.emit('refresh',true);}// make sure the level separation is the right way up
	if(hierarchical.direction==='RL'||hierarchical.direction==='DU'){if(hierarchical.levelSeparation>0){hierarchical.levelSeparation*=-1;}}else {if(hierarchical.levelSeparation<0){hierarchical.levelSeparation*=-1;}}this.setDirectionStrategy();this.body.emitter.emit('_resetHierarchicalLayout');// because the hierarchical system needs it's own physics and smooth curve settings,
	// we adapt the other options if needed.
	return this.adaptAllOptionsForHierarchicalLayout(allOptions);}else {if(prevHierarchicalState===true){// refresh the overridden options for nodes and edges.
	this.body.emitter.emit('refresh');return deepExtend(allOptions,this.optionsBackup);}}}return allOptions;}/**
	   * Reset the random number generator with given seed.
	   *
	   * @param {any} seed - The seed that will be forwarded the the RNG.
	   */},{key:"_resetRNG",value:function _resetRNG(seed){this.initialRandomSeed=seed;this._rng=Alea(this.initialRandomSeed);}/**
	   *
	   * @param {Object} allOptions
	   * @returns {Object}
	   */},{key:"adaptAllOptionsForHierarchicalLayout",value:function adaptAllOptionsForHierarchicalLayout(allOptions){if(this.options.hierarchical.enabled===true){var backupPhysics=this.optionsBackup.physics;// set the physics
	if(allOptions.physics===undefined||allOptions.physics===true){allOptions.physics={enabled:backupPhysics.enabled===undefined?true:backupPhysics.enabled,solver:'hierarchicalRepulsion'};backupPhysics.enabled=backupPhysics.enabled===undefined?true:backupPhysics.enabled;backupPhysics.solver=backupPhysics.solver||'barnesHut';}else if(_typeof_1(allOptions.physics)==='object'){backupPhysics.enabled=allOptions.physics.enabled===undefined?true:allOptions.physics.enabled;backupPhysics.solver=allOptions.physics.solver||'barnesHut';allOptions.physics.solver='hierarchicalRepulsion';}else if(allOptions.physics!==false){backupPhysics.solver='barnesHut';allOptions.physics={solver:'hierarchicalRepulsion'};}// get the type of static smooth curve in case it is required
	var type=this.direction.curveType();// disable smooth curves if nothing is defined. If smooth curves have been turned on,
	// turn them into static smooth curves.
	if(allOptions.edges===undefined){this.optionsBackup.edges={smooth:{enabled:true,type:'dynamic'}};allOptions.edges={smooth:false};}else if(allOptions.edges.smooth===undefined){this.optionsBackup.edges={smooth:{enabled:true,type:'dynamic'}};allOptions.edges.smooth=false;}else {if(typeof allOptions.edges.smooth==='boolean'){this.optionsBackup.edges={smooth:allOptions.edges.smooth};allOptions.edges.smooth={enabled:allOptions.edges.smooth,type:type};}else {var smooth=allOptions.edges.smooth;// allow custom types except for dynamic
	if(smooth.type!==undefined&&smooth.type!=='dynamic'){type=smooth.type;}// TODO: this is options merging; see if the standard routines can be used here.
	this.optionsBackup.edges={smooth:{enabled:smooth.enabled===undefined?true:smooth.enabled,type:smooth.type===undefined?'dynamic':smooth.type,roundness:smooth.roundness===undefined?0.5:smooth.roundness,forceDirection:smooth.forceDirection===undefined?false:smooth.forceDirection}};// NOTE: Copying an object to self; this is basically setting defaults for undefined variables
	allOptions.edges.smooth={enabled:smooth.enabled===undefined?true:smooth.enabled,type:type,roundness:smooth.roundness===undefined?0.5:smooth.roundness,forceDirection:smooth.forceDirection===undefined?false:smooth.forceDirection};}}// Force all edges into static smooth curves.
	// Only applies to edges that do not use the global options for smooth.
	this.body.emitter.emit('_forceDisableDynamicCurves',type);}return allOptions;}/**
	   *
	   * @param {Array.<Node>} nodesArray
	   */},{key:"positionInitially",value:function positionInitially(nodesArray){if(this.options.hierarchical.enabled!==true){this._resetRNG(this.initialRandomSeed);var radius=nodesArray.length+50;for(var i=0;i<nodesArray.length;i++){var node=nodesArray[i];var angle=2*Math.PI*this._rng();if(node.x===undefined){node.x=radius*Math.cos(angle);}if(node.y===undefined){node.y=radius*Math.sin(angle);}}}}/**
	   * Use Kamada Kawai to position nodes. This is quite a heavy algorithm so if there are a lot of nodes we
	   * cluster them first to reduce the amount.
	   */},{key:"layoutNetwork",value:function layoutNetwork(){if(this.options.hierarchical.enabled!==true&&this.options.improvedLayout===true){var indices=this.body.nodeIndices;// first check if we should Kamada Kawai to layout. The threshold is if less than half of the visible
	// nodes have predefined positions we use this.
	var positionDefined=0;for(var i=0;i<indices.length;i++){var node=this.body.nodes[indices[i]];if(node.predefinedPosition===true){positionDefined+=1;}}// if less than half of the nodes have a predefined position we continue
	if(positionDefined<0.5*indices.length){var MAX_LEVELS=10;var level=0;var clusterThreshold=this.options.clusterThreshold;//
	// Define the options for the hidden cluster nodes
	// These options don't propagate outside the clustering phase.
	//
	// Some options are explicitly disabled, because they may be set in group or default node options.
	// The clusters are never displayed, so most explicit settings here serve as performance optimizations.
	//
	// The explicit setting of 'shape' is to avoid `shape: 'image'`; images are not passed to the hidden
	// cluster nodes, leading to an exception on creation.
	//
	// All settings here are performance related, except when noted otherwise.
	//
	var clusterOptions={clusterNodeProperties:{shape:'ellipse',// Bugfix: avoid type 'image', no images supplied
	label:'',// avoid label handling
	group:'',// avoid group handling
	font:{multi:false}// avoid font propagation
	},clusterEdgeProperties:{label:'',// avoid label handling
	font:{multi:false},// avoid font propagation
	smooth:{enabled:false// avoid drawing penalty for complex edges
	}}};// if there are a lot of nodes, we cluster before we run the algorithm.
	// NOTE: this part fails to find clusters for large scale-free networks, which should
	//       be easily clusterable.
	// TODO: examine why this is so
	if(indices.length>clusterThreshold){var startLength=indices.length;while(indices.length>clusterThreshold&&level<=MAX_LEVELS){//console.time("clustering")
	level+=1;var before=indices.length;// if there are many nodes we do a hubsize cluster
	if(level%3===0){this.body.modules.clustering.clusterBridges(clusterOptions);}else {this.body.modules.clustering.clusterOutliers(clusterOptions);}var after=indices.length;if(before==after&&level%3!==0){this._declusterAll();this.body.emitter.emit("_layoutFailed");console.info("This network could not be positioned by this version of the improved layout algorithm."+" Please disable improvedLayout for better performance.");return;}//console.timeEnd("clustering")
	//console.log(before,level,after);
	}// increase the size of the edges
	this.body.modules.kamadaKawai.setOptions({springLength:Math.max(150,2*startLength)});}if(level>MAX_LEVELS){console.info("The clustering didn't succeed within the amount of interations allowed,"+" progressing with partial result.");}// position the system for these nodes and edges
	this.body.modules.kamadaKawai.solve(indices,this.body.edgeIndices,true);// shift to center point
	this._shiftToCenter();// perturb the nodes a little bit to force the physics to kick in
	var offset=70;for(var _i17=0;_i17<indices.length;_i17++){// Only perturb the nodes that aren't fixed
	var _node4=this.body.nodes[indices[_i17]];if(_node4.predefinedPosition===false){_node4.x+=(0.5-this._rng())*offset;_node4.y+=(0.5-this._rng())*offset;}}// uncluster all clusters
	this._declusterAll();// reposition all bezier nodes.
	this.body.emitter.emit("_repositionBezierNodes");}}}/**
	   * Move all the nodes towards to the center so gravitational pull wil not move the nodes away from view
	   * @private
	   */},{key:"_shiftToCenter",value:function _shiftToCenter(){var range=NetworkUtil.getRangeCore(this.body.nodes,this.body.nodeIndices);var center=NetworkUtil.findCenter(range);for(var i=0;i<this.body.nodeIndices.length;i++){var node=this.body.nodes[this.body.nodeIndices[i]];node.x-=center.x;node.y-=center.y;}}/**
	   * Expands all clusters
	   * @private
	   */},{key:"_declusterAll",value:function _declusterAll(){var clustersPresent=true;while(clustersPresent===true){clustersPresent=false;for(var i=0;i<this.body.nodeIndices.length;i++){if(this.body.nodes[this.body.nodeIndices[i]].isCluster===true){clustersPresent=true;this.body.modules.clustering.openCluster(this.body.nodeIndices[i],{},false);}}if(clustersPresent===true){this.body.emitter.emit('_dataChanged');}}}/**
	   *
	   * @returns {number|*}
	   */},{key:"getSeed",value:function getSeed(){return this.initialRandomSeed;}/**
	   * This is the main function to layout the nodes in a hierarchical way.
	   * It checks if the node details are supplied correctly
	   *
	   * @private
	   */},{key:"setupHierarchicalLayout",value:function setupHierarchicalLayout(){if(this.options.hierarchical.enabled===true&&this.body.nodeIndices.length>0){// get the size of the largest hubs and check if the user has defined a level for a node.
	var node,nodeId;var definedLevel=false;var undefinedLevel=false;this.lastNodeOnLevel={};this.hierarchical=new HierarchicalStatus();for(nodeId in this.body.nodes){if(this.body.nodes.hasOwnProperty(nodeId)){node=this.body.nodes[nodeId];if(node.options.level!==undefined){definedLevel=true;this.hierarchical.levels[nodeId]=node.options.level;}else {undefinedLevel=true;}}}// if the user defined some levels but not all, alert and run without hierarchical layout
	if(undefinedLevel===true&&definedLevel===true){throw new Error('To use the hierarchical layout, nodes require either no predefined levels'+' or levels have to be defined for all nodes.');}else {// define levels if undefined by the users. Based on hubsize.
	if(undefinedLevel===true){var sortMethod=this.options.hierarchical.sortMethod;if(sortMethod==='hubsize'){this._determineLevelsByHubsize();}else if(sortMethod==='directed'){this._determineLevelsDirected();}else if(sortMethod==='custom'){this._determineLevelsCustomCallback();}}// fallback for cases where there are nodes but no edges
	for(var _nodeId12 in this.body.nodes){if(this.body.nodes.hasOwnProperty(_nodeId12)){this.hierarchical.ensureLevel(_nodeId12);}}// check the distribution of the nodes per level.
	var distribution=this._getDistribution();// get the parent children relations.
	this._generateMap();// place the nodes on the canvas.
	this._placeNodesByHierarchy(distribution);// condense the whitespace.
	this._condenseHierarchy();// shift to center so gravity does not have to do much
	this._shiftToCenter();}}}/**
	   * @private
	   */},{key:"_condenseHierarchy",value:function _condenseHierarchy(){var _this53=this;// Global var in this scope to define when the movement has stopped.
	var stillShifting=false;var branches={};// first we have some methods to help shifting trees around.
	// the main method to shift the trees
	var shiftTrees=function shiftTrees(){var treeSizes=getTreeSizes();var shiftBy=0;for(var i=0;i<treeSizes.length-1;i++){var diff=treeSizes[i].max-treeSizes[i+1].min;shiftBy+=diff+_this53.options.hierarchical.treeSpacing;shiftTree(i+1,shiftBy);}};// shift a single tree by an offset
	var shiftTree=function shiftTree(index,offset){var trees=_this53.hierarchical.trees;for(var nodeId in trees){if(trees.hasOwnProperty(nodeId)){if(trees[nodeId]===index){_this53.direction.shift(nodeId,offset);}}}};// get the width of all trees
	var getTreeSizes=function getTreeSizes(){var treeWidths=[];for(var i=0;i<_this53.hierarchical.numTrees();i++){treeWidths.push(_this53.direction.getTreeSize(i));}return treeWidths;};// get a map of all nodes in this branch
	var getBranchNodes=function getBranchNodes(source,map){if(map[source.id]){return;}map[source.id]=true;if(_this53.hierarchical.childrenReference[source.id]){var children=_this53.hierarchical.childrenReference[source.id];if(children.length>0){for(var i=0;i<children.length;i++){getBranchNodes(_this53.body.nodes[children[i]],map);}}}};// get a min max width as well as the maximum movement space it has on either sides
	// we use min max terminology because width and height can interchange depending on the direction of the layout
	var getBranchBoundary=function getBranchBoundary(branchMap){var maxLevel=arguments.length>1&&arguments[1]!==undefined?arguments[1]:1e9;var minSpace=1e9;var maxSpace=1e9;var min=1e9;var max=-1e9;for(var branchNode in branchMap){if(branchMap.hasOwnProperty(branchNode)){var node=_this53.body.nodes[branchNode];var level=_this53.hierarchical.levels[node.id];var position=_this53.direction.getPosition(node);// get the space around the node.
	var _this53$_getSpaceArou=_this53._getSpaceAroundNode(node,branchMap),_this53$_getSpaceArou2=slicedToArray(_this53$_getSpaceArou,2),minSpaceNode=_this53$_getSpaceArou2[0],maxSpaceNode=_this53$_getSpaceArou2[1];minSpace=Math.min(minSpaceNode,minSpace);maxSpace=Math.min(maxSpaceNode,maxSpace);// the width is only relevant for the levels two nodes have in common. This is why we filter on this.
	if(level<=maxLevel){min=Math.min(position,min);max=Math.max(position,max);}}}return [min,max,minSpace,maxSpace];};// check what the maximum level is these nodes have in common.
	var getCollisionLevel=function getCollisionLevel(node1,node2){var maxLevel1=_this53.hierarchical.getMaxLevel(node1.id);var maxLevel2=_this53.hierarchical.getMaxLevel(node2.id);return Math.min(maxLevel1,maxLevel2);};/**
	     * Condense elements. These can be nodes or branches depending on the callback.
	     *
	     * @param {function} callback
	     * @param {Array.<number>} levels
	     * @param {*} centerParents
	     */var shiftElementsCloser=function shiftElementsCloser(callback,levels,centerParents){var hier=_this53.hierarchical;for(var i=0;i<levels.length;i++){var level=levels[i];var levelNodes=hier.distributionOrdering[level];if(levelNodes.length>1){for(var j=0;j<levelNodes.length-1;j++){var node1=levelNodes[j];var node2=levelNodes[j+1];// NOTE: logic maintained as it was; if nodes have same ancestor,
	//       then of course they are in the same sub-network.
	if(hier.hasSameParent(node1,node2)&&hier.inSameSubNetwork(node1,node2)){callback(node1,node2,centerParents);}}}}};// callback for shifting branches
	var branchShiftCallback=function branchShiftCallback(node1,node2){var centerParent=arguments.length>2&&arguments[2]!==undefined?arguments[2]:false;//window.CALLBACKS.push(() => {
	var pos1=_this53.direction.getPosition(node1);var pos2=_this53.direction.getPosition(node2);var diffAbs=Math.abs(pos2-pos1);var nodeSpacing=_this53.options.hierarchical.nodeSpacing;//console.log("NOW CHECKING:", node1.id, node2.id, diffAbs);
	if(diffAbs>nodeSpacing){var branchNodes1={};var branchNodes2={};getBranchNodes(node1,branchNodes1);getBranchNodes(node2,branchNodes2);// check the largest distance between the branches
	var maxLevel=getCollisionLevel(node1,node2);var branchNodeBoundary1=getBranchBoundary(branchNodes1,maxLevel);var branchNodeBoundary2=getBranchBoundary(branchNodes2,maxLevel);var max1=branchNodeBoundary1[1];var min2=branchNodeBoundary2[0];var minSpace2=branchNodeBoundary2[2];//console.log(node1.id, getBranchBoundary(branchNodes1, maxLevel), node2.id,
	//            getBranchBoundary(branchNodes2, maxLevel), maxLevel);
	var diffBranch=Math.abs(max1-min2);if(diffBranch>nodeSpacing){var offset=max1-min2+nodeSpacing;if(offset<-minSpace2+nodeSpacing){offset=-minSpace2+nodeSpacing;//console.log("RESETTING OFFSET", max1 - min2 + this.options.hierarchical.nodeSpacing, -minSpace2, offset);
	}if(offset<0){//console.log("SHIFTING", node2.id, offset);
	_this53._shiftBlock(node2.id,offset);stillShifting=true;if(centerParent===true)_this53._centerParent(node2);}}}//this.body.emitter.emit("_redraw");})
	};var minimizeEdgeLength=function minimizeEdgeLength(iterations,node){//window.CALLBACKS.push(() => {
	//  console.log("ts",node.id);
	var nodeId=node.id;var allEdges=node.edges;var nodeLevel=_this53.hierarchical.levels[node.id];// gather constants
	var C2=_this53.options.hierarchical.levelSeparation*_this53.options.hierarchical.levelSeparation;var referenceNodes={};var aboveEdges=[];for(var i=0;i<allEdges.length;i++){var edge=allEdges[i];if(edge.toId!=edge.fromId){var otherNode=edge.toId==nodeId?edge.from:edge.to;referenceNodes[allEdges[i].id]=otherNode;if(_this53.hierarchical.levels[otherNode.id]<nodeLevel){aboveEdges.push(edge);}}}// differentiated sum of lengths based on only moving one node over one axis
	var getFx=function getFx(point,edges){var sum=0;for(var _i18=0;_i18<edges.length;_i18++){if(referenceNodes[edges[_i18].id]!==undefined){var a=_this53.direction.getPosition(referenceNodes[edges[_i18].id])-point;sum+=a/Math.sqrt(a*a+C2);}}return sum;};// doubly differentiated sum of lengths based on only moving one node over one axis
	var getDFx=function getDFx(point,edges){var sum=0;for(var _i19=0;_i19<edges.length;_i19++){if(referenceNodes[edges[_i19].id]!==undefined){var a=_this53.direction.getPosition(referenceNodes[edges[_i19].id])-point;sum-=C2*Math.pow(a*a+C2,-1.5);}}return sum;};var getGuess=function getGuess(iterations,edges){var guess=_this53.direction.getPosition(node);// Newton's method for optimization
	var guessMap={};for(var _i20=0;_i20<iterations;_i20++){var fx=getFx(guess,edges);var dfx=getDFx(guess,edges);// we limit the movement to avoid instability.
	var limit=40;var ratio=Math.max(-limit,Math.min(limit,Math.round(fx/dfx)));guess=guess-ratio;// reduce duplicates
	if(guessMap[guess]!==undefined){break;}guessMap[guess]=_i20;}return guess;};var moveBranch=function moveBranch(guess){// position node if there is space
	var nodePosition=_this53.direction.getPosition(node);// check movable area of the branch
	if(branches[node.id]===undefined){var branchNodes={};getBranchNodes(node,branchNodes);branches[node.id]=branchNodes;}var branchBoundary=getBranchBoundary(branches[node.id]);var minSpaceBranch=branchBoundary[2];var maxSpaceBranch=branchBoundary[3];var diff=guess-nodePosition;// check if we are allowed to move the node:
	var branchOffset=0;if(diff>0){branchOffset=Math.min(diff,maxSpaceBranch-_this53.options.hierarchical.nodeSpacing);}else if(diff<0){branchOffset=-Math.min(-diff,minSpaceBranch-_this53.options.hierarchical.nodeSpacing);}if(branchOffset!=0){//console.log("moving branch:",branchOffset, maxSpaceBranch, minSpaceBranch)
	_this53._shiftBlock(node.id,branchOffset);//this.body.emitter.emit("_redraw");
	stillShifting=true;}};var moveNode=function moveNode(guess){var nodePosition=_this53.direction.getPosition(node);// position node if there is space
	var _this53$_getSpaceArou3=_this53._getSpaceAroundNode(node),_this53$_getSpaceArou4=slicedToArray(_this53$_getSpaceArou3,2),minSpace=_this53$_getSpaceArou4[0],maxSpace=_this53$_getSpaceArou4[1];var diff=guess-nodePosition;// check if we are allowed to move the node:
	var newPosition=nodePosition;if(diff>0){newPosition=Math.min(nodePosition+(maxSpace-_this53.options.hierarchical.nodeSpacing),guess);}else if(diff<0){newPosition=Math.max(nodePosition-(minSpace-_this53.options.hierarchical.nodeSpacing),guess);}if(newPosition!==nodePosition){//console.log("moving Node:",diff, minSpace, maxSpace);
	_this53.direction.setPosition(node,newPosition);//this.body.emitter.emit("_redraw");
	stillShifting=true;}};var guess=getGuess(iterations,aboveEdges);moveBranch(guess);guess=getGuess(iterations,allEdges);moveNode(guess);//})
	};// method to remove whitespace between branches. Because we do bottom up, we can center the parents.
	var minimizeEdgeLengthBottomUp=function minimizeEdgeLengthBottomUp(iterations){var levels=_this53.hierarchical.getLevels();levels=reverse$2(levels).call(levels);for(var i=0;i<iterations;i++){stillShifting=false;for(var j=0;j<levels.length;j++){var level=levels[j];var levelNodes=_this53.hierarchical.distributionOrdering[level];for(var k=0;k<levelNodes.length;k++){minimizeEdgeLength(1000,levelNodes[k]);}}if(stillShifting!==true){//console.log("FINISHED minimizeEdgeLengthBottomUp IN " + i);
	break;}}};// method to remove whitespace between branches. Because we do bottom up, we can center the parents.
	var shiftBranchesCloserBottomUp=function shiftBranchesCloserBottomUp(iterations){var levels=_this53.hierarchical.getLevels();levels=reverse$2(levels).call(levels);for(var i=0;i<iterations;i++){stillShifting=false;shiftElementsCloser(branchShiftCallback,levels,true);if(stillShifting!==true){//console.log("FINISHED shiftBranchesCloserBottomUp IN " + (i+1));
	break;}}};// center all parents
	var centerAllParents=function centerAllParents(){for(var nodeId in _this53.body.nodes){if(_this53.body.nodes.hasOwnProperty(nodeId))_this53._centerParent(_this53.body.nodes[nodeId]);}};// center all parents
	var centerAllParentsBottomUp=function centerAllParentsBottomUp(){var levels=_this53.hierarchical.getLevels();levels=reverse$2(levels).call(levels);for(var i=0;i<levels.length;i++){var level=levels[i];var levelNodes=_this53.hierarchical.distributionOrdering[level];for(var j=0;j<levelNodes.length;j++){_this53._centerParent(levelNodes[j]);}}};// the actual work is done here.
	if(this.options.hierarchical.blockShifting===true){shiftBranchesCloserBottomUp(5);centerAllParents();}// minimize edge length
	if(this.options.hierarchical.edgeMinimization===true){minimizeEdgeLengthBottomUp(20);}if(this.options.hierarchical.parentCentralization===true){centerAllParentsBottomUp();}shiftTrees();}/**
	   * This gives the space around the node. IF a map is supplied, it will only check against nodes NOT in the map.
	   * This is used to only get the distances to nodes outside of a branch.
	   * @param {Node} node
	   * @param {{Node.id: vis.Node}} map
	   * @returns {number[]}
	   * @private
	   */},{key:"_getSpaceAroundNode",value:function _getSpaceAroundNode(node,map){var useMap=true;if(map===undefined){useMap=false;}var level=this.hierarchical.levels[node.id];if(level!==undefined){var _index4=this.hierarchical.distributionIndex[node.id];var position=this.direction.getPosition(node);var ordering=this.hierarchical.distributionOrdering[level];var minSpace=1e9;var maxSpace=1e9;if(_index4!==0){var prevNode=ordering[_index4-1];if(useMap===true&&map[prevNode.id]===undefined||useMap===false){var prevPos=this.direction.getPosition(prevNode);minSpace=position-prevPos;}}if(_index4!=ordering.length-1){var nextNode=ordering[_index4+1];if(useMap===true&&map[nextNode.id]===undefined||useMap===false){var nextPos=this.direction.getPosition(nextNode);maxSpace=Math.min(maxSpace,nextPos-position);}}return [minSpace,maxSpace];}else {return [0,0];}}/**
	   * We use this method to center a parent node and check if it does not cross other nodes when it does.
	   * @param {Node} node
	   * @private
	   */},{key:"_centerParent",value:function _centerParent(node){if(this.hierarchical.parentReference[node.id]){var parents=this.hierarchical.parentReference[node.id];for(var i=0;i<parents.length;i++){var parentId=parents[i];var parentNode=this.body.nodes[parentId];var children=this.hierarchical.childrenReference[parentId];if(children!==undefined){// get the range of the children
	var newPosition=this._getCenterPosition(children);var position=this.direction.getPosition(parentNode);var _this$_getSpaceAround=this._getSpaceAroundNode(parentNode),_this$_getSpaceAround2=slicedToArray(_this$_getSpaceAround,2),minSpace=_this$_getSpaceAround2[0],maxSpace=_this$_getSpaceAround2[1];var diff=position-newPosition;if(diff<0&&Math.abs(diff)<maxSpace-this.options.hierarchical.nodeSpacing||diff>0&&Math.abs(diff)<minSpace-this.options.hierarchical.nodeSpacing){this.direction.setPosition(parentNode,newPosition);}}}}}/**
	   * This function places the nodes on the canvas based on the hierarchial distribution.
	   *
	   * @param {Object} distribution | obtained by the function this._getDistribution()
	   * @private
	   */},{key:"_placeNodesByHierarchy",value:function _placeNodesByHierarchy(distribution){this.positionedNodes={};// start placing all the level 0 nodes first. Then recursively position their branches.
	for(var level in distribution){if(distribution.hasOwnProperty(level)){var _context92;// sort nodes in level by position:
	var nodeArray=keys$6(distribution[level]);nodeArray=this._indexArrayToNodes(nodeArray);sort$2(_context92=this.direction).call(_context92,nodeArray);var handledNodeCount=0;for(var i=0;i<nodeArray.length;i++){var node=nodeArray[i];if(this.positionedNodes[node.id]===undefined){var spacing=this.options.hierarchical.nodeSpacing;var pos=spacing*handledNodeCount;// We get the X or Y values we need and store them in pos and previousPos.
	// The get and set make sure we get X or Y
	if(handledNodeCount>0){pos=this.direction.getPosition(nodeArray[i-1])+spacing;}this.direction.setPosition(node,pos,level);this._validatePositionAndContinue(node,level,pos);handledNodeCount++;}}}}}/**
	   * This is a recursively called function to enumerate the branches from the largest hubs and place the nodes
	   * on a X position that ensures there will be no overlap.
	   *
	   * @param {Node.id} parentId
	   * @param {number} parentLevel
	   * @private
	   */},{key:"_placeBranchNodes",value:function _placeBranchNodes(parentId,parentLevel){var _context93;var childRef=this.hierarchical.childrenReference[parentId];// if this is not a parent, cancel the placing. This can happen with multiple parents to one child.
	if(childRef===undefined){return;}// get a list of childNodes
	var childNodes=[];for(var i=0;i<childRef.length;i++){childNodes.push(this.body.nodes[childRef[i]]);}// use the positions to order the nodes.
	sort$2(_context93=this.direction).call(_context93,childNodes);// position the childNodes
	for(var _i21=0;_i21<childNodes.length;_i21++){var childNode=childNodes[_i21];var childNodeLevel=this.hierarchical.levels[childNode.id];// check if the child node is below the parent node and if it has already been positioned.
	if(childNodeLevel>parentLevel&&this.positionedNodes[childNode.id]===undefined){// get the amount of space required for this node. If parent the width is based on the amount of children.
	var spacing=this.options.hierarchical.nodeSpacing;var pos=void 0;// we get the X or Y values we need and store them in pos and previousPos.
	// The get and set make sure we get X or Y
	if(_i21===0){pos=this.direction.getPosition(this.body.nodes[parentId]);}else {pos=this.direction.getPosition(childNodes[_i21-1])+spacing;}this.direction.setPosition(childNode,pos,childNodeLevel);this._validatePositionAndContinue(childNode,childNodeLevel,pos);}else {return;}}// center the parent nodes.
	var center=this._getCenterPosition(childNodes);this.direction.setPosition(this.body.nodes[parentId],center,parentLevel);}/**
	   * This method checks for overlap and if required shifts the branch. It also keeps records of positioned nodes.
	   * Finally it will call _placeBranchNodes to place the branch nodes.
	   * @param {Node} node
	   * @param {number} level
	   * @param {number} pos
	   * @private
	   */},{key:"_validatePositionAndContinue",value:function _validatePositionAndContinue(node,level,pos){// This method only works for formal trees and formal forests
	// Early exit if this is not the case
	if(!this.hierarchical.isTree)return;// if overlap has been detected, we shift the branch
	if(this.lastNodeOnLevel[level]!==undefined){var previousPos=this.direction.getPosition(this.body.nodes[this.lastNodeOnLevel[level]]);if(pos-previousPos<this.options.hierarchical.nodeSpacing){var diff=previousPos+this.options.hierarchical.nodeSpacing-pos;var sharedParent=this._findCommonParent(this.lastNodeOnLevel[level],node.id);this._shiftBlock(sharedParent.withChild,diff);}}this.lastNodeOnLevel[level]=node.id;// store change in position.
	this.positionedNodes[node.id]=true;this._placeBranchNodes(node.id,level);}/**
	   * Receives an array with node indices and returns an array with the actual node references.
	   * Used for sorting based on node properties.
	   * @param {Array.<Node.id>} idArray
	   * @returns {Array.<Node>}
	   */},{key:"_indexArrayToNodes",value:function _indexArrayToNodes(idArray){var array=[];for(var i=0;i<idArray.length;i++){array.push(this.body.nodes[idArray[i]]);}return array;}/**
	   * This function get the distribution of levels based on hubsize
	   *
	   * @returns {Object}
	   * @private
	   */},{key:"_getDistribution",value:function _getDistribution(){var distribution={};var nodeId,node;// we fix Y because the hierarchy is vertical,
	// we fix X so we do not give a node an x position for a second time.
	// the fix of X is removed after the x value has been set.
	for(nodeId in this.body.nodes){if(this.body.nodes.hasOwnProperty(nodeId)){node=this.body.nodes[nodeId];var level=this.hierarchical.levels[nodeId]===undefined?0:this.hierarchical.levels[nodeId];this.direction.fix(node,level);if(distribution[level]===undefined){distribution[level]={};}distribution[level][nodeId]=node;}}return distribution;}/**
	   * Return the active (i.e. visible) edges for this node
	   *
	   * @param {Node} node
	   * @returns {Array.<vis.Edge>} Array of edge instances
	   * @private
	   */},{key:"_getActiveEdges",value:function _getActiveEdges(node){var _this54=this;var result=[];forEach$4(node.edges,function(edge){var _context94;if(indexOf$3(_context94=_this54.body.edgeIndices).call(_context94,edge.id)!==-1){result.push(edge);}});return result;}/**
	   * Get the hubsizes for all active nodes.
	   *
	   * @returns {number}
	   * @private
	   */},{key:"_getHubSizes",value:function _getHubSizes(){var _this55=this;var hubSizes={};var nodeIds=this.body.nodeIndices;forEach$4(nodeIds,function(nodeId){var node=_this55.body.nodes[nodeId];var hubSize=_this55._getActiveEdges(node).length;hubSizes[hubSize]=true;});// Make an array of the size sorted descending
	var result=[];forEach$4(hubSizes,function(size){result.push(Number(size));});sort$2(timsort$1).call(timsort$1,result,function(a,b){return b-a;});return result;}/**
	   * this function allocates nodes in levels based on the recursive branching from the largest hubs.
	   *
	   * @private
	   */},{key:"_determineLevelsByHubsize",value:function _determineLevelsByHubsize(){var _this56=this;var levelDownstream=function levelDownstream(nodeA,nodeB){_this56.hierarchical.levelDownstream(nodeA,nodeB);};var hubSizes=this._getHubSizes();var _loop6=function _loop6(i){var hubSize=hubSizes[i];if(hubSize===0)return "break";forEach$4(_this56.body.nodeIndices,function(nodeId){var node=_this56.body.nodes[nodeId];if(hubSize===_this56._getActiveEdges(node).length){_this56._crawlNetwork(levelDownstream,nodeId);}});};for(var i=0;i<hubSizes.length;++i){var _ret4=_loop6(i);if(_ret4==="break")break;}}/**
	   * TODO: release feature
	   * TODO: Determine if this feature is needed at all
	   *
	   * @private
	   */},{key:"_determineLevelsCustomCallback",value:function _determineLevelsCustomCallback(){var _this57=this;var minLevel=100000;// TODO: this should come from options.
	var customCallback=function customCallback(nodeA,nodeB,edge){// eslint-disable-line no-unused-vars
	};// TODO: perhaps move to HierarchicalStatus.
	//       But I currently don't see the point, this method is not used.
	var levelByDirection=function levelByDirection(nodeA,nodeB,edge){var levelA=_this57.hierarchical.levels[nodeA.id];// set initial level
	if(levelA===undefined){levelA=_this57.hierarchical.levels[nodeA.id]=minLevel;}var diff=customCallback(NetworkUtil.cloneOptions(nodeA,'node'),NetworkUtil.cloneOptions(nodeB,'node'),NetworkUtil.cloneOptions(edge,'edge'));_this57.hierarchical.levels[nodeB.id]=levelA+diff;};this._crawlNetwork(levelByDirection);this.hierarchical.setMinLevelToZero(this.body.nodes);}/**
	   * Allocate nodes in levels based on the direction of the edges.
	   *
	   * @private
	   */},{key:"_determineLevelsDirected",value:function _determineLevelsDirected(){var _context95,_this58=this;var nodes=reduce$2(_context95=this.body.nodeIndices).call(_context95,function(acc,id){acc.set(id,_this58.body.nodes[id]);return acc;},new map$2());var levels=this.hierarchical.levels;if(this.options.hierarchical.shakeTowards==="roots"){this.hierarchical.levels=fillLevelsByDirectionRoots(nodes,levels);}else {this.hierarchical.levels=fillLevelsByDirectionLeaves(nodes,levels);}this.hierarchical.setMinLevelToZero(this.body.nodes);}/**
	   * Update the bookkeeping of parent and child.
	   * @private
	   */},{key:"_generateMap",value:function _generateMap(){var _this59=this;var fillInRelations=function fillInRelations(parentNode,childNode){if(_this59.hierarchical.levels[childNode.id]>_this59.hierarchical.levels[parentNode.id]){_this59.hierarchical.addRelation(parentNode.id,childNode.id);}};this._crawlNetwork(fillInRelations);this.hierarchical.checkIfTree();}/**
	   * Crawl over the entire network and use a callback on each node couple that is connected to each other.
	   * @param {function} [callback=function(){}]          | will receive nodeA, nodeB and the connecting edge. A and B are distinct.
	   * @param {Node.id} startingNodeId
	   * @private
	   */},{key:"_crawlNetwork",value:function _crawlNetwork(){var _this60=this;var callback=arguments.length>0&&arguments[0]!==undefined?arguments[0]:function(){};var startingNodeId=arguments.length>1?arguments[1]:undefined;var progress={};var crawler=function crawler(node,tree){if(progress[node.id]===undefined){_this60.hierarchical.setTreeIndex(node,tree);progress[node.id]=true;var childNode;var edges=_this60._getActiveEdges(node);for(var i=0;i<edges.length;i++){var edge=edges[i];if(edge.connected===true){if(edge.toId==node.id){// Not '===' because id's can be string and numeric
	childNode=edge.from;}else {childNode=edge.to;}if(node.id!=childNode.id){// Not '!==' because id's can be string and numeric
	callback(node,childNode,edge);crawler(childNode,tree);}}}}};if(startingNodeId===undefined){// Crawl over all nodes
	var treeIndex=0;// Serves to pass a unique id for the current distinct tree
	for(var i=0;i<this.body.nodeIndices.length;i++){var nodeId=this.body.nodeIndices[i];if(progress[nodeId]===undefined){var node=this.body.nodes[nodeId];crawler(node,treeIndex);treeIndex+=1;}}}else {// Crawl from the given starting node
	var _node5=this.body.nodes[startingNodeId];if(_node5===undefined){console.error("Node not found:",startingNodeId);return;}crawler(_node5);}}/**
	   * Shift a branch a certain distance
	   * @param {Node.id} parentId
	   * @param {number} diff
	   * @private
	   */},{key:"_shiftBlock",value:function _shiftBlock(parentId,diff){var _this61=this;var progress={};var shifter=function shifter(parentId){if(progress[parentId]){return;}progress[parentId]=true;_this61.direction.shift(parentId,diff);var childRef=_this61.hierarchical.childrenReference[parentId];if(childRef!==undefined){for(var i=0;i<childRef.length;i++){shifter(childRef[i]);}}};shifter(parentId);}/**
	   * Find a common parent between branches.
	   * @param {Node.id} childA
	   * @param {Node.id} childB
	   * @returns {{foundParent, withChild}}
	   * @private
	   */},{key:"_findCommonParent",value:function _findCommonParent(childA,childB){var _this62=this;var parents={};var iterateParents=function iterateParents(parents,child){var parentRef=_this62.hierarchical.parentReference[child];if(parentRef!==undefined){for(var i=0;i<parentRef.length;i++){var parent=parentRef[i];parents[parent]=true;iterateParents(parents,parent);}}};var findParent=function findParent(parents,child){var parentRef=_this62.hierarchical.parentReference[child];if(parentRef!==undefined){for(var i=0;i<parentRef.length;i++){var parent=parentRef[i];if(parents[parent]!==undefined){return {foundParent:parent,withChild:child};}var branch=findParent(parents,parent);if(branch.foundParent!==null){return branch;}}}return {foundParent:null,withChild:child};};iterateParents(parents,childA);return findParent(parents,childB);}/**
	   * Set the strategy pattern for handling the coordinates given the current direction.
	   *
	   * The individual instances contain all the operations and data specific to a layout direction.
	   *
	   * @param {Node} node
	   * @param {{x: number, y: number}} position
	   * @param {number} level
	   * @param {boolean} [doNotUpdate=false]
	   * @private
	   */},{key:"setDirectionStrategy",value:function setDirectionStrategy(){var isVertical=this.options.hierarchical.direction==='UD'||this.options.hierarchical.direction==='DU';if(isVertical){this.direction=new VerticalStrategy(this);}else {this.direction=new HorizontalStrategy(this);}}/**
	   * Determine the center position of a branch from the passed list of child nodes
	   *
	   * This takes into account the positions of all the child nodes.
	   * @param {Array.<Node|vis.Node.id>} childNodes  Array of either child nodes or node id's
	   * @return {number}
	   * @private
	   */},{key:"_getCenterPosition",value:function _getCenterPosition(childNodes){var minPos=1e9;var maxPos=-1e9;for(var i=0;i<childNodes.length;i++){var childNode=void 0;if(childNodes[i].id!==undefined){childNode=childNodes[i];}else {var childNodeId=childNodes[i];childNode=this.body.nodes[childNodeId];}var position=this.direction.getPosition(childNode);minPos=Math.min(minPos,position);maxPos=Math.max(maxPos,position);}return 0.5*(minPos+maxPos);}}]);return LayoutEngine;}();/**
	 * Clears the toolbar div element of children
	 *
	 * @private
	 */var ManipulationSystem=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   * @param {Canvas} canvas
	   * @param {SelectionHandler} selectionHandler
	   */function ManipulationSystem(body,canvas,selectionHandler,interactionHandler){var _this63=this,_context96,_context97;classCallCheck(this,ManipulationSystem);this.body=body;this.canvas=canvas;this.selectionHandler=selectionHandler;this.interactionHandler=interactionHandler;this.editMode=false;this.manipulationDiv=undefined;this.editModeDiv=undefined;this.closeDiv=undefined;this.manipulationHammers=[];this.temporaryUIFunctions={};this.temporaryEventFunctions=[];this.touchTime=0;this.temporaryIds={nodes:[],edges:[]};this.guiEnabled=false;this.inMode=false;this.selectedControlNode=undefined;this.options={};this.defaultOptions={enabled:false,initiallyActive:false,addNode:true,addEdge:true,editNode:undefined,editEdge:true,deleteNode:true,deleteEdge:true,controlNodeStyle:{shape:'dot',size:6,color:{background:'#ff0000',border:'#3c3c3c',highlight:{background:'#07f968',border:'#3c3c3c'}},borderWidth:2,borderWidthSelected:2}};extend(this.options,this.defaultOptions);this.body.emitter.on('destroy',function(){_this63._clean();});this.body.emitter.on('_dataChanged',bind$2(_context96=this._restore).call(_context96,this));this.body.emitter.on('_resetData',bind$2(_context97=this._restore).call(_context97,this));}/**
	   * If something changes in the data during editing, switch back to the initial datamanipulation state and close all edit modes.
	   * @private
	   */createClass(ManipulationSystem,[{key:"_restore",value:function _restore(){if(this.inMode!==false){if(this.options.initiallyActive===true){this.enableEditMode();}else {this.disableEditMode();}}}/**
	   * Set the Options
	   *
	   * @param {Object} options
	   * @param {Object} allOptions
	   * @param {Object} globalOptions
	   */},{key:"setOptions",value:function setOptions(options,allOptions,globalOptions){if(allOptions!==undefined){if(allOptions.locale!==undefined){this.options.locale=allOptions.locale;}else {this.options.locale=globalOptions.locale;}if(allOptions.locales!==undefined){this.options.locales=allOptions.locales;}else {this.options.locales=globalOptions.locales;}}if(options!==undefined){if(typeof options==='boolean'){this.options.enabled=options;}else {this.options.enabled=true;deepExtend(this.options,options);}if(this.options.initiallyActive===true){this.editMode=true;}this._setup();}}/**
	   * Enable or disable edit-mode. Draws the DOM required and cleans up after itself.
	   *
	   * @private
	   */},{key:"toggleEditMode",value:function toggleEditMode(){if(this.editMode===true){this.disableEditMode();}else {this.enableEditMode();}}/**
	   * Enables Edit Mode
	   */},{key:"enableEditMode",value:function enableEditMode(){this.editMode=true;this._clean();if(this.guiEnabled===true){this.manipulationDiv.style.display='block';this.closeDiv.style.display='block';this.editModeDiv.style.display='none';this.showManipulatorToolbar();}}/**
	   * Disables Edit Mode
	   */},{key:"disableEditMode",value:function disableEditMode(){this.editMode=false;this._clean();if(this.guiEnabled===true){this.manipulationDiv.style.display='none';this.closeDiv.style.display='none';this.editModeDiv.style.display='block';this._createEditButton();}}/**
	   * Creates the main toolbar. Removes functions bound to the select event. Binds all the buttons of the toolbar.
	   *
	   * @private
	   */},{key:"showManipulatorToolbar",value:function showManipulatorToolbar(){// restore the state of any bound functions or events, remove control nodes, restore physics
	this._clean();// reset global variables
	this.manipulationDOM={};// if the gui is enabled, draw all elements.
	if(this.guiEnabled===true){var _context98,_context99;// a _restore will hide these menus
	this.editMode=true;this.manipulationDiv.style.display='block';this.closeDiv.style.display='block';var selectedNodeCount=this.selectionHandler._getSelectedNodeCount();var selectedEdgeCount=this.selectionHandler._getSelectedEdgeCount();var selectedTotalCount=selectedNodeCount+selectedEdgeCount;var locale=this.options.locales[this.options.locale];var needSeperator=false;if(this.options.addNode!==false){this._createAddNodeButton(locale);needSeperator=true;}if(this.options.addEdge!==false){if(needSeperator===true){this._createSeperator(1);}else {needSeperator=true;}this._createAddEdgeButton(locale);}if(selectedNodeCount===1&&typeof this.options.editNode==='function'){if(needSeperator===true){this._createSeperator(2);}else {needSeperator=true;}this._createEditNodeButton(locale);}else if(selectedEdgeCount===1&&selectedNodeCount===0&&this.options.editEdge!==false){if(needSeperator===true){this._createSeperator(3);}else {needSeperator=true;}this._createEditEdgeButton(locale);}// remove buttons
	if(selectedTotalCount!==0){if(selectedNodeCount>0&&this.options.deleteNode!==false){if(needSeperator===true){this._createSeperator(4);}this._createDeleteButton(locale);}else if(selectedNodeCount===0&&this.options.deleteEdge!==false){if(needSeperator===true){this._createSeperator(4);}this._createDeleteButton(locale);}}// bind the close button
	this._bindHammerToDiv(this.closeDiv,bind$2(_context98=this.toggleEditMode).call(_context98,this));// refresh this bar based on what has been selected
	this._temporaryBindEvent('select',bind$2(_context99=this.showManipulatorToolbar).call(_context99,this));}// redraw to show any possible changes
	this.body.emitter.emit('_redraw');}/**
	   * Create the toolbar for adding Nodes
	   */},{key:"addNodeMode",value:function addNodeMode(){var _context101;// when using the gui, enable edit mode if it wasnt already.
	if(this.editMode!==true){this.enableEditMode();}// restore the state of any bound functions or events, remove control nodes, restore physics
	this._clean();this.inMode='addNode';if(this.guiEnabled===true){var _context100;var locale=this.options.locales[this.options.locale];this.manipulationDOM={};this._createBackButton(locale);this._createSeperator();this._createDescription(locale['addDescription']||this.options.locales['en']['addDescription']);// bind the close button
	this._bindHammerToDiv(this.closeDiv,bind$2(_context100=this.toggleEditMode).call(_context100,this));}this._temporaryBindEvent('click',bind$2(_context101=this._performAddNode).call(_context101,this));}/**
	   * call the bound function to handle the editing of the node. The node has to be selected.
	   */},{key:"editNode",value:function editNode(){var _this64=this;// when using the gui, enable edit mode if it wasnt already.
	if(this.editMode!==true){this.enableEditMode();}// restore the state of any bound functions or events, remove control nodes, restore physics
	this._clean();var node=this.selectionHandler._getSelectedNode();if(node!==undefined){this.inMode='editNode';if(typeof this.options.editNode==='function'){if(node.isCluster!==true){var data=deepExtend({},node.options,false);data.x=node.x;data.y=node.y;if(this.options.editNode.length===2){this.options.editNode(data,function(finalizedData){if(finalizedData!==null&&finalizedData!==undefined&&_this64.inMode==='editNode'){// if for whatever reason the mode has changes (due to dataset change) disregard the callback) {
	_this64.body.data.nodes.getDataSet().update(finalizedData);}_this64.showManipulatorToolbar();});}else {throw new Error('The function for edit does not support two arguments (data, callback)');}}else {alert(this.options.locales[this.options.locale]['editClusterError']||this.options.locales['en']['editClusterError']);}}else {throw new Error('No function has been configured to handle the editing of nodes.');}}else {this.showManipulatorToolbar();}}/**
	   * create the toolbar to connect nodes
	   */},{key:"addEdgeMode",value:function addEdgeMode(){var _context103,_context104,_context105,_context106,_context107;// when using the gui, enable edit mode if it wasnt already.
	if(this.editMode!==true){this.enableEditMode();}// restore the state of any bound functions or events, remove control nodes, restore physics
	this._clean();this.inMode='addEdge';if(this.guiEnabled===true){var _context102;var locale=this.options.locales[this.options.locale];this.manipulationDOM={};this._createBackButton(locale);this._createSeperator();this._createDescription(locale['edgeDescription']||this.options.locales['en']['edgeDescription']);// bind the close button
	this._bindHammerToDiv(this.closeDiv,bind$2(_context102=this.toggleEditMode).call(_context102,this));}// temporarily overload functions
	this._temporaryBindUI('onTouch',bind$2(_context103=this._handleConnect).call(_context103,this));this._temporaryBindUI('onDragEnd',bind$2(_context104=this._finishConnect).call(_context104,this));this._temporaryBindUI('onDrag',bind$2(_context105=this._dragControlNode).call(_context105,this));this._temporaryBindUI('onRelease',bind$2(_context106=this._finishConnect).call(_context106,this));this._temporaryBindUI('onDragStart',bind$2(_context107=this._dragStartEdge).call(_context107,this));this._temporaryBindUI('onHold',function(){});}/**
	   * create the toolbar to edit edges
	   */},{key:"editEdgeMode",value:function editEdgeMode(){// when using the gui, enable edit mode if it wasn't already.
	if(this.editMode!==true){this.enableEditMode();}// restore the state of any bound functions or events, remove control nodes, restore physics
	this._clean();this.inMode='editEdge';if(_typeof_1(this.options.editEdge)==='object'&&typeof this.options.editEdge.editWithoutDrag==="function"){this.edgeBeingEditedId=this.selectionHandler.getSelectedEdges()[0];if(this.edgeBeingEditedId!==undefined){var edge=this.body.edges[this.edgeBeingEditedId];this._performEditEdge(edge.from.id,edge.to.id);return;}}if(this.guiEnabled===true){var _context108;var locale=this.options.locales[this.options.locale];this.manipulationDOM={};this._createBackButton(locale);this._createSeperator();this._createDescription(locale['editEdgeDescription']||this.options.locales['en']['editEdgeDescription']);// bind the close button
	this._bindHammerToDiv(this.closeDiv,bind$2(_context108=this.toggleEditMode).call(_context108,this));}this.edgeBeingEditedId=this.selectionHandler.getSelectedEdges()[0];if(this.edgeBeingEditedId!==undefined){var _context109,_context110,_context111,_context112;var _edge3=this.body.edges[this.edgeBeingEditedId];// create control nodes
	var controlNodeFrom=this._getNewTargetNode(_edge3.from.x,_edge3.from.y);var controlNodeTo=this._getNewTargetNode(_edge3.to.x,_edge3.to.y);this.temporaryIds.nodes.push(controlNodeFrom.id);this.temporaryIds.nodes.push(controlNodeTo.id);this.body.nodes[controlNodeFrom.id]=controlNodeFrom;this.body.nodeIndices.push(controlNodeFrom.id);this.body.nodes[controlNodeTo.id]=controlNodeTo;this.body.nodeIndices.push(controlNodeTo.id);// temporarily overload UI functions, cleaned up automatically because of _temporaryBindUI
	this._temporaryBindUI('onTouch',bind$2(_context109=this._controlNodeTouch).call(_context109,this));// used to get the position
	this._temporaryBindUI('onTap',function(){});// disabled
	this._temporaryBindUI('onHold',function(){});// disabled
	this._temporaryBindUI('onDragStart',bind$2(_context110=this._controlNodeDragStart).call(_context110,this));// used to select control node
	this._temporaryBindUI('onDrag',bind$2(_context111=this._controlNodeDrag).call(_context111,this));// used to drag control node
	this._temporaryBindUI('onDragEnd',bind$2(_context112=this._controlNodeDragEnd).call(_context112,this));// used to connect or revert control nodes
	this._temporaryBindUI('onMouseMove',function(){});// disabled
	// create function to position control nodes correctly on movement
	// automatically cleaned up because we use the temporary bind
	this._temporaryBindEvent('beforeDrawing',function(ctx){var positions=_edge3.edgeType.findBorderPositions(ctx);if(controlNodeFrom.selected===false){controlNodeFrom.x=positions.from.x;controlNodeFrom.y=positions.from.y;}if(controlNodeTo.selected===false){controlNodeTo.x=positions.to.x;controlNodeTo.y=positions.to.y;}});this.body.emitter.emit('_redraw');}else {this.showManipulatorToolbar();}}/**
	   * delete everything in the selection
	   */},{key:"deleteSelected",value:function deleteSelected(){var _this65=this;// when using the gui, enable edit mode if it wasnt already.
	if(this.editMode!==true){this.enableEditMode();}// restore the state of any bound functions or events, remove control nodes, restore physics
	this._clean();this.inMode='delete';var selectedNodes=this.selectionHandler.getSelectedNodes();var selectedEdges=this.selectionHandler.getSelectedEdges();var deleteFunction=undefined;if(selectedNodes.length>0){for(var i=0;i<selectedNodes.length;i++){if(this.body.nodes[selectedNodes[i]].isCluster===true){alert(this.options.locales[this.options.locale]['deleteClusterError']||this.options.locales['en']['deleteClusterError']);return;}}if(typeof this.options.deleteNode==='function'){deleteFunction=this.options.deleteNode;}}else if(selectedEdges.length>0){if(typeof this.options.deleteEdge==='function'){deleteFunction=this.options.deleteEdge;}}if(typeof deleteFunction==='function'){var data={nodes:selectedNodes,edges:selectedEdges};if(deleteFunction.length===2){deleteFunction(data,function(finalizedData){if(finalizedData!==null&&finalizedData!==undefined&&_this65.inMode==='delete'){// if for whatever reason the mode has changes (due to dataset change) disregard the callback) {
	_this65.body.data.edges.getDataSet().remove(finalizedData.edges);_this65.body.data.nodes.getDataSet().remove(finalizedData.nodes);_this65.body.emitter.emit('startSimulation');_this65.showManipulatorToolbar();}else {_this65.body.emitter.emit('startSimulation');_this65.showManipulatorToolbar();}});}else {throw new Error('The function for delete does not support two arguments (data, callback)');}}else {this.body.data.edges.getDataSet().remove(selectedEdges);this.body.data.nodes.getDataSet().remove(selectedNodes);this.body.emitter.emit('startSimulation');this.showManipulatorToolbar();}}//********************************************** PRIVATE ***************************************//
	/**
	   * draw or remove the DOM
	   * @private
	   */},{key:"_setup",value:function _setup(){if(this.options.enabled===true){// Enable the GUI
	this.guiEnabled=true;this._createWrappers();if(this.editMode===false){this._createEditButton();}else {this.showManipulatorToolbar();}}else {this._removeManipulationDOM();// disable the gui
	this.guiEnabled=false;}}/**
	   * create the div overlays that contain the DOM
	   * @private
	   */},{key:"_createWrappers",value:function _createWrappers(){// load the manipulator HTML elements. All styling done in css.
	if(this.manipulationDiv===undefined){this.manipulationDiv=document.createElement('div');this.manipulationDiv.className='vis-manipulation';if(this.editMode===true){this.manipulationDiv.style.display='block';}else {this.manipulationDiv.style.display='none';}this.canvas.frame.appendChild(this.manipulationDiv);}// container for the edit button.
	if(this.editModeDiv===undefined){this.editModeDiv=document.createElement('div');this.editModeDiv.className='vis-edit-mode';if(this.editMode===true){this.editModeDiv.style.display='none';}else {this.editModeDiv.style.display='block';}this.canvas.frame.appendChild(this.editModeDiv);}// container for the close div button
	if(this.closeDiv===undefined){this.closeDiv=document.createElement('div');this.closeDiv.className='vis-close';this.closeDiv.style.display=this.manipulationDiv.style.display;this.canvas.frame.appendChild(this.closeDiv);}}/**
	   * generate a new target node. Used for creating new edges and editing edges
	   *
	   * @param {number} x
	   * @param {number} y
	   * @returns {Node}
	   * @private
	   */},{key:"_getNewTargetNode",value:function _getNewTargetNode(x,y){var controlNodeStyle=deepExtend({},this.options.controlNodeStyle);controlNodeStyle.id='targetNode'+v4();controlNodeStyle.hidden=false;controlNodeStyle.physics=false;controlNodeStyle.x=x;controlNodeStyle.y=y;// we have to define the bounding box in order for the nodes to be drawn immediately
	var node=this.body.functions.createNode(controlNodeStyle);node.shape.boundingBox={left:x,right:x,top:y,bottom:y};return node;}/**
	   * Create the edit button
	   */},{key:"_createEditButton",value:function _createEditButton(){var _context113;// restore everything to it's original state (if applicable)
	this._clean();// reset the manipulationDOM
	this.manipulationDOM={};// empty the editModeDiv
	recursiveDOMDelete(this.editModeDiv);// create the contents for the editMode button
	var locale=this.options.locales[this.options.locale];var button=this._createButton('editMode','vis-button vis-edit vis-edit-mode',locale['edit']||this.options.locales['en']['edit']);this.editModeDiv.appendChild(button);// bind a hammer listener to the button, calling the function toggleEditMode.
	this._bindHammerToDiv(button,bind$2(_context113=this.toggleEditMode).call(_context113,this));}/**
	   * this function cleans up after everything this module does. Temporary elements, functions and events are removed, physics restored, hammers removed.
	   * @private
	   */},{key:"_clean",value:function _clean(){// not in mode
	this.inMode=false;// _clean the divs
	if(this.guiEnabled===true){recursiveDOMDelete(this.editModeDiv);recursiveDOMDelete(this.manipulationDiv);// removes all the bindings and overloads
	this._cleanManipulatorHammers();}// remove temporary nodes and edges
	this._cleanupTemporaryNodesAndEdges();// restore overloaded UI functions
	this._unbindTemporaryUIs();// remove the temporaryEventFunctions
	this._unbindTemporaryEvents();// restore the physics if required
	this.body.emitter.emit('restorePhysics');}/**
	   * Each dom element has it's own hammer. They are stored in this.manipulationHammers. This cleans them up.
	   * @private
	   */},{key:"_cleanManipulatorHammers",value:function _cleanManipulatorHammers(){// _clean hammer bindings
	if(this.manipulationHammers.length!=0){for(var i=0;i<this.manipulationHammers.length;i++){this.manipulationHammers[i].destroy();}this.manipulationHammers=[];}}/**
	   * Remove all DOM elements created by this module.
	   * @private
	   */},{key:"_removeManipulationDOM",value:function _removeManipulationDOM(){// removes all the bindings and overloads
	this._clean();// empty the manipulation divs
	recursiveDOMDelete(this.manipulationDiv);recursiveDOMDelete(this.editModeDiv);recursiveDOMDelete(this.closeDiv);// remove the manipulation divs
	if(this.manipulationDiv){this.canvas.frame.removeChild(this.manipulationDiv);}if(this.editModeDiv){this.canvas.frame.removeChild(this.editModeDiv);}if(this.closeDiv){this.canvas.frame.removeChild(this.closeDiv);}// set the references to undefined
	this.manipulationDiv=undefined;this.editModeDiv=undefined;this.closeDiv=undefined;}/**
	   * create a seperator line. the index is to differentiate in the manipulation dom
	   * @param {number} [index=1]
	   * @private
	   */},{key:"_createSeperator",value:function _createSeperator(){var index=arguments.length>0&&arguments[0]!==undefined?arguments[0]:1;this.manipulationDOM['seperatorLineDiv'+index]=document.createElement('div');this.manipulationDOM['seperatorLineDiv'+index].className='vis-separator-line';this.manipulationDiv.appendChild(this.manipulationDOM['seperatorLineDiv'+index]);}// ----------------------    DOM functions for buttons    --------------------------//
	/**
	   *
	   * @param {Locale} locale
	   * @private
	   */},{key:"_createAddNodeButton",value:function _createAddNodeButton(locale){var _context114;var button=this._createButton('addNode','vis-button vis-add',locale['addNode']||this.options.locales['en']['addNode']);this.manipulationDiv.appendChild(button);this._bindHammerToDiv(button,bind$2(_context114=this.addNodeMode).call(_context114,this));}/**
	   *
	   * @param {Locale} locale
	   * @private
	   */},{key:"_createAddEdgeButton",value:function _createAddEdgeButton(locale){var _context115;var button=this._createButton('addEdge','vis-button vis-connect',locale['addEdge']||this.options.locales['en']['addEdge']);this.manipulationDiv.appendChild(button);this._bindHammerToDiv(button,bind$2(_context115=this.addEdgeMode).call(_context115,this));}/**
	   *
	   * @param {Locale} locale
	   * @private
	   */},{key:"_createEditNodeButton",value:function _createEditNodeButton(locale){var _context116;var button=this._createButton('editNode','vis-button vis-edit',locale['editNode']||this.options.locales['en']['editNode']);this.manipulationDiv.appendChild(button);this._bindHammerToDiv(button,bind$2(_context116=this.editNode).call(_context116,this));}/**
	   *
	   * @param {Locale} locale
	   * @private
	   */},{key:"_createEditEdgeButton",value:function _createEditEdgeButton(locale){var _context117;var button=this._createButton('editEdge','vis-button vis-edit',locale['editEdge']||this.options.locales['en']['editEdge']);this.manipulationDiv.appendChild(button);this._bindHammerToDiv(button,bind$2(_context117=this.editEdgeMode).call(_context117,this));}/**
	   *
	   * @param {Locale} locale
	   * @private
	   */},{key:"_createDeleteButton",value:function _createDeleteButton(locale){var _context118;var deleteBtnClass;if(this.options.rtl){deleteBtnClass='vis-button vis-delete-rtl';}else {deleteBtnClass='vis-button vis-delete';}var button=this._createButton('delete',deleteBtnClass,locale['del']||this.options.locales['en']['del']);this.manipulationDiv.appendChild(button);this._bindHammerToDiv(button,bind$2(_context118=this.deleteSelected).call(_context118,this));}/**
	   *
	   * @param {Locale} locale
	   * @private
	   */},{key:"_createBackButton",value:function _createBackButton(locale){var _context119;var button=this._createButton('back','vis-button vis-back',locale['back']||this.options.locales['en']['back']);this.manipulationDiv.appendChild(button);this._bindHammerToDiv(button,bind$2(_context119=this.showManipulatorToolbar).call(_context119,this));}/**
	   *
	   * @param {number|string} id
	   * @param {string} className
	   * @param {label} label
	   * @param {string} labelClassName
	   * @returns {HTMLElement}
	   * @private
	   */},{key:"_createButton",value:function _createButton(id,className,label){var labelClassName=arguments.length>3&&arguments[3]!==undefined?arguments[3]:'vis-label';this.manipulationDOM[id+'Div']=document.createElement('div');this.manipulationDOM[id+'Div'].className=className;this.manipulationDOM[id+'Label']=document.createElement('div');this.manipulationDOM[id+'Label'].className=labelClassName;this.manipulationDOM[id+'Label'].innerHTML=label;this.manipulationDOM[id+'Div'].appendChild(this.manipulationDOM[id+'Label']);return this.manipulationDOM[id+'Div'];}/**
	   *
	   * @param {Label} label
	   * @private
	   */},{key:"_createDescription",value:function _createDescription(label){this.manipulationDiv.appendChild(this._createButton('description','vis-button vis-none',label));}// -------------------------- End of DOM functions for buttons ------------------------------//
	/**
	   * this binds an event until cleanup by the clean functions.
	   * @param {Event}  event   The event
	   * @param {function} newFunction
	   * @private
	   */},{key:"_temporaryBindEvent",value:function _temporaryBindEvent(event,newFunction){this.temporaryEventFunctions.push({event:event,boundFunction:newFunction});this.body.emitter.on(event,newFunction);}/**
	   * this overrides an UI function until cleanup by the clean function
	   * @param {string} UIfunctionName
	   * @param {function} newFunction
	   * @private
	   */},{key:"_temporaryBindUI",value:function _temporaryBindUI(UIfunctionName,newFunction){if(this.body.eventListeners[UIfunctionName]!==undefined){this.temporaryUIFunctions[UIfunctionName]=this.body.eventListeners[UIfunctionName];this.body.eventListeners[UIfunctionName]=newFunction;}else {throw new Error('This UI function does not exist. Typo? You tried: '+UIfunctionName+' possible are: '+stringify$2(keys$6(this.body.eventListeners)));}}/**
	   * Restore the overridden UI functions to their original state.
	   *
	   * @private
	   */},{key:"_unbindTemporaryUIs",value:function _unbindTemporaryUIs(){for(var functionName in this.temporaryUIFunctions){if(this.temporaryUIFunctions.hasOwnProperty(functionName)){this.body.eventListeners[functionName]=this.temporaryUIFunctions[functionName];delete this.temporaryUIFunctions[functionName];}}this.temporaryUIFunctions={};}/**
	   * Unbind the events created by _temporaryBindEvent
	   * @private
	   */},{key:"_unbindTemporaryEvents",value:function _unbindTemporaryEvents(){for(var i=0;i<this.temporaryEventFunctions.length;i++){var eventName=this.temporaryEventFunctions[i].event;var boundFunction=this.temporaryEventFunctions[i].boundFunction;this.body.emitter.off(eventName,boundFunction);}this.temporaryEventFunctions=[];}/**
	   * Bind an hammer instance to a DOM element.
	   *
	   * @param {Element} domElement
	   * @param {function} boundFunction
	   */},{key:"_bindHammerToDiv",value:function _bindHammerToDiv(domElement,boundFunction){var hammer$1=new hammer(domElement,{});hammerUtil.onTouch(hammer$1,boundFunction);this.manipulationHammers.push(hammer$1);}/**
	   * Neatly clean up temporary edges and nodes
	   * @private
	   */},{key:"_cleanupTemporaryNodesAndEdges",value:function _cleanupTemporaryNodesAndEdges(){// _clean temporary edges
	for(var i=0;i<this.temporaryIds.edges.length;i++){var _context120;this.body.edges[this.temporaryIds.edges[i]].disconnect();delete this.body.edges[this.temporaryIds.edges[i]];var indexTempEdge=indexOf$3(_context120=this.body.edgeIndices).call(_context120,this.temporaryIds.edges[i]);if(indexTempEdge!==-1){var _context121;splice$2(_context121=this.body.edgeIndices).call(_context121,indexTempEdge,1);}}// _clean temporary nodes
	for(var _i22=0;_i22<this.temporaryIds.nodes.length;_i22++){var _context122;delete this.body.nodes[this.temporaryIds.nodes[_i22]];var indexTempNode=indexOf$3(_context122=this.body.nodeIndices).call(_context122,this.temporaryIds.nodes[_i22]);if(indexTempNode!==-1){var _context123;splice$2(_context123=this.body.nodeIndices).call(_context123,indexTempNode,1);}}this.temporaryIds={nodes:[],edges:[]};}// ------------------------------------------ EDIT EDGE FUNCTIONS -----------------------------------------//
	/**
	   * the touch is used to get the position of the initial click
	   * @param {Event}  event   The event
	   * @private
	   */},{key:"_controlNodeTouch",value:function _controlNodeTouch(event){this.selectionHandler.unselectAll();this.lastTouch=this.body.functions.getPointer(event.center);this.lastTouch.translation=extend({},this.body.view.translation);// copy the object
	}/**
	   * the drag start is used to mark one of the control nodes as selected.
	   * @param {Event}  event   The event
	   * @private
	   */},{key:"_controlNodeDragStart",value:function _controlNodeDragStart(event){// eslint-disable-line no-unused-vars
	var pointer=this.lastTouch;var pointerObj=this.selectionHandler._pointerToPositionObject(pointer);var from=this.body.nodes[this.temporaryIds.nodes[0]];var to=this.body.nodes[this.temporaryIds.nodes[1]];var edge=this.body.edges[this.edgeBeingEditedId];this.selectedControlNode=undefined;var fromSelect=from.isOverlappingWith(pointerObj);var toSelect=to.isOverlappingWith(pointerObj);if(fromSelect===true){this.selectedControlNode=from;edge.edgeType.from=from;}else if(toSelect===true){this.selectedControlNode=to;edge.edgeType.to=to;}// we use the selection to find the node that is being dragged. We explicitly select it here.
	if(this.selectedControlNode!==undefined){this.selectionHandler.selectObject(this.selectedControlNode);}this.body.emitter.emit('_redraw');}/**
	   * dragging the control nodes or the canvas
	   * @param {Event}  event   The event
	   * @private
	   */},{key:"_controlNodeDrag",value:function _controlNodeDrag(event){this.body.emitter.emit('disablePhysics');var pointer=this.body.functions.getPointer(event.center);var pos=this.canvas.DOMtoCanvas(pointer);if(this.selectedControlNode!==undefined){this.selectedControlNode.x=pos.x;this.selectedControlNode.y=pos.y;}else {this.interactionHandler.onDrag(event);}this.body.emitter.emit('_redraw');}/**
	   * connecting or restoring the control nodes.
	   * @param {Event}  event   The event
	   * @private
	   */},{key:"_controlNodeDragEnd",value:function _controlNodeDragEnd(event){var pointer=this.body.functions.getPointer(event.center);var pointerObj=this.selectionHandler._pointerToPositionObject(pointer);var edge=this.body.edges[this.edgeBeingEditedId];// if the node that was dragged is not a control node, return
	if(this.selectedControlNode===undefined){return;}// we use the selection to find the node that is being dragged. We explicitly DEselect the control node here.
	this.selectionHandler.unselectAll();var overlappingNodeIds=this.selectionHandler._getAllNodesOverlappingWith(pointerObj);var node=undefined;for(var i=overlappingNodeIds.length-1;i>=0;i--){if(overlappingNodeIds[i]!==this.selectedControlNode.id){node=this.body.nodes[overlappingNodeIds[i]];break;}}// perform the connection
	if(node!==undefined&&this.selectedControlNode!==undefined){if(node.isCluster===true){alert(this.options.locales[this.options.locale]['createEdgeError']||this.options.locales['en']['createEdgeError']);}else {var from=this.body.nodes[this.temporaryIds.nodes[0]];if(this.selectedControlNode.id===from.id){this._performEditEdge(node.id,edge.to.id);}else {this._performEditEdge(edge.from.id,node.id);}}}else {edge.updateEdgeType();this.body.emitter.emit('restorePhysics');}this.body.emitter.emit('_redraw');}// ------------------------------------ END OF EDIT EDGE FUNCTIONS -----------------------------------------//
	// ------------------------------------------- ADD EDGE FUNCTIONS -----------------------------------------//
	/**
	   * the function bound to the selection event. It checks if you want to connect a cluster and changes the description
	   * to walk the user through the process.
	   *
	   * @param {Event} event
	   * @private
	   */},{key:"_handleConnect",value:function _handleConnect(event){// check to avoid double fireing of this function.
	if(new Date().valueOf()-this.touchTime>100){this.lastTouch=this.body.functions.getPointer(event.center);this.lastTouch.translation=extend({},this.body.view.translation);// copy the object
	this.interactionHandler.drag.pointer=this.lastTouch;// Drag pointer is not updated when adding edges
	this.interactionHandler.drag.translation=this.lastTouch.translation;var pointer=this.lastTouch;var node=this.selectionHandler.getNodeAt(pointer);if(node!==undefined){if(node.isCluster===true){alert(this.options.locales[this.options.locale]['createEdgeError']||this.options.locales['en']['createEdgeError']);}else {// create a node the temporary line can look at
	var targetNode=this._getNewTargetNode(node.x,node.y);this.body.nodes[targetNode.id]=targetNode;this.body.nodeIndices.push(targetNode.id);// create a temporary edge
	var connectionEdge=this.body.functions.createEdge({id:'connectionEdge'+v4(),from:node.id,to:targetNode.id,physics:false,smooth:{enabled:true,type:'continuous',roundness:0.5}});this.body.edges[connectionEdge.id]=connectionEdge;this.body.edgeIndices.push(connectionEdge.id);this.temporaryIds.nodes.push(targetNode.id);this.temporaryIds.edges.push(connectionEdge.id);}}this.touchTime=new Date().valueOf();}}/**
	   *
	   * @param {Event} event
	   * @private
	   */},{key:"_dragControlNode",value:function _dragControlNode(event){var pointer=this.body.functions.getPointer(event.center);var pointerObj=this.selectionHandler._pointerToPositionObject(pointer);// remember the edge id
	var connectFromId=undefined;if(this.temporaryIds.edges[0]!==undefined){connectFromId=this.body.edges[this.temporaryIds.edges[0]].fromId;}// get the overlapping node but NOT the temporary node;
	var overlappingNodeIds=this.selectionHandler._getAllNodesOverlappingWith(pointerObj);var node=undefined;for(var i=overlappingNodeIds.length-1;i>=0;i--){var _context124;// if the node id is NOT a temporary node, accept the node.
	if(indexOf$3(_context124=this.temporaryIds.nodes).call(_context124,overlappingNodeIds[i])===-1){node=this.body.nodes[overlappingNodeIds[i]];break;}}event.controlEdge={from:connectFromId,to:node?node.id:undefined};this.selectionHandler._generateClickEvent('controlNodeDragging',event,pointer);if(this.temporaryIds.nodes[0]!==undefined){var targetNode=this.body.nodes[this.temporaryIds.nodes[0]];// there is only one temp node in the add edge mode.
	targetNode.x=this.canvas._XconvertDOMtoCanvas(pointer.x);targetNode.y=this.canvas._YconvertDOMtoCanvas(pointer.y);this.body.emitter.emit('_redraw');}else {this.interactionHandler.onDrag(event);}}/**
	   * Connect the new edge to the target if one exists, otherwise remove temp line
	   * @param {Event}  event   The event
	   * @private
	   */},{key:"_finishConnect",value:function _finishConnect(event){var pointer=this.body.functions.getPointer(event.center);var pointerObj=this.selectionHandler._pointerToPositionObject(pointer);// remember the edge id
	var connectFromId=undefined;if(this.temporaryIds.edges[0]!==undefined){connectFromId=this.body.edges[this.temporaryIds.edges[0]].fromId;}// get the overlapping node but NOT the temporary node;
	var overlappingNodeIds=this.selectionHandler._getAllNodesOverlappingWith(pointerObj);var node=undefined;for(var i=overlappingNodeIds.length-1;i>=0;i--){var _context125;// if the node id is NOT a temporary node, accept the node.
	if(indexOf$3(_context125=this.temporaryIds.nodes).call(_context125,overlappingNodeIds[i])===-1){node=this.body.nodes[overlappingNodeIds[i]];break;}}// clean temporary nodes and edges.
	this._cleanupTemporaryNodesAndEdges();// perform the connection
	if(node!==undefined){if(node.isCluster===true){alert(this.options.locales[this.options.locale]['createEdgeError']||this.options.locales['en']['createEdgeError']);}else {if(this.body.nodes[connectFromId]!==undefined&&this.body.nodes[node.id]!==undefined){this._performAddEdge(connectFromId,node.id);}}}event.controlEdge={from:connectFromId,to:node?node.id:undefined};this.selectionHandler._generateClickEvent('controlNodeDragEnd',event,pointer);// No need to do _generateclickevent('dragEnd') here, the regular dragEnd event fires.
	this.body.emitter.emit('_redraw');}/**
	   *
	   * @param {Event} event
	   * @private
	   */},{key:"_dragStartEdge",value:function _dragStartEdge(event){var pointer=this.lastTouch;this.selectionHandler._generateClickEvent('dragStart',event,pointer,undefined,true);}// --------------------------------------- END OF ADD EDGE FUNCTIONS -------------------------------------//
	// ------------------------------ Performing all the actual data manipulation ------------------------//
	/**
	   * Adds a node on the specified location
	   *
	   * @param {Object} clickData
	   * @private
	   */},{key:"_performAddNode",value:function _performAddNode(clickData){var _this66=this;var defaultData={id:v4(),x:clickData.pointer.canvas.x,y:clickData.pointer.canvas.y,label:'new'};if(typeof this.options.addNode==='function'){if(this.options.addNode.length===2){this.options.addNode(defaultData,function(finalizedData){if(finalizedData!==null&&finalizedData!==undefined&&_this66.inMode==='addNode'){// if for whatever reason the mode has changes (due to dataset change) disregard the callback
	_this66.body.data.nodes.getDataSet().add(finalizedData);}_this66.showManipulatorToolbar();});}else {this.showManipulatorToolbar();throw new Error('The function for add does not support two arguments (data,callback)');}}else {this.body.data.nodes.getDataSet().add(defaultData);this.showManipulatorToolbar();}}/**
	   * connect two nodes with a new edge.
	   *
	   * @param {Node.id} sourceNodeId
	   * @param {Node.id} targetNodeId
	   * @private
	   */},{key:"_performAddEdge",value:function _performAddEdge(sourceNodeId,targetNodeId){var _this67=this;var defaultData={from:sourceNodeId,to:targetNodeId};if(typeof this.options.addEdge==='function'){if(this.options.addEdge.length===2){this.options.addEdge(defaultData,function(finalizedData){if(finalizedData!==null&&finalizedData!==undefined&&_this67.inMode==='addEdge'){// if for whatever reason the mode has changes (due to dataset change) disregard the callback
	_this67.body.data.edges.getDataSet().add(finalizedData);_this67.selectionHandler.unselectAll();_this67.showManipulatorToolbar();}});}else {throw new Error('The function for connect does not support two arguments (data,callback)');}}else {this.body.data.edges.getDataSet().add(defaultData);this.selectionHandler.unselectAll();this.showManipulatorToolbar();}}/**
	   * connect two nodes with a new edge.
	   *
	   * @param {Node.id} sourceNodeId
	   * @param {Node.id} targetNodeId
	   * @private
	   */},{key:"_performEditEdge",value:function _performEditEdge(sourceNodeId,targetNodeId){var _this68=this;var defaultData={id:this.edgeBeingEditedId,from:sourceNodeId,to:targetNodeId,label:this.body.data.edges.get(this.edgeBeingEditedId).label};var eeFunct=this.options.editEdge;if(_typeof_1(eeFunct)==='object'){eeFunct=eeFunct.editWithoutDrag;}if(typeof eeFunct==='function'){if(eeFunct.length===2){eeFunct(defaultData,function(finalizedData){if(finalizedData===null||finalizedData===undefined||_this68.inMode!=='editEdge'){// if for whatever reason the mode has changes (due to dataset change) disregard the callback) {
	_this68.body.edges[defaultData.id].updateEdgeType();_this68.body.emitter.emit('_redraw');_this68.showManipulatorToolbar();}else {_this68.body.data.edges.getDataSet().update(finalizedData);_this68.selectionHandler.unselectAll();_this68.showManipulatorToolbar();}});}else {throw new Error('The function for edit does not support two arguments (data, callback)');}}else {this.body.data.edges.getDataSet().update(defaultData);this.selectionHandler.unselectAll();this.showManipulatorToolbar();}}}]);return ManipulationSystem;}();var htmlColors={black:'#000000',navy:'#000080',darkblue:'#00008B',mediumblue:'#0000CD',blue:'#0000FF',darkgreen:'#006400',green:'#008000',teal:'#008080',darkcyan:'#008B8B',deepskyblue:'#00BFFF',darkturquoise:'#00CED1',mediumspringgreen:'#00FA9A',lime:'#00FF00',springgreen:'#00FF7F',aqua:'#00FFFF',cyan:'#00FFFF',midnightblue:'#191970',dodgerblue:'#1E90FF',lightseagreen:'#20B2AA',forestgreen:'#228B22',seagreen:'#2E8B57',darkslategray:'#2F4F4F',limegreen:'#32CD32',mediumseagreen:'#3CB371',turquoise:'#40E0D0',royalblue:'#4169E1',steelblue:'#4682B4',darkslateblue:'#483D8B',mediumturquoise:'#48D1CC',indigo:'#4B0082',darkolivegreen:'#556B2F',cadetblue:'#5F9EA0',cornflowerblue:'#6495ED',mediumaquamarine:'#66CDAA',dimgray:'#696969',slateblue:'#6A5ACD',olivedrab:'#6B8E23',slategray:'#708090',lightslategray:'#778899',mediumslateblue:'#7B68EE',lawngreen:'#7CFC00',chartreuse:'#7FFF00',aquamarine:'#7FFFD4',maroon:'#800000',purple:'#800080',olive:'#808000',gray:'#808080',skyblue:'#87CEEB',lightskyblue:'#87CEFA',blueviolet:'#8A2BE2',darkred:'#8B0000',darkmagenta:'#8B008B',saddlebrown:'#8B4513',darkseagreen:'#8FBC8F',lightgreen:'#90EE90',mediumpurple:'#9370D8',darkviolet:'#9400D3',palegreen:'#98FB98',darkorchid:'#9932CC',yellowgreen:'#9ACD32',sienna:'#A0522D',brown:'#A52A2A',darkgray:'#A9A9A9',lightblue:'#ADD8E6',greenyellow:'#ADFF2F',paleturquoise:'#AFEEEE',lightsteelblue:'#B0C4DE',powderblue:'#B0E0E6',firebrick:'#B22222',darkgoldenrod:'#B8860B',mediumorchid:'#BA55D3',rosybrown:'#BC8F8F',darkkhaki:'#BDB76B',silver:'#C0C0C0',mediumvioletred:'#C71585',indianred:'#CD5C5C',peru:'#CD853F',chocolate:'#D2691E',tan:'#D2B48C',lightgrey:'#D3D3D3',palevioletred:'#D87093',thistle:'#D8BFD8',orchid:'#DA70D6',goldenrod:'#DAA520',crimson:'#DC143C',gainsboro:'#DCDCDC',plum:'#DDA0DD',burlywood:'#DEB887',lightcyan:'#E0FFFF',lavender:'#E6E6FA',darksalmon:'#E9967A',violet:'#EE82EE',palegoldenrod:'#EEE8AA',lightcoral:'#F08080',khaki:'#F0E68C',aliceblue:'#F0F8FF',honeydew:'#F0FFF0',azure:'#F0FFFF',sandybrown:'#F4A460',wheat:'#F5DEB3',beige:'#F5F5DC',whitesmoke:'#F5F5F5',mintcream:'#F5FFFA',ghostwhite:'#F8F8FF',salmon:'#FA8072',antiquewhite:'#FAEBD7',linen:'#FAF0E6',lightgoldenrodyellow:'#FAFAD2',oldlace:'#FDF5E6',red:'#FF0000',fuchsia:'#FF00FF',magenta:'#FF00FF',deeppink:'#FF1493',orangered:'#FF4500',tomato:'#FF6347',hotpink:'#FF69B4',coral:'#FF7F50',darkorange:'#FF8C00',lightsalmon:'#FFA07A',orange:'#FFA500',lightpink:'#FFB6C1',pink:'#FFC0CB',gold:'#FFD700',peachpuff:'#FFDAB9',navajowhite:'#FFDEAD',moccasin:'#FFE4B5',bisque:'#FFE4C4',mistyrose:'#FFE4E1',blanchedalmond:'#FFEBCD',papayawhip:'#FFEFD5',lavenderblush:'#FFF0F5',seashell:'#FFF5EE',cornsilk:'#FFF8DC',lemonchiffon:'#FFFACD',floralwhite:'#FFFAF0',snow:'#FFFAFA',yellow:'#FFFF00',lightyellow:'#FFFFE0',ivory:'#FFFFF0',white:'#FFFFFF'};/**
	 * @param {number} [pixelRatio=1]
	 */var ColorPicker=/*#__PURE__*/function(){/**
	   * @param {number} [pixelRatio=1]
	   */function ColorPicker(){var pixelRatio=arguments.length>0&&arguments[0]!==undefined?arguments[0]:1;classCallCheck(this,ColorPicker);this.pixelRatio=pixelRatio;this.generated=false;this.centerCoordinates={x:289/2,y:289/2};this.r=289*0.49;this.color={r:255,g:255,b:255,a:1.0};this.hueCircle=undefined;this.initialColor={r:255,g:255,b:255,a:1.0};this.previousColor=undefined;this.applied=false;// bound by
	this.updateCallback=function(){};this.closeCallback=function(){};// create all DOM elements
	this._create();}/**
	   * this inserts the colorPicker into a div from the DOM
	   * @param {Element} container
	   */createClass(ColorPicker,[{key:"insertTo",value:function insertTo(container){if(this.hammer!==undefined){this.hammer.destroy();this.hammer=undefined;}this.container=container;this.container.appendChild(this.frame);this._bindHammer();this._setSize();}/**
	   * the callback is executed on apply and save. Bind it to the application
	   * @param {function} callback
	   */},{key:"setUpdateCallback",value:function setUpdateCallback(callback){if(typeof callback==='function'){this.updateCallback=callback;}else {throw new Error("Function attempted to set as colorPicker update callback is not a function.");}}/**
	   * the callback is executed on apply and save. Bind it to the application
	   * @param {function} callback
	   */},{key:"setCloseCallback",value:function setCloseCallback(callback){if(typeof callback==='function'){this.closeCallback=callback;}else {throw new Error("Function attempted to set as colorPicker closing callback is not a function.");}}/**
	   *
	   * @param {string} color
	   * @returns {String}
	   * @private
	   */},{key:"_isColorString",value:function _isColorString(color){if(typeof color==='string'){return htmlColors[color];}}/**
	   * Set the color of the colorPicker
	   * Supported formats:
	   * 'red'                   --> HTML color string
	   * '#ffffff'               --> hex string
	   * 'rgb(255,255,255)'      --> rgb string
	   * 'rgba(255,255,255,1.0)' --> rgba string
	   * {r:255,g:255,b:255}     --> rgb object
	   * {r:255,g:255,b:255,a:1.0} --> rgba object
	   * @param {string|Object} color
	   * @param {boolean} [setInitial=true]
	   */},{key:"setColor",value:function setColor(color){var setInitial=arguments.length>1&&arguments[1]!==undefined?arguments[1]:true;if(color==='none'){return;}var rgba;// if a html color shorthand is used, convert to hex
	var htmlColor=this._isColorString(color);if(htmlColor!==undefined){color=htmlColor;}// check format
	if(isString(color)===true){if(isValidRGB(color)===true){var rgbaArray=color.substr(4).substr(0,color.length-5).split(',');rgba={r:rgbaArray[0],g:rgbaArray[1],b:rgbaArray[2],a:1.0};}else if(isValidRGBA(color)===true){var _rgbaArray=color.substr(5).substr(0,color.length-6).split(',');rgba={r:_rgbaArray[0],g:_rgbaArray[1],b:_rgbaArray[2],a:_rgbaArray[3]};}else if(isValidHex(color)===true){var rgbObj=hexToRGB(color);rgba={r:rgbObj.r,g:rgbObj.g,b:rgbObj.b,a:1.0};}}else {if(color instanceof Object){if(color.r!==undefined&&color.g!==undefined&&color.b!==undefined){var alpha=color.a!==undefined?color.a:'1.0';rgba={r:color.r,g:color.g,b:color.b,a:alpha};}}}// set color
	if(rgba===undefined){throw new Error("Unknown color passed to the colorPicker. Supported are strings: rgb, hex, rgba. Object: rgb ({r:r,g:g,b:b,[a:a]}). Supplied: "+stringify$2(color));}else {this._setColor(rgba,setInitial);}}/**
	   * this shows the color picker.
	   * The hue circle is constructed once and stored.
	   */},{key:"show",value:function show(){if(this.closeCallback!==undefined){this.closeCallback();this.closeCallback=undefined;}this.applied=false;this.frame.style.display='block';this._generateHueCircle();}// ------------------------------------------ PRIVATE ----------------------------- //
	/**
	   * Hide the picker. Is called by the cancel button.
	   * Optional boolean to store the previous color for easy access later on.
	   * @param {boolean} [storePrevious=true]
	   * @private
	   */},{key:"_hide",value:function _hide(){var _this69=this;var storePrevious=arguments.length>0&&arguments[0]!==undefined?arguments[0]:true;// store the previous color for next time;
	if(storePrevious===true){this.previousColor=extend({},this.color);}if(this.applied===true){this.updateCallback(this.initialColor);}this.frame.style.display='none';// call the closing callback, restoring the onclick method.
	// this is in a setTimeout because it will trigger the show again before the click is done.
	setTimeout$2(function(){if(_this69.closeCallback!==undefined){_this69.closeCallback();_this69.closeCallback=undefined;}},0);}/**
	   * bound to the save button. Saves and hides.
	   * @private
	   */},{key:"_save",value:function _save(){this.updateCallback(this.color);this.applied=false;this._hide();}/**
	   * Bound to apply button. Saves but does not close. Is undone by the cancel button.
	   * @private
	   */},{key:"_apply",value:function _apply(){this.applied=true;this.updateCallback(this.color);this._updatePicker(this.color);}/**
	   * load the color from the previous session.
	   * @private
	   */},{key:"_loadLast",value:function _loadLast(){if(this.previousColor!==undefined){this.setColor(this.previousColor,false);}else {alert("There is no last color to load...");}}/**
	   * set the color, place the picker
	   * @param {Object} rgba
	   * @param {boolean} [setInitial=true]
	   * @private
	   */},{key:"_setColor",value:function _setColor(rgba){var setInitial=arguments.length>1&&arguments[1]!==undefined?arguments[1]:true;// store the initial color
	if(setInitial===true){this.initialColor=extend({},rgba);}this.color=rgba;var hsv=RGBToHSV(rgba.r,rgba.g,rgba.b);var angleConvert=2*Math.PI;var radius=this.r*hsv.s;var x=this.centerCoordinates.x+radius*Math.sin(angleConvert*hsv.h);var y=this.centerCoordinates.y+radius*Math.cos(angleConvert*hsv.h);this.colorPickerSelector.style.left=x-0.5*this.colorPickerSelector.clientWidth+'px';this.colorPickerSelector.style.top=y-0.5*this.colorPickerSelector.clientHeight+'px';this._updatePicker(rgba);}/**
	   * bound to opacity control
	   * @param {number} value
	   * @private
	   */},{key:"_setOpacity",value:function _setOpacity(value){this.color.a=value/100;this._updatePicker(this.color);}/**
	   * bound to brightness control
	   * @param {number} value
	   * @private
	   */},{key:"_setBrightness",value:function _setBrightness(value){var hsv=RGBToHSV(this.color.r,this.color.g,this.color.b);hsv.v=value/100;var rgba=HSVToRGB(hsv.h,hsv.s,hsv.v);rgba['a']=this.color.a;this.color=rgba;this._updatePicker();}/**
	   * update the color picker. A black circle overlays the hue circle to mimic the brightness decreasing.
	   * @param {Object} rgba
	   * @private
	   */},{key:"_updatePicker",value:function _updatePicker(){var rgba=arguments.length>0&&arguments[0]!==undefined?arguments[0]:this.color;var hsv=RGBToHSV(rgba.r,rgba.g,rgba.b);var ctx=this.colorPickerCanvas.getContext('2d');if(this.pixelRation===undefined){this.pixelRatio=(window.devicePixelRatio||1)/(ctx.webkitBackingStorePixelRatio||ctx.mozBackingStorePixelRatio||ctx.msBackingStorePixelRatio||ctx.oBackingStorePixelRatio||ctx.backingStorePixelRatio||1);}ctx.setTransform(this.pixelRatio,0,0,this.pixelRatio,0,0);// clear the canvas
	var w=this.colorPickerCanvas.clientWidth;var h=this.colorPickerCanvas.clientHeight;ctx.clearRect(0,0,w,h);ctx.putImageData(this.hueCircle,0,0);ctx.fillStyle='rgba(0,0,0,'+(1-hsv.v)+')';ctx.circle(this.centerCoordinates.x,this.centerCoordinates.y,this.r);fill$2(ctx).call(ctx);this.brightnessRange.value=100*hsv.v;this.opacityRange.value=100*rgba.a;this.initialColorDiv.style.backgroundColor='rgba('+this.initialColor.r+','+this.initialColor.g+','+this.initialColor.b+','+this.initialColor.a+')';this.newColorDiv.style.backgroundColor='rgba('+this.color.r+','+this.color.g+','+this.color.b+','+this.color.a+')';}/**
	   * used by create to set the size of the canvas.
	   * @private
	   */},{key:"_setSize",value:function _setSize(){this.colorPickerCanvas.style.width='100%';this.colorPickerCanvas.style.height='100%';this.colorPickerCanvas.width=289*this.pixelRatio;this.colorPickerCanvas.height=289*this.pixelRatio;}/**
	   * create all dom elements
	   * TODO: cleanup, lots of similar dom elements
	   * @private
	   */},{key:"_create",value:function _create(){var _context126,_context127,_context128,_context129;this.frame=document.createElement('div');this.frame.className='vis-color-picker';this.colorPickerDiv=document.createElement('div');this.colorPickerSelector=document.createElement('div');this.colorPickerSelector.className='vis-selector';this.colorPickerDiv.appendChild(this.colorPickerSelector);this.colorPickerCanvas=document.createElement('canvas');this.colorPickerDiv.appendChild(this.colorPickerCanvas);if(!this.colorPickerCanvas.getContext){var noCanvas=document.createElement('DIV');noCanvas.style.color='red';noCanvas.style.fontWeight='bold';noCanvas.style.padding='10px';noCanvas.innerHTML='Error: your browser does not support HTML canvas';this.colorPickerCanvas.appendChild(noCanvas);}else {var ctx=this.colorPickerCanvas.getContext("2d");this.pixelRatio=(window.devicePixelRatio||1)/(ctx.webkitBackingStorePixelRatio||ctx.mozBackingStorePixelRatio||ctx.msBackingStorePixelRatio||ctx.oBackingStorePixelRatio||ctx.backingStorePixelRatio||1);this.colorPickerCanvas.getContext("2d").setTransform(this.pixelRatio,0,0,this.pixelRatio,0,0);}this.colorPickerDiv.className='vis-color';this.opacityDiv=document.createElement('div');this.opacityDiv.className='vis-opacity';this.brightnessDiv=document.createElement('div');this.brightnessDiv.className='vis-brightness';this.arrowDiv=document.createElement('div');this.arrowDiv.className='vis-arrow';this.opacityRange=document.createElement('input');try{this.opacityRange.type='range';// Not supported on IE9
	this.opacityRange.min='0';this.opacityRange.max='100';}// TODO: Add some error handling and remove this lint exception
	catch(err){}// eslint-disable-line no-empty
	this.opacityRange.value='100';this.opacityRange.className='vis-range';this.brightnessRange=document.createElement('input');try{this.brightnessRange.type='range';// Not supported on IE9
	this.brightnessRange.min='0';this.brightnessRange.max='100';}// TODO: Add some error handling and remove this lint exception
	catch(err){}// eslint-disable-line no-empty
	this.brightnessRange.value='100';this.brightnessRange.className='vis-range';this.opacityDiv.appendChild(this.opacityRange);this.brightnessDiv.appendChild(this.brightnessRange);var me=this;this.opacityRange.onchange=function(){me._setOpacity(this.value);};this.opacityRange.oninput=function(){me._setOpacity(this.value);};this.brightnessRange.onchange=function(){me._setBrightness(this.value);};this.brightnessRange.oninput=function(){me._setBrightness(this.value);};this.brightnessLabel=document.createElement("div");this.brightnessLabel.className="vis-label vis-brightness";this.brightnessLabel.innerHTML='brightness:';this.opacityLabel=document.createElement("div");this.opacityLabel.className="vis-label vis-opacity";this.opacityLabel.innerHTML='opacity:';this.newColorDiv=document.createElement("div");this.newColorDiv.className="vis-new-color";this.newColorDiv.innerHTML='new';this.initialColorDiv=document.createElement("div");this.initialColorDiv.className="vis-initial-color";this.initialColorDiv.innerHTML='initial';this.cancelButton=document.createElement("div");this.cancelButton.className="vis-button vis-cancel";this.cancelButton.innerHTML='cancel';this.cancelButton.onclick=bind$2(_context126=this._hide).call(_context126,this,false);this.applyButton=document.createElement("div");this.applyButton.className="vis-button vis-apply";this.applyButton.innerHTML='apply';this.applyButton.onclick=bind$2(_context127=this._apply).call(_context127,this);this.saveButton=document.createElement("div");this.saveButton.className="vis-button vis-save";this.saveButton.innerHTML='save';this.saveButton.onclick=bind$2(_context128=this._save).call(_context128,this);this.loadButton=document.createElement("div");this.loadButton.className="vis-button vis-load";this.loadButton.innerHTML='load last';this.loadButton.onclick=bind$2(_context129=this._loadLast).call(_context129,this);this.frame.appendChild(this.colorPickerDiv);this.frame.appendChild(this.arrowDiv);this.frame.appendChild(this.brightnessLabel);this.frame.appendChild(this.brightnessDiv);this.frame.appendChild(this.opacityLabel);this.frame.appendChild(this.opacityDiv);this.frame.appendChild(this.newColorDiv);this.frame.appendChild(this.initialColorDiv);this.frame.appendChild(this.cancelButton);this.frame.appendChild(this.applyButton);this.frame.appendChild(this.saveButton);this.frame.appendChild(this.loadButton);}/**
	   * bind hammer to the color picker
	   * @private
	   */},{key:"_bindHammer",value:function _bindHammer(){var _this70=this;this.drag={};this.pinch={};this.hammer=new hammer(this.colorPickerCanvas);this.hammer.get('pinch').set({enable:true});hammerUtil.onTouch(this.hammer,function(event){_this70._moveSelector(event);});this.hammer.on('tap',function(event){_this70._moveSelector(event);});this.hammer.on('panstart',function(event){_this70._moveSelector(event);});this.hammer.on('panmove',function(event){_this70._moveSelector(event);});this.hammer.on('panend',function(event){_this70._moveSelector(event);});}/**
	   * generate the hue circle. This is relatively heavy (200ms) and is done only once on the first time it is shown.
	   * @private
	   */},{key:"_generateHueCircle",value:function _generateHueCircle(){if(this.generated===false){var ctx=this.colorPickerCanvas.getContext('2d');if(this.pixelRation===undefined){this.pixelRatio=(window.devicePixelRatio||1)/(ctx.webkitBackingStorePixelRatio||ctx.mozBackingStorePixelRatio||ctx.msBackingStorePixelRatio||ctx.oBackingStorePixelRatio||ctx.backingStorePixelRatio||1);}ctx.setTransform(this.pixelRatio,0,0,this.pixelRatio,0,0);// clear the canvas
	var w=this.colorPickerCanvas.clientWidth;var h=this.colorPickerCanvas.clientHeight;ctx.clearRect(0,0,w,h);// draw hue circle
	var x,y,hue,sat;this.centerCoordinates={x:w*0.5,y:h*0.5};this.r=0.49*w;var angleConvert=2*Math.PI/360;var hfac=1/360;var sfac=1/this.r;var rgb;for(hue=0;hue<360;hue++){for(sat=0;sat<this.r;sat++){x=this.centerCoordinates.x+sat*Math.sin(angleConvert*hue);y=this.centerCoordinates.y+sat*Math.cos(angleConvert*hue);rgb=HSVToRGB(hue*hfac,sat*sfac,1);ctx.fillStyle='rgb('+rgb.r+','+rgb.g+','+rgb.b+')';ctx.fillRect(x-0.5,y-0.5,2,2);}}ctx.strokeStyle='rgba(0,0,0,1)';ctx.circle(this.centerCoordinates.x,this.centerCoordinates.y,this.r);ctx.stroke();this.hueCircle=ctx.getImageData(0,0,w,h);}this.generated=true;}/**
	   * move the selector. This is called by hammer functions.
	   *
	   * @param {Event}  event   The event
	   * @private
	   */},{key:"_moveSelector",value:function _moveSelector(event){var rect=this.colorPickerDiv.getBoundingClientRect();var left=event.center.x-rect.left;var top=event.center.y-rect.top;var centerY=0.5*this.colorPickerDiv.clientHeight;var centerX=0.5*this.colorPickerDiv.clientWidth;var x=left-centerX;var y=top-centerY;var angle=Math.atan2(x,y);var radius=0.98*Math.min(Math.sqrt(x*x+y*y),centerX);var newTop=Math.cos(angle)*radius+centerY;var newLeft=Math.sin(angle)*radius+centerX;this.colorPickerSelector.style.top=newTop-0.5*this.colorPickerSelector.clientHeight+'px';this.colorPickerSelector.style.left=newLeft-0.5*this.colorPickerSelector.clientWidth+'px';// set color
	var h=angle/(2*Math.PI);h=h<0?h+1:h;var s=radius/this.r;var hsv=RGBToHSV(this.color.r,this.color.g,this.color.b);hsv.h=h;hsv.s=s;var rgba=HSVToRGB(hsv.h,hsv.s,hsv.v);rgba['a']=this.color.a;this.color=rgba;// update previews
	this.initialColorDiv.style.backgroundColor='rgba('+this.initialColor.r+','+this.initialColor.g+','+this.initialColor.b+','+this.initialColor.a+')';this.newColorDiv.style.backgroundColor='rgba('+this.color.r+','+this.color.g+','+this.color.b+','+this.color.a+')';}}]);return ColorPicker;}();/**
	 * The way this works is for all properties of this.possible options, you can supply the property name in any form to list the options.
	 * Boolean options are recognised as Boolean
	 * Number options should be written as array: [default value, min value, max value, stepsize]
	 * Colors should be written as array: ['color', '#ffffff']
	 * Strings with should be written as array: [option1, option2, option3, ..]
	 *
	 * The options are matched with their counterparts in each of the modules and the values used in the configuration are
	 */var Configurator=/*#__PURE__*/function(){/**
	   * @param {Object} parentModule        | the location where parentModule.setOptions() can be called
	   * @param {Object} defaultContainer    | the default container of the module
	   * @param {Object} configureOptions    | the fully configured and predefined options set found in allOptions.js
	   * @param {number} pixelRatio          | canvas pixel ratio
	   */function Configurator(parentModule,defaultContainer,configureOptions){var pixelRatio=arguments.length>3&&arguments[3]!==undefined?arguments[3]:1;classCallCheck(this,Configurator);this.parent=parentModule;this.changedOptions=[];this.container=defaultContainer;this.allowCreation=false;this.options={};this.initialized=false;this.popupCounter=0;this.defaultOptions={enabled:false,filter:true,container:undefined,showButton:true};extend(this.options,this.defaultOptions);this.configureOptions=configureOptions;this.moduleOptions={};this.domElements=[];this.popupDiv={};this.popupLimit=5;this.popupHistory={};this.colorPicker=new ColorPicker(pixelRatio);this.wrapper=undefined;}/**
	   * refresh all options.
	   * Because all modules parse their options by themselves, we just use their options. We copy them here.
	   *
	   * @param {Object} options
	   */createClass(Configurator,[{key:"setOptions",value:function setOptions(options){if(options!==undefined){// reset the popup history because the indices may have been changed.
	this.popupHistory={};this._removePopup();var enabled=true;if(typeof options==='string'){this.options.filter=options;}else if(options instanceof Array){this.options.filter=options.join();}else if(_typeof_1(options)==='object'){if(options==null){throw new TypeError('options cannot be null');}if(options.container!==undefined){this.options.container=options.container;}if(filter$2(options)!==undefined){this.options.filter=filter$2(options);}if(options.showButton!==undefined){this.options.showButton=options.showButton;}if(options.enabled!==undefined){enabled=options.enabled;}}else if(typeof options==='boolean'){this.options.filter=true;enabled=options;}else if(typeof options==='function'){this.options.filter=options;enabled=true;}if(filter$2(this.options)===false){enabled=false;}this.options.enabled=enabled;}this._clean();}/**
	   *
	   * @param {Object} moduleOptions
	   */},{key:"setModuleOptions",value:function setModuleOptions(moduleOptions){this.moduleOptions=moduleOptions;if(this.options.enabled===true){this._clean();if(this.options.container!==undefined){this.container=this.options.container;}this._create();}}/**
	   * Create all DOM elements
	   * @private
	   */},{key:"_create",value:function _create(){this._clean();this.changedOptions=[];var filter=filter$2(this.options);var counter=0;var show=false;for(var option in this.configureOptions){if(this.configureOptions.hasOwnProperty(option)){this.allowCreation=false;show=false;if(typeof filter==='function'){show=filter(option,[]);show=show||this._handleObject(this.configureOptions[option],[option],true);}else if(filter===true||indexOf$3(filter).call(filter,option)!==-1){show=true;}if(show!==false){this.allowCreation=true;// linebreak between categories
	if(counter>0){this._makeItem([]);}// a header for the category
	this._makeHeader(option);// get the sub options
	this._handleObject(this.configureOptions[option],[option]);}counter++;}}this._makeButton();this._push();//~ this.colorPicker.insertTo(this.container);
	}/**
	   * draw all DOM elements on the screen
	   * @private
	   */},{key:"_push",value:function _push(){this.wrapper=document.createElement('div');this.wrapper.className='vis-configuration-wrapper';this.container.appendChild(this.wrapper);for(var i=0;i<this.domElements.length;i++){this.wrapper.appendChild(this.domElements[i]);}this._showPopupIfNeeded();}/**
	   * delete all DOM elements
	   * @private
	   */},{key:"_clean",value:function _clean(){for(var i=0;i<this.domElements.length;i++){this.wrapper.removeChild(this.domElements[i]);}if(this.wrapper!==undefined){this.container.removeChild(this.wrapper);this.wrapper=undefined;}this.domElements=[];this._removePopup();}/**
	   * get the value from the actualOptions if it exists
	   * @param {array} path    | where to look for the actual option
	   * @returns {*}
	   * @private
	   */},{key:"_getValue",value:function _getValue(path){var base=this.moduleOptions;for(var i=0;i<path.length;i++){if(base[path[i]]!==undefined){base=base[path[i]];}else {base=undefined;break;}}return base;}/**
	   * all option elements are wrapped in an item
	   * @param {Array} path    | where to look for the actual option
	   * @param {Array.<Element>} domElements
	   * @returns {number}
	   * @private
	   */},{key:"_makeItem",value:function _makeItem(path){if(this.allowCreation===true){var item=document.createElement('div');item.className='vis-configuration vis-config-item vis-config-s'+path.length;for(var _len2=arguments.length,domElements=new Array(_len2>1?_len2-1:0),_key2=1;_key2<_len2;_key2++){domElements[_key2-1]=arguments[_key2];}forEach$3(domElements).call(domElements,function(element){item.appendChild(element);});this.domElements.push(item);return this.domElements.length;}return 0;}/**
	   * header for major subjects
	   * @param {string} name
	   * @private
	   */},{key:"_makeHeader",value:function _makeHeader(name){var div=document.createElement('div');div.className='vis-configuration vis-config-header';div.innerHTML=name;this._makeItem([],div);}/**
	   * make a label, if it is an object label, it gets different styling.
	   * @param {string} name
	   * @param {array} path    | where to look for the actual option
	   * @param {string} objectLabel
	   * @returns {HTMLElement}
	   * @private
	   */},{key:"_makeLabel",value:function _makeLabel(name,path){var objectLabel=arguments.length>2&&arguments[2]!==undefined?arguments[2]:false;var div=document.createElement('div');div.className='vis-configuration vis-config-label vis-config-s'+path.length;if(objectLabel===true){div.innerHTML='<i><b>'+name+':</b></i>';}else {div.innerHTML=name+':';}return div;}/**
	   * make a dropdown list for multiple possible string optoins
	   * @param {Array.<number>} arr
	   * @param {number} value
	   * @param {array} path    | where to look for the actual option
	   * @private
	   */},{key:"_makeDropdown",value:function _makeDropdown(arr,value,path){var select=document.createElement('select');select.className='vis-configuration vis-config-select';var selectedValue=0;if(value!==undefined){if(indexOf$3(arr).call(arr,value)!==-1){selectedValue=indexOf$3(arr).call(arr,value);}}for(var i=0;i<arr.length;i++){var option=document.createElement('option');option.value=arr[i];if(i===selectedValue){option.selected='selected';}option.innerHTML=arr[i];select.appendChild(option);}var me=this;select.onchange=function(){me._update(this.value,path);};var label=this._makeLabel(path[path.length-1],path);this._makeItem(path,label,select);}/**
	   * make a range object for numeric options
	   * @param {Array.<number>} arr
	   * @param {number} value
	   * @param {array} path    | where to look for the actual option
	   * @private
	   */},{key:"_makeRange",value:function _makeRange(arr,value,path){var defaultValue=arr[0];var min=arr[1];var max=arr[2];var step=arr[3];var range=document.createElement('input');range.className='vis-configuration vis-config-range';try{range.type='range';// not supported on IE9
	range.min=min;range.max=max;}// TODO: Add some error handling and remove this lint exception
	catch(err){}// eslint-disable-line no-empty
	range.step=step;// set up the popup settings in case they are needed.
	var popupString='';var popupValue=0;if(value!==undefined){var factor=1.20;if(value<0&&value*factor<min){range.min=Math.ceil(value*factor);popupValue=range.min;popupString='range increased';}else if(value/factor<min){range.min=Math.ceil(value/factor);popupValue=range.min;popupString='range increased';}if(value*factor>max&&max!==1){range.max=Math.ceil(value*factor);popupValue=range.max;popupString='range increased';}range.value=value;}else {range.value=defaultValue;}var input=document.createElement('input');input.className='vis-configuration vis-config-rangeinput';input.value=range.value;var me=this;range.onchange=function(){input.value=this.value;me._update(Number(this.value),path);};range.oninput=function(){input.value=this.value;};var label=this._makeLabel(path[path.length-1],path);var itemIndex=this._makeItem(path,label,range,input);// if a popup is needed AND it has not been shown for this value, show it.
	if(popupString!==''&&this.popupHistory[itemIndex]!==popupValue){this.popupHistory[itemIndex]=popupValue;this._setupPopup(popupString,itemIndex);}}/**
	   * make a button object
	   * @private
	   */},{key:"_makeButton",value:function _makeButton(){var _this71=this;if(this.options.showButton===true){var generateButton=document.createElement('div');generateButton.className='vis-configuration vis-config-button';generateButton.innerHTML='generate options';generateButton.onclick=function(){_this71._printOptions();};generateButton.onmouseover=function(){generateButton.className='vis-configuration vis-config-button hover';};generateButton.onmouseout=function(){generateButton.className='vis-configuration vis-config-button';};this.optionsContainer=document.createElement('div');this.optionsContainer.className='vis-configuration vis-config-option-container';this.domElements.push(this.optionsContainer);this.domElements.push(generateButton);}}/**
	   * prepare the popup
	   * @param {string} string
	   * @param {number} index
	   * @private
	   */},{key:"_setupPopup",value:function _setupPopup(string,index){var _this72=this;if(this.initialized===true&&this.allowCreation===true&&this.popupCounter<this.popupLimit){var div=document.createElement("div");div.id="vis-configuration-popup";div.className="vis-configuration-popup";div.innerHTML=string;div.onclick=function(){_this72._removePopup();};this.popupCounter+=1;this.popupDiv={html:div,index:index};}}/**
	   * remove the popup from the dom
	   * @private
	   */},{key:"_removePopup",value:function _removePopup(){if(this.popupDiv.html!==undefined){this.popupDiv.html.parentNode.removeChild(this.popupDiv.html);clearTimeout(this.popupDiv.hideTimeout);clearTimeout(this.popupDiv.deleteTimeout);this.popupDiv={};}}/**
	   * Show the popup if it is needed.
	   * @private
	   */},{key:"_showPopupIfNeeded",value:function _showPopupIfNeeded(){var _this73=this;if(this.popupDiv.html!==undefined){var correspondingElement=this.domElements[this.popupDiv.index];var rect=correspondingElement.getBoundingClientRect();this.popupDiv.html.style.left=rect.left+"px";this.popupDiv.html.style.top=rect.top-30+"px";// 30 is the height;
	document.body.appendChild(this.popupDiv.html);this.popupDiv.hideTimeout=setTimeout$2(function(){_this73.popupDiv.html.style.opacity=0;},1500);this.popupDiv.deleteTimeout=setTimeout$2(function(){_this73._removePopup();},1800);}}/**
	   * make a checkbox for boolean options.
	   * @param {number} defaultValue
	   * @param {number} value
	   * @param {array} path    | where to look for the actual option
	   * @private
	   */},{key:"_makeCheckbox",value:function _makeCheckbox(defaultValue,value,path){var checkbox=document.createElement('input');checkbox.type='checkbox';checkbox.className='vis-configuration vis-config-checkbox';checkbox.checked=defaultValue;if(value!==undefined){checkbox.checked=value;if(value!==defaultValue){if(_typeof_1(defaultValue)==='object'){if(value!==defaultValue.enabled){this.changedOptions.push({path:path,value:value});}}else {this.changedOptions.push({path:path,value:value});}}}var me=this;checkbox.onchange=function(){me._update(this.checked,path);};var label=this._makeLabel(path[path.length-1],path);this._makeItem(path,label,checkbox);}/**
	   * make a text input field for string options.
	   * @param {number} defaultValue
	   * @param {number} value
	   * @param {array} path    | where to look for the actual option
	   * @private
	   */},{key:"_makeTextInput",value:function _makeTextInput(defaultValue,value,path){var checkbox=document.createElement('input');checkbox.type='text';checkbox.className='vis-configuration vis-config-text';checkbox.value=value;if(value!==defaultValue){this.changedOptions.push({path:path,value:value});}var me=this;checkbox.onchange=function(){me._update(this.value,path);};var label=this._makeLabel(path[path.length-1],path);this._makeItem(path,label,checkbox);}/**
	   * make a color field with a color picker for color fields
	   * @param {Array.<number>} arr
	   * @param {number} value
	   * @param {array} path    | where to look for the actual option
	   * @private
	   */},{key:"_makeColorField",value:function _makeColorField(arr,value,path){var _this74=this;var defaultColor=arr[1];var div=document.createElement('div');value=value===undefined?defaultColor:value;if(value!=='none'){div.className='vis-configuration vis-config-colorBlock';div.style.backgroundColor=value;}else {div.className='vis-configuration vis-config-colorBlock none';}value=value===undefined?defaultColor:value;div.onclick=function(){_this74._showColorPicker(value,div,path);};var label=this._makeLabel(path[path.length-1],path);this._makeItem(path,label,div);}/**
	   * used by the color buttons to call the color picker.
	   * @param {number} value
	   * @param {HTMLElement} div
	   * @param {array} path    | where to look for the actual option
	   * @private
	   */},{key:"_showColorPicker",value:function _showColorPicker(value,div,path){var _this75=this;// clear the callback from this div
	div.onclick=function(){};this.colorPicker.insertTo(div);this.colorPicker.show();this.colorPicker.setColor(value);this.colorPicker.setUpdateCallback(function(color){var colorString='rgba('+color.r+','+color.g+','+color.b+','+color.a+')';div.style.backgroundColor=colorString;_this75._update(colorString,path);});// on close of the colorpicker, restore the callback.
	this.colorPicker.setCloseCallback(function(){div.onclick=function(){_this75._showColorPicker(value,div,path);};});}/**
	   * parse an object and draw the correct items
	   * @param {Object} obj
	   * @param {array} [path=[]]    | where to look for the actual option
	   * @param {boolean} [checkOnly=false]
	   * @returns {boolean}
	   * @private
	   */},{key:"_handleObject",value:function _handleObject(obj){var path=arguments.length>1&&arguments[1]!==undefined?arguments[1]:[];var checkOnly=arguments.length>2&&arguments[2]!==undefined?arguments[2]:false;var show=false;var filter=filter$2(this.options);var visibleInSet=false;for(var subObj in obj){if(obj.hasOwnProperty(subObj)){show=true;var item=obj[subObj];var newPath=copyAndExtendArray(path,subObj);if(typeof filter==='function'){show=filter(subObj,path);// if needed we must go deeper into the object.
	if(show===false){if(!(item instanceof Array)&&typeof item!=='string'&&typeof item!=='boolean'&&item instanceof Object){this.allowCreation=false;show=this._handleObject(item,newPath,true);this.allowCreation=checkOnly===false;}}}if(show!==false){visibleInSet=true;var value=this._getValue(newPath);if(item instanceof Array){this._handleArray(item,value,newPath);}else if(typeof item==='string'){this._makeTextInput(item,value,newPath);}else if(typeof item==='boolean'){this._makeCheckbox(item,value,newPath);}else if(item instanceof Object){// collapse the physics options that are not enabled
	var draw=true;if(indexOf$3(path).call(path,'physics')!==-1){if(this.moduleOptions.physics.solver!==subObj&&subObj!=='wind'){draw=false;}}if(draw===true){// initially collapse options with an disabled enabled option.
	if(item.enabled!==undefined){var enabledPath=copyAndExtendArray(newPath,'enabled');var enabledValue=this._getValue(enabledPath);if(enabledValue===true){var label=this._makeLabel(subObj,newPath,true);this._makeItem(newPath,label);visibleInSet=this._handleObject(item,newPath)||visibleInSet;}else {this._makeCheckbox(item,enabledValue,newPath);}}else {var _label=this._makeLabel(subObj,newPath,true);this._makeItem(newPath,_label);visibleInSet=this._handleObject(item,newPath)||visibleInSet;}}}else {console.error('dont know how to handle',item,subObj,newPath);}}}}return visibleInSet;}/**
	   * handle the array type of option
	   * @param {Array.<number>} arr
	   * @param {number} value
	   * @param {array} path    | where to look for the actual option
	   * @private
	   */},{key:"_handleArray",value:function _handleArray(arr,value,path){if(typeof arr[0]==='string'&&arr[0]==='color'){this._makeColorField(arr,value,path);if(arr[1]!==value){this.changedOptions.push({path:path,value:value});}}else if(typeof arr[0]==='string'){this._makeDropdown(arr,value,path);if(arr[0]!==value){this.changedOptions.push({path:path,value:value});}}else if(typeof arr[0]==='number'){this._makeRange(arr,value,path);if(arr[0]!==value){this.changedOptions.push({path:path,value:Number(value)});}}}/**
	   * called to update the network with the new settings.
	   * @param {number} value
	   * @param {array} path    | where to look for the actual option
	   * @private
	   */},{key:"_update",value:function _update(value,path){var options=this._constructOptions(value,path);if(this.parent.body&&this.parent.body.emitter&&this.parent.body.emitter.emit){this.parent.body.emitter.emit("configChange",options);}this.initialized=true;this.parent.setOptions(options);}/**
	   *
	   * @param {string|Boolean} value
	   * @param {Array.<string>} path
	   * @param {{}} optionsObj
	   * @returns {{}}
	   * @private
	   */},{key:"_constructOptions",value:function _constructOptions(value,path){var optionsObj=arguments.length>2&&arguments[2]!==undefined?arguments[2]:{};var pointer=optionsObj;// when dropdown boxes can be string or boolean, we typecast it into correct types
	value=value==='true'?true:value;value=value==='false'?false:value;for(var i=0;i<path.length;i++){if(path[i]!=='global'){if(pointer[path[i]]===undefined){pointer[path[i]]={};}if(i!==path.length-1){pointer=pointer[path[i]];}else {pointer[path[i]]=value;}}}return optionsObj;}/**
	   * @private
	   */},{key:"_printOptions",value:function _printOptions(){var options=this.getOptions();this.optionsContainer.innerHTML='<pre>var options = '+stringify$2(options,null,2)+'</pre>';}/**
	   *
	   * @returns {{}} options
	   */},{key:"getOptions",value:function getOptions(){var options={};for(var i=0;i<this.changedOptions.length;i++){this._constructOptions(this.changedOptions[i].value,this.changedOptions[i].path,options);}return options;}}]);return Configurator;}();/**
	 * This object contains all possible options. It will check if the types are correct, if required if the option is one
	 * of the allowed values.
	 *
	 * __any__ means that the name of the property does not matter.
	 * __type__ is a required field for all objects and contains the allowed types of all objects
	 */var string$1='string';var bool$1='boolean';var number$1='number';var array$1='array';var object$1='object';// should only be in a __type__ property
	var dom='dom';var any='any';// List of endpoints
	var endPoints=["arrow","bar","box","circle","crow","curve","diamond","image","inv_curve","inv_triangle","triangle","vee"];var allOptions$1$1={configure:{enabled:{boolean:bool$1},filter:{boolean:bool$1,string:string$1,array:array$1,'function':'function'},container:{dom:dom},showButton:{boolean:bool$1},__type__:{object:object$1,boolean:bool$1,string:string$1,array:array$1,'function':'function'}},edges:{arrows:{to:{enabled:{boolean:bool$1},scaleFactor:{number:number$1},type:{string:endPoints},imageHeight:{number:number$1},imageWidth:{number:number$1},src:{string:string$1},__type__:{object:object$1,boolean:bool$1}},middle:{enabled:{boolean:bool$1},scaleFactor:{number:number$1},type:{string:endPoints},imageWidth:{number:number$1},imageHeight:{number:number$1},src:{string:string$1},__type__:{object:object$1,boolean:bool$1}},from:{enabled:{boolean:bool$1},scaleFactor:{number:number$1},type:{string:endPoints},imageWidth:{number:number$1},imageHeight:{number:number$1},src:{string:string$1},__type__:{object:object$1,boolean:bool$1}},__type__:{string:["from","to","middle"],object:object$1}},arrowStrikethrough:{boolean:bool$1},background:{enabled:{boolean:bool$1},color:{string:string$1},size:{number:number$1},dashes:{boolean:bool$1,array:array$1},__type__:{object:object$1,boolean:bool$1}},chosen:{label:{boolean:bool$1,'function':'function'},edge:{boolean:bool$1,'function':'function'},__type__:{object:object$1,boolean:bool$1}},color:{color:{string:string$1},highlight:{string:string$1},hover:{string:string$1},inherit:{string:['from','to','both'],boolean:bool$1},opacity:{number:number$1},__type__:{object:object$1,string:string$1}},dashes:{boolean:bool$1,array:array$1},font:{color:{string:string$1},size:{number:number$1},// px
	face:{string:string$1},background:{string:string$1},strokeWidth:{number:number$1},// px
	strokeColor:{string:string$1},align:{string:['horizontal','top','middle','bottom']},vadjust:{number:number$1},multi:{boolean:bool$1,string:string$1},bold:{color:{string:string$1},size:{number:number$1},// px
	face:{string:string$1},mod:{string:string$1},vadjust:{number:number$1},__type__:{object:object$1,string:string$1}},boldital:{color:{string:string$1},size:{number:number$1},// px
	face:{string:string$1},mod:{string:string$1},vadjust:{number:number$1},__type__:{object:object$1,string:string$1}},ital:{color:{string:string$1},size:{number:number$1},// px
	face:{string:string$1},mod:{string:string$1},vadjust:{number:number$1},__type__:{object:object$1,string:string$1}},mono:{color:{string:string$1},size:{number:number$1},// px
	face:{string:string$1},mod:{string:string$1},vadjust:{number:number$1},__type__:{object:object$1,string:string$1}},__type__:{object:object$1,string:string$1}},hidden:{boolean:bool$1},hoverWidth:{'function':'function',number:number$1},label:{string:string$1,'undefined':'undefined'},labelHighlightBold:{boolean:bool$1},length:{number:number$1,'undefined':'undefined'},physics:{boolean:bool$1},scaling:{min:{number:number$1},max:{number:number$1},label:{enabled:{boolean:bool$1},min:{number:number$1},max:{number:number$1},maxVisible:{number:number$1},drawThreshold:{number:number$1},__type__:{object:object$1,boolean:bool$1}},customScalingFunction:{'function':'function'},__type__:{object:object$1}},selectionWidth:{'function':'function',number:number$1},selfReferenceSize:{number:number$1},selfReference:{size:{number:number$1},angle:{number:number$1},renderBehindTheNode:{boolean:bool$1},__type__:{object:object$1}},shadow:{enabled:{boolean:bool$1},color:{string:string$1},size:{number:number$1},x:{number:number$1},y:{number:number$1},__type__:{object:object$1,boolean:bool$1}},smooth:{enabled:{boolean:bool$1},type:{string:['dynamic','continuous','discrete','diagonalCross','straightCross','horizontal','vertical','curvedCW','curvedCCW','cubicBezier']},roundness:{number:number$1},forceDirection:{string:['horizontal','vertical','none'],boolean:bool$1},__type__:{object:object$1,boolean:bool$1}},title:{string:string$1,'undefined':'undefined'},width:{number:number$1},widthConstraint:{maximum:{number:number$1},__type__:{object:object$1,boolean:bool$1,number:number$1}},value:{number:number$1,'undefined':'undefined'},__type__:{object:object$1}},groups:{useDefaultGroups:{boolean:bool$1},__any__:'get from nodes, will be overwritten below',__type__:{object:object$1}},interaction:{dragNodes:{boolean:bool$1},dragView:{boolean:bool$1},hideEdgesOnDrag:{boolean:bool$1},hideEdgesOnZoom:{boolean:bool$1},hideNodesOnDrag:{boolean:bool$1},hover:{boolean:bool$1},keyboard:{enabled:{boolean:bool$1},speed:{x:{number:number$1},y:{number:number$1},zoom:{number:number$1},__type__:{object:object$1}},bindToWindow:{boolean:bool$1},__type__:{object:object$1,boolean:bool$1}},multiselect:{boolean:bool$1},navigationButtons:{boolean:bool$1},selectable:{boolean:bool$1},selectConnectedEdges:{boolean:bool$1},hoverConnectedEdges:{boolean:bool$1},tooltipDelay:{number:number$1},zoomView:{boolean:bool$1},zoomSpeed:{number:number$1},__type__:{object:object$1}},layout:{randomSeed:{'undefined':'undefined',number:number$1},improvedLayout:{boolean:bool$1},clusterThreshold:{number:number$1},hierarchical:{enabled:{boolean:bool$1},levelSeparation:{number:number$1},nodeSpacing:{number:number$1},treeSpacing:{number:number$1},blockShifting:{boolean:bool$1},edgeMinimization:{boolean:bool$1},parentCentralization:{boolean:bool$1},direction:{string:['UD','DU','LR','RL']},// UD, DU, LR, RL
	sortMethod:{string:['hubsize','directed']},// hubsize, directed
	shakeTowards:{string:['leaves','roots']},// leaves, roots
	__type__:{object:object$1,boolean:bool$1}},__type__:{object:object$1}},manipulation:{enabled:{boolean:bool$1},initiallyActive:{boolean:bool$1},addNode:{boolean:bool$1,'function':'function'},addEdge:{boolean:bool$1,'function':'function'},editNode:{'function':'function'},editEdge:{editWithoutDrag:{'function':'function'},__type__:{object:object$1,boolean:bool$1,'function':'function'}},deleteNode:{boolean:bool$1,'function':'function'},deleteEdge:{boolean:bool$1,'function':'function'},controlNodeStyle:'get from nodes, will be overwritten below',__type__:{object:object$1,boolean:bool$1}},nodes:{borderWidth:{number:number$1},borderWidthSelected:{number:number$1,'undefined':'undefined'},brokenImage:{string:string$1,'undefined':'undefined'},chosen:{label:{boolean:bool$1,'function':'function'},node:{boolean:bool$1,'function':'function'},__type__:{object:object$1,boolean:bool$1}},color:{border:{string:string$1},background:{string:string$1},highlight:{border:{string:string$1},background:{string:string$1},__type__:{object:object$1,string:string$1}},hover:{border:{string:string$1},background:{string:string$1},__type__:{object:object$1,string:string$1}},__type__:{object:object$1,string:string$1}},opacity:{number:number$1,'undefined':'undefined'},fixed:{x:{boolean:bool$1},y:{boolean:bool$1},__type__:{object:object$1,boolean:bool$1}},font:{align:{string:string$1},color:{string:string$1},size:{number:number$1},// px
	face:{string:string$1},background:{string:string$1},strokeWidth:{number:number$1},// px
	strokeColor:{string:string$1},vadjust:{number:number$1},multi:{boolean:bool$1,string:string$1},bold:{color:{string:string$1},size:{number:number$1},// px
	face:{string:string$1},mod:{string:string$1},vadjust:{number:number$1},__type__:{object:object$1,string:string$1}},boldital:{color:{string:string$1},size:{number:number$1},// px
	face:{string:string$1},mod:{string:string$1},vadjust:{number:number$1},__type__:{object:object$1,string:string$1}},ital:{color:{string:string$1},size:{number:number$1},// px
	face:{string:string$1},mod:{string:string$1},vadjust:{number:number$1},__type__:{object:object$1,string:string$1}},mono:{color:{string:string$1},size:{number:number$1},// px
	face:{string:string$1},mod:{string:string$1},vadjust:{number:number$1},__type__:{object:object$1,string:string$1}},__type__:{object:object$1,string:string$1}},group:{string:string$1,number:number$1,'undefined':'undefined'},heightConstraint:{minimum:{number:number$1},valign:{string:string$1},__type__:{object:object$1,boolean:bool$1,number:number$1}},hidden:{boolean:bool$1},icon:{face:{string:string$1},code:{string:string$1},//'\uf007',
	size:{number:number$1},//50,
	color:{string:string$1},weight:{string:string$1,number:number$1},__type__:{object:object$1}},id:{string:string$1,number:number$1},image:{selected:{string:string$1,'undefined':'undefined'},// --> URL
	unselected:{string:string$1,'undefined':'undefined'},// --> URL
	__type__:{object:object$1,string:string$1}},imagePadding:{top:{number:number$1},right:{number:number$1},bottom:{number:number$1},left:{number:number$1},__type__:{object:object$1,number:number$1}},label:{string:string$1,'undefined':'undefined'},labelHighlightBold:{boolean:bool$1},level:{number:number$1,'undefined':'undefined'},margin:{top:{number:number$1},right:{number:number$1},bottom:{number:number$1},left:{number:number$1},__type__:{object:object$1,number:number$1}},mass:{number:number$1},physics:{boolean:bool$1},scaling:{min:{number:number$1},max:{number:number$1},label:{enabled:{boolean:bool$1},min:{number:number$1},max:{number:number$1},maxVisible:{number:number$1},drawThreshold:{number:number$1},__type__:{object:object$1,boolean:bool$1}},customScalingFunction:{'function':'function'},__type__:{object:object$1}},shadow:{enabled:{boolean:bool$1},color:{string:string$1},size:{number:number$1},x:{number:number$1},y:{number:number$1},__type__:{object:object$1,boolean:bool$1}},shape:{string:['ellipse','circle','database','box','text','image','circularImage','diamond','dot','star','triangle','triangleDown','square','icon','hexagon']},shapeProperties:{borderDashes:{boolean:bool$1,array:array$1},borderRadius:{number:number$1},interpolation:{boolean:bool$1},useImageSize:{boolean:bool$1},useBorderWithImage:{boolean:bool$1},__type__:{object:object$1}},size:{number:number$1},title:{string:string$1,dom:dom,'undefined':'undefined'},value:{number:number$1,'undefined':'undefined'},widthConstraint:{minimum:{number:number$1},maximum:{number:number$1},__type__:{object:object$1,boolean:bool$1,number:number$1}},x:{number:number$1},y:{number:number$1},__type__:{object:object$1}},physics:{enabled:{boolean:bool$1},barnesHut:{theta:{number:number$1},gravitationalConstant:{number:number$1},centralGravity:{number:number$1},springLength:{number:number$1},springConstant:{number:number$1},damping:{number:number$1},avoidOverlap:{number:number$1},__type__:{object:object$1}},forceAtlas2Based:{theta:{number:number$1},gravitationalConstant:{number:number$1},centralGravity:{number:number$1},springLength:{number:number$1},springConstant:{number:number$1},damping:{number:number$1},avoidOverlap:{number:number$1},__type__:{object:object$1}},repulsion:{centralGravity:{number:number$1},springLength:{number:number$1},springConstant:{number:number$1},nodeDistance:{number:number$1},damping:{number:number$1},__type__:{object:object$1}},hierarchicalRepulsion:{centralGravity:{number:number$1},springLength:{number:number$1},springConstant:{number:number$1},nodeDistance:{number:number$1},damping:{number:number$1},avoidOverlap:{number:number$1},__type__:{object:object$1}},maxVelocity:{number:number$1},minVelocity:{number:number$1},// px/s
	solver:{string:['barnesHut','repulsion','hierarchicalRepulsion','forceAtlas2Based']},stabilization:{enabled:{boolean:bool$1},iterations:{number:number$1},// maximum number of iteration to stabilize
	updateInterval:{number:number$1},onlyDynamicEdges:{boolean:bool$1},fit:{boolean:bool$1},__type__:{object:object$1,boolean:bool$1}},timestep:{number:number$1},adaptiveTimestep:{boolean:bool$1},wind:{x:{number:number$1},y:{number:number$1},__type__:{object:object$1}},__type__:{object:object$1,boolean:bool$1}},//globals :
	autoResize:{boolean:bool$1},clickToUse:{boolean:bool$1},locale:{string:string$1},locales:{__any__:{any:any},__type__:{object:object$1}},height:{string:string$1},width:{string:string$1},__type__:{object:object$1}};allOptions$1$1.groups.__any__=allOptions$1$1.nodes;allOptions$1$1.manipulation.controlNodeStyle=allOptions$1$1.nodes;/**
	 * This provides ranges, initial values, steps and dropdown menu choices for the
	 * configuration.
	 *
	 * @remarks
	 * Checkbox: `boolean`
	 *   The value supllied will be used as the initial value.
	 *
	 * Text field: `string`
	 *   The passed text will be used as the initial value. Any text will be
	 *   accepted afterwards.
	 *
	 * Number range: `[number, number, number, number]`
	 *   The meanings are `[initial value, min, max, step]`.
	 *
	 * Dropdown: `[Exclude<string, "color">, ...(string | number | boolean)[]]`
	 *   Translations for people with poor understanding of TypeScript: the first
	 *   value always has to be a string but never `"color"`, the rest can be any
	 *   combination of strings, numbers and booleans.
	 *
	 * Color picker: `["color", string]`
	 *   The first value says this will be a color picker not a dropdown menu. The
	 *   next value is the initial color.
	 */var configureOptions={nodes:{borderWidth:[1,0,10,1],borderWidthSelected:[2,0,10,1],color:{border:['color','#2B7CE9'],background:['color','#97C2FC'],highlight:{border:['color','#2B7CE9'],background:['color','#D2E5FF']},hover:{border:['color','#2B7CE9'],background:['color','#D2E5FF']}},opacity:[0,0,1,0.1],fixed:{x:false,y:false},font:{color:['color','#343434'],size:[14,0,100,1],// px
	face:['arial','verdana','tahoma'],background:['color','none'],strokeWidth:[0,0,50,1],// px
	strokeColor:['color','#ffffff']},//group: 'string',
	hidden:false,labelHighlightBold:true,//icon: {
	//  face: 'string',  //'FontAwesome',
	//  code: 'string',  //'\uf007',
	//  size: [50, 0, 200, 1],  //50,
	//  color: ['color','#2B7CE9']   //'#aa00ff'
	//},
	//image: 'string', // --> URL
	physics:true,scaling:{min:[10,0,200,1],max:[30,0,200,1],label:{enabled:false,min:[14,0,200,1],max:[30,0,200,1],maxVisible:[30,0,200,1],drawThreshold:[5,0,20,1]}},shadow:{enabled:false,color:'rgba(0,0,0,0.5)',size:[10,0,20,1],x:[5,-30,30,1],y:[5,-30,30,1]},shape:['ellipse','box','circle','database','diamond','dot','square','star','text','triangle','triangleDown','hexagon'],shapeProperties:{borderDashes:false,borderRadius:[6,0,20,1],interpolation:true,useImageSize:false},size:[25,0,200,1]},edges:{arrows:{to:{enabled:false,scaleFactor:[1,0,3,0.05],type:'arrow'},middle:{enabled:false,scaleFactor:[1,0,3,0.05],type:'arrow'},from:{enabled:false,scaleFactor:[1,0,3,0.05],type:'arrow'}},arrowStrikethrough:true,color:{color:['color','#848484'],highlight:['color','#848484'],hover:['color','#848484'],inherit:['from','to','both',true,false],opacity:[1,0,1,0.05]},dashes:false,font:{color:['color','#343434'],size:[14,0,100,1],// px
	face:['arial','verdana','tahoma'],background:['color','none'],strokeWidth:[2,0,50,1],// px
	strokeColor:['color','#ffffff'],align:['horizontal','top','middle','bottom']},hidden:false,hoverWidth:[1.5,0,5,0.1],labelHighlightBold:true,physics:true,scaling:{min:[1,0,100,1],max:[15,0,100,1],label:{enabled:true,min:[14,0,200,1],max:[30,0,200,1],maxVisible:[30,0,200,1],drawThreshold:[5,0,20,1]}},selectionWidth:[1.5,0,5,0.1],selfReferenceSize:[20,0,200,1],selfReference:{size:[20,0,200,1],angle:[Math.PI/2,-6*Math.PI,6*Math.PI,Math.PI/8],renderBehindTheNode:true},shadow:{enabled:false,color:'rgba(0,0,0,0.5)',size:[10,0,20,1],x:[5,-30,30,1],y:[5,-30,30,1]},smooth:{enabled:true,type:['dynamic','continuous','discrete','diagonalCross','straightCross','horizontal','vertical','curvedCW','curvedCCW','cubicBezier'],forceDirection:['horizontal','vertical','none'],roundness:[0.5,0,1,0.05]},width:[1,0,30,1]},layout:{//randomSeed: [0, 0, 500, 1],
	//improvedLayout: true,
	hierarchical:{enabled:false,levelSeparation:[150,20,500,5],nodeSpacing:[100,20,500,5],treeSpacing:[200,20,500,5],blockShifting:true,edgeMinimization:true,parentCentralization:true,direction:['UD','DU','LR','RL'],// UD, DU, LR, RL
	sortMethod:['hubsize','directed'],// hubsize, directed
	shakeTowards:['leaves','roots']// leaves, roots
	}},interaction:{dragNodes:true,dragView:true,hideEdgesOnDrag:false,hideEdgesOnZoom:false,hideNodesOnDrag:false,hover:false,keyboard:{enabled:false,speed:{x:[10,0,40,1],y:[10,0,40,1],zoom:[0.02,0,0.1,0.005]},bindToWindow:true},multiselect:false,navigationButtons:false,selectable:true,selectConnectedEdges:true,hoverConnectedEdges:true,tooltipDelay:[300,0,1000,25],zoomView:true,zoomSpeed:[1,0.1,2,0.1]},manipulation:{enabled:false,initiallyActive:false},physics:{enabled:true,barnesHut:{theta:[0.5,0.1,1,0.05],gravitationalConstant:[-2000,-30000,0,50],centralGravity:[0.3,0,10,0.05],springLength:[95,0,500,5],springConstant:[0.04,0,1.2,0.005],damping:[0.09,0,1,0.01],avoidOverlap:[0,0,1,0.01]},forceAtlas2Based:{theta:[0.5,0.1,1,0.05],gravitationalConstant:[-50,-500,0,1],centralGravity:[0.01,0,1,0.005],springLength:[95,0,500,5],springConstant:[0.08,0,1.2,0.005],damping:[0.4,0,1,0.01],avoidOverlap:[0,0,1,0.01]},repulsion:{centralGravity:[0.2,0,10,0.05],springLength:[200,0,500,5],springConstant:[0.05,0,1.2,0.005],nodeDistance:[100,0,500,5],damping:[0.09,0,1,0.01]},hierarchicalRepulsion:{centralGravity:[0.2,0,10,0.05],springLength:[100,0,500,5],springConstant:[0.01,0,1.2,0.005],nodeDistance:[120,0,500,5],damping:[0.09,0,1,0.01],avoidOverlap:[0,0,1,0.01]},maxVelocity:[50,0,150,1],minVelocity:[0.1,0.01,0.5,0.01],solver:['barnesHut','forceAtlas2Based','repulsion','hierarchicalRepulsion'],timestep:[0.5,0.01,1,0.01],wind:{x:[0,-10,10,0.1],y:[0,-10,10,0.1]}//adaptiveTimestep: true
	}};var options=/*#__PURE__*/freeze$2({__proto__:null,allOptions:allOptions$1$1,configureOptions:configureOptions});/**
	 *  The Floyd–Warshall algorithm is an algorithm for finding shortest paths in
	 *  a weighted graph with positive or negative edge weights (but with no negative
	 *  cycles). - https://en.wikipedia.org/wiki/Floyd–Warshall_algorithm
	 */var FloydWarshall=/*#__PURE__*/function(){/**
	   * @ignore
	   */function FloydWarshall(){classCallCheck(this,FloydWarshall);}/**
	   *
	   * @param {Object} body
	   * @param {Array.<Node>} nodesArray
	   * @param {Array.<Edge>} edgesArray
	   * @returns {{}}
	   */createClass(FloydWarshall,[{key:"getDistances",value:function getDistances(body,nodesArray,edgesArray){var D_matrix={};var edges=body.edges;// prepare matrix with large numbers
	for(var i=0;i<nodesArray.length;i++){var node=nodesArray[i];var cell={};D_matrix[node]=cell;for(var j=0;j<nodesArray.length;j++){cell[nodesArray[j]]=i==j?0:1e9;}}// put the weights for the edges in. This assumes unidirectionality.
	for(var _i23=0;_i23<edgesArray.length;_i23++){var edge=edges[edgesArray[_i23]];// edge has to be connected if it counts to the distances. If it is connected to inner clusters it will crash so we also check if it is in the D_matrix
	if(edge.connected===true&&D_matrix[edge.fromId]!==undefined&&D_matrix[edge.toId]!==undefined){D_matrix[edge.fromId][edge.toId]=1;D_matrix[edge.toId][edge.fromId]=1;}}var nodeCount=nodesArray.length;// Adapted FloydWarshall based on unidirectionality to greatly reduce complexity.
	for(var k=0;k<nodeCount;k++){var knode=nodesArray[k];var kcolm=D_matrix[knode];for(var _i24=0;_i24<nodeCount-1;_i24++){var inode=nodesArray[_i24];var icolm=D_matrix[inode];for(var _j7=_i24+1;_j7<nodeCount;_j7++){var jnode=nodesArray[_j7];var jcolm=D_matrix[jnode];var val=Math.min(icolm[jnode],icolm[knode]+kcolm[jnode]);icolm[jnode]=val;jcolm[inode]=val;}}}return D_matrix;}}]);return FloydWarshall;}();// distance finding algorithm
	/**
	 * KamadaKawai positions the nodes initially based on
	 *
	 * "AN ALGORITHM FOR DRAWING GENERAL UNDIRECTED GRAPHS"
	 * -- Tomihisa KAMADA and Satoru KAWAI in 1989
	 *
	 * Possible optimizations in the distance calculation can be implemented.
	 */var KamadaKawai=/*#__PURE__*/function(){/**
	   * @param {Object} body
	   * @param {number} edgeLength
	   * @param {number} edgeStrength
	   */function KamadaKawai(body,edgeLength,edgeStrength){classCallCheck(this,KamadaKawai);this.body=body;this.springLength=edgeLength;this.springConstant=edgeStrength;this.distanceSolver=new FloydWarshall();}/**
	   * Not sure if needed but can be used to update the spring length and spring constant
	   * @param {Object} options
	   */createClass(KamadaKawai,[{key:"setOptions",value:function setOptions(options){if(options){if(options.springLength){this.springLength=options.springLength;}if(options.springConstant){this.springConstant=options.springConstant;}}}/**
	   * Position the system
	   * @param {Array.<Node>} nodesArray
	   * @param {Array.<vis.Edge>} edgesArray
	   * @param {boolean} [ignoreClusters=false]
	   */},{key:"solve",value:function solve(nodesArray,edgesArray){var ignoreClusters=arguments.length>2&&arguments[2]!==undefined?arguments[2]:false;// get distance matrix
	var D_matrix=this.distanceSolver.getDistances(this.body,nodesArray,edgesArray);// distance matrix
	// get the L Matrix
	this._createL_matrix(D_matrix);// get the K Matrix
	this._createK_matrix(D_matrix);// initial E Matrix
	this._createE_matrix();// calculate positions
	var threshold=0.01;var innerThreshold=1;var iterations=0;var maxIterations=Math.max(1000,Math.min(10*this.body.nodeIndices.length,6000));var maxInnerIterations=5;var maxEnergy=1e9;var highE_nodeId=0,dE_dx=0,dE_dy=0,delta_m=0,subIterations=0;while(maxEnergy>threshold&&iterations<maxIterations){iterations+=1;var _this$_getHighestEner=this._getHighestEnergyNode(ignoreClusters);var _this$_getHighestEner2=slicedToArray(_this$_getHighestEner,4);highE_nodeId=_this$_getHighestEner2[0];maxEnergy=_this$_getHighestEner2[1];dE_dx=_this$_getHighestEner2[2];dE_dy=_this$_getHighestEner2[3];delta_m=maxEnergy;subIterations=0;while(delta_m>innerThreshold&&subIterations<maxInnerIterations){subIterations+=1;this._moveNode(highE_nodeId,dE_dx,dE_dy);var _this$_getEnergy=this._getEnergy(highE_nodeId);var _this$_getEnergy2=slicedToArray(_this$_getEnergy,3);delta_m=_this$_getEnergy2[0];dE_dx=_this$_getEnergy2[1];dE_dy=_this$_getEnergy2[2];}}}/**
	   * get the node with the highest energy
	   * @param {boolean} ignoreClusters
	   * @returns {number[]}
	   * @private
	   */},{key:"_getHighestEnergyNode",value:function _getHighestEnergyNode(ignoreClusters){var nodesArray=this.body.nodeIndices;var nodes=this.body.nodes;var maxEnergy=0;var maxEnergyNodeId=nodesArray[0];var dE_dx_max=0,dE_dy_max=0;for(var nodeIdx=0;nodeIdx<nodesArray.length;nodeIdx++){var m=nodesArray[nodeIdx];// by not evaluating nodes with predefined positions we should only move nodes that have no positions.
	if(nodes[m].predefinedPosition===false||nodes[m].isCluster===true&&ignoreClusters===true||nodes[m].options.fixed.x===true||nodes[m].options.fixed.y===true){var _this$_getEnergy3=this._getEnergy(m),_this$_getEnergy4=slicedToArray(_this$_getEnergy3,3),delta_m=_this$_getEnergy4[0],dE_dx=_this$_getEnergy4[1],dE_dy=_this$_getEnergy4[2];if(maxEnergy<delta_m){maxEnergy=delta_m;maxEnergyNodeId=m;dE_dx_max=dE_dx;dE_dy_max=dE_dy;}}}return [maxEnergyNodeId,maxEnergy,dE_dx_max,dE_dy_max];}/**
	   * calculate the energy of a single node
	   * @param {Node.id} m
	   * @returns {number[]}
	   * @private
	   */},{key:"_getEnergy",value:function _getEnergy(m){var _this$E_sums$m=slicedToArray(this.E_sums[m],2),dE_dx=_this$E_sums$m[0],dE_dy=_this$E_sums$m[1];var delta_m=Math.sqrt(Math.pow(dE_dx,2)+Math.pow(dE_dy,2));return [delta_m,dE_dx,dE_dy];}/**
	   * move the node based on it's energy
	   * the dx and dy are calculated from the linear system proposed by Kamada and Kawai
	   * @param {number} m
	   * @param {number} dE_dx
	   * @param {number} dE_dy
	   * @private
	   */},{key:"_moveNode",value:function _moveNode(m,dE_dx,dE_dy){var nodesArray=this.body.nodeIndices;var nodes=this.body.nodes;var d2E_dx2=0;var d2E_dxdy=0;var d2E_dy2=0;var x_m=nodes[m].x;var y_m=nodes[m].y;var km=this.K_matrix[m];var lm=this.L_matrix[m];for(var iIdx=0;iIdx<nodesArray.length;iIdx++){var i=nodesArray[iIdx];if(i!==m){var x_i=nodes[i].x;var y_i=nodes[i].y;var kmat=km[i];var lmat=lm[i];var denominator=1.0/Math.pow(Math.pow(x_m-x_i,2)+Math.pow(y_m-y_i,2),1.5);d2E_dx2+=kmat*(1-lmat*Math.pow(y_m-y_i,2)*denominator);d2E_dxdy+=kmat*(lmat*(x_m-x_i)*(y_m-y_i)*denominator);d2E_dy2+=kmat*(1-lmat*Math.pow(x_m-x_i,2)*denominator);}}// make the variable names easier to make the solving of the linear system easier to read
	var A=d2E_dx2,B=d2E_dxdy,C=dE_dx,D=d2E_dy2,E=dE_dy;// solve the linear system for dx and dy
	var dy=(C/A+E/B)/(B/A-D/B);var dx=-(B*dy+C)/A;// move the node
	nodes[m].x+=dx;nodes[m].y+=dy;// Recalculate E_matrix (should be incremental)
	this._updateE_matrix(m);}/**
	   * Create the L matrix: edge length times shortest path
	   * @param {Object} D_matrix
	   * @private
	   */},{key:"_createL_matrix",value:function _createL_matrix(D_matrix){var nodesArray=this.body.nodeIndices;var edgeLength=this.springLength;this.L_matrix=[];for(var i=0;i<nodesArray.length;i++){this.L_matrix[nodesArray[i]]={};for(var j=0;j<nodesArray.length;j++){this.L_matrix[nodesArray[i]][nodesArray[j]]=edgeLength*D_matrix[nodesArray[i]][nodesArray[j]];}}}/**
	   * Create the K matrix: spring constants times shortest path
	   * @param {Object} D_matrix
	   * @private
	   */},{key:"_createK_matrix",value:function _createK_matrix(D_matrix){var nodesArray=this.body.nodeIndices;var edgeStrength=this.springConstant;this.K_matrix=[];for(var i=0;i<nodesArray.length;i++){this.K_matrix[nodesArray[i]]={};for(var j=0;j<nodesArray.length;j++){this.K_matrix[nodesArray[i]][nodesArray[j]]=edgeStrength*Math.pow(D_matrix[nodesArray[i]][nodesArray[j]],-2);}}}/**
	   *  Create matrix with all energies between nodes
	   *  @private
	   */},{key:"_createE_matrix",value:function _createE_matrix(){var nodesArray=this.body.nodeIndices;var nodes=this.body.nodes;this.E_matrix={};this.E_sums={};for(var mIdx=0;mIdx<nodesArray.length;mIdx++){this.E_matrix[nodesArray[mIdx]]=[];}for(var _mIdx=0;_mIdx<nodesArray.length;_mIdx++){var m=nodesArray[_mIdx];var x_m=nodes[m].x;var y_m=nodes[m].y;var dE_dx=0;var dE_dy=0;for(var iIdx=_mIdx;iIdx<nodesArray.length;iIdx++){var i=nodesArray[iIdx];if(i!==m){var x_i=nodes[i].x;var y_i=nodes[i].y;var denominator=1.0/Math.sqrt(Math.pow(x_m-x_i,2)+Math.pow(y_m-y_i,2));this.E_matrix[m][iIdx]=[this.K_matrix[m][i]*(x_m-x_i-this.L_matrix[m][i]*(x_m-x_i)*denominator),this.K_matrix[m][i]*(y_m-y_i-this.L_matrix[m][i]*(y_m-y_i)*denominator)];this.E_matrix[i][_mIdx]=this.E_matrix[m][iIdx];dE_dx+=this.E_matrix[m][iIdx][0];dE_dy+=this.E_matrix[m][iIdx][1];}}//Store sum
	this.E_sums[m]=[dE_dx,dE_dy];}}/**
	   * Update method, just doing single column (rows are auto-updated) (update all sums)
	   *
	   * @param {number} m
	   * @private
	   */},{key:"_updateE_matrix",value:function _updateE_matrix(m){var nodesArray=this.body.nodeIndices;var nodes=this.body.nodes;var colm=this.E_matrix[m];var kcolm=this.K_matrix[m];var lcolm=this.L_matrix[m];var x_m=nodes[m].x;var y_m=nodes[m].y;var dE_dx=0;var dE_dy=0;for(var iIdx=0;iIdx<nodesArray.length;iIdx++){var i=nodesArray[iIdx];if(i!==m){//Keep old energy value for sum modification below
	var cell=colm[iIdx];var oldDx=cell[0];var oldDy=cell[1];//Calc new energy:
	var x_i=nodes[i].x;var y_i=nodes[i].y;var denominator=1.0/Math.sqrt(Math.pow(x_m-x_i,2)+Math.pow(y_m-y_i,2));var dx=kcolm[i]*(x_m-x_i-lcolm[i]*(x_m-x_i)*denominator);var dy=kcolm[i]*(y_m-y_i-lcolm[i]*(y_m-y_i)*denominator);colm[iIdx]=[dx,dy];dE_dx+=dx;dE_dy+=dy;//add new energy to sum of each column
	var sum=this.E_sums[i];sum[0]+=dx-oldDx;sum[1]+=dy-oldDy;}}//Store sum at -1 index
	this.E_sums[m]=[dE_dx,dE_dy];}}]);return KamadaKawai;}();// Load custom shapes into CanvasRenderingContext2D
	/**
	 * Create a network visualization, displaying nodes and edges.
	 *
	 * @param {Element} container   The DOM element in which the Network will
	 *                                  be created. Normally a div element.
	 * @param {Object} data         An object containing parameters
	 *                              {Array} nodes
	 *                              {Array} edges
	 * @param {Object} options      Options
	 * @constructor Network
	 */function Network(container,data,options){var _context130,_context131,_context132,_context133,_this76=this;if(!(this instanceof Network)){throw new SyntaxError('Constructor must be called with the new operator');}// set constant values
	this.options={};this.defaultOptions={locale:'en',locales:locales,clickToUse:false};extend(this.options,this.defaultOptions);/**
	   * Containers for nodes and edges.
	   *
	   * 'edges' and 'nodes' contain the full definitions of all the network elements.
	   * 'nodeIndices' and 'edgeIndices' contain the id's of the active elements.
	   *
	   * The distinction is important, because a defined node need not be active, i.e.
	   * visible on the canvas. This happens in particular when clusters are defined, in
	   * that case there will be nodes and edges not displayed.
	   * The bottom line is that all code with actions related to visibility, *must* use
	   * 'nodeIndices' and 'edgeIndices', not 'nodes' and 'edges' directly.
	   */this.body={container:container,// See comment above for following fields
	nodes:{},nodeIndices:[],edges:{},edgeIndices:[],emitter:{on:bind$2(_context130=this.on).call(_context130,this),off:bind$2(_context131=this.off).call(_context131,this),emit:bind$2(_context132=this.emit).call(_context132,this),once:bind$2(_context133=this.once).call(_context133,this)},eventListeners:{onTap:function onTap(){},onTouch:function onTouch(){},onDoubleTap:function onDoubleTap(){},onHold:function onHold(){},onDragStart:function onDragStart(){},onDrag:function onDrag(){},onDragEnd:function onDragEnd(){},onMouseWheel:function onMouseWheel(){},onPinch:function onPinch(){},onMouseMove:function onMouseMove(){},onRelease:function onRelease(){},onContext:function onContext(){}},data:{nodes:null,// A DataSet or DataView
	edges:null// A DataSet or DataView
	},functions:{createNode:function createNode(){},createEdge:function createEdge(){},getPointer:function getPointer(){}},modules:{},view:{scale:1,translation:{x:0,y:0}}};// bind the event listeners
	this.bindEventListeners();// setting up all modules
	this.images=new Images(function(){return _this76.body.emitter.emit("_requestRedraw");});// object with images
	this.groups=new Groups();// object with groups
	this.canvas=new Canvas(this.body);// DOM handler
	this.selectionHandler=new SelectionHandler(this.body,this.canvas);// Selection handler
	this.interactionHandler=new InteractionHandler(this.body,this.canvas,this.selectionHandler);// Interaction handler handles all the hammer bindings (that are bound by canvas), key
	this.view=new View(this.body,this.canvas);// camera handler, does animations and zooms
	this.renderer=new CanvasRenderer(this.body,this.canvas);// renderer, starts renderloop, has events that modules can hook into
	this.physics=new PhysicsEngine(this.body);// physics engine, does all the simulations
	this.layoutEngine=new LayoutEngine(this.body);// layout engine for inital layout and hierarchical layout
	this.clustering=new ClusterEngine(this.body);// clustering api
	this.manipulation=new ManipulationSystem(this.body,this.canvas,this.selectionHandler,this.interactionHandler);// data manipulation system
	this.nodesHandler=new NodesHandler(this.body,this.images,this.groups,this.layoutEngine);// Handle adding, deleting and updating of nodes as well as global options
	this.edgesHandler=new EdgesHandler(this.body,this.images,this.groups);// Handle adding, deleting and updating of edges as well as global options
	this.body.modules["kamadaKawai"]=new KamadaKawai(this.body,150,0.05);// Layouting algorithm.
	this.body.modules["clustering"]=this.clustering;// create the DOM elements
	this.canvas._create();// apply options
	this.setOptions(options);// load data (the disable start variable will be the same as the enabled clustering)
	this.setData(data);}// Extend Network with an Emitter mixin
	componentEmitter(Network.prototype);/**
	 * Set options
	 * @param {Object} options
	 */Network.prototype.setOptions=function(options){var _this77=this;if(options===null){options=undefined;// This ensures that options handling doesn't crash in the handling
	}if(options!==undefined){var _errorFound=Validator$1.validate(options,allOptions$1$1);if(_errorFound===true){console.log('%cErrors have been found in the supplied options object.',printStyle$1);}// copy the global fields over
	var fields=['locale','locales','clickToUse'];selectiveDeepExtend(fields,this.options,options);// the hierarchical system can adapt the edges and the physics to it's own options because not all combinations work with the hierarichical system.
	options=this.layoutEngine.setOptions(options.layout,options);this.canvas.setOptions(options);// options for canvas are in globals
	// pass the options to the modules
	this.groups.setOptions(options.groups);this.nodesHandler.setOptions(options.nodes);this.edgesHandler.setOptions(options.edges);this.physics.setOptions(options.physics);this.manipulation.setOptions(options.manipulation,options,this.options);// manipulation uses the locales in the globals
	this.interactionHandler.setOptions(options.interaction);this.renderer.setOptions(options.interaction);// options for rendering are in interaction
	this.selectionHandler.setOptions(options.interaction);// options for selection are in interaction
	// reload the settings of the nodes to apply changes in groups that are not referenced by pointer.
	if(options.groups!==undefined){this.body.emitter.emit("refreshNodes");}// these two do not have options at the moment, here for completeness
	//this.view.setOptions(options.view);
	//this.clustering.setOptions(options.clustering);
	if('configure'in options){if(!this.configurator){this.configurator=new Configurator(this,this.body.container,configureOptions,this.canvas.pixelRatio);}this.configurator.setOptions(options.configure);}// if the configuration system is enabled, copy all options and put them into the config system
	if(this.configurator&&this.configurator.options.enabled===true){var networkOptions={nodes:{},edges:{},layout:{},interaction:{},manipulation:{},physics:{},global:{}};deepExtend(networkOptions.nodes,this.nodesHandler.options);deepExtend(networkOptions.edges,this.edgesHandler.options);deepExtend(networkOptions.layout,this.layoutEngine.options);// load the selectionHandler and render default options in to the interaction group
	deepExtend(networkOptions.interaction,this.selectionHandler.options);deepExtend(networkOptions.interaction,this.renderer.options);deepExtend(networkOptions.interaction,this.interactionHandler.options);deepExtend(networkOptions.manipulation,this.manipulation.options);deepExtend(networkOptions.physics,this.physics.options);// load globals into the global object
	deepExtend(networkOptions.global,this.canvas.options);deepExtend(networkOptions.global,this.options);this.configurator.setModuleOptions(networkOptions);}// handle network global options
	if(options.clickToUse!==undefined){if(options.clickToUse===true){if(this.activator===undefined){this.activator=new Activator_1(this.canvas.frame);this.activator.on('change',function(){_this77.body.emitter.emit("activate");});}}else {if(this.activator!==undefined){this.activator.destroy();delete this.activator;}this.body.emitter.emit("activate");}}else {this.body.emitter.emit("activate");}this.canvas.setSize();// start the physics simulation. Can be safely called multiple times.
	this.body.emitter.emit("startSimulation");}};/**
	 * Update the visible nodes and edges list with the most recent node state.
	 *
	 * Visible nodes are stored in this.body.nodeIndices.
	 * Visible edges are stored in this.body.edgeIndices.
	 * A node or edges is visible if it is not hidden or clustered.
	 *
	 * @private
	 */Network.prototype._updateVisibleIndices=function(){var nodes=this.body.nodes;var edges=this.body.edges;this.body.nodeIndices=[];this.body.edgeIndices=[];for(var nodeId in nodes){if(nodes.hasOwnProperty(nodeId)){if(!this.clustering._isClusteredNode(nodeId)&&nodes[nodeId].options.hidden===false){this.body.nodeIndices.push(nodes[nodeId].id);}}}for(var edgeId in edges){if(edges.hasOwnProperty(edgeId)){var edge=edges[edgeId];// It can happen that this is executed *after* a node edge has been removed,
	// but *before* the edge itself has been removed. Taking this into account.
	var fromNode=nodes[edge.fromId];var toNode=nodes[edge.toId];var edgeNodesPresent=fromNode!==undefined&&toNode!==undefined;var isVisible=!this.clustering._isClusteredEdge(edgeId)&&edge.options.hidden===false&&edgeNodesPresent&&fromNode.options.hidden===false// Also hidden if any of its connecting nodes are hidden
	&&toNode.options.hidden===false;// idem
	if(isVisible){this.body.edgeIndices.push(edge.id);}}}};/**
	 * Bind all events
	 */Network.prototype.bindEventListeners=function(){var _this78=this;// This event will trigger a rebuilding of the cache everything.
	// Used when nodes or edges have been added or removed.
	this.body.emitter.on("_dataChanged",function(){_this78.edgesHandler._updateState();_this78.body.emitter.emit("_dataUpdated");});// this is called when options of EXISTING nodes or edges have changed.
	this.body.emitter.on("_dataUpdated",function(){// Order important in following block
	_this78.clustering._updateState();_this78._updateVisibleIndices();_this78._updateValueRange(_this78.body.nodes);_this78._updateValueRange(_this78.body.edges);// start simulation (can be called safely, even if already running)
	_this78.body.emitter.emit("startSimulation");_this78.body.emitter.emit("_requestRedraw");});};/**
	 * Set nodes and edges, and optionally options as well.
	 *
	 * @param {Object} data              Object containing parameters:
	 *                                   {Array | DataSet | DataView} [nodes] Array with nodes
	 *                                   {Array | DataSet | DataView} [edges] Array with edges
	 *                                   {String} [dot] String containing data in DOT format
	 *                                   {String} [gephi] String containing data in gephi JSON format
	 *                                   {Options} [options] Object with options
	 */Network.prototype.setData=function(data){// reset the physics engine.
	this.body.emitter.emit("resetPhysics");this.body.emitter.emit("_resetData");// unselect all to ensure no selections from old data are carried over.
	this.selectionHandler.unselectAll();if(data&&data.dot&&(data.nodes||data.edges)){throw new SyntaxError('Data must contain either parameter "dot" or '+' parameter pair "nodes" and "edges", but not both.');}// set options
	this.setOptions(data&&data.options);// set all data
	if(data&&data.dot){console.log('The dot property has been deprecated. Please use the static convertDot method to convert DOT into vis.network format and use the normal data format with nodes and edges. This converter is used like this: var data = vis.network.convertDot(dotString);');// parse DOT file
	var dotData=dotparser.DOTToGraph(data.dot);this.setData(dotData);return;}else if(data&&data.gephi){// parse DOT file
	console.log('The gephi property has been deprecated. Please use the static convertGephi method to convert gephi into vis.network format and use the normal data format with nodes and edges. This converter is used like this: var data = vis.network.convertGephi(gephiJson);');var gephiData=parseGephi(data.gephi);this.setData(gephiData);return;}else {this.nodesHandler.setData(data&&data.nodes,true);this.edgesHandler.setData(data&&data.edges,true);}// emit change in data
	this.body.emitter.emit("_dataChanged");// emit data loaded
	this.body.emitter.emit("_dataLoaded");// find a stable position or start animating to a stable position
	this.body.emitter.emit("initPhysics");};/**
	 * Cleans up all bindings of the network, removing it fully from the memory IF the variable is set to null after calling this function.
	 * var network = new vis.Network(..);
	 * network.destroy();
	 * network = null;
	 */Network.prototype.destroy=function(){this.body.emitter.emit("destroy");// clear events
	this.body.emitter.off();this.off();// delete modules
	delete this.groups;delete this.canvas;delete this.selectionHandler;delete this.interactionHandler;delete this.view;delete this.renderer;delete this.physics;delete this.layoutEngine;delete this.clustering;delete this.manipulation;delete this.nodesHandler;delete this.edgesHandler;delete this.configurator;delete this.images;for(var nodeId in this.body.nodes){if(!this.body.nodes.hasOwnProperty(nodeId))continue;delete this.body.nodes[nodeId];}for(var edgeId in this.body.edges){if(!this.body.edges.hasOwnProperty(edgeId))continue;delete this.body.edges[edgeId];}// remove the container and everything inside it recursively
	recursiveDOMDelete(this.body.container);};/**
	 * Update the values of all object in the given array according to the current
	 * value range of the objects in the array.
	 * @param {Object} obj    An object containing a set of Edges or Nodes
	 *                        The objects must have a method getValue() and
	 *                        setValueRange(min, max).
	 * @private
	 */Network.prototype._updateValueRange=function(obj){var id;// determine the range of the objects
	var valueMin=undefined;var valueMax=undefined;var valueTotal=0;for(id in obj){if(obj.hasOwnProperty(id)){var value=obj[id].getValue();if(value!==undefined){valueMin=valueMin===undefined?value:Math.min(value,valueMin);valueMax=valueMax===undefined?value:Math.max(value,valueMax);valueTotal+=value;}}}// adjust the range of all objects
	if(valueMin!==undefined&&valueMax!==undefined){for(id in obj){if(obj.hasOwnProperty(id)){obj[id].setValueRange(valueMin,valueMax,valueTotal);}}}};/**
	 * Returns true when the Network is active.
	 * @returns {boolean}
	 */Network.prototype.isActive=function(){return !this.activator||this.activator.active;};Network.prototype.setSize=function(){return this.canvas.setSize.apply(this.canvas,arguments);};Network.prototype.canvasToDOM=function(){return this.canvas.canvasToDOM.apply(this.canvas,arguments);};Network.prototype.DOMtoCanvas=function(){return this.canvas.DOMtoCanvas.apply(this.canvas,arguments);};/**
	 * Nodes can be in clusters. Clusters can also be in clusters. This function returns and array of
	 * nodeIds showing where the node is.
	 *
	 * If any nodeId in the chain, especially the first passed in as a parameter, is not present in
	 * the current nodes list, an empty array is returned.
	 *
	 * Example:
	 * cluster 'A' contains cluster 'B',
	 * cluster 'B' contains cluster 'C',
	 * cluster 'C' contains node 'fred'.
	 * `jsnetwork.clustering.findNode('fred')` will return `['A','B','C','fred']`.
	 *
	 * @param {string|number} nodeId
	 * @returns {Array}
	 */Network.prototype.findNode=function(){return this.clustering.findNode.apply(this.clustering,arguments);};Network.prototype.isCluster=function(){return this.clustering.isCluster.apply(this.clustering,arguments);};Network.prototype.openCluster=function(){return this.clustering.openCluster.apply(this.clustering,arguments);};Network.prototype.cluster=function(){return this.clustering.cluster.apply(this.clustering,arguments);};Network.prototype.getNodesInCluster=function(){return this.clustering.getNodesInCluster.apply(this.clustering,arguments);};Network.prototype.clusterByConnection=function(){return this.clustering.clusterByConnection.apply(this.clustering,arguments);};Network.prototype.clusterByHubsize=function(){return this.clustering.clusterByHubsize.apply(this.clustering,arguments);};Network.prototype.updateClusteredNode=function(){return this.clustering.updateClusteredNode.apply(this.clustering,arguments);};Network.prototype.getClusteredEdges=function(){return this.clustering.getClusteredEdges.apply(this.clustering,arguments);};Network.prototype.getBaseEdge=function(){return this.clustering.getBaseEdge.apply(this.clustering,arguments);};Network.prototype.getBaseEdges=function(){return this.clustering.getBaseEdges.apply(this.clustering,arguments);};Network.prototype.updateEdge=function(){return this.clustering.updateEdge.apply(this.clustering,arguments);};/**
	 * This method will cluster all nodes with 1 edge with their respective connected node.
	 * The options object is explained in full <a data-scroll="" data-options="{ &quot;easing&quot;: &quot;easeInCubic&quot; }" href="#optionsObject">below</a>.
	 *
	 * @param {object} [options]
	 * @returns {undefined}
	 */Network.prototype.clusterOutliers=function(){return this.clustering.clusterOutliers.apply(this.clustering,arguments);};Network.prototype.getSeed=function(){return this.layoutEngine.getSeed.apply(this.layoutEngine,arguments);};Network.prototype.enableEditMode=function(){return this.manipulation.enableEditMode.apply(this.manipulation,arguments);};Network.prototype.disableEditMode=function(){return this.manipulation.disableEditMode.apply(this.manipulation,arguments);};Network.prototype.addNodeMode=function(){return this.manipulation.addNodeMode.apply(this.manipulation,arguments);};Network.prototype.editNode=function(){return this.manipulation.editNode.apply(this.manipulation,arguments);};Network.prototype.editNodeMode=function(){console.log("Deprecated: Please use editNode instead of editNodeMode.");return this.manipulation.editNode.apply(this.manipulation,arguments);};Network.prototype.addEdgeMode=function(){return this.manipulation.addEdgeMode.apply(this.manipulation,arguments);};Network.prototype.editEdgeMode=function(){return this.manipulation.editEdgeMode.apply(this.manipulation,arguments);};Network.prototype.deleteSelected=function(){return this.manipulation.deleteSelected.apply(this.manipulation,arguments);};Network.prototype.getPositions=function(){return this.nodesHandler.getPositions.apply(this.nodesHandler,arguments);};Network.prototype.getPosition=function(){return this.nodesHandler.getPosition.apply(this.nodesHandler,arguments);};Network.prototype.storePositions=function(){return this.nodesHandler.storePositions.apply(this.nodesHandler,arguments);};Network.prototype.moveNode=function(){return this.nodesHandler.moveNode.apply(this.nodesHandler,arguments);};Network.prototype.getBoundingBox=function(){return this.nodesHandler.getBoundingBox.apply(this.nodesHandler,arguments);};Network.prototype.getConnectedNodes=function(objectId){if(this.body.nodes[objectId]!==undefined){return this.nodesHandler.getConnectedNodes.apply(this.nodesHandler,arguments);}else {return this.edgesHandler.getConnectedNodes.apply(this.edgesHandler,arguments);}};Network.prototype.getConnectedEdges=function(){return this.nodesHandler.getConnectedEdges.apply(this.nodesHandler,arguments);};Network.prototype.startSimulation=function(){return this.physics.startSimulation.apply(this.physics,arguments);};Network.prototype.stopSimulation=function(){return this.physics.stopSimulation.apply(this.physics,arguments);};Network.prototype.stabilize=function(){return this.physics.stabilize.apply(this.physics,arguments);};Network.prototype.getSelection=function(){return this.selectionHandler.getSelection.apply(this.selectionHandler,arguments);};Network.prototype.setSelection=function(){return this.selectionHandler.setSelection.apply(this.selectionHandler,arguments);};Network.prototype.getSelectedNodes=function(){return this.selectionHandler.getSelectedNodes.apply(this.selectionHandler,arguments);};Network.prototype.getSelectedEdges=function(){return this.selectionHandler.getSelectedEdges.apply(this.selectionHandler,arguments);};Network.prototype.getNodeAt=function(){var node=this.selectionHandler.getNodeAt.apply(this.selectionHandler,arguments);if(node!==undefined&&node.id!==undefined){return node.id;}return node;};Network.prototype.getEdgeAt=function(){var edge=this.selectionHandler.getEdgeAt.apply(this.selectionHandler,arguments);if(edge!==undefined&&edge.id!==undefined){return edge.id;}return edge;};Network.prototype.selectNodes=function(){return this.selectionHandler.selectNodes.apply(this.selectionHandler,arguments);};Network.prototype.selectEdges=function(){return this.selectionHandler.selectEdges.apply(this.selectionHandler,arguments);};Network.prototype.unselectAll=function(){this.selectionHandler.unselectAll.apply(this.selectionHandler,arguments);this.redraw();};Network.prototype.redraw=function(){return this.renderer.redraw.apply(this.renderer,arguments);};Network.prototype.getScale=function(){return this.view.getScale.apply(this.view,arguments);};Network.prototype.getViewPosition=function(){return this.view.getViewPosition.apply(this.view,arguments);};Network.prototype.fit=function(){return this.view.fit.apply(this.view,arguments);};Network.prototype.moveTo=function(){return this.view.moveTo.apply(this.view,arguments);};Network.prototype.focus=function(){return this.view.focus.apply(this.view,arguments);};Network.prototype.releaseNode=function(){return this.view.releaseNode.apply(this.view,arguments);};Network.prototype.getOptionsFromConfigurator=function(){var options={};if(this.configurator){options=this.configurator.getOptions.apply(this.configurator);}return options;};var parseDOTNetwork=DOTToGraph_1;// DataSet, utils etc. can't be reexported here because that would cause stack

	var nsNetwork = /*#__PURE__*/Object.freeze({
		__proto__: null,
		Network: Network,
		NetworkImages: Images,
		networkDOTParser: dotparser$1,
		networkGephiParser: gephiParser,
		networkOptions: options,
		parseDOTNetwork: parseDOTNetwork,
		parseGephiNetwork: parseGephi
	});

	var $find = arrayIteration.find;
	var FIND = 'find';
	var SKIPS_HOLES = true;
	var USES_TO_LENGTH$a = arrayMethodUsesToLength(FIND); // Shouldn't skip holes

	if (FIND in []) Array(1)[FIND](function () {
	  SKIPS_HOLES = false;
	}); // `Array.prototype.find` method
	// https://tc39.github.io/ecma262/#sec-array.prototype.find

	_export({
	  target: 'Array',
	  proto: true,
	  forced: SKIPS_HOLES || !USES_TO_LENGTH$a
	}, {
	  find: function find(callbackfn
	  /* , that = undefined */
	  ) {
	    return $find(this, callbackfn, arguments.length > 1 ? arguments[1] : undefined);
	  }
	}); // https://tc39.github.io/ecma262/#sec-array.prototype-@@unscopables

	var find = entryVirtual('Array').find;

	var ArrayPrototype$j = Array.prototype;

	var find_1 = function (it) {
	  var own = it.find;
	  return it === ArrayPrototype$j || it instanceof Array && own === ArrayPrototype$j.find ? find : own;
	};

	var find$1 = find_1;

	var find$2 = find$1;

	// https://tc39.github.io/ecma262/#sec-string.prototype.repeat


	var stringRepeat = ''.repeat || function repeat(count) {
	  var str = String(requireObjectCoercible(this));
	  var result = '';
	  var n = toInteger(count);
	  if (n < 0 || n == Infinity) throw RangeError('Wrong number of repetitions');

	  for (; n > 0; (n >>>= 1) && (str += str)) if (n & 1) result += str;

	  return result;
	};

	// https://tc39.github.io/ecma262/#sec-string.prototype.repeat

	_export({
	  target: 'String',
	  proto: true
	}, {
	  repeat: stringRepeat
	});

	var repeat = entryVirtual('String').repeat;

	var StringPrototype$2 = String.prototype;

	var repeat_1 = function (it) {
	  var own = it.repeat;
	  return typeof it === 'string' || it === StringPrototype$2 || it instanceof String && own === StringPrototype$2.repeat ? repeat : own;
	};

	var repeat$1 = repeat_1;

	var repeat$2 = repeat$1;

	var propagating = createCommonjsModule(function (module, exports) {

	  (function (factory) {
	    {
	      // Node. Does not work with strict CommonJS, but
	      // only CommonJS-like environments that support module.exports,
	      // like Node.
	      module.exports = factory();
	    }
	  })(function () {
	    var _firstTarget = null; // singleton, will contain the target element where the touch event started

	    /**
	     * Extend an Hammer.js instance with event propagation.
	     *
	     * Features:
	     * - Events emitted by hammer will propagate in order from child to parent
	     *   elements.
	     * - Events are extended with a function `event.stopPropagation()` to stop
	     *   propagation to parent elements.
	     * - An option `preventDefault` to stop all default browser behavior.
	     *
	     * Usage:
	     *   var hammer = propagatingHammer(new Hammer(element));
	     *   var hammer = propagatingHammer(new Hammer(element), {preventDefault: true});
	     *
	     * @param {Hammer.Manager} hammer   An hammer instance.
	     * @param {Object} [options]        Available options:
	     *                                  - `preventDefault: true | false | 'mouse' | 'touch' | 'pen'`.
	     *                                    Enforce preventing the default browser behavior.
	     *                                    Cannot be set to `false`.
	     * @return {Hammer.Manager} Returns the same hammer instance with extended
	     *                          functionality
	     */

	    return function propagating(hammer, options) {
	      var _options = options || {
	        preventDefault: false
	      };

	      if (hammer.Manager) {
	        // This looks like the Hammer constructor.
	        // Overload the constructors with our own.
	        var Hammer = hammer;

	        var PropagatingHammer = function (element, options) {
	          var o = Object.create(_options);
	          if (options) Hammer.assign(o, options);
	          return propagating(new Hammer(element, o), o);
	        };

	        Hammer.assign(PropagatingHammer, Hammer);

	        PropagatingHammer.Manager = function (element, options) {
	          var o = Object.create(_options);
	          if (options) Hammer.assign(o, options);
	          return propagating(new Hammer.Manager(element, o), o);
	        };

	        return PropagatingHammer;
	      } // create a wrapper object which will override the functions
	      // `on`, `off`, `destroy`, and `emit` of the hammer instance


	      var wrapper = Object.create(hammer); // attach to DOM element

	      var element = hammer.element;
	      if (!element.hammer) element.hammer = [];
	      element.hammer.push(wrapper); // register an event to catch the start of a gesture and store the
	      // target in a singleton

	      hammer.on('hammer.input', function (event) {
	        if (_options.preventDefault === true || _options.preventDefault === event.pointerType) {
	          event.preventDefault();
	        }

	        if (event.isFirst) {
	          _firstTarget = event.target;
	        }
	      });
	      /** @type {Object.<String, Array.<function>>} */

	      wrapper._handlers = {};
	      /**
	       * Register a handler for one or multiple events
	       * @param {String} events    A space separated string with events
	       * @param {function} handler A callback function, called as handler(event)
	       * @returns {Hammer.Manager} Returns the hammer instance
	       */

	      wrapper.on = function (events, handler) {
	        // register the handler
	        split(events).forEach(function (event) {
	          var _handlers = wrapper._handlers[event];

	          if (!_handlers) {
	            wrapper._handlers[event] = _handlers = []; // register the static, propagated handler

	            hammer.on(event, propagatedHandler);
	          }

	          _handlers.push(handler);
	        });
	        return wrapper;
	      };
	      /**
	       * Unregister a handler for one or multiple events
	       * @param {String} events      A space separated string with events
	       * @param {function} [handler] Optional. The registered handler. If not
	       *                             provided, all handlers for given events
	       *                             are removed.
	       * @returns {Hammer.Manager}   Returns the hammer instance
	       */


	      wrapper.off = function (events, handler) {
	        // unregister the handler
	        split(events).forEach(function (event) {
	          var _handlers = wrapper._handlers[event];

	          if (_handlers) {
	            _handlers = handler ? _handlers.filter(function (h) {
	              return h !== handler;
	            }) : [];

	            if (_handlers.length > 0) {
	              wrapper._handlers[event] = _handlers;
	            } else {
	              // remove static, propagated handler
	              hammer.off(event, propagatedHandler);
	              delete wrapper._handlers[event];
	            }
	          }
	        });
	        return wrapper;
	      };
	      /**
	       * Emit to the event listeners
	       * @param {string} eventType
	       * @param {Event} event
	       */


	      wrapper.emit = function (eventType, event) {
	        _firstTarget = event.target;
	        hammer.emit(eventType, event);
	      };

	      wrapper.destroy = function () {
	        // Detach from DOM element
	        var hammers = hammer.element.hammer;
	        var idx = hammers.indexOf(wrapper);
	        if (idx !== -1) hammers.splice(idx, 1);
	        if (!hammers.length) delete hammer.element.hammer; // clear all handlers

	        wrapper._handlers = {}; // call original hammer destroy

	        hammer.destroy();
	      }; // split a string with space separated words


	      function split(events) {
	        return events.match(/[^ ]+/g);
	      }
	      /**
	       * A static event handler, applying event propagation.
	       * @param {Object} event
	       */


	      function propagatedHandler(event) {
	        // let only a single hammer instance handle this event
	        if (event.type !== 'hammer.input') {
	          // it is possible that the same srcEvent is used with multiple hammer events,
	          // we keep track on which events are handled in an object _handled
	          if (!event.srcEvent._handled) {
	            event.srcEvent._handled = {};
	          }

	          if (event.srcEvent._handled[event.type]) {
	            return;
	          } else {
	            event.srcEvent._handled[event.type] = true;
	          }
	        } // attach a stopPropagation function to the event


	        var stopped = false;

	        event.stopPropagation = function () {
	          stopped = true;
	        }; //wrap the srcEvent's stopPropagation to also stop hammer propagation:


	        var srcStop = event.srcEvent.stopPropagation.bind(event.srcEvent);

	        if (typeof srcStop == "function") {
	          event.srcEvent.stopPropagation = function () {
	            srcStop();
	            event.stopPropagation();
	          };
	        } // attach firstTarget property to the event


	        event.firstTarget = _firstTarget; // propagate over all elements (until stopped)

	        var elem = _firstTarget;

	        while (elem && !stopped) {
	          var elemHammer = elem.hammer;

	          if (elemHammer) {
	            var _handlers;

	            for (var k = 0; k < elemHammer.length; k++) {
	              _handlers = elemHammer[k]._handlers[event.type];
	              if (_handlers) for (var i = 0; i < _handlers.length && !stopped; i++) {
	                _handlers[i](event);
	              }
	            }
	          }

	          elem = elem.parentNode;
	        }
	      }

	      return wrapper;
	    };
	  });
	});

	function _createForOfIteratorHelper$3(o){if(typeof symbol$4==="undefined"||getIteratorMethod$1(o)==null){if(isArray$3(o)||(o=_unsupportedIterableToArray$4(o))){var i=0;var F=function F(){};return {s:F,n:function n(){if(i>=o.length)return {done:true};return {done:false,value:o[i++]};},e:function e(_e){throw _e;},f:F};}throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");}var it,normalCompletion=true,didErr=false,err;return {s:function s(){it=getIterator$1(o);},n:function n(){var step=it.next();normalCompletion=step.done;return step;},e:function e(_e2){didErr=true;err=_e2;},f:function f(){try{if(!normalCompletion&&it.return!=null)it.return();}finally{if(didErr)throw err;}}};}function _unsupportedIterableToArray$4(o,minLen){var _context95;if(!o)return;if(typeof o==="string")return _arrayLikeToArray$4(o,minLen);var n=slice$3(_context95=Object.prototype.toString.call(o)).call(_context95,8,-1);if(n==="Object"&&o.constructor)n=o.constructor.name;if(n==="Map"||n==="Set")return from_1$2(o);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return _arrayLikeToArray$4(o,minLen);}function _arrayLikeToArray$4(arr,len){if(len==null||len>arr.length)len=arr.length;for(var i=0,arr2=new Array(len);i<len;i++){arr2[i]=arr[i];}return arr2;}function _createSuper$2(Derived){var hasNativeReflectConstruct=_isNativeReflectConstruct$2();return function(){var Super=getPrototypeOf$3(Derived),result;if(hasNativeReflectConstruct){var NewTarget=getPrototypeOf$3(this).constructor;result=construct$3(Super,arguments,NewTarget);}else {result=Super.apply(this,arguments);}return possibleConstructorReturn(this,result);};}function _isNativeReflectConstruct$2(){if(typeof Reflect==="undefined"||!construct$3)return false;if(construct$3.sham)return false;if(typeof Proxy==="function")return true;try{Date.prototype.toString.call(construct$3(Date,[],function(){}));return true;}catch(e){return false;}}function ownKeys$4(object,enumerableOnly){var keys=keys$6(object);if(getOwnPropertySymbols$2){var symbols=getOwnPropertySymbols$2(object);if(enumerableOnly)symbols=filter$2(symbols).call(symbols,function(sym){return getOwnPropertyDescriptor$3(object,sym).enumerable;});keys.push.apply(keys,symbols);}return keys;}function _objectSpread$3(target){for(var i=1;i<arguments.length;i++){var source=arguments[i]!=null?arguments[i]:{};if(i%2){var _context191;forEach$3(_context191=ownKeys$4(Object(source),true)).call(_context191,function(key){defineProperty$8(target,key,source[key]);});}else if(getOwnPropertyDescriptors$2){defineProperties$1(target,getOwnPropertyDescriptors$2(source));}else {var _context192;forEach$3(_context192=ownKeys$4(Object(source))).call(_context192,function(key){defineProperty$2(target,key,getOwnPropertyDescriptor$3(source,key));});}}return target;}// use this instance. Else, load via commonjs.
	//
	// Note: This doesn't work in ESM.
	var moment$1=typeof window!=='undefined'&&window['moment']||moment;// utility functions
	// parse ASP.Net Date pattern,
	// for example '/Date(1198908717056)/' or '/Date(1198908717056-0700)/'
	// code from http://momentjs.com/
	var ASPDateRegex$2=/^\/?Date\((-?\d+)/i;/**
	 * Convert an object into another type
	 *
	 * @param object - Value of unknown type.
	 * @param type - Name of the desired type.
	 *
	 * @returns Object in the desired type.
	 * @throws Error
	 */function convert$1(object,type){var match;if(object===undefined){return undefined;}if(object===null){return null;}if(!type){return object;}if(!(typeof type==="string")&&!(type instanceof String)){throw new Error("Type must be a string");}//noinspection FallthroughInSwitchStatementJS
	switch(type){case"boolean":case"Boolean":return Boolean(object);case"number":case"Number":if(isString(object)&&!isNaN(Date.parse(object))){return moment(object).valueOf();}else {// @TODO: I don't think that Number and String constructors are a good idea.
	// This could also fail if the object doesn't have valueOf method or if it's redefined.
	// For example: Object.create(null) or { valueOf: 7 }.
	return Number(object.valueOf());}case"string":case"String":return String(object);case"Date":if(isNumber(object)){return new Date(object);}if(object instanceof Date){return new Date(object.valueOf());}else if(moment.isMoment(object)){return new Date(object.valueOf());}if(isString(object)){match=ASPDateRegex$2.exec(object);if(match){// object is an ASP date
	return new Date(Number(match[1]));// parse number
	}else {return moment(new Date(object)).toDate();// parse string
	}}else {throw new Error("Cannot convert object of type "+getType(object)+" to type Date");}case"Moment":if(isNumber(object)){return moment(object);}if(object instanceof Date){return moment(object.valueOf());}else if(moment.isMoment(object)){return moment(object);}if(isString(object)){match=ASPDateRegex$2.exec(object);if(match){// object is an ASP date
	return moment(Number(match[1]));// parse number
	}else {return moment(object);// parse string
	}}else {throw new Error("Cannot convert object of type "+getType(object)+" to type Date");}case"ISODate":if(isNumber(object)){return new Date(object);}else if(object instanceof Date){return object.toISOString();}else if(moment.isMoment(object)){return object.toDate().toISOString();}else if(isString(object)){match=ASPDateRegex$2.exec(object);if(match){// object is an ASP date
	return new Date(Number(match[1])).toISOString();// parse number
	}else {return moment(object).format();// ISO 8601
	}}else {throw new Error("Cannot convert object of type "+getType(object)+" to type ISODate");}case"ASPDate":if(isNumber(object)){return "/Date("+object+")/";}else if(object instanceof Date||moment.isMoment(object)){return "/Date("+object.valueOf()+")/";}else if(isString(object)){match=ASPDateRegex$2.exec(object);var value;if(match){// object is an ASP date
	value=new Date(Number(match[1])).valueOf();// parse number
	}else {value=new Date(object).valueOf();// parse string
	}return "/Date("+value+")/";}else {throw new Error("Cannot convert object of type "+getType(object)+" to type ASPDate");}default:throw new Error("Unknown type ".concat(type));}}/**
	 * Create a Data Set like wrapper to seamlessly coerce data types.
	 *
	 * @param rawDS - The Data Set with raw uncoerced data.
	 * @param type - A record assigning a data type to property name.
	 *
	 * @remarks
	 * The write operations (`add`, `remove`, `update` and `updateOnly`) write into
	 * the raw (uncoerced) data set. These values are then picked up by a pipe
	 * which coerces the values using the [[convert]] function and feeds them into
	 * the coerced data set. When querying (`forEach`, `get`, `getIds`, `off` and
	 * `on`) the values are then fetched from the coerced data set and already have
	 * the required data types. The values are coerced only once when inserted and
	 * then the same value is returned each time until it is updated or deleted.
	 *
	 * For example: `typeCoercedDataSet.add({ id: 7, start: "2020-01-21" })` would
	 * result in `typeCoercedDataSet.get(7)` returning `{ id: 7, start: moment(new
	 * Date("2020-01-21")).toDate() }`.
	 *
	 * Use the dispose method prior to throwing a reference to this away. Otherwise
	 * the pipe connecting the two Data Sets will keep the unaccessible coerced
	 * Data Set alive and updated as long as the raw Data Set exists.
	 *
	 * @returns A Data Set like object that saves data into the raw Data Set and
	 * retrieves them from the coerced Data Set.
	 */function typeCoerceDataSet(rawDS){var _context,_context3,_context4,_context5,_context6,_context7;var type=arguments.length>1&&arguments[1]!==undefined?arguments[1]:{start:"Date",end:"Date"};var idProp=rawDS._idProp;var coercedDS=new DataSet({fieldId:idProp});var pipe=map$5(_context=createNewDataPipeFrom(rawDS)).call(_context,function(item){var _context2;return reduce$2(_context2=keys$6(item)).call(_context2,function(acc,key){acc[key]=convert$1(item[key],type[key]);return acc;},{});}).to(coercedDS);pipe.all().start();return {// Write only.
	add:function add(){var _rawDS$getDataSet;return (_rawDS$getDataSet=rawDS.getDataSet()).add.apply(_rawDS$getDataSet,arguments);},remove:function remove(){var _rawDS$getDataSet2;return (_rawDS$getDataSet2=rawDS.getDataSet()).remove.apply(_rawDS$getDataSet2,arguments);},update:function update(){var _rawDS$getDataSet3;return (_rawDS$getDataSet3=rawDS.getDataSet()).update.apply(_rawDS$getDataSet3,arguments);},updateOnly:function updateOnly(){var _rawDS$getDataSet4;return (_rawDS$getDataSet4=rawDS.getDataSet()).updateOnly.apply(_rawDS$getDataSet4,arguments);},// Read only.
	forEach:bind$2(_context3=forEach$3(coercedDS)).call(_context3,coercedDS),get:bind$2(_context4=coercedDS.get).call(_context4,coercedDS),getIds:bind$2(_context5=coercedDS.getIds).call(_context5,coercedDS),off:bind$2(_context6=coercedDS.off).call(_context6,coercedDS),on:bind$2(_context7=coercedDS.on).call(_context7,coercedDS),get length(){return coercedDS.length;},// Non standard.
	idProp:idProp,type:type,rawDS:rawDS,coercedDS:coercedDS,dispose:function dispose(){return pipe.stop();}};}var util$1=_objectSpread$3(_objectSpread$3({},util),{},{convert:convert$1});/** Prototype for visual components */var Component=/*#__PURE__*/function(){/**
	 * @param {{dom: Object, domProps: Object, emitter: Emitter, range: Range}} [body]
	 * @param {Object} [options]
	 */function Component(body,options){classCallCheck(this,Component);// eslint-disable-line no-unused-vars
	this.options=null;this.props=null;}/**
	   * Set options for the component. The new options will be merged into the
	   * current options.
	   * @param {Object} options
	   */createClass(Component,[{key:"setOptions",value:function setOptions(options){if(options){util$1.extend(this.options,options);}}/**
	   * Repaint the component
	   * @return {boolean} Returns true if the component is resized
	   */},{key:"redraw",value:function redraw(){// should be implemented by the component
	return false;}/**
	   * Destroy the component. Cleanup DOM and event listeners
	   */},{key:"destroy",value:function destroy(){}// should be implemented by the component
	/**
	   * Test whether the component is resized since the last time _isResized() was
	   * called.
	   * @return {Boolean} Returns true if the component is resized
	   * @protected
	   */},{key:"_isResized",value:function _isResized(){var resized=this.props._previousWidth!==this.props.width||this.props._previousHeight!==this.props.height;this.props._previousWidth=this.props.width;this.props._previousHeight=this.props.height;return resized;}}]);return Component;}();/**
	 * used in Core to convert the options into a volatile variable
	 * 
	 * @param {function} moment
	 * @param {Object} body
	 * @param {Array | Object} hiddenDates
	 * @returns {number}
	 */function convertHiddenOptions(moment,body,hiddenDates){if(hiddenDates&&!isArray$3(hiddenDates)){return convertHiddenOptions(moment,body,[hiddenDates]);}body.hiddenDates=[];if(hiddenDates){if(isArray$3(hiddenDates)==true){var _context8;for(var i=0;i<hiddenDates.length;i++){if(repeat$2(hiddenDates[i])===undefined){var dateItem={};dateItem.start=moment(hiddenDates[i].start).toDate().valueOf();dateItem.end=moment(hiddenDates[i].end).toDate().valueOf();body.hiddenDates.push(dateItem);}}sort$2(_context8=body.hiddenDates).call(_context8,function(a,b){return a.start-b.start;});// sort by start time
	}}}/**
	 * create new entrees for the repeating hidden dates
	 *
	 * @param {function} moment
	 * @param {Object} body
	 * @param {Array | Object} hiddenDates
	 * @returns {null}
	 */function updateHiddenDates(moment,body,hiddenDates){if(hiddenDates&&!isArray$3(hiddenDates)){return updateHiddenDates(moment,body,[hiddenDates]);}if(hiddenDates&&body.domProps.centerContainer.width!==undefined){convertHiddenOptions(moment,body,hiddenDates);var start=moment(body.range.start);var end=moment(body.range.end);var totalRange=body.range.end-body.range.start;var pixelTime=totalRange/body.domProps.centerContainer.width;for(var i=0;i<hiddenDates.length;i++){if(repeat$2(hiddenDates[i])!==undefined){var startDate=moment(hiddenDates[i].start);var endDate=moment(hiddenDates[i].end);if(startDate._d=="Invalid Date"){throw new Error("Supplied start date is not valid: ".concat(hiddenDates[i].start));}if(endDate._d=="Invalid Date"){throw new Error("Supplied end date is not valid: ".concat(hiddenDates[i].end));}var duration=endDate-startDate;if(duration>=4*pixelTime){var offset=0;var runUntil=end.clone();switch(repeat$2(hiddenDates[i])){case"daily":// case of time
	if(startDate.day()!=endDate.day()){offset=1;}startDate.dayOfYear(start.dayOfYear());startDate.year(start.year());startDate.subtract(7,'days');endDate.dayOfYear(start.dayOfYear());endDate.year(start.year());endDate.subtract(7-offset,'days');runUntil.add(1,'weeks');break;case"weekly":{var dayOffset=endDate.diff(startDate,'days');var day=startDate.day();// set the start date to the range.start
	startDate.date(start.date());startDate.month(start.month());startDate.year(start.year());endDate=startDate.clone();// force
	startDate.day(day);endDate.day(day);endDate.add(dayOffset,'days');startDate.subtract(1,'weeks');endDate.subtract(1,'weeks');runUntil.add(1,'weeks');break;}case"monthly":if(startDate.month()!=endDate.month()){offset=1;}startDate.month(start.month());startDate.year(start.year());startDate.subtract(1,'months');endDate.month(start.month());endDate.year(start.year());endDate.subtract(1,'months');endDate.add(offset,'months');runUntil.add(1,'months');break;case"yearly":if(startDate.year()!=endDate.year()){offset=1;}startDate.year(start.year());startDate.subtract(1,'years');endDate.year(start.year());endDate.subtract(1,'years');endDate.add(offset,'years');runUntil.add(1,'years');break;default:console.log("Wrong repeat format, allowed are: daily, weekly, monthly, yearly. Given:",repeat$2(hiddenDates[i]));return;}while(startDate<runUntil){body.hiddenDates.push({start:startDate.valueOf(),end:endDate.valueOf()});switch(repeat$2(hiddenDates[i])){case"daily":startDate.add(1,'days');endDate.add(1,'days');break;case"weekly":startDate.add(1,'weeks');endDate.add(1,'weeks');break;case"monthly":startDate.add(1,'months');endDate.add(1,'months');break;case"yearly":startDate.add(1,'y');endDate.add(1,'y');break;default:console.log("Wrong repeat format, allowed are: daily, weekly, monthly, yearly. Given:",repeat$2(hiddenDates[i]));return;}}body.hiddenDates.push({start:startDate.valueOf(),end:endDate.valueOf()});}}}// remove duplicates, merge where possible
	removeDuplicates(body);// ensure the new positions are not on hidden dates
	var startHidden=getIsHidden(body.range.start,body.hiddenDates);var endHidden=getIsHidden(body.range.end,body.hiddenDates);var rangeStart=body.range.start;var rangeEnd=body.range.end;if(startHidden.hidden==true){rangeStart=body.range.startToFront==true?startHidden.startDate-1:startHidden.endDate+1;}if(endHidden.hidden==true){rangeEnd=body.range.endToFront==true?endHidden.startDate-1:endHidden.endDate+1;}if(startHidden.hidden==true||endHidden.hidden==true){body.range._applyRange(rangeStart,rangeEnd);}}}/**
	 * remove duplicates from the hidden dates list. Duplicates are evil. They mess everything up.
	 * Scales with N^2
	 *
	 * @param {Object} body
	 */function removeDuplicates(body){var _context9;var hiddenDates=body.hiddenDates;var safeDates=[];for(var i=0;i<hiddenDates.length;i++){for(var j=0;j<hiddenDates.length;j++){if(i!=j&&hiddenDates[j].remove!=true&&hiddenDates[i].remove!=true){// j inside i
	if(hiddenDates[j].start>=hiddenDates[i].start&&hiddenDates[j].end<=hiddenDates[i].end){hiddenDates[j].remove=true;}// j start inside i
	else if(hiddenDates[j].start>=hiddenDates[i].start&&hiddenDates[j].start<=hiddenDates[i].end){hiddenDates[i].end=hiddenDates[j].end;hiddenDates[j].remove=true;}// j end inside i
	else if(hiddenDates[j].end>=hiddenDates[i].start&&hiddenDates[j].end<=hiddenDates[i].end){hiddenDates[i].start=hiddenDates[j].start;hiddenDates[j].remove=true;}}}}for(i=0;i<hiddenDates.length;i++){if(hiddenDates[i].remove!==true){safeDates.push(hiddenDates[i]);}}body.hiddenDates=safeDates;sort$2(_context9=body.hiddenDates).call(_context9,function(a,b){return a.start-b.start;});// sort by start time
	}/**
	 * Prints dates to console
	 * @param {array} dates
	 */function printDates(dates){for(var i=0;i<dates.length;i++){console.log(i,new Date(dates[i].start),new Date(dates[i].end),dates[i].start,dates[i].end,dates[i].remove);}}/**
	 * Used in TimeStep to avoid the hidden times.
	 * @param {function} moment
	 * @param {TimeStep} timeStep
	 * @param {Date} previousTime
	 */function stepOverHiddenDates(moment,timeStep,previousTime){var stepInHidden=false;var currentValue=timeStep.current.valueOf();for(var i=0;i<timeStep.hiddenDates.length;i++){var startDate=timeStep.hiddenDates[i].start;var endDate=timeStep.hiddenDates[i].end;if(currentValue>=startDate&&currentValue<endDate){stepInHidden=true;break;}}if(stepInHidden==true&&currentValue<timeStep._end.valueOf()&&currentValue!=previousTime){var prevValue=moment(previousTime);var newValue=moment(endDate);//check if the next step should be major
	if(prevValue.year()!=newValue.year()){timeStep.switchedYear=true;}else if(prevValue.month()!=newValue.month()){timeStep.switchedMonth=true;}else if(prevValue.dayOfYear()!=newValue.dayOfYear()){timeStep.switchedDay=true;}timeStep.current=newValue;}}///**
	// * Used in TimeStep to avoid the hidden times.
	// * @param timeStep
	// * @param previousTime
	// */
	//checkFirstStep = function(timeStep) {
	//  var stepInHidden = false;
	//  var currentValue = timeStep.current.valueOf();
	//  for (var i = 0; i < timeStep.hiddenDates.length; i++) {
	//    var startDate = timeStep.hiddenDates[i].start;
	//    var endDate = timeStep.hiddenDates[i].end;
	//    if (currentValue >= startDate && currentValue < endDate) {
	//      stepInHidden = true;
	//      break;
	//    }
	//  }
	//
	//  if (stepInHidden == true && currentValue <= timeStep._end.valueOf()) {
	//    var newValue = moment(endDate);
	//    timeStep.current = newValue.toDate();
	//  }
	//};
	/**
	 * replaces the Core toScreen methods
	 *
	 * @param {timeline.Core} Core
	 * @param {Date} time
	 * @param {number} width
	 * @returns {number}
	 */function toScreen(Core,time,width){var conversion;if(Core.body.hiddenDates.length==0){conversion=Core.range.conversion(width);return (time.valueOf()-conversion.offset)*conversion.scale;}else {var hidden=getIsHidden(time,Core.body.hiddenDates);if(hidden.hidden==true){time=hidden.startDate;}var duration=getHiddenDurationBetween(Core.body.hiddenDates,Core.range.start,Core.range.end);if(time<Core.range.start){conversion=Core.range.conversion(width,duration);var hiddenBeforeStart=getHiddenDurationBeforeStart(Core.body.hiddenDates,time,conversion.offset);time=Core.options.moment(time).toDate().valueOf();time=time+hiddenBeforeStart;return -(conversion.offset-time.valueOf())*conversion.scale;}else if(time>Core.range.end){var rangeAfterEnd={start:Core.range.start,end:time};time=correctTimeForHidden(Core.options.moment,Core.body.hiddenDates,rangeAfterEnd,time);conversion=Core.range.conversion(width,duration);return (time.valueOf()-conversion.offset)*conversion.scale;}else {time=correctTimeForHidden(Core.options.moment,Core.body.hiddenDates,Core.range,time);conversion=Core.range.conversion(width,duration);return (time.valueOf()-conversion.offset)*conversion.scale;}}}/**
	 * Replaces the core toTime methods
	 *
	 * @param {timeline.Core} Core
	 * @param {number} x
	 * @param {number} width
	 * @returns {Date}
	 */function toTime(Core,x,width){if(Core.body.hiddenDates.length==0){var conversion=Core.range.conversion(width);return new Date(x/conversion.scale+conversion.offset);}else {var hiddenDuration=getHiddenDurationBetween(Core.body.hiddenDates,Core.range.start,Core.range.end);var totalDuration=Core.range.end-Core.range.start-hiddenDuration;var partialDuration=totalDuration*x/width;var accumulatedHiddenDuration=getAccumulatedHiddenDuration(Core.body.hiddenDates,Core.range,partialDuration);return new Date(accumulatedHiddenDuration+partialDuration+Core.range.start);}}/**
	 * Support function
	 *
	 * @param {Array.<{start: Window.start, end: *}>} hiddenDates
	 * @param {number} start
	 * @param {number} end
	 * @returns {number}
	 */function getHiddenDurationBetween(hiddenDates,start,end){var duration=0;for(var i=0;i<hiddenDates.length;i++){var startDate=hiddenDates[i].start;var endDate=hiddenDates[i].end;// if time after the cutout, and the
	if(startDate>=start&&endDate<end){duration+=endDate-startDate;}}return duration;}/**
	 * Support function
	 *
	 * @param {Array.<{start: Window.start, end: *}>} hiddenDates
	 * @param {number} start
	 * @param {number} end
	 * @returns {number}
	 */function getHiddenDurationBeforeStart(hiddenDates,start,end){var duration=0;for(var i=0;i<hiddenDates.length;i++){var startDate=hiddenDates[i].start;var endDate=hiddenDates[i].end;if(startDate>=start&&endDate<=end){duration+=endDate-startDate;}}return duration;}/**
	 * Support function
	 * @param {function} moment
	 * @param {Array.<{start: Window.start, end: *}>} hiddenDates
	 * @param {{start: number, end: number}} range
	 * @param {Date} time
	 * @returns {number}
	 */function correctTimeForHidden(moment,hiddenDates,range,time){time=moment(time).toDate().valueOf();time-=getHiddenDurationBefore(moment,hiddenDates,range,time);return time;}/**
	 * Support function
	 * @param {function} moment
	 * @param {Array.<{start: Window.start, end: *}>} hiddenDates
	 * @param {{start: number, end: number}} range
	 * @param {Date} time
	 * @returns {number}
	 */function getHiddenDurationBefore(moment,hiddenDates,range,time){var timeOffset=0;time=moment(time).toDate().valueOf();for(var i=0;i<hiddenDates.length;i++){var startDate=hiddenDates[i].start;var endDate=hiddenDates[i].end;// if time after the cutout, and the
	if(startDate>=range.start&&endDate<range.end){if(time>=endDate){timeOffset+=endDate-startDate;}}}return timeOffset;}/**
	 * sum the duration from start to finish, including the hidden duration,
	 * until the required amount has been reached, return the accumulated hidden duration
	 * @param {Array.<{start: Window.start, end: *}>} hiddenDates
	 * @param {{start: number, end: number}} range
	 * @param {number} [requiredDuration=0]
	 * @returns {number}
	 */function getAccumulatedHiddenDuration(hiddenDates,range,requiredDuration){var hiddenDuration=0;var duration=0;var previousPoint=range.start;//printDates(hiddenDates)
	for(var i=0;i<hiddenDates.length;i++){var startDate=hiddenDates[i].start;var endDate=hiddenDates[i].end;// if time after the cutout, and the
	if(startDate>=range.start&&endDate<range.end){duration+=startDate-previousPoint;previousPoint=endDate;if(duration>=requiredDuration){break;}else {hiddenDuration+=endDate-startDate;}}}return hiddenDuration;}/**
	 * used to step over to either side of a hidden block. Correction is disabled on tablets, might be set to true
	 * @param {Array.<{start: Window.start, end: *}>} hiddenDates
	 * @param {Date} time
	 * @param {number} direction
	 * @param {boolean} correctionEnabled
	 * @returns {Date|number}
	 */function snapAwayFromHidden(hiddenDates,time,direction,correctionEnabled){var isHidden=getIsHidden(time,hiddenDates);if(isHidden.hidden==true){if(direction<0){if(correctionEnabled==true){return isHidden.startDate-(isHidden.endDate-time)-1;}else {return isHidden.startDate-1;}}else {if(correctionEnabled==true){return isHidden.endDate+(time-isHidden.startDate)+1;}else {return isHidden.endDate+1;}}}else {return time;}}/**
	 * Check if a time is hidden
	 *
	 * @param {Date} time
	 * @param {Array.<{start: Window.start, end: *}>} hiddenDates
	 * @returns {{hidden: boolean, startDate: Window.start, endDate: *}}
	 */function getIsHidden(time,hiddenDates){for(var i=0;i<hiddenDates.length;i++){var startDate=hiddenDates[i].start;var endDate=hiddenDates[i].end;if(time>=startDate&&time<endDate){// if the start is entering a hidden zone
	return {hidden:true,startDate:startDate,endDate:endDate};}}return {hidden:false,startDate:startDate,endDate:endDate};}var DateUtil=/*#__PURE__*/freeze$2({__proto__:null,convertHiddenOptions:convertHiddenOptions,updateHiddenDates:updateHiddenDates,removeDuplicates:removeDuplicates,printDates:printDates,stepOverHiddenDates:stepOverHiddenDates,toScreen:toScreen,toTime:toTime,getHiddenDurationBetween:getHiddenDurationBetween,getHiddenDurationBeforeStart:getHiddenDurationBeforeStart,correctTimeForHidden:correctTimeForHidden,getHiddenDurationBefore:getHiddenDurationBefore,getAccumulatedHiddenDuration:getAccumulatedHiddenDuration,snapAwayFromHidden:snapAwayFromHidden,getIsHidden:getIsHidden});/**
	 * A Range controls a numeric range with a start and end value.
	 * The Range adjusts the range based on mouse events or programmatic changes,
	 * and triggers events when the range is changing or has been changed.
	 */var Range$1=/*#__PURE__*/function(_Component){inherits(Range,_Component);var _super=_createSuper$2(Range);/**
	 * @param {{dom: Object, domProps: Object, emitter: Emitter}} body
	 * @param {Object} [options]    See description at Range.setOptions
	 * @constructor Range
	 * @extends Component
	 */function Range(body,options){var _context10,_context11,_context12,_context13,_context14,_context15,_context16;var _this;classCallCheck(this,Range);_this=_super.call(this);var now=moment$1().hours(0).minutes(0).seconds(0).milliseconds(0);var start=now.clone().add(-3,'days').valueOf();var end=now.clone().add(3,'days').valueOf();_this.millisecondsPerPixelCache=undefined;if(options===undefined){_this.start=start;_this.end=end;}else {_this.start=options.start||start;_this.end=options.end||end;}_this.rolling=false;_this.body=body;_this.deltaDifference=0;_this.scaleOffset=0;_this.startToFront=false;_this.endToFront=true;// default options
	_this.defaultOptions={rtl:false,start:null,end:null,moment:moment$1,direction:'horizontal',// 'horizontal' or 'vertical'
	moveable:true,zoomable:true,min:null,max:null,zoomMin:10,// milliseconds
	zoomMax:1000*60*60*24*365*10000,// milliseconds
	rollingMode:{follow:false,offset:0.5}};_this.options=util$1.extend({},_this.defaultOptions);_this.props={touch:{}};_this.animationTimer=null;// drag listeners for dragging
	_this.body.emitter.on('panstart',bind$2(_context10=_this._onDragStart).call(_context10,assertThisInitialized(_this)));_this.body.emitter.on('panmove',bind$2(_context11=_this._onDrag).call(_context11,assertThisInitialized(_this)));_this.body.emitter.on('panend',bind$2(_context12=_this._onDragEnd).call(_context12,assertThisInitialized(_this)));// mouse wheel for zooming
	_this.body.emitter.on('mousewheel',bind$2(_context13=_this._onMouseWheel).call(_context13,assertThisInitialized(_this)));// pinch to zoom
	_this.body.emitter.on('touch',bind$2(_context14=_this._onTouch).call(_context14,assertThisInitialized(_this)));_this.body.emitter.on('pinch',bind$2(_context15=_this._onPinch).call(_context15,assertThisInitialized(_this)));// on click of rolling mode button
	_this.body.dom.rollingModeBtn.addEventListener('click',bind$2(_context16=_this.startRolling).call(_context16,assertThisInitialized(_this)));_this.setOptions(options);return _this;}/**
	   * Set options for the range controller
	   * @param {Object} options      Available options:
	   *                              {number | Date | String} start  Start date for the range
	   *                              {number | Date | String} end    End date for the range
	   *                              {number} min    Minimum value for start
	   *                              {number} max    Maximum value for end
	   *                              {number} zoomMin    Set a minimum value for
	   *                                                  (end - start).
	   *                              {number} zoomMax    Set a maximum value for
	   *                                                  (end - start).
	   *                              {boolean} moveable Enable moving of the range
	   *                                                 by dragging. True by default
	   *                              {boolean} zoomable Enable zooming of the range
	   *                                                 by pinching/scrolling. True by default
	   */createClass(Range,[{key:"setOptions",value:function setOptions(options){if(options){// copy the options that we know
	var fields=['animation','direction','min','max','zoomMin','zoomMax','moveable','zoomable','moment','activate','hiddenDates','zoomKey','zoomFriction','rtl','showCurrentTime','rollingMode','horizontalScroll'];util$1.selectiveExtend(fields,this.options,options);if(options.rollingMode&&options.rollingMode.follow){this.startRolling();}if('start'in options||'end'in options){// apply a new range. both start and end are optional
	this.setRange(options.start,options.end);}}}/**
	   * Start auto refreshing the current time bar
	   */},{key:"startRolling",value:function startRolling(){var me=this;/**
	     *  Updates the current time.
	     */function update(){me.stopRolling();me.rolling=true;var interval=me.end-me.start;var t=util$1.convert(new Date(),'Date').valueOf();var rollingModeOffset=me.options.rollingMode&&me.options.rollingMode.offset||0.5;var start=t-interval*rollingModeOffset;var end=t+interval*(1-rollingModeOffset);var options={animation:false};me.setRange(start,end,options);// determine interval to refresh
	var scale=me.conversion(me.body.domProps.center.width).scale;interval=1/scale/10;if(interval<30)interval=30;if(interval>1000)interval=1000;me.body.dom.rollingModeBtn.style.visibility="hidden";// start a renderTimer to adjust for the new time
	me.currentTimeTimer=setTimeout$2(update,interval);}update();}/**
	   * Stop auto refreshing the current time bar
	   */},{key:"stopRolling",value:function stopRolling(){if(this.currentTimeTimer!==undefined){clearTimeout(this.currentTimeTimer);this.rolling=false;this.body.dom.rollingModeBtn.style.visibility="visible";}}/**
	   * Set a new start and end range
	   * @param {Date | number | string} start
	   * @param {Date | number | string} end
	   * @param {Object} options      Available options:
	   *                              {boolean | {duration: number, easingFunction: string}} [animation=false]
	   *                                    If true, the range is animated
	   *                                    smoothly to the new window. An object can be
	   *                                    provided to specify duration and easing function.
	   *                                    Default duration is 500 ms, and default easing
	   *                                    function is 'easeInOutQuad'.
	   *                              {boolean} [byUser=false]
	   *                              {Event}  event  Mouse event
	   * @param {Function} callback     a callback function to be executed at the end of this function  
	   * @param {Function} frameCallback    a callback function executed each frame of the range animation.
	   *                                    The callback will be passed three parameters:
	   *                                    {number} easeCoefficient    an easing coefficent
	   *                                    {boolean} willDraw          If true the caller will redraw after the callback completes
	   *                                    {boolean} done              If true then animation is ending after the current frame
	   * @return {void}
	   */},{key:"setRange",value:function setRange(start,end,options,callback,frameCallback){if(!options){options={};}if(options.byUser!==true){options.byUser=false;}var me=this;var finalStart=start!=undefined?util$1.convert(start,'Date').valueOf():null;var finalEnd=end!=undefined?util$1.convert(end,'Date').valueOf():null;this._cancelAnimation();this.millisecondsPerPixelCache=undefined;if(options.animation){// true or an Object
	var initStart=this.start;var initEnd=this.end;var duration=_typeof_1(options.animation)==='object'&&'duration'in options.animation?options.animation.duration:500;var easingName=_typeof_1(options.animation)==='object'&&'easingFunction'in options.animation?options.animation.easingFunction:'easeInOutQuad';var easingFunction=util$1.easingFunctions[easingName];if(!easingFunction){var _context17;throw new Error(concat$2(_context17="Unknown easing function ".concat(stringify$2(easingName),". Choose from: ")).call(_context17,keys$6(util$1.easingFunctions).join(', ')));}var initTime=now$2();var anyChanged=false;var next=function next(){if(!me.props.touch.dragging){var now=now$2();var time=now-initTime;var ease=easingFunction(time/duration);var done=time>duration;var s=done||finalStart===null?finalStart:initStart+(finalStart-initStart)*ease;var e=done||finalEnd===null?finalEnd:initEnd+(finalEnd-initEnd)*ease;changed=me._applyRange(s,e);updateHiddenDates(me.options.moment,me.body,me.options.hiddenDates);anyChanged=anyChanged||changed;var params={start:new Date(me.start),end:new Date(me.end),byUser:options.byUser,event:options.event};if(frameCallback){frameCallback(ease,changed,done);}if(changed){me.body.emitter.emit('rangechange',params);}if(done){if(anyChanged){me.body.emitter.emit('rangechanged',params);if(callback){return callback();}}}else {// animate with as high as possible frame rate, leave 20 ms in between
	// each to prevent the browser from blocking
	me.animationTimer=setTimeout$2(next,20);}}};return next();}else {var changed=this._applyRange(finalStart,finalEnd);updateHiddenDates(this.options.moment,this.body,this.options.hiddenDates);if(changed){var params={start:new Date(this.start),end:new Date(this.end),byUser:options.byUser,event:options.event};this.body.emitter.emit('rangechange',params);clearTimeout(me.timeoutID);me.timeoutID=setTimeout$2(function(){me.body.emitter.emit('rangechanged',params);},200);if(callback){return callback();}}}}/**
	   * Get the number of milliseconds per pixel.
	   *
	   * @returns {undefined|number}
	   */},{key:"getMillisecondsPerPixel",value:function getMillisecondsPerPixel(){if(this.millisecondsPerPixelCache===undefined){this.millisecondsPerPixelCache=(this.end-this.start)/this.body.dom.center.clientWidth;}return this.millisecondsPerPixelCache;}/**
	   * Stop an animation
	   * @private
	   */},{key:"_cancelAnimation",value:function _cancelAnimation(){if(this.animationTimer){clearTimeout(this.animationTimer);this.animationTimer=null;}}/**
	   * Set a new start and end range. This method is the same as setRange, but
	   * does not trigger a range change and range changed event, and it returns
	   * true when the range is changed
	   * @param {number} [start]
	   * @param {number} [end]
	   * @return {boolean} changed
	   * @private
	   */},{key:"_applyRange",value:function _applyRange(start,end){var newStart=start!=null?util$1.convert(start,'Date').valueOf():this.start;var newEnd=end!=null?util$1.convert(end,'Date').valueOf():this.end;var max=this.options.max!=null?util$1.convert(this.options.max,'Date').valueOf():null;var min=this.options.min!=null?util$1.convert(this.options.min,'Date').valueOf():null;var diff;// check for valid number
	if(isNaN(newStart)||newStart===null){throw new Error("Invalid start \"".concat(start,"\""));}if(isNaN(newEnd)||newEnd===null){throw new Error("Invalid end \"".concat(end,"\""));}// prevent end < start
	if(newEnd<newStart){newEnd=newStart;}// prevent start < min
	if(min!==null){if(newStart<min){diff=min-newStart;newStart+=diff;newEnd+=diff;// prevent end > max
	if(max!=null){if(newEnd>max){newEnd=max;}}}}// prevent end > max
	if(max!==null){if(newEnd>max){diff=newEnd-max;newStart-=diff;newEnd-=diff;// prevent start < min
	if(min!=null){if(newStart<min){newStart=min;}}}}// prevent (end-start) < zoomMin
	if(this.options.zoomMin!==null){var zoomMin=_parseFloat$2(this.options.zoomMin);if(zoomMin<0){zoomMin=0;}if(newEnd-newStart<zoomMin){// compensate for a scale of 0.5 ms
	var compensation=0.5;if(this.end-this.start===zoomMin&&newStart>=this.start-compensation&&newEnd<=this.end){// ignore this action, we are already zoomed to the minimum
	newStart=this.start;newEnd=this.end;}else {// zoom to the minimum
	diff=zoomMin-(newEnd-newStart);newStart-=diff/2;newEnd+=diff/2;}}}// prevent (end-start) > zoomMax
	if(this.options.zoomMax!==null){var zoomMax=_parseFloat$2(this.options.zoomMax);if(zoomMax<0){zoomMax=0;}if(newEnd-newStart>zoomMax){if(this.end-this.start===zoomMax&&newStart<this.start&&newEnd>this.end){// ignore this action, we are already zoomed to the maximum
	newStart=this.start;newEnd=this.end;}else {// zoom to the maximum
	diff=newEnd-newStart-zoomMax;newStart+=diff/2;newEnd-=diff/2;}}}var changed=this.start!=newStart||this.end!=newEnd;// if the new range does NOT overlap with the old range, emit checkRangedItems to avoid not showing ranged items (ranged meaning has end time, not necessarily of type Range)
	if(!(newStart>=this.start&&newStart<=this.end||newEnd>=this.start&&newEnd<=this.end)&&!(this.start>=newStart&&this.start<=newEnd||this.end>=newStart&&this.end<=newEnd)){this.body.emitter.emit('checkRangedItems');}this.start=newStart;this.end=newEnd;return changed;}/**
	   * Retrieve the current range.
	   * @return {Object} An object with start and end properties
	   */},{key:"getRange",value:function getRange(){return {start:this.start,end:this.end};}/**
	   * Calculate the conversion offset and scale for current range, based on
	   * the provided width
	   * @param {number} width
	   * @param {number} [totalHidden=0]
	   * @returns {{offset: number, scale: number}} conversion
	   */},{key:"conversion",value:function conversion(width,totalHidden){return Range.conversion(this.start,this.end,width,totalHidden);}/**
	   * Static method to calculate the conversion offset and scale for a range,
	   * based on the provided start, end, and width
	   * @param {number} start
	   * @param {number} end
	   * @param {number} width
	   * @param {number} [totalHidden=0]
	   * @returns {{offset: number, scale: number}} conversion
	   */},{key:"_onDragStart",/**
	   * Start dragging horizontally or vertically
	   * @param {Event} event
	   * @private
	   */value:function _onDragStart(event){this.deltaDifference=0;this.previousDelta=0;// only allow dragging when configured as movable
	if(!this.options.moveable)return;// only start dragging when the mouse is inside the current range
	if(!this._isInsideRange(event))return;// refuse to drag when we where pinching to prevent the timeline make a jump
	// when releasing the fingers in opposite order from the touch screen
	if(!this.props.touch.allowDragging)return;this.stopRolling();this.props.touch.start=this.start;this.props.touch.end=this.end;this.props.touch.dragging=true;if(this.body.dom.root){this.body.dom.root.style.cursor='move';}}/**
	   * Perform dragging operation
	   * @param {Event} event
	   * @private
	   */},{key:"_onDrag",value:function _onDrag(event){if(!event)return;if(!this.props.touch.dragging)return;// only allow dragging when configured as movable
	if(!this.options.moveable)return;// TODO: this may be redundant in hammerjs2
	// refuse to drag when we where pinching to prevent the timeline make a jump
	// when releasing the fingers in opposite order from the touch screen
	if(!this.props.touch.allowDragging)return;var direction=this.options.direction;validateDirection(direction);var delta=direction=='horizontal'?event.deltaX:event.deltaY;delta-=this.deltaDifference;var interval=this.props.touch.end-this.props.touch.start;// normalize dragging speed if cutout is in between.
	var duration=getHiddenDurationBetween(this.body.hiddenDates,this.start,this.end);interval-=duration;var width=direction=='horizontal'?this.body.domProps.center.width:this.body.domProps.center.height;var diffRange;if(this.options.rtl){diffRange=delta/width*interval;}else {diffRange=-delta/width*interval;}var newStart=this.props.touch.start+diffRange;var newEnd=this.props.touch.end+diffRange;// snapping times away from hidden zones
	var safeStart=snapAwayFromHidden(this.body.hiddenDates,newStart,this.previousDelta-delta,true);var safeEnd=snapAwayFromHidden(this.body.hiddenDates,newEnd,this.previousDelta-delta,true);if(safeStart!=newStart||safeEnd!=newEnd){this.deltaDifference+=delta;this.props.touch.start=safeStart;this.props.touch.end=safeEnd;this._onDrag(event);return;}this.previousDelta=delta;this._applyRange(newStart,newEnd);var startDate=new Date(this.start);var endDate=new Date(this.end);// fire a rangechange event
	this.body.emitter.emit('rangechange',{start:startDate,end:endDate,byUser:true,event:event});// fire a panmove event
	this.body.emitter.emit('panmove');}/**
	   * Stop dragging operation
	   * @param {event} event
	   * @private
	   */},{key:"_onDragEnd",value:function _onDragEnd(event){if(!this.props.touch.dragging)return;// only allow dragging when configured as movable
	if(!this.options.moveable)return;// TODO: this may be redundant in hammerjs2
	// refuse to drag when we where pinching to prevent the timeline make a jump
	// when releasing the fingers in opposite order from the touch screen
	if(!this.props.touch.allowDragging)return;this.props.touch.dragging=false;if(this.body.dom.root){this.body.dom.root.style.cursor='auto';}// fire a rangechanged event
	this.body.emitter.emit('rangechanged',{start:new Date(this.start),end:new Date(this.end),byUser:true,event:event});}/**
	   * Event handler for mouse wheel event, used to zoom
	   * Code from http://adomas.org/javascript-mouse-wheel/
	   * @param {Event} event
	   * @private
	   */},{key:"_onMouseWheel",value:function _onMouseWheel(event){// retrieve delta
	var delta=0;if(event.wheelDelta){/* IE/Opera. */delta=event.wheelDelta/120;}else if(event.detail){/* Mozilla case. */ // In Mozilla, sign of delta is different than in IE.
	// Also, delta is multiple of 3.
	delta=-event.detail/3;}else if(event.deltaY){delta=-event.deltaY/3;}// don't allow zoom when the according key is pressed and the zoomKey option or not zoomable but movable
	if(this.options.zoomKey&&!event[this.options.zoomKey]&&this.options.zoomable||!this.options.zoomable&&this.options.moveable){return;}// only allow zooming when configured as zoomable and moveable
	if(!(this.options.zoomable&&this.options.moveable))return;// only zoom when the mouse is inside the current range
	if(!this._isInsideRange(event))return;// If delta is nonzero, handle it.
	// Basically, delta is now positive if wheel was scrolled up,
	// and negative, if wheel was scrolled down.
	if(delta){// perform the zoom action. Delta is normally 1 or -1
	// adjust a negative delta such that zooming in with delta 0.1
	// equals zooming out with a delta -0.1
	var zoomFriction=this.options.zoomFriction||5;var scale;if(delta<0){scale=1-delta/zoomFriction;}else {scale=1/(1+delta/zoomFriction);}// calculate center, the date to zoom around
	var pointerDate;if(this.rolling){var rollingModeOffset=this.options.rollingMode&&this.options.rollingMode.offset||0.5;pointerDate=this.start+(this.end-this.start)*rollingModeOffset;}else {var pointer=this.getPointer({x:event.clientX,y:event.clientY},this.body.dom.center);pointerDate=this._pointerToDate(pointer);}this.zoom(scale,pointerDate,delta,event);// Prevent default actions caused by mouse wheel
	// (else the page and timeline both scroll)
	event.preventDefault();}}/**
	   * Start of a touch gesture
	   * @param {Event} event
	   * @private
	   */},{key:"_onTouch",value:function _onTouch(event){// eslint-disable-line no-unused-vars
	this.props.touch.start=this.start;this.props.touch.end=this.end;this.props.touch.allowDragging=true;this.props.touch.center=null;this.props.touch.centerDate=null;this.scaleOffset=0;this.deltaDifference=0;// Disable the browser default handling of this event.
	util$1.preventDefault(event);}/**
	   * Handle pinch event
	   * @param {Event} event
	   * @private
	   */},{key:"_onPinch",value:function _onPinch(event){// only allow zooming when configured as zoomable and moveable
	if(!(this.options.zoomable&&this.options.moveable))return;// Disable the browser default handling of this event.
	util$1.preventDefault(event);this.props.touch.allowDragging=false;if(!this.props.touch.center){this.props.touch.center=this.getPointer(event.center,this.body.dom.center);this.props.touch.centerDate=this._pointerToDate(this.props.touch.center);}this.stopRolling();var scale=1/(event.scale+this.scaleOffset);var centerDate=this.props.touch.centerDate;var hiddenDuration=getHiddenDurationBetween(this.body.hiddenDates,this.start,this.end);var hiddenDurationBefore=getHiddenDurationBefore(this.options.moment,this.body.hiddenDates,this,centerDate);var hiddenDurationAfter=hiddenDuration-hiddenDurationBefore;// calculate new start and end
	var newStart=centerDate-hiddenDurationBefore+(this.props.touch.start-(centerDate-hiddenDurationBefore))*scale;var newEnd=centerDate+hiddenDurationAfter+(this.props.touch.end-(centerDate+hiddenDurationAfter))*scale;// snapping times away from hidden zones
	this.startToFront=1-scale<=0;// used to do the right auto correction with periodic hidden times
	this.endToFront=scale-1<=0;// used to do the right auto correction with periodic hidden times
	var safeStart=snapAwayFromHidden(this.body.hiddenDates,newStart,1-scale,true);var safeEnd=snapAwayFromHidden(this.body.hiddenDates,newEnd,scale-1,true);if(safeStart!=newStart||safeEnd!=newEnd){this.props.touch.start=safeStart;this.props.touch.end=safeEnd;this.scaleOffset=1-event.scale;newStart=safeStart;newEnd=safeEnd;}var options={animation:false,byUser:true,event:event};this.setRange(newStart,newEnd,options);this.startToFront=false;// revert to default
	this.endToFront=true;// revert to default
	}/**
	   * Test whether the mouse from a mouse event is inside the visible window,
	   * between the current start and end date
	   * @param {Object} event
	   * @return {boolean} Returns true when inside the visible window
	   * @private
	   */},{key:"_isInsideRange",value:function _isInsideRange(event){// calculate the time where the mouse is, check whether inside
	// and no scroll action should happen.
	var clientX=event.center?event.center.x:event.clientX;var centerContainerRect=this.body.dom.centerContainer.getBoundingClientRect();var x=this.options.rtl?clientX-centerContainerRect.left:centerContainerRect.right-clientX;var time=this.body.util.toTime(x);return time>=this.start&&time<=this.end;}/**
	   * Helper function to calculate the center date for zooming
	   * @param {{x: number, y: number}} pointer
	   * @return {number} date
	   * @private
	   */},{key:"_pointerToDate",value:function _pointerToDate(pointer){var conversion;var direction=this.options.direction;validateDirection(direction);if(direction=='horizontal'){return this.body.util.toTime(pointer.x).valueOf();}else {var height=this.body.domProps.center.height;conversion=this.conversion(height);return pointer.y/conversion.scale+conversion.offset;}}/**
	   * Get the pointer location relative to the location of the dom element
	   * @param {{x: number, y: number}} touch
	   * @param {Element} element   HTML DOM element
	   * @return {{x: number, y: number}} pointer
	   * @private
	   */},{key:"getPointer",value:function getPointer(touch,element){var elementRect=element.getBoundingClientRect();if(this.options.rtl){return {x:elementRect.right-touch.x,y:touch.y-elementRect.top};}else {return {x:touch.x-elementRect.left,y:touch.y-elementRect.top};}}/**
	   * Zoom the range the given scale in or out. Start and end date will
	   * be adjusted, and the timeline will be redrawn. You can optionally give a
	   * date around which to zoom.
	   * For example, try scale = 0.9 or 1.1
	   * @param {number} scale      Scaling factor. Values above 1 will zoom out,
	   *                            values below 1 will zoom in.
	   * @param {number} [center]   Value representing a date around which will
	   *                            be zoomed.
	   * @param {number} delta
	   * @param {Event} event
	   */},{key:"zoom",value:function zoom(scale,center,delta,event){// if centerDate is not provided, take it half between start Date and end Date
	if(center==null){center=(this.start+this.end)/2;}var hiddenDuration=getHiddenDurationBetween(this.body.hiddenDates,this.start,this.end);var hiddenDurationBefore=getHiddenDurationBefore(this.options.moment,this.body.hiddenDates,this,center);var hiddenDurationAfter=hiddenDuration-hiddenDurationBefore;// calculate new start and end
	var newStart=center-hiddenDurationBefore+(this.start-(center-hiddenDurationBefore))*scale;var newEnd=center+hiddenDurationAfter+(this.end-(center+hiddenDurationAfter))*scale;// snapping times away from hidden zones
	this.startToFront=delta>0?false:true;// used to do the right autocorrection with periodic hidden times
	this.endToFront=-delta>0?false:true;// used to do the right autocorrection with periodic hidden times
	var safeStart=snapAwayFromHidden(this.body.hiddenDates,newStart,delta,true);var safeEnd=snapAwayFromHidden(this.body.hiddenDates,newEnd,-delta,true);if(safeStart!=newStart||safeEnd!=newEnd){newStart=safeStart;newEnd=safeEnd;}var options={animation:false,byUser:true,event:event};this.setRange(newStart,newEnd,options);this.startToFront=false;// revert to default
	this.endToFront=true;// revert to default
	}/**
	   * Move the range with a given delta to the left or right. Start and end
	   * value will be adjusted. For example, try delta = 0.1 or -0.1
	   * @param {number}  delta     Moving amount. Positive value will move right,
	   *                            negative value will move left
	   */},{key:"move",value:function move(delta){// zoom start Date and end Date relative to the centerDate
	var diff=this.end-this.start;// apply new values
	var newStart=this.start+diff*delta;var newEnd=this.end+diff*delta;// TODO: reckon with min and max range
	this.start=newStart;this.end=newEnd;}/**
	   * Move the range to a new center point
	   * @param {number} moveTo      New center point of the range
	   */},{key:"moveTo",value:function moveTo(_moveTo){var center=(this.start+this.end)/2;var diff=center-_moveTo;// calculate new start and end
	var newStart=this.start-diff;var newEnd=this.end-diff;var options={animation:false,byUser:true,event:null};this.setRange(newStart,newEnd,options);}}],[{key:"conversion",value:function conversion(start,end,width,totalHidden){if(totalHidden===undefined){totalHidden=0;}if(width!=0&&end-start!=0){return {offset:start,scale:width/(end-start-totalHidden)};}else {return {offset:0,scale:1};}}}]);return Range;}(Component);/**
	 * Test whether direction has a valid value
	 * @param {string} direction    'horizontal' or 'vertical'
	 */function validateDirection(direction){if(direction!='horizontal'&&direction!='vertical'){throw new TypeError("Unknown direction \"".concat(direction,"\". Choose \"horizontal\" or \"vertical\"."));}}/**
	 * Setup a mock hammer.js object, for unit testing.
	 *
	 * Inspiration: https://github.com/uber/deck.gl/pull/658
	 *
	 * @returns {{on: noop, off: noop, destroy: noop, emit: noop, get: get}}
	 */function hammerMock(){var noop=function noop(){};return {on:noop,off:noop,destroy:noop,emit:noop,get:function get(m){//eslint-disable-line no-unused-vars
	return {set:noop};}};}var modifiedHammer;if(typeof window!=='undefined'){var OurHammer=window['Hammer']||Hammer;modifiedHammer=propagating(OurHammer,{preventDefault:'mouse'});}else {modifiedHammer=function modifiedHammer(){return(// hammer.js is only available in a browser, not in node.js. Replacing it with a mock object.
	hammerMock());};}var Hammer$1=modifiedHammer;/**
	 * Register a touch event, taking place before a gesture
	 * @param {Hammer} hammer       A hammer instance
	 * @param {function} callback   Callback, called as callback(event)
	 */function onTouch(hammer,callback){callback.inputHandler=function(event){if(event.isFirst){callback(event);}};hammer.on('hammer.input',callback.inputHandler);}/**
	 * Register a release event, taking place after a gesture
	 * @param {Hammer} hammer       A hammer instance
	 * @param {function} callback   Callback, called as callback(event)
	 * @returns {*}
	 */function onRelease(hammer,callback){callback.inputHandler=function(event){if(event.isFinal){callback(event);}};return hammer.on('hammer.input',callback.inputHandler);}/**
	 * Hack the PinchRecognizer such that it doesn't prevent default behavior
	 * for vertical panning.
	 *
	 * Yeah ... this is quite a hack ... see https://github.com/hammerjs/hammer.js/issues/932
	 *
	 * @param {Hammer.Pinch} pinchRecognizer
	 * @return {Hammer.Pinch} returns the pinchRecognizer
	 */function disablePreventDefaultVertically(pinchRecognizer){var TOUCH_ACTION_PAN_Y='pan-y';pinchRecognizer.getTouchAction=function(){// default method returns [TOUCH_ACTION_NONE]
	return [TOUCH_ACTION_PAN_Y];};return pinchRecognizer;}/**
	 * The class TimeStep is an iterator for dates. You provide a start date and an
	 * end date. The class itself determines the best scale (step size) based on the
	 * provided start Date, end Date, and minimumStep.
	 *
	 * If minimumStep is provided, the step size is chosen as close as possible
	 * to the minimumStep but larger than minimumStep. If minimumStep is not
	 * provided, the scale is set to 1 DAY.
	 * The minimumStep should correspond with the onscreen size of about 6 characters
	 *
	 * Alternatively, you can set a scale by hand.
	 * After creation, you can initialize the class by executing first(). Then you
	 * can iterate from the start date to the end date via next(). You can check if
	 * the end date is reached with the function hasNext(). After each step, you can
	 * retrieve the current date via getCurrent().
	 * The TimeStep has scales ranging from milliseconds, seconds, minutes, hours,
	 * days, to years.
	 *
	 * Version: 1.2
	 *
	 */var TimeStep=/*#__PURE__*/function(){/**
	    * @param {Date} [start]         The start date, for example new Date(2010, 9, 21)
	    *                               or new Date(2010, 9, 21, 23, 45, 00)
	    * @param {Date} [end]           The end date
	    * @param {number} [minimumStep] Optional. Minimum step size in milliseconds
	    * @param {Date|Array.<Date>} [hiddenDates] Optional.
	    * @param {{showMajorLabels: boolean, showWeekScale: boolean}} [options] Optional.
	    * @constructor  TimeStep
	    */function TimeStep(start,end,minimumStep,hiddenDates,options){classCallCheck(this,TimeStep);this.moment=options&&options.moment||moment$1;this.options=options?options:{};// variables
	this.current=this.moment();this._start=this.moment();this._end=this.moment();this.autoScale=true;this.scale='day';this.step=1;// initialize the range
	this.setRange(start,end,minimumStep);// hidden Dates options
	this.switchedDay=false;this.switchedMonth=false;this.switchedYear=false;if(isArray$3(hiddenDates)){this.hiddenDates=hiddenDates;}else if(hiddenDates!=undefined){this.hiddenDates=[hiddenDates];}else {this.hiddenDates=[];}this.format=TimeStep.FORMAT;// default formatting
	}/**
	   * Set custom constructor function for moment. Can be used to set dates
	   * to UTC or to set a utcOffset.
	   * @param {function} moment
	   */createClass(TimeStep,[{key:"setMoment",value:function setMoment(moment){this.moment=moment;// update the date properties, can have a new utcOffset
	this.current=this.moment(this.current.valueOf());this._start=this.moment(this._start.valueOf());this._end=this.moment(this._end.valueOf());}/**
	   * Set custom formatting for the minor an major labels of the TimeStep.
	   * Both `minorLabels` and `majorLabels` are an Object with properties:
	   * 'millisecond', 'second', 'minute', 'hour', 'weekday', 'day', 'week', 'month', 'year'.
	   * @param {{minorLabels: Object, majorLabels: Object}} format
	   */},{key:"setFormat",value:function setFormat(format){var defaultFormat=util$1.deepExtend({},TimeStep.FORMAT);this.format=util$1.deepExtend(defaultFormat,format);}/**
	   * Set a new range
	   * If minimumStep is provided, the step size is chosen as close as possible
	   * to the minimumStep but larger than minimumStep. If minimumStep is not
	   * provided, the scale is set to 1 DAY.
	   * The minimumStep should correspond with the onscreen size of about 6 characters
	   * @param {Date} [start]      The start date and time.
	   * @param {Date} [end]        The end date and time.
	   * @param {int} [minimumStep] Optional. Minimum step size in milliseconds
	   */},{key:"setRange",value:function setRange(start,end,minimumStep){if(!(start instanceof Date)||!(end instanceof Date)){throw "No legal start or end date in method setRange";}this._start=start!=undefined?this.moment(start.valueOf()):now$2();this._end=end!=undefined?this.moment(end.valueOf()):now$2();if(this.autoScale){this.setMinimumStep(minimumStep);}}/**
	   * Set the range iterator to the start date.
	   */},{key:"start",value:function start(){this.current=this._start.clone();this.roundToMinor();}/**
	   * Round the current date to the first minor date value
	   * This must be executed once when the current date is set to start Date
	   */},{key:"roundToMinor",value:function roundToMinor(){// round to floor
	// to prevent year & month scales rounding down to the first day of week we perform this separately
	if(this.scale=='week'){this.current.weekday(0);}// IMPORTANT: we have no breaks in this switch! (this is no bug)
	// noinspection FallThroughInSwitchStatementJS
	switch(this.scale){case'year':this.current.year(this.step*Math.floor(this.current.year()/this.step));this.current.month(0);case'month':this.current.date(1);// eslint-disable-line no-fallthrough
	case'week':// eslint-disable-line no-fallthrough
	case'day':// eslint-disable-line no-fallthrough
	case'weekday':this.current.hours(0);// eslint-disable-line no-fallthrough
	case'hour':this.current.minutes(0);// eslint-disable-line no-fallthrough
	case'minute':this.current.seconds(0);// eslint-disable-line no-fallthrough
	case'second':this.current.milliseconds(0);// eslint-disable-line no-fallthrough
	//case 'millisecond': // nothing to do for milliseconds
	}if(this.step!=1){// round down to the first minor value that is a multiple of the current step size
	var priorCurrent=this.current.clone();switch(this.scale){case'millisecond':this.current.subtract(this.current.milliseconds()%this.step,'milliseconds');break;case'second':this.current.subtract(this.current.seconds()%this.step,'seconds');break;case'minute':this.current.subtract(this.current.minutes()%this.step,'minutes');break;case'hour':this.current.subtract(this.current.hours()%this.step,'hours');break;case'weekday':// intentional fall through
	case'day':this.current.subtract((this.current.date()-1)%this.step,'day');break;case'week':this.current.subtract(this.current.week()%this.step,'week');break;case'month':this.current.subtract(this.current.month()%this.step,'month');break;case'year':this.current.subtract(this.current.year()%this.step,'year');break;}if(!priorCurrent.isSame(this.current)){this.current=this.moment(snapAwayFromHidden(this.hiddenDates,this.current.valueOf(),-1,true));}}}/**
	   * Check if the there is a next step
	   * @return {boolean}  true if the current date has not passed the end date
	   */},{key:"hasNext",value:function hasNext(){return this.current.valueOf()<=this._end.valueOf();}/**
	   * Do the next step
	   */},{key:"next",value:function next(){var prev=this.current.valueOf();// Two cases, needed to prevent issues with switching daylight savings
	// (end of March and end of October)
	switch(this.scale){case'millisecond':this.current.add(this.step,'millisecond');break;case'second':this.current.add(this.step,'second');break;case'minute':this.current.add(this.step,'minute');break;case'hour':this.current.add(this.step,'hour');if(this.current.month()<6){this.current.subtract(this.current.hours()%this.step,'hour');}else {if(this.current.hours()%this.step!==0){this.current.add(this.step-this.current.hours()%this.step,'hour');}}break;case'weekday':// intentional fall through
	case'day':this.current.add(this.step,'day');break;case'week':if(this.current.weekday()!==0){// we had a month break not correlating with a week's start before
	this.current.weekday(0);// switch back to week cycles
	this.current.add(this.step,'week');}else if(this.options.showMajorLabels===false){this.current.add(this.step,'week');// the default case
	}else {// first day of the week
	var nextWeek=this.current.clone();nextWeek.add(1,'week');if(nextWeek.isSame(this.current,'month')){// is the first day of the next week in the same month?
	this.current.add(this.step,'week');// the default case
	}else {// inject a step at each first day of the month
	this.current.add(this.step,'week');this.current.date(1);}}break;case'month':this.current.add(this.step,'month');break;case'year':this.current.add(this.step,'year');break;}if(this.step!=1){// round down to the correct major value
	switch(this.scale){case'millisecond':if(this.current.milliseconds()>0&&this.current.milliseconds()<this.step)this.current.milliseconds(0);break;case'second':if(this.current.seconds()>0&&this.current.seconds()<this.step)this.current.seconds(0);break;case'minute':if(this.current.minutes()>0&&this.current.minutes()<this.step)this.current.minutes(0);break;case'hour':if(this.current.hours()>0&&this.current.hours()<this.step)this.current.hours(0);break;case'weekday':// intentional fall through
	case'day':if(this.current.date()<this.step+1)this.current.date(1);break;case'week':if(this.current.week()<this.step)this.current.week(1);break;// week numbering starts at 1, not 0
	case'month':if(this.current.month()<this.step)this.current.month(0);break;}}// safety mechanism: if current time is still unchanged, move to the end
	if(this.current.valueOf()==prev){this.current=this._end.clone();}// Reset switches for year, month and day. Will get set to true where appropriate in DateUtil.stepOverHiddenDates
	this.switchedDay=false;this.switchedMonth=false;this.switchedYear=false;stepOverHiddenDates(this.moment,this,prev);}/**
	   * Get the current datetime
	   * @return {Moment}  current The current date
	   */},{key:"getCurrent",value:function getCurrent(){return this.current.clone();}/**
	   * Set a custom scale. Autoscaling will be disabled.
	   * For example setScale('minute', 5) will result
	   * in minor steps of 5 minutes, and major steps of an hour.
	   *
	   * @param {{scale: string, step: number}} params
	   *                               An object containing two properties:
	   *                               - A string 'scale'. Choose from 'millisecond', 'second',
	   *                                 'minute', 'hour', 'weekday', 'day', 'week', 'month', 'year'.
	   *                               - A number 'step'. A step size, by default 1.
	   *                                 Choose for example 1, 2, 5, or 10.
	   */},{key:"setScale",value:function setScale(params){if(params&&typeof params.scale=='string'){this.scale=params.scale;this.step=params.step>0?params.step:1;this.autoScale=false;}}/**
	   * Enable or disable autoscaling
	   * @param {boolean} enable  If true, autoascaling is set true
	   */},{key:"setAutoScale",value:function setAutoScale(enable){this.autoScale=enable;}/**
	   * Automatically determine the scale that bests fits the provided minimum step
	   * @param {number} [minimumStep]  The minimum step size in milliseconds
	   */},{key:"setMinimumStep",value:function setMinimumStep(minimumStep){if(minimumStep==undefined){return;}//var b = asc + ds;
	var stepYear=1000*60*60*24*30*12;var stepMonth=1000*60*60*24*30;var stepDay=1000*60*60*24;var stepHour=1000*60*60;var stepMinute=1000*60;var stepSecond=1000;var stepMillisecond=1;// find the smallest step that is larger than the provided minimumStep
	if(stepYear*1000>minimumStep){this.scale='year';this.step=1000;}if(stepYear*500>minimumStep){this.scale='year';this.step=500;}if(stepYear*100>minimumStep){this.scale='year';this.step=100;}if(stepYear*50>minimumStep){this.scale='year';this.step=50;}if(stepYear*10>minimumStep){this.scale='year';this.step=10;}if(stepYear*5>minimumStep){this.scale='year';this.step=5;}if(stepYear>minimumStep){this.scale='year';this.step=1;}if(stepMonth*3>minimumStep){this.scale='month';this.step=3;}if(stepMonth>minimumStep){this.scale='month';this.step=1;}if(stepDay*7>minimumStep&&this.options.showWeekScale){this.scale='week';this.step=1;}if(stepDay*2>minimumStep){this.scale='day';this.step=2;}if(stepDay>minimumStep){this.scale='day';this.step=1;}if(stepDay/2>minimumStep){this.scale='weekday';this.step=1;}if(stepHour*4>minimumStep){this.scale='hour';this.step=4;}if(stepHour>minimumStep){this.scale='hour';this.step=1;}if(stepMinute*15>minimumStep){this.scale='minute';this.step=15;}if(stepMinute*10>minimumStep){this.scale='minute';this.step=10;}if(stepMinute*5>minimumStep){this.scale='minute';this.step=5;}if(stepMinute>minimumStep){this.scale='minute';this.step=1;}if(stepSecond*15>minimumStep){this.scale='second';this.step=15;}if(stepSecond*10>minimumStep){this.scale='second';this.step=10;}if(stepSecond*5>minimumStep){this.scale='second';this.step=5;}if(stepSecond>minimumStep){this.scale='second';this.step=1;}if(stepMillisecond*200>minimumStep){this.scale='millisecond';this.step=200;}if(stepMillisecond*100>minimumStep){this.scale='millisecond';this.step=100;}if(stepMillisecond*50>minimumStep){this.scale='millisecond';this.step=50;}if(stepMillisecond*10>minimumStep){this.scale='millisecond';this.step=10;}if(stepMillisecond*5>minimumStep){this.scale='millisecond';this.step=5;}if(stepMillisecond>minimumStep){this.scale='millisecond';this.step=1;}}/**
	   * Snap a date to a rounded value.
	   * The snap intervals are dependent on the current scale and step.
	   * Static function
	   * @param {Date} date    the date to be snapped.
	   * @param {string} scale Current scale, can be 'millisecond', 'second',
	   *                       'minute', 'hour', 'weekday, 'day', 'week', 'month', 'year'.
	   * @param {number} step  Current step (1, 2, 4, 5, ...
	   * @return {Date} snappedDate
	   */},{key:"isMajor",/**
	   * Check if the current value is a major value (for example when the step
	   * is DAY, a major value is each first day of the MONTH)
	   * @return {boolean} true if current date is major, else false.
	   */value:function isMajor(){if(this.switchedYear==true){switch(this.scale){case'year':case'month':case'week':case'weekday':case'day':case'hour':case'minute':case'second':case'millisecond':return true;default:return false;}}else if(this.switchedMonth==true){switch(this.scale){case'week':case'weekday':case'day':case'hour':case'minute':case'second':case'millisecond':return true;default:return false;}}else if(this.switchedDay==true){switch(this.scale){case'millisecond':case'second':case'minute':case'hour':return true;default:return false;}}var date=this.moment(this.current);switch(this.scale){case'millisecond':return date.milliseconds()==0;case'second':return date.seconds()==0;case'minute':return date.hours()==0&&date.minutes()==0;case'hour':return date.hours()==0;case'weekday':// intentional fall through
	case'day':return date.date()==1;case'week':return date.date()==1;case'month':return date.month()==0;case'year':return false;default:return false;}}/**
	   * Returns formatted text for the minor axislabel, depending on the current
	   * date and the scale. For example when scale is MINUTE, the current time is
	   * formatted as "hh:mm".
	   * @param {Date} [date=this.current] custom date. if not provided, current date is taken
	   * @returns {String}
	   */},{key:"getLabelMinor",value:function getLabelMinor(date){if(date==undefined){date=this.current;}if(date instanceof Date){date=this.moment(date);}if(typeof this.format.minorLabels==="function"){return this.format.minorLabels(date,this.scale,this.step);}var format=this.format.minorLabels[this.scale];// noinspection FallThroughInSwitchStatementJS
	switch(this.scale){case'week':// Don't draw the minor label if this date is the first day of a month AND if it's NOT the start of the week.
	// The 'date' variable may actually be the 'next' step when called from TimeAxis' _repaintLabels.
	if(date.date()===1&&date.weekday()!==0){return "";}default:// eslint-disable-line no-fallthrough
	return format&&format.length>0?this.moment(date).format(format):'';}}/**
	   * Returns formatted text for the major axis label, depending on the current
	   * date and the scale. For example when scale is MINUTE, the major scale is
	   * hours, and the hour will be formatted as "hh".
	   * @param {Date} [date=this.current] custom date. if not provided, current date is taken
	   * @returns {String}
	   */},{key:"getLabelMajor",value:function getLabelMajor(date){if(date==undefined){date=this.current;}if(date instanceof Date){date=this.moment(date);}if(typeof this.format.majorLabels==="function"){return this.format.majorLabels(date,this.scale,this.step);}var format=this.format.majorLabels[this.scale];return format&&format.length>0?this.moment(date).format(format):'';}/**
	   * get class name
	   * @return {string} class name
	   */},{key:"getClassName",value:function getClassName(){var _context18;var _moment=this.moment;var m=this.moment(this.current);var current=m.locale?m.locale('en'):m.lang('en');// old versions of moment have .lang() function
	var step=this.step;var classNames=[];/**
	     *
	     * @param {number} value
	     * @returns {String}
	     */function even(value){return value/step%2==0?' vis-even':' vis-odd';}/**
	     *
	     * @param {Date} date
	     * @returns {String}
	     */function today(date){if(date.isSame(now$2(),'day')){return ' vis-today';}if(date.isSame(_moment().add(1,'day'),'day')){return ' vis-tomorrow';}if(date.isSame(_moment().add(-1,'day'),'day')){return ' vis-yesterday';}return '';}/**
	     *
	     * @param {Date} date
	     * @returns {String}
	     */function currentWeek(date){return date.isSame(now$2(),'week')?' vis-current-week':'';}/**
	     *
	     * @param {Date} date
	     * @returns {String}
	     */function currentMonth(date){return date.isSame(now$2(),'month')?' vis-current-month':'';}/**
	     *
	     * @param {Date} date
	     * @returns {String}
	     */function currentYear(date){return date.isSame(now$2(),'year')?' vis-current-year':'';}switch(this.scale){case'millisecond':classNames.push(today(current));classNames.push(even(current.milliseconds()));break;case'second':classNames.push(today(current));classNames.push(even(current.seconds()));break;case'minute':classNames.push(today(current));classNames.push(even(current.minutes()));break;case'hour':classNames.push(concat$2(_context18="vis-h".concat(current.hours())).call(_context18,this.step==4?'-h'+(current.hours()+4):''));classNames.push(today(current));classNames.push(even(current.hours()));break;case'weekday':classNames.push("vis-".concat(current.format('dddd').toLowerCase()));classNames.push(today(current));classNames.push(currentWeek(current));classNames.push(even(current.date()));break;case'day':classNames.push("vis-day".concat(current.date()));classNames.push("vis-".concat(current.format('MMMM').toLowerCase()));classNames.push(today(current));classNames.push(currentMonth(current));classNames.push(this.step<=2?today(current):'');classNames.push(this.step<=2?"vis-".concat(current.format('dddd').toLowerCase()):'');classNames.push(even(current.date()-1));break;case'week':classNames.push("vis-week".concat(current.format('w')));classNames.push(currentWeek(current));classNames.push(even(current.week()));break;case'month':classNames.push("vis-".concat(current.format('MMMM').toLowerCase()));classNames.push(currentMonth(current));classNames.push(even(current.month()));break;case'year':classNames.push("vis-year".concat(current.year()));classNames.push(currentYear(current));classNames.push(even(current.year()));break;}return filter$2(classNames).call(classNames,String).join(" ");}}],[{key:"snap",value:function snap(date,scale,step){var clone=moment$1(date);if(scale=='year'){var year=clone.year()+Math.round(clone.month()/12);clone.year(Math.round(year/step)*step);clone.month(0);clone.date(0);clone.hours(0);clone.minutes(0);clone.seconds(0);clone.milliseconds(0);}else if(scale=='month'){if(clone.date()>15){clone.date(1);clone.add(1,'month');// important: first set Date to 1, after that change the month.
	}else {clone.date(1);}clone.hours(0);clone.minutes(0);clone.seconds(0);clone.milliseconds(0);}else if(scale=='week'){if(clone.weekday()>2){// doing it the momentjs locale aware way
	clone.weekday(0);clone.add(1,'week');}else {clone.weekday(0);}clone.hours(0);clone.minutes(0);clone.seconds(0);clone.milliseconds(0);}else if(scale=='day'){//noinspection FallthroughInSwitchStatementJS
	switch(step){case 5:case 2:clone.hours(Math.round(clone.hours()/24)*24);break;default:clone.hours(Math.round(clone.hours()/12)*12);break;}clone.minutes(0);clone.seconds(0);clone.milliseconds(0);}else if(scale=='weekday'){//noinspection FallthroughInSwitchStatementJS
	switch(step){case 5:case 2:clone.hours(Math.round(clone.hours()/12)*12);break;default:clone.hours(Math.round(clone.hours()/6)*6);break;}clone.minutes(0);clone.seconds(0);clone.milliseconds(0);}else if(scale=='hour'){switch(step){case 4:clone.minutes(Math.round(clone.minutes()/60)*60);break;default:clone.minutes(Math.round(clone.minutes()/30)*30);break;}clone.seconds(0);clone.milliseconds(0);}else if(scale=='minute'){//noinspection FallthroughInSwitchStatementJS
	switch(step){case 15:case 10:clone.minutes(Math.round(clone.minutes()/5)*5);clone.seconds(0);break;case 5:clone.seconds(Math.round(clone.seconds()/60)*60);break;default:clone.seconds(Math.round(clone.seconds()/30)*30);break;}clone.milliseconds(0);}else if(scale=='second'){//noinspection FallthroughInSwitchStatementJS
	switch(step){case 15:case 10:clone.seconds(Math.round(clone.seconds()/5)*5);clone.milliseconds(0);break;case 5:clone.milliseconds(Math.round(clone.milliseconds()/1000)*1000);break;default:clone.milliseconds(Math.round(clone.milliseconds()/500)*500);break;}}else if(scale=='millisecond'){var _step=step>5?step/2:1;clone.milliseconds(Math.round(clone.milliseconds()/_step)*_step);}return clone;}}]);return TimeStep;}();// Time formatting
	TimeStep.FORMAT={minorLabels:{millisecond:'SSS',second:'s',minute:'HH:mm',hour:'HH:mm',weekday:'ddd D',day:'D',week:'w',month:'MMM',year:'YYYY'},majorLabels:{millisecond:'HH:mm:ss',second:'D MMMM HH:mm',minute:'ddd D MMMM',hour:'ddd D MMMM',weekday:'MMMM YYYY',day:'MMMM YYYY',week:'MMMM YYYY',month:'YYYY',year:''}};/** A horizontal time axis */var TimeAxis=/*#__PURE__*/function(_Component2){inherits(TimeAxis,_Component2);var _super2=_createSuper$2(TimeAxis);/**
	 * @param {{dom: Object, domProps: Object, emitter: Emitter, range: Range}} body
	 * @param {Object} [options]        See TimeAxis.setOptions for the available
	 *                                  options.
	 * @constructor TimeAxis
	 * @extends Component
	 */function TimeAxis(body,options){var _this2;classCallCheck(this,TimeAxis);_this2=_super2.call(this);_this2.dom={foreground:null,lines:[],majorTexts:[],minorTexts:[],redundant:{lines:[],majorTexts:[],minorTexts:[]}};_this2.props={range:{start:0,end:0,minimumStep:0},lineTop:0};_this2.defaultOptions={orientation:{axis:'bottom'},// axis orientation: 'top' or 'bottom'
	showMinorLabels:true,showMajorLabels:true,showWeekScale:false,maxMinorChars:7,format:TimeStep.FORMAT,moment:moment$1,timeAxis:null};_this2.options=util$1.extend({},_this2.defaultOptions);_this2.body=body;// create the HTML DOM
	_this2._create();_this2.setOptions(options);return _this2;}/**
	   * Set options for the TimeAxis.
	   * Parameters will be merged in current options.
	   * @param {Object} options  Available options:
	   *                          {string} [orientation.axis]
	   *                          {boolean} [showMinorLabels]
	   *                          {boolean} [showMajorLabels]
	   *                          {boolean} [showWeekScale]
	   */createClass(TimeAxis,[{key:"setOptions",value:function setOptions(options){if(options){// copy all options that we know
	util$1.selectiveExtend(['showMinorLabels','showMajorLabels','showWeekScale','maxMinorChars','hiddenDates','timeAxis','moment','rtl'],this.options,options);// deep copy the format options
	util$1.selectiveDeepExtend(['format'],this.options,options);if('orientation'in options){if(typeof options.orientation==='string'){this.options.orientation.axis=options.orientation;}else if(_typeof_1(options.orientation)==='object'&&'axis'in options.orientation){this.options.orientation.axis=options.orientation.axis;}}// apply locale to moment.js
	// TODO: not so nice, this is applied globally to moment.js
	if('locale'in options){if(typeof moment$1.locale==='function'){// moment.js 2.8.1+
	moment$1.locale(options.locale);}else {moment$1.lang(options.locale);}}}}/**
	   * Create the HTML DOM for the TimeAxis
	   */},{key:"_create",value:function _create(){this.dom.foreground=document.createElement('div');this.dom.background=document.createElement('div');this.dom.foreground.className='vis-time-axis vis-foreground';this.dom.background.className='vis-time-axis vis-background';}/**
	   * Destroy the TimeAxis
	   */},{key:"destroy",value:function destroy(){// remove from DOM
	if(this.dom.foreground.parentNode){this.dom.foreground.parentNode.removeChild(this.dom.foreground);}if(this.dom.background.parentNode){this.dom.background.parentNode.removeChild(this.dom.background);}this.body=null;}/**
	   * Repaint the component
	   * @return {boolean} Returns true if the component is resized
	   */},{key:"redraw",value:function redraw(){var props=this.props;var foreground=this.dom.foreground;var background=this.dom.background;// determine the correct parent DOM element (depending on option orientation)
	var parent=this.options.orientation.axis=='top'?this.body.dom.top:this.body.dom.bottom;var parentChanged=foreground.parentNode!==parent;// calculate character width and height
	this._calculateCharSize();// TODO: recalculate sizes only needed when parent is resized or options is changed
	var showMinorLabels=this.options.showMinorLabels&&this.options.orientation.axis!=='none';var showMajorLabels=this.options.showMajorLabels&&this.options.orientation.axis!=='none';// determine the width and height of the elemens for the axis
	props.minorLabelHeight=showMinorLabels?props.minorCharHeight:0;props.majorLabelHeight=showMajorLabels?props.majorCharHeight:0;props.height=props.minorLabelHeight+props.majorLabelHeight;props.width=foreground.offsetWidth;props.minorLineHeight=this.body.domProps.root.height-props.majorLabelHeight-(this.options.orientation.axis=='top'?this.body.domProps.bottom.height:this.body.domProps.top.height);props.minorLineWidth=1;// TODO: really calculate width
	props.majorLineHeight=props.minorLineHeight+props.majorLabelHeight;props.majorLineWidth=1;// TODO: really calculate width
	//  take foreground and background offline while updating (is almost twice as fast)
	var foregroundNextSibling=foreground.nextSibling;var backgroundNextSibling=background.nextSibling;foreground.parentNode&&foreground.parentNode.removeChild(foreground);background.parentNode&&background.parentNode.removeChild(background);foreground.style.height="".concat(this.props.height,"px");this._repaintLabels();// put DOM online again (at the same place)
	if(foregroundNextSibling){parent.insertBefore(foreground,foregroundNextSibling);}else {parent.appendChild(foreground);}if(backgroundNextSibling){this.body.dom.backgroundVertical.insertBefore(background,backgroundNextSibling);}else {this.body.dom.backgroundVertical.appendChild(background);}return this._isResized()||parentChanged;}/**
	   * Repaint major and minor text labels and vertical grid lines
	   * @private
	   */},{key:"_repaintLabels",value:function _repaintLabels(){var orientation=this.options.orientation.axis;// calculate range and step (step such that we have space for 7 characters per label)
	var start=util$1.convert(this.body.range.start,'Number');var end=util$1.convert(this.body.range.end,'Number');var timeLabelsize=this.body.util.toTime((this.props.minorCharWidth||10)*this.options.maxMinorChars).valueOf();var minimumStep=timeLabelsize-getHiddenDurationBefore(this.options.moment,this.body.hiddenDates,this.body.range,timeLabelsize);minimumStep-=this.body.util.toTime(0).valueOf();var step=new TimeStep(new Date(start),new Date(end),minimumStep,this.body.hiddenDates,this.options);step.setMoment(this.options.moment);if(this.options.format){step.setFormat(this.options.format);}if(this.options.timeAxis){step.setScale(this.options.timeAxis);}this.step=step;// Move all DOM elements to a "redundant" list, where they
	// can be picked for re-use, and clear the lists with lines and texts.
	// At the end of the function _repaintLabels, left over elements will be cleaned up
	var dom=this.dom;dom.redundant.lines=dom.lines;dom.redundant.majorTexts=dom.majorTexts;dom.redundant.minorTexts=dom.minorTexts;dom.lines=[];dom.majorTexts=[];dom.minorTexts=[];var current;var next;var x;var xNext;var isMajor;var showMinorGrid;var width=0;var prevWidth;var line;var xFirstMajorLabel=undefined;var count=0;var MAX=1000;var className;step.start();next=step.getCurrent();xNext=this.body.util.toScreen(next);while(step.hasNext()&&count<MAX){count++;isMajor=step.isMajor();className=step.getClassName();current=next;x=xNext;step.next();next=step.getCurrent();xNext=this.body.util.toScreen(next);prevWidth=width;width=xNext-x;switch(step.scale){case'week':showMinorGrid=true;break;default:showMinorGrid=width>=prevWidth*0.4;break;// prevent displaying of the 31th of the month on a scale of 5 days
	}if(this.options.showMinorLabels&&showMinorGrid){var label=this._repaintMinorText(x,step.getLabelMinor(current),orientation,className);label.style.width="".concat(width,"px");// set width to prevent overflow
	}if(isMajor&&this.options.showMajorLabels){if(x>0){if(xFirstMajorLabel==undefined){xFirstMajorLabel=x;}label=this._repaintMajorText(x,step.getLabelMajor(current),orientation,className);}line=this._repaintMajorLine(x,width,orientation,className);}else {// minor line
	if(showMinorGrid){line=this._repaintMinorLine(x,width,orientation,className);}else {if(line){// adjust the width of the previous grid
	line.style.width="".concat(_parseInt$2(line.style.width)+width,"px");}}}}if(count===MAX&&!warnedForOverflow){console.warn("Something is wrong with the Timeline scale. Limited drawing of grid lines to ".concat(MAX," lines."));warnedForOverflow=true;}// create a major label on the left when needed
	if(this.options.showMajorLabels){var leftTime=this.body.util.toTime(0);// upper bound estimation
	var leftText=step.getLabelMajor(leftTime);var widthText=leftText.length*(this.props.majorCharWidth||10)+10;if(xFirstMajorLabel==undefined||widthText<xFirstMajorLabel){this._repaintMajorText(0,leftText,orientation,className);}}// Cleanup leftover DOM elements from the redundant list
	forEach$3(util$1).call(util$1,this.dom.redundant,function(arr){while(arr.length){var elem=arr.pop();if(elem&&elem.parentNode){elem.parentNode.removeChild(elem);}}});}/**
	   * Create a minor label for the axis at position x
	   * @param {number} x
	   * @param {string} text
	   * @param {string} orientation   "top" or "bottom" (default)
	   * @param {string} className
	   * @return {Element} Returns the HTML element of the created label
	   * @private
	   */},{key:"_repaintMinorText",value:function _repaintMinorText(x,text,orientation,className){// reuse redundant label
	var label=this.dom.redundant.minorTexts.shift();if(!label){// create new label
	var content=document.createTextNode('');label=document.createElement('div');label.appendChild(content);this.dom.foreground.appendChild(label);}this.dom.minorTexts.push(label);label.innerHTML=text;var y=orientation=='top'?this.props.majorLabelHeight:0;this._setXY(label,x,y);label.className="vis-text vis-minor ".concat(className);//label.title = title;  // TODO: this is a heavy operation
	return label;}/**
	   * Create a Major label for the axis at position x
	   * @param {number} x
	   * @param {string} text
	   * @param {string} orientation   "top" or "bottom" (default)
	   * @param {string} className
	   * @return {Element} Returns the HTML element of the created label
	   * @private
	   */},{key:"_repaintMajorText",value:function _repaintMajorText(x,text,orientation,className){// reuse redundant label
	var label=this.dom.redundant.majorTexts.shift();if(!label){// create label
	var content=document.createElement('div');label=document.createElement('div');label.appendChild(content);this.dom.foreground.appendChild(label);}label.childNodes[0].innerHTML=text;label.className="vis-text vis-major ".concat(className);//label.title = title; // TODO: this is a heavy operation
	var y=orientation=='top'?0:this.props.minorLabelHeight;this._setXY(label,x,y);this.dom.majorTexts.push(label);return label;}/**
	   * sets xy
	   * @param {string} label
	   * @param {number} x
	   * @param {number} y
	   * @private
	   */},{key:"_setXY",value:function _setXY(label,x,y){var _context19;// If rtl is true, inverse x.
	var directionX=this.options.rtl?x*-1:x;label.style.transform=concat$2(_context19="translate(".concat(directionX,"px, ")).call(_context19,y,"px)");}/**
	   * Create a minor line for the axis at position x
	   * @param {number} left
	   * @param {number} width
	   * @param {string} orientation   "top" or "bottom" (default)
	   * @param {string} className
	   * @return {Element} Returns the created line
	   * @private
	   */},{key:"_repaintMinorLine",value:function _repaintMinorLine(left,width,orientation,className){var _context20;// reuse redundant line
	var line=this.dom.redundant.lines.shift();if(!line){// create vertical line
	line=document.createElement('div');this.dom.background.appendChild(line);}this.dom.lines.push(line);var props=this.props;line.style.width="".concat(width,"px");line.style.height="".concat(props.minorLineHeight,"px");var y=orientation=='top'?props.majorLabelHeight:this.body.domProps.top.height;var x=left-props.minorLineWidth/2;this._setXY(line,x,y);line.className=concat$2(_context20="vis-grid ".concat(this.options.rtl?'vis-vertical-rtl':'vis-vertical'," vis-minor ")).call(_context20,className);return line;}/**
	   * Create a Major line for the axis at position x
	   * @param {number} left
	   * @param {number} width
	   * @param {string} orientation   "top" or "bottom" (default)
	   * @param {string} className
	   * @return {Element} Returns the created line
	   * @private
	   */},{key:"_repaintMajorLine",value:function _repaintMajorLine(left,width,orientation,className){var _context21;// reuse redundant line
	var line=this.dom.redundant.lines.shift();if(!line){// create vertical line
	line=document.createElement('div');this.dom.background.appendChild(line);}this.dom.lines.push(line);var props=this.props;line.style.width="".concat(width,"px");line.style.height="".concat(props.majorLineHeight,"px");var y=orientation=='top'?0:this.body.domProps.top.height;var x=left-props.majorLineWidth/2;this._setXY(line,x,y);line.className=concat$2(_context21="vis-grid ".concat(this.options.rtl?'vis-vertical-rtl':'vis-vertical'," vis-major ")).call(_context21,className);return line;}/**
	   * Determine the size of text on the axis (both major and minor axis).
	   * The size is calculated only once and then cached in this.props.
	   * @private
	   */},{key:"_calculateCharSize",value:function _calculateCharSize(){// Note: We calculate char size with every redraw. Size may change, for
	// example when any of the timelines parents had display:none for example.
	// determine the char width and height on the minor axis
	if(!this.dom.measureCharMinor){this.dom.measureCharMinor=document.createElement('DIV');this.dom.measureCharMinor.className='vis-text vis-minor vis-measure';this.dom.measureCharMinor.style.position='absolute';this.dom.measureCharMinor.appendChild(document.createTextNode('0'));this.dom.foreground.appendChild(this.dom.measureCharMinor);}this.props.minorCharHeight=this.dom.measureCharMinor.clientHeight;this.props.minorCharWidth=this.dom.measureCharMinor.clientWidth;// determine the char width and height on the major axis
	if(!this.dom.measureCharMajor){this.dom.measureCharMajor=document.createElement('DIV');this.dom.measureCharMajor.className='vis-text vis-major vis-measure';this.dom.measureCharMajor.style.position='absolute';this.dom.measureCharMajor.appendChild(document.createTextNode('0'));this.dom.foreground.appendChild(this.dom.measureCharMajor);}this.props.majorCharHeight=this.dom.measureCharMajor.clientHeight;this.props.majorCharWidth=this.dom.measureCharMajor.clientWidth;}}]);return TimeAxis;}(Component);var warnedForOverflow=false;/**
	 * Turn an element into an clickToUse element.
	 * When not active, the element has a transparent overlay. When the overlay is
	 * clicked, the mode is changed to active.
	 * When active, the element is displayed with a blue border around it, and
	 * the interactive contents of the element can be used. When clicked outside
	 * the element, the elements mode is changed to inactive.
	 * @param {Element} container
	 * @constructor Activator
	 */function Activator$2(container){var _context22,_context23;this.active=false;this.dom={container:container};this.dom.overlay=document.createElement('div');this.dom.overlay.className='vis-overlay';this.dom.container.appendChild(this.dom.overlay);this.hammer=Hammer$1(this.dom.overlay);this.hammer.on('tap',bind$2(_context22=this._onTapOverlay).call(_context22,this));// block all touch events (except tap)
	var me=this;var events=['tap','doubletap','press','pinch','pan','panstart','panmove','panend'];forEach$3(events).call(events,function(event){me.hammer.on(event,function(event){event.stopPropagation();});});// attach a click event to the window, in order to deactivate when clicking outside the timeline
	if(document&&document.body){this.onClick=function(event){if(!_hasParent$1(event.target,container)){me.deactivate();}};document.body.addEventListener('click',this.onClick);}if(this.keycharm!==undefined){this.keycharm.destroy();}this.keycharm=keycharm();// keycharm listener only bounded when active)
	this.escListener=bind$2(_context23=this.deactivate).call(_context23,this);}// turn into an event emitter
	componentEmitter(Activator$2.prototype);// The currently active activator
	Activator$2.current=null;/**
	 * Destroy the activator. Cleans up all created DOM and event listeners
	 */Activator$2.prototype.destroy=function(){this.deactivate();// remove dom
	this.dom.overlay.parentNode.removeChild(this.dom.overlay);// remove global event listener
	if(this.onClick){document.body.removeEventListener('click',this.onClick);}// remove keycharm
	if(this.keycharm!==undefined){this.keycharm.destroy();}this.keycharm=null;// cleanup hammer instances
	this.hammer.destroy();this.hammer=null;// FIXME: cleaning up hammer instances doesn't work (Timeline not removed from memory)
	};/**
	 * Activate the element
	 * Overlay is hidden, element is decorated with a blue shadow border
	 */Activator$2.prototype.activate=function(){var _context24;// we allow only one active activator at a time
	if(Activator$2.current){Activator$2.current.deactivate();}Activator$2.current=this;this.active=true;this.dom.overlay.style.display='none';util$1.addClassName(this.dom.container,'vis-active');this.emit('change');this.emit('activate');// ugly hack: bind ESC after emitting the events, as the Network rebinds all
	// keyboard events on a 'change' event
	bind$2(_context24=this.keycharm).call(_context24,'esc',this.escListener);};/**
	 * Deactivate the element
	 * Overlay is displayed on top of the element
	 */Activator$2.prototype.deactivate=function(){if(Activator$2.current===this){Activator$2.current=null;}this.active=false;this.dom.overlay.style.display='';util$1.removeClassName(this.dom.container,'vis-active');this.keycharm.unbind('esc',this.escListener);this.emit('change');this.emit('deactivate');};/**
	 * Handle a tap event: activate the container
	 * @param {Event}  event   The event
	 * @private
	 */Activator$2.prototype._onTapOverlay=function(event){// activate the container
	this.activate();event.stopPropagation();};/**
	 * Test whether the element has the requested parent element somewhere in
	 * its chain of parent nodes.
	 * @param {HTMLElement} element
	 * @param {HTMLElement} parent
	 * @returns {boolean} Returns true when the parent is found somewhere in the
	 *                    chain of parent nodes.
	 * @private
	 */function _hasParent$1(element,parent){while(element){if(element===parent){return true;}element=element.parentNode;}return false;}/*
	 * IMPORTANT: Locales for Moment has to be imported in the legacy and standalone
	 * entry points. For the peer build it's users responsibility to do so.
	 */ // English
	var en={current:'current',time:'time',deleteSelected:'Delete selected'};var en_EN=en;var en_US=en;// Italiano
	var it={current:'attuale',time:'tempo',deleteSelected:'Cancella la selezione'};var it_IT=it;var it_CH=it;// Dutch
	var nl={current:'huidige',time:'tijd',deleteSelected:'Selectie verwijderen'};var nl_NL=nl;var nl_BE=nl;// German
	var de={current:'Aktuelle',time:'Zeit',deleteSelected:"L\xF6sche Auswahl"};var de_DE=de;// French
	var fr={current:'actuel',time:'heure',deleteSelected:'Effacer la selection'};var fr_FR=fr;var fr_CA=fr;var fr_BE=fr;// Espanol
	var es={current:'corriente',time:'hora',deleteSelected:"Eliminar selecci\xF3n"};var es_ES=es;// Ukrainian
	var uk={current:'поточний',time:'час',deleteSelected:'Видалити обране'};var uk_UA=uk;// Russian
	var ru={current:'текущее',time:'время',deleteSelected:'Удалить выбранное'};var ru_RU=ru;// Polish
	var pl={current:'aktualny',time:'czas',deleteSelected:'Usuń wybrane'};var pl_PL=pl;// Japanese
	var ja={current:'現在',time:'時刻',deleteSelected:'選択されたものを削除'};var ja_JP=ja;var locales$1={en:en,en_EN:en_EN,en_US:en_US,it:it,it_IT:it_IT,it_CH:it_CH,nl:nl,nl_NL:nl_NL,nl_BE:nl_BE,de:de,de_DE:de_DE,fr:fr,fr_FR:fr_FR,fr_CA:fr_CA,fr_BE:fr_BE,es:es,es_ES:es_ES,uk:uk,uk_UA:uk_UA,ru:ru,ru_RU:ru_RU,pl:pl,pl_PL:pl_PL,ja:ja,ja_JP:ja_JP};/** A custom time bar */var CustomTime=/*#__PURE__*/function(_Component3){inherits(CustomTime,_Component3);var _super3=_createSuper$2(CustomTime);/**
	 * @param {{range: Range, dom: Object}} body
	 * @param {Object} [options]        Available parameters:
	 *                                  {number | string} id
	 *                                  {string} locales
	 *                                  {string} locale
	 * @constructor CustomTime
	 * @extends Component
	 */function CustomTime(body,options){var _context25;var _this3;classCallCheck(this,CustomTime);_this3=_super3.call(this);_this3.body=body;// default options
	_this3.defaultOptions={moment:moment$1,locales:locales$1,locale:'en',id:undefined,title:undefined};_this3.options=util$1.extend({},_this3.defaultOptions);_this3.setOptions(options);_this3.options.locales=util$1.extend({},locales$1,_this3.options.locales);var defaultLocales=_this3.defaultOptions.locales[_this3.defaultOptions.locale];forEach$3(_context25=keys$6(_this3.options.locales)).call(_context25,function(locale){_this3.options.locales[locale]=util$1.extend({},defaultLocales,_this3.options.locales[locale]);});if(options&&options.time!=null){_this3.customTime=options.time;}else {_this3.customTime=new Date();}_this3.eventParams={};// stores state parameters while dragging the bar
	// create the DOM
	_this3._create();return _this3;}/**
	   * Set options for the component. Options will be merged in current options.
	   * @param {Object} options  Available parameters:
	   *                                  {number | string} id
	   *                                  {string} locales
	   *                                  {string} locale
	   */createClass(CustomTime,[{key:"setOptions",value:function setOptions(options){if(options){// copy all options that we know
	util$1.selectiveExtend(['moment','locale','locales','id','title','rtl','snap'],this.options,options);}}/**
	   * Create the DOM for the custom time
	   * @private
	   */},{key:"_create",value:function _create(){var _context26,_context27,_context28;var bar=document.createElement('div');bar['custom-time']=this;bar.className="vis-custom-time ".concat(this.options.id||'');bar.style.position='absolute';bar.style.top='0px';bar.style.height='100%';this.bar=bar;var drag=document.createElement('div');drag.style.position='relative';drag.style.top='0px';if(this.options.rtl){drag.style.right='-10px';}else {drag.style.left='-10px';}drag.style.height='100%';drag.style.width='20px';/**
	     *
	     * @param {WheelEvent} e
	     */function onMouseWheel(e){this.body.range._onMouseWheel(e);}if(drag.addEventListener){// IE9, Chrome, Safari, Opera
	drag.addEventListener("mousewheel",bind$2(onMouseWheel).call(onMouseWheel,this),false);// Firefox
	drag.addEventListener("DOMMouseScroll",bind$2(onMouseWheel).call(onMouseWheel,this),false);}else {// IE 6/7/8
	drag.attachEvent("onmousewheel",bind$2(onMouseWheel).call(onMouseWheel,this));}bar.appendChild(drag);// attach event listeners
	this.hammer=new Hammer$1(drag);this.hammer.on('panstart',bind$2(_context26=this._onDragStart).call(_context26,this));this.hammer.on('panmove',bind$2(_context27=this._onDrag).call(_context27,this));this.hammer.on('panend',bind$2(_context28=this._onDragEnd).call(_context28,this));this.hammer.get('pan').set({threshold:5,direction:Hammer$1.DIRECTION_ALL});}/**
	   * Destroy the CustomTime bar
	   */},{key:"destroy",value:function destroy(){this.hide();this.hammer.destroy();this.hammer=null;this.body=null;}/**
	   * Repaint the component
	   * @return {boolean} Returns true if the component is resized
	   */},{key:"redraw",value:function redraw(){var parent=this.body.dom.backgroundVertical;if(this.bar.parentNode!=parent){// attach to the dom
	if(this.bar.parentNode){this.bar.parentNode.removeChild(this.bar);}parent.appendChild(this.bar);}var x=this.body.util.toScreen(this.customTime);var locale=this.options.locales[this.options.locale];if(!locale){if(!this.warned){console.warn("WARNING: options.locales['".concat(this.options.locale,"'] not found. See https://visjs.github.io/vis-timeline/docs/timeline/#Localization"));this.warned=true;}locale=this.options.locales['en'];// fall back on english when not available
	}var title=this.options.title;// To hide the title completely use empty string ''.
	if(title===undefined){var _context29;title=concat$2(_context29="".concat(locale.time,": ")).call(_context29,this.options.moment(this.customTime).format('dddd, MMMM Do YYYY, H:mm:ss'));title=title.charAt(0).toUpperCase()+title.substring(1);}else if(typeof title==="function"){title=title.call(this,this.customTime);}this.options.rtl?this.bar.style.right="".concat(x,"px"):this.bar.style.left="".concat(x,"px");this.bar.title=title;return false;}/**
	   * Remove the CustomTime from the DOM
	   */},{key:"hide",value:function hide(){// remove the line from the DOM
	if(this.bar.parentNode){this.bar.parentNode.removeChild(this.bar);}}/**
	   * Set custom time.
	   * @param {Date | number | string} time
	   */},{key:"setCustomTime",value:function setCustomTime(time){this.customTime=util$1.convert(time,'Date');this.redraw();}/**
	   * Retrieve the current custom time.
	   * @return {Date} customTime
	   */},{key:"getCustomTime",value:function getCustomTime(){return new Date(this.customTime.valueOf());}/**
	   * Set custom marker.
	   * @param {string} [title] Title of the custom marker
	   * @param {boolean} [editable] Make the custom marker editable.
	   */},{key:"setCustomMarker",value:function setCustomMarker(title,editable){var marker=document.createElement('div');marker.className="vis-custom-time-marker";marker.innerHTML=title;marker.style.position='absolute';if(editable){var _context30,_context31;marker.setAttribute('contenteditable','true');marker.addEventListener('pointerdown',function(){marker.focus();});marker.addEventListener('input',bind$2(_context30=this._onMarkerChange).call(_context30,this));// The editable div element has no change event, so here emulates the change event.
	marker.title=title;marker.addEventListener('blur',bind$2(_context31=function _context31(event){if(this.title!=event.target.innerHTML){this._onMarkerChanged(event);this.title=event.target.innerHTML;}}).call(_context31,this));}this.bar.appendChild(marker);}/**
	    * Set custom title.
	    * @param {Date | number | string} title
	    */},{key:"setCustomTitle",value:function setCustomTitle(title){this.options.title=title;}/**
	   * Start moving horizontally
	   * @param {Event} event
	   * @private
	   */},{key:"_onDragStart",value:function _onDragStart(event){this.eventParams.dragging=true;this.eventParams.customTime=this.customTime;event.stopPropagation();}/**
	   * Perform moving operating.
	   * @param {Event} event
	   * @private
	   */},{key:"_onDrag",value:function _onDrag(event){if(!this.eventParams.dragging)return;var deltaX=this.options.rtl?-1*event.deltaX:event.deltaX;var x=this.body.util.toScreen(this.eventParams.customTime)+deltaX;var time=this.body.util.toTime(x);var scale=this.body.util.getScale();var step=this.body.util.getStep();var snap=this.options.snap;var snappedTime=snap?snap(time,scale,step):time;this.setCustomTime(snappedTime);// fire a timechange event
	this.body.emitter.emit('timechange',{id:this.options.id,time:new Date(this.customTime.valueOf()),event:event});event.stopPropagation();}/**
	   * Stop moving operating.
	   * @param {Event} event
	   * @private
	   */},{key:"_onDragEnd",value:function _onDragEnd(event){if(!this.eventParams.dragging)return;// fire a timechanged event
	this.body.emitter.emit('timechanged',{id:this.options.id,time:new Date(this.customTime.valueOf()),event:event});event.stopPropagation();}/**
	   * Perform input operating.
	   * @param {Event} event
	   * @private
	   */},{key:"_onMarkerChange",value:function _onMarkerChange(event){this.body.emitter.emit('markerchange',{id:this.options.id,title:event.target.innerHTML,event:event});event.stopPropagation();}/**
	   * Perform change operating.
	   * @param {Event} event
	   * @private
	   */},{key:"_onMarkerChanged",value:function _onMarkerChanged(event){this.body.emitter.emit('markerchanged',{id:this.options.id,title:event.target.innerHTML,event:event});event.stopPropagation();}/**
	   * Find a custom time from an event target:
	   * searches for the attribute 'custom-time' in the event target's element tree
	   * @param {Event} event
	   * @return {CustomTime | null} customTime
	   */}],[{key:"customTimeFromTarget",value:function customTimeFromTarget(event){var target=event.target;while(target){if(target.hasOwnProperty('custom-time')){return target['custom-time'];}target=target.parentNode;}return null;}}]);return CustomTime;}(Component);/**
	 * Create a timeline visualization
	 * @constructor Core
	 */var Core=/*#__PURE__*/function(){function Core(){classCallCheck(this,Core);}createClass(Core,[{key:"_create",/**
	   * Create the main DOM for the Core: a root panel containing left, right,
	   * top, bottom, content, and background panel.
	   * @param {Element} container  The container element where the Core will
	   *                             be attached.
	   * @protected
	   */value:function _create(container){var _this4=this,_context32,_context33,_context34;this.dom={};this.dom.container=container;this.dom.container.style.position='relative';this.dom.root=document.createElement('div');this.dom.background=document.createElement('div');this.dom.backgroundVertical=document.createElement('div');this.dom.backgroundHorizontal=document.createElement('div');this.dom.centerContainer=document.createElement('div');this.dom.leftContainer=document.createElement('div');this.dom.rightContainer=document.createElement('div');this.dom.center=document.createElement('div');this.dom.left=document.createElement('div');this.dom.right=document.createElement('div');this.dom.top=document.createElement('div');this.dom.bottom=document.createElement('div');this.dom.shadowTop=document.createElement('div');this.dom.shadowBottom=document.createElement('div');this.dom.shadowTopLeft=document.createElement('div');this.dom.shadowBottomLeft=document.createElement('div');this.dom.shadowTopRight=document.createElement('div');this.dom.shadowBottomRight=document.createElement('div');this.dom.rollingModeBtn=document.createElement('div');this.dom.loadingScreen=document.createElement('div');this.dom.root.className='vis-timeline';this.dom.background.className='vis-panel vis-background';this.dom.backgroundVertical.className='vis-panel vis-background vis-vertical';this.dom.backgroundHorizontal.className='vis-panel vis-background vis-horizontal';this.dom.centerContainer.className='vis-panel vis-center';this.dom.leftContainer.className='vis-panel vis-left';this.dom.rightContainer.className='vis-panel vis-right';this.dom.top.className='vis-panel vis-top';this.dom.bottom.className='vis-panel vis-bottom';this.dom.left.className='vis-content';this.dom.center.className='vis-content';this.dom.right.className='vis-content';this.dom.shadowTop.className='vis-shadow vis-top';this.dom.shadowBottom.className='vis-shadow vis-bottom';this.dom.shadowTopLeft.className='vis-shadow vis-top';this.dom.shadowBottomLeft.className='vis-shadow vis-bottom';this.dom.shadowTopRight.className='vis-shadow vis-top';this.dom.shadowBottomRight.className='vis-shadow vis-bottom';this.dom.rollingModeBtn.className='vis-rolling-mode-btn';this.dom.loadingScreen.className='vis-loading-screen';this.dom.root.appendChild(this.dom.background);this.dom.root.appendChild(this.dom.backgroundVertical);this.dom.root.appendChild(this.dom.backgroundHorizontal);this.dom.root.appendChild(this.dom.centerContainer);this.dom.root.appendChild(this.dom.leftContainer);this.dom.root.appendChild(this.dom.rightContainer);this.dom.root.appendChild(this.dom.top);this.dom.root.appendChild(this.dom.bottom);this.dom.root.appendChild(this.dom.rollingModeBtn);this.dom.centerContainer.appendChild(this.dom.center);this.dom.leftContainer.appendChild(this.dom.left);this.dom.rightContainer.appendChild(this.dom.right);this.dom.centerContainer.appendChild(this.dom.shadowTop);this.dom.centerContainer.appendChild(this.dom.shadowBottom);this.dom.leftContainer.appendChild(this.dom.shadowTopLeft);this.dom.leftContainer.appendChild(this.dom.shadowBottomLeft);this.dom.rightContainer.appendChild(this.dom.shadowTopRight);this.dom.rightContainer.appendChild(this.dom.shadowBottomRight);// size properties of each of the panels
	this.props={root:{},background:{},centerContainer:{},leftContainer:{},rightContainer:{},center:{},left:{},right:{},top:{},bottom:{},border:{},scrollTop:0,scrollTopMin:0};this.on('rangechange',function(){if(_this4.initialDrawDone===true){_this4._redraw();}});this.on('rangechanged',function(){if(!_this4.initialRangeChangeDone){_this4.initialRangeChangeDone=true;}});this.on('touch',bind$2(_context32=this._onTouch).call(_context32,this));this.on('panmove',bind$2(_context33=this._onDrag).call(_context33,this));var me=this;this._origRedraw=bind$2(_context34=this._redraw).call(_context34,this);this._redraw=util$1.throttle(this._origRedraw);this.on('_change',function(properties){if(me.itemSet&&me.itemSet.initialItemSetDrawn&&properties&&properties.queue==true){me._redraw();}else {me._origRedraw();}});// create event listeners for all interesting events, these events will be
	// emitted via emitter
	this.hammer=new Hammer$1(this.dom.root);var pinchRecognizer=this.hammer.get('pinch').set({enable:true});pinchRecognizer&&disablePreventDefaultVertically(pinchRecognizer);this.hammer.get('pan').set({threshold:5,direction:Hammer$1.DIRECTION_ALL});this.timelineListeners={};var events=['tap','doubletap','press','pinch','pan','panstart','panmove','panend'// TODO: cleanup
	//'touch', 'pinch',
	//'tap', 'doubletap', 'hold',
	//'dragstart', 'drag', 'dragend',
	//'mousewheel', 'DOMMouseScroll' // DOMMouseScroll is needed for Firefox
	];forEach$3(events).call(events,function(type){var listener=function listener(event){if(me.isActive()){me.emit(type,event);}};me.hammer.on(type,listener);me.timelineListeners[type]=listener;});// emulate a touch event (emitted before the start of a pan, pinch, tap, or press)
	onTouch(this.hammer,function(event){me.emit('touch',event);});// emulate a release event (emitted after a pan, pinch, tap, or press)
	onRelease(this.hammer,function(event){me.emit('release',event);});/**
	     *
	     * @param {WheelEvent} event
	     */function onMouseWheel(event){// Reasonable default wheel deltas
	var LINE_HEIGHT=40;var PAGE_HEIGHT=800;if(this.isActive()){this.emit('mousewheel',event);}// deltaX and deltaY normalization from jquery.mousewheel.js
	var deltaX=0;var deltaY=0;// Old school scrollwheel delta
	if('detail'in event){deltaY=event.detail*-1;}if('wheelDelta'in event){deltaY=event.wheelDelta;}if('wheelDeltaY'in event){deltaY=event.wheelDeltaY;}if('wheelDeltaX'in event){deltaX=event.wheelDeltaX*-1;}// Firefox < 17 horizontal scrolling related to DOMMouseScroll event
	if('axis'in event&&event.axis===event.HORIZONTAL_AXIS){deltaX=deltaY*-1;deltaY=0;}// New school wheel delta (wheel event)
	if('deltaY'in event){deltaY=event.deltaY*-1;}if('deltaX'in event){deltaX=event.deltaX;}// Normalize deltas
	if(event.deltaMode){if(event.deltaMode===1){// delta in LINE units
	deltaX*=LINE_HEIGHT;deltaY*=LINE_HEIGHT;}else {// delta in PAGE units
	deltaX*=LINE_HEIGHT;deltaY*=PAGE_HEIGHT;}}// Prevent scrolling when zooming (no zoom key, or pressing zoom key)
	if(this.options.preferZoom){if(!this.options.zoomKey||event[this.options.zoomKey])return;}else {if(this.options.zoomKey&&event[this.options.zoomKey])return;}// Don't preventDefault if you can't scroll
	if(!this.options.verticalScroll&&!this.options.horizontalScroll)return;if(this.options.verticalScroll&&Math.abs(deltaY)>=Math.abs(deltaX)){var current=this.props.scrollTop;var adjusted=current+deltaY;if(this.isActive()){var newScrollTop=this._setScrollTop(adjusted);if(newScrollTop!==current){this._redraw();this.emit('scroll',event);// Prevent default actions caused by mouse wheel
	// (else the page and timeline both scroll)
	event.preventDefault();}}}else if(this.options.horizontalScroll){var delta=Math.abs(deltaX)>=Math.abs(deltaY)?deltaX:deltaY;// calculate a single scroll jump relative to the range scale
	var diff=delta/120*(this.range.end-this.range.start)/20;// calculate new start and end
	var newStart=this.range.start+diff;var newEnd=this.range.end+diff;var options={animation:false,byUser:true,event:event};this.range.setRange(newStart,newEnd,options);event.preventDefault();}}// Add modern wheel event listener
	var wheelType="onwheel"in document.createElement("div")?"wheel":// Modern browsers support "wheel"
	document.onmousewheel!==undefined?"mousewheel":// Webkit and IE support at least "mousewheel"
	// DOMMouseScroll - Older Firefox versions use "DOMMouseScroll"
	// onmousewheel - All the use "onmousewheel"
	this.dom.centerContainer.addEventListener?"DOMMouseScroll":"onmousewheel";this.dom.top.addEventListener?"DOMMouseScroll":"onmousewheel";this.dom.bottom.addEventListener?"DOMMouseScroll":"onmousewheel";this.dom.centerContainer.addEventListener(wheelType,bind$2(onMouseWheel).call(onMouseWheel,this),false);this.dom.top.addEventListener(wheelType,bind$2(onMouseWheel).call(onMouseWheel,this),false);this.dom.bottom.addEventListener(wheelType,bind$2(onMouseWheel).call(onMouseWheel,this),false);/**
	     *
	     * @param {scroll} event
	     */function onMouseScrollSide(event){if(!me.options.verticalScroll)return;if(me._isProgramaticallyScrolled){me._isProgramaticallyScrolled=false;return;}event.preventDefault();if(me.isActive()){var adjusted=-event.target.scrollTop;me._setScrollTop(adjusted);me._redraw();me.emit('scrollSide',event);}}this.dom.left.parentNode.addEventListener('scroll',bind$2(onMouseScrollSide).call(onMouseScrollSide,this));this.dom.right.parentNode.addEventListener('scroll',bind$2(onMouseScrollSide).call(onMouseScrollSide,this));var itemAddedToTimeline=false;/**
	     *
	     * @param {dragover} event
	     * @returns {boolean}
	     */function handleDragOver(event){var _context35;if(event.preventDefault){me.emit('dragover',me.getEventProperties(event));event.preventDefault();// Necessary. Allows us to drop.
	}// make sure your target is a timeline element
	if(!(indexOf$3(_context35=event.target.className).call(_context35,"timeline")>-1))return;// make sure only one item is added every time you're over the timeline
	if(itemAddedToTimeline)return;event.dataTransfer.dropEffect='move';itemAddedToTimeline=true;return false;}/**
	     *
	     * @param {drop} event
	     * @returns {boolean}
	     */function handleDrop(event){// prevent redirect to blank page - Firefox
	if(event.preventDefault){event.preventDefault();}if(event.stopPropagation){event.stopPropagation();}// return when dropping non-timeline items
	try{var itemData=JSON.parse(event.dataTransfer.getData("text"));if(!itemData||!itemData.content)return;}catch(err){return false;}itemAddedToTimeline=false;event.center={x:event.clientX,y:event.clientY};if(itemData.target!=='item'){me.itemSet._onAddItem(event);}else {me.itemSet._onDropObjectOnItem(event);}me.emit('drop',me.getEventProperties(event));return false;}this.dom.center.addEventListener('dragover',bind$2(handleDragOver).call(handleDragOver,this),false);this.dom.center.addEventListener('drop',bind$2(handleDrop).call(handleDrop,this),false);this.customTimes=[];// store state information needed for touch events
	this.touch={};this.redrawCount=0;this.initialDrawDone=false;this.initialRangeChangeDone=false;// attach the root panel to the provided container
	if(!container)throw new Error('No container provided');container.appendChild(this.dom.root);container.appendChild(this.dom.loadingScreen);}/**
	   * Set options. Options will be passed to all components loaded in the Timeline.
	   * @param {Object} [options]
	   *                           {String} orientation
	   *                              Vertical orientation for the Timeline,
	   *                              can be 'bottom' (default) or 'top'.
	   *                           {string | number} width
	   *                              Width for the timeline, a number in pixels or
	   *                              a css string like '1000px' or '75%'. '100%' by default.
	   *                           {string | number} height
	   *                              Fixed height for the Timeline, a number in pixels or
	   *                              a css string like '400px' or '75%'. If undefined,
	   *                              The Timeline will automatically size such that
	   *                              its contents fit.
	   *                           {string | number} minHeight
	   *                              Minimum height for the Timeline, a number in pixels or
	   *                              a css string like '400px' or '75%'.
	   *                           {string | number} maxHeight
	   *                              Maximum height for the Timeline, a number in pixels or
	   *                              a css string like '400px' or '75%'.
	   *                           {number | Date | string} start
	   *                              Start date for the visible window
	   *                           {number | Date | string} end
	   *                              End date for the visible window
	   */},{key:"setOptions",value:function setOptions(options){var _context38;if(options){// copy the known options
	var fields=['width','height','minHeight','maxHeight','autoResize','start','end','clickToUse','dataAttributes','hiddenDates','locale','locales','moment','preferZoom','rtl','zoomKey','horizontalScroll','verticalScroll','longSelectPressTime'];util$1.selectiveExtend(fields,this.options,options);this.dom.rollingModeBtn.style.visibility='hidden';if(this.options.rtl){this.dom.container.style.direction="rtl";this.dom.backgroundVertical.className='vis-panel vis-background vis-vertical-rtl';}if(this.options.verticalScroll){if(this.options.rtl){this.dom.rightContainer.className='vis-panel vis-right vis-vertical-scroll';}else {this.dom.leftContainer.className='vis-panel vis-left vis-vertical-scroll';}}if(_typeof_1(this.options.orientation)!=='object'){this.options.orientation={item:undefined,axis:undefined};}if('orientation'in options){if(typeof options.orientation==='string'){this.options.orientation={item:options.orientation,axis:options.orientation};}else if(_typeof_1(options.orientation)==='object'){if('item'in options.orientation){this.options.orientation.item=options.orientation.item;}if('axis'in options.orientation){this.options.orientation.axis=options.orientation.axis;}}}if(this.options.orientation.axis==='both'){if(!this.timeAxis2){var timeAxis2=this.timeAxis2=new TimeAxis(this.body);timeAxis2.setOptions=function(options){var _options=options?util$1.extend({},options):{};_options.orientation='top';// override the orientation option, always top
	TimeAxis.prototype.setOptions.call(timeAxis2,_options);};this.components.push(timeAxis2);}}else {if(this.timeAxis2){var _context36;var index=indexOf$3(_context36=this.components).call(_context36,this.timeAxis2);if(index!==-1){var _context37;splice$2(_context37=this.components).call(_context37,index,1);}this.timeAxis2.destroy();this.timeAxis2=null;}}// if the graph2d's drawPoints is a function delegate the callback to the onRender property
	if(typeof options.drawPoints=='function'){options.drawPoints={onRender:options.drawPoints};}if('hiddenDates'in this.options){convertHiddenOptions(this.options.moment,this.body,this.options.hiddenDates);}if('clickToUse'in options){if(options.clickToUse){if(!this.activator){this.activator=new Activator$2(this.dom.root);}}else {if(this.activator){this.activator.destroy();delete this.activator;}}}// enable/disable autoResize
	this._initAutoResize();}// propagate options to all components
	forEach$3(_context38=this.components).call(_context38,function(component){return component.setOptions(options);});// enable/disable configure
	if('configure'in options){var _context39;if(!this.configurator){this.configurator=this._createConfigurator();}this.configurator.setOptions(options.configure);// collect the settings of all components, and pass them to the configuration system
	var appliedOptions=util$1.deepExtend({},this.options);forEach$3(_context39=this.components).call(_context39,function(component){util$1.deepExtend(appliedOptions,component.options);});this.configurator.setModuleOptions({global:appliedOptions});}this._redraw();}/**
	   * Returns true when the Timeline is active.
	   * @returns {boolean}
	   */},{key:"isActive",value:function isActive(){return !this.activator||this.activator.active;}/**
	   * Destroy the Core, clean up all DOM elements and event listeners.
	   */},{key:"destroy",value:function destroy(){var _context40;// unbind datasets
	this.setItems(null);this.setGroups(null);// remove all event listeners
	this.off();// stop checking for changed size
	this._stopAutoResize();// remove from DOM
	if(this.dom.root.parentNode){this.dom.root.parentNode.removeChild(this.dom.root);}this.dom=null;// remove Activator
	if(this.activator){this.activator.destroy();delete this.activator;}// cleanup hammer touch events
	for(var _event in this.timelineListeners){if(this.timelineListeners.hasOwnProperty(_event)){delete this.timelineListeners[_event];}}this.timelineListeners=null;this.hammer&&this.hammer.destroy();this.hammer=null;// give all components the opportunity to cleanup
	forEach$3(_context40=this.components).call(_context40,function(component){return component.destroy();});this.body=null;}/**
	   * Set a custom time bar
	   * @param {Date} time
	   * @param {number} [id=undefined] Optional id of the custom time bar to be adjusted.
	   */},{key:"setCustomTime",value:function setCustomTime(time,id){var _context41;var customTimes=filter$2(_context41=this.customTimes).call(_context41,function(component){return id===component.options.id;});if(customTimes.length===0){throw new Error("No custom time bar found with id ".concat(stringify$2(id)));}if(customTimes.length>0){customTimes[0].setCustomTime(time);}}/**
	   * Retrieve the current custom time.
	   * @param {number} [id=undefined]    Id of the custom time bar.
	   * @return {Date | undefined} customTime
	   */},{key:"getCustomTime",value:function getCustomTime(id){var _context42;var customTimes=filter$2(_context42=this.customTimes).call(_context42,function(component){return component.options.id===id;});if(customTimes.length===0){throw new Error("No custom time bar found with id ".concat(stringify$2(id)));}return customTimes[0].getCustomTime();}/**
	   * Set a custom marker for the custom time bar.
	   * @param {string} [title] Title of the custom marker.
	   * @param {number} [id=undefined] Id of the custom marker.
	   * @param {boolean} [editable=false] Make the custom marker editable.
	   */},{key:"setCustomTimeMarker",value:function setCustomTimeMarker(title,id,editable){var _context43;var customTimes=filter$2(_context43=this.customTimes).call(_context43,function(component){return component.options.id===id;});if(customTimes.length===0){throw new Error("No custom time bar found with id ".concat(stringify$2(id)));}if(customTimes.length>0){customTimes[0].setCustomMarker(title,editable);}}/**
	   * Set a custom title for the custom time bar.
	   * @param {string} [title] Custom title
	   * @param {number} [id=undefined]    Id of the custom time bar.
	   * @returns {*}
	   */},{key:"setCustomTimeTitle",value:function setCustomTimeTitle(title,id){var _context44;var customTimes=filter$2(_context44=this.customTimes).call(_context44,function(component){return component.options.id===id;});if(customTimes.length===0){throw new Error("No custom time bar found with id ".concat(stringify$2(id)));}if(customTimes.length>0){return customTimes[0].setCustomTitle(title);}}/**
	   * Retrieve meta information from an event.
	   * Should be overridden by classes extending Core
	   * @param {Event} event
	   * @return {Object} An object with related information.
	   */},{key:"getEventProperties",value:function getEventProperties(event){return {event:event};}/**
	   * Add custom vertical bar
	   * @param {Date | string | number} [time]  A Date, unix timestamp, or
	   *                                         ISO date string. Time point where
	   *                                         the new bar should be placed.
	   *                                         If not provided, `new Date()` will
	   *                                         be used.
	   * @param {number | string} [id=undefined] Id of the new bar. Optional
	   * @return {number | string}               Returns the id of the new bar
	   */},{key:"addCustomTime",value:function addCustomTime(time,id){var _context45;var timestamp=time!==undefined?util$1.convert(time,'Date'):new Date();var exists=some$2(_context45=this.customTimes).call(_context45,function(customTime){return customTime.options.id===id;});if(exists){throw new Error("A custom time with id ".concat(stringify$2(id)," already exists"));}var customTime=new CustomTime(this.body,util$1.extend({},this.options,{time:timestamp,id:id,snap:this.itemSet.options.snap}));this.customTimes.push(customTime);this.components.push(customTime);this._redraw();return id;}/**
	   * Remove previously added custom bar
	   * @param {int} id ID of the custom bar to be removed
	   * [at]returns {boolean} True if the bar exists and is removed, false otherwise
	   */},{key:"removeCustomTime",value:function removeCustomTime(id){var _context46,_this5=this;var customTimes=filter$2(_context46=this.customTimes).call(_context46,function(bar){return bar.options.id===id;});if(customTimes.length===0){throw new Error("No custom time bar found with id ".concat(stringify$2(id)));}forEach$3(customTimes).call(customTimes,function(customTime){var _context47,_context48,_context49,_context50;splice$2(_context47=_this5.customTimes).call(_context47,indexOf$3(_context48=_this5.customTimes).call(_context48,customTime),1);splice$2(_context49=_this5.components).call(_context49,indexOf$3(_context50=_this5.components).call(_context50,customTime),1);customTime.destroy();});}/**
	   * Get the id's of the currently visible items.
	   * @returns {Array} The ids of the visible items
	   */},{key:"getVisibleItems",value:function getVisibleItems(){return this.itemSet&&this.itemSet.getVisibleItems()||[];}/**
	   * Get the id's of the currently visible groups.
	   * @returns {Array} The ids of the visible groups
	   */},{key:"getVisibleGroups",value:function getVisibleGroups(){return this.itemSet&&this.itemSet.getVisibleGroups()||[];}/**
	   * Set Core window such that it fits all items
	   * @param {Object} [options]  Available options:
	   *                                `animation: boolean | {duration: number, easingFunction: string}`
	   *                                    If true (default), the range is animated
	   *                                    smoothly to the new window. An object can be
	   *                                    provided to specify duration and easing function.
	   *                                    Default duration is 500 ms, and default easing
	   *                                    function is 'easeInOutQuad'.
	   * @param {function} [callback] a callback funtion to be executed at the end of this function
	   */},{key:"fit",value:function fit(options,callback){var range=this.getDataRange();// skip range set if there is no min and max date
	if(range.min===null&&range.max===null){return;}// apply a margin of 1% left and right of the data
	var interval=range.max-range.min;var min=new Date(range.min.valueOf()-interval*0.01);var max=new Date(range.max.valueOf()+interval*0.01);var animation=options&&options.animation!==undefined?options.animation:true;this.range.setRange(min,max,{animation:animation},callback);}/**
	   * Calculate the data range of the items start and end dates
	   * [at]returns {{min: [Date], max: [Date]}}
	   * @protected
	   */},{key:"getDataRange",value:function getDataRange(){// must be implemented by Timeline and Graph2d
	throw new Error('Cannot invoke abstract method getDataRange');}/**
	   * Set the visible window. Both parameters are optional, you can change only
	   * start or only end. Syntax:
	   *
	   *     TimeLine.setWindow(start, end)
	   *     TimeLine.setWindow(start, end, options)
	   *     TimeLine.setWindow(range)
	   *
	   * Where start and end can be a Date, number, or string, and range is an
	   * object with properties start and end.
	   *
	   * @param {Date | number | string | Object} [start] Start date of visible window
	   * @param {Date | number | string} [end]            End date of visible window
	   * @param {Object} [options]  Available options:
	   *                                `animation: boolean | {duration: number, easingFunction: string}`
	   *                                    If true (default), the range is animated
	   *                                    smoothly to the new window. An object can be
	   *                                    provided to specify duration and easing function.
	   *                                    Default duration is 500 ms, and default easing
	   *                                    function is 'easeInOutQuad'.
	   * @param {function} [callback] a callback funtion to be executed at the end of this function
	   */},{key:"setWindow",value:function setWindow(start,end,options,callback){if(typeof arguments[2]=="function"){callback=arguments[2];options={};}var animation;var range;if(arguments.length==1){range=arguments[0];animation=range.animation!==undefined?range.animation:true;this.range.setRange(range.start,range.end,{animation:animation});}else if(arguments.length==2&&typeof arguments[1]=="function"){range=arguments[0];callback=arguments[1];animation=range.animation!==undefined?range.animation:true;this.range.setRange(range.start,range.end,{animation:animation},callback);}else {animation=options&&options.animation!==undefined?options.animation:true;this.range.setRange(start,end,{animation:animation},callback);}}/**
	   * Move the window such that given time is centered on screen.
	   * @param {Date | number | string} time
	   * @param {Object} [options]  Available options:
	   *                                `animation: boolean | {duration: number, easingFunction: string}`
	   *                                    If true (default), the range is animated
	   *                                    smoothly to the new window. An object can be
	   *                                    provided to specify duration and easing function.
	   *                                    Default duration is 500 ms, and default easing
	   *                                    function is 'easeInOutQuad'.
	   * @param {function} [callback] a callback funtion to be executed at the end of this function
	   */},{key:"moveTo",value:function moveTo(time,options,callback){if(typeof arguments[1]=="function"){callback=arguments[1];options={};}var interval=this.range.end-this.range.start;var t=util$1.convert(time,'Date').valueOf();var start=t-interval/2;var end=t+interval/2;var animation=options&&options.animation!==undefined?options.animation:true;this.range.setRange(start,end,{animation:animation},callback);}/**
	   * Get the visible window
	   * @return {{start: Date, end: Date}}   Visible range
	   */},{key:"getWindow",value:function getWindow(){var range=this.range.getRange();return {start:new Date(range.start),end:new Date(range.end)};}/**
	   * Zoom in the window such that given time is centered on screen.
	   * @param {number} percentage - must be between [0..1]
	   * @param {Object} [options]  Available options:
	   *                                `animation: boolean | {duration: number, easingFunction: string}`
	   *                                    If true (default), the range is animated
	   *                                    smoothly to the new window. An object can be
	   *                                    provided to specify duration and easing function.
	   *                                    Default duration is 500 ms, and default easing
	   *                                    function is 'easeInOutQuad'.
	   * @param {function} [callback] a callback funtion to be executed at the end of this function
	   */},{key:"zoomIn",value:function zoomIn(percentage,options,callback){if(!percentage||percentage<0||percentage>1)return;if(typeof arguments[1]=="function"){callback=arguments[1];options={};}var range=this.getWindow();var start=range.start.valueOf();var end=range.end.valueOf();var interval=end-start;var newInterval=interval/(1+percentage);var distance=(interval-newInterval)/2;var newStart=start+distance;var newEnd=end-distance;this.setWindow(newStart,newEnd,options,callback);}/**
	   * Zoom out the window such that given time is centered on screen.
	   * @param {number} percentage - must be between [0..1]
	   * @param {Object} [options]  Available options:
	   *                                `animation: boolean | {duration: number, easingFunction: string}`
	   *                                    If true (default), the range is animated
	   *                                    smoothly to the new window. An object can be
	   *                                    provided to specify duration and easing function.
	   *                                    Default duration is 500 ms, and default easing
	   *                                    function is 'easeInOutQuad'.
	   * @param {function} [callback] a callback funtion to be executed at the end of this function
	   */},{key:"zoomOut",value:function zoomOut(percentage,options,callback){if(!percentage||percentage<0||percentage>1)return;if(typeof arguments[1]=="function"){callback=arguments[1];options={};}var range=this.getWindow();var start=range.start.valueOf();var end=range.end.valueOf();var interval=end-start;var newStart=start-interval*percentage/2;var newEnd=end+interval*percentage/2;this.setWindow(newStart,newEnd,options,callback);}/**
	   * Force a redraw. Can be overridden by implementations of Core
	   *
	   * Note: this function will be overridden on construction with a trottled version
	   */},{key:"redraw",value:function redraw(){this._redraw();}/**
	   * Redraw for internal use. Redraws all components. See also the public
	   * method redraw.
	   * @protected
	   */},{key:"_redraw",value:function _redraw(){var _context51;this.redrawCount++;var dom=this.dom;if(!dom||!dom.container||dom.root.offsetWidth==0)return;// when destroyed, or invisible
	var resized=false;var options=this.options;var props=this.props;updateHiddenDates(this.options.moment,this.body,this.options.hiddenDates);// update class names
	if(options.orientation=='top'){util$1.addClassName(dom.root,'vis-top');util$1.removeClassName(dom.root,'vis-bottom');}else {util$1.removeClassName(dom.root,'vis-top');util$1.addClassName(dom.root,'vis-bottom');}if(options.rtl){util$1.addClassName(dom.root,'vis-rtl');util$1.removeClassName(dom.root,'vis-ltr');}else {util$1.addClassName(dom.root,'vis-ltr');util$1.removeClassName(dom.root,'vis-rtl');}// update root width and height options
	dom.root.style.maxHeight=util$1.option.asSize(options.maxHeight,'');dom.root.style.minHeight=util$1.option.asSize(options.minHeight,'');dom.root.style.width=util$1.option.asSize(options.width,'');var rootOffsetWidth=dom.root.offsetWidth;// calculate border widths
	props.border.left=1;props.border.right=1;props.border.top=1;props.border.bottom=1;// calculate the heights. If any of the side panels is empty, we set the height to
	// minus the border width, such that the border will be invisible
	props.center.height=dom.center.offsetHeight;props.left.height=dom.left.offsetHeight;props.right.height=dom.right.offsetHeight;props.top.height=dom.top.clientHeight||-props.border.top;props.bottom.height=dom.bottom.clientHeight||-props.border.bottom;// TODO: compensate borders when any of the panels is empty.
	// apply auto height
	// TODO: only calculate autoHeight when needed (else we cause an extra reflow/repaint of the DOM)
	var contentHeight=Math.max(props.left.height,props.center.height,props.right.height);var autoHeight=props.top.height+contentHeight+props.bottom.height+props.border.top+props.border.bottom;dom.root.style.height=util$1.option.asSize(options.height,"".concat(autoHeight,"px"));// calculate heights of the content panels
	props.root.height=dom.root.offsetHeight;props.background.height=props.root.height;var containerHeight=props.root.height-props.top.height-props.bottom.height;props.centerContainer.height=containerHeight;props.leftContainer.height=containerHeight;props.rightContainer.height=props.leftContainer.height;// calculate the widths of the panels
	props.root.width=rootOffsetWidth;props.background.width=props.root.width;if(!this.initialDrawDone){props.scrollbarWidth=util$1.getScrollBarWidth();}var leftContainerClientWidth=dom.leftContainer.clientWidth;var rightContainerClientWidth=dom.rightContainer.clientWidth;if(options.verticalScroll){if(options.rtl){props.left.width=leftContainerClientWidth||-props.border.left;props.right.width=rightContainerClientWidth+props.scrollbarWidth||-props.border.right;}else {props.left.width=leftContainerClientWidth+props.scrollbarWidth||-props.border.left;props.right.width=rightContainerClientWidth||-props.border.right;}}else {props.left.width=leftContainerClientWidth||-props.border.left;props.right.width=rightContainerClientWidth||-props.border.right;}this._setDOM();// update the scrollTop, feasible range for the offset can be changed
	// when the height of the Core or of the contents of the center changed
	var offset=this._updateScrollTop();// reposition the scrollable contents
	if(options.orientation.item!='top'){offset+=Math.max(props.centerContainer.height-props.center.height-props.border.top-props.border.bottom,0);}dom.center.style.transform="translateY(".concat(offset,"px)");// show shadows when vertical scrolling is available
	var visibilityTop=props.scrollTop==0?'hidden':'';var visibilityBottom=props.scrollTop==props.scrollTopMin?'hidden':'';dom.shadowTop.style.visibility=visibilityTop;dom.shadowBottom.style.visibility=visibilityBottom;dom.shadowTopLeft.style.visibility=visibilityTop;dom.shadowBottomLeft.style.visibility=visibilityBottom;dom.shadowTopRight.style.visibility=visibilityTop;dom.shadowBottomRight.style.visibility=visibilityBottom;if(options.verticalScroll){dom.rightContainer.className='vis-panel vis-right vis-vertical-scroll';dom.leftContainer.className='vis-panel vis-left vis-vertical-scroll';dom.shadowTopRight.style.visibility="hidden";dom.shadowBottomRight.style.visibility="hidden";dom.shadowTopLeft.style.visibility="hidden";dom.shadowBottomLeft.style.visibility="hidden";dom.left.style.top='0px';dom.right.style.top='0px';}if(!options.verticalScroll||props.center.height<props.centerContainer.height){dom.left.style.top="".concat(offset,"px");dom.right.style.top="".concat(offset,"px");dom.rightContainer.className=dom.rightContainer.className.replace(new RegExp('(?:^|\\s)'+'vis-vertical-scroll'+'(?:\\s|$)'),' ');dom.leftContainer.className=dom.leftContainer.className.replace(new RegExp('(?:^|\\s)'+'vis-vertical-scroll'+'(?:\\s|$)'),' ');props.left.width=leftContainerClientWidth||-props.border.left;props.right.width=rightContainerClientWidth||-props.border.right;this._setDOM();}// enable/disable vertical panning
	var contentsOverflow=props.center.height>props.centerContainer.height;this.hammer.get('pan').set({direction:contentsOverflow?Hammer$1.DIRECTION_ALL:Hammer$1.DIRECTION_HORIZONTAL});// set the long press time
	this.hammer.get('press').set({time:this.options.longSelectPressTime});// redraw all components
	forEach$3(_context51=this.components).call(_context51,function(component){resized=component.redraw()||resized;});var MAX_REDRAW=5;if(resized){if(this.redrawCount<MAX_REDRAW){this.body.emitter.emit('_change');return;}else {console.log('WARNING: infinite loop in redraw?');}}else {this.redrawCount=0;}//Emit public 'changed' event for UI updates, see issue #1592
	this.body.emitter.emit("changed");}/**
	   * sets the basic DOM components needed for the timeline\graph2d
	   */},{key:"_setDOM",value:function _setDOM(){var props=this.props;var dom=this.dom;props.leftContainer.width=props.left.width;props.rightContainer.width=props.right.width;var centerWidth=props.root.width-props.left.width-props.right.width;props.center.width=centerWidth;props.centerContainer.width=centerWidth;props.top.width=centerWidth;props.bottom.width=centerWidth;// resize the panels
	dom.background.style.height="".concat(props.background.height,"px");dom.backgroundVertical.style.height="".concat(props.background.height,"px");dom.backgroundHorizontal.style.height="".concat(props.centerContainer.height,"px");dom.centerContainer.style.height="".concat(props.centerContainer.height,"px");dom.leftContainer.style.height="".concat(props.leftContainer.height,"px");dom.rightContainer.style.height="".concat(props.rightContainer.height,"px");dom.background.style.width="".concat(props.background.width,"px");dom.backgroundVertical.style.width="".concat(props.centerContainer.width,"px");dom.backgroundHorizontal.style.width="".concat(props.background.width,"px");dom.centerContainer.style.width="".concat(props.center.width,"px");dom.top.style.width="".concat(props.top.width,"px");dom.bottom.style.width="".concat(props.bottom.width,"px");// reposition the panels
	dom.background.style.left='0';dom.background.style.top='0';dom.backgroundVertical.style.left="".concat(props.left.width+props.border.left,"px");dom.backgroundVertical.style.top='0';dom.backgroundHorizontal.style.left='0';dom.backgroundHorizontal.style.top="".concat(props.top.height,"px");dom.centerContainer.style.left="".concat(props.left.width,"px");dom.centerContainer.style.top="".concat(props.top.height,"px");dom.leftContainer.style.left='0';dom.leftContainer.style.top="".concat(props.top.height,"px");dom.rightContainer.style.left="".concat(props.left.width+props.center.width,"px");dom.rightContainer.style.top="".concat(props.top.height,"px");dom.top.style.left="".concat(props.left.width,"px");dom.top.style.top='0';dom.bottom.style.left="".concat(props.left.width,"px");dom.bottom.style.top="".concat(props.top.height+props.centerContainer.height,"px");dom.center.style.left='0';dom.left.style.left='0';dom.right.style.left='0';}/**
	   * Set a current time. This can be used for example to ensure that a client's
	   * time is synchronized with a shared server time.
	   * Only applicable when option `showCurrentTime` is true.
	   * @param {Date | string | number} time     A Date, unix timestamp, or
	   *                                          ISO date string.
	   */},{key:"setCurrentTime",value:function setCurrentTime(time){if(!this.currentTime){throw new Error('Option showCurrentTime must be true');}this.currentTime.setCurrentTime(time);}/**
	   * Get the current time.
	   * Only applicable when option `showCurrentTime` is true.
	   * @return {Date} Returns the current time.
	   */},{key:"getCurrentTime",value:function getCurrentTime(){if(!this.currentTime){throw new Error('Option showCurrentTime must be true');}return this.currentTime.getCurrentTime();}/**
	   * Convert a position on screen (pixels) to a datetime
	   * @param {int}     x    Position on the screen in pixels
	   * @return {Date}   time The datetime the corresponds with given position x
	   * @protected
	   * TODO: move this function to Range
	   */},{key:"_toTime",value:function _toTime(x){return toTime(this,x,this.props.center.width);}/**
	   * Convert a position on the global screen (pixels) to a datetime
	   * @param {int}     x    Position on the screen in pixels
	   * @return {Date}   time The datetime the corresponds with given position x
	   * @protected
	   * TODO: move this function to Range
	   */},{key:"_toGlobalTime",value:function _toGlobalTime(x){return toTime(this,x,this.props.root.width);//var conversion = this.range.conversion(this.props.root.width);
	//return new Date(x / conversion.scale + conversion.offset);
	}/**
	   * Convert a datetime (Date object) into a position on the screen
	   * @param {Date}   time A date
	   * @return {int}   x    The position on the screen in pixels which corresponds
	   *                      with the given date.
	   * @protected
	   * TODO: move this function to Range
	   */},{key:"_toScreen",value:function _toScreen(time){return toScreen(this,time,this.props.center.width);}/**
	   * Convert a datetime (Date object) into a position on the root
	   * This is used to get the pixel density estimate for the screen, not the center panel
	   * @param {Date}   time A date
	   * @return {int}   x    The position on root in pixels which corresponds
	   *                      with the given date.
	   * @protected
	   * TODO: move this function to Range
	   */},{key:"_toGlobalScreen",value:function _toGlobalScreen(time){return toScreen(this,time,this.props.root.width);//var conversion = this.range.conversion(this.props.root.width);
	//return (time.valueOf() - conversion.offset) * conversion.scale;
	}/**
	   * Initialize watching when option autoResize is true
	   * @private
	   */},{key:"_initAutoResize",value:function _initAutoResize(){if(this.options.autoResize==true){this._startAutoResize();}else {this._stopAutoResize();}}/**
	   * Watch for changes in the size of the container. On resize, the Panel will
	   * automatically redraw itself.
	   * @private
	   */},{key:"_startAutoResize",value:function _startAutoResize(){var me=this;this._stopAutoResize();this._onResize=function(){if(me.options.autoResize!=true){// stop watching when the option autoResize is changed to false
	me._stopAutoResize();return;}if(me.dom.root){var rootOffsetHeight=me.dom.root.offsetHeight;var rootOffsetWidth=me.dom.root.offsetWidth;// check whether the frame is resized
	// Note: we compare offsetWidth here, not clientWidth. For some reason,
	// IE does not restore the clientWidth from 0 to the actual width after
	// changing the timeline's container display style from none to visible
	if(rootOffsetWidth!=me.props.lastWidth||rootOffsetHeight!=me.props.lastHeight){me.props.lastWidth=rootOffsetWidth;me.props.lastHeight=rootOffsetHeight;me.props.scrollbarWidth=util$1.getScrollBarWidth();me.body.emitter.emit('_change');}}};// add event listener to window resize
	util$1.addEventListener(window,'resize',this._onResize);//Prevent initial unnecessary redraw
	if(me.dom.root){me.props.lastWidth=me.dom.root.offsetWidth;me.props.lastHeight=me.dom.root.offsetHeight;}this.watchTimer=setInterval$1(this._onResize,1000);}/**
	   * Stop watching for a resize of the frame.
	   * @private
	   */},{key:"_stopAutoResize",value:function _stopAutoResize(){if(this.watchTimer){clearInterval(this.watchTimer);this.watchTimer=undefined;}// remove event listener on window.resize
	if(this._onResize){util$1.removeEventListener(window,'resize',this._onResize);this._onResize=null;}}/**
	   * Start moving the timeline vertically
	   * @param {Event} event
	   * @private
	   */},{key:"_onTouch",value:function _onTouch(event){// eslint-disable-line no-unused-vars
	this.touch.allowDragging=true;this.touch.initialScrollTop=this.props.scrollTop;}/**
	   * Start moving the timeline vertically
	   * @param {Event} event
	   * @private
	   */},{key:"_onPinch",value:function _onPinch(event){// eslint-disable-line no-unused-vars
	this.touch.allowDragging=false;}/**
	   * Move the timeline vertically
	   * @param {Event} event
	   * @private
	   */},{key:"_onDrag",value:function _onDrag(event){if(!event)return;// refuse to drag when we where pinching to prevent the timeline make a jump
	// when releasing the fingers in opposite order from the touch screen
	if(!this.touch.allowDragging)return;var delta=event.deltaY;var oldScrollTop=this._getScrollTop();var newScrollTop=this._setScrollTop(this.touch.initialScrollTop+delta);if(this.options.verticalScroll){this.dom.left.parentNode.scrollTop=-this.props.scrollTop;this.dom.right.parentNode.scrollTop=-this.props.scrollTop;}if(newScrollTop!=oldScrollTop){this.emit("verticalDrag");}}/**
	   * Apply a scrollTop
	   * @param {number} scrollTop
	   * @returns {number} scrollTop  Returns the applied scrollTop
	   * @private
	   */},{key:"_setScrollTop",value:function _setScrollTop(scrollTop){this.props.scrollTop=scrollTop;this._updateScrollTop();return this.props.scrollTop;}/**
	   * Update the current scrollTop when the height of  the containers has been changed
	   * @returns {number} scrollTop  Returns the applied scrollTop
	   * @private
	   */},{key:"_updateScrollTop",value:function _updateScrollTop(){// recalculate the scrollTopMin
	var scrollTopMin=Math.min(this.props.centerContainer.height-this.props.center.height,0);// is negative or zero
	if(scrollTopMin!=this.props.scrollTopMin){// in case of bottom orientation, change the scrollTop such that the contents
	// do not move relative to the time axis at the bottom
	if(this.options.orientation.item!='top'){this.props.scrollTop+=scrollTopMin-this.props.scrollTopMin;}this.props.scrollTopMin=scrollTopMin;}// limit the scrollTop to the feasible scroll range
	if(this.props.scrollTop>0)this.props.scrollTop=0;if(this.props.scrollTop<scrollTopMin)this.props.scrollTop=scrollTopMin;if(this.options.verticalScroll){this.dom.left.parentNode.scrollTop=-this.props.scrollTop;this.dom.right.parentNode.scrollTop=-this.props.scrollTop;}this._isProgramaticallyScrolled=true;return this.props.scrollTop;}/**
	   * Get the current scrollTop
	   * @returns {number} scrollTop
	   * @private
	   */},{key:"_getScrollTop",value:function _getScrollTop(){return this.props.scrollTop;}/**
	   * Load a configurator
	   * [at]returns {Object}
	   * @private
	   */},{key:"_createConfigurator",value:function _createConfigurator(){throw new Error('Cannot invoke abstract method _createConfigurator');}}]);return Core;}();// turn Core into an event emitter
	componentEmitter(Core.prototype);/**
	 * A current time bar
	 */var CurrentTime=/*#__PURE__*/function(_Component4){inherits(CurrentTime,_Component4);var _super4=_createSuper$2(CurrentTime);/**
	 * @param {{range: Range, dom: Object, domProps: Object}} body
	 * @param {Object} [options]        Available parameters:
	 *                                  {Boolean} [showCurrentTime]
	 *                                  {String}  [alignCurrentTime]
	 * @constructor CurrentTime
	 * @extends Component
	 */function CurrentTime(body,options){var _context52;var _this6;classCallCheck(this,CurrentTime);_this6=_super4.call(this);_this6.body=body;// default options
	_this6.defaultOptions={rtl:false,showCurrentTime:true,alignCurrentTime:undefined,moment:moment$1,locales:locales$1,locale:'en'};_this6.options=util$1.extend({},_this6.defaultOptions);_this6.setOptions(options);_this6.options.locales=util$1.extend({},locales$1,_this6.options.locales);var defaultLocales=_this6.defaultOptions.locales[_this6.defaultOptions.locale];forEach$3(_context52=keys$6(_this6.options.locales)).call(_context52,function(locale){_this6.options.locales[locale]=util$1.extend({},defaultLocales,_this6.options.locales[locale]);});_this6.offset=0;_this6._create();return _this6;}/**
	   * Create the HTML DOM for the current time bar
	   * @private
	   */createClass(CurrentTime,[{key:"_create",value:function _create(){var bar=document.createElement('div');bar.className='vis-current-time';bar.style.position='absolute';bar.style.top='0px';bar.style.height='100%';this.bar=bar;}/**
	   * Destroy the CurrentTime bar
	   */},{key:"destroy",value:function destroy(){this.options.showCurrentTime=false;this.redraw();// will remove the bar from the DOM and stop refreshing
	this.body=null;}/**
	   * Set options for the component. Options will be merged in current options.
	   * @param {Object} options  Available parameters:
	   *                          {boolean} [showCurrentTime]
	   *                          {String}  [alignCurrentTime]
	   */},{key:"setOptions",value:function setOptions(options){if(options){// copy all options that we know
	util$1.selectiveExtend(['rtl','showCurrentTime','alignCurrentTime','moment','locale','locales'],this.options,options);}}/**
	   * Repaint the component
	   * @return {boolean} Returns true if the component is resized
	   */},{key:"redraw",value:function redraw(){if(this.options.showCurrentTime){var _context53,_context54;var parent=this.body.dom.backgroundVertical;if(this.bar.parentNode!=parent){// attach to the dom
	if(this.bar.parentNode){this.bar.parentNode.removeChild(this.bar);}parent.appendChild(this.bar);this.start();}var now=this.options.moment(now$2()+this.offset);if(this.options.alignCurrentTime){now=now.startOf(this.options.alignCurrentTime);}var x=this.body.util.toScreen(now);var locale=this.options.locales[this.options.locale];if(!locale){if(!this.warned){console.warn("WARNING: options.locales['".concat(this.options.locale,"'] not found. See https://visjs.github.io/vis-timeline/docs/timeline/#Localization"));this.warned=true;}locale=this.options.locales['en'];// fall back on english when not available
	}var title=concat$2(_context53=concat$2(_context54="".concat(locale.current," ")).call(_context54,locale.time,": ")).call(_context53,now.format('dddd, MMMM Do YYYY, H:mm:ss'));title=title.charAt(0).toUpperCase()+title.substring(1);if(this.options.rtl){this.bar.style.transform="translateX(".concat(x*-1,"px)");}else {this.bar.style.transform="translateX(".concat(x,"px)");}this.bar.title=title;}else {// remove the line from the DOM
	if(this.bar.parentNode){this.bar.parentNode.removeChild(this.bar);}this.stop();}return false;}/**
	   * Start auto refreshing the current time bar
	   */},{key:"start",value:function start(){var me=this;/**
	       *  Updates the current time.
	       */function update(){me.stop();// determine interval to refresh
	var scale=me.body.range.conversion(me.body.domProps.center.width).scale;var interval=1/scale/10;if(interval<30)interval=30;if(interval>1000)interval=1000;me.redraw();me.body.emitter.emit('currentTimeTick');// start a renderTimer to adjust for the new time
	me.currentTimeTimer=setTimeout$2(update,interval);}update();}/**
	   * Stop auto refreshing the current time bar
	   */},{key:"stop",value:function stop(){if(this.currentTimeTimer!==undefined){clearTimeout(this.currentTimeTimer);delete this.currentTimeTimer;}}/**
	   * Set a current time. This can be used for example to ensure that a client's
	   * time is synchronized with a shared server time.
	   * @param {Date | string | number} time     A Date, unix timestamp, or
	   *                                          ISO date string.
	   */},{key:"setCurrentTime",value:function setCurrentTime(time){var t=util$1.convert(time,'Date').valueOf();var now=now$2();this.offset=t-now;this.redraw();}/**
	   * Get the current time.
	   * @return {Date} Returns the current time.
	   */},{key:"getCurrentTime",value:function getCurrentTime(){return new Date(now$2()+this.offset);}}]);return CurrentTime;}(Component);// Utility functions for ordering and stacking of items
	var EPSILON=0.001;// used when checking collisions, to prevent round-off errors
	/**
	 * Order items by their start data
	 * @param {Item[]} items
	 */function orderByStart(items){sort$2(items).call(items,function(a,b){return a.data.start-b.data.start;});}/**
	 * Order items by their end date. If they have no end date, their start date
	 * is used.
	 * @param {Item[]} items
	 */function orderByEnd(items){sort$2(items).call(items,function(a,b){var aTime='end'in a.data?a.data.end:a.data.start;var bTime='end'in b.data?b.data.end:b.data.start;return aTime-bTime;});}/**
	 * Adjust vertical positions of the items such that they don't overlap each
	 * other.
	 * @param {Item[]} items
	 *            All visible items
	 * @param {{item: {horizontal: number, vertical: number}, axis: number}} margin
	 *            Margins between items and between items and the axis.
	 * @param {boolean} [force=false]
	 *            If true, all items will be repositioned. If false (default), only
	 *            items having a top===null will be re-stacked
	 * @param {function} shouldBailItemsRedrawFunction
	 *            bailing function
	 * @return {boolean} shouldBail
	 */function stack(items,margin,force,shouldBailItemsRedrawFunction){if(force){// reset top position of all items
	for(var i=0;i<items.length;i++){items[i].top=null;}}// calculate new, non-overlapping positions
	for(var i=0;i<items.length;i++){// eslint-disable-line no-redeclare
	var item=items[i];if(item.stack&&item.top===null){// initialize top position
	item.top=margin.axis;var shouldBail=false;do{// TODO: optimize checking for overlap. when there is a gap without items,
	//       you only need to check for items from the next item on, not from zero
	var collidingItem=null;for(var j=0,jj=items.length;j<jj;j++){var other=items[j];shouldBail=shouldBailItemsRedrawFunction()||false;if(shouldBail){return true;}if(other.top!==null&&other!==item&&other.stack&&collision(item,other,margin.item,other.options.rtl)){collidingItem=other;break;}}if(collidingItem!=null){// There is a collision. Reposition the items above the colliding element
	item.top=collidingItem.top+collidingItem.height+margin.item.vertical;}}while(collidingItem);}}return shouldBail;}/**
	 * Adjust vertical positions of the items within a single subgroup such that they
	 * don't overlap each other.
	 * @param {Item[]} items
	 *            All items withina subgroup
	 * @param {{item: {horizontal: number, vertical: number}, axis: number}} margin
	 *            Margins between items and between items and the axis.
	 * @param {subgroup} subgroup
	 *            The subgroup that is being stacked
	 */function substack(items,margin,subgroup){for(var i=0;i<items.length;i++){items[i].top=null;}// Set the initial height
	var subgroupHeight=subgroup.height;// calculate new, non-overlapping positions
	for(i=0;i<items.length;i++){var item=items[i];if(item.stack&&item.top===null){// initialize top position
	item.top=item.baseTop;//margin.axis + item.baseTop;
	do{// TODO: optimize checking for overlap. when there is a gap without items,
	//       you only need to check for items from the next item on, not from zero
	var collidingItem=null;for(var j=0,jj=items.length;j<jj;j++){var other=items[j];if(other.top!==null&&other!==item/*&& other.stack*/&&collision(item,other,margin.item,other.options.rtl)){collidingItem=other;break;}}if(collidingItem!=null){// There is a collision. Reposition the items above the colliding element
	item.top=collidingItem.top+collidingItem.height+margin.item.vertical;// + item.baseTop;
	}if(item.top+item.height>subgroupHeight){subgroupHeight=item.top+item.height;}}while(collidingItem);}}// Set the new height
	subgroup.height=subgroupHeight-subgroup.top+0.5*margin.item.vertical;}/**
	 * Adjust vertical positions of the items without stacking them
	 * @param {Item[]} items
	 *            All visible items
	 * @param {{item: {horizontal: number, vertical: number}, axis: number}} margin
	 *            Margins between items and between items and the axis.
	 * @param {subgroups[]} subgroups
	 *            All subgroups
	 * @param {boolean} isStackSubgroups
	 */function nostack(items,margin,subgroups,isStackSubgroups){for(var i=0;i<items.length;i++){if(items[i].data.subgroup==undefined){items[i].top=margin.item.vertical;}else if(items[i].data.subgroup!==undefined&&isStackSubgroups){var newTop=0;for(var subgroup in subgroups){if(subgroups.hasOwnProperty(subgroup)){if(subgroups[subgroup].visible==true&&subgroups[subgroup].index<subgroups[items[i].data.subgroup].index){newTop+=subgroups[subgroup].height;subgroups[items[i].data.subgroup].top=newTop;}}}items[i].top=newTop+0.5*margin.item.vertical;}}if(!isStackSubgroups){stackSubgroups(items,margin,subgroups);}}/**
	 * Adjust vertical positions of the subgroups such that they don't overlap each
	 * other.
	 * @param {Array.<timeline.Item>} items
	 * @param {{item: {horizontal: number, vertical: number}, axis: number}} margin Margins between items and between items and the axis.
	 * @param {subgroups[]} subgroups
	 *            All subgroups
	 */function stackSubgroups(items,margin,subgroups){for(var subgroup in subgroups){if(subgroups.hasOwnProperty(subgroup)){subgroups[subgroup].top=0;do{// TODO: optimize checking for overlap. when there is a gap without items,
	//       you only need to check for items from the next item on, not from zero
	var collidingItem=null;for(var otherSubgroup in subgroups){if(subgroups[otherSubgroup].top!==null&&otherSubgroup!==subgroup&&subgroups[subgroup].index>subgroups[otherSubgroup].index&&collisionByTimes(subgroups[subgroup],subgroups[otherSubgroup])){collidingItem=subgroups[otherSubgroup];break;}}if(collidingItem!=null){// There is a collision. Reposition the subgroups above the colliding element
	subgroups[subgroup].top=collidingItem.top+collidingItem.height;}}while(collidingItem);}}for(var i=0;i<items.length;i++){if(items[i].data.subgroup!==undefined){items[i].top=subgroups[items[i].data.subgroup].top+0.5*margin.item.vertical;}}}/**
	 * Adjust vertical positions of the subgroups such that they don't overlap each
	 * other, then stacks the contents of each subgroup individually.
	 * @param {Item[]} subgroupItems
	 *            All the items in a subgroup
	 * @param {{item: {horizontal: number, vertical: number}, axis: number}} margin
	 *            Margins between items and between items and the axis.
	 * @param {subgroups[]} subgroups
	 *            All subgroups
	 */function stackSubgroupsWithInnerStack(subgroupItems,margin,subgroups){var doSubStack=false;// Run subgroups in their order (if any)
	var subgroupOrder=[];for(var subgroup in subgroups){if(subgroups[subgroup].hasOwnProperty("index")){subgroupOrder[subgroups[subgroup].index]=subgroup;}else {subgroupOrder.push(subgroup);}}for(var j=0;j<subgroupOrder.length;j++){subgroup=subgroupOrder[j];if(subgroups.hasOwnProperty(subgroup)){doSubStack=doSubStack||subgroups[subgroup].stack;subgroups[subgroup].top=0;for(var otherSubgroup in subgroups){if(subgroups[otherSubgroup].visible&&subgroups[subgroup].index>subgroups[otherSubgroup].index){subgroups[subgroup].top+=subgroups[otherSubgroup].height;}}var items=subgroupItems[subgroup];for(var i=0;i<items.length;i++){if(items[i].data.subgroup!==undefined){items[i].top=subgroups[items[i].data.subgroup].top+0.5*margin.item.vertical;if(subgroups[subgroup].stack){items[i].baseTop=items[i].top;}}}if(doSubStack&&subgroups[subgroup].stack){substack(subgroupItems[subgroup],margin,subgroups[subgroup]);}}}}/**
	 * Test if the two provided items collide
	 * The items must have parameters left, width, top, and height.
	 * @param {Item} a          The first item
	 * @param {Item} b          The second item
	 * @param {{horizontal: number, vertical: number}} margin
	 *                          An object containing a horizontal and vertical
	 *                          minimum required margin.
	 * @param {boolean} rtl
	 * @return {boolean}        true if a and b collide, else false
	 */function collision(a,b,margin,rtl){if(rtl){return a.right-margin.horizontal+EPSILON<b.right+b.width&&a.right+a.width+margin.horizontal-EPSILON>b.right&&a.top-margin.vertical+EPSILON<b.top+b.height&&a.top+a.height+margin.vertical-EPSILON>b.top;}else {return a.left-margin.horizontal+EPSILON<b.left+b.width&&a.left+a.width+margin.horizontal-EPSILON>b.left&&a.top-margin.vertical+EPSILON<b.top+b.height&&a.top+a.height+margin.vertical-EPSILON>b.top;}}/**
	 * Test if the two provided objects collide
	 * The objects must have parameters start, end, top, and height.
	 * @param {Object} a          The first Object
	 * @param {Object} b          The second Object
	 * @return {boolean}        true if a and b collide, else false
	 */function collisionByTimes(a,b){// Check for overlap by time and height. Abutting is OK and
	// not considered a collision while overlap is considered a collision.
	var timeOverlap=a.start<b.end&&a.end>b.start;var heightOverlap=a.top<b.top+b.height&&a.top+a.height>b.top;return timeOverlap&&heightOverlap;}var stack$1=/*#__PURE__*/freeze$2({__proto__:null,orderByStart:orderByStart,orderByEnd:orderByEnd,stack:stack,substack:substack,nostack:nostack,stackSubgroups:stackSubgroups,stackSubgroupsWithInnerStack:stackSubgroupsWithInnerStack,collision:collision,collisionByTimes:collisionByTimes});var UNGROUPED='__ungrouped__';// reserved group id for ungrouped items
	var BACKGROUND='__background__';// reserved group id for background items without group
	var ReservedGroupIds={UNGROUPED:UNGROUPED,BACKGROUND:BACKGROUND};/**
	 * @constructor Group
	 */var Group=/*#__PURE__*/function(){/**
	 * @param {number | string} groupId
	 * @param {Object} data
	 * @param {ItemSet} itemSet
	 * @constructor Group
	 */function Group(groupId,data,itemSet){var _this7=this;classCallCheck(this,Group);this.groupId=groupId;this.subgroups={};this.subgroupStack={};this.subgroupStackAll=false;this.subgroupVisibility={};this.doInnerStack=false;this.shouldBailStackItems=false;this.subgroupIndex=0;this.subgroupOrderer=data&&data.subgroupOrder;this.itemSet=itemSet;this.isVisible=null;this.stackDirty=true;// if true, items will be restacked on next redraw
	// This is a stack of functions (`() => void`) that will be executed before
	// the instance is disposed off (method `dispose`). Anything that needs to
	// be manually disposed off before garbage collection happens (or so that
	// garbage collection can happen) should be added to this stack.
	this._disposeCallbacks=[];if(data&&data.nestedGroups){this.nestedGroups=data.nestedGroups;if(data.showNested==false){this.showNested=false;}else {this.showNested=true;}}if(data&&data.subgroupStack){if(typeof data.subgroupStack==="boolean"){this.doInnerStack=data.subgroupStack;this.subgroupStackAll=data.subgroupStack;}else {// We might be doing stacking on specific sub groups, but only
	// if at least one is set to do stacking
	for(var key in data.subgroupStack){this.subgroupStack[key]=data.subgroupStack[key];this.doInnerStack=this.doInnerStack||data.subgroupStack[key];}}}if(data&&data.heightMode){this.heightMode=data.heightMode;}else {this.heightMode=itemSet.options.groupHeightMode;}this.nestedInGroup=null;this.dom={};this.props={label:{width:0,height:0}};this.className=null;this.items={};// items filtered by groupId of this group
	this.visibleItems=[];// items currently visible in window
	this.itemsInRange=[];// items currently in range
	this.orderedItems={byStart:[],byEnd:[]};this.checkRangedItems=false;// needed to refresh the ranged items if the window is programatically changed with NO overlap.
	var handleCheckRangedItems=function handleCheckRangedItems(){_this7.checkRangedItems=true;};this.itemSet.body.emitter.on("checkRangedItems",handleCheckRangedItems);this._disposeCallbacks.push(function(){_this7.itemSet.body.emitter.off("checkRangedItems",handleCheckRangedItems);});this._create();this.setData(data);}/**
	   * Create DOM elements for the group
	   * @private
	   */createClass(Group,[{key:"_create",value:function _create(){var label=document.createElement('div');if(this.itemSet.options.groupEditable.order){label.className='vis-label draggable';}else {label.className='vis-label';}this.dom.label=label;var inner=document.createElement('div');inner.className='vis-inner';label.appendChild(inner);this.dom.inner=inner;var foreground=document.createElement('div');foreground.className='vis-group';foreground['vis-group']=this;this.dom.foreground=foreground;this.dom.background=document.createElement('div');this.dom.background.className='vis-group';this.dom.axis=document.createElement('div');this.dom.axis.className='vis-group';// create a hidden marker to detect when the Timelines container is attached
	// to the DOM, or the style of a parent of the Timeline is changed from
	// display:none is changed to visible.
	this.dom.marker=document.createElement('div');this.dom.marker.style.visibility='hidden';this.dom.marker.style.position='absolute';this.dom.marker.innerHTML='';this.dom.background.appendChild(this.dom.marker);}/**
	   * Set the group data for this group
	   * @param {Object} data   Group data, can contain properties content and className
	   */},{key:"setData",value:function setData(data){if(this.itemSet.groupTouchParams.isDragging)return;// update contents
	var content;var templateFunction;if(data&&data.subgroupVisibility){for(var key in data.subgroupVisibility){this.subgroupVisibility[key]=data.subgroupVisibility[key];}}if(this.itemSet.options&&this.itemSet.options.groupTemplate){var _context55;templateFunction=bind$2(_context55=this.itemSet.options.groupTemplate).call(_context55,this);content=templateFunction(data,this.dom.inner);}else {content=data&&data.content;}if(content instanceof Element){while(this.dom.inner.firstChild){this.dom.inner.removeChild(this.dom.inner.firstChild);}this.dom.inner.appendChild(content);}else if(content instanceof Object&&content.isReactComponent);else if(content instanceof Object){templateFunction(data,this.dom.inner);}else if(content!==undefined&&content!==null){this.dom.inner.innerHTML=content;}else {this.dom.inner.innerHTML=this.groupId||'';// groupId can be null
	}// update title
	this.dom.label.title=data&&data.title||'';if(!this.dom.inner.firstChild){util$1.addClassName(this.dom.inner,'vis-hidden');}else {util$1.removeClassName(this.dom.inner,'vis-hidden');}if(data&&data.nestedGroups){if(!this.nestedGroups||this.nestedGroups!=data.nestedGroups){this.nestedGroups=data.nestedGroups;}if(data.showNested!==undefined||this.showNested===undefined){if(data.showNested==false){this.showNested=false;}else {this.showNested=true;}}util$1.addClassName(this.dom.label,'vis-nesting-group');if(this.showNested){util$1.removeClassName(this.dom.label,'collapsed');util$1.addClassName(this.dom.label,'expanded');}else {util$1.removeClassName(this.dom.label,'expanded');util$1.addClassName(this.dom.label,'collapsed');}}else if(this.nestedGroups){this.nestedGroups=null;util$1.removeClassName(this.dom.label,'collapsed');util$1.removeClassName(this.dom.label,'expanded');util$1.removeClassName(this.dom.label,'vis-nesting-group');}if(data&&(data.treeLevel||data.nestedInGroup)){util$1.addClassName(this.dom.label,'vis-nested-group');if(data.treeLevel){util$1.addClassName(this.dom.label,'vis-group-level-'+data.treeLevel);}else {// Nesting level is unknown, but we're sure it's at least 1
	util$1.addClassName(this.dom.label,'vis-group-level-unknown-but-gte1');}}else {util$1.addClassName(this.dom.label,'vis-group-level-0');}// update className
	var className=data&&data.className||null;if(className!=this.className){if(this.className){util$1.removeClassName(this.dom.label,this.className);util$1.removeClassName(this.dom.foreground,this.className);util$1.removeClassName(this.dom.background,this.className);util$1.removeClassName(this.dom.axis,this.className);}util$1.addClassName(this.dom.label,className);util$1.addClassName(this.dom.foreground,className);util$1.addClassName(this.dom.background,className);util$1.addClassName(this.dom.axis,className);this.className=className;}// update style
	if(this.style){util$1.removeCssText(this.dom.label,this.style);this.style=null;}if(data&&data.style){util$1.addCssText(this.dom.label,data.style);this.style=data.style;}}/**
	   * Get the width of the group label
	   * @return {number} width
	   */},{key:"getLabelWidth",value:function getLabelWidth(){return this.props.label.width;}/**
	   * check if group has had an initial height hange
	   * @returns {boolean} 
	   */},{key:"_didMarkerHeightChange",value:function _didMarkerHeightChange(){var markerHeight=this.dom.marker.clientHeight;if(markerHeight!=this.lastMarkerHeight){this.lastMarkerHeight=markerHeight;var redrawQueue={};var redrawQueueLength=0;forEach$3(util$1).call(util$1,this.items,function(item,key){item.dirty=true;if(item.displayed){var returnQueue=true;redrawQueue[key]=item.redraw(returnQueue);redrawQueueLength=redrawQueue[key].length;}});var needRedraw=redrawQueueLength>0;if(needRedraw){var _loop=function _loop(i){forEach$3(util$1).call(util$1,redrawQueue,function(fns){fns[i]();});};// redraw all regular items
	for(var i=0;i<redrawQueueLength;i++){_loop(i);}}return true;}else {return false;}}/**
	   * calculate group dimentions and position
	   * @param {number} pixels
	   */},{key:"_calculateGroupSizeAndPosition",value:function _calculateGroupSizeAndPosition(){var _this$dom$foreground=this.dom.foreground,offsetTop=_this$dom$foreground.offsetTop,offsetLeft=_this$dom$foreground.offsetLeft,offsetWidth=_this$dom$foreground.offsetWidth;this.top=offsetTop;this.right=offsetLeft;this.width=offsetWidth;}/**
	   * checks if should bail redraw of items
	   * @returns {boolean} should bail 
	   */},{key:"_shouldBailItemsRedraw",value:function _shouldBailItemsRedraw(){var me=this;var timeoutOptions=this.itemSet.options.onTimeout;var bailOptions={relativeBailingTime:this.itemSet.itemsSettingTime,bailTimeMs:timeoutOptions&&timeoutOptions.timeoutMs,userBailFunction:timeoutOptions&&timeoutOptions.callback,shouldBailStackItems:this.shouldBailStackItems};var bail=null;if(!this.itemSet.initialDrawDone){if(bailOptions.shouldBailStackItems){return true;}if(Math.abs(now$2()-new Date(bailOptions.relativeBailingTime))>bailOptions.bailTimeMs){if(bailOptions.userBailFunction&&this.itemSet.userContinueNotBail==null){bailOptions.userBailFunction(function(didUserContinue){me.itemSet.userContinueNotBail=didUserContinue;bail=!didUserContinue;});}else if(me.itemSet.userContinueNotBail==false){bail=true;}else {bail=false;}}}return bail;}/**
	   * redraws items
	   * @param {boolean} forceRestack
	   * @param {boolean} lastIsVisible
	   * @param {number} margin
	   * @param {object} range
	   * @private
	   */},{key:"_redrawItems",value:function _redrawItems(forceRestack,lastIsVisible,margin,range){var _this8=this;var restack=forceRestack||this.stackDirty||this.isVisible&&!lastIsVisible;// if restacking, reposition visible items vertically
	if(restack){var _context56,_context57,_context58,_context59,_context60,_context61;var orderedItems={byEnd:filter$2(_context56=this.orderedItems.byEnd).call(_context56,function(item){return !item.isCluster;}),byStart:filter$2(_context57=this.orderedItems.byStart).call(_context57,function(item){return !item.isCluster;})};var orderedClusters={byEnd:toConsumableArray(new set$3(filter$2(_context58=map$5(_context59=this.orderedItems.byEnd).call(_context59,function(item){return item.cluster;})).call(_context58,function(item){return !!item;}))),byStart:toConsumableArray(new set$3(filter$2(_context60=map$5(_context61=this.orderedItems.byStart).call(_context61,function(item){return item.cluster;})).call(_context60,function(item){return !!item;})))};/**
	     * Get all visible items in range
	     * @return {array} items
	     */var getVisibleItems=function getVisibleItems(){var _context62,_context63,_context64;var visibleItems=_this8._updateItemsInRange(orderedItems,filter$2(_context62=_this8.visibleItems).call(_context62,function(item){return !item.isCluster;}),range);var visibleClusters=_this8._updateClustersInRange(orderedClusters,filter$2(_context63=_this8.visibleItems).call(_context63,function(item){return item.isCluster;}),range);return concat$2(_context64=[]).call(_context64,toConsumableArray(visibleItems),toConsumableArray(visibleClusters));};/**
	       * Get visible items grouped by subgroup
	       * @param {function} orderFn An optional function to order items inside the subgroups
	       * @return {Object}
	       */var getVisibleItemsGroupedBySubgroup=function getVisibleItemsGroupedBySubgroup(orderFn){var visibleSubgroupsItems={};var _loop2=function _loop2(subgroup){var _context65;var items=filter$2(_context65=_this8.visibleItems).call(_context65,function(item){return item.data.subgroup===subgroup;});visibleSubgroupsItems[subgroup]=orderFn?sort$2(items).call(items,function(a,b){return orderFn(a.data,b.data);}):items;};for(var subgroup in _this8.subgroups){_loop2(subgroup);}return visibleSubgroupsItems;};if(typeof this.itemSet.options.order==='function'){// a custom order function
	//show all items
	var me=this;if(this.doInnerStack&&this.itemSet.options.stackSubgroups){// Order the items within each subgroup
	var visibleSubgroupsItems=getVisibleItemsGroupedBySubgroup(this.itemSet.options.order);stackSubgroupsWithInnerStack(visibleSubgroupsItems,margin,this.subgroups);this.visibleItems=getVisibleItems();this._updateSubGroupHeights(margin);}else {var _context66,_context67,_context68,_context69;this.visibleItems=getVisibleItems();this._updateSubGroupHeights(margin);// order all items and force a restacking
	// order all items outside clusters and force a restacking
	var customOrderedItems=sort$2(_context66=filter$2(_context67=slice$3(_context68=this.visibleItems).call(_context68)).call(_context67,function(item){return item.isCluster||!item.isCluster&&!item.cluster;})).call(_context66,function(a,b){return me.itemSet.options.order(a.data,b.data);});this.shouldBailStackItems=stack(customOrderedItems,margin,true,bind$2(_context69=this._shouldBailItemsRedraw).call(_context69,this));}}else {// no custom order function, lazy stacking
	this.visibleItems=getVisibleItems();this._updateSubGroupHeights(margin);if(this.itemSet.options.stack){if(this.doInnerStack&&this.itemSet.options.stackSubgroups){var _visibleSubgroupsItems=getVisibleItemsGroupedBySubgroup();stackSubgroupsWithInnerStack(_visibleSubgroupsItems,margin,this.subgroups);}else {var _context70;// TODO: ugly way to access options...
	this.shouldBailStackItems=stack(this.visibleItems,margin,true,bind$2(_context70=this._shouldBailItemsRedraw).call(_context70,this));}}else {// no stacking
	nostack(this.visibleItems,margin,this.subgroups,this.itemSet.options.stackSubgroups);}}for(var i=0;i<this.visibleItems.length;i++){this.visibleItems[i].repositionX();if(this.subgroupVisibility[this.visibleItems[i].data.subgroup]!==undefined){if(!this.subgroupVisibility[this.visibleItems[i].data.subgroup]){this.visibleItems[i].hide();}}}if(this.itemSet.options.cluster){forEach$3(util$1).call(util$1,this.items,function(item){if(item.cluster&&item.displayed){item.hide();}});}if(this.shouldBailStackItems){this.itemSet.body.emitter.emit('destroyTimeline');}this.stackDirty=false;}}/**
	   * check if group resized
	   * @param {boolean} resized
	   * @param {number} height
	   * @return {boolean} did resize
	   */},{key:"_didResize",value:function _didResize(resized,height){resized=util$1.updateProperty(this,'height',height)||resized;// recalculate size of label
	var labelWidth=this.dom.inner.clientWidth;var labelHeight=this.dom.inner.clientHeight;resized=util$1.updateProperty(this.props.label,'width',labelWidth)||resized;resized=util$1.updateProperty(this.props.label,'height',labelHeight)||resized;return resized;}/**
	   * apply group height
	   * @param {number} height
	   */},{key:"_applyGroupHeight",value:function _applyGroupHeight(height){this.dom.background.style.height="".concat(height,"px");this.dom.foreground.style.height="".concat(height,"px");this.dom.label.style.height="".concat(height,"px");}/**
	   * update vertical position of items after they are re-stacked and the height of the group is calculated
	   * @param {number} margin
	   */},{key:"_updateItemsVerticalPosition",value:function _updateItemsVerticalPosition(margin){for(var i=0,ii=this.visibleItems.length;i<ii;i++){var _item2=this.visibleItems[i];_item2.repositionY(margin);if(!this.isVisible&&this.groupId!=ReservedGroupIds.BACKGROUND){if(_item2.displayed)_item2.hide();}}}/**
	   * Repaint this group
	   * @param {{start: number, end: number}} range
	   * @param {{item: {horizontal: number, vertical: number}, axis: number}} margin
	   * @param {boolean} [forceRestack=false]  Force restacking of all items
	   * @param {boolean} [returnQueue=false]  return the queue or if the group resized
	   * @return {boolean} Returns true if the group is resized or the redraw queue if returnQueue=true
	   */},{key:"redraw",value:function redraw(range,margin,forceRestack,returnQueue){var _this9=this,_context71,_context72,_context75,_context77,_context81;var resized=false;var lastIsVisible=this.isVisible;var height;var queue=[function(){forceRestack=_this9._didMarkerHeightChange.call(_this9)||forceRestack;},// recalculate the height of the subgroups
	bind$2(_context71=this._updateSubGroupHeights).call(_context71,this,margin),// calculate actual size and position
	bind$2(_context72=this._calculateGroupSizeAndPosition).call(_context72,this),function(){var _context73;_this9.isVisible=bind$2(_context73=_this9._isGroupVisible).call(_context73,_this9)(range,margin);},function(){var _context74;bind$2(_context74=_this9._redrawItems).call(_context74,_this9)(forceRestack,lastIsVisible,margin,range);},// update subgroups
	bind$2(_context75=this._updateSubgroupsSizes).call(_context75,this),function(){var _context76;height=bind$2(_context76=_this9._calculateHeight).call(_context76,_this9)(margin);},// calculate actual size and position again
	bind$2(_context77=this._calculateGroupSizeAndPosition).call(_context77,this),function(){var _context78;resized=bind$2(_context78=_this9._didResize).call(_context78,_this9)(resized,height);},function(){var _context79;bind$2(_context79=_this9._applyGroupHeight).call(_context79,_this9)(height);},function(){var _context80;bind$2(_context80=_this9._updateItemsVerticalPosition).call(_context80,_this9)(margin);},bind$2(_context81=function _context81(){if(!_this9.isVisible&&_this9.height){resized=false;}return resized;}).call(_context81,this)];if(returnQueue){return queue;}else {var result;forEach$3(queue).call(queue,function(fn){result=fn();});return result;}}/**
	   * recalculate the height of the subgroups
	   *
	   * @param {{item: timeline.Item}} margin
	   * @private
	   */},{key:"_updateSubGroupHeights",value:function _updateSubGroupHeights(margin){var _this10=this;if(keys$6(this.subgroups).length>0){var me=this;this._resetSubgroups();forEach$3(util$1).call(util$1,this.visibleItems,function(item){if(item.data.subgroup!==undefined){me.subgroups[item.data.subgroup].height=Math.max(me.subgroups[item.data.subgroup].height,item.height+margin.item.vertical);me.subgroups[item.data.subgroup].visible=typeof _this10.subgroupVisibility[item.data.subgroup]==='undefined'?true:Boolean(_this10.subgroupVisibility[item.data.subgroup]);}});}}/**
	   * check if group is visible
	   *
	   * @param {timeline.Range} range
	   * @param {{axis: timeline.DataAxis}} margin
	   * @returns {boolean} is visible
	   * @private
	   */},{key:"_isGroupVisible",value:function _isGroupVisible(range,margin){return this.top<=range.body.domProps.centerContainer.height-range.body.domProps.scrollTop+margin.axis&&this.top+this.height+margin.axis>=-range.body.domProps.scrollTop;}/**
	   * recalculate the height of the group
	   * @param {{item: {horizontal: number, vertical: number}, axis: number}} margin
	   * @returns {number} Returns the height
	   * @private
	   */},{key:"_calculateHeight",value:function _calculateHeight(margin){// recalculate the height of the group
	var height;var items;if(this.heightMode==='fixed'){items=util$1.toArray(this.items);}else {// default or 'auto'
	items=this.visibleItems;}if(items.length>0){var min=items[0].top;var max=items[0].top+items[0].height;forEach$3(util$1).call(util$1,items,function(item){min=Math.min(min,item.top);max=Math.max(max,item.top+item.height);});if(min>margin.axis){// there is an empty gap between the lowest item and the axis
	var offset=min-margin.axis;max-=offset;forEach$3(util$1).call(util$1,items,function(item){item.top-=offset;});}height=Math.ceil(max+margin.item.vertical/2);if(this.heightMode!=="fitItems"){height=Math.max(height,this.props.label.height);}}else {height=this.props.label.height;}return height;}/**
	   * Show this group: attach to the DOM
	   */},{key:"show",value:function show(){if(!this.dom.label.parentNode){this.itemSet.dom.labelSet.appendChild(this.dom.label);}if(!this.dom.foreground.parentNode){this.itemSet.dom.foreground.appendChild(this.dom.foreground);}if(!this.dom.background.parentNode){this.itemSet.dom.background.appendChild(this.dom.background);}if(!this.dom.axis.parentNode){this.itemSet.dom.axis.appendChild(this.dom.axis);}}/**
	   * Hide this group: remove from the DOM
	   */},{key:"hide",value:function hide(){var label=this.dom.label;if(label.parentNode){label.parentNode.removeChild(label);}var foreground=this.dom.foreground;if(foreground.parentNode){foreground.parentNode.removeChild(foreground);}var background=this.dom.background;if(background.parentNode){background.parentNode.removeChild(background);}var axis=this.dom.axis;if(axis.parentNode){axis.parentNode.removeChild(axis);}}/**
	   * Add an item to the group
	   * @param {Item} item
	   */},{key:"add",value:function add(item){var _context82;this.items[item.id]=item;item.setParent(this);this.stackDirty=true;// add to
	if(item.data.subgroup!==undefined){this._addToSubgroup(item);this.orderSubgroups();}if(!includes$4(_context82=this.visibleItems).call(_context82,item)){var range=this.itemSet.body.range;// TODO: not nice accessing the range like this
	this._checkIfVisible(item,this.visibleItems,range);}}/**
	   * add item to subgroup
	   * @param {object} item
	   * @param {string} subgroupId
	   */},{key:"_addToSubgroup",value:function _addToSubgroup(item){var subgroupId=arguments.length>1&&arguments[1]!==undefined?arguments[1]:item.data.subgroup;if(subgroupId!=undefined&&this.subgroups[subgroupId]===undefined){this.subgroups[subgroupId]={height:0,top:0,start:item.data.start,end:item.data.end||item.data.start,visible:false,index:this.subgroupIndex,items:[],stack:this.subgroupStackAll||this.subgroupStack[subgroupId]||false};this.subgroupIndex++;}if(new Date(item.data.start)<new Date(this.subgroups[subgroupId].start)){this.subgroups[subgroupId].start=item.data.start;}var itemEnd=item.data.end||item.data.start;if(new Date(itemEnd)>new Date(this.subgroups[subgroupId].end)){this.subgroups[subgroupId].end=itemEnd;}this.subgroups[subgroupId].items.push(item);}/**
	   * update subgroup sizes
	   */},{key:"_updateSubgroupsSizes",value:function _updateSubgroupsSizes(){var me=this;if(me.subgroups){var _loop3=function _loop3(subgroup){var _context83;var initialEnd=me.subgroups[subgroup].items[0].data.end||me.subgroups[subgroup].items[0].data.start;var newStart=me.subgroups[subgroup].items[0].data.start;var newEnd=initialEnd-1;forEach$3(_context83=me.subgroups[subgroup].items).call(_context83,function(item){if(new Date(item.data.start)<new Date(newStart)){newStart=item.data.start;}var itemEnd=item.data.end||item.data.start;if(new Date(itemEnd)>new Date(newEnd)){newEnd=itemEnd;}});me.subgroups[subgroup].start=newStart;me.subgroups[subgroup].end=new Date(newEnd-1);// -1 to compensate for colliding end to start subgroups;
	};for(var subgroup in me.subgroups){_loop3(subgroup);}}}/**
	   * order subgroups
	   */},{key:"orderSubgroups",value:function orderSubgroups(){if(this.subgroupOrderer!==undefined){var sortArray=[];if(typeof this.subgroupOrderer=='string'){for(var subgroup in this.subgroups){sortArray.push({subgroup:subgroup,sortField:this.subgroups[subgroup].items[0].data[this.subgroupOrderer]});}sort$2(sortArray).call(sortArray,function(a,b){return a.sortField-b.sortField;});}else if(typeof this.subgroupOrderer=='function'){for(var _subgroup in this.subgroups){sortArray.push(this.subgroups[_subgroup].items[0].data);}sort$2(sortArray).call(sortArray,this.subgroupOrderer);}if(sortArray.length>0){for(var i=0;i<sortArray.length;i++){this.subgroups[sortArray[i].subgroup].index=i;}}}}/**
	   * add item to subgroup
	   */},{key:"_resetSubgroups",value:function _resetSubgroups(){for(var subgroup in this.subgroups){if(this.subgroups.hasOwnProperty(subgroup)){this.subgroups[subgroup].visible=false;this.subgroups[subgroup].height=0;}}}/**
	   * Remove an item from the group
	   * @param {Item} item
	   */},{key:"remove",value:function remove(item){var _context84,_context85;delete this.items[item.id];item.setParent(null);this.stackDirty=true;// remove from visible items
	var index=indexOf$3(_context84=this.visibleItems).call(_context84,item);if(index!=-1)splice$2(_context85=this.visibleItems).call(_context85,index,1);if(item.data.subgroup!==undefined){this._removeFromSubgroup(item);this.orderSubgroups();}}/**
	   * remove item from subgroup
	   * @param {object} item
	   * @param {string} subgroupId
	   */},{key:"_removeFromSubgroup",value:function _removeFromSubgroup(item){var subgroupId=arguments.length>1&&arguments[1]!==undefined?arguments[1]:item.data.subgroup;if(subgroupId!=undefined){var subgroup=this.subgroups[subgroupId];if(subgroup){var _context86;var itemIndex=indexOf$3(_context86=subgroup.items).call(_context86,item);//  Check the item is actually in this subgroup. How should items not in the group be handled?
	if(itemIndex>=0){var _context87;splice$2(_context87=subgroup.items).call(_context87,itemIndex,1);if(!subgroup.items.length){delete this.subgroups[subgroupId];}else {this._updateSubgroupsSizes();}}}}}/**
	   * Remove an item from the corresponding DataSet
	   * @param {Item} item
	   */},{key:"removeFromDataSet",value:function removeFromDataSet(item){this.itemSet.removeItem(item.id);}/**
	   * Reorder the items
	   */},{key:"order",value:function order(){var array=util$1.toArray(this.items);var startArray=[];var endArray=[];for(var i=0;i<array.length;i++){if(array[i].data.end!==undefined){endArray.push(array[i]);}startArray.push(array[i]);}this.orderedItems={byStart:startArray,byEnd:endArray};orderByStart(this.orderedItems.byStart);orderByEnd(this.orderedItems.byEnd);}/**
	   * Update the visible items
	   * @param {{byStart: Item[], byEnd: Item[]}} orderedItems   All items ordered by start date and by end date
	   * @param {Item[]} oldVisibleItems                          The previously visible items.
	   * @param {{start: number, end: number}} range              Visible range
	   * @return {Item[]} visibleItems                            The new visible items.
	   * @private
	   */},{key:"_updateItemsInRange",value:function _updateItemsInRange(orderedItems,oldVisibleItems,range){var visibleItems=[];var visibleItemsLookup={};// we keep this to quickly look up if an item already exists in the list without using indexOf on visibleItems
	if(!this.isVisible&&this.groupId!=ReservedGroupIds.BACKGROUND){for(var i=0;i<oldVisibleItems.length;i++){var item=oldVisibleItems[i];if(item.displayed)item.hide();}return visibleItems;}var interval=(range.end-range.start)/4;var lowerBound=range.start-interval;var upperBound=range.end+interval;// this function is used to do the binary search for items having start date only.
	var startSearchFunction=function startSearchFunction(value){if(value<lowerBound){return -1;}else if(value<=upperBound){return 0;}else {return 1;}};// this function is used to do the binary search for items having start and end dates (range).
	var endSearchFunction=function endSearchFunction(data){var start=data.start,end=data.end;if(end<lowerBound){return -1;}else if(start<=upperBound){return 0;}else {return 1;}};// first check if the items that were in view previously are still in view.
	// IMPORTANT: this handles the case for the items with startdate before the window and enddate after the window!
	// also cleans up invisible items.
	if(oldVisibleItems.length>0){for(var _i=0;_i<oldVisibleItems.length;_i++){this._checkIfVisibleWithReference(oldVisibleItems[_i],visibleItems,visibleItemsLookup,range);}}// we do a binary search for the items that have only start values.
	var initialPosByStart=util$1.binarySearchCustom(orderedItems.byStart,startSearchFunction,'data','start');// trace the visible items from the inital start pos both ways until an invisible item is found, we only look at the start values.
	this._traceVisible(initialPosByStart,orderedItems.byStart,visibleItems,visibleItemsLookup,function(item){return item.data.start<lowerBound||item.data.start>upperBound;});// if the window has changed programmatically without overlapping the old window, the ranged items with start < lowerBound and end > upperbound are not shown.
	// We therefore have to brute force check all items in the byEnd list
	if(this.checkRangedItems==true){this.checkRangedItems=false;for(var _i2=0;_i2<orderedItems.byEnd.length;_i2++){this._checkIfVisibleWithReference(orderedItems.byEnd[_i2],visibleItems,visibleItemsLookup,range);}}else {// we do a binary search for the items that have defined end times.
	var initialPosByEnd=util$1.binarySearchCustom(orderedItems.byEnd,endSearchFunction,'data');// trace the visible items from the inital start pos both ways until an invisible item is found, we only look at the end values.
	this._traceVisible(initialPosByEnd,orderedItems.byEnd,visibleItems,visibleItemsLookup,function(item){return item.data.end<lowerBound||item.data.start>upperBound;});}var redrawQueue={};var redrawQueueLength=0;for(var _i3=0;_i3<visibleItems.length;_i3++){var _item3=visibleItems[_i3];if(!_item3.displayed){var returnQueue=true;redrawQueue[_i3]=_item3.redraw(returnQueue);redrawQueueLength=redrawQueue[_i3].length;}}var needRedraw=redrawQueueLength>0;if(needRedraw){var _loop4=function _loop4(j){forEach$3(util$1).call(util$1,redrawQueue,function(fns){fns[j]();});};// redraw all regular items
	for(var j=0;j<redrawQueueLength;j++){_loop4(j);}}for(var _i4=0;_i4<visibleItems.length;_i4++){visibleItems[_i4].repositionX();}return visibleItems;}/**
	   * trace visible items in group
	   * @param {number} initialPos
	   * @param {array} items
	   * @param {aray} visibleItems
	   * @param {object} visibleItemsLookup
	   * @param {function} breakCondition
	   */},{key:"_traceVisible",value:function _traceVisible(initialPos,items,visibleItems,visibleItemsLookup,breakCondition){if(initialPos!=-1){for(var i=initialPos;i>=0;i--){var _item4=items[i];if(breakCondition(_item4)){break;}else {if(!(_item4.isCluster&&!_item4.hasItems())&&!_item4.cluster){if(visibleItemsLookup[_item4.id]===undefined){visibleItemsLookup[_item4.id]=true;visibleItems.push(_item4);}}}}for(var _i5=initialPos+1;_i5<items.length;_i5++){var _item5=items[_i5];if(breakCondition(_item5)){break;}else {if(!(_item5.isCluster&&!_item5.hasItems())&&!_item5.cluster){if(visibleItemsLookup[_item5.id]===undefined){visibleItemsLookup[_item5.id]=true;visibleItems.push(_item5);}}}}}}/**
	   * this function is very similar to the _checkIfInvisible() but it does not
	   * return booleans, hides the item if it should not be seen and always adds to
	   * the visibleItems.
	   * this one is for brute forcing and hiding.
	   *
	   * @param {Item} item
	   * @param {Array} visibleItems
	   * @param {{start:number, end:number}} range
	   * @private
	   */},{key:"_checkIfVisible",value:function _checkIfVisible(item,visibleItems,range){if(item.isVisible(range)){if(!item.displayed)item.show();// reposition item horizontally
	item.repositionX();visibleItems.push(item);}else {if(item.displayed)item.hide();}}/**
	   * this function is very similar to the _checkIfInvisible() but it does not
	   * return booleans, hides the item if it should not be seen and always adds to
	   * the visibleItems.
	   * this one is for brute forcing and hiding.
	   *
	   * @param {Item} item
	   * @param {Array.<timeline.Item>} visibleItems
	   * @param {Object<number, boolean>} visibleItemsLookup
	   * @param {{start:number, end:number}} range
	   * @private
	   */},{key:"_checkIfVisibleWithReference",value:function _checkIfVisibleWithReference(item,visibleItems,visibleItemsLookup,range){if(item.isVisible(range)){if(visibleItemsLookup[item.id]===undefined){visibleItemsLookup[item.id]=true;visibleItems.push(item);}}else {if(item.displayed)item.hide();}}/**
	   * Update the visible items
	   * @param {array} orderedClusters 
	   * @param {array} oldVisibleClusters                         
	   * @param {{start: number, end: number}} range             
	   * @return {Item[]} visibleItems                            
	   * @private
	   */},{key:"_updateClustersInRange",value:function _updateClustersInRange(orderedClusters,oldVisibleClusters,range){// Clusters can overlap each other so we cannot use binary search here
	var visibleClusters=[];var visibleClustersLookup={};// we keep this to quickly look up if an item already exists in the list without using indexOf on visibleItems
	if(oldVisibleClusters.length>0){for(var i=0;i<oldVisibleClusters.length;i++){this._checkIfVisibleWithReference(oldVisibleClusters[i],visibleClusters,visibleClustersLookup,range);}}for(var _i6=0;_i6<orderedClusters.byStart.length;_i6++){this._checkIfVisibleWithReference(orderedClusters.byStart[_i6],visibleClusters,visibleClustersLookup,range);}for(var _i7=0;_i7<orderedClusters.byEnd.length;_i7++){this._checkIfVisibleWithReference(orderedClusters.byEnd[_i7],visibleClusters,visibleClustersLookup,range);}var redrawQueue={};var redrawQueueLength=0;for(var _i8=0;_i8<visibleClusters.length;_i8++){var _item6=visibleClusters[_i8];if(!_item6.displayed){var returnQueue=true;redrawQueue[_i8]=_item6.redraw(returnQueue);redrawQueueLength=redrawQueue[_i8].length;}}var needRedraw=redrawQueueLength>0;if(needRedraw){// redraw all regular items
	for(var j=0;j<redrawQueueLength;j++){forEach$3(util$1).call(util$1,redrawQueue,function(fns){fns[j]();});}}for(var _i9=0;_i9<visibleClusters.length;_i9++){visibleClusters[_i9].repositionX();}return visibleClusters;}/**
	   * change item subgroup
	   * @param {object} item
	   * @param {string} oldSubgroup
	   * @param {string} newSubgroup
	   */},{key:"changeSubgroup",value:function changeSubgroup(item,oldSubgroup,newSubgroup){this._removeFromSubgroup(item,oldSubgroup);this._addToSubgroup(item,newSubgroup);this.orderSubgroups();}/**
	   * Call this method before you lose the last reference to an instance of this.
	   * It will remove listeners etc.
	   */},{key:"dispose",value:function dispose(){this.hide();var disposeCallback;while(disposeCallback=this._disposeCallbacks.pop()){disposeCallback();}}}]);return Group;}();/**
	 * @constructor BackgroundGroup
	 * @extends Group
	 */var BackgroundGroup=/*#__PURE__*/function(_Group){inherits(BackgroundGroup,_Group);var _super5=_createSuper$2(BackgroundGroup);/**
	 * @param {number | string} groupId
	 * @param {Object} data
	 * @param {ItemSet} itemSet
	 */function BackgroundGroup(groupId,data,itemSet){var _this11;classCallCheck(this,BackgroundGroup);_this11=_super5.call(this,groupId,data,itemSet);// Group.call(this, groupId, data, itemSet);
	_this11.width=0;_this11.height=0;_this11.top=0;_this11.left=0;return _this11;}/**
	   * Repaint this group
	   * @param {{start: number, end: number}} range
	   * @param {{item: {horizontal: number, vertical: number}, axis: number}} margin
	   * @param {boolean} [forceRestack=false]  Force restacking of all items
	   * @return {boolean} Returns true if the group is resized
	   */createClass(BackgroundGroup,[{key:"redraw",value:function redraw(range,margin,forceRestack){// eslint-disable-line no-unused-vars
	var resized=false;this.visibleItems=this._updateItemsInRange(this.orderedItems,this.visibleItems,range);// calculate actual size
	this.width=this.dom.background.offsetWidth;// apply new height (just always zero for BackgroundGroup
	this.dom.background.style.height='0';// update vertical position of items after they are re-stacked and the height of the group is calculated
	for(var i=0,ii=this.visibleItems.length;i<ii;i++){var _item7=this.visibleItems[i];_item7.repositionY(margin);}return resized;}/**
	   * Show this group: attach to the DOM
	   */},{key:"show",value:function show(){if(!this.dom.background.parentNode){this.itemSet.dom.background.appendChild(this.dom.background);}}}]);return BackgroundGroup;}(Group);/**
	 * Item
	 */var Item=/*#__PURE__*/function(){/**
	 * @constructor Item
	 * @param {Object} data             Object containing (optional) parameters type,
	 *                                  start, end, content, group, className.
	 * @param {{toScreen: function, toTime: function}} conversion
	 *                                  Conversion functions from time to screen and vice versa
	 * @param {Object} options          Configuration options
	 *                                  // TODO: describe available options
	 */function Item(data,conversion,options){var _context88,_this12=this;classCallCheck(this,Item);this.id=null;this.parent=null;this.data=data;this.dom=null;this.conversion=conversion||{};this.defaultOptions={locales:locales$1,locale:'en'};this.options=util$1.extend({},this.defaultOptions,options);this.options.locales=util$1.extend({},locales$1,this.options.locales);var defaultLocales=this.defaultOptions.locales[this.defaultOptions.locale];forEach$3(_context88=keys$6(this.options.locales)).call(_context88,function(locale){_this12.options.locales[locale]=util$1.extend({},defaultLocales,_this12.options.locales[locale]);});this.selected=false;this.displayed=false;this.groupShowing=true;this.selectable=options&&options.selectable||false;this.dirty=true;this.top=null;this.right=null;this.left=null;this.width=null;this.height=null;this.setSelectability(data);this.editable=null;this._updateEditStatus();}/**
	   * Select current item
	   */createClass(Item,[{key:"select",value:function select(){if(this.selectable){this.selected=true;this.dirty=true;if(this.displayed)this.redraw();}}/**
	   * Unselect current item
	   */},{key:"unselect",value:function unselect(){this.selected=false;this.dirty=true;if(this.displayed)this.redraw();}/**
	   * Set data for the item. Existing data will be updated. The id should not
	   * be changed. When the item is displayed, it will be redrawn immediately.
	   * @param {Object} data
	   */},{key:"setData",value:function setData(data){var groupChanged=data.group!=undefined&&this.data.group!=data.group;if(groupChanged&&this.parent!=null){this.parent.itemSet._moveToGroup(this,data.group);}this.setSelectability(data);if(this.parent){this.parent.stackDirty=true;}var subGroupChanged=data.subgroup!=undefined&&this.data.subgroup!=data.subgroup;if(subGroupChanged&&this.parent!=null){this.parent.changeSubgroup(this,this.data.subgroup,data.subgroup);}this.data=data;this._updateEditStatus();this.dirty=true;if(this.displayed)this.redraw();}/**
	   * Set whether the item can be selected.
	   * Can only be set/unset if the timeline's `selectable` configuration option is `true`.
	   * @param {Object} data `data` from `constructor` and `setData`
	   */},{key:"setSelectability",value:function setSelectability(data){if(data){this.selectable=typeof data.selectable==='undefined'?true:Boolean(data.selectable);}}/**
	   * Set a parent for the item
	   * @param {Group} parent
	   */},{key:"setParent",value:function setParent(parent){if(this.displayed){this.hide();this.parent=parent;if(this.parent){this.show();}}else {this.parent=parent;}}/**
	   * Check whether this item is visible inside given range
	   * @param {timeline.Range} range with a timestamp for start and end
	   * @returns {boolean} True if visible
	   */},{key:"isVisible",value:function isVisible(range){// eslint-disable-line no-unused-vars
	return false;}/**
	   * Show the Item in the DOM (when not already visible)
	   * @return {Boolean} changed
	   */},{key:"show",value:function show(){return false;}/**
	   * Hide the Item from the DOM (when visible)
	   * @return {Boolean} changed
	   */},{key:"hide",value:function hide(){return false;}/**
	   * Repaint the item
	   */},{key:"redraw",value:function redraw(){}// should be implemented by the item
	/**
	   * Reposition the Item horizontally
	   */},{key:"repositionX",value:function repositionX(){}// should be implemented by the item
	/**
	   * Reposition the Item vertically
	   */},{key:"repositionY",value:function repositionY(){}// should be implemented by the item
	/**
	   * Repaint a drag area on the center of the item when the item is selected
	   * @protected
	   */},{key:"_repaintDragCenter",value:function _repaintDragCenter(){if(this.selected&&this.editable.updateTime&&!this.dom.dragCenter){var _context89,_context90;var me=this;// create and show drag area
	var dragCenter=document.createElement('div');dragCenter.className='vis-drag-center';dragCenter.dragCenterItem=this;this.hammerDragCenter=new Hammer$1(dragCenter);this.hammerDragCenter.on('tap',function(event){me.parent.itemSet.body.emitter.emit('click',{event:event,item:me.id});});this.hammerDragCenter.on('doubletap',function(event){event.stopPropagation();me.parent.itemSet._onUpdateItem(me);me.parent.itemSet.body.emitter.emit('doubleClick',{event:event,item:me.id});});this.hammerDragCenter.on('panstart',function(event){// do not allow this event to propagate to the Range
	event.stopPropagation();me.parent.itemSet._onDragStart(event);});this.hammerDragCenter.on('panmove',bind$2(_context89=me.parent.itemSet._onDrag).call(_context89,me.parent.itemSet));this.hammerDragCenter.on('panend',bind$2(_context90=me.parent.itemSet._onDragEnd).call(_context90,me.parent.itemSet));if(this.dom.box){if(this.dom.dragLeft){this.dom.box.insertBefore(dragCenter,this.dom.dragLeft);}else {this.dom.box.appendChild(dragCenter);}}else if(this.dom.point){this.dom.point.appendChild(dragCenter);}this.dom.dragCenter=dragCenter;}else if(!this.selected&&this.dom.dragCenter){// delete drag area
	if(this.dom.dragCenter.parentNode){this.dom.dragCenter.parentNode.removeChild(this.dom.dragCenter);}this.dom.dragCenter=null;if(this.hammerDragCenter){this.hammerDragCenter.destroy();this.hammerDragCenter=null;}}}/**
	   * Repaint a delete button on the top right of the item when the item is selected
	   * @param {HTMLElement} anchor
	   * @protected
	   */},{key:"_repaintDeleteButton",value:function _repaintDeleteButton(anchor){var editable=(this.options.editable.overrideItems||this.editable==null)&&this.options.editable.remove||!this.options.editable.overrideItems&&this.editable!=null&&this.editable.remove;if(this.selected&&editable&&!this.dom.deleteButton){// create and show button
	var me=this;var deleteButton=document.createElement('div');if(this.options.rtl){deleteButton.className='vis-delete-rtl';}else {deleteButton.className='vis-delete';}var optionsLocale=this.options.locales[this.options.locale];if(!optionsLocale){if(!this.warned){console.warn("WARNING: options.locales['".concat(this.options.locale,"'] not found. See https://visjs.github.io/vis-timeline/docs/timeline/#Localization"));this.warned=true;}optionsLocale=this.options.locales['en'];// fall back on english when not available
	}deleteButton.title=optionsLocale.deleteSelected;// TODO: be able to destroy the delete button
	this.hammerDeleteButton=new Hammer$1(deleteButton).on('tap',function(event){event.stopPropagation();me.parent.removeFromDataSet(me);});anchor.appendChild(deleteButton);this.dom.deleteButton=deleteButton;}else if(!this.selected&&this.dom.deleteButton){// remove button
	if(this.dom.deleteButton.parentNode){this.dom.deleteButton.parentNode.removeChild(this.dom.deleteButton);}this.dom.deleteButton=null;if(this.hammerDeleteButton){this.hammerDeleteButton.destroy();this.hammerDeleteButton=null;}}}/**
	   * Repaint a onChange tooltip on the top right of the item when the item is selected
	   * @param {HTMLElement} anchor
	   * @protected
	   */},{key:"_repaintOnItemUpdateTimeTooltip",value:function _repaintOnItemUpdateTimeTooltip(anchor){if(!this.options.tooltipOnItemUpdateTime)return;var editable=(this.options.editable.updateTime||this.data.editable===true)&&this.data.editable!==false;if(this.selected&&editable&&!this.dom.onItemUpdateTimeTooltip){var onItemUpdateTimeTooltip=document.createElement('div');onItemUpdateTimeTooltip.className='vis-onUpdateTime-tooltip';anchor.appendChild(onItemUpdateTimeTooltip);this.dom.onItemUpdateTimeTooltip=onItemUpdateTimeTooltip;}else if(!this.selected&&this.dom.onItemUpdateTimeTooltip){// remove button
	if(this.dom.onItemUpdateTimeTooltip.parentNode){this.dom.onItemUpdateTimeTooltip.parentNode.removeChild(this.dom.onItemUpdateTimeTooltip);}this.dom.onItemUpdateTimeTooltip=null;}// position onChange tooltip
	if(this.dom.onItemUpdateTimeTooltip){// only show when editing
	this.dom.onItemUpdateTimeTooltip.style.visibility=this.parent.itemSet.touchParams.itemIsDragging?'visible':'hidden';// position relative to item's content
	this.dom.onItemUpdateTimeTooltip.style.transform='translateX(-50%)';this.dom.onItemUpdateTimeTooltip.style.left='50%';// position above or below the item depending on the item's position in the window
	var tooltipOffset=50;// TODO: should be tooltip height (depends on template)
	var scrollTop=this.parent.itemSet.body.domProps.scrollTop;// TODO: this.top for orientation:true is actually the items distance from the bottom... 
	// (should be this.bottom)
	var itemDistanceFromTop;if(this.options.orientation.item=='top'){itemDistanceFromTop=this.top;}else {itemDistanceFromTop=this.parent.height-this.top-this.height;}var isCloseToTop=itemDistanceFromTop+this.parent.top-tooltipOffset<-scrollTop;if(isCloseToTop){this.dom.onItemUpdateTimeTooltip.style.bottom="";this.dom.onItemUpdateTimeTooltip.style.top="".concat(this.height+2,"px");}else {this.dom.onItemUpdateTimeTooltip.style.top="";this.dom.onItemUpdateTimeTooltip.style.bottom="".concat(this.height+2,"px");}// handle tooltip content
	var content;var templateFunction;if(this.options.tooltipOnItemUpdateTime&&this.options.tooltipOnItemUpdateTime.template){var _context91;templateFunction=bind$2(_context91=this.options.tooltipOnItemUpdateTime.template).call(_context91,this);content=templateFunction(this.data);}else {content="start: ".concat(moment$1(this.data.start).format('MM/DD/YYYY hh:mm'));if(this.data.end){content+="<br> end: ".concat(moment$1(this.data.end).format('MM/DD/YYYY hh:mm'));}}this.dom.onItemUpdateTimeTooltip.innerHTML=content;}}/**
	   * get item data
	   * @return {object}
	   * @private
	   */},{key:"_getItemData",value:function _getItemData(){return this.parent.itemSet.itemsData.get(this.id);}/**
	   * Set HTML contents for the item
	   * @param {Element} element   HTML element to fill with the contents
	   * @private
	   */},{key:"_updateContents",value:function _updateContents(element){var content;var changed;var templateFunction;var itemVisibleFrameContent;var visibleFrameTemplateFunction;var itemData=this._getItemData();// get a clone of the data from the dataset
	var frameElement=this.dom.box||this.dom.point;var itemVisibleFrameContentElement=frameElement.getElementsByClassName('vis-item-visible-frame')[0];if(this.options.visibleFrameTemplate){var _context92;visibleFrameTemplateFunction=bind$2(_context92=this.options.visibleFrameTemplate).call(_context92,this);itemVisibleFrameContent=visibleFrameTemplateFunction(itemData,itemVisibleFrameContentElement);}else {itemVisibleFrameContent='';}if(itemVisibleFrameContentElement){if(itemVisibleFrameContent instanceof Object&&!(itemVisibleFrameContent instanceof Element)){visibleFrameTemplateFunction(itemData,itemVisibleFrameContentElement);}else {changed=this._contentToString(this.itemVisibleFrameContent)!==this._contentToString(itemVisibleFrameContent);if(changed){// only replace the content when changed
	if(itemVisibleFrameContent instanceof Element){itemVisibleFrameContentElement.innerHTML='';itemVisibleFrameContentElement.appendChild(itemVisibleFrameContent);}else if(itemVisibleFrameContent!=undefined){itemVisibleFrameContentElement.innerHTML=itemVisibleFrameContent;}else {if(!(this.data.type=='background'&&this.data.content===undefined)){throw new Error("Property \"content\" missing in item ".concat(this.id));}}this.itemVisibleFrameContent=itemVisibleFrameContent;}}}if(this.options.template){var _context93;templateFunction=bind$2(_context93=this.options.template).call(_context93,this);content=templateFunction(itemData,element,this.data);}else {content=this.data.content;}if(content instanceof Object&&!(content instanceof Element)){templateFunction(itemData,element);}else {changed=this._contentToString(this.content)!==this._contentToString(content);if(changed){// only replace the content when changed
	if(content instanceof Element){element.innerHTML='';element.appendChild(content);}else if(content!=undefined){element.innerHTML=content;}else {if(!(this.data.type=='background'&&this.data.content===undefined)){throw new Error("Property \"content\" missing in item ".concat(this.id));}}this.content=content;}}}/**
	   * Process dataAttributes timeline option and set as data- attributes on dom.content
	   * @param {Element} element   HTML element to which the attributes will be attached
	   * @private
	   */},{key:"_updateDataAttributes",value:function _updateDataAttributes(element){if(this.options.dataAttributes&&this.options.dataAttributes.length>0){var attributes=[];if(isArray$3(this.options.dataAttributes)){attributes=this.options.dataAttributes;}else if(this.options.dataAttributes=='all'){attributes=keys$6(this.data);}else {return;}var _iterator=_createForOfIteratorHelper$3(attributes),_step2;try{for(_iterator.s();!(_step2=_iterator.n()).done;){var name=_step2.value;var value=this.data[name];if(value!=null){element.setAttribute("data-".concat(name),value);}else {element.removeAttribute("data-".concat(name));}}}catch(err){_iterator.e(err);}finally{_iterator.f();}}}/**
	   * Update custom styles of the element
	   * @param {Element} element
	   * @private
	   */},{key:"_updateStyle",value:function _updateStyle(element){// remove old styles
	if(this.style){util$1.removeCssText(element,this.style);this.style=null;}// append new styles
	if(this.data.style){util$1.addCssText(element,this.data.style);this.style=this.data.style;}}/**
	   * Stringify the items contents
	   * @param {string | Element | undefined} content
	   * @returns {string | undefined}
	   * @private
	   */},{key:"_contentToString",value:function _contentToString(content){if(typeof content==='string')return content;if(content&&'outerHTML'in content)return content.outerHTML;return content;}/**
	   * Update the editability of this item.
	   */},{key:"_updateEditStatus",value:function _updateEditStatus(){if(this.options){if(typeof this.options.editable==='boolean'){this.editable={updateTime:this.options.editable,updateGroup:this.options.editable,remove:this.options.editable};}else if(_typeof_1(this.options.editable)==='object'){this.editable={};util$1.selectiveExtend(['updateTime','updateGroup','remove'],this.editable,this.options.editable);}}// Item data overrides, except if options.editable.overrideItems is set.
	if(!this.options||!this.options.editable||this.options.editable.overrideItems!==true){if(this.data){if(typeof this.data.editable==='boolean'){this.editable={updateTime:this.data.editable,updateGroup:this.data.editable,remove:this.data.editable};}else if(_typeof_1(this.data.editable)==='object'){// TODO: in timeline.js 5.0, we should change this to not reset options from the timeline configuration.
	// Basically just remove the next line...
	this.editable={};util$1.selectiveExtend(['updateTime','updateGroup','remove'],this.editable,this.data.editable);}}}}/**
	   * Return the width of the item left from its start date
	   * @return {number}
	   */},{key:"getWidthLeft",value:function getWidthLeft(){return 0;}/**
	   * Return the width of the item right from the max of its start and end date
	   * @return {number}
	   */},{key:"getWidthRight",value:function getWidthRight(){return 0;}/**
	   * Return the title of the item
	   * @return {string | undefined}
	   */},{key:"getTitle",value:function getTitle(){if(this.options.tooltip&&this.options.tooltip.template){var _context94;var templateFunction=bind$2(_context94=this.options.tooltip.template).call(_context94,this);return templateFunction(this._getItemData(),this.data);}return this.data.title;}}]);return Item;}();Item.prototype.stack=true;/**
	 * @constructor BoxItem
	 * @extends Item
	 */var BoxItem=/*#__PURE__*/function(_Item){inherits(BoxItem,_Item);var _super6=_createSuper$2(BoxItem);/**
	 * @param {Object} data             Object containing parameters start
	 *                                  content, className.
	 * @param {{toScreen: function, toTime: function}} conversion
	 *                                  Conversion functions from time to screen and vice versa
	 * @param {Object} [options]        Configuration options
	 *                                  // TODO: describe available options
	 */function BoxItem(data,conversion,options){var _this13;classCallCheck(this,BoxItem);_this13=_super6.call(this,data,conversion,options);_this13.props={dot:{width:0,height:0},line:{width:0,height:0}};// validate data
	if(data){if(data.start==undefined){throw new Error("Property \"start\" missing in item ".concat(data));}}return _this13;}/**
	   * Check whether this item is visible inside given range
	   * @param {{start: number, end: number}} range with a timestamp for start and end
	   * @returns {boolean} True if visible
	   */createClass(BoxItem,[{key:"isVisible",value:function isVisible(range){if(this.cluster){return false;}// determine visibility
	var isVisible;var align=this.data.align||this.options.align;var widthInMs=this.width*range.getMillisecondsPerPixel();if(align=='right'){isVisible=this.data.start.getTime()>range.start&&this.data.start.getTime()-widthInMs<range.end;}else if(align=='left'){isVisible=this.data.start.getTime()+widthInMs>range.start&&this.data.start.getTime()<range.end;}else {// default or 'center'
	isVisible=this.data.start.getTime()+widthInMs/2>range.start&&this.data.start.getTime()-widthInMs/2<range.end;}return isVisible;}/**
	   * create DOM element
	   * @private
	   */},{key:"_createDomElement",value:function _createDomElement(){if(!this.dom){// create DOM
	this.dom={};// create main box
	this.dom.box=document.createElement('DIV');// contents box (inside the background box). used for making margins
	this.dom.content=document.createElement('DIV');this.dom.content.className='vis-item-content';this.dom.box.appendChild(this.dom.content);// line to axis
	this.dom.line=document.createElement('DIV');this.dom.line.className='vis-line';// dot on axis
	this.dom.dot=document.createElement('DIV');this.dom.dot.className='vis-dot';// attach this item as attribute
	this.dom.box['vis-item']=this;this.dirty=true;}}/**
	   * append DOM element
	   * @private
	   */},{key:"_appendDomElement",value:function _appendDomElement(){if(!this.parent){throw new Error('Cannot redraw item: no parent attached');}if(!this.dom.box.parentNode){var foreground=this.parent.dom.foreground;if(!foreground)throw new Error('Cannot redraw item: parent has no foreground container element');foreground.appendChild(this.dom.box);}if(!this.dom.line.parentNode){var background=this.parent.dom.background;if(!background)throw new Error('Cannot redraw item: parent has no background container element');background.appendChild(this.dom.line);}if(!this.dom.dot.parentNode){var axis=this.parent.dom.axis;if(!background)throw new Error('Cannot redraw item: parent has no axis container element');axis.appendChild(this.dom.dot);}this.displayed=true;}/**
	   * update dirty DOM element
	   * @private
	   */},{key:"_updateDirtyDomComponents",value:function _updateDirtyDomComponents(){// An item is marked dirty when:
	// - the item is not yet rendered
	// - the item's data is changed
	// - the item is selected/deselected
	if(this.dirty){this._updateContents(this.dom.content);this._updateDataAttributes(this.dom.box);this._updateStyle(this.dom.box);var editable=this.editable.updateTime||this.editable.updateGroup;// update class
	var className=(this.data.className?' '+this.data.className:'')+(this.selected?' vis-selected':'')+(editable?' vis-editable':' vis-readonly');this.dom.box.className="vis-item vis-box".concat(className);this.dom.line.className="vis-item vis-line".concat(className);this.dom.dot.className="vis-item vis-dot".concat(className);}}/**
	   * get DOM components sizes
	   * @return {object}
	   * @private
	   */},{key:"_getDomComponentsSizes",value:function _getDomComponentsSizes(){return {previous:{right:this.dom.box.style.right,left:this.dom.box.style.left},dot:{height:this.dom.dot.offsetHeight,width:this.dom.dot.offsetWidth},line:{width:this.dom.line.offsetWidth},box:{width:this.dom.box.offsetWidth,height:this.dom.box.offsetHeight}};}/**
	   * update DOM components sizes
	   * @param {object} sizes
	   * @private
	   */},{key:"_updateDomComponentsSizes",value:function _updateDomComponentsSizes(sizes){if(this.options.rtl){this.dom.box.style.right="0px";}else {this.dom.box.style.left="0px";}// recalculate size
	this.props.dot.height=sizes.dot.height;this.props.dot.width=sizes.dot.width;this.props.line.width=sizes.line.width;this.width=sizes.box.width;this.height=sizes.box.height;// restore previous position
	if(this.options.rtl){this.dom.box.style.right=sizes.previous.right;}else {this.dom.box.style.left=sizes.previous.left;}this.dirty=false;}/**
	   * repaint DOM additionals
	   * @private
	   */},{key:"_repaintDomAdditionals",value:function _repaintDomAdditionals(){this._repaintOnItemUpdateTimeTooltip(this.dom.box);this._repaintDragCenter();this._repaintDeleteButton(this.dom.box);}/**
	   * Repaint the item
	   * @param {boolean} [returnQueue=false]  return the queue
	   * @return {boolean} the redraw queue if returnQueue=true
	   */},{key:"redraw",value:function redraw(returnQueue){var _context96,_context97,_context98,_this14=this,_context100;var sizes;var queue=[// create item DOM
	bind$2(_context96=this._createDomElement).call(_context96,this),// append DOM to parent DOM
	bind$2(_context97=this._appendDomElement).call(_context97,this),// update dirty DOM
	bind$2(_context98=this._updateDirtyDomComponents).call(_context98,this),function(){if(_this14.dirty){sizes=_this14._getDomComponentsSizes();}},function(){if(_this14.dirty){var _context99;bind$2(_context99=_this14._updateDomComponentsSizes).call(_context99,_this14)(sizes);}},// repaint DOM additionals
	bind$2(_context100=this._repaintDomAdditionals).call(_context100,this)];if(returnQueue){return queue;}else {var result;forEach$3(queue).call(queue,function(fn){result=fn();});return result;}}/**
	   * Show the item in the DOM (when not already visible). The items DOM will
	   * be created when needed.
	   * @param {boolean} [returnQueue=false]  whether to return a queue of functions to execute instead of just executing them
	   * @return {boolean} the redraw queue if returnQueue=true
	   */},{key:"show",value:function show(returnQueue){if(!this.displayed){return this.redraw(returnQueue);}}/**
	   * Hide the item from the DOM (when visible)
	   */},{key:"hide",value:function hide(){if(this.displayed){var _dom=this.dom;if(_dom.box.remove)_dom.box.remove();else if(_dom.box.parentNode)_dom.box.parentNode.removeChild(_dom.box);// IE11
	if(_dom.line.remove)_dom.line.remove();else if(_dom.line.parentNode)_dom.line.parentNode.removeChild(_dom.line);// IE11
	if(_dom.dot.remove)_dom.dot.remove();else if(_dom.dot.parentNode)_dom.dot.parentNode.removeChild(_dom.dot);// IE11
	this.displayed=false;}}/**
	   * Reposition the item XY
	   */},{key:"repositionXY",value:function repositionXY(){var rtl=this.options.rtl;var repositionXY=function repositionXY(element,x,y){var _context101;var rtl=arguments.length>3&&arguments[3]!==undefined?arguments[3]:false;if(x===undefined&&y===undefined)return;// If rtl invert the number.
	var directionX=rtl?x*-1:x;//no y. translate x
	if(y===undefined){element.style.transform="translateX(".concat(directionX,"px)");return;}//no x. translate y
	if(x===undefined){element.style.transform="translateY(".concat(y,"px)");return;}element.style.transform=concat$2(_context101="translate(".concat(directionX,"px, ")).call(_context101,y,"px)");};repositionXY(this.dom.box,this.boxX,this.boxY,rtl);repositionXY(this.dom.dot,this.dotX,this.dotY,rtl);repositionXY(this.dom.line,this.lineX,this.lineY,rtl);}/**
	   * Reposition the item horizontally
	   * @Override
	   */},{key:"repositionX",value:function repositionX(){var start=this.conversion.toScreen(this.data.start);var align=this.data.align===undefined?this.options.align:this.data.align;var lineWidth=this.props.line.width;var dotWidth=this.props.dot.width;if(align=='right'){// calculate right position of the box
	this.boxX=start-this.width;this.lineX=start-lineWidth;this.dotX=start-lineWidth/2-dotWidth/2;}else if(align=='left'){// calculate left position of the box
	this.boxX=start;this.lineX=start;this.dotX=start+lineWidth/2-dotWidth/2;}else {// default or 'center'
	this.boxX=start-this.width/2;this.lineX=this.options.rtl?start-lineWidth:start-lineWidth/2;this.dotX=start-dotWidth/2;}if(this.options.rtl)this.right=this.boxX;else this.left=this.boxX;this.repositionXY();}/**
	   * Reposition the item vertically
	   * @Override
	   */},{key:"repositionY",value:function repositionY(){var orientation=this.options.orientation.item;var lineStyle=this.dom.line.style;if(orientation=='top'){var lineHeight=this.parent.top+this.top+1;this.boxY=this.top||0;lineStyle.height="".concat(lineHeight,"px");lineStyle.bottom='';lineStyle.top='0';}else {// orientation 'bottom'
	var itemSetHeight=this.parent.itemSet.props.height;// TODO: this is nasty
	var _lineHeight=itemSetHeight-this.parent.top-this.parent.height+this.top;this.boxY=this.parent.height-this.top-(this.height||0);lineStyle.height="".concat(_lineHeight,"px");lineStyle.top='';lineStyle.bottom='0';}this.dotY=-this.props.dot.height/2;this.repositionXY();}/**
	   * Return the width of the item left from its start date
	   * @return {number}
	   */},{key:"getWidthLeft",value:function getWidthLeft(){return this.width/2;}/**
	   * Return the width of the item right from its start date
	   * @return {number}
	   */},{key:"getWidthRight",value:function getWidthRight(){return this.width/2;}}]);return BoxItem;}(Item);/**
	 * @constructor PointItem
	 * @extends Item
	 */var PointItem=/*#__PURE__*/function(_Item2){inherits(PointItem,_Item2);var _super7=_createSuper$2(PointItem);/**
	 * @param {Object} data             Object containing parameters start
	 *                                  content, className.
	 * @param {{toScreen: function, toTime: function}} conversion
	 *                                  Conversion functions from time to screen and vice versa
	 * @param {Object} [options]        Configuration options
	 *                                  // TODO: describe available options
	 */function PointItem(data,conversion,options){var _this15;classCallCheck(this,PointItem);_this15=_super7.call(this,data,conversion,options);_this15.props={dot:{top:0,width:0,height:0},content:{height:0,marginLeft:0,marginRight:0}};// validate data
	if(data){if(data.start==undefined){throw new Error("Property \"start\" missing in item ".concat(data));}}return _this15;}/**
	   * Check whether this item is visible inside given range
	   * @param {{start: number, end: number}} range with a timestamp for start and end
	   * @returns {boolean} True if visible
	   */createClass(PointItem,[{key:"isVisible",value:function isVisible(range){if(this.cluster){return false;}// determine visibility
	var widthInMs=this.width*range.getMillisecondsPerPixel();return this.data.start.getTime()+widthInMs>range.start&&this.data.start<range.end;}/**
	   * create DOM element
	   * @private
	   */},{key:"_createDomElement",value:function _createDomElement(){if(!this.dom){// create DOM
	this.dom={};// background box
	this.dom.point=document.createElement('div');// className is updated in redraw()
	// contents box, right from the dot
	this.dom.content=document.createElement('div');this.dom.content.className='vis-item-content';this.dom.point.appendChild(this.dom.content);// dot at start
	this.dom.dot=document.createElement('div');this.dom.point.appendChild(this.dom.dot);// attach this item as attribute
	this.dom.point['vis-item']=this;this.dirty=true;}}/**
	   * append DOM element
	   * @private
	   */},{key:"_appendDomElement",value:function _appendDomElement(){if(!this.parent){throw new Error('Cannot redraw item: no parent attached');}if(!this.dom.point.parentNode){var foreground=this.parent.dom.foreground;if(!foreground){throw new Error('Cannot redraw item: parent has no foreground container element');}foreground.appendChild(this.dom.point);}this.displayed=true;}/**
	   * update dirty DOM components
	   * @private
	   */},{key:"_updateDirtyDomComponents",value:function _updateDirtyDomComponents(){// An item is marked dirty when:
	// - the item is not yet rendered
	// - the item's data is changed
	// - the item is selected/deselected
	if(this.dirty){this._updateContents(this.dom.content);this._updateDataAttributes(this.dom.point);this._updateStyle(this.dom.point);var editable=this.editable.updateTime||this.editable.updateGroup;// update class
	var className=(this.data.className?' '+this.data.className:'')+(this.selected?' vis-selected':'')+(editable?' vis-editable':' vis-readonly');this.dom.point.className="vis-item vis-point".concat(className);this.dom.dot.className="vis-item vis-dot".concat(className);}}/**
	   * get DOM component sizes
	   * @return {object}
	   * @private
	   */},{key:"_getDomComponentsSizes",value:function _getDomComponentsSizes(){return {dot:{width:this.dom.dot.offsetWidth,height:this.dom.dot.offsetHeight},content:{width:this.dom.content.offsetWidth,height:this.dom.content.offsetHeight},point:{width:this.dom.point.offsetWidth,height:this.dom.point.offsetHeight}};}/**
	   * update DOM components sizes
	   * @param {array} sizes
	   * @private
	   */},{key:"_updateDomComponentsSizes",value:function _updateDomComponentsSizes(sizes){// recalculate size of dot and contents
	this.props.dot.width=sizes.dot.width;this.props.dot.height=sizes.dot.height;this.props.content.height=sizes.content.height;// resize contents
	if(this.options.rtl){this.dom.content.style.marginRight="".concat(2*this.props.dot.width,"px");}else {this.dom.content.style.marginLeft="".concat(2*this.props.dot.width,"px");}//this.dom.content.style.marginRight = ... + 'px'; // TODO: margin right
	// recalculate size
	this.width=sizes.point.width;this.height=sizes.point.height;// reposition the dot
	this.dom.dot.style.top="".concat((this.height-this.props.dot.height)/2,"px");var dotWidth=this.props.dot.width;var translateX=this.options.rtl?dotWidth/2*-1:dotWidth/2;this.dom.dot.style.transform="translateX(".concat(translateX,"px");this.dirty=false;}/**
	   * Repain DOM additionals
	   * @private
	   */},{key:"_repaintDomAdditionals",value:function _repaintDomAdditionals(){this._repaintOnItemUpdateTimeTooltip(this.dom.point);this._repaintDragCenter();this._repaintDeleteButton(this.dom.point);}/**
	   * Repaint the item
	   * @param {boolean} [returnQueue=false]  return the queue
	   * @return {boolean} the redraw queue if returnQueue=true
	   */},{key:"redraw",value:function redraw(returnQueue){var _context102,_context103,_context104,_this16=this,_context106;var sizes;var queue=[// create item DOM
	bind$2(_context102=this._createDomElement).call(_context102,this),// append DOM to parent DOM
	bind$2(_context103=this._appendDomElement).call(_context103,this),// update dirty DOM
	bind$2(_context104=this._updateDirtyDomComponents).call(_context104,this),function(){if(_this16.dirty){sizes=_this16._getDomComponentsSizes();}},function(){if(_this16.dirty){var _context105;bind$2(_context105=_this16._updateDomComponentsSizes).call(_context105,_this16)(sizes);}},// repaint DOM additionals
	bind$2(_context106=this._repaintDomAdditionals).call(_context106,this)];if(returnQueue){return queue;}else {var result;forEach$3(queue).call(queue,function(fn){result=fn();});return result;}}/**
	   * Reposition XY
	   */},{key:"repositionXY",value:function repositionXY(){var rtl=this.options.rtl;var repositionXY=function repositionXY(element,x,y){var _context107;var rtl=arguments.length>3&&arguments[3]!==undefined?arguments[3]:false;if(x===undefined&&y===undefined)return;// If rtl invert the number.
	var directionX=rtl?x*-1:x;//no y. translate x
	if(y===undefined){element.style.transform="translateX(".concat(directionX,"px)");return;}//no x. translate y
	if(x===undefined){element.style.transform="translateY(".concat(y,"px)");return;}element.style.transform=concat$2(_context107="translate(".concat(directionX,"px, ")).call(_context107,y,"px)");};repositionXY(this.dom.point,this.pointX,this.pointY,rtl);}/**
	   * Show the item in the DOM (when not already visible). The items DOM will
	   * be created when needed.
	   * @param {boolean} [returnQueue=false]  whether to return a queue of functions to execute instead of just executing them
	   * @return {boolean} the redraw queue if returnQueue=true
	   */},{key:"show",value:function show(returnQueue){if(!this.displayed){return this.redraw(returnQueue);}}/**
	   * Hide the item from the DOM (when visible)
	   */},{key:"hide",value:function hide(){if(this.displayed){if(this.dom.point.parentNode){this.dom.point.parentNode.removeChild(this.dom.point);}this.displayed=false;}}/**
	   * Reposition the item horizontally
	   * @Override
	   */},{key:"repositionX",value:function repositionX(){var start=this.conversion.toScreen(this.data.start);this.pointX=start;if(this.options.rtl){this.right=start-this.props.dot.width;}else {this.left=start-this.props.dot.width;}this.repositionXY();}/**
	   * Reposition the item vertically
	   * @Override
	   */},{key:"repositionY",value:function repositionY(){var orientation=this.options.orientation.item;if(orientation=='top'){this.pointY=this.top;}else {this.pointY=this.parent.height-this.top-this.height;}this.repositionXY();}/**
	   * Return the width of the item left from its start date
	   * @return {number}
	   */},{key:"getWidthLeft",value:function getWidthLeft(){return this.props.dot.width;}/**
	   * Return the width of the item right from  its start date
	   * @return {number}
	   */},{key:"getWidthRight",value:function getWidthRight(){return this.props.dot.width;}}]);return PointItem;}(Item);/**
	 * @constructor RangeItem
	 * @extends Item
	 */var RangeItem=/*#__PURE__*/function(_Item3){inherits(RangeItem,_Item3);var _super8=_createSuper$2(RangeItem);/**
	 * @param {Object} data             Object containing parameters start, end
	 *                                  content, className.
	 * @param {{toScreen: function, toTime: function}} conversion
	 *                                  Conversion functions from time to screen and vice versa
	 * @param {Object} [options]        Configuration options
	 *                                  // TODO: describe options
	 */function RangeItem(data,conversion,options){var _this17;classCallCheck(this,RangeItem);_this17=_super8.call(this,data,conversion,options);_this17.props={content:{width:0}};_this17.overflow=false;// if contents can overflow (css styling), this flag is set to true
	// validate data
	if(data){if(data.start==undefined){throw new Error("Property \"start\" missing in item ".concat(data.id));}if(data.end==undefined){throw new Error("Property \"end\" missing in item ".concat(data.id));}}return _this17;}/**
	   * Check whether this item is visible inside given range
	   *
	   * @param {timeline.Range} range with a timestamp for start and end
	   * @returns {boolean} True if visible
	   */createClass(RangeItem,[{key:"isVisible",value:function isVisible(range){if(this.cluster){return false;}// determine visibility
	return this.data.start<range.end&&this.data.end>range.start;}/**
	   * create DOM elements
	   * @private
	   */},{key:"_createDomElement",value:function _createDomElement(){if(!this.dom){// create DOM
	this.dom={};// background box
	this.dom.box=document.createElement('div');// className is updated in redraw()
	// frame box (to prevent the item contents from overflowing)
	this.dom.frame=document.createElement('div');this.dom.frame.className='vis-item-overflow';this.dom.box.appendChild(this.dom.frame);// visible frame box (showing the frame that is always visible)
	this.dom.visibleFrame=document.createElement('div');this.dom.visibleFrame.className='vis-item-visible-frame';this.dom.box.appendChild(this.dom.visibleFrame);// contents box
	this.dom.content=document.createElement('div');this.dom.content.className='vis-item-content';this.dom.frame.appendChild(this.dom.content);// attach this item as attribute
	this.dom.box['vis-item']=this;this.dirty=true;}}/**
	   * append element to DOM
	   * @private
	   */},{key:"_appendDomElement",value:function _appendDomElement(){if(!this.parent){throw new Error('Cannot redraw item: no parent attached');}if(!this.dom.box.parentNode){var foreground=this.parent.dom.foreground;if(!foreground){throw new Error('Cannot redraw item: parent has no foreground container element');}foreground.appendChild(this.dom.box);}this.displayed=true;}/**
	   * update dirty DOM components
	   * @private
	   */},{key:"_updateDirtyDomComponents",value:function _updateDirtyDomComponents(){// update dirty DOM. An item is marked dirty when:
	// - the item is not yet rendered
	// - the item's data is changed
	// - the item is selected/deselected
	if(this.dirty){this._updateContents(this.dom.content);this._updateDataAttributes(this.dom.box);this._updateStyle(this.dom.box);var editable=this.editable.updateTime||this.editable.updateGroup;// update class
	var className=(this.data.className?' '+this.data.className:'')+(this.selected?' vis-selected':'')+(editable?' vis-editable':' vis-readonly');this.dom.box.className=this.baseClassName+className;// turn off max-width to be able to calculate the real width
	// this causes an extra browser repaint/reflow, but so be it
	this.dom.content.style.maxWidth='none';}}/**
	   * get DOM component sizes
	   * @return {object}
	   * @private
	   */},{key:"_getDomComponentsSizes",value:function _getDomComponentsSizes(){// determine from css whether this box has overflow
	this.overflow=window.getComputedStyle(this.dom.frame).overflow!=='hidden';this.whiteSpace=window.getComputedStyle(this.dom.content).whiteSpace!=='nowrap';return {content:{width:this.dom.content.offsetWidth},box:{height:this.dom.box.offsetHeight}};}/**
	   * update DOM component sizes
	   * @param {array} sizes
	   * @private
	   */},{key:"_updateDomComponentsSizes",value:function _updateDomComponentsSizes(sizes){this.props.content.width=sizes.content.width;this.height=sizes.box.height;this.dom.content.style.maxWidth='';this.dirty=false;}/**
	   * repaint DOM additional components
	   * @private
	   */},{key:"_repaintDomAdditionals",value:function _repaintDomAdditionals(){this._repaintOnItemUpdateTimeTooltip(this.dom.box);this._repaintDeleteButton(this.dom.box);this._repaintDragCenter();this._repaintDragLeft();this._repaintDragRight();}/**
	   * Repaint the item
	   * @param {boolean} [returnQueue=false]  return the queue
	   * @return {boolean} the redraw queue if returnQueue=true
	   */},{key:"redraw",value:function redraw(returnQueue){var _context108,_context109,_context110,_this18=this,_context113;var sizes;var queue=[// create item DOM
	bind$2(_context108=this._createDomElement).call(_context108,this),// append DOM to parent DOM
	bind$2(_context109=this._appendDomElement).call(_context109,this),// update dirty DOM 
	bind$2(_context110=this._updateDirtyDomComponents).call(_context110,this),function(){if(_this18.dirty){var _context111;sizes=bind$2(_context111=_this18._getDomComponentsSizes).call(_context111,_this18)();}},function(){if(_this18.dirty){var _context112;bind$2(_context112=_this18._updateDomComponentsSizes).call(_context112,_this18)(sizes);}},// repaint DOM additionals
	bind$2(_context113=this._repaintDomAdditionals).call(_context113,this)];if(returnQueue){return queue;}else {var result;forEach$3(queue).call(queue,function(fn){result=fn();});return result;}}/**
	   * Show the item in the DOM (when not already visible). The items DOM will
	   * be created when needed.
	   * @param {boolean} [returnQueue=false]  whether to return a queue of functions to execute instead of just executing them
	   * @return {boolean} the redraw queue if returnQueue=true
	   */},{key:"show",value:function show(returnQueue){if(!this.displayed){return this.redraw(returnQueue);}}/**
	   * Hide the item from the DOM (when visible)
	   */},{key:"hide",value:function hide(){if(this.displayed){var box=this.dom.box;if(box.parentNode){box.parentNode.removeChild(box);}this.displayed=false;}}/**
	   * Reposition the item horizontally
	   * @param {boolean} [limitSize=true] If true (default), the width of the range
	   *                                   item will be limited, as the browser cannot
	   *                                   display very wide divs. This means though
	   *                                   that the applied left and width may
	   *                                   not correspond to the ranges start and end
	   * @Override
	   */},{key:"repositionX",value:function repositionX(limitSize){var parentWidth=this.parent.width;var start=this.conversion.toScreen(this.data.start);var end=this.conversion.toScreen(this.data.end);var align=this.data.align===undefined?this.options.align:this.data.align;var contentStartPosition;var contentWidth;// limit the width of the range, as browsers cannot draw very wide divs
	// unless limitSize: false is explicitly set in item data
	if(this.data.limitSize!==false&&(limitSize===undefined||limitSize===true)){if(start<-parentWidth){start=-parentWidth;}if(end>2*parentWidth){end=2*parentWidth;}}//round to 3 decimals to compensate floating-point values rounding
	var boxWidth=Math.max(Math.round((end-start)*1000)/1000,1);if(this.overflow){if(this.options.rtl){this.right=start;}else {this.left=start;}this.width=boxWidth+this.props.content.width;contentWidth=this.props.content.width;// Note: The calculation of width is an optimistic calculation, giving
	//       a width which will not change when moving the Timeline
	//       So no re-stacking needed, which is nicer for the eye;
	}else {if(this.options.rtl){this.right=start;}else {this.left=start;}this.width=boxWidth;contentWidth=Math.min(end-start,this.props.content.width);}if(this.options.rtl){this.dom.box.style.transform="translateX(".concat(this.right*-1,"px)");}else {this.dom.box.style.transform="translateX(".concat(this.left,"px)");}this.dom.box.style.width="".concat(boxWidth,"px");if(this.whiteSpace){this.height=this.dom.box.offsetHeight;}switch(align){case'left':this.dom.content.style.transform='translateX(0)';break;case'right':if(this.options.rtl){var translateX=Math.max(boxWidth-contentWidth,0)*-1;this.dom.content.style.transform="translateX(".concat(translateX,"px)");}else {this.dom.content.style.transform="translateX(".concat(Math.max(boxWidth-contentWidth,0),"px)");}break;case'center':if(this.options.rtl){var _translateX=Math.max((boxWidth-contentWidth)/2,0)*-1;this.dom.content.style.transform="translateX(".concat(_translateX,"px)");}else {this.dom.content.style.transform="translateX(".concat(Math.max((boxWidth-contentWidth)/2,0),"px)");}break;default:// 'auto'
	// when range exceeds left of the window, position the contents at the left of the visible area
	if(this.overflow){if(end>0){contentStartPosition=Math.max(-start,0);}else {contentStartPosition=-contentWidth;// ensure it's not visible anymore
	}}else {if(start<0){contentStartPosition=-start;}else {contentStartPosition=0;}}if(this.options.rtl){var _translateX2=contentStartPosition*-1;this.dom.content.style.transform="translateX(".concat(_translateX2,"px)");}else {this.dom.content.style.transform="translateX(".concat(contentStartPosition,"px)");// this.dom.content.style.width = `calc(100% - ${contentStartPosition}px)`;
	}}}/**
	   * Reposition the item vertically
	   * @Override
	   */},{key:"repositionY",value:function repositionY(){var orientation=this.options.orientation.item;var box=this.dom.box;if(orientation=='top'){box.style.top="".concat(this.top,"px");}else {box.style.top="".concat(this.parent.height-this.top-this.height,"px");}}/**
	   * Repaint a drag area on the left side of the range when the range is selected
	   * @protected
	   */},{key:"_repaintDragLeft",value:function _repaintDragLeft(){if((this.selected||this.options.itemsAlwaysDraggable.range)&&this.editable.updateTime&&!this.dom.dragLeft){// create and show drag area
	var dragLeft=document.createElement('div');dragLeft.className='vis-drag-left';dragLeft.dragLeftItem=this;this.dom.box.appendChild(dragLeft);this.dom.dragLeft=dragLeft;}else if(!this.selected&&!this.options.itemsAlwaysDraggable.range&&this.dom.dragLeft){// delete drag area
	if(this.dom.dragLeft.parentNode){this.dom.dragLeft.parentNode.removeChild(this.dom.dragLeft);}this.dom.dragLeft=null;}}/**
	   * Repaint a drag area on the right side of the range when the range is selected
	   * @protected
	   */},{key:"_repaintDragRight",value:function _repaintDragRight(){if((this.selected||this.options.itemsAlwaysDraggable.range)&&this.editable.updateTime&&!this.dom.dragRight){// create and show drag area
	var dragRight=document.createElement('div');dragRight.className='vis-drag-right';dragRight.dragRightItem=this;this.dom.box.appendChild(dragRight);this.dom.dragRight=dragRight;}else if(!this.selected&&!this.options.itemsAlwaysDraggable.range&&this.dom.dragRight){// delete drag area
	if(this.dom.dragRight.parentNode){this.dom.dragRight.parentNode.removeChild(this.dom.dragRight);}this.dom.dragRight=null;}}}]);return RangeItem;}(Item);RangeItem.prototype.baseClassName='vis-item vis-range';/**
	 * @constructor BackgroundItem
	 * @extends Item
	 */var BackgroundItem=/*#__PURE__*/function(_Item4){inherits(BackgroundItem,_Item4);var _super9=_createSuper$2(BackgroundItem);/**
	 * @constructor BackgroundItem
	 * @param {Object} data             Object containing parameters start, end
	 *                                  content, className.
	 * @param {{toScreen: function, toTime: function}} conversion
	 *                                  Conversion functions from time to screen and vice versa
	 * @param {Object} [options]        Configuration options
	 *                                  // TODO: describe options
	 * // TODO: implement support for the BackgroundItem just having a start, then being displayed as a sort of an annotation
	 */function BackgroundItem(data,conversion,options){var _this19;classCallCheck(this,BackgroundItem);_this19=_super9.call(this,data,conversion,options);_this19.props={content:{width:0}};_this19.overflow=false;// if contents can overflow (css styling), this flag is set to true
	// validate data
	if(data){if(data.start==undefined){throw new Error("Property \"start\" missing in item ".concat(data.id));}if(data.end==undefined){throw new Error("Property \"end\" missing in item ".concat(data.id));}}return _this19;}/**
	   * Check whether this item is visible inside given range
	   * @param {timeline.Range} range with a timestamp for start and end
	   * @returns {boolean} True if visible
	   */createClass(BackgroundItem,[{key:"isVisible",value:function isVisible(range){// determine visibility
	return this.data.start<range.end&&this.data.end>range.start;}/**
	   * create DOM element
	   * @private
	   */},{key:"_createDomElement",value:function _createDomElement(){if(!this.dom){// create DOM
	this.dom={};// background box
	this.dom.box=document.createElement('div');// className is updated in redraw()
	// frame box (to prevent the item contents from overflowing
	this.dom.frame=document.createElement('div');this.dom.frame.className='vis-item-overflow';this.dom.box.appendChild(this.dom.frame);// contents box
	this.dom.content=document.createElement('div');this.dom.content.className='vis-item-content';this.dom.frame.appendChild(this.dom.content);// Note: we do NOT attach this item as attribute to the DOM,
	//       such that background items cannot be selected
	//this.dom.box['vis-item'] = this;
	this.dirty=true;}}/**
	   * append DOM element
	   * @private
	   */},{key:"_appendDomElement",value:function _appendDomElement(){if(!this.parent){throw new Error('Cannot redraw item: no parent attached');}if(!this.dom.box.parentNode){var background=this.parent.dom.background;if(!background){throw new Error('Cannot redraw item: parent has no background container element');}background.appendChild(this.dom.box);}this.displayed=true;}/**
	   * update DOM Dirty components
	   * @private
	   */},{key:"_updateDirtyDomComponents",value:function _updateDirtyDomComponents(){// update dirty DOM. An item is marked dirty when:
	// - the item is not yet rendered
	// - the item's data is changed
	// - the item is selected/deselected
	if(this.dirty){this._updateContents(this.dom.content);this._updateDataAttributes(this.dom.content);this._updateStyle(this.dom.box);// update class
	var className=(this.data.className?' '+this.data.className:'')+(this.selected?' vis-selected':'');this.dom.box.className=this.baseClassName+className;}}/**
	   * get DOM components sizes
	   * @return {object}
	   * @private
	   */},{key:"_getDomComponentsSizes",value:function _getDomComponentsSizes(){// determine from css whether this box has overflow
	this.overflow=window.getComputedStyle(this.dom.content).overflow!=='hidden';return {content:{width:this.dom.content.offsetWidth}};}/**
	   * update DOM components sizes
	   * @param {object} sizes
	   * @private
	   */},{key:"_updateDomComponentsSizes",value:function _updateDomComponentsSizes(sizes){// recalculate size
	this.props.content.width=sizes.content.width;this.height=0;// set height zero, so this item will be ignored when stacking items
	this.dirty=false;}/**
	   * repaint DOM additionals
	   * @private
	   */},{key:"_repaintDomAdditionals",value:function _repaintDomAdditionals(){}/**
	   * Repaint the item
	   * @param {boolean} [returnQueue=false]  return the queue
	   * @return {boolean} the redraw result or the redraw queue if returnQueue=true
	   */},{key:"redraw",value:function redraw(returnQueue){var _context114,_context115,_context116,_this20=this,_context119;var sizes;var queue=[// create item DOM
	bind$2(_context114=this._createDomElement).call(_context114,this),// append DOM to parent DOM
	bind$2(_context115=this._appendDomElement).call(_context115,this),bind$2(_context116=this._updateDirtyDomComponents).call(_context116,this),function(){if(_this20.dirty){var _context117;sizes=bind$2(_context117=_this20._getDomComponentsSizes).call(_context117,_this20)();}},function(){if(_this20.dirty){var _context118;bind$2(_context118=_this20._updateDomComponentsSizes).call(_context118,_this20)(sizes);}},// repaint DOM additionals
	bind$2(_context119=this._repaintDomAdditionals).call(_context119,this)];if(returnQueue){return queue;}else {var result;forEach$3(queue).call(queue,function(fn){result=fn();});return result;}}/**
	   * Reposition the item vertically
	   * @Override
	   */},{key:"repositionY",value:function repositionY(margin){// eslint-disable-line no-unused-vars
	var height;var orientation=this.options.orientation.item;// special positioning for subgroups
	if(this.data.subgroup!==undefined){// TODO: instead of calculating the top position of the subgroups here for every BackgroundItem, calculate the top of the subgroup once in Itemset
	var itemSubgroup=this.data.subgroup;this.dom.box.style.height="".concat(this.parent.subgroups[itemSubgroup].height,"px");if(orientation=='top'){this.dom.box.style.top="".concat(this.parent.top+this.parent.subgroups[itemSubgroup].top,"px");}else {this.dom.box.style.top="".concat(this.parent.top+this.parent.height-this.parent.subgroups[itemSubgroup].top-this.parent.subgroups[itemSubgroup].height,"px");}this.dom.box.style.bottom='';}// and in the case of no subgroups:
	else {// we want backgrounds with groups to only show in groups.
	if(this.parent instanceof BackgroundGroup){// if the item is not in a group:
	height=Math.max(this.parent.height,this.parent.itemSet.body.domProps.center.height,this.parent.itemSet.body.domProps.centerContainer.height);this.dom.box.style.bottom=orientation=='bottom'?'0':'';this.dom.box.style.top=orientation=='top'?'0':'';}else {height=this.parent.height;// same alignment for items when orientation is top or bottom
	this.dom.box.style.top="".concat(this.parent.top,"px");this.dom.box.style.bottom='';}}this.dom.box.style.height="".concat(height,"px");}}]);return BackgroundItem;}(Item);BackgroundItem.prototype.baseClassName='vis-item vis-background';BackgroundItem.prototype.stack=false;/**
	 * Show the item in the DOM (when not already visible). The items DOM will
	 * be created when needed.
	 */BackgroundItem.prototype.show=RangeItem.prototype.show;/**
	 * Hide the item from the DOM (when visible)
	 * @return {Boolean} changed
	 */BackgroundItem.prototype.hide=RangeItem.prototype.hide;/**
	 * Reposition the item horizontally
	 * @Override
	 */BackgroundItem.prototype.repositionX=RangeItem.prototype.repositionX;/**
	 * Popup is a class to create a popup window with some text
	 */var Popup$1=/*#__PURE__*/function(){/**
	   * @param {Element} container       The container object.
	   * @param {string}  overflowMethod  How the popup should act to overflowing ('flip', 'cap' or 'none')
	   */function Popup(container,overflowMethod){classCallCheck(this,Popup);this.container=container;this.overflowMethod=overflowMethod||'cap';this.x=0;this.y=0;this.padding=5;this.hidden=false;// create the frame
	this.frame=document.createElement('div');this.frame.className='vis-tooltip';this.container.appendChild(this.frame);}/**
	   * @param {number} x   Horizontal position of the popup window
	   * @param {number} y   Vertical position of the popup window
	   */createClass(Popup,[{key:"setPosition",value:function setPosition(x,y){this.x=_parseInt$2(x);this.y=_parseInt$2(y);}/**
	   * Set the content for the popup window. This can be HTML code or text.
	   * @param {string | Element} content
	   */},{key:"setText",value:function setText(content){if(content instanceof Element){this.frame.innerHTML='';this.frame.appendChild(content);}else {this.frame.innerHTML=content;// string containing text or HTML
	}}/**
	   * Show the popup window
	   * @param {boolean} [doShow]    Show or hide the window
	   */},{key:"show",value:function show(doShow){if(doShow===undefined){doShow=true;}if(doShow===true){var height=this.frame.clientHeight;var width=this.frame.clientWidth;var maxHeight=this.frame.parentNode.clientHeight;var maxWidth=this.frame.parentNode.clientWidth;var left=0,top=0;if(this.overflowMethod=='flip'||this.overflowMethod=='none'){var isLeft=false,isTop=true;// Where around the position it's located
	if(this.overflowMethod=='flip'){if(this.y-height<this.padding){isTop=false;}if(this.x+width>maxWidth-this.padding){isLeft=true;}}if(isLeft){left=this.x-width;}else {left=this.x;}if(isTop){top=this.y-height;}else {top=this.y;}}else {// this.overflowMethod == 'cap'
	top=this.y-height;if(top+height+this.padding>maxHeight){top=maxHeight-height-this.padding;}if(top<this.padding){top=this.padding;}left=this.x;if(left+width+this.padding>maxWidth){left=maxWidth-width-this.padding;}if(left<this.padding){left=this.padding;}}this.frame.style.left=left+"px";this.frame.style.top=top+"px";this.frame.style.visibility="visible";this.hidden=false;}else {this.hide();}}/**
	   * Hide the popup window
	   */},{key:"hide",value:function hide(){this.hidden=true;this.frame.style.left="0";this.frame.style.top="0";this.frame.style.visibility="hidden";}/**
	   * Remove the popup window
	   */},{key:"destroy",value:function destroy(){this.frame.parentNode.removeChild(this.frame);// Remove element from DOM
	}}]);return Popup;}();/**
	 * ClusterItem
	 */var ClusterItem=/*#__PURE__*/function(_Item5){inherits(ClusterItem,_Item5);var _super10=_createSuper$2(ClusterItem);/**
	 * @constructor Item
	 * @param {Object} data             Object containing (optional) parameters type,
	 *                                  start, end, content, group, className.
	 * @param {{toScreen: function, toTime: function}} conversion
	 *                                  Conversion functions from time to screen and vice versa
	 * @param {Object} options          Configuration options
	 *                                  // TODO: describe available options
	 */function ClusterItem(data,conversion,options){var _this21;classCallCheck(this,ClusterItem);var modifiedOptions=assign$2({},{fitOnDoubleClick:true},options,{editable:false});_this21=_super10.call(this,data,conversion,modifiedOptions);_this21.props={content:{width:0,height:0}};if(!data||data.uiItems==undefined){throw new Error('Property "uiItems" missing in item '+data.id);}_this21.id=v4();_this21.group=data.group;_this21._setupRange();_this21.emitter=_this21.data.eventEmitter;_this21.range=_this21.data.range;_this21.attached=false;_this21.isCluster=true;_this21.data.isCluster=true;return _this21;}/**
	   * check if there are items
	   * @return {boolean}
	   */createClass(ClusterItem,[{key:"hasItems",value:function hasItems(){return this.data.uiItems&&this.data.uiItems.length&&this.attached;}/**
	   * set UI items
	   * @param {array} items
	   */},{key:"setUiItems",value:function setUiItems(items){this.detach();this.data.uiItems=items;this._setupRange();this.attach();}/**
	   * check is visible
	   * @param {object} range
	   * @return {boolean}
	   */},{key:"isVisible",value:function isVisible(range){var rangeWidth=this.data.end?this.data.end-this.data.start:0;var widthInMs=this.width*range.getMillisecondsPerPixel();var end=Math.max(rangeWidth,this.data.start.getTime()+widthInMs);return this.data.start<range.end&&end>range.start&&this.hasItems();}/**
	   * get cluster data
	   * @return {object}
	   */},{key:"getData",value:function getData(){return {isCluster:true,id:this.id,items:this.data.items||[],data:this.data};}/**
	   * redraw cluster item
	   * @param {boolean} returnQueue
	   * @return {boolean}
	   */},{key:"redraw",value:function redraw(returnQueue){var _context120,_context121,_context122,_context123,_context124,_context126;var sizes;var queue=[// create item DOM
	bind$2(_context120=this._createDomElement).call(_context120,this),// append DOM to parent DOM
	bind$2(_context121=this._appendDomElement).call(_context121,this),// update dirty DOM
	bind$2(_context122=this._updateDirtyDomComponents).call(_context122,this),bind$2(_context123=function _context123(){if(this.dirty){sizes=this._getDomComponentsSizes();}}).call(_context123,this),bind$2(_context124=function _context124(){if(this.dirty){var _context125;bind$2(_context125=this._updateDomComponentsSizes).call(_context125,this)(sizes);}}).call(_context124,this),// repaint DOM additionals
	bind$2(_context126=this._repaintDomAdditionals).call(_context126,this)];if(returnQueue){return queue;}else {var result;forEach$3(queue).call(queue,function(fn){result=fn();});return result;}}/**
	   * show cluster item
	   */},{key:"show",value:function show(){if(!this.displayed){this.redraw();}}/**
	   * Hide the item from the DOM (when visible)
	   */},{key:"hide",value:function hide(){if(this.displayed){var dom=this.dom;if(dom.box.parentNode){dom.box.parentNode.removeChild(dom.box);}if(this.options.showStipes){if(dom.line.parentNode){dom.line.parentNode.removeChild(dom.line);}if(dom.dot.parentNode){dom.dot.parentNode.removeChild(dom.dot);}}this.displayed=false;}}/**
	   * reposition item x axis
	   */},{key:"repositionX",value:function repositionX(){var start=this.conversion.toScreen(this.data.start);var end=this.data.end?this.conversion.toScreen(this.data.end):0;if(end){this.repositionXWithRanges(start,end);}else {var align=this.data.align===undefined?this.options.align:this.data.align;this.repositionXWithoutRanges(start,align);}if(this.options.showStipes){this.dom.line.style.display=this._isStipeVisible()?'block':'none';this.dom.dot.style.display=this._isStipeVisible()?'block':'none';if(this._isStipeVisible()){this.repositionStype(start,end);}}}/**
	   * reposition item stype
	   * @param {date} start
	   * @param {date} end
	   */},{key:"repositionStype",value:function repositionStype(start,end){this.dom.line.style.display='block';this.dom.dot.style.display='block';var lineOffsetWidth=this.dom.line.offsetWidth;var dotOffsetWidth=this.dom.dot.offsetWidth;if(end){var lineOffset=lineOffsetWidth+start+(end-start)/2;var dotOffset=lineOffset-dotOffsetWidth/2;var lineOffsetDirection=this.options.rtl?lineOffset*-1:lineOffset;var dotOffsetDirection=this.options.rtl?dotOffset*-1:dotOffset;this.dom.line.style.transform="translateX(".concat(lineOffsetDirection,"px)");this.dom.dot.style.transform="translateX(".concat(dotOffsetDirection,"px)");}else {var _lineOffsetDirection=this.options.rtl?start*-1:start;var _dotOffsetDirection=this.options.rtl?(start-dotOffsetWidth/2)*-1:start-dotOffsetWidth/2;this.dom.line.style.transform="translateX(".concat(_lineOffsetDirection,"px)");this.dom.dot.style.transform="translateX(".concat(_dotOffsetDirection,"px)");}}/**
	   * reposition x without ranges
	   * @param {date} start
	   * @param {string} align
	   */},{key:"repositionXWithoutRanges",value:function repositionXWithoutRanges(start,align){// calculate left position of the box
	if(align=='right'){if(this.options.rtl){this.right=start-this.width;// reposition box, line, and dot
	this.dom.box.style.right=this.right+'px';}else {this.left=start-this.width;// reposition box, line, and dot
	this.dom.box.style.left=this.left+'px';}}else if(align=='left'){if(this.options.rtl){this.right=start;// reposition box, line, and dot
	this.dom.box.style.right=this.right+'px';}else {this.left=start;// reposition box, line, and dot
	this.dom.box.style.left=this.left+'px';}}else {// default or 'center'
	if(this.options.rtl){this.right=start-this.width/2;// reposition box, line, and dot
	this.dom.box.style.right=this.right+'px';}else {this.left=start-this.width/2;// reposition box, line, and dot
	this.dom.box.style.left=this.left+'px';}}}/**
	   * reposition x with ranges
	   * @param {date} start
	   * @param {date} end
	   */},{key:"repositionXWithRanges",value:function repositionXWithRanges(start,end){var boxWidth=Math.round(Math.max(end-start+0.5,1));if(this.options.rtl){this.right=start;}else {this.left=start;}this.width=Math.max(boxWidth,this.minWidth||0);if(this.options.rtl){this.dom.box.style.right=this.right+'px';}else {this.dom.box.style.left=this.left+'px';}this.dom.box.style.width=boxWidth+'px';}/**
	   * reposition item y axis
	   */},{key:"repositionY",value:function repositionY(){var orientation=this.options.orientation.item;var box=this.dom.box;if(orientation=='top'){box.style.top=(this.top||0)+'px';}else {// orientation 'bottom'
	box.style.top=(this.parent.height-this.top-this.height||0)+'px';}if(this.options.showStipes){if(orientation=='top'){this.dom.line.style.top='0';this.dom.line.style.height=this.parent.top+this.top+1+'px';this.dom.line.style.bottom='';}else {// orientation 'bottom'
	var itemSetHeight=this.parent.itemSet.props.height;var lineHeight=itemSetHeight-this.parent.top-this.parent.height+this.top;this.dom.line.style.top=itemSetHeight-lineHeight+'px';this.dom.line.style.bottom='0';}this.dom.dot.style.top=-this.dom.dot.offsetHeight/2+'px';}}/**
	   * get width left
	   * @return {number}
	   */},{key:"getWidthLeft",value:function getWidthLeft(){return this.width/2;}/**
	   * get width right
	   * @return {number}
	   */},{key:"getWidthRight",value:function getWidthRight(){return this.width/2;}/**
	   * move cluster item
	   */},{key:"move",value:function move(){this.repositionX();this.repositionY();}/**
	   * attach
	   */},{key:"attach",value:function attach(){var _context127;var _iterator2=_createForOfIteratorHelper$3(this.data.uiItems),_step3;try{for(_iterator2.s();!(_step3=_iterator2.n()).done;){var _item8=_step3.value;_item8.cluster=this;}}catch(err){_iterator2.e(err);}finally{_iterator2.f();}this.data.items=map$5(_context127=this.data.uiItems).call(_context127,function(item){return item.data;});this.attached=true;this.dirty=true;}/**
	   * detach
	   * @param {boolean} detachFromParent
	   * @return {void}
	   */},{key:"detach",value:function detach(){var detachFromParent=arguments.length>0&&arguments[0]!==undefined?arguments[0]:false;if(!this.hasItems()){return;}var _iterator3=_createForOfIteratorHelper$3(this.data.uiItems),_step4;try{for(_iterator3.s();!(_step4=_iterator3.n()).done;){var _item9=_step4.value;delete _item9.cluster;}}catch(err){_iterator3.e(err);}finally{_iterator3.f();}this.attached=false;if(detachFromParent&&this.group){this.group.remove(this);this.group=null;}this.data.items=[];this.dirty=true;}/**
	   * handle on double click
	   */},{key:"_onDoubleClick",value:function _onDoubleClick(){this._fit();}/**
	   * set range
	   */},{key:"_setupRange",value:function _setupRange(){var _context128,_context129,_context130;var stats=map$5(_context128=this.data.uiItems).call(_context128,function(item){return {start:item.data.start.valueOf(),end:item.data.end?item.data.end.valueOf():item.data.start.valueOf()};});this.data.min=Math.min.apply(Math,toConsumableArray(map$5(stats).call(stats,function(s){return Math.min(s.start,s.end||s.start);})));this.data.max=Math.max.apply(Math,toConsumableArray(map$5(stats).call(stats,function(s){return Math.max(s.start,s.end||s.start);})));var centers=map$5(_context129=this.data.uiItems).call(_context129,function(item){return item.center;});var avg=reduce$2(centers).call(centers,function(sum,value){return sum+value;},0)/this.data.uiItems.length;if(some$2(_context130=this.data.uiItems).call(_context130,function(item){return item.data.end;})){// contains ranges
	this.data.start=new Date(this.data.min);this.data.end=new Date(this.data.max);}else {this.data.start=new Date(avg);this.data.end=null;}}/**
	   * get UI items
	   * @return {array}
	   */},{key:"_getUiItems",value:function _getUiItems(){var _this22=this;if(this.data.uiItems&&this.data.uiItems.length){var _context131;return filter$2(_context131=this.data.uiItems).call(_context131,function(item){return item.cluster===_this22;});}return [];}/**
	   * create DOM element
	   */},{key:"_createDomElement",value:function _createDomElement(){if(!this.dom){// create DOM
	this.dom={};// create main box
	this.dom.box=document.createElement('DIV');// contents box (inside the background box). used for making margins
	this.dom.content=document.createElement('DIV');this.dom.content.className='vis-item-content';this.dom.box.appendChild(this.dom.content);if(this.options.showStipes){// line to axis
	this.dom.line=document.createElement('DIV');this.dom.line.className='vis-cluster-line';this.dom.line.style.display='none';// dot on axis
	this.dom.dot=document.createElement('DIV');this.dom.dot.className='vis-cluster-dot';this.dom.dot.style.display='none';}if(this.options.fitOnDoubleClick){var _context132;this.dom.box.ondblclick=bind$2(_context132=ClusterItem.prototype._onDoubleClick).call(_context132,this);}// attach this item as attribute
	this.dom.box['vis-item']=this;this.dirty=true;}}/**
	   * append element to DOM
	   */},{key:"_appendDomElement",value:function _appendDomElement(){if(!this.parent){throw new Error('Cannot redraw item: no parent attached');}if(!this.dom.box.parentNode){var foreground=this.parent.dom.foreground;if(!foreground){throw new Error('Cannot redraw item: parent has no foreground container element');}foreground.appendChild(this.dom.box);}var background=this.parent.dom.background;if(this.options.showStipes){if(!this.dom.line.parentNode){if(!background)throw new Error('Cannot redraw item: parent has no background container element');background.appendChild(this.dom.line);}if(!this.dom.dot.parentNode){var axis=this.parent.dom.axis;if(!background)throw new Error('Cannot redraw item: parent has no axis container element');axis.appendChild(this.dom.dot);}}this.displayed=true;}/**
	   * update dirty DOM components
	   */},{key:"_updateDirtyDomComponents",value:function _updateDirtyDomComponents(){// An item is marked dirty when:
	// - the item is not yet rendered
	// - the item's data is changed
	// - the item is selected/deselected
	if(this.dirty){this._updateContents(this.dom.content);this._updateDataAttributes(this.dom.box);this._updateStyle(this.dom.box);// update class
	var className=this.baseClassName+' '+(this.data.className?' '+this.data.className:'')+(this.selected?' vis-selected':'')+' vis-readonly';this.dom.box.className='vis-item '+className;if(this.options.showStipes){this.dom.line.className='vis-item vis-cluster-line '+(this.selected?' vis-selected':'');this.dom.dot.className='vis-item vis-cluster-dot '+(this.selected?' vis-selected':'');}if(this.data.end){// turn off max-width to be able to calculate the real width
	// this causes an extra browser repaint/reflow, but so be it
	this.dom.content.style.maxWidth='none';}}}/**
	   * get DOM components sizes
	   * @return {object}
	   */},{key:"_getDomComponentsSizes",value:function _getDomComponentsSizes(){var sizes={previous:{right:this.dom.box.style.right,left:this.dom.box.style.left},box:{width:this.dom.box.offsetWidth,height:this.dom.box.offsetHeight}};if(this.options.showStipes){sizes.dot={height:this.dom.dot.offsetHeight,width:this.dom.dot.offsetWidth};sizes.line={width:this.dom.line.offsetWidth};}return sizes;}/**
	   * update DOM components sizes
	   * @param {object} sizes
	   */},{key:"_updateDomComponentsSizes",value:function _updateDomComponentsSizes(sizes){if(this.options.rtl){this.dom.box.style.right="0px";}else {this.dom.box.style.left="0px";}// recalculate size
	if(!this.data.end){this.width=sizes.box.width;}else {this.minWidth=sizes.box.width;}this.height=sizes.box.height;// restore previous position
	if(this.options.rtl){this.dom.box.style.right=sizes.previous.right;}else {this.dom.box.style.left=sizes.previous.left;}this.dirty=false;}/**
	   * repaint DOM additional components
	   */},{key:"_repaintDomAdditionals",value:function _repaintDomAdditionals(){this._repaintOnItemUpdateTimeTooltip(this.dom.box);}/**
	   * check is stripe visible
	   * @return {number}
	   * @private
	   */},{key:"_isStipeVisible",value:function _isStipeVisible(){return this.minWidth>=this.width||!this.data.end;}/**
	   * get fit range
	   * @return {object}
	   * @private
	   */},{key:"_getFitRange",value:function _getFitRange(){var offset=0.05*(this.data.max-this.data.min)/2;return {fitStart:this.data.min-offset,fitEnd:this.data.max+offset};}/**
	   * fit
	   * @private
	   */},{key:"_fit",value:function _fit(){if(this.emitter){var _this$_getFitRange=this._getFitRange(),fitStart=_this$_getFitRange.fitStart,fitEnd=_this$_getFitRange.fitEnd;var fitArgs={start:new Date(fitStart),end:new Date(fitEnd),animation:true};this.emitter.emit('fit',fitArgs);}}/**
	   * get item data
	   * @return {object}
	   * @private
	   */},{key:"_getItemData",value:function _getItemData(){return this.data;}}]);return ClusterItem;}(Item);ClusterItem.prototype.baseClassName='vis-item vis-range vis-cluster';var UNGROUPED$1='__ungrouped__';// reserved group id for ungrouped items
	var BACKGROUND$1='__background__';// reserved group id for background items without group
	var ReservedGroupIds$1={UNGROUPED:UNGROUPED$1,BACKGROUND:BACKGROUND$1};/**
	 * An Cluster generator generates cluster items
	 */var ClusterGenerator=/*#__PURE__*/function(){/**
	     * @param {ItemSet} itemSet itemsSet instance
	     * @constructor ClusterGenerator
	    */function ClusterGenerator(itemSet){classCallCheck(this,ClusterGenerator);this.itemSet=itemSet;this.groups={};this.cache={};this.cache[-1]=[];}/**
	     * @param {Object} itemData             Object containing parameters start content, className.
	     * @param {{toScreen: function, toTime: function}} conversion
	     *                                  Conversion functions from time to screen and vice versa
	     * @param {Object} [options]        Configuration options
	     * @return {Object} newItem
	    */createClass(ClusterGenerator,[{key:"createClusterItem",value:function createClusterItem(itemData,conversion,options){var newItem=new ClusterItem(itemData,conversion,options);return newItem;}/**
	     * Set the items to be clustered.
	     * This will clear cached clusters.
	     * @param {Item[]} items
	     * @param {Object} [options]  Available options:
	     *                            {boolean} applyOnChangedLevel
	     *                                If true (default), the changed data is applied
	     *                                as soon the cluster level changes. If false,
	     *                                The changed data is applied immediately
	     */},{key:"setItems",value:function setItems(items,options){this.items=items||[];this.dataChanged=true;this.applyOnChangedLevel=false;if(options&&options.applyOnChangedLevel){this.applyOnChangedLevel=options.applyOnChangedLevel;}}/**
	     * Update the current data set: clear cache, and recalculate the clustering for
	     * the current level
	     */},{key:"updateData",value:function updateData(){this.dataChanged=true;this.applyOnChangedLevel=false;}/**
	     * Cluster the items which are too close together
	     * @param {array} oldClusters 
	     * @param {number} scale      The scale of the current window : (windowWidth / (endDate - startDate)) 
	     * @param {{maxItems: number, clusterCriteria: function, titleTemplate: string}} options             
	     * @return {array} clusters
	    */},{key:"getClusters",value:function getClusters(oldClusters,scale,options){var _ref=typeof options==="boolean"?{}:options,maxItems=_ref.maxItems,clusterCriteria=_ref.clusterCriteria;if(!clusterCriteria){clusterCriteria=function clusterCriteria(){return true;};}maxItems=maxItems||1;var level=-1;var granularity=2;var timeWindow=0;if(scale>0){if(scale>=1){return [];}level=Math.abs(Math.round(Math.log(100/scale)/Math.log(granularity)));timeWindow=Math.abs(Math.pow(granularity,level));}// clear the cache when and re-generate groups the data when needed.
	if(this.dataChanged){var levelChanged=level!=this.cacheLevel;var applyDataNow=this.applyOnChangedLevel?levelChanged:true;if(applyDataNow){this._dropLevelsCache();this._filterData();}}this.cacheLevel=level;var clusters=this.cache[level];if(!clusters){clusters=[];for(var groupName in this.groups){if(this.groups.hasOwnProperty(groupName)){var items=this.groups[groupName];var iMax=items.length;var i=0;while(i<iMax){// find all items around current item, within the timeWindow
	var _item10=items[i];var neighbors=1;// start at 1, to include itself)
	// loop through items left from the current item
	var j=i-1;while(j>=0&&_item10.center-items[j].center<timeWindow/2){if(!items[j].cluster&&clusterCriteria(_item10.data,items[j].data)){neighbors++;}j--;}// loop through items right from the current item
	var k=i+1;while(k<items.length&&items[k].center-_item10.center<timeWindow/2){if(clusterCriteria(_item10.data,items[k].data)){neighbors++;}k++;}// loop through the created clusters
	var l=clusters.length-1;while(l>=0&&_item10.center-clusters[l].center<timeWindow){if(_item10.group==clusters[l].group&&clusterCriteria(_item10.data,clusters[l].data)){neighbors++;}l--;}// aggregate until the number of items is within maxItems
	if(neighbors>maxItems){// too busy in this window.
	var num=neighbors-maxItems+1;var clusterItems=[];// append the items to the cluster,
	// and calculate the average start for the cluster
	var m=i;while(clusterItems.length<num&&m<items.length){if(clusterCriteria(items[m].data,items[m].data)){clusterItems.push(items[m]);}m++;}var groupId=this.itemSet.getGroupId(_item10.data);var group=this.itemSet.groups[groupId]||this.itemSet.groups[ReservedGroupIds$1.UNGROUPED];var cluster=this._getClusterForItems(clusterItems,group,oldClusters,options);clusters.push(cluster);i+=num;}else {delete _item10.cluster;i+=1;}}}}this.cache[level]=clusters;}return clusters;}/**
	     * Filter the items per group.
	     * @private
	     */},{key:"_filterData",value:function _filterData(){// filter per group
	var groups={};this.groups=groups;// split the items per group
	for(var _i10=0,_Object$values=values$5(this.items);_i10<_Object$values.length;_i10++){var _item11=_Object$values[_i10];// put the item in the correct group
	var groupName=_item11.parent?_item11.parent.groupId:'';var group=groups[groupName];if(!group){group=[];groups[groupName]=group;}group.push(_item11);// calculate the center of the item
	if(_item11.data.start){if(_item11.data.end){// range
	_item11.center=(_item11.data.start.valueOf()+_item11.data.end.valueOf())/2;}else {// box, dot
	_item11.center=_item11.data.start.valueOf();}}}// sort the items per group
	for(var currentGroupName in groups){if(groups.hasOwnProperty(currentGroupName)){var _context133;sort$2(_context133=groups[currentGroupName]).call(_context133,function(a,b){return a.center-b.center;});}}this.dataChanged=false;}/**
	     * Create new cluster or return existing
	     * @private
	     * @param {array} clusterItems    
	     * @param {object} group 
	     * @param {array} oldClusters 
	     * @param {object} options 
	     * @returns {object} cluster
	     */},{key:"_getClusterForItems",value:function _getClusterForItems(clusterItems,group,oldClusters,options){var _context134;var oldClustersLookup=map$5(_context134=oldClusters||[]).call(_context134,function(cluster){var _context135;return {cluster:cluster,itemsIds:new set$3(map$5(_context135=cluster.data.uiItems).call(_context135,function(item){return item.id;}))};});var cluster;if(oldClustersLookup.length){var _iterator4=_createForOfIteratorHelper$3(oldClustersLookup),_step5;try{var _loop5=function _loop5(){var oldClusterData=_step5.value;if(oldClusterData.itemsIds.size===clusterItems.length&&every$2(clusterItems).call(clusterItems,function(clusterItem){return oldClusterData.itemsIds.has(clusterItem.id);})){cluster=oldClusterData.cluster;return "break";}};for(_iterator4.s();!(_step5=_iterator4.n()).done;){var _ret=_loop5();if(_ret==="break")break;}}catch(err){_iterator4.e(err);}finally{_iterator4.f();}}if(cluster){cluster.setUiItems(clusterItems);if(cluster.group!==group){if(cluster.group){cluster.group.remove(cluster);}if(group){group.add(cluster);cluster.group=group;}}return cluster;}var titleTemplate=options.titleTemplate||'';var conversion={toScreen:this.itemSet.body.util.toScreen,toTime:this.itemSet.body.util.toTime};var title=titleTemplate.replace(/{count}/,clusterItems.length);var clusterContent='<div title="'+title+'">'+clusterItems.length+'</div>';var clusterOptions=assign$2({},options,this.itemSet.options);var data={'content':clusterContent,'title':title,'group':group,'uiItems':clusterItems,'eventEmitter':this.itemSet.body.emitter,'range':this.itemSet.body.range};cluster=this.createClusterItem(data,conversion,clusterOptions);if(group){group.add(cluster);cluster.group=group;}cluster.attach();return cluster;}/**
	     * Drop cache
	     * @private
	     */},{key:"_dropLevelsCache",value:function _dropLevelsCache(){this.cache={};this.cacheLevel=-1;this.cache[this.cacheLevel]=[];}}]);return ClusterGenerator;}();var UNGROUPED$2='__ungrouped__';// reserved group id for ungrouped items
	var BACKGROUND$2='__background__';// reserved group id for background items without group
	/**
	 * An ItemSet holds a set of items and ranges which can be displayed in a
	 * range. The width is determined by the parent of the ItemSet, and the height
	 * is determined by the size of the items.
	 */var ItemSet=/*#__PURE__*/function(_Component5){inherits(ItemSet,_Component5);var _super11=_createSuper$2(ItemSet);/**
	 * @param {{dom: Object, domProps: Object, emitter: Emitter, range: Range}} body
	 * @param {Object} [options]      See ItemSet.setOptions for the available options.
	 * @constructor ItemSet
	 * @extends Component
	 */function ItemSet(body,options){var _this23;classCallCheck(this,ItemSet);_this23=_super11.call(this);_this23.body=body;_this23.defaultOptions={type:null,// 'box', 'point', 'range', 'background'
	orientation:{item:'bottom'// item orientation: 'top' or 'bottom'
	},align:'auto',// alignment of box items
	stack:true,stackSubgroups:true,groupOrderSwap:function groupOrderSwap(fromGroup,toGroup,groups){// eslint-disable-line no-unused-vars
	var targetOrder=toGroup.order;toGroup.order=fromGroup.order;fromGroup.order=targetOrder;},groupOrder:'order',selectable:true,multiselect:false,longSelectPressTime:251,itemsAlwaysDraggable:{item:false,range:false},editable:{updateTime:false,updateGroup:false,add:false,remove:false,overrideItems:false},groupEditable:{order:false,add:false,remove:false},snap:TimeStep.snap,// Only called when `objectData.target === 'item'.
	onDropObjectOnItem:function onDropObjectOnItem(objectData,item,callback){callback(item);},onAdd:function onAdd(item,callback){callback(item);},onUpdate:function onUpdate(item,callback){callback(item);},onMove:function onMove(item,callback){callback(item);},onRemove:function onRemove(item,callback){callback(item);},onMoving:function onMoving(item,callback){callback(item);},onAddGroup:function onAddGroup(item,callback){callback(item);},onMoveGroup:function onMoveGroup(item,callback){callback(item);},onRemoveGroup:function onRemoveGroup(item,callback){callback(item);},margin:{item:{horizontal:10,vertical:10},axis:20},showTooltips:true,tooltip:{followMouse:false,overflowMethod:'flip',delay:500},tooltipOnItemUpdateTime:false};// options is shared by this ItemSet and all its items
	_this23.options=util$1.extend({},_this23.defaultOptions);_this23.options.rtl=options.rtl;_this23.options.onTimeout=options.onTimeout;_this23.conversion={toScreen:body.util.toScreen,toTime:body.util.toTime};_this23.dom={};_this23.props={};_this23.hammer=null;var me=assertThisInitialized(_this23);_this23.itemsData=null;// DataSet
	_this23.groupsData=null;// DataSet
	_this23.itemsSettingTime=null;_this23.initialItemSetDrawn=false;_this23.userContinueNotBail=null;_this23.sequentialSelection=false;// listeners for the DataSet of the items
	_this23.itemListeners={'add':function add(event,params,senderId){// eslint-disable-line no-unused-vars
	me._onAdd(params.items);if(me.options.cluster){me.clusterGenerator.setItems(me.items,{applyOnChangedLevel:false});}me.redraw();},'update':function update(event,params,senderId){// eslint-disable-line no-unused-vars
	me._onUpdate(params.items);if(me.options.cluster){me.clusterGenerator.setItems(me.items,{applyOnChangedLevel:false});}me.redraw();},'remove':function remove(event,params,senderId){// eslint-disable-line no-unused-vars
	me._onRemove(params.items);if(me.options.cluster){me.clusterGenerator.setItems(me.items,{applyOnChangedLevel:false});}me.redraw();}};// listeners for the DataSet of the groups
	_this23.groupListeners={'add':function add(event,params,senderId){// eslint-disable-line no-unused-vars
	me._onAddGroups(params.items);if(me.groupsData&&me.groupsData.length>0){var _context136;var groupsData=me.groupsData.getDataSet();forEach$3(_context136=groupsData.get()).call(_context136,function(groupData){if(groupData.nestedGroups){var _context137;if(groupData.showNested!=false){groupData.showNested=true;}var updatedGroups=[];forEach$3(_context137=groupData.nestedGroups).call(_context137,function(nestedGroupId){var updatedNestedGroup=groupsData.get(nestedGroupId);if(!updatedNestedGroup){return;}updatedNestedGroup.nestedInGroup=groupData.id;if(groupData.showNested==false){updatedNestedGroup.visible=false;}updatedGroups=concat$2(updatedGroups).call(updatedGroups,updatedNestedGroup);});groupsData.update(updatedGroups,senderId);}});}},'update':function update(event,params,senderId){// eslint-disable-line no-unused-vars
	me._onUpdateGroups(params.items);},'remove':function remove(event,params,senderId){// eslint-disable-line no-unused-vars
	me._onRemoveGroups(params.items);}};_this23.items={};// object with an Item for every data item
	_this23.groups={};// Group object for every group
	_this23.groupIds=[];_this23.selection=[];// list with the ids of all selected nodes
	_this23.popup=null;_this23.popupTimer=null;_this23.touchParams={};// stores properties while dragging
	_this23.groupTouchParams={group:null,isDragging:false};// create the HTML DOM
	_this23._create();_this23.setOptions(options);_this23.clusters=[];return _this23;}/**
	   * Create the HTML DOM for the ItemSet
	   */createClass(ItemSet,[{key:"_create",value:function _create(){var _this24=this,_context138,_context139,_context140,_context141,_context142,_context143,_context144,_context145,_context146,_context147,_context148,_context149,_context150,_context151,_context152;var frame=document.createElement('div');frame.className='vis-itemset';frame['vis-itemset']=this;this.dom.frame=frame;// create background panel
	var background=document.createElement('div');background.className='vis-background';frame.appendChild(background);this.dom.background=background;// create foreground panel
	var foreground=document.createElement('div');foreground.className='vis-foreground';frame.appendChild(foreground);this.dom.foreground=foreground;// create axis panel
	var axis=document.createElement('div');axis.className='vis-axis';this.dom.axis=axis;// create labelset
	var labelSet=document.createElement('div');labelSet.className='vis-labelset';this.dom.labelSet=labelSet;// create ungrouped Group
	this._updateUngrouped();// create background Group
	var backgroundGroup=new BackgroundGroup(BACKGROUND$2,null,this);backgroundGroup.show();this.groups[BACKGROUND$2]=backgroundGroup;// attach event listeners
	// Note: we bind to the centerContainer for the case where the height
	//       of the center container is larger than of the ItemSet, so we
	//       can click in the empty area to create a new item or deselect an item.
	this.hammer=new Hammer$1(this.body.dom.centerContainer);// drag items when selected
	this.hammer.on('hammer.input',function(event){if(event.isFirst){_this24._onTouch(event);}});this.hammer.on('panstart',bind$2(_context138=this._onDragStart).call(_context138,this));this.hammer.on('panmove',bind$2(_context139=this._onDrag).call(_context139,this));this.hammer.on('panend',bind$2(_context140=this._onDragEnd).call(_context140,this));this.hammer.get('pan').set({threshold:5,direction:Hammer$1.ALL});// single select (or unselect) when tapping an item
	this.hammer.on('tap',bind$2(_context141=this._onSelectItem).call(_context141,this));// multi select when holding mouse/touch, or on ctrl+click
	this.hammer.on('press',bind$2(_context142=this._onMultiSelectItem).call(_context142,this));// add item on doubletap
	this.hammer.on('doubletap',bind$2(_context143=this._onAddItem).call(_context143,this));if(this.options.rtl){this.groupHammer=new Hammer$1(this.body.dom.rightContainer);}else {this.groupHammer=new Hammer$1(this.body.dom.leftContainer);}this.groupHammer.on('tap',bind$2(_context144=this._onGroupClick).call(_context144,this));this.groupHammer.on('panstart',bind$2(_context145=this._onGroupDragStart).call(_context145,this));this.groupHammer.on('panmove',bind$2(_context146=this._onGroupDrag).call(_context146,this));this.groupHammer.on('panend',bind$2(_context147=this._onGroupDragEnd).call(_context147,this));this.groupHammer.get('pan').set({threshold:5,direction:Hammer$1.DIRECTION_VERTICAL});this.body.dom.centerContainer.addEventListener('mouseover',bind$2(_context148=this._onMouseOver).call(_context148,this));this.body.dom.centerContainer.addEventListener('mouseout',bind$2(_context149=this._onMouseOut).call(_context149,this));this.body.dom.centerContainer.addEventListener('mousemove',bind$2(_context150=this._onMouseMove).call(_context150,this));// right-click on timeline 
	this.body.dom.centerContainer.addEventListener('contextmenu',bind$2(_context151=this._onDragEnd).call(_context151,this));this.body.dom.centerContainer.addEventListener('mousewheel',bind$2(_context152=this._onMouseWheel).call(_context152,this));// attach to the DOM
	this.show();}/**
	   * Set options for the ItemSet. Existing options will be extended/overwritten.
	   * @param {Object} [options] The following options are available:
	   *                           {string} type
	   *                              Default type for the items. Choose from 'box'
	   *                              (default), 'point', 'range', or 'background'.
	   *                              The default style can be overwritten by
	   *                              individual items.
	   *                           {string} align
	   *                              Alignment for the items, only applicable for
	   *                              BoxItem. Choose 'center' (default), 'left', or
	   *                              'right'.
	   *                           {string} orientation.item
	   *                              Orientation of the item set. Choose 'top' or
	   *                              'bottom' (default).
	   *                           {Function} groupOrder
	   *                              A sorting function for ordering groups
	   *                           {boolean} stack
	   *                              If true (default), items will be stacked on
	   *                              top of each other.
	   *                           {number} margin.axis
	   *                              Margin between the axis and the items in pixels.
	   *                              Default is 20.
	   *                           {number} margin.item.horizontal
	   *                              Horizontal margin between items in pixels.
	   *                              Default is 10.
	   *                           {number} margin.item.vertical
	   *                              Vertical Margin between items in pixels.
	   *                              Default is 10.
	   *                           {number} margin.item
	   *                              Margin between items in pixels in both horizontal
	   *                              and vertical direction. Default is 10.
	   *                           {number} margin
	   *                              Set margin for both axis and items in pixels.
	   *                           {boolean} selectable
	   *                              If true (default), items can be selected.
	   *                           {boolean} multiselect
	   *                              If true, multiple items can be selected.
	   *                              False by default.
	   *                           {boolean} editable
	   *                              Set all editable options to true or false
	   *                           {boolean} editable.updateTime
	   *                              Allow dragging an item to an other moment in time
	   *                           {boolean} editable.updateGroup
	   *                              Allow dragging an item to an other group
	   *                           {boolean} editable.add
	   *                              Allow creating new items on double tap
	   *                           {boolean} editable.remove
	   *                              Allow removing items by clicking the delete button
	   *                              top right of a selected item.
	   *                           {Function(item: Item, callback: Function)} onAdd
	   *                              Callback function triggered when an item is about to be added:
	   *                              when the user double taps an empty space in the Timeline.
	   *                           {Function(item: Item, callback: Function)} onUpdate
	   *                              Callback function fired when an item is about to be updated.
	   *                              This function typically has to show a dialog where the user
	   *                              change the item. If not implemented, nothing happens.
	   *                           {Function(item: Item, callback: Function)} onMove
	   *                              Fired when an item has been moved. If not implemented,
	   *                              the move action will be accepted.
	   *                           {Function(item: Item, callback: Function)} onRemove
	   *                              Fired when an item is about to be deleted.
	   *                              If not implemented, the item will be always removed.
	   */},{key:"setOptions",value:function setOptions(options){var _this25=this;if(options){var _context153,_context155;// copy all options that we know
	var fields=['type','rtl','align','order','stack','stackSubgroups','selectable','multiselect','sequentialSelection','multiselectPerGroup','longSelectPressTime','groupOrder','dataAttributes','template','groupTemplate','visibleFrameTemplate','hide','snap','groupOrderSwap','showTooltips','tooltip','tooltipOnItemUpdateTime','groupHeightMode','onTimeout'];util$1.selectiveExtend(fields,this.options,options);if('itemsAlwaysDraggable'in options){if(typeof options.itemsAlwaysDraggable==='boolean'){this.options.itemsAlwaysDraggable.item=options.itemsAlwaysDraggable;this.options.itemsAlwaysDraggable.range=false;}else if(_typeof_1(options.itemsAlwaysDraggable)==='object'){util$1.selectiveExtend(['item','range'],this.options.itemsAlwaysDraggable,options.itemsAlwaysDraggable);// only allow range always draggable when item is always draggable as well
	if(!this.options.itemsAlwaysDraggable.item){this.options.itemsAlwaysDraggable.range=false;}}}if('sequentialSelection'in options){if(typeof options.sequentialSelection==='boolean'){this.options.sequentialSelection=options.sequentialSelection;}}if('orientation'in options){if(typeof options.orientation==='string'){this.options.orientation.item=options.orientation==='top'?'top':'bottom';}else if(_typeof_1(options.orientation)==='object'&&'item'in options.orientation){this.options.orientation.item=options.orientation.item;}}if('margin'in options){if(typeof options.margin==='number'){this.options.margin.axis=options.margin;this.options.margin.item.horizontal=options.margin;this.options.margin.item.vertical=options.margin;}else if(_typeof_1(options.margin)==='object'){util$1.selectiveExtend(['axis'],this.options.margin,options.margin);if('item'in options.margin){if(typeof options.margin.item==='number'){this.options.margin.item.horizontal=options.margin.item;this.options.margin.item.vertical=options.margin.item;}else if(_typeof_1(options.margin.item)==='object'){util$1.selectiveExtend(['horizontal','vertical'],this.options.margin.item,options.margin.item);}}}}forEach$3(_context153=['locale','locales']).call(_context153,function(key){if(key in options){_this25.options[key]=options[key];}});if('editable'in options){if(typeof options.editable==='boolean'){this.options.editable.updateTime=options.editable;this.options.editable.updateGroup=options.editable;this.options.editable.add=options.editable;this.options.editable.remove=options.editable;this.options.editable.overrideItems=false;}else if(_typeof_1(options.editable)==='object'){util$1.selectiveExtend(['updateTime','updateGroup','add','remove','overrideItems'],this.options.editable,options.editable);}}if('groupEditable'in options){if(typeof options.groupEditable==='boolean'){this.options.groupEditable.order=options.groupEditable;this.options.groupEditable.add=options.groupEditable;this.options.groupEditable.remove=options.groupEditable;}else if(_typeof_1(options.groupEditable)==='object'){util$1.selectiveExtend(['order','add','remove'],this.options.groupEditable,options.groupEditable);}}// callback functions
	var addCallback=function addCallback(name){var fn=options[name];if(fn){if(!(typeof fn==='function')){var _context154;throw new Error(concat$2(_context154="option ".concat(name," must be a function ")).call(_context154,name,"(item, callback)"));}_this25.options[name]=fn;}};forEach$3(_context155=['onDropObjectOnItem','onAdd','onUpdate','onRemove','onMove','onMoving','onAddGroup','onMoveGroup','onRemoveGroup']).call(_context155,addCallback);if(options.cluster){assign$2(this.options,{cluster:options.cluster});if(!this.clusterGenerator){this.clusterGenerator=new ClusterGenerator(this);}this.clusterGenerator.setItems(this.items,{applyOnChangedLevel:false});this.markDirty({refreshItems:true,restackGroups:true});this.redraw();}else if(this.clusterGenerator){this._detachAllClusters();this.clusters=[];this.clusterGenerator=null;this.options.cluster=undefined;this.markDirty({refreshItems:true,restackGroups:true});this.redraw();}else {// force the itemSet to refresh: options like orientation and margins may be changed
	this.markDirty();}}}/**
	   * Mark the ItemSet dirty so it will refresh everything with next redraw.
	   * Optionally, all items can be marked as dirty and be refreshed.
	   * @param {{refreshItems: boolean}} [options]
	   */},{key:"markDirty",value:function markDirty(options){this.groupIds=[];if(options){if(options.refreshItems){forEach$3(util$1).call(util$1,this.items,function(item){item.dirty=true;if(item.displayed)item.redraw();});}if(options.restackGroups){forEach$3(util$1).call(util$1,this.groups,function(group,key){if(key===BACKGROUND$2)return;group.stackDirty=true;});}}}/**
	   * Destroy the ItemSet
	   */},{key:"destroy",value:function destroy(){this.clearPopupTimer();this.hide();this.setItems(null);this.setGroups(null);this.hammer&&this.hammer.destroy();this.groupHammer&&this.groupHammer.destroy();this.hammer=null;this.body=null;this.conversion=null;}/**
	   * Hide the component from the DOM
	   */},{key:"hide",value:function hide(){// remove the frame containing the items
	if(this.dom.frame.parentNode){this.dom.frame.parentNode.removeChild(this.dom.frame);}// remove the axis with dots
	if(this.dom.axis.parentNode){this.dom.axis.parentNode.removeChild(this.dom.axis);}// remove the labelset containing all group labels
	if(this.dom.labelSet.parentNode){this.dom.labelSet.parentNode.removeChild(this.dom.labelSet);}}/**
	   * Show the component in the DOM (when not already visible).
	   */},{key:"show",value:function show(){// show frame containing the items
	if(!this.dom.frame.parentNode){this.body.dom.center.appendChild(this.dom.frame);}// show axis with dots
	if(!this.dom.axis.parentNode){this.body.dom.backgroundVertical.appendChild(this.dom.axis);}// show labelset containing labels
	if(!this.dom.labelSet.parentNode){if(this.options.rtl){this.body.dom.right.appendChild(this.dom.labelSet);}else {this.body.dom.left.appendChild(this.dom.labelSet);}}}/**
	   * Activates the popup timer to show the given popup after a fixed time.
	   * @param {Popup} popup
	   */},{key:"setPopupTimer",value:function setPopupTimer(popup){this.clearPopupTimer();if(popup){var delay=this.options.tooltip.delay||typeof this.options.tooltip.delay==='number'?this.options.tooltip.delay:500;this.popupTimer=setTimeout$2(function(){popup.show();},delay);}}/**
	   * Clears the popup timer for the tooltip.
	   */},{key:"clearPopupTimer",value:function clearPopupTimer(){if(this.popupTimer!=null){clearTimeout(this.popupTimer);this.popupTimer=null;}}/**
	   * Set selected items by their id. Replaces the current selection
	   * Unknown id's are silently ignored.
	   * @param {string[] | string} [ids] An array with zero or more id's of the items to be
	   *                                  selected, or a single item id. If ids is undefined
	   *                                  or an empty array, all items will be unselected.
	   */},{key:"setSelection",value:function setSelection(ids){var _context156;if(ids==undefined){ids=[];}if(!isArray$3(ids)){ids=[ids];}var idsToDeselect=filter$2(_context156=this.selection).call(_context156,function(id){return indexOf$3(ids).call(ids,id)===-1;});// unselect currently selected items
	var _iterator5=_createForOfIteratorHelper$3(idsToDeselect),_step6;try{for(_iterator5.s();!(_step6=_iterator5.n()).done;){var selectedId=_step6.value;var _item12=this.getItemById(selectedId);if(_item12){_item12.unselect();}}// select items
	}catch(err){_iterator5.e(err);}finally{_iterator5.f();}this.selection=toConsumableArray(ids);var _iterator6=_createForOfIteratorHelper$3(ids),_step7;try{for(_iterator6.s();!(_step7=_iterator6.n()).done;){var id=_step7.value;var _item13=this.getItemById(id);if(_item13){_item13.select();}}}catch(err){_iterator6.e(err);}finally{_iterator6.f();}}/**
	   * Get the selected items by their id
	   * @return {Array} ids  The ids of the selected items
	   */},{key:"getSelection",value:function getSelection(){var _context157;return concat$2(_context157=this.selection).call(_context157,[]);}/**
	   * Get the id's of the currently visible items.
	   * @returns {Array} The ids of the visible items
	   */},{key:"getVisibleItems",value:function getVisibleItems(){var range=this.body.range.getRange();var right;var left;if(this.options.rtl){right=this.body.util.toScreen(range.start);left=this.body.util.toScreen(range.end);}else {left=this.body.util.toScreen(range.start);right=this.body.util.toScreen(range.end);}var ids=[];for(var groupId in this.groups){if(this.groups.hasOwnProperty(groupId)){var group=this.groups[groupId];var rawVisibleItems=group.isVisible?group.visibleItems:[];// filter the "raw" set with visibleItems into a set which is really
	// visible by pixels
	var _iterator7=_createForOfIteratorHelper$3(rawVisibleItems),_step8;try{for(_iterator7.s();!(_step8=_iterator7.n()).done;){var _item14=_step8.value;// TODO: also check whether visible vertically
	if(this.options.rtl){if(_item14.right<left&&_item14.right+_item14.width>right){ids.push(_item14.id);}}else {if(_item14.left<right&&_item14.left+_item14.width>left){ids.push(_item14.id);}}}}catch(err){_iterator7.e(err);}finally{_iterator7.f();}}}return ids;}/**
	   * Get the id's of the currently visible groups.
	   * @returns {Array} The ids of the visible groups
	   */},{key:"getVisibleGroups",value:function getVisibleGroups(){var ids=[];for(var groupId in this.groups){if(this.groups.hasOwnProperty(groupId)){var group=this.groups[groupId];if(group.isVisible){ids.push(groupId);}}}return ids;}/**
	   * get item by id
	   * @param {string} id
	   * @return {object} item
	   */},{key:"getItemById",value:function getItemById(id){var _context158;return this.items[id]||find$2(_context158=this.clusters).call(_context158,function(cluster){return cluster.id===id;});}/**
	   * Deselect a selected item
	   * @param {string | number} id
	   * @private
	   */},{key:"_deselect",value:function _deselect(id){var selection=this.selection;for(var i=0,ii=selection.length;i<ii;i++){if(selection[i]==id){// non-strict comparison!
	splice$2(selection).call(selection,i,1);break;}}}/**
	   * Repaint the component
	   * @return {boolean} Returns true if the component is resized
	   */},{key:"redraw",value:function redraw(){var _this26=this;var margin=this.options.margin;var range=this.body.range;var asSize=util$1.option.asSize;var options=this.options;var orientation=options.orientation.item;var resized=false;var frame=this.dom.frame;// recalculate absolute position (before redrawing groups)
	this.props.top=this.body.domProps.top.height+this.body.domProps.border.top;if(this.options.rtl){this.props.right=this.body.domProps.right.width+this.body.domProps.border.right;}else {this.props.left=this.body.domProps.left.width+this.body.domProps.border.left;}// update class name
	frame.className='vis-itemset';if(this.options.cluster){this._clusterItems();}// reorder the groups (if needed)
	resized=this._orderGroups()||resized;// check whether zoomed (in that case we need to re-stack everything)
	// TODO: would be nicer to get this as a trigger from Range
	var visibleInterval=range.end-range.start;var zoomed=visibleInterval!=this.lastVisibleInterval||this.props.width!=this.props.lastWidth;var scrolled=range.start!=this.lastRangeStart;var changedStackOption=options.stack!=this.lastStack;var changedStackSubgroupsOption=options.stackSubgroups!=this.lastStackSubgroups;var forceRestack=zoomed||scrolled||changedStackOption||changedStackSubgroupsOption;this.lastVisibleInterval=visibleInterval;this.lastRangeStart=range.start;this.lastStack=options.stack;this.lastStackSubgroups=options.stackSubgroups;this.props.lastWidth=this.props.width;var firstGroup=this._firstGroup();var firstMargin={item:margin.item,axis:margin.axis};var nonFirstMargin={item:margin.item,axis:margin.item.vertical/2};var height=0;var minHeight=margin.axis+margin.item.vertical;// redraw the background group
	this.groups[BACKGROUND$2].redraw(range,nonFirstMargin,forceRestack);var redrawQueue={};var redrawQueueLength=0;// collect redraw functions
	forEach$3(util$1).call(util$1,this.groups,function(group,key){if(key===BACKGROUND$2)return;var groupMargin=group==firstGroup?firstMargin:nonFirstMargin;var returnQueue=true;redrawQueue[key]=group.redraw(range,groupMargin,forceRestack,returnQueue);redrawQueueLength=redrawQueue[key].length;});var needRedraw=redrawQueueLength>0;if(needRedraw){(function(){var redrawResults={};var _loop6=function _loop6(i){forEach$3(util$1).call(util$1,redrawQueue,function(fns,key){redrawResults[key]=fns[i]();});};for(var i=0;i<redrawQueueLength;i++){_loop6(i);}// redraw all regular groups
	forEach$3(util$1).call(util$1,_this26.groups,function(group,key){if(key===BACKGROUND$2)return;var groupResized=redrawResults[key];resized=groupResized||resized;height+=group.height;});height=Math.max(height,minHeight);})();}height=Math.max(height,minHeight);// update frame height
	frame.style.height=asSize(height);// calculate actual size
	this.props.width=frame.offsetWidth;this.props.height=height;// reposition axis
	this.dom.axis.style.top=asSize(orientation=='top'?this.body.domProps.top.height+this.body.domProps.border.top:this.body.domProps.top.height+this.body.domProps.centerContainer.height);if(this.options.rtl){this.dom.axis.style.right='0';}else {this.dom.axis.style.left='0';}this.hammer.get('press').set({time:this.options.longSelectPressTime});this.initialItemSetDrawn=true;// check if this component is resized
	resized=this._isResized()||resized;return resized;}/**
	   * Get the first group, aligned with the axis
	   * @return {Group | null} firstGroup
	   * @private
	   */},{key:"_firstGroup",value:function _firstGroup(){var firstGroupIndex=this.options.orientation.item=='top'?0:this.groupIds.length-1;var firstGroupId=this.groupIds[firstGroupIndex];var firstGroup=this.groups[firstGroupId]||this.groups[UNGROUPED$2];return firstGroup||null;}/**
	   * Create or delete the group holding all ungrouped items. This group is used when
	   * there are no groups specified.
	   * @protected
	   */},{key:"_updateUngrouped",value:function _updateUngrouped(){var ungrouped=this.groups[UNGROUPED$2];var item;var itemId;if(this.groupsData){// remove the group holding all ungrouped items
	if(ungrouped){ungrouped.dispose();delete this.groups[UNGROUPED$2];for(itemId in this.items){if(this.items.hasOwnProperty(itemId)){item=this.items[itemId];item.parent&&item.parent.remove(item);var groupId=this.getGroupId(item.data);var group=this.groups[groupId];group&&group.add(item)||item.hide();}}}}else {// create a group holding all (unfiltered) items
	if(!ungrouped){var id=null;var data=null;ungrouped=new Group(id,data,this);this.groups[UNGROUPED$2]=ungrouped;for(itemId in this.items){if(this.items.hasOwnProperty(itemId)){item=this.items[itemId];ungrouped.add(item);}}ungrouped.show();}}}/**
	   * Get the element for the labelset
	   * @return {HTMLElement} labelSet
	   */},{key:"getLabelSet",value:function getLabelSet(){return this.dom.labelSet;}/**
	   * Set items
	   * @param {vis.DataSet | null} items
	   */},{key:"setItems",value:function setItems(items){this.itemsSettingTime=new Date();var me=this;var ids;var oldItemsData=this.itemsData;// replace the dataset
	if(!items){this.itemsData=null;}else if(items instanceof DataSet||items instanceof DataView){this.itemsData=typeCoerceDataSet(items);}else {throw new TypeError('Data must be an instance of DataSet or DataView');}if(oldItemsData){// unsubscribe from old dataset
	forEach$3(util$1).call(util$1,this.itemListeners,function(callback,event){oldItemsData.off(event,callback);});// stop maintaining a coerced version of the old data set
	oldItemsData.dispose();// remove all drawn items
	ids=oldItemsData.getIds();this._onRemove(ids);}if(this.itemsData){// subscribe to new dataset
	var id=this.id;forEach$3(util$1).call(util$1,this.itemListeners,function(callback,event){me.itemsData.on(event,callback,id);});// add all new items
	ids=this.itemsData.getIds();this._onAdd(ids);// update the group holding all ungrouped items
	this._updateUngrouped();}this.body.emitter.emit('_change',{queue:true});}/**
	   * Get the current items
	   * @returns {vis.DataSet | null}
	   */},{key:"getItems",value:function getItems(){return this.itemsData!=null?this.itemsData.rawDS:null;}/**
	   * Set groups
	   * @param {vis.DataSet} groups
	   */},{key:"setGroups",value:function setGroups(groups){var me=this;var ids;// unsubscribe from current dataset
	if(this.groupsData){forEach$3(util$1).call(util$1,this.groupListeners,function(callback,event){me.groupsData.off(event,callback);});// remove all drawn groups
	ids=this.groupsData.getIds();this.groupsData=null;this._onRemoveGroups(ids);// note: this will cause a redraw
	}// replace the dataset
	if(!groups){this.groupsData=null;}else if(groups instanceof DataSet||groups instanceof DataView){this.groupsData=groups;}else {throw new TypeError('Data must be an instance of DataSet or DataView');}if(this.groupsData){var _context159;// go over all groups nesting
	var groupsData=this.groupsData;if(this.groupsData instanceof DataView){groupsData=this.groupsData.getDataSet();}forEach$3(_context159=groupsData.get()).call(_context159,function(group){if(group.nestedGroups){var _context160;forEach$3(_context160=group.nestedGroups).call(_context160,function(nestedGroupId){var updatedNestedGroup=groupsData.get(nestedGroupId);updatedNestedGroup.nestedInGroup=group.id;if(group.showNested==false){updatedNestedGroup.visible=false;}groupsData.update(updatedNestedGroup);});}});// subscribe to new dataset
	var id=this.id;forEach$3(util$1).call(util$1,this.groupListeners,function(callback,event){me.groupsData.on(event,callback,id);});// draw all ms
	ids=this.groupsData.getIds();this._onAddGroups(ids);}// update the group holding all ungrouped items
	this._updateUngrouped();// update the order of all items in each group
	this._order();if(this.options.cluster){this.clusterGenerator.updateData();this._clusterItems();this.markDirty({refreshItems:true,restackGroups:true});}this.body.emitter.emit('_change',{queue:true});}/**
	   * Get the current groups
	   * @returns {vis.DataSet | null} groups
	   */},{key:"getGroups",value:function getGroups(){return this.groupsData;}/**
	   * Remove an item by its id
	   * @param {string | number} id
	   */},{key:"removeItem",value:function removeItem(id){var _this27=this;var item=this.itemsData.get(id);if(item){// confirm deletion
	this.options.onRemove(item,function(item){if(item){// remove by id here, it is possible that an item has no id defined
	// itself, so better not delete by the item itself
	_this27.itemsData.remove(id);}});}}/**
	   * Get the time of an item based on it's data and options.type
	   * @param {Object} itemData
	   * @returns {string} Returns the type
	   * @private
	   */},{key:"_getType",value:function _getType(itemData){return itemData.type||this.options.type||(itemData.end?'range':'box');}/**
	   * Get the group id for an item
	   * @param {Object} itemData
	   * @returns {string} Returns the groupId
	   * @private
	   */},{key:"getGroupId",value:function getGroupId(itemData){var type=this._getType(itemData);if(type=='background'&&itemData.group==undefined){return BACKGROUND$2;}else {return this.groupsData?itemData.group:UNGROUPED$2;}}/**
	   * Handle updated items
	   * @param {number[]} ids
	   * @protected
	   */},{key:"_onUpdate",value:function _onUpdate(ids){var _this28=this;var me=this;forEach$3(ids).call(ids,function(id){var itemData=me.itemsData.get(id);var item=me.items[id];var type=itemData?me._getType(itemData):null;var constructor=ItemSet.types[type];var selected;if(item){// update item   	
	if(!constructor||!(item instanceof constructor)){// item type has changed, delete the item and recreate it
	selected=item.selected;// preserve selection of this item
	me._removeItem(item);item=null;}else {me._updateItem(item,itemData);}}if(!item&&itemData){// create item
	if(constructor){item=new constructor(itemData,me.conversion,me.options);item.id=id;// TODO: not so nice setting id afterwards
	me._addItem(item);if(selected){_this28.selection.push(id);item.select();}}else {throw new TypeError("Unknown item type \"".concat(type,"\""));}}});this._order();if(this.options.cluster){this.clusterGenerator.setItems(this.items,{applyOnChangedLevel:false});this._clusterItems();}this.body.emitter.emit('_change',{queue:true});}/**
	   * Handle removed items
	   * @param {number[]} ids
	   * @protected
	   */},{key:"_onRemove",value:function _onRemove(ids){var count=0;var me=this;forEach$3(ids).call(ids,function(id){var item=me.items[id];if(item){count++;me._removeItem(item);}});if(count){// update order
	this._order();this.body.emitter.emit('_change',{queue:true});}}/**
	   * Update the order of item in all groups
	   * @private
	   */},{key:"_order",value:function _order(){// reorder the items in all groups
	// TODO: optimization: only reorder groups affected by the changed items
	forEach$3(util$1).call(util$1,this.groups,function(group){group.order();});}/**
	   * Handle updated groups
	   * @param {number[]} ids
	   * @private
	   */},{key:"_onUpdateGroups",value:function _onUpdateGroups(ids){this._onAddGroups(ids);}/**
	   * Handle changed groups (added or updated)
	   * @param {number[]} ids
	   * @private
	   */},{key:"_onAddGroups",value:function _onAddGroups(ids){var me=this;forEach$3(ids).call(ids,function(id){var groupData=me.groupsData.get(id);var group=me.groups[id];if(!group){// check for reserved ids
	if(id==UNGROUPED$2||id==BACKGROUND$2){throw new Error("Illegal group id. ".concat(id," is a reserved id."));}var groupOptions=create$4(me.options);util$1.extend(groupOptions,{height:null});group=new Group(id,groupData,me);me.groups[id]=group;// add items with this groupId to the new group
	for(var itemId in me.items){if(me.items.hasOwnProperty(itemId)){var _item15=me.items[itemId];if(_item15.data.group==id){group.add(_item15);}}}group.order();group.show();}else {// update group
	group.setData(groupData);}});this.body.emitter.emit('_change',{queue:true});}/**
	   * Handle removed groups
	   * @param {number[]} ids
	   * @private
	   */},{key:"_onRemoveGroups",value:function _onRemoveGroups(ids){var _this29=this;forEach$3(ids).call(ids,function(id){var group=_this29.groups[id];if(group){group.dispose();delete _this29.groups[id];}});if(this.options.cluster){this.clusterGenerator.updateData();this._clusterItems();}this.markDirty({restackGroups:!!this.options.cluster});this.body.emitter.emit('_change',{queue:true});}/**
	   * Reorder the groups if needed
	   * @return {boolean} changed
	   * @private
	   */},{key:"_orderGroups",value:function _orderGroups(){if(this.groupsData){// reorder the groups
	var groupIds=this.groupsData.getIds({order:this.options.groupOrder});groupIds=this._orderNestedGroups(groupIds);var changed=!util$1.equalArray(groupIds,this.groupIds);if(changed){// hide all groups, removes them from the DOM
	var groups=this.groups;forEach$3(groupIds).call(groupIds,function(groupId){groups[groupId].hide();});// show the groups again, attach them to the DOM in correct order
	forEach$3(groupIds).call(groupIds,function(groupId){groups[groupId].show();});this.groupIds=groupIds;}return changed;}else {return false;}}/**
	   * Reorder the nested groups
	   *
	   * @param {Array.<number>} groupIds
	   * @returns {Array.<number>}
	   * @private
	   */},{key:"_orderNestedGroups",value:function _orderNestedGroups(groupIds){var _this30=this;/**
	     * Recursively order nested groups
	     *
	     * @param {ItemSet} t
	     * @param {Array.<number>} groupIds
	     * @returns {Array.<number>}
	     * @private
	     */function getOrderedNestedGroups(t,groupIds){var result=[];forEach$3(groupIds).call(groupIds,function(groupId){result.push(groupId);var groupData=t.groupsData.get(groupId);if(groupData.nestedGroups){var _context161;var nestedGroupIds=map$5(_context161=t.groupsData.get({filter:function filter(nestedGroup){return nestedGroup.nestedInGroup==groupId;},order:t.options.groupOrder})).call(_context161,function(nestedGroup){return nestedGroup.id;});result=concat$2(result).call(result,getOrderedNestedGroups(t,nestedGroupIds));}});return result;}var topGroupIds=filter$2(groupIds).call(groupIds,function(groupId){return !_this30.groupsData.get(groupId).nestedInGroup;});return getOrderedNestedGroups(this,topGroupIds);}/**
	   * Add a new item
	   * @param {Item} item
	   * @private
	   */},{key:"_addItem",value:function _addItem(item){this.items[item.id]=item;// add to group
	var groupId=this.getGroupId(item.data);var group=this.groups[groupId];if(!group){item.groupShowing=false;}else if(group&&group.data&&group.data.showNested){item.groupShowing=true;}if(group)group.add(item);}/**
	   * Update an existing item
	   * @param {Item} item
	   * @param {Object} itemData
	   * @private
	   */},{key:"_updateItem",value:function _updateItem(item,itemData){// update the items data (will redraw the item when displayed)
	item.setData(itemData);var groupId=this.getGroupId(item.data);var group=this.groups[groupId];if(!group){item.groupShowing=false;}else if(group&&group.data&&group.data.showNested){item.groupShowing=true;}}/**
	   * Delete an item from the ItemSet: remove it from the DOM, from the map
	   * with items, and from the map with visible items, and from the selection
	   * @param {Item} item
	   * @private
	   */},{key:"_removeItem",value:function _removeItem(item){var _context162,_context163;// remove from DOM
	item.hide();// remove from items
	delete this.items[item.id];// remove from selection
	var index=indexOf$3(_context162=this.selection).call(_context162,item.id);if(index!=-1)splice$2(_context163=this.selection).call(_context163,index,1);// remove from group
	item.parent&&item.parent.remove(item);// remove Tooltip from DOM
	if(this.popup!=null){this.popup.hide();}}/**
	   * Create an array containing all items being a range (having an end date)
	   * @param {Array.<Object>} array
	   * @returns {Array}
	   * @private
	   */},{key:"_constructByEndArray",value:function _constructByEndArray(array){var endArray=[];for(var i=0;i<array.length;i++){if(array[i]instanceof RangeItem){endArray.push(array[i]);}}return endArray;}/**
	   * Register the clicked item on touch, before dragStart is initiated.
	   *
	   * dragStart is initiated from a mousemove event, AFTER the mouse/touch is
	   * already moving. Therefore, the mouse/touch can sometimes be above an other
	   * DOM element than the item itself.
	   *
	   * @param {Event} event
	   * @private
	   */},{key:"_onTouch",value:function _onTouch(event){// store the touched item, used in _onDragStart
	this.touchParams.item=this.itemFromTarget(event);this.touchParams.dragLeftItem=event.target.dragLeftItem||false;this.touchParams.dragRightItem=event.target.dragRightItem||false;this.touchParams.itemProps=null;}/**
	   * Given an group id, returns the index it has.
	   *
	   * @param {number} groupId
	   * @returns {number} index / groupId
	   * @private
	   */},{key:"_getGroupIndex",value:function _getGroupIndex(groupId){for(var i=0;i<this.groupIds.length;i++){if(groupId==this.groupIds[i])return i;}}/**
	   * Start dragging the selected events
	   * @param {Event} event
	   * @private
	   */},{key:"_onDragStart",value:function _onDragStart(event){var _this31=this;if(this.touchParams.itemIsDragging){return;}var item=this.touchParams.item||null;var me=this;var props;if(item&&(item.selected||this.options.itemsAlwaysDraggable.item)){if(this.options.editable.overrideItems&&!this.options.editable.updateTime&&!this.options.editable.updateGroup){return;}// override options.editable
	if(item.editable!=null&&!item.editable.updateTime&&!item.editable.updateGroup&&!this.options.editable.overrideItems){return;}var dragLeftItem=this.touchParams.dragLeftItem;var dragRightItem=this.touchParams.dragRightItem;this.touchParams.itemIsDragging=true;this.touchParams.selectedItem=item;if(dragLeftItem){props={item:dragLeftItem,initialX:event.center.x,dragLeft:true,data:this._cloneItemData(item.data)};this.touchParams.itemProps=[props];}else if(dragRightItem){props={item:dragRightItem,initialX:event.center.x,dragRight:true,data:this._cloneItemData(item.data)};this.touchParams.itemProps=[props];}else if(this.options.editable.add&&(event.srcEvent.ctrlKey||event.srcEvent.metaKey)){// create a new range item when dragging with ctrl key down
	this._onDragStartAddItem(event);}else {if(this.groupIds.length<1){// Mitigates a race condition if _onDragStart() is
	// called after markDirty() without redraw() being called between.
	this.redraw();}var baseGroupIndex=this._getGroupIndex(item.data.group);var itemsToDrag=this.options.itemsAlwaysDraggable.item&&!item.selected?[item.id]:this.getSelection();this.touchParams.itemProps=map$5(itemsToDrag).call(itemsToDrag,function(id){var item=me.items[id];var groupIndex=me._getGroupIndex(item.data.group);return {item:item,initialX:event.center.x,groupOffset:baseGroupIndex-groupIndex,data:_this31._cloneItemData(item.data)};});}event.stopPropagation();}else if(this.options.editable.add&&(event.srcEvent.ctrlKey||event.srcEvent.metaKey)){// create a new range item when dragging with ctrl key down
	this._onDragStartAddItem(event);}}/**
	   * Start creating a new range item by dragging.
	   * @param {Event} event
	   * @private
	   */},{key:"_onDragStartAddItem",value:function _onDragStartAddItem(event){var snap=this.options.snap||null;var frameRect=this.dom.frame.getBoundingClientRect();// plus (if rtl) 10 to compensate for the drag starting as soon as you've moved 10px
	var x=this.options.rtl?frameRect.right-event.center.x+10:event.center.x-frameRect.left-10;var time=this.body.util.toTime(x);var scale=this.body.util.getScale();var step=this.body.util.getStep();var start=snap?snap(time,scale,step):time;var end=start;var itemData={type:'range',start:start,end:end,content:'new item'};var id=v4();itemData[this.itemsData.idProp]=id;var group=this.groupFromTarget(event);if(group){itemData.group=group.groupId;}var newItem=new RangeItem(itemData,this.conversion,this.options);newItem.id=id;// TODO: not so nice setting id afterwards
	newItem.data=this._cloneItemData(itemData);this._addItem(newItem);this.touchParams.selectedItem=newItem;var props={item:newItem,initialX:event.center.x,data:newItem.data};if(this.options.rtl){props.dragLeft=true;}else {props.dragRight=true;}this.touchParams.itemProps=[props];event.stopPropagation();}/**
	   * Drag selected items
	   * @param {Event} event
	   * @private
	   */},{key:"_onDrag",value:function _onDrag(event){var _this32=this;if(this.popup!=null&&this.options.showTooltips&&!this.popup.hidden){// this.popup.hide();
	var container=this.body.dom.centerContainer;var containerRect=container.getBoundingClientRect();this.popup.setPosition(event.center.x-containerRect.left+container.offsetLeft,event.center.y-containerRect.top+container.offsetTop);this.popup.show();// redraw
	}if(this.touchParams.itemProps){var _context164;event.stopPropagation();var me=this;var snap=this.options.snap||null;var domRootOffsetLeft=this.body.dom.root.offsetLeft;var xOffset=this.options.rtl?domRootOffsetLeft+this.body.domProps.right.width:domRootOffsetLeft+this.body.domProps.left.width;var scale=this.body.util.getScale();var step=this.body.util.getStep();//only calculate the new group for the item that's actually dragged
	var selectedItem=this.touchParams.selectedItem;var updateGroupAllowed=(this.options.editable.overrideItems||selectedItem.editable==null)&&this.options.editable.updateGroup||!this.options.editable.overrideItems&&selectedItem.editable!=null&&selectedItem.editable.updateGroup;var newGroupBase=null;if(updateGroupAllowed&&selectedItem){if(selectedItem.data.group!=undefined){// drag from one group to another
	var group=me.groupFromTarget(event);if(group){//we know the offset for all items, so the new group for all items
	//will be relative to this one.
	newGroupBase=this._getGroupIndex(group.groupId);}}}// move
	forEach$3(_context164=this.touchParams.itemProps).call(_context164,function(props){var current=me.body.util.toTime(event.center.x-xOffset);var initial=me.body.util.toTime(props.initialX-xOffset);var offset;var initialStart;var initialEnd;var start;var end;if(_this32.options.rtl){offset=-(current-initial);// ms
	}else {offset=current-initial;// ms
	}var itemData=_this32._cloneItemData(props.item.data);// clone the data
	if(props.item.editable!=null&&!props.item.editable.updateTime&&!props.item.editable.updateGroup&&!me.options.editable.overrideItems){return;}var updateTimeAllowed=(_this32.options.editable.overrideItems||selectedItem.editable==null)&&_this32.options.editable.updateTime||!_this32.options.editable.overrideItems&&selectedItem.editable!=null&&selectedItem.editable.updateTime;if(updateTimeAllowed){if(props.dragLeft){// drag left side of a range item
	if(_this32.options.rtl){if(itemData.end!=undefined){initialEnd=util$1.convert(props.data.end,'Date');end=new Date(initialEnd.valueOf()+offset);// TODO: pass a Moment instead of a Date to snap(). (Breaking change)
	itemData.end=snap?snap(end,scale,step):end;}}else {if(itemData.start!=undefined){initialStart=util$1.convert(props.data.start,'Date');start=new Date(initialStart.valueOf()+offset);// TODO: pass a Moment instead of a Date to snap(). (Breaking change)
	itemData.start=snap?snap(start,scale,step):start;}}}else if(props.dragRight){// drag right side of a range item
	if(_this32.options.rtl){if(itemData.start!=undefined){initialStart=util$1.convert(props.data.start,'Date');start=new Date(initialStart.valueOf()+offset);// TODO: pass a Moment instead of a Date to snap(). (Breaking change)
	itemData.start=snap?snap(start,scale,step):start;}}else {if(itemData.end!=undefined){initialEnd=util$1.convert(props.data.end,'Date');end=new Date(initialEnd.valueOf()+offset);// TODO: pass a Moment instead of a Date to snap(). (Breaking change)
	itemData.end=snap?snap(end,scale,step):end;}}}else {// drag both start and end
	if(itemData.start!=undefined){initialStart=util$1.convert(props.data.start,'Date').valueOf();start=new Date(initialStart+offset);if(itemData.end!=undefined){initialEnd=util$1.convert(props.data.end,'Date');var duration=initialEnd.valueOf()-initialStart.valueOf();// TODO: pass a Moment instead of a Date to snap(). (Breaking change)
	itemData.start=snap?snap(start,scale,step):start;itemData.end=new Date(itemData.start.valueOf()+duration);}else {// TODO: pass a Moment instead of a Date to snap(). (Breaking change)
	itemData.start=snap?snap(start,scale,step):start;}}}}if(updateGroupAllowed&&!props.dragLeft&&!props.dragRight&&newGroupBase!=null){if(itemData.group!=undefined){var newOffset=newGroupBase-props.groupOffset;//make sure we stay in bounds
	newOffset=Math.max(0,newOffset);newOffset=Math.min(me.groupIds.length-1,newOffset);itemData.group=me.groupIds[newOffset];}}// confirm moving the item
	itemData=_this32._cloneItemData(itemData);// convert start and end to the correct type
	me.options.onMoving(itemData,function(itemData){if(itemData){props.item.setData(_this32._cloneItemData(itemData,'Date'));}});});this.body.emitter.emit('_change');}}/**
	   * Move an item to another group
	   * @param {Item} item
	   * @param {string | number} groupId
	   * @private
	   */},{key:"_moveToGroup",value:function _moveToGroup(item,groupId){var group=this.groups[groupId];if(group&&group.groupId!=item.data.group){var oldGroup=item.parent;oldGroup.remove(item);oldGroup.order();item.data.group=group.groupId;group.add(item);group.order();}}/**
	   * End of dragging selected items
	   * @param {Event} event
	   * @private
	   */},{key:"_onDragEnd",value:function _onDragEnd(event){var _this33=this;this.touchParams.itemIsDragging=false;if(this.touchParams.itemProps){event.stopPropagation();var me=this;var itemProps=this.touchParams.itemProps;this.touchParams.itemProps=null;forEach$3(itemProps).call(itemProps,function(props){var id=props.item.id;var exists=me.itemsData.get(id)!=null;if(!exists){// add a new item
	me.options.onAdd(props.item.data,function(itemData){me._removeItem(props.item);// remove temporary item
	if(itemData){me.itemsData.add(itemData);}// force re-stacking of all items next redraw
	me.body.emitter.emit('_change');});}else {// update existing item
	var itemData=_this33._cloneItemData(props.item.data);// convert start and end to the correct type
	me.options.onMove(itemData,function(itemData){if(itemData){// apply changes
	itemData[_this33.itemsData.idProp]=id;// ensure the item contains its id (can be undefined)
	_this33.itemsData.update(itemData);}else {// restore original values
	props.item.setData(props.data);me.body.emitter.emit('_change');}});}});}}/**
	   * On group click
	   * @param {Event} event
	   * @private
	   */},{key:"_onGroupClick",value:function _onGroupClick(event){var _this34=this;var group=this.groupFromTarget(event);setTimeout$2(function(){_this34.toggleGroupShowNested(group);},1);}/**
	   * Toggle show nested
	   * @param {object} group
	   * @param {boolean} force
	   */},{key:"toggleGroupShowNested",value:function toggleGroupShowNested(group){var _context165;var force=arguments.length>1&&arguments[1]!==undefined?arguments[1]:undefined;if(!group||!group.nestedGroups)return;var groupsData=this.groupsData.getDataSet();if(force!=undefined){group.showNested=!!force;}else {group.showNested=!group.showNested;}var nestingGroup=groupsData.get(group.groupId);nestingGroup.showNested=group.showNested;var fullNestedGroups=group.nestedGroups;var nextLevel=fullNestedGroups;while(nextLevel.length>0){var current=nextLevel;nextLevel=[];for(var i=0;i<current.length;i++){var node=groupsData.get(current[i]);if(node.nestedGroups){nextLevel=concat$2(nextLevel).call(nextLevel,node.nestedGroups);}}if(nextLevel.length>0){fullNestedGroups=concat$2(fullNestedGroups).call(fullNestedGroups,nextLevel);}}var nestedGroups=map$5(_context165=groupsData.get(fullNestedGroups)).call(_context165,function(nestedGroup){if(nestedGroup.visible==undefined){nestedGroup.visible=true;}nestedGroup.visible=!!nestingGroup.showNested;return nestedGroup;});groupsData.update(concat$2(nestedGroups).call(nestedGroups,nestingGroup));if(nestingGroup.showNested){util$1.removeClassName(group.dom.label,'collapsed');util$1.addClassName(group.dom.label,'expanded');}else {util$1.removeClassName(group.dom.label,'expanded');util$1.addClassName(group.dom.label,'collapsed');}}/**
	   * Toggle group drag classname
	   * @param {object} group
	   */},{key:"toggleGroupDragClassName",value:function toggleGroupDragClassName(group){group.dom.label.classList.toggle('vis-group-is-dragging');group.dom.foreground.classList.toggle('vis-group-is-dragging');}/**
	   * on drag start
	   * @param {Event} event
	   * @return {void}   
	   * @private
	   */},{key:"_onGroupDragStart",value:function _onGroupDragStart(event){if(this.groupTouchParams.isDragging)return;if(this.options.groupEditable.order){this.groupTouchParams.group=this.groupFromTarget(event);if(this.groupTouchParams.group){event.stopPropagation();this.groupTouchParams.isDragging=true;this.toggleGroupDragClassName(this.groupTouchParams.group);this.groupTouchParams.originalOrder=this.groupsData.getIds({order:this.options.groupOrder});}}}/**
	   * on drag
	   * @param {Event} event
	   * @return {void}
	   * @private
	   */},{key:"_onGroupDrag",value:function _onGroupDrag(event){if(this.options.groupEditable.order&&this.groupTouchParams.group){event.stopPropagation();var groupsData=this.groupsData;if(this.groupsData instanceof DataView){groupsData=this.groupsData.getDataSet();}// drag from one group to another
	var group=this.groupFromTarget(event);// try to avoid toggling when groups differ in height
	if(group&&group.height!=this.groupTouchParams.group.height){var movingUp=group.top<this.groupTouchParams.group.top;var clientY=event.center?event.center.y:event.clientY;var targetGroup=group.dom.foreground.getBoundingClientRect();var draggedGroupHeight=this.groupTouchParams.group.height;if(movingUp){// skip swapping the groups when the dragged group is not below clientY afterwards
	if(targetGroup.top+draggedGroupHeight<clientY){return;}}else {var targetGroupHeight=group.height;// skip swapping the groups when the dragged group is not below clientY afterwards
	if(targetGroup.top+targetGroupHeight-draggedGroupHeight>clientY){return;}}}if(group&&group!=this.groupTouchParams.group){var _targetGroup=groupsData.get(group.groupId);var draggedGroup=groupsData.get(this.groupTouchParams.group.groupId);// switch groups
	if(draggedGroup&&_targetGroup){this.options.groupOrderSwap(draggedGroup,_targetGroup,groupsData);groupsData.update(draggedGroup);groupsData.update(_targetGroup);}// fetch current order of groups
	var newOrder=groupsData.getIds({order:this.options.groupOrder});// in case of changes since _onGroupDragStart
	if(!util$1.equalArray(newOrder,this.groupTouchParams.originalOrder)){var origOrder=this.groupTouchParams.originalOrder;var draggedId=this.groupTouchParams.group.groupId;var numGroups=Math.min(origOrder.length,newOrder.length);var curPos=0;var newOffset=0;var orgOffset=0;while(curPos<numGroups){// as long as the groups are where they should be step down along the groups order
	while(curPos+newOffset<numGroups&&curPos+orgOffset<numGroups&&newOrder[curPos+newOffset]==origOrder[curPos+orgOffset]){curPos++;}// all ok
	if(curPos+newOffset>=numGroups){break;}// not all ok
	// if dragged group was move upwards everything below should have an offset
	if(newOrder[curPos+newOffset]==draggedId){newOffset=1;}// if dragged group was move downwards everything above should have an offset
	else if(origOrder[curPos+orgOffset]==draggedId){orgOffset=1;}// found a group (apart from dragged group) that has the wrong position -> switch with the 
	// group at the position where other one should be, fix index arrays and continue
	else {var slippedPosition=indexOf$3(newOrder).call(newOrder,origOrder[curPos+orgOffset]);var switchGroup=groupsData.get(newOrder[curPos+newOffset]);var shouldBeGroup=groupsData.get(origOrder[curPos+orgOffset]);this.options.groupOrderSwap(switchGroup,shouldBeGroup,groupsData);groupsData.update(switchGroup);groupsData.update(shouldBeGroup);var switchGroupId=newOrder[curPos+newOffset];newOrder[curPos+newOffset]=origOrder[curPos+orgOffset];newOrder[slippedPosition]=switchGroupId;curPos++;}}}}}}/**
	   * on drag end
	   * @param {Event} event
	   * @return {void}
	   * @private
	   */},{key:"_onGroupDragEnd",value:function _onGroupDragEnd(event){this.groupTouchParams.isDragging=false;if(this.options.groupEditable.order&&this.groupTouchParams.group){event.stopPropagation();// update existing group
	var me=this;var id=me.groupTouchParams.group.groupId;var dataset=me.groupsData.getDataSet();var groupData=util$1.extend({},dataset.get(id));// clone the data
	me.options.onMoveGroup(groupData,function(groupData){if(groupData){// apply changes
	groupData[dataset._idProp]=id;// ensure the group contains its id (can be undefined)
	dataset.update(groupData);}else {// fetch current order of groups
	var newOrder=dataset.getIds({order:me.options.groupOrder});// restore original order
	if(!util$1.equalArray(newOrder,me.groupTouchParams.originalOrder)){var origOrder=me.groupTouchParams.originalOrder;var numGroups=Math.min(origOrder.length,newOrder.length);var curPos=0;while(curPos<numGroups){// as long as the groups are where they should be step down along the groups order
	while(curPos<numGroups&&newOrder[curPos]==origOrder[curPos]){curPos++;}// all ok
	if(curPos>=numGroups){break;}// found a group that has the wrong position -> switch with the
	// group at the position where other one should be, fix index arrays and continue
	var slippedPosition=indexOf$3(newOrder).call(newOrder,origOrder[curPos]);var switchGroup=dataset.get(newOrder[curPos]);var shouldBeGroup=dataset.get(origOrder[curPos]);me.options.groupOrderSwap(switchGroup,shouldBeGroup,dataset);dataset.update(switchGroup);dataset.update(shouldBeGroup);var switchGroupId=newOrder[curPos];newOrder[curPos]=origOrder[curPos];newOrder[slippedPosition]=switchGroupId;curPos++;}}}});me.body.emitter.emit('groupDragged',{groupId:id});this.toggleGroupDragClassName(this.groupTouchParams.group);this.groupTouchParams.group=null;}}/**
	   * Handle selecting/deselecting an item when tapping it
	   * @param {Event} event
	   * @private
	   */},{key:"_onSelectItem",value:function _onSelectItem(event){if(!this.options.selectable)return;var ctrlKey=event.srcEvent&&(event.srcEvent.ctrlKey||event.srcEvent.metaKey);var shiftKey=event.srcEvent&&event.srcEvent.shiftKey;if(ctrlKey||shiftKey){this._onMultiSelectItem(event);return;}var oldSelection=this.getSelection();var item=this.itemFromTarget(event);var selection=item&&item.selectable?[item.id]:[];this.setSelection(selection);var newSelection=this.getSelection();// emit a select event,
	// except when old selection is empty and new selection is still empty
	if(newSelection.length>0||oldSelection.length>0){this.body.emitter.emit('select',{items:newSelection,event:event});}}/**
	   * Handle hovering an item
	   * @param {Event} event
	   * @private
	   */},{key:"_onMouseOver",value:function _onMouseOver(event){var item=this.itemFromTarget(event);if(!item)return;// Item we just left
	var related=this.itemFromRelatedTarget(event);if(item===related){// We haven't changed item, just element in the item
	return;}var title=item.getTitle();if(this.options.showTooltips&&title){if(this.popup==null){this.popup=new Popup$1(this.body.dom.root,this.options.tooltip.overflowMethod||'flip');}this.popup.setText(title);var container=this.body.dom.centerContainer;var containerRect=container.getBoundingClientRect();this.popup.setPosition(event.clientX-containerRect.left+container.offsetLeft,event.clientY-containerRect.top+container.offsetTop);this.setPopupTimer(this.popup);}else {// Hovering over item without a title, hide popup
	// Needed instead of _just_ in _onMouseOut due to #2572
	this.clearPopupTimer();if(this.popup!=null){this.popup.hide();}}this.body.emitter.emit('itemover',{item:item.id,event:event});}/**
	   * on mouse start
	   * @param {Event} event
	   * @return {void}   
	   * @private
	   */},{key:"_onMouseOut",value:function _onMouseOut(event){var item=this.itemFromTarget(event);if(!item)return;// Item we are going to
	var related=this.itemFromRelatedTarget(event);if(item===related){// We aren't changing item, just element in the item
	return;}this.clearPopupTimer();if(this.popup!=null){this.popup.hide();}this.body.emitter.emit('itemout',{item:item.id,event:event});}/**
	   * on mouse move
	   * @param {Event} event
	   * @return {void}   
	   * @private
	   */},{key:"_onMouseMove",value:function _onMouseMove(event){var item=this.itemFromTarget(event);if(!item)return;if(this.popupTimer!=null){// restart timer
	this.setPopupTimer(this.popup);}if(this.options.showTooltips&&this.options.tooltip.followMouse&&this.popup&&!this.popup.hidden){var container=this.body.dom.centerContainer;var containerRect=container.getBoundingClientRect();this.popup.setPosition(event.clientX-containerRect.left+container.offsetLeft,event.clientY-containerRect.top+container.offsetTop);this.popup.show();// Redraw
	}}/**
	   * Handle mousewheel
	   * @param {Event}  event   The event
	   * @private
	   */},{key:"_onMouseWheel",value:function _onMouseWheel(event){if(this.touchParams.itemIsDragging){this._onDragEnd(event);}}/**
	   * Handle updates of an item on double tap
	   * @param {timeline.Item}  item   The item
	   * @private
	   */},{key:"_onUpdateItem",value:function _onUpdateItem(item){if(!this.options.selectable)return;if(!this.options.editable.updateTime&&!this.options.editable.updateGroup)return;var me=this;if(item){// execute async handler to update the item (or cancel it)
	var itemData=me.itemsData.get(item.id);// get a clone of the data from the dataset
	this.options.onUpdate(itemData,function(itemData){if(itemData){me.itemsData.update(itemData);}});}}/**
	   * Handle drop event of data on item
	   * Only called when `objectData.target === 'item'.
	   * @param {Event} event The event 
	   * @private
	   */},{key:"_onDropObjectOnItem",value:function _onDropObjectOnItem(event){var item=this.itemFromTarget(event);var objectData=JSON.parse(event.dataTransfer.getData("text"));this.options.onDropObjectOnItem(objectData,item);}/**
	   * Handle creation of an item on double tap or drop of a drag event
	   * @param {Event} event   The event
	   * @private
	   */},{key:"_onAddItem",value:function _onAddItem(event){if(!this.options.selectable)return;if(!this.options.editable.add)return;var me=this;var snap=this.options.snap||null;// add item
	var frameRect=this.dom.frame.getBoundingClientRect();var x=this.options.rtl?frameRect.right-event.center.x:event.center.x-frameRect.left;var start=this.body.util.toTime(x);var scale=this.body.util.getScale();var step=this.body.util.getStep();var end;var newItemData;if(event.type=='drop'){newItemData=JSON.parse(event.dataTransfer.getData("text"));newItemData.content=newItemData.content?newItemData.content:'new item';newItemData.start=newItemData.start?newItemData.start:snap?snap(start,scale,step):start;newItemData.type=newItemData.type||'box';newItemData[this.itemsData.idProp]=newItemData.id||v4();if(newItemData.type=='range'&&!newItemData.end){end=this.body.util.toTime(x+this.props.width/5);newItemData.end=snap?snap(end,scale,step):end;}}else {newItemData={start:snap?snap(start,scale,step):start,content:'new item'};newItemData[this.itemsData.idProp]=v4();// when default type is a range, add a default end date to the new item
	if(this.options.type==='range'){end=this.body.util.toTime(x+this.props.width/5);newItemData.end=snap?snap(end,scale,step):end;}}var group=this.groupFromTarget(event);if(group){newItemData.group=group.groupId;}// execute async handler to customize (or cancel) adding an item
	newItemData=this._cloneItemData(newItemData);// convert start and end to the correct type
	this.options.onAdd(newItemData,function(item){if(item){me.itemsData.add(item);if(event.type=='drop'){me.setSelection([item.id]);}// TODO: need to trigger a redraw?
	}});}/**
	   * Handle selecting/deselecting multiple items when holding an item
	   * @param {Event} event
	   * @private
	   */},{key:"_onMultiSelectItem",value:function _onMultiSelectItem(event){var _this35=this;if(!this.options.selectable)return;var item=this.itemFromTarget(event);if(item){// multi select items (if allowed)
	var selection=this.options.multiselect?this.getSelection()// take current selection
	:[];// deselect current selection
	var shiftKey=event.srcEvent&&event.srcEvent.shiftKey||false;if((shiftKey||this.options.sequentialSelection)&&this.options.multiselect){// select all items between the old selection and the tapped item
	var itemGroup=this.itemsData.get(item.id).group;// when filtering get the group of the last selected item
	var lastSelectedGroup=undefined;if(this.options.multiselectPerGroup){if(selection.length>0){lastSelectedGroup=this.itemsData.get(selection[0]).group;}}// determine the selection range
	if(!this.options.multiselectPerGroup||lastSelectedGroup==undefined||lastSelectedGroup==itemGroup){selection.push(item.id);}var range=ItemSet._getItemRange(this.itemsData.get(selection));if(!this.options.multiselectPerGroup||lastSelectedGroup==itemGroup){// select all items within the selection range
	selection=[];for(var id in this.items){if(this.items.hasOwnProperty(id)){var _item=this.items[id];var start=_item.data.start;var end=_item.data.end!==undefined?_item.data.end:start;if(start>=range.min&&end<=range.max&&(!this.options.multiselectPerGroup||lastSelectedGroup==this.itemsData.get(_item.id).group)&&!(_item instanceof BackgroundItem)){selection.push(_item.id);// do not use id but item.id, id itself is stringified
	}}}}}else {// add/remove this item from the current selection
	var index=indexOf$3(selection).call(selection,item.id);if(index==-1){// item is not yet selected -> select it
	selection.push(item.id);}else {// item is already selected -> deselect it
	splice$2(selection).call(selection,index,1);}}var filteredSelection=filter$2(selection).call(selection,function(item){return _this35.getItemById(item).selectable;});this.setSelection(filteredSelection);this.body.emitter.emit('select',{items:this.getSelection(),event:event});}}/**
	   * Calculate the time range of a list of items
	   * @param {Array.<Object>} itemsData
	   * @return {{min: Date, max: Date}} Returns the range of the provided items
	   * @private
	   */},{key:"itemFromElement",/**
	   * Find an item from an element:
	   * searches for the attribute 'vis-item' in the element's tree
	   * @param {HTMLElement} element
	   * @return {Item | null} item
	   */value:function itemFromElement(element){var cur=element;while(cur){if(cur.hasOwnProperty('vis-item')){return cur['vis-item'];}cur=cur.parentNode;}return null;}/**
	   * Find an item from an event target:
	   * searches for the attribute 'vis-item' in the event target's element tree
	   * @param {Event} event
	   * @return {Item | null} item
	   */},{key:"itemFromTarget",value:function itemFromTarget(event){return this.itemFromElement(event.target);}/**
	   * Find an item from an event's related target:
	   * searches for the attribute 'vis-item' in the related target's element tree
	   * @param {Event} event
	   * @return {Item | null} item
	   */},{key:"itemFromRelatedTarget",value:function itemFromRelatedTarget(event){return this.itemFromElement(event.relatedTarget);}/**
	   * Find the Group from an event target:
	   * searches for the attribute 'vis-group' in the event target's element tree
	   * @param {Event} event
	   * @return {Group | null} group
	   */},{key:"groupFromTarget",value:function groupFromTarget(event){var clientY=event.center?event.center.y:event.clientY;var groupIds=this.groupIds;if(groupIds.length<=0&&this.groupsData){groupIds=this.groupsData.getIds({order:this.options.groupOrder});}for(var i=0;i<groupIds.length;i++){var groupId=groupIds[i];var group=this.groups[groupId];var foreground=group.dom.foreground;var foregroundRect=foreground.getBoundingClientRect();if(clientY>=foregroundRect.top&&clientY<foregroundRect.top+foreground.offsetHeight){return group;}if(this.options.orientation.item==='top'){if(i===this.groupIds.length-1&&clientY>foregroundRect.top){return group;}}else {if(i===0&&clientY<foregroundRect.top+foreground.offset){return group;}}}return null;}/**
	   * Find the ItemSet from an event target:
	   * searches for the attribute 'vis-itemset' in the event target's element tree
	   * @param {Event} event
	   * @return {ItemSet | null} item
	   */},{key:"_cloneItemData",/**
	   * Clone the data of an item, and "normalize" it: convert the start and end date
	   * to the type (Date, Moment, ...) configured in the DataSet. If not configured,
	   * start and end are converted to Date.
	   * @param {Object} itemData, typically `item.data`
	   * @param {string} [type]  Optional Date type. If not provided, the type from the DataSet is taken
	   * @return {Object} The cloned object
	   * @private
	   */value:function _cloneItemData(itemData,type){var clone=util$1.extend({},itemData);if(!type){// convert start and end date to the type (Date, Moment, ...) configured in the DataSet
	type=this.itemsData.type;}if(clone.start!=undefined){clone.start=util$1.convert(clone.start,type&&type.start||'Date');}if(clone.end!=undefined){clone.end=util$1.convert(clone.end,type&&type.end||'Date');}return clone;}/**
	   * cluster items
	   * @return {void}   
	   * @private
	   */},{key:"_clusterItems",value:function _clusterItems(){if(!this.options.cluster){return;}var _this$body$range$conv=this.body.range.conversion(this.body.domProps.center.width),scale=_this$body$range$conv.scale;var clusters=this.clusterGenerator.getClusters(this.clusters,scale,this.options.cluster);if(this.clusters!=clusters){this._detachAllClusters();if(clusters){var _iterator8=_createForOfIteratorHelper$3(clusters),_step9;try{for(_iterator8.s();!(_step9=_iterator8.n()).done;){var cluster=_step9.value;cluster.attach();}}catch(err){_iterator8.e(err);}finally{_iterator8.f();}this.clusters=clusters;}this._updateClusters(clusters);}}/**
	   * detach all cluster items
	   * @private
	   */},{key:"_detachAllClusters",value:function _detachAllClusters(){if(this.options.cluster){if(this.clusters&&this.clusters.length){var _iterator9=_createForOfIteratorHelper$3(this.clusters),_step10;try{for(_iterator9.s();!(_step10=_iterator9.n()).done;){var cluster=_step10.value;cluster.detach();}}catch(err){_iterator9.e(err);}finally{_iterator9.f();}}}}/**
	   * update clusters
	   * @param {array} clusters
	   * @private
	   */},{key:"_updateClusters",value:function _updateClusters(clusters){if(this.clusters&&this.clusters.length){var _context166;var newClustersIds=new set$3(map$5(clusters).call(clusters,function(cluster){return cluster.id;}));var clustersToUnselect=filter$2(_context166=this.clusters).call(_context166,function(cluster){return !newClustersIds.has(cluster.id);});var selectionChanged=false;var _iterator10=_createForOfIteratorHelper$3(clustersToUnselect),_step11;try{for(_iterator10.s();!(_step11=_iterator10.n()).done;){var _context167;var cluster=_step11.value;var selectedIdx=indexOf$3(_context167=this.selection).call(_context167,cluster.id);if(selectedIdx!==-1){var _context168;cluster.unselect();splice$2(_context168=this.selection).call(_context168,selectedIdx,1);selectionChanged=true;}}}catch(err){_iterator10.e(err);}finally{_iterator10.f();}if(selectionChanged){var newSelection=this.getSelection();this.body.emitter.emit('select',{items:newSelection,event:event});}}this.clusters=clusters||[];}}],[{key:"_getItemRange",value:function _getItemRange(itemsData){var max=null;var min=null;forEach$3(itemsData).call(itemsData,function(data){if(min==null||data.start<min){min=data.start;}if(data.end!=undefined){if(max==null||data.end>max){max=data.end;}}else {if(max==null||data.start>max){max=data.start;}}});return {min:min,max:max};}},{key:"itemSetFromTarget",value:function itemSetFromTarget(event){var target=event.target;while(target){if(target.hasOwnProperty('vis-itemset')){return target['vis-itemset'];}target=target.parentNode;}return null;}}]);return ItemSet;}(Component);// available item types will be registered here
	ItemSet.types={background:BackgroundItem,box:BoxItem,range:RangeItem,point:PointItem};/**
	 * Handle added items
	 * @param {number[]} ids
	 * @protected
	 */ItemSet.prototype._onAdd=ItemSet.prototype._onUpdate;var errorFound$2=false;var allOptions$3;var printStyle$2='background: #FFeeee; color: #dd0000';/**
	 *  Used to validate options.
	 */var Validator$2=/*#__PURE__*/function(){/**
	   * @ignore
	   */function Validator(){classCallCheck(this,Validator);}/**
	   * Main function to be called
	   * @param {Object} options
	   * @param {Object} referenceOptions
	   * @param {Object} subObject
	   * @returns {boolean}
	   * @static
	   */createClass(Validator,null,[{key:"validate",value:function validate(options,referenceOptions,subObject){errorFound$2=false;allOptions$3=referenceOptions;var usedOptions=referenceOptions;if(subObject!==undefined){usedOptions=referenceOptions[subObject];}Validator.parse(options,usedOptions,[]);return errorFound$2;}/**
	   * Will traverse an object recursively and check every value
	   * @param {Object} options
	   * @param {Object} referenceOptions
	   * @param {array} path    | where to look for the actual option
	   * @static
	   */},{key:"parse",value:function parse(options,referenceOptions,path){for(var option in options){if(options.hasOwnProperty(option)){Validator.check(option,options,referenceOptions,path);}}}/**
	   * Check every value. If the value is an object, call the parse function on that object.
	   * @param {string} option
	   * @param {Object} options
	   * @param {Object} referenceOptions
	   * @param {array} path    | where to look for the actual option
	   * @static
	   */},{key:"check",value:function check(option,options,referenceOptions,path){if(referenceOptions[option]===undefined&&referenceOptions.__any__===undefined){Validator.getSuggestion(option,referenceOptions,path);return;}var referenceOption=option;var is_object=true;if(referenceOptions[option]===undefined&&referenceOptions.__any__!==undefined){// NOTE: This only triggers if the __any__ is in the top level of the options object.
	//       THAT'S A REALLY BAD PLACE TO ALLOW IT!!!!
	// TODO: Examine if needed, remove if possible
	// __any__ is a wildcard. Any value is accepted and will be further analysed by reference.
	referenceOption='__any__';// if the any-subgroup is not a predefined object in the configurator,
	// we do not look deeper into the object.
	is_object=Validator.getType(options[option])==='object';}var refOptionObj=referenceOptions[referenceOption];if(is_object&&refOptionObj.__type__!==undefined){refOptionObj=refOptionObj.__type__;}Validator.checkFields(option,options,referenceOptions,referenceOption,refOptionObj,path);}/**
	   *
	   * @param {string}  option           | the option property
	   * @param {Object}  options          | The supplied options object
	   * @param {Object}  referenceOptions | The reference options containing all options and their allowed formats
	   * @param {string}  referenceOption  | Usually this is the same as option, except when handling an __any__ tag.
	   * @param {string}  refOptionObj     | This is the type object from the reference options
	   * @param {Array}   path             | where in the object is the option
	   * @static
	   */},{key:"checkFields",value:function checkFields(option,options,referenceOptions,referenceOption,refOptionObj,path){var log=function log(message){console.log('%c'+message+Validator.printLocation(path,option),printStyle$2);};var optionType=Validator.getType(options[option]);var refOptionType=refOptionObj[optionType];if(refOptionType!==undefined){// if the type is correct, we check if it is supposed to be one of a few select values
	if(Validator.getType(refOptionType)==='array'&&indexOf$3(refOptionType).call(refOptionType,options[option])===-1){log('Invalid option detected in "'+option+'".'+' Allowed values are:'+Validator.print(refOptionType)+' not "'+options[option]+'". ');errorFound$2=true;}else if(optionType==='object'&&referenceOption!=="__any__"){path=util$1.copyAndExtendArray(path,option);Validator.parse(options[option],referenceOptions[referenceOption],path);}}else if(refOptionObj['any']===undefined){// type of the field is incorrect and the field cannot be any
	log('Invalid type received for "'+option+'". Expected: '+Validator.print(keys$6(refOptionObj))+'. Received ['+optionType+'] "'+options[option]+'"');errorFound$2=true;}}/**
	   *
	   * @param {Object|boolean|number|string|Array.<number>|Date|Node|Moment|undefined|null} object
	   * @returns {string}
	   * @static
	   */},{key:"getType",value:function getType(object){var type=_typeof_1(object);if(type==='object'){if(object===null){return 'null';}if(object instanceof Boolean){return 'boolean';}if(object instanceof Number){return 'number';}if(object instanceof String){return 'string';}if(isArray$3(object)){return 'array';}if(object instanceof Date){return 'date';}if(object.nodeType!==undefined){return 'dom';}if(object._isAMomentObject===true){return 'moment';}return 'object';}else if(type==='number'){return 'number';}else if(type==='boolean'){return 'boolean';}else if(type==='string'){return 'string';}else if(type===undefined){return 'undefined';}return type;}/**
	   * @param {string} option
	   * @param {Object} options
	   * @param {Array.<string>} path
	   * @static
	   */},{key:"getSuggestion",value:function getSuggestion(option,options,path){var localSearch=Validator.findInOptions(option,options,path,false);var globalSearch=Validator.findInOptions(option,allOptions$3,[],true);var localSearchThreshold=8;var globalSearchThreshold=4;var msg;if(localSearch.indexMatch!==undefined){msg=' in '+Validator.printLocation(localSearch.path,option,'')+'Perhaps it was incomplete? Did you mean: "'+localSearch.indexMatch+'"?\n\n';}else if(globalSearch.distance<=globalSearchThreshold&&localSearch.distance>globalSearch.distance){msg=' in '+Validator.printLocation(localSearch.path,option,'')+'Perhaps it was misplaced? Matching option found at: '+Validator.printLocation(globalSearch.path,globalSearch.closestMatch,'');}else if(localSearch.distance<=localSearchThreshold){msg='. Did you mean "'+localSearch.closestMatch+'"?'+Validator.printLocation(localSearch.path,option);}else {msg='. Did you mean one of these: '+Validator.print(keys$6(options))+Validator.printLocation(path,option);}console.log('%cUnknown option detected: "'+option+'"'+msg,printStyle$2);errorFound$2=true;}/**
	   * traverse the options in search for a match.
	   * @param {string} option
	   * @param {Object} options
	   * @param {Array} path    | where to look for the actual option
	   * @param {boolean} [recursive=false]
	   * @returns {{closestMatch: string, path: Array, distance: number}}
	   * @static
	   */},{key:"findInOptions",value:function findInOptions(option,options,path){var recursive=arguments.length>3&&arguments[3]!==undefined?arguments[3]:false;var min=1e9;var closestMatch='';var closestMatchPath=[];var lowerCaseOption=option.toLowerCase();var indexMatch=undefined;for(var op in options){// eslint-disable-line guard-for-in
	var distance=void 0;if(options[op].__type__!==undefined&&recursive===true){var result=Validator.findInOptions(option,options[op],util$1.copyAndExtendArray(path,op));if(min>result.distance){closestMatch=result.closestMatch;closestMatchPath=result.path;min=result.distance;indexMatch=result.indexMatch;}}else {var _context169;if(indexOf$3(_context169=op.toLowerCase()).call(_context169,lowerCaseOption)!==-1){indexMatch=op;}distance=Validator.levenshteinDistance(option,op);if(min>distance){closestMatch=op;closestMatchPath=util$1.copyArray(path);min=distance;}}}return {closestMatch:closestMatch,path:closestMatchPath,distance:min,indexMatch:indexMatch};}/**
	   * @param {Array.<string>} path
	   * @param {Object} option
	   * @param {string} prefix
	   * @returns {String}
	   * @static
	   */},{key:"printLocation",value:function printLocation(path,option){var prefix=arguments.length>2&&arguments[2]!==undefined?arguments[2]:'Problem value found at: \n';var str='\n\n'+prefix+'options = {\n';for(var i=0;i<path.length;i++){for(var j=0;j<i+1;j++){str+='  ';}str+=path[i]+': {\n';}for(var _j=0;_j<path.length+1;_j++){str+='  ';}str+=option+'\n';for(var _i11=0;_i11<path.length+1;_i11++){for(var _j2=0;_j2<path.length-_i11;_j2++){str+='  ';}str+='}\n';}return str+'\n\n';}/**
	   * @param {Object} options
	   * @returns {String}
	   * @static
	   */},{key:"print",value:function print(options){return stringify$2(options).replace(/(\")|(\[)|(\])|(,"__type__")/g,"").replace(/(\,)/g,', ');}/**
	   *  Compute the edit distance between the two given strings
	   * http://en.wikibooks.org/wiki/Algorithm_Implementation/Strings/Levenshtein_distance#JavaScript
	   *
	   * Copyright (c) 2011 Andrei Mackenzie
	   *
	   * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
	   *
	   * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
	   *
	   * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
	   *
	   * @param {string} a
	   * @param {string} b
	   * @returns {Array.<Array.<number>>}}
	   * @static
	   */},{key:"levenshteinDistance",value:function levenshteinDistance(a,b){if(a.length===0)return b.length;if(b.length===0)return a.length;var matrix=[];// increment along the first column of each row
	var i;for(i=0;i<=b.length;i++){matrix[i]=[i];}// increment each column in the first row
	var j;for(j=0;j<=a.length;j++){matrix[0][j]=j;}// Fill in the rest of the matrix
	for(i=1;i<=b.length;i++){for(j=1;j<=a.length;j++){if(b.charAt(i-1)==a.charAt(j-1)){matrix[i][j]=matrix[i-1][j-1];}else {matrix[i][j]=Math.min(matrix[i-1][j-1]+1,// substitution
	Math.min(matrix[i][j-1]+1,// insertion
	matrix[i-1][j]+1));// deletion
	}}}return matrix[b.length][a.length];}}]);return Validator;}();/**
	 * This object contains all possible options. It will check if the types are correct, if required if the option is one
	 * of the allowed values.
	 *
	 * __any__ means that the name of the property does not matter.
	 * __type__ is a required field for all objects and contains the allowed types of all objects
	 */var string$2='string';var bool$2='boolean';var number$2='number';var array$2='array';var date='date';var object$2='object';// should only be in a __type__ property
	var dom$1='dom';var moment$1$1='moment';var any$1='any';var allOptions$1$2={configure:{enabled:{'boolean':bool$2},filter:{'boolean':bool$2,'function':'function'},container:{dom:dom$1},__type__:{object:object$2,'boolean':bool$2,'function':'function'}},//globals :
	align:{string:string$2},alignCurrentTime:{string:string$2,'undefined':'undefined'},rtl:{'boolean':bool$2,'undefined':'undefined'},rollingMode:{follow:{'boolean':bool$2},offset:{number:number$2,'undefined':'undefined'},__type__:{object:object$2}},onTimeout:{timeoutMs:{number:number$2},callback:{'function':'function'},__type__:{object:object$2}},verticalScroll:{'boolean':bool$2,'undefined':'undefined'},horizontalScroll:{'boolean':bool$2,'undefined':'undefined'},autoResize:{'boolean':bool$2},throttleRedraw:{number:number$2},// TODO: DEPRICATED see https://github.com/almende/vis/issues/2511
	clickToUse:{'boolean':bool$2},dataAttributes:{string:string$2,array:array$2},editable:{add:{'boolean':bool$2,'undefined':'undefined'},remove:{'boolean':bool$2,'undefined':'undefined'},updateGroup:{'boolean':bool$2,'undefined':'undefined'},updateTime:{'boolean':bool$2,'undefined':'undefined'},overrideItems:{'boolean':bool$2,'undefined':'undefined'},__type__:{'boolean':bool$2,object:object$2}},end:{number:number$2,date:date,string:string$2,moment:moment$1$1},format:{minorLabels:{millisecond:{string:string$2,'undefined':'undefined'},second:{string:string$2,'undefined':'undefined'},minute:{string:string$2,'undefined':'undefined'},hour:{string:string$2,'undefined':'undefined'},weekday:{string:string$2,'undefined':'undefined'},day:{string:string$2,'undefined':'undefined'},week:{string:string$2,'undefined':'undefined'},month:{string:string$2,'undefined':'undefined'},year:{string:string$2,'undefined':'undefined'},__type__:{object:object$2,'function':'function'}},majorLabels:{millisecond:{string:string$2,'undefined':'undefined'},second:{string:string$2,'undefined':'undefined'},minute:{string:string$2,'undefined':'undefined'},hour:{string:string$2,'undefined':'undefined'},weekday:{string:string$2,'undefined':'undefined'},day:{string:string$2,'undefined':'undefined'},week:{string:string$2,'undefined':'undefined'},month:{string:string$2,'undefined':'undefined'},year:{string:string$2,'undefined':'undefined'},__type__:{object:object$2,'function':'function'}},__type__:{object:object$2}},moment:{'function':'function'},groupHeightMode:{string:string$2},groupOrder:{string:string$2,'function':'function'},groupEditable:{add:{'boolean':bool$2,'undefined':'undefined'},remove:{'boolean':bool$2,'undefined':'undefined'},order:{'boolean':bool$2,'undefined':'undefined'},__type__:{'boolean':bool$2,object:object$2}},groupOrderSwap:{'function':'function'},height:{string:string$2,number:number$2},hiddenDates:{start:{date:date,number:number$2,string:string$2,moment:moment$1$1},end:{date:date,number:number$2,string:string$2,moment:moment$1$1},repeat:{string:string$2},__type__:{object:object$2,array:array$2}},itemsAlwaysDraggable:{item:{'boolean':bool$2,'undefined':'undefined'},range:{'boolean':bool$2,'undefined':'undefined'},__type__:{'boolean':bool$2,object:object$2}},limitSize:{'boolean':bool$2},locale:{string:string$2},locales:{__any__:{any:any$1},__type__:{object:object$2}},longSelectPressTime:{number:number$2},margin:{axis:{number:number$2},item:{horizontal:{number:number$2,'undefined':'undefined'},vertical:{number:number$2,'undefined':'undefined'},__type__:{object:object$2,number:number$2}},__type__:{object:object$2,number:number$2}},max:{date:date,number:number$2,string:string$2,moment:moment$1$1},maxHeight:{number:number$2,string:string$2},maxMinorChars:{number:number$2},min:{date:date,number:number$2,string:string$2,moment:moment$1$1},minHeight:{number:number$2,string:string$2},moveable:{'boolean':bool$2},multiselect:{'boolean':bool$2},multiselectPerGroup:{'boolean':bool$2},onAdd:{'function':'function'},onDropObjectOnItem:{'function':'function'},onUpdate:{'function':'function'},onMove:{'function':'function'},onMoving:{'function':'function'},onRemove:{'function':'function'},onAddGroup:{'function':'function'},onMoveGroup:{'function':'function'},onRemoveGroup:{'function':'function'},onInitialDrawComplete:{'function':'function'},order:{'function':'function'},orientation:{axis:{string:string$2,'undefined':'undefined'},item:{string:string$2,'undefined':'undefined'},__type__:{string:string$2,object:object$2}},selectable:{'boolean':bool$2},sequentialSelection:{'boolean':bool$2},showCurrentTime:{'boolean':bool$2},showMajorLabels:{'boolean':bool$2},showMinorLabels:{'boolean':bool$2},showWeekScale:{'boolean':bool$2},stack:{'boolean':bool$2},stackSubgroups:{'boolean':bool$2},cluster:{maxItems:{'number':number$2,'undefined':'undefined'},titleTemplate:{'string':string$2,'undefined':'undefined'},clusterCriteria:{'function':'function','undefined':'undefined'},showStipes:{'boolean':bool$2,'undefined':'undefined'},fitOnDoubleClick:{'boolean':bool$2,'undefined':'undefined'},__type__:{'boolean':bool$2,object:object$2}},snap:{'function':'function','null':'null'},start:{date:date,number:number$2,string:string$2,moment:moment$1$1},template:{'function':'function'},loadingScreenTemplate:{'function':'function'},groupTemplate:{'function':'function'},visibleFrameTemplate:{string:string$2,'function':'function'},showTooltips:{'boolean':bool$2},tooltip:{followMouse:{'boolean':bool$2},overflowMethod:{'string':['cap','flip','none']},delay:{number:number$2},template:{'function':'function'},__type__:{object:object$2}},tooltipOnItemUpdateTime:{template:{'function':'function'},__type__:{'boolean':bool$2,object:object$2}},timeAxis:{scale:{string:string$2,'undefined':'undefined'},step:{number:number$2,'undefined':'undefined'},__type__:{object:object$2}},type:{string:string$2},width:{string:string$2,number:number$2},preferZoom:{'boolean':bool$2},zoomable:{'boolean':bool$2},zoomKey:{string:['ctrlKey','altKey','shiftKey','metaKey','']},zoomFriction:{number:number$2},zoomMax:{number:number$2},zoomMin:{number:number$2},__type__:{object:object$2}};var configureOptions$1={global:{align:['center','left','right'],alignCurrentTime:['none','year','month','quarter','week','isoWeek','day','date','hour','minute','second'],direction:false,autoResize:true,clickToUse:false,// dataAttributes: ['all'], // FIXME: can be 'all' or string[]
	editable:{add:false,remove:false,updateGroup:false,updateTime:false},end:'',format:{minorLabels:{millisecond:'SSS',second:'s',minute:'HH:mm',hour:'HH:mm',weekday:'ddd D',day:'D',week:'w',month:'MMM',year:'YYYY'},majorLabels:{millisecond:'HH:mm:ss',second:'D MMMM HH:mm',minute:'ddd D MMMM',hour:'ddd D MMMM',weekday:'MMMM YYYY',day:'MMMM YYYY',week:'MMMM YYYY',month:'YYYY',year:''}},groupHeightMode:['auto','fixed','fitItems'],//groupOrder: {string, 'function': 'function'},
	groupsDraggable:false,height:'',//hiddenDates: {object, array},
	locale:'',longSelectPressTime:251,margin:{axis:[20,0,100,1],item:{horizontal:[10,0,100,1],vertical:[10,0,100,1]}},max:'',maxHeight:'',maxMinorChars:[7,0,20,1],min:'',minHeight:'',moveable:false,multiselect:false,multiselectPerGroup:false,//onAdd: {'function': 'function'},
	//onUpdate: {'function': 'function'},
	//onMove: {'function': 'function'},
	//onMoving: {'function': 'function'},
	//onRename: {'function': 'function'},
	//order: {'function': 'function'},
	orientation:{axis:['both','bottom','top'],item:['bottom','top']},preferZoom:false,selectable:true,showCurrentTime:false,showMajorLabels:true,showMinorLabels:true,stack:true,stackSubgroups:true,cluster:false,//snap: {'function': 'function', nada},
	start:'',//template: {'function': 'function'},
	//timeAxis: {
	//  scale: ['millisecond', 'second', 'minute', 'hour', 'weekday', 'day', 'week', 'month', 'year'],
	//  step: [1, 1, 10, 1]
	//},
	showTooltips:true,tooltip:{followMouse:false,overflowMethod:'flip',delay:[500,0,99999,100]},tooltipOnItemUpdateTime:false,type:['box','point','range','background'],width:'100%',zoomable:true,zoomKey:['ctrlKey','altKey','shiftKey','metaKey',''],zoomMax:[315360000000000,10,315360000000000,1],zoomMin:[10,10,315360000000000,1]}};var htmlColors$1={black:'#000000',navy:'#000080',darkblue:'#00008B',mediumblue:'#0000CD',blue:'#0000FF',darkgreen:'#006400',green:'#008000',teal:'#008080',darkcyan:'#008B8B',deepskyblue:'#00BFFF',darkturquoise:'#00CED1',mediumspringgreen:'#00FA9A',lime:'#00FF00',springgreen:'#00FF7F',aqua:'#00FFFF',cyan:'#00FFFF',midnightblue:'#191970',dodgerblue:'#1E90FF',lightseagreen:'#20B2AA',forestgreen:'#228B22',seagreen:'#2E8B57',darkslategray:'#2F4F4F',limegreen:'#32CD32',mediumseagreen:'#3CB371',turquoise:'#40E0D0',royalblue:'#4169E1',steelblue:'#4682B4',darkslateblue:'#483D8B',mediumturquoise:'#48D1CC',indigo:'#4B0082',darkolivegreen:'#556B2F',cadetblue:'#5F9EA0',cornflowerblue:'#6495ED',mediumaquamarine:'#66CDAA',dimgray:'#696969',slateblue:'#6A5ACD',olivedrab:'#6B8E23',slategray:'#708090',lightslategray:'#778899',mediumslateblue:'#7B68EE',lawngreen:'#7CFC00',chartreuse:'#7FFF00',aquamarine:'#7FFFD4',maroon:'#800000',purple:'#800080',olive:'#808000',gray:'#808080',skyblue:'#87CEEB',lightskyblue:'#87CEFA',blueviolet:'#8A2BE2',darkred:'#8B0000',darkmagenta:'#8B008B',saddlebrown:'#8B4513',darkseagreen:'#8FBC8F',lightgreen:'#90EE90',mediumpurple:'#9370D8',darkviolet:'#9400D3',palegreen:'#98FB98',darkorchid:'#9932CC',yellowgreen:'#9ACD32',sienna:'#A0522D',brown:'#A52A2A',darkgray:'#A9A9A9',lightblue:'#ADD8E6',greenyellow:'#ADFF2F',paleturquoise:'#AFEEEE',lightsteelblue:'#B0C4DE',powderblue:'#B0E0E6',firebrick:'#B22222',darkgoldenrod:'#B8860B',mediumorchid:'#BA55D3',rosybrown:'#BC8F8F',darkkhaki:'#BDB76B',silver:'#C0C0C0',mediumvioletred:'#C71585',indianred:'#CD5C5C',peru:'#CD853F',chocolate:'#D2691E',tan:'#D2B48C',lightgrey:'#D3D3D3',palevioletred:'#D87093',thistle:'#D8BFD8',orchid:'#DA70D6',goldenrod:'#DAA520',crimson:'#DC143C',gainsboro:'#DCDCDC',plum:'#DDA0DD',burlywood:'#DEB887',lightcyan:'#E0FFFF',lavender:'#E6E6FA',darksalmon:'#E9967A',violet:'#EE82EE',palegoldenrod:'#EEE8AA',lightcoral:'#F08080',khaki:'#F0E68C',aliceblue:'#F0F8FF',honeydew:'#F0FFF0',azure:'#F0FFFF',sandybrown:'#F4A460',wheat:'#F5DEB3',beige:'#F5F5DC',whitesmoke:'#F5F5F5',mintcream:'#F5FFFA',ghostwhite:'#F8F8FF',salmon:'#FA8072',antiquewhite:'#FAEBD7',linen:'#FAF0E6',lightgoldenrodyellow:'#FAFAD2',oldlace:'#FDF5E6',red:'#FF0000',fuchsia:'#FF00FF',magenta:'#FF00FF',deeppink:'#FF1493',orangered:'#FF4500',tomato:'#FF6347',hotpink:'#FF69B4',coral:'#FF7F50',darkorange:'#FF8C00',lightsalmon:'#FFA07A',orange:'#FFA500',lightpink:'#FFB6C1',pink:'#FFC0CB',gold:'#FFD700',peachpuff:'#FFDAB9',navajowhite:'#FFDEAD',moccasin:'#FFE4B5',bisque:'#FFE4C4',mistyrose:'#FFE4E1',blanchedalmond:'#FFEBCD',papayawhip:'#FFEFD5',lavenderblush:'#FFF0F5',seashell:'#FFF5EE',cornsilk:'#FFF8DC',lemonchiffon:'#FFFACD',floralwhite:'#FFFAF0',snow:'#FFFAFA',yellow:'#FFFF00',lightyellow:'#FFFFE0',ivory:'#FFFFF0',white:'#FFFFFF'};/**
	 * @param {number} [pixelRatio=1]
	 */var ColorPicker$1=/*#__PURE__*/function(){/**
	   * @param {number} [pixelRatio=1]
	   */function ColorPicker(){var pixelRatio=arguments.length>0&&arguments[0]!==undefined?arguments[0]:1;classCallCheck(this,ColorPicker);this.pixelRatio=pixelRatio;this.generated=false;this.centerCoordinates={x:289/2,y:289/2};this.r=289*0.49;this.color={r:255,g:255,b:255,a:1.0};this.hueCircle=undefined;this.initialColor={r:255,g:255,b:255,a:1.0};this.previousColor=undefined;this.applied=false;// bound by
	this.updateCallback=function(){};this.closeCallback=function(){};// create all DOM elements
	this._create();}/**
	   * this inserts the colorPicker into a div from the DOM
	   * @param {Element} container
	   */createClass(ColorPicker,[{key:"insertTo",value:function insertTo(container){if(this.hammer!==undefined){this.hammer.destroy();this.hammer=undefined;}this.container=container;this.container.appendChild(this.frame);this._bindHammer();this._setSize();}/**
	   * the callback is executed on apply and save. Bind it to the application
	   * @param {function} callback
	   */},{key:"setUpdateCallback",value:function setUpdateCallback(callback){if(typeof callback==='function'){this.updateCallback=callback;}else {throw new Error("Function attempted to set as colorPicker update callback is not a function.");}}/**
	   * the callback is executed on apply and save. Bind it to the application
	   * @param {function} callback
	   */},{key:"setCloseCallback",value:function setCloseCallback(callback){if(typeof callback==='function'){this.closeCallback=callback;}else {throw new Error("Function attempted to set as colorPicker closing callback is not a function.");}}/**
	   *
	   * @param {string} color
	   * @returns {String}
	   * @private
	   */},{key:"_isColorString",value:function _isColorString(color){if(typeof color==='string'){return htmlColors$1[color];}}/**
	   * Set the color of the colorPicker
	   * Supported formats:
	   * 'red'                   --> HTML color string
	   * '#ffffff'               --> hex string
	   * 'rgb(255,255,255)'      --> rgb string
	   * 'rgba(255,255,255,1.0)' --> rgba string
	   * {r:255,g:255,b:255}     --> rgb object
	   * {r:255,g:255,b:255,a:1.0} --> rgba object
	   * @param {string|Object} color
	   * @param {boolean} [setInitial=true]
	   */},{key:"setColor",value:function setColor(color){var setInitial=arguments.length>1&&arguments[1]!==undefined?arguments[1]:true;if(color==='none'){return;}var rgba;// if a html color shorthand is used, convert to hex
	var htmlColor=this._isColorString(color);if(htmlColor!==undefined){color=htmlColor;}// check format
	if(util$1.isString(color)===true){if(util$1.isValidRGB(color)===true){var rgbaArray=color.substr(4).substr(0,color.length-5).split(',');rgba={r:rgbaArray[0],g:rgbaArray[1],b:rgbaArray[2],a:1.0};}else if(util$1.isValidRGBA(color)===true){var _rgbaArray=color.substr(5).substr(0,color.length-6).split(',');rgba={r:_rgbaArray[0],g:_rgbaArray[1],b:_rgbaArray[2],a:_rgbaArray[3]};}else if(util$1.isValidHex(color)===true){var rgbObj=util$1.hexToRGB(color);rgba={r:rgbObj.r,g:rgbObj.g,b:rgbObj.b,a:1.0};}}else {if(color instanceof Object){if(color.r!==undefined&&color.g!==undefined&&color.b!==undefined){var alpha=color.a!==undefined?color.a:'1.0';rgba={r:color.r,g:color.g,b:color.b,a:alpha};}}}// set color
	if(rgba===undefined){throw new Error("Unknown color passed to the colorPicker. Supported are strings: rgb, hex, rgba. Object: rgb ({r:r,g:g,b:b,[a:a]}). Supplied: "+stringify$2(color));}else {this._setColor(rgba,setInitial);}}/**
	   * this shows the color picker.
	   * The hue circle is constructed once and stored.
	   */},{key:"show",value:function show(){if(this.closeCallback!==undefined){this.closeCallback();this.closeCallback=undefined;}this.applied=false;this.frame.style.display='block';this._generateHueCircle();}// ------------------------------------------ PRIVATE ----------------------------- //
	/**
	   * Hide the picker. Is called by the cancel button.
	   * Optional boolean to store the previous color for easy access later on.
	   * @param {boolean} [storePrevious=true]
	   * @private
	   */},{key:"_hide",value:function _hide(){var _this36=this;var storePrevious=arguments.length>0&&arguments[0]!==undefined?arguments[0]:true;// store the previous color for next time;
	if(storePrevious===true){this.previousColor=util$1.extend({},this.color);}if(this.applied===true){this.updateCallback(this.initialColor);}this.frame.style.display='none';// call the closing callback, restoring the onclick method.
	// this is in a setTimeout because it will trigger the show again before the click is done.
	setTimeout$2(function(){if(_this36.closeCallback!==undefined){_this36.closeCallback();_this36.closeCallback=undefined;}},0);}/**
	   * bound to the save button. Saves and hides.
	   * @private
	   */},{key:"_save",value:function _save(){this.updateCallback(this.color);this.applied=false;this._hide();}/**
	   * Bound to apply button. Saves but does not close. Is undone by the cancel button.
	   * @private
	   */},{key:"_apply",value:function _apply(){this.applied=true;this.updateCallback(this.color);this._updatePicker(this.color);}/**
	   * load the color from the previous session.
	   * @private
	   */},{key:"_loadLast",value:function _loadLast(){if(this.previousColor!==undefined){this.setColor(this.previousColor,false);}else {alert("There is no last color to load...");}}/**
	   * set the color, place the picker
	   * @param {Object} rgba
	   * @param {boolean} [setInitial=true]
	   * @private
	   */},{key:"_setColor",value:function _setColor(rgba){var setInitial=arguments.length>1&&arguments[1]!==undefined?arguments[1]:true;// store the initial color
	if(setInitial===true){this.initialColor=util$1.extend({},rgba);}this.color=rgba;var hsv=util$1.RGBToHSV(rgba.r,rgba.g,rgba.b);var angleConvert=2*Math.PI;var radius=this.r*hsv.s;var x=this.centerCoordinates.x+radius*Math.sin(angleConvert*hsv.h);var y=this.centerCoordinates.y+radius*Math.cos(angleConvert*hsv.h);this.colorPickerSelector.style.left=x-0.5*this.colorPickerSelector.clientWidth+'px';this.colorPickerSelector.style.top=y-0.5*this.colorPickerSelector.clientHeight+'px';this._updatePicker(rgba);}/**
	   * bound to opacity control
	   * @param {number} value
	   * @private
	   */},{key:"_setOpacity",value:function _setOpacity(value){this.color.a=value/100;this._updatePicker(this.color);}/**
	   * bound to brightness control
	   * @param {number} value
	   * @private
	   */},{key:"_setBrightness",value:function _setBrightness(value){var hsv=util$1.RGBToHSV(this.color.r,this.color.g,this.color.b);hsv.v=value/100;var rgba=util$1.HSVToRGB(hsv.h,hsv.s,hsv.v);rgba['a']=this.color.a;this.color=rgba;this._updatePicker();}/**
	   * update the color picker. A black circle overlays the hue circle to mimic the brightness decreasing.
	   * @param {Object} rgba
	   * @private
	   */},{key:"_updatePicker",value:function _updatePicker(){var rgba=arguments.length>0&&arguments[0]!==undefined?arguments[0]:this.color;var hsv=util$1.RGBToHSV(rgba.r,rgba.g,rgba.b);var ctx=this.colorPickerCanvas.getContext('2d');if(this.pixelRation===undefined){this.pixelRatio=(window.devicePixelRatio||1)/(ctx.webkitBackingStorePixelRatio||ctx.mozBackingStorePixelRatio||ctx.msBackingStorePixelRatio||ctx.oBackingStorePixelRatio||ctx.backingStorePixelRatio||1);}ctx.setTransform(this.pixelRatio,0,0,this.pixelRatio,0,0);// clear the canvas
	var w=this.colorPickerCanvas.clientWidth;var h=this.colorPickerCanvas.clientHeight;ctx.clearRect(0,0,w,h);ctx.putImageData(this.hueCircle,0,0);ctx.fillStyle='rgba(0,0,0,'+(1-hsv.v)+')';ctx.circle(this.centerCoordinates.x,this.centerCoordinates.y,this.r);fill$2(ctx).call(ctx);this.brightnessRange.value=100*hsv.v;this.opacityRange.value=100*rgba.a;this.initialColorDiv.style.backgroundColor='rgba('+this.initialColor.r+','+this.initialColor.g+','+this.initialColor.b+','+this.initialColor.a+')';this.newColorDiv.style.backgroundColor='rgba('+this.color.r+','+this.color.g+','+this.color.b+','+this.color.a+')';}/**
	   * used by create to set the size of the canvas.
	   * @private
	   */},{key:"_setSize",value:function _setSize(){this.colorPickerCanvas.style.width='100%';this.colorPickerCanvas.style.height='100%';this.colorPickerCanvas.width=289*this.pixelRatio;this.colorPickerCanvas.height=289*this.pixelRatio;}/**
	   * create all dom elements
	   * TODO: cleanup, lots of similar dom elements
	   * @private
	   */},{key:"_create",value:function _create(){var _context170,_context171,_context172,_context173;this.frame=document.createElement('div');this.frame.className='vis-color-picker';this.colorPickerDiv=document.createElement('div');this.colorPickerSelector=document.createElement('div');this.colorPickerSelector.className='vis-selector';this.colorPickerDiv.appendChild(this.colorPickerSelector);this.colorPickerCanvas=document.createElement('canvas');this.colorPickerDiv.appendChild(this.colorPickerCanvas);if(!this.colorPickerCanvas.getContext){var noCanvas=document.createElement('DIV');noCanvas.style.color='red';noCanvas.style.fontWeight='bold';noCanvas.style.padding='10px';noCanvas.innerHTML='Error: your browser does not support HTML canvas';this.colorPickerCanvas.appendChild(noCanvas);}else {var ctx=this.colorPickerCanvas.getContext("2d");this.pixelRatio=(window.devicePixelRatio||1)/(ctx.webkitBackingStorePixelRatio||ctx.mozBackingStorePixelRatio||ctx.msBackingStorePixelRatio||ctx.oBackingStorePixelRatio||ctx.backingStorePixelRatio||1);this.colorPickerCanvas.getContext("2d").setTransform(this.pixelRatio,0,0,this.pixelRatio,0,0);}this.colorPickerDiv.className='vis-color';this.opacityDiv=document.createElement('div');this.opacityDiv.className='vis-opacity';this.brightnessDiv=document.createElement('div');this.brightnessDiv.className='vis-brightness';this.arrowDiv=document.createElement('div');this.arrowDiv.className='vis-arrow';this.opacityRange=document.createElement('input');try{this.opacityRange.type='range';// Not supported on IE9
	this.opacityRange.min='0';this.opacityRange.max='100';}// TODO: Add some error handling and remove this lint exception
	catch(err){}// eslint-disable-line no-empty
	this.opacityRange.value='100';this.opacityRange.className='vis-range';this.brightnessRange=document.createElement('input');try{this.brightnessRange.type='range';// Not supported on IE9
	this.brightnessRange.min='0';this.brightnessRange.max='100';}// TODO: Add some error handling and remove this lint exception
	catch(err){}// eslint-disable-line no-empty
	this.brightnessRange.value='100';this.brightnessRange.className='vis-range';this.opacityDiv.appendChild(this.opacityRange);this.brightnessDiv.appendChild(this.brightnessRange);var me=this;this.opacityRange.onchange=function(){me._setOpacity(this.value);};this.opacityRange.oninput=function(){me._setOpacity(this.value);};this.brightnessRange.onchange=function(){me._setBrightness(this.value);};this.brightnessRange.oninput=function(){me._setBrightness(this.value);};this.brightnessLabel=document.createElement("div");this.brightnessLabel.className="vis-label vis-brightness";this.brightnessLabel.innerHTML='brightness:';this.opacityLabel=document.createElement("div");this.opacityLabel.className="vis-label vis-opacity";this.opacityLabel.innerHTML='opacity:';this.newColorDiv=document.createElement("div");this.newColorDiv.className="vis-new-color";this.newColorDiv.innerHTML='new';this.initialColorDiv=document.createElement("div");this.initialColorDiv.className="vis-initial-color";this.initialColorDiv.innerHTML='initial';this.cancelButton=document.createElement("div");this.cancelButton.className="vis-button vis-cancel";this.cancelButton.innerHTML='cancel';this.cancelButton.onclick=bind$2(_context170=this._hide).call(_context170,this,false);this.applyButton=document.createElement("div");this.applyButton.className="vis-button vis-apply";this.applyButton.innerHTML='apply';this.applyButton.onclick=bind$2(_context171=this._apply).call(_context171,this);this.saveButton=document.createElement("div");this.saveButton.className="vis-button vis-save";this.saveButton.innerHTML='save';this.saveButton.onclick=bind$2(_context172=this._save).call(_context172,this);this.loadButton=document.createElement("div");this.loadButton.className="vis-button vis-load";this.loadButton.innerHTML='load last';this.loadButton.onclick=bind$2(_context173=this._loadLast).call(_context173,this);this.frame.appendChild(this.colorPickerDiv);this.frame.appendChild(this.arrowDiv);this.frame.appendChild(this.brightnessLabel);this.frame.appendChild(this.brightnessDiv);this.frame.appendChild(this.opacityLabel);this.frame.appendChild(this.opacityDiv);this.frame.appendChild(this.newColorDiv);this.frame.appendChild(this.initialColorDiv);this.frame.appendChild(this.cancelButton);this.frame.appendChild(this.applyButton);this.frame.appendChild(this.saveButton);this.frame.appendChild(this.loadButton);}/**
	   * bind hammer to the color picker
	   * @private
	   */},{key:"_bindHammer",value:function _bindHammer(){var _this37=this;this.drag={};this.pinch={};this.hammer=new Hammer$1(this.colorPickerCanvas);this.hammer.get('pinch').set({enable:true});onTouch(this.hammer,function(event){_this37._moveSelector(event);});this.hammer.on('tap',function(event){_this37._moveSelector(event);});this.hammer.on('panstart',function(event){_this37._moveSelector(event);});this.hammer.on('panmove',function(event){_this37._moveSelector(event);});this.hammer.on('panend',function(event){_this37._moveSelector(event);});}/**
	   * generate the hue circle. This is relatively heavy (200ms) and is done only once on the first time it is shown.
	   * @private
	   */},{key:"_generateHueCircle",value:function _generateHueCircle(){if(this.generated===false){var ctx=this.colorPickerCanvas.getContext('2d');if(this.pixelRation===undefined){this.pixelRatio=(window.devicePixelRatio||1)/(ctx.webkitBackingStorePixelRatio||ctx.mozBackingStorePixelRatio||ctx.msBackingStorePixelRatio||ctx.oBackingStorePixelRatio||ctx.backingStorePixelRatio||1);}ctx.setTransform(this.pixelRatio,0,0,this.pixelRatio,0,0);// clear the canvas
	var w=this.colorPickerCanvas.clientWidth;var h=this.colorPickerCanvas.clientHeight;ctx.clearRect(0,0,w,h);// draw hue circle
	var x,y,hue,sat;this.centerCoordinates={x:w*0.5,y:h*0.5};this.r=0.49*w;var angleConvert=2*Math.PI/360;var hfac=1/360;var sfac=1/this.r;var rgb;for(hue=0;hue<360;hue++){for(sat=0;sat<this.r;sat++){x=this.centerCoordinates.x+sat*Math.sin(angleConvert*hue);y=this.centerCoordinates.y+sat*Math.cos(angleConvert*hue);rgb=util$1.HSVToRGB(hue*hfac,sat*sfac,1);ctx.fillStyle='rgb('+rgb.r+','+rgb.g+','+rgb.b+')';ctx.fillRect(x-0.5,y-0.5,2,2);}}ctx.strokeStyle='rgba(0,0,0,1)';ctx.circle(this.centerCoordinates.x,this.centerCoordinates.y,this.r);ctx.stroke();this.hueCircle=ctx.getImageData(0,0,w,h);}this.generated=true;}/**
	   * move the selector. This is called by hammer functions.
	   *
	   * @param {Event}  event   The event
	   * @private
	   */},{key:"_moveSelector",value:function _moveSelector(event){var rect=this.colorPickerDiv.getBoundingClientRect();var left=event.center.x-rect.left;var top=event.center.y-rect.top;var centerY=0.5*this.colorPickerDiv.clientHeight;var centerX=0.5*this.colorPickerDiv.clientWidth;var x=left-centerX;var y=top-centerY;var angle=Math.atan2(x,y);var radius=0.98*Math.min(Math.sqrt(x*x+y*y),centerX);var newTop=Math.cos(angle)*radius+centerY;var newLeft=Math.sin(angle)*radius+centerX;this.colorPickerSelector.style.top=newTop-0.5*this.colorPickerSelector.clientHeight+'px';this.colorPickerSelector.style.left=newLeft-0.5*this.colorPickerSelector.clientWidth+'px';// set color
	var h=angle/(2*Math.PI);h=h<0?h+1:h;var s=radius/this.r;var hsv=util$1.RGBToHSV(this.color.r,this.color.g,this.color.b);hsv.h=h;hsv.s=s;var rgba=util$1.HSVToRGB(hsv.h,hsv.s,hsv.v);rgba['a']=this.color.a;this.color=rgba;// update previews
	this.initialColorDiv.style.backgroundColor='rgba('+this.initialColor.r+','+this.initialColor.g+','+this.initialColor.b+','+this.initialColor.a+')';this.newColorDiv.style.backgroundColor='rgba('+this.color.r+','+this.color.g+','+this.color.b+','+this.color.a+')';}}]);return ColorPicker;}();/**
	 * The way this works is for all properties of this.possible options, you can supply the property name in any form to list the options.
	 * Boolean options are recognised as Boolean
	 * Number options should be written as array: [default value, min value, max value, stepsize]
	 * Colors should be written as array: ['color', '#ffffff']
	 * Strings with should be written as array: [option1, option2, option3, ..]
	 *
	 * The options are matched with their counterparts in each of the modules and the values used in the configuration are
	 */var Configurator$1=/*#__PURE__*/function(){/**
	   * @param {Object} parentModule        | the location where parentModule.setOptions() can be called
	   * @param {Object} defaultContainer    | the default container of the module
	   * @param {Object} configureOptions    | the fully configured and predefined options set found in allOptions.js
	   * @param {number} pixelRatio          | canvas pixel ratio
	   */function Configurator(parentModule,defaultContainer,configureOptions){var pixelRatio=arguments.length>3&&arguments[3]!==undefined?arguments[3]:1;classCallCheck(this,Configurator);this.parent=parentModule;this.changedOptions=[];this.container=defaultContainer;this.allowCreation=false;this.options={};this.initialized=false;this.popupCounter=0;this.defaultOptions={enabled:false,filter:true,container:undefined,showButton:true};util$1.extend(this.options,this.defaultOptions);this.configureOptions=configureOptions;this.moduleOptions={};this.domElements=[];this.popupDiv={};this.popupLimit=5;this.popupHistory={};this.colorPicker=new ColorPicker$1(pixelRatio);this.wrapper=undefined;}/**
	   * refresh all options.
	   * Because all modules parse their options by themselves, we just use their options. We copy them here.
	   *
	   * @param {Object} options
	   */createClass(Configurator,[{key:"setOptions",value:function setOptions(options){if(options!==undefined){// reset the popup history because the indices may have been changed.
	this.popupHistory={};this._removePopup();var enabled=true;if(typeof options==='string'){this.options.filter=options;}else if(options instanceof Array){this.options.filter=options.join();}else if(_typeof_1(options)==='object'){if(options==null){throw new TypeError('options cannot be null');}if(options.container!==undefined){this.options.container=options.container;}if(filter$2(options)!==undefined){this.options.filter=filter$2(options);}if(options.showButton!==undefined){this.options.showButton=options.showButton;}if(options.enabled!==undefined){enabled=options.enabled;}}else if(typeof options==='boolean'){this.options.filter=true;enabled=options;}else if(typeof options==='function'){this.options.filter=options;enabled=true;}if(filter$2(this.options)===false){enabled=false;}this.options.enabled=enabled;}this._clean();}/**
	   *
	   * @param {Object} moduleOptions
	   */},{key:"setModuleOptions",value:function setModuleOptions(moduleOptions){this.moduleOptions=moduleOptions;if(this.options.enabled===true){this._clean();if(this.options.container!==undefined){this.container=this.options.container;}this._create();}}/**
	   * Create all DOM elements
	   * @private
	   */},{key:"_create",value:function _create(){this._clean();this.changedOptions=[];var filter=filter$2(this.options);var counter=0;var show=false;for(var option in this.configureOptions){if(this.configureOptions.hasOwnProperty(option)){this.allowCreation=false;show=false;if(typeof filter==='function'){show=filter(option,[]);show=show||this._handleObject(this.configureOptions[option],[option],true);}else if(filter===true||indexOf$3(filter).call(filter,option)!==-1){show=true;}if(show!==false){this.allowCreation=true;// linebreak between categories
	if(counter>0){this._makeItem([]);}// a header for the category
	this._makeHeader(option);// get the sub options
	this._handleObject(this.configureOptions[option],[option]);}counter++;}}this._makeButton();this._push();//~ this.colorPicker.insertTo(this.container);
	}/**
	   * draw all DOM elements on the screen
	   * @private
	   */},{key:"_push",value:function _push(){this.wrapper=document.createElement('div');this.wrapper.className='vis-configuration-wrapper';this.container.appendChild(this.wrapper);for(var i=0;i<this.domElements.length;i++){this.wrapper.appendChild(this.domElements[i]);}this._showPopupIfNeeded();}/**
	   * delete all DOM elements
	   * @private
	   */},{key:"_clean",value:function _clean(){for(var i=0;i<this.domElements.length;i++){this.wrapper.removeChild(this.domElements[i]);}if(this.wrapper!==undefined){this.container.removeChild(this.wrapper);this.wrapper=undefined;}this.domElements=[];this._removePopup();}/**
	   * get the value from the actualOptions if it exists
	   * @param {array} path    | where to look for the actual option
	   * @returns {*}
	   * @private
	   */},{key:"_getValue",value:function _getValue(path){var base=this.moduleOptions;for(var i=0;i<path.length;i++){if(base[path[i]]!==undefined){base=base[path[i]];}else {base=undefined;break;}}return base;}/**
	   * all option elements are wrapped in an item
	   * @param {Array} path    | where to look for the actual option
	   * @param {Array.<Element>} domElements
	   * @returns {number}
	   * @private
	   */},{key:"_makeItem",value:function _makeItem(path){if(this.allowCreation===true){var _item16=document.createElement('div');_item16.className='vis-configuration vis-config-item vis-config-s'+path.length;for(var _len=arguments.length,domElements=new Array(_len>1?_len-1:0),_key=1;_key<_len;_key++){domElements[_key-1]=arguments[_key];}forEach$3(domElements).call(domElements,function(element){_item16.appendChild(element);});this.domElements.push(_item16);return this.domElements.length;}return 0;}/**
	   * header for major subjects
	   * @param {string} name
	   * @private
	   */},{key:"_makeHeader",value:function _makeHeader(name){var div=document.createElement('div');div.className='vis-configuration vis-config-header';div.innerHTML=name;this._makeItem([],div);}/**
	   * make a label, if it is an object label, it gets different styling.
	   * @param {string} name
	   * @param {array} path    | where to look for the actual option
	   * @param {string} objectLabel
	   * @returns {HTMLElement}
	   * @private
	   */},{key:"_makeLabel",value:function _makeLabel(name,path){var objectLabel=arguments.length>2&&arguments[2]!==undefined?arguments[2]:false;var div=document.createElement('div');div.className='vis-configuration vis-config-label vis-config-s'+path.length;if(objectLabel===true){div.innerHTML='<i><b>'+name+':</b></i>';}else {div.innerHTML=name+':';}return div;}/**
	   * make a dropdown list for multiple possible string optoins
	   * @param {Array.<number>} arr
	   * @param {number} value
	   * @param {array} path    | where to look for the actual option
	   * @private
	   */},{key:"_makeDropdown",value:function _makeDropdown(arr,value,path){var select=document.createElement('select');select.className='vis-configuration vis-config-select';var selectedValue=0;if(value!==undefined){if(indexOf$3(arr).call(arr,value)!==-1){selectedValue=indexOf$3(arr).call(arr,value);}}for(var i=0;i<arr.length;i++){var option=document.createElement('option');option.value=arr[i];if(i===selectedValue){option.selected='selected';}option.innerHTML=arr[i];select.appendChild(option);}var me=this;select.onchange=function(){me._update(this.value,path);};var label=this._makeLabel(path[path.length-1],path);this._makeItem(path,label,select);}/**
	   * make a range object for numeric options
	   * @param {Array.<number>} arr
	   * @param {number} value
	   * @param {array} path    | where to look for the actual option
	   * @private
	   */},{key:"_makeRange",value:function _makeRange(arr,value,path){var defaultValue=arr[0];var min=arr[1];var max=arr[2];var step=arr[3];var range=document.createElement('input');range.className='vis-configuration vis-config-range';try{range.type='range';// not supported on IE9
	range.min=min;range.max=max;}// TODO: Add some error handling and remove this lint exception
	catch(err){}// eslint-disable-line no-empty
	range.step=step;// set up the popup settings in case they are needed.
	var popupString='';var popupValue=0;if(value!==undefined){var factor=1.20;if(value<0&&value*factor<min){range.min=Math.ceil(value*factor);popupValue=range.min;popupString='range increased';}else if(value/factor<min){range.min=Math.ceil(value/factor);popupValue=range.min;popupString='range increased';}if(value*factor>max&&max!==1){range.max=Math.ceil(value*factor);popupValue=range.max;popupString='range increased';}range.value=value;}else {range.value=defaultValue;}var input=document.createElement('input');input.className='vis-configuration vis-config-rangeinput';input.value=Number(range.value);var me=this;range.onchange=function(){input.value=this.value;me._update(Number(this.value),path);};range.oninput=function(){input.value=this.value;};var label=this._makeLabel(path[path.length-1],path);var itemIndex=this._makeItem(path,label,range,input);// if a popup is needed AND it has not been shown for this value, show it.
	if(popupString!==''&&this.popupHistory[itemIndex]!==popupValue){this.popupHistory[itemIndex]=popupValue;this._setupPopup(popupString,itemIndex);}}/**
	   * make a button object
	   * @private
	   */},{key:"_makeButton",value:function _makeButton(){var _this38=this;if(this.options.showButton===true){var generateButton=document.createElement('div');generateButton.className='vis-configuration vis-config-button';generateButton.innerHTML='generate options';generateButton.onclick=function(){_this38._printOptions();};generateButton.onmouseover=function(){generateButton.className='vis-configuration vis-config-button hover';};generateButton.onmouseout=function(){generateButton.className='vis-configuration vis-config-button';};this.optionsContainer=document.createElement('div');this.optionsContainer.className='vis-configuration vis-config-option-container';this.domElements.push(this.optionsContainer);this.domElements.push(generateButton);}}/**
	   * prepare the popup
	   * @param {string} string
	   * @param {number} index
	   * @private
	   */},{key:"_setupPopup",value:function _setupPopup(string,index){var _this39=this;if(this.initialized===true&&this.allowCreation===true&&this.popupCounter<this.popupLimit){var div=document.createElement("div");div.id="vis-configuration-popup";div.className="vis-configuration-popup";div.innerHTML=string;div.onclick=function(){_this39._removePopup();};this.popupCounter+=1;this.popupDiv={html:div,index:index};}}/**
	   * remove the popup from the dom
	   * @private
	   */},{key:"_removePopup",value:function _removePopup(){if(this.popupDiv.html!==undefined){this.popupDiv.html.parentNode.removeChild(this.popupDiv.html);clearTimeout(this.popupDiv.hideTimeout);clearTimeout(this.popupDiv.deleteTimeout);this.popupDiv={};}}/**
	   * Show the popup if it is needed.
	   * @private
	   */},{key:"_showPopupIfNeeded",value:function _showPopupIfNeeded(){var _this40=this;if(this.popupDiv.html!==undefined){var correspondingElement=this.domElements[this.popupDiv.index];var rect=correspondingElement.getBoundingClientRect();this.popupDiv.html.style.left=rect.left+"px";this.popupDiv.html.style.top=rect.top-30+"px";// 30 is the height;
	document.body.appendChild(this.popupDiv.html);this.popupDiv.hideTimeout=setTimeout$2(function(){_this40.popupDiv.html.style.opacity=0;},1500);this.popupDiv.deleteTimeout=setTimeout$2(function(){_this40._removePopup();},1800);}}/**
	   * make a checkbox for boolean options.
	   * @param {number} defaultValue
	   * @param {number} value
	   * @param {array} path    | where to look for the actual option
	   * @private
	   */},{key:"_makeCheckbox",value:function _makeCheckbox(defaultValue,value,path){var checkbox=document.createElement('input');checkbox.type='checkbox';checkbox.className='vis-configuration vis-config-checkbox';checkbox.checked=defaultValue;if(value!==undefined){checkbox.checked=value;if(value!==defaultValue){if(_typeof_1(defaultValue)==='object'){if(value!==defaultValue.enabled){this.changedOptions.push({path:path,value:value});}}else {this.changedOptions.push({path:path,value:value});}}}var me=this;checkbox.onchange=function(){me._update(this.checked,path);};var label=this._makeLabel(path[path.length-1],path);this._makeItem(path,label,checkbox);}/**
	   * make a text input field for string options.
	   * @param {number} defaultValue
	   * @param {number} value
	   * @param {array} path    | where to look for the actual option
	   * @private
	   */},{key:"_makeTextInput",value:function _makeTextInput(defaultValue,value,path){var checkbox=document.createElement('input');checkbox.type='text';checkbox.className='vis-configuration vis-config-text';checkbox.value=value;if(value!==defaultValue){this.changedOptions.push({path:path,value:value});}var me=this;checkbox.onchange=function(){me._update(this.value,path);};var label=this._makeLabel(path[path.length-1],path);this._makeItem(path,label,checkbox);}/**
	   * make a color field with a color picker for color fields
	   * @param {Array.<number>} arr
	   * @param {number} value
	   * @param {array} path    | where to look for the actual option
	   * @private
	   */},{key:"_makeColorField",value:function _makeColorField(arr,value,path){var _this41=this;var defaultColor=arr[1];var div=document.createElement('div');value=value===undefined?defaultColor:value;if(value!=='none'){div.className='vis-configuration vis-config-colorBlock';div.style.backgroundColor=value;}else {div.className='vis-configuration vis-config-colorBlock none';}value=value===undefined?defaultColor:value;div.onclick=function(){_this41._showColorPicker(value,div,path);};var label=this._makeLabel(path[path.length-1],path);this._makeItem(path,label,div);}/**
	   * used by the color buttons to call the color picker.
	   * @param {number} value
	   * @param {HTMLElement} div
	   * @param {array} path    | where to look for the actual option
	   * @private
	   */},{key:"_showColorPicker",value:function _showColorPicker(value,div,path){var _this42=this;// clear the callback from this div
	div.onclick=function(){};this.colorPicker.insertTo(div);this.colorPicker.show();this.colorPicker.setColor(value);this.colorPicker.setUpdateCallback(function(color){var colorString='rgba('+color.r+','+color.g+','+color.b+','+color.a+')';div.style.backgroundColor=colorString;_this42._update(colorString,path);});// on close of the colorpicker, restore the callback.
	this.colorPicker.setCloseCallback(function(){div.onclick=function(){_this42._showColorPicker(value,div,path);};});}/**
	   * parse an object and draw the correct items
	   * @param {Object} obj
	   * @param {array} [path=[]]    | where to look for the actual option
	   * @param {boolean} [checkOnly=false]
	   * @returns {boolean}
	   * @private
	   */},{key:"_handleObject",value:function _handleObject(obj){var path=arguments.length>1&&arguments[1]!==undefined?arguments[1]:[];var checkOnly=arguments.length>2&&arguments[2]!==undefined?arguments[2]:false;var show=false;var filter=filter$2(this.options);var visibleInSet=false;for(var subObj in obj){if(obj.hasOwnProperty(subObj)){show=true;var _item17=obj[subObj];var newPath=util$1.copyAndExtendArray(path,subObj);if(typeof filter==='function'){show=filter(subObj,path);// if needed we must go deeper into the object.
	if(show===false){if(!(_item17 instanceof Array)&&typeof _item17!=='string'&&typeof _item17!=='boolean'&&_item17 instanceof Object){this.allowCreation=false;show=this._handleObject(_item17,newPath,true);this.allowCreation=checkOnly===false;}}}if(show!==false){visibleInSet=true;var value=this._getValue(newPath);if(_item17 instanceof Array){this._handleArray(_item17,value,newPath);}else if(typeof _item17==='string'){this._makeTextInput(_item17,value,newPath);}else if(typeof _item17==='boolean'){this._makeCheckbox(_item17,value,newPath);}else if(_item17 instanceof Object){// collapse the physics options that are not enabled
	var draw=true;if(indexOf$3(path).call(path,'physics')!==-1){if(this.moduleOptions.physics.solver!==subObj){draw=false;}}if(draw===true){// initially collapse options with an disabled enabled option.
	if(_item17.enabled!==undefined){var enabledPath=util$1.copyAndExtendArray(newPath,'enabled');var enabledValue=this._getValue(enabledPath);if(enabledValue===true){var label=this._makeLabel(subObj,newPath,true);this._makeItem(newPath,label);visibleInSet=this._handleObject(_item17,newPath)||visibleInSet;}else {this._makeCheckbox(_item17,enabledValue,newPath);}}else {var _label=this._makeLabel(subObj,newPath,true);this._makeItem(newPath,_label);visibleInSet=this._handleObject(_item17,newPath)||visibleInSet;}}}else {console.error('dont know how to handle',_item17,subObj,newPath);}}}}return visibleInSet;}/**
	   * handle the array type of option
	   * @param {Array.<number>} arr
	   * @param {number} value
	   * @param {array} path    | where to look for the actual option
	   * @private
	   */},{key:"_handleArray",value:function _handleArray(arr,value,path){if(typeof arr[0]==='string'&&arr[0]==='color'){this._makeColorField(arr,value,path);if(arr[1]!==value){this.changedOptions.push({path:path,value:value});}}else if(typeof arr[0]==='string'){this._makeDropdown(arr,value,path);if(arr[0]!==value){this.changedOptions.push({path:path,value:value});}}else if(typeof arr[0]==='number'){this._makeRange(arr,value,path);if(arr[0]!==value){this.changedOptions.push({path:path,value:Number(value)});}}}/**
	   * called to update the network with the new settings.
	   * @param {number} value
	   * @param {array} path    | where to look for the actual option
	   * @private
	   */},{key:"_update",value:function _update(value,path){var options=this._constructOptions(value,path);if(this.parent.body&&this.parent.body.emitter&&this.parent.body.emitter.emit){this.parent.body.emitter.emit("configChange",options);}this.initialized=true;this.parent.setOptions(options);}/**
	   *
	   * @param {string|Boolean} value
	   * @param {Array.<string>} path
	   * @param {{}} optionsObj
	   * @returns {{}}
	   * @private
	   */},{key:"_constructOptions",value:function _constructOptions(value,path){var optionsObj=arguments.length>2&&arguments[2]!==undefined?arguments[2]:{};var pointer=optionsObj;// when dropdown boxes can be string or boolean, we typecast it into correct types
	value=value==='true'?true:value;value=value==='false'?false:value;for(var i=0;i<path.length;i++){if(path[i]!=='global'){if(pointer[path[i]]===undefined){pointer[path[i]]={};}if(i!==path.length-1){pointer=pointer[path[i]];}else {pointer[path[i]]=value;}}}return optionsObj;}/**
	   * @private
	   */},{key:"_printOptions",value:function _printOptions(){var options=this.getOptions();this.optionsContainer.innerHTML='<pre>var options = '+stringify$2(options,null,2)+'</pre>';}/**
	   *
	   * @returns {{}} options
	   */},{key:"getOptions",value:function getOptions(){var options={};for(var i=0;i<this.changedOptions.length;i++){this._constructOptions(this.changedOptions[i].value,this.changedOptions[i].path,options);}return options;}}]);return Configurator;}();/**
	 * Create a timeline visualization
	 * @extends Core
	 */var Timeline=/*#__PURE__*/function(_Core){inherits(Timeline,_Core);var _super12=_createSuper$2(Timeline);/**
	 * @param {HTMLElement} container
	 * @param {vis.DataSet | vis.DataView | Array} [items]
	 * @param {vis.DataSet | vis.DataView | Array} [groups]
	 * @param {Object} [options]  See Timeline.setOptions for the available options.
	 * @constructor Timeline
	 */function Timeline(container,items,groups,options){var _context175,_context176,_context177,_context178,_context179,_context180,_context181;var _this43;classCallCheck(this,Timeline);_this43=_super12.call(this);_this43.initTime=new Date();_this43.itemsDone=false;if(!(assertThisInitialized(_this43)instanceof Timeline)){throw new SyntaxError('Constructor must be called with the new operator');}// if the third element is options, the forth is groups (optionally);
	if(!(isArray$3(groups)||groups instanceof DataSet||groups instanceof DataView)&&groups instanceof Object){var forthArgument=options;options=groups;groups=forthArgument;}// TODO: REMOVE THIS in the next MAJOR release
	// see https://github.com/almende/vis/issues/2511
	if(options&&options.throttleRedraw){console.warn("Timeline option \"throttleRedraw\" is DEPRICATED and no longer supported. It will be removed in the next MAJOR release.");}var me=assertThisInitialized(_this43);_this43.defaultOptions={autoResize:true,longSelectPressTime:251,orientation:{axis:'bottom',// axis orientation: 'bottom', 'top', or 'both'
	item:'bottom'// not relevant
	},moment:moment$1};_this43.options=util$1.deepExtend({},_this43.defaultOptions);// Create the DOM, props, and emitter
	_this43._create(container);if(!options||options&&typeof options.rtl=="undefined"){_this43.dom.root.style.visibility='hidden';var directionFromDom;var domNode=_this43.dom.root;while(!directionFromDom&&domNode){directionFromDom=window.getComputedStyle(domNode,null).direction;domNode=domNode.parentElement;}_this43.options.rtl=directionFromDom&&directionFromDom.toLowerCase()=="rtl";}else {_this43.options.rtl=options.rtl;}if(options){if(options.rollingMode){_this43.options.rollingMode=options.rollingMode;}if(options.onInitialDrawComplete){_this43.options.onInitialDrawComplete=options.onInitialDrawComplete;}if(options.onTimeout){_this43.options.onTimeout=options.onTimeout;}if(options.loadingScreenTemplate){_this43.options.loadingScreenTemplate=options.loadingScreenTemplate;}}// Prepare loading screen
	var loadingScreenFragment=document.createElement('div');if(_this43.options.loadingScreenTemplate){var _context174;var templateFunction=bind$2(_context174=_this43.options.loadingScreenTemplate).call(_context174,assertThisInitialized(_this43));var loadingScreen=templateFunction(_this43.dom.loadingScreen);if(loadingScreen instanceof Object&&!(loadingScreen instanceof Element)){templateFunction(loadingScreenFragment);}else {if(loadingScreen instanceof Element){loadingScreenFragment.innerHTML='';loadingScreenFragment.appendChild(loadingScreen);}else if(loadingScreen!=undefined){loadingScreenFragment.innerHTML=loadingScreen;}}}_this43.dom.loadingScreen.appendChild(loadingScreenFragment);// all components listed here will be repainted automatically
	_this43.components=[];_this43.body={dom:_this43.dom,domProps:_this43.props,emitter:{on:bind$2(_context175=_this43.on).call(_context175,assertThisInitialized(_this43)),off:bind$2(_context176=_this43.off).call(_context176,assertThisInitialized(_this43)),emit:bind$2(_context177=_this43.emit).call(_context177,assertThisInitialized(_this43))},hiddenDates:[],util:{getScale:function getScale(){return me.timeAxis.step.scale;},getStep:function getStep(){return me.timeAxis.step.step;},toScreen:bind$2(_context178=me._toScreen).call(_context178,me),toGlobalScreen:bind$2(_context179=me._toGlobalScreen).call(_context179,me),// this refers to the root.width
	toTime:bind$2(_context180=me._toTime).call(_context180,me),toGlobalTime:bind$2(_context181=me._toGlobalTime).call(_context181,me)}};// range
	_this43.range=new Range$1(_this43.body,_this43.options);_this43.components.push(_this43.range);_this43.body.range=_this43.range;// time axis
	_this43.timeAxis=new TimeAxis(_this43.body,_this43.options);_this43.timeAxis2=null;// used in case of orientation option 'both'
	_this43.components.push(_this43.timeAxis);// current time bar
	_this43.currentTime=new CurrentTime(_this43.body,_this43.options);_this43.components.push(_this43.currentTime);// item set
	_this43.itemSet=new ItemSet(_this43.body,_this43.options);_this43.components.push(_this43.itemSet);_this43.itemsData=null;// DataSet
	_this43.groupsData=null;// DataSet
	function emit(eventName,event){if(!me.hasListeners(eventName)){return;}me.emit(eventName,me.getEventProperties(event));}_this43.dom.root.onclick=function(event){emit('click',event);};_this43.dom.root.ondblclick=function(event){emit('doubleClick',event);};_this43.dom.root.oncontextmenu=function(event){emit('contextmenu',event);};_this43.dom.root.onmouseover=function(event){emit('mouseOver',event);};if(window.PointerEvent){_this43.dom.root.onpointerdown=function(event){emit('mouseDown',event);};_this43.dom.root.onpointermove=function(event){emit('mouseMove',event);};_this43.dom.root.onpointerup=function(event){emit('mouseUp',event);};}else {_this43.dom.root.onmousemove=function(event){emit('mouseMove',event);};_this43.dom.root.onmousedown=function(event){emit('mouseDown',event);};_this43.dom.root.onmouseup=function(event){emit('mouseUp',event);};}//Single time autoscale/fit
	_this43.initialFitDone=false;_this43.on('changed',function(){if(me.itemsData==null)return;if(!me.initialFitDone&&!me.options.rollingMode){me.initialFitDone=true;if(me.options.start!=undefined||me.options.end!=undefined){if(me.options.start==undefined||me.options.end==undefined){var range=me.getItemRange();}var start=me.options.start!=undefined?me.options.start:range.min;var end=me.options.end!=undefined?me.options.end:range.max;me.setWindow(start,end,{animation:false});}else {me.fit({animation:false});}}if(!me.initialDrawDone&&(me.initialRangeChangeDone||!me.options.start&&!me.options.end||me.options.rollingMode)){me.initialDrawDone=true;me.itemSet.initialDrawDone=true;me.dom.root.style.visibility='visible';me.dom.loadingScreen.parentNode.removeChild(me.dom.loadingScreen);if(me.options.onInitialDrawComplete){setTimeout$2(function(){return me.options.onInitialDrawComplete();},0);}}});_this43.on('destroyTimeline',function(){me.destroy();});// apply options
	if(options){_this43.setOptions(options);}_this43.body.emitter.on('fit',function(args){_this43._onFit(args);_this43.redraw();});// IMPORTANT: THIS HAPPENS BEFORE SET ITEMS!
	if(groups){_this43.setGroups(groups);}// create itemset
	if(items){_this43.setItems(items);}// draw for the first time
	_this43._redraw();return _this43;}/**
	   * Load a configurator
	   * @return {Object}
	   * @private
	   */createClass(Timeline,[{key:"_createConfigurator",value:function _createConfigurator(){return new Configurator$1(this,this.dom.container,configureOptions$1);}/**
	   * Force a redraw. The size of all items will be recalculated.
	   * Can be useful to manually redraw when option autoResize=false and the window
	   * has been resized, or when the items CSS has been changed.
	   *
	   * Note: this function will be overridden on construction with a trottled version
	   */},{key:"redraw",value:function redraw(){this.itemSet&&this.itemSet.markDirty({refreshItems:true});this._redraw();}/**
	   * Remove an item from the group
	   * @param {object} options
	   */},{key:"setOptions",value:function setOptions(options){// validate options
	var errorFound=Validator$2.validate(options,allOptions$1$2);if(errorFound===true){console.log('%cErrors have been found in the supplied options object.',printStyle$2);}Core.prototype.setOptions.call(this,options);if('type'in options){if(options.type!==this.options.type){this.options.type=options.type;// force recreation of all items
	var itemsData=this.itemsData;if(itemsData){var selection=this.getSelection();this.setItems(null);// remove all
	this.setItems(itemsData.rawDS);// add all
	this.setSelection(selection);// restore selection
	}}}}/**
	   * Set items
	   * @param {vis.DataSet | Array | null} items
	   */},{key:"setItems",value:function setItems(items){this.itemsDone=false;// convert to type DataSet when needed
	var newDataSet;if(!items){newDataSet=null;}else if(items instanceof DataSet||items instanceof DataView){newDataSet=typeCoerceDataSet(items);}else {// turn an array into a dataset
	newDataSet=typeCoerceDataSet(new DataSet(items));}// set items
	if(this.itemsData){// stop maintaining a coerced version of the old data set
	this.itemsData.dispose();}this.itemsData=newDataSet;this.itemSet&&this.itemSet.setItems(newDataSet!=null?newDataSet.rawDS:null);}/**
	   * Set groups
	   * @param {vis.DataSet | Array} groups
	   */},{key:"setGroups",value:function setGroups(groups){// convert to type DataSet when needed
	var newDataSet;if(!groups){newDataSet=null;}else {var filter=function filter(group){return group.visible!==false;};if(groups instanceof DataSet||groups instanceof DataView){newDataSet=new DataView(groups,{filter:filter});}else {// turn an array into a dataset
	newDataSet=new DataSet(filter$2(groups).call(groups,filter));}}// This looks weird but it's necessary to prevent memory leaks.
	//
	// The problem is that the DataView will exist as long as the DataSet it's
	// connected to. This will force it to swap the groups DataSet for it's own
	// DataSet. In this arrangement it will become unreferenced from the outside
	// and garbage collected.
	//
	// IMPORTANT NOTE: If `this.groupsData` is a DataView was created in this
	// method. Even if the original is a DataView already a new one has been
	// created and assigned to `this.groupsData`. In case this changes in the
	// future it will be necessary to rework this!!!!
	if(this.groupsData instanceof DataView){this.groupsData.setData(null);}this.groupsData=newDataSet;this.itemSet.setGroups(newDataSet);}/**
	   * Set both items and groups in one go
	   * @param {{items: (Array | vis.DataSet), groups: (Array | vis.DataSet)}} data
	   */},{key:"setData",value:function setData(data){if(data&&data.groups){this.setGroups(data.groups);}if(data&&data.items){this.setItems(data.items);}}/**
	   * Set selected items by their id. Replaces the current selection
	   * Unknown id's are silently ignored.
	   * @param {string[] | string} [ids]  An array with zero or more id's of the items to be
	   *                                selected. If ids is an empty array, all items will be
	   *                                unselected.
	   * @param {Object} [options]      Available options:
	   *                                `focus: boolean`
	   *                                    If true, focus will be set to the selected item(s)
	   *                                `animation: boolean | {duration: number, easingFunction: string}`
	   *                                    If true (default), the range is animated
	   *                                    smoothly to the new window. An object can be
	   *                                    provided to specify duration and easing function.
	   *                                    Default duration is 500 ms, and default easing
	   *                                    function is 'easeInOutQuad'.
	   *                                    Only applicable when option focus is true.
	   */},{key:"setSelection",value:function setSelection(ids,options){this.itemSet&&this.itemSet.setSelection(ids);if(options&&options.focus){this.focus(ids,options);}}/**
	   * Get the selected items by their id
	   * @return {Array} ids  The ids of the selected items
	   */},{key:"getSelection",value:function getSelection(){return this.itemSet&&this.itemSet.getSelection()||[];}/**
	   * Adjust the visible window such that the selected item (or multiple items)
	   * are centered on screen.
	   * @param {string | String[]} id     An item id or array with item ids
	   * @param {Object} [options]      Available options:
	   *                                `animation: boolean | {duration: number, easingFunction: string}`
	   *                                    If true (default), the range is animated
	   *                                    smoothly to the new window. An object can be
	   *                                    provided to specify duration and easing function.
	   *                                    Default duration is 500 ms, and default easing
	   *                                    function is 'easeInOutQuad'.
	   *                                `zoom: boolean`
	   *                                    If true (default), the timeline will
	   *                                    zoom on the element after focus it.
	   */},{key:"focus",value:function focus(id,options){if(!this.itemsData||id==undefined)return;var ids=isArray$3(id)?id:[id];// get the specified item(s)
	var itemsData=this.itemsData.get(ids);// calculate minimum start and maximum end of specified items
	var start=null;var end=null;forEach$3(itemsData).call(itemsData,function(itemData){var s=itemData.start.valueOf();var e='end'in itemData?itemData.end.valueOf():itemData.start.valueOf();if(start===null||s<start){start=s;}if(end===null||e>end){end=e;}});if(start!==null&&end!==null){var _me=this;// Use the first item for the vertical focus
	var _item18=this.itemSet.items[ids[0]];var startPos=this._getScrollTop()*-1;var initialVerticalScroll=null;// Setup a handler for each frame of the vertical scroll
	var verticalAnimationFrame=function verticalAnimationFrame(ease,willDraw,done){var verticalScroll=getItemVerticalScroll(_me,_item18);if(verticalScroll===false){return;// We don't need to scroll, so do nothing
	}if(!initialVerticalScroll){initialVerticalScroll=verticalScroll;}if(initialVerticalScroll.itemTop==verticalScroll.itemTop&&!initialVerticalScroll.shouldScroll){return;// We don't need to scroll, so do nothing
	}else if(initialVerticalScroll.itemTop!=verticalScroll.itemTop&&verticalScroll.shouldScroll){// The redraw shifted elements, so reset the animation to correct
	initialVerticalScroll=verticalScroll;startPos=_me._getScrollTop()*-1;}var from=startPos;var to=initialVerticalScroll.scrollOffset;var scrollTop=done?to:from+(to-from)*ease;_me._setScrollTop(-scrollTop);if(!willDraw){_me._redraw();}};// Enforces the final vertical scroll position
	var setFinalVerticalPosition=function setFinalVerticalPosition(){var finalVerticalScroll=getItemVerticalScroll(_me,_item18);if(finalVerticalScroll.shouldScroll&&finalVerticalScroll.itemTop!=initialVerticalScroll.itemTop){_me._setScrollTop(-finalVerticalScroll.scrollOffset);_me._redraw();}};// Perform one last check at the end to make sure the final vertical
	// position is correct
	var finalVerticalCallback=function finalVerticalCallback(){// Double check we ended at the proper scroll position
	setFinalVerticalPosition();// Let the redraw settle and finalize the position.      
	setTimeout$2(setFinalVerticalPosition,100);};// calculate the new middle and interval for the window
	var zoom=options&&options.zoom!==undefined?options.zoom:true;var middle=(start+end)/2;var interval=zoom?(end-start)*1.1:Math.max(this.range.end-this.range.start,(end-start)*1.1);var animation=options&&options.animation!==undefined?options.animation:true;if(!animation){// We aren't animating so set a default so that the final callback forces the vertical location
	initialVerticalScroll={shouldScroll:false,scrollOffset:-1,itemTop:-1};}this.range.setRange(middle-interval/2,middle+interval/2,{animation:animation},finalVerticalCallback,verticalAnimationFrame);}}/**
	   * Set Timeline window such that it fits all items
	   * @param {Object} [options]  Available options:
	   *                                `animation: boolean | {duration: number, easingFunction: string}`
	   *                                    If true (default), the range is animated
	   *                                    smoothly to the new window. An object can be
	   *                                    provided to specify duration and easing function.
	   *                                    Default duration is 500 ms, and default easing
	   *                                    function is 'easeInOutQuad'.
	   * @param {function} [callback]
	   */},{key:"fit",value:function fit(options,callback){var animation=options&&options.animation!==undefined?options.animation:true;var range;if(this.itemsData.length===1&&this.itemsData.get()[0].end===undefined){// a single item -> don't fit, just show a range around the item from -4 to +3 days
	range=this.getDataRange();this.moveTo(range.min.valueOf(),{animation:animation},callback);}else {// exactly fit the items (plus a small margin)
	range=this.getItemRange();this.range.setRange(range.min,range.max,{animation:animation},callback);}}/**
	   * Determine the range of the items, taking into account their actual width
	   * and a margin of 10 pixels on both sides.
	   *
	   * @returns {{min: Date, max: Date}}
	   */},{key:"getItemRange",value:function getItemRange(){var _this44=this;// get a rough approximation for the range based on the items start and end dates
	var range=this.getDataRange();var min=range.min!==null?range.min.valueOf():null;var max=range.max!==null?range.max.valueOf():null;var minItem=null;var maxItem=null;if(min!=null&&max!=null){var interval=max-min;// ms
	if(interval<=0){interval=10;}var factor=interval/this.props.center.width;var redrawQueue={};var redrawQueueLength=0;// collect redraw functions
	forEach$3(util$1).call(util$1,this.itemSet.items,function(item,key){if(item.groupShowing){var returnQueue=true;redrawQueue[key]=item.redraw(returnQueue);redrawQueueLength=redrawQueue[key].length;}});var needRedraw=redrawQueueLength>0;if(needRedraw){var _loop7=function _loop7(i){forEach$3(util$1).call(util$1,redrawQueue,function(fns){fns[i]();});};// redraw all regular items
	for(var i=0;i<redrawQueueLength;i++){_loop7(i);}}// calculate the date of the left side and right side of the items given
	forEach$3(util$1).call(util$1,this.itemSet.items,function(item){var start=getStart(item);var end=getEnd(item);var startSide;var endSide;if(_this44.options.rtl){startSide=start-(item.getWidthRight()+10)*factor;endSide=end+(item.getWidthLeft()+10)*factor;}else {startSide=start-(item.getWidthLeft()+10)*factor;endSide=end+(item.getWidthRight()+10)*factor;}if(startSide<min){min=startSide;minItem=item;}if(endSide>max){max=endSide;maxItem=item;}});if(minItem&&maxItem){var lhs=minItem.getWidthLeft()+10;var rhs=maxItem.getWidthRight()+10;var delta=this.props.center.width-lhs-rhs;// px
	if(delta>0){if(this.options.rtl){min=getStart(minItem)-rhs*interval/delta;// ms
	max=getEnd(maxItem)+lhs*interval/delta;// ms
	}else {min=getStart(minItem)-lhs*interval/delta;// ms
	max=getEnd(maxItem)+rhs*interval/delta;// ms
	}}}}return {min:min!=null?new Date(min):null,max:max!=null?new Date(max):null};}/**
	   * Calculate the data range of the items start and end dates
	   * @returns {{min: Date, max: Date}}
	   */},{key:"getDataRange",value:function getDataRange(){var min=null;var max=null;if(this.itemsData){var _context182;forEach$3(_context182=this.itemsData).call(_context182,function(item){var start=util$1.convert(item.start,'Date').valueOf();var end=util$1.convert(item.end!=undefined?item.end:item.start,'Date').valueOf();if(min===null||start<min){min=start;}if(max===null||end>max){max=end;}});}return {min:min!=null?new Date(min):null,max:max!=null?new Date(max):null};}/**
	   * Generate Timeline related information from an event
	   * @param {Event} event
	   * @return {Object} An object with related information, like on which area
	   *                  The event happened, whether clicked on an item, etc.
	   */},{key:"getEventProperties",value:function getEventProperties(event){var clientX=event.center?event.center.x:event.clientX;var clientY=event.center?event.center.y:event.clientY;var centerContainerRect=this.dom.centerContainer.getBoundingClientRect();var x=this.options.rtl?centerContainerRect.right-clientX:clientX-centerContainerRect.left;var y=clientY-centerContainerRect.top;var item=this.itemSet.itemFromTarget(event);var group=this.itemSet.groupFromTarget(event);var customTime=CustomTime.customTimeFromTarget(event);var snap=this.itemSet.options.snap||null;var scale=this.body.util.getScale();var step=this.body.util.getStep();var time=this._toTime(x);var snappedTime=snap?snap(time,scale,step):time;var element=util$1.getTarget(event);var what=null;if(item!=null){what='item';}else if(customTime!=null){what='custom-time';}else if(util$1.hasParent(element,this.timeAxis.dom.foreground)){what='axis';}else if(this.timeAxis2&&util$1.hasParent(element,this.timeAxis2.dom.foreground)){what='axis';}else if(util$1.hasParent(element,this.itemSet.dom.labelSet)){what='group-label';}else if(util$1.hasParent(element,this.currentTime.bar)){what='current-time';}else if(util$1.hasParent(element,this.dom.center)){what='background';}return {event:event,item:item?item.id:null,isCluster:item?!!item.isCluster:false,items:item?item.items||[]:null,group:group?group.groupId:null,customTime:customTime?customTime.options.id:null,what:what,pageX:event.srcEvent?event.srcEvent.pageX:event.pageX,pageY:event.srcEvent?event.srcEvent.pageY:event.pageY,x:x,y:y,time:time,snappedTime:snappedTime};}/**
	   * Toggle Timeline rolling mode
	   */},{key:"toggleRollingMode",value:function toggleRollingMode(){if(this.range.rolling){this.range.stopRolling();}else {if(this.options.rollingMode==undefined){this.setOptions(this.options);}this.range.startRolling();}}/**
	   * redraw
	   * @private
	   */},{key:"_redraw",value:function _redraw(){Core.prototype._redraw.call(this);}/**
	   * on fit callback
	   * @param {object} args
	   * @private
	   */},{key:"_onFit",value:function _onFit(args){var start=args.start,end=args.end,animation=args.animation;if(!end){this.moveTo(start.valueOf(),{animation:animation});}else {this.range.setRange(start,end,{animation:animation});}}}]);return Timeline;}(Core);/**
	 *
	 * @param {timeline.Item} item
	 * @returns {number}
	 */function getStart(item){return util$1.convert(item.data.start,'Date').valueOf();}/**
	 *
	 * @param {timeline.Item} item
	 * @returns {number}
	 */function getEnd(item){var end=item.data.end!=undefined?item.data.end:item.data.start;return util$1.convert(end,'Date').valueOf();}/**
	 * @param {vis.Timeline} timeline
	 * @param {timeline.Item} item
	 * @return {{shouldScroll: bool, scrollOffset: number, itemTop: number}}
	 */function getItemVerticalScroll(timeline,item){if(!item.parent){// The item no longer exists, so ignore this focus.
	return false;}var itemsetHeight=timeline.options.rtl?timeline.props.rightContainer.height:timeline.props.leftContainer.height;var contentHeight=timeline.props.center.height;var group=item.parent;var offset=group.top;var shouldScroll=true;var orientation=timeline.timeAxis.options.orientation.axis;var itemTop=function itemTop(){if(orientation=="bottom"){return group.height-item.top-item.height;}else {return item.top;}};var currentScrollHeight=timeline._getScrollTop()*-1;var targetOffset=offset+itemTop();var height=item.height;if(targetOffset<currentScrollHeight){if(offset+itemsetHeight<=offset+itemTop()+height){offset+=itemTop()-timeline.itemSet.options.margin.item.vertical;}}else if(targetOffset+height>currentScrollHeight+itemsetHeight){offset+=itemTop()+height-itemsetHeight+timeline.itemSet.options.margin.item.vertical;}else {shouldScroll=false;}offset=Math.min(offset,contentHeight-itemsetHeight);return {shouldScroll:shouldScroll,scrollOffset:offset,itemTop:targetOffset};}// DOM utility methods
	/**
	 * this prepares the JSON container for allocating SVG elements
	 * @param {Object} JSONcontainer
	 * @private
	 */function prepareElements(JSONcontainer){// cleanup the redundant svgElements;
	for(var elementType in JSONcontainer){if(JSONcontainer.hasOwnProperty(elementType)){JSONcontainer[elementType].redundant=JSONcontainer[elementType].used;JSONcontainer[elementType].used=[];}}}/**
	 * this cleans up all the unused SVG elements. By asking for the parentNode, we only need to supply the JSON container from
	 * which to remove the redundant elements.
	 *
	 * @param {Object} JSONcontainer
	 * @private
	 */function cleanupElements(JSONcontainer){// cleanup the redundant svgElements;
	for(var elementType in JSONcontainer){if(JSONcontainer.hasOwnProperty(elementType)){if(JSONcontainer[elementType].redundant){for(var i=0;i<JSONcontainer[elementType].redundant.length;i++){JSONcontainer[elementType].redundant[i].parentNode.removeChild(JSONcontainer[elementType].redundant[i]);}JSONcontainer[elementType].redundant=[];}}}}/**
	 * Ensures that all elements are removed first up so they can be recreated cleanly
	 * @param {Object} JSONcontainer
	 */function resetElements(JSONcontainer){prepareElements(JSONcontainer);cleanupElements(JSONcontainer);prepareElements(JSONcontainer);}/**
	 * Allocate or generate an SVG element if needed. Store a reference to it in the JSON container and draw it in the svgContainer
	 * the JSON container and the SVG container have to be supplied so other svg containers (like the legend) can use this.
	 *
	 * @param {string} elementType
	 * @param {Object} JSONcontainer
	 * @param {Object} svgContainer
	 * @returns {Element}
	 * @private
	 */function getSVGElement(elementType,JSONcontainer,svgContainer){var element;// allocate SVG element, if it doesnt yet exist, create one.
	if(JSONcontainer.hasOwnProperty(elementType)){// this element has been created before
	// check if there is an redundant element
	if(JSONcontainer[elementType].redundant.length>0){element=JSONcontainer[elementType].redundant[0];JSONcontainer[elementType].redundant.shift();}else {// create a new element and add it to the SVG
	element=document.createElementNS('http://www.w3.org/2000/svg',elementType);svgContainer.appendChild(element);}}else {// create a new element and add it to the SVG, also create a new object in the svgElements to keep track of it.
	element=document.createElementNS('http://www.w3.org/2000/svg',elementType);JSONcontainer[elementType]={used:[],redundant:[]};svgContainer.appendChild(element);}JSONcontainer[elementType].used.push(element);return element;}/**
	 * Allocate or generate an SVG element if needed. Store a reference to it in the JSON container and draw it in the svgContainer
	 * the JSON container and the SVG container have to be supplied so other svg containers (like the legend) can use this.
	 *
	 * @param {string} elementType
	 * @param {Object} JSONcontainer
	 * @param {Element} DOMContainer
	 * @param {Element} insertBefore
	 * @returns {*}
	 */function getDOMElement(elementType,JSONcontainer,DOMContainer,insertBefore){var element;// allocate DOM element, if it doesnt yet exist, create one.
	if(JSONcontainer.hasOwnProperty(elementType)){// this element has been created before
	// check if there is an redundant element
	if(JSONcontainer[elementType].redundant.length>0){element=JSONcontainer[elementType].redundant[0];JSONcontainer[elementType].redundant.shift();}else {// create a new element and add it to the SVG
	element=document.createElement(elementType);if(insertBefore!==undefined){DOMContainer.insertBefore(element,insertBefore);}else {DOMContainer.appendChild(element);}}}else {// create a new element and add it to the SVG, also create a new object in the svgElements to keep track of it.
	element=document.createElement(elementType);JSONcontainer[elementType]={used:[],redundant:[]};if(insertBefore!==undefined){DOMContainer.insertBefore(element,insertBefore);}else {DOMContainer.appendChild(element);}}JSONcontainer[elementType].used.push(element);return element;}/**
	 * Draw a point object. This is a separate function because it can also be called by the legend.
	 * The reason the JSONcontainer and the target SVG svgContainer have to be supplied is so the legend can use these functions
	 * as well.
	 *
	 * @param {number} x
	 * @param {number} y
	 * @param {Object} groupTemplate: A template containing the necessary information to draw the datapoint e.g., {style: 'circle', size: 5, className: 'className' }
	 * @param {Object} JSONcontainer
	 * @param {Object} svgContainer
	 * @param {Object} labelObj
	 * @returns {vis.PointItem}
	 */function drawPoint(x,y,groupTemplate,JSONcontainer,svgContainer,labelObj){var point;if(groupTemplate.style=='circle'){point=getSVGElement('circle',JSONcontainer,svgContainer);point.setAttributeNS(null,"cx",x);point.setAttributeNS(null,"cy",y);point.setAttributeNS(null,"r",0.5*groupTemplate.size);}else {point=getSVGElement('rect',JSONcontainer,svgContainer);point.setAttributeNS(null,"x",x-0.5*groupTemplate.size);point.setAttributeNS(null,"y",y-0.5*groupTemplate.size);point.setAttributeNS(null,"width",groupTemplate.size);point.setAttributeNS(null,"height",groupTemplate.size);}if(groupTemplate.styles!==undefined){point.setAttributeNS(null,"style",groupTemplate.styles);}point.setAttributeNS(null,"class",groupTemplate.className+" vis-point");//handle label
	if(labelObj){var label=getSVGElement('text',JSONcontainer,svgContainer);if(labelObj.xOffset){x=x+labelObj.xOffset;}if(labelObj.yOffset){y=y+labelObj.yOffset;}if(labelObj.content){label.textContent=labelObj.content;}if(labelObj.className){label.setAttributeNS(null,"class",labelObj.className+" vis-label");}label.setAttributeNS(null,"x",x);label.setAttributeNS(null,"y",y);}return point;}/**
	 * draw a bar SVG element centered on the X coordinate
	 *
	 * @param {number} x
	 * @param {number} y
	 * @param {number} width
	 * @param {number} height
	 * @param {string} className
	 * @param {Object} JSONcontainer
	 * @param {Object} svgContainer
	 * @param {string} style
	 */function drawBar(x,y,width,height,className,JSONcontainer,svgContainer,style){if(height!=0){if(height<0){height*=-1;y-=height;}var rect=getSVGElement('rect',JSONcontainer,svgContainer);rect.setAttributeNS(null,"x",x-0.5*width);rect.setAttributeNS(null,"y",y);rect.setAttributeNS(null,"width",width);rect.setAttributeNS(null,"height",height);rect.setAttributeNS(null,"class",className);if(style){rect.setAttributeNS(null,"style",style);}}}/**
	 * get default language
	 * @returns {string}
	 */function getNavigatorLanguage(){try{if(!navigator)return 'en';if(navigator.languages&&navigator.languages.length){return navigator.languages;}else {return navigator.userLanguage||navigator.language||navigator.browserLanguage||'en';}}catch(error){return 'en';}}/** DataScale */var DataScale=/*#__PURE__*/function(){/**
	   *
	   * @param {number} start
	   * @param {number} end
	   * @param {boolean} autoScaleStart
	   * @param {boolean} autoScaleEnd
	   * @param {number} containerHeight
	   * @param {number} majorCharHeight
	   * @param {boolean} zeroAlign
	   * @param {function} formattingFunction
	   * @constructor DataScale
	   */function DataScale(start,end,autoScaleStart,autoScaleEnd,containerHeight,majorCharHeight){var zeroAlign=arguments.length>6&&arguments[6]!==undefined?arguments[6]:false;var formattingFunction=arguments.length>7&&arguments[7]!==undefined?arguments[7]:false;classCallCheck(this,DataScale);this.majorSteps=[1,2,5,10];this.minorSteps=[0.25,0.5,1,2];this.customLines=null;this.containerHeight=containerHeight;this.majorCharHeight=majorCharHeight;this._start=start;this._end=end;this.scale=1;this.minorStepIdx=-1;this.magnitudefactor=1;this.determineScale();this.zeroAlign=zeroAlign;this.autoScaleStart=autoScaleStart;this.autoScaleEnd=autoScaleEnd;this.formattingFunction=formattingFunction;if(autoScaleStart||autoScaleEnd){var _me2=this;var roundToMinor=function roundToMinor(value){var rounded=value-value%(_me2.magnitudefactor*_me2.minorSteps[_me2.minorStepIdx]);if(value%(_me2.magnitudefactor*_me2.minorSteps[_me2.minorStepIdx])>0.5*(_me2.magnitudefactor*_me2.minorSteps[_me2.minorStepIdx])){return rounded+_me2.magnitudefactor*_me2.minorSteps[_me2.minorStepIdx];}else {return rounded;}};if(autoScaleStart){this._start-=this.magnitudefactor*2*this.minorSteps[this.minorStepIdx];this._start=roundToMinor(this._start);}if(autoScaleEnd){this._end+=this.magnitudefactor*this.minorSteps[this.minorStepIdx];this._end=roundToMinor(this._end);}this.determineScale();}}/**
	   * set chart height
	   * @param {number} majorCharHeight 
	   */createClass(DataScale,[{key:"setCharHeight",value:function setCharHeight(majorCharHeight){this.majorCharHeight=majorCharHeight;}/**
	   * set height
	   * @param {number} containerHeight 
	   */},{key:"setHeight",value:function setHeight(containerHeight){this.containerHeight=containerHeight;}/**
	   * determine scale
	   */},{key:"determineScale",value:function determineScale(){var range=this._end-this._start;this.scale=this.containerHeight/range;var minimumStepValue=this.majorCharHeight/this.scale;var orderOfMagnitude=range>0?Math.round(Math.log(range)/Math.LN10):0;this.minorStepIdx=-1;this.magnitudefactor=Math.pow(10,orderOfMagnitude);var start=0;if(orderOfMagnitude<0){start=orderOfMagnitude;}var solutionFound=false;for(var l=start;Math.abs(l)<=Math.abs(orderOfMagnitude);l++){this.magnitudefactor=Math.pow(10,l);for(var j=0;j<this.minorSteps.length;j++){var stepSize=this.magnitudefactor*this.minorSteps[j];if(stepSize>=minimumStepValue){solutionFound=true;this.minorStepIdx=j;break;}}if(solutionFound===true){break;}}}/**
	   * returns if value is major
	   * @param {number} value
	   * @returns {boolean} 
	   */},{key:"is_major",value:function is_major(value){return value%(this.magnitudefactor*this.majorSteps[this.minorStepIdx])===0;}/**
	   * returns step size
	   * @returns {number} 
	   */},{key:"getStep",value:function getStep(){return this.magnitudefactor*this.minorSteps[this.minorStepIdx];}/**
	   * returns first major
	   * @returns {number} 
	   */},{key:"getFirstMajor",value:function getFirstMajor(){var majorStep=this.magnitudefactor*this.majorSteps[this.minorStepIdx];return this.convertValue(this._start+(majorStep-this._start%majorStep)%majorStep);}/**
	   * returns first major
	   * @param {date} current
	   * @returns {date} formatted date
	   */},{key:"formatValue",value:function formatValue(current){var returnValue=current.toPrecision(5);if(typeof this.formattingFunction==='function'){returnValue=this.formattingFunction(current);}if(typeof returnValue==='number'){return "".concat(returnValue);}else if(typeof returnValue==='string'){return returnValue;}else {return current.toPrecision(5);}}/**
	   * returns lines
	   * @returns {object} lines
	   */},{key:"getLines",value:function getLines(){var lines=[];var step=this.getStep();var bottomOffset=(step-this._start%step)%step;for(var i=this._start+bottomOffset;this._end-i>0.00001;i+=step){if(i!=this._start){//Skip the bottom line
	lines.push({major:this.is_major(i),y:this.convertValue(i),val:this.formatValue(i)});}}return lines;}/**
	   * follow scale
	   * @param {object} other
	   */},{key:"followScale",value:function followScale(other){var oldStepIdx=this.minorStepIdx;var oldStart=this._start;var oldEnd=this._end;var me=this;var increaseMagnitude=function increaseMagnitude(){me.magnitudefactor*=2;};var decreaseMagnitude=function decreaseMagnitude(){me.magnitudefactor/=2;};if(other.minorStepIdx<=1&&this.minorStepIdx<=1||other.minorStepIdx>1&&this.minorStepIdx>1);else if(other.minorStepIdx<this.minorStepIdx){//I'm 5, they are 4 per major.
	this.minorStepIdx=1;if(oldStepIdx==2){increaseMagnitude();}else {increaseMagnitude();increaseMagnitude();}}else {//I'm 4, they are 5 per major
	this.minorStepIdx=2;if(oldStepIdx==1){decreaseMagnitude();}else {decreaseMagnitude();decreaseMagnitude();}}//Get masters stats:
	var otherZero=other.convertValue(0);var otherStep=other.getStep()*other.scale;var done=false;var count=0;//Loop until magnitude is correct for given constrains.
	while(!done&&count++<5){//Get my stats:
	this.scale=otherStep/(this.minorSteps[this.minorStepIdx]*this.magnitudefactor);var newRange=this.containerHeight/this.scale;//For the case the magnitudefactor has changed:
	this._start=oldStart;this._end=this._start+newRange;var myOriginalZero=this._end*this.scale;var majorStep=this.magnitudefactor*this.majorSteps[this.minorStepIdx];var majorOffset=this.getFirstMajor()-other.getFirstMajor();if(this.zeroAlign){var zeroOffset=otherZero-myOriginalZero;this._end+=zeroOffset/this.scale;this._start=this._end-newRange;}else {if(!this.autoScaleStart){this._start+=majorStep-majorOffset/this.scale;this._end=this._start+newRange;}else {this._start-=majorOffset/this.scale;this._end=this._start+newRange;}}if(!this.autoScaleEnd&&this._end>oldEnd+0.00001){//Need to decrease magnitude to prevent scale overshoot! (end)
	decreaseMagnitude();done=false;continue;}if(!this.autoScaleStart&&this._start<oldStart-0.00001){if(this.zeroAlign&&oldStart>=0){console.warn("Can't adhere to given 'min' range, due to zeroalign");}else {//Need to decrease magnitude to prevent scale overshoot! (start)
	decreaseMagnitude();done=false;continue;}}if(this.autoScaleStart&&this.autoScaleEnd&&newRange<oldEnd-oldStart){increaseMagnitude();done=false;continue;}done=true;}}/**
	   * convert value
	   * @param {number} value
	   * @returns {number} 
	   */},{key:"convertValue",value:function convertValue(value){return this.containerHeight-(value-this._start)*this.scale;}/**
	   * returns screen to value
	   * @param {number} pixels
	   * @returns {number} 
	   */},{key:"screenToValue",value:function screenToValue(pixels){return (this.containerHeight-pixels)/this.scale+this._start;}}]);return DataScale;}();/** A horizontal time axis */var DataAxis=/*#__PURE__*/function(_Component6){inherits(DataAxis,_Component6);var _super13=_createSuper$2(DataAxis);/**
	 * @param {Object} body
	 * @param {Object} [options]        See DataAxis.setOptions for the available
	 *                                  options.
	 * @param {SVGElement} svg
	 * @param {timeline.LineGraph.options} linegraphOptions
	 * @constructor DataAxis
	 * @extends Component
	 */function DataAxis(body,options,svg,linegraphOptions){var _this45;classCallCheck(this,DataAxis);_this45=_super13.call(this);_this45.id=v4();_this45.body=body;_this45.defaultOptions={orientation:'left',// supported: 'left', 'right'
	showMinorLabels:true,showMajorLabels:true,showWeekScale:false,icons:false,majorLinesOffset:7,minorLinesOffset:4,labelOffsetX:10,labelOffsetY:2,iconWidth:20,width:'40px',visible:true,alignZeros:true,left:{range:{min:undefined,max:undefined},format:function format(value){return "".concat(_parseFloat$2(value.toPrecision(3)));},title:{text:undefined,style:undefined}},right:{range:{min:undefined,max:undefined},format:function format(value){return "".concat(_parseFloat$2(value.toPrecision(3)));},title:{text:undefined,style:undefined}}};_this45.linegraphOptions=linegraphOptions;_this45.linegraphSVG=svg;_this45.props={};_this45.DOMelements={// dynamic elements
	lines:{},labels:{},title:{}};_this45.dom={};_this45.scale=undefined;_this45.range={start:0,end:0};_this45.options=util$1.extend({},_this45.defaultOptions);_this45.conversionFactor=1;_this45.setOptions(options);_this45.width=Number("".concat(_this45.options.width).replace("px",""));_this45.minWidth=_this45.width;_this45.height=_this45.linegraphSVG.getBoundingClientRect().height;_this45.hidden=false;_this45.stepPixels=25;_this45.zeroCrossing=-1;_this45.amountOfSteps=-1;_this45.lineOffset=0;_this45.master=true;_this45.masterAxis=null;_this45.svgElements={};_this45.iconsRemoved=false;_this45.groups={};_this45.amountOfGroups=0;// create the HTML DOM
	_this45._create();if(_this45.scale==undefined){_this45._redrawLabels();}_this45.framework={svg:_this45.svg,svgElements:_this45.svgElements,options:_this45.options,groups:_this45.groups};var me=assertThisInitialized(_this45);_this45.body.emitter.on("verticalDrag",function(){me.dom.lineContainer.style.top="".concat(me.body.domProps.scrollTop,"px");});return _this45;}/**
	   * Adds group to data axis
	   * @param {string} label 
	   * @param {object} graphOptions
	   */createClass(DataAxis,[{key:"addGroup",value:function addGroup(label,graphOptions){if(!this.groups.hasOwnProperty(label)){this.groups[label]=graphOptions;}this.amountOfGroups+=1;}/**
	   * updates group of data axis
	   * @param {string} label 
	   * @param {object} graphOptions
	   */},{key:"updateGroup",value:function updateGroup(label,graphOptions){if(!this.groups.hasOwnProperty(label)){this.amountOfGroups+=1;}this.groups[label]=graphOptions;}/**
	   * removes group of data axis
	   * @param {string} label 
	   */},{key:"removeGroup",value:function removeGroup(label){if(this.groups.hasOwnProperty(label)){delete this.groups[label];this.amountOfGroups-=1;}}/**
	   * sets options
	   * @param {object} options
	   */},{key:"setOptions",value:function setOptions(options){if(options){var redraw=false;if(this.options.orientation!=options.orientation&&options.orientation!==undefined){redraw=true;}var fields=['orientation','showMinorLabels','showMajorLabels','icons','majorLinesOffset','minorLinesOffset','labelOffsetX','labelOffsetY','iconWidth','width','visible','left','right','alignZeros'];util$1.selectiveDeepExtend(fields,this.options,options);this.minWidth=Number("".concat(this.options.width).replace("px",""));if(redraw===true&&this.dom.frame){this.hide();this.show();}}}/**
	   * Create the HTML DOM for the DataAxis
	   */},{key:"_create",value:function _create(){this.dom.frame=document.createElement('div');this.dom.frame.style.width=this.options.width;this.dom.frame.style.height=this.height;this.dom.lineContainer=document.createElement('div');this.dom.lineContainer.style.width='100%';this.dom.lineContainer.style.height=this.height;this.dom.lineContainer.style.position='relative';this.dom.lineContainer.style.visibility='visible';this.dom.lineContainer.style.display='block';// create svg element for graph drawing.
	this.svg=document.createElementNS('http://www.w3.org/2000/svg',"svg");this.svg.style.position="absolute";this.svg.style.top='0px';this.svg.style.height='100%';this.svg.style.width='100%';this.svg.style.display="block";this.dom.frame.appendChild(this.svg);}/**
	   * redraws groups icons
	   */},{key:"_redrawGroupIcons",value:function _redrawGroupIcons(){prepareElements(this.svgElements);var x;var iconWidth=this.options.iconWidth;var iconHeight=15;var iconOffset=4;var y=iconOffset+0.5*iconHeight;if(this.options.orientation==='left'){x=iconOffset;}else {x=this.width-iconWidth-iconOffset;}var groupArray=keys$6(this.groups);sort$2(groupArray).call(groupArray,function(a,b){return a<b?-1:1;});var _iterator11=_createForOfIteratorHelper$3(groupArray),_step12;try{for(_iterator11.s();!(_step12=_iterator11.n()).done;){var groupId=_step12.value;if(this.groups[groupId].visible===true&&(this.linegraphOptions.visibility[groupId]===undefined||this.linegraphOptions.visibility[groupId]===true)){this.groups[groupId].getLegend(iconWidth,iconHeight,this.framework,x,y);y+=iconHeight+iconOffset;}}}catch(err){_iterator11.e(err);}finally{_iterator11.f();}cleanupElements(this.svgElements);this.iconsRemoved=false;}/**
	   * Cleans up icons
	   */},{key:"_cleanupIcons",value:function _cleanupIcons(){if(this.iconsRemoved===false){prepareElements(this.svgElements);cleanupElements(this.svgElements);this.iconsRemoved=true;}}/**
	   * Create the HTML DOM for the DataAxis
	   */},{key:"show",value:function show(){this.hidden=false;if(!this.dom.frame.parentNode){if(this.options.orientation==='left'){this.body.dom.left.appendChild(this.dom.frame);}else {this.body.dom.right.appendChild(this.dom.frame);}}if(!this.dom.lineContainer.parentNode){this.body.dom.backgroundHorizontal.appendChild(this.dom.lineContainer);}this.dom.lineContainer.style.display='block';}/**
	   * Create the HTML DOM for the DataAxis
	   */},{key:"hide",value:function hide(){this.hidden=true;if(this.dom.frame.parentNode){this.dom.frame.parentNode.removeChild(this.dom.frame);}this.dom.lineContainer.style.display='none';}/**
	   * Set a range (start and end)
	   * @param {number} start
	   * @param {number} end
	   */},{key:"setRange",value:function setRange(start,end){this.range.start=start;this.range.end=end;}/**
	   * Repaint the component
	   * @return {boolean} Returns true if the component is resized
	   */},{key:"redraw",value:function redraw(){var resized=false;var activeGroups=0;// Make sure the line container adheres to the vertical scrolling.
	this.dom.lineContainer.style.top="".concat(this.body.domProps.scrollTop,"px");for(var groupId in this.groups){if(this.groups.hasOwnProperty(groupId)){if(this.groups[groupId].visible===true&&(this.linegraphOptions.visibility[groupId]===undefined||this.linegraphOptions.visibility[groupId]===true)){activeGroups++;}}}if(this.amountOfGroups===0||activeGroups===0){this.hide();}else {this.show();this.height=Number(this.linegraphSVG.style.height.replace("px",""));// svg offsetheight did not work in firefox and explorer...
	this.dom.lineContainer.style.height="".concat(this.height,"px");this.width=this.options.visible===true?Number("".concat(this.options.width).replace("px","")):0;var props=this.props;var frame=this.dom.frame;// update classname
	frame.className='vis-data-axis';// calculate character width and height
	this._calculateCharSize();var orientation=this.options.orientation;var showMinorLabels=this.options.showMinorLabels;var showMajorLabels=this.options.showMajorLabels;var backgroundHorizontalOffsetWidth=this.body.dom.backgroundHorizontal.offsetWidth;// determine the width and height of the elements for the axis
	props.minorLabelHeight=showMinorLabels?props.minorCharHeight:0;props.majorLabelHeight=showMajorLabels?props.majorCharHeight:0;props.minorLineWidth=backgroundHorizontalOffsetWidth-this.lineOffset-this.width+2*this.options.minorLinesOffset;props.minorLineHeight=1;props.majorLineWidth=backgroundHorizontalOffsetWidth-this.lineOffset-this.width+2*this.options.majorLinesOffset;props.majorLineHeight=1;//  take frame offline while updating (is almost twice as fast)
	if(orientation==='left'){frame.style.top='0';frame.style.left='0';frame.style.bottom='';frame.style.width="".concat(this.width,"px");frame.style.height="".concat(this.height,"px");this.props.width=this.body.domProps.left.width;this.props.height=this.body.domProps.left.height;}else {// right
	frame.style.top='';frame.style.bottom='0';frame.style.left='0';frame.style.width="".concat(this.width,"px");frame.style.height="".concat(this.height,"px");this.props.width=this.body.domProps.right.width;this.props.height=this.body.domProps.right.height;}resized=this._redrawLabels();resized=this._isResized()||resized;if(this.options.icons===true){this._redrawGroupIcons();}else {this._cleanupIcons();}this._redrawTitle(orientation);}return resized;}/**
	   * Repaint major and minor text labels and vertical grid lines
	   *
	   * @returns {boolean}
	   * @private
	   */},{key:"_redrawLabels",value:function _redrawLabels(){var _this46=this;var resized=false;prepareElements(this.DOMelements.lines);prepareElements(this.DOMelements.labels);var orientation=this.options['orientation'];var customRange=this.options[orientation].range!=undefined?this.options[orientation].range:{};//Override range with manual options:
	var autoScaleEnd=true;if(customRange.max!=undefined){this.range.end=customRange.max;autoScaleEnd=false;}var autoScaleStart=true;if(customRange.min!=undefined){this.range.start=customRange.min;autoScaleStart=false;}this.scale=new DataScale(this.range.start,this.range.end,autoScaleStart,autoScaleEnd,this.dom.frame.offsetHeight,this.props.majorCharHeight,this.options.alignZeros,this.options[orientation].format);if(this.master===false&&this.masterAxis!=undefined){this.scale.followScale(this.masterAxis.scale);this.dom.lineContainer.style.display='none';}else {this.dom.lineContainer.style.display='block';}//Is updated in side-effect of _redrawLabel():
	this.maxLabelSize=0;var lines=this.scale.getLines();forEach$3(lines).call(lines,function(line){var y=line.y;var isMajor=line.major;if(_this46.options['showMinorLabels']&&isMajor===false){_this46._redrawLabel(y-2,line.val,orientation,'vis-y-axis vis-minor',_this46.props.minorCharHeight);}if(isMajor){if(y>=0){_this46._redrawLabel(y-2,line.val,orientation,'vis-y-axis vis-major',_this46.props.majorCharHeight);}}if(_this46.master===true){if(isMajor){_this46._redrawLine(y,orientation,'vis-grid vis-horizontal vis-major',_this46.options.majorLinesOffset,_this46.props.majorLineWidth);}else {_this46._redrawLine(y,orientation,'vis-grid vis-horizontal vis-minor',_this46.options.minorLinesOffset,_this46.props.minorLineWidth);}}});// Note that title is rotated, so we're using the height, not width!
	var titleWidth=0;if(this.options[orientation].title!==undefined&&this.options[orientation].title.text!==undefined){titleWidth=this.props.titleCharHeight;}var offset=this.options.icons===true?Math.max(this.options.iconWidth,titleWidth)+this.options.labelOffsetX+15:titleWidth+this.options.labelOffsetX+15;// this will resize the yAxis to accommodate the labels.
	if(this.maxLabelSize>this.width-offset&&this.options.visible===true){this.width=this.maxLabelSize+offset;this.options.width="".concat(this.width,"px");cleanupElements(this.DOMelements.lines);cleanupElements(this.DOMelements.labels);this.redraw();resized=true;}// this will resize the yAxis if it is too big for the labels.
	else if(this.maxLabelSize<this.width-offset&&this.options.visible===true&&this.width>this.minWidth){this.width=Math.max(this.minWidth,this.maxLabelSize+offset);this.options.width="".concat(this.width,"px");cleanupElements(this.DOMelements.lines);cleanupElements(this.DOMelements.labels);this.redraw();resized=true;}else {cleanupElements(this.DOMelements.lines);cleanupElements(this.DOMelements.labels);resized=false;}return resized;}/**
	   * converts value
	   * @param {number} value
	   * @returns {number} converted number
	   */},{key:"convertValue",value:function convertValue(value){return this.scale.convertValue(value);}/**
	   * converts value
	   * @param {number} x
	   * @returns {number} screen value
	   */},{key:"screenToValue",value:function screenToValue(x){return this.scale.screenToValue(x);}/**
	   * Create a label for the axis at position x
	   *
	   * @param {number} y
	   * @param {string} text
	   * @param {'top'|'right'|'bottom'|'left'} orientation
	   * @param {string} className
	   * @param {number} characterHeight
	   * @private
	   */},{key:"_redrawLabel",value:function _redrawLabel(y,text,orientation,className,characterHeight){// reuse redundant label
	var label=getDOMElement('div',this.DOMelements.labels,this.dom.frame);//this.dom.redundant.labels.shift();
	label.className=className;label.innerHTML=text;if(orientation==='left'){label.style.left="-".concat(this.options.labelOffsetX,"px");label.style.textAlign="right";}else {label.style.right="-".concat(this.options.labelOffsetX,"px");label.style.textAlign="left";}label.style.top="".concat(y-0.5*characterHeight+this.options.labelOffsetY,"px");text+='';var largestWidth=Math.max(this.props.majorCharWidth,this.props.minorCharWidth);if(this.maxLabelSize<text.length*largestWidth){this.maxLabelSize=text.length*largestWidth;}}/**
	   * Create a minor line for the axis at position y
	   * @param {number} y
	   * @param {'top'|'right'|'bottom'|'left'} orientation
	   * @param {string} className
	   * @param {number} offset
	   * @param {number} width
	   */},{key:"_redrawLine",value:function _redrawLine(y,orientation,className,offset,width){if(this.master===true){var line=getDOMElement('div',this.DOMelements.lines,this.dom.lineContainer);//this.dom.redundant.lines.shift();
	line.className=className;line.innerHTML='';if(orientation==='left'){line.style.left="".concat(this.width-offset,"px");}else {line.style.right="".concat(this.width-offset,"px");}line.style.width="".concat(width,"px");line.style.top="".concat(y,"px");}}/**
	   * Create a title for the axis
	   * @private
	   * @param {'top'|'right'|'bottom'|'left'} orientation
	   */},{key:"_redrawTitle",value:function _redrawTitle(orientation){prepareElements(this.DOMelements.title);// Check if the title is defined for this axes
	if(this.options[orientation].title!==undefined&&this.options[orientation].title.text!==undefined){var title=getDOMElement('div',this.DOMelements.title,this.dom.frame);title.className="vis-y-axis vis-title vis-".concat(orientation);title.innerHTML=this.options[orientation].title.text;// Add style - if provided
	if(this.options[orientation].title.style!==undefined){util$1.addCssText(title,this.options[orientation].title.style);}if(orientation==='left'){title.style.left="".concat(this.props.titleCharHeight,"px");}else {title.style.right="".concat(this.props.titleCharHeight,"px");}title.style.width="".concat(this.height,"px");}// we need to clean up in case we did not use all elements.
	cleanupElements(this.DOMelements.title);}/**
	   * Determine the size of text on the axis (both major and minor axis).
	   * The size is calculated only once and then cached in this.props.
	   * @private
	   */},{key:"_calculateCharSize",value:function _calculateCharSize(){// determine the char width and height on the minor axis
	if(!('minorCharHeight'in this.props)){var textMinor=document.createTextNode('0');var measureCharMinor=document.createElement('div');measureCharMinor.className='vis-y-axis vis-minor vis-measure';measureCharMinor.appendChild(textMinor);this.dom.frame.appendChild(measureCharMinor);this.props.minorCharHeight=measureCharMinor.clientHeight;this.props.minorCharWidth=measureCharMinor.clientWidth;this.dom.frame.removeChild(measureCharMinor);}if(!('majorCharHeight'in this.props)){var textMajor=document.createTextNode('0');var measureCharMajor=document.createElement('div');measureCharMajor.className='vis-y-axis vis-major vis-measure';measureCharMajor.appendChild(textMajor);this.dom.frame.appendChild(measureCharMajor);this.props.majorCharHeight=measureCharMajor.clientHeight;this.props.majorCharWidth=measureCharMajor.clientWidth;this.dom.frame.removeChild(measureCharMajor);}if(!('titleCharHeight'in this.props)){var textTitle=document.createTextNode('0');var measureCharTitle=document.createElement('div');measureCharTitle.className='vis-y-axis vis-title vis-measure';measureCharTitle.appendChild(textTitle);this.dom.frame.appendChild(measureCharTitle);this.props.titleCharHeight=measureCharTitle.clientHeight;this.props.titleCharWidth=measureCharTitle.clientWidth;this.dom.frame.removeChild(measureCharTitle);}}}]);return DataAxis;}(Component);/**
	 *
	 * @param {number | string} groupId
	 * @param {Object} options   // TODO: Describe options
	 *
	 * @constructor Points
	 */function Points(groupId,options){}// eslint-disable-line no-unused-vars
	/**
	 * draw the data points
	 *
	 * @param {Array} dataset
	 * @param {GraphGroup} group
	 * @param {Object} framework            | SVG DOM element
	 * @param {number} [offset]
	 */Points.draw=function(dataset,group,framework,offset){offset=offset||0;var callback=getCallback(framework,group);for(var i=0;i<dataset.length;i++){if(!callback){// draw the point the simple way.
	drawPoint(dataset[i].screen_x+offset,dataset[i].screen_y,getGroupTemplate(group),framework.svgElements,framework.svg,dataset[i].label);}else {var callbackResult=callback(dataset[i],group);// result might be true, false or an object
	if(callbackResult===true||_typeof_1(callbackResult)==='object'){drawPoint(dataset[i].screen_x+offset,dataset[i].screen_y,getGroupTemplate(group,callbackResult),framework.svgElements,framework.svg,dataset[i].label);}}}};Points.drawIcon=function(group,x,y,iconWidth,iconHeight,framework){var fillHeight=iconHeight*0.5;var outline=getSVGElement("rect",framework.svgElements,framework.svg);outline.setAttributeNS(null,"x",x);outline.setAttributeNS(null,"y",y-fillHeight);outline.setAttributeNS(null,"width",iconWidth);outline.setAttributeNS(null,"height",2*fillHeight);outline.setAttributeNS(null,"class","vis-outline");//Don't call callback on icon
	drawPoint(x+0.5*iconWidth,y,getGroupTemplate(group),framework.svgElements,framework.svg);};/**
	 *
	 * @param {vis.Group} group
	 * @param {any} callbackResult
	 * @returns {{style: *, styles: (*|string), size: *, className: *}}
	 */function getGroupTemplate(group,callbackResult){callbackResult=typeof callbackResult==='undefined'?{}:callbackResult;return {style:callbackResult.style||group.options.drawPoints.style,styles:callbackResult.styles||group.options.drawPoints.styles,size:callbackResult.size||group.options.drawPoints.size,className:callbackResult.className||group.className};}/**
	 *
	 * @param {Object} framework            | SVG DOM element
	 * @param {vis.Group} group
	 * @returns {function}
	 */function getCallback(framework,group){var callback=undefined;// check for the graph2d onRender
	if(framework.options&&framework.options.drawPoints&&framework.options.drawPoints.onRender&&typeof framework.options.drawPoints.onRender=='function'){callback=framework.options.drawPoints.onRender;}// override it with the group onRender if defined
	if(group.group.options&&group.group.options.drawPoints&&group.group.options.drawPoints.onRender&&typeof group.group.options.drawPoints.onRender=='function'){callback=group.group.options.drawPoints.onRender;}return callback;}/**
	 *
	 * @param {vis.GraphGroup.id} groupId
	 * @param {Object} options   // TODO: Describe options
	 * @constructor Bargraph
	 */function Bargraph(groupId,options){// eslint-disable-line no-unused-vars
	}Bargraph.drawIcon=function(group,x,y,iconWidth,iconHeight,framework){var fillHeight=iconHeight*0.5;var outline=getSVGElement("rect",framework.svgElements,framework.svg);outline.setAttributeNS(null,"x",x);outline.setAttributeNS(null,"y",y-fillHeight);outline.setAttributeNS(null,"width",iconWidth);outline.setAttributeNS(null,"height",2*fillHeight);outline.setAttributeNS(null,"class","vis-outline");var barWidth=Math.round(0.3*iconWidth);var originalWidth=group.options.barChart.width;var scale=originalWidth/barWidth;var bar1Height=Math.round(0.4*iconHeight);var bar2Height=Math.round(0.75*iconHeight);var offset=Math.round((iconWidth-2*barWidth)/3);drawBar(x+0.5*barWidth+offset,y+fillHeight-bar1Height-1,barWidth,bar1Height,group.className+' vis-bar',framework.svgElements,framework.svg,group.style);drawBar(x+1.5*barWidth+offset+2,y+fillHeight-bar2Height-1,barWidth,bar2Height,group.className+' vis-bar',framework.svgElements,framework.svg,group.style);if(group.options.drawPoints.enabled==true){var groupTemplate={style:group.options.drawPoints.style,styles:group.options.drawPoints.styles,size:group.options.drawPoints.size/scale,className:group.className};drawPoint(x+0.5*barWidth+offset,y+fillHeight-bar1Height-1,groupTemplate,framework.svgElements,framework.svg);drawPoint(x+1.5*barWidth+offset+2,y+fillHeight-bar2Height-1,groupTemplate,framework.svgElements,framework.svg);}};/**
	 * draw a bar graph
	 *
	 * @param {Array.<vis.GraphGroup.id>} groupIds
	 * @param {Object} processedGroupData
	 * @param {{svg: Object, svgElements: Array.<Object>, options: Object, groups: Array.<vis.Group>}} framework
	 */Bargraph.draw=function(groupIds,processedGroupData,framework){var combinedData=[];var intersections={};var coreDistance;var key,drawData;var group;var i,j;var barPoints=0;// combine all barchart data
	for(i=0;i<groupIds.length;i++){group=framework.groups[groupIds[i]];if(group.options.style==='bar'){if(group.visible===true&&(framework.options.groups.visibility[groupIds[i]]===undefined||framework.options.groups.visibility[groupIds[i]]===true)){for(j=0;j<processedGroupData[groupIds[i]].length;j++){combinedData.push({screen_x:processedGroupData[groupIds[i]][j].screen_x,screen_end:processedGroupData[groupIds[i]][j].screen_end,screen_y:processedGroupData[groupIds[i]][j].screen_y,x:processedGroupData[groupIds[i]][j].x,end:processedGroupData[groupIds[i]][j].end,y:processedGroupData[groupIds[i]][j].y,groupId:groupIds[i],label:processedGroupData[groupIds[i]][j].label});barPoints+=1;}}}}if(barPoints===0){return;}// sort by time and by group
	sort$2(combinedData).call(combinedData,function(a,b){if(a.screen_x===b.screen_x){return a.groupId<b.groupId?-1:1;}else {return a.screen_x-b.screen_x;}});// get intersections
	Bargraph._getDataIntersections(intersections,combinedData);// plot barchart
	for(i=0;i<combinedData.length;i++){group=framework.groups[combinedData[i].groupId];var minWidth=group.options.barChart.minWidth!=undefined?group.options.barChart.minWidth:0.1*group.options.barChart.width;key=combinedData[i].screen_x;var heightOffset=0;if(intersections[key]===undefined){if(i+1<combinedData.length){coreDistance=Math.abs(combinedData[i+1].screen_x-key);}drawData=Bargraph._getSafeDrawData(coreDistance,group,minWidth);}else {var nextKey=i+(intersections[key].amount-intersections[key].resolved);if(nextKey<combinedData.length){coreDistance=Math.abs(combinedData[nextKey].screen_x-key);}drawData=Bargraph._getSafeDrawData(coreDistance,group,minWidth);intersections[key].resolved+=1;if(group.options.stack===true&&group.options.excludeFromStacking!==true){if(combinedData[i].screen_y<group.zeroPosition){heightOffset=intersections[key].accumulatedNegative;intersections[key].accumulatedNegative+=group.zeroPosition-combinedData[i].screen_y;}else {heightOffset=intersections[key].accumulatedPositive;intersections[key].accumulatedPositive+=group.zeroPosition-combinedData[i].screen_y;}}else if(group.options.barChart.sideBySide===true){drawData.width=drawData.width/intersections[key].amount;drawData.offset+=intersections[key].resolved*drawData.width-0.5*drawData.width*(intersections[key].amount+1);}}var dataWidth=drawData.width;var start=combinedData[i].screen_x;// are we drawing explicit boxes? (we supplied an end value)
	if(combinedData[i].screen_end!=undefined){dataWidth=combinedData[i].screen_end-combinedData[i].screen_x;start+=dataWidth*0.5;}else {start+=drawData.offset;}drawBar(start,combinedData[i].screen_y-heightOffset,dataWidth,group.zeroPosition-combinedData[i].screen_y,group.className+' vis-bar',framework.svgElements,framework.svg,group.style);// draw points
	if(group.options.drawPoints.enabled===true){var pointData={screen_x:combinedData[i].screen_x,screen_y:combinedData[i].screen_y-heightOffset,x:combinedData[i].x,y:combinedData[i].y,groupId:combinedData[i].groupId,label:combinedData[i].label};Points.draw([pointData],group,framework,drawData.offset);//DOMutil.drawPoint(combinedData[i].x + drawData.offset, combinedData[i].y, group, framework.svgElements, framework.svg);
	}}};/**
	 * Fill the intersections object with counters of how many datapoints share the same x coordinates
	 * @param {Object} intersections
	 * @param {Array.<Object>} combinedData
	 * @private
	 */Bargraph._getDataIntersections=function(intersections,combinedData){// get intersections
	var coreDistance;for(var i=0;i<combinedData.length;i++){if(i+1<combinedData.length){coreDistance=Math.abs(combinedData[i+1].screen_x-combinedData[i].screen_x);}if(i>0){coreDistance=Math.min(coreDistance,Math.abs(combinedData[i-1].screen_x-combinedData[i].screen_x));}if(coreDistance===0){if(intersections[combinedData[i].screen_x]===undefined){intersections[combinedData[i].screen_x]={amount:0,resolved:0,accumulatedPositive:0,accumulatedNegative:0};}intersections[combinedData[i].screen_x].amount+=1;}}};/**
	 * Get the width and offset for bargraphs based on the coredistance between datapoints
	 *
	 * @param {number} coreDistance
	 * @param {vis.Group} group
	 * @param {number} minWidth
	 * @returns {{width: number, offset: number}}
	 * @private
	 */Bargraph._getSafeDrawData=function(coreDistance,group,minWidth){var width,offset;if(coreDistance<group.options.barChart.width&&coreDistance>0){width=coreDistance<minWidth?minWidth:coreDistance;offset=0;// recalculate offset with the new width;
	if(group.options.barChart.align==='left'){offset-=0.5*coreDistance;}else if(group.options.barChart.align==='right'){offset+=0.5*coreDistance;}}else {// default settings
	width=group.options.barChart.width;offset=0;if(group.options.barChart.align==='left'){offset-=0.5*group.options.barChart.width;}else if(group.options.barChart.align==='right'){offset+=0.5*group.options.barChart.width;}}return {width:width,offset:offset};};Bargraph.getStackedYRange=function(combinedData,groupRanges,groupIds,groupLabel,orientation){if(combinedData.length>0){// sort by time and by group
	sort$2(combinedData).call(combinedData,function(a,b){if(a.screen_x===b.screen_x){return a.groupId<b.groupId?-1:1;}else {return a.screen_x-b.screen_x;}});var intersections={};Bargraph._getDataIntersections(intersections,combinedData);groupRanges[groupLabel]=Bargraph._getStackedYRange(intersections,combinedData);groupRanges[groupLabel].yAxisOrientation=orientation;groupIds.push(groupLabel);}};Bargraph._getStackedYRange=function(intersections,combinedData){var key;var yMin=combinedData[0].screen_y;var yMax=combinedData[0].screen_y;for(var i=0;i<combinedData.length;i++){key=combinedData[i].screen_x;if(intersections[key]===undefined){yMin=yMin>combinedData[i].screen_y?combinedData[i].screen_y:yMin;yMax=yMax<combinedData[i].screen_y?combinedData[i].screen_y:yMax;}else {if(combinedData[i].screen_y<0){intersections[key].accumulatedNegative+=combinedData[i].screen_y;}else {intersections[key].accumulatedPositive+=combinedData[i].screen_y;}}}for(var xpos in intersections){if(intersections.hasOwnProperty(xpos)){yMin=yMin>intersections[xpos].accumulatedNegative?intersections[xpos].accumulatedNegative:yMin;yMin=yMin>intersections[xpos].accumulatedPositive?intersections[xpos].accumulatedPositive:yMin;yMax=yMax<intersections[xpos].accumulatedNegative?intersections[xpos].accumulatedNegative:yMax;yMax=yMax<intersections[xpos].accumulatedPositive?intersections[xpos].accumulatedPositive:yMax;}}return {min:yMin,max:yMax};};/**
	 *
	 * @param {vis.GraphGroup.id} groupId
	 * @param {Object} options   // TODO: Describe options
	 * @constructor Line
	 */function Line(groupId,options){// eslint-disable-line no-unused-vars
	}Line.calcPath=function(dataset,group){if(dataset!=null){if(dataset.length>0){var d=[];// construct path from dataset
	if(group.options.interpolation.enabled==true){d=Line._catmullRom(dataset,group);}else {d=Line._linear(dataset);}return d;}}};Line.drawIcon=function(group,x,y,iconWidth,iconHeight,framework){var fillHeight=iconHeight*0.5;var path,fillPath;var outline=getSVGElement("rect",framework.svgElements,framework.svg);outline.setAttributeNS(null,"x",x);outline.setAttributeNS(null,"y",y-fillHeight);outline.setAttributeNS(null,"width",iconWidth);outline.setAttributeNS(null,"height",2*fillHeight);outline.setAttributeNS(null,"class","vis-outline");path=getSVGElement("path",framework.svgElements,framework.svg);path.setAttributeNS(null,"class",group.className);if(group.style!==undefined){path.setAttributeNS(null,"style",group.style);}path.setAttributeNS(null,"d","M"+x+","+y+" L"+(x+iconWidth)+","+y+"");if(group.options.shaded.enabled==true){fillPath=getSVGElement("path",framework.svgElements,framework.svg);if(group.options.shaded.orientation=='top'){fillPath.setAttributeNS(null,"d","M"+x+", "+(y-fillHeight)+"L"+x+","+y+" L"+(x+iconWidth)+","+y+" L"+(x+iconWidth)+","+(y-fillHeight));}else {fillPath.setAttributeNS(null,"d","M"+x+","+y+" "+"L"+x+","+(y+fillHeight)+" "+"L"+(x+iconWidth)+","+(y+fillHeight)+"L"+(x+iconWidth)+","+y);}fillPath.setAttributeNS(null,"class",group.className+" vis-icon-fill");if(group.options.shaded.style!==undefined&&group.options.shaded.style!==""){fillPath.setAttributeNS(null,"style",group.options.shaded.style);}}if(group.options.drawPoints.enabled==true){var groupTemplate={style:group.options.drawPoints.style,styles:group.options.drawPoints.styles,size:group.options.drawPoints.size,className:group.className};drawPoint(x+0.5*iconWidth,y,groupTemplate,framework.svgElements,framework.svg);}};Line.drawShading=function(pathArray,group,subPathArray,framework){// append shading to the path
	if(group.options.shaded.enabled==true){var svgHeight=Number(framework.svg.style.height.replace('px',''));var fillPath=getSVGElement('path',framework.svgElements,framework.svg);var type="L";if(group.options.interpolation.enabled==true){type="C";}var dFill;var zero=0;if(group.options.shaded.orientation=='top'){zero=0;}else if(group.options.shaded.orientation=='bottom'){zero=svgHeight;}else {zero=Math.min(Math.max(0,group.zeroPosition),svgHeight);}if(group.options.shaded.orientation=='group'&&subPathArray!=null&&subPathArray!=undefined){dFill='M'+pathArray[0][0]+","+pathArray[0][1]+" "+this.serializePath(pathArray,type,false)+' L'+subPathArray[subPathArray.length-1][0]+","+subPathArray[subPathArray.length-1][1]+" "+this.serializePath(subPathArray,type,true)+subPathArray[0][0]+","+subPathArray[0][1]+" Z";}else {dFill='M'+pathArray[0][0]+","+pathArray[0][1]+" "+this.serializePath(pathArray,type,false)+' V'+zero+' H'+pathArray[0][0]+" Z";}fillPath.setAttributeNS(null,'class',group.className+' vis-fill');if(group.options.shaded.style!==undefined){fillPath.setAttributeNS(null,'style',group.options.shaded.style);}fillPath.setAttributeNS(null,'d',dFill);}};/**
	 * draw a line graph
	 *
	 * @param {Array.<Object>} pathArray
	 * @param {vis.Group} group
	 * @param {{svg: Object, svgElements: Array.<Object>, options: Object, groups: Array.<vis.Group>}} framework
	 */Line.draw=function(pathArray,group,framework){if(pathArray!=null&&pathArray!=undefined){var path=getSVGElement('path',framework.svgElements,framework.svg);path.setAttributeNS(null,"class",group.className);if(group.style!==undefined){path.setAttributeNS(null,"style",group.style);}var type="L";if(group.options.interpolation.enabled==true){type="C";}// copy properties to path for drawing.
	path.setAttributeNS(null,'d','M'+pathArray[0][0]+","+pathArray[0][1]+" "+this.serializePath(pathArray,type,false));}};Line.serializePath=function(pathArray,type,inverse){if(pathArray.length<2){//Too little data to create a path.
	return "";}var d=type;var i;if(inverse){for(i=pathArray.length-2;i>0;i--){d+=pathArray[i][0]+","+pathArray[i][1]+" ";}}else {for(i=1;i<pathArray.length;i++){d+=pathArray[i][0]+","+pathArray[i][1]+" ";}}return d;};/**
	 * This uses an uniform parametrization of the interpolation algorithm:
	 * 'On the Parameterization of Catmull-Rom Curves' by Cem Yuksel et al.
	 * @param {Array.<Object>} data
	 * @returns {string}
	 * @private
	 */Line._catmullRomUniform=function(data){// catmull rom
	var p0,p1,p2,p3,bp1,bp2;var d=[];d.push([Math.round(data[0].screen_x),Math.round(data[0].screen_y)]);var normalization=1/6;var length=data.length;for(var i=0;i<length-1;i++){p0=i==0?data[0]:data[i-1];p1=data[i];p2=data[i+1];p3=i+2<length?data[i+2]:p2;// Catmull-Rom to Cubic Bezier conversion matrix
	//    0       1       0       0
	//  -1/6      1      1/6      0
	//    0      1/6      1     -1/6
	//    0       0       1       0
	//    bp0 = { x: p1.x,                               y: p1.y };
	bp1={screen_x:(-p0.screen_x+6*p1.screen_x+p2.screen_x)*normalization,screen_y:(-p0.screen_y+6*p1.screen_y+p2.screen_y)*normalization};bp2={screen_x:(p1.screen_x+6*p2.screen_x-p3.screen_x)*normalization,screen_y:(p1.screen_y+6*p2.screen_y-p3.screen_y)*normalization};//    bp0 = { x: p2.x,                               y: p2.y };
	d.push([bp1.screen_x,bp1.screen_y]);d.push([bp2.screen_x,bp2.screen_y]);d.push([p2.screen_x,p2.screen_y]);}return d;};/**
	 * This uses either the chordal or centripetal parameterization of the catmull-rom algorithm.
	 * By default, the centripetal parameterization is used because this gives the nicest results.
	 * These parameterizations are relatively heavy because the distance between 4 points have to be calculated.
	 *
	 * One optimization can be used to reuse distances since this is a sliding window approach.
	 * @param {Array.<Object>} data
	 * @param {vis.GraphGroup} group
	 * @returns {string}
	 * @private
	 */Line._catmullRom=function(data,group){var alpha=group.options.interpolation.alpha;if(alpha==0||alpha===undefined){return this._catmullRomUniform(data);}else {var p0,p1,p2,p3,bp1,bp2,d1,d2,d3,A,B,N,M;var d3powA,d2powA,d3pow2A,d2pow2A,d1pow2A,d1powA;var d=[];d.push([Math.round(data[0].screen_x),Math.round(data[0].screen_y)]);var length=data.length;for(var i=0;i<length-1;i++){p0=i==0?data[0]:data[i-1];p1=data[i];p2=data[i+1];p3=i+2<length?data[i+2]:p2;d1=Math.sqrt(Math.pow(p0.screen_x-p1.screen_x,2)+Math.pow(p0.screen_y-p1.screen_y,2));d2=Math.sqrt(Math.pow(p1.screen_x-p2.screen_x,2)+Math.pow(p1.screen_y-p2.screen_y,2));d3=Math.sqrt(Math.pow(p2.screen_x-p3.screen_x,2)+Math.pow(p2.screen_y-p3.screen_y,2));// Catmull-Rom to Cubic Bezier conversion matrix
	// A = 2d1^2a + 3d1^a * d2^a + d3^2a
	// B = 2d3^2a + 3d3^a * d2^a + d2^2a
	// [   0             1            0          0          ]
	// [   -d2^2a /N     A/N          d1^2a /N   0          ]
	// [   0             d3^2a /M     B/M        -d2^2a /M  ]
	// [   0             0            1          0          ]
	d3powA=Math.pow(d3,alpha);d3pow2A=Math.pow(d3,2*alpha);d2powA=Math.pow(d2,alpha);d2pow2A=Math.pow(d2,2*alpha);d1powA=Math.pow(d1,alpha);d1pow2A=Math.pow(d1,2*alpha);A=2*d1pow2A+3*d1powA*d2powA+d2pow2A;B=2*d3pow2A+3*d3powA*d2powA+d2pow2A;N=3*d1powA*(d1powA+d2powA);if(N>0){N=1/N;}M=3*d3powA*(d3powA+d2powA);if(M>0){M=1/M;}bp1={screen_x:(-d2pow2A*p0.screen_x+A*p1.screen_x+d1pow2A*p2.screen_x)*N,screen_y:(-d2pow2A*p0.screen_y+A*p1.screen_y+d1pow2A*p2.screen_y)*N};bp2={screen_x:(d3pow2A*p1.screen_x+B*p2.screen_x-d2pow2A*p3.screen_x)*M,screen_y:(d3pow2A*p1.screen_y+B*p2.screen_y-d2pow2A*p3.screen_y)*M};if(bp1.screen_x==0&&bp1.screen_y==0){bp1=p1;}if(bp2.screen_x==0&&bp2.screen_y==0){bp2=p2;}d.push([bp1.screen_x,bp1.screen_y]);d.push([bp2.screen_x,bp2.screen_y]);d.push([p2.screen_x,p2.screen_y]);}return d;}};/**
	 * this generates the SVG path for a linear drawing between datapoints.
	 * @param {Array.<Object>} data
	 * @returns {string}
	 * @private
	 */Line._linear=function(data){// linear
	var d=[];for(var i=0;i<data.length;i++){d.push([data[i].screen_x,data[i].screen_y]);}return d;};/**
	 * /**
	 * @param {object} group            | the object of the group from the dataset
	 * @param {string} groupId          | ID of the group
	 * @param {object} options          | the default options
	 * @param {array} groupsUsingDefaultStyles  | this array has one entree.
	 *                                            It is passed as an array so it is passed by reference.
	 *                                            It enumerates through the default styles
	 * @constructor GraphGroup
	 */function GraphGroup(group,groupId,options,groupsUsingDefaultStyles){this.id=groupId;var fields=['sampling','style','sort','yAxisOrientation','barChart','drawPoints','shaded','interpolation','zIndex','excludeFromStacking','excludeFromLegend'];this.options=util$1.selectiveBridgeObject(fields,options);this.usingDefaultStyle=group.className===undefined;this.groupsUsingDefaultStyles=groupsUsingDefaultStyles;this.zeroPosition=0;this.update(group);if(this.usingDefaultStyle==true){this.groupsUsingDefaultStyles[0]+=1;}this.itemsData=[];this.visible=group.visible===undefined?true:group.visible;}/**
	 * this loads a reference to all items in this group into this group.
	 * @param {array} items
	 */GraphGroup.prototype.setItems=function(items){if(items!=null){this.itemsData=items;if(sort$2(this.options)==true){util$1.insertSort(this.itemsData,function(a,b){return a.x>b.x?1:-1;});}}else {this.itemsData=[];}};GraphGroup.prototype.getItems=function(){return this.itemsData;};/**
	 * this is used for barcharts and shading, this way, we only have to calculate it once.
	 * @param {number} pos
	 */GraphGroup.prototype.setZeroPosition=function(pos){this.zeroPosition=pos;};/**
	 * set the options of the graph group over the default options.
	 * @param {Object} options
	 */GraphGroup.prototype.setOptions=function(options){if(options!==undefined){var fields=['sampling','style','sort','yAxisOrientation','barChart','zIndex','excludeFromStacking','excludeFromLegend'];util$1.selectiveDeepExtend(fields,this.options,options);// if the group's drawPoints is a function delegate the callback to the onRender property
	if(typeof options.drawPoints=='function'){options.drawPoints={onRender:options.drawPoints};}util$1.mergeOptions(this.options,options,'interpolation');util$1.mergeOptions(this.options,options,'drawPoints');util$1.mergeOptions(this.options,options,'shaded');if(options.interpolation){if(_typeof_1(options.interpolation)=='object'){if(options.interpolation.parametrization){if(options.interpolation.parametrization=='uniform'){this.options.interpolation.alpha=0;}else if(options.interpolation.parametrization=='chordal'){this.options.interpolation.alpha=1.0;}else {this.options.interpolation.parametrization='centripetal';this.options.interpolation.alpha=0.5;}}}}}};/**
	 * this updates the current group class with the latest group dataset entree, used in _updateGroup in linegraph
	 * @param {vis.Group} group
	 */GraphGroup.prototype.update=function(group){this.group=group;this.content=group.content||'graph';this.className=group.className||this.className||'vis-graph-group'+this.groupsUsingDefaultStyles[0]%10;this.visible=group.visible===undefined?true:group.visible;this.style=group.style;this.setOptions(group.options);};/**
	 * return the legend entree for this group.
	 *
	 * @param {number} iconWidth
	 * @param {number} iconHeight
	 * @param {{svg: (*|Element), svgElements: Object, options: Object, groups: Array.<Object>}} framework
	 * @param {number} x
	 * @param {number} y
	 * @returns {{icon: (*|Element), label: (*|string), orientation: *}}
	 */GraphGroup.prototype.getLegend=function(iconWidth,iconHeight,framework,x,y){if(framework==undefined||framework==null){var svg=document.createElementNS('http://www.w3.org/2000/svg',"svg");framework={svg:svg,svgElements:{},options:this.options,groups:[this]};}if(x==undefined||x==null){x=0;}if(y==undefined||y==null){y=0.5*iconHeight;}switch(this.options.style){case"line":Line.drawIcon(this,x,y,iconWidth,iconHeight,framework);break;case"points"://explicit no break
	case"point":Points.drawIcon(this,x,y,iconWidth,iconHeight,framework);break;case"bar":Bargraph.drawIcon(this,x,y,iconWidth,iconHeight,framework);break;}return {icon:framework.svg,label:this.content,orientation:this.options.yAxisOrientation};};GraphGroup.prototype.getYRange=function(groupData){var yMin=groupData[0].y;var yMax=groupData[0].y;for(var j=0;j<groupData.length;j++){yMin=yMin>groupData[j].y?groupData[j].y:yMin;yMax=yMax<groupData[j].y?groupData[j].y:yMax;}return {min:yMin,max:yMax,yAxisOrientation:this.options.yAxisOrientation};};/**
	 * Legend for Graph2d
	 *
	 * @param {vis.Graph2d.body} body
	 * @param {vis.Graph2d.options} options
	 * @param {number} side
	 * @param {vis.LineGraph.options} linegraphOptions
	 * @constructor Legend
	 * @extends Component
	 */function Legend(body,options,side,linegraphOptions){this.body=body;this.defaultOptions={enabled:false,icons:true,iconSize:20,iconSpacing:6,left:{visible:true,position:'top-left'// top/bottom - left,center,right
	},right:{visible:true,position:'top-right'// top/bottom - left,center,right
	}};this.side=side;this.options=util$1.extend({},this.defaultOptions);this.linegraphOptions=linegraphOptions;this.svgElements={};this.dom={};this.groups={};this.amountOfGroups=0;this._create();this.framework={svg:this.svg,svgElements:this.svgElements,options:this.options,groups:this.groups};this.setOptions(options);}Legend.prototype=new Component();Legend.prototype.clear=function(){this.groups={};this.amountOfGroups=0;};Legend.prototype.addGroup=function(label,graphOptions){// Include a group only if the group option 'excludeFromLegend: false' is not set.
	if(graphOptions.options.excludeFromLegend!=true){if(!this.groups.hasOwnProperty(label)){this.groups[label]=graphOptions;}this.amountOfGroups+=1;}};Legend.prototype.updateGroup=function(label,graphOptions){this.groups[label]=graphOptions;};Legend.prototype.removeGroup=function(label){if(this.groups.hasOwnProperty(label)){delete this.groups[label];this.amountOfGroups-=1;}};Legend.prototype._create=function(){this.dom.frame=document.createElement('div');this.dom.frame.className='vis-legend';this.dom.frame.style.position="absolute";this.dom.frame.style.top="10px";this.dom.frame.style.display="block";this.dom.textArea=document.createElement('div');this.dom.textArea.className='vis-legend-text';this.dom.textArea.style.position="relative";this.dom.textArea.style.top="0px";this.svg=document.createElementNS('http://www.w3.org/2000/svg',"svg");this.svg.style.position='absolute';this.svg.style.top=0+'px';this.svg.style.width=this.options.iconSize+5+'px';this.svg.style.height='100%';this.dom.frame.appendChild(this.svg);this.dom.frame.appendChild(this.dom.textArea);};/**
	 * Hide the component from the DOM
	 */Legend.prototype.hide=function(){// remove the frame containing the items
	if(this.dom.frame.parentNode){this.dom.frame.parentNode.removeChild(this.dom.frame);}};/**
	 * Show the component in the DOM (when not already visible).
	 */Legend.prototype.show=function(){// show frame containing the items
	if(!this.dom.frame.parentNode){this.body.dom.center.appendChild(this.dom.frame);}};Legend.prototype.setOptions=function(options){var fields=['enabled','orientation','icons','left','right'];util$1.selectiveDeepExtend(fields,this.options,options);};Legend.prototype.redraw=function(){var activeGroups=0;var groupArray=keys$6(this.groups);sort$2(groupArray).call(groupArray,function(a,b){return a<b?-1:1;});for(var i=0;i<groupArray.length;i++){var groupId=groupArray[i];if(this.groups[groupId].visible==true&&(this.linegraphOptions.visibility[groupId]===undefined||this.linegraphOptions.visibility[groupId]==true)){activeGroups++;}}if(this.options[this.side].visible==false||this.amountOfGroups==0||this.options.enabled==false||activeGroups==0){this.hide();}else {this.show();if(this.options[this.side].position=='top-left'||this.options[this.side].position=='bottom-left'){this.dom.frame.style.left='4px';this.dom.frame.style.textAlign="left";this.dom.textArea.style.textAlign="left";this.dom.textArea.style.left=this.options.iconSize+15+'px';this.dom.textArea.style.right='';this.svg.style.left=0+'px';this.svg.style.right='';}else {this.dom.frame.style.right='4px';this.dom.frame.style.textAlign="right";this.dom.textArea.style.textAlign="right";this.dom.textArea.style.right=this.options.iconSize+15+'px';this.dom.textArea.style.left='';this.svg.style.right=0+'px';this.svg.style.left='';}if(this.options[this.side].position=='top-left'||this.options[this.side].position=='top-right'){this.dom.frame.style.top=4-Number(this.body.dom.center.style.top.replace("px",""))+'px';this.dom.frame.style.bottom='';}else {var scrollableHeight=this.body.domProps.center.height-this.body.domProps.centerContainer.height;this.dom.frame.style.bottom=4+scrollableHeight+Number(this.body.dom.center.style.top.replace("px",""))+'px';this.dom.frame.style.top='';}if(this.options.icons==false){this.dom.frame.style.width=this.dom.textArea.offsetWidth+10+'px';this.dom.textArea.style.right='';this.dom.textArea.style.left='';this.svg.style.width='0px';}else {this.dom.frame.style.width=this.options.iconSize+15+this.dom.textArea.offsetWidth+10+'px';this.drawLegendIcons();}var content='';for(i=0;i<groupArray.length;i++){groupId=groupArray[i];if(this.groups[groupId].visible==true&&(this.linegraphOptions.visibility[groupId]===undefined||this.linegraphOptions.visibility[groupId]==true)){content+=this.groups[groupId].content+'<br />';}}this.dom.textArea.innerHTML=content;this.dom.textArea.style.lineHeight=0.75*this.options.iconSize+this.options.iconSpacing+'px';}};Legend.prototype.drawLegendIcons=function(){if(this.dom.frame.parentNode){var groupArray=keys$6(this.groups);sort$2(groupArray).call(groupArray,function(a,b){return a<b?-1:1;});// this resets the elements so the order is maintained
	resetElements(this.svgElements);var padding=window.getComputedStyle(this.dom.frame).paddingTop;var iconOffset=Number(padding.replace('px',''));var x=iconOffset;var iconWidth=this.options.iconSize;var iconHeight=0.75*this.options.iconSize;var y=iconOffset+0.5*iconHeight+3;this.svg.style.width=iconWidth+5+iconOffset+'px';for(var i=0;i<groupArray.length;i++){var groupId=groupArray[i];if(this.groups[groupId].visible==true&&(this.linegraphOptions.visibility[groupId]===undefined||this.linegraphOptions.visibility[groupId]==true)){this.groups[groupId].getLegend(iconWidth,iconHeight,this.framework,x,y);y+=iconHeight+this.options.iconSpacing;}}}};var UNGROUPED$3='__ungrouped__';// reserved group id for ungrouped items
	/**
	 * This is the constructor of the LineGraph. It requires a Timeline body and options.
	 *
	 * @param {vis.Timeline.body} body
	 * @param {Object} options
	 * @constructor LineGraph
	 * @extends Component
	 */function LineGraph(body,options){this.id=v4();this.body=body;this.defaultOptions={yAxisOrientation:'left',defaultGroup:'default',sort:true,sampling:true,stack:false,graphHeight:'400px',shaded:{enabled:false,orientation:'bottom'// top, bottom, zero
	},style:'line',// line, bar
	barChart:{width:50,sideBySide:false,align:'center'// left, center, right
	},interpolation:{enabled:true,parametrization:'centripetal',// uniform (alpha = 0.0), chordal (alpha = 1.0), centripetal (alpha = 0.5)
	alpha:0.5},drawPoints:{enabled:true,size:6,style:'square'// square, circle
	},dataAxis:{},//Defaults are done on DataAxis level
	legend:{},//Defaults are done on Legend level
	groups:{visibility:{}}};// options is shared by this lineGraph and all its items
	this.options=util$1.extend({},this.defaultOptions);this.dom={};this.props={};this.hammer=null;this.groups={};this.abortedGraphUpdate=false;this.updateSVGheight=false;this.updateSVGheightOnResize=false;this.forceGraphUpdate=true;var me=this;this.itemsData=null;// DataSet
	this.groupsData=null;// DataSet
	// listeners for the DataSet of the items
	this.itemListeners={'add':function add(event,params,senderId){// eslint-disable-line no-unused-vars
	me._onAdd(params.items);},'update':function update(event,params,senderId){// eslint-disable-line no-unused-vars
	me._onUpdate(params.items);},'remove':function remove(event,params,senderId){// eslint-disable-line no-unused-vars
	me._onRemove(params.items);}};// listeners for the DataSet of the groups
	this.groupListeners={'add':function add(event,params,senderId){// eslint-disable-line no-unused-vars
	me._onAddGroups(params.items);},'update':function update(event,params,senderId){// eslint-disable-line no-unused-vars
	me._onUpdateGroups(params.items);},'remove':function remove(event,params,senderId){// eslint-disable-line no-unused-vars
	me._onRemoveGroups(params.items);}};this.items={};// object with an Item for every data item
	this.selection=[];// list with the ids of all selected nodes
	this.lastStart=this.body.range.start;this.touchParams={};// stores properties while dragging
	this.svgElements={};this.setOptions(options);this.groupsUsingDefaultStyles=[0];this.body.emitter.on('rangechanged',function(){me.lastStart=me.body.range.start;me.svg.style.left=util$1.option.asSize(-me.props.width);me.forceGraphUpdate=true;//Is this local redraw necessary? (Core also does a change event!)
	me.redraw.call(me);});// create the HTML DOM
	this._create();this.framework={svg:this.svg,svgElements:this.svgElements,options:this.options,groups:this.groups};}LineGraph.prototype=new Component();/**
	 * Create the HTML DOM for the ItemSet
	 */LineGraph.prototype._create=function(){var frame=document.createElement('div');frame.className='vis-line-graph';this.dom.frame=frame;// create svg element for graph drawing.
	this.svg=document.createElementNS('http://www.w3.org/2000/svg','svg');this.svg.style.position='relative';this.svg.style.height=(''+this.options.graphHeight).replace('px','')+'px';this.svg.style.display='block';frame.appendChild(this.svg);// data axis
	this.options.dataAxis.orientation='left';this.yAxisLeft=new DataAxis(this.body,this.options.dataAxis,this.svg,this.options.groups);this.options.dataAxis.orientation='right';this.yAxisRight=new DataAxis(this.body,this.options.dataAxis,this.svg,this.options.groups);delete this.options.dataAxis.orientation;// legends
	this.legendLeft=new Legend(this.body,this.options.legend,'left',this.options.groups);this.legendRight=new Legend(this.body,this.options.legend,'right',this.options.groups);this.show();};/**
	 * set the options of the LineGraph. the mergeOptions is used for subObjects that have an enabled element.
	 * @param {object} options
	 */LineGraph.prototype.setOptions=function(options){if(options){var fields=['sampling','defaultGroup','stack','height','graphHeight','yAxisOrientation','style','barChart','dataAxis','sort','groups'];if(options.graphHeight===undefined&&options.height!==undefined){this.updateSVGheight=true;this.updateSVGheightOnResize=true;}else if(this.body.domProps.centerContainer.height!==undefined&&options.graphHeight!==undefined){if(_parseInt$2((options.graphHeight+'').replace("px",''))<this.body.domProps.centerContainer.height){this.updateSVGheight=true;}}util$1.selectiveDeepExtend(fields,this.options,options);util$1.mergeOptions(this.options,options,'interpolation');util$1.mergeOptions(this.options,options,'drawPoints');util$1.mergeOptions(this.options,options,'shaded');util$1.mergeOptions(this.options,options,'legend');if(options.interpolation){if(_typeof_1(options.interpolation)=='object'){if(options.interpolation.parametrization){if(options.interpolation.parametrization=='uniform'){this.options.interpolation.alpha=0;}else if(options.interpolation.parametrization=='chordal'){this.options.interpolation.alpha=1.0;}else {this.options.interpolation.parametrization='centripetal';this.options.interpolation.alpha=0.5;}}}}if(this.yAxisLeft){if(options.dataAxis!==undefined){this.yAxisLeft.setOptions(this.options.dataAxis);this.yAxisRight.setOptions(this.options.dataAxis);}}if(this.legendLeft){if(options.legend!==undefined){this.legendLeft.setOptions(this.options.legend);this.legendRight.setOptions(this.options.legend);}}if(this.groups.hasOwnProperty(UNGROUPED$3)){this.groups[UNGROUPED$3].setOptions(options);}}// this is used to redraw the graph if the visibility of the groups is changed.
	if(this.dom.frame){//not on initial run?
	this.forceGraphUpdate=true;this.body.emitter.emit("_change",{queue:true});}};/**
	 * Hide the component from the DOM
	 */LineGraph.prototype.hide=function(){// remove the frame containing the items
	if(this.dom.frame.parentNode){this.dom.frame.parentNode.removeChild(this.dom.frame);}};/**
	 * Show the component in the DOM (when not already visible).
	 */LineGraph.prototype.show=function(){// show frame containing the items
	if(!this.dom.frame.parentNode){this.body.dom.center.appendChild(this.dom.frame);}};/**
	 * Set items
	 * @param {vis.DataSet | null} items
	 */LineGraph.prototype.setItems=function(items){var me=this,ids,oldItemsData=this.itemsData;// replace the dataset
	if(!items){this.itemsData=null;}else if(items instanceof DataSet||items instanceof DataView){this.itemsData=typeCoerceDataSet(items);}else {throw new TypeError('Data must be an instance of DataSet or DataView');}if(oldItemsData){// unsubscribe from old dataset
	forEach$3(util$1).call(util$1,this.itemListeners,function(callback,event){oldItemsData.off(event,callback);});// stop maintaining a coerced version of the old data set
	oldItemsData.dispose();// remove all drawn items
	ids=oldItemsData.getIds();this._onRemove(ids);}if(this.itemsData){// subscribe to new dataset
	var id=this.id;forEach$3(util$1).call(util$1,this.itemListeners,function(callback,event){me.itemsData.on(event,callback,id);});// add all new items
	ids=this.itemsData.getIds();this._onAdd(ids);}};/**
	 * Set groups
	 * @param {vis.DataSet} groups
	 */LineGraph.prototype.setGroups=function(groups){var me=this;var ids;// unsubscribe from current dataset
	if(this.groupsData){forEach$3(util$1).call(util$1,this.groupListeners,function(callback,event){me.groupsData.off(event,callback);});// remove all drawn groups
	ids=this.groupsData.getIds();this.groupsData=null;for(var i=0;i<ids.length;i++){this._removeGroup(ids[i]);}}// replace the dataset
	if(!groups){this.groupsData=null;}else if(groups instanceof DataSet||groups instanceof DataView){this.groupsData=groups;}else {throw new TypeError('Data must be an instance of DataSet or DataView');}if(this.groupsData){// subscribe to new dataset
	var id=this.id;forEach$3(util$1).call(util$1,this.groupListeners,function(callback,event){me.groupsData.on(event,callback,id);});// draw all ms
	ids=this.groupsData.getIds();this._onAddGroups(ids);}};LineGraph.prototype._onUpdate=function(ids){this._updateAllGroupData(ids);};LineGraph.prototype._onAdd=function(ids){this._onUpdate(ids);};LineGraph.prototype._onRemove=function(ids){this._onUpdate(ids);};LineGraph.prototype._onUpdateGroups=function(groupIds){this._updateAllGroupData(null,groupIds);};LineGraph.prototype._onAddGroups=function(groupIds){this._onUpdateGroups(groupIds);};/**
	 * this cleans the group out off the legends and the dataaxis, updates the ungrouped and updates the graph
	 * @param {Array} groupIds
	 * @private
	 */LineGraph.prototype._onRemoveGroups=function(groupIds){for(var i=0;i<groupIds.length;i++){this._removeGroup(groupIds[i]);}this.forceGraphUpdate=true;this.body.emitter.emit("_change",{queue:true});};/**
	 * this cleans the group out off the legends and the dataaxis
	 * @param {vis.GraphGroup.id} groupId
	 * @private
	 */LineGraph.prototype._removeGroup=function(groupId){if(this.groups.hasOwnProperty(groupId)){if(this.groups[groupId].options.yAxisOrientation=='right'){this.yAxisRight.removeGroup(groupId);this.legendRight.removeGroup(groupId);this.legendRight.redraw();}else {this.yAxisLeft.removeGroup(groupId);this.legendLeft.removeGroup(groupId);this.legendLeft.redraw();}delete this.groups[groupId];}};/**
	 * update a group object with the group dataset entree
	 *
	 * @param {vis.GraphGroup} group
	 * @param {vis.GraphGroup.id} groupId
	 * @private
	 */LineGraph.prototype._updateGroup=function(group,groupId){if(!this.groups.hasOwnProperty(groupId)){this.groups[groupId]=new GraphGroup(group,groupId,this.options,this.groupsUsingDefaultStyles);if(this.groups[groupId].options.yAxisOrientation=='right'){this.yAxisRight.addGroup(groupId,this.groups[groupId]);this.legendRight.addGroup(groupId,this.groups[groupId]);}else {this.yAxisLeft.addGroup(groupId,this.groups[groupId]);this.legendLeft.addGroup(groupId,this.groups[groupId]);}}else {this.groups[groupId].update(group);if(this.groups[groupId].options.yAxisOrientation=='right'){this.yAxisRight.updateGroup(groupId,this.groups[groupId]);this.legendRight.updateGroup(groupId,this.groups[groupId]);//If yAxisOrientation changed, clean out the group from the other axis.
	this.yAxisLeft.removeGroup(groupId);this.legendLeft.removeGroup(groupId);}else {this.yAxisLeft.updateGroup(groupId,this.groups[groupId]);this.legendLeft.updateGroup(groupId,this.groups[groupId]);//If yAxisOrientation changed, clean out the group from the other axis.
	this.yAxisRight.removeGroup(groupId);this.legendRight.removeGroup(groupId);}}this.legendLeft.redraw();this.legendRight.redraw();};/**
	 * this updates all groups, it is used when there is an update the the itemset.
	 *
	 * @param  {Array} ids
	 * @param  {Array} groupIds
	 * @private
	 */LineGraph.prototype._updateAllGroupData=function(ids,groupIds){if(this.itemsData!=null){var groupsContent={};var items=this.itemsData.get();var fieldId=this.itemsData.idProp;var idMap={};if(ids){map$5(ids).call(ids,function(id){idMap[id]=id;});}//pre-Determine array sizes, for more efficient memory claim
	var groupCounts={};for(var i=0;i<items.length;i++){var item=items[i];var groupId=item.group;if(groupId===null||groupId===undefined){groupId=UNGROUPED$3;}groupCounts.hasOwnProperty(groupId)?groupCounts[groupId]++:groupCounts[groupId]=1;}//Pre-load arrays from existing groups if items are not changed (not in ids)
	var existingItemsMap={};if(!groupIds&&ids){for(groupId in this.groups){if(this.groups.hasOwnProperty(groupId)){group=this.groups[groupId];var existing_items=group.getItems();groupsContent[groupId]=filter$2(existing_items).call(existing_items,function(item){existingItemsMap[item[fieldId]]=item[fieldId];return item[fieldId]!==idMap[item[fieldId]];});var newLength=groupCounts[groupId];groupCounts[groupId]-=groupsContent[groupId].length;if(groupsContent[groupId].length<newLength){groupsContent[groupId][newLength-1]={};}}}}//Now insert data into the arrays.
	for(i=0;i<items.length;i++){item=items[i];groupId=item.group;if(groupId===null||groupId===undefined){groupId=UNGROUPED$3;}if(!groupIds&&ids&&item[fieldId]!==idMap[item[fieldId]]&&existingItemsMap.hasOwnProperty(item[fieldId])){continue;}if(!groupsContent.hasOwnProperty(groupId)){groupsContent[groupId]=new Array(groupCounts[groupId]);}//Copy data (because of unmodifiable DataView input.
	var extended=util$1.bridgeObject(item);extended.x=util$1.convert(item.x,'Date');extended.end=util$1.convert(item.end,'Date');extended.orginalY=item.y;//real Y
	extended.y=Number(item.y);extended[fieldId]=item[fieldId];var index=groupsContent[groupId].length-groupCounts[groupId]--;groupsContent[groupId][index]=extended;}//Make sure all groups are present, to allow removal of old groups
	for(groupId in this.groups){if(this.groups.hasOwnProperty(groupId)){if(!groupsContent.hasOwnProperty(groupId)){groupsContent[groupId]=new Array(0);}}}//Update legendas, style and axis
	for(groupId in groupsContent){if(groupsContent.hasOwnProperty(groupId)){if(groupsContent[groupId].length==0){if(this.groups.hasOwnProperty(groupId)){this._removeGroup(groupId);}}else {var group=undefined;if(this.groupsData!=undefined){group=this.groupsData.get(groupId);}if(group==undefined){group={id:groupId,content:this.options.defaultGroup+groupId};}this._updateGroup(group,groupId);this.groups[groupId].setItems(groupsContent[groupId]);}}}this.forceGraphUpdate=true;this.body.emitter.emit("_change",{queue:true});}};/**
	 * Redraw the component, mandatory function
	 * @return {boolean} Returns true if the component is resized
	 */LineGraph.prototype.redraw=function(){var resized=false;// calculate actual size and position
	this.props.width=this.dom.frame.offsetWidth;this.props.height=this.body.domProps.centerContainer.height-this.body.domProps.border.top-this.body.domProps.border.bottom;// check if this component is resized
	resized=this._isResized()||resized;// check whether zoomed (in that case we need to re-stack everything)
	var visibleInterval=this.body.range.end-this.body.range.start;var zoomed=visibleInterval!=this.lastVisibleInterval;this.lastVisibleInterval=visibleInterval;// the svg element is three times as big as the width, this allows for fully dragging left and right
	// without reloading the graph. the controls for this are bound to events in the constructor
	if(resized==true){var _context183;this.svg.style.width=util$1.option.asSize(3*this.props.width);this.svg.style.left=util$1.option.asSize(-this.props.width);// if the height of the graph is set as proportional, change the height of the svg
	if(indexOf$3(_context183=this.options.height+'').call(_context183,"%")!=-1||this.updateSVGheightOnResize==true){this.updateSVGheight=true;}}// update the height of the graph on each redraw of the graph.
	if(this.updateSVGheight==true){if(this.options.graphHeight!=this.props.height+'px'){this.options.graphHeight=this.props.height+'px';this.svg.style.height=this.props.height+'px';}this.updateSVGheight=false;}else {this.svg.style.height=(''+this.options.graphHeight).replace('px','')+'px';}// zoomed is here to ensure that animations are shown correctly.
	if(resized==true||zoomed==true||this.abortedGraphUpdate==true||this.forceGraphUpdate==true){resized=this._updateGraph()||resized;this.forceGraphUpdate=false;}else {// move the whole svg while dragging
	if(this.lastStart!=0){var offset=this.body.range.start-this.lastStart;var range=this.body.range.end-this.body.range.start;if(this.props.width!=0){var rangePerPixelInv=this.props.width/range;var xOffset=offset*rangePerPixelInv;this.svg.style.left=-this.props.width-xOffset+'px';}}}this.legendLeft.redraw();this.legendRight.redraw();return resized;};LineGraph.prototype._getSortedGroupIds=function(){// getting group Ids
	var grouplist=[];for(var groupId in this.groups){if(this.groups.hasOwnProperty(groupId)){var group=this.groups[groupId];if(group.visible==true&&(this.options.groups.visibility[groupId]===undefined||this.options.groups.visibility[groupId]==true)){grouplist.push({id:groupId,zIndex:group.options.zIndex});}}}util$1.insertSort(grouplist,function(a,b){var az=a.zIndex;var bz=b.zIndex;if(az===undefined)az=0;if(bz===undefined)bz=0;return az==bz?0:az<bz?-1:1;});var groupIds=new Array(grouplist.length);for(var i=0;i<grouplist.length;i++){groupIds[i]=grouplist[i].id;}return groupIds;};/**
	 * Update and redraw the graph.
	 *
	 * @returns {boolean}
	 * @private
	 */LineGraph.prototype._updateGraph=function(){// reset the svg elements
	prepareElements(this.svgElements);if(this.props.width!=0&&this.itemsData!=null){var group,i;var groupRanges={};var changeCalled=false;// this is the range of the SVG canvas
	var minDate=this.body.util.toGlobalTime(-this.body.domProps.root.width);var maxDate=this.body.util.toGlobalTime(2*this.body.domProps.root.width);// getting group Ids
	var groupIds=this._getSortedGroupIds();if(groupIds.length>0){var groupsData={};// fill groups data, this only loads the data we require based on the timewindow
	this._getRelevantData(groupIds,groupsData,minDate,maxDate);// apply sampling, if disabled, it will pass through this function.
	this._applySampling(groupIds,groupsData);// we transform the X coordinates to detect collisions
	for(i=0;i<groupIds.length;i++){this._convertXcoordinates(groupsData[groupIds[i]]);}// now all needed data has been collected we start the processing.
	this._getYRanges(groupIds,groupsData,groupRanges);// update the Y axis first, we use this data to draw at the correct Y points
	changeCalled=this._updateYAxis(groupIds,groupRanges);//  at changeCalled, abort this update cycle as the graph needs another update with new Width input from the Redraw container.
	//  Cleanup SVG elements on abort.
	if(changeCalled==true){cleanupElements(this.svgElements);this.abortedGraphUpdate=true;return true;}this.abortedGraphUpdate=false;// With the yAxis scaled correctly, use this to get the Y values of the points.
	var below=undefined;for(i=0;i<groupIds.length;i++){group=this.groups[groupIds[i]];if(this.options.stack===true&&this.options.style==='line'){if(group.options.excludeFromStacking==undefined||!group.options.excludeFromStacking){if(below!=undefined){this._stack(groupsData[group.id],groupsData[below.id]);if(group.options.shaded.enabled==true&&group.options.shaded.orientation!=="group"){if(group.options.shaded.orientation=="top"&&below.options.shaded.orientation!=="group"){below.options.shaded.orientation="group";below.options.shaded.groupId=group.id;}else {group.options.shaded.orientation="group";group.options.shaded.groupId=below.id;}}}below=group;}}this._convertYcoordinates(groupsData[groupIds[i]],group);}//Precalculate paths and draw shading if appropriate. This will make sure the shading is always behind any lines.
	var paths={};for(i=0;i<groupIds.length;i++){group=this.groups[groupIds[i]];if(group.options.style==='line'&&group.options.shaded.enabled==true){var dataset=groupsData[groupIds[i]];if(dataset==null||dataset.length==0){continue;}if(!paths.hasOwnProperty(groupIds[i])){paths[groupIds[i]]=Line.calcPath(dataset,group);}if(group.options.shaded.orientation==="group"){var subGroupId=group.options.shaded.groupId;if(indexOf$3(groupIds).call(groupIds,subGroupId)===-1){console.log(group.id+": Unknown shading group target given:"+subGroupId);continue;}if(!paths.hasOwnProperty(subGroupId)){paths[subGroupId]=Line.calcPath(groupsData[subGroupId],this.groups[subGroupId]);}Line.drawShading(paths[groupIds[i]],group,paths[subGroupId],this.framework);}else {Line.drawShading(paths[groupIds[i]],group,undefined,this.framework);}}}// draw the groups, calculating paths if still necessary.
	Bargraph.draw(groupIds,groupsData,this.framework);for(i=0;i<groupIds.length;i++){group=this.groups[groupIds[i]];if(groupsData[groupIds[i]].length>0){switch(group.options.style){case"line":if(!paths.hasOwnProperty(groupIds[i])){paths[groupIds[i]]=Line.calcPath(groupsData[groupIds[i]],group);}Line.draw(paths[groupIds[i]],group,this.framework);// eslint-disable-line no-fallthrough
	case"point":// eslint-disable-line no-fallthrough
	case"points":if(group.options.style=="point"||group.options.style=="points"||group.options.drawPoints.enabled==true){Points.draw(groupsData[groupIds[i]],group,this.framework);}break;//do nothing...
	}}}}}// cleanup unused svg elements
	cleanupElements(this.svgElements);return false;};LineGraph.prototype._stack=function(data,subData){var index,dx,dy,subPrevPoint,subNextPoint;index=0;// for each data point we look for a matching on in the set below
	for(var j=0;j<data.length;j++){subPrevPoint=undefined;subNextPoint=undefined;// we look for time matches or a before-after point
	for(var k=index;k<subData.length;k++){// if times match exactly
	if(subData[k].x===data[j].x){subPrevPoint=subData[k];subNextPoint=subData[k];index=k;break;}else if(subData[k].x>data[j].x){// overshoot
	subNextPoint=subData[k];if(k==0){subPrevPoint=subNextPoint;}else {subPrevPoint=subData[k-1];}index=k;break;}}// in case the last data point has been used, we assume it stays like this.
	if(subNextPoint===undefined){subPrevPoint=subData[subData.length-1];subNextPoint=subData[subData.length-1];}// linear interpolation
	dx=subNextPoint.x-subPrevPoint.x;dy=subNextPoint.y-subPrevPoint.y;if(dx==0){data[j].y=data[j].orginalY+subNextPoint.y;}else {data[j].y=data[j].orginalY+dy/dx*(data[j].x-subPrevPoint.x)+subPrevPoint.y;// ax + b where b is data[j].y
	}}};/**
	 * first select and preprocess the data from the datasets.
	 * the groups have their preselection of data, we now loop over this data to see
	 * what data we need to draw. Sorted data is much faster.
	 * more optimization is possible by doing the sampling before and using the binary search
	 * to find the end date to determine the increment.
	 *
	 * @param {array}  groupIds
	 * @param {object} groupsData
	 * @param {date}   minDate
	 * @param {date}   maxDate
	 * @private
	 */LineGraph.prototype._getRelevantData=function(groupIds,groupsData,minDate,maxDate){var group,i,j,item;if(groupIds.length>0){for(i=0;i<groupIds.length;i++){group=this.groups[groupIds[i]];var itemsData=group.getItems();// optimization for sorted data
	if(sort$2(group.options)==true){var dateComparator=function dateComparator(a,b){return a.getTime()==b.getTime()?0:a<b?-1:1;};var first=Math.max(0,util$1.binarySearchValue(itemsData,minDate,'x','before',dateComparator));var last=Math.min(itemsData.length,util$1.binarySearchValue(itemsData,maxDate,'x','after',dateComparator)+1);if(last<=0){last=itemsData.length;}var dataContainer=new Array(last-first);for(j=first;j<last;j++){item=group.itemsData[j];dataContainer[j-first]=item;}groupsData[groupIds[i]]=dataContainer;}else {// If unsorted data, all data is relevant, just returning entire structure
	groupsData[groupIds[i]]=group.itemsData;}}}};/**
	 *
	 * @param {Array.<vis.GraphGroup.id>} groupIds
	 * @param {vis.DataSet} groupsData
	 * @private
	 */LineGraph.prototype._applySampling=function(groupIds,groupsData){var group;if(groupIds.length>0){for(var i=0;i<groupIds.length;i++){group=this.groups[groupIds[i]];if(group.options.sampling==true){var dataContainer=groupsData[groupIds[i]];if(dataContainer.length>0){var increment=1;var amountOfPoints=dataContainer.length;// the global screen is used because changing the width of the yAxis may affect the increment, resulting in an endless loop
	// of width changing of the yAxis.
	//TODO: This assumes sorted data, but that's not guaranteed!
	var xDistance=this.body.util.toGlobalScreen(dataContainer[dataContainer.length-1].x)-this.body.util.toGlobalScreen(dataContainer[0].x);var pointsPerPixel=amountOfPoints/xDistance;increment=Math.min(Math.ceil(0.2*amountOfPoints),Math.max(1,Math.round(pointsPerPixel)));var sampledData=new Array(amountOfPoints);for(var j=0;j<amountOfPoints;j+=increment){var idx=Math.round(j/increment);sampledData[idx]=dataContainer[j];}groupsData[groupIds[i]]=splice$2(sampledData).call(sampledData,0,Math.round(amountOfPoints/increment));}}}}};/**
	 *
	 * @param {Array.<vis.GraphGroup.id>} groupIds
	 * @param {vis.DataSet} groupsData
	 * @param {object} groupRanges  | this is being filled here
	 * @private
	 */LineGraph.prototype._getYRanges=function(groupIds,groupsData,groupRanges){var groupData,group,i;var combinedDataLeft=[];var combinedDataRight=[];var options;if(groupIds.length>0){for(i=0;i<groupIds.length;i++){groupData=groupsData[groupIds[i]];options=this.groups[groupIds[i]].options;if(groupData.length>0){group=this.groups[groupIds[i]];// if bar graphs are stacked, their range need to be handled differently and accumulated over all groups.
	if(options.stack===true&&options.style==='bar'){if(options.yAxisOrientation==='left'){combinedDataLeft=concat$2(combinedDataLeft).call(combinedDataLeft,groupData);}else {combinedDataRight=concat$2(combinedDataRight).call(combinedDataRight,groupData);}}else {groupRanges[groupIds[i]]=group.getYRange(groupData,groupIds[i]);}}}// if bar graphs are stacked, their range need to be handled differently and accumulated over all groups.
	Bargraph.getStackedYRange(combinedDataLeft,groupRanges,groupIds,'__barStackLeft','left');Bargraph.getStackedYRange(combinedDataRight,groupRanges,groupIds,'__barStackRight','right');}};/**
	 * this sets the Y ranges for the Y axis. It also determines which of the axis should be shown or hidden.
	 * @param {Array.<vis.GraphGroup.id>} groupIds
	 * @param {Object} groupRanges
	 * @returns {boolean} resized
	 * @private
	 */LineGraph.prototype._updateYAxis=function(groupIds,groupRanges){var resized=false;var yAxisLeftUsed=false;var yAxisRightUsed=false;var minLeft=1e9,minRight=1e9,maxLeft=-1e9,maxRight=-1e9,minVal,maxVal;// if groups are present
	if(groupIds.length>0){// this is here to make sure that if there are no items in the axis but there are groups, that there is no infinite draw/redraw loop.
	for(var i=0;i<groupIds.length;i++){var group=this.groups[groupIds[i]];if(group&&group.options.yAxisOrientation!='right'){yAxisLeftUsed=true;minLeft=1e9;maxLeft=-1e9;}else if(group&&group.options.yAxisOrientation){yAxisRightUsed=true;minRight=1e9;maxRight=-1e9;}}// if there are items:
	for(i=0;i<groupIds.length;i++){if(groupRanges.hasOwnProperty(groupIds[i])){if(groupRanges[groupIds[i]].ignore!==true){minVal=groupRanges[groupIds[i]].min;maxVal=groupRanges[groupIds[i]].max;if(groupRanges[groupIds[i]].yAxisOrientation!='right'){yAxisLeftUsed=true;minLeft=minLeft>minVal?minVal:minLeft;maxLeft=maxLeft<maxVal?maxVal:maxLeft;}else {yAxisRightUsed=true;minRight=minRight>minVal?minVal:minRight;maxRight=maxRight<maxVal?maxVal:maxRight;}}}}if(yAxisLeftUsed==true){this.yAxisLeft.setRange(minLeft,maxLeft);}if(yAxisRightUsed==true){this.yAxisRight.setRange(minRight,maxRight);}}resized=this._toggleAxisVisiblity(yAxisLeftUsed,this.yAxisLeft)||resized;resized=this._toggleAxisVisiblity(yAxisRightUsed,this.yAxisRight)||resized;if(yAxisRightUsed==true&&yAxisLeftUsed==true){this.yAxisLeft.drawIcons=true;this.yAxisRight.drawIcons=true;}else {this.yAxisLeft.drawIcons=false;this.yAxisRight.drawIcons=false;}this.yAxisRight.master=!yAxisLeftUsed;this.yAxisRight.masterAxis=this.yAxisLeft;if(this.yAxisRight.master==false){if(yAxisRightUsed==true){this.yAxisLeft.lineOffset=this.yAxisRight.width;}else {this.yAxisLeft.lineOffset=0;}resized=this.yAxisLeft.redraw()||resized;resized=this.yAxisRight.redraw()||resized;}else {resized=this.yAxisRight.redraw()||resized;}// clean the accumulated lists
	var tempGroups=['__barStackLeft','__barStackRight','__lineStackLeft','__lineStackRight'];for(i=0;i<tempGroups.length;i++){if(indexOf$3(groupIds).call(groupIds,tempGroups[i])!=-1){splice$2(groupIds).call(groupIds,indexOf$3(groupIds).call(groupIds,tempGroups[i]),1);}}return resized;};/**
	 * This shows or hides the Y axis if needed. If there is a change, the changed event is emitted by the updateYAxis function
	 *
	 * @param {boolean} axisUsed
	 * @param {vis.DataAxis}  axis
	 * @returns {boolean}
	 * @private
	 */LineGraph.prototype._toggleAxisVisiblity=function(axisUsed,axis){var changed=false;if(axisUsed==false){if(axis.dom.frame.parentNode&&axis.hidden==false){axis.hide();changed=true;}}else {if(!axis.dom.frame.parentNode&&axis.hidden==true){axis.show();changed=true;}}return changed;};/**
	 * This uses the DataAxis object to generate the correct X coordinate on the SVG window. It uses the
	 * util function toScreen to get the x coordinate from the timestamp. It also pre-filters the data and get the minMax ranges for
	 * the yAxis.
	 *
	 * @param {Array.<Object>} datapoints
	 * @private
	 */LineGraph.prototype._convertXcoordinates=function(datapoints){var toScreen=this.body.util.toScreen;for(var i=0;i<datapoints.length;i++){datapoints[i].screen_x=toScreen(datapoints[i].x)+this.props.width;datapoints[i].screen_y=datapoints[i].y;//starting point for range calculations
	if(datapoints[i].end!=undefined){datapoints[i].screen_end=toScreen(datapoints[i].end)+this.props.width;}else {datapoints[i].screen_end=undefined;}}};/**
	 * This uses the DataAxis object to generate the correct X coordinate on the SVG window. It uses the
	 * util function toScreen to get the x coordinate from the timestamp. It also pre-filters the data and get the minMax ranges for
	 * the yAxis.
	 *
	 * @param {Array.<Object>} datapoints
	 * @param {vis.GraphGroup} group
	 * @private
	 */LineGraph.prototype._convertYcoordinates=function(datapoints,group){var axis=this.yAxisLeft;var svgHeight=Number(this.svg.style.height.replace('px',''));if(group.options.yAxisOrientation=='right'){axis=this.yAxisRight;}for(var i=0;i<datapoints.length;i++){datapoints[i].screen_y=Math.round(axis.convertValue(datapoints[i].y));}group.setZeroPosition(Math.min(svgHeight,axis.convertValue(0)));};/**
	 * This object contains all possible options. It will check if the types are correct, if required if the option is one
	 * of the allowed values.
	 *
	 * __any__ means that the name of the property does not matter.
	 * __type__ is a required field for all objects and contains the allowed types of all objects
	 */var string$1$1='string';var bool$1$1='boolean';var number$1$1='number';var array$1$1='array';var date$1='date';var object$1$1='object';// should only be in a __type__ property
	var dom$1$1='dom';var moment$2='moment';var any$1$1='any';var allOptions$2$1={configure:{enabled:{'boolean':bool$1$1},filter:{'boolean':bool$1$1,'function':'function'},container:{dom:dom$1$1},__type__:{object:object$1$1,'boolean':bool$1$1,'function':'function'}},//globals :
	alignCurrentTime:{string:string$1$1,'undefined':'undefined'},yAxisOrientation:{string:['left','right']},defaultGroup:{string:string$1$1},sort:{'boolean':bool$1$1},sampling:{'boolean':bool$1$1},stack:{'boolean':bool$1$1},graphHeight:{string:string$1$1,number:number$1$1},shaded:{enabled:{'boolean':bool$1$1},orientation:{string:['bottom','top','zero','group']},// top, bottom, zero, group
	groupId:{object:object$1$1},__type__:{'boolean':bool$1$1,object:object$1$1}},style:{string:['line','bar','points']},// line, bar
	barChart:{width:{number:number$1$1},minWidth:{number:number$1$1},sideBySide:{'boolean':bool$1$1},align:{string:['left','center','right']},__type__:{object:object$1$1}},interpolation:{enabled:{'boolean':bool$1$1},parametrization:{string:['centripetal','chordal','uniform']},// uniform (alpha = 0.0), chordal (alpha = 1.0), centripetal (alpha = 0.5)
	alpha:{number:number$1$1},__type__:{object:object$1$1,'boolean':bool$1$1}},drawPoints:{enabled:{'boolean':bool$1$1},onRender:{'function':'function'},size:{number:number$1$1},style:{string:['square','circle']},// square, circle
	__type__:{object:object$1$1,'boolean':bool$1$1,'function':'function'}},dataAxis:{showMinorLabels:{'boolean':bool$1$1},showMajorLabels:{'boolean':bool$1$1},showWeekScale:{'boolean':bool$1$1},icons:{'boolean':bool$1$1},width:{string:string$1$1,number:number$1$1},visible:{'boolean':bool$1$1},alignZeros:{'boolean':bool$1$1},left:{range:{min:{number:number$1$1,'undefined':'undefined'},max:{number:number$1$1,'undefined':'undefined'},__type__:{object:object$1$1}},format:{'function':'function'},title:{text:{string:string$1$1,number:number$1$1,'undefined':'undefined'},style:{string:string$1$1,'undefined':'undefined'},__type__:{object:object$1$1}},__type__:{object:object$1$1}},right:{range:{min:{number:number$1$1,'undefined':'undefined'},max:{number:number$1$1,'undefined':'undefined'},__type__:{object:object$1$1}},format:{'function':'function'},title:{text:{string:string$1$1,number:number$1$1,'undefined':'undefined'},style:{string:string$1$1,'undefined':'undefined'},__type__:{object:object$1$1}},__type__:{object:object$1$1}},__type__:{object:object$1$1}},legend:{enabled:{'boolean':bool$1$1},icons:{'boolean':bool$1$1},left:{visible:{'boolean':bool$1$1},position:{string:['top-right','bottom-right','top-left','bottom-left']},__type__:{object:object$1$1}},right:{visible:{'boolean':bool$1$1},position:{string:['top-right','bottom-right','top-left','bottom-left']},__type__:{object:object$1$1}},__type__:{object:object$1$1,'boolean':bool$1$1}},groups:{visibility:{any:any$1$1},__type__:{object:object$1$1}},autoResize:{'boolean':bool$1$1},throttleRedraw:{number:number$1$1},// TODO: DEPRICATED see https://github.com/almende/vis/issues/2511
	clickToUse:{'boolean':bool$1$1},end:{number:number$1$1,date:date$1,string:string$1$1,moment:moment$2},format:{minorLabels:{millisecond:{string:string$1$1,'undefined':'undefined'},second:{string:string$1$1,'undefined':'undefined'},minute:{string:string$1$1,'undefined':'undefined'},hour:{string:string$1$1,'undefined':'undefined'},weekday:{string:string$1$1,'undefined':'undefined'},day:{string:string$1$1,'undefined':'undefined'},week:{string:string$1$1,'undefined':'undefined'},month:{string:string$1$1,'undefined':'undefined'},quarter:{string:string$1$1,'undefined':'undefined'},year:{string:string$1$1,'undefined':'undefined'},__type__:{object:object$1$1}},majorLabels:{millisecond:{string:string$1$1,'undefined':'undefined'},second:{string:string$1$1,'undefined':'undefined'},minute:{string:string$1$1,'undefined':'undefined'},hour:{string:string$1$1,'undefined':'undefined'},weekday:{string:string$1$1,'undefined':'undefined'},day:{string:string$1$1,'undefined':'undefined'},week:{string:string$1$1,'undefined':'undefined'},month:{string:string$1$1,'undefined':'undefined'},quarter:{string:string$1$1,'undefined':'undefined'},year:{string:string$1$1,'undefined':'undefined'},__type__:{object:object$1$1}},__type__:{object:object$1$1}},moment:{'function':'function'},height:{string:string$1$1,number:number$1$1},hiddenDates:{start:{date:date$1,number:number$1$1,string:string$1$1,moment:moment$2},end:{date:date$1,number:number$1$1,string:string$1$1,moment:moment$2},repeat:{string:string$1$1},__type__:{object:object$1$1,array:array$1$1}},locale:{string:string$1$1},locales:{__any__:{any:any$1$1},__type__:{object:object$1$1}},max:{date:date$1,number:number$1$1,string:string$1$1,moment:moment$2},maxHeight:{number:number$1$1,string:string$1$1},maxMinorChars:{number:number$1$1},min:{date:date$1,number:number$1$1,string:string$1$1,moment:moment$2},minHeight:{number:number$1$1,string:string$1$1},moveable:{'boolean':bool$1$1},multiselect:{'boolean':bool$1$1},orientation:{string:string$1$1},showCurrentTime:{'boolean':bool$1$1},showMajorLabels:{'boolean':bool$1$1},showMinorLabels:{'boolean':bool$1$1},showWeekScale:{'boolean':bool$1$1},start:{date:date$1,number:number$1$1,string:string$1$1,moment:moment$2},timeAxis:{scale:{string:string$1$1,'undefined':'undefined'},step:{number:number$1$1,'undefined':'undefined'},__type__:{object:object$1$1}},width:{string:string$1$1,number:number$1$1},zoomable:{'boolean':bool$1$1},zoomKey:{string:['ctrlKey','altKey','metaKey','']},zoomMax:{number:number$1$1},zoomMin:{number:number$1$1},zIndex:{number:number$1$1},__type__:{object:object$1$1}};var configureOptions$1$1={global:{alignCurrentTime:['none','year','month','quarter','week','isoWeek','day','date','hour','minute','second'],//yAxisOrientation: ['left','right'], // TDOO: enable as soon as Grahp2d doesn't crash when changing this on the fly
	sort:true,sampling:true,stack:false,shaded:{enabled:false,orientation:['zero','top','bottom','group']// zero, top, bottom
	},style:['line','bar','points'],// line, bar
	barChart:{width:[50,5,100,5],minWidth:[50,5,100,5],sideBySide:false,align:['left','center','right']// left, center, right
	},interpolation:{enabled:true,parametrization:['centripetal','chordal','uniform']// uniform (alpha = 0.0), chordal (alpha = 1.0), centripetal (alpha = 0.5)
	},drawPoints:{enabled:true,size:[6,2,30,1],style:['square','circle']// square, circle
	},dataAxis:{showMinorLabels:true,showMajorLabels:true,showWeekScale:false,icons:false,width:[40,0,200,1],visible:true,alignZeros:true,left:{//range: {min:'undefined': 'undefined'ined,max:'undefined': 'undefined'ined},
	//format: function (value) {return value;},
	title:{text:'',style:''}},right:{//range: {min:'undefined': 'undefined'ined,max:'undefined': 'undefined'ined},
	//format: function (value) {return value;},
	title:{text:'',style:''}}},legend:{enabled:false,icons:true,left:{visible:true,position:['top-right','bottom-right','top-left','bottom-left']// top/bottom - left,right
	},right:{visible:true,position:['top-right','bottom-right','top-left','bottom-left']// top/bottom - left,right
	}},autoResize:true,clickToUse:false,end:'',format:{minorLabels:{millisecond:'SSS',second:'s',minute:'HH:mm',hour:'HH:mm',weekday:'ddd D',day:'D',week:'w',month:'MMM',quarter:'[Q]Q',year:'YYYY'},majorLabels:{millisecond:'HH:mm:ss',second:'D MMMM HH:mm',minute:'ddd D MMMM',hour:'ddd D MMMM',weekday:'MMMM YYYY',day:'MMMM YYYY',week:'MMMM YYYY',month:'YYYY',quarter:'YYYY',year:''}},height:'',locale:'',max:'',maxHeight:'',maxMinorChars:[7,0,20,1],min:'',minHeight:'',moveable:true,orientation:['both','bottom','top'],showCurrentTime:false,showMajorLabels:true,showMinorLabels:true,showWeekScale:false,start:'',width:'100%',zoomable:true,zoomKey:['ctrlKey','altKey','metaKey',''],zoomMax:[315360000000000,10,315360000000000,1],zoomMin:[10,10,315360000000000,1],zIndex:0}};/**
	 * Create a timeline visualization
	 * @param {HTMLElement} container
	 * @param {vis.DataSet | Array} [items]
	 * @param {vis.DataSet | Array | vis.DataView | Object} [groups]
	 * @param {Object} [options]  See Graph2d.setOptions for the available options.
	 * @constructor Graph2d
	 * @extends Core
	 */function Graph2d(container,items,groups,options){var _context184,_context185,_context186,_context187,_context188,_context189,_context190;// if the third element is options, the forth is groups (optionally);
	if(!(isArray$3(groups)||groups instanceof DataSet||groups instanceof DataView)&&groups instanceof Object){var forthArgument=options;options=groups;groups=forthArgument;}// TODO: REMOVE THIS in the next MAJOR release
	// see https://github.com/almende/vis/issues/2511
	if(options&&options.throttleRedraw){console.warn("Graph2d option \"throttleRedraw\" is DEPRICATED and no longer supported. It will be removed in the next MAJOR release.");}var me=this;this.defaultOptions={start:null,end:null,autoResize:true,orientation:{axis:'bottom',// axis orientation: 'bottom', 'top', or 'both'
	item:'bottom'// not relevant for Graph2d
	},moment:moment$1,width:null,height:null,maxHeight:null,minHeight:null};this.options=util$1.deepExtend({},this.defaultOptions);// Create the DOM, props, and emitter
	this._create(container);// all components listed here will be repainted automatically
	this.components=[];this.body={dom:this.dom,domProps:this.props,emitter:{on:bind$2(_context184=this.on).call(_context184,this),off:bind$2(_context185=this.off).call(_context185,this),emit:bind$2(_context186=this.emit).call(_context186,this)},hiddenDates:[],util:{toScreen:bind$2(_context187=me._toScreen).call(_context187,me),toGlobalScreen:bind$2(_context188=me._toGlobalScreen).call(_context188,me),// this refers to the root.width
	toTime:bind$2(_context189=me._toTime).call(_context189,me),toGlobalTime:bind$2(_context190=me._toGlobalTime).call(_context190,me)}};// range
	this.range=new Range$1(this.body);this.components.push(this.range);this.body.range=this.range;// time axis
	this.timeAxis=new TimeAxis(this.body);this.components.push(this.timeAxis);//this.body.util.snap = this.timeAxis.snap.bind(this.timeAxis);
	// current time bar
	this.currentTime=new CurrentTime(this.body);this.components.push(this.currentTime);// item set
	this.linegraph=new LineGraph(this.body);this.components.push(this.linegraph);this.itemsData=null;// DataSet
	this.groupsData=null;// DataSet
	this.on('tap',function(event){me.emit('click',me.getEventProperties(event));});this.on('doubletap',function(event){me.emit('doubleClick',me.getEventProperties(event));});this.dom.root.oncontextmenu=function(event){me.emit('contextmenu',me.getEventProperties(event));};//Single time autoscale/fit
	this.initialFitDone=false;this.on('changed',function(){if(me.itemsData==null)return;if(!me.initialFitDone&&!me.options.rollingMode){me.initialFitDone=true;if(me.options.start!=undefined||me.options.end!=undefined){if(me.options.start==undefined||me.options.end==undefined){var range=me.getItemRange();}var start=me.options.start!=undefined?me.options.start:range.min;var end=me.options.end!=undefined?me.options.end:range.max;me.setWindow(start,end,{animation:false});}else {me.fit({animation:false});}}if(!me.initialDrawDone&&(me.initialRangeChangeDone||!me.options.start&&!me.options.end||me.options.rollingMode)){me.initialDrawDone=true;me.dom.root.style.visibility='visible';me.dom.loadingScreen.parentNode.removeChild(me.dom.loadingScreen);if(me.options.onInitialDrawComplete){setTimeout$2(function(){return me.options.onInitialDrawComplete();},0);}}});// apply options
	if(options){this.setOptions(options);}// IMPORTANT: THIS HAPPENS BEFORE SET ITEMS!
	if(groups){this.setGroups(groups);}// create itemset
	if(items){this.setItems(items);}// draw for the first time
	this._redraw();}// Extend the functionality from Core
	Graph2d.prototype=new Core();Graph2d.prototype.setOptions=function(options){// validate options
	var errorFound=Validator$2.validate(options,allOptions$2$1);if(errorFound===true){console.log('%cErrors have been found in the supplied options object.',printStyle$2);}Core.prototype.setOptions.call(this,options);};/**
	 * Set items
	 * @param {vis.DataSet | Array | null} items
	 */Graph2d.prototype.setItems=function(items){var initialLoad=this.itemsData==null;// convert to type DataSet when needed
	var newDataSet;if(!items){newDataSet=null;}else if(items instanceof DataSet||items instanceof DataView){newDataSet=typeCoerceDataSet(items);}else {// turn an array into a dataset
	newDataSet=typeCoerceDataSet(new DataSet(items));}// set items
	if(this.itemsData){// stop maintaining a coerced version of the old data set
	this.itemsData.dispose();}this.itemsData=newDataSet;this.linegraph&&this.linegraph.setItems(newDataSet!=null?newDataSet.rawDS:null);if(initialLoad){if(this.options.start!=undefined||this.options.end!=undefined){var start=this.options.start!=undefined?this.options.start:null;var end=this.options.end!=undefined?this.options.end:null;this.setWindow(start,end,{animation:false});}else {this.fit({animation:false});}}};/**
	 * Set groups
	 * @param {vis.DataSet | Array} groups
	 */Graph2d.prototype.setGroups=function(groups){// convert to type DataSet when needed
	var newDataSet;if(!groups){newDataSet=null;}else if(groups instanceof DataSet||groups instanceof DataView){newDataSet=groups;}else {// turn an array into a dataset
	newDataSet=new DataSet(groups);}this.groupsData=newDataSet;this.linegraph.setGroups(newDataSet);};/**
	 * Returns an object containing an SVG element with the icon of the group (size determined by iconWidth and iconHeight), the label of the group (content) and the yAxisOrientation of the group (left or right).
	 * @param {vis.GraphGroup.id} groupId
	 * @param {number} width
	 * @param {number} height
	 * @returns {{icon: SVGElement, label: string, orientation: string}|string}
	 */Graph2d.prototype.getLegend=function(groupId,width,height){if(width===undefined){width=15;}if(height===undefined){height=15;}if(this.linegraph.groups[groupId]!==undefined){return this.linegraph.groups[groupId].getLegend(width,height);}else {return "cannot find group:'"+groupId+"'";}};/**
	 * This checks if the visible option of the supplied group (by ID) is true or false.
	 * @param {vis.GraphGroup.id} groupId
	 * @returns {boolean}
	 */Graph2d.prototype.isGroupVisible=function(groupId){if(this.linegraph.groups[groupId]!==undefined){return this.linegraph.groups[groupId].visible&&(this.linegraph.options.groups.visibility[groupId]===undefined||this.linegraph.options.groups.visibility[groupId]==true);}else {return false;}};/**
	 * Get the data range of the item set.
	 * @returns {{min: Date, max: Date}} range  A range with a start and end Date.
	 *                                          When no minimum is found, min==null
	 *                                          When no maximum is found, max==null
	 */Graph2d.prototype.getDataRange=function(){var min=null;var max=null;// calculate min from start filed
	for(var groupId in this.linegraph.groups){if(this.linegraph.groups.hasOwnProperty(groupId)){if(this.linegraph.groups[groupId].visible==true){for(var i=0;i<this.linegraph.groups[groupId].itemsData.length;i++){var item=this.linegraph.groups[groupId].itemsData[i];var value=util$1.convert(item.x,'Date').valueOf();min=min==null?value:min>value?value:min;max=max==null?value:max<value?value:max;}}}}return {min:min!=null?new Date(min):null,max:max!=null?new Date(max):null};};/**
	 * Generate Timeline related information from an event
	 * @param {Event} event
	 * @return {Object} An object with related information, like on which area
	 *                  The event happened, whether clicked on an item, etc.
	 */Graph2d.prototype.getEventProperties=function(event){var clientX=event.center?event.center.x:event.clientX;var clientY=event.center?event.center.y:event.clientY;var x=clientX-util$1.getAbsoluteLeft(this.dom.centerContainer);var y=clientY-util$1.getAbsoluteTop(this.dom.centerContainer);var time=this._toTime(x);var customTime=CustomTime.customTimeFromTarget(event);var element=util$1.getTarget(event);var what=null;if(util$1.hasParent(element,this.timeAxis.dom.foreground)){what='axis';}else if(this.timeAxis2&&util$1.hasParent(element,this.timeAxis2.dom.foreground)){what='axis';}else if(util$1.hasParent(element,this.linegraph.yAxisLeft.dom.frame)){what='data-axis';}else if(util$1.hasParent(element,this.linegraph.yAxisRight.dom.frame)){what='data-axis';}else if(util$1.hasParent(element,this.linegraph.legendLeft.dom.frame)){what='legend';}else if(util$1.hasParent(element,this.linegraph.legendRight.dom.frame)){what='legend';}else if(customTime!=null){what='custom-time';}else if(util$1.hasParent(element,this.currentTime.bar)){what='current-time';}else if(util$1.hasParent(element,this.dom.center)){what='background';}var value=[];var yAxisLeft=this.linegraph.yAxisLeft;var yAxisRight=this.linegraph.yAxisRight;if(!yAxisLeft.hidden&&this.itemsData.length>0){value.push(yAxisLeft.screenToValue(y));}if(!yAxisRight.hidden&&this.itemsData.length>0){value.push(yAxisRight.screenToValue(y));}return {event:event,customTime:customTime?customTime.options.id:null,what:what,pageX:event.srcEvent?event.srcEvent.pageX:event.pageX,pageY:event.srcEvent?event.srcEvent.pageY:event.pageY,x:x,y:y,time:time,value:value};};/**
	 * Load a configurator
	 * @return {Object}
	 * @private
	 */Graph2d.prototype._createConfigurator=function(){return new Configurator$1(this,this.dom.container,configureOptions$1$1);};// Locales have to be supplied by the user.
	var defaultLanguage=getNavigatorLanguage();moment.locale(defaultLanguage);var timeline={Core:Core,DateUtil:DateUtil,Range:Range$1,stack:stack$1,TimeStep:TimeStep,components:{items:{Item:Item,BackgroundItem:BackgroundItem,BoxItem:BoxItem,PointItem:PointItem,RangeItem:RangeItem},BackgroundGroup:BackgroundGroup,Component:Component,CurrentTime:CurrentTime,CustomTime:CustomTime,DataAxis:DataAxis,DataScale:DataScale,GraphGroup:GraphGroup,Group:Group,ItemSet:ItemSet,Legend:Legend,LineGraph:LineGraph,TimeAxis:TimeAxis}};

	var nsTimeline = /*#__PURE__*/Object.freeze({
		__proto__: null,
		Graph2d: Graph2d,
		Timeline: Timeline,
		timeline: timeline
	});

	var de$1 = createCommonjsModule(function (module, exports) {

	  (function (global, factory) {
	     typeof commonjsRequire === 'function' ? factory(moment) :  factory(global.moment);
	  })(commonjsGlobal, function (moment) {

	    function processRelativeTime(number, withoutSuffix, key, isFuture) {
	      var format = {
	        m: ['eine Minute', 'einer Minute'],
	        h: ['eine Stunde', 'einer Stunde'],
	        d: ['ein Tag', 'einem Tag'],
	        dd: [number + ' Tage', number + ' Tagen'],
	        M: ['ein Monat', 'einem Monat'],
	        MM: [number + ' Monate', number + ' Monaten'],
	        y: ['ein Jahr', 'einem Jahr'],
	        yy: [number + ' Jahre', number + ' Jahren']
	      };
	      return withoutSuffix ? format[key][0] : format[key][1];
	    }

	    var de = moment.defineLocale('de', {
	      months: 'Januar_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember'.split('_'),
	      monthsShort: 'Jan._Feb._März_Apr._Mai_Juni_Juli_Aug._Sep._Okt._Nov._Dez.'.split('_'),
	      monthsParseExact: true,
	      weekdays: 'Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag'.split('_'),
	      weekdaysShort: 'So._Mo._Di._Mi._Do._Fr._Sa.'.split('_'),
	      weekdaysMin: 'So_Mo_Di_Mi_Do_Fr_Sa'.split('_'),
	      weekdaysParseExact: true,
	      longDateFormat: {
	        LT: 'HH:mm',
	        LTS: 'HH:mm:ss',
	        L: 'DD.MM.YYYY',
	        LL: 'D. MMMM YYYY',
	        LLL: 'D. MMMM YYYY HH:mm',
	        LLLL: 'dddd, D. MMMM YYYY HH:mm'
	      },
	      calendar: {
	        sameDay: '[heute um] LT [Uhr]',
	        sameElse: 'L',
	        nextDay: '[morgen um] LT [Uhr]',
	        nextWeek: 'dddd [um] LT [Uhr]',
	        lastDay: '[gestern um] LT [Uhr]',
	        lastWeek: '[letzten] dddd [um] LT [Uhr]'
	      },
	      relativeTime: {
	        future: 'in %s',
	        past: 'vor %s',
	        s: 'ein paar Sekunden',
	        ss: '%d Sekunden',
	        m: processRelativeTime,
	        mm: '%d Minuten',
	        h: processRelativeTime,
	        hh: '%d Stunden',
	        d: processRelativeTime,
	        dd: processRelativeTime,
	        M: processRelativeTime,
	        MM: processRelativeTime,
	        y: processRelativeTime,
	        yy: processRelativeTime
	      },
	      dayOfMonthOrdinalParse: /\d{1,2}\./,
	      ordinal: '%d.',
	      week: {
	        dow: 1,
	        // Monday is the first day of the week.
	        doy: 4 // The week that contains Jan 4th is the first week of the year.

	      }
	    });
	    return de;
	  });
	});

	var es$1 = createCommonjsModule(function (module, exports) {

	  (function (global, factory) {
	     typeof commonjsRequire === 'function' ? factory(moment) :  factory(global.moment);
	  })(commonjsGlobal, function (moment) {

	    var monthsShortDot = 'ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.'.split('_'),
	        monthsShort = 'ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic'.split('_'),
	        monthsParse = [/^ene/i, /^feb/i, /^mar/i, /^abr/i, /^may/i, /^jun/i, /^jul/i, /^ago/i, /^sep/i, /^oct/i, /^nov/i, /^dic/i],
	        monthsRegex = /^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i;
	    var es = moment.defineLocale('es', {
	      months: 'enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre'.split('_'),
	      monthsShort: function (m, format) {
	        if (!m) {
	          return monthsShortDot;
	        } else if (/-MMM-/.test(format)) {
	          return monthsShort[m.month()];
	        } else {
	          return monthsShortDot[m.month()];
	        }
	      },
	      monthsRegex: monthsRegex,
	      monthsShortRegex: monthsRegex,
	      monthsStrictRegex: /^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,
	      monthsShortStrictRegex: /^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,
	      monthsParse: monthsParse,
	      longMonthsParse: monthsParse,
	      shortMonthsParse: monthsParse,
	      weekdays: 'domingo_lunes_martes_miércoles_jueves_viernes_sábado'.split('_'),
	      weekdaysShort: 'dom._lun._mar._mié._jue._vie._sáb.'.split('_'),
	      weekdaysMin: 'do_lu_ma_mi_ju_vi_sá'.split('_'),
	      weekdaysParseExact: true,
	      longDateFormat: {
	        LT: 'H:mm',
	        LTS: 'H:mm:ss',
	        L: 'DD/MM/YYYY',
	        LL: 'D [de] MMMM [de] YYYY',
	        LLL: 'D [de] MMMM [de] YYYY H:mm',
	        LLLL: 'dddd, D [de] MMMM [de] YYYY H:mm'
	      },
	      calendar: {
	        sameDay: function () {
	          return '[hoy a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
	        },
	        nextDay: function () {
	          return '[mañana a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
	        },
	        nextWeek: function () {
	          return 'dddd [a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
	        },
	        lastDay: function () {
	          return '[ayer a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
	        },
	        lastWeek: function () {
	          return '[el] dddd [pasado a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
	        },
	        sameElse: 'L'
	      },
	      relativeTime: {
	        future: 'en %s',
	        past: 'hace %s',
	        s: 'unos segundos',
	        ss: '%d segundos',
	        m: 'un minuto',
	        mm: '%d minutos',
	        h: 'una hora',
	        hh: '%d horas',
	        d: 'un día',
	        dd: '%d días',
	        M: 'un mes',
	        MM: '%d meses',
	        y: 'un año',
	        yy: '%d años'
	      },
	      dayOfMonthOrdinalParse: /\d{1,2}º/,
	      ordinal: '%dº',
	      week: {
	        dow: 1,
	        // Monday is the first day of the week.
	        doy: 4 // The week that contains Jan 4th is the first week of the year.

	      },
	      invalidDate: 'Fecha invalida'
	    });
	    return es;
	  });
	});

	var fr$1 = createCommonjsModule(function (module, exports) {

	  (function (global, factory) {
	     typeof commonjsRequire === 'function' ? factory(moment) :  factory(global.moment);
	  })(commonjsGlobal, function (moment) {

	    var fr = moment.defineLocale('fr', {
	      months: 'janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre'.split('_'),
	      monthsShort: 'janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.'.split('_'),
	      monthsParseExact: true,
	      weekdays: 'dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi'.split('_'),
	      weekdaysShort: 'dim._lun._mar._mer._jeu._ven._sam.'.split('_'),
	      weekdaysMin: 'di_lu_ma_me_je_ve_sa'.split('_'),
	      weekdaysParseExact: true,
	      longDateFormat: {
	        LT: 'HH:mm',
	        LTS: 'HH:mm:ss',
	        L: 'DD/MM/YYYY',
	        LL: 'D MMMM YYYY',
	        LLL: 'D MMMM YYYY HH:mm',
	        LLLL: 'dddd D MMMM YYYY HH:mm'
	      },
	      calendar: {
	        sameDay: '[Aujourd’hui à] LT',
	        nextDay: '[Demain à] LT',
	        nextWeek: 'dddd [à] LT',
	        lastDay: '[Hier à] LT',
	        lastWeek: 'dddd [dernier à] LT',
	        sameElse: 'L'
	      },
	      relativeTime: {
	        future: 'dans %s',
	        past: 'il y a %s',
	        s: 'quelques secondes',
	        ss: '%d secondes',
	        m: 'une minute',
	        mm: '%d minutes',
	        h: 'une heure',
	        hh: '%d heures',
	        d: 'un jour',
	        dd: '%d jours',
	        M: 'un mois',
	        MM: '%d mois',
	        y: 'un an',
	        yy: '%d ans'
	      },
	      dayOfMonthOrdinalParse: /\d{1,2}(er|)/,
	      ordinal: function (number, period) {
	        switch (period) {
	          // TODO: Return 'e' when day of month > 1. Move this case inside
	          // block for masculine words below.
	          // See https://github.com/moment/moment/issues/3375
	          case 'D':
	            return number + (number === 1 ? 'er' : '');
	          // Words with masculine grammatical gender: mois, trimestre, jour

	          default:
	          case 'M':
	          case 'Q':
	          case 'DDD':
	          case 'd':
	            return number + (number === 1 ? 'er' : 'e');
	          // Words with feminine grammatical gender: semaine

	          case 'w':
	          case 'W':
	            return number + (number === 1 ? 're' : 'e');
	        }
	      },
	      week: {
	        dow: 1,
	        // Monday is the first day of the week.
	        doy: 4 // The week that contains Jan 4th is the first week of the year.

	      }
	    });
	    return fr;
	  });
	});

	var it$1 = createCommonjsModule(function (module, exports) {

	  (function (global, factory) {
	     typeof commonjsRequire === 'function' ? factory(moment) :  factory(global.moment);
	  })(commonjsGlobal, function (moment) {

	    var it = moment.defineLocale('it', {
	      months: 'gennaio_febbraio_marzo_aprile_maggio_giugno_luglio_agosto_settembre_ottobre_novembre_dicembre'.split('_'),
	      monthsShort: 'gen_feb_mar_apr_mag_giu_lug_ago_set_ott_nov_dic'.split('_'),
	      weekdays: 'domenica_lunedì_martedì_mercoledì_giovedì_venerdì_sabato'.split('_'),
	      weekdaysShort: 'dom_lun_mar_mer_gio_ven_sab'.split('_'),
	      weekdaysMin: 'do_lu_ma_me_gi_ve_sa'.split('_'),
	      longDateFormat: {
	        LT: 'HH:mm',
	        LTS: 'HH:mm:ss',
	        L: 'DD/MM/YYYY',
	        LL: 'D MMMM YYYY',
	        LLL: 'D MMMM YYYY HH:mm',
	        LLLL: 'dddd D MMMM YYYY HH:mm'
	      },
	      calendar: {
	        sameDay: function () {
	          return '[Oggi a' + (this.hours() > 1 ? 'lle ' : this.hours() === 0 ? ' ' : "ll'") + ']LT';
	        },
	        nextDay: function () {
	          return '[Domani a' + (this.hours() > 1 ? 'lle ' : this.hours() === 0 ? ' ' : "ll'") + ']LT';
	        },
	        nextWeek: function () {
	          return 'dddd [a' + (this.hours() > 1 ? 'lle ' : this.hours() === 0 ? ' ' : "ll'") + ']LT';
	        },
	        lastDay: function () {
	          return '[Ieri a' + (this.hours() > 1 ? 'lle ' : this.hours() === 0 ? ' ' : "ll'") + ']LT';
	        },
	        lastWeek: function () {
	          switch (this.day()) {
	            case 0:
	              return '[La scorsa] dddd [a' + (this.hours() > 1 ? 'lle ' : this.hours() === 0 ? ' ' : "ll'") + ']LT';

	            default:
	              return '[Lo scorso] dddd [a' + (this.hours() > 1 ? 'lle ' : this.hours() === 0 ? ' ' : "ll'") + ']LT';
	          }
	        },
	        sameElse: 'L'
	      },
	      relativeTime: {
	        future: function (s) {
	          return (/^[0-9].+$/.test(s) ? 'tra' : 'in') + ' ' + s;
	        },
	        past: '%s fa',
	        s: 'alcuni secondi',
	        ss: '%d secondi',
	        m: 'un minuto',
	        mm: '%d minuti',
	        h: "un'ora",
	        hh: '%d ore',
	        d: 'un giorno',
	        dd: '%d giorni',
	        M: 'un mese',
	        MM: '%d mesi',
	        y: 'un anno',
	        yy: '%d anni'
	      },
	      dayOfMonthOrdinalParse: /\d{1,2}º/,
	      ordinal: '%dº',
	      week: {
	        dow: 1,
	        // Monday is the first day of the week.
	        doy: 4 // The week that contains Jan 4th is the first week of the year.

	      }
	    });
	    return it;
	  });
	});

	var ja$1 = createCommonjsModule(function (module, exports) {

	  (function (global, factory) {
	     typeof commonjsRequire === 'function' ? factory(moment) :  factory(global.moment);
	  })(commonjsGlobal, function (moment) {

	    var ja = moment.defineLocale('ja', {
	      eras: [{
	        since: '2019-05-01',
	        offset: 1,
	        name: '令和',
	        narrow: '㋿',
	        abbr: 'R'
	      }, {
	        since: '1989-01-08',
	        until: '2019-04-30',
	        offset: 1,
	        name: '平成',
	        narrow: '㍻',
	        abbr: 'H'
	      }, {
	        since: '1926-12-25',
	        until: '1989-01-07',
	        offset: 1,
	        name: '昭和',
	        narrow: '㍼',
	        abbr: 'S'
	      }, {
	        since: '1912-07-30',
	        until: '1926-12-24',
	        offset: 1,
	        name: '大正',
	        narrow: '㍽',
	        abbr: 'T'
	      }, {
	        since: '1873-01-01',
	        until: '1912-07-29',
	        offset: 6,
	        name: '明治',
	        narrow: '㍾',
	        abbr: 'M'
	      }, {
	        since: '0001-01-01',
	        until: '1873-12-31',
	        offset: 1,
	        name: '西暦',
	        narrow: 'AD',
	        abbr: 'AD'
	      }, {
	        since: '0000-12-31',
	        until: -Infinity,
	        offset: 1,
	        name: '紀元前',
	        narrow: 'BC',
	        abbr: 'BC'
	      }],
	      eraYearOrdinalRegex: /(元|\d+)年/,
	      eraYearOrdinalParse: function (input, match) {
	        return match[1] === '元' ? 1 : parseInt(match[1] || input, 10);
	      },
	      months: '1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月'.split('_'),
	      monthsShort: '1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月'.split('_'),
	      weekdays: '日曜日_月曜日_火曜日_水曜日_木曜日_金曜日_土曜日'.split('_'),
	      weekdaysShort: '日_月_火_水_木_金_土'.split('_'),
	      weekdaysMin: '日_月_火_水_木_金_土'.split('_'),
	      longDateFormat: {
	        LT: 'HH:mm',
	        LTS: 'HH:mm:ss',
	        L: 'YYYY/MM/DD',
	        LL: 'YYYY年M月D日',
	        LLL: 'YYYY年M月D日 HH:mm',
	        LLLL: 'YYYY年M月D日 dddd HH:mm',
	        l: 'YYYY/MM/DD',
	        ll: 'YYYY年M月D日',
	        lll: 'YYYY年M月D日 HH:mm',
	        llll: 'YYYY年M月D日(ddd) HH:mm'
	      },
	      meridiemParse: /午前|午後/i,
	      isPM: function (input) {
	        return input === '午後';
	      },
	      meridiem: function (hour, minute, isLower) {
	        if (hour < 12) {
	          return '午前';
	        } else {
	          return '午後';
	        }
	      },
	      calendar: {
	        sameDay: '[今日] LT',
	        nextDay: '[明日] LT',
	        nextWeek: function (now) {
	          if (now.week() !== this.week()) {
	            return '[来週]dddd LT';
	          } else {
	            return 'dddd LT';
	          }
	        },
	        lastDay: '[昨日] LT',
	        lastWeek: function (now) {
	          if (this.week() !== now.week()) {
	            return '[先週]dddd LT';
	          } else {
	            return 'dddd LT';
	          }
	        },
	        sameElse: 'L'
	      },
	      dayOfMonthOrdinalParse: /\d{1,2}日/,
	      ordinal: function (number, period) {
	        switch (period) {
	          case 'y':
	            return number === 1 ? '元年' : number + '年';

	          case 'd':
	          case 'D':
	          case 'DDD':
	            return number + '日';

	          default:
	            return number;
	        }
	      },
	      relativeTime: {
	        future: '%s後',
	        past: '%s前',
	        s: '数秒',
	        ss: '%d秒',
	        m: '1分',
	        mm: '%d分',
	        h: '1時間',
	        hh: '%d時間',
	        d: '1日',
	        dd: '%d日',
	        M: '1ヶ月',
	        MM: '%dヶ月',
	        y: '1年',
	        yy: '%d年'
	      }
	    });
	    return ja;
	  });
	});

	var nl$1 = createCommonjsModule(function (module, exports) {

	  (function (global, factory) {
	     typeof commonjsRequire === 'function' ? factory(moment) :  factory(global.moment);
	  })(commonjsGlobal, function (moment) {

	    var monthsShortWithDots = 'jan._feb._mrt._apr._mei_jun._jul._aug._sep._okt._nov._dec.'.split('_'),
	        monthsShortWithoutDots = 'jan_feb_mrt_apr_mei_jun_jul_aug_sep_okt_nov_dec'.split('_'),
	        monthsParse = [/^jan/i, /^feb/i, /^maart|mrt.?$/i, /^apr/i, /^mei$/i, /^jun[i.]?$/i, /^jul[i.]?$/i, /^aug/i, /^sep/i, /^okt/i, /^nov/i, /^dec/i],
	        monthsRegex = /^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december|jan\.?|feb\.?|mrt\.?|apr\.?|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i;
	    var nl = moment.defineLocale('nl', {
	      months: 'januari_februari_maart_april_mei_juni_juli_augustus_september_oktober_november_december'.split('_'),
	      monthsShort: function (m, format) {
	        if (!m) {
	          return monthsShortWithDots;
	        } else if (/-MMM-/.test(format)) {
	          return monthsShortWithoutDots[m.month()];
	        } else {
	          return monthsShortWithDots[m.month()];
	        }
	      },
	      monthsRegex: monthsRegex,
	      monthsShortRegex: monthsRegex,
	      monthsStrictRegex: /^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december)/i,
	      monthsShortStrictRegex: /^(jan\.?|feb\.?|mrt\.?|apr\.?|mei|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,
	      monthsParse: monthsParse,
	      longMonthsParse: monthsParse,
	      shortMonthsParse: monthsParse,
	      weekdays: 'zondag_maandag_dinsdag_woensdag_donderdag_vrijdag_zaterdag'.split('_'),
	      weekdaysShort: 'zo._ma._di._wo._do._vr._za.'.split('_'),
	      weekdaysMin: 'zo_ma_di_wo_do_vr_za'.split('_'),
	      weekdaysParseExact: true,
	      longDateFormat: {
	        LT: 'HH:mm',
	        LTS: 'HH:mm:ss',
	        L: 'DD-MM-YYYY',
	        LL: 'D MMMM YYYY',
	        LLL: 'D MMMM YYYY HH:mm',
	        LLLL: 'dddd D MMMM YYYY HH:mm'
	      },
	      calendar: {
	        sameDay: '[vandaag om] LT',
	        nextDay: '[morgen om] LT',
	        nextWeek: 'dddd [om] LT',
	        lastDay: '[gisteren om] LT',
	        lastWeek: '[afgelopen] dddd [om] LT',
	        sameElse: 'L'
	      },
	      relativeTime: {
	        future: 'over %s',
	        past: '%s geleden',
	        s: 'een paar seconden',
	        ss: '%d seconden',
	        m: 'één minuut',
	        mm: '%d minuten',
	        h: 'één uur',
	        hh: '%d uur',
	        d: 'één dag',
	        dd: '%d dagen',
	        M: 'één maand',
	        MM: '%d maanden',
	        y: 'één jaar',
	        yy: '%d jaar'
	      },
	      dayOfMonthOrdinalParse: /\d{1,2}(ste|de)/,
	      ordinal: function (number) {
	        return number + (number === 1 || number === 8 || number >= 20 ? 'ste' : 'de');
	      },
	      week: {
	        dow: 1,
	        // Monday is the first day of the week.
	        doy: 4 // The week that contains Jan 4th is the first week of the year.

	      }
	    });
	    return nl;
	  });
	});

	var pl$1 = createCommonjsModule(function (module, exports) {

	  (function (global, factory) {
	     typeof commonjsRequire === 'function' ? factory(moment) :  factory(global.moment);
	  })(commonjsGlobal, function (moment) {

	    var monthsNominative = 'styczeń_luty_marzec_kwiecień_maj_czerwiec_lipiec_sierpień_wrzesień_październik_listopad_grudzień'.split('_'),
	        monthsSubjective = 'stycznia_lutego_marca_kwietnia_maja_czerwca_lipca_sierpnia_września_października_listopada_grudnia'.split('_');

	    function plural(n) {
	      return n % 10 < 5 && n % 10 > 1 && ~~(n / 10) % 10 !== 1;
	    }

	    function translate(number, withoutSuffix, key) {
	      var result = number + ' ';

	      switch (key) {
	        case 'ss':
	          return result + (plural(number) ? 'sekundy' : 'sekund');

	        case 'm':
	          return withoutSuffix ? 'minuta' : 'minutę';

	        case 'mm':
	          return result + (plural(number) ? 'minuty' : 'minut');

	        case 'h':
	          return withoutSuffix ? 'godzina' : 'godzinę';

	        case 'hh':
	          return result + (plural(number) ? 'godziny' : 'godzin');

	        case 'MM':
	          return result + (plural(number) ? 'miesiące' : 'miesięcy');

	        case 'yy':
	          return result + (plural(number) ? 'lata' : 'lat');
	      }
	    }

	    var pl = moment.defineLocale('pl', {
	      months: function (momentToFormat, format) {
	        if (!momentToFormat) {
	          return monthsNominative;
	        } else if (format === '') {
	          // Hack: if format empty we know this is used to generate
	          // RegExp by moment. Give then back both valid forms of months
	          // in RegExp ready format.
	          return '(' + monthsSubjective[momentToFormat.month()] + '|' + monthsNominative[momentToFormat.month()] + ')';
	        } else if (/D MMMM/.test(format)) {
	          return monthsSubjective[momentToFormat.month()];
	        } else {
	          return monthsNominative[momentToFormat.month()];
	        }
	      },
	      monthsShort: 'sty_lut_mar_kwi_maj_cze_lip_sie_wrz_paź_lis_gru'.split('_'),
	      weekdays: 'niedziela_poniedziałek_wtorek_środa_czwartek_piątek_sobota'.split('_'),
	      weekdaysShort: 'ndz_pon_wt_śr_czw_pt_sob'.split('_'),
	      weekdaysMin: 'Nd_Pn_Wt_Śr_Cz_Pt_So'.split('_'),
	      longDateFormat: {
	        LT: 'HH:mm',
	        LTS: 'HH:mm:ss',
	        L: 'DD.MM.YYYY',
	        LL: 'D MMMM YYYY',
	        LLL: 'D MMMM YYYY HH:mm',
	        LLLL: 'dddd, D MMMM YYYY HH:mm'
	      },
	      calendar: {
	        sameDay: '[Dziś o] LT',
	        nextDay: '[Jutro o] LT',
	        nextWeek: function () {
	          switch (this.day()) {
	            case 0:
	              return '[W niedzielę o] LT';

	            case 2:
	              return '[We wtorek o] LT';

	            case 3:
	              return '[W środę o] LT';

	            case 6:
	              return '[W sobotę o] LT';

	            default:
	              return '[W] dddd [o] LT';
	          }
	        },
	        lastDay: '[Wczoraj o] LT',
	        lastWeek: function () {
	          switch (this.day()) {
	            case 0:
	              return '[W zeszłą niedzielę o] LT';

	            case 3:
	              return '[W zeszłą środę o] LT';

	            case 6:
	              return '[W zeszłą sobotę o] LT';

	            default:
	              return '[W zeszły] dddd [o] LT';
	          }
	        },
	        sameElse: 'L'
	      },
	      relativeTime: {
	        future: 'za %s',
	        past: '%s temu',
	        s: 'kilka sekund',
	        ss: translate,
	        m: translate,
	        mm: translate,
	        h: translate,
	        hh: translate,
	        d: '1 dzień',
	        dd: '%d dni',
	        M: 'miesiąc',
	        MM: translate,
	        y: 'rok',
	        yy: translate
	      },
	      dayOfMonthOrdinalParse: /\d{1,2}\./,
	      ordinal: '%d.',
	      week: {
	        dow: 1,
	        // Monday is the first day of the week.
	        doy: 4 // The week that contains Jan 4th is the first week of the year.

	      }
	    });
	    return pl;
	  });
	});

	var ru$1 = createCommonjsModule(function (module, exports) {

	  (function (global, factory) {
	     typeof commonjsRequire === 'function' ? factory(moment) :  factory(global.moment);
	  })(commonjsGlobal, function (moment) {

	    function plural(word, num) {
	      var forms = word.split('_');
	      return num % 10 === 1 && num % 100 !== 11 ? forms[0] : num % 10 >= 2 && num % 10 <= 4 && (num % 100 < 10 || num % 100 >= 20) ? forms[1] : forms[2];
	    }

	    function relativeTimeWithPlural(number, withoutSuffix, key) {
	      var format = {
	        ss: withoutSuffix ? 'секунда_секунды_секунд' : 'секунду_секунды_секунд',
	        mm: withoutSuffix ? 'минута_минуты_минут' : 'минуту_минуты_минут',
	        hh: 'час_часа_часов',
	        dd: 'день_дня_дней',
	        MM: 'месяц_месяца_месяцев',
	        yy: 'год_года_лет'
	      };

	      if (key === 'm') {
	        return withoutSuffix ? 'минута' : 'минуту';
	      } else {
	        return number + ' ' + plural(format[key], +number);
	      }
	    }

	    var monthsParse = [/^янв/i, /^фев/i, /^мар/i, /^апр/i, /^ма[йя]/i, /^июн/i, /^июл/i, /^авг/i, /^сен/i, /^окт/i, /^ноя/i, /^дек/i]; // http://new.gramota.ru/spravka/rules/139-prop : § 103
	    // Сокращения месяцев: http://new.gramota.ru/spravka/buro/search-answer?s=242637
	    // CLDR data:          http://www.unicode.org/cldr/charts/28/summary/ru.html#1753

	    var ru = moment.defineLocale('ru', {
	      months: {
	        format: 'января_февраля_марта_апреля_мая_июня_июля_августа_сентября_октября_ноября_декабря'.split('_'),
	        standalone: 'январь_февраль_март_апрель_май_июнь_июль_август_сентябрь_октябрь_ноябрь_декабрь'.split('_')
	      },
	      monthsShort: {
	        // по CLDR именно "июл." и "июн.", но какой смысл менять букву на точку ?
	        format: 'янв._февр._мар._апр._мая_июня_июля_авг._сент._окт._нояб._дек.'.split('_'),
	        standalone: 'янв._февр._март_апр._май_июнь_июль_авг._сент._окт._нояб._дек.'.split('_')
	      },
	      weekdays: {
	        standalone: 'воскресенье_понедельник_вторник_среда_четверг_пятница_суббота'.split('_'),
	        format: 'воскресенье_понедельник_вторник_среду_четверг_пятницу_субботу'.split('_'),
	        isFormat: /\[ ?[Вв] ?(?:прошлую|следующую|эту)? ?\] ?dddd/
	      },
	      weekdaysShort: 'вс_пн_вт_ср_чт_пт_сб'.split('_'),
	      weekdaysMin: 'вс_пн_вт_ср_чт_пт_сб'.split('_'),
	      monthsParse: monthsParse,
	      longMonthsParse: monthsParse,
	      shortMonthsParse: monthsParse,
	      // полные названия с падежами, по три буквы, для некоторых, по 4 буквы, сокращения с точкой и без точки
	      monthsRegex: /^(январ[ья]|янв\.?|феврал[ья]|февр?\.?|марта?|мар\.?|апрел[ья]|апр\.?|ма[йя]|июн[ья]|июн\.?|июл[ья]|июл\.?|августа?|авг\.?|сентябр[ья]|сент?\.?|октябр[ья]|окт\.?|ноябр[ья]|нояб?\.?|декабр[ья]|дек\.?)/i,
	      // копия предыдущего
	      monthsShortRegex: /^(январ[ья]|янв\.?|феврал[ья]|февр?\.?|марта?|мар\.?|апрел[ья]|апр\.?|ма[йя]|июн[ья]|июн\.?|июл[ья]|июл\.?|августа?|авг\.?|сентябр[ья]|сент?\.?|октябр[ья]|окт\.?|ноябр[ья]|нояб?\.?|декабр[ья]|дек\.?)/i,
	      // полные названия с падежами
	      monthsStrictRegex: /^(январ[яь]|феврал[яь]|марта?|апрел[яь]|ма[яй]|июн[яь]|июл[яь]|августа?|сентябр[яь]|октябр[яь]|ноябр[яь]|декабр[яь])/i,
	      // Выражение, которое соотвествует только сокращённым формам
	      monthsShortStrictRegex: /^(янв\.|февр?\.|мар[т.]|апр\.|ма[яй]|июн[ья.]|июл[ья.]|авг\.|сент?\.|окт\.|нояб?\.|дек\.)/i,
	      longDateFormat: {
	        LT: 'H:mm',
	        LTS: 'H:mm:ss',
	        L: 'DD.MM.YYYY',
	        LL: 'D MMMM YYYY г.',
	        LLL: 'D MMMM YYYY г., H:mm',
	        LLLL: 'dddd, D MMMM YYYY г., H:mm'
	      },
	      calendar: {
	        sameDay: '[Сегодня, в] LT',
	        nextDay: '[Завтра, в] LT',
	        lastDay: '[Вчера, в] LT',
	        nextWeek: function (now) {
	          if (now.week() !== this.week()) {
	            switch (this.day()) {
	              case 0:
	                return '[В следующее] dddd, [в] LT';

	              case 1:
	              case 2:
	              case 4:
	                return '[В следующий] dddd, [в] LT';

	              case 3:
	              case 5:
	              case 6:
	                return '[В следующую] dddd, [в] LT';
	            }
	          } else {
	            if (this.day() === 2) {
	              return '[Во] dddd, [в] LT';
	            } else {
	              return '[В] dddd, [в] LT';
	            }
	          }
	        },
	        lastWeek: function (now) {
	          if (now.week() !== this.week()) {
	            switch (this.day()) {
	              case 0:
	                return '[В прошлое] dddd, [в] LT';

	              case 1:
	              case 2:
	              case 4:
	                return '[В прошлый] dddd, [в] LT';

	              case 3:
	              case 5:
	              case 6:
	                return '[В прошлую] dddd, [в] LT';
	            }
	          } else {
	            if (this.day() === 2) {
	              return '[Во] dddd, [в] LT';
	            } else {
	              return '[В] dddd, [в] LT';
	            }
	          }
	        },
	        sameElse: 'L'
	      },
	      relativeTime: {
	        future: 'через %s',
	        past: '%s назад',
	        s: 'несколько секунд',
	        ss: relativeTimeWithPlural,
	        m: relativeTimeWithPlural,
	        mm: relativeTimeWithPlural,
	        h: 'час',
	        hh: relativeTimeWithPlural,
	        d: 'день',
	        dd: relativeTimeWithPlural,
	        M: 'месяц',
	        MM: relativeTimeWithPlural,
	        y: 'год',
	        yy: relativeTimeWithPlural
	      },
	      meridiemParse: /ночи|утра|дня|вечера/i,
	      isPM: function (input) {
	        return /^(дня|вечера)$/.test(input);
	      },
	      meridiem: function (hour, minute, isLower) {
	        if (hour < 4) {
	          return 'ночи';
	        } else if (hour < 12) {
	          return 'утра';
	        } else if (hour < 17) {
	          return 'дня';
	        } else {
	          return 'вечера';
	        }
	      },
	      dayOfMonthOrdinalParse: /\d{1,2}-(й|го|я)/,
	      ordinal: function (number, period) {
	        switch (period) {
	          case 'M':
	          case 'd':
	          case 'DDD':
	            return number + '-й';

	          case 'D':
	            return number + '-го';

	          case 'w':
	          case 'W':
	            return number + '-я';

	          default:
	            return number;
	        }
	      },
	      week: {
	        dow: 1,
	        // Monday is the first day of the week.
	        doy: 4 // The week that contains Jan 4th is the first week of the year.

	      }
	    });
	    return ru;
	  });
	});

	var uk$1 = createCommonjsModule(function (module, exports) {

	  (function (global, factory) {
	     typeof commonjsRequire === 'function' ? factory(moment) :  factory(global.moment);
	  })(commonjsGlobal, function (moment) {

	    function plural(word, num) {
	      var forms = word.split('_');
	      return num % 10 === 1 && num % 100 !== 11 ? forms[0] : num % 10 >= 2 && num % 10 <= 4 && (num % 100 < 10 || num % 100 >= 20) ? forms[1] : forms[2];
	    }

	    function relativeTimeWithPlural(number, withoutSuffix, key) {
	      var format = {
	        ss: withoutSuffix ? 'секунда_секунди_секунд' : 'секунду_секунди_секунд',
	        mm: withoutSuffix ? 'хвилина_хвилини_хвилин' : 'хвилину_хвилини_хвилин',
	        hh: withoutSuffix ? 'година_години_годин' : 'годину_години_годин',
	        dd: 'день_дні_днів',
	        MM: 'місяць_місяці_місяців',
	        yy: 'рік_роки_років'
	      };

	      if (key === 'm') {
	        return withoutSuffix ? 'хвилина' : 'хвилину';
	      } else if (key === 'h') {
	        return withoutSuffix ? 'година' : 'годину';
	      } else {
	        return number + ' ' + plural(format[key], +number);
	      }
	    }

	    function weekdaysCaseReplace(m, format) {
	      var weekdays = {
	        nominative: 'неділя_понеділок_вівторок_середа_четвер_п’ятниця_субота'.split('_'),
	        accusative: 'неділю_понеділок_вівторок_середу_четвер_п’ятницю_суботу'.split('_'),
	        genitive: 'неділі_понеділка_вівторка_середи_четверга_п’ятниці_суботи'.split('_')
	      },
	          nounCase;

	      if (m === true) {
	        return weekdays['nominative'].slice(1, 7).concat(weekdays['nominative'].slice(0, 1));
	      }

	      if (!m) {
	        return weekdays['nominative'];
	      }

	      nounCase = /(\[[ВвУу]\]) ?dddd/.test(format) ? 'accusative' : /\[?(?:минулої|наступної)? ?\] ?dddd/.test(format) ? 'genitive' : 'nominative';
	      return weekdays[nounCase][m.day()];
	    }

	    function processHoursFunction(str) {
	      return function () {
	        return str + 'о' + (this.hours() === 11 ? 'б' : '') + '] LT';
	      };
	    }

	    var uk = moment.defineLocale('uk', {
	      months: {
	        format: 'січня_лютого_березня_квітня_травня_червня_липня_серпня_вересня_жовтня_листопада_грудня'.split('_'),
	        standalone: 'січень_лютий_березень_квітень_травень_червень_липень_серпень_вересень_жовтень_листопад_грудень'.split('_')
	      },
	      monthsShort: 'січ_лют_бер_квіт_трав_черв_лип_серп_вер_жовт_лист_груд'.split('_'),
	      weekdays: weekdaysCaseReplace,
	      weekdaysShort: 'нд_пн_вт_ср_чт_пт_сб'.split('_'),
	      weekdaysMin: 'нд_пн_вт_ср_чт_пт_сб'.split('_'),
	      longDateFormat: {
	        LT: 'HH:mm',
	        LTS: 'HH:mm:ss',
	        L: 'DD.MM.YYYY',
	        LL: 'D MMMM YYYY р.',
	        LLL: 'D MMMM YYYY р., HH:mm',
	        LLLL: 'dddd, D MMMM YYYY р., HH:mm'
	      },
	      calendar: {
	        sameDay: processHoursFunction('[Сьогодні '),
	        nextDay: processHoursFunction('[Завтра '),
	        lastDay: processHoursFunction('[Вчора '),
	        nextWeek: processHoursFunction('[У] dddd ['),
	        lastWeek: function () {
	          switch (this.day()) {
	            case 0:
	            case 3:
	            case 5:
	            case 6:
	              return processHoursFunction('[Минулої] dddd [').call(this);

	            case 1:
	            case 2:
	            case 4:
	              return processHoursFunction('[Минулого] dddd [').call(this);
	          }
	        },
	        sameElse: 'L'
	      },
	      relativeTime: {
	        future: 'за %s',
	        past: '%s тому',
	        s: 'декілька секунд',
	        ss: relativeTimeWithPlural,
	        m: relativeTimeWithPlural,
	        mm: relativeTimeWithPlural,
	        h: 'годину',
	        hh: relativeTimeWithPlural,
	        d: 'день',
	        dd: relativeTimeWithPlural,
	        M: 'місяць',
	        MM: relativeTimeWithPlural,
	        y: 'рік',
	        yy: relativeTimeWithPlural
	      },
	      // M. E.: those two are virtually unused but a user might want to implement them for his/her website for some reason
	      meridiemParse: /ночі|ранку|дня|вечора/,
	      isPM: function (input) {
	        return /^(дня|вечора)$/.test(input);
	      },
	      meridiem: function (hour, minute, isLower) {
	        if (hour < 4) {
	          return 'ночі';
	        } else if (hour < 12) {
	          return 'ранку';
	        } else if (hour < 17) {
	          return 'дня';
	        } else {
	          return 'вечора';
	        }
	      },
	      dayOfMonthOrdinalParse: /\d{1,2}-(й|го)/,
	      ordinal: function (number, period) {
	        switch (period) {
	          case 'M':
	          case 'd':
	          case 'DDD':
	          case 'w':
	          case 'W':
	            return number + '-й';

	          case 'D':
	            return number + '-го';

	          default:
	            return number;
	        }
	      },
	      week: {
	        dow: 1,
	        // Monday is the first day of the week.
	        doy: 7 // The week that contains Jan 7th is the first week of the year.

	      }
	    });
	    return uk;
	  });
	});

	// DOM utility methods

	/**
	 * this prepares the JSON container for allocating SVG elements
	 * @param {Object} JSONcontainer
	 * @private
	 */
	function prepareElements$1(JSONcontainer) {
	  // cleanup the redundant svgElements;
	  for (var elementType in JSONcontainer) {
	    if (JSONcontainer.hasOwnProperty(elementType)) {
	      JSONcontainer[elementType].redundant = JSONcontainer[elementType].used;
	      JSONcontainer[elementType].used = [];
	    }
	  }
	}
	/**
	 * this cleans up all the unused SVG elements. By asking for the parentNode, we only need to supply the JSON container from
	 * which to remove the redundant elements.
	 *
	 * @param {Object} JSONcontainer
	 * @private
	 */

	function cleanupElements$1(JSONcontainer) {
	  // cleanup the redundant svgElements;
	  for (var elementType in JSONcontainer) {
	    if (JSONcontainer.hasOwnProperty(elementType)) {
	      if (JSONcontainer[elementType].redundant) {
	        for (var i = 0; i < JSONcontainer[elementType].redundant.length; i++) {
	          JSONcontainer[elementType].redundant[i].parentNode.removeChild(JSONcontainer[elementType].redundant[i]);
	        }

	        JSONcontainer[elementType].redundant = [];
	      }
	    }
	  }
	}
	/**
	 * Ensures that all elements are removed first up so they can be recreated cleanly
	 * @param {Object} JSONcontainer
	 */

	function resetElements$1(JSONcontainer) {
	  prepareElements$1(JSONcontainer);
	  cleanupElements$1(JSONcontainer);
	  prepareElements$1(JSONcontainer);
	}
	/**
	 * Allocate or generate an SVG element if needed. Store a reference to it in the JSON container and draw it in the svgContainer
	 * the JSON container and the SVG container have to be supplied so other svg containers (like the legend) can use this.
	 *
	 * @param {string} elementType
	 * @param {Object} JSONcontainer
	 * @param {Object} svgContainer
	 * @returns {Element}
	 * @private
	 */

	function getSVGElement$1(elementType, JSONcontainer, svgContainer) {
	  var element; // allocate SVG element, if it doesnt yet exist, create one.

	  if (JSONcontainer.hasOwnProperty(elementType)) {
	    // this element has been created before
	    // check if there is an redundant element
	    if (JSONcontainer[elementType].redundant.length > 0) {
	      element = JSONcontainer[elementType].redundant[0];
	      JSONcontainer[elementType].redundant.shift();
	    } else {
	      // create a new element and add it to the SVG
	      element = document.createElementNS("http://www.w3.org/2000/svg", elementType);
	      svgContainer.appendChild(element);
	    }
	  } else {
	    // create a new element and add it to the SVG, also create a new object in the svgElements to keep track of it.
	    element = document.createElementNS("http://www.w3.org/2000/svg", elementType);
	    JSONcontainer[elementType] = {
	      used: [],
	      redundant: []
	    };
	    svgContainer.appendChild(element);
	  }

	  JSONcontainer[elementType].used.push(element);
	  return element;
	}
	/**
	 * Allocate or generate an SVG element if needed. Store a reference to it in the JSON container and draw it in the svgContainer
	 * the JSON container and the SVG container have to be supplied so other svg containers (like the legend) can use this.
	 *
	 * @param {string} elementType
	 * @param {Object} JSONcontainer
	 * @param {Element} DOMContainer
	 * @param {Element} insertBefore
	 * @returns {*}
	 */

	function getDOMElement$1(elementType, JSONcontainer, DOMContainer, insertBefore) {
	  var element; // allocate DOM element, if it doesnt yet exist, create one.

	  if (JSONcontainer.hasOwnProperty(elementType)) {
	    // this element has been created before
	    // check if there is an redundant element
	    if (JSONcontainer[elementType].redundant.length > 0) {
	      element = JSONcontainer[elementType].redundant[0];
	      JSONcontainer[elementType].redundant.shift();
	    } else {
	      // create a new element and add it to the SVG
	      element = document.createElement(elementType);

	      if (insertBefore !== undefined) {
	        DOMContainer.insertBefore(element, insertBefore);
	      } else {
	        DOMContainer.appendChild(element);
	      }
	    }
	  } else {
	    // create a new element and add it to the SVG, also create a new object in the svgElements to keep track of it.
	    element = document.createElement(elementType);
	    JSONcontainer[elementType] = {
	      used: [],
	      redundant: []
	    };

	    if (insertBefore !== undefined) {
	      DOMContainer.insertBefore(element, insertBefore);
	    } else {
	      DOMContainer.appendChild(element);
	    }
	  }

	  JSONcontainer[elementType].used.push(element);
	  return element;
	}
	/**
	 * Draw a point object. This is a separate function because it can also be called by the legend.
	 * The reason the JSONcontainer and the target SVG svgContainer have to be supplied is so the legend can use these functions
	 * as well.
	 *
	 * @param {number} x
	 * @param {number} y
	 * @param {Object} groupTemplate: A template containing the necessary information to draw the datapoint e.g., {style: 'circle', size: 5, className: 'className' }
	 * @param {Object} JSONcontainer
	 * @param {Object} svgContainer
	 * @param {Object} labelObj
	 * @returns {vis.PointItem}
	 */

	function drawPoint$1(x, y, groupTemplate, JSONcontainer, svgContainer, labelObj) {
	  var point;

	  if (groupTemplate.style == "circle") {
	    point = getSVGElement$1("circle", JSONcontainer, svgContainer);
	    point.setAttributeNS(null, "cx", x);
	    point.setAttributeNS(null, "cy", y);
	    point.setAttributeNS(null, "r", 0.5 * groupTemplate.size);
	  } else {
	    point = getSVGElement$1("rect", JSONcontainer, svgContainer);
	    point.setAttributeNS(null, "x", x - 0.5 * groupTemplate.size);
	    point.setAttributeNS(null, "y", y - 0.5 * groupTemplate.size);
	    point.setAttributeNS(null, "width", groupTemplate.size);
	    point.setAttributeNS(null, "height", groupTemplate.size);
	  }

	  if (groupTemplate.styles !== undefined) {
	    point.setAttributeNS(null, "style", groupTemplate.styles);
	  }

	  point.setAttributeNS(null, "class", groupTemplate.className + " vis-point"); //handle label

	  if (labelObj) {
	    var label = getSVGElement$1("text", JSONcontainer, svgContainer);

	    if (labelObj.xOffset) {
	      x = x + labelObj.xOffset;
	    }

	    if (labelObj.yOffset) {
	      y = y + labelObj.yOffset;
	    }

	    if (labelObj.content) {
	      label.textContent = labelObj.content;
	    }

	    if (labelObj.className) {
	      label.setAttributeNS(null, "class", labelObj.className + " vis-label");
	    }

	    label.setAttributeNS(null, "x", x);
	    label.setAttributeNS(null, "y", y);
	  }

	  return point;
	}
	/**
	 * draw a bar SVG element centered on the X coordinate
	 *
	 * @param {number} x
	 * @param {number} y
	 * @param {number} width
	 * @param {number} height
	 * @param {string} className
	 * @param {Object} JSONcontainer
	 * @param {Object} svgContainer
	 * @param {string} style
	 */

	function drawBar$1(x, y, width, height, className, JSONcontainer, svgContainer, style) {
	  if (height != 0) {
	    if (height < 0) {
	      height *= -1;
	      y -= height;
	    }

	    var rect = getSVGElement$1("rect", JSONcontainer, svgContainer);
	    rect.setAttributeNS(null, "x", x - 0.5 * width);
	    rect.setAttributeNS(null, "y", y);
	    rect.setAttributeNS(null, "width", width);
	    rect.setAttributeNS(null, "height", height);
	    rect.setAttributeNS(null, "class", className);

	    if (style) {
	      rect.setAttributeNS(null, "style", style);
	    }
	  }
	}
	/**
	 * get default language
	 * @returns {string}
	 */

	function getNavigatorLanguage$1() {
	  try {
	    if (!navigator) return "en";

	    if (navigator.languages && navigator.languages.length) {
	      return navigator.languages;
	    } else {
	      return navigator.userLanguage || navigator.language || navigator.browserLanguage || "en";
	    }
	  } catch (error) {
	    return "en";
	  }
	}

	var DOMutil = /*#__PURE__*/Object.freeze({
		__proto__: null,
		prepareElements: prepareElements$1,
		cleanupElements: cleanupElements$1,
		resetElements: resetElements$1,
		getSVGElement: getSVGElement$1,
		getDOMElement: getDOMElement$1,
		drawPoint: drawPoint$1,
		drawBar: drawBar$1,
		getNavigatorLanguage: getNavigatorLanguage$1
	});

	// use this instance. Else, load via commonjs.
	//
	// Note: This doesn't work in ESM.

	var moment$3 = typeof window !== "undefined" && window["moment"] || moment;

	var moment$4 = /*#__PURE__*/Object.freeze({
		__proto__: null,
		'default': moment$3,
		__moduleExports: moment$3
	});

	/**
	 * Setup a mock hammer.js object, for unit testing.
	 *
	 * Inspiration: https://github.com/uber/deck.gl/pull/658
	 *
	 * @returns {{on: noop, off: noop, destroy: noop, emit: noop, get: get}}
	 */

	function hammerMock$1() {
	  var noop = function noop() {};

	  return {
	    on: noop,
	    off: noop,
	    destroy: noop,
	    emit: noop,
	    get: function get(m) {
	      //eslint-disable-line no-unused-vars
	      return {
	        set: noop
	      };
	    }
	  };
	}

	var modifiedHammer$1;

	if (typeof window !== "undefined") {
	  var OurHammer$1 = window["Hammer"] || Hammer;
	  modifiedHammer$1 = propagating(OurHammer$1, {
	    preventDefault: "mouse"
	  });
	} else {
	  modifiedHammer$1 = function modifiedHammer() {
	    return (// hammer.js is only available in a browser, not in node.js. Replacing it with a mock object.
	      hammerMock$1()
	    );
	  };
	}

	var modifiedHammer$2 = modifiedHammer$1;

	var Hammer$2 = /*#__PURE__*/Object.freeze({
		__proto__: null,
		'default': modifiedHammer$2
	});

	var defaultExport = {};

	assign$2(defaultExport, nsData);

	assign$2(defaultExport, nsGraph3d);

	assign$2(defaultExport, nsNetwork);

	assign$2(defaultExport, nsTimeline);
	var network = {
	  Images: Images,
	  dotparser: dotparser$1,
	  gephiParser: gephiParser,
	  allOptions: options,
	  convertDot: parseDOTNetwork,
	  convertGephi: parseGephi
	};
	defaultExport.network = network;
	defaultExport.util = util;
	defaultExport.DOMutil = DOMutil;
	defaultExport.moment = moment$4;
	defaultExport.Hammer = Hammer$2;
	defaultExport.keycharm = keycharm$1;

	exports.DOMutil = DOMutil;
	exports.DataSet = DataSet;
	exports.DataStream = DataStream;
	exports.DataView = DataView;
	exports.Graph2d = Graph2d;
	exports.Graph3d = Graph3d;
	exports.Graph3dCamera = Camera;
	exports.Graph3dFilter = Filter;
	exports.Graph3dPoint2d = Point2d_1;
	exports.Graph3dPoint3d = Point3d_1;
	exports.Graph3dSlider = Slider;
	exports.Graph3dStepNumber = StepNumber_1;
	exports.Hammer = Hammer$2;
	exports.Network = Network;
	exports.NetworkImages = Images;
	exports.Queue = Queue;
	exports.Timeline = Timeline;
	exports.createNewDataPipeFrom = createNewDataPipeFrom;
	exports.keycharm = keycharm$1;
	exports.moment = moment$4;
	exports.network = network;
	exports.networkDOTParser = dotparser$1;
	exports.networkGephiParser = gephiParser;
	exports.networkOptions = options;
	exports.parseDOTNetwork = parseDOTNetwork;
	exports.parseGephiNetwork = parseGephi;
	exports.timeline = timeline;
	exports.util = util;

	Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=vis-charts.js.map
